var xA=Object.defineProperty;var SA=(e,t,n)=>t in e?xA(e,t,{enumerable:!0,configurable:!0,writable:!0,value:n}):e[t]=n;var qi=(e,t,n)=>SA(e,typeof t!="symbol"?t+"":t,n);(function(){const t=document.createElement("link").relList;if(t&&t.supports&&t.supports("modulepreload"))return;for(const a of document.querySelectorAll('link[rel="modulepreload"]'))r(a);new MutationObserver(a=>{for(const s of a)if(s.type==="childList")for(const o of s.addedNodes)o.tagName==="LINK"&&o.rel==="modulepreload"&&r(o)}).observe(document,{childList:!0,subtree:!0});function n(a){const s={};return a.integrity&&(s.integrity=a.integrity),a.referrerPolicy&&(s.referrerPolicy=a.referrerPolicy),a.crossOrigin==="use-credentials"?s.credentials="include":a.crossOrigin==="anonymous"?s.credentials="omit":s.credentials="same-origin",s}function r(a){if(a.ep)return;a.ep=!0;const s=n(a);fetch(a.href,s)}})();function Mf(e){return e&&e.__esModule&&Object.prototype.hasOwnProperty.call(e,"default")?e.default:e}var u1={exports:{}},Ec={};/**
 * @license React
 * react-jsx-runtime.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var By;function wA(){if(By)return Ec;By=1;var e=Symbol.for("react.transitional.element"),t=Symbol.for("react.fragment");function n(r,a,s){var o=null;if(s!==void 0&&(o=""+s),a.key!==void 0&&(o=""+a.key),"key"in a){s={};for(var c in a)c!=="key"&&(s[c]=a[c])}else s=a;return a=s.ref,{$$typeof:e,type:r,key:o,ref:a!==void 0?a:null,props:s}}return Ec.Fragment=t,Ec.jsx=n,Ec.jsxs=n,Ec}var Uy;function AA(){return Uy||(Uy=1,u1.exports=wA()),u1.exports}var R=AA(),c1={exports:{}},Tt={};/**
 * @license React
 * react.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var Fy;function kA(){if(Fy)return Tt;Fy=1;var e=Symbol.for("react.transitional.element"),t=Symbol.for("react.portal"),n=Symbol.for("react.fragment"),r=Symbol.for("react.strict_mode"),a=Symbol.for("react.profiler"),s=Symbol.for("react.consumer"),o=Symbol.for("react.context"),c=Symbol.for("react.forward_ref"),d=Symbol.for("react.suspense"),f=Symbol.for("react.memo"),m=Symbol.for("react.lazy"),p=Symbol.iterator;function b(k){return k===null||typeof k!="object"?null:(k=p&&k[p]||k["@@iterator"],typeof k=="function"?k:null)}var y={isMounted:function(){return!1},enqueueForceUpdate:function(){},enqueueReplaceState:function(){},enqueueSetState:function(){}},T=Object.assign,x={};function C(k,ie,ge){this.props=k,this.context=ie,this.refs=x,this.updater=ge||y}C.prototype.isReactComponent={},C.prototype.setState=function(k,ie){if(typeof k!="object"&&typeof k!="function"&&k!=null)throw Error("takes an object of state variables to update or a function which returns an object of state variables.");this.updater.enqueueSetState(this,k,ie,"setState")},C.prototype.forceUpdate=function(k){this.updater.enqueueForceUpdate(this,k,"forceUpdate")};function w(){}w.prototype=C.prototype;function O(k,ie,ge){this.props=k,this.context=ie,this.refs=x,this.updater=ge||y}var D=O.prototype=new w;D.constructor=O,T(D,C.prototype),D.isPureReactComponent=!0;var Y=Array.isArray,H={H:null,A:null,T:null,S:null,V:null},B=Object.prototype.hasOwnProperty;function ee(k,ie,ge,L,Oe,je){return ge=je.ref,{$$typeof:e,type:k,key:ie,ref:ge!==void 0?ge:null,props:je}}function Z(k,ie){return ee(k.type,ie,void 0,void 0,void 0,k.props)}function Q(k){return typeof k=="object"&&k!==null&&k.$$typeof===e}function G(k){var ie={"=":"=0",":":"=2"};return"$"+k.replace(/[=:]/g,function(ge){return ie[ge]})}var se=/\/+/g;function oe(k,ie){return typeof k=="object"&&k!==null&&k.key!=null?G(""+k.key):ie.toString(36)}function ye(){}function fe(k){switch(k.status){case"fulfilled":return k.value;case"rejected":throw k.reason;default:switch(typeof k.status=="string"?k.then(ye,ye):(k.status="pending",k.then(function(ie){k.status==="pending"&&(k.status="fulfilled",k.value=ie)},function(ie){k.status==="pending"&&(k.status="rejected",k.reason=ie)})),k.status){case"fulfilled":return k.value;case"rejected":throw k.reason}}throw k}function de(k,ie,ge,L,Oe){var je=typeof k;(je==="undefined"||je==="boolean")&&(k=null);var De=!1;if(k===null)De=!0;else switch(je){case"bigint":case"string":case"number":De=!0;break;case"object":switch(k.$$typeof){case e:case t:De=!0;break;case m:return De=k._init,de(De(k._payload),ie,ge,L,Oe)}}if(De)return Oe=Oe(k),De=L===""?"."+oe(k,0):L,Y(Oe)?(ge="",De!=null&&(ge=De.replace(se,"$&/")+"/"),de(Oe,ie,ge,"",function(zt){return zt})):Oe!=null&&(Q(Oe)&&(Oe=Z(Oe,ge+(Oe.key==null||k&&k.key===Oe.key?"":(""+Oe.key).replace(se,"$&/")+"/")+De)),ie.push(Oe)),1;De=0;var Ot=L===""?".":L+":";if(Y(k))for(var ot=0;ot<k.length;ot++)L=k[ot],je=Ot+oe(L,ot),De+=de(L,ie,ge,je,Oe);else if(ot=b(k),typeof ot=="function")for(k=ot.call(k),ot=0;!(L=k.next()).done;)L=L.value,je=Ot+oe(L,ot++),De+=de(L,ie,ge,je,Oe);else if(je==="object"){if(typeof k.then=="function")return de(fe(k),ie,ge,L,Oe);throw ie=String(k),Error("Objects are not valid as a React child (found: "+(ie==="[object Object]"?"object with keys {"+Object.keys(k).join(", ")+"}":ie)+"). If you meant to render a collection of children, use an array instead.")}return De}function V(k,ie,ge){if(k==null)return k;var L=[],Oe=0;return de(k,L,"","",function(je){return ie.call(ge,je,Oe++)}),L}function me(k){if(k._status===-1){var ie=k._result;ie=ie(),ie.then(function(ge){(k._status===0||k._status===-1)&&(k._status=1,k._result=ge)},function(ge){(k._status===0||k._status===-1)&&(k._status=2,k._result=ge)}),k._status===-1&&(k._status=0,k._result=ie)}if(k._status===1)return k._result.default;throw k._result}var pe=typeof reportError=="function"?reportError:function(k){if(typeof window=="object"&&typeof window.ErrorEvent=="function"){var ie=new window.ErrorEvent("error",{bubbles:!0,cancelable:!0,message:typeof k=="object"&&k!==null&&typeof k.message=="string"?String(k.message):String(k),error:k});if(!window.dispatchEvent(ie))return}else if(typeof process=="object"&&typeof process.emit=="function"){process.emit("uncaughtException",k);return}console.error(k)};function Be(){}return Tt.Children={map:V,forEach:function(k,ie,ge){V(k,function(){ie.apply(this,arguments)},ge)},count:function(k){var ie=0;return V(k,function(){ie++}),ie},toArray:function(k){return V(k,function(ie){return ie})||[]},only:function(k){if(!Q(k))throw Error("React.Children.only expected to receive a single React element child.");return k}},Tt.Component=C,Tt.Fragment=n,Tt.Profiler=a,Tt.PureComponent=O,Tt.StrictMode=r,Tt.Suspense=d,Tt.__CLIENT_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE=H,Tt.__COMPILER_RUNTIME={__proto__:null,c:function(k){return H.H.useMemoCache(k)}},Tt.cache=function(k){return function(){return k.apply(null,arguments)}},Tt.cloneElement=function(k,ie,ge){if(k==null)throw Error("The argument must be a React element, but you passed "+k+".");var L=T({},k.props),Oe=k.key,je=void 0;if(ie!=null)for(De in ie.ref!==void 0&&(je=void 0),ie.key!==void 0&&(Oe=""+ie.key),ie)!B.call(ie,De)||De==="key"||De==="__self"||De==="__source"||De==="ref"&&ie.ref===void 0||(L[De]=ie[De]);var De=arguments.length-2;if(De===1)L.children=ge;else if(1<De){for(var Ot=Array(De),ot=0;ot<De;ot++)Ot[ot]=arguments[ot+2];L.children=Ot}return ee(k.type,Oe,void 0,void 0,je,L)},Tt.createContext=function(k){return k={$$typeof:o,_currentValue:k,_currentValue2:k,_threadCount:0,Provider:null,Consumer:null},k.Provider=k,k.Consumer={$$typeof:s,_context:k},k},Tt.createElement=function(k,ie,ge){var L,Oe={},je=null;if(ie!=null)for(L in ie.key!==void 0&&(je=""+ie.key),ie)B.call(ie,L)&&L!=="key"&&L!=="__self"&&L!=="__source"&&(Oe[L]=ie[L]);var De=arguments.length-2;if(De===1)Oe.children=ge;else if(1<De){for(var Ot=Array(De),ot=0;ot<De;ot++)Ot[ot]=arguments[ot+2];Oe.children=Ot}if(k&&k.defaultProps)for(L in De=k.defaultProps,De)Oe[L]===void 0&&(Oe[L]=De[L]);return ee(k,je,void 0,void 0,null,Oe)},Tt.createRef=function(){return{current:null}},Tt.forwardRef=function(k){return{$$typeof:c,render:k}},Tt.isValidElement=Q,Tt.lazy=function(k){return{$$typeof:m,_payload:{_status:-1,_result:k},_init:me}},Tt.memo=function(k,ie){return{$$typeof:f,type:k,compare:ie===void 0?null:ie}},Tt.startTransition=function(k){var ie=H.T,ge={};H.T=ge;try{var L=k(),Oe=H.S;Oe!==null&&Oe(ge,L),typeof L=="object"&&L!==null&&typeof L.then=="function"&&L.then(Be,pe)}catch(je){pe(je)}finally{H.T=ie}},Tt.unstable_useCacheRefresh=function(){return H.H.useCacheRefresh()},Tt.use=function(k){return H.H.use(k)},Tt.useActionState=function(k,ie,ge){return H.H.useActionState(k,ie,ge)},Tt.useCallback=function(k,ie){return H.H.useCallback(k,ie)},Tt.useContext=function(k){return H.H.useContext(k)},Tt.useDebugValue=function(){},Tt.useDeferredValue=function(k,ie){return H.H.useDeferredValue(k,ie)},Tt.useEffect=function(k,ie,ge){var L=H.H;if(typeof ge=="function")throw Error("useEffect CRUD overload is not enabled in this build of React.");return L.useEffect(k,ie)},Tt.useId=function(){return H.H.useId()},Tt.useImperativeHandle=function(k,ie,ge){return H.H.useImperativeHandle(k,ie,ge)},Tt.useInsertionEffect=function(k,ie){return H.H.useInsertionEffect(k,ie)},Tt.useLayoutEffect=function(k,ie){return H.H.useLayoutEffect(k,ie)},Tt.useMemo=function(k,ie){return H.H.useMemo(k,ie)},Tt.useOptimistic=function(k,ie){return H.H.useOptimistic(k,ie)},Tt.useReducer=function(k,ie,ge){return H.H.useReducer(k,ie,ge)},Tt.useRef=function(k){return H.H.useRef(k)},Tt.useState=function(k){return H.H.useState(k)},Tt.useSyncExternalStore=function(k,ie,ge){return H.H.useSyncExternalStore(k,ie,ge)},Tt.useTransition=function(){return H.H.useTransition()},Tt.version="19.1.0",Tt}var zy;function If(){return zy||(zy=1,c1.exports=kA()),c1.exports}var z=If();const ce=Mf(z);var d1={exports:{}},Tc={},h1={exports:{}},f1={};/**
 * @license React
 * scheduler.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var Hy;function CA(){return Hy||(Hy=1,function(e){function t(V,me){var pe=V.length;V.push(me);e:for(;0<pe;){var Be=pe-1>>>1,k=V[Be];if(0<a(k,me))V[Be]=me,V[pe]=k,pe=Be;else break e}}function n(V){return V.length===0?null:V[0]}function r(V){if(V.length===0)return null;var me=V[0],pe=V.pop();if(pe!==me){V[0]=pe;e:for(var Be=0,k=V.length,ie=k>>>1;Be<ie;){var ge=2*(Be+1)-1,L=V[ge],Oe=ge+1,je=V[Oe];if(0>a(L,pe))Oe<k&&0>a(je,L)?(V[Be]=je,V[Oe]=pe,Be=Oe):(V[Be]=L,V[ge]=pe,Be=ge);else if(Oe<k&&0>a(je,pe))V[Be]=je,V[Oe]=pe,Be=Oe;else break e}}return me}function a(V,me){var pe=V.sortIndex-me.sortIndex;return pe!==0?pe:V.id-me.id}if(e.unstable_now=void 0,typeof performance=="object"&&typeof performance.now=="function"){var s=performance;e.unstable_now=function(){return s.now()}}else{var o=Date,c=o.now();e.unstable_now=function(){return o.now()-c}}var d=[],f=[],m=1,p=null,b=3,y=!1,T=!1,x=!1,C=!1,w=typeof setTimeout=="function"?setTimeout:null,O=typeof clearTimeout=="function"?clearTimeout:null,D=typeof setImmediate<"u"?setImmediate:null;function Y(V){for(var me=n(f);me!==null;){if(me.callback===null)r(f);else if(me.startTime<=V)r(f),me.sortIndex=me.expirationTime,t(d,me);else break;me=n(f)}}function H(V){if(x=!1,Y(V),!T)if(n(d)!==null)T=!0,B||(B=!0,oe());else{var me=n(f);me!==null&&de(H,me.startTime-V)}}var B=!1,ee=-1,Z=5,Q=-1;function G(){return C?!0:!(e.unstable_now()-Q<Z)}function se(){if(C=!1,B){var V=e.unstable_now();Q=V;var me=!0;try{e:{T=!1,x&&(x=!1,O(ee),ee=-1),y=!0;var pe=b;try{t:{for(Y(V),p=n(d);p!==null&&!(p.expirationTime>V&&G());){var Be=p.callback;if(typeof Be=="function"){p.callback=null,b=p.priorityLevel;var k=Be(p.expirationTime<=V);if(V=e.unstable_now(),typeof k=="function"){p.callback=k,Y(V),me=!0;break t}p===n(d)&&r(d),Y(V)}else r(d);p=n(d)}if(p!==null)me=!0;else{var ie=n(f);ie!==null&&de(H,ie.startTime-V),me=!1}}break e}finally{p=null,b=pe,y=!1}me=void 0}}finally{me?oe():B=!1}}}var oe;if(typeof D=="function")oe=function(){D(se)};else if(typeof MessageChannel<"u"){var ye=new MessageChannel,fe=ye.port2;ye.port1.onmessage=se,oe=function(){fe.postMessage(null)}}else oe=function(){w(se,0)};function de(V,me){ee=w(function(){V(e.unstable_now())},me)}e.unstable_IdlePriority=5,e.unstable_ImmediatePriority=1,e.unstable_LowPriority=4,e.unstable_NormalPriority=3,e.unstable_Profiling=null,e.unstable_UserBlockingPriority=2,e.unstable_cancelCallback=function(V){V.callback=null},e.unstable_forceFrameRate=function(V){0>V||125<V?console.error("forceFrameRate takes a positive int between 0 and 125, forcing frame rates higher than 125 fps is not supported"):Z=0<V?Math.floor(1e3/V):5},e.unstable_getCurrentPriorityLevel=function(){return b},e.unstable_next=function(V){switch(b){case 1:case 2:case 3:var me=3;break;default:me=b}var pe=b;b=me;try{return V()}finally{b=pe}},e.unstable_requestPaint=function(){C=!0},e.unstable_runWithPriority=function(V,me){switch(V){case 1:case 2:case 3:case 4:case 5:break;default:V=3}var pe=b;b=V;try{return me()}finally{b=pe}},e.unstable_scheduleCallback=function(V,me,pe){var Be=e.unstable_now();switch(typeof pe=="object"&&pe!==null?(pe=pe.delay,pe=typeof pe=="number"&&0<pe?Be+pe:Be):pe=Be,V){case 1:var k=-1;break;case 2:k=250;break;case 5:k=1073741823;break;case 4:k=1e4;break;default:k=5e3}return k=pe+k,V={id:m++,callback:me,priorityLevel:V,startTime:pe,expirationTime:k,sortIndex:-1},pe>Be?(V.sortIndex=pe,t(f,V),n(d)===null&&V===n(f)&&(x?(O(ee),ee=-1):x=!0,de(H,pe-Be))):(V.sortIndex=k,t(d,V),T||y||(T=!0,B||(B=!0,oe()))),V},e.unstable_shouldYield=G,e.unstable_wrapCallback=function(V){var me=b;return function(){var pe=b;b=me;try{return V.apply(this,arguments)}finally{b=pe}}}}(f1)),f1}var qy;function NA(){return qy||(qy=1,h1.exports=CA()),h1.exports}var m1={exports:{}},Sr={};/**
 * @license React
 * react-dom.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var jy;function OA(){if(jy)return Sr;jy=1;var e=If();function t(d){var f="https://react.dev/errors/"+d;if(1<arguments.length){f+="?args[]="+encodeURIComponent(arguments[1]);for(var m=2;m<arguments.length;m++)f+="&args[]="+encodeURIComponent(arguments[m])}return"Minified React error #"+d+"; visit "+f+" for the full message or use the non-minified dev environment for full errors and additional helpful warnings."}function n(){}var r={d:{f:n,r:function(){throw Error(t(522))},D:n,C:n,L:n,m:n,X:n,S:n,M:n},p:0,findDOMNode:null},a=Symbol.for("react.portal");function s(d,f,m){var p=3<arguments.length&&arguments[3]!==void 0?arguments[3]:null;return{$$typeof:a,key:p==null?null:""+p,children:d,containerInfo:f,implementation:m}}var o=e.__CLIENT_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE;function c(d,f){if(d==="font")return"";if(typeof f=="string")return f==="use-credentials"?f:""}return Sr.__DOM_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE=r,Sr.createPortal=function(d,f){var m=2<arguments.length&&arguments[2]!==void 0?arguments[2]:null;if(!f||f.nodeType!==1&&f.nodeType!==9&&f.nodeType!==11)throw Error(t(299));return s(d,f,null,m)},Sr.flushSync=function(d){var f=o.T,m=r.p;try{if(o.T=null,r.p=2,d)return d()}finally{o.T=f,r.p=m,r.d.f()}},Sr.preconnect=function(d,f){typeof d=="string"&&(f?(f=f.crossOrigin,f=typeof f=="string"?f==="use-credentials"?f:"":void 0):f=null,r.d.C(d,f))},Sr.prefetchDNS=function(d){typeof d=="string"&&r.d.D(d)},Sr.preinit=function(d,f){if(typeof d=="string"&&f&&typeof f.as=="string"){var m=f.as,p=c(m,f.crossOrigin),b=typeof f.integrity=="string"?f.integrity:void 0,y=typeof f.fetchPriority=="string"?f.fetchPriority:void 0;m==="style"?r.d.S(d,typeof f.precedence=="string"?f.precedence:void 0,{crossOrigin:p,integrity:b,fetchPriority:y}):m==="script"&&r.d.X(d,{crossOrigin:p,integrity:b,fetchPriority:y,nonce:typeof f.nonce=="string"?f.nonce:void 0})}},Sr.preinitModule=function(d,f){if(typeof d=="string")if(typeof f=="object"&&f!==null){if(f.as==null||f.as==="script"){var m=c(f.as,f.crossOrigin);r.d.M(d,{crossOrigin:m,integrity:typeof f.integrity=="string"?f.integrity:void 0,nonce:typeof f.nonce=="string"?f.nonce:void 0})}}else f==null&&r.d.M(d)},Sr.preload=function(d,f){if(typeof d=="string"&&typeof f=="object"&&f!==null&&typeof f.as=="string"){var m=f.as,p=c(m,f.crossOrigin);r.d.L(d,m,{crossOrigin:p,integrity:typeof f.integrity=="string"?f.integrity:void 0,nonce:typeof f.nonce=="string"?f.nonce:void 0,type:typeof f.type=="string"?f.type:void 0,fetchPriority:typeof f.fetchPriority=="string"?f.fetchPriority:void 0,referrerPolicy:typeof f.referrerPolicy=="string"?f.referrerPolicy:void 0,imageSrcSet:typeof f.imageSrcSet=="string"?f.imageSrcSet:void 0,imageSizes:typeof f.imageSizes=="string"?f.imageSizes:void 0,media:typeof f.media=="string"?f.media:void 0})}},Sr.preloadModule=function(d,f){if(typeof d=="string")if(f){var m=c(f.as,f.crossOrigin);r.d.m(d,{as:typeof f.as=="string"&&f.as!=="script"?f.as:void 0,crossOrigin:m,integrity:typeof f.integrity=="string"?f.integrity:void 0})}else r.d.m(d)},Sr.requestFormReset=function(d){r.d.r(d)},Sr.unstable_batchedUpdates=function(d,f){return d(f)},Sr.useFormState=function(d,f,m){return o.H.useFormState(d,f,m)},Sr.useFormStatus=function(){return o.H.useHostTransitionStatus()},Sr.version="19.1.0",Sr}var $y;function H5(){if($y)return m1.exports;$y=1;function e(){if(!(typeof __REACT_DEVTOOLS_GLOBAL_HOOK__>"u"||typeof __REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE!="function"))try{__REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE(e)}catch(t){console.error(t)}}return e(),m1.exports=OA(),m1.exports}/**
 * @license React
 * react-dom-client.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var Yy;function RA(){if(Yy)return Tc;Yy=1;var e=NA(),t=If(),n=H5();function r(i){var l="https://react.dev/errors/"+i;if(1<arguments.length){l+="?args[]="+encodeURIComponent(arguments[1]);for(var u=2;u<arguments.length;u++)l+="&args[]="+encodeURIComponent(arguments[u])}return"Minified React error #"+i+"; visit "+l+" for the full message or use the non-minified dev environment for full errors and additional helpful warnings."}function a(i){return!(!i||i.nodeType!==1&&i.nodeType!==9&&i.nodeType!==11)}function s(i){var l=i,u=i;if(i.alternate)for(;l.return;)l=l.return;else{i=l;do l=i,(l.flags&4098)!==0&&(u=l.return),i=l.return;while(i)}return l.tag===3?u:null}function o(i){if(i.tag===13){var l=i.memoizedState;if(l===null&&(i=i.alternate,i!==null&&(l=i.memoizedState)),l!==null)return l.dehydrated}return null}function c(i){if(s(i)!==i)throw Error(r(188))}function d(i){var l=i.alternate;if(!l){if(l=s(i),l===null)throw Error(r(188));return l!==i?null:i}for(var u=i,h=l;;){var g=u.return;if(g===null)break;var v=g.alternate;if(v===null){if(h=g.return,h!==null){u=h;continue}break}if(g.child===v.child){for(v=g.child;v;){if(v===u)return c(g),i;if(v===h)return c(g),l;v=v.sibling}throw Error(r(188))}if(u.return!==h.return)u=g,h=v;else{for(var A=!1,M=g.child;M;){if(M===u){A=!0,u=g,h=v;break}if(M===h){A=!0,h=g,u=v;break}M=M.sibling}if(!A){for(M=v.child;M;){if(M===u){A=!0,u=v,h=g;break}if(M===h){A=!0,h=v,u=g;break}M=M.sibling}if(!A)throw Error(r(189))}}if(u.alternate!==h)throw Error(r(190))}if(u.tag!==3)throw Error(r(188));return u.stateNode.current===u?i:l}function f(i){var l=i.tag;if(l===5||l===26||l===27||l===6)return i;for(i=i.child;i!==null;){if(l=f(i),l!==null)return l;i=i.sibling}return null}var m=Object.assign,p=Symbol.for("react.element"),b=Symbol.for("react.transitional.element"),y=Symbol.for("react.portal"),T=Symbol.for("react.fragment"),x=Symbol.for("react.strict_mode"),C=Symbol.for("react.profiler"),w=Symbol.for("react.provider"),O=Symbol.for("react.consumer"),D=Symbol.for("react.context"),Y=Symbol.for("react.forward_ref"),H=Symbol.for("react.suspense"),B=Symbol.for("react.suspense_list"),ee=Symbol.for("react.memo"),Z=Symbol.for("react.lazy"),Q=Symbol.for("react.activity"),G=Symbol.for("react.memo_cache_sentinel"),se=Symbol.iterator;function oe(i){return i===null||typeof i!="object"?null:(i=se&&i[se]||i["@@iterator"],typeof i=="function"?i:null)}var ye=Symbol.for("react.client.reference");function fe(i){if(i==null)return null;if(typeof i=="function")return i.$$typeof===ye?null:i.displayName||i.name||null;if(typeof i=="string")return i;switch(i){case T:return"Fragment";case C:return"Profiler";case x:return"StrictMode";case H:return"Suspense";case B:return"SuspenseList";case Q:return"Activity"}if(typeof i=="object")switch(i.$$typeof){case y:return"Portal";case D:return(i.displayName||"Context")+".Provider";case O:return(i._context.displayName||"Context")+".Consumer";case Y:var l=i.render;return i=i.displayName,i||(i=l.displayName||l.name||"",i=i!==""?"ForwardRef("+i+")":"ForwardRef"),i;case ee:return l=i.displayName||null,l!==null?l:fe(i.type)||"Memo";case Z:l=i._payload,i=i._init;try{return fe(i(l))}catch{}}return null}var de=Array.isArray,V=t.__CLIENT_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE,me=n.__DOM_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE,pe={pending:!1,data:null,method:null,action:null},Be=[],k=-1;function ie(i){return{current:i}}function ge(i){0>k||(i.current=Be[k],Be[k]=null,k--)}function L(i,l){k++,Be[k]=i.current,i.current=l}var Oe=ie(null),je=ie(null),De=ie(null),Ot=ie(null);function ot(i,l){switch(L(De,l),L(je,i),L(Oe,null),l.nodeType){case 9:case 11:i=(i=l.documentElement)&&(i=i.namespaceURI)?dy(i):0;break;default:if(i=l.tagName,l=l.namespaceURI)l=dy(l),i=hy(l,i);else switch(i){case"svg":i=1;break;case"math":i=2;break;default:i=0}}ge(Oe),L(Oe,i)}function zt(){ge(Oe),ge(je),ge(De)}function wt(i){i.memoizedState!==null&&L(Ot,i);var l=Oe.current,u=hy(l,i.type);l!==u&&(L(je,i),L(Oe,u))}function vn(i){je.current===i&&(ge(Oe),ge(je)),Ot.current===i&&(ge(Ot),gc._currentValue=pe)}var mr=Object.prototype.hasOwnProperty,vt=e.unstable_scheduleCallback,gt=e.unstable_cancelCallback,Bt=e.unstable_shouldYield,Je=e.unstable_requestPaint,yt=e.unstable_now,mn=e.unstable_getCurrentPriorityLevel,Lt=e.unstable_ImmediatePriority,ln=e.unstable_UserBlockingPriority,We=e.unstable_NormalPriority,on=e.unstable_LowPriority,qn=e.unstable_IdlePriority,Ln=e.log,nn=e.unstable_setDisableYieldValue,he=null,Se=null;function Ue(i){if(typeof Ln=="function"&&nn(i),Se&&typeof Se.setStrictMode=="function")try{Se.setStrictMode(he,i)}catch{}}var ze=Math.clz32?Math.clz32:$r,tt=Math.log,er=Math.LN2;function $r(i){return i>>>=0,i===0?32:31-(tt(i)/er|0)|0}var Pn=256,tr=4194304;function Or(i){var l=i&42;if(l!==0)return l;switch(i&-i){case 1:return 1;case 2:return 2;case 4:return 4;case 8:return 8;case 16:return 16;case 32:return 32;case 64:return 64;case 128:return 128;case 256:case 512:case 1024:case 2048:case 4096:case 8192:case 16384:case 32768:case 65536:case 131072:case 262144:case 524288:case 1048576:case 2097152:return i&4194048;case 4194304:case 8388608:case 16777216:case 33554432:return i&62914560;case 67108864:return 67108864;case 134217728:return 134217728;case 268435456:return 268435456;case 536870912:return 536870912;case 1073741824:return 0;default:return i}}function jn(i,l,u){var h=i.pendingLanes;if(h===0)return 0;var g=0,v=i.suspendedLanes,A=i.pingedLanes;i=i.warmLanes;var M=h&134217727;return M!==0?(h=M&~v,h!==0?g=Or(h):(A&=M,A!==0?g=Or(A):u||(u=M&~i,u!==0&&(g=Or(u))))):(M=h&~v,M!==0?g=Or(M):A!==0?g=Or(A):u||(u=h&~i,u!==0&&(g=Or(u)))),g===0?0:l!==0&&l!==g&&(l&v)===0&&(v=g&-g,u=l&-l,v>=u||v===32&&(u&4194048)!==0)?l:g}function Yr(i,l){return(i.pendingLanes&~(i.suspendedLanes&~i.pingedLanes)&l)===0}function Rr(i,l){switch(i){case 1:case 2:case 4:case 8:case 64:return l+250;case 16:case 32:case 128:case 256:case 512:case 1024:case 2048:case 4096:case 8192:case 16384:case 32768:case 65536:case 131072:case 262144:case 524288:case 1048576:case 2097152:return l+5e3;case 4194304:case 8388608:case 16777216:case 33554432:return-1;case 67108864:case 134217728:case 268435456:case 536870912:case 1073741824:return-1;default:return-1}}function Tu(){var i=Pn;return Pn<<=1,(Pn&4194048)===0&&(Pn=256),i}function xu(){var i=tr;return tr<<=1,(tr&62914560)===0&&(tr=4194304),i}function Su(i){for(var l=[],u=0;31>u;u++)l.push(i);return l}function hs(i,l){i.pendingLanes|=l,l!==268435456&&(i.suspendedLanes=0,i.pingedLanes=0,i.warmLanes=0)}function fs(i,l,u,h,g,v){var A=i.pendingLanes;i.pendingLanes=u,i.suspendedLanes=0,i.pingedLanes=0,i.warmLanes=0,i.expiredLanes&=u,i.entangledLanes&=u,i.errorRecoveryDisabledLanes&=u,i.shellSuspendCounter=0;var M=i.entanglements,$=i.expirationTimes,re=i.hiddenUpdates;for(u=A&~u;0<u;){var _e=31-ze(u),xe=1<<_e;M[_e]=0,$[_e]=-1;var le=re[_e];if(le!==null)for(re[_e]=null,_e=0;_e<le.length;_e++){var ue=le[_e];ue!==null&&(ue.lane&=-536870913)}u&=~xe}h!==0&&K(i,h,0),v!==0&&g===0&&i.tag!==0&&(i.suspendedLanes|=v&~(A&~l))}function K(i,l,u){i.pendingLanes|=l,i.suspendedLanes&=~l;var h=31-ze(l);i.entangledLanes|=l,i.entanglements[h]=i.entanglements[h]|1073741824|u&4194090}function Ee(i,l){var u=i.entangledLanes|=l;for(i=i.entanglements;u;){var h=31-ze(u),g=1<<h;g&l|i[h]&l&&(i[h]|=l),u&=~g}}function Fe(i){switch(i){case 2:i=1;break;case 8:i=4;break;case 32:i=16;break;case 256:case 512:case 1024:case 2048:case 4096:case 8192:case 16384:case 32768:case 65536:case 131072:case 262144:case 524288:case 1048576:case 2097152:case 4194304:case 8388608:case 16777216:case 33554432:i=128;break;case 268435456:i=134217728;break;default:i=0}return i}function ht(i){return i&=-i,2<i?8<i?(i&134217727)!==0?32:268435456:8:2}function yn(){var i=me.p;return i!==0?i:(i=window.event,i===void 0?32:Ry(i.type))}function wn(i,l){var u=me.p;try{return me.p=i,l()}finally{me.p=u}}var Ve=Math.random().toString(36).slice(2),Pe="__reactFiber$"+Ve,et="__reactProps$"+Ve,pn="__reactContainer$"+Ve,_n="__reactEvents$"+Ve,cl="__reactListeners$"+Ve,wu="__reactHandles$"+Ve,no="__reactResources$"+Ve,dl="__reactMarker$"+Ve;function hl(i){delete i[Pe],delete i[et],delete i[_n],delete i[cl],delete i[wu]}function ms(i){var l=i[Pe];if(l)return l;for(var u=i.parentNode;u;){if(l=u[pn]||u[Pe]){if(u=l.alternate,l.child!==null||u!==null&&u.child!==null)for(i=gy(i);i!==null;){if(u=i[Pe])return u;i=gy(i)}return l}i=u,u=i.parentNode}return null}function ps(i){if(i=i[Pe]||i[pn]){var l=i.tag;if(l===5||l===6||l===13||l===26||l===27||l===3)return i}return null}function fl(i){var l=i.tag;if(l===5||l===26||l===27||l===6)return i.stateNode;throw Error(r(33))}function Si(i){var l=i[no];return l||(l=i[no]={hoistableStyles:new Map,hoistableScripts:new Map}),l}function An(i){i[dl]=!0}var $0=new Set,Y0={};function wi(i,l){Mr(i,l),Mr(i+"Capture",l)}function Mr(i,l){for(Y0[i]=l,i=0;i<l.length;i++)$0.add(l[i])}var G0=RegExp("^[:A-Z_a-z\\u00C0-\\u00D6\\u00D8-\\u00F6\\u00F8-\\u02FF\\u0370-\\u037D\\u037F-\\u1FFF\\u200C-\\u200D\\u2070-\\u218F\\u2C00-\\u2FEF\\u3001-\\uD7FF\\uF900-\\uFDCF\\uFDF0-\\uFFFD][:A-Z_a-z\\u00C0-\\u00D6\\u00D8-\\u00F6\\u00F8-\\u02FF\\u0370-\\u037D\\u037F-\\u1FFF\\u200C-\\u200D\\u2070-\\u218F\\u2C00-\\u2FEF\\u3001-\\uD7FF\\uF900-\\uFDCF\\uFDF0-\\uFFFD\\-.0-9\\u00B7\\u0300-\\u036F\\u203F-\\u2040]*$"),Au={},V0={};function cm(i){return mr.call(V0,i)?!0:mr.call(Au,i)?!1:G0.test(i)?V0[i]=!0:(Au[i]=!0,!1)}function ro(i,l,u){if(cm(l))if(u===null)i.removeAttribute(l);else{switch(typeof u){case"undefined":case"function":case"symbol":i.removeAttribute(l);return;case"boolean":var h=l.toLowerCase().slice(0,5);if(h!=="data-"&&h!=="aria-"){i.removeAttribute(l);return}}i.setAttribute(l,""+u)}}function Ai(i,l,u){if(u===null)i.removeAttribute(l);else{switch(typeof u){case"undefined":case"function":case"symbol":case"boolean":i.removeAttribute(l);return}i.setAttribute(l,""+u)}}function Ha(i,l,u,h){if(h===null)i.removeAttribute(u);else{switch(typeof h){case"undefined":case"function":case"symbol":case"boolean":i.removeAttribute(u);return}i.setAttributeNS(l,u,""+h)}}var Re,rt;function _t(i){if(Re===void 0)try{throw Error()}catch(u){var l=u.stack.trim().match(/\n( *(at )?)/);Re=l&&l[1]||"",rt=-1<u.stack.indexOf(`
    at`)?" (<anonymous>)":-1<u.stack.indexOf("@")?"@unknown:0:0":""}return`
`+Re+i+rt}var $t=!1;function kn(i,l){if(!i||$t)return"";$t=!0;var u=Error.prepareStackTrace;Error.prepareStackTrace=void 0;try{var h={DetermineComponentFrameRoot:function(){try{if(l){var xe=function(){throw Error()};if(Object.defineProperty(xe.prototype,"props",{set:function(){throw Error()}}),typeof Reflect=="object"&&Reflect.construct){try{Reflect.construct(xe,[])}catch(ue){var le=ue}Reflect.construct(i,[],xe)}else{try{xe.call()}catch(ue){le=ue}i.call(xe.prototype)}}else{try{throw Error()}catch(ue){le=ue}(xe=i())&&typeof xe.catch=="function"&&xe.catch(function(){})}}catch(ue){if(ue&&le&&typeof ue.stack=="string")return[ue.stack,le.stack]}return[null,null]}};h.DetermineComponentFrameRoot.displayName="DetermineComponentFrameRoot";var g=Object.getOwnPropertyDescriptor(h.DetermineComponentFrameRoot,"name");g&&g.configurable&&Object.defineProperty(h.DetermineComponentFrameRoot,"name",{value:"DetermineComponentFrameRoot"});var v=h.DetermineComponentFrameRoot(),A=v[0],M=v[1];if(A&&M){var $=A.split(`
`),re=M.split(`
`);for(g=h=0;h<$.length&&!$[h].includes("DetermineComponentFrameRoot");)h++;for(;g<re.length&&!re[g].includes("DetermineComponentFrameRoot");)g++;if(h===$.length||g===re.length)for(h=$.length-1,g=re.length-1;1<=h&&0<=g&&$[h]!==re[g];)g--;for(;1<=h&&0<=g;h--,g--)if($[h]!==re[g]){if(h!==1||g!==1)do if(h--,g--,0>g||$[h]!==re[g]){var _e=`
`+$[h].replace(" at new "," at ");return i.displayName&&_e.includes("<anonymous>")&&(_e=_e.replace("<anonymous>",i.displayName)),_e}while(1<=h&&0<=g);break}}}finally{$t=!1,Error.prepareStackTrace=u}return(u=i?i.displayName||i.name:"")?_t(u):""}function ia(i){switch(i.tag){case 26:case 27:case 5:return _t(i.type);case 16:return _t("Lazy");case 13:return _t("Suspense");case 19:return _t("SuspenseList");case 0:case 15:return kn(i.type,!1);case 11:return kn(i.type.render,!1);case 1:return kn(i.type,!0);case 31:return _t("Activity");default:return""}}function qa(i){try{var l="";do l+=ia(i),i=i.return;while(i);return l}catch(u){return`
Error generating stack: `+u.message+`
`+u.stack}}function pr(i){switch(typeof i){case"bigint":case"boolean":case"number":case"string":case"undefined":return i;case"object":return i;default:return""}}function Tr(i){var l=i.type;return(i=i.nodeName)&&i.toLowerCase()==="input"&&(l==="checkbox"||l==="radio")}function Ta(i){var l=Tr(i)?"checked":"value",u=Object.getOwnPropertyDescriptor(i.constructor.prototype,l),h=""+i[l];if(!i.hasOwnProperty(l)&&typeof u<"u"&&typeof u.get=="function"&&typeof u.set=="function"){var g=u.get,v=u.set;return Object.defineProperty(i,l,{configurable:!0,get:function(){return g.call(this)},set:function(A){h=""+A,v.call(this,A)}}),Object.defineProperty(i,l,{enumerable:u.enumerable}),{getValue:function(){return h},setValue:function(A){h=""+A},stopTracking:function(){i._valueTracker=null,delete i[l]}}}}function ml(i){i._valueTracker||(i._valueTracker=Ta(i))}function ku(i){if(!i)return!1;var l=i._valueTracker;if(!l)return!0;var u=l.getValue(),h="";return i&&(h=Tr(i)?i.checked?"true":"false":i.value),i=h,i!==u?(l.setValue(i),!0):!1}function pl(i){if(i=i||(typeof document<"u"?document:void 0),typeof i>"u")return null;try{return i.activeElement||i.body}catch{return i.body}}var dm=/[\n"\\]/g;function Gr(i){return i.replace(dm,function(l){return"\\"+l.charCodeAt(0).toString(16)+" "})}function Cu(i,l,u,h,g,v,A,M){i.name="",A!=null&&typeof A!="function"&&typeof A!="symbol"&&typeof A!="boolean"?i.type=A:i.removeAttribute("type"),l!=null?A==="number"?(l===0&&i.value===""||i.value!=l)&&(i.value=""+pr(l)):i.value!==""+pr(l)&&(i.value=""+pr(l)):A!=="submit"&&A!=="reset"||i.removeAttribute("value"),l!=null?gs(i,A,pr(l)):u!=null?gs(i,A,pr(u)):h!=null&&i.removeAttribute("value"),g==null&&v!=null&&(i.defaultChecked=!!v),g!=null&&(i.checked=g&&typeof g!="function"&&typeof g!="symbol"),M!=null&&typeof M!="function"&&typeof M!="symbol"&&typeof M!="boolean"?i.name=""+pr(M):i.removeAttribute("name")}function K0(i,l,u,h,g,v,A,M){if(v!=null&&typeof v!="function"&&typeof v!="symbol"&&typeof v!="boolean"&&(i.type=v),l!=null||u!=null){if(!(v!=="submit"&&v!=="reset"||l!=null))return;u=u!=null?""+pr(u):"",l=l!=null?""+pr(l):u,M||l===i.value||(i.value=l),i.defaultValue=l}h=h??g,h=typeof h!="function"&&typeof h!="symbol"&&!!h,i.checked=M?i.checked:!!h,i.defaultChecked=!!h,A!=null&&typeof A!="function"&&typeof A!="symbol"&&typeof A!="boolean"&&(i.name=A)}function gs(i,l,u){l==="number"&&pl(i.ownerDocument)===i||i.defaultValue===""+u||(i.defaultValue=""+u)}function ki(i,l,u,h){if(i=i.options,l){l={};for(var g=0;g<u.length;g++)l["$"+u[g]]=!0;for(u=0;u<i.length;u++)g=l.hasOwnProperty("$"+i[u].value),i[u].selected!==g&&(i[u].selected=g),g&&h&&(i[u].defaultSelected=!0)}else{for(u=""+pr(u),l=null,g=0;g<i.length;g++){if(i[g].value===u){i[g].selected=!0,h&&(i[g].defaultSelected=!0);return}l!==null||i[g].disabled||(l=i[g])}l!==null&&(l.selected=!0)}}function sa(i,l,u){if(l!=null&&(l=""+pr(l),l!==i.value&&(i.value=l),u==null)){i.defaultValue!==l&&(i.defaultValue=l);return}i.defaultValue=u!=null?""+pr(u):""}function W0(i,l,u,h){if(l==null){if(h!=null){if(u!=null)throw Error(r(92));if(de(h)){if(1<h.length)throw Error(r(93));h=h[0]}u=h}u==null&&(u=""),l=u}u=pr(l),i.defaultValue=u,h=i.textContent,h===u&&h!==""&&h!==null&&(i.value=h)}function ei(i,l){if(l){var u=i.firstChild;if(u&&u===i.lastChild&&u.nodeType===3){u.nodeValue=l;return}}i.textContent=l}var bt=new Set("animationIterationCount aspectRatio borderImageOutset borderImageSlice borderImageWidth boxFlex boxFlexGroup boxOrdinalGroup columnCount columns flex flexGrow flexPositive flexShrink flexNegative flexOrder gridArea gridRow gridRowEnd gridRowSpan gridRowStart gridColumn gridColumnEnd gridColumnSpan gridColumnStart fontWeight lineClamp lineHeight opacity order orphans scale tabSize widows zIndex zoom fillOpacity floodOpacity stopOpacity strokeDasharray strokeDashoffset strokeMiterlimit strokeOpacity strokeWidth MozAnimationIterationCount MozBoxFlex MozBoxFlexGroup MozLineClamp msAnimationIterationCount msFlex msZoom msFlexGrow msFlexNegative msFlexOrder msFlexPositive msFlexShrink msGridColumn msGridColumnSpan msGridRow msGridRowSpan WebkitAnimationIterationCount WebkitBoxFlex WebKitBoxFlexGroup WebkitBoxOrdinalGroup WebkitColumnCount WebkitColumns WebkitFlex WebkitFlexGrow WebkitFlexPositive WebkitFlexShrink WebkitLineClamp".split(" "));function Nu(i,l,u){var h=l.indexOf("--")===0;u==null||typeof u=="boolean"||u===""?h?i.setProperty(l,""):l==="float"?i.cssFloat="":i[l]="":h?i.setProperty(l,u):typeof u!="number"||u===0||bt.has(l)?l==="float"?i.cssFloat=u:i[l]=(""+u).trim():i[l]=u+"px"}function $n(i,l,u){if(l!=null&&typeof l!="object")throw Error(r(62));if(i=i.style,u!=null){for(var h in u)!u.hasOwnProperty(h)||l!=null&&l.hasOwnProperty(h)||(h.indexOf("--")===0?i.setProperty(h,""):h==="float"?i.cssFloat="":i[h]="");for(var g in l)h=l[g],l.hasOwnProperty(g)&&u[g]!==h&&Nu(i,g,h)}else for(var v in l)l.hasOwnProperty(v)&&Nu(i,v,l[v])}function Yt(i){if(i.indexOf("-")===-1)return!1;switch(i){case"annotation-xml":case"color-profile":case"font-face":case"font-face-src":case"font-face-uri":case"font-face-format":case"font-face-name":case"missing-glyph":return!1;default:return!0}}var ao=new Map([["acceptCharset","accept-charset"],["htmlFor","for"],["httpEquiv","http-equiv"],["crossOrigin","crossorigin"],["accentHeight","accent-height"],["alignmentBaseline","alignment-baseline"],["arabicForm","arabic-form"],["baselineShift","baseline-shift"],["capHeight","cap-height"],["clipPath","clip-path"],["clipRule","clip-rule"],["colorInterpolation","color-interpolation"],["colorInterpolationFilters","color-interpolation-filters"],["colorProfile","color-profile"],["colorRendering","color-rendering"],["dominantBaseline","dominant-baseline"],["enableBackground","enable-background"],["fillOpacity","fill-opacity"],["fillRule","fill-rule"],["floodColor","flood-color"],["floodOpacity","flood-opacity"],["fontFamily","font-family"],["fontSize","font-size"],["fontSizeAdjust","font-size-adjust"],["fontStretch","font-stretch"],["fontStyle","font-style"],["fontVariant","font-variant"],["fontWeight","font-weight"],["glyphName","glyph-name"],["glyphOrientationHorizontal","glyph-orientation-horizontal"],["glyphOrientationVertical","glyph-orientation-vertical"],["horizAdvX","horiz-adv-x"],["horizOriginX","horiz-origin-x"],["imageRendering","image-rendering"],["letterSpacing","letter-spacing"],["lightingColor","lighting-color"],["markerEnd","marker-end"],["markerMid","marker-mid"],["markerStart","marker-start"],["overlinePosition","overline-position"],["overlineThickness","overline-thickness"],["paintOrder","paint-order"],["panose-1","panose-1"],["pointerEvents","pointer-events"],["renderingIntent","rendering-intent"],["shapeRendering","shape-rendering"],["stopColor","stop-color"],["stopOpacity","stop-opacity"],["strikethroughPosition","strikethrough-position"],["strikethroughThickness","strikethrough-thickness"],["strokeDasharray","stroke-dasharray"],["strokeDashoffset","stroke-dashoffset"],["strokeLinecap","stroke-linecap"],["strokeLinejoin","stroke-linejoin"],["strokeMiterlimit","stroke-miterlimit"],["strokeOpacity","stroke-opacity"],["strokeWidth","stroke-width"],["textAnchor","text-anchor"],["textDecoration","text-decoration"],["textRendering","text-rendering"],["transformOrigin","transform-origin"],["underlinePosition","underline-position"],["underlineThickness","underline-thickness"],["unicodeBidi","unicode-bidi"],["unicodeRange","unicode-range"],["unitsPerEm","units-per-em"],["vAlphabetic","v-alphabetic"],["vHanging","v-hanging"],["vIdeographic","v-ideographic"],["vMathematical","v-mathematical"],["vectorEffect","vector-effect"],["vertAdvY","vert-adv-y"],["vertOriginX","vert-origin-x"],["vertOriginY","vert-origin-y"],["wordSpacing","word-spacing"],["writingMode","writing-mode"],["xmlnsXlink","xmlns:xlink"],["xHeight","x-height"]]),Ci=/^[\u0000-\u001F ]*j[\r\n\t]*a[\r\n\t]*v[\r\n\t]*a[\r\n\t]*s[\r\n\t]*c[\r\n\t]*r[\r\n\t]*i[\r\n\t]*p[\r\n\t]*t[\r\n\t]*:/i;function bs(i){return Ci.test(""+i)?"javascript:throw new Error('React has blocked a javascript: URL as a security precaution.')":i}var gl=null;function He(i){return i=i.target||i.srcElement||window,i.correspondingUseElement&&(i=i.correspondingUseElement),i.nodeType===3?i.parentNode:i}var Qe=null,pt=null;function Ht(i){var l=ps(i);if(l&&(i=l.stateNode)){var u=i[et]||null;e:switch(i=l.stateNode,l.type){case"input":if(Cu(i,u.value,u.defaultValue,u.defaultValue,u.checked,u.defaultChecked,u.type,u.name),l=u.name,u.type==="radio"&&l!=null){for(u=i;u.parentNode;)u=u.parentNode;for(u=u.querySelectorAll('input[name="'+Gr(""+l)+'"][type="radio"]'),l=0;l<u.length;l++){var h=u[l];if(h!==i&&h.form===i.form){var g=h[et]||null;if(!g)throw Error(r(90));Cu(h,g.value,g.defaultValue,g.defaultValue,g.checked,g.defaultChecked,g.type,g.name)}}for(l=0;l<u.length;l++)h=u[l],h.form===i.form&&ku(h)}break e;case"textarea":sa(i,u.value,u.defaultValue);break e;case"select":l=u.value,l!=null&&ki(i,!!u.multiple,l,!1)}}}var Yn=!1;function xa(i,l,u){if(Yn)return i(l,u);Yn=!0;try{var h=i(l);return h}finally{if(Yn=!1,(Qe!==null||pt!==null)&&(Dd(),Qe&&(l=Qe,i=pt,pt=Qe=null,Ht(l),i)))for(l=0;l<i.length;l++)Ht(i[l])}}function bl(i,l){var u=i.stateNode;if(u===null)return null;var h=u[et]||null;if(h===null)return null;u=h[l];e:switch(l){case"onClick":case"onClickCapture":case"onDoubleClick":case"onDoubleClickCapture":case"onMouseDown":case"onMouseDownCapture":case"onMouseMove":case"onMouseMoveCapture":case"onMouseUp":case"onMouseUpCapture":case"onMouseEnter":(h=!h.disabled)||(i=i.type,h=!(i==="button"||i==="input"||i==="select"||i==="textarea")),i=!h;break e;default:i=!1}if(i)return null;if(u&&typeof u!="function")throw Error(r(231,l,typeof u));return u}var ja=!(typeof window>"u"||typeof window.document>"u"||typeof window.document.createElement>"u"),hm=!1;if(ja)try{var Ou={};Object.defineProperty(Ou,"passive",{get:function(){hm=!0}}),window.addEventListener("test",Ou,Ou),window.removeEventListener("test",Ou,Ou)}catch{hm=!1}var vs=null,fm=null,X0=null;function pv(){if(X0)return X0;var i,l=fm,u=l.length,h,g="value"in vs?vs.value:vs.textContent,v=g.length;for(i=0;i<u&&l[i]===g[i];i++);var A=u-i;for(h=1;h<=A&&l[u-h]===g[v-h];h++);return X0=g.slice(i,1<h?1-h:void 0)}function Q0(i){var l=i.keyCode;return"charCode"in i?(i=i.charCode,i===0&&l===13&&(i=13)):i=l,i===10&&(i=13),32<=i||i===13?i:0}function Z0(){return!0}function gv(){return!1}function Vr(i){function l(u,h,g,v,A){this._reactName=u,this._targetInst=g,this.type=h,this.nativeEvent=v,this.target=A,this.currentTarget=null;for(var M in i)i.hasOwnProperty(M)&&(u=i[M],this[M]=u?u(v):v[M]);return this.isDefaultPrevented=(v.defaultPrevented!=null?v.defaultPrevented:v.returnValue===!1)?Z0:gv,this.isPropagationStopped=gv,this}return m(l.prototype,{preventDefault:function(){this.defaultPrevented=!0;var u=this.nativeEvent;u&&(u.preventDefault?u.preventDefault():typeof u.returnValue!="unknown"&&(u.returnValue=!1),this.isDefaultPrevented=Z0)},stopPropagation:function(){var u=this.nativeEvent;u&&(u.stopPropagation?u.stopPropagation():typeof u.cancelBubble!="unknown"&&(u.cancelBubble=!0),this.isPropagationStopped=Z0)},persist:function(){},isPersistent:Z0}),l}var vl={eventPhase:0,bubbles:0,cancelable:0,timeStamp:function(i){return i.timeStamp||Date.now()},defaultPrevented:0,isTrusted:0},J0=Vr(vl),Ru=m({},vl,{view:0,detail:0}),x9=Vr(Ru),mm,pm,Mu,ed=m({},Ru,{screenX:0,screenY:0,clientX:0,clientY:0,pageX:0,pageY:0,ctrlKey:0,shiftKey:0,altKey:0,metaKey:0,getModifierState:bm,button:0,buttons:0,relatedTarget:function(i){return i.relatedTarget===void 0?i.fromElement===i.srcElement?i.toElement:i.fromElement:i.relatedTarget},movementX:function(i){return"movementX"in i?i.movementX:(i!==Mu&&(Mu&&i.type==="mousemove"?(mm=i.screenX-Mu.screenX,pm=i.screenY-Mu.screenY):pm=mm=0,Mu=i),mm)},movementY:function(i){return"movementY"in i?i.movementY:pm}}),bv=Vr(ed),S9=m({},ed,{dataTransfer:0}),w9=Vr(S9),A9=m({},Ru,{relatedTarget:0}),gm=Vr(A9),k9=m({},vl,{animationName:0,elapsedTime:0,pseudoElement:0}),C9=Vr(k9),N9=m({},vl,{clipboardData:function(i){return"clipboardData"in i?i.clipboardData:window.clipboardData}}),O9=Vr(N9),R9=m({},vl,{data:0}),vv=Vr(R9),M9={Esc:"Escape",Spacebar:" ",Left:"ArrowLeft",Up:"ArrowUp",Right:"ArrowRight",Down:"ArrowDown",Del:"Delete",Win:"OS",Menu:"ContextMenu",Apps:"ContextMenu",Scroll:"ScrollLock",MozPrintableKey:"Unidentified"},I9={8:"Backspace",9:"Tab",12:"Clear",13:"Enter",16:"Shift",17:"Control",18:"Alt",19:"Pause",20:"CapsLock",27:"Escape",32:" ",33:"PageUp",34:"PageDown",35:"End",36:"Home",37:"ArrowLeft",38:"ArrowUp",39:"ArrowRight",40:"ArrowDown",45:"Insert",46:"Delete",112:"F1",113:"F2",114:"F3",115:"F4",116:"F5",117:"F6",118:"F7",119:"F8",120:"F9",121:"F10",122:"F11",123:"F12",144:"NumLock",145:"ScrollLock",224:"Meta"},D9={Alt:"altKey",Control:"ctrlKey",Meta:"metaKey",Shift:"shiftKey"};function L9(i){var l=this.nativeEvent;return l.getModifierState?l.getModifierState(i):(i=D9[i])?!!l[i]:!1}function bm(){return L9}var P9=m({},Ru,{key:function(i){if(i.key){var l=M9[i.key]||i.key;if(l!=="Unidentified")return l}return i.type==="keypress"?(i=Q0(i),i===13?"Enter":String.fromCharCode(i)):i.type==="keydown"||i.type==="keyup"?I9[i.keyCode]||"Unidentified":""},code:0,location:0,ctrlKey:0,shiftKey:0,altKey:0,metaKey:0,repeat:0,locale:0,getModifierState:bm,charCode:function(i){return i.type==="keypress"?Q0(i):0},keyCode:function(i){return i.type==="keydown"||i.type==="keyup"?i.keyCode:0},which:function(i){return i.type==="keypress"?Q0(i):i.type==="keydown"||i.type==="keyup"?i.keyCode:0}}),B9=Vr(P9),U9=m({},ed,{pointerId:0,width:0,height:0,pressure:0,tangentialPressure:0,tiltX:0,tiltY:0,twist:0,pointerType:0,isPrimary:0}),yv=Vr(U9),F9=m({},Ru,{touches:0,targetTouches:0,changedTouches:0,altKey:0,metaKey:0,ctrlKey:0,shiftKey:0,getModifierState:bm}),z9=Vr(F9),H9=m({},vl,{propertyName:0,elapsedTime:0,pseudoElement:0}),q9=Vr(H9),j9=m({},ed,{deltaX:function(i){return"deltaX"in i?i.deltaX:"wheelDeltaX"in i?-i.wheelDeltaX:0},deltaY:function(i){return"deltaY"in i?i.deltaY:"wheelDeltaY"in i?-i.wheelDeltaY:"wheelDelta"in i?-i.wheelDelta:0},deltaZ:0,deltaMode:0}),$9=Vr(j9),Y9=m({},vl,{newState:0,oldState:0}),G9=Vr(Y9),V9=[9,13,27,32],vm=ja&&"CompositionEvent"in window,Iu=null;ja&&"documentMode"in document&&(Iu=document.documentMode);var K9=ja&&"TextEvent"in window&&!Iu,_v=ja&&(!vm||Iu&&8<Iu&&11>=Iu),Ev=" ",Tv=!1;function xv(i,l){switch(i){case"keyup":return V9.indexOf(l.keyCode)!==-1;case"keydown":return l.keyCode!==229;case"keypress":case"mousedown":case"focusout":return!0;default:return!1}}function Sv(i){return i=i.detail,typeof i=="object"&&"data"in i?i.data:null}var io=!1;function W9(i,l){switch(i){case"compositionend":return Sv(l);case"keypress":return l.which!==32?null:(Tv=!0,Ev);case"textInput":return i=l.data,i===Ev&&Tv?null:i;default:return null}}function X9(i,l){if(io)return i==="compositionend"||!vm&&xv(i,l)?(i=pv(),X0=fm=vs=null,io=!1,i):null;switch(i){case"paste":return null;case"keypress":if(!(l.ctrlKey||l.altKey||l.metaKey)||l.ctrlKey&&l.altKey){if(l.char&&1<l.char.length)return l.char;if(l.which)return String.fromCharCode(l.which)}return null;case"compositionend":return _v&&l.locale!=="ko"?null:l.data;default:return null}}var Q9={color:!0,date:!0,datetime:!0,"datetime-local":!0,email:!0,month:!0,number:!0,password:!0,range:!0,search:!0,tel:!0,text:!0,time:!0,url:!0,week:!0};function wv(i){var l=i&&i.nodeName&&i.nodeName.toLowerCase();return l==="input"?!!Q9[i.type]:l==="textarea"}function Av(i,l,u,h){Qe?pt?pt.push(h):pt=[h]:Qe=h,l=zd(l,"onChange"),0<l.length&&(u=new J0("onChange","change",null,u,h),i.push({event:u,listeners:l}))}var Du=null,Lu=null;function Z9(i){sy(i,0)}function td(i){var l=fl(i);if(ku(l))return i}function kv(i,l){if(i==="change")return l}var Cv=!1;if(ja){var ym;if(ja){var _m="oninput"in document;if(!_m){var Nv=document.createElement("div");Nv.setAttribute("oninput","return;"),_m=typeof Nv.oninput=="function"}ym=_m}else ym=!1;Cv=ym&&(!document.documentMode||9<document.documentMode)}function Ov(){Du&&(Du.detachEvent("onpropertychange",Rv),Lu=Du=null)}function Rv(i){if(i.propertyName==="value"&&td(Lu)){var l=[];Av(l,Lu,i,He(i)),xa(Z9,l)}}function J9(i,l,u){i==="focusin"?(Ov(),Du=l,Lu=u,Du.attachEvent("onpropertychange",Rv)):i==="focusout"&&Ov()}function ew(i){if(i==="selectionchange"||i==="keyup"||i==="keydown")return td(Lu)}function tw(i,l){if(i==="click")return td(l)}function nw(i,l){if(i==="input"||i==="change")return td(l)}function rw(i,l){return i===l&&(i!==0||1/i===1/l)||i!==i&&l!==l}var la=typeof Object.is=="function"?Object.is:rw;function Pu(i,l){if(la(i,l))return!0;if(typeof i!="object"||i===null||typeof l!="object"||l===null)return!1;var u=Object.keys(i),h=Object.keys(l);if(u.length!==h.length)return!1;for(h=0;h<u.length;h++){var g=u[h];if(!mr.call(l,g)||!la(i[g],l[g]))return!1}return!0}function Mv(i){for(;i&&i.firstChild;)i=i.firstChild;return i}function Iv(i,l){var u=Mv(i);i=0;for(var h;u;){if(u.nodeType===3){if(h=i+u.textContent.length,i<=l&&h>=l)return{node:u,offset:l-i};i=h}e:{for(;u;){if(u.nextSibling){u=u.nextSibling;break e}u=u.parentNode}u=void 0}u=Mv(u)}}function Dv(i,l){return i&&l?i===l?!0:i&&i.nodeType===3?!1:l&&l.nodeType===3?Dv(i,l.parentNode):"contains"in i?i.contains(l):i.compareDocumentPosition?!!(i.compareDocumentPosition(l)&16):!1:!1}function Lv(i){i=i!=null&&i.ownerDocument!=null&&i.ownerDocument.defaultView!=null?i.ownerDocument.defaultView:window;for(var l=pl(i.document);l instanceof i.HTMLIFrameElement;){try{var u=typeof l.contentWindow.location.href=="string"}catch{u=!1}if(u)i=l.contentWindow;else break;l=pl(i.document)}return l}function Em(i){var l=i&&i.nodeName&&i.nodeName.toLowerCase();return l&&(l==="input"&&(i.type==="text"||i.type==="search"||i.type==="tel"||i.type==="url"||i.type==="password")||l==="textarea"||i.contentEditable==="true")}var aw=ja&&"documentMode"in document&&11>=document.documentMode,so=null,Tm=null,Bu=null,xm=!1;function Pv(i,l,u){var h=u.window===u?u.document:u.nodeType===9?u:u.ownerDocument;xm||so==null||so!==pl(h)||(h=so,"selectionStart"in h&&Em(h)?h={start:h.selectionStart,end:h.selectionEnd}:(h=(h.ownerDocument&&h.ownerDocument.defaultView||window).getSelection(),h={anchorNode:h.anchorNode,anchorOffset:h.anchorOffset,focusNode:h.focusNode,focusOffset:h.focusOffset}),Bu&&Pu(Bu,h)||(Bu=h,h=zd(Tm,"onSelect"),0<h.length&&(l=new J0("onSelect","select",null,l,u),i.push({event:l,listeners:h}),l.target=so)))}function yl(i,l){var u={};return u[i.toLowerCase()]=l.toLowerCase(),u["Webkit"+i]="webkit"+l,u["Moz"+i]="moz"+l,u}var lo={animationend:yl("Animation","AnimationEnd"),animationiteration:yl("Animation","AnimationIteration"),animationstart:yl("Animation","AnimationStart"),transitionrun:yl("Transition","TransitionRun"),transitionstart:yl("Transition","TransitionStart"),transitioncancel:yl("Transition","TransitionCancel"),transitionend:yl("Transition","TransitionEnd")},Sm={},Bv={};ja&&(Bv=document.createElement("div").style,"AnimationEvent"in window||(delete lo.animationend.animation,delete lo.animationiteration.animation,delete lo.animationstart.animation),"TransitionEvent"in window||delete lo.transitionend.transition);function _l(i){if(Sm[i])return Sm[i];if(!lo[i])return i;var l=lo[i],u;for(u in l)if(l.hasOwnProperty(u)&&u in Bv)return Sm[i]=l[u];return i}var Uv=_l("animationend"),Fv=_l("animationiteration"),zv=_l("animationstart"),iw=_l("transitionrun"),sw=_l("transitionstart"),lw=_l("transitioncancel"),Hv=_l("transitionend"),qv=new Map,wm="abort auxClick beforeToggle cancel canPlay canPlayThrough click close contextMenu copy cut drag dragEnd dragEnter dragExit dragLeave dragOver dragStart drop durationChange emptied encrypted ended error gotPointerCapture input invalid keyDown keyPress keyUp load loadedData loadedMetadata loadStart lostPointerCapture mouseDown mouseMove mouseOut mouseOver mouseUp paste pause play playing pointerCancel pointerDown pointerMove pointerOut pointerOver pointerUp progress rateChange reset resize seeked seeking stalled submit suspend timeUpdate touchCancel touchEnd touchStart volumeChange scroll toggle touchMove waiting wheel".split(" ");wm.push("scrollEnd");function $a(i,l){qv.set(i,l),wi(l,[i])}var jv=new WeakMap;function Sa(i,l){if(typeof i=="object"&&i!==null){var u=jv.get(i);return u!==void 0?u:(l={value:i,source:l,stack:qa(l)},jv.set(i,l),l)}return{value:i,source:l,stack:qa(l)}}var wa=[],oo=0,Am=0;function nd(){for(var i=oo,l=Am=oo=0;l<i;){var u=wa[l];wa[l++]=null;var h=wa[l];wa[l++]=null;var g=wa[l];wa[l++]=null;var v=wa[l];if(wa[l++]=null,h!==null&&g!==null){var A=h.pending;A===null?g.next=g:(g.next=A.next,A.next=g),h.pending=g}v!==0&&$v(u,g,v)}}function rd(i,l,u,h){wa[oo++]=i,wa[oo++]=l,wa[oo++]=u,wa[oo++]=h,Am|=h,i.lanes|=h,i=i.alternate,i!==null&&(i.lanes|=h)}function km(i,l,u,h){return rd(i,l,u,h),ad(i)}function uo(i,l){return rd(i,null,null,l),ad(i)}function $v(i,l,u){i.lanes|=u;var h=i.alternate;h!==null&&(h.lanes|=u);for(var g=!1,v=i.return;v!==null;)v.childLanes|=u,h=v.alternate,h!==null&&(h.childLanes|=u),v.tag===22&&(i=v.stateNode,i===null||i._visibility&1||(g=!0)),i=v,v=v.return;return i.tag===3?(v=i.stateNode,g&&l!==null&&(g=31-ze(u),i=v.hiddenUpdates,h=i[g],h===null?i[g]=[l]:h.push(l),l.lane=u|536870912),v):null}function ad(i){if(50<oc)throw oc=0,Ip=null,Error(r(185));for(var l=i.return;l!==null;)i=l,l=i.return;return i.tag===3?i.stateNode:null}var co={};function ow(i,l,u,h){this.tag=i,this.key=u,this.sibling=this.child=this.return=this.stateNode=this.type=this.elementType=null,this.index=0,this.refCleanup=this.ref=null,this.pendingProps=l,this.dependencies=this.memoizedState=this.updateQueue=this.memoizedProps=null,this.mode=h,this.subtreeFlags=this.flags=0,this.deletions=null,this.childLanes=this.lanes=0,this.alternate=null}function oa(i,l,u,h){return new ow(i,l,u,h)}function Cm(i){return i=i.prototype,!(!i||!i.isReactComponent)}function Ni(i,l){var u=i.alternate;return u===null?(u=oa(i.tag,l,i.key,i.mode),u.elementType=i.elementType,u.type=i.type,u.stateNode=i.stateNode,u.alternate=i,i.alternate=u):(u.pendingProps=l,u.type=i.type,u.flags=0,u.subtreeFlags=0,u.deletions=null),u.flags=i.flags&65011712,u.childLanes=i.childLanes,u.lanes=i.lanes,u.child=i.child,u.memoizedProps=i.memoizedProps,u.memoizedState=i.memoizedState,u.updateQueue=i.updateQueue,l=i.dependencies,u.dependencies=l===null?null:{lanes:l.lanes,firstContext:l.firstContext},u.sibling=i.sibling,u.index=i.index,u.ref=i.ref,u.refCleanup=i.refCleanup,u}function Yv(i,l){i.flags&=65011714;var u=i.alternate;return u===null?(i.childLanes=0,i.lanes=l,i.child=null,i.subtreeFlags=0,i.memoizedProps=null,i.memoizedState=null,i.updateQueue=null,i.dependencies=null,i.stateNode=null):(i.childLanes=u.childLanes,i.lanes=u.lanes,i.child=u.child,i.subtreeFlags=0,i.deletions=null,i.memoizedProps=u.memoizedProps,i.memoizedState=u.memoizedState,i.updateQueue=u.updateQueue,i.type=u.type,l=u.dependencies,i.dependencies=l===null?null:{lanes:l.lanes,firstContext:l.firstContext}),i}function id(i,l,u,h,g,v){var A=0;if(h=i,typeof i=="function")Cm(i)&&(A=1);else if(typeof i=="string")A=cA(i,u,Oe.current)?26:i==="html"||i==="head"||i==="body"?27:5;else e:switch(i){case Q:return i=oa(31,u,l,g),i.elementType=Q,i.lanes=v,i;case T:return El(u.children,g,v,l);case x:A=8,g|=24;break;case C:return i=oa(12,u,l,g|2),i.elementType=C,i.lanes=v,i;case H:return i=oa(13,u,l,g),i.elementType=H,i.lanes=v,i;case B:return i=oa(19,u,l,g),i.elementType=B,i.lanes=v,i;default:if(typeof i=="object"&&i!==null)switch(i.$$typeof){case w:case D:A=10;break e;case O:A=9;break e;case Y:A=11;break e;case ee:A=14;break e;case Z:A=16,h=null;break e}A=29,u=Error(r(130,i===null?"null":typeof i,"")),h=null}return l=oa(A,u,l,g),l.elementType=i,l.type=h,l.lanes=v,l}function El(i,l,u,h){return i=oa(7,i,h,l),i.lanes=u,i}function Nm(i,l,u){return i=oa(6,i,null,l),i.lanes=u,i}function Om(i,l,u){return l=oa(4,i.children!==null?i.children:[],i.key,l),l.lanes=u,l.stateNode={containerInfo:i.containerInfo,pendingChildren:null,implementation:i.implementation},l}var ho=[],fo=0,sd=null,ld=0,Aa=[],ka=0,Tl=null,Oi=1,Ri="";function xl(i,l){ho[fo++]=ld,ho[fo++]=sd,sd=i,ld=l}function Gv(i,l,u){Aa[ka++]=Oi,Aa[ka++]=Ri,Aa[ka++]=Tl,Tl=i;var h=Oi;i=Ri;var g=32-ze(h)-1;h&=~(1<<g),u+=1;var v=32-ze(l)+g;if(30<v){var A=g-g%5;v=(h&(1<<A)-1).toString(32),h>>=A,g-=A,Oi=1<<32-ze(l)+g|u<<g|h,Ri=v+i}else Oi=1<<v|u<<g|h,Ri=i}function Rm(i){i.return!==null&&(xl(i,1),Gv(i,1,0))}function Mm(i){for(;i===sd;)sd=ho[--fo],ho[fo]=null,ld=ho[--fo],ho[fo]=null;for(;i===Tl;)Tl=Aa[--ka],Aa[ka]=null,Ri=Aa[--ka],Aa[ka]=null,Oi=Aa[--ka],Aa[ka]=null}var Ir=null,Mn=null,jt=!1,Sl=null,ti=!1,Im=Error(r(519));function wl(i){var l=Error(r(418,""));throw zu(Sa(l,i)),Im}function Vv(i){var l=i.stateNode,u=i.type,h=i.memoizedProps;switch(l[Pe]=i,l[et]=h,u){case"dialog":Mt("cancel",l),Mt("close",l);break;case"iframe":case"object":case"embed":Mt("load",l);break;case"video":case"audio":for(u=0;u<cc.length;u++)Mt(cc[u],l);break;case"source":Mt("error",l);break;case"img":case"image":case"link":Mt("error",l),Mt("load",l);break;case"details":Mt("toggle",l);break;case"input":Mt("invalid",l),K0(l,h.value,h.defaultValue,h.checked,h.defaultChecked,h.type,h.name,!0),ml(l);break;case"select":Mt("invalid",l);break;case"textarea":Mt("invalid",l),W0(l,h.value,h.defaultValue,h.children),ml(l)}u=h.children,typeof u!="string"&&typeof u!="number"&&typeof u!="bigint"||l.textContent===""+u||h.suppressHydrationWarning===!0||cy(l.textContent,u)?(h.popover!=null&&(Mt("beforetoggle",l),Mt("toggle",l)),h.onScroll!=null&&Mt("scroll",l),h.onScrollEnd!=null&&Mt("scrollend",l),h.onClick!=null&&(l.onclick=Hd),l=!0):l=!1,l||wl(i)}function Kv(i){for(Ir=i.return;Ir;)switch(Ir.tag){case 5:case 13:ti=!1;return;case 27:case 3:ti=!0;return;default:Ir=Ir.return}}function Uu(i){if(i!==Ir)return!1;if(!jt)return Kv(i),jt=!0,!1;var l=i.tag,u;if((u=l!==3&&l!==27)&&((u=l===5)&&(u=i.type,u=!(u!=="form"&&u!=="button")||Wp(i.type,i.memoizedProps)),u=!u),u&&Mn&&wl(i),Kv(i),l===13){if(i=i.memoizedState,i=i!==null?i.dehydrated:null,!i)throw Error(r(317));e:{for(i=i.nextSibling,l=0;i;){if(i.nodeType===8)if(u=i.data,u==="/$"){if(l===0){Mn=Ga(i.nextSibling);break e}l--}else u!=="$"&&u!=="$!"&&u!=="$?"||l++;i=i.nextSibling}Mn=null}}else l===27?(l=Mn,Ds(i.type)?(i=Jp,Jp=null,Mn=i):Mn=l):Mn=Ir?Ga(i.stateNode.nextSibling):null;return!0}function Fu(){Mn=Ir=null,jt=!1}function Wv(){var i=Sl;return i!==null&&(Xr===null?Xr=i:Xr.push.apply(Xr,i),Sl=null),i}function zu(i){Sl===null?Sl=[i]:Sl.push(i)}var Dm=ie(null),Al=null,Mi=null;function ys(i,l,u){L(Dm,l._currentValue),l._currentValue=u}function Ii(i){i._currentValue=Dm.current,ge(Dm)}function Lm(i,l,u){for(;i!==null;){var h=i.alternate;if((i.childLanes&l)!==l?(i.childLanes|=l,h!==null&&(h.childLanes|=l)):h!==null&&(h.childLanes&l)!==l&&(h.childLanes|=l),i===u)break;i=i.return}}function Pm(i,l,u,h){var g=i.child;for(g!==null&&(g.return=i);g!==null;){var v=g.dependencies;if(v!==null){var A=g.child;v=v.firstContext;e:for(;v!==null;){var M=v;v=g;for(var $=0;$<l.length;$++)if(M.context===l[$]){v.lanes|=u,M=v.alternate,M!==null&&(M.lanes|=u),Lm(v.return,u,i),h||(A=null);break e}v=M.next}}else if(g.tag===18){if(A=g.return,A===null)throw Error(r(341));A.lanes|=u,v=A.alternate,v!==null&&(v.lanes|=u),Lm(A,u,i),A=null}else A=g.child;if(A!==null)A.return=g;else for(A=g;A!==null;){if(A===i){A=null;break}if(g=A.sibling,g!==null){g.return=A.return,A=g;break}A=A.return}g=A}}function Hu(i,l,u,h){i=null;for(var g=l,v=!1;g!==null;){if(!v){if((g.flags&524288)!==0)v=!0;else if((g.flags&262144)!==0)break}if(g.tag===10){var A=g.alternate;if(A===null)throw Error(r(387));if(A=A.memoizedProps,A!==null){var M=g.type;la(g.pendingProps.value,A.value)||(i!==null?i.push(M):i=[M])}}else if(g===Ot.current){if(A=g.alternate,A===null)throw Error(r(387));A.memoizedState.memoizedState!==g.memoizedState.memoizedState&&(i!==null?i.push(gc):i=[gc])}g=g.return}i!==null&&Pm(l,i,u,h),l.flags|=262144}function od(i){for(i=i.firstContext;i!==null;){if(!la(i.context._currentValue,i.memoizedValue))return!0;i=i.next}return!1}function kl(i){Al=i,Mi=null,i=i.dependencies,i!==null&&(i.firstContext=null)}function xr(i){return Xv(Al,i)}function ud(i,l){return Al===null&&kl(i),Xv(i,l)}function Xv(i,l){var u=l._currentValue;if(l={context:l,memoizedValue:u,next:null},Mi===null){if(i===null)throw Error(r(308));Mi=l,i.dependencies={lanes:0,firstContext:l},i.flags|=524288}else Mi=Mi.next=l;return u}var uw=typeof AbortController<"u"?AbortController:function(){var i=[],l=this.signal={aborted:!1,addEventListener:function(u,h){i.push(h)}};this.abort=function(){l.aborted=!0,i.forEach(function(u){return u()})}},cw=e.unstable_scheduleCallback,dw=e.unstable_NormalPriority,nr={$$typeof:D,Consumer:null,Provider:null,_currentValue:null,_currentValue2:null,_threadCount:0};function Bm(){return{controller:new uw,data:new Map,refCount:0}}function qu(i){i.refCount--,i.refCount===0&&cw(dw,function(){i.controller.abort()})}var ju=null,Um=0,mo=0,po=null;function hw(i,l){if(ju===null){var u=ju=[];Um=0,mo=zp(),po={status:"pending",value:void 0,then:function(h){u.push(h)}}}return Um++,l.then(Qv,Qv),l}function Qv(){if(--Um===0&&ju!==null){po!==null&&(po.status="fulfilled");var i=ju;ju=null,mo=0,po=null;for(var l=0;l<i.length;l++)(0,i[l])()}}function fw(i,l){var u=[],h={status:"pending",value:null,reason:null,then:function(g){u.push(g)}};return i.then(function(){h.status="fulfilled",h.value=l;for(var g=0;g<u.length;g++)(0,u[g])(l)},function(g){for(h.status="rejected",h.reason=g,g=0;g<u.length;g++)(0,u[g])(void 0)}),h}var Zv=V.S;V.S=function(i,l){typeof l=="object"&&l!==null&&typeof l.then=="function"&&hw(i,l),Zv!==null&&Zv(i,l)};var Cl=ie(null);function Fm(){var i=Cl.current;return i!==null?i:un.pooledCache}function cd(i,l){l===null?L(Cl,Cl.current):L(Cl,l.pool)}function Jv(){var i=Fm();return i===null?null:{parent:nr._currentValue,pool:i}}var $u=Error(r(460)),e4=Error(r(474)),dd=Error(r(542)),zm={then:function(){}};function t4(i){return i=i.status,i==="fulfilled"||i==="rejected"}function hd(){}function n4(i,l,u){switch(u=i[u],u===void 0?i.push(l):u!==l&&(l.then(hd,hd),l=u),l.status){case"fulfilled":return l.value;case"rejected":throw i=l.reason,a4(i),i;default:if(typeof l.status=="string")l.then(hd,hd);else{if(i=un,i!==null&&100<i.shellSuspendCounter)throw Error(r(482));i=l,i.status="pending",i.then(function(h){if(l.status==="pending"){var g=l;g.status="fulfilled",g.value=h}},function(h){if(l.status==="pending"){var g=l;g.status="rejected",g.reason=h}})}switch(l.status){case"fulfilled":return l.value;case"rejected":throw i=l.reason,a4(i),i}throw Yu=l,$u}}var Yu=null;function r4(){if(Yu===null)throw Error(r(459));var i=Yu;return Yu=null,i}function a4(i){if(i===$u||i===dd)throw Error(r(483))}var _s=!1;function Hm(i){i.updateQueue={baseState:i.memoizedState,firstBaseUpdate:null,lastBaseUpdate:null,shared:{pending:null,lanes:0,hiddenCallbacks:null},callbacks:null}}function qm(i,l){i=i.updateQueue,l.updateQueue===i&&(l.updateQueue={baseState:i.baseState,firstBaseUpdate:i.firstBaseUpdate,lastBaseUpdate:i.lastBaseUpdate,shared:i.shared,callbacks:null})}function Es(i){return{lane:i,tag:0,payload:null,callback:null,next:null}}function Ts(i,l,u){var h=i.updateQueue;if(h===null)return null;if(h=h.shared,(Kt&2)!==0){var g=h.pending;return g===null?l.next=l:(l.next=g.next,g.next=l),h.pending=l,l=ad(i),$v(i,null,u),l}return rd(i,h,l,u),ad(i)}function Gu(i,l,u){if(l=l.updateQueue,l!==null&&(l=l.shared,(u&4194048)!==0)){var h=l.lanes;h&=i.pendingLanes,u|=h,l.lanes=u,Ee(i,u)}}function jm(i,l){var u=i.updateQueue,h=i.alternate;if(h!==null&&(h=h.updateQueue,u===h)){var g=null,v=null;if(u=u.firstBaseUpdate,u!==null){do{var A={lane:u.lane,tag:u.tag,payload:u.payload,callback:null,next:null};v===null?g=v=A:v=v.next=A,u=u.next}while(u!==null);v===null?g=v=l:v=v.next=l}else g=v=l;u={baseState:h.baseState,firstBaseUpdate:g,lastBaseUpdate:v,shared:h.shared,callbacks:h.callbacks},i.updateQueue=u;return}i=u.lastBaseUpdate,i===null?u.firstBaseUpdate=l:i.next=l,u.lastBaseUpdate=l}var $m=!1;function Vu(){if($m){var i=po;if(i!==null)throw i}}function Ku(i,l,u,h){$m=!1;var g=i.updateQueue;_s=!1;var v=g.firstBaseUpdate,A=g.lastBaseUpdate,M=g.shared.pending;if(M!==null){g.shared.pending=null;var $=M,re=$.next;$.next=null,A===null?v=re:A.next=re,A=$;var _e=i.alternate;_e!==null&&(_e=_e.updateQueue,M=_e.lastBaseUpdate,M!==A&&(M===null?_e.firstBaseUpdate=re:M.next=re,_e.lastBaseUpdate=$))}if(v!==null){var xe=g.baseState;A=0,_e=re=$=null,M=v;do{var le=M.lane&-536870913,ue=le!==M.lane;if(ue?(Pt&le)===le:(h&le)===le){le!==0&&le===mo&&($m=!0),_e!==null&&(_e=_e.next={lane:0,tag:M.tag,payload:M.payload,callback:null,next:null});e:{var ft=i,ut=M;le=l;var en=u;switch(ut.tag){case 1:if(ft=ut.payload,typeof ft=="function"){xe=ft.call(en,xe,le);break e}xe=ft;break e;case 3:ft.flags=ft.flags&-65537|128;case 0:if(ft=ut.payload,le=typeof ft=="function"?ft.call(en,xe,le):ft,le==null)break e;xe=m({},xe,le);break e;case 2:_s=!0}}le=M.callback,le!==null&&(i.flags|=64,ue&&(i.flags|=8192),ue=g.callbacks,ue===null?g.callbacks=[le]:ue.push(le))}else ue={lane:le,tag:M.tag,payload:M.payload,callback:M.callback,next:null},_e===null?(re=_e=ue,$=xe):_e=_e.next=ue,A|=le;if(M=M.next,M===null){if(M=g.shared.pending,M===null)break;ue=M,M=ue.next,ue.next=null,g.lastBaseUpdate=ue,g.shared.pending=null}}while(!0);_e===null&&($=xe),g.baseState=$,g.firstBaseUpdate=re,g.lastBaseUpdate=_e,v===null&&(g.shared.lanes=0),Os|=A,i.lanes=A,i.memoizedState=xe}}function i4(i,l){if(typeof i!="function")throw Error(r(191,i));i.call(l)}function s4(i,l){var u=i.callbacks;if(u!==null)for(i.callbacks=null,i=0;i<u.length;i++)i4(u[i],l)}var go=ie(null),fd=ie(0);function l4(i,l){i=zi,L(fd,i),L(go,l),zi=i|l.baseLanes}function Ym(){L(fd,zi),L(go,go.current)}function Gm(){zi=fd.current,ge(go),ge(fd)}var xs=0,At=null,Zt=null,Gn=null,md=!1,bo=!1,Nl=!1,pd=0,Wu=0,vo=null,mw=0;function Bn(){throw Error(r(321))}function Vm(i,l){if(l===null)return!1;for(var u=0;u<l.length&&u<i.length;u++)if(!la(i[u],l[u]))return!1;return!0}function Km(i,l,u,h,g,v){return xs=v,At=l,l.memoizedState=null,l.updateQueue=null,l.lanes=0,V.H=i===null||i.memoizedState===null?$4:Y4,Nl=!1,v=u(h,g),Nl=!1,bo&&(v=u4(l,u,h,g)),o4(i),v}function o4(i){V.H=Ed;var l=Zt!==null&&Zt.next!==null;if(xs=0,Gn=Zt=At=null,md=!1,Wu=0,vo=null,l)throw Error(r(300));i===null||lr||(i=i.dependencies,i!==null&&od(i)&&(lr=!0))}function u4(i,l,u,h){At=i;var g=0;do{if(bo&&(vo=null),Wu=0,bo=!1,25<=g)throw Error(r(301));if(g+=1,Gn=Zt=null,i.updateQueue!=null){var v=i.updateQueue;v.lastEffect=null,v.events=null,v.stores=null,v.memoCache!=null&&(v.memoCache.index=0)}V.H=Ew,v=l(u,h)}while(bo);return v}function pw(){var i=V.H,l=i.useState()[0];return l=typeof l.then=="function"?Xu(l):l,i=i.useState()[0],(Zt!==null?Zt.memoizedState:null)!==i&&(At.flags|=1024),l}function Wm(){var i=pd!==0;return pd=0,i}function Xm(i,l,u){l.updateQueue=i.updateQueue,l.flags&=-2053,i.lanes&=~u}function Qm(i){if(md){for(i=i.memoizedState;i!==null;){var l=i.queue;l!==null&&(l.pending=null),i=i.next}md=!1}xs=0,Gn=Zt=At=null,bo=!1,Wu=pd=0,vo=null}function Kr(){var i={memoizedState:null,baseState:null,baseQueue:null,queue:null,next:null};return Gn===null?At.memoizedState=Gn=i:Gn=Gn.next=i,Gn}function Vn(){if(Zt===null){var i=At.alternate;i=i!==null?i.memoizedState:null}else i=Zt.next;var l=Gn===null?At.memoizedState:Gn.next;if(l!==null)Gn=l,Zt=i;else{if(i===null)throw At.alternate===null?Error(r(467)):Error(r(310));Zt=i,i={memoizedState:Zt.memoizedState,baseState:Zt.baseState,baseQueue:Zt.baseQueue,queue:Zt.queue,next:null},Gn===null?At.memoizedState=Gn=i:Gn=Gn.next=i}return Gn}function Zm(){return{lastEffect:null,events:null,stores:null,memoCache:null}}function Xu(i){var l=Wu;return Wu+=1,vo===null&&(vo=[]),i=n4(vo,i,l),l=At,(Gn===null?l.memoizedState:Gn.next)===null&&(l=l.alternate,V.H=l===null||l.memoizedState===null?$4:Y4),i}function gd(i){if(i!==null&&typeof i=="object"){if(typeof i.then=="function")return Xu(i);if(i.$$typeof===D)return xr(i)}throw Error(r(438,String(i)))}function Jm(i){var l=null,u=At.updateQueue;if(u!==null&&(l=u.memoCache),l==null){var h=At.alternate;h!==null&&(h=h.updateQueue,h!==null&&(h=h.memoCache,h!=null&&(l={data:h.data.map(function(g){return g.slice()}),index:0})))}if(l==null&&(l={data:[],index:0}),u===null&&(u=Zm(),At.updateQueue=u),u.memoCache=l,u=l.data[l.index],u===void 0)for(u=l.data[l.index]=Array(i),h=0;h<i;h++)u[h]=G;return l.index++,u}function Di(i,l){return typeof l=="function"?l(i):l}function bd(i){var l=Vn();return ep(l,Zt,i)}function ep(i,l,u){var h=i.queue;if(h===null)throw Error(r(311));h.lastRenderedReducer=u;var g=i.baseQueue,v=h.pending;if(v!==null){if(g!==null){var A=g.next;g.next=v.next,v.next=A}l.baseQueue=g=v,h.pending=null}if(v=i.baseState,g===null)i.memoizedState=v;else{l=g.next;var M=A=null,$=null,re=l,_e=!1;do{var xe=re.lane&-536870913;if(xe!==re.lane?(Pt&xe)===xe:(xs&xe)===xe){var le=re.revertLane;if(le===0)$!==null&&($=$.next={lane:0,revertLane:0,action:re.action,hasEagerState:re.hasEagerState,eagerState:re.eagerState,next:null}),xe===mo&&(_e=!0);else if((xs&le)===le){re=re.next,le===mo&&(_e=!0);continue}else xe={lane:0,revertLane:re.revertLane,action:re.action,hasEagerState:re.hasEagerState,eagerState:re.eagerState,next:null},$===null?(M=$=xe,A=v):$=$.next=xe,At.lanes|=le,Os|=le;xe=re.action,Nl&&u(v,xe),v=re.hasEagerState?re.eagerState:u(v,xe)}else le={lane:xe,revertLane:re.revertLane,action:re.action,hasEagerState:re.hasEagerState,eagerState:re.eagerState,next:null},$===null?(M=$=le,A=v):$=$.next=le,At.lanes|=xe,Os|=xe;re=re.next}while(re!==null&&re!==l);if($===null?A=v:$.next=M,!la(v,i.memoizedState)&&(lr=!0,_e&&(u=po,u!==null)))throw u;i.memoizedState=v,i.baseState=A,i.baseQueue=$,h.lastRenderedState=v}return g===null&&(h.lanes=0),[i.memoizedState,h.dispatch]}function tp(i){var l=Vn(),u=l.queue;if(u===null)throw Error(r(311));u.lastRenderedReducer=i;var h=u.dispatch,g=u.pending,v=l.memoizedState;if(g!==null){u.pending=null;var A=g=g.next;do v=i(v,A.action),A=A.next;while(A!==g);la(v,l.memoizedState)||(lr=!0),l.memoizedState=v,l.baseQueue===null&&(l.baseState=v),u.lastRenderedState=v}return[v,h]}function c4(i,l,u){var h=At,g=Vn(),v=jt;if(v){if(u===void 0)throw Error(r(407));u=u()}else u=l();var A=!la((Zt||g).memoizedState,u);A&&(g.memoizedState=u,lr=!0),g=g.queue;var M=f4.bind(null,h,g,i);if(Qu(2048,8,M,[i]),g.getSnapshot!==l||A||Gn!==null&&Gn.memoizedState.tag&1){if(h.flags|=2048,yo(9,vd(),h4.bind(null,h,g,u,l),null),un===null)throw Error(r(349));v||(xs&124)!==0||d4(h,l,u)}return u}function d4(i,l,u){i.flags|=16384,i={getSnapshot:l,value:u},l=At.updateQueue,l===null?(l=Zm(),At.updateQueue=l,l.stores=[i]):(u=l.stores,u===null?l.stores=[i]:u.push(i))}function h4(i,l,u,h){l.value=u,l.getSnapshot=h,m4(l)&&p4(i)}function f4(i,l,u){return u(function(){m4(l)&&p4(i)})}function m4(i){var l=i.getSnapshot;i=i.value;try{var u=l();return!la(i,u)}catch{return!0}}function p4(i){var l=uo(i,2);l!==null&&fa(l,i,2)}function np(i){var l=Kr();if(typeof i=="function"){var u=i;if(i=u(),Nl){Ue(!0);try{u()}finally{Ue(!1)}}}return l.memoizedState=l.baseState=i,l.queue={pending:null,lanes:0,dispatch:null,lastRenderedReducer:Di,lastRenderedState:i},l}function g4(i,l,u,h){return i.baseState=u,ep(i,Zt,typeof h=="function"?h:Di)}function gw(i,l,u,h,g){if(_d(i))throw Error(r(485));if(i=l.action,i!==null){var v={payload:g,action:i,next:null,isTransition:!0,status:"pending",value:null,reason:null,listeners:[],then:function(A){v.listeners.push(A)}};V.T!==null?u(!0):v.isTransition=!1,h(v),u=l.pending,u===null?(v.next=l.pending=v,b4(l,v)):(v.next=u.next,l.pending=u.next=v)}}function b4(i,l){var u=l.action,h=l.payload,g=i.state;if(l.isTransition){var v=V.T,A={};V.T=A;try{var M=u(g,h),$=V.S;$!==null&&$(A,M),v4(i,l,M)}catch(re){rp(i,l,re)}finally{V.T=v}}else try{v=u(g,h),v4(i,l,v)}catch(re){rp(i,l,re)}}function v4(i,l,u){u!==null&&typeof u=="object"&&typeof u.then=="function"?u.then(function(h){y4(i,l,h)},function(h){return rp(i,l,h)}):y4(i,l,u)}function y4(i,l,u){l.status="fulfilled",l.value=u,_4(l),i.state=u,l=i.pending,l!==null&&(u=l.next,u===l?i.pending=null:(u=u.next,l.next=u,b4(i,u)))}function rp(i,l,u){var h=i.pending;if(i.pending=null,h!==null){h=h.next;do l.status="rejected",l.reason=u,_4(l),l=l.next;while(l!==h)}i.action=null}function _4(i){i=i.listeners;for(var l=0;l<i.length;l++)(0,i[l])()}function E4(i,l){return l}function T4(i,l){if(jt){var u=un.formState;if(u!==null){e:{var h=At;if(jt){if(Mn){t:{for(var g=Mn,v=ti;g.nodeType!==8;){if(!v){g=null;break t}if(g=Ga(g.nextSibling),g===null){g=null;break t}}v=g.data,g=v==="F!"||v==="F"?g:null}if(g){Mn=Ga(g.nextSibling),h=g.data==="F!";break e}}wl(h)}h=!1}h&&(l=u[0])}}return u=Kr(),u.memoizedState=u.baseState=l,h={pending:null,lanes:0,dispatch:null,lastRenderedReducer:E4,lastRenderedState:l},u.queue=h,u=H4.bind(null,At,h),h.dispatch=u,h=np(!1),v=op.bind(null,At,!1,h.queue),h=Kr(),g={state:l,dispatch:null,action:i,pending:null},h.queue=g,u=gw.bind(null,At,g,v,u),g.dispatch=u,h.memoizedState=i,[l,u,!1]}function x4(i){var l=Vn();return S4(l,Zt,i)}function S4(i,l,u){if(l=ep(i,l,E4)[0],i=bd(Di)[0],typeof l=="object"&&l!==null&&typeof l.then=="function")try{var h=Xu(l)}catch(A){throw A===$u?dd:A}else h=l;l=Vn();var g=l.queue,v=g.dispatch;return u!==l.memoizedState&&(At.flags|=2048,yo(9,vd(),bw.bind(null,g,u),null)),[h,v,i]}function bw(i,l){i.action=l}function w4(i){var l=Vn(),u=Zt;if(u!==null)return S4(l,u,i);Vn(),l=l.memoizedState,u=Vn();var h=u.queue.dispatch;return u.memoizedState=i,[l,h,!1]}function yo(i,l,u,h){return i={tag:i,create:u,deps:h,inst:l,next:null},l=At.updateQueue,l===null&&(l=Zm(),At.updateQueue=l),u=l.lastEffect,u===null?l.lastEffect=i.next=i:(h=u.next,u.next=i,i.next=h,l.lastEffect=i),i}function vd(){return{destroy:void 0,resource:void 0}}function A4(){return Vn().memoizedState}function yd(i,l,u,h){var g=Kr();h=h===void 0?null:h,At.flags|=i,g.memoizedState=yo(1|l,vd(),u,h)}function Qu(i,l,u,h){var g=Vn();h=h===void 0?null:h;var v=g.memoizedState.inst;Zt!==null&&h!==null&&Vm(h,Zt.memoizedState.deps)?g.memoizedState=yo(l,v,u,h):(At.flags|=i,g.memoizedState=yo(1|l,v,u,h))}function k4(i,l){yd(8390656,8,i,l)}function C4(i,l){Qu(2048,8,i,l)}function N4(i,l){return Qu(4,2,i,l)}function O4(i,l){return Qu(4,4,i,l)}function R4(i,l){if(typeof l=="function"){i=i();var u=l(i);return function(){typeof u=="function"?u():l(null)}}if(l!=null)return i=i(),l.current=i,function(){l.current=null}}function M4(i,l,u){u=u!=null?u.concat([i]):null,Qu(4,4,R4.bind(null,l,i),u)}function ap(){}function I4(i,l){var u=Vn();l=l===void 0?null:l;var h=u.memoizedState;return l!==null&&Vm(l,h[1])?h[0]:(u.memoizedState=[i,l],i)}function D4(i,l){var u=Vn();l=l===void 0?null:l;var h=u.memoizedState;if(l!==null&&Vm(l,h[1]))return h[0];if(h=i(),Nl){Ue(!0);try{i()}finally{Ue(!1)}}return u.memoizedState=[h,l],h}function ip(i,l,u){return u===void 0||(xs&1073741824)!==0?i.memoizedState=l:(i.memoizedState=u,i=B3(),At.lanes|=i,Os|=i,u)}function L4(i,l,u,h){return la(u,l)?u:go.current!==null?(i=ip(i,u,h),la(i,l)||(lr=!0),i):(xs&42)===0?(lr=!0,i.memoizedState=u):(i=B3(),At.lanes|=i,Os|=i,l)}function P4(i,l,u,h,g){var v=me.p;me.p=v!==0&&8>v?v:8;var A=V.T,M={};V.T=M,op(i,!1,l,u);try{var $=g(),re=V.S;if(re!==null&&re(M,$),$!==null&&typeof $=="object"&&typeof $.then=="function"){var _e=fw($,h);Zu(i,l,_e,ha(i))}else Zu(i,l,h,ha(i))}catch(xe){Zu(i,l,{then:function(){},status:"rejected",reason:xe},ha())}finally{me.p=v,V.T=A}}function vw(){}function sp(i,l,u,h){if(i.tag!==5)throw Error(r(476));var g=B4(i).queue;P4(i,g,l,pe,u===null?vw:function(){return U4(i),u(h)})}function B4(i){var l=i.memoizedState;if(l!==null)return l;l={memoizedState:pe,baseState:pe,baseQueue:null,queue:{pending:null,lanes:0,dispatch:null,lastRenderedReducer:Di,lastRenderedState:pe},next:null};var u={};return l.next={memoizedState:u,baseState:u,baseQueue:null,queue:{pending:null,lanes:0,dispatch:null,lastRenderedReducer:Di,lastRenderedState:u},next:null},i.memoizedState=l,i=i.alternate,i!==null&&(i.memoizedState=l),l}function U4(i){var l=B4(i).next.queue;Zu(i,l,{},ha())}function lp(){return xr(gc)}function F4(){return Vn().memoizedState}function z4(){return Vn().memoizedState}function yw(i){for(var l=i.return;l!==null;){switch(l.tag){case 24:case 3:var u=ha();i=Es(u);var h=Ts(l,i,u);h!==null&&(fa(h,l,u),Gu(h,l,u)),l={cache:Bm()},i.payload=l;return}l=l.return}}function _w(i,l,u){var h=ha();u={lane:h,revertLane:0,action:u,hasEagerState:!1,eagerState:null,next:null},_d(i)?q4(l,u):(u=km(i,l,u,h),u!==null&&(fa(u,i,h),j4(u,l,h)))}function H4(i,l,u){var h=ha();Zu(i,l,u,h)}function Zu(i,l,u,h){var g={lane:h,revertLane:0,action:u,hasEagerState:!1,eagerState:null,next:null};if(_d(i))q4(l,g);else{var v=i.alternate;if(i.lanes===0&&(v===null||v.lanes===0)&&(v=l.lastRenderedReducer,v!==null))try{var A=l.lastRenderedState,M=v(A,u);if(g.hasEagerState=!0,g.eagerState=M,la(M,A))return rd(i,l,g,0),un===null&&nd(),!1}catch{}finally{}if(u=km(i,l,g,h),u!==null)return fa(u,i,h),j4(u,l,h),!0}return!1}function op(i,l,u,h){if(h={lane:2,revertLane:zp(),action:h,hasEagerState:!1,eagerState:null,next:null},_d(i)){if(l)throw Error(r(479))}else l=km(i,u,h,2),l!==null&&fa(l,i,2)}function _d(i){var l=i.alternate;return i===At||l!==null&&l===At}function q4(i,l){bo=md=!0;var u=i.pending;u===null?l.next=l:(l.next=u.next,u.next=l),i.pending=l}function j4(i,l,u){if((u&4194048)!==0){var h=l.lanes;h&=i.pendingLanes,u|=h,l.lanes=u,Ee(i,u)}}var Ed={readContext:xr,use:gd,useCallback:Bn,useContext:Bn,useEffect:Bn,useImperativeHandle:Bn,useLayoutEffect:Bn,useInsertionEffect:Bn,useMemo:Bn,useReducer:Bn,useRef:Bn,useState:Bn,useDebugValue:Bn,useDeferredValue:Bn,useTransition:Bn,useSyncExternalStore:Bn,useId:Bn,useHostTransitionStatus:Bn,useFormState:Bn,useActionState:Bn,useOptimistic:Bn,useMemoCache:Bn,useCacheRefresh:Bn},$4={readContext:xr,use:gd,useCallback:function(i,l){return Kr().memoizedState=[i,l===void 0?null:l],i},useContext:xr,useEffect:k4,useImperativeHandle:function(i,l,u){u=u!=null?u.concat([i]):null,yd(4194308,4,R4.bind(null,l,i),u)},useLayoutEffect:function(i,l){return yd(4194308,4,i,l)},useInsertionEffect:function(i,l){yd(4,2,i,l)},useMemo:function(i,l){var u=Kr();l=l===void 0?null:l;var h=i();if(Nl){Ue(!0);try{i()}finally{Ue(!1)}}return u.memoizedState=[h,l],h},useReducer:function(i,l,u){var h=Kr();if(u!==void 0){var g=u(l);if(Nl){Ue(!0);try{u(l)}finally{Ue(!1)}}}else g=l;return h.memoizedState=h.baseState=g,i={pending:null,lanes:0,dispatch:null,lastRenderedReducer:i,lastRenderedState:g},h.queue=i,i=i.dispatch=_w.bind(null,At,i),[h.memoizedState,i]},useRef:function(i){var l=Kr();return i={current:i},l.memoizedState=i},useState:function(i){i=np(i);var l=i.queue,u=H4.bind(null,At,l);return l.dispatch=u,[i.memoizedState,u]},useDebugValue:ap,useDeferredValue:function(i,l){var u=Kr();return ip(u,i,l)},useTransition:function(){var i=np(!1);return i=P4.bind(null,At,i.queue,!0,!1),Kr().memoizedState=i,[!1,i]},useSyncExternalStore:function(i,l,u){var h=At,g=Kr();if(jt){if(u===void 0)throw Error(r(407));u=u()}else{if(u=l(),un===null)throw Error(r(349));(Pt&124)!==0||d4(h,l,u)}g.memoizedState=u;var v={value:u,getSnapshot:l};return g.queue=v,k4(f4.bind(null,h,v,i),[i]),h.flags|=2048,yo(9,vd(),h4.bind(null,h,v,u,l),null),u},useId:function(){var i=Kr(),l=un.identifierPrefix;if(jt){var u=Ri,h=Oi;u=(h&~(1<<32-ze(h)-1)).toString(32)+u,l="«"+l+"R"+u,u=pd++,0<u&&(l+="H"+u.toString(32)),l+="»"}else u=mw++,l="«"+l+"r"+u.toString(32)+"»";return i.memoizedState=l},useHostTransitionStatus:lp,useFormState:T4,useActionState:T4,useOptimistic:function(i){var l=Kr();l.memoizedState=l.baseState=i;var u={pending:null,lanes:0,dispatch:null,lastRenderedReducer:null,lastRenderedState:null};return l.queue=u,l=op.bind(null,At,!0,u),u.dispatch=l,[i,l]},useMemoCache:Jm,useCacheRefresh:function(){return Kr().memoizedState=yw.bind(null,At)}},Y4={readContext:xr,use:gd,useCallback:I4,useContext:xr,useEffect:C4,useImperativeHandle:M4,useInsertionEffect:N4,useLayoutEffect:O4,useMemo:D4,useReducer:bd,useRef:A4,useState:function(){return bd(Di)},useDebugValue:ap,useDeferredValue:function(i,l){var u=Vn();return L4(u,Zt.memoizedState,i,l)},useTransition:function(){var i=bd(Di)[0],l=Vn().memoizedState;return[typeof i=="boolean"?i:Xu(i),l]},useSyncExternalStore:c4,useId:F4,useHostTransitionStatus:lp,useFormState:x4,useActionState:x4,useOptimistic:function(i,l){var u=Vn();return g4(u,Zt,i,l)},useMemoCache:Jm,useCacheRefresh:z4},Ew={readContext:xr,use:gd,useCallback:I4,useContext:xr,useEffect:C4,useImperativeHandle:M4,useInsertionEffect:N4,useLayoutEffect:O4,useMemo:D4,useReducer:tp,useRef:A4,useState:function(){return tp(Di)},useDebugValue:ap,useDeferredValue:function(i,l){var u=Vn();return Zt===null?ip(u,i,l):L4(u,Zt.memoizedState,i,l)},useTransition:function(){var i=tp(Di)[0],l=Vn().memoizedState;return[typeof i=="boolean"?i:Xu(i),l]},useSyncExternalStore:c4,useId:F4,useHostTransitionStatus:lp,useFormState:w4,useActionState:w4,useOptimistic:function(i,l){var u=Vn();return Zt!==null?g4(u,Zt,i,l):(u.baseState=i,[i,u.queue.dispatch])},useMemoCache:Jm,useCacheRefresh:z4},_o=null,Ju=0;function Td(i){var l=Ju;return Ju+=1,_o===null&&(_o=[]),n4(_o,i,l)}function ec(i,l){l=l.props.ref,i.ref=l!==void 0?l:null}function xd(i,l){throw l.$$typeof===p?Error(r(525)):(i=Object.prototype.toString.call(l),Error(r(31,i==="[object Object]"?"object with keys {"+Object.keys(l).join(", ")+"}":i)))}function G4(i){var l=i._init;return l(i._payload)}function V4(i){function l(J,W){if(i){var te=J.deletions;te===null?(J.deletions=[W],J.flags|=16):te.push(W)}}function u(J,W){if(!i)return null;for(;W!==null;)l(J,W),W=W.sibling;return null}function h(J){for(var W=new Map;J!==null;)J.key!==null?W.set(J.key,J):W.set(J.index,J),J=J.sibling;return W}function g(J,W){return J=Ni(J,W),J.index=0,J.sibling=null,J}function v(J,W,te){return J.index=te,i?(te=J.alternate,te!==null?(te=te.index,te<W?(J.flags|=67108866,W):te):(J.flags|=67108866,W)):(J.flags|=1048576,W)}function A(J){return i&&J.alternate===null&&(J.flags|=67108866),J}function M(J,W,te,Te){return W===null||W.tag!==6?(W=Nm(te,J.mode,Te),W.return=J,W):(W=g(W,te),W.return=J,W)}function $(J,W,te,Te){var Ze=te.type;return Ze===T?_e(J,W,te.props.children,Te,te.key):W!==null&&(W.elementType===Ze||typeof Ze=="object"&&Ze!==null&&Ze.$$typeof===Z&&G4(Ze)===W.type)?(W=g(W,te.props),ec(W,te),W.return=J,W):(W=id(te.type,te.key,te.props,null,J.mode,Te),ec(W,te),W.return=J,W)}function re(J,W,te,Te){return W===null||W.tag!==4||W.stateNode.containerInfo!==te.containerInfo||W.stateNode.implementation!==te.implementation?(W=Om(te,J.mode,Te),W.return=J,W):(W=g(W,te.children||[]),W.return=J,W)}function _e(J,W,te,Te,Ze){return W===null||W.tag!==7?(W=El(te,J.mode,Te,Ze),W.return=J,W):(W=g(W,te),W.return=J,W)}function xe(J,W,te){if(typeof W=="string"&&W!==""||typeof W=="number"||typeof W=="bigint")return W=Nm(""+W,J.mode,te),W.return=J,W;if(typeof W=="object"&&W!==null){switch(W.$$typeof){case b:return te=id(W.type,W.key,W.props,null,J.mode,te),ec(te,W),te.return=J,te;case y:return W=Om(W,J.mode,te),W.return=J,W;case Z:var Te=W._init;return W=Te(W._payload),xe(J,W,te)}if(de(W)||oe(W))return W=El(W,J.mode,te,null),W.return=J,W;if(typeof W.then=="function")return xe(J,Td(W),te);if(W.$$typeof===D)return xe(J,ud(J,W),te);xd(J,W)}return null}function le(J,W,te,Te){var Ze=W!==null?W.key:null;if(typeof te=="string"&&te!==""||typeof te=="number"||typeof te=="bigint")return Ze!==null?null:M(J,W,""+te,Te);if(typeof te=="object"&&te!==null){switch(te.$$typeof){case b:return te.key===Ze?$(J,W,te,Te):null;case y:return te.key===Ze?re(J,W,te,Te):null;case Z:return Ze=te._init,te=Ze(te._payload),le(J,W,te,Te)}if(de(te)||oe(te))return Ze!==null?null:_e(J,W,te,Te,null);if(typeof te.then=="function")return le(J,W,Td(te),Te);if(te.$$typeof===D)return le(J,W,ud(J,te),Te);xd(J,te)}return null}function ue(J,W,te,Te,Ze){if(typeof Te=="string"&&Te!==""||typeof Te=="number"||typeof Te=="bigint")return J=J.get(te)||null,M(W,J,""+Te,Ze);if(typeof Te=="object"&&Te!==null){switch(Te.$$typeof){case b:return J=J.get(Te.key===null?te:Te.key)||null,$(W,J,Te,Ze);case y:return J=J.get(Te.key===null?te:Te.key)||null,re(W,J,Te,Ze);case Z:var kt=Te._init;return Te=kt(Te._payload),ue(J,W,te,Te,Ze)}if(de(Te)||oe(Te))return J=J.get(te)||null,_e(W,J,Te,Ze,null);if(typeof Te.then=="function")return ue(J,W,te,Td(Te),Ze);if(Te.$$typeof===D)return ue(J,W,te,ud(W,Te),Ze);xd(W,Te)}return null}function ft(J,W,te,Te){for(var Ze=null,kt=null,at=W,ct=W=0,ur=null;at!==null&&ct<te.length;ct++){at.index>ct?(ur=at,at=null):ur=at.sibling;var qt=le(J,at,te[ct],Te);if(qt===null){at===null&&(at=ur);break}i&&at&&qt.alternate===null&&l(J,at),W=v(qt,W,ct),kt===null?Ze=qt:kt.sibling=qt,kt=qt,at=ur}if(ct===te.length)return u(J,at),jt&&xl(J,ct),Ze;if(at===null){for(;ct<te.length;ct++)at=xe(J,te[ct],Te),at!==null&&(W=v(at,W,ct),kt===null?Ze=at:kt.sibling=at,kt=at);return jt&&xl(J,ct),Ze}for(at=h(at);ct<te.length;ct++)ur=ue(at,J,ct,te[ct],Te),ur!==null&&(i&&ur.alternate!==null&&at.delete(ur.key===null?ct:ur.key),W=v(ur,W,ct),kt===null?Ze=ur:kt.sibling=ur,kt=ur);return i&&at.forEach(function(Fs){return l(J,Fs)}),jt&&xl(J,ct),Ze}function ut(J,W,te,Te){if(te==null)throw Error(r(151));for(var Ze=null,kt=null,at=W,ct=W=0,ur=null,qt=te.next();at!==null&&!qt.done;ct++,qt=te.next()){at.index>ct?(ur=at,at=null):ur=at.sibling;var Fs=le(J,at,qt.value,Te);if(Fs===null){at===null&&(at=ur);break}i&&at&&Fs.alternate===null&&l(J,at),W=v(Fs,W,ct),kt===null?Ze=Fs:kt.sibling=Fs,kt=Fs,at=ur}if(qt.done)return u(J,at),jt&&xl(J,ct),Ze;if(at===null){for(;!qt.done;ct++,qt=te.next())qt=xe(J,qt.value,Te),qt!==null&&(W=v(qt,W,ct),kt===null?Ze=qt:kt.sibling=qt,kt=qt);return jt&&xl(J,ct),Ze}for(at=h(at);!qt.done;ct++,qt=te.next())qt=ue(at,J,ct,qt.value,Te),qt!==null&&(i&&qt.alternate!==null&&at.delete(qt.key===null?ct:qt.key),W=v(qt,W,ct),kt===null?Ze=qt:kt.sibling=qt,kt=qt);return i&&at.forEach(function(TA){return l(J,TA)}),jt&&xl(J,ct),Ze}function en(J,W,te,Te){if(typeof te=="object"&&te!==null&&te.type===T&&te.key===null&&(te=te.props.children),typeof te=="object"&&te!==null){switch(te.$$typeof){case b:e:{for(var Ze=te.key;W!==null;){if(W.key===Ze){if(Ze=te.type,Ze===T){if(W.tag===7){u(J,W.sibling),Te=g(W,te.props.children),Te.return=J,J=Te;break e}}else if(W.elementType===Ze||typeof Ze=="object"&&Ze!==null&&Ze.$$typeof===Z&&G4(Ze)===W.type){u(J,W.sibling),Te=g(W,te.props),ec(Te,te),Te.return=J,J=Te;break e}u(J,W);break}else l(J,W);W=W.sibling}te.type===T?(Te=El(te.props.children,J.mode,Te,te.key),Te.return=J,J=Te):(Te=id(te.type,te.key,te.props,null,J.mode,Te),ec(Te,te),Te.return=J,J=Te)}return A(J);case y:e:{for(Ze=te.key;W!==null;){if(W.key===Ze)if(W.tag===4&&W.stateNode.containerInfo===te.containerInfo&&W.stateNode.implementation===te.implementation){u(J,W.sibling),Te=g(W,te.children||[]),Te.return=J,J=Te;break e}else{u(J,W);break}else l(J,W);W=W.sibling}Te=Om(te,J.mode,Te),Te.return=J,J=Te}return A(J);case Z:return Ze=te._init,te=Ze(te._payload),en(J,W,te,Te)}if(de(te))return ft(J,W,te,Te);if(oe(te)){if(Ze=oe(te),typeof Ze!="function")throw Error(r(150));return te=Ze.call(te),ut(J,W,te,Te)}if(typeof te.then=="function")return en(J,W,Td(te),Te);if(te.$$typeof===D)return en(J,W,ud(J,te),Te);xd(J,te)}return typeof te=="string"&&te!==""||typeof te=="number"||typeof te=="bigint"?(te=""+te,W!==null&&W.tag===6?(u(J,W.sibling),Te=g(W,te),Te.return=J,J=Te):(u(J,W),Te=Nm(te,J.mode,Te),Te.return=J,J=Te),A(J)):u(J,W)}return function(J,W,te,Te){try{Ju=0;var Ze=en(J,W,te,Te);return _o=null,Ze}catch(at){if(at===$u||at===dd)throw at;var kt=oa(29,at,null,J.mode);return kt.lanes=Te,kt.return=J,kt}finally{}}}var Eo=V4(!0),K4=V4(!1),Ca=ie(null),ni=null;function Ss(i){var l=i.alternate;L(rr,rr.current&1),L(Ca,i),ni===null&&(l===null||go.current!==null||l.memoizedState!==null)&&(ni=i)}function W4(i){if(i.tag===22){if(L(rr,rr.current),L(Ca,i),ni===null){var l=i.alternate;l!==null&&l.memoizedState!==null&&(ni=i)}}else ws()}function ws(){L(rr,rr.current),L(Ca,Ca.current)}function Li(i){ge(Ca),ni===i&&(ni=null),ge(rr)}var rr=ie(0);function Sd(i){for(var l=i;l!==null;){if(l.tag===13){var u=l.memoizedState;if(u!==null&&(u=u.dehydrated,u===null||u.data==="$?"||Zp(u)))return l}else if(l.tag===19&&l.memoizedProps.revealOrder!==void 0){if((l.flags&128)!==0)return l}else if(l.child!==null){l.child.return=l,l=l.child;continue}if(l===i)break;for(;l.sibling===null;){if(l.return===null||l.return===i)return null;l=l.return}l.sibling.return=l.return,l=l.sibling}return null}function up(i,l,u,h){l=i.memoizedState,u=u(h,l),u=u==null?l:m({},l,u),i.memoizedState=u,i.lanes===0&&(i.updateQueue.baseState=u)}var cp={enqueueSetState:function(i,l,u){i=i._reactInternals;var h=ha(),g=Es(h);g.payload=l,u!=null&&(g.callback=u),l=Ts(i,g,h),l!==null&&(fa(l,i,h),Gu(l,i,h))},enqueueReplaceState:function(i,l,u){i=i._reactInternals;var h=ha(),g=Es(h);g.tag=1,g.payload=l,u!=null&&(g.callback=u),l=Ts(i,g,h),l!==null&&(fa(l,i,h),Gu(l,i,h))},enqueueForceUpdate:function(i,l){i=i._reactInternals;var u=ha(),h=Es(u);h.tag=2,l!=null&&(h.callback=l),l=Ts(i,h,u),l!==null&&(fa(l,i,u),Gu(l,i,u))}};function X4(i,l,u,h,g,v,A){return i=i.stateNode,typeof i.shouldComponentUpdate=="function"?i.shouldComponentUpdate(h,v,A):l.prototype&&l.prototype.isPureReactComponent?!Pu(u,h)||!Pu(g,v):!0}function Q4(i,l,u,h){i=l.state,typeof l.componentWillReceiveProps=="function"&&l.componentWillReceiveProps(u,h),typeof l.UNSAFE_componentWillReceiveProps=="function"&&l.UNSAFE_componentWillReceiveProps(u,h),l.state!==i&&cp.enqueueReplaceState(l,l.state,null)}function Ol(i,l){var u=l;if("ref"in l){u={};for(var h in l)h!=="ref"&&(u[h]=l[h])}if(i=i.defaultProps){u===l&&(u=m({},u));for(var g in i)u[g]===void 0&&(u[g]=i[g])}return u}var wd=typeof reportError=="function"?reportError:function(i){if(typeof window=="object"&&typeof window.ErrorEvent=="function"){var l=new window.ErrorEvent("error",{bubbles:!0,cancelable:!0,message:typeof i=="object"&&i!==null&&typeof i.message=="string"?String(i.message):String(i),error:i});if(!window.dispatchEvent(l))return}else if(typeof process=="object"&&typeof process.emit=="function"){process.emit("uncaughtException",i);return}console.error(i)};function Z4(i){wd(i)}function J4(i){console.error(i)}function e3(i){wd(i)}function Ad(i,l){try{var u=i.onUncaughtError;u(l.value,{componentStack:l.stack})}catch(h){setTimeout(function(){throw h})}}function t3(i,l,u){try{var h=i.onCaughtError;h(u.value,{componentStack:u.stack,errorBoundary:l.tag===1?l.stateNode:null})}catch(g){setTimeout(function(){throw g})}}function dp(i,l,u){return u=Es(u),u.tag=3,u.payload={element:null},u.callback=function(){Ad(i,l)},u}function n3(i){return i=Es(i),i.tag=3,i}function r3(i,l,u,h){var g=u.type.getDerivedStateFromError;if(typeof g=="function"){var v=h.value;i.payload=function(){return g(v)},i.callback=function(){t3(l,u,h)}}var A=u.stateNode;A!==null&&typeof A.componentDidCatch=="function"&&(i.callback=function(){t3(l,u,h),typeof g!="function"&&(Rs===null?Rs=new Set([this]):Rs.add(this));var M=h.stack;this.componentDidCatch(h.value,{componentStack:M!==null?M:""})})}function Tw(i,l,u,h,g){if(u.flags|=32768,h!==null&&typeof h=="object"&&typeof h.then=="function"){if(l=u.alternate,l!==null&&Hu(l,u,g,!0),u=Ca.current,u!==null){switch(u.tag){case 13:return ni===null?Lp():u.alternate===null&&In===0&&(In=3),u.flags&=-257,u.flags|=65536,u.lanes=g,h===zm?u.flags|=16384:(l=u.updateQueue,l===null?u.updateQueue=new Set([h]):l.add(h),Bp(i,h,g)),!1;case 22:return u.flags|=65536,h===zm?u.flags|=16384:(l=u.updateQueue,l===null?(l={transitions:null,markerInstances:null,retryQueue:new Set([h])},u.updateQueue=l):(u=l.retryQueue,u===null?l.retryQueue=new Set([h]):u.add(h)),Bp(i,h,g)),!1}throw Error(r(435,u.tag))}return Bp(i,h,g),Lp(),!1}if(jt)return l=Ca.current,l!==null?((l.flags&65536)===0&&(l.flags|=256),l.flags|=65536,l.lanes=g,h!==Im&&(i=Error(r(422),{cause:h}),zu(Sa(i,u)))):(h!==Im&&(l=Error(r(423),{cause:h}),zu(Sa(l,u))),i=i.current.alternate,i.flags|=65536,g&=-g,i.lanes|=g,h=Sa(h,u),g=dp(i.stateNode,h,g),jm(i,g),In!==4&&(In=2)),!1;var v=Error(r(520),{cause:h});if(v=Sa(v,u),lc===null?lc=[v]:lc.push(v),In!==4&&(In=2),l===null)return!0;h=Sa(h,u),u=l;do{switch(u.tag){case 3:return u.flags|=65536,i=g&-g,u.lanes|=i,i=dp(u.stateNode,h,i),jm(u,i),!1;case 1:if(l=u.type,v=u.stateNode,(u.flags&128)===0&&(typeof l.getDerivedStateFromError=="function"||v!==null&&typeof v.componentDidCatch=="function"&&(Rs===null||!Rs.has(v))))return u.flags|=65536,g&=-g,u.lanes|=g,g=n3(g),r3(g,i,u,h),jm(u,g),!1}u=u.return}while(u!==null);return!1}var a3=Error(r(461)),lr=!1;function gr(i,l,u,h){l.child=i===null?K4(l,null,u,h):Eo(l,i.child,u,h)}function i3(i,l,u,h,g){u=u.render;var v=l.ref;if("ref"in h){var A={};for(var M in h)M!=="ref"&&(A[M]=h[M])}else A=h;return kl(l),h=Km(i,l,u,A,v,g),M=Wm(),i!==null&&!lr?(Xm(i,l,g),Pi(i,l,g)):(jt&&M&&Rm(l),l.flags|=1,gr(i,l,h,g),l.child)}function s3(i,l,u,h,g){if(i===null){var v=u.type;return typeof v=="function"&&!Cm(v)&&v.defaultProps===void 0&&u.compare===null?(l.tag=15,l.type=v,l3(i,l,v,h,g)):(i=id(u.type,null,h,l,l.mode,g),i.ref=l.ref,i.return=l,l.child=i)}if(v=i.child,!yp(i,g)){var A=v.memoizedProps;if(u=u.compare,u=u!==null?u:Pu,u(A,h)&&i.ref===l.ref)return Pi(i,l,g)}return l.flags|=1,i=Ni(v,h),i.ref=l.ref,i.return=l,l.child=i}function l3(i,l,u,h,g){if(i!==null){var v=i.memoizedProps;if(Pu(v,h)&&i.ref===l.ref)if(lr=!1,l.pendingProps=h=v,yp(i,g))(i.flags&131072)!==0&&(lr=!0);else return l.lanes=i.lanes,Pi(i,l,g)}return hp(i,l,u,h,g)}function o3(i,l,u){var h=l.pendingProps,g=h.children,v=i!==null?i.memoizedState:null;if(h.mode==="hidden"){if((l.flags&128)!==0){if(h=v!==null?v.baseLanes|u:u,i!==null){for(g=l.child=i.child,v=0;g!==null;)v=v|g.lanes|g.childLanes,g=g.sibling;l.childLanes=v&~h}else l.childLanes=0,l.child=null;return u3(i,l,h,u)}if((u&536870912)!==0)l.memoizedState={baseLanes:0,cachePool:null},i!==null&&cd(l,v!==null?v.cachePool:null),v!==null?l4(l,v):Ym(),W4(l);else return l.lanes=l.childLanes=536870912,u3(i,l,v!==null?v.baseLanes|u:u,u)}else v!==null?(cd(l,v.cachePool),l4(l,v),ws(),l.memoizedState=null):(i!==null&&cd(l,null),Ym(),ws());return gr(i,l,g,u),l.child}function u3(i,l,u,h){var g=Fm();return g=g===null?null:{parent:nr._currentValue,pool:g},l.memoizedState={baseLanes:u,cachePool:g},i!==null&&cd(l,null),Ym(),W4(l),i!==null&&Hu(i,l,h,!0),null}function kd(i,l){var u=l.ref;if(u===null)i!==null&&i.ref!==null&&(l.flags|=4194816);else{if(typeof u!="function"&&typeof u!="object")throw Error(r(284));(i===null||i.ref!==u)&&(l.flags|=4194816)}}function hp(i,l,u,h,g){return kl(l),u=Km(i,l,u,h,void 0,g),h=Wm(),i!==null&&!lr?(Xm(i,l,g),Pi(i,l,g)):(jt&&h&&Rm(l),l.flags|=1,gr(i,l,u,g),l.child)}function c3(i,l,u,h,g,v){return kl(l),l.updateQueue=null,u=u4(l,h,u,g),o4(i),h=Wm(),i!==null&&!lr?(Xm(i,l,v),Pi(i,l,v)):(jt&&h&&Rm(l),l.flags|=1,gr(i,l,u,v),l.child)}function d3(i,l,u,h,g){if(kl(l),l.stateNode===null){var v=co,A=u.contextType;typeof A=="object"&&A!==null&&(v=xr(A)),v=new u(h,v),l.memoizedState=v.state!==null&&v.state!==void 0?v.state:null,v.updater=cp,l.stateNode=v,v._reactInternals=l,v=l.stateNode,v.props=h,v.state=l.memoizedState,v.refs={},Hm(l),A=u.contextType,v.context=typeof A=="object"&&A!==null?xr(A):co,v.state=l.memoizedState,A=u.getDerivedStateFromProps,typeof A=="function"&&(up(l,u,A,h),v.state=l.memoizedState),typeof u.getDerivedStateFromProps=="function"||typeof v.getSnapshotBeforeUpdate=="function"||typeof v.UNSAFE_componentWillMount!="function"&&typeof v.componentWillMount!="function"||(A=v.state,typeof v.componentWillMount=="function"&&v.componentWillMount(),typeof v.UNSAFE_componentWillMount=="function"&&v.UNSAFE_componentWillMount(),A!==v.state&&cp.enqueueReplaceState(v,v.state,null),Ku(l,h,v,g),Vu(),v.state=l.memoizedState),typeof v.componentDidMount=="function"&&(l.flags|=4194308),h=!0}else if(i===null){v=l.stateNode;var M=l.memoizedProps,$=Ol(u,M);v.props=$;var re=v.context,_e=u.contextType;A=co,typeof _e=="object"&&_e!==null&&(A=xr(_e));var xe=u.getDerivedStateFromProps;_e=typeof xe=="function"||typeof v.getSnapshotBeforeUpdate=="function",M=l.pendingProps!==M,_e||typeof v.UNSAFE_componentWillReceiveProps!="function"&&typeof v.componentWillReceiveProps!="function"||(M||re!==A)&&Q4(l,v,h,A),_s=!1;var le=l.memoizedState;v.state=le,Ku(l,h,v,g),Vu(),re=l.memoizedState,M||le!==re||_s?(typeof xe=="function"&&(up(l,u,xe,h),re=l.memoizedState),($=_s||X4(l,u,$,h,le,re,A))?(_e||typeof v.UNSAFE_componentWillMount!="function"&&typeof v.componentWillMount!="function"||(typeof v.componentWillMount=="function"&&v.componentWillMount(),typeof v.UNSAFE_componentWillMount=="function"&&v.UNSAFE_componentWillMount()),typeof v.componentDidMount=="function"&&(l.flags|=4194308)):(typeof v.componentDidMount=="function"&&(l.flags|=4194308),l.memoizedProps=h,l.memoizedState=re),v.props=h,v.state=re,v.context=A,h=$):(typeof v.componentDidMount=="function"&&(l.flags|=4194308),h=!1)}else{v=l.stateNode,qm(i,l),A=l.memoizedProps,_e=Ol(u,A),v.props=_e,xe=l.pendingProps,le=v.context,re=u.contextType,$=co,typeof re=="object"&&re!==null&&($=xr(re)),M=u.getDerivedStateFromProps,(re=typeof M=="function"||typeof v.getSnapshotBeforeUpdate=="function")||typeof v.UNSAFE_componentWillReceiveProps!="function"&&typeof v.componentWillReceiveProps!="function"||(A!==xe||le!==$)&&Q4(l,v,h,$),_s=!1,le=l.memoizedState,v.state=le,Ku(l,h,v,g),Vu();var ue=l.memoizedState;A!==xe||le!==ue||_s||i!==null&&i.dependencies!==null&&od(i.dependencies)?(typeof M=="function"&&(up(l,u,M,h),ue=l.memoizedState),(_e=_s||X4(l,u,_e,h,le,ue,$)||i!==null&&i.dependencies!==null&&od(i.dependencies))?(re||typeof v.UNSAFE_componentWillUpdate!="function"&&typeof v.componentWillUpdate!="function"||(typeof v.componentWillUpdate=="function"&&v.componentWillUpdate(h,ue,$),typeof v.UNSAFE_componentWillUpdate=="function"&&v.UNSAFE_componentWillUpdate(h,ue,$)),typeof v.componentDidUpdate=="function"&&(l.flags|=4),typeof v.getSnapshotBeforeUpdate=="function"&&(l.flags|=1024)):(typeof v.componentDidUpdate!="function"||A===i.memoizedProps&&le===i.memoizedState||(l.flags|=4),typeof v.getSnapshotBeforeUpdate!="function"||A===i.memoizedProps&&le===i.memoizedState||(l.flags|=1024),l.memoizedProps=h,l.memoizedState=ue),v.props=h,v.state=ue,v.context=$,h=_e):(typeof v.componentDidUpdate!="function"||A===i.memoizedProps&&le===i.memoizedState||(l.flags|=4),typeof v.getSnapshotBeforeUpdate!="function"||A===i.memoizedProps&&le===i.memoizedState||(l.flags|=1024),h=!1)}return v=h,kd(i,l),h=(l.flags&128)!==0,v||h?(v=l.stateNode,u=h&&typeof u.getDerivedStateFromError!="function"?null:v.render(),l.flags|=1,i!==null&&h?(l.child=Eo(l,i.child,null,g),l.child=Eo(l,null,u,g)):gr(i,l,u,g),l.memoizedState=v.state,i=l.child):i=Pi(i,l,g),i}function h3(i,l,u,h){return Fu(),l.flags|=256,gr(i,l,u,h),l.child}var fp={dehydrated:null,treeContext:null,retryLane:0,hydrationErrors:null};function mp(i){return{baseLanes:i,cachePool:Jv()}}function pp(i,l,u){return i=i!==null?i.childLanes&~u:0,l&&(i|=Na),i}function f3(i,l,u){var h=l.pendingProps,g=!1,v=(l.flags&128)!==0,A;if((A=v)||(A=i!==null&&i.memoizedState===null?!1:(rr.current&2)!==0),A&&(g=!0,l.flags&=-129),A=(l.flags&32)!==0,l.flags&=-33,i===null){if(jt){if(g?Ss(l):ws(),jt){var M=Mn,$;if($=M){e:{for($=M,M=ti;$.nodeType!==8;){if(!M){M=null;break e}if($=Ga($.nextSibling),$===null){M=null;break e}}M=$}M!==null?(l.memoizedState={dehydrated:M,treeContext:Tl!==null?{id:Oi,overflow:Ri}:null,retryLane:536870912,hydrationErrors:null},$=oa(18,null,null,0),$.stateNode=M,$.return=l,l.child=$,Ir=l,Mn=null,$=!0):$=!1}$||wl(l)}if(M=l.memoizedState,M!==null&&(M=M.dehydrated,M!==null))return Zp(M)?l.lanes=32:l.lanes=536870912,null;Li(l)}return M=h.children,h=h.fallback,g?(ws(),g=l.mode,M=Cd({mode:"hidden",children:M},g),h=El(h,g,u,null),M.return=l,h.return=l,M.sibling=h,l.child=M,g=l.child,g.memoizedState=mp(u),g.childLanes=pp(i,A,u),l.memoizedState=fp,h):(Ss(l),gp(l,M))}if($=i.memoizedState,$!==null&&(M=$.dehydrated,M!==null)){if(v)l.flags&256?(Ss(l),l.flags&=-257,l=bp(i,l,u)):l.memoizedState!==null?(ws(),l.child=i.child,l.flags|=128,l=null):(ws(),g=h.fallback,M=l.mode,h=Cd({mode:"visible",children:h.children},M),g=El(g,M,u,null),g.flags|=2,h.return=l,g.return=l,h.sibling=g,l.child=h,Eo(l,i.child,null,u),h=l.child,h.memoizedState=mp(u),h.childLanes=pp(i,A,u),l.memoizedState=fp,l=g);else if(Ss(l),Zp(M)){if(A=M.nextSibling&&M.nextSibling.dataset,A)var re=A.dgst;A=re,h=Error(r(419)),h.stack="",h.digest=A,zu({value:h,source:null,stack:null}),l=bp(i,l,u)}else if(lr||Hu(i,l,u,!1),A=(u&i.childLanes)!==0,lr||A){if(A=un,A!==null&&(h=u&-u,h=(h&42)!==0?1:Fe(h),h=(h&(A.suspendedLanes|u))!==0?0:h,h!==0&&h!==$.retryLane))throw $.retryLane=h,uo(i,h),fa(A,i,h),a3;M.data==="$?"||Lp(),l=bp(i,l,u)}else M.data==="$?"?(l.flags|=192,l.child=i.child,l=null):(i=$.treeContext,Mn=Ga(M.nextSibling),Ir=l,jt=!0,Sl=null,ti=!1,i!==null&&(Aa[ka++]=Oi,Aa[ka++]=Ri,Aa[ka++]=Tl,Oi=i.id,Ri=i.overflow,Tl=l),l=gp(l,h.children),l.flags|=4096);return l}return g?(ws(),g=h.fallback,M=l.mode,$=i.child,re=$.sibling,h=Ni($,{mode:"hidden",children:h.children}),h.subtreeFlags=$.subtreeFlags&65011712,re!==null?g=Ni(re,g):(g=El(g,M,u,null),g.flags|=2),g.return=l,h.return=l,h.sibling=g,l.child=h,h=g,g=l.child,M=i.child.memoizedState,M===null?M=mp(u):($=M.cachePool,$!==null?(re=nr._currentValue,$=$.parent!==re?{parent:re,pool:re}:$):$=Jv(),M={baseLanes:M.baseLanes|u,cachePool:$}),g.memoizedState=M,g.childLanes=pp(i,A,u),l.memoizedState=fp,h):(Ss(l),u=i.child,i=u.sibling,u=Ni(u,{mode:"visible",children:h.children}),u.return=l,u.sibling=null,i!==null&&(A=l.deletions,A===null?(l.deletions=[i],l.flags|=16):A.push(i)),l.child=u,l.memoizedState=null,u)}function gp(i,l){return l=Cd({mode:"visible",children:l},i.mode),l.return=i,i.child=l}function Cd(i,l){return i=oa(22,i,null,l),i.lanes=0,i.stateNode={_visibility:1,_pendingMarkers:null,_retryCache:null,_transitions:null},i}function bp(i,l,u){return Eo(l,i.child,null,u),i=gp(l,l.pendingProps.children),i.flags|=2,l.memoizedState=null,i}function m3(i,l,u){i.lanes|=l;var h=i.alternate;h!==null&&(h.lanes|=l),Lm(i.return,l,u)}function vp(i,l,u,h,g){var v=i.memoizedState;v===null?i.memoizedState={isBackwards:l,rendering:null,renderingStartTime:0,last:h,tail:u,tailMode:g}:(v.isBackwards=l,v.rendering=null,v.renderingStartTime=0,v.last=h,v.tail=u,v.tailMode=g)}function p3(i,l,u){var h=l.pendingProps,g=h.revealOrder,v=h.tail;if(gr(i,l,h.children,u),h=rr.current,(h&2)!==0)h=h&1|2,l.flags|=128;else{if(i!==null&&(i.flags&128)!==0)e:for(i=l.child;i!==null;){if(i.tag===13)i.memoizedState!==null&&m3(i,u,l);else if(i.tag===19)m3(i,u,l);else if(i.child!==null){i.child.return=i,i=i.child;continue}if(i===l)break e;for(;i.sibling===null;){if(i.return===null||i.return===l)break e;i=i.return}i.sibling.return=i.return,i=i.sibling}h&=1}switch(L(rr,h),g){case"forwards":for(u=l.child,g=null;u!==null;)i=u.alternate,i!==null&&Sd(i)===null&&(g=u),u=u.sibling;u=g,u===null?(g=l.child,l.child=null):(g=u.sibling,u.sibling=null),vp(l,!1,g,u,v);break;case"backwards":for(u=null,g=l.child,l.child=null;g!==null;){if(i=g.alternate,i!==null&&Sd(i)===null){l.child=g;break}i=g.sibling,g.sibling=u,u=g,g=i}vp(l,!0,u,null,v);break;case"together":vp(l,!1,null,null,void 0);break;default:l.memoizedState=null}return l.child}function Pi(i,l,u){if(i!==null&&(l.dependencies=i.dependencies),Os|=l.lanes,(u&l.childLanes)===0)if(i!==null){if(Hu(i,l,u,!1),(u&l.childLanes)===0)return null}else return null;if(i!==null&&l.child!==i.child)throw Error(r(153));if(l.child!==null){for(i=l.child,u=Ni(i,i.pendingProps),l.child=u,u.return=l;i.sibling!==null;)i=i.sibling,u=u.sibling=Ni(i,i.pendingProps),u.return=l;u.sibling=null}return l.child}function yp(i,l){return(i.lanes&l)!==0?!0:(i=i.dependencies,!!(i!==null&&od(i)))}function xw(i,l,u){switch(l.tag){case 3:ot(l,l.stateNode.containerInfo),ys(l,nr,i.memoizedState.cache),Fu();break;case 27:case 5:wt(l);break;case 4:ot(l,l.stateNode.containerInfo);break;case 10:ys(l,l.type,l.memoizedProps.value);break;case 13:var h=l.memoizedState;if(h!==null)return h.dehydrated!==null?(Ss(l),l.flags|=128,null):(u&l.child.childLanes)!==0?f3(i,l,u):(Ss(l),i=Pi(i,l,u),i!==null?i.sibling:null);Ss(l);break;case 19:var g=(i.flags&128)!==0;if(h=(u&l.childLanes)!==0,h||(Hu(i,l,u,!1),h=(u&l.childLanes)!==0),g){if(h)return p3(i,l,u);l.flags|=128}if(g=l.memoizedState,g!==null&&(g.rendering=null,g.tail=null,g.lastEffect=null),L(rr,rr.current),h)break;return null;case 22:case 23:return l.lanes=0,o3(i,l,u);case 24:ys(l,nr,i.memoizedState.cache)}return Pi(i,l,u)}function g3(i,l,u){if(i!==null)if(i.memoizedProps!==l.pendingProps)lr=!0;else{if(!yp(i,u)&&(l.flags&128)===0)return lr=!1,xw(i,l,u);lr=(i.flags&131072)!==0}else lr=!1,jt&&(l.flags&1048576)!==0&&Gv(l,ld,l.index);switch(l.lanes=0,l.tag){case 16:e:{i=l.pendingProps;var h=l.elementType,g=h._init;if(h=g(h._payload),l.type=h,typeof h=="function")Cm(h)?(i=Ol(h,i),l.tag=1,l=d3(null,l,h,i,u)):(l.tag=0,l=hp(null,l,h,i,u));else{if(h!=null){if(g=h.$$typeof,g===Y){l.tag=11,l=i3(null,l,h,i,u);break e}else if(g===ee){l.tag=14,l=s3(null,l,h,i,u);break e}}throw l=fe(h)||h,Error(r(306,l,""))}}return l;case 0:return hp(i,l,l.type,l.pendingProps,u);case 1:return h=l.type,g=Ol(h,l.pendingProps),d3(i,l,h,g,u);case 3:e:{if(ot(l,l.stateNode.containerInfo),i===null)throw Error(r(387));h=l.pendingProps;var v=l.memoizedState;g=v.element,qm(i,l),Ku(l,h,null,u);var A=l.memoizedState;if(h=A.cache,ys(l,nr,h),h!==v.cache&&Pm(l,[nr],u,!0),Vu(),h=A.element,v.isDehydrated)if(v={element:h,isDehydrated:!1,cache:A.cache},l.updateQueue.baseState=v,l.memoizedState=v,l.flags&256){l=h3(i,l,h,u);break e}else if(h!==g){g=Sa(Error(r(424)),l),zu(g),l=h3(i,l,h,u);break e}else{switch(i=l.stateNode.containerInfo,i.nodeType){case 9:i=i.body;break;default:i=i.nodeName==="HTML"?i.ownerDocument.body:i}for(Mn=Ga(i.firstChild),Ir=l,jt=!0,Sl=null,ti=!0,u=K4(l,null,h,u),l.child=u;u;)u.flags=u.flags&-3|4096,u=u.sibling}else{if(Fu(),h===g){l=Pi(i,l,u);break e}gr(i,l,h,u)}l=l.child}return l;case 26:return kd(i,l),i===null?(u=_y(l.type,null,l.pendingProps,null))?l.memoizedState=u:jt||(u=l.type,i=l.pendingProps,h=qd(De.current).createElement(u),h[Pe]=l,h[et]=i,vr(h,u,i),An(h),l.stateNode=h):l.memoizedState=_y(l.type,i.memoizedProps,l.pendingProps,i.memoizedState),null;case 27:return wt(l),i===null&&jt&&(h=l.stateNode=by(l.type,l.pendingProps,De.current),Ir=l,ti=!0,g=Mn,Ds(l.type)?(Jp=g,Mn=Ga(h.firstChild)):Mn=g),gr(i,l,l.pendingProps.children,u),kd(i,l),i===null&&(l.flags|=4194304),l.child;case 5:return i===null&&jt&&((g=h=Mn)&&(h=Qw(h,l.type,l.pendingProps,ti),h!==null?(l.stateNode=h,Ir=l,Mn=Ga(h.firstChild),ti=!1,g=!0):g=!1),g||wl(l)),wt(l),g=l.type,v=l.pendingProps,A=i!==null?i.memoizedProps:null,h=v.children,Wp(g,v)?h=null:A!==null&&Wp(g,A)&&(l.flags|=32),l.memoizedState!==null&&(g=Km(i,l,pw,null,null,u),gc._currentValue=g),kd(i,l),gr(i,l,h,u),l.child;case 6:return i===null&&jt&&((i=u=Mn)&&(u=Zw(u,l.pendingProps,ti),u!==null?(l.stateNode=u,Ir=l,Mn=null,i=!0):i=!1),i||wl(l)),null;case 13:return f3(i,l,u);case 4:return ot(l,l.stateNode.containerInfo),h=l.pendingProps,i===null?l.child=Eo(l,null,h,u):gr(i,l,h,u),l.child;case 11:return i3(i,l,l.type,l.pendingProps,u);case 7:return gr(i,l,l.pendingProps,u),l.child;case 8:return gr(i,l,l.pendingProps.children,u),l.child;case 12:return gr(i,l,l.pendingProps.children,u),l.child;case 10:return h=l.pendingProps,ys(l,l.type,h.value),gr(i,l,h.children,u),l.child;case 9:return g=l.type._context,h=l.pendingProps.children,kl(l),g=xr(g),h=h(g),l.flags|=1,gr(i,l,h,u),l.child;case 14:return s3(i,l,l.type,l.pendingProps,u);case 15:return l3(i,l,l.type,l.pendingProps,u);case 19:return p3(i,l,u);case 31:return h=l.pendingProps,u=l.mode,h={mode:h.mode,children:h.children},i===null?(u=Cd(h,u),u.ref=l.ref,l.child=u,u.return=l,l=u):(u=Ni(i.child,h),u.ref=l.ref,l.child=u,u.return=l,l=u),l;case 22:return o3(i,l,u);case 24:return kl(l),h=xr(nr),i===null?(g=Fm(),g===null&&(g=un,v=Bm(),g.pooledCache=v,v.refCount++,v!==null&&(g.pooledCacheLanes|=u),g=v),l.memoizedState={parent:h,cache:g},Hm(l),ys(l,nr,g)):((i.lanes&u)!==0&&(qm(i,l),Ku(l,null,null,u),Vu()),g=i.memoizedState,v=l.memoizedState,g.parent!==h?(g={parent:h,cache:h},l.memoizedState=g,l.lanes===0&&(l.memoizedState=l.updateQueue.baseState=g),ys(l,nr,h)):(h=v.cache,ys(l,nr,h),h!==g.cache&&Pm(l,[nr],u,!0))),gr(i,l,l.pendingProps.children,u),l.child;case 29:throw l.pendingProps}throw Error(r(156,l.tag))}function Bi(i){i.flags|=4}function b3(i,l){if(l.type!=="stylesheet"||(l.state.loading&4)!==0)i.flags&=-16777217;else if(i.flags|=16777216,!wy(l)){if(l=Ca.current,l!==null&&((Pt&4194048)===Pt?ni!==null:(Pt&62914560)!==Pt&&(Pt&536870912)===0||l!==ni))throw Yu=zm,e4;i.flags|=8192}}function Nd(i,l){l!==null&&(i.flags|=4),i.flags&16384&&(l=i.tag!==22?xu():536870912,i.lanes|=l,wo|=l)}function tc(i,l){if(!jt)switch(i.tailMode){case"hidden":l=i.tail;for(var u=null;l!==null;)l.alternate!==null&&(u=l),l=l.sibling;u===null?i.tail=null:u.sibling=null;break;case"collapsed":u=i.tail;for(var h=null;u!==null;)u.alternate!==null&&(h=u),u=u.sibling;h===null?l||i.tail===null?i.tail=null:i.tail.sibling=null:h.sibling=null}}function Cn(i){var l=i.alternate!==null&&i.alternate.child===i.child,u=0,h=0;if(l)for(var g=i.child;g!==null;)u|=g.lanes|g.childLanes,h|=g.subtreeFlags&65011712,h|=g.flags&65011712,g.return=i,g=g.sibling;else for(g=i.child;g!==null;)u|=g.lanes|g.childLanes,h|=g.subtreeFlags,h|=g.flags,g.return=i,g=g.sibling;return i.subtreeFlags|=h,i.childLanes=u,l}function Sw(i,l,u){var h=l.pendingProps;switch(Mm(l),l.tag){case 31:case 16:case 15:case 0:case 11:case 7:case 8:case 12:case 9:case 14:return Cn(l),null;case 1:return Cn(l),null;case 3:return u=l.stateNode,h=null,i!==null&&(h=i.memoizedState.cache),l.memoizedState.cache!==h&&(l.flags|=2048),Ii(nr),zt(),u.pendingContext&&(u.context=u.pendingContext,u.pendingContext=null),(i===null||i.child===null)&&(Uu(l)?Bi(l):i===null||i.memoizedState.isDehydrated&&(l.flags&256)===0||(l.flags|=1024,Wv())),Cn(l),null;case 26:return u=l.memoizedState,i===null?(Bi(l),u!==null?(Cn(l),b3(l,u)):(Cn(l),l.flags&=-16777217)):u?u!==i.memoizedState?(Bi(l),Cn(l),b3(l,u)):(Cn(l),l.flags&=-16777217):(i.memoizedProps!==h&&Bi(l),Cn(l),l.flags&=-16777217),null;case 27:vn(l),u=De.current;var g=l.type;if(i!==null&&l.stateNode!=null)i.memoizedProps!==h&&Bi(l);else{if(!h){if(l.stateNode===null)throw Error(r(166));return Cn(l),null}i=Oe.current,Uu(l)?Vv(l):(i=by(g,h,u),l.stateNode=i,Bi(l))}return Cn(l),null;case 5:if(vn(l),u=l.type,i!==null&&l.stateNode!=null)i.memoizedProps!==h&&Bi(l);else{if(!h){if(l.stateNode===null)throw Error(r(166));return Cn(l),null}if(i=Oe.current,Uu(l))Vv(l);else{switch(g=qd(De.current),i){case 1:i=g.createElementNS("http://www.w3.org/2000/svg",u);break;case 2:i=g.createElementNS("http://www.w3.org/1998/Math/MathML",u);break;default:switch(u){case"svg":i=g.createElementNS("http://www.w3.org/2000/svg",u);break;case"math":i=g.createElementNS("http://www.w3.org/1998/Math/MathML",u);break;case"script":i=g.createElement("div"),i.innerHTML="<script><\/script>",i=i.removeChild(i.firstChild);break;case"select":i=typeof h.is=="string"?g.createElement("select",{is:h.is}):g.createElement("select"),h.multiple?i.multiple=!0:h.size&&(i.size=h.size);break;default:i=typeof h.is=="string"?g.createElement(u,{is:h.is}):g.createElement(u)}}i[Pe]=l,i[et]=h;e:for(g=l.child;g!==null;){if(g.tag===5||g.tag===6)i.appendChild(g.stateNode);else if(g.tag!==4&&g.tag!==27&&g.child!==null){g.child.return=g,g=g.child;continue}if(g===l)break e;for(;g.sibling===null;){if(g.return===null||g.return===l)break e;g=g.return}g.sibling.return=g.return,g=g.sibling}l.stateNode=i;e:switch(vr(i,u,h),u){case"button":case"input":case"select":case"textarea":i=!!h.autoFocus;break e;case"img":i=!0;break e;default:i=!1}i&&Bi(l)}}return Cn(l),l.flags&=-16777217,null;case 6:if(i&&l.stateNode!=null)i.memoizedProps!==h&&Bi(l);else{if(typeof h!="string"&&l.stateNode===null)throw Error(r(166));if(i=De.current,Uu(l)){if(i=l.stateNode,u=l.memoizedProps,h=null,g=Ir,g!==null)switch(g.tag){case 27:case 5:h=g.memoizedProps}i[Pe]=l,i=!!(i.nodeValue===u||h!==null&&h.suppressHydrationWarning===!0||cy(i.nodeValue,u)),i||wl(l)}else i=qd(i).createTextNode(h),i[Pe]=l,l.stateNode=i}return Cn(l),null;case 13:if(h=l.memoizedState,i===null||i.memoizedState!==null&&i.memoizedState.dehydrated!==null){if(g=Uu(l),h!==null&&h.dehydrated!==null){if(i===null){if(!g)throw Error(r(318));if(g=l.memoizedState,g=g!==null?g.dehydrated:null,!g)throw Error(r(317));g[Pe]=l}else Fu(),(l.flags&128)===0&&(l.memoizedState=null),l.flags|=4;Cn(l),g=!1}else g=Wv(),i!==null&&i.memoizedState!==null&&(i.memoizedState.hydrationErrors=g),g=!0;if(!g)return l.flags&256?(Li(l),l):(Li(l),null)}if(Li(l),(l.flags&128)!==0)return l.lanes=u,l;if(u=h!==null,i=i!==null&&i.memoizedState!==null,u){h=l.child,g=null,h.alternate!==null&&h.alternate.memoizedState!==null&&h.alternate.memoizedState.cachePool!==null&&(g=h.alternate.memoizedState.cachePool.pool);var v=null;h.memoizedState!==null&&h.memoizedState.cachePool!==null&&(v=h.memoizedState.cachePool.pool),v!==g&&(h.flags|=2048)}return u!==i&&u&&(l.child.flags|=8192),Nd(l,l.updateQueue),Cn(l),null;case 4:return zt(),i===null&&$p(l.stateNode.containerInfo),Cn(l),null;case 10:return Ii(l.type),Cn(l),null;case 19:if(ge(rr),g=l.memoizedState,g===null)return Cn(l),null;if(h=(l.flags&128)!==0,v=g.rendering,v===null)if(h)tc(g,!1);else{if(In!==0||i!==null&&(i.flags&128)!==0)for(i=l.child;i!==null;){if(v=Sd(i),v!==null){for(l.flags|=128,tc(g,!1),i=v.updateQueue,l.updateQueue=i,Nd(l,i),l.subtreeFlags=0,i=u,u=l.child;u!==null;)Yv(u,i),u=u.sibling;return L(rr,rr.current&1|2),l.child}i=i.sibling}g.tail!==null&&yt()>Md&&(l.flags|=128,h=!0,tc(g,!1),l.lanes=4194304)}else{if(!h)if(i=Sd(v),i!==null){if(l.flags|=128,h=!0,i=i.updateQueue,l.updateQueue=i,Nd(l,i),tc(g,!0),g.tail===null&&g.tailMode==="hidden"&&!v.alternate&&!jt)return Cn(l),null}else 2*yt()-g.renderingStartTime>Md&&u!==536870912&&(l.flags|=128,h=!0,tc(g,!1),l.lanes=4194304);g.isBackwards?(v.sibling=l.child,l.child=v):(i=g.last,i!==null?i.sibling=v:l.child=v,g.last=v)}return g.tail!==null?(l=g.tail,g.rendering=l,g.tail=l.sibling,g.renderingStartTime=yt(),l.sibling=null,i=rr.current,L(rr,h?i&1|2:i&1),l):(Cn(l),null);case 22:case 23:return Li(l),Gm(),h=l.memoizedState!==null,i!==null?i.memoizedState!==null!==h&&(l.flags|=8192):h&&(l.flags|=8192),h?(u&536870912)!==0&&(l.flags&128)===0&&(Cn(l),l.subtreeFlags&6&&(l.flags|=8192)):Cn(l),u=l.updateQueue,u!==null&&Nd(l,u.retryQueue),u=null,i!==null&&i.memoizedState!==null&&i.memoizedState.cachePool!==null&&(u=i.memoizedState.cachePool.pool),h=null,l.memoizedState!==null&&l.memoizedState.cachePool!==null&&(h=l.memoizedState.cachePool.pool),h!==u&&(l.flags|=2048),i!==null&&ge(Cl),null;case 24:return u=null,i!==null&&(u=i.memoizedState.cache),l.memoizedState.cache!==u&&(l.flags|=2048),Ii(nr),Cn(l),null;case 25:return null;case 30:return null}throw Error(r(156,l.tag))}function ww(i,l){switch(Mm(l),l.tag){case 1:return i=l.flags,i&65536?(l.flags=i&-65537|128,l):null;case 3:return Ii(nr),zt(),i=l.flags,(i&65536)!==0&&(i&128)===0?(l.flags=i&-65537|128,l):null;case 26:case 27:case 5:return vn(l),null;case 13:if(Li(l),i=l.memoizedState,i!==null&&i.dehydrated!==null){if(l.alternate===null)throw Error(r(340));Fu()}return i=l.flags,i&65536?(l.flags=i&-65537|128,l):null;case 19:return ge(rr),null;case 4:return zt(),null;case 10:return Ii(l.type),null;case 22:case 23:return Li(l),Gm(),i!==null&&ge(Cl),i=l.flags,i&65536?(l.flags=i&-65537|128,l):null;case 24:return Ii(nr),null;case 25:return null;default:return null}}function v3(i,l){switch(Mm(l),l.tag){case 3:Ii(nr),zt();break;case 26:case 27:case 5:vn(l);break;case 4:zt();break;case 13:Li(l);break;case 19:ge(rr);break;case 10:Ii(l.type);break;case 22:case 23:Li(l),Gm(),i!==null&&ge(Cl);break;case 24:Ii(nr)}}function nc(i,l){try{var u=l.updateQueue,h=u!==null?u.lastEffect:null;if(h!==null){var g=h.next;u=g;do{if((u.tag&i)===i){h=void 0;var v=u.create,A=u.inst;h=v(),A.destroy=h}u=u.next}while(u!==g)}}catch(M){rn(l,l.return,M)}}function As(i,l,u){try{var h=l.updateQueue,g=h!==null?h.lastEffect:null;if(g!==null){var v=g.next;h=v;do{if((h.tag&i)===i){var A=h.inst,M=A.destroy;if(M!==void 0){A.destroy=void 0,g=l;var $=u,re=M;try{re()}catch(_e){rn(g,$,_e)}}}h=h.next}while(h!==v)}}catch(_e){rn(l,l.return,_e)}}function y3(i){var l=i.updateQueue;if(l!==null){var u=i.stateNode;try{s4(l,u)}catch(h){rn(i,i.return,h)}}}function _3(i,l,u){u.props=Ol(i.type,i.memoizedProps),u.state=i.memoizedState;try{u.componentWillUnmount()}catch(h){rn(i,l,h)}}function rc(i,l){try{var u=i.ref;if(u!==null){switch(i.tag){case 26:case 27:case 5:var h=i.stateNode;break;case 30:h=i.stateNode;break;default:h=i.stateNode}typeof u=="function"?i.refCleanup=u(h):u.current=h}}catch(g){rn(i,l,g)}}function ri(i,l){var u=i.ref,h=i.refCleanup;if(u!==null)if(typeof h=="function")try{h()}catch(g){rn(i,l,g)}finally{i.refCleanup=null,i=i.alternate,i!=null&&(i.refCleanup=null)}else if(typeof u=="function")try{u(null)}catch(g){rn(i,l,g)}else u.current=null}function E3(i){var l=i.type,u=i.memoizedProps,h=i.stateNode;try{e:switch(l){case"button":case"input":case"select":case"textarea":u.autoFocus&&h.focus();break e;case"img":u.src?h.src=u.src:u.srcSet&&(h.srcset=u.srcSet)}}catch(g){rn(i,i.return,g)}}function _p(i,l,u){try{var h=i.stateNode;Gw(h,i.type,u,l),h[et]=l}catch(g){rn(i,i.return,g)}}function T3(i){return i.tag===5||i.tag===3||i.tag===26||i.tag===27&&Ds(i.type)||i.tag===4}function Ep(i){e:for(;;){for(;i.sibling===null;){if(i.return===null||T3(i.return))return null;i=i.return}for(i.sibling.return=i.return,i=i.sibling;i.tag!==5&&i.tag!==6&&i.tag!==18;){if(i.tag===27&&Ds(i.type)||i.flags&2||i.child===null||i.tag===4)continue e;i.child.return=i,i=i.child}if(!(i.flags&2))return i.stateNode}}function Tp(i,l,u){var h=i.tag;if(h===5||h===6)i=i.stateNode,l?(u.nodeType===9?u.body:u.nodeName==="HTML"?u.ownerDocument.body:u).insertBefore(i,l):(l=u.nodeType===9?u.body:u.nodeName==="HTML"?u.ownerDocument.body:u,l.appendChild(i),u=u._reactRootContainer,u!=null||l.onclick!==null||(l.onclick=Hd));else if(h!==4&&(h===27&&Ds(i.type)&&(u=i.stateNode,l=null),i=i.child,i!==null))for(Tp(i,l,u),i=i.sibling;i!==null;)Tp(i,l,u),i=i.sibling}function Od(i,l,u){var h=i.tag;if(h===5||h===6)i=i.stateNode,l?u.insertBefore(i,l):u.appendChild(i);else if(h!==4&&(h===27&&Ds(i.type)&&(u=i.stateNode),i=i.child,i!==null))for(Od(i,l,u),i=i.sibling;i!==null;)Od(i,l,u),i=i.sibling}function x3(i){var l=i.stateNode,u=i.memoizedProps;try{for(var h=i.type,g=l.attributes;g.length;)l.removeAttributeNode(g[0]);vr(l,h,u),l[Pe]=i,l[et]=u}catch(v){rn(i,i.return,v)}}var Ui=!1,Un=!1,xp=!1,S3=typeof WeakSet=="function"?WeakSet:Set,or=null;function Aw(i,l){if(i=i.containerInfo,Vp=Kd,i=Lv(i),Em(i)){if("selectionStart"in i)var u={start:i.selectionStart,end:i.selectionEnd};else e:{u=(u=i.ownerDocument)&&u.defaultView||window;var h=u.getSelection&&u.getSelection();if(h&&h.rangeCount!==0){u=h.anchorNode;var g=h.anchorOffset,v=h.focusNode;h=h.focusOffset;try{u.nodeType,v.nodeType}catch{u=null;break e}var A=0,M=-1,$=-1,re=0,_e=0,xe=i,le=null;t:for(;;){for(var ue;xe!==u||g!==0&&xe.nodeType!==3||(M=A+g),xe!==v||h!==0&&xe.nodeType!==3||($=A+h),xe.nodeType===3&&(A+=xe.nodeValue.length),(ue=xe.firstChild)!==null;)le=xe,xe=ue;for(;;){if(xe===i)break t;if(le===u&&++re===g&&(M=A),le===v&&++_e===h&&($=A),(ue=xe.nextSibling)!==null)break;xe=le,le=xe.parentNode}xe=ue}u=M===-1||$===-1?null:{start:M,end:$}}else u=null}u=u||{start:0,end:0}}else u=null;for(Kp={focusedElem:i,selectionRange:u},Kd=!1,or=l;or!==null;)if(l=or,i=l.child,(l.subtreeFlags&1024)!==0&&i!==null)i.return=l,or=i;else for(;or!==null;){switch(l=or,v=l.alternate,i=l.flags,l.tag){case 0:break;case 11:case 15:break;case 1:if((i&1024)!==0&&v!==null){i=void 0,u=l,g=v.memoizedProps,v=v.memoizedState,h=u.stateNode;try{var ft=Ol(u.type,g,u.elementType===u.type);i=h.getSnapshotBeforeUpdate(ft,v),h.__reactInternalSnapshotBeforeUpdate=i}catch(ut){rn(u,u.return,ut)}}break;case 3:if((i&1024)!==0){if(i=l.stateNode.containerInfo,u=i.nodeType,u===9)Qp(i);else if(u===1)switch(i.nodeName){case"HEAD":case"HTML":case"BODY":Qp(i);break;default:i.textContent=""}}break;case 5:case 26:case 27:case 6:case 4:case 17:break;default:if((i&1024)!==0)throw Error(r(163))}if(i=l.sibling,i!==null){i.return=l.return,or=i;break}or=l.return}}function w3(i,l,u){var h=u.flags;switch(u.tag){case 0:case 11:case 15:ks(i,u),h&4&&nc(5,u);break;case 1:if(ks(i,u),h&4)if(i=u.stateNode,l===null)try{i.componentDidMount()}catch(A){rn(u,u.return,A)}else{var g=Ol(u.type,l.memoizedProps);l=l.memoizedState;try{i.componentDidUpdate(g,l,i.__reactInternalSnapshotBeforeUpdate)}catch(A){rn(u,u.return,A)}}h&64&&y3(u),h&512&&rc(u,u.return);break;case 3:if(ks(i,u),h&64&&(i=u.updateQueue,i!==null)){if(l=null,u.child!==null)switch(u.child.tag){case 27:case 5:l=u.child.stateNode;break;case 1:l=u.child.stateNode}try{s4(i,l)}catch(A){rn(u,u.return,A)}}break;case 27:l===null&&h&4&&x3(u);case 26:case 5:ks(i,u),l===null&&h&4&&E3(u),h&512&&rc(u,u.return);break;case 12:ks(i,u);break;case 13:ks(i,u),h&4&&C3(i,u),h&64&&(i=u.memoizedState,i!==null&&(i=i.dehydrated,i!==null&&(u=Lw.bind(null,u),Jw(i,u))));break;case 22:if(h=u.memoizedState!==null||Ui,!h){l=l!==null&&l.memoizedState!==null||Un,g=Ui;var v=Un;Ui=h,(Un=l)&&!v?Cs(i,u,(u.subtreeFlags&8772)!==0):ks(i,u),Ui=g,Un=v}break;case 30:break;default:ks(i,u)}}function A3(i){var l=i.alternate;l!==null&&(i.alternate=null,A3(l)),i.child=null,i.deletions=null,i.sibling=null,i.tag===5&&(l=i.stateNode,l!==null&&hl(l)),i.stateNode=null,i.return=null,i.dependencies=null,i.memoizedProps=null,i.memoizedState=null,i.pendingProps=null,i.stateNode=null,i.updateQueue=null}var En=null,Wr=!1;function Fi(i,l,u){for(u=u.child;u!==null;)k3(i,l,u),u=u.sibling}function k3(i,l,u){if(Se&&typeof Se.onCommitFiberUnmount=="function")try{Se.onCommitFiberUnmount(he,u)}catch{}switch(u.tag){case 26:Un||ri(u,l),Fi(i,l,u),u.memoizedState?u.memoizedState.count--:u.stateNode&&(u=u.stateNode,u.parentNode.removeChild(u));break;case 27:Un||ri(u,l);var h=En,g=Wr;Ds(u.type)&&(En=u.stateNode,Wr=!1),Fi(i,l,u),hc(u.stateNode),En=h,Wr=g;break;case 5:Un||ri(u,l);case 6:if(h=En,g=Wr,En=null,Fi(i,l,u),En=h,Wr=g,En!==null)if(Wr)try{(En.nodeType===9?En.body:En.nodeName==="HTML"?En.ownerDocument.body:En).removeChild(u.stateNode)}catch(v){rn(u,l,v)}else try{En.removeChild(u.stateNode)}catch(v){rn(u,l,v)}break;case 18:En!==null&&(Wr?(i=En,py(i.nodeType===9?i.body:i.nodeName==="HTML"?i.ownerDocument.body:i,u.stateNode),_c(i)):py(En,u.stateNode));break;case 4:h=En,g=Wr,En=u.stateNode.containerInfo,Wr=!0,Fi(i,l,u),En=h,Wr=g;break;case 0:case 11:case 14:case 15:Un||As(2,u,l),Un||As(4,u,l),Fi(i,l,u);break;case 1:Un||(ri(u,l),h=u.stateNode,typeof h.componentWillUnmount=="function"&&_3(u,l,h)),Fi(i,l,u);break;case 21:Fi(i,l,u);break;case 22:Un=(h=Un)||u.memoizedState!==null,Fi(i,l,u),Un=h;break;default:Fi(i,l,u)}}function C3(i,l){if(l.memoizedState===null&&(i=l.alternate,i!==null&&(i=i.memoizedState,i!==null&&(i=i.dehydrated,i!==null))))try{_c(i)}catch(u){rn(l,l.return,u)}}function kw(i){switch(i.tag){case 13:case 19:var l=i.stateNode;return l===null&&(l=i.stateNode=new S3),l;case 22:return i=i.stateNode,l=i._retryCache,l===null&&(l=i._retryCache=new S3),l;default:throw Error(r(435,i.tag))}}function Sp(i,l){var u=kw(i);l.forEach(function(h){var g=Pw.bind(null,i,h);u.has(h)||(u.add(h),h.then(g,g))})}function ua(i,l){var u=l.deletions;if(u!==null)for(var h=0;h<u.length;h++){var g=u[h],v=i,A=l,M=A;e:for(;M!==null;){switch(M.tag){case 27:if(Ds(M.type)){En=M.stateNode,Wr=!1;break e}break;case 5:En=M.stateNode,Wr=!1;break e;case 3:case 4:En=M.stateNode.containerInfo,Wr=!0;break e}M=M.return}if(En===null)throw Error(r(160));k3(v,A,g),En=null,Wr=!1,v=g.alternate,v!==null&&(v.return=null),g.return=null}if(l.subtreeFlags&13878)for(l=l.child;l!==null;)N3(l,i),l=l.sibling}var Ya=null;function N3(i,l){var u=i.alternate,h=i.flags;switch(i.tag){case 0:case 11:case 14:case 15:ua(l,i),ca(i),h&4&&(As(3,i,i.return),nc(3,i),As(5,i,i.return));break;case 1:ua(l,i),ca(i),h&512&&(Un||u===null||ri(u,u.return)),h&64&&Ui&&(i=i.updateQueue,i!==null&&(h=i.callbacks,h!==null&&(u=i.shared.hiddenCallbacks,i.shared.hiddenCallbacks=u===null?h:u.concat(h))));break;case 26:var g=Ya;if(ua(l,i),ca(i),h&512&&(Un||u===null||ri(u,u.return)),h&4){var v=u!==null?u.memoizedState:null;if(h=i.memoizedState,u===null)if(h===null)if(i.stateNode===null){e:{h=i.type,u=i.memoizedProps,g=g.ownerDocument||g;t:switch(h){case"title":v=g.getElementsByTagName("title")[0],(!v||v[dl]||v[Pe]||v.namespaceURI==="http://www.w3.org/2000/svg"||v.hasAttribute("itemprop"))&&(v=g.createElement(h),g.head.insertBefore(v,g.querySelector("head > title"))),vr(v,h,u),v[Pe]=i,An(v),h=v;break e;case"link":var A=xy("link","href",g).get(h+(u.href||""));if(A){for(var M=0;M<A.length;M++)if(v=A[M],v.getAttribute("href")===(u.href==null||u.href===""?null:u.href)&&v.getAttribute("rel")===(u.rel==null?null:u.rel)&&v.getAttribute("title")===(u.title==null?null:u.title)&&v.getAttribute("crossorigin")===(u.crossOrigin==null?null:u.crossOrigin)){A.splice(M,1);break t}}v=g.createElement(h),vr(v,h,u),g.head.appendChild(v);break;case"meta":if(A=xy("meta","content",g).get(h+(u.content||""))){for(M=0;M<A.length;M++)if(v=A[M],v.getAttribute("content")===(u.content==null?null:""+u.content)&&v.getAttribute("name")===(u.name==null?null:u.name)&&v.getAttribute("property")===(u.property==null?null:u.property)&&v.getAttribute("http-equiv")===(u.httpEquiv==null?null:u.httpEquiv)&&v.getAttribute("charset")===(u.charSet==null?null:u.charSet)){A.splice(M,1);break t}}v=g.createElement(h),vr(v,h,u),g.head.appendChild(v);break;default:throw Error(r(468,h))}v[Pe]=i,An(v),h=v}i.stateNode=h}else Sy(g,i.type,i.stateNode);else i.stateNode=Ty(g,h,i.memoizedProps);else v!==h?(v===null?u.stateNode!==null&&(u=u.stateNode,u.parentNode.removeChild(u)):v.count--,h===null?Sy(g,i.type,i.stateNode):Ty(g,h,i.memoizedProps)):h===null&&i.stateNode!==null&&_p(i,i.memoizedProps,u.memoizedProps)}break;case 27:ua(l,i),ca(i),h&512&&(Un||u===null||ri(u,u.return)),u!==null&&h&4&&_p(i,i.memoizedProps,u.memoizedProps);break;case 5:if(ua(l,i),ca(i),h&512&&(Un||u===null||ri(u,u.return)),i.flags&32){g=i.stateNode;try{ei(g,"")}catch(ue){rn(i,i.return,ue)}}h&4&&i.stateNode!=null&&(g=i.memoizedProps,_p(i,g,u!==null?u.memoizedProps:g)),h&1024&&(xp=!0);break;case 6:if(ua(l,i),ca(i),h&4){if(i.stateNode===null)throw Error(r(162));h=i.memoizedProps,u=i.stateNode;try{u.nodeValue=h}catch(ue){rn(i,i.return,ue)}}break;case 3:if(Yd=null,g=Ya,Ya=jd(l.containerInfo),ua(l,i),Ya=g,ca(i),h&4&&u!==null&&u.memoizedState.isDehydrated)try{_c(l.containerInfo)}catch(ue){rn(i,i.return,ue)}xp&&(xp=!1,O3(i));break;case 4:h=Ya,Ya=jd(i.stateNode.containerInfo),ua(l,i),ca(i),Ya=h;break;case 12:ua(l,i),ca(i);break;case 13:ua(l,i),ca(i),i.child.flags&8192&&i.memoizedState!==null!=(u!==null&&u.memoizedState!==null)&&(Op=yt()),h&4&&(h=i.updateQueue,h!==null&&(i.updateQueue=null,Sp(i,h)));break;case 22:g=i.memoizedState!==null;var $=u!==null&&u.memoizedState!==null,re=Ui,_e=Un;if(Ui=re||g,Un=_e||$,ua(l,i),Un=_e,Ui=re,ca(i),h&8192)e:for(l=i.stateNode,l._visibility=g?l._visibility&-2:l._visibility|1,g&&(u===null||$||Ui||Un||Rl(i)),u=null,l=i;;){if(l.tag===5||l.tag===26){if(u===null){$=u=l;try{if(v=$.stateNode,g)A=v.style,typeof A.setProperty=="function"?A.setProperty("display","none","important"):A.display="none";else{M=$.stateNode;var xe=$.memoizedProps.style,le=xe!=null&&xe.hasOwnProperty("display")?xe.display:null;M.style.display=le==null||typeof le=="boolean"?"":(""+le).trim()}}catch(ue){rn($,$.return,ue)}}}else if(l.tag===6){if(u===null){$=l;try{$.stateNode.nodeValue=g?"":$.memoizedProps}catch(ue){rn($,$.return,ue)}}}else if((l.tag!==22&&l.tag!==23||l.memoizedState===null||l===i)&&l.child!==null){l.child.return=l,l=l.child;continue}if(l===i)break e;for(;l.sibling===null;){if(l.return===null||l.return===i)break e;u===l&&(u=null),l=l.return}u===l&&(u=null),l.sibling.return=l.return,l=l.sibling}h&4&&(h=i.updateQueue,h!==null&&(u=h.retryQueue,u!==null&&(h.retryQueue=null,Sp(i,u))));break;case 19:ua(l,i),ca(i),h&4&&(h=i.updateQueue,h!==null&&(i.updateQueue=null,Sp(i,h)));break;case 30:break;case 21:break;default:ua(l,i),ca(i)}}function ca(i){var l=i.flags;if(l&2){try{for(var u,h=i.return;h!==null;){if(T3(h)){u=h;break}h=h.return}if(u==null)throw Error(r(160));switch(u.tag){case 27:var g=u.stateNode,v=Ep(i);Od(i,v,g);break;case 5:var A=u.stateNode;u.flags&32&&(ei(A,""),u.flags&=-33);var M=Ep(i);Od(i,M,A);break;case 3:case 4:var $=u.stateNode.containerInfo,re=Ep(i);Tp(i,re,$);break;default:throw Error(r(161))}}catch(_e){rn(i,i.return,_e)}i.flags&=-3}l&4096&&(i.flags&=-4097)}function O3(i){if(i.subtreeFlags&1024)for(i=i.child;i!==null;){var l=i;O3(l),l.tag===5&&l.flags&1024&&l.stateNode.reset(),i=i.sibling}}function ks(i,l){if(l.subtreeFlags&8772)for(l=l.child;l!==null;)w3(i,l.alternate,l),l=l.sibling}function Rl(i){for(i=i.child;i!==null;){var l=i;switch(l.tag){case 0:case 11:case 14:case 15:As(4,l,l.return),Rl(l);break;case 1:ri(l,l.return);var u=l.stateNode;typeof u.componentWillUnmount=="function"&&_3(l,l.return,u),Rl(l);break;case 27:hc(l.stateNode);case 26:case 5:ri(l,l.return),Rl(l);break;case 22:l.memoizedState===null&&Rl(l);break;case 30:Rl(l);break;default:Rl(l)}i=i.sibling}}function Cs(i,l,u){for(u=u&&(l.subtreeFlags&8772)!==0,l=l.child;l!==null;){var h=l.alternate,g=i,v=l,A=v.flags;switch(v.tag){case 0:case 11:case 15:Cs(g,v,u),nc(4,v);break;case 1:if(Cs(g,v,u),h=v,g=h.stateNode,typeof g.componentDidMount=="function")try{g.componentDidMount()}catch(re){rn(h,h.return,re)}if(h=v,g=h.updateQueue,g!==null){var M=h.stateNode;try{var $=g.shared.hiddenCallbacks;if($!==null)for(g.shared.hiddenCallbacks=null,g=0;g<$.length;g++)i4($[g],M)}catch(re){rn(h,h.return,re)}}u&&A&64&&y3(v),rc(v,v.return);break;case 27:x3(v);case 26:case 5:Cs(g,v,u),u&&h===null&&A&4&&E3(v),rc(v,v.return);break;case 12:Cs(g,v,u);break;case 13:Cs(g,v,u),u&&A&4&&C3(g,v);break;case 22:v.memoizedState===null&&Cs(g,v,u),rc(v,v.return);break;case 30:break;default:Cs(g,v,u)}l=l.sibling}}function wp(i,l){var u=null;i!==null&&i.memoizedState!==null&&i.memoizedState.cachePool!==null&&(u=i.memoizedState.cachePool.pool),i=null,l.memoizedState!==null&&l.memoizedState.cachePool!==null&&(i=l.memoizedState.cachePool.pool),i!==u&&(i!=null&&i.refCount++,u!=null&&qu(u))}function Ap(i,l){i=null,l.alternate!==null&&(i=l.alternate.memoizedState.cache),l=l.memoizedState.cache,l!==i&&(l.refCount++,i!=null&&qu(i))}function ai(i,l,u,h){if(l.subtreeFlags&10256)for(l=l.child;l!==null;)R3(i,l,u,h),l=l.sibling}function R3(i,l,u,h){var g=l.flags;switch(l.tag){case 0:case 11:case 15:ai(i,l,u,h),g&2048&&nc(9,l);break;case 1:ai(i,l,u,h);break;case 3:ai(i,l,u,h),g&2048&&(i=null,l.alternate!==null&&(i=l.alternate.memoizedState.cache),l=l.memoizedState.cache,l!==i&&(l.refCount++,i!=null&&qu(i)));break;case 12:if(g&2048){ai(i,l,u,h),i=l.stateNode;try{var v=l.memoizedProps,A=v.id,M=v.onPostCommit;typeof M=="function"&&M(A,l.alternate===null?"mount":"update",i.passiveEffectDuration,-0)}catch($){rn(l,l.return,$)}}else ai(i,l,u,h);break;case 13:ai(i,l,u,h);break;case 23:break;case 22:v=l.stateNode,A=l.alternate,l.memoizedState!==null?v._visibility&2?ai(i,l,u,h):ac(i,l):v._visibility&2?ai(i,l,u,h):(v._visibility|=2,To(i,l,u,h,(l.subtreeFlags&10256)!==0)),g&2048&&wp(A,l);break;case 24:ai(i,l,u,h),g&2048&&Ap(l.alternate,l);break;default:ai(i,l,u,h)}}function To(i,l,u,h,g){for(g=g&&(l.subtreeFlags&10256)!==0,l=l.child;l!==null;){var v=i,A=l,M=u,$=h,re=A.flags;switch(A.tag){case 0:case 11:case 15:To(v,A,M,$,g),nc(8,A);break;case 23:break;case 22:var _e=A.stateNode;A.memoizedState!==null?_e._visibility&2?To(v,A,M,$,g):ac(v,A):(_e._visibility|=2,To(v,A,M,$,g)),g&&re&2048&&wp(A.alternate,A);break;case 24:To(v,A,M,$,g),g&&re&2048&&Ap(A.alternate,A);break;default:To(v,A,M,$,g)}l=l.sibling}}function ac(i,l){if(l.subtreeFlags&10256)for(l=l.child;l!==null;){var u=i,h=l,g=h.flags;switch(h.tag){case 22:ac(u,h),g&2048&&wp(h.alternate,h);break;case 24:ac(u,h),g&2048&&Ap(h.alternate,h);break;default:ac(u,h)}l=l.sibling}}var ic=8192;function xo(i){if(i.subtreeFlags&ic)for(i=i.child;i!==null;)M3(i),i=i.sibling}function M3(i){switch(i.tag){case 26:xo(i),i.flags&ic&&i.memoizedState!==null&&hA(Ya,i.memoizedState,i.memoizedProps);break;case 5:xo(i);break;case 3:case 4:var l=Ya;Ya=jd(i.stateNode.containerInfo),xo(i),Ya=l;break;case 22:i.memoizedState===null&&(l=i.alternate,l!==null&&l.memoizedState!==null?(l=ic,ic=16777216,xo(i),ic=l):xo(i));break;default:xo(i)}}function I3(i){var l=i.alternate;if(l!==null&&(i=l.child,i!==null)){l.child=null;do l=i.sibling,i.sibling=null,i=l;while(i!==null)}}function sc(i){var l=i.deletions;if((i.flags&16)!==0){if(l!==null)for(var u=0;u<l.length;u++){var h=l[u];or=h,L3(h,i)}I3(i)}if(i.subtreeFlags&10256)for(i=i.child;i!==null;)D3(i),i=i.sibling}function D3(i){switch(i.tag){case 0:case 11:case 15:sc(i),i.flags&2048&&As(9,i,i.return);break;case 3:sc(i);break;case 12:sc(i);break;case 22:var l=i.stateNode;i.memoizedState!==null&&l._visibility&2&&(i.return===null||i.return.tag!==13)?(l._visibility&=-3,Rd(i)):sc(i);break;default:sc(i)}}function Rd(i){var l=i.deletions;if((i.flags&16)!==0){if(l!==null)for(var u=0;u<l.length;u++){var h=l[u];or=h,L3(h,i)}I3(i)}for(i=i.child;i!==null;){switch(l=i,l.tag){case 0:case 11:case 15:As(8,l,l.return),Rd(l);break;case 22:u=l.stateNode,u._visibility&2&&(u._visibility&=-3,Rd(l));break;default:Rd(l)}i=i.sibling}}function L3(i,l){for(;or!==null;){var u=or;switch(u.tag){case 0:case 11:case 15:As(8,u,l);break;case 23:case 22:if(u.memoizedState!==null&&u.memoizedState.cachePool!==null){var h=u.memoizedState.cachePool.pool;h!=null&&h.refCount++}break;case 24:qu(u.memoizedState.cache)}if(h=u.child,h!==null)h.return=u,or=h;else e:for(u=i;or!==null;){h=or;var g=h.sibling,v=h.return;if(A3(h),h===u){or=null;break e}if(g!==null){g.return=v,or=g;break e}or=v}}}var Cw={getCacheForType:function(i){var l=xr(nr),u=l.data.get(i);return u===void 0&&(u=i(),l.data.set(i,u)),u}},Nw=typeof WeakMap=="function"?WeakMap:Map,Kt=0,un=null,Rt=null,Pt=0,Wt=0,da=null,Ns=!1,So=!1,kp=!1,zi=0,In=0,Os=0,Ml=0,Cp=0,Na=0,wo=0,lc=null,Xr=null,Np=!1,Op=0,Md=1/0,Id=null,Rs=null,br=0,Ms=null,Ao=null,ko=0,Rp=0,Mp=null,P3=null,oc=0,Ip=null;function ha(){if((Kt&2)!==0&&Pt!==0)return Pt&-Pt;if(V.T!==null){var i=mo;return i!==0?i:zp()}return yn()}function B3(){Na===0&&(Na=(Pt&536870912)===0||jt?Tu():536870912);var i=Ca.current;return i!==null&&(i.flags|=32),Na}function fa(i,l,u){(i===un&&(Wt===2||Wt===9)||i.cancelPendingCommit!==null)&&(Co(i,0),Is(i,Pt,Na,!1)),hs(i,u),((Kt&2)===0||i!==un)&&(i===un&&((Kt&2)===0&&(Ml|=u),In===4&&Is(i,Pt,Na,!1)),ii(i))}function U3(i,l,u){if((Kt&6)!==0)throw Error(r(327));var h=!u&&(l&124)===0&&(l&i.expiredLanes)===0||Yr(i,l),g=h?Mw(i,l):Pp(i,l,!0),v=h;do{if(g===0){So&&!h&&Is(i,l,0,!1);break}else{if(u=i.current.alternate,v&&!Ow(u)){g=Pp(i,l,!1),v=!1;continue}if(g===2){if(v=l,i.errorRecoveryDisabledLanes&v)var A=0;else A=i.pendingLanes&-536870913,A=A!==0?A:A&536870912?536870912:0;if(A!==0){l=A;e:{var M=i;g=lc;var $=M.current.memoizedState.isDehydrated;if($&&(Co(M,A).flags|=256),A=Pp(M,A,!1),A!==2){if(kp&&!$){M.errorRecoveryDisabledLanes|=v,Ml|=v,g=4;break e}v=Xr,Xr=g,v!==null&&(Xr===null?Xr=v:Xr.push.apply(Xr,v))}g=A}if(v=!1,g!==2)continue}}if(g===1){Co(i,0),Is(i,l,0,!0);break}e:{switch(h=i,v=g,v){case 0:case 1:throw Error(r(345));case 4:if((l&4194048)!==l)break;case 6:Is(h,l,Na,!Ns);break e;case 2:Xr=null;break;case 3:case 5:break;default:throw Error(r(329))}if((l&62914560)===l&&(g=Op+300-yt(),10<g)){if(Is(h,l,Na,!Ns),jn(h,0,!0)!==0)break e;h.timeoutHandle=fy(F3.bind(null,h,u,Xr,Id,Np,l,Na,Ml,wo,Ns,v,2,-0,0),g);break e}F3(h,u,Xr,Id,Np,l,Na,Ml,wo,Ns,v,0,-0,0)}}break}while(!0);ii(i)}function F3(i,l,u,h,g,v,A,M,$,re,_e,xe,le,ue){if(i.timeoutHandle=-1,xe=l.subtreeFlags,(xe&8192||(xe&16785408)===16785408)&&(pc={stylesheets:null,count:0,unsuspend:dA},M3(l),xe=fA(),xe!==null)){i.cancelPendingCommit=xe(G3.bind(null,i,l,v,u,h,g,A,M,$,_e,1,le,ue)),Is(i,v,A,!re);return}G3(i,l,v,u,h,g,A,M,$)}function Ow(i){for(var l=i;;){var u=l.tag;if((u===0||u===11||u===15)&&l.flags&16384&&(u=l.updateQueue,u!==null&&(u=u.stores,u!==null)))for(var h=0;h<u.length;h++){var g=u[h],v=g.getSnapshot;g=g.value;try{if(!la(v(),g))return!1}catch{return!1}}if(u=l.child,l.subtreeFlags&16384&&u!==null)u.return=l,l=u;else{if(l===i)break;for(;l.sibling===null;){if(l.return===null||l.return===i)return!0;l=l.return}l.sibling.return=l.return,l=l.sibling}}return!0}function Is(i,l,u,h){l&=~Cp,l&=~Ml,i.suspendedLanes|=l,i.pingedLanes&=~l,h&&(i.warmLanes|=l),h=i.expirationTimes;for(var g=l;0<g;){var v=31-ze(g),A=1<<v;h[v]=-1,g&=~A}u!==0&&K(i,u,l)}function Dd(){return(Kt&6)===0?(uc(0),!1):!0}function Dp(){if(Rt!==null){if(Wt===0)var i=Rt.return;else i=Rt,Mi=Al=null,Qm(i),_o=null,Ju=0,i=Rt;for(;i!==null;)v3(i.alternate,i),i=i.return;Rt=null}}function Co(i,l){var u=i.timeoutHandle;u!==-1&&(i.timeoutHandle=-1,Kw(u)),u=i.cancelPendingCommit,u!==null&&(i.cancelPendingCommit=null,u()),Dp(),un=i,Rt=u=Ni(i.current,null),Pt=l,Wt=0,da=null,Ns=!1,So=Yr(i,l),kp=!1,wo=Na=Cp=Ml=Os=In=0,Xr=lc=null,Np=!1,(l&8)!==0&&(l|=l&32);var h=i.entangledLanes;if(h!==0)for(i=i.entanglements,h&=l;0<h;){var g=31-ze(h),v=1<<g;l|=i[g],h&=~v}return zi=l,nd(),u}function z3(i,l){At=null,V.H=Ed,l===$u||l===dd?(l=r4(),Wt=3):l===e4?(l=r4(),Wt=4):Wt=l===a3?8:l!==null&&typeof l=="object"&&typeof l.then=="function"?6:1,da=l,Rt===null&&(In=1,Ad(i,Sa(l,i.current)))}function H3(){var i=V.H;return V.H=Ed,i===null?Ed:i}function q3(){var i=V.A;return V.A=Cw,i}function Lp(){In=4,Ns||(Pt&4194048)!==Pt&&Ca.current!==null||(So=!0),(Os&134217727)===0&&(Ml&134217727)===0||un===null||Is(un,Pt,Na,!1)}function Pp(i,l,u){var h=Kt;Kt|=2;var g=H3(),v=q3();(un!==i||Pt!==l)&&(Id=null,Co(i,l)),l=!1;var A=In;e:do try{if(Wt!==0&&Rt!==null){var M=Rt,$=da;switch(Wt){case 8:Dp(),A=6;break e;case 3:case 2:case 9:case 6:Ca.current===null&&(l=!0);var re=Wt;if(Wt=0,da=null,No(i,M,$,re),u&&So){A=0;break e}break;default:re=Wt,Wt=0,da=null,No(i,M,$,re)}}Rw(),A=In;break}catch(_e){z3(i,_e)}while(!0);return l&&i.shellSuspendCounter++,Mi=Al=null,Kt=h,V.H=g,V.A=v,Rt===null&&(un=null,Pt=0,nd()),A}function Rw(){for(;Rt!==null;)j3(Rt)}function Mw(i,l){var u=Kt;Kt|=2;var h=H3(),g=q3();un!==i||Pt!==l?(Id=null,Md=yt()+500,Co(i,l)):So=Yr(i,l);e:do try{if(Wt!==0&&Rt!==null){l=Rt;var v=da;t:switch(Wt){case 1:Wt=0,da=null,No(i,l,v,1);break;case 2:case 9:if(t4(v)){Wt=0,da=null,$3(l);break}l=function(){Wt!==2&&Wt!==9||un!==i||(Wt=7),ii(i)},v.then(l,l);break e;case 3:Wt=7;break e;case 4:Wt=5;break e;case 7:t4(v)?(Wt=0,da=null,$3(l)):(Wt=0,da=null,No(i,l,v,7));break;case 5:var A=null;switch(Rt.tag){case 26:A=Rt.memoizedState;case 5:case 27:var M=Rt;if(!A||wy(A)){Wt=0,da=null;var $=M.sibling;if($!==null)Rt=$;else{var re=M.return;re!==null?(Rt=re,Ld(re)):Rt=null}break t}}Wt=0,da=null,No(i,l,v,5);break;case 6:Wt=0,da=null,No(i,l,v,6);break;case 8:Dp(),In=6;break e;default:throw Error(r(462))}}Iw();break}catch(_e){z3(i,_e)}while(!0);return Mi=Al=null,V.H=h,V.A=g,Kt=u,Rt!==null?0:(un=null,Pt=0,nd(),In)}function Iw(){for(;Rt!==null&&!Bt();)j3(Rt)}function j3(i){var l=g3(i.alternate,i,zi);i.memoizedProps=i.pendingProps,l===null?Ld(i):Rt=l}function $3(i){var l=i,u=l.alternate;switch(l.tag){case 15:case 0:l=c3(u,l,l.pendingProps,l.type,void 0,Pt);break;case 11:l=c3(u,l,l.pendingProps,l.type.render,l.ref,Pt);break;case 5:Qm(l);default:v3(u,l),l=Rt=Yv(l,zi),l=g3(u,l,zi)}i.memoizedProps=i.pendingProps,l===null?Ld(i):Rt=l}function No(i,l,u,h){Mi=Al=null,Qm(l),_o=null,Ju=0;var g=l.return;try{if(Tw(i,g,l,u,Pt)){In=1,Ad(i,Sa(u,i.current)),Rt=null;return}}catch(v){if(g!==null)throw Rt=g,v;In=1,Ad(i,Sa(u,i.current)),Rt=null;return}l.flags&32768?(jt||h===1?i=!0:So||(Pt&536870912)!==0?i=!1:(Ns=i=!0,(h===2||h===9||h===3||h===6)&&(h=Ca.current,h!==null&&h.tag===13&&(h.flags|=16384))),Y3(l,i)):Ld(l)}function Ld(i){var l=i;do{if((l.flags&32768)!==0){Y3(l,Ns);return}i=l.return;var u=Sw(l.alternate,l,zi);if(u!==null){Rt=u;return}if(l=l.sibling,l!==null){Rt=l;return}Rt=l=i}while(l!==null);In===0&&(In=5)}function Y3(i,l){do{var u=ww(i.alternate,i);if(u!==null){u.flags&=32767,Rt=u;return}if(u=i.return,u!==null&&(u.flags|=32768,u.subtreeFlags=0,u.deletions=null),!l&&(i=i.sibling,i!==null)){Rt=i;return}Rt=i=u}while(i!==null);In=6,Rt=null}function G3(i,l,u,h,g,v,A,M,$){i.cancelPendingCommit=null;do Pd();while(br!==0);if((Kt&6)!==0)throw Error(r(327));if(l!==null){if(l===i.current)throw Error(r(177));if(v=l.lanes|l.childLanes,v|=Am,fs(i,u,v,A,M,$),i===un&&(Rt=un=null,Pt=0),Ao=l,Ms=i,ko=u,Rp=v,Mp=g,P3=h,(l.subtreeFlags&10256)!==0||(l.flags&10256)!==0?(i.callbackNode=null,i.callbackPriority=0,Bw(We,function(){return Q3(),null})):(i.callbackNode=null,i.callbackPriority=0),h=(l.flags&13878)!==0,(l.subtreeFlags&13878)!==0||h){h=V.T,V.T=null,g=me.p,me.p=2,A=Kt,Kt|=4;try{Aw(i,l,u)}finally{Kt=A,me.p=g,V.T=h}}br=1,V3(),K3(),W3()}}function V3(){if(br===1){br=0;var i=Ms,l=Ao,u=(l.flags&13878)!==0;if((l.subtreeFlags&13878)!==0||u){u=V.T,V.T=null;var h=me.p;me.p=2;var g=Kt;Kt|=4;try{N3(l,i);var v=Kp,A=Lv(i.containerInfo),M=v.focusedElem,$=v.selectionRange;if(A!==M&&M&&M.ownerDocument&&Dv(M.ownerDocument.documentElement,M)){if($!==null&&Em(M)){var re=$.start,_e=$.end;if(_e===void 0&&(_e=re),"selectionStart"in M)M.selectionStart=re,M.selectionEnd=Math.min(_e,M.value.length);else{var xe=M.ownerDocument||document,le=xe&&xe.defaultView||window;if(le.getSelection){var ue=le.getSelection(),ft=M.textContent.length,ut=Math.min($.start,ft),en=$.end===void 0?ut:Math.min($.end,ft);!ue.extend&&ut>en&&(A=en,en=ut,ut=A);var J=Iv(M,ut),W=Iv(M,en);if(J&&W&&(ue.rangeCount!==1||ue.anchorNode!==J.node||ue.anchorOffset!==J.offset||ue.focusNode!==W.node||ue.focusOffset!==W.offset)){var te=xe.createRange();te.setStart(J.node,J.offset),ue.removeAllRanges(),ut>en?(ue.addRange(te),ue.extend(W.node,W.offset)):(te.setEnd(W.node,W.offset),ue.addRange(te))}}}}for(xe=[],ue=M;ue=ue.parentNode;)ue.nodeType===1&&xe.push({element:ue,left:ue.scrollLeft,top:ue.scrollTop});for(typeof M.focus=="function"&&M.focus(),M=0;M<xe.length;M++){var Te=xe[M];Te.element.scrollLeft=Te.left,Te.element.scrollTop=Te.top}}Kd=!!Vp,Kp=Vp=null}finally{Kt=g,me.p=h,V.T=u}}i.current=l,br=2}}function K3(){if(br===2){br=0;var i=Ms,l=Ao,u=(l.flags&8772)!==0;if((l.subtreeFlags&8772)!==0||u){u=V.T,V.T=null;var h=me.p;me.p=2;var g=Kt;Kt|=4;try{w3(i,l.alternate,l)}finally{Kt=g,me.p=h,V.T=u}}br=3}}function W3(){if(br===4||br===3){br=0,Je();var i=Ms,l=Ao,u=ko,h=P3;(l.subtreeFlags&10256)!==0||(l.flags&10256)!==0?br=5:(br=0,Ao=Ms=null,X3(i,i.pendingLanes));var g=i.pendingLanes;if(g===0&&(Rs=null),ht(u),l=l.stateNode,Se&&typeof Se.onCommitFiberRoot=="function")try{Se.onCommitFiberRoot(he,l,void 0,(l.current.flags&128)===128)}catch{}if(h!==null){l=V.T,g=me.p,me.p=2,V.T=null;try{for(var v=i.onRecoverableError,A=0;A<h.length;A++){var M=h[A];v(M.value,{componentStack:M.stack})}}finally{V.T=l,me.p=g}}(ko&3)!==0&&Pd(),ii(i),g=i.pendingLanes,(u&4194090)!==0&&(g&42)!==0?i===Ip?oc++:(oc=0,Ip=i):oc=0,uc(0)}}function X3(i,l){(i.pooledCacheLanes&=l)===0&&(l=i.pooledCache,l!=null&&(i.pooledCache=null,qu(l)))}function Pd(i){return V3(),K3(),W3(),Q3()}function Q3(){if(br!==5)return!1;var i=Ms,l=Rp;Rp=0;var u=ht(ko),h=V.T,g=me.p;try{me.p=32>u?32:u,V.T=null,u=Mp,Mp=null;var v=Ms,A=ko;if(br=0,Ao=Ms=null,ko=0,(Kt&6)!==0)throw Error(r(331));var M=Kt;if(Kt|=4,D3(v.current),R3(v,v.current,A,u),Kt=M,uc(0,!1),Se&&typeof Se.onPostCommitFiberRoot=="function")try{Se.onPostCommitFiberRoot(he,v)}catch{}return!0}finally{me.p=g,V.T=h,X3(i,l)}}function Z3(i,l,u){l=Sa(u,l),l=dp(i.stateNode,l,2),i=Ts(i,l,2),i!==null&&(hs(i,2),ii(i))}function rn(i,l,u){if(i.tag===3)Z3(i,i,u);else for(;l!==null;){if(l.tag===3){Z3(l,i,u);break}else if(l.tag===1){var h=l.stateNode;if(typeof l.type.getDerivedStateFromError=="function"||typeof h.componentDidCatch=="function"&&(Rs===null||!Rs.has(h))){i=Sa(u,i),u=n3(2),h=Ts(l,u,2),h!==null&&(r3(u,h,l,i),hs(h,2),ii(h));break}}l=l.return}}function Bp(i,l,u){var h=i.pingCache;if(h===null){h=i.pingCache=new Nw;var g=new Set;h.set(l,g)}else g=h.get(l),g===void 0&&(g=new Set,h.set(l,g));g.has(u)||(kp=!0,g.add(u),i=Dw.bind(null,i,l,u),l.then(i,i))}function Dw(i,l,u){var h=i.pingCache;h!==null&&h.delete(l),i.pingedLanes|=i.suspendedLanes&u,i.warmLanes&=~u,un===i&&(Pt&u)===u&&(In===4||In===3&&(Pt&62914560)===Pt&&300>yt()-Op?(Kt&2)===0&&Co(i,0):Cp|=u,wo===Pt&&(wo=0)),ii(i)}function J3(i,l){l===0&&(l=xu()),i=uo(i,l),i!==null&&(hs(i,l),ii(i))}function Lw(i){var l=i.memoizedState,u=0;l!==null&&(u=l.retryLane),J3(i,u)}function Pw(i,l){var u=0;switch(i.tag){case 13:var h=i.stateNode,g=i.memoizedState;g!==null&&(u=g.retryLane);break;case 19:h=i.stateNode;break;case 22:h=i.stateNode._retryCache;break;default:throw Error(r(314))}h!==null&&h.delete(l),J3(i,u)}function Bw(i,l){return vt(i,l)}var Bd=null,Oo=null,Up=!1,Ud=!1,Fp=!1,Il=0;function ii(i){i!==Oo&&i.next===null&&(Oo===null?Bd=Oo=i:Oo=Oo.next=i),Ud=!0,Up||(Up=!0,Fw())}function uc(i,l){if(!Fp&&Ud){Fp=!0;do for(var u=!1,h=Bd;h!==null;){if(i!==0){var g=h.pendingLanes;if(g===0)var v=0;else{var A=h.suspendedLanes,M=h.pingedLanes;v=(1<<31-ze(42|i)+1)-1,v&=g&~(A&~M),v=v&201326741?v&201326741|1:v?v|2:0}v!==0&&(u=!0,ry(h,v))}else v=Pt,v=jn(h,h===un?v:0,h.cancelPendingCommit!==null||h.timeoutHandle!==-1),(v&3)===0||Yr(h,v)||(u=!0,ry(h,v));h=h.next}while(u);Fp=!1}}function Uw(){ey()}function ey(){Ud=Up=!1;var i=0;Il!==0&&(Vw()&&(i=Il),Il=0);for(var l=yt(),u=null,h=Bd;h!==null;){var g=h.next,v=ty(h,l);v===0?(h.next=null,u===null?Bd=g:u.next=g,g===null&&(Oo=u)):(u=h,(i!==0||(v&3)!==0)&&(Ud=!0)),h=g}uc(i)}function ty(i,l){for(var u=i.suspendedLanes,h=i.pingedLanes,g=i.expirationTimes,v=i.pendingLanes&-62914561;0<v;){var A=31-ze(v),M=1<<A,$=g[A];$===-1?((M&u)===0||(M&h)!==0)&&(g[A]=Rr(M,l)):$<=l&&(i.expiredLanes|=M),v&=~M}if(l=un,u=Pt,u=jn(i,i===l?u:0,i.cancelPendingCommit!==null||i.timeoutHandle!==-1),h=i.callbackNode,u===0||i===l&&(Wt===2||Wt===9)||i.cancelPendingCommit!==null)return h!==null&&h!==null&&gt(h),i.callbackNode=null,i.callbackPriority=0;if((u&3)===0||Yr(i,u)){if(l=u&-u,l===i.callbackPriority)return l;switch(h!==null&&gt(h),ht(u)){case 2:case 8:u=ln;break;case 32:u=We;break;case 268435456:u=qn;break;default:u=We}return h=ny.bind(null,i),u=vt(u,h),i.callbackPriority=l,i.callbackNode=u,l}return h!==null&&h!==null&&gt(h),i.callbackPriority=2,i.callbackNode=null,2}function ny(i,l){if(br!==0&&br!==5)return i.callbackNode=null,i.callbackPriority=0,null;var u=i.callbackNode;if(Pd()&&i.callbackNode!==u)return null;var h=Pt;return h=jn(i,i===un?h:0,i.cancelPendingCommit!==null||i.timeoutHandle!==-1),h===0?null:(U3(i,h,l),ty(i,yt()),i.callbackNode!=null&&i.callbackNode===u?ny.bind(null,i):null)}function ry(i,l){if(Pd())return null;U3(i,l,!0)}function Fw(){Ww(function(){(Kt&6)!==0?vt(Lt,Uw):ey()})}function zp(){return Il===0&&(Il=Tu()),Il}function ay(i){return i==null||typeof i=="symbol"||typeof i=="boolean"?null:typeof i=="function"?i:bs(""+i)}function iy(i,l){var u=l.ownerDocument.createElement("input");return u.name=l.name,u.value=l.value,i.id&&u.setAttribute("form",i.id),l.parentNode.insertBefore(u,l),i=new FormData(i),u.parentNode.removeChild(u),i}function zw(i,l,u,h,g){if(l==="submit"&&u&&u.stateNode===g){var v=ay((g[et]||null).action),A=h.submitter;A&&(l=(l=A[et]||null)?ay(l.formAction):A.getAttribute("formAction"),l!==null&&(v=l,A=null));var M=new J0("action","action",null,h,g);i.push({event:M,listeners:[{instance:null,listener:function(){if(h.defaultPrevented){if(Il!==0){var $=A?iy(g,A):new FormData(g);sp(u,{pending:!0,data:$,method:g.method,action:v},null,$)}}else typeof v=="function"&&(M.preventDefault(),$=A?iy(g,A):new FormData(g),sp(u,{pending:!0,data:$,method:g.method,action:v},v,$))},currentTarget:g}]})}}for(var Hp=0;Hp<wm.length;Hp++){var qp=wm[Hp],Hw=qp.toLowerCase(),qw=qp[0].toUpperCase()+qp.slice(1);$a(Hw,"on"+qw)}$a(Uv,"onAnimationEnd"),$a(Fv,"onAnimationIteration"),$a(zv,"onAnimationStart"),$a("dblclick","onDoubleClick"),$a("focusin","onFocus"),$a("focusout","onBlur"),$a(iw,"onTransitionRun"),$a(sw,"onTransitionStart"),$a(lw,"onTransitionCancel"),$a(Hv,"onTransitionEnd"),Mr("onMouseEnter",["mouseout","mouseover"]),Mr("onMouseLeave",["mouseout","mouseover"]),Mr("onPointerEnter",["pointerout","pointerover"]),Mr("onPointerLeave",["pointerout","pointerover"]),wi("onChange","change click focusin focusout input keydown keyup selectionchange".split(" ")),wi("onSelect","focusout contextmenu dragend focusin keydown keyup mousedown mouseup selectionchange".split(" ")),wi("onBeforeInput",["compositionend","keypress","textInput","paste"]),wi("onCompositionEnd","compositionend focusout keydown keypress keyup mousedown".split(" ")),wi("onCompositionStart","compositionstart focusout keydown keypress keyup mousedown".split(" ")),wi("onCompositionUpdate","compositionupdate focusout keydown keypress keyup mousedown".split(" "));var cc="abort canplay canplaythrough durationchange emptied encrypted ended error loadeddata loadedmetadata loadstart pause play playing progress ratechange resize seeked seeking stalled suspend timeupdate volumechange waiting".split(" "),jw=new Set("beforetoggle cancel close invalid load scroll scrollend toggle".split(" ").concat(cc));function sy(i,l){l=(l&4)!==0;for(var u=0;u<i.length;u++){var h=i[u],g=h.event;h=h.listeners;e:{var v=void 0;if(l)for(var A=h.length-1;0<=A;A--){var M=h[A],$=M.instance,re=M.currentTarget;if(M=M.listener,$!==v&&g.isPropagationStopped())break e;v=M,g.currentTarget=re;try{v(g)}catch(_e){wd(_e)}g.currentTarget=null,v=$}else for(A=0;A<h.length;A++){if(M=h[A],$=M.instance,re=M.currentTarget,M=M.listener,$!==v&&g.isPropagationStopped())break e;v=M,g.currentTarget=re;try{v(g)}catch(_e){wd(_e)}g.currentTarget=null,v=$}}}}function Mt(i,l){var u=l[_n];u===void 0&&(u=l[_n]=new Set);var h=i+"__bubble";u.has(h)||(ly(l,i,2,!1),u.add(h))}function jp(i,l,u){var h=0;l&&(h|=4),ly(u,i,h,l)}var Fd="_reactListening"+Math.random().toString(36).slice(2);function $p(i){if(!i[Fd]){i[Fd]=!0,$0.forEach(function(u){u!=="selectionchange"&&(jw.has(u)||jp(u,!1,i),jp(u,!0,i))});var l=i.nodeType===9?i:i.ownerDocument;l===null||l[Fd]||(l[Fd]=!0,jp("selectionchange",!1,l))}}function ly(i,l,u,h){switch(Ry(l)){case 2:var g=gA;break;case 8:g=bA;break;default:g=a1}u=g.bind(null,l,u,i),g=void 0,!hm||l!=="touchstart"&&l!=="touchmove"&&l!=="wheel"||(g=!0),h?g!==void 0?i.addEventListener(l,u,{capture:!0,passive:g}):i.addEventListener(l,u,!0):g!==void 0?i.addEventListener(l,u,{passive:g}):i.addEventListener(l,u,!1)}function Yp(i,l,u,h,g){var v=h;if((l&1)===0&&(l&2)===0&&h!==null)e:for(;;){if(h===null)return;var A=h.tag;if(A===3||A===4){var M=h.stateNode.containerInfo;if(M===g)break;if(A===4)for(A=h.return;A!==null;){var $=A.tag;if(($===3||$===4)&&A.stateNode.containerInfo===g)return;A=A.return}for(;M!==null;){if(A=ms(M),A===null)return;if($=A.tag,$===5||$===6||$===26||$===27){h=v=A;continue e}M=M.parentNode}}h=h.return}xa(function(){var re=v,_e=He(u),xe=[];e:{var le=qv.get(i);if(le!==void 0){var ue=J0,ft=i;switch(i){case"keypress":if(Q0(u)===0)break e;case"keydown":case"keyup":ue=B9;break;case"focusin":ft="focus",ue=gm;break;case"focusout":ft="blur",ue=gm;break;case"beforeblur":case"afterblur":ue=gm;break;case"click":if(u.button===2)break e;case"auxclick":case"dblclick":case"mousedown":case"mousemove":case"mouseup":case"mouseout":case"mouseover":case"contextmenu":ue=bv;break;case"drag":case"dragend":case"dragenter":case"dragexit":case"dragleave":case"dragover":case"dragstart":case"drop":ue=w9;break;case"touchcancel":case"touchend":case"touchmove":case"touchstart":ue=z9;break;case Uv:case Fv:case zv:ue=C9;break;case Hv:ue=q9;break;case"scroll":case"scrollend":ue=x9;break;case"wheel":ue=$9;break;case"copy":case"cut":case"paste":ue=O9;break;case"gotpointercapture":case"lostpointercapture":case"pointercancel":case"pointerdown":case"pointermove":case"pointerout":case"pointerover":case"pointerup":ue=yv;break;case"toggle":case"beforetoggle":ue=G9}var ut=(l&4)!==0,en=!ut&&(i==="scroll"||i==="scrollend"),J=ut?le!==null?le+"Capture":null:le;ut=[];for(var W=re,te;W!==null;){var Te=W;if(te=Te.stateNode,Te=Te.tag,Te!==5&&Te!==26&&Te!==27||te===null||J===null||(Te=bl(W,J),Te!=null&&ut.push(dc(W,Te,te))),en)break;W=W.return}0<ut.length&&(le=new ue(le,ft,null,u,_e),xe.push({event:le,listeners:ut}))}}if((l&7)===0){e:{if(le=i==="mouseover"||i==="pointerover",ue=i==="mouseout"||i==="pointerout",le&&u!==gl&&(ft=u.relatedTarget||u.fromElement)&&(ms(ft)||ft[pn]))break e;if((ue||le)&&(le=_e.window===_e?_e:(le=_e.ownerDocument)?le.defaultView||le.parentWindow:window,ue?(ft=u.relatedTarget||u.toElement,ue=re,ft=ft?ms(ft):null,ft!==null&&(en=s(ft),ut=ft.tag,ft!==en||ut!==5&&ut!==27&&ut!==6)&&(ft=null)):(ue=null,ft=re),ue!==ft)){if(ut=bv,Te="onMouseLeave",J="onMouseEnter",W="mouse",(i==="pointerout"||i==="pointerover")&&(ut=yv,Te="onPointerLeave",J="onPointerEnter",W="pointer"),en=ue==null?le:fl(ue),te=ft==null?le:fl(ft),le=new ut(Te,W+"leave",ue,u,_e),le.target=en,le.relatedTarget=te,Te=null,ms(_e)===re&&(ut=new ut(J,W+"enter",ft,u,_e),ut.target=te,ut.relatedTarget=en,Te=ut),en=Te,ue&&ft)t:{for(ut=ue,J=ft,W=0,te=ut;te;te=Ro(te))W++;for(te=0,Te=J;Te;Te=Ro(Te))te++;for(;0<W-te;)ut=Ro(ut),W--;for(;0<te-W;)J=Ro(J),te--;for(;W--;){if(ut===J||J!==null&&ut===J.alternate)break t;ut=Ro(ut),J=Ro(J)}ut=null}else ut=null;ue!==null&&oy(xe,le,ue,ut,!1),ft!==null&&en!==null&&oy(xe,en,ft,ut,!0)}}e:{if(le=re?fl(re):window,ue=le.nodeName&&le.nodeName.toLowerCase(),ue==="select"||ue==="input"&&le.type==="file")var Ze=kv;else if(wv(le))if(Cv)Ze=nw;else{Ze=ew;var kt=J9}else ue=le.nodeName,!ue||ue.toLowerCase()!=="input"||le.type!=="checkbox"&&le.type!=="radio"?re&&Yt(re.elementType)&&(Ze=kv):Ze=tw;if(Ze&&(Ze=Ze(i,re))){Av(xe,Ze,u,_e);break e}kt&&kt(i,le,re),i==="focusout"&&re&&le.type==="number"&&re.memoizedProps.value!=null&&gs(le,"number",le.value)}switch(kt=re?fl(re):window,i){case"focusin":(wv(kt)||kt.contentEditable==="true")&&(so=kt,Tm=re,Bu=null);break;case"focusout":Bu=Tm=so=null;break;case"mousedown":xm=!0;break;case"contextmenu":case"mouseup":case"dragend":xm=!1,Pv(xe,u,_e);break;case"selectionchange":if(aw)break;case"keydown":case"keyup":Pv(xe,u,_e)}var at;if(vm)e:{switch(i){case"compositionstart":var ct="onCompositionStart";break e;case"compositionend":ct="onCompositionEnd";break e;case"compositionupdate":ct="onCompositionUpdate";break e}ct=void 0}else io?xv(i,u)&&(ct="onCompositionEnd"):i==="keydown"&&u.keyCode===229&&(ct="onCompositionStart");ct&&(_v&&u.locale!=="ko"&&(io||ct!=="onCompositionStart"?ct==="onCompositionEnd"&&io&&(at=pv()):(vs=_e,fm="value"in vs?vs.value:vs.textContent,io=!0)),kt=zd(re,ct),0<kt.length&&(ct=new vv(ct,i,null,u,_e),xe.push({event:ct,listeners:kt}),at?ct.data=at:(at=Sv(u),at!==null&&(ct.data=at)))),(at=K9?W9(i,u):X9(i,u))&&(ct=zd(re,"onBeforeInput"),0<ct.length&&(kt=new vv("onBeforeInput","beforeinput",null,u,_e),xe.push({event:kt,listeners:ct}),kt.data=at)),zw(xe,i,re,u,_e)}sy(xe,l)})}function dc(i,l,u){return{instance:i,listener:l,currentTarget:u}}function zd(i,l){for(var u=l+"Capture",h=[];i!==null;){var g=i,v=g.stateNode;if(g=g.tag,g!==5&&g!==26&&g!==27||v===null||(g=bl(i,u),g!=null&&h.unshift(dc(i,g,v)),g=bl(i,l),g!=null&&h.push(dc(i,g,v))),i.tag===3)return h;i=i.return}return[]}function Ro(i){if(i===null)return null;do i=i.return;while(i&&i.tag!==5&&i.tag!==27);return i||null}function oy(i,l,u,h,g){for(var v=l._reactName,A=[];u!==null&&u!==h;){var M=u,$=M.alternate,re=M.stateNode;if(M=M.tag,$!==null&&$===h)break;M!==5&&M!==26&&M!==27||re===null||($=re,g?(re=bl(u,v),re!=null&&A.unshift(dc(u,re,$))):g||(re=bl(u,v),re!=null&&A.push(dc(u,re,$)))),u=u.return}A.length!==0&&i.push({event:l,listeners:A})}var $w=/\r\n?/g,Yw=/\u0000|\uFFFD/g;function uy(i){return(typeof i=="string"?i:""+i).replace($w,`
`).replace(Yw,"")}function cy(i,l){return l=uy(l),uy(i)===l}function Hd(){}function Jt(i,l,u,h,g,v){switch(u){case"children":typeof h=="string"?l==="body"||l==="textarea"&&h===""||ei(i,h):(typeof h=="number"||typeof h=="bigint")&&l!=="body"&&ei(i,""+h);break;case"className":Ai(i,"class",h);break;case"tabIndex":Ai(i,"tabindex",h);break;case"dir":case"role":case"viewBox":case"width":case"height":Ai(i,u,h);break;case"style":$n(i,h,v);break;case"data":if(l!=="object"){Ai(i,"data",h);break}case"src":case"href":if(h===""&&(l!=="a"||u!=="href")){i.removeAttribute(u);break}if(h==null||typeof h=="function"||typeof h=="symbol"||typeof h=="boolean"){i.removeAttribute(u);break}h=bs(""+h),i.setAttribute(u,h);break;case"action":case"formAction":if(typeof h=="function"){i.setAttribute(u,"javascript:throw new Error('A React form was unexpectedly submitted. If you called form.submit() manually, consider using form.requestSubmit() instead. If you\\'re trying to use event.stopPropagation() in a submit event handler, consider also calling event.preventDefault().')");break}else typeof v=="function"&&(u==="formAction"?(l!=="input"&&Jt(i,l,"name",g.name,g,null),Jt(i,l,"formEncType",g.formEncType,g,null),Jt(i,l,"formMethod",g.formMethod,g,null),Jt(i,l,"formTarget",g.formTarget,g,null)):(Jt(i,l,"encType",g.encType,g,null),Jt(i,l,"method",g.method,g,null),Jt(i,l,"target",g.target,g,null)));if(h==null||typeof h=="symbol"||typeof h=="boolean"){i.removeAttribute(u);break}h=bs(""+h),i.setAttribute(u,h);break;case"onClick":h!=null&&(i.onclick=Hd);break;case"onScroll":h!=null&&Mt("scroll",i);break;case"onScrollEnd":h!=null&&Mt("scrollend",i);break;case"dangerouslySetInnerHTML":if(h!=null){if(typeof h!="object"||!("__html"in h))throw Error(r(61));if(u=h.__html,u!=null){if(g.children!=null)throw Error(r(60));i.innerHTML=u}}break;case"multiple":i.multiple=h&&typeof h!="function"&&typeof h!="symbol";break;case"muted":i.muted=h&&typeof h!="function"&&typeof h!="symbol";break;case"suppressContentEditableWarning":case"suppressHydrationWarning":case"defaultValue":case"defaultChecked":case"innerHTML":case"ref":break;case"autoFocus":break;case"xlinkHref":if(h==null||typeof h=="function"||typeof h=="boolean"||typeof h=="symbol"){i.removeAttribute("xlink:href");break}u=bs(""+h),i.setAttributeNS("http://www.w3.org/1999/xlink","xlink:href",u);break;case"contentEditable":case"spellCheck":case"draggable":case"value":case"autoReverse":case"externalResourcesRequired":case"focusable":case"preserveAlpha":h!=null&&typeof h!="function"&&typeof h!="symbol"?i.setAttribute(u,""+h):i.removeAttribute(u);break;case"inert":case"allowFullScreen":case"async":case"autoPlay":case"controls":case"default":case"defer":case"disabled":case"disablePictureInPicture":case"disableRemotePlayback":case"formNoValidate":case"hidden":case"loop":case"noModule":case"noValidate":case"open":case"playsInline":case"readOnly":case"required":case"reversed":case"scoped":case"seamless":case"itemScope":h&&typeof h!="function"&&typeof h!="symbol"?i.setAttribute(u,""):i.removeAttribute(u);break;case"capture":case"download":h===!0?i.setAttribute(u,""):h!==!1&&h!=null&&typeof h!="function"&&typeof h!="symbol"?i.setAttribute(u,h):i.removeAttribute(u);break;case"cols":case"rows":case"size":case"span":h!=null&&typeof h!="function"&&typeof h!="symbol"&&!isNaN(h)&&1<=h?i.setAttribute(u,h):i.removeAttribute(u);break;case"rowSpan":case"start":h==null||typeof h=="function"||typeof h=="symbol"||isNaN(h)?i.removeAttribute(u):i.setAttribute(u,h);break;case"popover":Mt("beforetoggle",i),Mt("toggle",i),ro(i,"popover",h);break;case"xlinkActuate":Ha(i,"http://www.w3.org/1999/xlink","xlink:actuate",h);break;case"xlinkArcrole":Ha(i,"http://www.w3.org/1999/xlink","xlink:arcrole",h);break;case"xlinkRole":Ha(i,"http://www.w3.org/1999/xlink","xlink:role",h);break;case"xlinkShow":Ha(i,"http://www.w3.org/1999/xlink","xlink:show",h);break;case"xlinkTitle":Ha(i,"http://www.w3.org/1999/xlink","xlink:title",h);break;case"xlinkType":Ha(i,"http://www.w3.org/1999/xlink","xlink:type",h);break;case"xmlBase":Ha(i,"http://www.w3.org/XML/1998/namespace","xml:base",h);break;case"xmlLang":Ha(i,"http://www.w3.org/XML/1998/namespace","xml:lang",h);break;case"xmlSpace":Ha(i,"http://www.w3.org/XML/1998/namespace","xml:space",h);break;case"is":ro(i,"is",h);break;case"innerText":case"textContent":break;default:(!(2<u.length)||u[0]!=="o"&&u[0]!=="O"||u[1]!=="n"&&u[1]!=="N")&&(u=ao.get(u)||u,ro(i,u,h))}}function Gp(i,l,u,h,g,v){switch(u){case"style":$n(i,h,v);break;case"dangerouslySetInnerHTML":if(h!=null){if(typeof h!="object"||!("__html"in h))throw Error(r(61));if(u=h.__html,u!=null){if(g.children!=null)throw Error(r(60));i.innerHTML=u}}break;case"children":typeof h=="string"?ei(i,h):(typeof h=="number"||typeof h=="bigint")&&ei(i,""+h);break;case"onScroll":h!=null&&Mt("scroll",i);break;case"onScrollEnd":h!=null&&Mt("scrollend",i);break;case"onClick":h!=null&&(i.onclick=Hd);break;case"suppressContentEditableWarning":case"suppressHydrationWarning":case"innerHTML":case"ref":break;case"innerText":case"textContent":break;default:if(!Y0.hasOwnProperty(u))e:{if(u[0]==="o"&&u[1]==="n"&&(g=u.endsWith("Capture"),l=u.slice(2,g?u.length-7:void 0),v=i[et]||null,v=v!=null?v[u]:null,typeof v=="function"&&i.removeEventListener(l,v,g),typeof h=="function")){typeof v!="function"&&v!==null&&(u in i?i[u]=null:i.hasAttribute(u)&&i.removeAttribute(u)),i.addEventListener(l,h,g);break e}u in i?i[u]=h:h===!0?i.setAttribute(u,""):ro(i,u,h)}}}function vr(i,l,u){switch(l){case"div":case"span":case"svg":case"path":case"a":case"g":case"p":case"li":break;case"img":Mt("error",i),Mt("load",i);var h=!1,g=!1,v;for(v in u)if(u.hasOwnProperty(v)){var A=u[v];if(A!=null)switch(v){case"src":h=!0;break;case"srcSet":g=!0;break;case"children":case"dangerouslySetInnerHTML":throw Error(r(137,l));default:Jt(i,l,v,A,u,null)}}g&&Jt(i,l,"srcSet",u.srcSet,u,null),h&&Jt(i,l,"src",u.src,u,null);return;case"input":Mt("invalid",i);var M=v=A=g=null,$=null,re=null;for(h in u)if(u.hasOwnProperty(h)){var _e=u[h];if(_e!=null)switch(h){case"name":g=_e;break;case"type":A=_e;break;case"checked":$=_e;break;case"defaultChecked":re=_e;break;case"value":v=_e;break;case"defaultValue":M=_e;break;case"children":case"dangerouslySetInnerHTML":if(_e!=null)throw Error(r(137,l));break;default:Jt(i,l,h,_e,u,null)}}K0(i,v,M,$,re,A,g,!1),ml(i);return;case"select":Mt("invalid",i),h=A=v=null;for(g in u)if(u.hasOwnProperty(g)&&(M=u[g],M!=null))switch(g){case"value":v=M;break;case"defaultValue":A=M;break;case"multiple":h=M;default:Jt(i,l,g,M,u,null)}l=v,u=A,i.multiple=!!h,l!=null?ki(i,!!h,l,!1):u!=null&&ki(i,!!h,u,!0);return;case"textarea":Mt("invalid",i),v=g=h=null;for(A in u)if(u.hasOwnProperty(A)&&(M=u[A],M!=null))switch(A){case"value":h=M;break;case"defaultValue":g=M;break;case"children":v=M;break;case"dangerouslySetInnerHTML":if(M!=null)throw Error(r(91));break;default:Jt(i,l,A,M,u,null)}W0(i,h,g,v),ml(i);return;case"option":for($ in u)if(u.hasOwnProperty($)&&(h=u[$],h!=null))switch($){case"selected":i.selected=h&&typeof h!="function"&&typeof h!="symbol";break;default:Jt(i,l,$,h,u,null)}return;case"dialog":Mt("beforetoggle",i),Mt("toggle",i),Mt("cancel",i),Mt("close",i);break;case"iframe":case"object":Mt("load",i);break;case"video":case"audio":for(h=0;h<cc.length;h++)Mt(cc[h],i);break;case"image":Mt("error",i),Mt("load",i);break;case"details":Mt("toggle",i);break;case"embed":case"source":case"link":Mt("error",i),Mt("load",i);case"area":case"base":case"br":case"col":case"hr":case"keygen":case"meta":case"param":case"track":case"wbr":case"menuitem":for(re in u)if(u.hasOwnProperty(re)&&(h=u[re],h!=null))switch(re){case"children":case"dangerouslySetInnerHTML":throw Error(r(137,l));default:Jt(i,l,re,h,u,null)}return;default:if(Yt(l)){for(_e in u)u.hasOwnProperty(_e)&&(h=u[_e],h!==void 0&&Gp(i,l,_e,h,u,void 0));return}}for(M in u)u.hasOwnProperty(M)&&(h=u[M],h!=null&&Jt(i,l,M,h,u,null))}function Gw(i,l,u,h){switch(l){case"div":case"span":case"svg":case"path":case"a":case"g":case"p":case"li":break;case"input":var g=null,v=null,A=null,M=null,$=null,re=null,_e=null;for(ue in u){var xe=u[ue];if(u.hasOwnProperty(ue)&&xe!=null)switch(ue){case"checked":break;case"value":break;case"defaultValue":$=xe;default:h.hasOwnProperty(ue)||Jt(i,l,ue,null,h,xe)}}for(var le in h){var ue=h[le];if(xe=u[le],h.hasOwnProperty(le)&&(ue!=null||xe!=null))switch(le){case"type":v=ue;break;case"name":g=ue;break;case"checked":re=ue;break;case"defaultChecked":_e=ue;break;case"value":A=ue;break;case"defaultValue":M=ue;break;case"children":case"dangerouslySetInnerHTML":if(ue!=null)throw Error(r(137,l));break;default:ue!==xe&&Jt(i,l,le,ue,h,xe)}}Cu(i,A,M,$,re,_e,v,g);return;case"select":ue=A=M=le=null;for(v in u)if($=u[v],u.hasOwnProperty(v)&&$!=null)switch(v){case"value":break;case"multiple":ue=$;default:h.hasOwnProperty(v)||Jt(i,l,v,null,h,$)}for(g in h)if(v=h[g],$=u[g],h.hasOwnProperty(g)&&(v!=null||$!=null))switch(g){case"value":le=v;break;case"defaultValue":M=v;break;case"multiple":A=v;default:v!==$&&Jt(i,l,g,v,h,$)}l=M,u=A,h=ue,le!=null?ki(i,!!u,le,!1):!!h!=!!u&&(l!=null?ki(i,!!u,l,!0):ki(i,!!u,u?[]:"",!1));return;case"textarea":ue=le=null;for(M in u)if(g=u[M],u.hasOwnProperty(M)&&g!=null&&!h.hasOwnProperty(M))switch(M){case"value":break;case"children":break;default:Jt(i,l,M,null,h,g)}for(A in h)if(g=h[A],v=u[A],h.hasOwnProperty(A)&&(g!=null||v!=null))switch(A){case"value":le=g;break;case"defaultValue":ue=g;break;case"children":break;case"dangerouslySetInnerHTML":if(g!=null)throw Error(r(91));break;default:g!==v&&Jt(i,l,A,g,h,v)}sa(i,le,ue);return;case"option":for(var ft in u)if(le=u[ft],u.hasOwnProperty(ft)&&le!=null&&!h.hasOwnProperty(ft))switch(ft){case"selected":i.selected=!1;break;default:Jt(i,l,ft,null,h,le)}for($ in h)if(le=h[$],ue=u[$],h.hasOwnProperty($)&&le!==ue&&(le!=null||ue!=null))switch($){case"selected":i.selected=le&&typeof le!="function"&&typeof le!="symbol";break;default:Jt(i,l,$,le,h,ue)}return;case"img":case"link":case"area":case"base":case"br":case"col":case"embed":case"hr":case"keygen":case"meta":case"param":case"source":case"track":case"wbr":case"menuitem":for(var ut in u)le=u[ut],u.hasOwnProperty(ut)&&le!=null&&!h.hasOwnProperty(ut)&&Jt(i,l,ut,null,h,le);for(re in h)if(le=h[re],ue=u[re],h.hasOwnProperty(re)&&le!==ue&&(le!=null||ue!=null))switch(re){case"children":case"dangerouslySetInnerHTML":if(le!=null)throw Error(r(137,l));break;default:Jt(i,l,re,le,h,ue)}return;default:if(Yt(l)){for(var en in u)le=u[en],u.hasOwnProperty(en)&&le!==void 0&&!h.hasOwnProperty(en)&&Gp(i,l,en,void 0,h,le);for(_e in h)le=h[_e],ue=u[_e],!h.hasOwnProperty(_e)||le===ue||le===void 0&&ue===void 0||Gp(i,l,_e,le,h,ue);return}}for(var J in u)le=u[J],u.hasOwnProperty(J)&&le!=null&&!h.hasOwnProperty(J)&&Jt(i,l,J,null,h,le);for(xe in h)le=h[xe],ue=u[xe],!h.hasOwnProperty(xe)||le===ue||le==null&&ue==null||Jt(i,l,xe,le,h,ue)}var Vp=null,Kp=null;function qd(i){return i.nodeType===9?i:i.ownerDocument}function dy(i){switch(i){case"http://www.w3.org/2000/svg":return 1;case"http://www.w3.org/1998/Math/MathML":return 2;default:return 0}}function hy(i,l){if(i===0)switch(l){case"svg":return 1;case"math":return 2;default:return 0}return i===1&&l==="foreignObject"?0:i}function Wp(i,l){return i==="textarea"||i==="noscript"||typeof l.children=="string"||typeof l.children=="number"||typeof l.children=="bigint"||typeof l.dangerouslySetInnerHTML=="object"&&l.dangerouslySetInnerHTML!==null&&l.dangerouslySetInnerHTML.__html!=null}var Xp=null;function Vw(){var i=window.event;return i&&i.type==="popstate"?i===Xp?!1:(Xp=i,!0):(Xp=null,!1)}var fy=typeof setTimeout=="function"?setTimeout:void 0,Kw=typeof clearTimeout=="function"?clearTimeout:void 0,my=typeof Promise=="function"?Promise:void 0,Ww=typeof queueMicrotask=="function"?queueMicrotask:typeof my<"u"?function(i){return my.resolve(null).then(i).catch(Xw)}:fy;function Xw(i){setTimeout(function(){throw i})}function Ds(i){return i==="head"}function py(i,l){var u=l,h=0,g=0;do{var v=u.nextSibling;if(i.removeChild(u),v&&v.nodeType===8)if(u=v.data,u==="/$"){if(0<h&&8>h){u=h;var A=i.ownerDocument;if(u&1&&hc(A.documentElement),u&2&&hc(A.body),u&4)for(u=A.head,hc(u),A=u.firstChild;A;){var M=A.nextSibling,$=A.nodeName;A[dl]||$==="SCRIPT"||$==="STYLE"||$==="LINK"&&A.rel.toLowerCase()==="stylesheet"||u.removeChild(A),A=M}}if(g===0){i.removeChild(v),_c(l);return}g--}else u==="$"||u==="$?"||u==="$!"?g++:h=u.charCodeAt(0)-48;else h=0;u=v}while(u);_c(l)}function Qp(i){var l=i.firstChild;for(l&&l.nodeType===10&&(l=l.nextSibling);l;){var u=l;switch(l=l.nextSibling,u.nodeName){case"HTML":case"HEAD":case"BODY":Qp(u),hl(u);continue;case"SCRIPT":case"STYLE":continue;case"LINK":if(u.rel.toLowerCase()==="stylesheet")continue}i.removeChild(u)}}function Qw(i,l,u,h){for(;i.nodeType===1;){var g=u;if(i.nodeName.toLowerCase()!==l.toLowerCase()){if(!h&&(i.nodeName!=="INPUT"||i.type!=="hidden"))break}else if(h){if(!i[dl])switch(l){case"meta":if(!i.hasAttribute("itemprop"))break;return i;case"link":if(v=i.getAttribute("rel"),v==="stylesheet"&&i.hasAttribute("data-precedence"))break;if(v!==g.rel||i.getAttribute("href")!==(g.href==null||g.href===""?null:g.href)||i.getAttribute("crossorigin")!==(g.crossOrigin==null?null:g.crossOrigin)||i.getAttribute("title")!==(g.title==null?null:g.title))break;return i;case"style":if(i.hasAttribute("data-precedence"))break;return i;case"script":if(v=i.getAttribute("src"),(v!==(g.src==null?null:g.src)||i.getAttribute("type")!==(g.type==null?null:g.type)||i.getAttribute("crossorigin")!==(g.crossOrigin==null?null:g.crossOrigin))&&v&&i.hasAttribute("async")&&!i.hasAttribute("itemprop"))break;return i;default:return i}}else if(l==="input"&&i.type==="hidden"){var v=g.name==null?null:""+g.name;if(g.type==="hidden"&&i.getAttribute("name")===v)return i}else return i;if(i=Ga(i.nextSibling),i===null)break}return null}function Zw(i,l,u){if(l==="")return null;for(;i.nodeType!==3;)if((i.nodeType!==1||i.nodeName!=="INPUT"||i.type!=="hidden")&&!u||(i=Ga(i.nextSibling),i===null))return null;return i}function Zp(i){return i.data==="$!"||i.data==="$?"&&i.ownerDocument.readyState==="complete"}function Jw(i,l){var u=i.ownerDocument;if(i.data!=="$?"||u.readyState==="complete")l();else{var h=function(){l(),u.removeEventListener("DOMContentLoaded",h)};u.addEventListener("DOMContentLoaded",h),i._reactRetry=h}}function Ga(i){for(;i!=null;i=i.nextSibling){var l=i.nodeType;if(l===1||l===3)break;if(l===8){if(l=i.data,l==="$"||l==="$!"||l==="$?"||l==="F!"||l==="F")break;if(l==="/$")return null}}return i}var Jp=null;function gy(i){i=i.previousSibling;for(var l=0;i;){if(i.nodeType===8){var u=i.data;if(u==="$"||u==="$!"||u==="$?"){if(l===0)return i;l--}else u==="/$"&&l++}i=i.previousSibling}return null}function by(i,l,u){switch(l=qd(u),i){case"html":if(i=l.documentElement,!i)throw Error(r(452));return i;case"head":if(i=l.head,!i)throw Error(r(453));return i;case"body":if(i=l.body,!i)throw Error(r(454));return i;default:throw Error(r(451))}}function hc(i){for(var l=i.attributes;l.length;)i.removeAttributeNode(l[0]);hl(i)}var Oa=new Map,vy=new Set;function jd(i){return typeof i.getRootNode=="function"?i.getRootNode():i.nodeType===9?i:i.ownerDocument}var Hi=me.d;me.d={f:eA,r:tA,D:nA,C:rA,L:aA,m:iA,X:lA,S:sA,M:oA};function eA(){var i=Hi.f(),l=Dd();return i||l}function tA(i){var l=ps(i);l!==null&&l.tag===5&&l.type==="form"?U4(l):Hi.r(i)}var Mo=typeof document>"u"?null:document;function yy(i,l,u){var h=Mo;if(h&&typeof l=="string"&&l){var g=Gr(l);g='link[rel="'+i+'"][href="'+g+'"]',typeof u=="string"&&(g+='[crossorigin="'+u+'"]'),vy.has(g)||(vy.add(g),i={rel:i,crossOrigin:u,href:l},h.querySelector(g)===null&&(l=h.createElement("link"),vr(l,"link",i),An(l),h.head.appendChild(l)))}}function nA(i){Hi.D(i),yy("dns-prefetch",i,null)}function rA(i,l){Hi.C(i,l),yy("preconnect",i,l)}function aA(i,l,u){Hi.L(i,l,u);var h=Mo;if(h&&i&&l){var g='link[rel="preload"][as="'+Gr(l)+'"]';l==="image"&&u&&u.imageSrcSet?(g+='[imagesrcset="'+Gr(u.imageSrcSet)+'"]',typeof u.imageSizes=="string"&&(g+='[imagesizes="'+Gr(u.imageSizes)+'"]')):g+='[href="'+Gr(i)+'"]';var v=g;switch(l){case"style":v=Io(i);break;case"script":v=Do(i)}Oa.has(v)||(i=m({rel:"preload",href:l==="image"&&u&&u.imageSrcSet?void 0:i,as:l},u),Oa.set(v,i),h.querySelector(g)!==null||l==="style"&&h.querySelector(fc(v))||l==="script"&&h.querySelector(mc(v))||(l=h.createElement("link"),vr(l,"link",i),An(l),h.head.appendChild(l)))}}function iA(i,l){Hi.m(i,l);var u=Mo;if(u&&i){var h=l&&typeof l.as=="string"?l.as:"script",g='link[rel="modulepreload"][as="'+Gr(h)+'"][href="'+Gr(i)+'"]',v=g;switch(h){case"audioworklet":case"paintworklet":case"serviceworker":case"sharedworker":case"worker":case"script":v=Do(i)}if(!Oa.has(v)&&(i=m({rel:"modulepreload",href:i},l),Oa.set(v,i),u.querySelector(g)===null)){switch(h){case"audioworklet":case"paintworklet":case"serviceworker":case"sharedworker":case"worker":case"script":if(u.querySelector(mc(v)))return}h=u.createElement("link"),vr(h,"link",i),An(h),u.head.appendChild(h)}}}function sA(i,l,u){Hi.S(i,l,u);var h=Mo;if(h&&i){var g=Si(h).hoistableStyles,v=Io(i);l=l||"default";var A=g.get(v);if(!A){var M={loading:0,preload:null};if(A=h.querySelector(fc(v)))M.loading=5;else{i=m({rel:"stylesheet",href:i,"data-precedence":l},u),(u=Oa.get(v))&&e1(i,u);var $=A=h.createElement("link");An($),vr($,"link",i),$._p=new Promise(function(re,_e){$.onload=re,$.onerror=_e}),$.addEventListener("load",function(){M.loading|=1}),$.addEventListener("error",function(){M.loading|=2}),M.loading|=4,$d(A,l,h)}A={type:"stylesheet",instance:A,count:1,state:M},g.set(v,A)}}}function lA(i,l){Hi.X(i,l);var u=Mo;if(u&&i){var h=Si(u).hoistableScripts,g=Do(i),v=h.get(g);v||(v=u.querySelector(mc(g)),v||(i=m({src:i,async:!0},l),(l=Oa.get(g))&&t1(i,l),v=u.createElement("script"),An(v),vr(v,"link",i),u.head.appendChild(v)),v={type:"script",instance:v,count:1,state:null},h.set(g,v))}}function oA(i,l){Hi.M(i,l);var u=Mo;if(u&&i){var h=Si(u).hoistableScripts,g=Do(i),v=h.get(g);v||(v=u.querySelector(mc(g)),v||(i=m({src:i,async:!0,type:"module"},l),(l=Oa.get(g))&&t1(i,l),v=u.createElement("script"),An(v),vr(v,"link",i),u.head.appendChild(v)),v={type:"script",instance:v,count:1,state:null},h.set(g,v))}}function _y(i,l,u,h){var g=(g=De.current)?jd(g):null;if(!g)throw Error(r(446));switch(i){case"meta":case"title":return null;case"style":return typeof u.precedence=="string"&&typeof u.href=="string"?(l=Io(u.href),u=Si(g).hoistableStyles,h=u.get(l),h||(h={type:"style",instance:null,count:0,state:null},u.set(l,h)),h):{type:"void",instance:null,count:0,state:null};case"link":if(u.rel==="stylesheet"&&typeof u.href=="string"&&typeof u.precedence=="string"){i=Io(u.href);var v=Si(g).hoistableStyles,A=v.get(i);if(A||(g=g.ownerDocument||g,A={type:"stylesheet",instance:null,count:0,state:{loading:0,preload:null}},v.set(i,A),(v=g.querySelector(fc(i)))&&!v._p&&(A.instance=v,A.state.loading=5),Oa.has(i)||(u={rel:"preload",as:"style",href:u.href,crossOrigin:u.crossOrigin,integrity:u.integrity,media:u.media,hrefLang:u.hrefLang,referrerPolicy:u.referrerPolicy},Oa.set(i,u),v||uA(g,i,u,A.state))),l&&h===null)throw Error(r(528,""));return A}if(l&&h!==null)throw Error(r(529,""));return null;case"script":return l=u.async,u=u.src,typeof u=="string"&&l&&typeof l!="function"&&typeof l!="symbol"?(l=Do(u),u=Si(g).hoistableScripts,h=u.get(l),h||(h={type:"script",instance:null,count:0,state:null},u.set(l,h)),h):{type:"void",instance:null,count:0,state:null};default:throw Error(r(444,i))}}function Io(i){return'href="'+Gr(i)+'"'}function fc(i){return'link[rel="stylesheet"]['+i+"]"}function Ey(i){return m({},i,{"data-precedence":i.precedence,precedence:null})}function uA(i,l,u,h){i.querySelector('link[rel="preload"][as="style"]['+l+"]")?h.loading=1:(l=i.createElement("link"),h.preload=l,l.addEventListener("load",function(){return h.loading|=1}),l.addEventListener("error",function(){return h.loading|=2}),vr(l,"link",u),An(l),i.head.appendChild(l))}function Do(i){return'[src="'+Gr(i)+'"]'}function mc(i){return"script[async]"+i}function Ty(i,l,u){if(l.count++,l.instance===null)switch(l.type){case"style":var h=i.querySelector('style[data-href~="'+Gr(u.href)+'"]');if(h)return l.instance=h,An(h),h;var g=m({},u,{"data-href":u.href,"data-precedence":u.precedence,href:null,precedence:null});return h=(i.ownerDocument||i).createElement("style"),An(h),vr(h,"style",g),$d(h,u.precedence,i),l.instance=h;case"stylesheet":g=Io(u.href);var v=i.querySelector(fc(g));if(v)return l.state.loading|=4,l.instance=v,An(v),v;h=Ey(u),(g=Oa.get(g))&&e1(h,g),v=(i.ownerDocument||i).createElement("link"),An(v);var A=v;return A._p=new Promise(function(M,$){A.onload=M,A.onerror=$}),vr(v,"link",h),l.state.loading|=4,$d(v,u.precedence,i),l.instance=v;case"script":return v=Do(u.src),(g=i.querySelector(mc(v)))?(l.instance=g,An(g),g):(h=u,(g=Oa.get(v))&&(h=m({},u),t1(h,g)),i=i.ownerDocument||i,g=i.createElement("script"),An(g),vr(g,"link",h),i.head.appendChild(g),l.instance=g);case"void":return null;default:throw Error(r(443,l.type))}else l.type==="stylesheet"&&(l.state.loading&4)===0&&(h=l.instance,l.state.loading|=4,$d(h,u.precedence,i));return l.instance}function $d(i,l,u){for(var h=u.querySelectorAll('link[rel="stylesheet"][data-precedence],style[data-precedence]'),g=h.length?h[h.length-1]:null,v=g,A=0;A<h.length;A++){var M=h[A];if(M.dataset.precedence===l)v=M;else if(v!==g)break}v?v.parentNode.insertBefore(i,v.nextSibling):(l=u.nodeType===9?u.head:u,l.insertBefore(i,l.firstChild))}function e1(i,l){i.crossOrigin==null&&(i.crossOrigin=l.crossOrigin),i.referrerPolicy==null&&(i.referrerPolicy=l.referrerPolicy),i.title==null&&(i.title=l.title)}function t1(i,l){i.crossOrigin==null&&(i.crossOrigin=l.crossOrigin),i.referrerPolicy==null&&(i.referrerPolicy=l.referrerPolicy),i.integrity==null&&(i.integrity=l.integrity)}var Yd=null;function xy(i,l,u){if(Yd===null){var h=new Map,g=Yd=new Map;g.set(u,h)}else g=Yd,h=g.get(u),h||(h=new Map,g.set(u,h));if(h.has(i))return h;for(h.set(i,null),u=u.getElementsByTagName(i),g=0;g<u.length;g++){var v=u[g];if(!(v[dl]||v[Pe]||i==="link"&&v.getAttribute("rel")==="stylesheet")&&v.namespaceURI!=="http://www.w3.org/2000/svg"){var A=v.getAttribute(l)||"";A=i+A;var M=h.get(A);M?M.push(v):h.set(A,[v])}}return h}function Sy(i,l,u){i=i.ownerDocument||i,i.head.insertBefore(u,l==="title"?i.querySelector("head > title"):null)}function cA(i,l,u){if(u===1||l.itemProp!=null)return!1;switch(i){case"meta":case"title":return!0;case"style":if(typeof l.precedence!="string"||typeof l.href!="string"||l.href==="")break;return!0;case"link":if(typeof l.rel!="string"||typeof l.href!="string"||l.href===""||l.onLoad||l.onError)break;switch(l.rel){case"stylesheet":return i=l.disabled,typeof l.precedence=="string"&&i==null;default:return!0}case"script":if(l.async&&typeof l.async!="function"&&typeof l.async!="symbol"&&!l.onLoad&&!l.onError&&l.src&&typeof l.src=="string")return!0}return!1}function wy(i){return!(i.type==="stylesheet"&&(i.state.loading&3)===0)}var pc=null;function dA(){}function hA(i,l,u){if(pc===null)throw Error(r(475));var h=pc;if(l.type==="stylesheet"&&(typeof u.media!="string"||matchMedia(u.media).matches!==!1)&&(l.state.loading&4)===0){if(l.instance===null){var g=Io(u.href),v=i.querySelector(fc(g));if(v){i=v._p,i!==null&&typeof i=="object"&&typeof i.then=="function"&&(h.count++,h=Gd.bind(h),i.then(h,h)),l.state.loading|=4,l.instance=v,An(v);return}v=i.ownerDocument||i,u=Ey(u),(g=Oa.get(g))&&e1(u,g),v=v.createElement("link"),An(v);var A=v;A._p=new Promise(function(M,$){A.onload=M,A.onerror=$}),vr(v,"link",u),l.instance=v}h.stylesheets===null&&(h.stylesheets=new Map),h.stylesheets.set(l,i),(i=l.state.preload)&&(l.state.loading&3)===0&&(h.count++,l=Gd.bind(h),i.addEventListener("load",l),i.addEventListener("error",l))}}function fA(){if(pc===null)throw Error(r(475));var i=pc;return i.stylesheets&&i.count===0&&n1(i,i.stylesheets),0<i.count?function(l){var u=setTimeout(function(){if(i.stylesheets&&n1(i,i.stylesheets),i.unsuspend){var h=i.unsuspend;i.unsuspend=null,h()}},6e4);return i.unsuspend=l,function(){i.unsuspend=null,clearTimeout(u)}}:null}function Gd(){if(this.count--,this.count===0){if(this.stylesheets)n1(this,this.stylesheets);else if(this.unsuspend){var i=this.unsuspend;this.unsuspend=null,i()}}}var Vd=null;function n1(i,l){i.stylesheets=null,i.unsuspend!==null&&(i.count++,Vd=new Map,l.forEach(mA,i),Vd=null,Gd.call(i))}function mA(i,l){if(!(l.state.loading&4)){var u=Vd.get(i);if(u)var h=u.get(null);else{u=new Map,Vd.set(i,u);for(var g=i.querySelectorAll("link[data-precedence],style[data-precedence]"),v=0;v<g.length;v++){var A=g[v];(A.nodeName==="LINK"||A.getAttribute("media")!=="not all")&&(u.set(A.dataset.precedence,A),h=A)}h&&u.set(null,h)}g=l.instance,A=g.getAttribute("data-precedence"),v=u.get(A)||h,v===h&&u.set(null,g),u.set(A,g),this.count++,h=Gd.bind(this),g.addEventListener("load",h),g.addEventListener("error",h),v?v.parentNode.insertBefore(g,v.nextSibling):(i=i.nodeType===9?i.head:i,i.insertBefore(g,i.firstChild)),l.state.loading|=4}}var gc={$$typeof:D,Provider:null,Consumer:null,_currentValue:pe,_currentValue2:pe,_threadCount:0};function pA(i,l,u,h,g,v,A,M){this.tag=1,this.containerInfo=i,this.pingCache=this.current=this.pendingChildren=null,this.timeoutHandle=-1,this.callbackNode=this.next=this.pendingContext=this.context=this.cancelPendingCommit=null,this.callbackPriority=0,this.expirationTimes=Su(-1),this.entangledLanes=this.shellSuspendCounter=this.errorRecoveryDisabledLanes=this.expiredLanes=this.warmLanes=this.pingedLanes=this.suspendedLanes=this.pendingLanes=0,this.entanglements=Su(0),this.hiddenUpdates=Su(null),this.identifierPrefix=h,this.onUncaughtError=g,this.onCaughtError=v,this.onRecoverableError=A,this.pooledCache=null,this.pooledCacheLanes=0,this.formState=M,this.incompleteTransitions=new Map}function Ay(i,l,u,h,g,v,A,M,$,re,_e,xe){return i=new pA(i,l,u,A,M,$,re,xe),l=1,v===!0&&(l|=24),v=oa(3,null,null,l),i.current=v,v.stateNode=i,l=Bm(),l.refCount++,i.pooledCache=l,l.refCount++,v.memoizedState={element:h,isDehydrated:u,cache:l},Hm(v),i}function ky(i){return i?(i=co,i):co}function Cy(i,l,u,h,g,v){g=ky(g),h.context===null?h.context=g:h.pendingContext=g,h=Es(l),h.payload={element:u},v=v===void 0?null:v,v!==null&&(h.callback=v),u=Ts(i,h,l),u!==null&&(fa(u,i,l),Gu(u,i,l))}function Ny(i,l){if(i=i.memoizedState,i!==null&&i.dehydrated!==null){var u=i.retryLane;i.retryLane=u!==0&&u<l?u:l}}function r1(i,l){Ny(i,l),(i=i.alternate)&&Ny(i,l)}function Oy(i){if(i.tag===13){var l=uo(i,67108864);l!==null&&fa(l,i,67108864),r1(i,67108864)}}var Kd=!0;function gA(i,l,u,h){var g=V.T;V.T=null;var v=me.p;try{me.p=2,a1(i,l,u,h)}finally{me.p=v,V.T=g}}function bA(i,l,u,h){var g=V.T;V.T=null;var v=me.p;try{me.p=8,a1(i,l,u,h)}finally{me.p=v,V.T=g}}function a1(i,l,u,h){if(Kd){var g=i1(h);if(g===null)Yp(i,l,h,Wd,u),My(i,h);else if(yA(g,i,l,u,h))h.stopPropagation();else if(My(i,h),l&4&&-1<vA.indexOf(i)){for(;g!==null;){var v=ps(g);if(v!==null)switch(v.tag){case 3:if(v=v.stateNode,v.current.memoizedState.isDehydrated){var A=Or(v.pendingLanes);if(A!==0){var M=v;for(M.pendingLanes|=2,M.entangledLanes|=2;A;){var $=1<<31-ze(A);M.entanglements[1]|=$,A&=~$}ii(v),(Kt&6)===0&&(Md=yt()+500,uc(0))}}break;case 13:M=uo(v,2),M!==null&&fa(M,v,2),Dd(),r1(v,2)}if(v=i1(h),v===null&&Yp(i,l,h,Wd,u),v===g)break;g=v}g!==null&&h.stopPropagation()}else Yp(i,l,h,null,u)}}function i1(i){return i=He(i),s1(i)}var Wd=null;function s1(i){if(Wd=null,i=ms(i),i!==null){var l=s(i);if(l===null)i=null;else{var u=l.tag;if(u===13){if(i=o(l),i!==null)return i;i=null}else if(u===3){if(l.stateNode.current.memoizedState.isDehydrated)return l.tag===3?l.stateNode.containerInfo:null;i=null}else l!==i&&(i=null)}}return Wd=i,null}function Ry(i){switch(i){case"beforetoggle":case"cancel":case"click":case"close":case"contextmenu":case"copy":case"cut":case"auxclick":case"dblclick":case"dragend":case"dragstart":case"drop":case"focusin":case"focusout":case"input":case"invalid":case"keydown":case"keypress":case"keyup":case"mousedown":case"mouseup":case"paste":case"pause":case"play":case"pointercancel":case"pointerdown":case"pointerup":case"ratechange":case"reset":case"resize":case"seeked":case"submit":case"toggle":case"touchcancel":case"touchend":case"touchstart":case"volumechange":case"change":case"selectionchange":case"textInput":case"compositionstart":case"compositionend":case"compositionupdate":case"beforeblur":case"afterblur":case"beforeinput":case"blur":case"fullscreenchange":case"focus":case"hashchange":case"popstate":case"select":case"selectstart":return 2;case"drag":case"dragenter":case"dragexit":case"dragleave":case"dragover":case"mousemove":case"mouseout":case"mouseover":case"pointermove":case"pointerout":case"pointerover":case"scroll":case"touchmove":case"wheel":case"mouseenter":case"mouseleave":case"pointerenter":case"pointerleave":return 8;case"message":switch(mn()){case Lt:return 2;case ln:return 8;case We:case on:return 32;case qn:return 268435456;default:return 32}default:return 32}}var l1=!1,Ls=null,Ps=null,Bs=null,bc=new Map,vc=new Map,Us=[],vA="mousedown mouseup touchcancel touchend touchstart auxclick dblclick pointercancel pointerdown pointerup dragend dragstart drop compositionend compositionstart keydown keypress keyup input textInput copy cut paste click change contextmenu reset".split(" ");function My(i,l){switch(i){case"focusin":case"focusout":Ls=null;break;case"dragenter":case"dragleave":Ps=null;break;case"mouseover":case"mouseout":Bs=null;break;case"pointerover":case"pointerout":bc.delete(l.pointerId);break;case"gotpointercapture":case"lostpointercapture":vc.delete(l.pointerId)}}function yc(i,l,u,h,g,v){return i===null||i.nativeEvent!==v?(i={blockedOn:l,domEventName:u,eventSystemFlags:h,nativeEvent:v,targetContainers:[g]},l!==null&&(l=ps(l),l!==null&&Oy(l)),i):(i.eventSystemFlags|=h,l=i.targetContainers,g!==null&&l.indexOf(g)===-1&&l.push(g),i)}function yA(i,l,u,h,g){switch(l){case"focusin":return Ls=yc(Ls,i,l,u,h,g),!0;case"dragenter":return Ps=yc(Ps,i,l,u,h,g),!0;case"mouseover":return Bs=yc(Bs,i,l,u,h,g),!0;case"pointerover":var v=g.pointerId;return bc.set(v,yc(bc.get(v)||null,i,l,u,h,g)),!0;case"gotpointercapture":return v=g.pointerId,vc.set(v,yc(vc.get(v)||null,i,l,u,h,g)),!0}return!1}function Iy(i){var l=ms(i.target);if(l!==null){var u=s(l);if(u!==null){if(l=u.tag,l===13){if(l=o(u),l!==null){i.blockedOn=l,wn(i.priority,function(){if(u.tag===13){var h=ha();h=Fe(h);var g=uo(u,h);g!==null&&fa(g,u,h),r1(u,h)}});return}}else if(l===3&&u.stateNode.current.memoizedState.isDehydrated){i.blockedOn=u.tag===3?u.stateNode.containerInfo:null;return}}}i.blockedOn=null}function Xd(i){if(i.blockedOn!==null)return!1;for(var l=i.targetContainers;0<l.length;){var u=i1(i.nativeEvent);if(u===null){u=i.nativeEvent;var h=new u.constructor(u.type,u);gl=h,u.target.dispatchEvent(h),gl=null}else return l=ps(u),l!==null&&Oy(l),i.blockedOn=u,!1;l.shift()}return!0}function Dy(i,l,u){Xd(i)&&u.delete(l)}function _A(){l1=!1,Ls!==null&&Xd(Ls)&&(Ls=null),Ps!==null&&Xd(Ps)&&(Ps=null),Bs!==null&&Xd(Bs)&&(Bs=null),bc.forEach(Dy),vc.forEach(Dy)}function Qd(i,l){i.blockedOn===l&&(i.blockedOn=null,l1||(l1=!0,e.unstable_scheduleCallback(e.unstable_NormalPriority,_A)))}var Zd=null;function Ly(i){Zd!==i&&(Zd=i,e.unstable_scheduleCallback(e.unstable_NormalPriority,function(){Zd===i&&(Zd=null);for(var l=0;l<i.length;l+=3){var u=i[l],h=i[l+1],g=i[l+2];if(typeof h!="function"){if(s1(h||u)===null)continue;break}var v=ps(u);v!==null&&(i.splice(l,3),l-=3,sp(v,{pending:!0,data:g,method:u.method,action:h},h,g))}}))}function _c(i){function l($){return Qd($,i)}Ls!==null&&Qd(Ls,i),Ps!==null&&Qd(Ps,i),Bs!==null&&Qd(Bs,i),bc.forEach(l),vc.forEach(l);for(var u=0;u<Us.length;u++){var h=Us[u];h.blockedOn===i&&(h.blockedOn=null)}for(;0<Us.length&&(u=Us[0],u.blockedOn===null);)Iy(u),u.blockedOn===null&&Us.shift();if(u=(i.ownerDocument||i).$$reactFormReplay,u!=null)for(h=0;h<u.length;h+=3){var g=u[h],v=u[h+1],A=g[et]||null;if(typeof v=="function")A||Ly(u);else if(A){var M=null;if(v&&v.hasAttribute("formAction")){if(g=v,A=v[et]||null)M=A.formAction;else if(s1(g)!==null)continue}else M=A.action;typeof M=="function"?u[h+1]=M:(u.splice(h,3),h-=3),Ly(u)}}}function o1(i){this._internalRoot=i}Jd.prototype.render=o1.prototype.render=function(i){var l=this._internalRoot;if(l===null)throw Error(r(409));var u=l.current,h=ha();Cy(u,h,i,l,null,null)},Jd.prototype.unmount=o1.prototype.unmount=function(){var i=this._internalRoot;if(i!==null){this._internalRoot=null;var l=i.containerInfo;Cy(i.current,2,null,i,null,null),Dd(),l[pn]=null}};function Jd(i){this._internalRoot=i}Jd.prototype.unstable_scheduleHydration=function(i){if(i){var l=yn();i={blockedOn:null,target:i,priority:l};for(var u=0;u<Us.length&&l!==0&&l<Us[u].priority;u++);Us.splice(u,0,i),u===0&&Iy(i)}};var Py=t.version;if(Py!=="19.1.0")throw Error(r(527,Py,"19.1.0"));me.findDOMNode=function(i){var l=i._reactInternals;if(l===void 0)throw typeof i.render=="function"?Error(r(188)):(i=Object.keys(i).join(","),Error(r(268,i)));return i=d(l),i=i!==null?f(i):null,i=i===null?null:i.stateNode,i};var EA={bundleType:0,version:"19.1.0",rendererPackageName:"react-dom",currentDispatcherRef:V,reconcilerVersion:"19.1.0"};if(typeof __REACT_DEVTOOLS_GLOBAL_HOOK__<"u"){var eh=__REACT_DEVTOOLS_GLOBAL_HOOK__;if(!eh.isDisabled&&eh.supportsFiber)try{he=eh.inject(EA),Se=eh}catch{}}return Tc.createRoot=function(i,l){if(!a(i))throw Error(r(299));var u=!1,h="",g=Z4,v=J4,A=e3,M=null;return l!=null&&(l.unstable_strictMode===!0&&(u=!0),l.identifierPrefix!==void 0&&(h=l.identifierPrefix),l.onUncaughtError!==void 0&&(g=l.onUncaughtError),l.onCaughtError!==void 0&&(v=l.onCaughtError),l.onRecoverableError!==void 0&&(A=l.onRecoverableError),l.unstable_transitionCallbacks!==void 0&&(M=l.unstable_transitionCallbacks)),l=Ay(i,1,!1,null,null,u,h,g,v,A,M,null),i[pn]=l.current,$p(i),new o1(l)},Tc.hydrateRoot=function(i,l,u){if(!a(i))throw Error(r(299));var h=!1,g="",v=Z4,A=J4,M=e3,$=null,re=null;return u!=null&&(u.unstable_strictMode===!0&&(h=!0),u.identifierPrefix!==void 0&&(g=u.identifierPrefix),u.onUncaughtError!==void 0&&(v=u.onUncaughtError),u.onCaughtError!==void 0&&(A=u.onCaughtError),u.onRecoverableError!==void 0&&(M=u.onRecoverableError),u.unstable_transitionCallbacks!==void 0&&($=u.unstable_transitionCallbacks),u.formState!==void 0&&(re=u.formState)),l=Ay(i,1,!0,l,u??null,h,g,v,A,M,$,re),l.context=ky(null),u=l.current,h=ha(),h=Fe(h),g=Es(h),g.callback=null,Ts(u,g,h),u=h,l.current.lanes=u,hs(l,u),ii(l),i[pn]=l.current,$p(i),new Jd(l)},Tc.version="19.1.0",Tc}var Gy;function MA(){if(Gy)return d1.exports;Gy=1;function e(){if(!(typeof __REACT_DEVTOOLS_GLOBAL_HOOK__>"u"||typeof __REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE!="function"))try{__REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE(e)}catch(t){console.error(t)}}return e(),d1.exports=RA(),d1.exports}var IA=MA();const q5=z.createContext({theme:"system",resolvedTheme:"light",setTheme:()=>{},toggleTheme:()=>{}}),j5=()=>z.useContext(q5),Pg="cycls_theme_preference",Vy=()=>typeof window>"u"?"light":window.matchMedia("(prefers-color-scheme: dark)").matches?"dark":"light",Ky=()=>{if(typeof window>"u")return"system";try{return localStorage.getItem(Pg)||"system"}catch{return"system"}},Wy=(e,t)=>{const n=document.documentElement;n.classList.remove("light","dark"),n.removeAttribute("data-theme"),e==="system"?n.setAttribute("data-resolved-theme",t):(n.classList.add(e),n.setAttribute("data-theme",e),n.setAttribute("data-resolved-theme",e))},p1=({children:e})=>{const[t,n]=z.useState(()=>Ky()),[r,a]=z.useState(()=>{const d=Ky();return d==="system"?Vy():d});z.useEffect(()=>{const d=window.matchMedia("(prefers-color-scheme: dark)"),f=m=>{if(t==="system"){const p=m.matches?"dark":"light";a(p),Wy("system",p)}};return d.addEventListener("change",f),()=>d.removeEventListener("change",f)},[t]),z.useEffect(()=>{const d=t==="system"?Vy():t;Wy(t,d),requestAnimationFrame(()=>{a(d)})},[t]);const s=z.useCallback(d=>{n(d);try{d==="system"?localStorage.removeItem(Pg):localStorage.setItem(Pg,d)}catch{}},[]),o=z.useCallback(()=>{s(r==="dark"?"light":"dark")},[r,s]),c={theme:t,resolvedTheme:r,setTheme:s,toggleTheme:o};return R.jsx(q5.Provider,{value:c,children:e})},$5=({className:e=""})=>{const{resolvedTheme:t,toggleTheme:n}=j5(),r=t==="dark";return R.jsx("button",{onClick:n,className:`
        theme-toggle
        inline-flex items-center justify-center
        rounded-xl p-2
        hover:bg-[var(--bg-hover)] active:bg-[var(--bg-active)]
        focus:outline-none focus-visible:ring-2 focus-visible:ring-[var(--text-primary)]/20
        transition-colors duration-200
        ${e}
      `,"aria-label":r?"Switch to light mode":"Switch to dark mode",title:r?"Light mode":"Dark mode",children:r?R.jsxs("svg",{className:"theme-toggle-icon w-5 h-5 text-[var(--text-primary)]",viewBox:"0 0 24 24",fill:"none",stroke:"currentColor",strokeWidth:"1.7",strokeLinecap:"round",strokeLinejoin:"round",children:[R.jsx("circle",{cx:"12",cy:"12",r:"4"}),R.jsx("path",{d:"M12 2v2"}),R.jsx("path",{d:"M12 20v2"}),R.jsx("path",{d:"M4.93 4.93l1.41 1.41"}),R.jsx("path",{d:"M17.66 17.66l1.41 1.41"}),R.jsx("path",{d:"M2 12h2"}),R.jsx("path",{d:"M20 12h2"}),R.jsx("path",{d:"M6.34 17.66l-1.41 1.41"}),R.jsx("path",{d:"M19.07 4.93l-1.41 1.41"})]}):R.jsx("svg",{className:"theme-toggle-icon w-5 h-5 text-[var(--text-primary)]",viewBox:"0 0 24 24",fill:"none",stroke:"currentColor",strokeWidth:"1.7",strokeLinecap:"round",strokeLinejoin:"round",children:R.jsx("path",{d:"M12 3a6 6 0 0 0 9 9 9 9 0 1 1-9-9Z"})})})},Bg=z.createContext(!1),DA=()=>z.useContext(Bg);/**
 * react-router v7.9.6
 *
 * Copyright (c) Remix Software Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE.md file in the root directory of this source tree.
 *
 * @license MIT
 */var Xy="popstate";function LA(e={}){function t(r,a){let{pathname:s,search:o,hash:c}=r.location;return Ug("",{pathname:s,search:o,hash:c},a.state&&a.state.usr||null,a.state&&a.state.key||"default")}function n(r,a){return typeof a=="string"?a:u0(a)}return BA(t,n,null,e)}function xn(e,t){if(e===!1||e===null||typeof e>"u")throw new Error(t)}function Ba(e,t){if(!e){typeof console<"u"&&console.warn(t);try{throw new Error(t)}catch{}}}function PA(){return Math.random().toString(36).substring(2,10)}function Qy(e,t){return{usr:e.state,key:e.key,idx:t}}function Ug(e,t,n=null,r){return{pathname:typeof e=="string"?e:e.pathname,search:"",hash:"",...typeof t=="string"?ou(t):t,state:n,key:t&&t.key||r||PA()}}function u0({pathname:e="/",search:t="",hash:n=""}){return t&&t!=="?"&&(e+=t.charAt(0)==="?"?t:"?"+t),n&&n!=="#"&&(e+=n.charAt(0)==="#"?n:"#"+n),e}function ou(e){let t={};if(e){let n=e.indexOf("#");n>=0&&(t.hash=e.substring(n),e=e.substring(0,n));let r=e.indexOf("?");r>=0&&(t.search=e.substring(r),e=e.substring(0,r)),e&&(t.pathname=e)}return t}function BA(e,t,n,r={}){let{window:a=document.defaultView,v5Compat:s=!1}=r,o=a.history,c="POP",d=null,f=m();f==null&&(f=0,o.replaceState({...o.state,idx:f},""));function m(){return(o.state||{idx:null}).idx}function p(){c="POP";let C=m(),w=C==null?null:C-f;f=C,d&&d({action:c,location:x.location,delta:w})}function b(C,w){c="PUSH";let O=Ug(x.location,C,w);f=m()+1;let D=Qy(O,f),Y=x.createHref(O);try{o.pushState(D,"",Y)}catch(H){if(H instanceof DOMException&&H.name==="DataCloneError")throw H;a.location.assign(Y)}s&&d&&d({action:c,location:x.location,delta:1})}function y(C,w){c="REPLACE";let O=Ug(x.location,C,w);f=m();let D=Qy(O,f),Y=x.createHref(O);o.replaceState(D,"",Y),s&&d&&d({action:c,location:x.location,delta:0})}function T(C){return UA(C)}let x={get action(){return c},get location(){return e(a,o)},listen(C){if(d)throw new Error("A history only accepts one active listener");return a.addEventListener(Xy,p),d=C,()=>{a.removeEventListener(Xy,p),d=null}},createHref(C){return t(a,C)},createURL:T,encodeLocation(C){let w=T(C);return{pathname:w.pathname,search:w.search,hash:w.hash}},push:b,replace:y,go(C){return o.go(C)}};return x}function UA(e,t=!1){let n="http://localhost";typeof window<"u"&&(n=window.location.origin!=="null"?window.location.origin:window.location.href),xn(n,"No window.location.(origin|href) available to create URL");let r=typeof e=="string"?e:u0(e);return r=r.replace(/ $/,"%20"),!t&&r.startsWith("//")&&(r=n+r),new URL(r,n)}function Y5(e,t,n="/"){return FA(e,t,n,!1)}function FA(e,t,n,r){let a=typeof t=="string"?ou(t):t,s=ns(a.pathname||"/",n);if(s==null)return null;let o=G5(e);zA(o);let c=null;for(let d=0;c==null&&d<o.length;++d){let f=QA(s);c=WA(o[d],f,r)}return c}function G5(e,t=[],n=[],r="",a=!1){let s=(o,c,d=a,f)=>{let m={relativePath:f===void 0?o.path||"":f,caseSensitive:o.caseSensitive===!0,childrenIndex:c,route:o};if(m.relativePath.startsWith("/")){if(!m.relativePath.startsWith(r)&&d)return;xn(m.relativePath.startsWith(r),`Absolute route path "${m.relativePath}" nested under path "${r}" is not valid. An absolute child route path must start with the combined path of all its parent routes.`),m.relativePath=m.relativePath.slice(r.length)}let p=Ji([r,m.relativePath]),b=n.concat(m);o.children&&o.children.length>0&&(xn(o.index!==!0,`Index routes must not have child routes. Please remove all child routes from route path "${p}".`),G5(o.children,t,b,p,d)),!(o.path==null&&!o.index)&&t.push({path:p,score:VA(p,o.index),routesMeta:b})};return e.forEach((o,c)=>{var d;if(o.path===""||!((d=o.path)!=null&&d.includes("?")))s(o,c);else for(let f of V5(o.path))s(o,c,!0,f)}),t}function V5(e){let t=e.split("/");if(t.length===0)return[];let[n,...r]=t,a=n.endsWith("?"),s=n.replace(/\?$/,"");if(r.length===0)return a?[s,""]:[s];let o=V5(r.join("/")),c=[];return c.push(...o.map(d=>d===""?s:[s,d].join("/"))),a&&c.push(...o),c.map(d=>e.startsWith("/")&&d===""?"/":d)}function zA(e){e.sort((t,n)=>t.score!==n.score?n.score-t.score:KA(t.routesMeta.map(r=>r.childrenIndex),n.routesMeta.map(r=>r.childrenIndex)))}var HA=/^:[\w-]+$/,qA=3,jA=2,$A=1,YA=10,GA=-2,Zy=e=>e==="*";function VA(e,t){let n=e.split("/"),r=n.length;return n.some(Zy)&&(r+=GA),t&&(r+=jA),n.filter(a=>!Zy(a)).reduce((a,s)=>a+(HA.test(s)?qA:s===""?$A:YA),r)}function KA(e,t){return e.length===t.length&&e.slice(0,-1).every((r,a)=>r===t[a])?e[e.length-1]-t[t.length-1]:0}function WA(e,t,n=!1){let{routesMeta:r}=e,a={},s="/",o=[];for(let c=0;c<r.length;++c){let d=r[c],f=c===r.length-1,m=s==="/"?t:t.slice(s.length)||"/",p=Xh({path:d.relativePath,caseSensitive:d.caseSensitive,end:f},m),b=d.route;if(!p&&f&&n&&!r[r.length-1].route.index&&(p=Xh({path:d.relativePath,caseSensitive:d.caseSensitive,end:!1},m)),!p)return null;Object.assign(a,p.params),o.push({params:a,pathname:Ji([s,p.pathname]),pathnameBase:nk(Ji([s,p.pathnameBase])),route:b}),p.pathnameBase!=="/"&&(s=Ji([s,p.pathnameBase]))}return o}function Xh(e,t){typeof e=="string"&&(e={path:e,caseSensitive:!1,end:!0});let[n,r]=XA(e.path,e.caseSensitive,e.end),a=t.match(n);if(!a)return null;let s=a[0],o=s.replace(/(.)\/+$/,"$1"),c=a.slice(1);return{params:r.reduce((f,{paramName:m,isOptional:p},b)=>{if(m==="*"){let T=c[b]||"";o=s.slice(0,s.length-T.length).replace(/(.)\/+$/,"$1")}const y=c[b];return p&&!y?f[m]=void 0:f[m]=(y||"").replace(/%2F/g,"/"),f},{}),pathname:s,pathnameBase:o,pattern:e}}function XA(e,t=!1,n=!0){Ba(e==="*"||!e.endsWith("*")||e.endsWith("/*"),`Route path "${e}" will be treated as if it were "${e.replace(/\*$/,"/*")}" because the \`*\` character must always follow a \`/\` in the pattern. To get rid of this warning, please change the route path to "${e.replace(/\*$/,"/*")}".`);let r=[],a="^"+e.replace(/\/*\*?$/,"").replace(/^\/*/,"/").replace(/[\\.*+^${}|()[\]]/g,"\\$&").replace(/\/:([\w-]+)(\?)?/g,(o,c,d)=>(r.push({paramName:c,isOptional:d!=null}),d?"/?([^\\/]+)?":"/([^\\/]+)")).replace(/\/([\w-]+)\?(\/|$)/g,"(/$1)?$2");return e.endsWith("*")?(r.push({paramName:"*"}),a+=e==="*"||e==="/*"?"(.*)$":"(?:\\/(.+)|\\/*)$"):n?a+="\\/*$":e!==""&&e!=="/"&&(a+="(?:(?=\\/|$))"),[new RegExp(a,t?void 0:"i"),r]}function QA(e){try{return e.split("/").map(t=>decodeURIComponent(t).replace(/\//g,"%2F")).join("/")}catch(t){return Ba(!1,`The URL path "${e}" could not be decoded because it is a malformed URL segment. This is probably due to a bad percent encoding (${t}).`),e}}function ns(e,t){if(t==="/")return e;if(!e.toLowerCase().startsWith(t.toLowerCase()))return null;let n=t.endsWith("/")?t.length-1:t.length,r=e.charAt(n);return r&&r!=="/"?null:e.slice(n)||"/"}var ZA=/^(?:[a-z][a-z0-9+.-]*:|\/\/)/i,JA=e=>ZA.test(e);function ek(e,t="/"){let{pathname:n,search:r="",hash:a=""}=typeof e=="string"?ou(e):e,s;if(n)if(JA(n))s=n;else{if(n.includes("//")){let o=n;n=n.replace(/\/\/+/g,"/"),Ba(!1,`Pathnames cannot have embedded double slashes - normalizing ${o} -> ${n}`)}n.startsWith("/")?s=Jy(n.substring(1),"/"):s=Jy(n,t)}else s=t;return{pathname:s,search:rk(r),hash:ak(a)}}function Jy(e,t){let n=t.replace(/\/+$/,"").split("/");return e.split("/").forEach(a=>{a===".."?n.length>1&&n.pop():a!=="."&&n.push(a)}),n.length>1?n.join("/"):"/"}function g1(e,t,n,r){return`Cannot include a '${e}' character in a manually specified \`to.${t}\` field [${JSON.stringify(r)}].  Please separate it out to the \`to.${n}\` field. Alternatively you may provide the full path as a string in <Link to="..."> and the router will parse it for you.`}function tk(e){return e.filter((t,n)=>n===0||t.route.path&&t.route.path.length>0)}function Zb(e){let t=tk(e);return t.map((n,r)=>r===t.length-1?n.pathname:n.pathnameBase)}function Jb(e,t,n,r=!1){let a;typeof e=="string"?a=ou(e):(a={...e},xn(!a.pathname||!a.pathname.includes("?"),g1("?","pathname","search",a)),xn(!a.pathname||!a.pathname.includes("#"),g1("#","pathname","hash",a)),xn(!a.search||!a.search.includes("#"),g1("#","search","hash",a)));let s=e===""||a.pathname==="",o=s?"/":a.pathname,c;if(o==null)c=n;else{let p=t.length-1;if(!r&&o.startsWith("..")){let b=o.split("/");for(;b[0]==="..";)b.shift(),p-=1;a.pathname=b.join("/")}c=p>=0?t[p]:"/"}let d=ek(a,c),f=o&&o!=="/"&&o.endsWith("/"),m=(s||o===".")&&n.endsWith("/");return!d.pathname.endsWith("/")&&(f||m)&&(d.pathname+="/"),d}var Ji=e=>e.join("/").replace(/\/\/+/g,"/"),nk=e=>e.replace(/\/+$/,"").replace(/^\/*/,"/"),rk=e=>!e||e==="?"?"":e.startsWith("?")?e:"?"+e,ak=e=>!e||e==="#"?"":e.startsWith("#")?e:"#"+e;function ik(e){return e!=null&&typeof e.status=="number"&&typeof e.statusText=="string"&&typeof e.internal=="boolean"&&"data"in e}Object.getOwnPropertyNames(Object.prototype).sort().join("\0");var K5=["POST","PUT","PATCH","DELETE"];new Set(K5);var sk=["GET",...K5];new Set(sk);var uu=z.createContext(null);uu.displayName="DataRouter";var Df=z.createContext(null);Df.displayName="DataRouterState";z.createContext(!1);var W5=z.createContext({isTransitioning:!1});W5.displayName="ViewTransition";var lk=z.createContext(new Map);lk.displayName="Fetchers";var ok=z.createContext(null);ok.displayName="Await";var Za=z.createContext(null);Za.displayName="Navigation";var T0=z.createContext(null);T0.displayName="Location";var vi=z.createContext({outlet:null,matches:[],isDataRoute:!1});vi.displayName="Route";var e2=z.createContext(null);e2.displayName="RouteError";function uk(e,{relative:t}={}){xn(cu(),"useHref() may be used only in the context of a <Router> component.");let{basename:n,navigator:r}=z.useContext(Za),{hash:a,pathname:s,search:o}=x0(e,{relative:t}),c=s;return n!=="/"&&(c=s==="/"?n:Ji([n,s])),r.createHref({pathname:c,search:o,hash:a})}function cu(){return z.useContext(T0)!=null}function ls(){return xn(cu(),"useLocation() may be used only in the context of a <Router> component."),z.useContext(T0).location}var X5="You should call navigate() in a React.useEffect(), not when your component is first rendered.";function Q5(e){z.useContext(Za).static||z.useLayoutEffect(e)}function Z5(){let{isDataRoute:e}=z.useContext(vi);return e?Tk():ck()}function ck(){xn(cu(),"useNavigate() may be used only in the context of a <Router> component.");let e=z.useContext(uu),{basename:t,navigator:n}=z.useContext(Za),{matches:r}=z.useContext(vi),{pathname:a}=ls(),s=JSON.stringify(Zb(r)),o=z.useRef(!1);return Q5(()=>{o.current=!0}),z.useCallback((d,f={})=>{if(Ba(o.current,X5),!o.current)return;if(typeof d=="number"){n.go(d);return}let m=Jb(d,JSON.parse(s),a,f.relative==="path");e==null&&t!=="/"&&(m.pathname=m.pathname==="/"?t:Ji([t,m.pathname])),(f.replace?n.replace:n.push)(m,f.state,f)},[t,n,s,a,e])}z.createContext(null);function x0(e,{relative:t}={}){let{matches:n}=z.useContext(vi),{pathname:r}=ls(),a=JSON.stringify(Zb(n));return z.useMemo(()=>Jb(e,JSON.parse(a),r,t==="path"),[e,a,r,t])}function dk(e,t){return J5(e,t)}function J5(e,t,n,r,a){var O;xn(cu(),"useRoutes() may be used only in the context of a <Router> component.");let{navigator:s}=z.useContext(Za),{matches:o}=z.useContext(vi),c=o[o.length-1],d=c?c.params:{},f=c?c.pathname:"/",m=c?c.pathnameBase:"/",p=c&&c.route;{let D=p&&p.path||"";e8(f,!p||D.endsWith("*")||D.endsWith("*?"),`You rendered descendant <Routes> (or called \`useRoutes()\`) at "${f}" (under <Route path="${D}">) but the parent route path has no trailing "*". This means if you navigate deeper, the parent won't match anymore and therefore the child routes will never render.

Please change the parent <Route path="${D}"> to <Route path="${D==="/"?"*":`${D}/*`}">.`)}let b=ls(),y;if(t){let D=typeof t=="string"?ou(t):t;xn(m==="/"||((O=D.pathname)==null?void 0:O.startsWith(m)),`When overriding the location using \`<Routes location>\` or \`useRoutes(routes, location)\`, the location pathname must begin with the portion of the URL pathname that was matched by all parent routes. The current pathname base is "${m}" but pathname "${D.pathname}" was given in the \`location\` prop.`),y=D}else y=b;let T=y.pathname||"/",x=T;if(m!=="/"){let D=m.replace(/^\//,"").split("/");x="/"+T.replace(/^\//,"").split("/").slice(D.length).join("/")}let C=Y5(e,{pathname:x});Ba(p||C!=null,`No routes matched location "${y.pathname}${y.search}${y.hash}" `),Ba(C==null||C[C.length-1].route.element!==void 0||C[C.length-1].route.Component!==void 0||C[C.length-1].route.lazy!==void 0,`Matched leaf route at location "${y.pathname}${y.search}${y.hash}" does not have an element or Component. This means it will render an <Outlet /> with a null value by default resulting in an "empty" page.`);let w=gk(C&&C.map(D=>Object.assign({},D,{params:Object.assign({},d,D.params),pathname:Ji([m,s.encodeLocation?s.encodeLocation(D.pathname.replace(/\?/g,"%3F").replace(/#/g,"%23")).pathname:D.pathname]),pathnameBase:D.pathnameBase==="/"?m:Ji([m,s.encodeLocation?s.encodeLocation(D.pathnameBase.replace(/\?/g,"%3F").replace(/#/g,"%23")).pathname:D.pathnameBase])})),o,n,r,a);return t&&w?z.createElement(T0.Provider,{value:{location:{pathname:"/",search:"",hash:"",state:null,key:"default",...y},navigationType:"POP"}},w):w}function hk(){let e=Ek(),t=ik(e)?`${e.status} ${e.statusText}`:e instanceof Error?e.message:JSON.stringify(e),n=e instanceof Error?e.stack:null,r="rgba(200,200,200, 0.5)",a={padding:"0.5rem",backgroundColor:r},s={padding:"2px 4px",backgroundColor:r},o=null;return console.error("Error handled by React Router default ErrorBoundary:",e),o=z.createElement(z.Fragment,null,z.createElement("p",null,"💿 Hey developer 👋"),z.createElement("p",null,"You can provide a way better UX than this when your app throws errors by providing your own ",z.createElement("code",{style:s},"ErrorBoundary")," or"," ",z.createElement("code",{style:s},"errorElement")," prop on your route.")),z.createElement(z.Fragment,null,z.createElement("h2",null,"Unexpected Application Error!"),z.createElement("h3",{style:{fontStyle:"italic"}},t),n?z.createElement("pre",{style:a},n):null,o)}var fk=z.createElement(hk,null),mk=class extends z.Component{constructor(e){super(e),this.state={location:e.location,revalidation:e.revalidation,error:e.error}}static getDerivedStateFromError(e){return{error:e}}static getDerivedStateFromProps(e,t){return t.location!==e.location||t.revalidation!=="idle"&&e.revalidation==="idle"?{error:e.error,location:e.location,revalidation:e.revalidation}:{error:e.error!==void 0?e.error:t.error,location:t.location,revalidation:e.revalidation||t.revalidation}}componentDidCatch(e,t){this.props.onError?this.props.onError(e,t):console.error("React Router caught the following error during render",e)}render(){return this.state.error!==void 0?z.createElement(vi.Provider,{value:this.props.routeContext},z.createElement(e2.Provider,{value:this.state.error,children:this.props.component})):this.props.children}};function pk({routeContext:e,match:t,children:n}){let r=z.useContext(uu);return r&&r.static&&r.staticContext&&(t.route.errorElement||t.route.ErrorBoundary)&&(r.staticContext._deepestRenderedBoundaryId=t.route.id),z.createElement(vi.Provider,{value:e},n)}function gk(e,t=[],n=null,r=null,a=null){if(e==null){if(!n)return null;if(n.errors)e=n.matches;else if(t.length===0&&!n.initialized&&n.matches.length>0)e=n.matches;else return null}let s=e,o=n==null?void 0:n.errors;if(o!=null){let m=s.findIndex(p=>p.route.id&&(o==null?void 0:o[p.route.id])!==void 0);xn(m>=0,`Could not find a matching route for errors on route IDs: ${Object.keys(o).join(",")}`),s=s.slice(0,Math.min(s.length,m+1))}let c=!1,d=-1;if(n)for(let m=0;m<s.length;m++){let p=s[m];if((p.route.HydrateFallback||p.route.hydrateFallbackElement)&&(d=m),p.route.id){let{loaderData:b,errors:y}=n,T=p.route.loader&&!b.hasOwnProperty(p.route.id)&&(!y||y[p.route.id]===void 0);if(p.route.lazy||T){c=!0,d>=0?s=s.slice(0,d+1):s=[s[0]];break}}}let f=n&&r?(m,p)=>{var b,y;r(m,{location:n.location,params:((y=(b=n.matches)==null?void 0:b[0])==null?void 0:y.params)??{},errorInfo:p})}:void 0;return s.reduceRight((m,p,b)=>{let y,T=!1,x=null,C=null;n&&(y=o&&p.route.id?o[p.route.id]:void 0,x=p.route.errorElement||fk,c&&(d<0&&b===0?(e8("route-fallback",!1,"No `HydrateFallback` element provided to render during initial hydration"),T=!0,C=null):d===b&&(T=!0,C=p.route.hydrateFallbackElement||null)));let w=t.concat(s.slice(0,b+1)),O=()=>{let D;return y?D=x:T?D=C:p.route.Component?D=z.createElement(p.route.Component,null):p.route.element?D=p.route.element:D=m,z.createElement(pk,{match:p,routeContext:{outlet:m,matches:w,isDataRoute:n!=null},children:D})};return n&&(p.route.ErrorBoundary||p.route.errorElement||b===0)?z.createElement(mk,{location:n.location,revalidation:n.revalidation,component:x,error:y,children:O(),routeContext:{outlet:null,matches:w,isDataRoute:!0},onError:f}):O()},null)}function t2(e){return`${e} must be used within a data router.  See https://reactrouter.com/en/main/routers/picking-a-router.`}function bk(e){let t=z.useContext(uu);return xn(t,t2(e)),t}function vk(e){let t=z.useContext(Df);return xn(t,t2(e)),t}function yk(e){let t=z.useContext(vi);return xn(t,t2(e)),t}function n2(e){let t=yk(e),n=t.matches[t.matches.length-1];return xn(n.route.id,`${e} can only be used on routes that contain a unique "id"`),n.route.id}function _k(){return n2("useRouteId")}function Ek(){var r;let e=z.useContext(e2),t=vk("useRouteError"),n=n2("useRouteError");return e!==void 0?e:(r=t.errors)==null?void 0:r[n]}function Tk(){let{router:e}=bk("useNavigate"),t=n2("useNavigate"),n=z.useRef(!1);return Q5(()=>{n.current=!0}),z.useCallback(async(a,s={})=>{Ba(n.current,X5),n.current&&(typeof a=="number"?e.navigate(a):await e.navigate(a,{fromRouteId:t,...s}))},[e,t])}var e_={};function e8(e,t,n){!t&&!e_[e]&&(e_[e]=!0,Ba(!1,n))}z.memo(xk);function xk({routes:e,future:t,state:n,unstable_onError:r}){return J5(e,void 0,n,r,t)}function t8({to:e,replace:t,state:n,relative:r}){xn(cu(),"<Navigate> may be used only in the context of a <Router> component.");let{static:a}=z.useContext(Za);Ba(!a,"<Navigate> must not be used on the initial render in a <StaticRouter>. This is a no-op, but you should modify your code so the <Navigate> is only ever rendered in response to some user interaction or state change.");let{matches:s}=z.useContext(vi),{pathname:o}=ls(),c=Z5(),d=Jb(e,Zb(s),o,r==="path"),f=JSON.stringify(d);return z.useEffect(()=>{c(JSON.parse(f),{replace:t,state:n,relative:r})},[c,f,r,t,n]),null}function Dh(e){xn(!1,"A <Route> is only ever to be used as the child of <Routes> element, never rendered directly. Please wrap your <Route> in a <Routes>.")}function Sk({basename:e="/",children:t=null,location:n,navigationType:r="POP",navigator:a,static:s=!1}){xn(!cu(),"You cannot render a <Router> inside another <Router>. You should never have more than one in your app.");let o=e.replace(/^\/*/,"/"),c=z.useMemo(()=>({basename:o,navigator:a,static:s,future:{}}),[o,a,s]);typeof n=="string"&&(n=ou(n));let{pathname:d="/",search:f="",hash:m="",state:p=null,key:b="default"}=n,y=z.useMemo(()=>{let T=ns(d,o);return T==null?null:{location:{pathname:T,search:f,hash:m,state:p,key:b},navigationType:r}},[o,d,f,m,p,b,r]);return Ba(y!=null,`<Router basename="${o}"> is not able to match the URL "${d}${f}${m}" because it does not start with the basename, so the <Router> won't render anything.`),y==null?null:z.createElement(Za.Provider,{value:c},z.createElement(T0.Provider,{children:t,value:y}))}function wk({children:e,location:t}){return dk(Fg(e),t)}function Fg(e,t=[]){let n=[];return z.Children.forEach(e,(r,a)=>{if(!z.isValidElement(r))return;let s=[...t,a];if(r.type===z.Fragment){n.push.apply(n,Fg(r.props.children,s));return}xn(r.type===Dh,`[${typeof r.type=="string"?r.type:r.type.name}] is not a <Route> component. All component children of <Routes> must be a <Route> or <React.Fragment>`),xn(!r.props.index||!r.props.children,"An index route cannot have child routes.");let o={id:r.props.id||s.join("-"),caseSensitive:r.props.caseSensitive,element:r.props.element,Component:r.props.Component,index:r.props.index,path:r.props.path,middleware:r.props.middleware,loader:r.props.loader,action:r.props.action,hydrateFallbackElement:r.props.hydrateFallbackElement,HydrateFallback:r.props.HydrateFallback,errorElement:r.props.errorElement,ErrorBoundary:r.props.ErrorBoundary,hasErrorBoundary:r.props.hasErrorBoundary===!0||r.props.ErrorBoundary!=null||r.props.errorElement!=null,shouldRevalidate:r.props.shouldRevalidate,handle:r.props.handle,lazy:r.props.lazy};r.props.children&&(o.children=Fg(r.props.children,s)),n.push(o)}),n}var Lh="get",Ph="application/x-www-form-urlencoded";function Lf(e){return e!=null&&typeof e.tagName=="string"}function Ak(e){return Lf(e)&&e.tagName.toLowerCase()==="button"}function kk(e){return Lf(e)&&e.tagName.toLowerCase()==="form"}function Ck(e){return Lf(e)&&e.tagName.toLowerCase()==="input"}function Nk(e){return!!(e.metaKey||e.altKey||e.ctrlKey||e.shiftKey)}function Ok(e,t){return e.button===0&&(!t||t==="_self")&&!Nk(e)}var th=null;function Rk(){if(th===null)try{new FormData(document.createElement("form"),0),th=!1}catch{th=!0}return th}var Mk=new Set(["application/x-www-form-urlencoded","multipart/form-data","text/plain"]);function b1(e){return e!=null&&!Mk.has(e)?(Ba(!1,`"${e}" is not a valid \`encType\` for \`<Form>\`/\`<fetcher.Form>\` and will default to "${Ph}"`),null):e}function Ik(e,t){let n,r,a,s,o;if(kk(e)){let c=e.getAttribute("action");r=c?ns(c,t):null,n=e.getAttribute("method")||Lh,a=b1(e.getAttribute("enctype"))||Ph,s=new FormData(e)}else if(Ak(e)||Ck(e)&&(e.type==="submit"||e.type==="image")){let c=e.form;if(c==null)throw new Error('Cannot submit a <button> or <input type="submit"> without a <form>');let d=e.getAttribute("formaction")||c.getAttribute("action");if(r=d?ns(d,t):null,n=e.getAttribute("formmethod")||c.getAttribute("method")||Lh,a=b1(e.getAttribute("formenctype"))||b1(c.getAttribute("enctype"))||Ph,s=new FormData(c,e),!Rk()){let{name:f,type:m,value:p}=e;if(m==="image"){let b=f?`${f}.`:"";s.append(`${b}x`,"0"),s.append(`${b}y`,"0")}else f&&s.append(f,p)}}else{if(Lf(e))throw new Error('Cannot submit element that is not <form>, <button>, or <input type="submit|image">');n=Lh,r=null,a=Ph,o=e}return s&&a==="text/plain"&&(o=s,s=void 0),{action:r,method:n.toLowerCase(),encType:a,formData:s,body:o}}Object.getOwnPropertyNames(Object.prototype).sort().join("\0");function r2(e,t){if(e===!1||e===null||typeof e>"u")throw new Error(t)}function Dk(e,t,n){let r=typeof e=="string"?new URL(e,typeof window>"u"?"server://singlefetch/":window.location.origin):e;return r.pathname==="/"?r.pathname=`_root.${n}`:t&&ns(r.pathname,t)==="/"?r.pathname=`${t.replace(/\/$/,"")}/_root.${n}`:r.pathname=`${r.pathname.replace(/\/$/,"")}.${n}`,r}async function Lk(e,t){if(e.id in t)return t[e.id];try{let n=await import(e.module);return t[e.id]=n,n}catch(n){return console.error(`Error loading route module \`${e.module}\`, reloading page...`),console.error(n),window.__reactRouterContext&&window.__reactRouterContext.isSpaMode,window.location.reload(),new Promise(()=>{})}}function Pk(e){return e==null?!1:e.href==null?e.rel==="preload"&&typeof e.imageSrcSet=="string"&&typeof e.imageSizes=="string":typeof e.rel=="string"&&typeof e.href=="string"}async function Bk(e,t,n){let r=await Promise.all(e.map(async a=>{let s=t.routes[a.route.id];if(s){let o=await Lk(s,n);return o.links?o.links():[]}return[]}));return Hk(r.flat(1).filter(Pk).filter(a=>a.rel==="stylesheet"||a.rel==="preload").map(a=>a.rel==="stylesheet"?{...a,rel:"prefetch",as:"style"}:{...a,rel:"prefetch"}))}function t_(e,t,n,r,a,s){let o=(d,f)=>n[f]?d.route.id!==n[f].route.id:!0,c=(d,f)=>{var m;return n[f].pathname!==d.pathname||((m=n[f].route.path)==null?void 0:m.endsWith("*"))&&n[f].params["*"]!==d.params["*"]};return s==="assets"?t.filter((d,f)=>o(d,f)||c(d,f)):s==="data"?t.filter((d,f)=>{var p;let m=r.routes[d.route.id];if(!m||!m.hasLoader)return!1;if(o(d,f)||c(d,f))return!0;if(d.route.shouldRevalidate){let b=d.route.shouldRevalidate({currentUrl:new URL(a.pathname+a.search+a.hash,window.origin),currentParams:((p=n[0])==null?void 0:p.params)||{},nextUrl:new URL(e,window.origin),nextParams:d.params,defaultShouldRevalidate:!0});if(typeof b=="boolean")return b}return!0}):[]}function Uk(e,t,{includeHydrateFallback:n}={}){return Fk(e.map(r=>{let a=t.routes[r.route.id];if(!a)return[];let s=[a.module];return a.clientActionModule&&(s=s.concat(a.clientActionModule)),a.clientLoaderModule&&(s=s.concat(a.clientLoaderModule)),n&&a.hydrateFallbackModule&&(s=s.concat(a.hydrateFallbackModule)),a.imports&&(s=s.concat(a.imports)),s}).flat(1))}function Fk(e){return[...new Set(e)]}function zk(e){let t={},n=Object.keys(e).sort();for(let r of n)t[r]=e[r];return t}function Hk(e,t){let n=new Set;return new Set(t),e.reduce((r,a)=>{let s=JSON.stringify(zk(a));return n.has(s)||(n.add(s),r.push({key:s,link:a})),r},[])}function n8(){let e=z.useContext(uu);return r2(e,"You must render this element inside a <DataRouterContext.Provider> element"),e}function qk(){let e=z.useContext(Df);return r2(e,"You must render this element inside a <DataRouterStateContext.Provider> element"),e}var a2=z.createContext(void 0);a2.displayName="FrameworkContext";function r8(){let e=z.useContext(a2);return r2(e,"You must render this element inside a <HydratedRouter> element"),e}function jk(e,t){let n=z.useContext(a2),[r,a]=z.useState(!1),[s,o]=z.useState(!1),{onFocus:c,onBlur:d,onMouseEnter:f,onMouseLeave:m,onTouchStart:p}=t,b=z.useRef(null);z.useEffect(()=>{if(e==="render"&&o(!0),e==="viewport"){let x=w=>{w.forEach(O=>{o(O.isIntersecting)})},C=new IntersectionObserver(x,{threshold:.5});return b.current&&C.observe(b.current),()=>{C.disconnect()}}},[e]),z.useEffect(()=>{if(r){let x=setTimeout(()=>{o(!0)},100);return()=>{clearTimeout(x)}}},[r]);let y=()=>{a(!0)},T=()=>{a(!1),o(!1)};return n?e!=="intent"?[s,b,{}]:[s,b,{onFocus:xc(c,y),onBlur:xc(d,T),onMouseEnter:xc(f,y),onMouseLeave:xc(m,T),onTouchStart:xc(p,y)}]:[!1,b,{}]}function xc(e,t){return n=>{e&&e(n),n.defaultPrevented||t(n)}}function $k({page:e,...t}){let{router:n}=n8(),r=z.useMemo(()=>Y5(n.routes,e,n.basename),[n.routes,e,n.basename]);return r?z.createElement(Gk,{page:e,matches:r,...t}):null}function Yk(e){let{manifest:t,routeModules:n}=r8(),[r,a]=z.useState([]);return z.useEffect(()=>{let s=!1;return Bk(e,t,n).then(o=>{s||a(o)}),()=>{s=!0}},[e,t,n]),r}function Gk({page:e,matches:t,...n}){let r=ls(),{manifest:a,routeModules:s}=r8(),{basename:o}=n8(),{loaderData:c,matches:d}=qk(),f=z.useMemo(()=>t_(e,t,d,a,r,"data"),[e,t,d,a,r]),m=z.useMemo(()=>t_(e,t,d,a,r,"assets"),[e,t,d,a,r]),p=z.useMemo(()=>{if(e===r.pathname+r.search+r.hash)return[];let T=new Set,x=!1;if(t.forEach(w=>{var D;let O=a.routes[w.route.id];!O||!O.hasLoader||(!f.some(Y=>Y.route.id===w.route.id)&&w.route.id in c&&((D=s[w.route.id])!=null&&D.shouldRevalidate)||O.hasClientLoader?x=!0:T.add(w.route.id))}),T.size===0)return[];let C=Dk(e,o,"data");return x&&T.size>0&&C.searchParams.set("_routes",t.filter(w=>T.has(w.route.id)).map(w=>w.route.id).join(",")),[C.pathname+C.search]},[o,c,r,a,f,t,e,s]),b=z.useMemo(()=>Uk(m,a),[m,a]),y=Yk(m);return z.createElement(z.Fragment,null,p.map(T=>z.createElement("link",{key:T,rel:"prefetch",as:"fetch",href:T,...n})),b.map(T=>z.createElement("link",{key:T,rel:"modulepreload",href:T,...n})),y.map(({key:T,link:x})=>z.createElement("link",{key:T,nonce:n.nonce,...x})))}function Vk(...e){return t=>{e.forEach(n=>{typeof n=="function"?n(t):n!=null&&(n.current=t)})}}var a8=typeof window<"u"&&typeof window.document<"u"&&typeof window.document.createElement<"u";try{a8&&(window.__reactRouterVersion="7.9.6")}catch{}function Kk({basename:e,children:t,window:n}){let r=z.useRef();r.current==null&&(r.current=LA({window:n,v5Compat:!0}));let a=r.current,[s,o]=z.useState({action:a.action,location:a.location}),c=z.useCallback(d=>{z.startTransition(()=>o(d))},[o]);return z.useLayoutEffect(()=>a.listen(c),[a,c]),z.createElement(Sk,{basename:e,children:t,location:s.location,navigationType:s.action,navigator:a})}var i8=/^(?:[a-z][a-z0-9+.-]*:|\/\/)/i,s8=z.forwardRef(function({onClick:t,discover:n="render",prefetch:r="none",relative:a,reloadDocument:s,replace:o,state:c,target:d,to:f,preventScrollReset:m,viewTransition:p,...b},y){let{basename:T}=z.useContext(Za),x=typeof f=="string"&&i8.test(f),C,w=!1;if(typeof f=="string"&&x&&(C=f,a8))try{let Q=new URL(window.location.href),G=f.startsWith("//")?new URL(Q.protocol+f):new URL(f),se=ns(G.pathname,T);G.origin===Q.origin&&se!=null?f=se+G.search+G.hash:w=!0}catch{Ba(!1,`<Link to="${f}"> contains an invalid URL which will probably break when clicked - please update to a valid URL path.`)}let O=uk(f,{relative:a}),[D,Y,H]=jk(r,b),B=Zk(f,{replace:o,state:c,target:d,preventScrollReset:m,relative:a,viewTransition:p});function ee(Q){t&&t(Q),Q.defaultPrevented||B(Q)}let Z=z.createElement("a",{...b,...H,href:C||O,onClick:w||s?t:ee,ref:Vk(y,Y),target:d,"data-discover":!x&&n==="render"?"true":void 0});return D&&!x?z.createElement(z.Fragment,null,Z,z.createElement($k,{page:O})):Z});s8.displayName="Link";var Wk=z.forwardRef(function({"aria-current":t="page",caseSensitive:n=!1,className:r="",end:a=!1,style:s,to:o,viewTransition:c,children:d,...f},m){let p=x0(o,{relative:f.relative}),b=ls(),y=z.useContext(Df),{navigator:T,basename:x}=z.useContext(Za),C=y!=null&&rC(p)&&c===!0,w=T.encodeLocation?T.encodeLocation(p).pathname:p.pathname,O=b.pathname,D=y&&y.navigation&&y.navigation.location?y.navigation.location.pathname:null;n||(O=O.toLowerCase(),D=D?D.toLowerCase():null,w=w.toLowerCase()),D&&x&&(D=ns(D,x)||D);const Y=w!=="/"&&w.endsWith("/")?w.length-1:w.length;let H=O===w||!a&&O.startsWith(w)&&O.charAt(Y)==="/",B=D!=null&&(D===w||!a&&D.startsWith(w)&&D.charAt(w.length)==="/"),ee={isActive:H,isPending:B,isTransitioning:C},Z=H?t:void 0,Q;typeof r=="function"?Q=r(ee):Q=[r,H?"active":null,B?"pending":null,C?"transitioning":null].filter(Boolean).join(" ");let G=typeof s=="function"?s(ee):s;return z.createElement(s8,{...f,"aria-current":Z,className:Q,ref:m,style:G,to:o,viewTransition:c},typeof d=="function"?d(ee):d)});Wk.displayName="NavLink";var Xk=z.forwardRef(({discover:e="render",fetcherKey:t,navigate:n,reloadDocument:r,replace:a,state:s,method:o=Lh,action:c,onSubmit:d,relative:f,preventScrollReset:m,viewTransition:p,...b},y)=>{let T=tC(),x=nC(c,{relative:f}),C=o.toLowerCase()==="get"?"get":"post",w=typeof c=="string"&&i8.test(c),O=D=>{if(d&&d(D),D.defaultPrevented)return;D.preventDefault();let Y=D.nativeEvent.submitter,H=(Y==null?void 0:Y.getAttribute("formmethod"))||o;T(Y||D.currentTarget,{fetcherKey:t,method:H,navigate:n,replace:a,state:s,relative:f,preventScrollReset:m,viewTransition:p})};return z.createElement("form",{ref:y,method:C,action:x,onSubmit:r?d:O,...b,"data-discover":!w&&e==="render"?"true":void 0})});Xk.displayName="Form";function Qk(e){return`${e} must be used within a data router.  See https://reactrouter.com/en/main/routers/picking-a-router.`}function l8(e){let t=z.useContext(uu);return xn(t,Qk(e)),t}function Zk(e,{target:t,replace:n,state:r,preventScrollReset:a,relative:s,viewTransition:o}={}){let c=Z5(),d=ls(),f=x0(e,{relative:s});return z.useCallback(m=>{if(Ok(m,t)){m.preventDefault();let p=n!==void 0?n:u0(d)===u0(f);c(e,{replace:p,state:r,preventScrollReset:a,relative:s,viewTransition:o})}},[d,c,f,n,r,t,e,a,s,o])}var Jk=0,eC=()=>`__${String(++Jk)}__`;function tC(){let{router:e}=l8("useSubmit"),{basename:t}=z.useContext(Za),n=_k();return z.useCallback(async(r,a={})=>{let{action:s,method:o,encType:c,formData:d,body:f}=Ik(r,t);if(a.navigate===!1){let m=a.fetcherKey||eC();await e.fetch(m,n,a.action||s,{preventScrollReset:a.preventScrollReset,formData:d,body:f,formMethod:a.method||o,formEncType:a.encType||c,flushSync:a.flushSync})}else await e.navigate(a.action||s,{preventScrollReset:a.preventScrollReset,formData:d,body:f,formMethod:a.method||o,formEncType:a.encType||c,replace:a.replace,state:a.state,fromRouteId:n,flushSync:a.flushSync,viewTransition:a.viewTransition})},[e,t,n])}function nC(e,{relative:t}={}){let{basename:n}=z.useContext(Za),r=z.useContext(vi);xn(r,"useFormAction must be used inside a RouteContext");let[a]=r.matches.slice(-1),s={...x0(e||".",{relative:t})},o=ls();if(e==null){s.search=o.search;let c=new URLSearchParams(s.search),d=c.getAll("index");if(d.some(m=>m==="")){c.delete("index"),d.filter(p=>p).forEach(p=>c.append("index",p));let m=c.toString();s.search=m?`?${m}`:""}}return(!e||e===".")&&a.route.index&&(s.search=s.search?s.search.replace(/^\?/,"?index&"):"?index"),n!=="/"&&(s.pathname=s.pathname==="/"?n:Ji([n,s.pathname])),u0(s)}function rC(e,{relative:t}={}){let n=z.useContext(W5);xn(n!=null,"`useViewTransitionState` must be used within `react-router-dom`'s `RouterProvider`.  Did you accidentally import `RouterProvider` from `react-router`?");let{basename:r}=l8("useViewTransitionState"),a=x0(e,{relative:t});if(!n.isTransitioning)return!1;let s=ns(n.currentLocation.pathname,r)||n.currentLocation.pathname,o=ns(n.nextLocation.pathname,r)||n.nextLocation.pathname;return Xh(a.pathname,o)!=null||Xh(a.pathname,s)!=null}var ql,aC=(ql=class extends Error{constructor(n){super(new.target.formatMessage(new.target.kind,n.message,n.code,n.docsUrl),{cause:n.cause});qi(this,"clerkError",!0);qi(this,"code");qi(this,"longMessage");qi(this,"docsUrl");qi(this,"cause");Object.setPrototypeOf(this,ql.prototype),this.code=n.code,this.docsUrl=n.docsUrl,this.longMessage=n.longMessage,this.cause=n.cause}get name(){return this.constructor.name}toString(){return`[${this.name}]
Message:${this.message}`}static formatMessage(n,r,a,s){const o="Clerk:",c=new RegExp(o.replace(" ","\\s*"),"i");return r=r.replace(c,""),r=`${o} ${r.trim()}

(code="${a}")

`,s&&(r+=`

Docs: ${s}`),r}},qi(ql,"kind","ClerkError"),ql);const iC=Object.freeze({InvalidProxyUrlErrorMessage:"The proxyUrl passed to Clerk is invalid. The expected value for proxyUrl is an absolute URL or a relative path with a leading '/'. (key={{url}})",InvalidPublishableKeyErrorMessage:"The publishableKey passed to Clerk is invalid. You can get your Publishable key at https://dashboard.clerk.com/last-active?path=api-keys. (key={{key}})",MissingPublishableKeyErrorMessage:"Missing publishableKey. You can get your key at https://dashboard.clerk.com/last-active?path=api-keys.",MissingSecretKeyErrorMessage:"Missing secretKey. You can get your key at https://dashboard.clerk.com/last-active?path=api-keys.",MissingClerkProvider:"{{source}} can only be used within the <ClerkProvider /> component. Learn more: https://clerk.com/docs/components/clerk-provider"});function o8({packageName:e,customMessages:t}){let n=e;function r(s,o){if(!o)return`${n}: ${s}`;let c=s;const d=s.matchAll(/{{([a-zA-Z0-9-_]+)}}/g);for(const f of d){const m=(o[f[1]]||"").toString();c=c.replace(`{{${f[1]}}}`,m)}return`${n}: ${c}`}const a={...iC,...t};return{setPackageName({packageName:s}){return typeof s=="string"&&(n=s),this},setMessages({customMessages:s}){return Object.assign(a,s||{}),this},throwInvalidPublishableKeyError(s){throw new Error(r(a.InvalidPublishableKeyErrorMessage,s))},throwInvalidProxyUrl(s){throw new Error(r(a.InvalidProxyUrlErrorMessage,s))},throwMissingPublishableKeyError(){throw new Error(r(a.MissingPublishableKeyErrorMessage))},throwMissingSecretKeyError(){throw new Error(r(a.MissingSecretKeyErrorMessage))},throwMissingClerkProviderError(s){throw new Error(r(a.MissingClerkProvider,s))},throw(s){throw new Error(r(s))}}}var jl,zg=(jl=class extends aC{constructor(n,r){super({...r,message:n});qi(this,"clerkRuntimeError",!0);Object.setPrototypeOf(this,jl.prototype)}},qi(jl,"kind","ClerkRuntimeError"),jl);const sC={strict_mfa:{afterMinutes:10,level:"multi_factor"},strict:{afterMinutes:10,level:"second_factor"},moderate:{afterMinutes:60,level:"second_factor"},lax:{afterMinutes:1440,level:"second_factor"}},lC=new Set(["first_factor","second_factor","multi_factor"]),oC=new Set(["strict_mfa","strict","moderate","lax"]),uC=e=>typeof e=="number"&&e>0,cC=e=>lC.has(e),dC=e=>oC.has(e),v1=e=>e.replace(/^(org:)*/,"org:"),hC=(e,t)=>{const{orgId:n,orgRole:r,orgPermissions:a}=t;return!e.role&&!e.permission||!n||!r||!a?null:e.permission?a.includes(v1(e.permission)):e.role?v1(r)===v1(e.role):null},n_=(e,t)=>{const{org:n,user:r}=mC(e),[a,s]=t.split(":"),o=s||a;return a==="org"?n.includes(o):a==="user"?r.includes(o):[...n,...r].includes(o)},fC=(e,t)=>{const{features:n,plans:r}=t;return e.feature&&n?n_(n,e.feature):e.plan&&r?n_(r,e.plan):null},mC=e=>{const t=e?e.split(",").map(n=>n.trim()):[];return{org:t.filter(n=>n.split(":")[0].includes("o")).map(n=>n.split(":")[1]),user:t.filter(n=>n.split(":")[0].includes("u")).map(n=>n.split(":")[1])}},pC=e=>{if(!e)return!1;const t=a=>typeof a=="string"?sC[a]:a,n=typeof e=="string"&&dC(e),r=typeof e=="object"&&cC(e.level)&&uC(e.afterMinutes);return n||r?t.bind(null,e):!1},gC=(e,{factorVerificationAge:t})=>{if(!e.reverification||!t)return null;const n=pC(e.reverification);if(!n)return null;const{level:r,afterMinutes:a}=n(),[s,o]=t,c=s!==-1?a>s:null,d=o!==-1?a>o:null;switch(r){case"first_factor":return c;case"second_factor":return o!==-1?d:c;case"multi_factor":return o===-1?c:c&&d}},bC=e=>t=>{if(!e.userId)return!1;const n=fC(t,e),r=hC(t,e),a=gC(t,e);return[n||r,a].some(s=>s===null)?[n||r,a].some(s=>s===!0):[n||r,a].every(s=>s===!0)},vC=({authObject:{sessionId:e,sessionStatus:t,userId:n,actor:r,orgId:a,orgRole:s,orgSlug:o,signOut:c,getToken:d,has:f,sessionClaims:m},options:{treatPendingAsSignedOut:p=!0}})=>{if(e===void 0&&n===void 0)return{isLoaded:!1,isSignedIn:void 0,sessionId:e,sessionClaims:void 0,userId:n,actor:void 0,orgId:void 0,orgRole:void 0,orgSlug:void 0,has:void 0,signOut:c,getToken:d};if(e===null&&n===null)return{isLoaded:!0,isSignedIn:!1,sessionId:e,userId:n,sessionClaims:null,actor:null,orgId:null,orgRole:null,orgSlug:null,has:()=>!1,signOut:c,getToken:d};if(p&&t==="pending")return{isLoaded:!0,isSignedIn:!1,sessionId:null,userId:null,sessionClaims:null,actor:null,orgId:null,orgRole:null,orgSlug:null,has:()=>!1,signOut:c,getToken:d};if(e&&m&&n&&a&&s)return{isLoaded:!0,isSignedIn:!0,sessionId:e,sessionClaims:m,userId:n,actor:r||null,orgId:a,orgRole:s,orgSlug:o||null,has:f,signOut:c,getToken:d};if(e&&m&&n&&!a)return{isLoaded:!0,isSignedIn:!0,sessionId:e,sessionClaims:m,userId:n,actor:r||null,orgId:null,orgRole:null,orgSlug:null,has:f,signOut:c,getToken:d}},yC=[".lcl.dev",".stg.dev",".lclstage.dev",".stgstage.dev",".dev.lclclerk.com",".stg.lclclerk.com",".accounts.lclclerk.com","accountsstage.dev","accounts.dev"],u8=e=>typeof atob<"u"&&typeof atob=="function"?atob(e):typeof global<"u"&&global.Buffer?new global.Buffer(e,"base64").toString():e,c8="pk_live_",_C="pk_test_";function d8(e){if(!e.endsWith("$"))return!1;const t=e.slice(0,-1);return t.includes("$")?!1:t.includes(".")}function r_(e,t={}){if(e=e||"",!e||!Hg(e)){if(t.fatal&&!e)throw new Error("Publishable key is missing. Ensure that your publishable key is correctly configured. Double-check your environment configuration for your keys, or access them here: https://dashboard.clerk.com/last-active?path=api-keys");if(t.fatal&&!Hg(e))throw new Error("Publishable key not valid.");return null}const n=e.startsWith(c8)?"production":"development";let r;try{r=u8(e.split("_")[2])}catch{if(t.fatal)throw new Error("Publishable key not valid: Failed to decode key.");return null}if(!d8(r)){if(t.fatal)throw new Error("Publishable key not valid: Decoded key has invalid format.");return null}let a=r.slice(0,-1);return t.proxyUrl?a=t.proxyUrl:n!=="development"&&t.domain&&t.isSatellite&&(a=`clerk.${t.domain}`),{instanceType:n,frontendApi:a}}function Hg(e=""){try{if(!(e.startsWith(c8)||e.startsWith(_C)))return!1;const t=e.split("_");if(t.length!==3)return!1;const n=t[2];return n?d8(u8(n)):!1}catch{return!1}}function EC(){const e=new Map;return{isDevOrStagingUrl:t=>{if(!t)return!1;const n=typeof t=="string"?t:t.hostname;let r=e.get(n);return r===void 0&&(r=yC.some(a=>n.endsWith(a)),e.set(n,r)),r}}}const TC="METHOD_CALLED",xC=.1;function i2(e,t){return{event:TC,eventSamplingRate:xC,payload:{method:e,...t}}}var y1={exports:{}},_1={};/**
 * @license React
 * use-sync-external-store-shim.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var a_;function SC(){if(a_)return _1;a_=1;var e=If();function t(p,b){return p===b&&(p!==0||1/p===1/b)||p!==p&&b!==b}var n=typeof Object.is=="function"?Object.is:t,r=e.useState,a=e.useEffect,s=e.useLayoutEffect,o=e.useDebugValue;function c(p,b){var y=b(),T=r({inst:{value:y,getSnapshot:b}}),x=T[0].inst,C=T[1];return s(function(){x.value=y,x.getSnapshot=b,d(x)&&C({inst:x})},[p,y,b]),a(function(){return d(x)&&C({inst:x}),p(function(){d(x)&&C({inst:x})})},[p]),o(y),y}function d(p){var b=p.getSnapshot;p=p.value;try{var y=b();return!n(p,y)}catch{return!0}}function f(p,b){return b()}var m=typeof window>"u"||typeof window.document>"u"||typeof window.document.createElement>"u"?f:c;return _1.useSyncExternalStore=e.useSyncExternalStore!==void 0?e.useSyncExternalStore:m,_1}var i_;function wC(){return i_||(i_=1,y1.exports=SC()),y1.exports}var AC=wC();const h8=0,f8=1,m8=2,s_=3;var l_=Object.prototype.hasOwnProperty;function qg(e,t){var n,r;if(e===t)return!0;if(e&&t&&(n=e.constructor)===t.constructor){if(n===Date)return e.getTime()===t.getTime();if(n===RegExp)return e.toString()===t.toString();if(n===Array){if((r=e.length)===t.length)for(;r--&&qg(e[r],t[r]););return r===-1}if(!n||typeof e=="object"){r=0;for(n in e)if(l_.call(e,n)&&++r&&!l_.call(t,n)||!(n in t)||!qg(e[n],t[n]))return!1;return Object.keys(t).length===r}}return e!==e&&t!==t}const Wi=new WeakMap,Qs=()=>{},Ar=Qs(),Qh=Object,Ut=e=>e===Ar,Wa=e=>typeof e=="function",rs=(e,t)=>({...e,...t}),p8=e=>Wa(e.then),E1={},nh={},s2="undefined",S0=typeof window!=s2,jg=typeof document!=s2,kC=S0&&"Deno"in window,CC=()=>S0&&typeof window.requestAnimationFrame!=s2,g8=(e,t)=>{const n=Wi.get(e);return[()=>!Ut(t)&&e.get(t)||E1,r=>{if(!Ut(t)){const a=e.get(t);t in nh||(nh[t]=a),n[5](t,rs(a,r),a||E1)}},n[6],()=>!Ut(t)&&t in nh?nh[t]:!Ut(t)&&e.get(t)||E1]};let $g=!0;const NC=()=>$g,[Yg,Gg]=S0&&window.addEventListener?[window.addEventListener.bind(window),window.removeEventListener.bind(window)]:[Qs,Qs],OC=()=>{const e=jg&&document.visibilityState;return Ut(e)||e!=="hidden"},RC=e=>(jg&&document.addEventListener("visibilitychange",e),Yg("focus",e),()=>{jg&&document.removeEventListener("visibilitychange",e),Gg("focus",e)}),MC=e=>{const t=()=>{$g=!0,e()},n=()=>{$g=!1};return Yg("online",t),Yg("offline",n),()=>{Gg("online",t),Gg("offline",n)}},IC={isOnline:NC,isVisible:OC},DC={initFocus:RC,initReconnect:MC},o_=!ce.useId,c0=!S0||kC,LC=e=>CC()?window.requestAnimationFrame(e):setTimeout(e,1),Bh=c0?z.useEffect:z.useLayoutEffect,T1=typeof navigator<"u"&&navigator.connection,u_=!c0&&T1&&(["slow-2g","2g"].includes(T1.effectiveType)||T1.saveData),rh=new WeakMap,PC=e=>Qh.prototype.toString.call(e),x1=(e,t)=>e===`[object ${t}]`;let BC=0;const Vg=e=>{const t=typeof e,n=PC(e),r=x1(n,"Date"),a=x1(n,"RegExp"),s=x1(n,"Object");let o,c;if(Qh(e)===e&&!r&&!a){if(o=rh.get(e),o)return o;if(o=++BC+"~",rh.set(e,o),Array.isArray(e)){for(o="@",c=0;c<e.length;c++)o+=Vg(e[c])+",";rh.set(e,o)}if(s){o="#";const d=Qh.keys(e).sort();for(;!Ut(c=d.pop());)Ut(e[c])||(o+=c+":"+Vg(e[c])+",");rh.set(e,o)}}else o=r?e.toJSON():t=="symbol"?e.toString():t=="string"?JSON.stringify(e):""+e;return o},l2=e=>{if(Wa(e))try{e=e()}catch{e=""}const t=e;return e=typeof e=="string"?e:(Array.isArray(e)?e.length:e)?Vg(e):"",[e,t]};let UC=0;const Kg=()=>++UC;async function b8(...e){const[t,n,r,a]=e,s=rs({populateCache:!0,throwOnError:!0},typeof a=="boolean"?{revalidate:a}:a||{});let o=s.populateCache;const c=s.rollbackOnError;let d=s.optimisticData;const f=b=>typeof c=="function"?c(b):c!==!1,m=s.throwOnError;if(Wa(n)){const b=n,y=[],T=t.keys();for(const x of T)!/^\$(inf|sub)\$/.test(x)&&b(t.get(x)._k)&&y.push(x);return Promise.all(y.map(p))}return p(n);async function p(b){const[y]=l2(b);if(!y)return;const[T,x]=g8(t,y),[C,w,O,D]=Wi.get(t),Y=()=>{const fe=C[y];return(Wa(s.revalidate)?s.revalidate(T().data,b):s.revalidate!==!1)&&(delete O[y],delete D[y],fe&&fe[0])?fe[0](m8).then(()=>T().data):T().data};if(e.length<3)return Y();let H=r,B,ee=!1;const Z=Kg();w[y]=[Z,0];const Q=!Ut(d),G=T(),se=G.data,oe=G._c,ye=Ut(oe)?se:oe;if(Q&&(d=Wa(d)?d(ye,se):d,x({data:d,_c:ye})),Wa(H))try{H=H(ye)}catch(fe){B=fe,ee=!0}if(H&&p8(H))if(H=await H.catch(fe=>{B=fe,ee=!0}),Z!==w[y][0]){if(ee)throw B;return H}else ee&&Q&&f(B)&&(o=!0,x({data:ye,_c:Ar}));if(o&&!ee)if(Wa(o)){const fe=o(H,ye);x({data:fe,error:Ar,_c:Ar})}else x({data:H,error:Ar,_c:Ar});if(w[y][1]=Kg(),Promise.resolve(Y()).then(()=>{x({_c:Ar})}),ee){if(m)throw B;return}return H}}const c_=(e,t)=>{for(const n in e)e[n][0]&&e[n][0](t)},v8=(e,t)=>{if(!Wi.has(e)){const n=rs(DC,t),r=Object.create(null),a=b8.bind(Ar,e);let s=Qs;const o=Object.create(null),c=(m,p)=>{const b=o[m]||[];return o[m]=b,b.push(p),()=>b.splice(b.indexOf(p),1)},d=(m,p,b)=>{e.set(m,p);const y=o[m];if(y)for(const T of y)T(p,b)},f=()=>{if(!Wi.has(e)&&(Wi.set(e,[r,Object.create(null),Object.create(null),Object.create(null),a,d,c]),!c0)){const m=n.initFocus(setTimeout.bind(Ar,c_.bind(Ar,r,h8))),p=n.initReconnect(setTimeout.bind(Ar,c_.bind(Ar,r,f8)));s=()=>{m&&m(),p&&p(),Wi.delete(e)}}};return f(),[e,a,f,s]}return[e,Wi.get(e)[4]]},FC=(e,t,n,r,a)=>{const s=n.errorRetryCount,o=a.retryCount,c=~~((Math.random()+.5)*(1<<(o<8?o:8)))*n.errorRetryInterval;!Ut(s)&&o>s||setTimeout(r,c,a)},zC=qg,[o2,HC]=v8(new Map),y8=rs({onLoadingSlow:Qs,onSuccess:Qs,onError:Qs,onErrorRetry:FC,onDiscarded:Qs,revalidateOnFocus:!0,revalidateOnReconnect:!0,revalidateIfStale:!0,shouldRetryOnError:!0,errorRetryInterval:u_?1e4:5e3,focusThrottleInterval:5*1e3,dedupingInterval:2*1e3,loadingTimeout:u_?5e3:3e3,compare:zC,isPaused:()=>!1,cache:o2,mutate:HC,fallback:{}},IC),_8=(e,t)=>{const n=rs(e,t);if(t){const{use:r,fallback:a}=e,{use:s,fallback:o}=t;r&&s&&(n.use=r.concat(s)),a&&o&&(n.fallback=rs(a,o))}return n},Wg=z.createContext({}),qC=e=>{const{value:t}=e,n=z.useContext(Wg),r=Wa(t),a=z.useMemo(()=>r?t(n):t,[r,n,t]),s=z.useMemo(()=>r?a:_8(n,a),[r,n,a]),o=a&&a.provider,c=z.useRef(Ar);o&&!c.current&&(c.current=v8(o(s.cache||o2),a));const d=c.current;return d&&(s.cache=d[0],s.mutate=d[1]),Bh(()=>{if(d)return d[2]&&d[2](),d[3]},[]),z.createElement(Wg.Provider,rs(e,{value:s}))},jC="$inf$",E8=S0&&window.__SWR_DEVTOOLS_USE__,$C=E8?window.__SWR_DEVTOOLS_USE__:[],YC=()=>{E8&&(window.__SWR_DEVTOOLS_REACT__=ce)},GC=e=>Wa(e[1])?[e[0],e[1],e[2]||{}]:[e[0],null,(e[1]===null?e[2]:e[1])||{}],VC=()=>rs(y8,z.useContext(Wg)),KC=e=>(t,n,r)=>e(t,n&&((...s)=>{const[o]=l2(t),[,,,c]=Wi.get(o2);if(o.startsWith(jC))return n(...s);const d=c[o];return Ut(d)?n(...s):(delete c[o],d)}),r),WC=$C.concat(KC),XC=e=>function(...n){const r=VC(),[a,s,o]=GC(n),c=_8(r,o);let d=e;const{use:f}=c,m=(f||[]).concat(WC);for(let p=m.length;p--;)d=m[p](d);return d(a,s||c.fetcher||null,c)},QC=(e,t,n)=>{const r=t[e]||(t[e]=[]);return r.push(n),()=>{const a=r.indexOf(n);a>=0&&(r[a]=r[r.length-1],r.pop())}};YC();const S1=ce.use||(e=>{switch(e.status){case"pending":throw e;case"fulfilled":return e.value;case"rejected":throw e.reason;default:throw e.status="pending",e.then(t=>{e.status="fulfilled",e.value=t},t=>{e.status="rejected",e.reason=t}),e}}),w1={dedupe:!0},ZC=(e,t,n)=>{const{cache:r,compare:a,suspense:s,fallbackData:o,revalidateOnMount:c,revalidateIfStale:d,refreshInterval:f,refreshWhenHidden:m,refreshWhenOffline:p,keepPreviousData:b}=n,[y,T,x,C]=Wi.get(r),[w,O]=l2(e),D=z.useRef(!1),Y=z.useRef(!1),H=z.useRef(w),B=z.useRef(t),ee=z.useRef(n),Z=()=>ee.current,Q=()=>Z().isVisible()&&Z().isOnline(),[G,se,oe,ye]=g8(r,w),fe=z.useRef({}).current,de=Ut(o)?Ut(n.fallback)?Ar:n.fallback[w]:o,V=(vt,gt)=>{for(const Bt in fe){const Je=Bt;if(Je==="data"){if(!a(vt[Je],gt[Je])&&(!Ut(vt[Je])||!a(je,gt[Je])))return!1}else if(gt[Je]!==vt[Je])return!1}return!0},me=z.useMemo(()=>{const vt=!w||!t?!1:Ut(c)?Z().isPaused()||s?!1:d!==!1:c,gt=ln=>{const We=rs(ln);return delete We._k,vt?{isValidating:!0,isLoading:!0,...We}:We},Bt=G(),Je=ye(),yt=gt(Bt),mn=Bt===Je?yt:gt(Je);let Lt=yt;return[()=>{const ln=gt(G());return V(ln,Lt)?(Lt.data=ln.data,Lt.isLoading=ln.isLoading,Lt.isValidating=ln.isValidating,Lt.error=ln.error,Lt):(Lt=ln,ln)},()=>mn]},[r,w]),pe=AC.useSyncExternalStore(z.useCallback(vt=>oe(w,(gt,Bt)=>{V(Bt,gt)||vt()}),[r,w]),me[0],me[1]),Be=!D.current,k=y[w]&&y[w].length>0,ie=pe.data,ge=Ut(ie)?de&&p8(de)?S1(de):de:ie,L=pe.error,Oe=z.useRef(ge),je=b?Ut(ie)?Ut(Oe.current)?ge:Oe.current:ie:ge,De=k&&!Ut(L)?!1:Be&&!Ut(c)?c:Z().isPaused()?!1:s?Ut(ge)?!1:d:Ut(ge)||d,Ot=!!(w&&t&&Be&&De),ot=Ut(pe.isValidating)?Ot:pe.isValidating,zt=Ut(pe.isLoading)?Ot:pe.isLoading,wt=z.useCallback(async vt=>{const gt=B.current;if(!w||!gt||Y.current||Z().isPaused())return!1;let Bt,Je,yt=!0;const mn=vt||{},Lt=!x[w]||!mn.dedupe,ln=()=>o_?!Y.current&&w===H.current&&D.current:w===H.current,We={isValidating:!1,isLoading:!1},on=()=>{se(We)},qn=()=>{const nn=x[w];nn&&nn[1]===Je&&delete x[w]},Ln={isValidating:!0};Ut(G().data)&&(Ln.isLoading=!0);try{if(Lt&&(se(Ln),n.loadingTimeout&&Ut(G().data)&&setTimeout(()=>{yt&&ln()&&Z().onLoadingSlow(w,n)},n.loadingTimeout),x[w]=[gt(O),Kg()]),[Bt,Je]=x[w],Bt=await Bt,Lt&&setTimeout(qn,n.dedupingInterval),!x[w]||x[w][1]!==Je)return Lt&&ln()&&Z().onDiscarded(w),!1;We.error=Ar;const nn=T[w];if(!Ut(nn)&&(Je<=nn[0]||Je<=nn[1]||nn[1]===0))return on(),Lt&&ln()&&Z().onDiscarded(w),!1;const he=G().data;We.data=a(he,Bt)?he:Bt,Lt&&ln()&&Z().onSuccess(Bt,w,n)}catch(nn){qn();const he=Z(),{shouldRetryOnError:Se}=he;he.isPaused()||(We.error=nn,Lt&&ln()&&(he.onError(nn,w,he),(Se===!0||Wa(Se)&&Se(nn))&&(!Z().revalidateOnFocus||!Z().revalidateOnReconnect||Q())&&he.onErrorRetry(nn,w,he,Ue=>{const ze=y[w];ze&&ze[0]&&ze[0](s_,Ue)},{retryCount:(mn.retryCount||0)+1,dedupe:!0})))}return yt=!1,on(),!0},[w,r]),vn=z.useCallback((...vt)=>b8(r,H.current,...vt),[]);if(Bh(()=>{B.current=t,ee.current=n,Ut(ie)||(Oe.current=ie)}),Bh(()=>{if(!w)return;const vt=wt.bind(Ar,w1);let gt=0;Z().revalidateOnFocus&&(gt=Date.now()+Z().focusThrottleInterval);const Je=QC(w,y,(yt,mn={})=>{if(yt==h8){const Lt=Date.now();Z().revalidateOnFocus&&Lt>gt&&Q()&&(gt=Lt+Z().focusThrottleInterval,vt())}else if(yt==f8)Z().revalidateOnReconnect&&Q()&&vt();else{if(yt==m8)return wt();if(yt==s_)return wt(mn)}});return Y.current=!1,H.current=w,D.current=!0,se({_k:O}),De&&(x[w]||(Ut(ge)||c0?vt():LC(vt))),()=>{Y.current=!0,Je()}},[w]),Bh(()=>{let vt;function gt(){const Je=Wa(f)?f(G().data):f;Je&&vt!==-1&&(vt=setTimeout(Bt,Je))}function Bt(){!G().error&&(m||Z().isVisible())&&(p||Z().isOnline())?wt(w1).then(gt):gt()}return gt(),()=>{vt&&(clearTimeout(vt),vt=-1)}},[f,m,p,w]),z.useDebugValue(je),s&&Ut(ge)&&w){if(!o_&&c0)throw new Error("Fallback data is required when using Suspense in SSR.");B.current=t,ee.current=n,Y.current=!1;const vt=C[w];if(!Ut(vt)){const gt=vn(vt);S1(gt)}if(Ut(L)){const gt=wt(w1);Ut(je)||(gt.status="fulfilled",gt.value=!0),S1(gt)}else throw L}return{mutate:vn,get data(){return fe.data=!0,je},get error(){return fe.error=!0,L},get isValidating(){return fe.isValidating=!0,ot},get isLoading(){return fe.isLoading=!0,zt}}},JC=Qh.defineProperty(qC,"defaultValue",{value:y8}),eN=XC(ZC);var d_=Object.prototype.hasOwnProperty;function h_(e,t,n){for(n of e.keys())if(Xc(n,t))return n}function Xc(e,t){var n,r,a;if(e===t)return!0;if(e&&t&&(n=e.constructor)===t.constructor){if(n===Date)return e.getTime()===t.getTime();if(n===RegExp)return e.toString()===t.toString();if(n===Array){if((r=e.length)===t.length)for(;r--&&Xc(e[r],t[r]););return r===-1}if(n===Set){if(e.size!==t.size)return!1;for(r of e)if(a=r,a&&typeof a=="object"&&(a=h_(t,a),!a)||!t.has(a))return!1;return!0}if(n===Map){if(e.size!==t.size)return!1;for(r of e)if(a=r[0],a&&typeof a=="object"&&(a=h_(t,a),!a)||!Xc(r[1],t.get(a)))return!1;return!0}if(n===ArrayBuffer)e=new Uint8Array(e),t=new Uint8Array(t);else if(n===DataView){if((r=e.byteLength)===t.byteLength)for(;r--&&e.getInt8(r)===t.getInt8(r););return r===-1}if(ArrayBuffer.isView(e)){if((r=e.byteLength)===t.byteLength)for(;r--&&e[r]===t[r];);return r===-1}if(!n||typeof e=="object"){r=0;for(n in e)if(d_.call(e,n)&&++r&&!d_.call(t,n)||!(n in t)||!Xc(e[n],t[n]))return!1;return Object.keys(t).length===r}}return e!==e&&t!==t}function tN(e,t){if(!e)throw typeof t=="string"?new Error(t):new Error(`${t.displayName} not found`)}const os=(e,t)=>{const{assertCtxFn:n=tN}={},r=ce.createContext(void 0);return r.displayName=e,[r,()=>{const o=ce.useContext(r);return n(o,`${e} not found`),o.value},()=>{const o=ce.useContext(r);return o?o.value:{}}]};function nN({swrConfig:e,children:t}){return ce.createElement(JC,{value:e},t)}const[T8,Pf]=os("ClerkInstanceContext"),[rN,x8]=os("UserContext"),[aN,hK]=os("ClientContext"),[iN,fK]=os("SessionContext");ce.createContext({});const[sN,mK]=os("CheckoutContext"),lN=({children:e,...t})=>ce.createElement(sN.Provider,{value:{value:t}},e),[oN,uN]=os("OrganizationContext"),cN=({children:e,organization:t,swrConfig:n})=>ce.createElement(nN,{swrConfig:n},ce.createElement(oN.Provider,{value:{value:{organization:t}}},e));function Bf(e){if(!ce.useContext(T8)){if(typeof e=="function"){e();return}throw new Error(`${e} can only be used within the <ClerkProvider /> component.

Possible fixes:
1. Ensure that the <ClerkProvider /> is correctly wrapping your application where this component is used.
2. Check for multiple versions of the \`@clerk/shared\` package in your project. Use a tool like \`npm ls @clerk/shared\` to identify multiple versions, and update your dependencies to only rely on one.

Learn more: https://clerk.com/docs/components/clerk-provider`.trim())}}const dN="billing-subscription",hN={SUBSCRIPTION_KEY:dN};function fN(e){return{queryKey:[e.stablePrefix,e.authenticated,e.tracked,e.untracked],invalidationKey:[e.stablePrefix,e.authenticated,e.tracked],stableKey:e.stablePrefix,authenticated:e.authenticated}}const mN=()=>(Bf("useClerk"),Pf());typeof window<"u"?ce.useLayoutEffect:ce.useEffect;const f_="useUser";function pN(){var t;Bf(f_);const e=x8();return(t=Pf().telemetry)==null||t.record(i2(f_)),e===void 0?{isLoaded:!1,isSignedIn:void 0,user:void 0}:e===null?{isLoaded:!0,isSignedIn:!1,user:null}:{isLoaded:!0,isSignedIn:!0,user:e}}const m_=Xc;function gN(e){const{userId:t,orgId:n,for:r}=e;return z.useMemo(()=>{const a=r==="organization"?n:void 0;return fN({stablePrefix:hN.SUBSCRIPTION_KEY,authenticated:!0,tracked:{userId:t,orgId:a},untracked:{args:{orgId:a}}})},[t,n,r])}const p_="useSubscription";function bN(e){var m;Bf(p_);const t=Pf(),n=x8(),{organization:r}=uN(),a=t.__unstable__environment;(m=t.telemetry)==null||m.record(i2(p_));const s=(e==null?void 0:e.for)==="organization"?a==null?void 0:a.commerceSettings.billing.organization.enabled:a==null?void 0:a.commerceSettings.billing.user.enabled,o=((e==null?void 0:e.enabled)??!0)&&s,{queryKey:c}=gN({userId:n==null?void 0:n.id,orgId:r==null?void 0:r.id,for:e==null?void 0:e.for}),d=eN(o?{queryKey:c}:null,({queryKey:p})=>{const b=p[3].args;return p[2].userId?t.billing.getSubscription(b):null},{dedupingInterval:1e3*60,keepPreviousData:e==null?void 0:e.keepPreviousData}),f=z.useCallback(()=>{d.mutate()},[d]);return{data:d.data,error:d.error,isLoading:d.isLoading,isFetching:d.isValidating,revalidate:f}}const vN=e=>{const t=z.useRef(e);return z.useEffect(()=>{t.current=e},[e]),t.current},ma=(e,t,n)=>{const r=!!n,a=z.useRef(n);z.useEffect(()=>{a.current=n},[n]),z.useEffect(()=>{if(!r||!e)return()=>{};const s=(...o)=>{a.current&&a.current(...o)};return e.on(t,s),()=>{e.off(t,s)}},[r,t,e,a])},S8=ce.createContext(null);S8.displayName="ElementsContext";const yN=(e,t)=>{if(!e)throw new Error(`Could not find Elements context; You need to wrap the part of your app that ${t} in an <Elements> provider.`);return e},Zh=e=>e!==null&&typeof e=="object",_N=(e,t,n)=>Zh(e)?Object.keys(e).reduce((r,a)=>{const s=!Zh(t)||!w8(e[a],t[a]);return n.includes(a)?(s&&console.warn(`Unsupported prop change: options.${a} is not a mutable property.`),r):s?{...r||{},[a]:e[a]}:r},null):null,g_="[object Object]",w8=(e,t)=>{if(!Zh(e)||!Zh(t))return e===t;const n=Array.isArray(e);if(n!==Array.isArray(t))return!1;const r=Object.prototype.toString.call(e)===g_;if(r!==(Object.prototype.toString.call(t)===g_))return!1;if(!r&&!n)return e===t;const a=Object.keys(e),s=Object.keys(t);if(a.length!==s.length)return!1;const o={};for(let p=0;p<a.length;p+=1)o[a[p]]=!0;for(let p=0;p<s.length;p+=1)o[s[p]]=!0;const c=Object.keys(o);if(c.length!==a.length)return!1;const d=e,f=t,m=p=>w8(d[p],f[p]);return c.every(m)},b_=e=>yN(ce.useContext(S8),e),EN=e=>e.charAt(0).toUpperCase()+e.slice(1),TN=(e,t)=>{const n=`${EN(e)}Element`,s=t?o=>{b_(`mounts <${n}>`);const{id:c,className:d}=o;return ce.createElement("div",{id:c,className:d})}:({id:o,className:c,fallback:d,options:f={},onBlur:m,onFocus:p,onReady:b,onChange:y,onEscape:T,onClick:x,onLoadError:C,onLoaderStart:w,onNetworksChange:O,onConfirm:D,onCancel:Y,onShippingAddressChange:H,onShippingRateChange:B})=>{const ee=b_(`mounts <${n}>`),Z="elements"in ee?ee.elements:null,[Q,G]=ce.useState(null),se=ce.useRef(null),oe=ce.useRef(null),[ye,fe]=z.useState(!1);ma(Q,"blur",m),ma(Q,"focus",p),ma(Q,"escape",T),ma(Q,"click",x),ma(Q,"loaderror",C),ma(Q,"loaderstart",w),ma(Q,"networkschange",O),ma(Q,"confirm",D),ma(Q,"cancel",Y),ma(Q,"shippingaddresschange",H),ma(Q,"shippingratechange",B),ma(Q,"change",y);let de;b&&(de=()=>{fe(!0),b(Q)}),ma(Q,"ready",de),ce.useLayoutEffect(()=>{if(se.current===null&&oe.current!==null&&Z){let me=null;Z&&(me=Z.create(e,f)),se.current=me,G(me),me&&me.mount(oe.current)}},[Z,f]);const V=vN(f);return ce.useEffect(()=>{if(!se.current)return;const me=_N(f,V,["paymentRequest"]);me&&"update"in se.current&&se.current.update(me)},[f,V]),ce.useLayoutEffect(()=>()=>{if(se.current&&typeof se.current.destroy=="function")try{se.current.destroy(),se.current=null}catch{}},[]),ce.createElement(ce.Fragment,null,!ye&&d,ce.createElement("div",{id:o,style:{height:ye?"unset":"0px",visibility:ye?"visible":"hidden"},className:c,ref:oe}))};return s.displayName=n,s.__elementType=e,s};TN("payment",typeof window>"u");os("PaymentElementContext");os("StripeUtilsContext");var Xa=o8({packageName:"@clerk/clerk-react"});function xN(e){Xa.setMessages(e).setPackageName(e)}var[SN,wN]=os("AuthContext"),AN=T8,A8=Pf,kN="You've added multiple <ClerkProvider> components in your React component tree. Wrap your components in a single <ClerkProvider>.",CN=e=>`You've passed multiple children components to <${e}/>. You can only pass a single child component or text.`,NN="Invalid state. Feel free to submit a bug or reach out to support here: https://clerk.com/support",A1="Unsupported usage of isSatellite, domain or proxyUrl. The usage of isSatellite, domain or proxyUrl as function is not supported in non-browser environments.",ON="<UserProfile.Page /> component needs to be a direct child of `<UserProfile />` or `<UserButton />`.",RN="<UserProfile.Link /> component needs to be a direct child of `<UserProfile />` or `<UserButton />`.",MN="<OrganizationProfile.Page /> component needs to be a direct child of `<OrganizationProfile />` or `<OrganizationSwitcher />`.",IN="<OrganizationProfile.Link /> component needs to be a direct child of `<OrganizationProfile />` or `<OrganizationSwitcher />`.",DN=e=>`<${e} /> can only accept <${e}.Page /> and <${e}.Link /> as its children. Any other provided component will be ignored. Additionally, please ensure that the component is rendered in a client component.`,LN=e=>`Missing props. <${e}.Page /> component requires the following props: url, label, labelIcon, alongside with children to be rendered inside the page.`,PN=e=>`Missing props. <${e}.Link /> component requires the following props: url, label and labelIcon.`,BN="<UserButton /> can only accept <UserButton.UserProfilePage />, <UserButton.UserProfileLink /> and <UserButton.MenuItems /> as its children. Any other provided component will be ignored. Additionally, please ensure that the component is rendered in a client component.",UN="<UserButton.MenuItems /> component can only accept <UserButton.Action /> and <UserButton.Link /> as its children. Any other provided component will be ignored. Additionally, please ensure that the component is rendered in a client component.",FN="<UserButton.MenuItems /> component needs to be a direct child of `<UserButton />`.",zN="<UserButton.Action /> component needs to be a direct child of `<UserButton.MenuItems />`.",HN="<UserButton.Link /> component needs to be a direct child of `<UserButton.MenuItems />`.",qN="Missing props. <UserButton.Link /> component requires the following props: href, label and labelIcon.",jN="Missing props. <UserButton.Action /> component requires the following props: label.",Uf=e=>{Bf(()=>{Xa.throwMissingClerkProviderError({source:e})})},k8=e=>new Promise(t=>{const n=r=>{["ready","degraded"].includes(r)&&(t(),e.off("status",n))};e.on("status",n,{notify:!0})}),$N=e=>async t=>(await k8(e),e.session?e.session.getToken(t):null),YN=e=>async(...t)=>(await k8(e),e.signOut(...t)),w0=(e={})=>{var t;Uf("useAuth");const{treatPendingAsSignedOut:n,...r}=e??{},a=r;let o=wN();o.sessionId===void 0&&o.userId===void 0&&(o=a??{});const c=A8(),d=z.useCallback($N(c),[c]),f=z.useCallback(YN(c),[c]);return(t=c.telemetry)==null||t.record(i2("useAuth",{treatPendingAsSignedOut:n})),GN({...o,getToken:d,signOut:f},{treatPendingAsSignedOut:n})};function GN(e,{treatPendingAsSignedOut:t=!0}={}){const{userId:n,orgId:r,orgRole:a,has:s,signOut:o,getToken:c,orgPermissions:d,factorVerificationAge:f,sessionClaims:m}=e??{},p=z.useCallback(y=>s?s(y):bC({userId:n,orgId:r,orgRole:a,orgPermissions:d,factorVerificationAge:f,features:(m==null?void 0:m.fea)||"",plans:(m==null?void 0:m.pla)||""})(y),[s,n,r,a,d,f,m]),b=vC({authObject:{...e,getToken:c,signOut:o,has:p},options:{treatPendingAsSignedOut:t}});return b||Xa.throw(NN)}var Qt=(e,t)=>{const r=(typeof t=="string"?t:t==null?void 0:t.component)||e.displayName||e.name||"Component";e.displayName=r;const a=typeof t=="string"?void 0:t,s=o=>{Uf(r||"withClerk");const c=A8();return!c.loaded&&!(a!=null&&a.renderWhileLoading)?null:ce.createElement(e,{...o,component:r,clerk:c})};return s.displayName=`withClerk(${r})`,s};const VN=()=>{try{return!1}catch{}return!1},KN=()=>{try{return!1}catch{}return!1},WN=()=>{try{return!0}catch{}return!1},v_=new Set,u2=(e,t,n)=>{const r=KN()||WN(),a=e;v_.has(a)||r||(v_.add(a),console.warn(`Clerk - DEPRECATION WARNING: "${e}" is deprecated and will be removed in the next major release.
${t}`))};var XN=({children:e,treatPendingAsSignedOut:t})=>{Uf("SignedIn");const{userId:n}=w0({treatPendingAsSignedOut:t});return n?e:null},QN=({children:e,treatPendingAsSignedOut:t})=>{Uf("SignedOut");const{userId:n}=w0({treatPendingAsSignedOut:t});return n===null?e:null};Qt(({clerk:e,...t})=>{const{client:n,session:r}=e,a=n.signedInSessions?n.signedInSessions.length>0:n.activeSessions&&n.activeSessions.length>0;return ce.useEffect(()=>{r===null&&a?e.redirectToAfterSignOut():e.redirectToSignIn(t)},[]),null},"RedirectToSignIn");Qt(({clerk:e,...t})=>(ce.useEffect(()=>{e.redirectToSignUp(t)},[]),null),"RedirectToSignUp");Qt(({clerk:e,...t})=>(ce.useEffect(()=>{e.redirectToTasks(t)},[]),null),"RedirectToTasks");Qt(({clerk:e})=>(ce.useEffect(()=>{u2("RedirectToUserProfile","Use the `redirectToUserProfile()` method instead."),e.redirectToUserProfile()},[]),null),"RedirectToUserProfile");Qt(({clerk:e})=>(ce.useEffect(()=>{u2("RedirectToOrganizationProfile","Use the `redirectToOrganizationProfile()` method instead."),e.redirectToOrganizationProfile()},[]),null),"RedirectToOrganizationProfile");Qt(({clerk:e})=>(ce.useEffect(()=>{u2("RedirectToCreateOrganization","Use the `redirectToCreateOrganization()` method instead."),e.redirectToCreateOrganization()},[]),null),"RedirectToCreateOrganization");Qt(({clerk:e,...t})=>(ce.useEffect(()=>{e.handleRedirectCallback(t)},[]),null),"AuthenticateWithRedirectCallback");function k1(e,t,n){if(typeof e=="function")return e(t);if(typeof e<"u")return e;if(typeof n<"u")return n}const Fr=e=>{VN()&&console.error(`Clerk: ${e}`)};var ZN=H5();const y_=(e,...t)=>{const n={...e};for(const r of t)delete n[r];return n};var Kl=e=>t=>{try{return ce.Children.only(e)}catch{return Xa.throw(CN(t))}},Wl=(e,t)=>(e||(e=t),typeof e=="string"&&(e=ce.createElement("button",null,e)),e),Xl=e=>(...t)=>{if(e&&typeof e=="function")return e(...t)};function JN(e){return typeof e=="function"}var ah=new Map;function eO(e,t,n=1){ce.useEffect(()=>{const r=ah.get(e)||0;return r==n?Xa.throw(t):(ah.set(e,r+1),()=>{ah.set(e,(ah.get(e)||1)-1)})},[])}function tO(e,t,n){const r=e.displayName||e.name||t||"Component",a=s=>(eO(t,n),ce.createElement(e,{...s}));return a.displayName=`withMaxAllowedInstancesGuard(${r})`,a}var Qc=e=>{const[t,n]=z.useState(new Map);return e.map(r=>({id:r.id,mount:a=>n(s=>new Map(s).set(String(r.id),a)),unmount:()=>n(a=>{const s=new Map(a);return s.set(String(r.id),null),s}),portal:()=>{const a=t.get(String(r.id));return a?ZN.createPortal(r.component,a):null}}))},Ur=(e,t)=>!!e&&ce.isValidElement(e)&&(e==null?void 0:e.type)===t,C8=(e,t)=>R8({children:e,reorderItemsLabels:["account","security","billing","apiKeys"],LinkComponent:k0,PageComponent:A0,MenuItemsComponent:zf,componentName:"UserProfile"},t),N8=(e,t)=>R8({children:e,reorderItemsLabels:["general","members","billing","apiKeys"],LinkComponent:qf,PageComponent:Hf,componentName:"OrganizationProfile"},t),O8=e=>{const t=[],n=[qf,Hf,zf,A0,k0];return ce.Children.forEach(e,r=>{n.some(a=>Ur(r,a))||t.push(r)}),t},R8=(e,t)=>{const{children:n,LinkComponent:r,PageComponent:a,MenuItemsComponent:s,reorderItemsLabels:o,componentName:c}=e,{allowForAnyChildren:d=!1}=t||{},f=[];ce.Children.forEach(n,O=>{if(!Ur(O,a)&&!Ur(O,r)&&!Ur(O,s)){O&&!d&&Fr(DN(c));return}const{props:D}=O,{children:Y,label:H,url:B,labelIcon:ee}=D;if(Ur(O,a))if(__(D,o))f.push({label:H});else if(C1(D))f.push({label:H,labelIcon:ee,children:Y,url:B});else{Fr(LN(c));return}if(Ur(O,r))if(N1(D))f.push({label:H,labelIcon:ee,url:B});else{Fr(PN(c));return}});const m=[],p=[],b=[];f.forEach((O,D)=>{if(C1(O)){m.push({component:O.children,id:D}),p.push({component:O.labelIcon,id:D});return}N1(O)&&b.push({component:O.labelIcon,id:D})});const y=Qc(m),T=Qc(p),x=Qc(b),C=[],w=[];return f.forEach((O,D)=>{if(__(O,o)){C.push({label:O.label});return}if(C1(O)){const{portal:Y,mount:H,unmount:B}=y.find(G=>G.id===D),{portal:ee,mount:Z,unmount:Q}=T.find(G=>G.id===D);C.push({label:O.label,url:O.url,mount:H,unmount:B,mountIcon:Z,unmountIcon:Q}),w.push(Y),w.push(ee);return}if(N1(O)){const{portal:Y,mount:H,unmount:B}=x.find(ee=>ee.id===D);C.push({label:O.label,url:O.url,mountIcon:H,unmountIcon:B}),w.push(Y);return}}),{customPages:C,customPagesPortals:w}},__=(e,t)=>{const{children:n,label:r,url:a,labelIcon:s}=e;return!n&&!a&&!s&&t.some(o=>o===r)},C1=e=>{const{children:t,label:n,url:r,labelIcon:a}=e;return!!t&&!!r&&!!a&&!!n},N1=e=>{const{children:t,label:n,url:r,labelIcon:a}=e;return!t&&!!r&&!!a&&!!n},nO=(e,t)=>{var n;return rO({children:e,reorderItemsLabels:["manageAccount","signOut"],MenuItemsComponent:zf,MenuActionComponent:I8,MenuLinkComponent:D8,UserProfileLinkComponent:k0,UserProfilePageComponent:A0,allowForAnyChildren:(n=t==null?void 0:t.allowForAnyChildren)!=null?n:!1})},rO=({children:e,MenuItemsComponent:t,MenuActionComponent:n,MenuLinkComponent:r,UserProfileLinkComponent:a,UserProfilePageComponent:s,reorderItemsLabels:o,allowForAnyChildren:c=!1})=>{const d=[],f=[],m=[];ce.Children.forEach(e,x=>{if(!Ur(x,t)&&!Ur(x,a)&&!Ur(x,s)){x&&!c&&Fr(BN);return}if(Ur(x,a)||Ur(x,s))return;const{props:C}=x;ce.Children.forEach(C.children,w=>{if(!Ur(w,n)&&!Ur(w,r)){w&&Fr(UN);return}const{props:O}=w,{label:D,labelIcon:Y,href:H,onClick:B,open:ee}=O;if(Ur(w,n))if(E_(O,o))d.push({label:D});else if(O1(O)){const Z={label:D,labelIcon:Y};if(B!==void 0)d.push({...Z,onClick:B});else if(ee!==void 0)d.push({...Z,open:ee.startsWith("/")?ee:`/${ee}`});else{Fr("Custom menu item must have either onClick or open property");return}}else{Fr(jN);return}if(Ur(w,r))if(R1(O))d.push({label:D,labelIcon:Y,href:H});else{Fr(qN);return}})});const p=[],b=[];d.forEach((x,C)=>{O1(x)&&p.push({component:x.labelIcon,id:C}),R1(x)&&b.push({component:x.labelIcon,id:C})});const y=Qc(p),T=Qc(b);return d.forEach((x,C)=>{if(E_(x,o)&&f.push({label:x.label}),O1(x)){const{portal:w,mount:O,unmount:D}=y.find(H=>H.id===C),Y={label:x.label,mountIcon:O,unmountIcon:D};"onClick"in x?Y.onClick=x.onClick:"open"in x&&(Y.open=x.open),f.push(Y),m.push(w)}if(R1(x)){const{portal:w,mount:O,unmount:D}=T.find(Y=>Y.id===C);f.push({label:x.label,href:x.href,mountIcon:O,unmountIcon:D}),m.push(w)}}),{customMenuItems:f,customMenuItemsPortals:m}},E_=(e,t)=>{const{children:n,label:r,onClick:a,labelIcon:s}=e;return!n&&!a&&!s&&t.some(o=>o===r)},O1=e=>{const{label:t,labelIcon:n,onClick:r,open:a}=e;return!!n&&!!t&&(typeof r=="function"||typeof a=="string")},R1=e=>{const{label:t,href:n,labelIcon:r}=e;return!!n&&!!r&&!!t},aO=e=>{const t=e==null?void 0:e.isReady;return n=>new Promise((r,a)=>{const{root:s=document==null?void 0:document.body,selector:o,timeout:c=0}=n;if(!s){a(new Error("No root element provided"));return}let d=s;if(o&&(d=s==null?void 0:s.querySelector(o)),t(d,o)){r();return}const f=new MutationObserver(m=>{for(const p of m)if(!d&&o&&(d=s==null?void 0:s.querySelector(o)),(e.childList&&p.type==="childList"||e.attributes&&p.type==="attributes")&&t(d,o)){f.disconnect(),r();return}});f.observe(s,e),c>0&&setTimeout(()=>{f.disconnect(),a(new Error(`Timeout waiting for ${o}`))},c)})},iO=aO({childList:!0,subtree:!0,isReady:(e,t)=>{var n;return!!(e!=null&&e.childElementCount)&&((n=e==null?void 0:e.matches)==null?void 0:n.call(e,t))&&e.childElementCount>0}});function qr(e,t){const n=z.useRef(),[r,a]=z.useState("rendering");return z.useEffect(()=>{if(!e)throw new Error("Clerk: no component name provided, unable to detect mount.");if(typeof window<"u"&&!n.current){const s=`[data-clerk-component="${e}"]`,o=t==null?void 0:t.selector;n.current=iO({selector:o?s+o:s}).then(()=>{a("rendered")}).catch(()=>{a("error")})}},[e,t==null?void 0:t.selector]),r}var ih=e=>"mount"in e,T_=e=>"open"in e,x_=e=>e==null?void 0:e.map(({mountIcon:t,unmountIcon:n,...r})=>r),Er=class extends ce.PureComponent{constructor(){super(...arguments),this.rootRef=ce.createRef()}componentDidUpdate(e){var t,n,r,a;if(!ih(e)||!ih(this.props))return;const s=y_(e.props,"customPages","customMenuItems","children"),o=y_(this.props.props,"customPages","customMenuItems","children"),c=((t=s.customPages)==null?void 0:t.length)!==((n=o.customPages)==null?void 0:n.length),d=((r=s.customMenuItems)==null?void 0:r.length)!==((a=o.customMenuItems)==null?void 0:a.length),f=x_(e.props.customMenuItems),m=x_(this.props.props.customMenuItems);(!m_(s,o)||!m_(f,m)||c||d)&&this.rootRef.current&&this.props.updateProps({node:this.rootRef.current,props:this.props.props})}componentDidMount(){this.rootRef.current&&(ih(this.props)&&this.props.mount(this.rootRef.current,this.props.props),T_(this.props)&&this.props.open(this.props.props))}componentWillUnmount(){this.rootRef.current&&(ih(this.props)&&this.props.unmount(this.rootRef.current),T_(this.props)&&this.props.close())}render(){const{hideRootHtmlElement:e=!1}=this.props,t={ref:this.rootRef,...this.props.rootProps,...this.props.component&&{"data-clerk-component":this.props.component}};return ce.createElement(ce.Fragment,null,!e&&ce.createElement("div",{...t}),this.props.children)}},Ff=e=>{var t,n;return ce.createElement(ce.Fragment,null,(t=e==null?void 0:e.customPagesPortals)==null?void 0:t.map((r,a)=>z.createElement(r,{key:a})),(n=e==null?void 0:e.customMenuItemsPortals)==null?void 0:n.map((r,a)=>z.createElement(r,{key:a})))},sO=Qt(({clerk:e,component:t,fallback:n,...r})=>{const s=qr(t)==="rendering"||!e.loaded,o={...s&&n&&{style:{display:"none"}}};return ce.createElement(ce.Fragment,null,s&&n,e.loaded&&ce.createElement(Er,{component:t,mount:e.mountSignIn,unmount:e.unmountSignIn,updateProps:e.__unstable__updateProps,props:r,rootProps:o}))},{component:"SignIn",renderWhileLoading:!0}),lO=Qt(({clerk:e,component:t,fallback:n,...r})=>{const s=qr(t)==="rendering"||!e.loaded,o={...s&&n&&{style:{display:"none"}}};return ce.createElement(ce.Fragment,null,s&&n,e.loaded&&ce.createElement(Er,{component:t,mount:e.mountSignUp,unmount:e.unmountSignUp,updateProps:e.__unstable__updateProps,props:r,rootProps:o}))},{component:"SignUp",renderWhileLoading:!0});function A0({children:e}){return Fr(ON),ce.createElement(ce.Fragment,null,e)}function k0({children:e}){return Fr(RN),ce.createElement(ce.Fragment,null,e)}var oO=Qt(({clerk:e,component:t,fallback:n,...r})=>{const s=qr(t)==="rendering"||!e.loaded,o={...s&&n&&{style:{display:"none"}}},{customPages:c,customPagesPortals:d}=C8(r.children);return ce.createElement(ce.Fragment,null,s&&n,ce.createElement(Er,{component:t,mount:e.mountUserProfile,unmount:e.unmountUserProfile,updateProps:e.__unstable__updateProps,props:{...r,customPages:c},rootProps:o},ce.createElement(Ff,{customPagesPortals:d})))},{component:"UserProfile",renderWhileLoading:!0});Object.assign(oO,{Page:A0,Link:k0});var M8=z.createContext({mount:()=>{},unmount:()=>{},updateProps:()=>{}}),uO=Qt(({clerk:e,component:t,fallback:n,...r})=>{const s=qr(t)==="rendering"||!e.loaded,o={...s&&n&&{style:{display:"none"}}},{customPages:c,customPagesPortals:d}=C8(r.children,{allowForAnyChildren:!!r.__experimental_asProvider}),f={...r.userProfileProps,customPages:c},{customMenuItems:m,customMenuItemsPortals:p}=nO(r.children,{allowForAnyChildren:!!r.__experimental_asProvider}),b=O8(r.children),y={mount:e.mountUserButton,unmount:e.unmountUserButton,updateProps:e.__unstable__updateProps,props:{...r,userProfileProps:f,customMenuItems:m}},T={customPagesPortals:d,customMenuItemsPortals:p};return ce.createElement(M8.Provider,{value:y},s&&n,e.loaded&&ce.createElement(Er,{component:t,...y,hideRootHtmlElement:!!r.__experimental_asProvider,rootProps:o},r.__experimental_asProvider?b:null,ce.createElement(Ff,{...T})))},{component:"UserButton",renderWhileLoading:!0});function zf({children:e}){return Fr(FN),ce.createElement(ce.Fragment,null,e)}function I8({children:e}){return Fr(zN),ce.createElement(ce.Fragment,null,e)}function D8({children:e}){return Fr(HN),ce.createElement(ce.Fragment,null,e)}function cO(e){const t=z.useContext(M8),n={...t,props:{...t.props,...e}};return ce.createElement(Er,{...n})}var dO=Object.assign(uO,{UserProfilePage:A0,UserProfileLink:k0,MenuItems:zf,Action:I8,Link:D8,__experimental_Outlet:cO});function Hf({children:e}){return Fr(MN),ce.createElement(ce.Fragment,null,e)}function qf({children:e}){return Fr(IN),ce.createElement(ce.Fragment,null,e)}var hO=Qt(({clerk:e,component:t,fallback:n,...r})=>{const s=qr(t)==="rendering"||!e.loaded,o={...s&&n&&{style:{display:"none"}}},{customPages:c,customPagesPortals:d}=N8(r.children);return ce.createElement(ce.Fragment,null,s&&n,e.loaded&&ce.createElement(Er,{component:t,mount:e.mountOrganizationProfile,unmount:e.unmountOrganizationProfile,updateProps:e.__unstable__updateProps,props:{...r,customPages:c},rootProps:o},ce.createElement(Ff,{customPagesPortals:d})))},{component:"OrganizationProfile",renderWhileLoading:!0});Object.assign(hO,{Page:Hf,Link:qf});Qt(({clerk:e,component:t,fallback:n,...r})=>{const s=qr(t)==="rendering"||!e.loaded,o={...s&&n&&{style:{display:"none"}}};return ce.createElement(ce.Fragment,null,s&&n,e.loaded&&ce.createElement(Er,{component:t,mount:e.mountCreateOrganization,unmount:e.unmountCreateOrganization,updateProps:e.__unstable__updateProps,props:r,rootProps:o}))},{component:"CreateOrganization",renderWhileLoading:!0});var L8=z.createContext({mount:()=>{},unmount:()=>{},updateProps:()=>{}}),fO=Qt(({clerk:e,component:t,fallback:n,...r})=>{const s=qr(t)==="rendering"||!e.loaded,o={...s&&n&&{style:{display:"none"}}},{customPages:c,customPagesPortals:d}=N8(r.children,{allowForAnyChildren:!!r.__experimental_asProvider}),f={...r.organizationProfileProps,customPages:c},m=O8(r.children),p={mount:e.mountOrganizationSwitcher,unmount:e.unmountOrganizationSwitcher,updateProps:e.__unstable__updateProps,props:{...r,organizationProfileProps:f},rootProps:o,component:t};return e.__experimental_prefetchOrganizationSwitcher(),ce.createElement(L8.Provider,{value:p},ce.createElement(ce.Fragment,null,s&&n,e.loaded&&ce.createElement(Er,{...p,hideRootHtmlElement:!!r.__experimental_asProvider},r.__experimental_asProvider?m:null,ce.createElement(Ff,{customPagesPortals:d}))))},{component:"OrganizationSwitcher",renderWhileLoading:!0});function mO(e){const t=z.useContext(L8),n={...t,props:{...t.props,...e}};return ce.createElement(Er,{...n})}Object.assign(fO,{OrganizationProfilePage:Hf,OrganizationProfileLink:qf,__experimental_Outlet:mO});Qt(({clerk:e,component:t,fallback:n,...r})=>{const s=qr(t)==="rendering"||!e.loaded,o={...s&&n&&{style:{display:"none"}}};return ce.createElement(ce.Fragment,null,s&&n,e.loaded&&ce.createElement(Er,{component:t,mount:e.mountOrganizationList,unmount:e.unmountOrganizationList,updateProps:e.__unstable__updateProps,props:r,rootProps:o}))},{component:"OrganizationList",renderWhileLoading:!0});Qt(({clerk:e,component:t,fallback:n,...r})=>{const s=qr(t)==="rendering"||!e.loaded,o={...s&&n&&{style:{display:"none"}}};return ce.createElement(ce.Fragment,null,s&&n,e.loaded&&ce.createElement(Er,{component:t,open:e.openGoogleOneTap,close:e.closeGoogleOneTap,updateProps:e.__unstable__updateProps,props:r,rootProps:o}))},{component:"GoogleOneTap",renderWhileLoading:!0});Qt(({clerk:e,component:t,fallback:n,...r})=>{const s=qr(t)==="rendering"||!e.loaded,o={...s&&n&&{style:{display:"none"}}};return ce.createElement(ce.Fragment,null,s&&n,e.loaded&&ce.createElement(Er,{component:t,mount:e.mountWaitlist,unmount:e.unmountWaitlist,updateProps:e.__unstable__updateProps,props:r,rootProps:o}))},{component:"Waitlist",renderWhileLoading:!0});var pO=Qt(({clerk:e,component:t,fallback:n,...r})=>{const s=qr(t,{selector:'[data-component-status="ready"]'})==="rendering"||!e.loaded,o={...s&&n&&{style:{display:"none"}}};return ce.createElement(ce.Fragment,null,s&&n,e.loaded&&ce.createElement(Er,{component:t,mount:e.mountPricingTable,unmount:e.unmountPricingTable,updateProps:e.__unstable__updateProps,props:r,rootProps:o}))},{component:"PricingTable",renderWhileLoading:!0});Qt(({clerk:e,component:t,fallback:n,...r})=>{const s=qr(t)==="rendering"||!e.loaded,o={...s&&n&&{style:{display:"none"}}};return ce.createElement(ce.Fragment,null,s&&n,e.loaded&&ce.createElement(Er,{component:t,mount:e.mountAPIKeys,unmount:e.unmountAPIKeys,updateProps:e.__unstable__updateProps,props:r,rootProps:o}))},{component:"ApiKeys",renderWhileLoading:!0});Qt(({clerk:e,component:t,fallback:n,...r})=>{const s=qr(t)==="rendering"||!e.loaded,o={...s&&n&&{style:{display:"none"}}};return ce.createElement(ce.Fragment,null,s&&n,e.loaded&&ce.createElement(Er,{component:t,mount:e.mountUserAvatar,unmount:e.unmountUserAvatar,updateProps:e.__unstable__updateProps,props:r,rootProps:o}))},{component:"UserAvatar",renderWhileLoading:!0});Qt(({clerk:e,component:t,fallback:n,...r})=>{const s=qr(t)==="rendering"||!e.loaded,o={...s&&n&&{style:{display:"none"}}};return ce.createElement(ce.Fragment,null,s&&n,e.loaded&&ce.createElement(Er,{component:t,mount:e.mountTaskChooseOrganization,unmount:e.unmountTaskChooseOrganization,updateProps:e.__unstable__updateProps,props:r,rootProps:o}))},{component:"TaskChooseOrganization",renderWhileLoading:!0});Qt(({clerk:e,component:t,fallback:n,...r})=>{const s=qr(t)==="rendering"||!e.loaded,o={...s&&n&&{style:{display:"none"}}};return ce.createElement(ce.Fragment,null,s&&n,e.loaded&&ce.createElement(Er,{component:t,mount:e.mountTaskResetPassword,unmount:e.unmountTaskResetPassword,updateProps:e.__unstable__updateProps,props:r,rootProps:o}))},{component:"TaskResetPassword",renderWhileLoading:!0});var P8=e=>{throw TypeError(e)},c2=(e,t,n)=>t.has(e)||P8("Cannot "+n),gn=(e,t,n)=>(c2(e,t,"read from private field"),n?n.call(e):t.get(e)),Ys=(e,t,n)=>t.has(e)?P8("Cannot add the same private member more than once"):t instanceof WeakSet?t.add(e):t.set(e,n),Dl=(e,t,n,r)=>(c2(e,t,"write to private field"),t.set(e,n),n),M1=(e,t,n)=>(c2(e,t,"access private method"),n);const gO={initialDelay:125,maxDelayBetweenRetries:0,factor:2,shouldRetry:(e,t)=>t<5,retryImmediately:!1,jitter:!0},bO=100,B8=async e=>new Promise(t=>setTimeout(t,e)),U8=(e,t)=>t?e*(1+Math.random()):e,vO=e=>{let t=0;const n=()=>{const r=e.initialDelay,a=e.factor;let s=r*Math.pow(a,t);return s=U8(s,e.jitter),Math.min(e.maxDelayBetweenRetries||s,s)};return async()=>{await B8(n()),t++}},yO=async(e,t={})=>{let n=0;const{shouldRetry:r,initialDelay:a,maxDelayBetweenRetries:s,factor:o,retryImmediately:c,jitter:d,onBeforeRetry:f}={...gO,...t},m=vO({initialDelay:a,maxDelayBetweenRetries:s,factor:o,jitter:d});for(;;)try{return await e()}catch(p){if(n++,!r(p,n))throw p;f&&await f(n),c&&n===1?await B8(U8(bO,d)):await m()}},_O="loadScript cannot be called when document does not exist",EO="loadScript cannot be called without a src";async function TO(e="",t){const{async:n,defer:r,beforeLoad:a,crossOrigin:s,nonce:o}=t||{};return yO(()=>new Promise((d,f)=>{e||f(new Error(EO)),(!document||!document.body)&&f(new Error(_O));const m=document.createElement("script");s&&m.setAttribute("crossorigin",s),m.async=n||!1,m.defer=r||!1,m.addEventListener("load",()=>{m.remove(),d(m)}),m.addEventListener("error",p=>{m.remove(),f(p.error??new Error(`failed to load script: ${e}`))}),m.src=e,m.nonce=o,a==null||a(m),document.body.appendChild(m)}),{shouldRetry:(d,f)=>f<=5})}function xO(e){return e?SO(e)||F8(e):!0}function SO(e){return/^http(s)?:\/\//.test(e||"")}function F8(e){return e.startsWith("/")}function wO(e){return e?F8(e)?new URL(e,window.location.origin).toString():e:""}function AO(e){if(!e)return"";let t;if(e.match(/^(clerk\.)+\w*$/))t=/(clerk\.)*(?=clerk\.)/;else{if(e.match(/\.clerk.accounts/))return e;t=/^(clerk\.)*/gi}return`clerk.${e.replace(t,"")}`}const kO=(e,t="5.114.0")=>{if(e)return e;const n=CO(t);return n?n==="snapshot"?"5.114.0":n:NO(t)},CO=e=>{var t;return(t=e.trim().replace(/^v/,"").match(/-(.+?)(\.|$)/))==null?void 0:t[1]},NO=e=>e.trim().replace(/^v/,"").split(".")[0],z8="failed_to_load_clerk_js",OO="failed_to_load_clerk_js_timeout",Xg="Failed to load Clerk",{isDevOrStagingUrl:RO}=EC(),H8=o8({packageName:"@clerk/shared"});function MO(e){H8.setPackageName({packageName:e})}function Qg(){if(typeof window>"u"||!window.Clerk)return!1;const e=window.Clerk;return typeof e=="object"&&typeof e.load=="function"}function IO(e){if(typeof window>"u"||!window.performance)return!1;const t=performance.getEntriesByName(e,"resource");if(t.length===0)return!1;const n=t[t.length-1];return n.transferSize===0&&n.decodedBodySize===0&&(n.responseEnd===0||n.responseEnd>0&&n.responseStart>0||"responseStatus"in n&&(n.responseStatus>=400||n.responseStatus===0))}function S_(e,t){return new Promise((n,r)=>{let a=!1;const s=(m,p)=>{clearTimeout(m),clearInterval(p)};t==null||t.addEventListener("error",()=>{s(d,f),r(new zg(Xg,{code:z8}))});const o=()=>{a||Qg()&&(a=!0,s(d,f),n(null))},d=setTimeout(()=>{a||(a=!0,s(d,f),Qg()?n(null):r(new zg(Xg,{code:OO})))},e);o();const f=setInterval(()=>{if(a){clearInterval(f);return}o()},100)})}const DO=async e=>{const t=(e==null?void 0:e.scriptLoadTimeout)??15e3;if(Qg())return null;if(!(e!=null&&e.publishableKey))return H8.throwMissingPublishableKeyError(),null;const n=LO(e),r=document.querySelector("script[data-clerk-js-script]");if(r)if(IO(n))r.remove();else try{return await S_(t,r),null}catch{r.remove()}const a=S_(t);return TO(n,{async:!0,crossOrigin:"anonymous",nonce:e.nonce,beforeLoad:BO(e)}).catch(s=>{throw new zg(Xg+(s.message?`, ${s.message}`:""),{code:z8,cause:s})}),a},LO=e=>{var m,p;const{clerkJSUrl:t,clerkJSVariant:n,clerkJSVersion:r,proxyUrl:a,domain:s,publishableKey:o}=e;if(t)return t;let c="";a&&xO(a)?c=wO(a).replace(/http(s)?:\/\//,""):s&&!RO(((m=r_(o))==null?void 0:m.frontendApi)||"")?c=AO(s):c=((p=r_(o))==null?void 0:p.frontendApi)||"";const d=n?`${n.replace(/\.+$/,"")}.`:"",f=kO(r);return`https://${c}/npm/@clerk/clerk-js@${f}/dist/clerk.${d}browser.js`},PO=e=>{const t={};return e.publishableKey&&(t["data-clerk-publishable-key"]=e.publishableKey),e.proxyUrl&&(t["data-clerk-proxy-url"]=e.proxyUrl),e.domain&&(t["data-clerk-domain"]=e.domain),e.nonce&&(t.nonce=e.nonce),t},BO=e=>t=>{const n=PO(e);for(const r in n)t.setAttribute(r,n[r])},UO=(e,t,n)=>!e&&n?FO(n):zO(t),FO=e=>{const t=e.userId,n=e.user,r=e.sessionId,a=e.sessionStatus,s=e.sessionClaims;return{userId:t,user:n,sessionId:r,session:e.session,sessionStatus:a,sessionClaims:s,organization:e.organization,orgId:e.orgId,orgRole:e.orgRole,orgPermissions:e.orgPermissions,orgSlug:e.orgSlug,actor:e.actor,factorVerificationAge:e.factorVerificationAge}},zO=e=>{var T,x,C,w;const t=e.user?e.user.id:e.user,n=e.user,r=e.session?e.session.id:e.session,a=e.session,s=(T=e.session)==null?void 0:T.status,o=e.session?(C=(x=e.session.lastActiveToken)==null?void 0:x.jwt)==null?void 0:C.claims:null,c=e.session?e.session.factorVerificationAge:null,d=a==null?void 0:a.actor,f=e.organization,m=e.organization?e.organization.id:e.organization,p=f==null?void 0:f.slug,b=f&&((w=n==null?void 0:n.organizationMemberships)==null?void 0:w.find(O=>O.organization.id===m)),y=b&&b.permissions;return{userId:t,user:n,sessionId:r,session:a,sessionStatus:s,sessionClaims:o,organization:f,orgId:m,orgRole:b&&b.role,orgSlug:p,orgPermissions:y,actor:d,factorVerificationAge:c}};function Jh(){return typeof window<"u"}const w_=(e,t,n,r,a)=>{const{notify:s}=a||{};let o=e.get(n);o||(o=[],e.set(n,o)),o.push(r),s&&t.has(n)&&r(t.get(n))},A_=(e,t,n)=>(e.get(t)||[]).map(r=>r(n)),k_=(e,t,n)=>{const r=e.get(t);r&&(n?r.splice(r.indexOf(n)>>>0,1):e.set(t,[]))},HO=()=>{const e=new Map,t=new Map,n=new Map;return{on:(...a)=>w_(e,t,...a),prioritizedOn:(...a)=>w_(n,t,...a),emit:(a,s)=>{t.set(a,s),A_(n,a,s),A_(e,a,s)},off:(...a)=>k_(e,...a),prioritizedOff:(...a)=>k_(n,...a),internal:{retrieveListeners:a=>e.get(a)||[]}}},sh={Status:"status"},qO=()=>HO();typeof window<"u"&&!window.global&&(window.global=typeof global>"u"?window:global);Qt(({clerk:e,children:t,...n})=>{const{signUpFallbackRedirectUrl:r,forceRedirectUrl:a,fallbackRedirectUrl:s,signUpForceRedirectUrl:o,mode:c,initialValues:d,withSignUp:f,oauthFlow:m,...p}=n;t=Wl(t,"Sign in");const b=Kl(t)("SignInButton"),y=()=>{const C={forceRedirectUrl:a,fallbackRedirectUrl:s,signUpFallbackRedirectUrl:r,signUpForceRedirectUrl:o,initialValues:d,withSignUp:f,oauthFlow:m};return c==="modal"?e.openSignIn({...C,appearance:n.appearance}):e.redirectToSignIn({...C,signInFallbackRedirectUrl:s,signInForceRedirectUrl:a})},x={...p,onClick:async C=>(b&&typeof b=="object"&&"props"in b&&await Xl(b.props.onClick)(C),y())};return ce.cloneElement(b,x)},{component:"SignInButton",renderWhileLoading:!0});Qt(({clerk:e,children:t,...n})=>{const{redirectUrl:r,...a}=n;t=Wl(t,"Sign in with Metamask");const s=Kl(t)("SignInWithMetamaskButton"),o=async()=>{async function f(){await e.authenticateWithMetamask({redirectUrl:r||void 0})}f()},d={...a,onClick:async f=>(await Xl(s.props.onClick)(f),o())};return ce.cloneElement(s,d)},{component:"SignInWithMetamask",renderWhileLoading:!0});Qt(({clerk:e,children:t,...n})=>{const{redirectUrl:r="/",signOutOptions:a,...s}=n;t=Wl(t,"Sign out");const o=Kl(t)("SignOutButton"),c=()=>e.signOut({redirectUrl:r,...a}),f={...s,onClick:async m=>(await Xl(o.props.onClick)(m),c())};return ce.cloneElement(o,f)},{component:"SignOutButton",renderWhileLoading:!0});Qt(({clerk:e,children:t,...n})=>{const{fallbackRedirectUrl:r,forceRedirectUrl:a,signInFallbackRedirectUrl:s,signInForceRedirectUrl:o,mode:c,initialValues:d,oauthFlow:f,...m}=n;t=Wl(t,"Sign up");const p=Kl(t)("SignUpButton"),b=()=>{const x={fallbackRedirectUrl:r,forceRedirectUrl:a,signInFallbackRedirectUrl:s,signInForceRedirectUrl:o,initialValues:d,oauthFlow:f};return c==="modal"?e.openSignUp({...x,appearance:n.appearance,unsafeMetadata:n.unsafeMetadata}):e.redirectToSignUp({...x,signUpFallbackRedirectUrl:r,signUpForceRedirectUrl:a})},T={...m,onClick:async x=>(p&&typeof p=="object"&&"props"in p&&await Xl(p.props.onClick)(x),b())};return ce.cloneElement(p,T)},{component:"SignUpButton",renderWhileLoading:!0});var jO=()=>({fields:{identifier:null,password:null,code:null},raw:null,global:null}),$O=()=>({fields:{firstName:null,lastName:null,emailAddress:null,phoneNumber:null,password:null,username:null,code:null,captcha:null,legalAccepted:null},raw:null,global:null}),YO=class{constructor(e){this.isomorphicClerk=e,this.signInSignalProxy=this.buildSignInProxy(),this.signUpSignalProxy=this.buildSignUpProxy()}signInSignal(){return this.signInSignalProxy}signUpSignal(){return this.signUpSignalProxy}buildSignInProxy(){const e=this.gateProperty.bind(this),t=()=>this.client.signIn.__internal_future;return{errors:jO(),fetchStatus:"idle",signIn:{status:"needs_identifier",availableStrategies:[],isTransferable:!1,get id(){return e(t,"id",void 0)},get supportedFirstFactors(){return e(t,"supportedFirstFactors",[])},get supportedSecondFactors(){return e(t,"supportedSecondFactors",[])},get secondFactorVerification(){return e(t,"secondFactorVerification",{status:null,error:null,expireAt:null,externalVerificationRedirectURL:null,nonce:null,attempts:null,message:null,strategy:null,verifiedAtClient:null,verifiedFromTheSameClient:()=>!1,__internal_toSnapshot:()=>{throw new Error("__internal_toSnapshot called before Clerk is loaded")},pathRoot:"",reload:()=>{throw new Error("__internal_toSnapshot called before Clerk is loaded")}})},get identifier(){return e(t,"identifier",null)},get createdSessionId(){return e(t,"createdSessionId",null)},get userData(){return e(t,"userData",{})},get firstFactorVerification(){return e(t,"firstFactorVerification",{status:null,error:null,expireAt:null,externalVerificationRedirectURL:null,nonce:null,attempts:null,message:null,strategy:null,verifiedAtClient:null,verifiedFromTheSameClient:()=>!1,__internal_toSnapshot:()=>{throw new Error("__internal_toSnapshot called before Clerk is loaded")},pathRoot:"",reload:()=>{throw new Error("__internal_toSnapshot called before Clerk is loaded")}})},create:this.gateMethod(t,"create"),password:this.gateMethod(t,"password"),sso:this.gateMethod(t,"sso"),finalize:this.gateMethod(t,"finalize"),emailCode:this.wrapMethods(()=>t().emailCode,["sendCode","verifyCode"]),emailLink:this.wrapStruct(()=>t().emailLink,["sendLink","waitForVerification"],["verification"],{verification:null}),resetPasswordEmailCode:this.wrapMethods(()=>t().resetPasswordEmailCode,["sendCode","verifyCode","submitPassword"]),phoneCode:this.wrapMethods(()=>t().phoneCode,["sendCode","verifyCode"]),mfa:this.wrapMethods(()=>t().mfa,["sendPhoneCode","verifyPhoneCode","verifyTOTP","verifyBackupCode"]),ticket:this.gateMethod(t,"ticket"),passkey:this.gateMethod(t,"passkey"),web3:this.gateMethod(t,"web3")}}}buildSignUpProxy(){const e=this.gateProperty.bind(this),t=this.gateMethod.bind(this),n=this.wrapMethods.bind(this),r=()=>this.client.signUp.__internal_future;return{errors:$O(),fetchStatus:"idle",signUp:{get id(){return e(r,"id",void 0)},get requiredFields(){return e(r,"requiredFields",[])},get optionalFields(){return e(r,"optionalFields",[])},get missingFields(){return e(r,"missingFields",[])},get username(){return e(r,"username",null)},get firstName(){return e(r,"firstName",null)},get lastName(){return e(r,"lastName",null)},get emailAddress(){return e(r,"emailAddress",null)},get phoneNumber(){return e(r,"phoneNumber",null)},get web3Wallet(){return e(r,"web3Wallet",null)},get hasPassword(){return e(r,"hasPassword",!1)},get unsafeMetadata(){return e(r,"unsafeMetadata",{})},get createdSessionId(){return e(r,"createdSessionId",null)},get createdUserId(){return e(r,"createdUserId",null)},get abandonAt(){return e(r,"abandonAt",null)},get legalAcceptedAt(){return e(r,"legalAcceptedAt",null)},get locale(){return e(r,"locale",null)},get status(){return e(r,"status","missing_requirements")},get unverifiedFields(){return e(r,"unverifiedFields",[])},get isTransferable(){return e(r,"isTransferable",!1)},create:t(r,"create"),update:t(r,"update"),sso:t(r,"sso"),password:t(r,"password"),ticket:t(r,"ticket"),web3:t(r,"web3"),finalize:t(r,"finalize"),verifications:n(()=>r().verifications,["sendEmailCode","verifyEmailCode","sendPhoneCode","verifyPhoneCode"])}}}__internal_effect(e){throw new Error("__internal_effect called before Clerk is loaded")}__internal_computed(e){throw new Error("__internal_computed called before Clerk is loaded")}get client(){const e=this.isomorphicClerk.client;if(!e)throw new Error("Clerk client not ready");return e}gateProperty(e,t,n){return!Jh()||!this.isomorphicClerk.loaded?n:e()[t]}gateMethod(e,t){return async(...n)=>{if(!Jh())return Xa.throw(`Attempted to call a method (${t}) that is not supported on the server.`);this.isomorphicClerk.loaded||await new Promise(a=>this.isomorphicClerk.addOnLoaded(a));const r=e();return r[t].apply(r,n)}}wrapMethods(e,t){return Object.fromEntries(t.map(n=>[n,this.gateMethod(e,n)]))}wrapStruct(e,t,n,r){const a={};for(const s of t)a[s]=this.gateMethod(e,s);for(const s of n)Object.defineProperty(a,s,{get:()=>this.gateProperty(e,s,r[s]),enumerable:!0});return a}};typeof globalThis.__BUILD_DISABLE_RHC__>"u"&&(globalThis.__BUILD_DISABLE_RHC__=!1);var GO={name:"@clerk/clerk-react",version:"5.58.1",environment:"production"},Uh,zo,Ho,Gs,si,Fh,Xs,zc,zh,q8=class j8{constructor(t){Ys(this,zc),this.clerkjs=null,this.preopenOneTap=null,this.preopenUserVerification=null,this.preopenEnableOrganizationsPrompt=null,this.preopenSignIn=null,this.preopenCheckout=null,this.preopenPlanDetails=null,this.preopenSubscriptionDetails=null,this.preopenSignUp=null,this.preopenUserProfile=null,this.preopenOrganizationProfile=null,this.preopenCreateOrganization=null,this.preOpenWaitlist=null,this.premountSignInNodes=new Map,this.premountSignUpNodes=new Map,this.premountUserAvatarNodes=new Map,this.premountUserProfileNodes=new Map,this.premountUserButtonNodes=new Map,this.premountOrganizationProfileNodes=new Map,this.premountCreateOrganizationNodes=new Map,this.premountOrganizationSwitcherNodes=new Map,this.premountOrganizationListNodes=new Map,this.premountMethodCalls=new Map,this.premountWaitlistNodes=new Map,this.premountPricingTableNodes=new Map,this.premountAPIKeysNodes=new Map,this.premountOAuthConsentNodes=new Map,this.premountTaskChooseOrganizationNodes=new Map,this.premountTaskResetPasswordNodes=new Map,this.premountAddListenerCalls=new Map,this.loadedListeners=[],Ys(this,Uh,"loading"),Ys(this,zo),Ys(this,Ho),Ys(this,Gs),Ys(this,si,qO()),Ys(this,Fh),this.buildSignInUrl=a=>{const s=()=>{var o;return((o=this.clerkjs)==null?void 0:o.buildSignInUrl(a))||""};if(this.clerkjs&&this.loaded)return s();this.premountMethodCalls.set("buildSignInUrl",s)},this.buildSignUpUrl=a=>{const s=()=>{var o;return((o=this.clerkjs)==null?void 0:o.buildSignUpUrl(a))||""};if(this.clerkjs&&this.loaded)return s();this.premountMethodCalls.set("buildSignUpUrl",s)},this.buildAfterSignInUrl=(...a)=>{const s=()=>{var o;return((o=this.clerkjs)==null?void 0:o.buildAfterSignInUrl(...a))||""};if(this.clerkjs&&this.loaded)return s();this.premountMethodCalls.set("buildAfterSignInUrl",s)},this.buildAfterSignUpUrl=(...a)=>{const s=()=>{var o;return((o=this.clerkjs)==null?void 0:o.buildAfterSignUpUrl(...a))||""};if(this.clerkjs&&this.loaded)return s();this.premountMethodCalls.set("buildAfterSignUpUrl",s)},this.buildAfterSignOutUrl=()=>{const a=()=>{var s;return((s=this.clerkjs)==null?void 0:s.buildAfterSignOutUrl())||""};if(this.clerkjs&&this.loaded)return a();this.premountMethodCalls.set("buildAfterSignOutUrl",a)},this.buildNewSubscriptionRedirectUrl=()=>{const a=()=>{var s;return((s=this.clerkjs)==null?void 0:s.buildNewSubscriptionRedirectUrl())||""};if(this.clerkjs&&this.loaded)return a();this.premountMethodCalls.set("buildNewSubscriptionRedirectUrl",a)},this.buildAfterMultiSessionSingleSignOutUrl=()=>{const a=()=>{var s;return((s=this.clerkjs)==null?void 0:s.buildAfterMultiSessionSingleSignOutUrl())||""};if(this.clerkjs&&this.loaded)return a();this.premountMethodCalls.set("buildAfterMultiSessionSingleSignOutUrl",a)},this.buildUserProfileUrl=()=>{const a=()=>{var s;return((s=this.clerkjs)==null?void 0:s.buildUserProfileUrl())||""};if(this.clerkjs&&this.loaded)return a();this.premountMethodCalls.set("buildUserProfileUrl",a)},this.buildCreateOrganizationUrl=()=>{const a=()=>{var s;return((s=this.clerkjs)==null?void 0:s.buildCreateOrganizationUrl())||""};if(this.clerkjs&&this.loaded)return a();this.premountMethodCalls.set("buildCreateOrganizationUrl",a)},this.buildOrganizationProfileUrl=()=>{const a=()=>{var s;return((s=this.clerkjs)==null?void 0:s.buildOrganizationProfileUrl())||""};if(this.clerkjs&&this.loaded)return a();this.premountMethodCalls.set("buildOrganizationProfileUrl",a)},this.buildWaitlistUrl=()=>{const a=()=>{var s;return((s=this.clerkjs)==null?void 0:s.buildWaitlistUrl())||""};if(this.clerkjs&&this.loaded)return a();this.premountMethodCalls.set("buildWaitlistUrl",a)},this.buildTasksUrl=()=>{const a=()=>{var s;return((s=this.clerkjs)==null?void 0:s.buildTasksUrl())||""};if(this.clerkjs&&this.loaded)return a();this.premountMethodCalls.set("buildTasksUrl",a)},this.buildUrlWithAuth=a=>{const s=()=>{var o;return((o=this.clerkjs)==null?void 0:o.buildUrlWithAuth(a))||""};if(this.clerkjs&&this.loaded)return s();this.premountMethodCalls.set("buildUrlWithAuth",s)},this.handleUnauthenticated=async()=>{const a=()=>{var s;return(s=this.clerkjs)==null?void 0:s.handleUnauthenticated()};this.clerkjs&&this.loaded?a():this.premountMethodCalls.set("handleUnauthenticated",a)},this.on=(...a)=>{var s;if((s=this.clerkjs)!=null&&s.on)return this.clerkjs.on(...a);gn(this,si).on(...a)},this.off=(...a)=>{var s;if((s=this.clerkjs)!=null&&s.off)return this.clerkjs.off(...a);gn(this,si).off(...a)},this.addOnLoaded=a=>{this.loadedListeners.push(a),this.loaded&&this.emitLoaded()},this.emitLoaded=()=>{this.loadedListeners.forEach(a=>a()),this.loadedListeners=[]},this.beforeLoad=a=>{if(!a)throw new Error("Failed to hydrate latest Clerk JS")},this.hydrateClerkJS=a=>{var s,o;if(!a)throw new Error("Failed to hydrate latest Clerk JS");return this.clerkjs=a,this.premountMethodCalls.forEach(c=>c()),this.premountAddListenerCalls.forEach((c,d)=>{c.nativeUnsubscribe=a.addListener(d)}),(s=gn(this,si).internal.retrieveListeners("status"))==null||s.forEach(c=>{this.on("status",c,{notify:!0})}),(o=gn(this,si).internal.retrieveListeners("queryClientStatus"))==null||o.forEach(c=>{this.on("queryClientStatus",c,{notify:!0})}),this.preopenSignIn!==null&&a.openSignIn(this.preopenSignIn),this.preopenCheckout!==null&&a.__internal_openCheckout(this.preopenCheckout),this.preopenPlanDetails!==null&&a.__internal_openPlanDetails(this.preopenPlanDetails),this.preopenSubscriptionDetails!==null&&a.__internal_openSubscriptionDetails(this.preopenSubscriptionDetails),this.preopenSignUp!==null&&a.openSignUp(this.preopenSignUp),this.preopenUserProfile!==null&&a.openUserProfile(this.preopenUserProfile),this.preopenUserVerification!==null&&a.__internal_openReverification(this.preopenUserVerification),this.preopenOneTap!==null&&a.openGoogleOneTap(this.preopenOneTap),this.preopenOrganizationProfile!==null&&a.openOrganizationProfile(this.preopenOrganizationProfile),this.preopenCreateOrganization!==null&&a.openCreateOrganization(this.preopenCreateOrganization),this.preOpenWaitlist!==null&&a.openWaitlist(this.preOpenWaitlist),this.preopenEnableOrganizationsPrompt&&a.__internal_openEnableOrganizationsPrompt(this.preopenEnableOrganizationsPrompt),this.premountSignInNodes.forEach((c,d)=>{a.mountSignIn(d,c)}),this.premountSignUpNodes.forEach((c,d)=>{a.mountSignUp(d,c)}),this.premountUserProfileNodes.forEach((c,d)=>{a.mountUserProfile(d,c)}),this.premountUserAvatarNodes.forEach((c,d)=>{a.mountUserAvatar(d,c)}),this.premountUserButtonNodes.forEach((c,d)=>{a.mountUserButton(d,c)}),this.premountOrganizationListNodes.forEach((c,d)=>{a.mountOrganizationList(d,c)}),this.premountWaitlistNodes.forEach((c,d)=>{a.mountWaitlist(d,c)}),this.premountPricingTableNodes.forEach((c,d)=>{a.mountPricingTable(d,c)}),this.premountAPIKeysNodes.forEach((c,d)=>{a.mountAPIKeys(d,c)}),this.premountOAuthConsentNodes.forEach((c,d)=>{a.__internal_mountOAuthConsent(d,c)}),this.premountTaskChooseOrganizationNodes.forEach((c,d)=>{a.mountTaskChooseOrganization(d,c)}),this.premountTaskResetPasswordNodes.forEach((c,d)=>{a.mountTaskResetPassword(d,c)}),typeof this.clerkjs.status>"u"&&gn(this,si).emit(sh.Status,"ready"),this.emitLoaded(),this.clerkjs},this.__experimental_checkout=(...a)=>{var s;return(s=this.clerkjs)==null?void 0:s.__experimental_checkout(...a)},this.__unstable__updateProps=async a=>{const s=await M1(this,zc,zh).call(this);if(s&&"__unstable__updateProps"in s)return s.__unstable__updateProps(a)},this.setActive=a=>this.clerkjs?this.clerkjs.setActive(a):Promise.reject(),this.openSignIn=a=>{this.clerkjs&&this.loaded?this.clerkjs.openSignIn(a):this.preopenSignIn=a},this.closeSignIn=()=>{this.clerkjs&&this.loaded?this.clerkjs.closeSignIn():this.preopenSignIn=null},this.__internal_openCheckout=a=>{this.clerkjs&&this.loaded?this.clerkjs.__internal_openCheckout(a):this.preopenCheckout=a},this.__internal_closeCheckout=()=>{this.clerkjs&&this.loaded?this.clerkjs.__internal_closeCheckout():this.preopenCheckout=null},this.__internal_openPlanDetails=a=>{this.clerkjs&&this.loaded?this.clerkjs.__internal_openPlanDetails(a):this.preopenPlanDetails=a},this.__internal_closePlanDetails=()=>{this.clerkjs&&this.loaded?this.clerkjs.__internal_closePlanDetails():this.preopenPlanDetails=null},this.__internal_openSubscriptionDetails=a=>{this.clerkjs&&this.loaded?this.clerkjs.__internal_openSubscriptionDetails(a):this.preopenSubscriptionDetails=a??null},this.__internal_closeSubscriptionDetails=()=>{this.clerkjs&&this.loaded?this.clerkjs.__internal_closeSubscriptionDetails():this.preopenSubscriptionDetails=null},this.__internal_openReverification=a=>{this.clerkjs&&this.loaded?this.clerkjs.__internal_openReverification(a):this.preopenUserVerification=a},this.__internal_closeReverification=()=>{this.clerkjs&&this.loaded?this.clerkjs.__internal_closeReverification():this.preopenUserVerification=null},this.__internal_openEnableOrganizationsPrompt=a=>{this.clerkjs&&this.loaded?this.clerkjs.__internal_openEnableOrganizationsPrompt(a):this.preopenEnableOrganizationsPrompt=a},this.__internal_closeEnableOrganizationsPrompt=()=>{this.clerkjs&&this.loaded?this.clerkjs.__internal_closeEnableOrganizationsPrompt():this.preopenEnableOrganizationsPrompt=null},this.openGoogleOneTap=a=>{this.clerkjs&&this.loaded?this.clerkjs.openGoogleOneTap(a):this.preopenOneTap=a},this.closeGoogleOneTap=()=>{this.clerkjs&&this.loaded?this.clerkjs.closeGoogleOneTap():this.preopenOneTap=null},this.openUserProfile=a=>{this.clerkjs&&this.loaded?this.clerkjs.openUserProfile(a):this.preopenUserProfile=a},this.closeUserProfile=()=>{this.clerkjs&&this.loaded?this.clerkjs.closeUserProfile():this.preopenUserProfile=null},this.openOrganizationProfile=a=>{this.clerkjs&&this.loaded?this.clerkjs.openOrganizationProfile(a):this.preopenOrganizationProfile=a},this.closeOrganizationProfile=()=>{this.clerkjs&&this.loaded?this.clerkjs.closeOrganizationProfile():this.preopenOrganizationProfile=null},this.openCreateOrganization=a=>{this.clerkjs&&this.loaded?this.clerkjs.openCreateOrganization(a):this.preopenCreateOrganization=a},this.closeCreateOrganization=()=>{this.clerkjs&&this.loaded?this.clerkjs.closeCreateOrganization():this.preopenCreateOrganization=null},this.openWaitlist=a=>{this.clerkjs&&this.loaded?this.clerkjs.openWaitlist(a):this.preOpenWaitlist=a},this.closeWaitlist=()=>{this.clerkjs&&this.loaded?this.clerkjs.closeWaitlist():this.preOpenWaitlist=null},this.openSignUp=a=>{this.clerkjs&&this.loaded?this.clerkjs.openSignUp(a):this.preopenSignUp=a},this.closeSignUp=()=>{this.clerkjs&&this.loaded?this.clerkjs.closeSignUp():this.preopenSignUp=null},this.mountSignIn=(a,s)=>{this.clerkjs&&this.loaded?this.clerkjs.mountSignIn(a,s):this.premountSignInNodes.set(a,s)},this.unmountSignIn=a=>{this.clerkjs&&this.loaded?this.clerkjs.unmountSignIn(a):this.premountSignInNodes.delete(a)},this.mountSignUp=(a,s)=>{this.clerkjs&&this.loaded?this.clerkjs.mountSignUp(a,s):this.premountSignUpNodes.set(a,s)},this.unmountSignUp=a=>{this.clerkjs&&this.loaded?this.clerkjs.unmountSignUp(a):this.premountSignUpNodes.delete(a)},this.mountUserAvatar=(a,s)=>{this.clerkjs&&this.loaded?this.clerkjs.mountUserAvatar(a,s):this.premountUserAvatarNodes.set(a,s)},this.unmountUserAvatar=a=>{this.clerkjs&&this.loaded?this.clerkjs.unmountUserAvatar(a):this.premountUserAvatarNodes.delete(a)},this.mountUserProfile=(a,s)=>{this.clerkjs&&this.loaded?this.clerkjs.mountUserProfile(a,s):this.premountUserProfileNodes.set(a,s)},this.unmountUserProfile=a=>{this.clerkjs&&this.loaded?this.clerkjs.unmountUserProfile(a):this.premountUserProfileNodes.delete(a)},this.mountOrganizationProfile=(a,s)=>{this.clerkjs&&this.loaded?this.clerkjs.mountOrganizationProfile(a,s):this.premountOrganizationProfileNodes.set(a,s)},this.unmountOrganizationProfile=a=>{this.clerkjs&&this.loaded?this.clerkjs.unmountOrganizationProfile(a):this.premountOrganizationProfileNodes.delete(a)},this.mountCreateOrganization=(a,s)=>{this.clerkjs&&this.loaded?this.clerkjs.mountCreateOrganization(a,s):this.premountCreateOrganizationNodes.set(a,s)},this.unmountCreateOrganization=a=>{this.clerkjs&&this.loaded?this.clerkjs.unmountCreateOrganization(a):this.premountCreateOrganizationNodes.delete(a)},this.mountOrganizationSwitcher=(a,s)=>{this.clerkjs&&this.loaded?this.clerkjs.mountOrganizationSwitcher(a,s):this.premountOrganizationSwitcherNodes.set(a,s)},this.unmountOrganizationSwitcher=a=>{this.clerkjs&&this.loaded?this.clerkjs.unmountOrganizationSwitcher(a):this.premountOrganizationSwitcherNodes.delete(a)},this.__experimental_prefetchOrganizationSwitcher=()=>{const a=()=>{var s;return(s=this.clerkjs)==null?void 0:s.__experimental_prefetchOrganizationSwitcher()};this.clerkjs&&this.loaded?a():this.premountMethodCalls.set("__experimental_prefetchOrganizationSwitcher",a)},this.mountOrganizationList=(a,s)=>{this.clerkjs&&this.loaded?this.clerkjs.mountOrganizationList(a,s):this.premountOrganizationListNodes.set(a,s)},this.unmountOrganizationList=a=>{this.clerkjs&&this.loaded?this.clerkjs.unmountOrganizationList(a):this.premountOrganizationListNodes.delete(a)},this.mountUserButton=(a,s)=>{this.clerkjs&&this.loaded?this.clerkjs.mountUserButton(a,s):this.premountUserButtonNodes.set(a,s)},this.unmountUserButton=a=>{this.clerkjs&&this.loaded?this.clerkjs.unmountUserButton(a):this.premountUserButtonNodes.delete(a)},this.mountWaitlist=(a,s)=>{this.clerkjs&&this.loaded?this.clerkjs.mountWaitlist(a,s):this.premountWaitlistNodes.set(a,s)},this.unmountWaitlist=a=>{this.clerkjs&&this.loaded?this.clerkjs.unmountWaitlist(a):this.premountWaitlistNodes.delete(a)},this.mountPricingTable=(a,s)=>{this.clerkjs&&this.loaded?this.clerkjs.mountPricingTable(a,s):this.premountPricingTableNodes.set(a,s)},this.unmountPricingTable=a=>{this.clerkjs&&this.loaded?this.clerkjs.unmountPricingTable(a):this.premountPricingTableNodes.delete(a)},this.mountAPIKeys=(a,s)=>{this.clerkjs&&this.loaded?this.clerkjs.mountAPIKeys(a,s):this.premountAPIKeysNodes.set(a,s)},this.unmountAPIKeys=a=>{this.clerkjs&&this.loaded?this.clerkjs.unmountAPIKeys(a):this.premountAPIKeysNodes.delete(a)},this.__internal_mountOAuthConsent=(a,s)=>{this.clerkjs&&this.loaded?this.clerkjs.__internal_mountOAuthConsent(a,s):this.premountOAuthConsentNodes.set(a,s)},this.__internal_unmountOAuthConsent=a=>{this.clerkjs&&this.loaded?this.clerkjs.__internal_unmountOAuthConsent(a):this.premountOAuthConsentNodes.delete(a)},this.mountTaskChooseOrganization=(a,s)=>{this.clerkjs&&this.loaded?this.clerkjs.mountTaskChooseOrganization(a,s):this.premountTaskChooseOrganizationNodes.set(a,s)},this.unmountTaskChooseOrganization=a=>{this.clerkjs&&this.loaded?this.clerkjs.unmountTaskChooseOrganization(a):this.premountTaskChooseOrganizationNodes.delete(a)},this.mountTaskResetPassword=(a,s)=>{this.clerkjs&&this.loaded?this.clerkjs.mountTaskResetPassword(a,s):this.premountTaskResetPasswordNodes.set(a,s)},this.unmountTaskResetPassword=a=>{this.clerkjs&&this.loaded?this.clerkjs.unmountTaskResetPassword(a):this.premountTaskResetPasswordNodes.delete(a)},this.addListener=a=>{if(this.clerkjs)return this.clerkjs.addListener(a);{const s=()=>{var o;const c=this.premountAddListenerCalls.get(a);c&&((o=c.nativeUnsubscribe)==null||o.call(c),this.premountAddListenerCalls.delete(a))};return this.premountAddListenerCalls.set(a,{unsubscribe:s,nativeUnsubscribe:void 0}),s}},this.navigate=a=>{const s=()=>{var o;return(o=this.clerkjs)==null?void 0:o.navigate(a)};this.clerkjs&&this.loaded?s():this.premountMethodCalls.set("navigate",s)},this.redirectWithAuth=async(...a)=>{const s=()=>{var o;return(o=this.clerkjs)==null?void 0:o.redirectWithAuth(...a)};if(this.clerkjs&&this.loaded)return s();this.premountMethodCalls.set("redirectWithAuth",s)},this.redirectToSignIn=async a=>{const s=()=>{var o;return(o=this.clerkjs)==null?void 0:o.redirectToSignIn(a)};if(this.clerkjs&&this.loaded)return s();this.premountMethodCalls.set("redirectToSignIn",s)},this.redirectToSignUp=async a=>{const s=()=>{var o;return(o=this.clerkjs)==null?void 0:o.redirectToSignUp(a)};if(this.clerkjs&&this.loaded)return s();this.premountMethodCalls.set("redirectToSignUp",s)},this.redirectToUserProfile=async()=>{const a=()=>{var s;return(s=this.clerkjs)==null?void 0:s.redirectToUserProfile()};if(this.clerkjs&&this.loaded)return a();this.premountMethodCalls.set("redirectToUserProfile",a)},this.redirectToAfterSignUp=()=>{const a=()=>{var s;return(s=this.clerkjs)==null?void 0:s.redirectToAfterSignUp()};if(this.clerkjs&&this.loaded)return a();this.premountMethodCalls.set("redirectToAfterSignUp",a)},this.redirectToAfterSignIn=()=>{const a=()=>{var s;return(s=this.clerkjs)==null?void 0:s.redirectToAfterSignIn()};this.clerkjs&&this.loaded?a():this.premountMethodCalls.set("redirectToAfterSignIn",a)},this.redirectToAfterSignOut=()=>{const a=()=>{var s;return(s=this.clerkjs)==null?void 0:s.redirectToAfterSignOut()};this.clerkjs&&this.loaded?a():this.premountMethodCalls.set("redirectToAfterSignOut",a)},this.redirectToOrganizationProfile=async()=>{const a=()=>{var s;return(s=this.clerkjs)==null?void 0:s.redirectToOrganizationProfile()};if(this.clerkjs&&this.loaded)return a();this.premountMethodCalls.set("redirectToOrganizationProfile",a)},this.redirectToCreateOrganization=async()=>{const a=()=>{var s;return(s=this.clerkjs)==null?void 0:s.redirectToCreateOrganization()};if(this.clerkjs&&this.loaded)return a();this.premountMethodCalls.set("redirectToCreateOrganization",a)},this.redirectToWaitlist=async()=>{const a=()=>{var s;return(s=this.clerkjs)==null?void 0:s.redirectToWaitlist()};if(this.clerkjs&&this.loaded)return a();this.premountMethodCalls.set("redirectToWaitlist",a)},this.redirectToTasks=async a=>{const s=()=>{var o;return(o=this.clerkjs)==null?void 0:o.redirectToTasks(a)};if(this.clerkjs&&this.loaded)return s();this.premountMethodCalls.set("redirectToTasks",s)},this.handleRedirectCallback=async a=>{var s;const o=()=>{var c;return(c=this.clerkjs)==null?void 0:c.handleRedirectCallback(a)};this.clerkjs&&this.loaded?(s=o())==null||s.catch(()=>{}):this.premountMethodCalls.set("handleRedirectCallback",o)},this.handleGoogleOneTapCallback=async(a,s)=>{var o;const c=()=>{var d;return(d=this.clerkjs)==null?void 0:d.handleGoogleOneTapCallback(a,s)};this.clerkjs&&this.loaded?(o=c())==null||o.catch(()=>{}):this.premountMethodCalls.set("handleGoogleOneTapCallback",c)},this.handleEmailLinkVerification=async a=>{const s=()=>{var o;return(o=this.clerkjs)==null?void 0:o.handleEmailLinkVerification(a)};if(this.clerkjs&&this.loaded)return s();this.premountMethodCalls.set("handleEmailLinkVerification",s)},this.authenticateWithMetamask=async a=>{const s=()=>{var o;return(o=this.clerkjs)==null?void 0:o.authenticateWithMetamask(a)};if(this.clerkjs&&this.loaded)return s();this.premountMethodCalls.set("authenticateWithMetamask",s)},this.authenticateWithCoinbaseWallet=async a=>{const s=()=>{var o;return(o=this.clerkjs)==null?void 0:o.authenticateWithCoinbaseWallet(a)};if(this.clerkjs&&this.loaded)return s();this.premountMethodCalls.set("authenticateWithCoinbaseWallet",s)},this.authenticateWithBase=async a=>{const s=()=>{var o;return(o=this.clerkjs)==null?void 0:o.authenticateWithBase(a)};if(this.clerkjs&&this.loaded)return s();this.premountMethodCalls.set("authenticateWithBase",s)},this.authenticateWithOKXWallet=async a=>{const s=()=>{var o;return(o=this.clerkjs)==null?void 0:o.authenticateWithOKXWallet(a)};if(this.clerkjs&&this.loaded)return s();this.premountMethodCalls.set("authenticateWithOKXWallet",s)},this.authenticateWithWeb3=async a=>{const s=()=>{var o;return(o=this.clerkjs)==null?void 0:o.authenticateWithWeb3(a)};if(this.clerkjs&&this.loaded)return s();this.premountMethodCalls.set("authenticateWithWeb3",s)},this.authenticateWithGoogleOneTap=async a=>(await M1(this,zc,zh).call(this)).authenticateWithGoogleOneTap(a),this.__internal_loadStripeJs=async()=>(await M1(this,zc,zh).call(this)).__internal_loadStripeJs(),this.createOrganization=async a=>{const s=()=>{var o;return(o=this.clerkjs)==null?void 0:o.createOrganization(a)};if(this.clerkjs&&this.loaded)return s();this.premountMethodCalls.set("createOrganization",s)},this.getOrganization=async a=>{const s=()=>{var o;return(o=this.clerkjs)==null?void 0:o.getOrganization(a)};if(this.clerkjs&&this.loaded)return s();this.premountMethodCalls.set("getOrganization",s)},this.joinWaitlist=async a=>{const s=()=>{var o;return(o=this.clerkjs)==null?void 0:o.joinWaitlist(a)};if(this.clerkjs&&this.loaded)return s();this.premountMethodCalls.set("joinWaitlist",s)},this.signOut=async(...a)=>{const s=()=>{var o;return(o=this.clerkjs)==null?void 0:o.signOut(...a)};if(this.clerkjs&&this.loaded)return s();this.premountMethodCalls.set("signOut",s)},this.__internal_attemptToEnableEnvironmentSetting=a=>{const s=()=>{var o;return(o=this.clerkjs)==null?void 0:o.__internal_attemptToEnableEnvironmentSetting(a)};if(this.clerkjs&&this.loaded)return s();this.premountMethodCalls.set("__internal_attemptToEnableEnvironmentSetting",s)};const{Clerk:n=null,publishableKey:r}=t||{};Dl(this,Gs,r),Dl(this,Ho,t==null?void 0:t.proxyUrl),Dl(this,zo,t==null?void 0:t.domain),this.options=t,this.Clerk=n,this.mode=Jh()?"browser":"server",Dl(this,Fh,new YO(this)),this.options.sdkMetadata||(this.options.sdkMetadata=GO),gn(this,si).emit(sh.Status,"loading"),gn(this,si).prioritizedOn(sh.Status,a=>Dl(this,Uh,a)),gn(this,Gs)&&this.loadClerkJS()}get publishableKey(){return gn(this,Gs)}get loaded(){var t;return((t=this.clerkjs)==null?void 0:t.loaded)||!1}get status(){var t;return this.clerkjs?((t=this.clerkjs)==null?void 0:t.status)||(this.clerkjs.loaded?"ready":"loading"):gn(this,Uh)}static getOrCreateInstance(t){return(!Jh()||!gn(this,Xs)||t.Clerk&&gn(this,Xs).Clerk!==t.Clerk||gn(this,Xs).publishableKey!==t.publishableKey)&&Dl(this,Xs,new j8(t)),gn(this,Xs)}static clearInstance(){Dl(this,Xs,null)}get domain(){return typeof window<"u"&&window.location?k1(gn(this,zo),new URL(window.location.href),""):typeof gn(this,zo)=="function"?Xa.throw(A1):gn(this,zo)||""}get proxyUrl(){return typeof window<"u"&&window.location?k1(gn(this,Ho),new URL(window.location.href),""):typeof gn(this,Ho)=="function"?Xa.throw(A1):gn(this,Ho)||""}__internal_getOption(t){var n,r;return(n=this.clerkjs)!=null&&n.__internal_getOption?(r=this.clerkjs)==null?void 0:r.__internal_getOption(t):this.options[t]}get sdkMetadata(){var t;return((t=this.clerkjs)==null?void 0:t.sdkMetadata)||this.options.sdkMetadata||void 0}get instanceType(){var t;return(t=this.clerkjs)==null?void 0:t.instanceType}get frontendApi(){var t;return((t=this.clerkjs)==null?void 0:t.frontendApi)||""}get isStandardBrowser(){var t;return((t=this.clerkjs)==null?void 0:t.isStandardBrowser)||this.options.standardBrowser||!1}get __internal_queryClient(){var t;return(t=this.clerkjs)==null?void 0:t.__internal_queryClient}get isSatellite(){return typeof window<"u"&&window.location?k1(this.options.isSatellite,new URL(window.location.href),!1):typeof this.options.isSatellite=="function"?Xa.throw(A1):!1}async loadClerkJS(){var t;if(!(this.mode!=="browser"||this.loaded)){typeof window<"u"&&(window.__clerk_publishable_key=gn(this,Gs),window.__clerk_proxy_url=this.proxyUrl,window.__clerk_domain=this.domain);try{if(this.Clerk){let n;JN(this.Clerk)?(n=new this.Clerk(gn(this,Gs),{proxyUrl:this.proxyUrl,domain:this.domain}),this.beforeLoad(n),await n.load(this.options)):(n=this.Clerk,n.loaded||(this.beforeLoad(n),await n.load(this.options))),global.Clerk=n}else if(!__BUILD_DISABLE_RHC__){if(global.Clerk||await DO({...this.options,publishableKey:gn(this,Gs),proxyUrl:this.proxyUrl,domain:this.domain,nonce:this.options.nonce}),!global.Clerk)throw new Error("Failed to download latest ClerkJS. Contact support@clerk.com.");this.beforeLoad(global.Clerk),await global.Clerk.load(this.options)}return(t=global.Clerk)!=null&&t.loaded?this.hydrateClerkJS(global.Clerk):void 0}catch(n){const r=n;gn(this,si).emit(sh.Status,"error"),console.error(r.stack||r.message||r);return}}}get version(){var t;return(t=this.clerkjs)==null?void 0:t.version}get client(){if(this.clerkjs)return this.clerkjs.client}get session(){if(this.clerkjs)return this.clerkjs.session}get user(){if(this.clerkjs)return this.clerkjs.user}get organization(){if(this.clerkjs)return this.clerkjs.organization}get telemetry(){if(this.clerkjs)return this.clerkjs.telemetry}get __unstable__environment(){if(this.clerkjs)return this.clerkjs.__unstable__environment}get isSignedIn(){return this.clerkjs?this.clerkjs.isSignedIn:!1}get billing(){var t;return(t=this.clerkjs)==null?void 0:t.billing}get __internal_state(){return this.loaded&&this.clerkjs?this.clerkjs.__internal_state:gn(this,Fh)}get apiKeys(){var t;return(t=this.clerkjs)==null?void 0:t.apiKeys}__unstable__setEnvironment(...t){if(this.clerkjs&&"__unstable__setEnvironment"in this.clerkjs)this.clerkjs.__unstable__setEnvironment(t);else return}};Uh=new WeakMap;zo=new WeakMap;Ho=new WeakMap;Gs=new WeakMap;si=new WeakMap;Fh=new WeakMap;Xs=new WeakMap;zc=new WeakSet;zh=function(){return new Promise(e=>{this.addOnLoaded(()=>e(this.clerkjs))})};Ys(q8,Xs);var C_=q8;function VO(e){const{isomorphicClerkOptions:t,initialState:n,children:r}=e,{isomorphicClerk:a,clerkStatus:s}=KO(t),[o,c]=ce.useState({client:a.client,session:a.session,user:a.user,organization:a.organization});ce.useEffect(()=>a.addListener(oe=>c({...oe})),[]);const d=UO(a.loaded,o,n),f=ce.useMemo(()=>({value:a}),[s]),m=ce.useMemo(()=>({value:o.client}),[o.client]),{sessionId:p,sessionStatus:b,sessionClaims:y,session:T,userId:x,user:C,orgId:w,actor:O,organization:D,orgRole:Y,orgSlug:H,orgPermissions:B,factorVerificationAge:ee}=d,Z=ce.useMemo(()=>({value:{sessionId:p,sessionStatus:b,sessionClaims:y,userId:x,actor:O,orgId:w,orgRole:Y,orgSlug:H,orgPermissions:B,factorVerificationAge:ee}}),[p,b,x,O,w,Y,H,ee,y==null?void 0:y.__raw]),Q=ce.useMemo(()=>({value:T}),[p,T]),G=ce.useMemo(()=>({value:C}),[x,C]),se=ce.useMemo(()=>({value:{organization:D}}),[w,D]);return ce.createElement(AN.Provider,{value:f},ce.createElement(aN.Provider,{value:m},ce.createElement(iN.Provider,{value:Q},ce.createElement(cN,{...se.value},ce.createElement(SN.Provider,{value:Z},ce.createElement(rN.Provider,{value:G},ce.createElement(lN,{value:void 0},r)))))))}var KO=e=>{const t=ce.useRef(C_.getOrCreateInstance(e)),[n,r]=ce.useState(t.current.status);return ce.useEffect(()=>{t.current.__unstable__updateProps({appearance:e.appearance})},[e.appearance]),ce.useEffect(()=>{t.current.__unstable__updateProps({options:e})},[e.localization]),ce.useEffect(()=>(t.current.on("status",r),()=>{t.current&&t.current.off("status",r),C_.clearInstance()}),[]),{isomorphicClerk:t.current,clerkStatus:n}};function WO(e){const{initialState:t,children:n,__internal_bypassMissingPublishableKey:r,...a}=e,{publishableKey:s="",Clerk:o}=a;return!o&&!r&&(s?s&&!Hg(s)&&Xa.throwInvalidPublishableKeyError({key:s}):Xa.throwMissingPublishableKeyError()),ce.createElement(VO,{initialState:t,isomorphicClerkOptions:a},n)}var $8=tO(WO,"ClerkProvider",kN);$8.displayName="ClerkProvider";xN({packageName:"@clerk/clerk-react"});MO("@clerk/clerk-react");function N_(e){const t=[],n=String(e||"");let r=n.indexOf(","),a=0,s=!1;for(;!s;){r===-1&&(r=n.length,s=!0);const o=n.slice(a,r).trim();(o||!s)&&t.push(o),a=r+1,r=n.indexOf(",",a)}return t}function Y8(e,t){const n={};return(e[e.length-1]===""?[...e,""]:e).join((n.padRight?" ":"")+","+(n.padLeft===!1?"":" ")).trim()}const XO=/^[$_\p{ID_Start}][$_\u{200C}\u{200D}\p{ID_Continue}]*$/u,QO=/^[$_\p{ID_Start}][-$_\u{200C}\u{200D}\p{ID_Continue}]*$/u,ZO={};function O_(e,t){return(ZO.jsx?QO:XO).test(e)}const JO=/[ \t\n\f\r]/g;function eR(e){return typeof e=="object"?e.type==="text"?R_(e.value):!1:R_(e)}function R_(e){return e.replace(JO,"")===""}let C0=class{constructor(t,n,r){this.normal=n,this.property=t,r&&(this.space=r)}};C0.prototype.normal={};C0.prototype.property={};C0.prototype.space=void 0;function G8(e,t){const n={},r={};for(const a of e)Object.assign(n,a.property),Object.assign(r,a.normal);return new C0(n,r,t)}function d0(e){return e.toLowerCase()}let ra=class{constructor(t,n){this.attribute=n,this.property=t}};ra.prototype.attribute="";ra.prototype.booleanish=!1;ra.prototype.boolean=!1;ra.prototype.commaOrSpaceSeparated=!1;ra.prototype.commaSeparated=!1;ra.prototype.defined=!1;ra.prototype.mustUseProperty=!1;ra.prototype.number=!1;ra.prototype.overloadedBoolean=!1;ra.prototype.property="";ra.prototype.spaceSeparated=!1;ra.prototype.space=void 0;let tR=0;const xt=Ql(),Kn=Ql(),Zg=Ql(),Me=Ql(),cn=Ql(),Wo=Ql(),pa=Ql();function Ql(){return 2**++tR}const Jg=Object.freeze(Object.defineProperty({__proto__:null,boolean:xt,booleanish:Kn,commaOrSpaceSeparated:pa,commaSeparated:Wo,number:Me,overloadedBoolean:Zg,spaceSeparated:cn},Symbol.toStringTag,{value:"Module"})),I1=Object.keys(Jg);let d2=class extends ra{constructor(t,n,r,a){let s=-1;if(super(t,n),M_(this,"space",a),typeof r=="number")for(;++s<I1.length;){const o=I1[s];M_(this,I1[s],(r&Jg[o])===Jg[o])}}};d2.prototype.defined=!0;function M_(e,t,n){n&&(e[t]=n)}function du(e){const t={},n={};for(const[r,a]of Object.entries(e.properties)){const s=new d2(r,e.transform(e.attributes||{},r),a,e.space);e.mustUseProperty&&e.mustUseProperty.includes(r)&&(s.mustUseProperty=!0),t[r]=s,n[d0(r)]=r,n[d0(s.attribute)]=r}return new C0(t,n,e.space)}const V8=du({properties:{ariaActiveDescendant:null,ariaAtomic:Kn,ariaAutoComplete:null,ariaBusy:Kn,ariaChecked:Kn,ariaColCount:Me,ariaColIndex:Me,ariaColSpan:Me,ariaControls:cn,ariaCurrent:null,ariaDescribedBy:cn,ariaDetails:null,ariaDisabled:Kn,ariaDropEffect:cn,ariaErrorMessage:null,ariaExpanded:Kn,ariaFlowTo:cn,ariaGrabbed:Kn,ariaHasPopup:null,ariaHidden:Kn,ariaInvalid:null,ariaKeyShortcuts:null,ariaLabel:null,ariaLabelledBy:cn,ariaLevel:Me,ariaLive:null,ariaModal:Kn,ariaMultiLine:Kn,ariaMultiSelectable:Kn,ariaOrientation:null,ariaOwns:cn,ariaPlaceholder:null,ariaPosInSet:Me,ariaPressed:Kn,ariaReadOnly:Kn,ariaRelevant:null,ariaRequired:Kn,ariaRoleDescription:cn,ariaRowCount:Me,ariaRowIndex:Me,ariaRowSpan:Me,ariaSelected:Kn,ariaSetSize:Me,ariaSort:null,ariaValueMax:Me,ariaValueMin:Me,ariaValueNow:Me,ariaValueText:null,role:null},transform(e,t){return t==="role"?t:"aria-"+t.slice(4).toLowerCase()}});function K8(e,t){return t in e?e[t]:t}function W8(e,t){return K8(e,t.toLowerCase())}const nR=du({attributes:{acceptcharset:"accept-charset",classname:"class",htmlfor:"for",httpequiv:"http-equiv"},mustUseProperty:["checked","multiple","muted","selected"],properties:{abbr:null,accept:Wo,acceptCharset:cn,accessKey:cn,action:null,allow:null,allowFullScreen:xt,allowPaymentRequest:xt,allowUserMedia:xt,alt:null,as:null,async:xt,autoCapitalize:null,autoComplete:cn,autoFocus:xt,autoPlay:xt,blocking:cn,capture:null,charSet:null,checked:xt,cite:null,className:cn,cols:Me,colSpan:null,content:null,contentEditable:Kn,controls:xt,controlsList:cn,coords:Me|Wo,crossOrigin:null,data:null,dateTime:null,decoding:null,default:xt,defer:xt,dir:null,dirName:null,disabled:xt,download:Zg,draggable:Kn,encType:null,enterKeyHint:null,fetchPriority:null,form:null,formAction:null,formEncType:null,formMethod:null,formNoValidate:xt,formTarget:null,headers:cn,height:Me,hidden:Zg,high:Me,href:null,hrefLang:null,htmlFor:cn,httpEquiv:cn,id:null,imageSizes:null,imageSrcSet:null,inert:xt,inputMode:null,integrity:null,is:null,isMap:xt,itemId:null,itemProp:cn,itemRef:cn,itemScope:xt,itemType:cn,kind:null,label:null,lang:null,language:null,list:null,loading:null,loop:xt,low:Me,manifest:null,max:null,maxLength:Me,media:null,method:null,min:null,minLength:Me,multiple:xt,muted:xt,name:null,nonce:null,noModule:xt,noValidate:xt,onAbort:null,onAfterPrint:null,onAuxClick:null,onBeforeMatch:null,onBeforePrint:null,onBeforeToggle:null,onBeforeUnload:null,onBlur:null,onCancel:null,onCanPlay:null,onCanPlayThrough:null,onChange:null,onClick:null,onClose:null,onContextLost:null,onContextMenu:null,onContextRestored:null,onCopy:null,onCueChange:null,onCut:null,onDblClick:null,onDrag:null,onDragEnd:null,onDragEnter:null,onDragExit:null,onDragLeave:null,onDragOver:null,onDragStart:null,onDrop:null,onDurationChange:null,onEmptied:null,onEnded:null,onError:null,onFocus:null,onFormData:null,onHashChange:null,onInput:null,onInvalid:null,onKeyDown:null,onKeyPress:null,onKeyUp:null,onLanguageChange:null,onLoad:null,onLoadedData:null,onLoadedMetadata:null,onLoadEnd:null,onLoadStart:null,onMessage:null,onMessageError:null,onMouseDown:null,onMouseEnter:null,onMouseLeave:null,onMouseMove:null,onMouseOut:null,onMouseOver:null,onMouseUp:null,onOffline:null,onOnline:null,onPageHide:null,onPageShow:null,onPaste:null,onPause:null,onPlay:null,onPlaying:null,onPopState:null,onProgress:null,onRateChange:null,onRejectionHandled:null,onReset:null,onResize:null,onScroll:null,onScrollEnd:null,onSecurityPolicyViolation:null,onSeeked:null,onSeeking:null,onSelect:null,onSlotChange:null,onStalled:null,onStorage:null,onSubmit:null,onSuspend:null,onTimeUpdate:null,onToggle:null,onUnhandledRejection:null,onUnload:null,onVolumeChange:null,onWaiting:null,onWheel:null,open:xt,optimum:Me,pattern:null,ping:cn,placeholder:null,playsInline:xt,popover:null,popoverTarget:null,popoverTargetAction:null,poster:null,preload:null,readOnly:xt,referrerPolicy:null,rel:cn,required:xt,reversed:xt,rows:Me,rowSpan:Me,sandbox:cn,scope:null,scoped:xt,seamless:xt,selected:xt,shadowRootClonable:xt,shadowRootDelegatesFocus:xt,shadowRootMode:null,shape:null,size:Me,sizes:null,slot:null,span:Me,spellCheck:Kn,src:null,srcDoc:null,srcLang:null,srcSet:null,start:Me,step:null,style:null,tabIndex:Me,target:null,title:null,translate:null,type:null,typeMustMatch:xt,useMap:null,value:Kn,width:Me,wrap:null,writingSuggestions:null,align:null,aLink:null,archive:cn,axis:null,background:null,bgColor:null,border:Me,borderColor:null,bottomMargin:Me,cellPadding:null,cellSpacing:null,char:null,charOff:null,classId:null,clear:null,code:null,codeBase:null,codeType:null,color:null,compact:xt,declare:xt,event:null,face:null,frame:null,frameBorder:null,hSpace:Me,leftMargin:Me,link:null,longDesc:null,lowSrc:null,marginHeight:Me,marginWidth:Me,noResize:xt,noHref:xt,noShade:xt,noWrap:xt,object:null,profile:null,prompt:null,rev:null,rightMargin:Me,rules:null,scheme:null,scrolling:Kn,standby:null,summary:null,text:null,topMargin:Me,valueType:null,version:null,vAlign:null,vLink:null,vSpace:Me,allowTransparency:null,autoCorrect:null,autoSave:null,disablePictureInPicture:xt,disableRemotePlayback:xt,prefix:null,property:null,results:Me,security:null,unselectable:null},space:"html",transform:W8}),rR=du({attributes:{accentHeight:"accent-height",alignmentBaseline:"alignment-baseline",arabicForm:"arabic-form",baselineShift:"baseline-shift",capHeight:"cap-height",className:"class",clipPath:"clip-path",clipRule:"clip-rule",colorInterpolation:"color-interpolation",colorInterpolationFilters:"color-interpolation-filters",colorProfile:"color-profile",colorRendering:"color-rendering",crossOrigin:"crossorigin",dataType:"datatype",dominantBaseline:"dominant-baseline",enableBackground:"enable-background",fillOpacity:"fill-opacity",fillRule:"fill-rule",floodColor:"flood-color",floodOpacity:"flood-opacity",fontFamily:"font-family",fontSize:"font-size",fontSizeAdjust:"font-size-adjust",fontStretch:"font-stretch",fontStyle:"font-style",fontVariant:"font-variant",fontWeight:"font-weight",glyphName:"glyph-name",glyphOrientationHorizontal:"glyph-orientation-horizontal",glyphOrientationVertical:"glyph-orientation-vertical",hrefLang:"hreflang",horizAdvX:"horiz-adv-x",horizOriginX:"horiz-origin-x",horizOriginY:"horiz-origin-y",imageRendering:"image-rendering",letterSpacing:"letter-spacing",lightingColor:"lighting-color",markerEnd:"marker-end",markerMid:"marker-mid",markerStart:"marker-start",navDown:"nav-down",navDownLeft:"nav-down-left",navDownRight:"nav-down-right",navLeft:"nav-left",navNext:"nav-next",navPrev:"nav-prev",navRight:"nav-right",navUp:"nav-up",navUpLeft:"nav-up-left",navUpRight:"nav-up-right",onAbort:"onabort",onActivate:"onactivate",onAfterPrint:"onafterprint",onBeforePrint:"onbeforeprint",onBegin:"onbegin",onCancel:"oncancel",onCanPlay:"oncanplay",onCanPlayThrough:"oncanplaythrough",onChange:"onchange",onClick:"onclick",onClose:"onclose",onCopy:"oncopy",onCueChange:"oncuechange",onCut:"oncut",onDblClick:"ondblclick",onDrag:"ondrag",onDragEnd:"ondragend",onDragEnter:"ondragenter",onDragExit:"ondragexit",onDragLeave:"ondragleave",onDragOver:"ondragover",onDragStart:"ondragstart",onDrop:"ondrop",onDurationChange:"ondurationchange",onEmptied:"onemptied",onEnd:"onend",onEnded:"onended",onError:"onerror",onFocus:"onfocus",onFocusIn:"onfocusin",onFocusOut:"onfocusout",onHashChange:"onhashchange",onInput:"oninput",onInvalid:"oninvalid",onKeyDown:"onkeydown",onKeyPress:"onkeypress",onKeyUp:"onkeyup",onLoad:"onload",onLoadedData:"onloadeddata",onLoadedMetadata:"onloadedmetadata",onLoadStart:"onloadstart",onMessage:"onmessage",onMouseDown:"onmousedown",onMouseEnter:"onmouseenter",onMouseLeave:"onmouseleave",onMouseMove:"onmousemove",onMouseOut:"onmouseout",onMouseOver:"onmouseover",onMouseUp:"onmouseup",onMouseWheel:"onmousewheel",onOffline:"onoffline",onOnline:"ononline",onPageHide:"onpagehide",onPageShow:"onpageshow",onPaste:"onpaste",onPause:"onpause",onPlay:"onplay",onPlaying:"onplaying",onPopState:"onpopstate",onProgress:"onprogress",onRateChange:"onratechange",onRepeat:"onrepeat",onReset:"onreset",onResize:"onresize",onScroll:"onscroll",onSeeked:"onseeked",onSeeking:"onseeking",onSelect:"onselect",onShow:"onshow",onStalled:"onstalled",onStorage:"onstorage",onSubmit:"onsubmit",onSuspend:"onsuspend",onTimeUpdate:"ontimeupdate",onToggle:"ontoggle",onUnload:"onunload",onVolumeChange:"onvolumechange",onWaiting:"onwaiting",onZoom:"onzoom",overlinePosition:"overline-position",overlineThickness:"overline-thickness",paintOrder:"paint-order",panose1:"panose-1",pointerEvents:"pointer-events",referrerPolicy:"referrerpolicy",renderingIntent:"rendering-intent",shapeRendering:"shape-rendering",stopColor:"stop-color",stopOpacity:"stop-opacity",strikethroughPosition:"strikethrough-position",strikethroughThickness:"strikethrough-thickness",strokeDashArray:"stroke-dasharray",strokeDashOffset:"stroke-dashoffset",strokeLineCap:"stroke-linecap",strokeLineJoin:"stroke-linejoin",strokeMiterLimit:"stroke-miterlimit",strokeOpacity:"stroke-opacity",strokeWidth:"stroke-width",tabIndex:"tabindex",textAnchor:"text-anchor",textDecoration:"text-decoration",textRendering:"text-rendering",transformOrigin:"transform-origin",typeOf:"typeof",underlinePosition:"underline-position",underlineThickness:"underline-thickness",unicodeBidi:"unicode-bidi",unicodeRange:"unicode-range",unitsPerEm:"units-per-em",vAlphabetic:"v-alphabetic",vHanging:"v-hanging",vIdeographic:"v-ideographic",vMathematical:"v-mathematical",vectorEffect:"vector-effect",vertAdvY:"vert-adv-y",vertOriginX:"vert-origin-x",vertOriginY:"vert-origin-y",wordSpacing:"word-spacing",writingMode:"writing-mode",xHeight:"x-height",playbackOrder:"playbackorder",timelineBegin:"timelinebegin"},properties:{about:pa,accentHeight:Me,accumulate:null,additive:null,alignmentBaseline:null,alphabetic:Me,amplitude:Me,arabicForm:null,ascent:Me,attributeName:null,attributeType:null,azimuth:Me,bandwidth:null,baselineShift:null,baseFrequency:null,baseProfile:null,bbox:null,begin:null,bias:Me,by:null,calcMode:null,capHeight:Me,className:cn,clip:null,clipPath:null,clipPathUnits:null,clipRule:null,color:null,colorInterpolation:null,colorInterpolationFilters:null,colorProfile:null,colorRendering:null,content:null,contentScriptType:null,contentStyleType:null,crossOrigin:null,cursor:null,cx:null,cy:null,d:null,dataType:null,defaultAction:null,descent:Me,diffuseConstant:Me,direction:null,display:null,dur:null,divisor:Me,dominantBaseline:null,download:xt,dx:null,dy:null,edgeMode:null,editable:null,elevation:Me,enableBackground:null,end:null,event:null,exponent:Me,externalResourcesRequired:null,fill:null,fillOpacity:Me,fillRule:null,filter:null,filterRes:null,filterUnits:null,floodColor:null,floodOpacity:null,focusable:null,focusHighlight:null,fontFamily:null,fontSize:null,fontSizeAdjust:null,fontStretch:null,fontStyle:null,fontVariant:null,fontWeight:null,format:null,fr:null,from:null,fx:null,fy:null,g1:Wo,g2:Wo,glyphName:Wo,glyphOrientationHorizontal:null,glyphOrientationVertical:null,glyphRef:null,gradientTransform:null,gradientUnits:null,handler:null,hanging:Me,hatchContentUnits:null,hatchUnits:null,height:null,href:null,hrefLang:null,horizAdvX:Me,horizOriginX:Me,horizOriginY:Me,id:null,ideographic:Me,imageRendering:null,initialVisibility:null,in:null,in2:null,intercept:Me,k:Me,k1:Me,k2:Me,k3:Me,k4:Me,kernelMatrix:pa,kernelUnitLength:null,keyPoints:null,keySplines:null,keyTimes:null,kerning:null,lang:null,lengthAdjust:null,letterSpacing:null,lightingColor:null,limitingConeAngle:Me,local:null,markerEnd:null,markerMid:null,markerStart:null,markerHeight:null,markerUnits:null,markerWidth:null,mask:null,maskContentUnits:null,maskUnits:null,mathematical:null,max:null,media:null,mediaCharacterEncoding:null,mediaContentEncodings:null,mediaSize:Me,mediaTime:null,method:null,min:null,mode:null,name:null,navDown:null,navDownLeft:null,navDownRight:null,navLeft:null,navNext:null,navPrev:null,navRight:null,navUp:null,navUpLeft:null,navUpRight:null,numOctaves:null,observer:null,offset:null,onAbort:null,onActivate:null,onAfterPrint:null,onBeforePrint:null,onBegin:null,onCancel:null,onCanPlay:null,onCanPlayThrough:null,onChange:null,onClick:null,onClose:null,onCopy:null,onCueChange:null,onCut:null,onDblClick:null,onDrag:null,onDragEnd:null,onDragEnter:null,onDragExit:null,onDragLeave:null,onDragOver:null,onDragStart:null,onDrop:null,onDurationChange:null,onEmptied:null,onEnd:null,onEnded:null,onError:null,onFocus:null,onFocusIn:null,onFocusOut:null,onHashChange:null,onInput:null,onInvalid:null,onKeyDown:null,onKeyPress:null,onKeyUp:null,onLoad:null,onLoadedData:null,onLoadedMetadata:null,onLoadStart:null,onMessage:null,onMouseDown:null,onMouseEnter:null,onMouseLeave:null,onMouseMove:null,onMouseOut:null,onMouseOver:null,onMouseUp:null,onMouseWheel:null,onOffline:null,onOnline:null,onPageHide:null,onPageShow:null,onPaste:null,onPause:null,onPlay:null,onPlaying:null,onPopState:null,onProgress:null,onRateChange:null,onRepeat:null,onReset:null,onResize:null,onScroll:null,onSeeked:null,onSeeking:null,onSelect:null,onShow:null,onStalled:null,onStorage:null,onSubmit:null,onSuspend:null,onTimeUpdate:null,onToggle:null,onUnload:null,onVolumeChange:null,onWaiting:null,onZoom:null,opacity:null,operator:null,order:null,orient:null,orientation:null,origin:null,overflow:null,overlay:null,overlinePosition:Me,overlineThickness:Me,paintOrder:null,panose1:null,path:null,pathLength:Me,patternContentUnits:null,patternTransform:null,patternUnits:null,phase:null,ping:cn,pitch:null,playbackOrder:null,pointerEvents:null,points:null,pointsAtX:Me,pointsAtY:Me,pointsAtZ:Me,preserveAlpha:null,preserveAspectRatio:null,primitiveUnits:null,propagate:null,property:pa,r:null,radius:null,referrerPolicy:null,refX:null,refY:null,rel:pa,rev:pa,renderingIntent:null,repeatCount:null,repeatDur:null,requiredExtensions:pa,requiredFeatures:pa,requiredFonts:pa,requiredFormats:pa,resource:null,restart:null,result:null,rotate:null,rx:null,ry:null,scale:null,seed:null,shapeRendering:null,side:null,slope:null,snapshotTime:null,specularConstant:Me,specularExponent:Me,spreadMethod:null,spacing:null,startOffset:null,stdDeviation:null,stemh:null,stemv:null,stitchTiles:null,stopColor:null,stopOpacity:null,strikethroughPosition:Me,strikethroughThickness:Me,string:null,stroke:null,strokeDashArray:pa,strokeDashOffset:null,strokeLineCap:null,strokeLineJoin:null,strokeMiterLimit:Me,strokeOpacity:Me,strokeWidth:null,style:null,surfaceScale:Me,syncBehavior:null,syncBehaviorDefault:null,syncMaster:null,syncTolerance:null,syncToleranceDefault:null,systemLanguage:pa,tabIndex:Me,tableValues:null,target:null,targetX:Me,targetY:Me,textAnchor:null,textDecoration:null,textRendering:null,textLength:null,timelineBegin:null,title:null,transformBehavior:null,type:null,typeOf:pa,to:null,transform:null,transformOrigin:null,u1:null,u2:null,underlinePosition:Me,underlineThickness:Me,unicode:null,unicodeBidi:null,unicodeRange:null,unitsPerEm:Me,values:null,vAlphabetic:Me,vMathematical:Me,vectorEffect:null,vHanging:Me,vIdeographic:Me,version:null,vertAdvY:Me,vertOriginX:Me,vertOriginY:Me,viewBox:null,viewTarget:null,visibility:null,width:null,widths:null,wordSpacing:null,writingMode:null,x:null,x1:null,x2:null,xChannelSelector:null,xHeight:Me,y:null,y1:null,y2:null,yChannelSelector:null,z:null,zoomAndPan:null},space:"svg",transform:K8}),X8=du({properties:{xLinkActuate:null,xLinkArcRole:null,xLinkHref:null,xLinkRole:null,xLinkShow:null,xLinkTitle:null,xLinkType:null},space:"xlink",transform(e,t){return"xlink:"+t.slice(5).toLowerCase()}}),Q8=du({attributes:{xmlnsxlink:"xmlns:xlink"},properties:{xmlnsXLink:null,xmlns:null},space:"xmlns",transform:W8}),Z8=du({properties:{xmlBase:null,xmlLang:null,xmlSpace:null},space:"xml",transform(e,t){return"xml:"+t.slice(3).toLowerCase()}}),aR={classId:"classID",dataType:"datatype",itemId:"itemID",strokeDashArray:"strokeDasharray",strokeDashOffset:"strokeDashoffset",strokeLineCap:"strokeLinecap",strokeLineJoin:"strokeLinejoin",strokeMiterLimit:"strokeMiterlimit",typeOf:"typeof",xLinkActuate:"xlinkActuate",xLinkArcRole:"xlinkArcrole",xLinkHref:"xlinkHref",xLinkRole:"xlinkRole",xLinkShow:"xlinkShow",xLinkTitle:"xlinkTitle",xLinkType:"xlinkType",xmlnsXLink:"xmlnsXlink"},iR=/[A-Z]/g,I_=/-[a-z]/g,sR=/^data[-\w.:]+$/i;function h2(e,t){const n=d0(t);let r=t,a=ra;if(n in e.normal)return e.property[e.normal[n]];if(n.length>4&&n.slice(0,4)==="data"&&sR.test(t)){if(t.charAt(4)==="-"){const s=t.slice(5).replace(I_,oR);r="data"+s.charAt(0).toUpperCase()+s.slice(1)}else{const s=t.slice(4);if(!I_.test(s)){let o=s.replace(iR,lR);o.charAt(0)!=="-"&&(o="-"+o),t="data"+o}}a=d2}return new a(r,t)}function lR(e){return"-"+e.toLowerCase()}function oR(e){return e.charAt(1).toUpperCase()}const jf=G8([V8,nR,X8,Q8,Z8],"html"),hu=G8([V8,rR,X8,Q8,Z8],"svg");function D_(e){const t=String(e||"").trim();return t?t.split(/[ \t\n\r\f]+/g):[]}function J8(e){return e.join(" ").trim()}var Lo={},D1,L_;function uR(){if(L_)return D1;L_=1;var e=/\/\*[^*]*\*+([^/*][^*]*\*+)*\//g,t=/\n/g,n=/^\s*/,r=/^(\*?[-#/*\\\w]+(\[[0-9a-z_-]+\])?)\s*/,a=/^:\s*/,s=/^((?:'(?:\\'|.)*?'|"(?:\\"|.)*?"|\([^)]*?\)|[^};])+)/,o=/^[;\s]*/,c=/^\s+|\s+$/g,d=`
`,f="/",m="*",p="",b="comment",y="declaration";D1=function(x,C){if(typeof x!="string")throw new TypeError("First argument must be a string");if(!x)return[];C=C||{};var w=1,O=1;function D(ye){var fe=ye.match(t);fe&&(w+=fe.length);var de=ye.lastIndexOf(d);O=~de?ye.length-de:O+ye.length}function Y(){var ye={line:w,column:O};return function(fe){return fe.position=new H(ye),Z(),fe}}function H(ye){this.start=ye,this.end={line:w,column:O},this.source=C.source}H.prototype.content=x;function B(ye){var fe=new Error(C.source+":"+w+":"+O+": "+ye);if(fe.reason=ye,fe.filename=C.source,fe.line=w,fe.column=O,fe.source=x,!C.silent)throw fe}function ee(ye){var fe=ye.exec(x);if(fe){var de=fe[0];return D(de),x=x.slice(de.length),fe}}function Z(){ee(n)}function Q(ye){var fe;for(ye=ye||[];fe=G();)fe!==!1&&ye.push(fe);return ye}function G(){var ye=Y();if(!(f!=x.charAt(0)||m!=x.charAt(1))){for(var fe=2;p!=x.charAt(fe)&&(m!=x.charAt(fe)||f!=x.charAt(fe+1));)++fe;if(fe+=2,p===x.charAt(fe-1))return B("End of comment missing");var de=x.slice(2,fe-2);return O+=2,D(de),x=x.slice(fe),O+=2,ye({type:b,comment:de})}}function se(){var ye=Y(),fe=ee(r);if(fe){if(G(),!ee(a))return B("property missing ':'");var de=ee(s),V=ye({type:y,property:T(fe[0].replace(e,p)),value:de?T(de[0].replace(e,p)):p});return ee(o),V}}function oe(){var ye=[];Q(ye);for(var fe;fe=se();)fe!==!1&&(ye.push(fe),Q(ye));return ye}return Z(),oe()};function T(x){return x?x.replace(c,p):p}return D1}var P_;function cR(){if(P_)return Lo;P_=1;var e=Lo&&Lo.__importDefault||function(r){return r&&r.__esModule?r:{default:r}};Object.defineProperty(Lo,"__esModule",{value:!0}),Lo.default=n;var t=e(uR());function n(r,a){var s=null;if(!r||typeof r!="string")return s;var o=(0,t.default)(r),c=typeof a=="function";return o.forEach(function(d){if(d.type==="declaration"){var f=d.property,m=d.value;c?a(f,m,d):m&&(s=s||{},s[f]=m)}}),s}return Lo}var Sc={},B_;function dR(){if(B_)return Sc;B_=1,Object.defineProperty(Sc,"__esModule",{value:!0}),Sc.camelCase=void 0;var e=/^--[a-zA-Z0-9_-]+$/,t=/-([a-z])/g,n=/^[^-]+$/,r=/^-(webkit|moz|ms|o|khtml)-/,a=/^-(ms)-/,s=function(f){return!f||n.test(f)||e.test(f)},o=function(f,m){return m.toUpperCase()},c=function(f,m){return"".concat(m,"-")},d=function(f,m){return m===void 0&&(m={}),s(f)?f:(f=f.toLowerCase(),m.reactCompat?f=f.replace(a,c):f=f.replace(r,c),f.replace(t,o))};return Sc.camelCase=d,Sc}var wc,U_;function hR(){if(U_)return wc;U_=1;var e=wc&&wc.__importDefault||function(a){return a&&a.__esModule?a:{default:a}},t=e(cR()),n=dR();function r(a,s){var o={};return!a||typeof a!="string"||(0,t.default)(a,function(c,d){c&&d&&(o[(0,n.camelCase)(c,s)]=d)}),o}return r.default=r,wc=r,wc}var fR=hR();const mR=Mf(fR),$f=e7("end"),yi=e7("start");function e7(e){return t;function t(n){const r=n&&n.position&&n.position[e]||{};if(typeof r.line=="number"&&r.line>0&&typeof r.column=="number"&&r.column>0)return{line:r.line,column:r.column,offset:typeof r.offset=="number"&&r.offset>-1?r.offset:void 0}}}function pR(e){const t=yi(e),n=$f(e);if(t&&n)return{start:t,end:n}}function Zc(e){return!e||typeof e!="object"?"":"position"in e||"type"in e?F_(e.position):"start"in e||"end"in e?F_(e):"line"in e||"column"in e?eb(e):""}function eb(e){return z_(e&&e.line)+":"+z_(e&&e.column)}function F_(e){return eb(e&&e.start)+"-"+eb(e&&e.end)}function z_(e){return e&&typeof e=="number"?e:1}class Cr extends Error{constructor(t,n,r){super(),typeof n=="string"&&(r=n,n=void 0);let a="",s={},o=!1;if(n&&("line"in n&&"column"in n?s={place:n}:"start"in n&&"end"in n?s={place:n}:"type"in n?s={ancestors:[n],place:n.position}:s={...n}),typeof t=="string"?a=t:!s.cause&&t&&(o=!0,a=t.message,s.cause=t),!s.ruleId&&!s.source&&typeof r=="string"){const d=r.indexOf(":");d===-1?s.ruleId=r:(s.source=r.slice(0,d),s.ruleId=r.slice(d+1))}if(!s.place&&s.ancestors&&s.ancestors){const d=s.ancestors[s.ancestors.length-1];d&&(s.place=d.position)}const c=s.place&&"start"in s.place?s.place.start:s.place;this.ancestors=s.ancestors||void 0,this.cause=s.cause||void 0,this.column=c?c.column:void 0,this.fatal=void 0,this.file,this.message=a,this.line=c?c.line:void 0,this.name=Zc(s.place)||"1:1",this.place=s.place||void 0,this.reason=this.message,this.ruleId=s.ruleId||void 0,this.source=s.source||void 0,this.stack=o&&s.cause&&typeof s.cause.stack=="string"?s.cause.stack:"",this.actual,this.expected,this.note,this.url}}Cr.prototype.file="";Cr.prototype.name="";Cr.prototype.reason="";Cr.prototype.message="";Cr.prototype.stack="";Cr.prototype.column=void 0;Cr.prototype.line=void 0;Cr.prototype.ancestors=void 0;Cr.prototype.cause=void 0;Cr.prototype.fatal=void 0;Cr.prototype.place=void 0;Cr.prototype.ruleId=void 0;Cr.prototype.source=void 0;const f2={}.hasOwnProperty,gR=new Map,bR=/[A-Z]/g,vR=new Set(["table","tbody","thead","tfoot","tr"]),yR=new Set(["td","th"]),t7="https://github.com/syntax-tree/hast-util-to-jsx-runtime";function _R(e,t){if(!t||t.Fragment===void 0)throw new TypeError("Expected `Fragment` in options");const n=t.filePath||void 0;let r;if(t.development){if(typeof t.jsxDEV!="function")throw new TypeError("Expected `jsxDEV` in options when `development: true`");r=CR(n,t.jsxDEV)}else{if(typeof t.jsx!="function")throw new TypeError("Expected `jsx` in production options");if(typeof t.jsxs!="function")throw new TypeError("Expected `jsxs` in production options");r=kR(n,t.jsx,t.jsxs)}const a={Fragment:t.Fragment,ancestors:[],components:t.components||{},create:r,elementAttributeNameCase:t.elementAttributeNameCase||"react",evaluater:t.createEvaluater?t.createEvaluater():void 0,filePath:n,ignoreInvalidStyle:t.ignoreInvalidStyle||!1,passKeys:t.passKeys!==!1,passNode:t.passNode||!1,schema:t.space==="svg"?hu:jf,stylePropertyNameCase:t.stylePropertyNameCase||"dom",tableCellAlignToStyle:t.tableCellAlignToStyle!==!1},s=n7(a,e,void 0);return s&&typeof s!="string"?s:a.create(e,a.Fragment,{children:s||void 0},void 0)}function n7(e,t,n){if(t.type==="element")return ER(e,t,n);if(t.type==="mdxFlowExpression"||t.type==="mdxTextExpression")return TR(e,t);if(t.type==="mdxJsxFlowElement"||t.type==="mdxJsxTextElement")return SR(e,t,n);if(t.type==="mdxjsEsm")return xR(e,t);if(t.type==="root")return wR(e,t,n);if(t.type==="text")return AR(e,t)}function ER(e,t,n){const r=e.schema;let a=r;t.tagName.toLowerCase()==="svg"&&r.space==="html"&&(a=hu,e.schema=a),e.ancestors.push(t);const s=a7(e,t.tagName,!1),o=NR(e,t);let c=p2(e,t);return vR.has(t.tagName)&&(c=c.filter(function(d){return typeof d=="string"?!eR(d):!0})),r7(e,o,s,t),m2(o,c),e.ancestors.pop(),e.schema=r,e.create(t,s,o,n)}function TR(e,t){if(t.data&&t.data.estree&&e.evaluater){const r=t.data.estree.body[0];return r.type,e.evaluater.evaluateExpression(r.expression)}h0(e,t.position)}function xR(e,t){if(t.data&&t.data.estree&&e.evaluater)return e.evaluater.evaluateProgram(t.data.estree);h0(e,t.position)}function SR(e,t,n){const r=e.schema;let a=r;t.name==="svg"&&r.space==="html"&&(a=hu,e.schema=a),e.ancestors.push(t);const s=t.name===null?e.Fragment:a7(e,t.name,!0),o=OR(e,t),c=p2(e,t);return r7(e,o,s,t),m2(o,c),e.ancestors.pop(),e.schema=r,e.create(t,s,o,n)}function wR(e,t,n){const r={};return m2(r,p2(e,t)),e.create(t,e.Fragment,r,n)}function AR(e,t){return t.value}function r7(e,t,n,r){typeof n!="string"&&n!==e.Fragment&&e.passNode&&(t.node=r)}function m2(e,t){if(t.length>0){const n=t.length>1?t:t[0];n&&(e.children=n)}}function kR(e,t,n){return r;function r(a,s,o,c){const f=Array.isArray(o.children)?n:t;return c?f(s,o,c):f(s,o)}}function CR(e,t){return n;function n(r,a,s,o){const c=Array.isArray(s.children),d=yi(r);return t(a,s,o,c,{columnNumber:d?d.column-1:void 0,fileName:e,lineNumber:d?d.line:void 0},void 0)}}function NR(e,t){const n={};let r,a;for(a in t.properties)if(a!=="children"&&f2.call(t.properties,a)){const s=RR(e,a,t.properties[a]);if(s){const[o,c]=s;e.tableCellAlignToStyle&&o==="align"&&typeof c=="string"&&yR.has(t.tagName)?r=c:n[o]=c}}if(r){const s=n.style||(n.style={});s[e.stylePropertyNameCase==="css"?"text-align":"textAlign"]=r}return n}function OR(e,t){const n={};for(const r of t.attributes)if(r.type==="mdxJsxExpressionAttribute")if(r.data&&r.data.estree&&e.evaluater){const s=r.data.estree.body[0];s.type;const o=s.expression;o.type;const c=o.properties[0];c.type,Object.assign(n,e.evaluater.evaluateExpression(c.argument))}else h0(e,t.position);else{const a=r.name;let s;if(r.value&&typeof r.value=="object")if(r.value.data&&r.value.data.estree&&e.evaluater){const c=r.value.data.estree.body[0];c.type,s=e.evaluater.evaluateExpression(c.expression)}else h0(e,t.position);else s=r.value===null?!0:r.value;n[a]=s}return n}function p2(e,t){const n=[];let r=-1;const a=e.passKeys?new Map:gR;for(;++r<t.children.length;){const s=t.children[r];let o;if(e.passKeys){const d=s.type==="element"?s.tagName:s.type==="mdxJsxFlowElement"||s.type==="mdxJsxTextElement"?s.name:void 0;if(d){const f=a.get(d)||0;o=d+"-"+f,a.set(d,f+1)}}const c=n7(e,s,o);c!==void 0&&n.push(c)}return n}function RR(e,t,n){const r=h2(e.schema,t);if(!(n==null||typeof n=="number"&&Number.isNaN(n))){if(Array.isArray(n)&&(n=r.commaSeparated?Y8(n):J8(n)),r.property==="style"){let a=typeof n=="object"?n:MR(e,String(n));return e.stylePropertyNameCase==="css"&&(a=IR(a)),["style",a]}return[e.elementAttributeNameCase==="react"&&r.space?aR[r.property]||r.property:r.attribute,n]}}function MR(e,t){try{return mR(t,{reactCompat:!0})}catch(n){if(e.ignoreInvalidStyle)return{};const r=n,a=new Cr("Cannot parse `style` attribute",{ancestors:e.ancestors,cause:r,ruleId:"style",source:"hast-util-to-jsx-runtime"});throw a.file=e.filePath||void 0,a.url=t7+"#cannot-parse-style-attribute",a}}function a7(e,t,n){let r;if(!n)r={type:"Literal",value:t};else if(t.includes(".")){const a=t.split(".");let s=-1,o;for(;++s<a.length;){const c=O_(a[s])?{type:"Identifier",name:a[s]}:{type:"Literal",value:a[s]};o=o?{type:"MemberExpression",object:o,property:c,computed:!!(s&&c.type==="Literal"),optional:!1}:c}r=o}else r=O_(t)&&!/^[a-z]/.test(t)?{type:"Identifier",name:t}:{type:"Literal",value:t};if(r.type==="Literal"){const a=r.value;return f2.call(e.components,a)?e.components[a]:a}if(e.evaluater)return e.evaluater.evaluateExpression(r);h0(e)}function h0(e,t){const n=new Cr("Cannot handle MDX estrees without `createEvaluater`",{ancestors:e.ancestors,place:t,ruleId:"mdx-estree",source:"hast-util-to-jsx-runtime"});throw n.file=e.filePath||void 0,n.url=t7+"#cannot-handle-mdx-estrees-without-createevaluater",n}function IR(e){const t={};let n;for(n in e)f2.call(e,n)&&(t[DR(n)]=e[n]);return t}function DR(e){let t=e.replace(bR,LR);return t.slice(0,3)==="ms-"&&(t="-"+t),t}function LR(e){return"-"+e.toLowerCase()}const L1={action:["form"],cite:["blockquote","del","ins","q"],data:["object"],formAction:["button","input"],href:["a","area","base","link"],icon:["menuitem"],itemId:null,manifest:["html"],ping:["a","area"],poster:["video"],src:["audio","embed","iframe","img","input","script","source","track","video"]},PR={};function g2(e,t){const n=PR,r=typeof n.includeImageAlt=="boolean"?n.includeImageAlt:!0,a=typeof n.includeHtml=="boolean"?n.includeHtml:!0;return i7(e,r,a)}function i7(e,t,n){if(BR(e)){if("value"in e)return e.type==="html"&&!n?"":e.value;if(t&&"alt"in e&&e.alt)return e.alt;if("children"in e)return H_(e.children,t,n)}return Array.isArray(e)?H_(e,t,n):""}function H_(e,t,n){const r=[];let a=-1;for(;++a<e.length;)r[a]=i7(e[a],t,n);return r.join("")}function BR(e){return!!(e&&typeof e=="object")}const q_=document.createElement("i");function b2(e){const t="&"+e+";";q_.innerHTML=t;const n=q_.textContent;return n.charCodeAt(n.length-1)===59&&e!=="semi"||n===t?!1:n}function _a(e,t,n,r){const a=e.length;let s=0,o;if(t<0?t=-t>a?0:a+t:t=t>a?a:t,n=n>0?n:0,r.length<1e4)o=Array.from(r),o.unshift(t,n),e.splice(...o);else for(n&&e.splice(t,n);s<r.length;)o=r.slice(s,s+1e4),o.unshift(t,0),e.splice(...o),s+=1e4,t+=1e4}function Ra(e,t){return e.length>0?(_a(e,e.length,0,t),e):t}const j_={}.hasOwnProperty;function s7(e){const t={};let n=-1;for(;++n<e.length;)UR(t,e[n]);return t}function UR(e,t){let n;for(n in t){const a=(j_.call(e,n)?e[n]:void 0)||(e[n]={}),s=t[n];let o;if(s)for(o in s){j_.call(a,o)||(a[o]=[]);const c=s[o];FR(a[o],Array.isArray(c)?c:c?[c]:[])}}}function FR(e,t){let n=-1;const r=[];for(;++n<t.length;)(t[n].add==="after"?e:r).push(t[n]);_a(e,0,0,r)}function l7(e,t){const n=Number.parseInt(e,t);return n<9||n===11||n>13&&n<32||n>126&&n<160||n>55295&&n<57344||n>64975&&n<65008||(n&65535)===65535||(n&65535)===65534||n>1114111?"�":String.fromCodePoint(n)}function Qa(e){return e.replace(/[\t\n\r ]+/g," ").replace(/^ | $/g,"").toLowerCase().toUpperCase()}const zr=ol(/[A-Za-z]/),kr=ol(/[\dA-Za-z]/),zR=ol(/[#-'*+\--9=?A-Z^-~]/);function ef(e){return e!==null&&(e<32||e===127)}const tb=ol(/\d/),HR=ol(/[\dA-Fa-f]/),qR=ol(/[!-/:-@[-`{-~]/);function nt(e){return e!==null&&e<-2}function an(e){return e!==null&&(e<0||e===32)}function Dt(e){return e===-2||e===-1||e===32}const Yf=ol(new RegExp("\\p{P}|\\p{S}","u")),Yl=ol(/\s/);function ol(e){return t;function t(n){return n!==null&&n>-1&&e.test(String.fromCharCode(n))}}function fu(e){const t=[];let n=-1,r=0,a=0;for(;++n<e.length;){const s=e.charCodeAt(n);let o="";if(s===37&&kr(e.charCodeAt(n+1))&&kr(e.charCodeAt(n+2)))a=2;else if(s<128)/[!#$&-;=?-Z_a-z~]/.test(String.fromCharCode(s))||(o=String.fromCharCode(s));else if(s>55295&&s<57344){const c=e.charCodeAt(n+1);s<56320&&c>56319&&c<57344?(o=String.fromCharCode(s,c),a=1):o="�"}else o=String.fromCharCode(s);o&&(t.push(e.slice(r,n),encodeURIComponent(o)),r=n+a+1,o=""),a&&(n+=a,a=0)}return t.join("")+e.slice(r)}function Nt(e,t,n,r){const a=r?r-1:Number.POSITIVE_INFINITY;let s=0;return o;function o(d){return Dt(d)?(e.enter(n),c(d)):t(d)}function c(d){return Dt(d)&&s++<a?(e.consume(d),c):(e.exit(n),t(d))}}const jR={tokenize:$R};function $R(e){const t=e.attempt(this.parser.constructs.contentInitial,r,a);let n;return t;function r(c){if(c===null){e.consume(c);return}return e.enter("lineEnding"),e.consume(c),e.exit("lineEnding"),Nt(e,t,"linePrefix")}function a(c){return e.enter("paragraph"),s(c)}function s(c){const d=e.enter("chunkText",{contentType:"text",previous:n});return n&&(n.next=d),n=d,o(c)}function o(c){if(c===null){e.exit("chunkText"),e.exit("paragraph"),e.consume(c);return}return nt(c)?(e.consume(c),e.exit("chunkText"),s):(e.consume(c),o)}}const YR={tokenize:GR},$_={tokenize:VR};function GR(e){const t=this,n=[];let r=0,a,s,o;return c;function c(D){if(r<n.length){const Y=n[r];return t.containerState=Y[1],e.attempt(Y[0].continuation,d,f)(D)}return f(D)}function d(D){if(r++,t.containerState._closeFlow){t.containerState._closeFlow=void 0,a&&O();const Y=t.events.length;let H=Y,B;for(;H--;)if(t.events[H][0]==="exit"&&t.events[H][1].type==="chunkFlow"){B=t.events[H][1].end;break}w(r);let ee=Y;for(;ee<t.events.length;)t.events[ee][1].end={...B},ee++;return _a(t.events,H+1,0,t.events.slice(Y)),t.events.length=ee,f(D)}return c(D)}function f(D){if(r===n.length){if(!a)return b(D);if(a.currentConstruct&&a.currentConstruct.concrete)return T(D);t.interrupt=!!(a.currentConstruct&&!a._gfmTableDynamicInterruptHack)}return t.containerState={},e.check($_,m,p)(D)}function m(D){return a&&O(),w(r),b(D)}function p(D){return t.parser.lazy[t.now().line]=r!==n.length,o=t.now().offset,T(D)}function b(D){return t.containerState={},e.attempt($_,y,T)(D)}function y(D){return r++,n.push([t.currentConstruct,t.containerState]),b(D)}function T(D){if(D===null){a&&O(),w(0),e.consume(D);return}return a=a||t.parser.flow(t.now()),e.enter("chunkFlow",{_tokenizer:a,contentType:"flow",previous:s}),x(D)}function x(D){if(D===null){C(e.exit("chunkFlow"),!0),w(0),e.consume(D);return}return nt(D)?(e.consume(D),C(e.exit("chunkFlow")),r=0,t.interrupt=void 0,c):(e.consume(D),x)}function C(D,Y){const H=t.sliceStream(D);if(Y&&H.push(null),D.previous=s,s&&(s.next=D),s=D,a.defineSkip(D.start),a.write(H),t.parser.lazy[D.start.line]){let B=a.events.length;for(;B--;)if(a.events[B][1].start.offset<o&&(!a.events[B][1].end||a.events[B][1].end.offset>o))return;const ee=t.events.length;let Z=ee,Q,G;for(;Z--;)if(t.events[Z][0]==="exit"&&t.events[Z][1].type==="chunkFlow"){if(Q){G=t.events[Z][1].end;break}Q=!0}for(w(r),B=ee;B<t.events.length;)t.events[B][1].end={...G},B++;_a(t.events,Z+1,0,t.events.slice(ee)),t.events.length=B}}function w(D){let Y=n.length;for(;Y-- >D;){const H=n[Y];t.containerState=H[1],H[0].exit.call(t,e)}n.length=D}function O(){a.write([null]),s=void 0,a=void 0,t.containerState._closeFlow=void 0}}function VR(e,t,n){return Nt(e,e.attempt(this.parser.constructs.document,t,n),"linePrefix",this.parser.constructs.disable.null.includes("codeIndented")?void 0:4)}function eu(e){if(e===null||an(e)||Yl(e))return 1;if(Yf(e))return 2}function Gf(e,t,n){const r=[];let a=-1;for(;++a<e.length;){const s=e[a].resolveAll;s&&!r.includes(s)&&(t=s(t,n),r.push(s))}return t}const nb={name:"attention",resolveAll:KR,tokenize:WR};function KR(e,t){let n=-1,r,a,s,o,c,d,f,m;for(;++n<e.length;)if(e[n][0]==="enter"&&e[n][1].type==="attentionSequence"&&e[n][1]._close){for(r=n;r--;)if(e[r][0]==="exit"&&e[r][1].type==="attentionSequence"&&e[r][1]._open&&t.sliceSerialize(e[r][1]).charCodeAt(0)===t.sliceSerialize(e[n][1]).charCodeAt(0)){if((e[r][1]._close||e[n][1]._open)&&(e[n][1].end.offset-e[n][1].start.offset)%3&&!((e[r][1].end.offset-e[r][1].start.offset+e[n][1].end.offset-e[n][1].start.offset)%3))continue;d=e[r][1].end.offset-e[r][1].start.offset>1&&e[n][1].end.offset-e[n][1].start.offset>1?2:1;const p={...e[r][1].end},b={...e[n][1].start};Y_(p,-d),Y_(b,d),o={type:d>1?"strongSequence":"emphasisSequence",start:p,end:{...e[r][1].end}},c={type:d>1?"strongSequence":"emphasisSequence",start:{...e[n][1].start},end:b},s={type:d>1?"strongText":"emphasisText",start:{...e[r][1].end},end:{...e[n][1].start}},a={type:d>1?"strong":"emphasis",start:{...o.start},end:{...c.end}},e[r][1].end={...o.start},e[n][1].start={...c.end},f=[],e[r][1].end.offset-e[r][1].start.offset&&(f=Ra(f,[["enter",e[r][1],t],["exit",e[r][1],t]])),f=Ra(f,[["enter",a,t],["enter",o,t],["exit",o,t],["enter",s,t]]),f=Ra(f,Gf(t.parser.constructs.insideSpan.null,e.slice(r+1,n),t)),f=Ra(f,[["exit",s,t],["enter",c,t],["exit",c,t],["exit",a,t]]),e[n][1].end.offset-e[n][1].start.offset?(m=2,f=Ra(f,[["enter",e[n][1],t],["exit",e[n][1],t]])):m=0,_a(e,r-1,n-r+3,f),n=r+f.length-m-2;break}}for(n=-1;++n<e.length;)e[n][1].type==="attentionSequence"&&(e[n][1].type="data");return e}function WR(e,t){const n=this.parser.constructs.attentionMarkers.null,r=this.previous,a=eu(r);let s;return o;function o(d){return s=d,e.enter("attentionSequence"),c(d)}function c(d){if(d===s)return e.consume(d),c;const f=e.exit("attentionSequence"),m=eu(d),p=!m||m===2&&a||n.includes(d),b=!a||a===2&&m||n.includes(r);return f._open=!!(s===42?p:p&&(a||!b)),f._close=!!(s===42?b:b&&(m||!p)),t(d)}}function Y_(e,t){e.column+=t,e.offset+=t,e._bufferIndex+=t}const XR={name:"autolink",tokenize:QR};function QR(e,t,n){let r=0;return a;function a(y){return e.enter("autolink"),e.enter("autolinkMarker"),e.consume(y),e.exit("autolinkMarker"),e.enter("autolinkProtocol"),s}function s(y){return zr(y)?(e.consume(y),o):y===64?n(y):f(y)}function o(y){return y===43||y===45||y===46||kr(y)?(r=1,c(y)):f(y)}function c(y){return y===58?(e.consume(y),r=0,d):(y===43||y===45||y===46||kr(y))&&r++<32?(e.consume(y),c):(r=0,f(y))}function d(y){return y===62?(e.exit("autolinkProtocol"),e.enter("autolinkMarker"),e.consume(y),e.exit("autolinkMarker"),e.exit("autolink"),t):y===null||y===32||y===60||ef(y)?n(y):(e.consume(y),d)}function f(y){return y===64?(e.consume(y),m):zR(y)?(e.consume(y),f):n(y)}function m(y){return kr(y)?p(y):n(y)}function p(y){return y===46?(e.consume(y),r=0,m):y===62?(e.exit("autolinkProtocol").type="autolinkEmail",e.enter("autolinkMarker"),e.consume(y),e.exit("autolinkMarker"),e.exit("autolink"),t):b(y)}function b(y){if((y===45||kr(y))&&r++<63){const T=y===45?b:p;return e.consume(y),T}return n(y)}}const N0={partial:!0,tokenize:ZR};function ZR(e,t,n){return r;function r(s){return Dt(s)?Nt(e,a,"linePrefix")(s):a(s)}function a(s){return s===null||nt(s)?t(s):n(s)}}const o7={continuation:{tokenize:eM},exit:tM,name:"blockQuote",tokenize:JR};function JR(e,t,n){const r=this;return a;function a(o){if(o===62){const c=r.containerState;return c.open||(e.enter("blockQuote",{_container:!0}),c.open=!0),e.enter("blockQuotePrefix"),e.enter("blockQuoteMarker"),e.consume(o),e.exit("blockQuoteMarker"),s}return n(o)}function s(o){return Dt(o)?(e.enter("blockQuotePrefixWhitespace"),e.consume(o),e.exit("blockQuotePrefixWhitespace"),e.exit("blockQuotePrefix"),t):(e.exit("blockQuotePrefix"),t(o))}}function eM(e,t,n){const r=this;return a;function a(o){return Dt(o)?Nt(e,s,"linePrefix",r.parser.constructs.disable.null.includes("codeIndented")?void 0:4)(o):s(o)}function s(o){return e.attempt(o7,t,n)(o)}}function tM(e){e.exit("blockQuote")}const u7={name:"characterEscape",tokenize:nM};function nM(e,t,n){return r;function r(s){return e.enter("characterEscape"),e.enter("escapeMarker"),e.consume(s),e.exit("escapeMarker"),a}function a(s){return qR(s)?(e.enter("characterEscapeValue"),e.consume(s),e.exit("characterEscapeValue"),e.exit("characterEscape"),t):n(s)}}const c7={name:"characterReference",tokenize:rM};function rM(e,t,n){const r=this;let a=0,s,o;return c;function c(p){return e.enter("characterReference"),e.enter("characterReferenceMarker"),e.consume(p),e.exit("characterReferenceMarker"),d}function d(p){return p===35?(e.enter("characterReferenceMarkerNumeric"),e.consume(p),e.exit("characterReferenceMarkerNumeric"),f):(e.enter("characterReferenceValue"),s=31,o=kr,m(p))}function f(p){return p===88||p===120?(e.enter("characterReferenceMarkerHexadecimal"),e.consume(p),e.exit("characterReferenceMarkerHexadecimal"),e.enter("characterReferenceValue"),s=6,o=HR,m):(e.enter("characterReferenceValue"),s=7,o=tb,m(p))}function m(p){if(p===59&&a){const b=e.exit("characterReferenceValue");return o===kr&&!b2(r.sliceSerialize(b))?n(p):(e.enter("characterReferenceMarker"),e.consume(p),e.exit("characterReferenceMarker"),e.exit("characterReference"),t)}return o(p)&&a++<s?(e.consume(p),m):n(p)}}const G_={partial:!0,tokenize:iM},V_={concrete:!0,name:"codeFenced",tokenize:aM};function aM(e,t,n){const r=this,a={partial:!0,tokenize:H};let s=0,o=0,c;return d;function d(B){return f(B)}function f(B){const ee=r.events[r.events.length-1];return s=ee&&ee[1].type==="linePrefix"?ee[2].sliceSerialize(ee[1],!0).length:0,c=B,e.enter("codeFenced"),e.enter("codeFencedFence"),e.enter("codeFencedFenceSequence"),m(B)}function m(B){return B===c?(o++,e.consume(B),m):o<3?n(B):(e.exit("codeFencedFenceSequence"),Dt(B)?Nt(e,p,"whitespace")(B):p(B))}function p(B){return B===null||nt(B)?(e.exit("codeFencedFence"),r.interrupt?t(B):e.check(G_,x,Y)(B)):(e.enter("codeFencedFenceInfo"),e.enter("chunkString",{contentType:"string"}),b(B))}function b(B){return B===null||nt(B)?(e.exit("chunkString"),e.exit("codeFencedFenceInfo"),p(B)):Dt(B)?(e.exit("chunkString"),e.exit("codeFencedFenceInfo"),Nt(e,y,"whitespace")(B)):B===96&&B===c?n(B):(e.consume(B),b)}function y(B){return B===null||nt(B)?p(B):(e.enter("codeFencedFenceMeta"),e.enter("chunkString",{contentType:"string"}),T(B))}function T(B){return B===null||nt(B)?(e.exit("chunkString"),e.exit("codeFencedFenceMeta"),p(B)):B===96&&B===c?n(B):(e.consume(B),T)}function x(B){return e.attempt(a,Y,C)(B)}function C(B){return e.enter("lineEnding"),e.consume(B),e.exit("lineEnding"),w}function w(B){return s>0&&Dt(B)?Nt(e,O,"linePrefix",s+1)(B):O(B)}function O(B){return B===null||nt(B)?e.check(G_,x,Y)(B):(e.enter("codeFlowValue"),D(B))}function D(B){return B===null||nt(B)?(e.exit("codeFlowValue"),O(B)):(e.consume(B),D)}function Y(B){return e.exit("codeFenced"),t(B)}function H(B,ee,Z){let Q=0;return G;function G(de){return B.enter("lineEnding"),B.consume(de),B.exit("lineEnding"),se}function se(de){return B.enter("codeFencedFence"),Dt(de)?Nt(B,oe,"linePrefix",r.parser.constructs.disable.null.includes("codeIndented")?void 0:4)(de):oe(de)}function oe(de){return de===c?(B.enter("codeFencedFenceSequence"),ye(de)):Z(de)}function ye(de){return de===c?(Q++,B.consume(de),ye):Q>=o?(B.exit("codeFencedFenceSequence"),Dt(de)?Nt(B,fe,"whitespace")(de):fe(de)):Z(de)}function fe(de){return de===null||nt(de)?(B.exit("codeFencedFence"),ee(de)):Z(de)}}}function iM(e,t,n){const r=this;return a;function a(o){return o===null?n(o):(e.enter("lineEnding"),e.consume(o),e.exit("lineEnding"),s)}function s(o){return r.parser.lazy[r.now().line]?n(o):t(o)}}const P1={name:"codeIndented",tokenize:lM},sM={partial:!0,tokenize:oM};function lM(e,t,n){const r=this;return a;function a(f){return e.enter("codeIndented"),Nt(e,s,"linePrefix",5)(f)}function s(f){const m=r.events[r.events.length-1];return m&&m[1].type==="linePrefix"&&m[2].sliceSerialize(m[1],!0).length>=4?o(f):n(f)}function o(f){return f===null?d(f):nt(f)?e.attempt(sM,o,d)(f):(e.enter("codeFlowValue"),c(f))}function c(f){return f===null||nt(f)?(e.exit("codeFlowValue"),o(f)):(e.consume(f),c)}function d(f){return e.exit("codeIndented"),t(f)}}function oM(e,t,n){const r=this;return a;function a(o){return r.parser.lazy[r.now().line]?n(o):nt(o)?(e.enter("lineEnding"),e.consume(o),e.exit("lineEnding"),a):Nt(e,s,"linePrefix",5)(o)}function s(o){const c=r.events[r.events.length-1];return c&&c[1].type==="linePrefix"&&c[2].sliceSerialize(c[1],!0).length>=4?t(o):nt(o)?a(o):n(o)}}const uM={name:"codeText",previous:dM,resolve:cM,tokenize:hM};function cM(e){let t=e.length-4,n=3,r,a;if((e[n][1].type==="lineEnding"||e[n][1].type==="space")&&(e[t][1].type==="lineEnding"||e[t][1].type==="space")){for(r=n;++r<t;)if(e[r][1].type==="codeTextData"){e[n][1].type="codeTextPadding",e[t][1].type="codeTextPadding",n+=2,t-=2;break}}for(r=n-1,t++;++r<=t;)a===void 0?r!==t&&e[r][1].type!=="lineEnding"&&(a=r):(r===t||e[r][1].type==="lineEnding")&&(e[a][1].type="codeTextData",r!==a+2&&(e[a][1].end=e[r-1][1].end,e.splice(a+2,r-a-2),t-=r-a-2,r=a+2),a=void 0);return e}function dM(e){return e!==96||this.events[this.events.length-1][1].type==="characterEscape"}function hM(e,t,n){let r=0,a,s;return o;function o(p){return e.enter("codeText"),e.enter("codeTextSequence"),c(p)}function c(p){return p===96?(e.consume(p),r++,c):(e.exit("codeTextSequence"),d(p))}function d(p){return p===null?n(p):p===32?(e.enter("space"),e.consume(p),e.exit("space"),d):p===96?(s=e.enter("codeTextSequence"),a=0,m(p)):nt(p)?(e.enter("lineEnding"),e.consume(p),e.exit("lineEnding"),d):(e.enter("codeTextData"),f(p))}function f(p){return p===null||p===32||p===96||nt(p)?(e.exit("codeTextData"),d(p)):(e.consume(p),f)}function m(p){return p===96?(e.consume(p),a++,m):a===r?(e.exit("codeTextSequence"),e.exit("codeText"),t(p)):(s.type="codeTextData",f(p))}}class fM{constructor(t){this.left=t?[...t]:[],this.right=[]}get(t){if(t<0||t>=this.left.length+this.right.length)throw new RangeError("Cannot access index `"+t+"` in a splice buffer of size `"+(this.left.length+this.right.length)+"`");return t<this.left.length?this.left[t]:this.right[this.right.length-t+this.left.length-1]}get length(){return this.left.length+this.right.length}shift(){return this.setCursor(0),this.right.pop()}slice(t,n){const r=n??Number.POSITIVE_INFINITY;return r<this.left.length?this.left.slice(t,r):t>this.left.length?this.right.slice(this.right.length-r+this.left.length,this.right.length-t+this.left.length).reverse():this.left.slice(t).concat(this.right.slice(this.right.length-r+this.left.length).reverse())}splice(t,n,r){const a=n||0;this.setCursor(Math.trunc(t));const s=this.right.splice(this.right.length-a,Number.POSITIVE_INFINITY);return r&&Ac(this.left,r),s.reverse()}pop(){return this.setCursor(Number.POSITIVE_INFINITY),this.left.pop()}push(t){this.setCursor(Number.POSITIVE_INFINITY),this.left.push(t)}pushMany(t){this.setCursor(Number.POSITIVE_INFINITY),Ac(this.left,t)}unshift(t){this.setCursor(0),this.right.push(t)}unshiftMany(t){this.setCursor(0),Ac(this.right,t.reverse())}setCursor(t){if(!(t===this.left.length||t>this.left.length&&this.right.length===0||t<0&&this.left.length===0))if(t<this.left.length){const n=this.left.splice(t,Number.POSITIVE_INFINITY);Ac(this.right,n.reverse())}else{const n=this.right.splice(this.left.length+this.right.length-t,Number.POSITIVE_INFINITY);Ac(this.left,n.reverse())}}}function Ac(e,t){let n=0;if(t.length<1e4)e.push(...t);else for(;n<t.length;)e.push(...t.slice(n,n+1e4)),n+=1e4}function d7(e){const t={};let n=-1,r,a,s,o,c,d,f;const m=new fM(e);for(;++n<m.length;){for(;n in t;)n=t[n];if(r=m.get(n),n&&r[1].type==="chunkFlow"&&m.get(n-1)[1].type==="listItemPrefix"&&(d=r[1]._tokenizer.events,s=0,s<d.length&&d[s][1].type==="lineEndingBlank"&&(s+=2),s<d.length&&d[s][1].type==="content"))for(;++s<d.length&&d[s][1].type!=="content";)d[s][1].type==="chunkText"&&(d[s][1]._isInFirstContentOfListItem=!0,s++);if(r[0]==="enter")r[1].contentType&&(Object.assign(t,mM(m,n)),n=t[n],f=!0);else if(r[1]._container){for(s=n,a=void 0;s--;)if(o=m.get(s),o[1].type==="lineEnding"||o[1].type==="lineEndingBlank")o[0]==="enter"&&(a&&(m.get(a)[1].type="lineEndingBlank"),o[1].type="lineEnding",a=s);else if(!(o[1].type==="linePrefix"||o[1].type==="listItemIndent"))break;a&&(r[1].end={...m.get(a)[1].start},c=m.slice(a,n),c.unshift(r),m.splice(a,n-a+1,c))}}return _a(e,0,Number.POSITIVE_INFINITY,m.slice(0)),!f}function mM(e,t){const n=e.get(t)[1],r=e.get(t)[2];let a=t-1;const s=[];let o=n._tokenizer;o||(o=r.parser[n.contentType](n.start),n._contentTypeTextTrailing&&(o._contentTypeTextTrailing=!0));const c=o.events,d=[],f={};let m,p,b=-1,y=n,T=0,x=0;const C=[x];for(;y;){for(;e.get(++a)[1]!==y;);s.push(a),y._tokenizer||(m=r.sliceStream(y),y.next||m.push(null),p&&o.defineSkip(y.start),y._isInFirstContentOfListItem&&(o._gfmTasklistFirstContentOfListItem=!0),o.write(m),y._isInFirstContentOfListItem&&(o._gfmTasklistFirstContentOfListItem=void 0)),p=y,y=y.next}for(y=n;++b<c.length;)c[b][0]==="exit"&&c[b-1][0]==="enter"&&c[b][1].type===c[b-1][1].type&&c[b][1].start.line!==c[b][1].end.line&&(x=b+1,C.push(x),y._tokenizer=void 0,y.previous=void 0,y=y.next);for(o.events=[],y?(y._tokenizer=void 0,y.previous=void 0):C.pop(),b=C.length;b--;){const w=c.slice(C[b],C[b+1]),O=s.pop();d.push([O,O+w.length-1]),e.splice(O,2,w)}for(d.reverse(),b=-1;++b<d.length;)f[T+d[b][0]]=T+d[b][1],T+=d[b][1]-d[b][0]-1;return f}const pM={resolve:bM,tokenize:vM},gM={partial:!0,tokenize:yM};function bM(e){return d7(e),e}function vM(e,t){let n;return r;function r(c){return e.enter("content"),n=e.enter("chunkContent",{contentType:"content"}),a(c)}function a(c){return c===null?s(c):nt(c)?e.check(gM,o,s)(c):(e.consume(c),a)}function s(c){return e.exit("chunkContent"),e.exit("content"),t(c)}function o(c){return e.consume(c),e.exit("chunkContent"),n.next=e.enter("chunkContent",{contentType:"content",previous:n}),n=n.next,a}}function yM(e,t,n){const r=this;return a;function a(o){return e.exit("chunkContent"),e.enter("lineEnding"),e.consume(o),e.exit("lineEnding"),Nt(e,s,"linePrefix")}function s(o){if(o===null||nt(o))return n(o);const c=r.events[r.events.length-1];return!r.parser.constructs.disable.null.includes("codeIndented")&&c&&c[1].type==="linePrefix"&&c[2].sliceSerialize(c[1],!0).length>=4?t(o):e.interrupt(r.parser.constructs.flow,n,t)(o)}}function h7(e,t,n,r,a,s,o,c,d){const f=d||Number.POSITIVE_INFINITY;let m=0;return p;function p(w){return w===60?(e.enter(r),e.enter(a),e.enter(s),e.consume(w),e.exit(s),b):w===null||w===32||w===41||ef(w)?n(w):(e.enter(r),e.enter(o),e.enter(c),e.enter("chunkString",{contentType:"string"}),x(w))}function b(w){return w===62?(e.enter(s),e.consume(w),e.exit(s),e.exit(a),e.exit(r),t):(e.enter(c),e.enter("chunkString",{contentType:"string"}),y(w))}function y(w){return w===62?(e.exit("chunkString"),e.exit(c),b(w)):w===null||w===60||nt(w)?n(w):(e.consume(w),w===92?T:y)}function T(w){return w===60||w===62||w===92?(e.consume(w),y):y(w)}function x(w){return!m&&(w===null||w===41||an(w))?(e.exit("chunkString"),e.exit(c),e.exit(o),e.exit(r),t(w)):m<f&&w===40?(e.consume(w),m++,x):w===41?(e.consume(w),m--,x):w===null||w===32||w===40||ef(w)?n(w):(e.consume(w),w===92?C:x)}function C(w){return w===40||w===41||w===92?(e.consume(w),x):x(w)}}function f7(e,t,n,r,a,s){const o=this;let c=0,d;return f;function f(y){return e.enter(r),e.enter(a),e.consume(y),e.exit(a),e.enter(s),m}function m(y){return c>999||y===null||y===91||y===93&&!d||y===94&&!c&&"_hiddenFootnoteSupport"in o.parser.constructs?n(y):y===93?(e.exit(s),e.enter(a),e.consume(y),e.exit(a),e.exit(r),t):nt(y)?(e.enter("lineEnding"),e.consume(y),e.exit("lineEnding"),m):(e.enter("chunkString",{contentType:"string"}),p(y))}function p(y){return y===null||y===91||y===93||nt(y)||c++>999?(e.exit("chunkString"),m(y)):(e.consume(y),d||(d=!Dt(y)),y===92?b:p)}function b(y){return y===91||y===92||y===93?(e.consume(y),c++,p):p(y)}}function m7(e,t,n,r,a,s){let o;return c;function c(b){return b===34||b===39||b===40?(e.enter(r),e.enter(a),e.consume(b),e.exit(a),o=b===40?41:b,d):n(b)}function d(b){return b===o?(e.enter(a),e.consume(b),e.exit(a),e.exit(r),t):(e.enter(s),f(b))}function f(b){return b===o?(e.exit(s),d(o)):b===null?n(b):nt(b)?(e.enter("lineEnding"),e.consume(b),e.exit("lineEnding"),Nt(e,f,"linePrefix")):(e.enter("chunkString",{contentType:"string"}),m(b))}function m(b){return b===o||b===null||nt(b)?(e.exit("chunkString"),f(b)):(e.consume(b),b===92?p:m)}function p(b){return b===o||b===92?(e.consume(b),m):m(b)}}function Jc(e,t){let n;return r;function r(a){return nt(a)?(e.enter("lineEnding"),e.consume(a),e.exit("lineEnding"),n=!0,r):Dt(a)?Nt(e,r,n?"linePrefix":"lineSuffix")(a):t(a)}}const _M={name:"definition",tokenize:TM},EM={partial:!0,tokenize:xM};function TM(e,t,n){const r=this;let a;return s;function s(y){return e.enter("definition"),o(y)}function o(y){return f7.call(r,e,c,n,"definitionLabel","definitionLabelMarker","definitionLabelString")(y)}function c(y){return a=Qa(r.sliceSerialize(r.events[r.events.length-1][1]).slice(1,-1)),y===58?(e.enter("definitionMarker"),e.consume(y),e.exit("definitionMarker"),d):n(y)}function d(y){return an(y)?Jc(e,f)(y):f(y)}function f(y){return h7(e,m,n,"definitionDestination","definitionDestinationLiteral","definitionDestinationLiteralMarker","definitionDestinationRaw","definitionDestinationString")(y)}function m(y){return e.attempt(EM,p,p)(y)}function p(y){return Dt(y)?Nt(e,b,"whitespace")(y):b(y)}function b(y){return y===null||nt(y)?(e.exit("definition"),r.parser.defined.push(a),t(y)):n(y)}}function xM(e,t,n){return r;function r(c){return an(c)?Jc(e,a)(c):n(c)}function a(c){return m7(e,s,n,"definitionTitle","definitionTitleMarker","definitionTitleString")(c)}function s(c){return Dt(c)?Nt(e,o,"whitespace")(c):o(c)}function o(c){return c===null||nt(c)?t(c):n(c)}}const SM={name:"hardBreakEscape",tokenize:wM};function wM(e,t,n){return r;function r(s){return e.enter("hardBreakEscape"),e.consume(s),a}function a(s){return nt(s)?(e.exit("hardBreakEscape"),t(s)):n(s)}}const AM={name:"headingAtx",resolve:kM,tokenize:CM};function kM(e,t){let n=e.length-2,r=3,a,s;return e[r][1].type==="whitespace"&&(r+=2),n-2>r&&e[n][1].type==="whitespace"&&(n-=2),e[n][1].type==="atxHeadingSequence"&&(r===n-1||n-4>r&&e[n-2][1].type==="whitespace")&&(n-=r+1===n?2:4),n>r&&(a={type:"atxHeadingText",start:e[r][1].start,end:e[n][1].end},s={type:"chunkText",start:e[r][1].start,end:e[n][1].end,contentType:"text"},_a(e,r,n-r+1,[["enter",a,t],["enter",s,t],["exit",s,t],["exit",a,t]])),e}function CM(e,t,n){let r=0;return a;function a(m){return e.enter("atxHeading"),s(m)}function s(m){return e.enter("atxHeadingSequence"),o(m)}function o(m){return m===35&&r++<6?(e.consume(m),o):m===null||an(m)?(e.exit("atxHeadingSequence"),c(m)):n(m)}function c(m){return m===35?(e.enter("atxHeadingSequence"),d(m)):m===null||nt(m)?(e.exit("atxHeading"),t(m)):Dt(m)?Nt(e,c,"whitespace")(m):(e.enter("atxHeadingText"),f(m))}function d(m){return m===35?(e.consume(m),d):(e.exit("atxHeadingSequence"),c(m))}function f(m){return m===null||m===35||an(m)?(e.exit("atxHeadingText"),c(m)):(e.consume(m),f)}}const NM=["address","article","aside","base","basefont","blockquote","body","caption","center","col","colgroup","dd","details","dialog","dir","div","dl","dt","fieldset","figcaption","figure","footer","form","frame","frameset","h1","h2","h3","h4","h5","h6","head","header","hr","html","iframe","legend","li","link","main","menu","menuitem","nav","noframes","ol","optgroup","option","p","param","search","section","summary","table","tbody","td","tfoot","th","thead","title","tr","track","ul"],K_=["pre","script","style","textarea"],OM={concrete:!0,name:"htmlFlow",resolveTo:IM,tokenize:DM},RM={partial:!0,tokenize:PM},MM={partial:!0,tokenize:LM};function IM(e){let t=e.length;for(;t--&&!(e[t][0]==="enter"&&e[t][1].type==="htmlFlow"););return t>1&&e[t-2][1].type==="linePrefix"&&(e[t][1].start=e[t-2][1].start,e[t+1][1].start=e[t-2][1].start,e.splice(t-2,2)),e}function DM(e,t,n){const r=this;let a,s,o,c,d;return f;function f(L){return m(L)}function m(L){return e.enter("htmlFlow"),e.enter("htmlFlowData"),e.consume(L),p}function p(L){return L===33?(e.consume(L),b):L===47?(e.consume(L),s=!0,x):L===63?(e.consume(L),a=3,r.interrupt?t:k):zr(L)?(e.consume(L),o=String.fromCharCode(L),C):n(L)}function b(L){return L===45?(e.consume(L),a=2,y):L===91?(e.consume(L),a=5,c=0,T):zr(L)?(e.consume(L),a=4,r.interrupt?t:k):n(L)}function y(L){return L===45?(e.consume(L),r.interrupt?t:k):n(L)}function T(L){const Oe="CDATA[";return L===Oe.charCodeAt(c++)?(e.consume(L),c===Oe.length?r.interrupt?t:oe:T):n(L)}function x(L){return zr(L)?(e.consume(L),o=String.fromCharCode(L),C):n(L)}function C(L){if(L===null||L===47||L===62||an(L)){const Oe=L===47,je=o.toLowerCase();return!Oe&&!s&&K_.includes(je)?(a=1,r.interrupt?t(L):oe(L)):NM.includes(o.toLowerCase())?(a=6,Oe?(e.consume(L),w):r.interrupt?t(L):oe(L)):(a=7,r.interrupt&&!r.parser.lazy[r.now().line]?n(L):s?O(L):D(L))}return L===45||kr(L)?(e.consume(L),o+=String.fromCharCode(L),C):n(L)}function w(L){return L===62?(e.consume(L),r.interrupt?t:oe):n(L)}function O(L){return Dt(L)?(e.consume(L),O):G(L)}function D(L){return L===47?(e.consume(L),G):L===58||L===95||zr(L)?(e.consume(L),Y):Dt(L)?(e.consume(L),D):G(L)}function Y(L){return L===45||L===46||L===58||L===95||kr(L)?(e.consume(L),Y):H(L)}function H(L){return L===61?(e.consume(L),B):Dt(L)?(e.consume(L),H):D(L)}function B(L){return L===null||L===60||L===61||L===62||L===96?n(L):L===34||L===39?(e.consume(L),d=L,ee):Dt(L)?(e.consume(L),B):Z(L)}function ee(L){return L===d?(e.consume(L),d=null,Q):L===null||nt(L)?n(L):(e.consume(L),ee)}function Z(L){return L===null||L===34||L===39||L===47||L===60||L===61||L===62||L===96||an(L)?H(L):(e.consume(L),Z)}function Q(L){return L===47||L===62||Dt(L)?D(L):n(L)}function G(L){return L===62?(e.consume(L),se):n(L)}function se(L){return L===null||nt(L)?oe(L):Dt(L)?(e.consume(L),se):n(L)}function oe(L){return L===45&&a===2?(e.consume(L),V):L===60&&a===1?(e.consume(L),me):L===62&&a===4?(e.consume(L),ie):L===63&&a===3?(e.consume(L),k):L===93&&a===5?(e.consume(L),Be):nt(L)&&(a===6||a===7)?(e.exit("htmlFlowData"),e.check(RM,ge,ye)(L)):L===null||nt(L)?(e.exit("htmlFlowData"),ye(L)):(e.consume(L),oe)}function ye(L){return e.check(MM,fe,ge)(L)}function fe(L){return e.enter("lineEnding"),e.consume(L),e.exit("lineEnding"),de}function de(L){return L===null||nt(L)?ye(L):(e.enter("htmlFlowData"),oe(L))}function V(L){return L===45?(e.consume(L),k):oe(L)}function me(L){return L===47?(e.consume(L),o="",pe):oe(L)}function pe(L){if(L===62){const Oe=o.toLowerCase();return K_.includes(Oe)?(e.consume(L),ie):oe(L)}return zr(L)&&o.length<8?(e.consume(L),o+=String.fromCharCode(L),pe):oe(L)}function Be(L){return L===93?(e.consume(L),k):oe(L)}function k(L){return L===62?(e.consume(L),ie):L===45&&a===2?(e.consume(L),k):oe(L)}function ie(L){return L===null||nt(L)?(e.exit("htmlFlowData"),ge(L)):(e.consume(L),ie)}function ge(L){return e.exit("htmlFlow"),t(L)}}function LM(e,t,n){const r=this;return a;function a(o){return nt(o)?(e.enter("lineEnding"),e.consume(o),e.exit("lineEnding"),s):n(o)}function s(o){return r.parser.lazy[r.now().line]?n(o):t(o)}}function PM(e,t,n){return r;function r(a){return e.enter("lineEnding"),e.consume(a),e.exit("lineEnding"),e.attempt(N0,t,n)}}const BM={name:"htmlText",tokenize:UM};function UM(e,t,n){const r=this;let a,s,o;return c;function c(k){return e.enter("htmlText"),e.enter("htmlTextData"),e.consume(k),d}function d(k){return k===33?(e.consume(k),f):k===47?(e.consume(k),H):k===63?(e.consume(k),D):zr(k)?(e.consume(k),Z):n(k)}function f(k){return k===45?(e.consume(k),m):k===91?(e.consume(k),s=0,T):zr(k)?(e.consume(k),O):n(k)}function m(k){return k===45?(e.consume(k),y):n(k)}function p(k){return k===null?n(k):k===45?(e.consume(k),b):nt(k)?(o=p,me(k)):(e.consume(k),p)}function b(k){return k===45?(e.consume(k),y):p(k)}function y(k){return k===62?V(k):k===45?b(k):p(k)}function T(k){const ie="CDATA[";return k===ie.charCodeAt(s++)?(e.consume(k),s===ie.length?x:T):n(k)}function x(k){return k===null?n(k):k===93?(e.consume(k),C):nt(k)?(o=x,me(k)):(e.consume(k),x)}function C(k){return k===93?(e.consume(k),w):x(k)}function w(k){return k===62?V(k):k===93?(e.consume(k),w):x(k)}function O(k){return k===null||k===62?V(k):nt(k)?(o=O,me(k)):(e.consume(k),O)}function D(k){return k===null?n(k):k===63?(e.consume(k),Y):nt(k)?(o=D,me(k)):(e.consume(k),D)}function Y(k){return k===62?V(k):D(k)}function H(k){return zr(k)?(e.consume(k),B):n(k)}function B(k){return k===45||kr(k)?(e.consume(k),B):ee(k)}function ee(k){return nt(k)?(o=ee,me(k)):Dt(k)?(e.consume(k),ee):V(k)}function Z(k){return k===45||kr(k)?(e.consume(k),Z):k===47||k===62||an(k)?Q(k):n(k)}function Q(k){return k===47?(e.consume(k),V):k===58||k===95||zr(k)?(e.consume(k),G):nt(k)?(o=Q,me(k)):Dt(k)?(e.consume(k),Q):V(k)}function G(k){return k===45||k===46||k===58||k===95||kr(k)?(e.consume(k),G):se(k)}function se(k){return k===61?(e.consume(k),oe):nt(k)?(o=se,me(k)):Dt(k)?(e.consume(k),se):Q(k)}function oe(k){return k===null||k===60||k===61||k===62||k===96?n(k):k===34||k===39?(e.consume(k),a=k,ye):nt(k)?(o=oe,me(k)):Dt(k)?(e.consume(k),oe):(e.consume(k),fe)}function ye(k){return k===a?(e.consume(k),a=void 0,de):k===null?n(k):nt(k)?(o=ye,me(k)):(e.consume(k),ye)}function fe(k){return k===null||k===34||k===39||k===60||k===61||k===96?n(k):k===47||k===62||an(k)?Q(k):(e.consume(k),fe)}function de(k){return k===47||k===62||an(k)?Q(k):n(k)}function V(k){return k===62?(e.consume(k),e.exit("htmlTextData"),e.exit("htmlText"),t):n(k)}function me(k){return e.exit("htmlTextData"),e.enter("lineEnding"),e.consume(k),e.exit("lineEnding"),pe}function pe(k){return Dt(k)?Nt(e,Be,"linePrefix",r.parser.constructs.disable.null.includes("codeIndented")?void 0:4)(k):Be(k)}function Be(k){return e.enter("htmlTextData"),o(k)}}const v2={name:"labelEnd",resolveAll:qM,resolveTo:jM,tokenize:$M},FM={tokenize:YM},zM={tokenize:GM},HM={tokenize:VM};function qM(e){let t=-1;const n=[];for(;++t<e.length;){const r=e[t][1];if(n.push(e[t]),r.type==="labelImage"||r.type==="labelLink"||r.type==="labelEnd"){const a=r.type==="labelImage"?4:2;r.type="data",t+=a}}return e.length!==n.length&&_a(e,0,e.length,n),e}function jM(e,t){let n=e.length,r=0,a,s,o,c;for(;n--;)if(a=e[n][1],s){if(a.type==="link"||a.type==="labelLink"&&a._inactive)break;e[n][0]==="enter"&&a.type==="labelLink"&&(a._inactive=!0)}else if(o){if(e[n][0]==="enter"&&(a.type==="labelImage"||a.type==="labelLink")&&!a._balanced&&(s=n,a.type!=="labelLink")){r=2;break}}else a.type==="labelEnd"&&(o=n);const d={type:e[s][1].type==="labelLink"?"link":"image",start:{...e[s][1].start},end:{...e[e.length-1][1].end}},f={type:"label",start:{...e[s][1].start},end:{...e[o][1].end}},m={type:"labelText",start:{...e[s+r+2][1].end},end:{...e[o-2][1].start}};return c=[["enter",d,t],["enter",f,t]],c=Ra(c,e.slice(s+1,s+r+3)),c=Ra(c,[["enter",m,t]]),c=Ra(c,Gf(t.parser.constructs.insideSpan.null,e.slice(s+r+4,o-3),t)),c=Ra(c,[["exit",m,t],e[o-2],e[o-1],["exit",f,t]]),c=Ra(c,e.slice(o+1)),c=Ra(c,[["exit",d,t]]),_a(e,s,e.length,c),e}function $M(e,t,n){const r=this;let a=r.events.length,s,o;for(;a--;)if((r.events[a][1].type==="labelImage"||r.events[a][1].type==="labelLink")&&!r.events[a][1]._balanced){s=r.events[a][1];break}return c;function c(b){return s?s._inactive?p(b):(o=r.parser.defined.includes(Qa(r.sliceSerialize({start:s.end,end:r.now()}))),e.enter("labelEnd"),e.enter("labelMarker"),e.consume(b),e.exit("labelMarker"),e.exit("labelEnd"),d):n(b)}function d(b){return b===40?e.attempt(FM,m,o?m:p)(b):b===91?e.attempt(zM,m,o?f:p)(b):o?m(b):p(b)}function f(b){return e.attempt(HM,m,p)(b)}function m(b){return t(b)}function p(b){return s._balanced=!0,n(b)}}function YM(e,t,n){return r;function r(p){return e.enter("resource"),e.enter("resourceMarker"),e.consume(p),e.exit("resourceMarker"),a}function a(p){return an(p)?Jc(e,s)(p):s(p)}function s(p){return p===41?m(p):h7(e,o,c,"resourceDestination","resourceDestinationLiteral","resourceDestinationLiteralMarker","resourceDestinationRaw","resourceDestinationString",32)(p)}function o(p){return an(p)?Jc(e,d)(p):m(p)}function c(p){return n(p)}function d(p){return p===34||p===39||p===40?m7(e,f,n,"resourceTitle","resourceTitleMarker","resourceTitleString")(p):m(p)}function f(p){return an(p)?Jc(e,m)(p):m(p)}function m(p){return p===41?(e.enter("resourceMarker"),e.consume(p),e.exit("resourceMarker"),e.exit("resource"),t):n(p)}}function GM(e,t,n){const r=this;return a;function a(c){return f7.call(r,e,s,o,"reference","referenceMarker","referenceString")(c)}function s(c){return r.parser.defined.includes(Qa(r.sliceSerialize(r.events[r.events.length-1][1]).slice(1,-1)))?t(c):n(c)}function o(c){return n(c)}}function VM(e,t,n){return r;function r(s){return e.enter("reference"),e.enter("referenceMarker"),e.consume(s),e.exit("referenceMarker"),a}function a(s){return s===93?(e.enter("referenceMarker"),e.consume(s),e.exit("referenceMarker"),e.exit("reference"),t):n(s)}}const KM={name:"labelStartImage",resolveAll:v2.resolveAll,tokenize:WM};function WM(e,t,n){const r=this;return a;function a(c){return e.enter("labelImage"),e.enter("labelImageMarker"),e.consume(c),e.exit("labelImageMarker"),s}function s(c){return c===91?(e.enter("labelMarker"),e.consume(c),e.exit("labelMarker"),e.exit("labelImage"),o):n(c)}function o(c){return c===94&&"_hiddenFootnoteSupport"in r.parser.constructs?n(c):t(c)}}const XM={name:"labelStartLink",resolveAll:v2.resolveAll,tokenize:QM};function QM(e,t,n){const r=this;return a;function a(o){return e.enter("labelLink"),e.enter("labelMarker"),e.consume(o),e.exit("labelMarker"),e.exit("labelLink"),s}function s(o){return o===94&&"_hiddenFootnoteSupport"in r.parser.constructs?n(o):t(o)}}const B1={name:"lineEnding",tokenize:ZM};function ZM(e,t){return n;function n(r){return e.enter("lineEnding"),e.consume(r),e.exit("lineEnding"),Nt(e,t,"linePrefix")}}const Hh={name:"thematicBreak",tokenize:JM};function JM(e,t,n){let r=0,a;return s;function s(f){return e.enter("thematicBreak"),o(f)}function o(f){return a=f,c(f)}function c(f){return f===a?(e.enter("thematicBreakSequence"),d(f)):r>=3&&(f===null||nt(f))?(e.exit("thematicBreak"),t(f)):n(f)}function d(f){return f===a?(e.consume(f),r++,d):(e.exit("thematicBreakSequence"),Dt(f)?Nt(e,c,"whitespace")(f):c(f))}}const Zr={continuation:{tokenize:rI},exit:iI,name:"list",tokenize:nI},eI={partial:!0,tokenize:sI},tI={partial:!0,tokenize:aI};function nI(e,t,n){const r=this,a=r.events[r.events.length-1];let s=a&&a[1].type==="linePrefix"?a[2].sliceSerialize(a[1],!0).length:0,o=0;return c;function c(y){const T=r.containerState.type||(y===42||y===43||y===45?"listUnordered":"listOrdered");if(T==="listUnordered"?!r.containerState.marker||y===r.containerState.marker:tb(y)){if(r.containerState.type||(r.containerState.type=T,e.enter(T,{_container:!0})),T==="listUnordered")return e.enter("listItemPrefix"),y===42||y===45?e.check(Hh,n,f)(y):f(y);if(!r.interrupt||y===49)return e.enter("listItemPrefix"),e.enter("listItemValue"),d(y)}return n(y)}function d(y){return tb(y)&&++o<10?(e.consume(y),d):(!r.interrupt||o<2)&&(r.containerState.marker?y===r.containerState.marker:y===41||y===46)?(e.exit("listItemValue"),f(y)):n(y)}function f(y){return e.enter("listItemMarker"),e.consume(y),e.exit("listItemMarker"),r.containerState.marker=r.containerState.marker||y,e.check(N0,r.interrupt?n:m,e.attempt(eI,b,p))}function m(y){return r.containerState.initialBlankLine=!0,s++,b(y)}function p(y){return Dt(y)?(e.enter("listItemPrefixWhitespace"),e.consume(y),e.exit("listItemPrefixWhitespace"),b):n(y)}function b(y){return r.containerState.size=s+r.sliceSerialize(e.exit("listItemPrefix"),!0).length,t(y)}}function rI(e,t,n){const r=this;return r.containerState._closeFlow=void 0,e.check(N0,a,s);function a(c){return r.containerState.furtherBlankLines=r.containerState.furtherBlankLines||r.containerState.initialBlankLine,Nt(e,t,"listItemIndent",r.containerState.size+1)(c)}function s(c){return r.containerState.furtherBlankLines||!Dt(c)?(r.containerState.furtherBlankLines=void 0,r.containerState.initialBlankLine=void 0,o(c)):(r.containerState.furtherBlankLines=void 0,r.containerState.initialBlankLine=void 0,e.attempt(tI,t,o)(c))}function o(c){return r.containerState._closeFlow=!0,r.interrupt=void 0,Nt(e,e.attempt(Zr,t,n),"linePrefix",r.parser.constructs.disable.null.includes("codeIndented")?void 0:4)(c)}}function aI(e,t,n){const r=this;return Nt(e,a,"listItemIndent",r.containerState.size+1);function a(s){const o=r.events[r.events.length-1];return o&&o[1].type==="listItemIndent"&&o[2].sliceSerialize(o[1],!0).length===r.containerState.size?t(s):n(s)}}function iI(e){e.exit(this.containerState.type)}function sI(e,t,n){const r=this;return Nt(e,a,"listItemPrefixWhitespace",r.parser.constructs.disable.null.includes("codeIndented")?void 0:5);function a(s){const o=r.events[r.events.length-1];return!Dt(s)&&o&&o[1].type==="listItemPrefixWhitespace"?t(s):n(s)}}const W_={name:"setextUnderline",resolveTo:lI,tokenize:oI};function lI(e,t){let n=e.length,r,a,s;for(;n--;)if(e[n][0]==="enter"){if(e[n][1].type==="content"){r=n;break}e[n][1].type==="paragraph"&&(a=n)}else e[n][1].type==="content"&&e.splice(n,1),!s&&e[n][1].type==="definition"&&(s=n);const o={type:"setextHeading",start:{...e[r][1].start},end:{...e[e.length-1][1].end}};return e[a][1].type="setextHeadingText",s?(e.splice(a,0,["enter",o,t]),e.splice(s+1,0,["exit",e[r][1],t]),e[r][1].end={...e[s][1].end}):e[r][1]=o,e.push(["exit",o,t]),e}function oI(e,t,n){const r=this;let a;return s;function s(f){let m=r.events.length,p;for(;m--;)if(r.events[m][1].type!=="lineEnding"&&r.events[m][1].type!=="linePrefix"&&r.events[m][1].type!=="content"){p=r.events[m][1].type==="paragraph";break}return!r.parser.lazy[r.now().line]&&(r.interrupt||p)?(e.enter("setextHeadingLine"),a=f,o(f)):n(f)}function o(f){return e.enter("setextHeadingLineSequence"),c(f)}function c(f){return f===a?(e.consume(f),c):(e.exit("setextHeadingLineSequence"),Dt(f)?Nt(e,d,"lineSuffix")(f):d(f))}function d(f){return f===null||nt(f)?(e.exit("setextHeadingLine"),t(f)):n(f)}}const uI={tokenize:cI};function cI(e){const t=this,n=e.attempt(N0,r,e.attempt(this.parser.constructs.flowInitial,a,Nt(e,e.attempt(this.parser.constructs.flow,a,e.attempt(pM,a)),"linePrefix")));return n;function r(s){if(s===null){e.consume(s);return}return e.enter("lineEndingBlank"),e.consume(s),e.exit("lineEndingBlank"),t.currentConstruct=void 0,n}function a(s){if(s===null){e.consume(s);return}return e.enter("lineEnding"),e.consume(s),e.exit("lineEnding"),t.currentConstruct=void 0,n}}const dI={resolveAll:g7()},hI=p7("string"),fI=p7("text");function p7(e){return{resolveAll:g7(e==="text"?mI:void 0),tokenize:t};function t(n){const r=this,a=this.parser.constructs[e],s=n.attempt(a,o,c);return o;function o(m){return f(m)?s(m):c(m)}function c(m){if(m===null){n.consume(m);return}return n.enter("data"),n.consume(m),d}function d(m){return f(m)?(n.exit("data"),s(m)):(n.consume(m),d)}function f(m){if(m===null)return!0;const p=a[m];let b=-1;if(p)for(;++b<p.length;){const y=p[b];if(!y.previous||y.previous.call(r,r.previous))return!0}return!1}}}function g7(e){return t;function t(n,r){let a=-1,s;for(;++a<=n.length;)s===void 0?n[a]&&n[a][1].type==="data"&&(s=a,a++):(!n[a]||n[a][1].type!=="data")&&(a!==s+2&&(n[s][1].end=n[a-1][1].end,n.splice(s+2,a-s-2),a=s+2),s=void 0);return e?e(n,r):n}}function mI(e,t){let n=0;for(;++n<=e.length;)if((n===e.length||e[n][1].type==="lineEnding")&&e[n-1][1].type==="data"){const r=e[n-1][1],a=t.sliceStream(r);let s=a.length,o=-1,c=0,d;for(;s--;){const f=a[s];if(typeof f=="string"){for(o=f.length;f.charCodeAt(o-1)===32;)c++,o--;if(o)break;o=-1}else if(f===-2)d=!0,c++;else if(f!==-1){s++;break}}if(t._contentTypeTextTrailing&&n===e.length&&(c=0),c){const f={type:n===e.length||d||c<2?"lineSuffix":"hardBreakTrailing",start:{_bufferIndex:s?o:r.start._bufferIndex+o,_index:r.start._index+s,line:r.end.line,column:r.end.column-c,offset:r.end.offset-c},end:{...r.end}};r.end={...f.start},r.start.offset===r.end.offset?Object.assign(r,f):(e.splice(n,0,["enter",f,t],["exit",f,t]),n+=2)}n++}return e}const pI={42:Zr,43:Zr,45:Zr,48:Zr,49:Zr,50:Zr,51:Zr,52:Zr,53:Zr,54:Zr,55:Zr,56:Zr,57:Zr,62:o7},gI={91:_M},bI={[-2]:P1,[-1]:P1,32:P1},vI={35:AM,42:Hh,45:[W_,Hh],60:OM,61:W_,95:Hh,96:V_,126:V_},yI={38:c7,92:u7},_I={[-5]:B1,[-4]:B1,[-3]:B1,33:KM,38:c7,42:nb,60:[XR,BM],91:XM,92:[SM,u7],93:v2,95:nb,96:uM},EI={null:[nb,dI]},TI={null:[42,95]},xI={null:[]},SI=Object.freeze(Object.defineProperty({__proto__:null,attentionMarkers:TI,contentInitial:gI,disable:xI,document:pI,flow:vI,flowInitial:bI,insideSpan:EI,string:yI,text:_I},Symbol.toStringTag,{value:"Module"}));function wI(e,t,n){let r={_bufferIndex:-1,_index:0,line:n&&n.line||1,column:n&&n.column||1,offset:n&&n.offset||0};const a={},s=[];let o=[],c=[];const d={attempt:ee(H),check:ee(B),consume:O,enter:D,exit:Y,interrupt:ee(B,{interrupt:!0})},f={code:null,containerState:{},defineSkip:x,events:[],now:T,parser:e,previous:null,sliceSerialize:b,sliceStream:y,write:p};let m=t.tokenize.call(f,d);return t.resolveAll&&s.push(t),f;function p(se){return o=Ra(o,se),C(),o[o.length-1]!==null?[]:(Z(t,0),f.events=Gf(s,f.events,f),f.events)}function b(se,oe){return kI(y(se),oe)}function y(se){return AI(o,se)}function T(){const{_bufferIndex:se,_index:oe,line:ye,column:fe,offset:de}=r;return{_bufferIndex:se,_index:oe,line:ye,column:fe,offset:de}}function x(se){a[se.line]=se.column,G()}function C(){let se;for(;r._index<o.length;){const oe=o[r._index];if(typeof oe=="string")for(se=r._index,r._bufferIndex<0&&(r._bufferIndex=0);r._index===se&&r._bufferIndex<oe.length;)w(oe.charCodeAt(r._bufferIndex));else w(oe)}}function w(se){m=m(se)}function O(se){nt(se)?(r.line++,r.column=1,r.offset+=se===-3?2:1,G()):se!==-1&&(r.column++,r.offset++),r._bufferIndex<0?r._index++:(r._bufferIndex++,r._bufferIndex===o[r._index].length&&(r._bufferIndex=-1,r._index++)),f.previous=se}function D(se,oe){const ye=oe||{};return ye.type=se,ye.start=T(),f.events.push(["enter",ye,f]),c.push(ye),ye}function Y(se){const oe=c.pop();return oe.end=T(),f.events.push(["exit",oe,f]),oe}function H(se,oe){Z(se,oe.from)}function B(se,oe){oe.restore()}function ee(se,oe){return ye;function ye(fe,de,V){let me,pe,Be,k;return Array.isArray(fe)?ge(fe):"tokenize"in fe?ge([fe]):ie(fe);function ie(De){return Ot;function Ot(ot){const zt=ot!==null&&De[ot],wt=ot!==null&&De.null,vn=[...Array.isArray(zt)?zt:zt?[zt]:[],...Array.isArray(wt)?wt:wt?[wt]:[]];return ge(vn)(ot)}}function ge(De){return me=De,pe=0,De.length===0?V:L(De[pe])}function L(De){return Ot;function Ot(ot){return k=Q(),Be=De,De.partial||(f.currentConstruct=De),De.name&&f.parser.constructs.disable.null.includes(De.name)?je():De.tokenize.call(oe?Object.assign(Object.create(f),oe):f,d,Oe,je)(ot)}}function Oe(De){return se(Be,k),de}function je(De){return k.restore(),++pe<me.length?L(me[pe]):V}}}function Z(se,oe){se.resolveAll&&!s.includes(se)&&s.push(se),se.resolve&&_a(f.events,oe,f.events.length-oe,se.resolve(f.events.slice(oe),f)),se.resolveTo&&(f.events=se.resolveTo(f.events,f))}function Q(){const se=T(),oe=f.previous,ye=f.currentConstruct,fe=f.events.length,de=Array.from(c);return{from:fe,restore:V};function V(){r=se,f.previous=oe,f.currentConstruct=ye,f.events.length=fe,c=de,G()}}function G(){r.line in a&&r.column<2&&(r.column=a[r.line],r.offset+=a[r.line]-1)}}function AI(e,t){const n=t.start._index,r=t.start._bufferIndex,a=t.end._index,s=t.end._bufferIndex;let o;if(n===a)o=[e[n].slice(r,s)];else{if(o=e.slice(n,a),r>-1){const c=o[0];typeof c=="string"?o[0]=c.slice(r):o.shift()}s>0&&o.push(e[a].slice(0,s))}return o}function kI(e,t){let n=-1;const r=[];let a;for(;++n<e.length;){const s=e[n];let o;if(typeof s=="string")o=s;else switch(s){case-5:{o="\r";break}case-4:{o=`
`;break}case-3:{o=`\r
`;break}case-2:{o=t?" ":"	";break}case-1:{if(!t&&a)continue;o=" ";break}default:o=String.fromCharCode(s)}a=s===-2,r.push(o)}return r.join("")}function CI(e){const r={constructs:s7([SI,...(e||{}).extensions||[]]),content:a(jR),defined:[],document:a(YR),flow:a(uI),lazy:{},string:a(hI),text:a(fI)};return r;function a(s){return o;function o(c){return wI(r,s,c)}}}function NI(e){for(;!d7(e););return e}const X_=/[\0\t\n\r]/g;function OI(){let e=1,t="",n=!0,r;return a;function a(s,o,c){const d=[];let f,m,p,b,y;for(s=t+(typeof s=="string"?s.toString():new TextDecoder(o||void 0).decode(s)),p=0,t="",n&&(s.charCodeAt(0)===65279&&p++,n=void 0);p<s.length;){if(X_.lastIndex=p,f=X_.exec(s),b=f&&f.index!==void 0?f.index:s.length,y=s.charCodeAt(b),!f){t=s.slice(p);break}if(y===10&&p===b&&r)d.push(-3),r=void 0;else switch(r&&(d.push(-5),r=void 0),p<b&&(d.push(s.slice(p,b)),e+=b-p),y){case 0:{d.push(65533),e++;break}case 9:{for(m=Math.ceil(e/4)*4,d.push(-2);e++<m;)d.push(-1);break}case 10:{d.push(-4),e=1;break}default:r=!0,e=1}p=b+1}return c&&(r&&d.push(-5),t&&d.push(t),d.push(null)),d}}const RI=/\\([!-/:-@[-`{-~])|&(#(?:\d{1,7}|x[\da-f]{1,6})|[\da-z]{1,31});/gi;function MI(e){return e.replace(RI,II)}function II(e,t,n){if(t)return t;if(n.charCodeAt(0)===35){const a=n.charCodeAt(1),s=a===120||a===88;return l7(n.slice(s?2:1),s?16:10)}return b2(n)||e}const b7={}.hasOwnProperty;function DI(e,t,n){return typeof t!="string"&&(n=t,t=void 0),LI(n)(NI(CI(n).document().write(OI()(e,t,!0))))}function LI(e){const t={transforms:[],canContainEols:["emphasis","fragment","heading","paragraph","strong"],enter:{autolink:s(Lt),autolinkProtocol:Q,autolinkEmail:Q,atxHeading:s(Bt),blockQuote:s(wt),characterEscape:Q,characterReference:Q,codeFenced:s(vn),codeFencedFenceInfo:o,codeFencedFenceMeta:o,codeIndented:s(vn,o),codeText:s(mr,o),codeTextData:Q,data:Q,codeFlowValue:Q,definition:s(vt),definitionDestinationString:o,definitionLabelString:o,definitionTitleString:o,emphasis:s(gt),hardBreakEscape:s(Je),hardBreakTrailing:s(Je),htmlFlow:s(yt,o),htmlFlowData:Q,htmlText:s(yt,o),htmlTextData:Q,image:s(mn),label:o,link:s(Lt),listItem:s(We),listItemValue:b,listOrdered:s(ln,p),listUnordered:s(ln),paragraph:s(on),reference:L,referenceString:o,resourceDestinationString:o,resourceTitleString:o,setextHeading:s(Bt),strong:s(qn),thematicBreak:s(nn)},exit:{atxHeading:d(),atxHeadingSequence:H,autolink:d(),autolinkEmail:zt,autolinkProtocol:ot,blockQuote:d(),characterEscapeValue:G,characterReferenceMarkerHexadecimal:je,characterReferenceMarkerNumeric:je,characterReferenceValue:De,characterReference:Ot,codeFenced:d(C),codeFencedFence:x,codeFencedFenceInfo:y,codeFencedFenceMeta:T,codeFlowValue:G,codeIndented:d(w),codeText:d(de),codeTextData:G,data:G,definition:d(),definitionDestinationString:Y,definitionLabelString:O,definitionTitleString:D,emphasis:d(),hardBreakEscape:d(oe),hardBreakTrailing:d(oe),htmlFlow:d(ye),htmlFlowData:G,htmlText:d(fe),htmlTextData:G,image:d(me),label:Be,labelText:pe,lineEnding:se,link:d(V),listItem:d(),listOrdered:d(),listUnordered:d(),paragraph:d(),referenceString:Oe,resourceDestinationString:k,resourceTitleString:ie,resource:ge,setextHeading:d(Z),setextHeadingLineSequence:ee,setextHeadingText:B,strong:d(),thematicBreak:d()}};v7(t,(e||{}).mdastExtensions||[]);const n={};return r;function r(he){let Se={type:"root",children:[]};const Ue={stack:[Se],tokenStack:[],config:t,enter:c,exit:f,buffer:o,resume:m,data:n},ze=[];let tt=-1;for(;++tt<he.length;)if(he[tt][1].type==="listOrdered"||he[tt][1].type==="listUnordered")if(he[tt][0]==="enter")ze.push(tt);else{const er=ze.pop();tt=a(he,er,tt)}for(tt=-1;++tt<he.length;){const er=t[he[tt][0]];b7.call(er,he[tt][1].type)&&er[he[tt][1].type].call(Object.assign({sliceSerialize:he[tt][2].sliceSerialize},Ue),he[tt][1])}if(Ue.tokenStack.length>0){const er=Ue.tokenStack[Ue.tokenStack.length-1];(er[1]||Q_).call(Ue,void 0,er[0])}for(Se.position={start:zs(he.length>0?he[0][1].start:{line:1,column:1,offset:0}),end:zs(he.length>0?he[he.length-2][1].end:{line:1,column:1,offset:0})},tt=-1;++tt<t.transforms.length;)Se=t.transforms[tt](Se)||Se;return Se}function a(he,Se,Ue){let ze=Se-1,tt=-1,er=!1,$r,Pn,tr,Or;for(;++ze<=Ue;){const jn=he[ze];switch(jn[1].type){case"listUnordered":case"listOrdered":case"blockQuote":{jn[0]==="enter"?tt++:tt--,Or=void 0;break}case"lineEndingBlank":{jn[0]==="enter"&&($r&&!Or&&!tt&&!tr&&(tr=ze),Or=void 0);break}case"linePrefix":case"listItemValue":case"listItemMarker":case"listItemPrefix":case"listItemPrefixWhitespace":break;default:Or=void 0}if(!tt&&jn[0]==="enter"&&jn[1].type==="listItemPrefix"||tt===-1&&jn[0]==="exit"&&(jn[1].type==="listUnordered"||jn[1].type==="listOrdered")){if($r){let Yr=ze;for(Pn=void 0;Yr--;){const Rr=he[Yr];if(Rr[1].type==="lineEnding"||Rr[1].type==="lineEndingBlank"){if(Rr[0]==="exit")continue;Pn&&(he[Pn][1].type="lineEndingBlank",er=!0),Rr[1].type="lineEnding",Pn=Yr}else if(!(Rr[1].type==="linePrefix"||Rr[1].type==="blockQuotePrefix"||Rr[1].type==="blockQuotePrefixWhitespace"||Rr[1].type==="blockQuoteMarker"||Rr[1].type==="listItemIndent"))break}tr&&(!Pn||tr<Pn)&&($r._spread=!0),$r.end=Object.assign({},Pn?he[Pn][1].start:jn[1].end),he.splice(Pn||ze,0,["exit",$r,jn[2]]),ze++,Ue++}if(jn[1].type==="listItemPrefix"){const Yr={type:"listItem",_spread:!1,start:Object.assign({},jn[1].start),end:void 0};$r=Yr,he.splice(ze,0,["enter",Yr,jn[2]]),ze++,Ue++,tr=void 0,Or=!0}}}return he[Se][1]._spread=er,Ue}function s(he,Se){return Ue;function Ue(ze){c.call(this,he(ze),ze),Se&&Se.call(this,ze)}}function o(){this.stack.push({type:"fragment",children:[]})}function c(he,Se,Ue){this.stack[this.stack.length-1].children.push(he),this.stack.push(he),this.tokenStack.push([Se,Ue||void 0]),he.position={start:zs(Se.start),end:void 0}}function d(he){return Se;function Se(Ue){he&&he.call(this,Ue),f.call(this,Ue)}}function f(he,Se){const Ue=this.stack.pop(),ze=this.tokenStack.pop();if(ze)ze[0].type!==he.type&&(Se?Se.call(this,he,ze[0]):(ze[1]||Q_).call(this,he,ze[0]));else throw new Error("Cannot close `"+he.type+"` ("+Zc({start:he.start,end:he.end})+"): it’s not open");Ue.position.end=zs(he.end)}function m(){return g2(this.stack.pop())}function p(){this.data.expectingFirstListItemValue=!0}function b(he){if(this.data.expectingFirstListItemValue){const Se=this.stack[this.stack.length-2];Se.start=Number.parseInt(this.sliceSerialize(he),10),this.data.expectingFirstListItemValue=void 0}}function y(){const he=this.resume(),Se=this.stack[this.stack.length-1];Se.lang=he}function T(){const he=this.resume(),Se=this.stack[this.stack.length-1];Se.meta=he}function x(){this.data.flowCodeInside||(this.buffer(),this.data.flowCodeInside=!0)}function C(){const he=this.resume(),Se=this.stack[this.stack.length-1];Se.value=he.replace(/^(\r?\n|\r)|(\r?\n|\r)$/g,""),this.data.flowCodeInside=void 0}function w(){const he=this.resume(),Se=this.stack[this.stack.length-1];Se.value=he.replace(/(\r?\n|\r)$/g,"")}function O(he){const Se=this.resume(),Ue=this.stack[this.stack.length-1];Ue.label=Se,Ue.identifier=Qa(this.sliceSerialize(he)).toLowerCase()}function D(){const he=this.resume(),Se=this.stack[this.stack.length-1];Se.title=he}function Y(){const he=this.resume(),Se=this.stack[this.stack.length-1];Se.url=he}function H(he){const Se=this.stack[this.stack.length-1];if(!Se.depth){const Ue=this.sliceSerialize(he).length;Se.depth=Ue}}function B(){this.data.setextHeadingSlurpLineEnding=!0}function ee(he){const Se=this.stack[this.stack.length-1];Se.depth=this.sliceSerialize(he).codePointAt(0)===61?1:2}function Z(){this.data.setextHeadingSlurpLineEnding=void 0}function Q(he){const Ue=this.stack[this.stack.length-1].children;let ze=Ue[Ue.length-1];(!ze||ze.type!=="text")&&(ze=Ln(),ze.position={start:zs(he.start),end:void 0},Ue.push(ze)),this.stack.push(ze)}function G(he){const Se=this.stack.pop();Se.value+=this.sliceSerialize(he),Se.position.end=zs(he.end)}function se(he){const Se=this.stack[this.stack.length-1];if(this.data.atHardBreak){const Ue=Se.children[Se.children.length-1];Ue.position.end=zs(he.end),this.data.atHardBreak=void 0;return}!this.data.setextHeadingSlurpLineEnding&&t.canContainEols.includes(Se.type)&&(Q.call(this,he),G.call(this,he))}function oe(){this.data.atHardBreak=!0}function ye(){const he=this.resume(),Se=this.stack[this.stack.length-1];Se.value=he}function fe(){const he=this.resume(),Se=this.stack[this.stack.length-1];Se.value=he}function de(){const he=this.resume(),Se=this.stack[this.stack.length-1];Se.value=he}function V(){const he=this.stack[this.stack.length-1];if(this.data.inReference){const Se=this.data.referenceType||"shortcut";he.type+="Reference",he.referenceType=Se,delete he.url,delete he.title}else delete he.identifier,delete he.label;this.data.referenceType=void 0}function me(){const he=this.stack[this.stack.length-1];if(this.data.inReference){const Se=this.data.referenceType||"shortcut";he.type+="Reference",he.referenceType=Se,delete he.url,delete he.title}else delete he.identifier,delete he.label;this.data.referenceType=void 0}function pe(he){const Se=this.sliceSerialize(he),Ue=this.stack[this.stack.length-2];Ue.label=MI(Se),Ue.identifier=Qa(Se).toLowerCase()}function Be(){const he=this.stack[this.stack.length-1],Se=this.resume(),Ue=this.stack[this.stack.length-1];if(this.data.inReference=!0,Ue.type==="link"){const ze=he.children;Ue.children=ze}else Ue.alt=Se}function k(){const he=this.resume(),Se=this.stack[this.stack.length-1];Se.url=he}function ie(){const he=this.resume(),Se=this.stack[this.stack.length-1];Se.title=he}function ge(){this.data.inReference=void 0}function L(){this.data.referenceType="collapsed"}function Oe(he){const Se=this.resume(),Ue=this.stack[this.stack.length-1];Ue.label=Se,Ue.identifier=Qa(this.sliceSerialize(he)).toLowerCase(),this.data.referenceType="full"}function je(he){this.data.characterReferenceType=he.type}function De(he){const Se=this.sliceSerialize(he),Ue=this.data.characterReferenceType;let ze;Ue?(ze=l7(Se,Ue==="characterReferenceMarkerNumeric"?10:16),this.data.characterReferenceType=void 0):ze=b2(Se);const tt=this.stack[this.stack.length-1];tt.value+=ze}function Ot(he){const Se=this.stack.pop();Se.position.end=zs(he.end)}function ot(he){G.call(this,he);const Se=this.stack[this.stack.length-1];Se.url=this.sliceSerialize(he)}function zt(he){G.call(this,he);const Se=this.stack[this.stack.length-1];Se.url="mailto:"+this.sliceSerialize(he)}function wt(){return{type:"blockquote",children:[]}}function vn(){return{type:"code",lang:null,meta:null,value:""}}function mr(){return{type:"inlineCode",value:""}}function vt(){return{type:"definition",identifier:"",label:null,title:null,url:""}}function gt(){return{type:"emphasis",children:[]}}function Bt(){return{type:"heading",depth:0,children:[]}}function Je(){return{type:"break"}}function yt(){return{type:"html",value:""}}function mn(){return{type:"image",title:null,url:"",alt:null}}function Lt(){return{type:"link",title:null,url:"",children:[]}}function ln(he){return{type:"list",ordered:he.type==="listOrdered",start:null,spread:he._spread,children:[]}}function We(he){return{type:"listItem",spread:he._spread,checked:null,children:[]}}function on(){return{type:"paragraph",children:[]}}function qn(){return{type:"strong",children:[]}}function Ln(){return{type:"text",value:""}}function nn(){return{type:"thematicBreak"}}}function zs(e){return{line:e.line,column:e.column,offset:e.offset}}function v7(e,t){let n=-1;for(;++n<t.length;){const r=t[n];Array.isArray(r)?v7(e,r):PI(e,r)}}function PI(e,t){let n;for(n in t)if(b7.call(t,n))switch(n){case"canContainEols":{const r=t[n];r&&e[n].push(...r);break}case"transforms":{const r=t[n];r&&e[n].push(...r);break}case"enter":case"exit":{const r=t[n];r&&Object.assign(e[n],r);break}}}function Q_(e,t){throw e?new Error("Cannot close `"+e.type+"` ("+Zc({start:e.start,end:e.end})+"): a different token (`"+t.type+"`, "+Zc({start:t.start,end:t.end})+") is open"):new Error("Cannot close document, a token (`"+t.type+"`, "+Zc({start:t.start,end:t.end})+") is still open")}function BI(e){const t=this;t.parser=n;function n(r){return DI(r,{...t.data("settings"),...e,extensions:t.data("micromarkExtensions")||[],mdastExtensions:t.data("fromMarkdownExtensions")||[]})}}function UI(e,t){const n={type:"element",tagName:"blockquote",properties:{},children:e.wrap(e.all(t),!0)};return e.patch(t,n),e.applyData(t,n)}function FI(e,t){const n={type:"element",tagName:"br",properties:{},children:[]};return e.patch(t,n),[e.applyData(t,n),{type:"text",value:`
`}]}function zI(e,t){const n=t.value?t.value+`
`:"",r={};t.lang&&(r.className=["language-"+t.lang]);let a={type:"element",tagName:"code",properties:r,children:[{type:"text",value:n}]};return t.meta&&(a.data={meta:t.meta}),e.patch(t,a),a=e.applyData(t,a),a={type:"element",tagName:"pre",properties:{},children:[a]},e.patch(t,a),a}function HI(e,t){const n={type:"element",tagName:"del",properties:{},children:e.all(t)};return e.patch(t,n),e.applyData(t,n)}function qI(e,t){const n={type:"element",tagName:"em",properties:{},children:e.all(t)};return e.patch(t,n),e.applyData(t,n)}function jI(e,t){const n=typeof e.options.clobberPrefix=="string"?e.options.clobberPrefix:"user-content-",r=String(t.identifier).toUpperCase(),a=fu(r.toLowerCase()),s=e.footnoteOrder.indexOf(r);let o,c=e.footnoteCounts.get(r);c===void 0?(c=0,e.footnoteOrder.push(r),o=e.footnoteOrder.length):o=s+1,c+=1,e.footnoteCounts.set(r,c);const d={type:"element",tagName:"a",properties:{href:"#"+n+"fn-"+a,id:n+"fnref-"+a+(c>1?"-"+c:""),dataFootnoteRef:!0,ariaDescribedBy:["footnote-label"]},children:[{type:"text",value:String(o)}]};e.patch(t,d);const f={type:"element",tagName:"sup",properties:{},children:[d]};return e.patch(t,f),e.applyData(t,f)}function $I(e,t){const n={type:"element",tagName:"h"+t.depth,properties:{},children:e.all(t)};return e.patch(t,n),e.applyData(t,n)}function YI(e,t){if(e.options.allowDangerousHtml){const n={type:"raw",value:t.value};return e.patch(t,n),e.applyData(t,n)}}function y7(e,t){const n=t.referenceType;let r="]";if(n==="collapsed"?r+="[]":n==="full"&&(r+="["+(t.label||t.identifier)+"]"),t.type==="imageReference")return[{type:"text",value:"!["+t.alt+r}];const a=e.all(t),s=a[0];s&&s.type==="text"?s.value="["+s.value:a.unshift({type:"text",value:"["});const o=a[a.length-1];return o&&o.type==="text"?o.value+=r:a.push({type:"text",value:r}),a}function GI(e,t){const n=String(t.identifier).toUpperCase(),r=e.definitionById.get(n);if(!r)return y7(e,t);const a={src:fu(r.url||""),alt:t.alt};r.title!==null&&r.title!==void 0&&(a.title=r.title);const s={type:"element",tagName:"img",properties:a,children:[]};return e.patch(t,s),e.applyData(t,s)}function VI(e,t){const n={src:fu(t.url)};t.alt!==null&&t.alt!==void 0&&(n.alt=t.alt),t.title!==null&&t.title!==void 0&&(n.title=t.title);const r={type:"element",tagName:"img",properties:n,children:[]};return e.patch(t,r),e.applyData(t,r)}function KI(e,t){const n={type:"text",value:t.value.replace(/\r?\n|\r/g," ")};e.patch(t,n);const r={type:"element",tagName:"code",properties:{},children:[n]};return e.patch(t,r),e.applyData(t,r)}function WI(e,t){const n=String(t.identifier).toUpperCase(),r=e.definitionById.get(n);if(!r)return y7(e,t);const a={href:fu(r.url||"")};r.title!==null&&r.title!==void 0&&(a.title=r.title);const s={type:"element",tagName:"a",properties:a,children:e.all(t)};return e.patch(t,s),e.applyData(t,s)}function XI(e,t){const n={href:fu(t.url)};t.title!==null&&t.title!==void 0&&(n.title=t.title);const r={type:"element",tagName:"a",properties:n,children:e.all(t)};return e.patch(t,r),e.applyData(t,r)}function QI(e,t,n){const r=e.all(t),a=n?ZI(n):_7(t),s={},o=[];if(typeof t.checked=="boolean"){const m=r[0];let p;m&&m.type==="element"&&m.tagName==="p"?p=m:(p={type:"element",tagName:"p",properties:{},children:[]},r.unshift(p)),p.children.length>0&&p.children.unshift({type:"text",value:" "}),p.children.unshift({type:"element",tagName:"input",properties:{type:"checkbox",checked:t.checked,disabled:!0},children:[]}),s.className=["task-list-item"]}let c=-1;for(;++c<r.length;){const m=r[c];(a||c!==0||m.type!=="element"||m.tagName!=="p")&&o.push({type:"text",value:`
`}),m.type==="element"&&m.tagName==="p"&&!a?o.push(...m.children):o.push(m)}const d=r[r.length-1];d&&(a||d.type!=="element"||d.tagName!=="p")&&o.push({type:"text",value:`
`});const f={type:"element",tagName:"li",properties:s,children:o};return e.patch(t,f),e.applyData(t,f)}function ZI(e){let t=!1;if(e.type==="list"){t=e.spread||!1;const n=e.children;let r=-1;for(;!t&&++r<n.length;)t=_7(n[r])}return t}function _7(e){const t=e.spread;return t??e.children.length>1}function JI(e,t){const n={},r=e.all(t);let a=-1;for(typeof t.start=="number"&&t.start!==1&&(n.start=t.start);++a<r.length;){const o=r[a];if(o.type==="element"&&o.tagName==="li"&&o.properties&&Array.isArray(o.properties.className)&&o.properties.className.includes("task-list-item")){n.className=["contains-task-list"];break}}const s={type:"element",tagName:t.ordered?"ol":"ul",properties:n,children:e.wrap(r,!0)};return e.patch(t,s),e.applyData(t,s)}function eD(e,t){const n={type:"element",tagName:"p",properties:{},children:e.all(t)};return e.patch(t,n),e.applyData(t,n)}function tD(e,t){const n={type:"root",children:e.wrap(e.all(t))};return e.patch(t,n),e.applyData(t,n)}function nD(e,t){const n={type:"element",tagName:"strong",properties:{},children:e.all(t)};return e.patch(t,n),e.applyData(t,n)}function rD(e,t){const n=e.all(t),r=n.shift(),a=[];if(r){const o={type:"element",tagName:"thead",properties:{},children:e.wrap([r],!0)};e.patch(t.children[0],o),a.push(o)}if(n.length>0){const o={type:"element",tagName:"tbody",properties:{},children:e.wrap(n,!0)},c=yi(t.children[1]),d=$f(t.children[t.children.length-1]);c&&d&&(o.position={start:c,end:d}),a.push(o)}const s={type:"element",tagName:"table",properties:{},children:e.wrap(a,!0)};return e.patch(t,s),e.applyData(t,s)}function aD(e,t,n){const r=n?n.children:void 0,s=(r?r.indexOf(t):1)===0?"th":"td",o=n&&n.type==="table"?n.align:void 0,c=o?o.length:t.children.length;let d=-1;const f=[];for(;++d<c;){const p=t.children[d],b={},y=o?o[d]:void 0;y&&(b.align=y);let T={type:"element",tagName:s,properties:b,children:[]};p&&(T.children=e.all(p),e.patch(p,T),T=e.applyData(p,T)),f.push(T)}const m={type:"element",tagName:"tr",properties:{},children:e.wrap(f,!0)};return e.patch(t,m),e.applyData(t,m)}function iD(e,t){const n={type:"element",tagName:"td",properties:{},children:e.all(t)};return e.patch(t,n),e.applyData(t,n)}const Z_=9,J_=32;function sD(e){const t=String(e),n=/\r?\n|\r/g;let r=n.exec(t),a=0;const s=[];for(;r;)s.push(eE(t.slice(a,r.index),a>0,!0),r[0]),a=r.index+r[0].length,r=n.exec(t);return s.push(eE(t.slice(a),a>0,!1)),s.join("")}function eE(e,t,n){let r=0,a=e.length;if(t){let s=e.codePointAt(r);for(;s===Z_||s===J_;)r++,s=e.codePointAt(r)}if(n){let s=e.codePointAt(a-1);for(;s===Z_||s===J_;)a--,s=e.codePointAt(a-1)}return a>r?e.slice(r,a):""}function lD(e,t){const n={type:"text",value:sD(String(t.value))};return e.patch(t,n),e.applyData(t,n)}function oD(e,t){const n={type:"element",tagName:"hr",properties:{},children:[]};return e.patch(t,n),e.applyData(t,n)}const uD={blockquote:UI,break:FI,code:zI,delete:HI,emphasis:qI,footnoteReference:jI,heading:$I,html:YI,imageReference:GI,image:VI,inlineCode:KI,linkReference:WI,link:XI,listItem:QI,list:JI,paragraph:eD,root:tD,strong:nD,table:rD,tableCell:iD,tableRow:aD,text:lD,thematicBreak:oD,toml:lh,yaml:lh,definition:lh,footnoteDefinition:lh};function lh(){}const E7=-1,Vf=0,e0=1,tf=2,y2=3,_2=4,E2=5,T2=6,T7=7,x7=8,tE=typeof self=="object"?self:globalThis,cD=(e,t)=>{const n=(a,s)=>(e.set(s,a),a),r=a=>{if(e.has(a))return e.get(a);const[s,o]=t[a];switch(s){case Vf:case E7:return n(o,a);case e0:{const c=n([],a);for(const d of o)c.push(r(d));return c}case tf:{const c=n({},a);for(const[d,f]of o)c[r(d)]=r(f);return c}case y2:return n(new Date(o),a);case _2:{const{source:c,flags:d}=o;return n(new RegExp(c,d),a)}case E2:{const c=n(new Map,a);for(const[d,f]of o)c.set(r(d),r(f));return c}case T2:{const c=n(new Set,a);for(const d of o)c.add(r(d));return c}case T7:{const{name:c,message:d}=o;return n(new tE[c](d),a)}case x7:return n(BigInt(o),a);case"BigInt":return n(Object(BigInt(o)),a);case"ArrayBuffer":return n(new Uint8Array(o).buffer,o);case"DataView":{const{buffer:c}=new Uint8Array(o);return n(new DataView(c),o)}}return n(new tE[s](o),a)};return r},nE=e=>cD(new Map,e)(0),Po="",{toString:dD}={},{keys:hD}=Object,kc=e=>{const t=typeof e;if(t!=="object"||!e)return[Vf,t];const n=dD.call(e).slice(8,-1);switch(n){case"Array":return[e0,Po];case"Object":return[tf,Po];case"Date":return[y2,Po];case"RegExp":return[_2,Po];case"Map":return[E2,Po];case"Set":return[T2,Po];case"DataView":return[e0,n]}return n.includes("Array")?[e0,n]:n.includes("Error")?[T7,n]:[tf,n]},oh=([e,t])=>e===Vf&&(t==="function"||t==="symbol"),fD=(e,t,n,r)=>{const a=(o,c)=>{const d=r.push(o)-1;return n.set(c,d),d},s=o=>{if(n.has(o))return n.get(o);let[c,d]=kc(o);switch(c){case Vf:{let m=o;switch(d){case"bigint":c=x7,m=o.toString();break;case"function":case"symbol":if(e)throw new TypeError("unable to serialize "+d);m=null;break;case"undefined":return a([E7],o)}return a([c,m],o)}case e0:{if(d){let b=o;return d==="DataView"?b=new Uint8Array(o.buffer):d==="ArrayBuffer"&&(b=new Uint8Array(o)),a([d,[...b]],o)}const m=[],p=a([c,m],o);for(const b of o)m.push(s(b));return p}case tf:{if(d)switch(d){case"BigInt":return a([d,o.toString()],o);case"Boolean":case"Number":case"String":return a([d,o.valueOf()],o)}if(t&&"toJSON"in o)return s(o.toJSON());const m=[],p=a([c,m],o);for(const b of hD(o))(e||!oh(kc(o[b])))&&m.push([s(b),s(o[b])]);return p}case y2:return a([c,o.toISOString()],o);case _2:{const{source:m,flags:p}=o;return a([c,{source:m,flags:p}],o)}case E2:{const m=[],p=a([c,m],o);for(const[b,y]of o)(e||!(oh(kc(b))||oh(kc(y))))&&m.push([s(b),s(y)]);return p}case T2:{const m=[],p=a([c,m],o);for(const b of o)(e||!oh(kc(b)))&&m.push(s(b));return p}}const{message:f}=o;return a([c,{name:d,message:f}],o)};return s},rE=(e,{json:t,lossy:n}={})=>{const r=[];return fD(!(t||n),!!t,new Map,r)(e),r},tu=typeof structuredClone=="function"?(e,t)=>t&&("json"in t||"lossy"in t)?nE(rE(e,t)):structuredClone(e):(e,t)=>nE(rE(e,t));function mD(e,t){const n=[{type:"text",value:"↩"}];return t>1&&n.push({type:"element",tagName:"sup",properties:{},children:[{type:"text",value:String(t)}]}),n}function pD(e,t){return"Back to reference "+(e+1)+(t>1?"-"+t:"")}function gD(e){const t=typeof e.options.clobberPrefix=="string"?e.options.clobberPrefix:"user-content-",n=e.options.footnoteBackContent||mD,r=e.options.footnoteBackLabel||pD,a=e.options.footnoteLabel||"Footnotes",s=e.options.footnoteLabelTagName||"h2",o=e.options.footnoteLabelProperties||{className:["sr-only"]},c=[];let d=-1;for(;++d<e.footnoteOrder.length;){const f=e.footnoteById.get(e.footnoteOrder[d]);if(!f)continue;const m=e.all(f),p=String(f.identifier).toUpperCase(),b=fu(p.toLowerCase());let y=0;const T=[],x=e.footnoteCounts.get(p);for(;x!==void 0&&++y<=x;){T.length>0&&T.push({type:"text",value:" "});let O=typeof n=="string"?n:n(d,y);typeof O=="string"&&(O={type:"text",value:O}),T.push({type:"element",tagName:"a",properties:{href:"#"+t+"fnref-"+b+(y>1?"-"+y:""),dataFootnoteBackref:"",ariaLabel:typeof r=="string"?r:r(d,y),className:["data-footnote-backref"]},children:Array.isArray(O)?O:[O]})}const C=m[m.length-1];if(C&&C.type==="element"&&C.tagName==="p"){const O=C.children[C.children.length-1];O&&O.type==="text"?O.value+=" ":C.children.push({type:"text",value:" "}),C.children.push(...T)}else m.push(...T);const w={type:"element",tagName:"li",properties:{id:t+"fn-"+b},children:e.wrap(m,!0)};e.patch(f,w),c.push(w)}if(c.length!==0)return{type:"element",tagName:"section",properties:{dataFootnotes:!0,className:["footnotes"]},children:[{type:"element",tagName:s,properties:{...tu(o),id:"footnote-label"},children:[{type:"text",value:a}]},{type:"text",value:`
`},{type:"element",tagName:"ol",properties:{},children:e.wrap(c,!0)},{type:"text",value:`
`}]}}const O0=function(e){if(e==null)return _D;if(typeof e=="function")return Kf(e);if(typeof e=="object")return Array.isArray(e)?bD(e):vD(e);if(typeof e=="string")return yD(e);throw new Error("Expected function, string, or object as test")};function bD(e){const t=[];let n=-1;for(;++n<e.length;)t[n]=O0(e[n]);return Kf(r);function r(...a){let s=-1;for(;++s<t.length;)if(t[s].apply(this,a))return!0;return!1}}function vD(e){const t=e;return Kf(n);function n(r){const a=r;let s;for(s in e)if(a[s]!==t[s])return!1;return!0}}function yD(e){return Kf(t);function t(n){return n&&n.type===e}}function Kf(e){return t;function t(n,r,a){return!!(ED(n)&&e.call(this,n,typeof r=="number"?r:void 0,a||void 0))}}function _D(){return!0}function ED(e){return e!==null&&typeof e=="object"&&"type"in e}const S7=[],TD=!0,rb=!1,w7="skip";function x2(e,t,n,r){let a;typeof t=="function"&&typeof n!="function"?(r=n,n=t):a=t;const s=O0(a),o=r?-1:1;c(e,void 0,[])();function c(d,f,m){const p=d&&typeof d=="object"?d:{};if(typeof p.type=="string"){const y=typeof p.tagName=="string"?p.tagName:typeof p.name=="string"?p.name:void 0;Object.defineProperty(b,"name",{value:"node ("+(d.type+(y?"<"+y+">":""))+")"})}return b;function b(){let y=S7,T,x,C;if((!t||s(d,f,m[m.length-1]||void 0))&&(y=xD(n(d,m)),y[0]===rb))return y;if("children"in d&&d.children){const w=d;if(w.children&&y[0]!==w7)for(x=(r?w.children.length:-1)+o,C=m.concat(w);x>-1&&x<w.children.length;){const O=w.children[x];if(T=c(O,x,C)(),T[0]===rb)return T;x=typeof T[1]=="number"?T[1]:x+o}}return y}}}function xD(e){return Array.isArray(e)?e:typeof e=="number"?[TD,e]:e==null?S7:[e]}function R0(e,t,n,r){let a,s,o;typeof t=="function"&&typeof n!="function"?(s=void 0,o=t,a=n):(s=t,o=n,a=r),x2(e,s,c,a);function c(d,f){const m=f[f.length-1],p=m?m.children.indexOf(d):void 0;return o(d,p,m)}}const ab={}.hasOwnProperty,SD={};function wD(e,t){const n=t||SD,r=new Map,a=new Map,s=new Map,o={...uD,...n.handlers},c={all:f,applyData:kD,definitionById:r,footnoteById:a,footnoteCounts:s,footnoteOrder:[],handlers:o,one:d,options:n,patch:AD,wrap:ND};return R0(e,function(m){if(m.type==="definition"||m.type==="footnoteDefinition"){const p=m.type==="definition"?r:a,b=String(m.identifier).toUpperCase();p.has(b)||p.set(b,m)}}),c;function d(m,p){const b=m.type,y=c.handlers[b];if(ab.call(c.handlers,b)&&y)return y(c,m,p);if(c.options.passThrough&&c.options.passThrough.includes(b)){if("children"in m){const{children:x,...C}=m,w=tu(C);return w.children=c.all(m),w}return tu(m)}return(c.options.unknownHandler||CD)(c,m,p)}function f(m){const p=[];if("children"in m){const b=m.children;let y=-1;for(;++y<b.length;){const T=c.one(b[y],m);if(T){if(y&&b[y-1].type==="break"&&(!Array.isArray(T)&&T.type==="text"&&(T.value=aE(T.value)),!Array.isArray(T)&&T.type==="element")){const x=T.children[0];x&&x.type==="text"&&(x.value=aE(x.value))}Array.isArray(T)?p.push(...T):p.push(T)}}}return p}}function AD(e,t){e.position&&(t.position=pR(e))}function kD(e,t){let n=t;if(e&&e.data){const r=e.data.hName,a=e.data.hChildren,s=e.data.hProperties;if(typeof r=="string")if(n.type==="element")n.tagName=r;else{const o="children"in n?n.children:[n];n={type:"element",tagName:r,properties:{},children:o}}n.type==="element"&&s&&Object.assign(n.properties,tu(s)),"children"in n&&n.children&&a!==null&&a!==void 0&&(n.children=a)}return n}function CD(e,t){const n=t.data||{},r="value"in t&&!(ab.call(n,"hProperties")||ab.call(n,"hChildren"))?{type:"text",value:t.value}:{type:"element",tagName:"div",properties:{},children:e.all(t)};return e.patch(t,r),e.applyData(t,r)}function ND(e,t){const n=[];let r=-1;for(t&&n.push({type:"text",value:`
`});++r<e.length;)r&&n.push({type:"text",value:`
`}),n.push(e[r]);return t&&e.length>0&&n.push({type:"text",value:`
`}),n}function aE(e){let t=0,n=e.charCodeAt(t);for(;n===9||n===32;)t++,n=e.charCodeAt(t);return e.slice(t)}function iE(e,t){const n=wD(e,t),r=n.one(e,void 0),a=gD(n),s=Array.isArray(r)?{type:"root",children:r}:r||{type:"root",children:[]};return a&&s.children.push({type:"text",value:`
`},a),s}function OD(e,t){return e&&"run"in e?async function(n,r){const a=iE(n,{file:r,...t});await e.run(a,r)}:function(n,r){return iE(n,{file:r,...e||t})}}function sE(e){if(e)throw e}var U1,lE;function RD(){if(lE)return U1;lE=1;var e=Object.prototype.hasOwnProperty,t=Object.prototype.toString,n=Object.defineProperty,r=Object.getOwnPropertyDescriptor,a=function(f){return typeof Array.isArray=="function"?Array.isArray(f):t.call(f)==="[object Array]"},s=function(f){if(!f||t.call(f)!=="[object Object]")return!1;var m=e.call(f,"constructor"),p=f.constructor&&f.constructor.prototype&&e.call(f.constructor.prototype,"isPrototypeOf");if(f.constructor&&!m&&!p)return!1;var b;for(b in f);return typeof b>"u"||e.call(f,b)},o=function(f,m){n&&m.name==="__proto__"?n(f,m.name,{enumerable:!0,configurable:!0,value:m.newValue,writable:!0}):f[m.name]=m.newValue},c=function(f,m){if(m==="__proto__")if(e.call(f,m)){if(r)return r(f,m).value}else return;return f[m]};return U1=function d(){var f,m,p,b,y,T,x=arguments[0],C=1,w=arguments.length,O=!1;for(typeof x=="boolean"&&(O=x,x=arguments[1]||{},C=2),(x==null||typeof x!="object"&&typeof x!="function")&&(x={});C<w;++C)if(f=arguments[C],f!=null)for(m in f)p=c(x,m),b=c(f,m),x!==b&&(O&&b&&(s(b)||(y=a(b)))?(y?(y=!1,T=p&&a(p)?p:[]):T=p&&s(p)?p:{},o(x,{name:m,newValue:d(O,T,b)})):typeof b<"u"&&o(x,{name:m,newValue:b}));return x},U1}var MD=RD();const F1=Mf(MD);function ib(e){if(typeof e!="object"||e===null)return!1;const t=Object.getPrototypeOf(e);return(t===null||t===Object.prototype||Object.getPrototypeOf(t)===null)&&!(Symbol.toStringTag in e)&&!(Symbol.iterator in e)}function ID(){const e=[],t={run:n,use:r};return t;function n(...a){let s=-1;const o=a.pop();if(typeof o!="function")throw new TypeError("Expected function as last argument, not "+o);c(null,...a);function c(d,...f){const m=e[++s];let p=-1;if(d){o(d);return}for(;++p<a.length;)(f[p]===null||f[p]===void 0)&&(f[p]=a[p]);a=f,m?DD(m,c)(...f):o(null,...f)}}function r(a){if(typeof a!="function")throw new TypeError("Expected `middelware` to be a function, not "+a);return e.push(a),t}}function DD(e,t){let n;return r;function r(...o){const c=e.length>o.length;let d;c&&o.push(a);try{d=e.apply(this,o)}catch(f){const m=f;if(c&&n)throw m;return a(m)}c||(d&&d.then&&typeof d.then=="function"?d.then(s,a):d instanceof Error?a(d):s(d))}function a(o,...c){n||(n=!0,t(o,...c))}function s(o){a(null,o)}}const oi={basename:LD,dirname:PD,extname:BD,join:UD,sep:"/"};function LD(e,t){if(t!==void 0&&typeof t!="string")throw new TypeError('"ext" argument must be a string');M0(e);let n=0,r=-1,a=e.length,s;if(t===void 0||t.length===0||t.length>e.length){for(;a--;)if(e.codePointAt(a)===47){if(s){n=a+1;break}}else r<0&&(s=!0,r=a+1);return r<0?"":e.slice(n,r)}if(t===e)return"";let o=-1,c=t.length-1;for(;a--;)if(e.codePointAt(a)===47){if(s){n=a+1;break}}else o<0&&(s=!0,o=a+1),c>-1&&(e.codePointAt(a)===t.codePointAt(c--)?c<0&&(r=a):(c=-1,r=o));return n===r?r=o:r<0&&(r=e.length),e.slice(n,r)}function PD(e){if(M0(e),e.length===0)return".";let t=-1,n=e.length,r;for(;--n;)if(e.codePointAt(n)===47){if(r){t=n;break}}else r||(r=!0);return t<0?e.codePointAt(0)===47?"/":".":t===1&&e.codePointAt(0)===47?"//":e.slice(0,t)}function BD(e){M0(e);let t=e.length,n=-1,r=0,a=-1,s=0,o;for(;t--;){const c=e.codePointAt(t);if(c===47){if(o){r=t+1;break}continue}n<0&&(o=!0,n=t+1),c===46?a<0?a=t:s!==1&&(s=1):a>-1&&(s=-1)}return a<0||n<0||s===0||s===1&&a===n-1&&a===r+1?"":e.slice(a,n)}function UD(...e){let t=-1,n;for(;++t<e.length;)M0(e[t]),e[t]&&(n=n===void 0?e[t]:n+"/"+e[t]);return n===void 0?".":FD(n)}function FD(e){M0(e);const t=e.codePointAt(0)===47;let n=zD(e,!t);return n.length===0&&!t&&(n="."),n.length>0&&e.codePointAt(e.length-1)===47&&(n+="/"),t?"/"+n:n}function zD(e,t){let n="",r=0,a=-1,s=0,o=-1,c,d;for(;++o<=e.length;){if(o<e.length)c=e.codePointAt(o);else{if(c===47)break;c=47}if(c===47){if(!(a===o-1||s===1))if(a!==o-1&&s===2){if(n.length<2||r!==2||n.codePointAt(n.length-1)!==46||n.codePointAt(n.length-2)!==46){if(n.length>2){if(d=n.lastIndexOf("/"),d!==n.length-1){d<0?(n="",r=0):(n=n.slice(0,d),r=n.length-1-n.lastIndexOf("/")),a=o,s=0;continue}}else if(n.length>0){n="",r=0,a=o,s=0;continue}}t&&(n=n.length>0?n+"/..":"..",r=2)}else n.length>0?n+="/"+e.slice(a+1,o):n=e.slice(a+1,o),r=o-a-1;a=o,s=0}else c===46&&s>-1?s++:s=-1}return n}function M0(e){if(typeof e!="string")throw new TypeError("Path must be a string. Received "+JSON.stringify(e))}const HD={cwd:qD};function qD(){return"/"}function sb(e){return!!(e!==null&&typeof e=="object"&&"href"in e&&e.href&&"protocol"in e&&e.protocol&&e.auth===void 0)}function jD(e){if(typeof e=="string")e=new URL(e);else if(!sb(e)){const t=new TypeError('The "path" argument must be of type string or an instance of URL. Received `'+e+"`");throw t.code="ERR_INVALID_ARG_TYPE",t}if(e.protocol!=="file:"){const t=new TypeError("The URL must be of scheme file");throw t.code="ERR_INVALID_URL_SCHEME",t}return $D(e)}function $D(e){if(e.hostname!==""){const r=new TypeError('File URL host must be "localhost" or empty on darwin');throw r.code="ERR_INVALID_FILE_URL_HOST",r}const t=e.pathname;let n=-1;for(;++n<t.length;)if(t.codePointAt(n)===37&&t.codePointAt(n+1)===50){const r=t.codePointAt(n+2);if(r===70||r===102){const a=new TypeError("File URL path must not include encoded / characters");throw a.code="ERR_INVALID_FILE_URL_PATH",a}}return decodeURIComponent(t)}const z1=["history","path","basename","stem","extname","dirname"];class A7{constructor(t){let n;t?sb(t)?n={path:t}:typeof t=="string"||YD(t)?n={value:t}:n=t:n={},this.cwd="cwd"in n?"":HD.cwd(),this.data={},this.history=[],this.messages=[],this.value,this.map,this.result,this.stored;let r=-1;for(;++r<z1.length;){const s=z1[r];s in n&&n[s]!==void 0&&n[s]!==null&&(this[s]=s==="history"?[...n[s]]:n[s])}let a;for(a in n)z1.includes(a)||(this[a]=n[a])}get basename(){return typeof this.path=="string"?oi.basename(this.path):void 0}set basename(t){q1(t,"basename"),H1(t,"basename"),this.path=oi.join(this.dirname||"",t)}get dirname(){return typeof this.path=="string"?oi.dirname(this.path):void 0}set dirname(t){oE(this.basename,"dirname"),this.path=oi.join(t||"",this.basename)}get extname(){return typeof this.path=="string"?oi.extname(this.path):void 0}set extname(t){if(H1(t,"extname"),oE(this.dirname,"extname"),t){if(t.codePointAt(0)!==46)throw new Error("`extname` must start with `.`");if(t.includes(".",1))throw new Error("`extname` cannot contain multiple dots")}this.path=oi.join(this.dirname,this.stem+(t||""))}get path(){return this.history[this.history.length-1]}set path(t){sb(t)&&(t=jD(t)),q1(t,"path"),this.path!==t&&this.history.push(t)}get stem(){return typeof this.path=="string"?oi.basename(this.path,this.extname):void 0}set stem(t){q1(t,"stem"),H1(t,"stem"),this.path=oi.join(this.dirname||"",t+(this.extname||""))}fail(t,n,r){const a=this.message(t,n,r);throw a.fatal=!0,a}info(t,n,r){const a=this.message(t,n,r);return a.fatal=void 0,a}message(t,n,r){const a=new Cr(t,n,r);return this.path&&(a.name=this.path+":"+a.name,a.file=this.path),a.fatal=!1,this.messages.push(a),a}toString(t){return this.value===void 0?"":typeof this.value=="string"?this.value:new TextDecoder(t||void 0).decode(this.value)}}function H1(e,t){if(e&&e.includes(oi.sep))throw new Error("`"+t+"` cannot be a path: did not expect `"+oi.sep+"`")}function q1(e,t){if(!e)throw new Error("`"+t+"` cannot be empty")}function oE(e,t){if(!e)throw new Error("Setting `"+t+"` requires `path` to be set too")}function YD(e){return!!(e&&typeof e=="object"&&"byteLength"in e&&"byteOffset"in e)}const GD=function(e){const r=this.constructor.prototype,a=r[e],s=function(){return a.apply(s,arguments)};return Object.setPrototypeOf(s,r),s},VD={}.hasOwnProperty;class S2 extends GD{constructor(){super("copy"),this.Compiler=void 0,this.Parser=void 0,this.attachers=[],this.compiler=void 0,this.freezeIndex=-1,this.frozen=void 0,this.namespace={},this.parser=void 0,this.transformers=ID()}copy(){const t=new S2;let n=-1;for(;++n<this.attachers.length;){const r=this.attachers[n];t.use(...r)}return t.data(F1(!0,{},this.namespace)),t}data(t,n){return typeof t=="string"?arguments.length===2?(Y1("data",this.frozen),this.namespace[t]=n,this):VD.call(this.namespace,t)&&this.namespace[t]||void 0:t?(Y1("data",this.frozen),this.namespace=t,this):this.namespace}freeze(){if(this.frozen)return this;const t=this;for(;++this.freezeIndex<this.attachers.length;){const[n,...r]=this.attachers[this.freezeIndex];if(r[0]===!1)continue;r[0]===!0&&(r[0]=void 0);const a=n.call(t,...r);typeof a=="function"&&this.transformers.use(a)}return this.frozen=!0,this.freezeIndex=Number.POSITIVE_INFINITY,this}parse(t){this.freeze();const n=uh(t),r=this.parser||this.Parser;return j1("parse",r),r(String(n),n)}process(t,n){const r=this;return this.freeze(),j1("process",this.parser||this.Parser),$1("process",this.compiler||this.Compiler),n?a(void 0,n):new Promise(a);function a(s,o){const c=uh(t),d=r.parse(c);r.run(d,c,function(m,p,b){if(m||!p||!b)return f(m);const y=p,T=r.stringify(y,b);XD(T)?b.value=T:b.result=T,f(m,b)});function f(m,p){m||!p?o(m):s?s(p):n(void 0,p)}}}processSync(t){let n=!1,r;return this.freeze(),j1("processSync",this.parser||this.Parser),$1("processSync",this.compiler||this.Compiler),this.process(t,a),cE("processSync","process",n),r;function a(s,o){n=!0,sE(s),r=o}}run(t,n,r){uE(t),this.freeze();const a=this.transformers;return!r&&typeof n=="function"&&(r=n,n=void 0),r?s(void 0,r):new Promise(s);function s(o,c){const d=uh(n);a.run(t,d,f);function f(m,p,b){const y=p||t;m?c(m):o?o(y):r(void 0,y,b)}}}runSync(t,n){let r=!1,a;return this.run(t,n,s),cE("runSync","run",r),a;function s(o,c){sE(o),a=c,r=!0}}stringify(t,n){this.freeze();const r=uh(n),a=this.compiler||this.Compiler;return $1("stringify",a),uE(t),a(t,r)}use(t,...n){const r=this.attachers,a=this.namespace;if(Y1("use",this.frozen),t!=null)if(typeof t=="function")d(t,n);else if(typeof t=="object")Array.isArray(t)?c(t):o(t);else throw new TypeError("Expected usable value, not `"+t+"`");return this;function s(f){if(typeof f=="function")d(f,[]);else if(typeof f=="object")if(Array.isArray(f)){const[m,...p]=f;d(m,p)}else o(f);else throw new TypeError("Expected usable value, not `"+f+"`")}function o(f){if(!("plugins"in f)&&!("settings"in f))throw new Error("Expected usable value but received an empty preset, which is probably a mistake: presets typically come with `plugins` and sometimes with `settings`, but this has neither");c(f.plugins),f.settings&&(a.settings=F1(!0,a.settings,f.settings))}function c(f){let m=-1;if(f!=null)if(Array.isArray(f))for(;++m<f.length;){const p=f[m];s(p)}else throw new TypeError("Expected a list of plugins, not `"+f+"`")}function d(f,m){let p=-1,b=-1;for(;++p<r.length;)if(r[p][0]===f){b=p;break}if(b===-1)r.push([f,...m]);else if(m.length>0){let[y,...T]=m;const x=r[b][1];ib(x)&&ib(y)&&(y=F1(!0,x,y)),r[b]=[f,y,...T]}}}}const KD=new S2().freeze();function j1(e,t){if(typeof t!="function")throw new TypeError("Cannot `"+e+"` without `parser`")}function $1(e,t){if(typeof t!="function")throw new TypeError("Cannot `"+e+"` without `compiler`")}function Y1(e,t){if(t)throw new Error("Cannot call `"+e+"` on a frozen processor.\nCreate a new processor first, by calling it: use `processor()` instead of `processor`.")}function uE(e){if(!ib(e)||typeof e.type!="string")throw new TypeError("Expected node, got `"+e+"`")}function cE(e,t,n){if(!n)throw new Error("`"+e+"` finished async. Use `"+t+"` instead")}function uh(e){return WD(e)?e:new A7(e)}function WD(e){return!!(e&&typeof e=="object"&&"message"in e&&"messages"in e)}function XD(e){return typeof e=="string"||QD(e)}function QD(e){return!!(e&&typeof e=="object"&&"byteLength"in e&&"byteOffset"in e)}const ZD="https://github.com/remarkjs/react-markdown/blob/main/changelog.md",dE=[],hE={allowDangerousHtml:!0},JD=/^(https?|ircs?|mailto|xmpp)$/i,eL=[{from:"astPlugins",id:"remove-buggy-html-in-markdown-parser"},{from:"allowDangerousHtml",id:"remove-buggy-html-in-markdown-parser"},{from:"allowNode",id:"replace-allownode-allowedtypes-and-disallowedtypes",to:"allowElement"},{from:"allowedTypes",id:"replace-allownode-allowedtypes-and-disallowedtypes",to:"allowedElements"},{from:"className",id:"remove-classname"},{from:"disallowedTypes",id:"replace-allownode-allowedtypes-and-disallowedtypes",to:"disallowedElements"},{from:"escapeHtml",id:"remove-buggy-html-in-markdown-parser"},{from:"includeElementIndex",id:"#remove-includeelementindex"},{from:"includeNodeIndex",id:"change-includenodeindex-to-includeelementindex"},{from:"linkTarget",id:"remove-linktarget"},{from:"plugins",id:"change-plugins-to-remarkplugins",to:"remarkPlugins"},{from:"rawSourcePos",id:"#remove-rawsourcepos"},{from:"renderers",id:"change-renderers-to-components",to:"components"},{from:"source",id:"change-source-to-children",to:"children"},{from:"sourcePos",id:"#remove-sourcepos"},{from:"transformImageUri",id:"#add-urltransform",to:"urlTransform"},{from:"transformLinkUri",id:"#add-urltransform",to:"urlTransform"}];function tL(e){const t=nL(e),n=rL(e);return aL(t.runSync(t.parse(n),n),e)}function nL(e){const t=e.rehypePlugins||dE,n=e.remarkPlugins||dE,r=e.remarkRehypeOptions?{...e.remarkRehypeOptions,...hE}:hE;return KD().use(BI).use(n).use(OD,r).use(t)}function rL(e){const t=e.children||"",n=new A7;return typeof t=="string"&&(n.value=t),n}function aL(e,t){const n=t.allowedElements,r=t.allowElement,a=t.components,s=t.disallowedElements,o=t.skipHtml,c=t.unwrapDisallowed,d=t.urlTransform||iL;for(const m of eL)Object.hasOwn(t,m.from)&&(""+m.from+(m.to?"use `"+m.to+"` instead":"remove it")+ZD+m.id,void 0);return R0(e,f),_R(e,{Fragment:R.Fragment,components:a,ignoreInvalidStyle:!0,jsx:R.jsx,jsxs:R.jsxs,passKeys:!0,passNode:!0});function f(m,p,b){if(m.type==="raw"&&b&&typeof p=="number")return o?b.children.splice(p,1):b.children[p]={type:"text",value:m.value},p;if(m.type==="element"){let y;for(y in L1)if(Object.hasOwn(L1,y)&&Object.hasOwn(m.properties,y)){const T=m.properties[y],x=L1[y];(x===null||x.includes(m.tagName))&&(m.properties[y]=d(String(T||""),y,m))}}if(m.type==="element"){let y=n?!n.includes(m.tagName):s?s.includes(m.tagName):!1;if(!y&&r&&typeof p=="number"&&(y=!r(m,p,b)),y&&b&&typeof p=="number")return c&&m.children?b.children.splice(p,1,...m.children):b.children.splice(p,1),p}}}function iL(e){const t=e.indexOf(":"),n=e.indexOf("?"),r=e.indexOf("#"),a=e.indexOf("/");return t===-1||a!==-1&&t>a||n!==-1&&t>n||r!==-1&&t>r||JD.test(e.slice(0,t))?e:""}function fE(e,t){const n=String(e);if(typeof t!="string")throw new TypeError("Expected character");let r=0,a=n.indexOf(t);for(;a!==-1;)r++,a=n.indexOf(t,a+t.length);return r}function sL(e){if(typeof e!="string")throw new TypeError("Expected a string");return e.replace(/[|\\{}()[\]^$+*?.]/g,"\\$&").replace(/-/g,"\\x2d")}function lL(e,t,n){const a=O0((n||{}).ignore||[]),s=oL(t);let o=-1;for(;++o<s.length;)x2(e,"text",c);function c(f,m){let p=-1,b;for(;++p<m.length;){const y=m[p],T=b?b.children:void 0;if(a(y,T?T.indexOf(y):void 0,b))return;b=y}if(b)return d(f,m)}function d(f,m){const p=m[m.length-1],b=s[o][0],y=s[o][1];let T=0;const C=p.children.indexOf(f);let w=!1,O=[];b.lastIndex=0;let D=b.exec(f.value);for(;D;){const Y=D.index,H={index:D.index,input:D.input,stack:[...m,f]};let B=y(...D,H);if(typeof B=="string"&&(B=B.length>0?{type:"text",value:B}:void 0),B===!1?b.lastIndex=Y+1:(T!==Y&&O.push({type:"text",value:f.value.slice(T,Y)}),Array.isArray(B)?O.push(...B):B&&O.push(B),T=Y+D[0].length,w=!0),!b.global)break;D=b.exec(f.value)}return w?(T<f.value.length&&O.push({type:"text",value:f.value.slice(T)}),p.children.splice(C,1,...O)):O=[f],C+O.length}}function oL(e){const t=[];if(!Array.isArray(e))throw new TypeError("Expected find and replace tuple or list of tuples");const n=!e[0]||Array.isArray(e[0])?e:[e];let r=-1;for(;++r<n.length;){const a=n[r];t.push([uL(a[0]),cL(a[1])])}return t}function uL(e){return typeof e=="string"?new RegExp(sL(e),"g"):e}function cL(e){return typeof e=="function"?e:function(){return e}}const G1="phrasing",V1=["autolink","link","image","label"];function dL(){return{transforms:[vL],enter:{literalAutolink:fL,literalAutolinkEmail:K1,literalAutolinkHttp:K1,literalAutolinkWww:K1},exit:{literalAutolink:bL,literalAutolinkEmail:gL,literalAutolinkHttp:mL,literalAutolinkWww:pL}}}function hL(){return{unsafe:[{character:"@",before:"[+\\-.\\w]",after:"[\\-.\\w]",inConstruct:G1,notInConstruct:V1},{character:".",before:"[Ww]",after:"[\\-.\\w]",inConstruct:G1,notInConstruct:V1},{character:":",before:"[ps]",after:"\\/",inConstruct:G1,notInConstruct:V1}]}}function fL(e){this.enter({type:"link",title:null,url:"",children:[]},e)}function K1(e){this.config.enter.autolinkProtocol.call(this,e)}function mL(e){this.config.exit.autolinkProtocol.call(this,e)}function pL(e){this.config.exit.data.call(this,e);const t=this.stack[this.stack.length-1];t.type,t.url="http://"+this.sliceSerialize(e)}function gL(e){this.config.exit.autolinkEmail.call(this,e)}function bL(e){this.exit(e)}function vL(e){lL(e,[[/(https?:\/\/|www(?=\.))([-.\w]+)([^ \t\r\n]*)/gi,yL],[new RegExp("(?<=^|\\s|\\p{P}|\\p{S})([-.\\w+]+)@([-\\w]+(?:\\.[-\\w]+)+)","gu"),_L]],{ignore:["link","linkReference"]})}function yL(e,t,n,r,a){let s="";if(!k7(a)||(/^w/i.test(t)&&(n=t+n,t="",s="http://"),!EL(n)))return!1;const o=TL(n+r);if(!o[0])return!1;const c={type:"link",title:null,url:s+t+o[0],children:[{type:"text",value:t+o[0]}]};return o[1]?[c,{type:"text",value:o[1]}]:c}function _L(e,t,n,r){return!k7(r,!0)||/[-\d_]$/.test(n)?!1:{type:"link",title:null,url:"mailto:"+t+"@"+n,children:[{type:"text",value:t+"@"+n}]}}function EL(e){const t=e.split(".");return!(t.length<2||t[t.length-1]&&(/_/.test(t[t.length-1])||!/[a-zA-Z\d]/.test(t[t.length-1]))||t[t.length-2]&&(/_/.test(t[t.length-2])||!/[a-zA-Z\d]/.test(t[t.length-2])))}function TL(e){const t=/[!"&'),.:;<>?\]}]+$/.exec(e);if(!t)return[e,void 0];e=e.slice(0,t.index);let n=t[0],r=n.indexOf(")");const a=fE(e,"(");let s=fE(e,")");for(;r!==-1&&a>s;)e+=n.slice(0,r+1),n=n.slice(r+1),r=n.indexOf(")"),s++;return[e,n]}function k7(e,t){const n=e.input.charCodeAt(e.index-1);return(e.index===0||Yl(n)||Yf(n))&&(!t||n!==47)}C7.peek=RL;function xL(){this.buffer()}function SL(e){this.enter({type:"footnoteReference",identifier:"",label:""},e)}function wL(){this.buffer()}function AL(e){this.enter({type:"footnoteDefinition",identifier:"",label:"",children:[]},e)}function kL(e){const t=this.resume(),n=this.stack[this.stack.length-1];n.type,n.identifier=Qa(this.sliceSerialize(e)).toLowerCase(),n.label=t}function CL(e){this.exit(e)}function NL(e){const t=this.resume(),n=this.stack[this.stack.length-1];n.type,n.identifier=Qa(this.sliceSerialize(e)).toLowerCase(),n.label=t}function OL(e){this.exit(e)}function RL(){return"["}function C7(e,t,n,r){const a=n.createTracker(r);let s=a.move("[^");const o=n.enter("footnoteReference"),c=n.enter("reference");return s+=a.move(n.safe(n.associationId(e),{after:"]",before:s})),c(),o(),s+=a.move("]"),s}function ML(){return{enter:{gfmFootnoteCallString:xL,gfmFootnoteCall:SL,gfmFootnoteDefinitionLabelString:wL,gfmFootnoteDefinition:AL},exit:{gfmFootnoteCallString:kL,gfmFootnoteCall:CL,gfmFootnoteDefinitionLabelString:NL,gfmFootnoteDefinition:OL}}}function IL(e){let t=!1;return e&&e.firstLineBlank&&(t=!0),{handlers:{footnoteDefinition:n,footnoteReference:C7},unsafe:[{character:"[",inConstruct:["label","phrasing","reference"]}]};function n(r,a,s,o){const c=s.createTracker(o);let d=c.move("[^");const f=s.enter("footnoteDefinition"),m=s.enter("label");return d+=c.move(s.safe(s.associationId(r),{before:d,after:"]"})),m(),d+=c.move("]:"),r.children&&r.children.length>0&&(c.shift(4),d+=c.move((t?`
`:" ")+s.indentLines(s.containerFlow(r,c.current()),t?N7:DL))),f(),d}}function DL(e,t,n){return t===0?e:N7(e,t,n)}function N7(e,t,n){return(n?"":"    ")+e}const LL=["autolink","destinationLiteral","destinationRaw","reference","titleQuote","titleApostrophe"];O7.peek=zL;function PL(){return{canContainEols:["delete"],enter:{strikethrough:UL},exit:{strikethrough:FL}}}function BL(){return{unsafe:[{character:"~",inConstruct:"phrasing",notInConstruct:LL}],handlers:{delete:O7}}}function UL(e){this.enter({type:"delete",children:[]},e)}function FL(e){this.exit(e)}function O7(e,t,n,r){const a=n.createTracker(r),s=n.enter("strikethrough");let o=a.move("~~");return o+=n.containerPhrasing(e,{...a.current(),before:o,after:"~"}),o+=a.move("~~"),s(),o}function zL(){return"~"}function HL(e){return e.length}function qL(e,t){const n=t||{},r=(n.align||[]).concat(),a=n.stringLength||HL,s=[],o=[],c=[],d=[];let f=0,m=-1;for(;++m<e.length;){const x=[],C=[];let w=-1;for(e[m].length>f&&(f=e[m].length);++w<e[m].length;){const O=jL(e[m][w]);if(n.alignDelimiters!==!1){const D=a(O);C[w]=D,(d[w]===void 0||D>d[w])&&(d[w]=D)}x.push(O)}o[m]=x,c[m]=C}let p=-1;if(typeof r=="object"&&"length"in r)for(;++p<f;)s[p]=mE(r[p]);else{const x=mE(r);for(;++p<f;)s[p]=x}p=-1;const b=[],y=[];for(;++p<f;){const x=s[p];let C="",w="";x===99?(C=":",w=":"):x===108?C=":":x===114&&(w=":");let O=n.alignDelimiters===!1?1:Math.max(1,d[p]-C.length-w.length);const D=C+"-".repeat(O)+w;n.alignDelimiters!==!1&&(O=C.length+O+w.length,O>d[p]&&(d[p]=O),y[p]=O),b[p]=D}o.splice(1,0,b),c.splice(1,0,y),m=-1;const T=[];for(;++m<o.length;){const x=o[m],C=c[m];p=-1;const w=[];for(;++p<f;){const O=x[p]||"";let D="",Y="";if(n.alignDelimiters!==!1){const H=d[p]-(C[p]||0),B=s[p];B===114?D=" ".repeat(H):B===99?H%2?(D=" ".repeat(H/2+.5),Y=" ".repeat(H/2-.5)):(D=" ".repeat(H/2),Y=D):Y=" ".repeat(H)}n.delimiterStart!==!1&&!p&&w.push("|"),n.padding!==!1&&!(n.alignDelimiters===!1&&O==="")&&(n.delimiterStart!==!1||p)&&w.push(" "),n.alignDelimiters!==!1&&w.push(D),w.push(O),n.alignDelimiters!==!1&&w.push(Y),n.padding!==!1&&w.push(" "),(n.delimiterEnd!==!1||p!==f-1)&&w.push("|")}T.push(n.delimiterEnd===!1?w.join("").replace(/ +$/,""):w.join(""))}return T.join(`
`)}function jL(e){return e==null?"":String(e)}function mE(e){const t=typeof e=="string"?e.codePointAt(0):0;return t===67||t===99?99:t===76||t===108?108:t===82||t===114?114:0}const pE={}.hasOwnProperty;function R7(e,t){const n=t||{};function r(a,...s){let o=r.invalid;const c=r.handlers;if(a&&pE.call(a,e)){const d=String(a[e]);o=pE.call(c,d)?c[d]:r.unknown}if(o)return o.call(this,a,...s)}return r.handlers=n.handlers||{},r.invalid=n.invalid,r.unknown=n.unknown,r}function $L(e,t,n,r){const a=n.enter("blockquote"),s=n.createTracker(r);s.move("> "),s.shift(2);const o=n.indentLines(n.containerFlow(e,s.current()),YL);return a(),o}function YL(e,t,n){return">"+(n?"":" ")+e}function GL(e,t){return gE(e,t.inConstruct,!0)&&!gE(e,t.notInConstruct,!1)}function gE(e,t,n){if(typeof t=="string"&&(t=[t]),!t||t.length===0)return n;let r=-1;for(;++r<t.length;)if(e.includes(t[r]))return!0;return!1}function bE(e,t,n,r){let a=-1;for(;++a<n.unsafe.length;)if(n.unsafe[a].character===`
`&&GL(n.stack,n.unsafe[a]))return/[ \t]/.test(r.before)?"":" ";return`\\
`}function M7(e,t){const n=String(e);let r=n.indexOf(t),a=r,s=0,o=0;if(typeof t!="string")throw new TypeError("Expected substring");for(;r!==-1;)r===a?++s>o&&(o=s):s=1,a=r+t.length,r=n.indexOf(t,a);return o}function VL(e,t){return!!(t.options.fences===!1&&e.value&&!e.lang&&/[^ \r\n]/.test(e.value)&&!/^[\t ]*(?:[\r\n]|$)|(?:^|[\r\n])[\t ]*$/.test(e.value))}function KL(e){const t=e.options.fence||"`";if(t!=="`"&&t!=="~")throw new Error("Cannot serialize code with `"+t+"` for `options.fence`, expected `` ` `` or `~`");return t}function WL(e,t,n,r){const a=KL(n),s=e.value||"",o=a==="`"?"GraveAccent":"Tilde";if(VL(e,n)){const p=n.enter("codeIndented"),b=n.indentLines(s,XL);return p(),b}const c=n.createTracker(r),d=a.repeat(Math.max(M7(s,a)+1,3)),f=n.enter("codeFenced");let m=c.move(d);if(e.lang){const p=n.enter(`codeFencedLang${o}`);m+=c.move(n.safe(e.lang,{before:m,after:" ",encode:["`"],...c.current()})),p()}if(e.lang&&e.meta){const p=n.enter(`codeFencedMeta${o}`);m+=c.move(" "),m+=c.move(n.safe(e.meta,{before:m,after:`
`,encode:["`"],...c.current()})),p()}return m+=c.move(`
`),s&&(m+=c.move(s+`
`)),m+=c.move(d),f(),m}function XL(e,t,n){return(n?"":"    ")+e}function w2(e){const t=e.options.quote||'"';if(t!=='"'&&t!=="'")throw new Error("Cannot serialize title with `"+t+"` for `options.quote`, expected `\"`, or `'`");return t}function QL(e,t,n,r){const a=w2(n),s=a==='"'?"Quote":"Apostrophe",o=n.enter("definition");let c=n.enter("label");const d=n.createTracker(r);let f=d.move("[");return f+=d.move(n.safe(n.associationId(e),{before:f,after:"]",...d.current()})),f+=d.move("]: "),c(),!e.url||/[\0- \u007F]/.test(e.url)?(c=n.enter("destinationLiteral"),f+=d.move("<"),f+=d.move(n.safe(e.url,{before:f,after:">",...d.current()})),f+=d.move(">")):(c=n.enter("destinationRaw"),f+=d.move(n.safe(e.url,{before:f,after:e.title?" ":`
`,...d.current()}))),c(),e.title&&(c=n.enter(`title${s}`),f+=d.move(" "+a),f+=d.move(n.safe(e.title,{before:f,after:a,...d.current()})),f+=d.move(a),c()),o(),f}function ZL(e){const t=e.options.emphasis||"*";if(t!=="*"&&t!=="_")throw new Error("Cannot serialize emphasis with `"+t+"` for `options.emphasis`, expected `*`, or `_`");return t}function f0(e){return"&#x"+e.toString(16).toUpperCase()+";"}function nf(e,t,n){const r=eu(e),a=eu(t);return r===void 0?a===void 0?n==="_"?{inside:!0,outside:!0}:{inside:!1,outside:!1}:a===1?{inside:!0,outside:!0}:{inside:!1,outside:!0}:r===1?a===void 0?{inside:!1,outside:!1}:a===1?{inside:!0,outside:!0}:{inside:!1,outside:!1}:a===void 0?{inside:!1,outside:!1}:a===1?{inside:!0,outside:!1}:{inside:!1,outside:!1}}I7.peek=JL;function I7(e,t,n,r){const a=ZL(n),s=n.enter("emphasis"),o=n.createTracker(r),c=o.move(a);let d=o.move(n.containerPhrasing(e,{after:a,before:c,...o.current()}));const f=d.charCodeAt(0),m=nf(r.before.charCodeAt(r.before.length-1),f,a);m.inside&&(d=f0(f)+d.slice(1));const p=d.charCodeAt(d.length-1),b=nf(r.after.charCodeAt(0),p,a);b.inside&&(d=d.slice(0,-1)+f0(p));const y=o.move(a);return s(),n.attentionEncodeSurroundingInfo={after:b.outside,before:m.outside},c+d+y}function JL(e,t,n){return n.options.emphasis||"*"}function eP(e,t){let n=!1;return R0(e,function(r){if("value"in r&&/\r?\n|\r/.test(r.value)||r.type==="break")return n=!0,rb}),!!((!e.depth||e.depth<3)&&g2(e)&&(t.options.setext||n))}function tP(e,t,n,r){const a=Math.max(Math.min(6,e.depth||1),1),s=n.createTracker(r);if(eP(e,n)){const m=n.enter("headingSetext"),p=n.enter("phrasing"),b=n.containerPhrasing(e,{...s.current(),before:`
`,after:`
`});return p(),m(),b+`
`+(a===1?"=":"-").repeat(b.length-(Math.max(b.lastIndexOf("\r"),b.lastIndexOf(`
`))+1))}const o="#".repeat(a),c=n.enter("headingAtx"),d=n.enter("phrasing");s.move(o+" ");let f=n.containerPhrasing(e,{before:"# ",after:`
`,...s.current()});return/^[\t ]/.test(f)&&(f=f0(f.charCodeAt(0))+f.slice(1)),f=f?o+" "+f:o,n.options.closeAtx&&(f+=" "+o),d(),c(),f}D7.peek=nP;function D7(e){return e.value||""}function nP(){return"<"}L7.peek=rP;function L7(e,t,n,r){const a=w2(n),s=a==='"'?"Quote":"Apostrophe",o=n.enter("image");let c=n.enter("label");const d=n.createTracker(r);let f=d.move("![");return f+=d.move(n.safe(e.alt,{before:f,after:"]",...d.current()})),f+=d.move("]("),c(),!e.url&&e.title||/[\0- \u007F]/.test(e.url)?(c=n.enter("destinationLiteral"),f+=d.move("<"),f+=d.move(n.safe(e.url,{before:f,after:">",...d.current()})),f+=d.move(">")):(c=n.enter("destinationRaw"),f+=d.move(n.safe(e.url,{before:f,after:e.title?" ":")",...d.current()}))),c(),e.title&&(c=n.enter(`title${s}`),f+=d.move(" "+a),f+=d.move(n.safe(e.title,{before:f,after:a,...d.current()})),f+=d.move(a),c()),f+=d.move(")"),o(),f}function rP(){return"!"}P7.peek=aP;function P7(e,t,n,r){const a=e.referenceType,s=n.enter("imageReference");let o=n.enter("label");const c=n.createTracker(r);let d=c.move("![");const f=n.safe(e.alt,{before:d,after:"]",...c.current()});d+=c.move(f+"]["),o();const m=n.stack;n.stack=[],o=n.enter("reference");const p=n.safe(n.associationId(e),{before:d,after:"]",...c.current()});return o(),n.stack=m,s(),a==="full"||!f||f!==p?d+=c.move(p+"]"):a==="shortcut"?d=d.slice(0,-1):d+=c.move("]"),d}function aP(){return"!"}B7.peek=iP;function B7(e,t,n){let r=e.value||"",a="`",s=-1;for(;new RegExp("(^|[^`])"+a+"([^`]|$)").test(r);)a+="`";for(/[^ \r\n]/.test(r)&&(/^[ \r\n]/.test(r)&&/[ \r\n]$/.test(r)||/^`|`$/.test(r))&&(r=" "+r+" ");++s<n.unsafe.length;){const o=n.unsafe[s],c=n.compilePattern(o);let d;if(o.atBreak)for(;d=c.exec(r);){let f=d.index;r.charCodeAt(f)===10&&r.charCodeAt(f-1)===13&&f--,r=r.slice(0,f)+" "+r.slice(d.index+1)}}return a+r+a}function iP(){return"`"}function U7(e,t){const n=g2(e);return!!(!t.options.resourceLink&&e.url&&!e.title&&e.children&&e.children.length===1&&e.children[0].type==="text"&&(n===e.url||"mailto:"+n===e.url)&&/^[a-z][a-z+.-]+:/i.test(e.url)&&!/[\0- <>\u007F]/.test(e.url))}F7.peek=sP;function F7(e,t,n,r){const a=w2(n),s=a==='"'?"Quote":"Apostrophe",o=n.createTracker(r);let c,d;if(U7(e,n)){const m=n.stack;n.stack=[],c=n.enter("autolink");let p=o.move("<");return p+=o.move(n.containerPhrasing(e,{before:p,after:">",...o.current()})),p+=o.move(">"),c(),n.stack=m,p}c=n.enter("link"),d=n.enter("label");let f=o.move("[");return f+=o.move(n.containerPhrasing(e,{before:f,after:"](",...o.current()})),f+=o.move("]("),d(),!e.url&&e.title||/[\0- \u007F]/.test(e.url)?(d=n.enter("destinationLiteral"),f+=o.move("<"),f+=o.move(n.safe(e.url,{before:f,after:">",...o.current()})),f+=o.move(">")):(d=n.enter("destinationRaw"),f+=o.move(n.safe(e.url,{before:f,after:e.title?" ":")",...o.current()}))),d(),e.title&&(d=n.enter(`title${s}`),f+=o.move(" "+a),f+=o.move(n.safe(e.title,{before:f,after:a,...o.current()})),f+=o.move(a),d()),f+=o.move(")"),c(),f}function sP(e,t,n){return U7(e,n)?"<":"["}z7.peek=lP;function z7(e,t,n,r){const a=e.referenceType,s=n.enter("linkReference");let o=n.enter("label");const c=n.createTracker(r);let d=c.move("[");const f=n.containerPhrasing(e,{before:d,after:"]",...c.current()});d+=c.move(f+"]["),o();const m=n.stack;n.stack=[],o=n.enter("reference");const p=n.safe(n.associationId(e),{before:d,after:"]",...c.current()});return o(),n.stack=m,s(),a==="full"||!f||f!==p?d+=c.move(p+"]"):a==="shortcut"?d=d.slice(0,-1):d+=c.move("]"),d}function lP(){return"["}function A2(e){const t=e.options.bullet||"*";if(t!=="*"&&t!=="+"&&t!=="-")throw new Error("Cannot serialize items with `"+t+"` for `options.bullet`, expected `*`, `+`, or `-`");return t}function oP(e){const t=A2(e),n=e.options.bulletOther;if(!n)return t==="*"?"-":"*";if(n!=="*"&&n!=="+"&&n!=="-")throw new Error("Cannot serialize items with `"+n+"` for `options.bulletOther`, expected `*`, `+`, or `-`");if(n===t)throw new Error("Expected `bullet` (`"+t+"`) and `bulletOther` (`"+n+"`) to be different");return n}function uP(e){const t=e.options.bulletOrdered||".";if(t!=="."&&t!==")")throw new Error("Cannot serialize items with `"+t+"` for `options.bulletOrdered`, expected `.` or `)`");return t}function H7(e){const t=e.options.rule||"*";if(t!=="*"&&t!=="-"&&t!=="_")throw new Error("Cannot serialize rules with `"+t+"` for `options.rule`, expected `*`, `-`, or `_`");return t}function cP(e,t,n,r){const a=n.enter("list"),s=n.bulletCurrent;let o=e.ordered?uP(n):A2(n);const c=e.ordered?o==="."?")":".":oP(n);let d=t&&n.bulletLastUsed?o===n.bulletLastUsed:!1;if(!e.ordered){const m=e.children?e.children[0]:void 0;if((o==="*"||o==="-")&&m&&(!m.children||!m.children[0])&&n.stack[n.stack.length-1]==="list"&&n.stack[n.stack.length-2]==="listItem"&&n.stack[n.stack.length-3]==="list"&&n.stack[n.stack.length-4]==="listItem"&&n.indexStack[n.indexStack.length-1]===0&&n.indexStack[n.indexStack.length-2]===0&&n.indexStack[n.indexStack.length-3]===0&&(d=!0),H7(n)===o&&m){let p=-1;for(;++p<e.children.length;){const b=e.children[p];if(b&&b.type==="listItem"&&b.children&&b.children[0]&&b.children[0].type==="thematicBreak"){d=!0;break}}}}d&&(o=c),n.bulletCurrent=o;const f=n.containerFlow(e,r);return n.bulletLastUsed=o,n.bulletCurrent=s,a(),f}function dP(e){const t=e.options.listItemIndent||"one";if(t!=="tab"&&t!=="one"&&t!=="mixed")throw new Error("Cannot serialize items with `"+t+"` for `options.listItemIndent`, expected `tab`, `one`, or `mixed`");return t}function hP(e,t,n,r){const a=dP(n);let s=n.bulletCurrent||A2(n);t&&t.type==="list"&&t.ordered&&(s=(typeof t.start=="number"&&t.start>-1?t.start:1)+(n.options.incrementListMarker===!1?0:t.children.indexOf(e))+s);let o=s.length+1;(a==="tab"||a==="mixed"&&(t&&t.type==="list"&&t.spread||e.spread))&&(o=Math.ceil(o/4)*4);const c=n.createTracker(r);c.move(s+" ".repeat(o-s.length)),c.shift(o);const d=n.enter("listItem"),f=n.indentLines(n.containerFlow(e,c.current()),m);return d(),f;function m(p,b,y){return b?(y?"":" ".repeat(o))+p:(y?s:s+" ".repeat(o-s.length))+p}}function fP(e,t,n,r){const a=n.enter("paragraph"),s=n.enter("phrasing"),o=n.containerPhrasing(e,r);return s(),a(),o}const mP=O0(["break","delete","emphasis","footnote","footnoteReference","image","imageReference","inlineCode","inlineMath","link","linkReference","mdxJsxTextElement","mdxTextExpression","strong","text","textDirective"]);function pP(e,t,n,r){return(e.children.some(function(o){return mP(o)})?n.containerPhrasing:n.containerFlow).call(n,e,r)}function gP(e){const t=e.options.strong||"*";if(t!=="*"&&t!=="_")throw new Error("Cannot serialize strong with `"+t+"` for `options.strong`, expected `*`, or `_`");return t}q7.peek=bP;function q7(e,t,n,r){const a=gP(n),s=n.enter("strong"),o=n.createTracker(r),c=o.move(a+a);let d=o.move(n.containerPhrasing(e,{after:a,before:c,...o.current()}));const f=d.charCodeAt(0),m=nf(r.before.charCodeAt(r.before.length-1),f,a);m.inside&&(d=f0(f)+d.slice(1));const p=d.charCodeAt(d.length-1),b=nf(r.after.charCodeAt(0),p,a);b.inside&&(d=d.slice(0,-1)+f0(p));const y=o.move(a+a);return s(),n.attentionEncodeSurroundingInfo={after:b.outside,before:m.outside},c+d+y}function bP(e,t,n){return n.options.strong||"*"}function vP(e,t,n,r){return n.safe(e.value,r)}function yP(e){const t=e.options.ruleRepetition||3;if(t<3)throw new Error("Cannot serialize rules with repetition `"+t+"` for `options.ruleRepetition`, expected `3` or more");return t}function _P(e,t,n){const r=(H7(n)+(n.options.ruleSpaces?" ":"")).repeat(yP(n));return n.options.ruleSpaces?r.slice(0,-1):r}const j7={blockquote:$L,break:bE,code:WL,definition:QL,emphasis:I7,hardBreak:bE,heading:tP,html:D7,image:L7,imageReference:P7,inlineCode:B7,link:F7,linkReference:z7,list:cP,listItem:hP,paragraph:fP,root:pP,strong:q7,text:vP,thematicBreak:_P};function EP(){return{enter:{table:TP,tableData:vE,tableHeader:vE,tableRow:SP},exit:{codeText:wP,table:xP,tableData:W1,tableHeader:W1,tableRow:W1}}}function TP(e){const t=e._align;this.enter({type:"table",align:t.map(function(n){return n==="none"?null:n}),children:[]},e),this.data.inTable=!0}function xP(e){this.exit(e),this.data.inTable=void 0}function SP(e){this.enter({type:"tableRow",children:[]},e)}function W1(e){this.exit(e)}function vE(e){this.enter({type:"tableCell",children:[]},e)}function wP(e){let t=this.resume();this.data.inTable&&(t=t.replace(/\\([\\|])/g,AP));const n=this.stack[this.stack.length-1];n.type,n.value=t,this.exit(e)}function AP(e,t){return t==="|"?t:e}function kP(e){const t=e||{},n=t.tableCellPadding,r=t.tablePipeAlign,a=t.stringLength,s=n?" ":"|";return{unsafe:[{character:"\r",inConstruct:"tableCell"},{character:`
`,inConstruct:"tableCell"},{atBreak:!0,character:"|",after:"[	 :-]"},{character:"|",inConstruct:"tableCell"},{atBreak:!0,character:":",after:"-"},{atBreak:!0,character:"-",after:"[:|-]"}],handlers:{inlineCode:b,table:o,tableCell:d,tableRow:c}};function o(y,T,x,C){return f(m(y,x,C),y.align)}function c(y,T,x,C){const w=p(y,x,C),O=f([w]);return O.slice(0,O.indexOf(`
`))}function d(y,T,x,C){const w=x.enter("tableCell"),O=x.enter("phrasing"),D=x.containerPhrasing(y,{...C,before:s,after:s});return O(),w(),D}function f(y,T){return qL(y,{align:T,alignDelimiters:r,padding:n,stringLength:a})}function m(y,T,x){const C=y.children;let w=-1;const O=[],D=T.enter("table");for(;++w<C.length;)O[w]=p(C[w],T,x);return D(),O}function p(y,T,x){const C=y.children;let w=-1;const O=[],D=T.enter("tableRow");for(;++w<C.length;)O[w]=d(C[w],y,T,x);return D(),O}function b(y,T,x){let C=j7.inlineCode(y,T,x);return x.stack.includes("tableCell")&&(C=C.replace(/\|/g,"\\$&")),C}}function CP(){return{exit:{taskListCheckValueChecked:yE,taskListCheckValueUnchecked:yE,paragraph:OP}}}function NP(){return{unsafe:[{atBreak:!0,character:"-",after:"[:|-]"}],handlers:{listItem:RP}}}function yE(e){const t=this.stack[this.stack.length-2];t.type,t.checked=e.type==="taskListCheckValueChecked"}function OP(e){const t=this.stack[this.stack.length-2];if(t&&t.type==="listItem"&&typeof t.checked=="boolean"){const n=this.stack[this.stack.length-1];n.type;const r=n.children[0];if(r&&r.type==="text"){const a=t.children;let s=-1,o;for(;++s<a.length;){const c=a[s];if(c.type==="paragraph"){o=c;break}}o===n&&(r.value=r.value.slice(1),r.value.length===0?n.children.shift():n.position&&r.position&&typeof r.position.start.offset=="number"&&(r.position.start.column++,r.position.start.offset++,n.position.start=Object.assign({},r.position.start)))}}this.exit(e)}function RP(e,t,n,r){const a=e.children[0],s=typeof e.checked=="boolean"&&a&&a.type==="paragraph",o="["+(e.checked?"x":" ")+"] ",c=n.createTracker(r);s&&c.move(o);let d=j7.listItem(e,t,n,{...r,...c.current()});return s&&(d=d.replace(/^(?:[*+-]|\d+\.)([\r\n]| {1,3})/,f)),d;function f(m){return m+o}}function MP(){return[dL(),ML(),PL(),EP(),CP()]}function IP(e){return{extensions:[hL(),IL(e),BL(),kP(e),NP()]}}const DP={tokenize:zP,partial:!0},$7={tokenize:HP,partial:!0},Y7={tokenize:qP,partial:!0},G7={tokenize:jP,partial:!0},LP={tokenize:$P,partial:!0},V7={name:"wwwAutolink",tokenize:UP,previous:W7},K7={name:"protocolAutolink",tokenize:FP,previous:X7},us={name:"emailAutolink",tokenize:BP,previous:Q7},_i={};function PP(){return{text:_i}}let Ll=48;for(;Ll<123;)_i[Ll]=us,Ll++,Ll===58?Ll=65:Ll===91&&(Ll=97);_i[43]=us;_i[45]=us;_i[46]=us;_i[95]=us;_i[72]=[us,K7];_i[104]=[us,K7];_i[87]=[us,V7];_i[119]=[us,V7];function BP(e,t,n){const r=this;let a,s;return o;function o(p){return!lb(p)||!Q7.call(r,r.previous)||k2(r.events)?n(p):(e.enter("literalAutolink"),e.enter("literalAutolinkEmail"),c(p))}function c(p){return lb(p)?(e.consume(p),c):p===64?(e.consume(p),d):n(p)}function d(p){return p===46?e.check(LP,m,f)(p):p===45||p===95||kr(p)?(s=!0,e.consume(p),d):m(p)}function f(p){return e.consume(p),a=!0,d}function m(p){return s&&a&&zr(r.previous)?(e.exit("literalAutolinkEmail"),e.exit("literalAutolink"),t(p)):n(p)}}function UP(e,t,n){const r=this;return a;function a(o){return o!==87&&o!==119||!W7.call(r,r.previous)||k2(r.events)?n(o):(e.enter("literalAutolink"),e.enter("literalAutolinkWww"),e.check(DP,e.attempt($7,e.attempt(Y7,s),n),n)(o))}function s(o){return e.exit("literalAutolinkWww"),e.exit("literalAutolink"),t(o)}}function FP(e,t,n){const r=this;let a="",s=!1;return o;function o(p){return(p===72||p===104)&&X7.call(r,r.previous)&&!k2(r.events)?(e.enter("literalAutolink"),e.enter("literalAutolinkHttp"),a+=String.fromCodePoint(p),e.consume(p),c):n(p)}function c(p){if(zr(p)&&a.length<5)return a+=String.fromCodePoint(p),e.consume(p),c;if(p===58){const b=a.toLowerCase();if(b==="http"||b==="https")return e.consume(p),d}return n(p)}function d(p){return p===47?(e.consume(p),s?f:(s=!0,d)):n(p)}function f(p){return p===null||ef(p)||an(p)||Yl(p)||Yf(p)?n(p):e.attempt($7,e.attempt(Y7,m),n)(p)}function m(p){return e.exit("literalAutolinkHttp"),e.exit("literalAutolink"),t(p)}}function zP(e,t,n){let r=0;return a;function a(o){return(o===87||o===119)&&r<3?(r++,e.consume(o),a):o===46&&r===3?(e.consume(o),s):n(o)}function s(o){return o===null?n(o):t(o)}}function HP(e,t,n){let r,a,s;return o;function o(f){return f===46||f===95?e.check(G7,d,c)(f):f===null||an(f)||Yl(f)||f!==45&&Yf(f)?d(f):(s=!0,e.consume(f),o)}function c(f){return f===95?r=!0:(a=r,r=void 0),e.consume(f),o}function d(f){return a||r||!s?n(f):t(f)}}function qP(e,t){let n=0,r=0;return a;function a(o){return o===40?(n++,e.consume(o),a):o===41&&r<n?s(o):o===33||o===34||o===38||o===39||o===41||o===42||o===44||o===46||o===58||o===59||o===60||o===63||o===93||o===95||o===126?e.check(G7,t,s)(o):o===null||an(o)||Yl(o)?t(o):(e.consume(o),a)}function s(o){return o===41&&r++,e.consume(o),a}}function jP(e,t,n){return r;function r(c){return c===33||c===34||c===39||c===41||c===42||c===44||c===46||c===58||c===59||c===63||c===95||c===126?(e.consume(c),r):c===38?(e.consume(c),s):c===93?(e.consume(c),a):c===60||c===null||an(c)||Yl(c)?t(c):n(c)}function a(c){return c===null||c===40||c===91||an(c)||Yl(c)?t(c):r(c)}function s(c){return zr(c)?o(c):n(c)}function o(c){return c===59?(e.consume(c),r):zr(c)?(e.consume(c),o):n(c)}}function $P(e,t,n){return r;function r(s){return e.consume(s),a}function a(s){return kr(s)?n(s):t(s)}}function W7(e){return e===null||e===40||e===42||e===95||e===91||e===93||e===126||an(e)}function X7(e){return!zr(e)}function Q7(e){return!(e===47||lb(e))}function lb(e){return e===43||e===45||e===46||e===95||kr(e)}function k2(e){let t=e.length,n=!1;for(;t--;){const r=e[t][1];if((r.type==="labelLink"||r.type==="labelImage")&&!r._balanced){n=!0;break}if(r._gfmAutolinkLiteralWalkedInto){n=!1;break}}return e.length>0&&!n&&(e[e.length-1][1]._gfmAutolinkLiteralWalkedInto=!0),n}const YP={tokenize:JP,partial:!0};function GP(){return{document:{91:{name:"gfmFootnoteDefinition",tokenize:XP,continuation:{tokenize:QP},exit:ZP}},text:{91:{name:"gfmFootnoteCall",tokenize:WP},93:{name:"gfmPotentialFootnoteCall",add:"after",tokenize:VP,resolveTo:KP}}}}function VP(e,t,n){const r=this;let a=r.events.length;const s=r.parser.gfmFootnotes||(r.parser.gfmFootnotes=[]);let o;for(;a--;){const d=r.events[a][1];if(d.type==="labelImage"){o=d;break}if(d.type==="gfmFootnoteCall"||d.type==="labelLink"||d.type==="label"||d.type==="image"||d.type==="link")break}return c;function c(d){if(!o||!o._balanced)return n(d);const f=Qa(r.sliceSerialize({start:o.end,end:r.now()}));return f.codePointAt(0)!==94||!s.includes(f.slice(1))?n(d):(e.enter("gfmFootnoteCallLabelMarker"),e.consume(d),e.exit("gfmFootnoteCallLabelMarker"),t(d))}}function KP(e,t){let n=e.length;for(;n--;)if(e[n][1].type==="labelImage"&&e[n][0]==="enter"){e[n][1];break}e[n+1][1].type="data",e[n+3][1].type="gfmFootnoteCallLabelMarker";const r={type:"gfmFootnoteCall",start:Object.assign({},e[n+3][1].start),end:Object.assign({},e[e.length-1][1].end)},a={type:"gfmFootnoteCallMarker",start:Object.assign({},e[n+3][1].end),end:Object.assign({},e[n+3][1].end)};a.end.column++,a.end.offset++,a.end._bufferIndex++;const s={type:"gfmFootnoteCallString",start:Object.assign({},a.end),end:Object.assign({},e[e.length-1][1].start)},o={type:"chunkString",contentType:"string",start:Object.assign({},s.start),end:Object.assign({},s.end)},c=[e[n+1],e[n+2],["enter",r,t],e[n+3],e[n+4],["enter",a,t],["exit",a,t],["enter",s,t],["enter",o,t],["exit",o,t],["exit",s,t],e[e.length-2],e[e.length-1],["exit",r,t]];return e.splice(n,e.length-n+1,...c),e}function WP(e,t,n){const r=this,a=r.parser.gfmFootnotes||(r.parser.gfmFootnotes=[]);let s=0,o;return c;function c(p){return e.enter("gfmFootnoteCall"),e.enter("gfmFootnoteCallLabelMarker"),e.consume(p),e.exit("gfmFootnoteCallLabelMarker"),d}function d(p){return p!==94?n(p):(e.enter("gfmFootnoteCallMarker"),e.consume(p),e.exit("gfmFootnoteCallMarker"),e.enter("gfmFootnoteCallString"),e.enter("chunkString").contentType="string",f)}function f(p){if(s>999||p===93&&!o||p===null||p===91||an(p))return n(p);if(p===93){e.exit("chunkString");const b=e.exit("gfmFootnoteCallString");return a.includes(Qa(r.sliceSerialize(b)))?(e.enter("gfmFootnoteCallLabelMarker"),e.consume(p),e.exit("gfmFootnoteCallLabelMarker"),e.exit("gfmFootnoteCall"),t):n(p)}return an(p)||(o=!0),s++,e.consume(p),p===92?m:f}function m(p){return p===91||p===92||p===93?(e.consume(p),s++,f):f(p)}}function XP(e,t,n){const r=this,a=r.parser.gfmFootnotes||(r.parser.gfmFootnotes=[]);let s,o=0,c;return d;function d(T){return e.enter("gfmFootnoteDefinition")._container=!0,e.enter("gfmFootnoteDefinitionLabel"),e.enter("gfmFootnoteDefinitionLabelMarker"),e.consume(T),e.exit("gfmFootnoteDefinitionLabelMarker"),f}function f(T){return T===94?(e.enter("gfmFootnoteDefinitionMarker"),e.consume(T),e.exit("gfmFootnoteDefinitionMarker"),e.enter("gfmFootnoteDefinitionLabelString"),e.enter("chunkString").contentType="string",m):n(T)}function m(T){if(o>999||T===93&&!c||T===null||T===91||an(T))return n(T);if(T===93){e.exit("chunkString");const x=e.exit("gfmFootnoteDefinitionLabelString");return s=Qa(r.sliceSerialize(x)),e.enter("gfmFootnoteDefinitionLabelMarker"),e.consume(T),e.exit("gfmFootnoteDefinitionLabelMarker"),e.exit("gfmFootnoteDefinitionLabel"),b}return an(T)||(c=!0),o++,e.consume(T),T===92?p:m}function p(T){return T===91||T===92||T===93?(e.consume(T),o++,m):m(T)}function b(T){return T===58?(e.enter("definitionMarker"),e.consume(T),e.exit("definitionMarker"),a.includes(s)||a.push(s),Nt(e,y,"gfmFootnoteDefinitionWhitespace")):n(T)}function y(T){return t(T)}}function QP(e,t,n){return e.check(N0,t,e.attempt(YP,t,n))}function ZP(e){e.exit("gfmFootnoteDefinition")}function JP(e,t,n){const r=this;return Nt(e,a,"gfmFootnoteDefinitionIndent",5);function a(s){const o=r.events[r.events.length-1];return o&&o[1].type==="gfmFootnoteDefinitionIndent"&&o[2].sliceSerialize(o[1],!0).length===4?t(s):n(s)}}function eB(e){let n=(e||{}).singleTilde;const r={name:"strikethrough",tokenize:s,resolveAll:a};return n==null&&(n=!0),{text:{126:r},insideSpan:{null:[r]},attentionMarkers:{null:[126]}};function a(o,c){let d=-1;for(;++d<o.length;)if(o[d][0]==="enter"&&o[d][1].type==="strikethroughSequenceTemporary"&&o[d][1]._close){let f=d;for(;f--;)if(o[f][0]==="exit"&&o[f][1].type==="strikethroughSequenceTemporary"&&o[f][1]._open&&o[d][1].end.offset-o[d][1].start.offset===o[f][1].end.offset-o[f][1].start.offset){o[d][1].type="strikethroughSequence",o[f][1].type="strikethroughSequence";const m={type:"strikethrough",start:Object.assign({},o[f][1].start),end:Object.assign({},o[d][1].end)},p={type:"strikethroughText",start:Object.assign({},o[f][1].end),end:Object.assign({},o[d][1].start)},b=[["enter",m,c],["enter",o[f][1],c],["exit",o[f][1],c],["enter",p,c]],y=c.parser.constructs.insideSpan.null;y&&_a(b,b.length,0,Gf(y,o.slice(f+1,d),c)),_a(b,b.length,0,[["exit",p,c],["enter",o[d][1],c],["exit",o[d][1],c],["exit",m,c]]),_a(o,f-1,d-f+3,b),d=f+b.length-2;break}}for(d=-1;++d<o.length;)o[d][1].type==="strikethroughSequenceTemporary"&&(o[d][1].type="data");return o}function s(o,c,d){const f=this.previous,m=this.events;let p=0;return b;function b(T){return f===126&&m[m.length-1][1].type!=="characterEscape"?d(T):(o.enter("strikethroughSequenceTemporary"),y(T))}function y(T){const x=eu(f);if(T===126)return p>1?d(T):(o.consume(T),p++,y);if(p<2&&!n)return d(T);const C=o.exit("strikethroughSequenceTemporary"),w=eu(T);return C._open=!w||w===2&&!!x,C._close=!x||x===2&&!!w,c(T)}}}class tB{constructor(){this.map=[]}add(t,n,r){nB(this,t,n,r)}consume(t){if(this.map.sort(function(s,o){return s[0]-o[0]}),this.map.length===0)return;let n=this.map.length;const r=[];for(;n>0;)n-=1,r.push(t.slice(this.map[n][0]+this.map[n][1]),this.map[n][2]),t.length=this.map[n][0];r.push(t.slice()),t.length=0;let a=r.pop();for(;a;){for(const s of a)t.push(s);a=r.pop()}this.map.length=0}}function nB(e,t,n,r){let a=0;if(!(n===0&&r.length===0)){for(;a<e.map.length;){if(e.map[a][0]===t){e.map[a][1]+=n,e.map[a][2].push(...r);return}a+=1}e.map.push([t,n,r])}}function rB(e,t){let n=!1;const r=[];for(;t<e.length;){const a=e[t];if(n){if(a[0]==="enter")a[1].type==="tableContent"&&r.push(e[t+1][1].type==="tableDelimiterMarker"?"left":"none");else if(a[1].type==="tableContent"){if(e[t-1][1].type==="tableDelimiterMarker"){const s=r.length-1;r[s]=r[s]==="left"?"center":"right"}}else if(a[1].type==="tableDelimiterRow")break}else a[0]==="enter"&&a[1].type==="tableDelimiterRow"&&(n=!0);t+=1}return r}function aB(){return{flow:{null:{name:"table",tokenize:iB,resolveAll:sB}}}}function iB(e,t,n){const r=this;let a=0,s=0,o;return c;function c(G){let se=r.events.length-1;for(;se>-1;){const fe=r.events[se][1].type;if(fe==="lineEnding"||fe==="linePrefix")se--;else break}const oe=se>-1?r.events[se][1].type:null,ye=oe==="tableHead"||oe==="tableRow"?B:d;return ye===B&&r.parser.lazy[r.now().line]?n(G):ye(G)}function d(G){return e.enter("tableHead"),e.enter("tableRow"),f(G)}function f(G){return G===124||(o=!0,s+=1),m(G)}function m(G){return G===null?n(G):nt(G)?s>1?(s=0,r.interrupt=!0,e.exit("tableRow"),e.enter("lineEnding"),e.consume(G),e.exit("lineEnding"),y):n(G):Dt(G)?Nt(e,m,"whitespace")(G):(s+=1,o&&(o=!1,a+=1),G===124?(e.enter("tableCellDivider"),e.consume(G),e.exit("tableCellDivider"),o=!0,m):(e.enter("data"),p(G)))}function p(G){return G===null||G===124||an(G)?(e.exit("data"),m(G)):(e.consume(G),G===92?b:p)}function b(G){return G===92||G===124?(e.consume(G),p):p(G)}function y(G){return r.interrupt=!1,r.parser.lazy[r.now().line]?n(G):(e.enter("tableDelimiterRow"),o=!1,Dt(G)?Nt(e,T,"linePrefix",r.parser.constructs.disable.null.includes("codeIndented")?void 0:4)(G):T(G))}function T(G){return G===45||G===58?C(G):G===124?(o=!0,e.enter("tableCellDivider"),e.consume(G),e.exit("tableCellDivider"),x):H(G)}function x(G){return Dt(G)?Nt(e,C,"whitespace")(G):C(G)}function C(G){return G===58?(s+=1,o=!0,e.enter("tableDelimiterMarker"),e.consume(G),e.exit("tableDelimiterMarker"),w):G===45?(s+=1,w(G)):G===null||nt(G)?Y(G):H(G)}function w(G){return G===45?(e.enter("tableDelimiterFiller"),O(G)):H(G)}function O(G){return G===45?(e.consume(G),O):G===58?(o=!0,e.exit("tableDelimiterFiller"),e.enter("tableDelimiterMarker"),e.consume(G),e.exit("tableDelimiterMarker"),D):(e.exit("tableDelimiterFiller"),D(G))}function D(G){return Dt(G)?Nt(e,Y,"whitespace")(G):Y(G)}function Y(G){return G===124?T(G):G===null||nt(G)?!o||a!==s?H(G):(e.exit("tableDelimiterRow"),e.exit("tableHead"),t(G)):H(G)}function H(G){return n(G)}function B(G){return e.enter("tableRow"),ee(G)}function ee(G){return G===124?(e.enter("tableCellDivider"),e.consume(G),e.exit("tableCellDivider"),ee):G===null||nt(G)?(e.exit("tableRow"),t(G)):Dt(G)?Nt(e,ee,"whitespace")(G):(e.enter("data"),Z(G))}function Z(G){return G===null||G===124||an(G)?(e.exit("data"),ee(G)):(e.consume(G),G===92?Q:Z)}function Q(G){return G===92||G===124?(e.consume(G),Z):Z(G)}}function sB(e,t){let n=-1,r=!0,a=0,s=[0,0,0,0],o=[0,0,0,0],c=!1,d=0,f,m,p;const b=new tB;for(;++n<e.length;){const y=e[n],T=y[1];y[0]==="enter"?T.type==="tableHead"?(c=!1,d!==0&&(_E(b,t,d,f,m),m=void 0,d=0),f={type:"table",start:Object.assign({},T.start),end:Object.assign({},T.end)},b.add(n,0,[["enter",f,t]])):T.type==="tableRow"||T.type==="tableDelimiterRow"?(r=!0,p=void 0,s=[0,0,0,0],o=[0,n+1,0,0],c&&(c=!1,m={type:"tableBody",start:Object.assign({},T.start),end:Object.assign({},T.end)},b.add(n,0,[["enter",m,t]])),a=T.type==="tableDelimiterRow"?2:m?3:1):a&&(T.type==="data"||T.type==="tableDelimiterMarker"||T.type==="tableDelimiterFiller")?(r=!1,o[2]===0&&(s[1]!==0&&(o[0]=o[1],p=ch(b,t,s,a,void 0,p),s=[0,0,0,0]),o[2]=n)):T.type==="tableCellDivider"&&(r?r=!1:(s[1]!==0&&(o[0]=o[1],p=ch(b,t,s,a,void 0,p)),s=o,o=[s[1],n,0,0])):T.type==="tableHead"?(c=!0,d=n):T.type==="tableRow"||T.type==="tableDelimiterRow"?(d=n,s[1]!==0?(o[0]=o[1],p=ch(b,t,s,a,n,p)):o[1]!==0&&(p=ch(b,t,o,a,n,p)),a=0):a&&(T.type==="data"||T.type==="tableDelimiterMarker"||T.type==="tableDelimiterFiller")&&(o[3]=n)}for(d!==0&&_E(b,t,d,f,m),b.consume(t.events),n=-1;++n<t.events.length;){const y=t.events[n];y[0]==="enter"&&y[1].type==="table"&&(y[1]._align=rB(t.events,n))}return e}function ch(e,t,n,r,a,s){const o=r===1?"tableHeader":r===2?"tableDelimiter":"tableData",c="tableContent";n[0]!==0&&(s.end=Object.assign({},qo(t.events,n[0])),e.add(n[0],0,[["exit",s,t]]));const d=qo(t.events,n[1]);if(s={type:o,start:Object.assign({},d),end:Object.assign({},d)},e.add(n[1],0,[["enter",s,t]]),n[2]!==0){const f=qo(t.events,n[2]),m=qo(t.events,n[3]),p={type:c,start:Object.assign({},f),end:Object.assign({},m)};if(e.add(n[2],0,[["enter",p,t]]),r!==2){const b=t.events[n[2]],y=t.events[n[3]];if(b[1].end=Object.assign({},y[1].end),b[1].type="chunkText",b[1].contentType="text",n[3]>n[2]+1){const T=n[2]+1,x=n[3]-n[2]-1;e.add(T,x,[])}}e.add(n[3]+1,0,[["exit",p,t]])}return a!==void 0&&(s.end=Object.assign({},qo(t.events,a)),e.add(a,0,[["exit",s,t]]),s=void 0),s}function _E(e,t,n,r,a){const s=[],o=qo(t.events,n);a&&(a.end=Object.assign({},o),s.push(["exit",a,t])),r.end=Object.assign({},o),s.push(["exit",r,t]),e.add(n+1,0,s)}function qo(e,t){const n=e[t],r=n[0]==="enter"?"start":"end";return n[1][r]}const lB={name:"tasklistCheck",tokenize:uB};function oB(){return{text:{91:lB}}}function uB(e,t,n){const r=this;return a;function a(d){return r.previous!==null||!r._gfmTasklistFirstContentOfListItem?n(d):(e.enter("taskListCheck"),e.enter("taskListCheckMarker"),e.consume(d),e.exit("taskListCheckMarker"),s)}function s(d){return an(d)?(e.enter("taskListCheckValueUnchecked"),e.consume(d),e.exit("taskListCheckValueUnchecked"),o):d===88||d===120?(e.enter("taskListCheckValueChecked"),e.consume(d),e.exit("taskListCheckValueChecked"),o):n(d)}function o(d){return d===93?(e.enter("taskListCheckMarker"),e.consume(d),e.exit("taskListCheckMarker"),e.exit("taskListCheck"),c):n(d)}function c(d){return nt(d)?t(d):Dt(d)?e.check({tokenize:cB},t,n)(d):n(d)}}function cB(e,t,n){return Nt(e,r,"whitespace");function r(a){return a===null?n(a):t(a)}}function dB(e){return s7([PP(),GP(),eB(e),aB(),oB()])}const hB={};function fB(e){const t=this,n=e||hB,r=t.data(),a=r.micromarkExtensions||(r.micromarkExtensions=[]),s=r.fromMarkdownExtensions||(r.fromMarkdownExtensions=[]),o=r.toMarkdownExtensions||(r.toMarkdownExtensions=[]);a.push(dB(n)),s.push(MP()),o.push(IP(n))}const EE=/[#.]/g;function mB(e,t){const n=e||"",r={};let a=0,s,o;for(;a<n.length;){EE.lastIndex=a;const c=EE.exec(n),d=n.slice(a,c?c.index:n.length);d&&(s?s==="#"?r.id=d:Array.isArray(r.className)?r.className.push(d):r.className=[d]:o=d,a+=d.length),c&&(s=c[0],a++)}return{type:"element",tagName:o||t||"div",properties:r,children:[]}}function Z7(e,t,n){const r=n?vB(n):void 0;function a(s,o,...c){let d;if(s==null){d={type:"root",children:[]};const f=o;c.unshift(f)}else{d=mB(s,t);const f=d.tagName.toLowerCase(),m=r?r.get(f):void 0;if(d.tagName=m||f,pB(o))c.unshift(o);else for(const[p,b]of Object.entries(o))gB(e,d.properties,p,b)}for(const f of c)ob(d.children,f);return d.type==="element"&&d.tagName==="template"&&(d.content={type:"root",children:d.children},d.children=[]),d}return a}function pB(e){if(e===null||typeof e!="object"||Array.isArray(e))return!0;if(typeof e.type!="string")return!1;const t=e,n=Object.keys(e);for(const r of n){const a=t[r];if(a&&typeof a=="object"){if(!Array.isArray(a))return!0;const s=a;for(const o of s)if(typeof o!="number"&&typeof o!="string")return!0}}return!!("children"in e&&Array.isArray(e.children))}function gB(e,t,n,r){const a=h2(e,n);let s;if(r!=null){if(typeof r=="number"){if(Number.isNaN(r))return;s=r}else typeof r=="boolean"?s=r:typeof r=="string"?a.spaceSeparated?s=D_(r):a.commaSeparated?s=N_(r):a.commaOrSpaceSeparated?s=D_(N_(r).join(" ")):s=TE(a,a.property,r):Array.isArray(r)?s=[...r]:s=a.property==="style"?bB(r):String(r);if(Array.isArray(s)){const o=[];for(const c of s)o.push(TE(a,a.property,c));s=o}a.property==="className"&&Array.isArray(t.className)&&(s=t.className.concat(s)),t[a.property]=s}}function ob(e,t){if(t!=null)if(typeof t=="number"||typeof t=="string")e.push({type:"text",value:String(t)});else if(Array.isArray(t))for(const n of t)ob(e,n);else if(typeof t=="object"&&"type"in t)t.type==="root"?ob(e,t.children):e.push(t);else throw new Error("Expected node, nodes, or string, got `"+t+"`")}function TE(e,t,n){if(typeof n=="string"){if(e.number&&n&&!Number.isNaN(Number(n)))return Number(n);if((e.boolean||e.overloadedBoolean)&&(n===""||d0(n)===d0(t)))return!0}return n}function bB(e){const t=[];for(const[n,r]of Object.entries(e))t.push([n,r].join(": "));return t.join("; ")}function vB(e){const t=new Map;for(const n of e)t.set(n.toLowerCase(),n);return t}const yB=["altGlyph","altGlyphDef","altGlyphItem","animateColor","animateMotion","animateTransform","clipPath","feBlend","feColorMatrix","feComponentTransfer","feComposite","feConvolveMatrix","feDiffuseLighting","feDisplacementMap","feDistantLight","feDropShadow","feFlood","feFuncA","feFuncB","feFuncG","feFuncR","feGaussianBlur","feImage","feMerge","feMergeNode","feMorphology","feOffset","fePointLight","feSpecularLighting","feSpotLight","feTile","feTurbulence","foreignObject","glyphRef","linearGradient","radialGradient","solidColor","textArea","textPath"],J7=Z7(jf,"div"),eT=Z7(hu,"g",yB);function _B(e){const t=String(e),n=[];return{toOffset:a,toPoint:r};function r(s){if(typeof s=="number"&&s>-1&&s<=t.length){let o=0;for(;;){let c=n[o];if(c===void 0){const d=xE(t,n[o-1]);c=d===-1?t.length+1:d+1,n[o]=c}if(c>s)return{line:o+1,column:s-(o>0?n[o-1]:0)+1,offset:s};o++}}}function a(s){if(s&&typeof s.line=="number"&&typeof s.column=="number"&&!Number.isNaN(s.line)&&!Number.isNaN(s.column)){for(;n.length<s.line;){const c=n[n.length-1],d=xE(t,c),f=d===-1?t.length+1:d+1;if(c===f)break;n.push(f)}const o=(s.line>1?n[s.line-2]:0)+s.column-1;if(o<n[s.line-1])return o}}}function xE(e,t){const n=e.indexOf("\r",t),r=e.indexOf(`
`,t);return r===-1?n:n===-1||n+1===r?r:n<r?n:r}const di={html:"http://www.w3.org/1999/xhtml",mathml:"http://www.w3.org/1998/Math/MathML",svg:"http://www.w3.org/2000/svg",xlink:"http://www.w3.org/1999/xlink",xml:"http://www.w3.org/XML/1998/namespace",xmlns:"http://www.w3.org/2000/xmlns/"},tT={}.hasOwnProperty,EB=Object.prototype;function TB(e,t){const n=t||{};return C2({file:n.file||void 0,location:!1,schema:n.space==="svg"?hu:jf,verbose:n.verbose||!1},e)}function C2(e,t){let n;switch(t.nodeName){case"#comment":{const r=t;return n={type:"comment",value:r.data},qh(e,r,n),n}case"#document":case"#document-fragment":{const r=t,a="mode"in r?r.mode==="quirks"||r.mode==="limited-quirks":!1;if(n={type:"root",children:nT(e,t.childNodes),data:{quirksMode:a}},e.file&&e.location){const s=String(e.file),o=_B(s),c=o.toPoint(0),d=o.toPoint(s.length);n.position={start:c,end:d}}return n}case"#documentType":{const r=t;return n={type:"doctype"},qh(e,r,n),n}case"#text":{const r=t;return n={type:"text",value:r.value},qh(e,r,n),n}default:return n=xB(e,t),n}}function nT(e,t){let n=-1;const r=[];for(;++n<t.length;){const a=C2(e,t[n]);r.push(a)}return r}function xB(e,t){const n=e.schema;e.schema=t.namespaceURI===di.svg?hu:jf;let r=-1;const a={};for(;++r<t.attrs.length;){const c=t.attrs[r],d=(c.prefix?c.prefix+":":"")+c.name;tT.call(EB,d)||(a[d]=c.value)}const o=(e.schema.space==="svg"?eT:J7)(t.tagName,a,nT(e,t.childNodes));if(qh(e,t,o),o.tagName==="template"){const c=t,d=c.sourceCodeLocation,f=d&&d.startTag&&Go(d.startTag),m=d&&d.endTag&&Go(d.endTag),p=C2(e,c.content);f&&m&&e.file&&(p.position={start:f.end,end:m.start}),o.content=p}return e.schema=n,o}function qh(e,t,n){if("sourceCodeLocation"in t&&t.sourceCodeLocation&&e.file){const r=SB(e,n,t.sourceCodeLocation);r&&(e.location=!0,n.position=r)}}function SB(e,t,n){const r=Go(n);if(t.type==="element"){const a=t.children[t.children.length-1];if(r&&!n.endTag&&a&&a.position&&a.position.end&&(r.end=Object.assign({},a.position.end)),e.verbose){const s={};let o;if(n.attrs)for(o in n.attrs)tT.call(n.attrs,o)&&(s[h2(e.schema,o).property]=Go(n.attrs[o]));n.startTag;const c=Go(n.startTag),d=n.endTag?Go(n.endTag):void 0,f={opening:c};d&&(f.closing=d),f.properties=s,t.data={position:f}}}return r}function Go(e){const t=SE({line:e.startLine,column:e.startCol,offset:e.startOffset}),n=SE({line:e.endLine,column:e.endCol,offset:e.endOffset});return t||n?{start:t,end:n}:void 0}function SE(e){return e.line&&e.column?e:void 0}class I0{constructor(t,n,r){this.property=t,this.normal=n,r&&(this.space=r)}}I0.prototype.property={};I0.prototype.normal={};I0.prototype.space=null;function rT(e,t){const n={},r={};let a=-1;for(;++a<e.length;)Object.assign(n,e[a].property),Object.assign(r,e[a].normal);return new I0(n,r,t)}function ub(e){return e.toLowerCase()}class za{constructor(t,n){this.property=t,this.attribute=n}}za.prototype.space=null;za.prototype.boolean=!1;za.prototype.booleanish=!1;za.prototype.overloadedBoolean=!1;za.prototype.number=!1;za.prototype.commaSeparated=!1;za.prototype.spaceSeparated=!1;za.prototype.commaOrSpaceSeparated=!1;za.prototype.mustUseProperty=!1;za.prototype.defined=!1;let wB=0;const Et=Zl(),Wn=Zl(),aT=Zl(),Ie=Zl(),dn=Zl(),Xo=Zl(),ga=Zl();function Zl(){return 2**++wB}const cb=Object.freeze(Object.defineProperty({__proto__:null,boolean:Et,booleanish:Wn,commaOrSpaceSeparated:ga,commaSeparated:Xo,number:Ie,overloadedBoolean:aT,spaceSeparated:dn},Symbol.toStringTag,{value:"Module"})),X1=Object.keys(cb);class N2 extends za{constructor(t,n,r,a){let s=-1;if(super(t,n),wE(this,"space",a),typeof r=="number")for(;++s<X1.length;){const o=X1[s];wE(this,X1[s],(r&cb[o])===cb[o])}}}N2.prototype.defined=!0;function wE(e,t,n){n&&(e[t]=n)}const AB={}.hasOwnProperty;function mu(e){const t={},n={};let r;for(r in e.properties)if(AB.call(e.properties,r)){const a=e.properties[r],s=new N2(r,e.transform(e.attributes||{},r),a,e.space);e.mustUseProperty&&e.mustUseProperty.includes(r)&&(s.mustUseProperty=!0),t[r]=s,n[ub(r)]=r,n[ub(s.attribute)]=r}return new I0(t,n,e.space)}const iT=mu({space:"xlink",transform(e,t){return"xlink:"+t.slice(5).toLowerCase()},properties:{xLinkActuate:null,xLinkArcRole:null,xLinkHref:null,xLinkRole:null,xLinkShow:null,xLinkTitle:null,xLinkType:null}}),sT=mu({space:"xml",transform(e,t){return"xml:"+t.slice(3).toLowerCase()},properties:{xmlLang:null,xmlBase:null,xmlSpace:null}});function lT(e,t){return t in e?e[t]:t}function oT(e,t){return lT(e,t.toLowerCase())}const uT=mu({space:"xmlns",attributes:{xmlnsxlink:"xmlns:xlink"},transform:oT,properties:{xmlns:null,xmlnsXLink:null}}),cT=mu({transform(e,t){return t==="role"?t:"aria-"+t.slice(4).toLowerCase()},properties:{ariaActiveDescendant:null,ariaAtomic:Wn,ariaAutoComplete:null,ariaBusy:Wn,ariaChecked:Wn,ariaColCount:Ie,ariaColIndex:Ie,ariaColSpan:Ie,ariaControls:dn,ariaCurrent:null,ariaDescribedBy:dn,ariaDetails:null,ariaDisabled:Wn,ariaDropEffect:dn,ariaErrorMessage:null,ariaExpanded:Wn,ariaFlowTo:dn,ariaGrabbed:Wn,ariaHasPopup:null,ariaHidden:Wn,ariaInvalid:null,ariaKeyShortcuts:null,ariaLabel:null,ariaLabelledBy:dn,ariaLevel:Ie,ariaLive:null,ariaModal:Wn,ariaMultiLine:Wn,ariaMultiSelectable:Wn,ariaOrientation:null,ariaOwns:dn,ariaPlaceholder:null,ariaPosInSet:Ie,ariaPressed:Wn,ariaReadOnly:Wn,ariaRelevant:null,ariaRequired:Wn,ariaRoleDescription:dn,ariaRowCount:Ie,ariaRowIndex:Ie,ariaRowSpan:Ie,ariaSelected:Wn,ariaSetSize:Ie,ariaSort:null,ariaValueMax:Ie,ariaValueMin:Ie,ariaValueNow:Ie,ariaValueText:null,role:null}}),kB=mu({space:"html",attributes:{acceptcharset:"accept-charset",classname:"class",htmlfor:"for",httpequiv:"http-equiv"},transform:oT,mustUseProperty:["checked","multiple","muted","selected"],properties:{abbr:null,accept:Xo,acceptCharset:dn,accessKey:dn,action:null,allow:null,allowFullScreen:Et,allowPaymentRequest:Et,allowUserMedia:Et,alt:null,as:null,async:Et,autoCapitalize:null,autoComplete:dn,autoFocus:Et,autoPlay:Et,blocking:dn,capture:null,charSet:null,checked:Et,cite:null,className:dn,cols:Ie,colSpan:null,content:null,contentEditable:Wn,controls:Et,controlsList:dn,coords:Ie|Xo,crossOrigin:null,data:null,dateTime:null,decoding:null,default:Et,defer:Et,dir:null,dirName:null,disabled:Et,download:aT,draggable:Wn,encType:null,enterKeyHint:null,fetchPriority:null,form:null,formAction:null,formEncType:null,formMethod:null,formNoValidate:Et,formTarget:null,headers:dn,height:Ie,hidden:Et,high:Ie,href:null,hrefLang:null,htmlFor:dn,httpEquiv:dn,id:null,imageSizes:null,imageSrcSet:null,inert:Et,inputMode:null,integrity:null,is:null,isMap:Et,itemId:null,itemProp:dn,itemRef:dn,itemScope:Et,itemType:dn,kind:null,label:null,lang:null,language:null,list:null,loading:null,loop:Et,low:Ie,manifest:null,max:null,maxLength:Ie,media:null,method:null,min:null,minLength:Ie,multiple:Et,muted:Et,name:null,nonce:null,noModule:Et,noValidate:Et,onAbort:null,onAfterPrint:null,onAuxClick:null,onBeforeMatch:null,onBeforePrint:null,onBeforeToggle:null,onBeforeUnload:null,onBlur:null,onCancel:null,onCanPlay:null,onCanPlayThrough:null,onChange:null,onClick:null,onClose:null,onContextLost:null,onContextMenu:null,onContextRestored:null,onCopy:null,onCueChange:null,onCut:null,onDblClick:null,onDrag:null,onDragEnd:null,onDragEnter:null,onDragExit:null,onDragLeave:null,onDragOver:null,onDragStart:null,onDrop:null,onDurationChange:null,onEmptied:null,onEnded:null,onError:null,onFocus:null,onFormData:null,onHashChange:null,onInput:null,onInvalid:null,onKeyDown:null,onKeyPress:null,onKeyUp:null,onLanguageChange:null,onLoad:null,onLoadedData:null,onLoadedMetadata:null,onLoadEnd:null,onLoadStart:null,onMessage:null,onMessageError:null,onMouseDown:null,onMouseEnter:null,onMouseLeave:null,onMouseMove:null,onMouseOut:null,onMouseOver:null,onMouseUp:null,onOffline:null,onOnline:null,onPageHide:null,onPageShow:null,onPaste:null,onPause:null,onPlay:null,onPlaying:null,onPopState:null,onProgress:null,onRateChange:null,onRejectionHandled:null,onReset:null,onResize:null,onScroll:null,onScrollEnd:null,onSecurityPolicyViolation:null,onSeeked:null,onSeeking:null,onSelect:null,onSlotChange:null,onStalled:null,onStorage:null,onSubmit:null,onSuspend:null,onTimeUpdate:null,onToggle:null,onUnhandledRejection:null,onUnload:null,onVolumeChange:null,onWaiting:null,onWheel:null,open:Et,optimum:Ie,pattern:null,ping:dn,placeholder:null,playsInline:Et,popover:null,popoverTarget:null,popoverTargetAction:null,poster:null,preload:null,readOnly:Et,referrerPolicy:null,rel:dn,required:Et,reversed:Et,rows:Ie,rowSpan:Ie,sandbox:dn,scope:null,scoped:Et,seamless:Et,selected:Et,shadowRootClonable:Et,shadowRootDelegatesFocus:Et,shadowRootMode:null,shape:null,size:Ie,sizes:null,slot:null,span:Ie,spellCheck:Wn,src:null,srcDoc:null,srcLang:null,srcSet:null,start:Ie,step:null,style:null,tabIndex:Ie,target:null,title:null,translate:null,type:null,typeMustMatch:Et,useMap:null,value:Wn,width:Ie,wrap:null,writingSuggestions:null,align:null,aLink:null,archive:dn,axis:null,background:null,bgColor:null,border:Ie,borderColor:null,bottomMargin:Ie,cellPadding:null,cellSpacing:null,char:null,charOff:null,classId:null,clear:null,code:null,codeBase:null,codeType:null,color:null,compact:Et,declare:Et,event:null,face:null,frame:null,frameBorder:null,hSpace:Ie,leftMargin:Ie,link:null,longDesc:null,lowSrc:null,marginHeight:Ie,marginWidth:Ie,noResize:Et,noHref:Et,noShade:Et,noWrap:Et,object:null,profile:null,prompt:null,rev:null,rightMargin:Ie,rules:null,scheme:null,scrolling:Wn,standby:null,summary:null,text:null,topMargin:Ie,valueType:null,version:null,vAlign:null,vLink:null,vSpace:Ie,allowTransparency:null,autoCorrect:null,autoSave:null,disablePictureInPicture:Et,disableRemotePlayback:Et,prefix:null,property:null,results:Ie,security:null,unselectable:null}}),CB=mu({space:"svg",attributes:{accentHeight:"accent-height",alignmentBaseline:"alignment-baseline",arabicForm:"arabic-form",baselineShift:"baseline-shift",capHeight:"cap-height",className:"class",clipPath:"clip-path",clipRule:"clip-rule",colorInterpolation:"color-interpolation",colorInterpolationFilters:"color-interpolation-filters",colorProfile:"color-profile",colorRendering:"color-rendering",crossOrigin:"crossorigin",dataType:"datatype",dominantBaseline:"dominant-baseline",enableBackground:"enable-background",fillOpacity:"fill-opacity",fillRule:"fill-rule",floodColor:"flood-color",floodOpacity:"flood-opacity",fontFamily:"font-family",fontSize:"font-size",fontSizeAdjust:"font-size-adjust",fontStretch:"font-stretch",fontStyle:"font-style",fontVariant:"font-variant",fontWeight:"font-weight",glyphName:"glyph-name",glyphOrientationHorizontal:"glyph-orientation-horizontal",glyphOrientationVertical:"glyph-orientation-vertical",hrefLang:"hreflang",horizAdvX:"horiz-adv-x",horizOriginX:"horiz-origin-x",horizOriginY:"horiz-origin-y",imageRendering:"image-rendering",letterSpacing:"letter-spacing",lightingColor:"lighting-color",markerEnd:"marker-end",markerMid:"marker-mid",markerStart:"marker-start",navDown:"nav-down",navDownLeft:"nav-down-left",navDownRight:"nav-down-right",navLeft:"nav-left",navNext:"nav-next",navPrev:"nav-prev",navRight:"nav-right",navUp:"nav-up",navUpLeft:"nav-up-left",navUpRight:"nav-up-right",onAbort:"onabort",onActivate:"onactivate",onAfterPrint:"onafterprint",onBeforePrint:"onbeforeprint",onBegin:"onbegin",onCancel:"oncancel",onCanPlay:"oncanplay",onCanPlayThrough:"oncanplaythrough",onChange:"onchange",onClick:"onclick",onClose:"onclose",onCopy:"oncopy",onCueChange:"oncuechange",onCut:"oncut",onDblClick:"ondblclick",onDrag:"ondrag",onDragEnd:"ondragend",onDragEnter:"ondragenter",onDragExit:"ondragexit",onDragLeave:"ondragleave",onDragOver:"ondragover",onDragStart:"ondragstart",onDrop:"ondrop",onDurationChange:"ondurationchange",onEmptied:"onemptied",onEnd:"onend",onEnded:"onended",onError:"onerror",onFocus:"onfocus",onFocusIn:"onfocusin",onFocusOut:"onfocusout",onHashChange:"onhashchange",onInput:"oninput",onInvalid:"oninvalid",onKeyDown:"onkeydown",onKeyPress:"onkeypress",onKeyUp:"onkeyup",onLoad:"onload",onLoadedData:"onloadeddata",onLoadedMetadata:"onloadedmetadata",onLoadStart:"onloadstart",onMessage:"onmessage",onMouseDown:"onmousedown",onMouseEnter:"onmouseenter",onMouseLeave:"onmouseleave",onMouseMove:"onmousemove",onMouseOut:"onmouseout",onMouseOver:"onmouseover",onMouseUp:"onmouseup",onMouseWheel:"onmousewheel",onOffline:"onoffline",onOnline:"ononline",onPageHide:"onpagehide",onPageShow:"onpageshow",onPaste:"onpaste",onPause:"onpause",onPlay:"onplay",onPlaying:"onplaying",onPopState:"onpopstate",onProgress:"onprogress",onRateChange:"onratechange",onRepeat:"onrepeat",onReset:"onreset",onResize:"onresize",onScroll:"onscroll",onSeeked:"onseeked",onSeeking:"onseeking",onSelect:"onselect",onShow:"onshow",onStalled:"onstalled",onStorage:"onstorage",onSubmit:"onsubmit",onSuspend:"onsuspend",onTimeUpdate:"ontimeupdate",onToggle:"ontoggle",onUnload:"onunload",onVolumeChange:"onvolumechange",onWaiting:"onwaiting",onZoom:"onzoom",overlinePosition:"overline-position",overlineThickness:"overline-thickness",paintOrder:"paint-order",panose1:"panose-1",pointerEvents:"pointer-events",referrerPolicy:"referrerpolicy",renderingIntent:"rendering-intent",shapeRendering:"shape-rendering",stopColor:"stop-color",stopOpacity:"stop-opacity",strikethroughPosition:"strikethrough-position",strikethroughThickness:"strikethrough-thickness",strokeDashArray:"stroke-dasharray",strokeDashOffset:"stroke-dashoffset",strokeLineCap:"stroke-linecap",strokeLineJoin:"stroke-linejoin",strokeMiterLimit:"stroke-miterlimit",strokeOpacity:"stroke-opacity",strokeWidth:"stroke-width",tabIndex:"tabindex",textAnchor:"text-anchor",textDecoration:"text-decoration",textRendering:"text-rendering",transformOrigin:"transform-origin",typeOf:"typeof",underlinePosition:"underline-position",underlineThickness:"underline-thickness",unicodeBidi:"unicode-bidi",unicodeRange:"unicode-range",unitsPerEm:"units-per-em",vAlphabetic:"v-alphabetic",vHanging:"v-hanging",vIdeographic:"v-ideographic",vMathematical:"v-mathematical",vectorEffect:"vector-effect",vertAdvY:"vert-adv-y",vertOriginX:"vert-origin-x",vertOriginY:"vert-origin-y",wordSpacing:"word-spacing",writingMode:"writing-mode",xHeight:"x-height",playbackOrder:"playbackorder",timelineBegin:"timelinebegin"},transform:lT,properties:{about:ga,accentHeight:Ie,accumulate:null,additive:null,alignmentBaseline:null,alphabetic:Ie,amplitude:Ie,arabicForm:null,ascent:Ie,attributeName:null,attributeType:null,azimuth:Ie,bandwidth:null,baselineShift:null,baseFrequency:null,baseProfile:null,bbox:null,begin:null,bias:Ie,by:null,calcMode:null,capHeight:Ie,className:dn,clip:null,clipPath:null,clipPathUnits:null,clipRule:null,color:null,colorInterpolation:null,colorInterpolationFilters:null,colorProfile:null,colorRendering:null,content:null,contentScriptType:null,contentStyleType:null,crossOrigin:null,cursor:null,cx:null,cy:null,d:null,dataType:null,defaultAction:null,descent:Ie,diffuseConstant:Ie,direction:null,display:null,dur:null,divisor:Ie,dominantBaseline:null,download:Et,dx:null,dy:null,edgeMode:null,editable:null,elevation:Ie,enableBackground:null,end:null,event:null,exponent:Ie,externalResourcesRequired:null,fill:null,fillOpacity:Ie,fillRule:null,filter:null,filterRes:null,filterUnits:null,floodColor:null,floodOpacity:null,focusable:null,focusHighlight:null,fontFamily:null,fontSize:null,fontSizeAdjust:null,fontStretch:null,fontStyle:null,fontVariant:null,fontWeight:null,format:null,fr:null,from:null,fx:null,fy:null,g1:Xo,g2:Xo,glyphName:Xo,glyphOrientationHorizontal:null,glyphOrientationVertical:null,glyphRef:null,gradientTransform:null,gradientUnits:null,handler:null,hanging:Ie,hatchContentUnits:null,hatchUnits:null,height:null,href:null,hrefLang:null,horizAdvX:Ie,horizOriginX:Ie,horizOriginY:Ie,id:null,ideographic:Ie,imageRendering:null,initialVisibility:null,in:null,in2:null,intercept:Ie,k:Ie,k1:Ie,k2:Ie,k3:Ie,k4:Ie,kernelMatrix:ga,kernelUnitLength:null,keyPoints:null,keySplines:null,keyTimes:null,kerning:null,lang:null,lengthAdjust:null,letterSpacing:null,lightingColor:null,limitingConeAngle:Ie,local:null,markerEnd:null,markerMid:null,markerStart:null,markerHeight:null,markerUnits:null,markerWidth:null,mask:null,maskContentUnits:null,maskUnits:null,mathematical:null,max:null,media:null,mediaCharacterEncoding:null,mediaContentEncodings:null,mediaSize:Ie,mediaTime:null,method:null,min:null,mode:null,name:null,navDown:null,navDownLeft:null,navDownRight:null,navLeft:null,navNext:null,navPrev:null,navRight:null,navUp:null,navUpLeft:null,navUpRight:null,numOctaves:null,observer:null,offset:null,onAbort:null,onActivate:null,onAfterPrint:null,onBeforePrint:null,onBegin:null,onCancel:null,onCanPlay:null,onCanPlayThrough:null,onChange:null,onClick:null,onClose:null,onCopy:null,onCueChange:null,onCut:null,onDblClick:null,onDrag:null,onDragEnd:null,onDragEnter:null,onDragExit:null,onDragLeave:null,onDragOver:null,onDragStart:null,onDrop:null,onDurationChange:null,onEmptied:null,onEnd:null,onEnded:null,onError:null,onFocus:null,onFocusIn:null,onFocusOut:null,onHashChange:null,onInput:null,onInvalid:null,onKeyDown:null,onKeyPress:null,onKeyUp:null,onLoad:null,onLoadedData:null,onLoadedMetadata:null,onLoadStart:null,onMessage:null,onMouseDown:null,onMouseEnter:null,onMouseLeave:null,onMouseMove:null,onMouseOut:null,onMouseOver:null,onMouseUp:null,onMouseWheel:null,onOffline:null,onOnline:null,onPageHide:null,onPageShow:null,onPaste:null,onPause:null,onPlay:null,onPlaying:null,onPopState:null,onProgress:null,onRateChange:null,onRepeat:null,onReset:null,onResize:null,onScroll:null,onSeeked:null,onSeeking:null,onSelect:null,onShow:null,onStalled:null,onStorage:null,onSubmit:null,onSuspend:null,onTimeUpdate:null,onToggle:null,onUnload:null,onVolumeChange:null,onWaiting:null,onZoom:null,opacity:null,operator:null,order:null,orient:null,orientation:null,origin:null,overflow:null,overlay:null,overlinePosition:Ie,overlineThickness:Ie,paintOrder:null,panose1:null,path:null,pathLength:Ie,patternContentUnits:null,patternTransform:null,patternUnits:null,phase:null,ping:dn,pitch:null,playbackOrder:null,pointerEvents:null,points:null,pointsAtX:Ie,pointsAtY:Ie,pointsAtZ:Ie,preserveAlpha:null,preserveAspectRatio:null,primitiveUnits:null,propagate:null,property:ga,r:null,radius:null,referrerPolicy:null,refX:null,refY:null,rel:ga,rev:ga,renderingIntent:null,repeatCount:null,repeatDur:null,requiredExtensions:ga,requiredFeatures:ga,requiredFonts:ga,requiredFormats:ga,resource:null,restart:null,result:null,rotate:null,rx:null,ry:null,scale:null,seed:null,shapeRendering:null,side:null,slope:null,snapshotTime:null,specularConstant:Ie,specularExponent:Ie,spreadMethod:null,spacing:null,startOffset:null,stdDeviation:null,stemh:null,stemv:null,stitchTiles:null,stopColor:null,stopOpacity:null,strikethroughPosition:Ie,strikethroughThickness:Ie,string:null,stroke:null,strokeDashArray:ga,strokeDashOffset:null,strokeLineCap:null,strokeLineJoin:null,strokeMiterLimit:Ie,strokeOpacity:Ie,strokeWidth:null,style:null,surfaceScale:Ie,syncBehavior:null,syncBehaviorDefault:null,syncMaster:null,syncTolerance:null,syncToleranceDefault:null,systemLanguage:ga,tabIndex:Ie,tableValues:null,target:null,targetX:Ie,targetY:Ie,textAnchor:null,textDecoration:null,textRendering:null,textLength:null,timelineBegin:null,title:null,transformBehavior:null,type:null,typeOf:ga,to:null,transform:null,transformOrigin:null,u1:null,u2:null,underlinePosition:Ie,underlineThickness:Ie,unicode:null,unicodeBidi:null,unicodeRange:null,unitsPerEm:Ie,values:null,vAlphabetic:Ie,vMathematical:Ie,vectorEffect:null,vHanging:Ie,vIdeographic:Ie,version:null,vertAdvY:Ie,vertOriginX:Ie,vertOriginY:Ie,viewBox:null,viewTarget:null,visibility:null,width:null,widths:null,wordSpacing:null,writingMode:null,x:null,x1:null,x2:null,xChannelSelector:null,xHeight:Ie,y:null,y1:null,y2:null,yChannelSelector:null,z:null,zoomAndPan:null}}),NB=/^data[-\w.:]+$/i,AE=/-[a-z]/g,OB=/[A-Z]/g;function RB(e,t){const n=ub(t);let r=t,a=za;if(n in e.normal)return e.property[e.normal[n]];if(n.length>4&&n.slice(0,4)==="data"&&NB.test(t)){if(t.charAt(4)==="-"){const s=t.slice(5).replace(AE,IB);r="data"+s.charAt(0).toUpperCase()+s.slice(1)}else{const s=t.slice(4);if(!AE.test(s)){let o=s.replace(OB,MB);o.charAt(0)!=="-"&&(o="-"+o),t="data"+o}}a=N2}return new a(r,t)}function MB(e){return"-"+e.toLowerCase()}function IB(e){return e.charAt(1).toUpperCase()}const DB=rT([sT,iT,uT,cT,kB],"html"),dT=rT([sT,iT,uT,cT,CB],"svg"),LB={},PB={}.hasOwnProperty,hT=R7("type",{handlers:{root:UB,element:jB,text:HB,comment:qB,doctype:zB}});function BB(e,t){const r=(t||LB).space;return hT(e,r==="svg"?dT:DB)}function UB(e,t){const n={nodeName:"#document",mode:(e.data||{}).quirksMode?"quirks":"no-quirks",childNodes:[]};return n.childNodes=O2(e.children,n,t),pu(e,n),n}function FB(e,t){const n={nodeName:"#document-fragment",childNodes:[]};return n.childNodes=O2(e.children,n,t),pu(e,n),n}function zB(e){const t={nodeName:"#documentType",name:"html",publicId:"",systemId:"",parentNode:null};return pu(e,t),t}function HB(e){const t={nodeName:"#text",value:e.value,parentNode:null};return pu(e,t),t}function qB(e){const t={nodeName:"#comment",data:e.value,parentNode:null};return pu(e,t),t}function jB(e,t){const n=t;let r=n;e.type==="element"&&e.tagName.toLowerCase()==="svg"&&n.space==="html"&&(r=dT);const a=[];let s;if(e.properties){for(s in e.properties)if(s!=="children"&&PB.call(e.properties,s)){const d=$B(r,s,e.properties[s]);d&&a.push(d)}}const o=r.space,c={nodeName:e.tagName,tagName:e.tagName,attrs:a,namespaceURI:di[o],childNodes:[],parentNode:null};return c.childNodes=O2(e.children,c,r),pu(e,c),e.tagName==="template"&&e.content&&(c.content=FB(e.content,r)),c}function $B(e,t,n){const r=RB(e,t);if(n===!1||n===null||n===void 0||typeof n=="number"&&Number.isNaN(n)||!n&&r.boolean)return;Array.isArray(n)&&(n=r.commaSeparated?Y8(n):J8(n));const a={name:r.attribute,value:n===!0?"":String(n)};if(r.space&&r.space!=="html"&&r.space!=="svg"){const s=a.name.indexOf(":");s<0?a.prefix="":(a.name=a.name.slice(s+1),a.prefix=r.attribute.slice(0,s)),a.namespace=di[r.space]}return a}function O2(e,t,n){let r=-1;const a=[];if(e)for(;++r<e.length;){const s=hT(e[r],n);s.parentNode=t,a.push(s)}return a}function pu(e,t){const n=e.position;n&&n.start&&n.end&&(n.start.offset,n.end.offset,t.sourceCodeLocation={startLine:n.start.line,startCol:n.start.column,startOffset:n.start.offset,endLine:n.end.line,endCol:n.end.column,endOffset:n.end.offset})}const YB=["area","base","basefont","bgsound","br","col","command","embed","frame","hr","image","img","input","keygen","link","meta","param","source","track","wbr"],GB=new Set([65534,65535,131070,131071,196606,196607,262142,262143,327678,327679,393214,393215,458750,458751,524286,524287,589822,589823,655358,655359,720894,720895,786430,786431,851966,851967,917502,917503,983038,983039,1048574,1048575,1114110,1114111]),Tn="�";var P;(function(e){e[e.EOF=-1]="EOF",e[e.NULL=0]="NULL",e[e.TABULATION=9]="TABULATION",e[e.CARRIAGE_RETURN=13]="CARRIAGE_RETURN",e[e.LINE_FEED=10]="LINE_FEED",e[e.FORM_FEED=12]="FORM_FEED",e[e.SPACE=32]="SPACE",e[e.EXCLAMATION_MARK=33]="EXCLAMATION_MARK",e[e.QUOTATION_MARK=34]="QUOTATION_MARK",e[e.AMPERSAND=38]="AMPERSAND",e[e.APOSTROPHE=39]="APOSTROPHE",e[e.HYPHEN_MINUS=45]="HYPHEN_MINUS",e[e.SOLIDUS=47]="SOLIDUS",e[e.DIGIT_0=48]="DIGIT_0",e[e.DIGIT_9=57]="DIGIT_9",e[e.SEMICOLON=59]="SEMICOLON",e[e.LESS_THAN_SIGN=60]="LESS_THAN_SIGN",e[e.EQUALS_SIGN=61]="EQUALS_SIGN",e[e.GREATER_THAN_SIGN=62]="GREATER_THAN_SIGN",e[e.QUESTION_MARK=63]="QUESTION_MARK",e[e.LATIN_CAPITAL_A=65]="LATIN_CAPITAL_A",e[e.LATIN_CAPITAL_Z=90]="LATIN_CAPITAL_Z",e[e.RIGHT_SQUARE_BRACKET=93]="RIGHT_SQUARE_BRACKET",e[e.GRAVE_ACCENT=96]="GRAVE_ACCENT",e[e.LATIN_SMALL_A=97]="LATIN_SMALL_A",e[e.LATIN_SMALL_Z=122]="LATIN_SMALL_Z"})(P||(P={}));const Qr={DASH_DASH:"--",CDATA_START:"[CDATA[",DOCTYPE:"doctype",SCRIPT:"script",PUBLIC:"public",SYSTEM:"system"};function fT(e){return e>=55296&&e<=57343}function VB(e){return e>=56320&&e<=57343}function KB(e,t){return(e-55296)*1024+9216+t}function mT(e){return e!==32&&e!==10&&e!==13&&e!==9&&e!==12&&e>=1&&e<=31||e>=127&&e<=159}function pT(e){return e>=64976&&e<=65007||GB.has(e)}var ve;(function(e){e.controlCharacterInInputStream="control-character-in-input-stream",e.noncharacterInInputStream="noncharacter-in-input-stream",e.surrogateInInputStream="surrogate-in-input-stream",e.nonVoidHtmlElementStartTagWithTrailingSolidus="non-void-html-element-start-tag-with-trailing-solidus",e.endTagWithAttributes="end-tag-with-attributes",e.endTagWithTrailingSolidus="end-tag-with-trailing-solidus",e.unexpectedSolidusInTag="unexpected-solidus-in-tag",e.unexpectedNullCharacter="unexpected-null-character",e.unexpectedQuestionMarkInsteadOfTagName="unexpected-question-mark-instead-of-tag-name",e.invalidFirstCharacterOfTagName="invalid-first-character-of-tag-name",e.unexpectedEqualsSignBeforeAttributeName="unexpected-equals-sign-before-attribute-name",e.missingEndTagName="missing-end-tag-name",e.unexpectedCharacterInAttributeName="unexpected-character-in-attribute-name",e.unknownNamedCharacterReference="unknown-named-character-reference",e.missingSemicolonAfterCharacterReference="missing-semicolon-after-character-reference",e.unexpectedCharacterAfterDoctypeSystemIdentifier="unexpected-character-after-doctype-system-identifier",e.unexpectedCharacterInUnquotedAttributeValue="unexpected-character-in-unquoted-attribute-value",e.eofBeforeTagName="eof-before-tag-name",e.eofInTag="eof-in-tag",e.missingAttributeValue="missing-attribute-value",e.missingWhitespaceBetweenAttributes="missing-whitespace-between-attributes",e.missingWhitespaceAfterDoctypePublicKeyword="missing-whitespace-after-doctype-public-keyword",e.missingWhitespaceBetweenDoctypePublicAndSystemIdentifiers="missing-whitespace-between-doctype-public-and-system-identifiers",e.missingWhitespaceAfterDoctypeSystemKeyword="missing-whitespace-after-doctype-system-keyword",e.missingQuoteBeforeDoctypePublicIdentifier="missing-quote-before-doctype-public-identifier",e.missingQuoteBeforeDoctypeSystemIdentifier="missing-quote-before-doctype-system-identifier",e.missingDoctypePublicIdentifier="missing-doctype-public-identifier",e.missingDoctypeSystemIdentifier="missing-doctype-system-identifier",e.abruptDoctypePublicIdentifier="abrupt-doctype-public-identifier",e.abruptDoctypeSystemIdentifier="abrupt-doctype-system-identifier",e.cdataInHtmlContent="cdata-in-html-content",e.incorrectlyOpenedComment="incorrectly-opened-comment",e.eofInScriptHtmlCommentLikeText="eof-in-script-html-comment-like-text",e.eofInDoctype="eof-in-doctype",e.nestedComment="nested-comment",e.abruptClosingOfEmptyComment="abrupt-closing-of-empty-comment",e.eofInComment="eof-in-comment",e.incorrectlyClosedComment="incorrectly-closed-comment",e.eofInCdata="eof-in-cdata",e.absenceOfDigitsInNumericCharacterReference="absence-of-digits-in-numeric-character-reference",e.nullCharacterReference="null-character-reference",e.surrogateCharacterReference="surrogate-character-reference",e.characterReferenceOutsideUnicodeRange="character-reference-outside-unicode-range",e.controlCharacterReference="control-character-reference",e.noncharacterCharacterReference="noncharacter-character-reference",e.missingWhitespaceBeforeDoctypeName="missing-whitespace-before-doctype-name",e.missingDoctypeName="missing-doctype-name",e.invalidCharacterSequenceAfterDoctypeName="invalid-character-sequence-after-doctype-name",e.duplicateAttribute="duplicate-attribute",e.nonConformingDoctype="non-conforming-doctype",e.missingDoctype="missing-doctype",e.misplacedDoctype="misplaced-doctype",e.endTagWithoutMatchingOpenElement="end-tag-without-matching-open-element",e.closingOfElementWithOpenChildElements="closing-of-element-with-open-child-elements",e.disallowedContentInNoscriptInHead="disallowed-content-in-noscript-in-head",e.openElementsLeftAfterEof="open-elements-left-after-eof",e.abandonedHeadElementChild="abandoned-head-element-child",e.misplacedStartTagForHeadElement="misplaced-start-tag-for-head-element",e.nestedNoscriptInHead="nested-noscript-in-head",e.eofInElementThatCanContainOnlyText="eof-in-element-that-can-contain-only-text"})(ve||(ve={}));const WB=65536;class XB{constructor(t){this.handler=t,this.html="",this.pos=-1,this.lastGapPos=-2,this.gapStack=[],this.skipNextNewLine=!1,this.lastChunkWritten=!1,this.endOfChunkHit=!1,this.bufferWaterline=WB,this.isEol=!1,this.lineStartPos=0,this.droppedBufferSize=0,this.line=1,this.lastErrOffset=-1}get col(){return this.pos-this.lineStartPos+ +(this.lastGapPos!==this.pos)}get offset(){return this.droppedBufferSize+this.pos}getError(t,n){const{line:r,col:a,offset:s}=this,o=a+n,c=s+n;return{code:t,startLine:r,endLine:r,startCol:o,endCol:o,startOffset:c,endOffset:c}}_err(t){this.handler.onParseError&&this.lastErrOffset!==this.offset&&(this.lastErrOffset=this.offset,this.handler.onParseError(this.getError(t,0)))}_addGap(){this.gapStack.push(this.lastGapPos),this.lastGapPos=this.pos}_processSurrogate(t){if(this.pos!==this.html.length-1){const n=this.html.charCodeAt(this.pos+1);if(VB(n))return this.pos++,this._addGap(),KB(t,n)}else if(!this.lastChunkWritten)return this.endOfChunkHit=!0,P.EOF;return this._err(ve.surrogateInInputStream),t}willDropParsedChunk(){return this.pos>this.bufferWaterline}dropParsedChunk(){this.willDropParsedChunk()&&(this.html=this.html.substring(this.pos),this.lineStartPos-=this.pos,this.droppedBufferSize+=this.pos,this.pos=0,this.lastGapPos=-2,this.gapStack.length=0)}write(t,n){this.html.length>0?this.html+=t:this.html=t,this.endOfChunkHit=!1,this.lastChunkWritten=n}insertHtmlAtCurrentPos(t){this.html=this.html.substring(0,this.pos+1)+t+this.html.substring(this.pos+1),this.endOfChunkHit=!1}startsWith(t,n){if(this.pos+t.length>this.html.length)return this.endOfChunkHit=!this.lastChunkWritten,!1;if(n)return this.html.startsWith(t,this.pos);for(let r=0;r<t.length;r++)if((this.html.charCodeAt(this.pos+r)|32)!==t.charCodeAt(r))return!1;return!0}peek(t){const n=this.pos+t;if(n>=this.html.length)return this.endOfChunkHit=!this.lastChunkWritten,P.EOF;const r=this.html.charCodeAt(n);return r===P.CARRIAGE_RETURN?P.LINE_FEED:r}advance(){if(this.pos++,this.isEol&&(this.isEol=!1,this.line++,this.lineStartPos=this.pos),this.pos>=this.html.length)return this.endOfChunkHit=!this.lastChunkWritten,P.EOF;let t=this.html.charCodeAt(this.pos);return t===P.CARRIAGE_RETURN?(this.isEol=!0,this.skipNextNewLine=!0,P.LINE_FEED):t===P.LINE_FEED&&(this.isEol=!0,this.skipNextNewLine)?(this.line--,this.skipNextNewLine=!1,this._addGap(),this.advance()):(this.skipNextNewLine=!1,fT(t)&&(t=this._processSurrogate(t)),this.handler.onParseError===null||t>31&&t<127||t===P.LINE_FEED||t===P.CARRIAGE_RETURN||t>159&&t<64976||this._checkForProblematicCharacters(t),t)}_checkForProblematicCharacters(t){mT(t)?this._err(ve.controlCharacterInInputStream):pT(t)&&this._err(ve.noncharacterInInputStream)}retreat(t){for(this.pos-=t;this.pos<this.lastGapPos;)this.lastGapPos=this.gapStack.pop(),this.pos--;this.isEol=!1}}var Ct;(function(e){e[e.CHARACTER=0]="CHARACTER",e[e.NULL_CHARACTER=1]="NULL_CHARACTER",e[e.WHITESPACE_CHARACTER=2]="WHITESPACE_CHARACTER",e[e.START_TAG=3]="START_TAG",e[e.END_TAG=4]="END_TAG",e[e.COMMENT=5]="COMMENT",e[e.DOCTYPE=6]="DOCTYPE",e[e.EOF=7]="EOF",e[e.HIBERNATION=8]="HIBERNATION"})(Ct||(Ct={}));function gT(e,t){for(let n=e.attrs.length-1;n>=0;n--)if(e.attrs[n].name===t)return e.attrs[n].value;return null}const QB=new Uint16Array('ᵁ<Õıʊҝջאٵ۞ޢߖࠏ੊ઑඡ๭༉༦჊ረዡᐕᒝᓃᓟᔥ\0\0\0\0\0\0ᕫᛍᦍᰒᷝ὾⁠↰⊍⏀⏻⑂⠤⤒ⴈ⹈⿎〖㊺㘹㞬㣾㨨㩱㫠㬮ࠀEMabcfglmnoprstu\\bfms¦³¹ÈÏlig耻Æ䃆P耻&䀦cute耻Á䃁reve;䄂Āiyx}rc耻Â䃂;䐐r;쀀𝔄rave耻À䃀pha;䎑acr;䄀d;橓Āgp¡on;䄄f;쀀𝔸plyFunction;恡ing耻Å䃅Ācs¾Ãr;쀀𝒜ign;扔ilde耻Ã䃃ml耻Ä䃄ЀaceforsuåûþėĜĢħĪĀcrêòkslash;或Ŷöø;櫧ed;挆y;䐑ƀcrtąċĔause;戵noullis;愬a;䎒r;쀀𝔅pf;쀀𝔹eve;䋘còēmpeq;扎܀HOacdefhilorsuōőŖƀƞƢƵƷƺǜȕɳɸɾcy;䐧PY耻©䂩ƀcpyŝŢźute;䄆Ā;iŧŨ拒talDifferentialD;慅leys;愭ȀaeioƉƎƔƘron;䄌dil耻Ç䃇rc;䄈nint;戰ot;䄊ĀdnƧƭilla;䂸terDot;䂷òſi;䎧rcleȀDMPTǇǋǑǖot;抙inus;抖lus;投imes;抗oĀcsǢǸkwiseContourIntegral;戲eCurlyĀDQȃȏoubleQuote;思uote;怙ȀlnpuȞȨɇɕonĀ;eȥȦ户;橴ƀgitȯȶȺruent;扡nt;戯ourIntegral;戮ĀfrɌɎ;愂oduct;成nterClockwiseContourIntegral;戳oss;樯cr;쀀𝒞pĀ;Cʄʅ拓ap;才րDJSZacefiosʠʬʰʴʸˋ˗ˡ˦̳ҍĀ;oŹʥtrahd;椑cy;䐂cy;䐅cy;䐏ƀgrsʿ˄ˇger;怡r;憡hv;櫤Āayː˕ron;䄎;䐔lĀ;t˝˞戇a;䎔r;쀀𝔇Āaf˫̧Ācm˰̢riticalȀADGT̖̜̀̆cute;䂴oŴ̋̍;䋙bleAcute;䋝rave;䁠ilde;䋜ond;拄ferentialD;慆Ѱ̽\0\0\0͔͂\0Ѕf;쀀𝔻ƀ;DE͈͉͍䂨ot;惜qual;扐blèCDLRUVͣͲ΂ϏϢϸontourIntegraìȹoɴ͹\0\0ͻ»͉nArrow;懓Āeo·ΤftƀARTΐΖΡrrow;懐ightArrow;懔eåˊngĀLRΫτeftĀARγιrrow;柸ightArrow;柺ightArrow;柹ightĀATϘϞrrow;懒ee;抨pɁϩ\0\0ϯrrow;懑ownArrow;懕erticalBar;戥ǹABLRTaВЪаўѿͼrrowƀ;BUНОТ憓ar;椓pArrow;懵reve;䌑eft˒к\0ц\0ѐightVector;楐eeVector;楞ectorĀ;Bљњ憽ar;楖ightǔѧ\0ѱeeVector;楟ectorĀ;BѺѻ懁ar;楗eeĀ;A҆҇护rrow;憧ĀctҒҗr;쀀𝒟rok;䄐ࠀNTacdfglmopqstuxҽӀӄӋӞӢӧӮӵԡԯԶՒ՝ՠեG;䅊H耻Ð䃐cute耻É䃉ƀaiyӒӗӜron;䄚rc耻Ê䃊;䐭ot;䄖r;쀀𝔈rave耻È䃈ement;戈ĀapӺӾcr;䄒tyɓԆ\0\0ԒmallSquare;旻erySmallSquare;斫ĀgpԦԪon;䄘f;쀀𝔼silon;䎕uĀaiԼՉlĀ;TՂՃ橵ilde;扂librium;懌Āci՗՚r;愰m;橳a;䎗ml耻Ë䃋Āipժկsts;戃onentialE;慇ʀcfiosօֈ֍ֲ׌y;䐤r;쀀𝔉lledɓ֗\0\0֣mallSquare;旼erySmallSquare;斪Ͱֺ\0ֿ\0\0ׄf;쀀𝔽All;戀riertrf;愱cò׋؀JTabcdfgorstר׬ׯ׺؀ؒؖ؛؝أ٬ٲcy;䐃耻>䀾mmaĀ;d׷׸䎓;䏜reve;䄞ƀeiy؇،ؐdil;䄢rc;䄜;䐓ot;䄠r;쀀𝔊;拙pf;쀀𝔾eater̀EFGLSTصلَٖٛ٦qualĀ;Lؾؿ扥ess;招ullEqual;执reater;檢ess;扷lantEqual;橾ilde;扳cr;쀀𝒢;扫ЀAacfiosuڅڋږڛڞڪھۊRDcy;䐪Āctڐڔek;䋇;䁞irc;䄤r;愌lbertSpace;愋ǰگ\0ڲf;愍izontalLine;攀Āctۃۅòکrok;䄦mpńېۘownHumðįqual;扏܀EJOacdfgmnostuۺ۾܃܇܎ܚܞܡܨ݄ݸދޏޕcy;䐕lig;䄲cy;䐁cute耻Í䃍Āiyܓܘrc耻Î䃎;䐘ot;䄰r;愑rave耻Ì䃌ƀ;apܠܯܿĀcgܴܷr;䄪inaryI;慈lieóϝǴ݉\0ݢĀ;eݍݎ戬Āgrݓݘral;戫section;拂isibleĀCTݬݲomma;恣imes;恢ƀgptݿރވon;䄮f;쀀𝕀a;䎙cr;愐ilde;䄨ǫޚ\0ޞcy;䐆l耻Ï䃏ʀcfosuެ޷޼߂ߐĀiyޱ޵rc;䄴;䐙r;쀀𝔍pf;쀀𝕁ǣ߇\0ߌr;쀀𝒥rcy;䐈kcy;䐄΀HJacfosߤߨ߽߬߱ࠂࠈcy;䐥cy;䐌ppa;䎚Āey߶߻dil;䄶;䐚r;쀀𝔎pf;쀀𝕂cr;쀀𝒦րJTaceflmostࠥࠩࠬࡐࡣ঳সে্਷ੇcy;䐉耻<䀼ʀcmnpr࠷࠼ࡁࡄࡍute;䄹bda;䎛g;柪lacetrf;愒r;憞ƀaeyࡗ࡜ࡡron;䄽dil;䄻;䐛Āfsࡨ॰tԀACDFRTUVarࡾࢩࢱࣦ࣠ࣼयज़ΐ४Ānrࢃ࢏gleBracket;柨rowƀ;BR࢙࢚࢞憐ar;懤ightArrow;懆eiling;挈oǵࢷ\0ࣃbleBracket;柦nǔࣈ\0࣒eeVector;楡ectorĀ;Bࣛࣜ懃ar;楙loor;挊ightĀAV࣯ࣵrrow;憔ector;楎Āerँगeƀ;AVउऊऐ抣rrow;憤ector;楚iangleƀ;BEतथऩ抲ar;槏qual;抴pƀDTVषूौownVector;楑eeVector;楠ectorĀ;Bॖॗ憿ar;楘ectorĀ;B॥०憼ar;楒ightáΜs̀EFGLSTॾঋকঝঢভqualGreater;拚ullEqual;扦reater;扶ess;檡lantEqual;橽ilde;扲r;쀀𝔏Ā;eঽা拘ftarrow;懚idot;䄿ƀnpw৔ਖਛgȀLRlr৞৷ਂਐeftĀAR০৬rrow;柵ightArrow;柷ightArrow;柶eftĀarγਊightáοightáϊf;쀀𝕃erĀLRਢਬeftArrow;憙ightArrow;憘ƀchtਾੀੂòࡌ;憰rok;䅁;扪Ѐacefiosuਗ਼੝੠੷੼અઋ઎p;椅y;䐜Ādl੥੯iumSpace;恟lintrf;愳r;쀀𝔐nusPlus;戓pf;쀀𝕄cò੶;䎜ҀJacefostuણધભીଔଙඑ඗ඞcy;䐊cute;䅃ƀaey઴હાron;䅇dil;䅅;䐝ƀgswે૰଎ativeƀMTV૓૟૨ediumSpace;怋hiĀcn૦૘ë૙eryThiî૙tedĀGL૸ଆreaterGreateòٳessLesóੈLine;䀊r;쀀𝔑ȀBnptଢନଷ଺reak;恠BreakingSpace;䂠f;愕ڀ;CDEGHLNPRSTV୕ୖ୪୼஡௫ఄ౞಄ದ೘ൡඅ櫬Āou୛୤ngruent;扢pCap;扭oubleVerticalBar;戦ƀlqxஃஊ஛ement;戉ualĀ;Tஒஓ扠ilde;쀀≂̸ists;戄reater΀;EFGLSTஶஷ஽௉௓௘௥扯qual;扱ullEqual;쀀≧̸reater;쀀≫̸ess;批lantEqual;쀀⩾̸ilde;扵umpń௲௽ownHump;쀀≎̸qual;쀀≏̸eĀfsఊధtTriangleƀ;BEచఛడ拪ar;쀀⧏̸qual;括s̀;EGLSTవశ఼ౄోౘ扮qual;扰reater;扸ess;쀀≪̸lantEqual;쀀⩽̸ilde;扴estedĀGL౨౹reaterGreater;쀀⪢̸essLess;쀀⪡̸recedesƀ;ESಒಓಛ技qual;쀀⪯̸lantEqual;拠ĀeiಫಹverseElement;戌ghtTriangleƀ;BEೋೌ೒拫ar;쀀⧐̸qual;拭ĀquೝഌuareSuĀbp೨೹setĀ;E೰ೳ쀀⊏̸qual;拢ersetĀ;Eഃആ쀀⊐̸qual;拣ƀbcpഓതൎsetĀ;Eഛഞ쀀⊂⃒qual;抈ceedsȀ;ESTലള഻െ抁qual;쀀⪰̸lantEqual;拡ilde;쀀≿̸ersetĀ;E൘൛쀀⊃⃒qual;抉ildeȀ;EFT൮൯൵ൿ扁qual;扄ullEqual;扇ilde;扉erticalBar;戤cr;쀀𝒩ilde耻Ñ䃑;䎝܀Eacdfgmoprstuvලෂ෉෕ෛ෠෧෼ขภยา฿ไlig;䅒cute耻Ó䃓Āiy෎ීrc耻Ô䃔;䐞blac;䅐r;쀀𝔒rave耻Ò䃒ƀaei෮ෲ෶cr;䅌ga;䎩cron;䎟pf;쀀𝕆enCurlyĀDQฎบoubleQuote;怜uote;怘;橔Āclวฬr;쀀𝒪ash耻Ø䃘iŬื฼de耻Õ䃕es;樷ml耻Ö䃖erĀBP๋๠Āar๐๓r;怾acĀek๚๜;揞et;掴arenthesis;揜Ҁacfhilors๿ງຊຏຒດຝະ໼rtialD;戂y;䐟r;쀀𝔓i;䎦;䎠usMinus;䂱Āipຢອncareplanåڝf;愙Ȁ;eio຺ູ໠໤檻cedesȀ;EST່້໏໚扺qual;檯lantEqual;扼ilde;找me;怳Ādp໩໮uct;戏ortionĀ;aȥ໹l;戝Āci༁༆r;쀀𝒫;䎨ȀUfos༑༖༛༟OT耻"䀢r;쀀𝔔pf;愚cr;쀀𝒬؀BEacefhiorsu༾གྷཇའཱིྦྷྪྭ႖ႩႴႾarr;椐G耻®䂮ƀcnrཎནབute;䅔g;柫rĀ;tཛྷཝ憠l;椖ƀaeyཧཬཱron;䅘dil;䅖;䐠Ā;vླྀཹ愜erseĀEUྂྙĀlq྇ྎement;戋uilibrium;懋pEquilibrium;楯r»ཹo;䎡ghtЀACDFTUVa࿁࿫࿳ဢဨၛႇϘĀnr࿆࿒gleBracket;柩rowƀ;BL࿜࿝࿡憒ar;懥eftArrow;懄eiling;按oǵ࿹\0စbleBracket;柧nǔည\0နeeVector;楝ectorĀ;Bဝသ懂ar;楕loor;挋Āerိ၃eƀ;AVဵံြ抢rrow;憦ector;楛iangleƀ;BEၐၑၕ抳ar;槐qual;抵pƀDTVၣၮၸownVector;楏eeVector;楜ectorĀ;Bႂႃ憾ar;楔ectorĀ;B႑႒懀ar;楓Āpuႛ႞f;愝ndImplies;楰ightarrow;懛ĀchႹႼr;愛;憱leDelayed;槴ڀHOacfhimoqstuფჱჷჽᄙᄞᅑᅖᅡᅧᆵᆻᆿĀCcჩხHcy;䐩y;䐨FTcy;䐬cute;䅚ʀ;aeiyᄈᄉᄎᄓᄗ檼ron;䅠dil;䅞rc;䅜;䐡r;쀀𝔖ortȀDLRUᄪᄴᄾᅉownArrow»ОeftArrow»࢚ightArrow»࿝pArrow;憑gma;䎣allCircle;战pf;쀀𝕊ɲᅭ\0\0ᅰt;戚areȀ;ISUᅻᅼᆉᆯ斡ntersection;抓uĀbpᆏᆞsetĀ;Eᆗᆘ抏qual;抑ersetĀ;Eᆨᆩ抐qual;抒nion;抔cr;쀀𝒮ar;拆ȀbcmpᇈᇛሉላĀ;sᇍᇎ拐etĀ;Eᇍᇕqual;抆ĀchᇠህeedsȀ;ESTᇭᇮᇴᇿ扻qual;檰lantEqual;扽ilde;承Tháྌ;我ƀ;esሒሓሣ拑rsetĀ;Eሜም抃qual;抇et»ሓրHRSacfhiorsሾቄ቉ቕ቞ቱቶኟዂወዑORN耻Þ䃞ADE;愢ĀHc቎ቒcy;䐋y;䐦Ābuቚቜ;䀉;䎤ƀaeyብቪቯron;䅤dil;䅢;䐢r;쀀𝔗Āeiቻ኉ǲኀ\0ኇefore;戴a;䎘Ācn኎ኘkSpace;쀀  Space;怉ldeȀ;EFTካኬኲኼ戼qual;扃ullEqual;扅ilde;扈pf;쀀𝕋ipleDot;惛Āctዖዛr;쀀𝒯rok;䅦ૡዷጎጚጦ\0ጬጱ\0\0\0\0\0ጸጽ፷ᎅ\0᏿ᐄᐊᐐĀcrዻጁute耻Ú䃚rĀ;oጇገ憟cir;楉rǣጓ\0጖y;䐎ve;䅬Āiyጞጣrc耻Û䃛;䐣blac;䅰r;쀀𝔘rave耻Ù䃙acr;䅪Ādiፁ፩erĀBPፈ፝Āarፍፐr;䁟acĀekፗፙ;揟et;掵arenthesis;揝onĀ;P፰፱拃lus;抎Āgp፻፿on;䅲f;쀀𝕌ЀADETadps᎕ᎮᎸᏄϨᏒᏗᏳrrowƀ;BDᅐᎠᎤar;椒ownArrow;懅ownArrow;憕quilibrium;楮eeĀ;AᏋᏌ报rrow;憥ownáϳerĀLRᏞᏨeftArrow;憖ightArrow;憗iĀ;lᏹᏺ䏒on;䎥ing;䅮cr;쀀𝒰ilde;䅨ml耻Ü䃜ҀDbcdefosvᐧᐬᐰᐳᐾᒅᒊᒐᒖash;披ar;櫫y;䐒ashĀ;lᐻᐼ抩;櫦Āerᑃᑅ;拁ƀbtyᑌᑐᑺar;怖Ā;iᑏᑕcalȀBLSTᑡᑥᑪᑴar;戣ine;䁼eparator;杘ilde;所ThinSpace;怊r;쀀𝔙pf;쀀𝕍cr;쀀𝒱dash;抪ʀcefosᒧᒬᒱᒶᒼirc;䅴dge;拀r;쀀𝔚pf;쀀𝕎cr;쀀𝒲Ȁfiosᓋᓐᓒᓘr;쀀𝔛;䎞pf;쀀𝕏cr;쀀𝒳ҀAIUacfosuᓱᓵᓹᓽᔄᔏᔔᔚᔠcy;䐯cy;䐇cy;䐮cute耻Ý䃝Āiyᔉᔍrc;䅶;䐫r;쀀𝔜pf;쀀𝕐cr;쀀𝒴ml;䅸ЀHacdefosᔵᔹᔿᕋᕏᕝᕠᕤcy;䐖cute;䅹Āayᕄᕉron;䅽;䐗ot;䅻ǲᕔ\0ᕛoWidtè૙a;䎖r;愨pf;愤cr;쀀𝒵௡ᖃᖊᖐ\0ᖰᖶᖿ\0\0\0\0ᗆᗛᗫᙟ᙭\0ᚕ᚛ᚲᚹ\0ᚾcute耻á䃡reve;䄃̀;Ediuyᖜᖝᖡᖣᖨᖭ戾;쀀∾̳;房rc耻â䃢te肻´̆;䐰lig耻æ䃦Ā;r²ᖺ;쀀𝔞rave耻à䃠ĀepᗊᗖĀfpᗏᗔsym;愵èᗓha;䎱ĀapᗟcĀclᗤᗧr;䄁g;樿ɤᗰ\0\0ᘊʀ;adsvᗺᗻᗿᘁᘇ戧nd;橕;橜lope;橘;橚΀;elmrszᘘᘙᘛᘞᘿᙏᙙ戠;榤e»ᘙsdĀ;aᘥᘦ戡ѡᘰᘲᘴᘶᘸᘺᘼᘾ;榨;榩;榪;榫;榬;榭;榮;榯tĀ;vᙅᙆ戟bĀ;dᙌᙍ抾;榝Āptᙔᙗh;戢»¹arr;捼Āgpᙣᙧon;䄅f;쀀𝕒΀;Eaeiop዁ᙻᙽᚂᚄᚇᚊ;橰cir;橯;扊d;手s;䀧roxĀ;e዁ᚒñᚃing耻å䃥ƀctyᚡᚦᚨr;쀀𝒶;䀪mpĀ;e዁ᚯñʈilde耻ã䃣ml耻ä䃤Āciᛂᛈoninôɲnt;樑ࠀNabcdefiklnoprsu᛭ᛱᜰ᜼ᝃᝈ᝸᝽០៦ᠹᡐᜍ᤽᥈ᥰot;櫭Ācrᛶ᜞kȀcepsᜀᜅᜍᜓong;扌psilon;䏶rime;怵imĀ;e᜚᜛戽q;拍Ŷᜢᜦee;抽edĀ;gᜬᜭ挅e»ᜭrkĀ;t፜᜷brk;掶Āoyᜁᝁ;䐱quo;怞ʀcmprtᝓ᝛ᝡᝤᝨausĀ;eĊĉptyv;榰séᜌnoõēƀahwᝯ᝱ᝳ;䎲;愶een;扬r;쀀𝔟g΀costuvwឍឝឳេ៕៛៞ƀaiuបពរðݠrc;旯p»፱ƀdptឤឨឭot;樀lus;樁imes;樂ɱឹ\0\0ើcup;樆ar;昅riangleĀdu៍្own;施p;斳plus;樄eåᑄåᒭarow;植ƀako៭ᠦᠵĀcn៲ᠣkƀlst៺֫᠂ozenge;槫riangleȀ;dlr᠒᠓᠘᠝斴own;斾eft;旂ight;斸k;搣Ʊᠫ\0ᠳƲᠯ\0ᠱ;斒;斑4;斓ck;斈ĀeoᠾᡍĀ;qᡃᡆ쀀=⃥uiv;쀀≡⃥t;挐Ȁptwxᡙᡞᡧᡬf;쀀𝕓Ā;tᏋᡣom»Ꮜtie;拈؀DHUVbdhmptuvᢅᢖᢪᢻᣗᣛᣬ᣿ᤅᤊᤐᤡȀLRlrᢎᢐᢒᢔ;敗;敔;敖;敓ʀ;DUduᢡᢢᢤᢦᢨ敐;敦;敩;敤;敧ȀLRlrᢳᢵᢷᢹ;敝;敚;敜;教΀;HLRhlrᣊᣋᣍᣏᣑᣓᣕ救;敬;散;敠;敫;敢;敟ox;槉ȀLRlrᣤᣦᣨᣪ;敕;敒;攐;攌ʀ;DUduڽ᣷᣹᣻᣽;敥;敨;攬;攴inus;抟lus;択imes;抠ȀLRlrᤙᤛᤝ᤟;敛;敘;攘;攔΀;HLRhlrᤰᤱᤳᤵᤷ᤻᤹攂;敪;敡;敞;攼;攤;攜Āevģ᥂bar耻¦䂦Ȁceioᥑᥖᥚᥠr;쀀𝒷mi;恏mĀ;e᜚᜜lƀ;bhᥨᥩᥫ䁜;槅sub;柈Ŭᥴ᥾lĀ;e᥹᥺怢t»᥺pƀ;Eeįᦅᦇ;檮Ā;qۜۛೡᦧ\0᧨ᨑᨕᨲ\0ᨷᩐ\0\0᪴\0\0᫁\0\0ᬡᬮ᭍᭒\0᯽\0ᰌƀcpr᦭ᦲ᧝ute;䄇̀;abcdsᦿᧀᧄ᧊᧕᧙戩nd;橄rcup;橉Āau᧏᧒p;橋p;橇ot;橀;쀀∩︀Āeo᧢᧥t;恁îړȀaeiu᧰᧻ᨁᨅǰ᧵\0᧸s;橍on;䄍dil耻ç䃧rc;䄉psĀ;sᨌᨍ橌m;橐ot;䄋ƀdmnᨛᨠᨦil肻¸ƭptyv;榲t脀¢;eᨭᨮ䂢räƲr;쀀𝔠ƀceiᨽᩀᩍy;䑇ckĀ;mᩇᩈ朓ark»ᩈ;䏇r΀;Ecefms᩟᩠ᩢᩫ᪤᪪᪮旋;槃ƀ;elᩩᩪᩭ䋆q;扗eɡᩴ\0\0᪈rrowĀlr᩼᪁eft;憺ight;憻ʀRSacd᪒᪔᪖᪚᪟»ཇ;擈st;抛irc;抚ash;抝nint;樐id;櫯cir;槂ubsĀ;u᪻᪼晣it»᪼ˬ᫇᫔᫺\0ᬊonĀ;eᫍᫎ䀺Ā;qÇÆɭ᫙\0\0᫢aĀ;t᫞᫟䀬;䁀ƀ;fl᫨᫩᫫戁îᅠeĀmx᫱᫶ent»᫩eóɍǧ᫾\0ᬇĀ;dኻᬂot;橭nôɆƀfryᬐᬔᬗ;쀀𝕔oäɔ脀©;sŕᬝr;愗Āaoᬥᬩrr;憵ss;朗Ācuᬲᬷr;쀀𝒸Ābpᬼ᭄Ā;eᭁᭂ櫏;櫑Ā;eᭉᭊ櫐;櫒dot;拯΀delprvw᭠᭬᭷ᮂᮬᯔ᯹arrĀlr᭨᭪;椸;椵ɰ᭲\0\0᭵r;拞c;拟arrĀ;p᭿ᮀ憶;椽̀;bcdosᮏᮐᮖᮡᮥᮨ截rcap;橈Āauᮛᮞp;橆p;橊ot;抍r;橅;쀀∪︀Ȁalrv᮵ᮿᯞᯣrrĀ;mᮼᮽ憷;椼yƀevwᯇᯔᯘqɰᯎ\0\0ᯒreã᭳uã᭵ee;拎edge;拏en耻¤䂤earrowĀlrᯮ᯳eft»ᮀight»ᮽeäᯝĀciᰁᰇoninôǷnt;戱lcty;挭ঀAHabcdefhijlorstuwz᰸᰻᰿ᱝᱩᱵᲊᲞᲬᲷ᳻᳿ᴍᵻᶑᶫᶻ᷆᷍rò΁ar;楥Ȁglrs᱈ᱍ᱒᱔ger;怠eth;愸òᄳhĀ;vᱚᱛ怐»ऊūᱡᱧarow;椏aã̕Āayᱮᱳron;䄏;䐴ƀ;ao̲ᱼᲄĀgrʿᲁr;懊tseq;橷ƀglmᲑᲔᲘ耻°䂰ta;䎴ptyv;榱ĀirᲣᲨsht;楿;쀀𝔡arĀlrᲳᲵ»ࣜ»သʀaegsv᳂͸᳖᳜᳠mƀ;oș᳊᳔ndĀ;ș᳑uit;晦amma;䏝in;拲ƀ;io᳧᳨᳸䃷de脀÷;o᳧ᳰntimes;拇nø᳷cy;䑒cɯᴆ\0\0ᴊrn;挞op;挍ʀlptuwᴘᴝᴢᵉᵕlar;䀤f;쀀𝕕ʀ;emps̋ᴭᴷᴽᵂqĀ;d͒ᴳot;扑inus;戸lus;戔quare;抡blebarwedgåúnƀadhᄮᵝᵧownarrowóᲃarpoonĀlrᵲᵶefôᲴighôᲶŢᵿᶅkaro÷གɯᶊ\0\0ᶎrn;挟op;挌ƀcotᶘᶣᶦĀryᶝᶡ;쀀𝒹;䑕l;槶rok;䄑Ādrᶰᶴot;拱iĀ;fᶺ᠖斿Āah᷀᷃ròЩaòྦangle;榦Āci᷒ᷕy;䑟grarr;柿ऀDacdefglmnopqrstuxḁḉḙḸոḼṉṡṾấắẽỡἪἷὄ὎὚ĀDoḆᴴoôᲉĀcsḎḔute耻é䃩ter;橮ȀaioyḢḧḱḶron;䄛rĀ;cḭḮ扖耻ê䃪lon;払;䑍ot;䄗ĀDrṁṅot;扒;쀀𝔢ƀ;rsṐṑṗ檚ave耻è䃨Ā;dṜṝ檖ot;檘Ȁ;ilsṪṫṲṴ檙nters;揧;愓Ā;dṹṺ檕ot;檗ƀapsẅẉẗcr;䄓tyƀ;svẒẓẕ戅et»ẓpĀ1;ẝẤĳạả;怄;怅怃ĀgsẪẬ;䅋p;怂ĀgpẴẸon;䄙f;쀀𝕖ƀalsỄỎỒrĀ;sỊị拕l;槣us;橱iƀ;lvỚớở䎵on»ớ;䏵ȀcsuvỪỳἋἣĀioữḱrc»Ḯɩỹ\0\0ỻíՈantĀglἂἆtr»ṝess»Ṻƀaeiἒ἖Ἒls;䀽st;扟vĀ;DȵἠD;橸parsl;槥ĀDaἯἳot;打rr;楱ƀcdiἾὁỸr;愯oô͒ĀahὉὋ;䎷耻ð䃰Āmrὓὗl耻ë䃫o;悬ƀcipὡὤὧl;䀡sôծĀeoὬὴctatioîՙnentialåչৡᾒ\0ᾞ\0ᾡᾧ\0\0ῆῌ\0ΐ\0ῦῪ \0 ⁚llingdotseñṄy;䑄male;晀ƀilrᾭᾳ῁lig;耀ﬃɩᾹ\0\0᾽g;耀ﬀig;耀ﬄ;쀀𝔣lig;耀ﬁlig;쀀fjƀaltῙ῜ῡt;晭ig;耀ﬂns;斱of;䆒ǰ΅\0ῳf;쀀𝕗ĀakֿῷĀ;vῼ´拔;櫙artint;樍Āao‌⁕Ācs‑⁒α‚‰‸⁅⁈\0⁐β•‥‧‪‬\0‮耻½䂽;慓耻¼䂼;慕;慙;慛Ƴ‴\0‶;慔;慖ʴ‾⁁\0\0⁃耻¾䂾;慗;慜5;慘ƶ⁌\0⁎;慚;慝8;慞l;恄wn;挢cr;쀀𝒻ࢀEabcdefgijlnorstv₂₉₟₥₰₴⃰⃵⃺⃿℃ℒℸ̗ℾ⅒↞Ā;lٍ₇;檌ƀcmpₐₕ₝ute;䇵maĀ;dₜ᳚䎳;檆reve;䄟Āiy₪₮rc;䄝;䐳ot;䄡Ȁ;lqsؾق₽⃉ƀ;qsؾٌ⃄lanô٥Ȁ;cdl٥⃒⃥⃕c;檩otĀ;o⃜⃝檀Ā;l⃢⃣檂;檄Ā;e⃪⃭쀀⋛︀s;檔r;쀀𝔤Ā;gٳ؛mel;愷cy;䑓Ȁ;Eajٚℌℎℐ;檒;檥;檤ȀEaesℛℝ℩ℴ;扩pĀ;p℣ℤ檊rox»ℤĀ;q℮ℯ檈Ā;q℮ℛim;拧pf;쀀𝕘Āci⅃ⅆr;愊mƀ;el٫ⅎ⅐;檎;檐茀>;cdlqr׮ⅠⅪⅮⅳⅹĀciⅥⅧ;檧r;橺ot;拗Par;榕uest;橼ʀadelsↄⅪ←ٖ↛ǰ↉\0↎proø₞r;楸qĀlqؿ↖lesó₈ií٫Āen↣↭rtneqq;쀀≩︀Å↪ԀAabcefkosy⇄⇇⇱⇵⇺∘∝∯≨≽ròΠȀilmr⇐⇔⇗⇛rsðᒄf»․ilôکĀdr⇠⇤cy;䑊ƀ;cwࣴ⇫⇯ir;楈;憭ar;意irc;䄥ƀalr∁∎∓rtsĀ;u∉∊晥it»∊lip;怦con;抹r;쀀𝔥sĀew∣∩arow;椥arow;椦ʀamopr∺∾≃≞≣rr;懿tht;戻kĀlr≉≓eftarrow;憩ightarrow;憪f;쀀𝕙bar;怕ƀclt≯≴≸r;쀀𝒽asè⇴rok;䄧Ābp⊂⊇ull;恃hen»ᱛૡ⊣\0⊪\0⊸⋅⋎\0⋕⋳\0\0⋸⌢⍧⍢⍿\0⎆⎪⎴cute耻í䃭ƀ;iyݱ⊰⊵rc耻î䃮;䐸Ācx⊼⊿y;䐵cl耻¡䂡ĀfrΟ⋉;쀀𝔦rave耻ì䃬Ȁ;inoܾ⋝⋩⋮Āin⋢⋦nt;樌t;戭fin;槜ta;愩lig;䄳ƀaop⋾⌚⌝ƀcgt⌅⌈⌗r;䄫ƀelpܟ⌏⌓inåގarôܠh;䄱f;抷ed;䆵ʀ;cfotӴ⌬⌱⌽⍁are;愅inĀ;t⌸⌹戞ie;槝doô⌙ʀ;celpݗ⍌⍐⍛⍡al;抺Āgr⍕⍙eróᕣã⍍arhk;樗rod;樼Ȁcgpt⍯⍲⍶⍻y;䑑on;䄯f;쀀𝕚a;䎹uest耻¿䂿Āci⎊⎏r;쀀𝒾nʀ;EdsvӴ⎛⎝⎡ӳ;拹ot;拵Ā;v⎦⎧拴;拳Ā;iݷ⎮lde;䄩ǫ⎸\0⎼cy;䑖l耻ï䃯̀cfmosu⏌⏗⏜⏡⏧⏵Āiy⏑⏕rc;䄵;䐹r;쀀𝔧ath;䈷pf;쀀𝕛ǣ⏬\0⏱r;쀀𝒿rcy;䑘kcy;䑔Ѐacfghjos␋␖␢␧␭␱␵␻ppaĀ;v␓␔䎺;䏰Āey␛␠dil;䄷;䐺r;쀀𝔨reen;䄸cy;䑅cy;䑜pf;쀀𝕜cr;쀀𝓀஀ABEHabcdefghjlmnoprstuv⑰⒁⒆⒍⒑┎┽╚▀♎♞♥♹♽⚚⚲⛘❝❨➋⟀⠁⠒ƀart⑷⑺⑼rò৆òΕail;椛arr;椎Ā;gঔ⒋;檋ar;楢ॣ⒥\0⒪\0⒱\0\0\0\0\0⒵Ⓔ\0ⓆⓈⓍ\0⓹ute;䄺mptyv;榴raîࡌbda;䎻gƀ;dlࢎⓁⓃ;榑åࢎ;檅uo耻«䂫rЀ;bfhlpst࢙ⓞⓦⓩ⓫⓮⓱⓵Ā;f࢝ⓣs;椟s;椝ë≒p;憫l;椹im;楳l;憢ƀ;ae⓿─┄檫il;椙Ā;s┉┊檭;쀀⪭︀ƀabr┕┙┝rr;椌rk;杲Āak┢┬cĀek┨┪;䁻;䁛Āes┱┳;榋lĀdu┹┻;榏;榍Ȁaeuy╆╋╖╘ron;䄾Ādi═╔il;䄼ìࢰâ┩;䐻Ȁcqrs╣╦╭╽a;椶uoĀ;rนᝆĀdu╲╷har;楧shar;楋h;憲ʀ;fgqs▋▌উ◳◿扤tʀahlrt▘▤▷◂◨rrowĀ;t࢙□aé⓶arpoonĀdu▯▴own»њp»०eftarrows;懇ightƀahs◍◖◞rrowĀ;sࣴࢧarpoonó྘quigarro÷⇰hreetimes;拋ƀ;qs▋ও◺lanôবʀ;cdgsব☊☍☝☨c;檨otĀ;o☔☕橿Ā;r☚☛檁;檃Ā;e☢☥쀀⋚︀s;檓ʀadegs☳☹☽♉♋pproøⓆot;拖qĀgq♃♅ôউgtò⒌ôছiíলƀilr♕࣡♚sht;楼;쀀𝔩Ā;Eজ♣;檑š♩♶rĀdu▲♮Ā;l॥♳;楪lk;斄cy;䑙ʀ;achtੈ⚈⚋⚑⚖rò◁orneòᴈard;楫ri;旺Āio⚟⚤dot;䅀ustĀ;a⚬⚭掰che»⚭ȀEaes⚻⚽⛉⛔;扨pĀ;p⛃⛄檉rox»⛄Ā;q⛎⛏檇Ā;q⛎⚻im;拦Ѐabnoptwz⛩⛴⛷✚✯❁❇❐Ānr⛮⛱g;柬r;懽rëࣁgƀlmr⛿✍✔eftĀar০✇ightá৲apsto;柼ightá৽parrowĀlr✥✩efô⓭ight;憬ƀafl✶✹✽r;榅;쀀𝕝us;樭imes;樴š❋❏st;戗áፎƀ;ef❗❘᠀旊nge»❘arĀ;l❤❥䀨t;榓ʀachmt❳❶❼➅➇ròࢨorneòᶌarĀ;d྘➃;業;怎ri;抿̀achiqt➘➝ੀ➢➮➻quo;怹r;쀀𝓁mƀ;egল➪➬;檍;檏Ābu┪➳oĀ;rฟ➹;怚rok;䅂萀<;cdhilqrࠫ⟒☹⟜⟠⟥⟪⟰Āci⟗⟙;檦r;橹reå◲mes;拉arr;楶uest;橻ĀPi⟵⟹ar;榖ƀ;ef⠀भ᠛旃rĀdu⠇⠍shar;楊har;楦Āen⠗⠡rtneqq;쀀≨︀Å⠞܀Dacdefhilnopsu⡀⡅⢂⢎⢓⢠⢥⢨⣚⣢⣤ઃ⣳⤂Dot;戺Ȁclpr⡎⡒⡣⡽r耻¯䂯Āet⡗⡙;時Ā;e⡞⡟朠se»⡟Ā;sျ⡨toȀ;dluျ⡳⡷⡻owîҌefôएðᏑker;斮Āoy⢇⢌mma;権;䐼ash;怔asuredangle»ᘦr;쀀𝔪o;愧ƀcdn⢯⢴⣉ro耻µ䂵Ȁ;acdᑤ⢽⣀⣄sôᚧir;櫰ot肻·Ƶusƀ;bd⣒ᤃ⣓戒Ā;uᴼ⣘;横ţ⣞⣡p;櫛ò−ðઁĀdp⣩⣮els;抧f;쀀𝕞Āct⣸⣽r;쀀𝓂pos»ᖝƀ;lm⤉⤊⤍䎼timap;抸ఀGLRVabcdefghijlmoprstuvw⥂⥓⥾⦉⦘⧚⧩⨕⨚⩘⩝⪃⪕⪤⪨⬄⬇⭄⭿⮮ⰴⱧⱼ⳩Āgt⥇⥋;쀀⋙̸Ā;v⥐௏쀀≫⃒ƀelt⥚⥲⥶ftĀar⥡⥧rrow;懍ightarrow;懎;쀀⋘̸Ā;v⥻ే쀀≪⃒ightarrow;懏ĀDd⦎⦓ash;抯ash;抮ʀbcnpt⦣⦧⦬⦱⧌la»˞ute;䅄g;쀀∠⃒ʀ;Eiop඄⦼⧀⧅⧈;쀀⩰̸d;쀀≋̸s;䅉roø඄urĀ;a⧓⧔普lĀ;s⧓ସǳ⧟\0⧣p肻 ଷmpĀ;e௹ఀʀaeouy⧴⧾⨃⨐⨓ǰ⧹\0⧻;橃on;䅈dil;䅆ngĀ;dൾ⨊ot;쀀⩭̸p;橂;䐽ash;怓΀;Aadqsxஒ⨩⨭⨻⩁⩅⩐rr;懗rĀhr⨳⨶k;椤Ā;oᏲᏰot;쀀≐̸uiöୣĀei⩊⩎ar;椨í஘istĀ;s஠டr;쀀𝔫ȀEest௅⩦⩹⩼ƀ;qs஼⩭௡ƀ;qs஼௅⩴lanô௢ií௪Ā;rஶ⪁»ஷƀAap⪊⪍⪑rò⥱rr;憮ar;櫲ƀ;svྍ⪜ྌĀ;d⪡⪢拼;拺cy;䑚΀AEadest⪷⪺⪾⫂⫅⫶⫹rò⥦;쀀≦̸rr;憚r;急Ȁ;fqs఻⫎⫣⫯tĀar⫔⫙rro÷⫁ightarro÷⪐ƀ;qs఻⪺⫪lanôౕĀ;sౕ⫴»శiíౝĀ;rవ⫾iĀ;eచథiäඐĀpt⬌⬑f;쀀𝕟膀¬;in⬙⬚⬶䂬nȀ;Edvஉ⬤⬨⬮;쀀⋹̸ot;쀀⋵̸ǡஉ⬳⬵;拷;拶iĀ;vಸ⬼ǡಸ⭁⭃;拾;拽ƀaor⭋⭣⭩rȀ;ast୻⭕⭚⭟lleì୻l;쀀⫽⃥;쀀∂̸lint;樔ƀ;ceಒ⭰⭳uåಥĀ;cಘ⭸Ā;eಒ⭽ñಘȀAait⮈⮋⮝⮧rò⦈rrƀ;cw⮔⮕⮙憛;쀀⤳̸;쀀↝̸ghtarrow»⮕riĀ;eೋೖ΀chimpqu⮽⯍⯙⬄୸⯤⯯Ȁ;cerല⯆ഷ⯉uå൅;쀀𝓃ortɭ⬅\0\0⯖ará⭖mĀ;e൮⯟Ā;q൴൳suĀbp⯫⯭å೸åഋƀbcp⯶ⰑⰙȀ;Ees⯿ⰀഢⰄ抄;쀀⫅̸etĀ;eഛⰋqĀ;qണⰀcĀ;eലⰗñസȀ;EesⰢⰣൟⰧ抅;쀀⫆̸etĀ;e൘ⰮqĀ;qൠⰣȀgilrⰽⰿⱅⱇìௗlde耻ñ䃱çృiangleĀlrⱒⱜeftĀ;eచⱚñదightĀ;eೋⱥñ೗Ā;mⱬⱭ䎽ƀ;esⱴⱵⱹ䀣ro;愖p;怇ҀDHadgilrsⲏⲔⲙⲞⲣⲰⲶⳓⳣash;抭arr;椄p;쀀≍⃒ash;抬ĀetⲨⲬ;쀀≥⃒;쀀>⃒nfin;槞ƀAetⲽⳁⳅrr;椂;쀀≤⃒Ā;rⳊⳍ쀀<⃒ie;쀀⊴⃒ĀAtⳘⳜrr;椃rie;쀀⊵⃒im;쀀∼⃒ƀAan⳰⳴ⴂrr;懖rĀhr⳺⳽k;椣Ā;oᏧᏥear;椧ቓ᪕\0\0\0\0\0\0\0\0\0\0\0\0\0ⴭ\0ⴸⵈⵠⵥ⵲ⶄᬇ\0\0ⶍⶫ\0ⷈⷎ\0ⷜ⸙⸫⸾⹃Ācsⴱ᪗ute耻ó䃳ĀiyⴼⵅrĀ;c᪞ⵂ耻ô䃴;䐾ʀabios᪠ⵒⵗǈⵚlac;䅑v;樸old;榼lig;䅓Ācr⵩⵭ir;榿;쀀𝔬ͯ⵹\0\0⵼\0ⶂn;䋛ave耻ò䃲;槁Ābmⶈ෴ar;榵Ȁacitⶕ⶘ⶥⶨrò᪀Āir⶝ⶠr;榾oss;榻nå๒;槀ƀaeiⶱⶵⶹcr;䅍ga;䏉ƀcdnⷀⷅǍron;䎿;榶pf;쀀𝕠ƀaelⷔ⷗ǒr;榷rp;榹΀;adiosvⷪⷫⷮ⸈⸍⸐⸖戨rò᪆Ȁ;efmⷷⷸ⸂⸅橝rĀ;oⷾⷿ愴f»ⷿ耻ª䂪耻º䂺gof;抶r;橖lope;橗;橛ƀclo⸟⸡⸧ò⸁ash耻ø䃸l;折iŬⸯ⸴de耻õ䃵esĀ;aǛ⸺s;樶ml耻ö䃶bar;挽ૡ⹞\0⹽\0⺀⺝\0⺢⺹\0\0⻋ຜ\0⼓\0\0⼫⾼\0⿈rȀ;astЃ⹧⹲຅脀¶;l⹭⹮䂶leìЃɩ⹸\0\0⹻m;櫳;櫽y;䐿rʀcimpt⺋⺏⺓ᡥ⺗nt;䀥od;䀮il;怰enk;怱r;쀀𝔭ƀimo⺨⺰⺴Ā;v⺭⺮䏆;䏕maô੶ne;明ƀ;tv⺿⻀⻈䏀chfork»´;䏖Āau⻏⻟nĀck⻕⻝kĀ;h⇴⻛;愎ö⇴sҀ;abcdemst⻳⻴ᤈ⻹⻽⼄⼆⼊⼎䀫cir;樣ir;樢Āouᵀ⼂;樥;橲n肻±ຝim;樦wo;樧ƀipu⼙⼠⼥ntint;樕f;쀀𝕡nd耻£䂣Ԁ;Eaceinosu່⼿⽁⽄⽇⾁⾉⾒⽾⾶;檳p;檷uå໙Ā;c໎⽌̀;acens່⽙⽟⽦⽨⽾pproø⽃urlyeñ໙ñ໎ƀaes⽯⽶⽺pprox;檹qq;檵im;拨iíໟmeĀ;s⾈ຮ怲ƀEas⽸⾐⽺ð⽵ƀdfp໬⾙⾯ƀals⾠⾥⾪lar;挮ine;挒urf;挓Ā;t໻⾴ï໻rel;抰Āci⿀⿅r;쀀𝓅;䏈ncsp;怈̀fiopsu⿚⋢⿟⿥⿫⿱r;쀀𝔮pf;쀀𝕢rime;恗cr;쀀𝓆ƀaeo⿸〉〓tĀei⿾々rnionóڰnt;樖stĀ;e【】䀿ñἙô༔઀ABHabcdefhilmnoprstux぀けさすムㄎㄫㅇㅢㅲㆎ㈆㈕㈤㈩㉘㉮㉲㊐㊰㊷ƀartぇおがròႳòϝail;検aròᱥar;楤΀cdenqrtとふへみわゔヌĀeuねぱ;쀀∽̱te;䅕iãᅮmptyv;榳gȀ;del࿑らるろ;榒;榥å࿑uo耻»䂻rր;abcfhlpstw࿜ガクシスゼゾダッデナp;極Ā;f࿠ゴs;椠;椳s;椞ë≝ð✮l;楅im;楴l;憣;憝Āaiパフil;椚oĀ;nホボ戶aló༞ƀabrョリヮrò៥rk;杳ĀakンヽcĀekヹ・;䁽;䁝Āes㄂㄄;榌lĀduㄊㄌ;榎;榐Ȁaeuyㄗㄜㄧㄩron;䅙Ādiㄡㄥil;䅗ì࿲âヺ;䑀Ȁclqsㄴㄷㄽㅄa;椷dhar;楩uoĀ;rȎȍh;憳ƀacgㅎㅟངlȀ;ipsླྀㅘㅛႜnåႻarôྩt;断ƀilrㅩဣㅮsht;楽;쀀𝔯ĀaoㅷㆆrĀduㅽㅿ»ѻĀ;l႑ㆄ;楬Ā;vㆋㆌ䏁;䏱ƀgns㆕ㇹㇼht̀ahlrstㆤㆰ㇂㇘㇤㇮rrowĀ;t࿜ㆭaéトarpoonĀduㆻㆿowîㅾp»႒eftĀah㇊㇐rrowó࿪arpoonóՑightarrows;應quigarro÷ニhreetimes;拌g;䋚ingdotseñἲƀahm㈍㈐㈓rò࿪aòՑ;怏oustĀ;a㈞㈟掱che»㈟mid;櫮Ȁabpt㈲㈽㉀㉒Ānr㈷㈺g;柭r;懾rëဃƀafl㉇㉊㉎r;榆;쀀𝕣us;樮imes;樵Āap㉝㉧rĀ;g㉣㉤䀩t;榔olint;樒arò㇣Ȁachq㉻㊀Ⴜ㊅quo;怺r;쀀𝓇Ābu・㊊oĀ;rȔȓƀhir㊗㊛㊠reåㇸmes;拊iȀ;efl㊪ၙᠡ㊫方tri;槎luhar;楨;愞ൡ㋕㋛㋟㌬㌸㍱\0㍺㎤\0\0㏬㏰\0㐨㑈㑚㒭㒱㓊㓱\0㘖\0\0㘳cute;䅛quï➺Ԁ;Eaceinpsyᇭ㋳㋵㋿㌂㌋㌏㌟㌦㌩;檴ǰ㋺\0㋼;檸on;䅡uåᇾĀ;dᇳ㌇il;䅟rc;䅝ƀEas㌖㌘㌛;檶p;檺im;择olint;樓iíሄ;䑁otƀ;be㌴ᵇ㌵担;橦΀Aacmstx㍆㍊㍗㍛㍞㍣㍭rr;懘rĀhr㍐㍒ë∨Ā;oਸ਼਴t耻§䂧i;䀻war;椩mĀin㍩ðnuóñt;朶rĀ;o㍶⁕쀀𝔰Ȁacoy㎂㎆㎑㎠rp;景Āhy㎋㎏cy;䑉;䑈rtɭ㎙\0\0㎜iäᑤaraì⹯耻­䂭Āgm㎨㎴maƀ;fv㎱㎲㎲䏃;䏂Ѐ;deglnprካ㏅㏉㏎㏖㏞㏡㏦ot;橪Ā;q኱ኰĀ;E㏓㏔檞;檠Ā;E㏛㏜檝;檟e;扆lus;樤arr;楲aròᄽȀaeit㏸㐈㐏㐗Āls㏽㐄lsetmé㍪hp;樳parsl;槤Ādlᑣ㐔e;挣Ā;e㐜㐝檪Ā;s㐢㐣檬;쀀⪬︀ƀflp㐮㐳㑂tcy;䑌Ā;b㐸㐹䀯Ā;a㐾㐿槄r;挿f;쀀𝕤aĀdr㑍ЂesĀ;u㑔㑕晠it»㑕ƀcsu㑠㑹㒟Āau㑥㑯pĀ;sᆈ㑫;쀀⊓︀pĀ;sᆴ㑵;쀀⊔︀uĀbp㑿㒏ƀ;esᆗᆜ㒆etĀ;eᆗ㒍ñᆝƀ;esᆨᆭ㒖etĀ;eᆨ㒝ñᆮƀ;afᅻ㒦ְrť㒫ֱ»ᅼaròᅈȀcemt㒹㒾㓂㓅r;쀀𝓈tmîñiì㐕aræᆾĀar㓎㓕rĀ;f㓔ឿ昆Āan㓚㓭ightĀep㓣㓪psiloîỠhé⺯s»⡒ʀbcmnp㓻㕞ሉ㖋㖎Ҁ;Edemnprs㔎㔏㔑㔕㔞㔣㔬㔱㔶抂;櫅ot;檽Ā;dᇚ㔚ot;櫃ult;櫁ĀEe㔨㔪;櫋;把lus;檿arr;楹ƀeiu㔽㕒㕕tƀ;en㔎㕅㕋qĀ;qᇚ㔏eqĀ;q㔫㔨m;櫇Ābp㕚㕜;櫕;櫓c̀;acensᇭ㕬㕲㕹㕻㌦pproø㋺urlyeñᇾñᇳƀaes㖂㖈㌛pproø㌚qñ㌗g;晪ڀ123;Edehlmnps㖩㖬㖯ሜ㖲㖴㗀㗉㗕㗚㗟㗨㗭耻¹䂹耻²䂲耻³䂳;櫆Āos㖹㖼t;檾ub;櫘Ā;dሢ㗅ot;櫄sĀou㗏㗒l;柉b;櫗arr;楻ult;櫂ĀEe㗤㗦;櫌;抋lus;櫀ƀeiu㗴㘉㘌tƀ;enሜ㗼㘂qĀ;qሢ㖲eqĀ;q㗧㗤m;櫈Ābp㘑㘓;櫔;櫖ƀAan㘜㘠㘭rr;懙rĀhr㘦㘨ë∮Ā;oਫ਩war;椪lig耻ß䃟௡㙑㙝㙠ዎ㙳㙹\0㙾㛂\0\0\0\0\0㛛㜃\0㜉㝬\0\0\0㞇ɲ㙖\0\0㙛get;挖;䏄rë๟ƀaey㙦㙫㙰ron;䅥dil;䅣;䑂lrec;挕r;쀀𝔱Ȁeiko㚆㚝㚵㚼ǲ㚋\0㚑eĀ4fኄኁaƀ;sv㚘㚙㚛䎸ym;䏑Ācn㚢㚲kĀas㚨㚮pproø዁im»ኬsðኞĀas㚺㚮ð዁rn耻þ䃾Ǭ̟㛆⋧es膀×;bd㛏㛐㛘䃗Ā;aᤏ㛕r;樱;樰ƀeps㛡㛣㜀á⩍Ȁ;bcf҆㛬㛰㛴ot;挶ir;櫱Ā;o㛹㛼쀀𝕥rk;櫚á㍢rime;怴ƀaip㜏㜒㝤dåቈ΀adempst㜡㝍㝀㝑㝗㝜㝟ngleʀ;dlqr㜰㜱㜶㝀㝂斵own»ᶻeftĀ;e⠀㜾ñम;扜ightĀ;e㊪㝋ñၚot;旬inus;樺lus;樹b;槍ime;樻ezium;揢ƀcht㝲㝽㞁Āry㝷㝻;쀀𝓉;䑆cy;䑛rok;䅧Āio㞋㞎xô᝷headĀlr㞗㞠eftarro÷ࡏightarrow»ཝऀAHabcdfghlmoprstuw㟐㟓㟗㟤㟰㟼㠎㠜㠣㠴㡑㡝㡫㢩㣌㣒㣪㣶ròϭar;楣Ācr㟜㟢ute耻ú䃺òᅐrǣ㟪\0㟭y;䑞ve;䅭Āiy㟵㟺rc耻û䃻;䑃ƀabh㠃㠆㠋ròᎭlac;䅱aòᏃĀir㠓㠘sht;楾;쀀𝔲rave耻ù䃹š㠧㠱rĀlr㠬㠮»ॗ»ႃlk;斀Āct㠹㡍ɯ㠿\0\0㡊rnĀ;e㡅㡆挜r»㡆op;挏ri;旸Āal㡖㡚cr;䅫肻¨͉Āgp㡢㡦on;䅳f;쀀𝕦̀adhlsuᅋ㡸㡽፲㢑㢠ownáᎳarpoonĀlr㢈㢌efô㠭ighô㠯iƀ;hl㢙㢚㢜䏅»ᏺon»㢚parrows;懈ƀcit㢰㣄㣈ɯ㢶\0\0㣁rnĀ;e㢼㢽挝r»㢽op;挎ng;䅯ri;旹cr;쀀𝓊ƀdir㣙㣝㣢ot;拰lde;䅩iĀ;f㜰㣨»᠓Āam㣯㣲rò㢨l耻ü䃼angle;榧ހABDacdeflnoprsz㤜㤟㤩㤭㦵㦸㦽㧟㧤㧨㧳㧹㧽㨁㨠ròϷarĀ;v㤦㤧櫨;櫩asèϡĀnr㤲㤷grt;榜΀eknprst㓣㥆㥋㥒㥝㥤㦖appá␕othinçẖƀhir㓫⻈㥙opô⾵Ā;hᎷ㥢ïㆍĀiu㥩㥭gmá㎳Ābp㥲㦄setneqĀ;q㥽㦀쀀⊊︀;쀀⫋︀setneqĀ;q㦏㦒쀀⊋︀;쀀⫌︀Āhr㦛㦟etá㚜iangleĀlr㦪㦯eft»थight»ၑy;䐲ash»ံƀelr㧄㧒㧗ƀ;beⷪ㧋㧏ar;抻q;扚lip;拮Ābt㧜ᑨaòᑩr;쀀𝔳tré㦮suĀbp㧯㧱»ജ»൙pf;쀀𝕧roð໻tré㦴Ācu㨆㨋r;쀀𝓋Ābp㨐㨘nĀEe㦀㨖»㥾nĀEe㦒㨞»㦐igzag;榚΀cefoprs㨶㨻㩖㩛㩔㩡㩪irc;䅵Ādi㩀㩑Ābg㩅㩉ar;機eĀ;qᗺ㩏;扙erp;愘r;쀀𝔴pf;쀀𝕨Ā;eᑹ㩦atèᑹcr;쀀𝓌ૣណ㪇\0㪋\0㪐㪛\0\0㪝㪨㪫㪯\0\0㫃㫎\0㫘ៜ៟tré៑r;쀀𝔵ĀAa㪔㪗ròσrò৶;䎾ĀAa㪡㪤ròθrò৫að✓is;拻ƀdptឤ㪵㪾Āfl㪺ឩ;쀀𝕩imåឲĀAa㫇㫊ròώròਁĀcq㫒ីr;쀀𝓍Āpt៖㫜ré។Ѐacefiosu㫰㫽㬈㬌㬑㬕㬛㬡cĀuy㫶㫻te耻ý䃽;䑏Āiy㬂㬆rc;䅷;䑋n耻¥䂥r;쀀𝔶cy;䑗pf;쀀𝕪cr;쀀𝓎Ācm㬦㬩y;䑎l耻ÿ䃿Ԁacdefhiosw㭂㭈㭔㭘㭤㭩㭭㭴㭺㮀cute;䅺Āay㭍㭒ron;䅾;䐷ot;䅼Āet㭝㭡træᕟa;䎶r;쀀𝔷cy;䐶grarr;懝pf;쀀𝕫cr;쀀𝓏Ājn㮅㮇;怍j;怌'.split("").map(e=>e.charCodeAt(0))),ZB=new Map([[0,65533],[128,8364],[130,8218],[131,402],[132,8222],[133,8230],[134,8224],[135,8225],[136,710],[137,8240],[138,352],[139,8249],[140,338],[142,381],[145,8216],[146,8217],[147,8220],[148,8221],[149,8226],[150,8211],[151,8212],[152,732],[153,8482],[154,353],[155,8250],[156,339],[158,382],[159,376]]);function JB(e){var t;return e>=55296&&e<=57343||e>1114111?65533:(t=ZB.get(e))!==null&&t!==void 0?t:e}var fr;(function(e){e[e.NUM=35]="NUM",e[e.SEMI=59]="SEMI",e[e.EQUALS=61]="EQUALS",e[e.ZERO=48]="ZERO",e[e.NINE=57]="NINE",e[e.LOWER_A=97]="LOWER_A",e[e.LOWER_F=102]="LOWER_F",e[e.LOWER_X=120]="LOWER_X",e[e.LOWER_Z=122]="LOWER_Z",e[e.UPPER_A=65]="UPPER_A",e[e.UPPER_F=70]="UPPER_F",e[e.UPPER_Z=90]="UPPER_Z"})(fr||(fr={}));const eU=32;var Zs;(function(e){e[e.VALUE_LENGTH=49152]="VALUE_LENGTH",e[e.BRANCH_LENGTH=16256]="BRANCH_LENGTH",e[e.JUMP_TABLE=127]="JUMP_TABLE"})(Zs||(Zs={}));function db(e){return e>=fr.ZERO&&e<=fr.NINE}function tU(e){return e>=fr.UPPER_A&&e<=fr.UPPER_F||e>=fr.LOWER_A&&e<=fr.LOWER_F}function nU(e){return e>=fr.UPPER_A&&e<=fr.UPPER_Z||e>=fr.LOWER_A&&e<=fr.LOWER_Z||db(e)}function rU(e){return e===fr.EQUALS||nU(e)}var cr;(function(e){e[e.EntityStart=0]="EntityStart",e[e.NumericStart=1]="NumericStart",e[e.NumericDecimal=2]="NumericDecimal",e[e.NumericHex=3]="NumericHex",e[e.NamedEntity=4]="NamedEntity"})(cr||(cr={}));var Xi;(function(e){e[e.Legacy=0]="Legacy",e[e.Strict=1]="Strict",e[e.Attribute=2]="Attribute"})(Xi||(Xi={}));class aU{constructor(t,n,r){this.decodeTree=t,this.emitCodePoint=n,this.errors=r,this.state=cr.EntityStart,this.consumed=1,this.result=0,this.treeIndex=0,this.excess=1,this.decodeMode=Xi.Strict}startEntity(t){this.decodeMode=t,this.state=cr.EntityStart,this.result=0,this.treeIndex=0,this.excess=1,this.consumed=1}write(t,n){switch(this.state){case cr.EntityStart:return t.charCodeAt(n)===fr.NUM?(this.state=cr.NumericStart,this.consumed+=1,this.stateNumericStart(t,n+1)):(this.state=cr.NamedEntity,this.stateNamedEntity(t,n));case cr.NumericStart:return this.stateNumericStart(t,n);case cr.NumericDecimal:return this.stateNumericDecimal(t,n);case cr.NumericHex:return this.stateNumericHex(t,n);case cr.NamedEntity:return this.stateNamedEntity(t,n)}}stateNumericStart(t,n){return n>=t.length?-1:(t.charCodeAt(n)|eU)===fr.LOWER_X?(this.state=cr.NumericHex,this.consumed+=1,this.stateNumericHex(t,n+1)):(this.state=cr.NumericDecimal,this.stateNumericDecimal(t,n))}addToNumericResult(t,n,r,a){if(n!==r){const s=r-n;this.result=this.result*Math.pow(a,s)+Number.parseInt(t.substr(n,s),a),this.consumed+=s}}stateNumericHex(t,n){const r=n;for(;n<t.length;){const a=t.charCodeAt(n);if(db(a)||tU(a))n+=1;else return this.addToNumericResult(t,r,n,16),this.emitNumericEntity(a,3)}return this.addToNumericResult(t,r,n,16),-1}stateNumericDecimal(t,n){const r=n;for(;n<t.length;){const a=t.charCodeAt(n);if(db(a))n+=1;else return this.addToNumericResult(t,r,n,10),this.emitNumericEntity(a,2)}return this.addToNumericResult(t,r,n,10),-1}emitNumericEntity(t,n){var r;if(this.consumed<=n)return(r=this.errors)===null||r===void 0||r.absenceOfDigitsInNumericCharacterReference(this.consumed),0;if(t===fr.SEMI)this.consumed+=1;else if(this.decodeMode===Xi.Strict)return 0;return this.emitCodePoint(JB(this.result),this.consumed),this.errors&&(t!==fr.SEMI&&this.errors.missingSemicolonAfterCharacterReference(),this.errors.validateNumericCharacterReference(this.result)),this.consumed}stateNamedEntity(t,n){const{decodeTree:r}=this;let a=r[this.treeIndex],s=(a&Zs.VALUE_LENGTH)>>14;for(;n<t.length;n++,this.excess++){const o=t.charCodeAt(n);if(this.treeIndex=iU(r,a,this.treeIndex+Math.max(1,s),o),this.treeIndex<0)return this.result===0||this.decodeMode===Xi.Attribute&&(s===0||rU(o))?0:this.emitNotTerminatedNamedEntity();if(a=r[this.treeIndex],s=(a&Zs.VALUE_LENGTH)>>14,s!==0){if(o===fr.SEMI)return this.emitNamedEntityData(this.treeIndex,s,this.consumed+this.excess);this.decodeMode!==Xi.Strict&&(this.result=this.treeIndex,this.consumed+=this.excess,this.excess=0)}}return-1}emitNotTerminatedNamedEntity(){var t;const{result:n,decodeTree:r}=this,a=(r[n]&Zs.VALUE_LENGTH)>>14;return this.emitNamedEntityData(n,a,this.consumed),(t=this.errors)===null||t===void 0||t.missingSemicolonAfterCharacterReference(),this.consumed}emitNamedEntityData(t,n,r){const{decodeTree:a}=this;return this.emitCodePoint(n===1?a[t]&~Zs.VALUE_LENGTH:a[t+1],r),n===3&&this.emitCodePoint(a[t+2],r),r}end(){var t;switch(this.state){case cr.NamedEntity:return this.result!==0&&(this.decodeMode!==Xi.Attribute||this.result===this.treeIndex)?this.emitNotTerminatedNamedEntity():0;case cr.NumericDecimal:return this.emitNumericEntity(0,2);case cr.NumericHex:return this.emitNumericEntity(0,3);case cr.NumericStart:return(t=this.errors)===null||t===void 0||t.absenceOfDigitsInNumericCharacterReference(this.consumed),0;case cr.EntityStart:return 0}}}function iU(e,t,n,r){const a=(t&Zs.BRANCH_LENGTH)>>7,s=t&Zs.JUMP_TABLE;if(a===0)return s!==0&&r===s?n:-1;if(s){const d=r-s;return d<0||d>=a?-1:e[n+d]-1}let o=n,c=o+a-1;for(;o<=c;){const d=o+c>>>1,f=e[d];if(f<r)o=d+1;else if(f>r)c=d-1;else return e[d+a]}return-1}var Ae;(function(e){e.HTML="http://www.w3.org/1999/xhtml",e.MATHML="http://www.w3.org/1998/Math/MathML",e.SVG="http://www.w3.org/2000/svg",e.XLINK="http://www.w3.org/1999/xlink",e.XML="http://www.w3.org/XML/1998/namespace",e.XMLNS="http://www.w3.org/2000/xmlns/"})(Ae||(Ae={}));var $l;(function(e){e.TYPE="type",e.ACTION="action",e.ENCODING="encoding",e.PROMPT="prompt",e.NAME="name",e.COLOR="color",e.FACE="face",e.SIZE="size"})($l||($l={}));var Ma;(function(e){e.NO_QUIRKS="no-quirks",e.QUIRKS="quirks",e.LIMITED_QUIRKS="limited-quirks"})(Ma||(Ma={}));var ne;(function(e){e.A="a",e.ADDRESS="address",e.ANNOTATION_XML="annotation-xml",e.APPLET="applet",e.AREA="area",e.ARTICLE="article",e.ASIDE="aside",e.B="b",e.BASE="base",e.BASEFONT="basefont",e.BGSOUND="bgsound",e.BIG="big",e.BLOCKQUOTE="blockquote",e.BODY="body",e.BR="br",e.BUTTON="button",e.CAPTION="caption",e.CENTER="center",e.CODE="code",e.COL="col",e.COLGROUP="colgroup",e.DD="dd",e.DESC="desc",e.DETAILS="details",e.DIALOG="dialog",e.DIR="dir",e.DIV="div",e.DL="dl",e.DT="dt",e.EM="em",e.EMBED="embed",e.FIELDSET="fieldset",e.FIGCAPTION="figcaption",e.FIGURE="figure",e.FONT="font",e.FOOTER="footer",e.FOREIGN_OBJECT="foreignObject",e.FORM="form",e.FRAME="frame",e.FRAMESET="frameset",e.H1="h1",e.H2="h2",e.H3="h3",e.H4="h4",e.H5="h5",e.H6="h6",e.HEAD="head",e.HEADER="header",e.HGROUP="hgroup",e.HR="hr",e.HTML="html",e.I="i",e.IMG="img",e.IMAGE="image",e.INPUT="input",e.IFRAME="iframe",e.KEYGEN="keygen",e.LABEL="label",e.LI="li",e.LINK="link",e.LISTING="listing",e.MAIN="main",e.MALIGNMARK="malignmark",e.MARQUEE="marquee",e.MATH="math",e.MENU="menu",e.META="meta",e.MGLYPH="mglyph",e.MI="mi",e.MO="mo",e.MN="mn",e.MS="ms",e.MTEXT="mtext",e.NAV="nav",e.NOBR="nobr",e.NOFRAMES="noframes",e.NOEMBED="noembed",e.NOSCRIPT="noscript",e.OBJECT="object",e.OL="ol",e.OPTGROUP="optgroup",e.OPTION="option",e.P="p",e.PARAM="param",e.PLAINTEXT="plaintext",e.PRE="pre",e.RB="rb",e.RP="rp",e.RT="rt",e.RTC="rtc",e.RUBY="ruby",e.S="s",e.SCRIPT="script",e.SEARCH="search",e.SECTION="section",e.SELECT="select",e.SOURCE="source",e.SMALL="small",e.SPAN="span",e.STRIKE="strike",e.STRONG="strong",e.STYLE="style",e.SUB="sub",e.SUMMARY="summary",e.SUP="sup",e.TABLE="table",e.TBODY="tbody",e.TEMPLATE="template",e.TEXTAREA="textarea",e.TFOOT="tfoot",e.TD="td",e.TH="th",e.THEAD="thead",e.TITLE="title",e.TR="tr",e.TRACK="track",e.TT="tt",e.U="u",e.UL="ul",e.SVG="svg",e.VAR="var",e.WBR="wbr",e.XMP="xmp"})(ne||(ne={}));var _;(function(e){e[e.UNKNOWN=0]="UNKNOWN",e[e.A=1]="A",e[e.ADDRESS=2]="ADDRESS",e[e.ANNOTATION_XML=3]="ANNOTATION_XML",e[e.APPLET=4]="APPLET",e[e.AREA=5]="AREA",e[e.ARTICLE=6]="ARTICLE",e[e.ASIDE=7]="ASIDE",e[e.B=8]="B",e[e.BASE=9]="BASE",e[e.BASEFONT=10]="BASEFONT",e[e.BGSOUND=11]="BGSOUND",e[e.BIG=12]="BIG",e[e.BLOCKQUOTE=13]="BLOCKQUOTE",e[e.BODY=14]="BODY",e[e.BR=15]="BR",e[e.BUTTON=16]="BUTTON",e[e.CAPTION=17]="CAPTION",e[e.CENTER=18]="CENTER",e[e.CODE=19]="CODE",e[e.COL=20]="COL",e[e.COLGROUP=21]="COLGROUP",e[e.DD=22]="DD",e[e.DESC=23]="DESC",e[e.DETAILS=24]="DETAILS",e[e.DIALOG=25]="DIALOG",e[e.DIR=26]="DIR",e[e.DIV=27]="DIV",e[e.DL=28]="DL",e[e.DT=29]="DT",e[e.EM=30]="EM",e[e.EMBED=31]="EMBED",e[e.FIELDSET=32]="FIELDSET",e[e.FIGCAPTION=33]="FIGCAPTION",e[e.FIGURE=34]="FIGURE",e[e.FONT=35]="FONT",e[e.FOOTER=36]="FOOTER",e[e.FOREIGN_OBJECT=37]="FOREIGN_OBJECT",e[e.FORM=38]="FORM",e[e.FRAME=39]="FRAME",e[e.FRAMESET=40]="FRAMESET",e[e.H1=41]="H1",e[e.H2=42]="H2",e[e.H3=43]="H3",e[e.H4=44]="H4",e[e.H5=45]="H5",e[e.H6=46]="H6",e[e.HEAD=47]="HEAD",e[e.HEADER=48]="HEADER",e[e.HGROUP=49]="HGROUP",e[e.HR=50]="HR",e[e.HTML=51]="HTML",e[e.I=52]="I",e[e.IMG=53]="IMG",e[e.IMAGE=54]="IMAGE",e[e.INPUT=55]="INPUT",e[e.IFRAME=56]="IFRAME",e[e.KEYGEN=57]="KEYGEN",e[e.LABEL=58]="LABEL",e[e.LI=59]="LI",e[e.LINK=60]="LINK",e[e.LISTING=61]="LISTING",e[e.MAIN=62]="MAIN",e[e.MALIGNMARK=63]="MALIGNMARK",e[e.MARQUEE=64]="MARQUEE",e[e.MATH=65]="MATH",e[e.MENU=66]="MENU",e[e.META=67]="META",e[e.MGLYPH=68]="MGLYPH",e[e.MI=69]="MI",e[e.MO=70]="MO",e[e.MN=71]="MN",e[e.MS=72]="MS",e[e.MTEXT=73]="MTEXT",e[e.NAV=74]="NAV",e[e.NOBR=75]="NOBR",e[e.NOFRAMES=76]="NOFRAMES",e[e.NOEMBED=77]="NOEMBED",e[e.NOSCRIPT=78]="NOSCRIPT",e[e.OBJECT=79]="OBJECT",e[e.OL=80]="OL",e[e.OPTGROUP=81]="OPTGROUP",e[e.OPTION=82]="OPTION",e[e.P=83]="P",e[e.PARAM=84]="PARAM",e[e.PLAINTEXT=85]="PLAINTEXT",e[e.PRE=86]="PRE",e[e.RB=87]="RB",e[e.RP=88]="RP",e[e.RT=89]="RT",e[e.RTC=90]="RTC",e[e.RUBY=91]="RUBY",e[e.S=92]="S",e[e.SCRIPT=93]="SCRIPT",e[e.SEARCH=94]="SEARCH",e[e.SECTION=95]="SECTION",e[e.SELECT=96]="SELECT",e[e.SOURCE=97]="SOURCE",e[e.SMALL=98]="SMALL",e[e.SPAN=99]="SPAN",e[e.STRIKE=100]="STRIKE",e[e.STRONG=101]="STRONG",e[e.STYLE=102]="STYLE",e[e.SUB=103]="SUB",e[e.SUMMARY=104]="SUMMARY",e[e.SUP=105]="SUP",e[e.TABLE=106]="TABLE",e[e.TBODY=107]="TBODY",e[e.TEMPLATE=108]="TEMPLATE",e[e.TEXTAREA=109]="TEXTAREA",e[e.TFOOT=110]="TFOOT",e[e.TD=111]="TD",e[e.TH=112]="TH",e[e.THEAD=113]="THEAD",e[e.TITLE=114]="TITLE",e[e.TR=115]="TR",e[e.TRACK=116]="TRACK",e[e.TT=117]="TT",e[e.U=118]="U",e[e.UL=119]="UL",e[e.SVG=120]="SVG",e[e.VAR=121]="VAR",e[e.WBR=122]="WBR",e[e.XMP=123]="XMP"})(_||(_={}));const sU=new Map([[ne.A,_.A],[ne.ADDRESS,_.ADDRESS],[ne.ANNOTATION_XML,_.ANNOTATION_XML],[ne.APPLET,_.APPLET],[ne.AREA,_.AREA],[ne.ARTICLE,_.ARTICLE],[ne.ASIDE,_.ASIDE],[ne.B,_.B],[ne.BASE,_.BASE],[ne.BASEFONT,_.BASEFONT],[ne.BGSOUND,_.BGSOUND],[ne.BIG,_.BIG],[ne.BLOCKQUOTE,_.BLOCKQUOTE],[ne.BODY,_.BODY],[ne.BR,_.BR],[ne.BUTTON,_.BUTTON],[ne.CAPTION,_.CAPTION],[ne.CENTER,_.CENTER],[ne.CODE,_.CODE],[ne.COL,_.COL],[ne.COLGROUP,_.COLGROUP],[ne.DD,_.DD],[ne.DESC,_.DESC],[ne.DETAILS,_.DETAILS],[ne.DIALOG,_.DIALOG],[ne.DIR,_.DIR],[ne.DIV,_.DIV],[ne.DL,_.DL],[ne.DT,_.DT],[ne.EM,_.EM],[ne.EMBED,_.EMBED],[ne.FIELDSET,_.FIELDSET],[ne.FIGCAPTION,_.FIGCAPTION],[ne.FIGURE,_.FIGURE],[ne.FONT,_.FONT],[ne.FOOTER,_.FOOTER],[ne.FOREIGN_OBJECT,_.FOREIGN_OBJECT],[ne.FORM,_.FORM],[ne.FRAME,_.FRAME],[ne.FRAMESET,_.FRAMESET],[ne.H1,_.H1],[ne.H2,_.H2],[ne.H3,_.H3],[ne.H4,_.H4],[ne.H5,_.H5],[ne.H6,_.H6],[ne.HEAD,_.HEAD],[ne.HEADER,_.HEADER],[ne.HGROUP,_.HGROUP],[ne.HR,_.HR],[ne.HTML,_.HTML],[ne.I,_.I],[ne.IMG,_.IMG],[ne.IMAGE,_.IMAGE],[ne.INPUT,_.INPUT],[ne.IFRAME,_.IFRAME],[ne.KEYGEN,_.KEYGEN],[ne.LABEL,_.LABEL],[ne.LI,_.LI],[ne.LINK,_.LINK],[ne.LISTING,_.LISTING],[ne.MAIN,_.MAIN],[ne.MALIGNMARK,_.MALIGNMARK],[ne.MARQUEE,_.MARQUEE],[ne.MATH,_.MATH],[ne.MENU,_.MENU],[ne.META,_.META],[ne.MGLYPH,_.MGLYPH],[ne.MI,_.MI],[ne.MO,_.MO],[ne.MN,_.MN],[ne.MS,_.MS],[ne.MTEXT,_.MTEXT],[ne.NAV,_.NAV],[ne.NOBR,_.NOBR],[ne.NOFRAMES,_.NOFRAMES],[ne.NOEMBED,_.NOEMBED],[ne.NOSCRIPT,_.NOSCRIPT],[ne.OBJECT,_.OBJECT],[ne.OL,_.OL],[ne.OPTGROUP,_.OPTGROUP],[ne.OPTION,_.OPTION],[ne.P,_.P],[ne.PARAM,_.PARAM],[ne.PLAINTEXT,_.PLAINTEXT],[ne.PRE,_.PRE],[ne.RB,_.RB],[ne.RP,_.RP],[ne.RT,_.RT],[ne.RTC,_.RTC],[ne.RUBY,_.RUBY],[ne.S,_.S],[ne.SCRIPT,_.SCRIPT],[ne.SEARCH,_.SEARCH],[ne.SECTION,_.SECTION],[ne.SELECT,_.SELECT],[ne.SOURCE,_.SOURCE],[ne.SMALL,_.SMALL],[ne.SPAN,_.SPAN],[ne.STRIKE,_.STRIKE],[ne.STRONG,_.STRONG],[ne.STYLE,_.STYLE],[ne.SUB,_.SUB],[ne.SUMMARY,_.SUMMARY],[ne.SUP,_.SUP],[ne.TABLE,_.TABLE],[ne.TBODY,_.TBODY],[ne.TEMPLATE,_.TEMPLATE],[ne.TEXTAREA,_.TEXTAREA],[ne.TFOOT,_.TFOOT],[ne.TD,_.TD],[ne.TH,_.TH],[ne.THEAD,_.THEAD],[ne.TITLE,_.TITLE],[ne.TR,_.TR],[ne.TRACK,_.TRACK],[ne.TT,_.TT],[ne.U,_.U],[ne.UL,_.UL],[ne.SVG,_.SVG],[ne.VAR,_.VAR],[ne.WBR,_.WBR],[ne.XMP,_.XMP]]);function gu(e){var t;return(t=sU.get(e))!==null&&t!==void 0?t:_.UNKNOWN}const ke=_,lU={[Ae.HTML]:new Set([ke.ADDRESS,ke.APPLET,ke.AREA,ke.ARTICLE,ke.ASIDE,ke.BASE,ke.BASEFONT,ke.BGSOUND,ke.BLOCKQUOTE,ke.BODY,ke.BR,ke.BUTTON,ke.CAPTION,ke.CENTER,ke.COL,ke.COLGROUP,ke.DD,ke.DETAILS,ke.DIR,ke.DIV,ke.DL,ke.DT,ke.EMBED,ke.FIELDSET,ke.FIGCAPTION,ke.FIGURE,ke.FOOTER,ke.FORM,ke.FRAME,ke.FRAMESET,ke.H1,ke.H2,ke.H3,ke.H4,ke.H5,ke.H6,ke.HEAD,ke.HEADER,ke.HGROUP,ke.HR,ke.HTML,ke.IFRAME,ke.IMG,ke.INPUT,ke.LI,ke.LINK,ke.LISTING,ke.MAIN,ke.MARQUEE,ke.MENU,ke.META,ke.NAV,ke.NOEMBED,ke.NOFRAMES,ke.NOSCRIPT,ke.OBJECT,ke.OL,ke.P,ke.PARAM,ke.PLAINTEXT,ke.PRE,ke.SCRIPT,ke.SECTION,ke.SELECT,ke.SOURCE,ke.STYLE,ke.SUMMARY,ke.TABLE,ke.TBODY,ke.TD,ke.TEMPLATE,ke.TEXTAREA,ke.TFOOT,ke.TH,ke.THEAD,ke.TITLE,ke.TR,ke.TRACK,ke.UL,ke.WBR,ke.XMP]),[Ae.MATHML]:new Set([ke.MI,ke.MO,ke.MN,ke.MS,ke.MTEXT,ke.ANNOTATION_XML]),[Ae.SVG]:new Set([ke.TITLE,ke.FOREIGN_OBJECT,ke.DESC]),[Ae.XLINK]:new Set,[Ae.XML]:new Set,[Ae.XMLNS]:new Set},hb=new Set([ke.H1,ke.H2,ke.H3,ke.H4,ke.H5,ke.H6]);ne.STYLE,ne.SCRIPT,ne.XMP,ne.IFRAME,ne.NOEMBED,ne.NOFRAMES,ne.PLAINTEXT;var U;(function(e){e[e.DATA=0]="DATA",e[e.RCDATA=1]="RCDATA",e[e.RAWTEXT=2]="RAWTEXT",e[e.SCRIPT_DATA=3]="SCRIPT_DATA",e[e.PLAINTEXT=4]="PLAINTEXT",e[e.TAG_OPEN=5]="TAG_OPEN",e[e.END_TAG_OPEN=6]="END_TAG_OPEN",e[e.TAG_NAME=7]="TAG_NAME",e[e.RCDATA_LESS_THAN_SIGN=8]="RCDATA_LESS_THAN_SIGN",e[e.RCDATA_END_TAG_OPEN=9]="RCDATA_END_TAG_OPEN",e[e.RCDATA_END_TAG_NAME=10]="RCDATA_END_TAG_NAME",e[e.RAWTEXT_LESS_THAN_SIGN=11]="RAWTEXT_LESS_THAN_SIGN",e[e.RAWTEXT_END_TAG_OPEN=12]="RAWTEXT_END_TAG_OPEN",e[e.RAWTEXT_END_TAG_NAME=13]="RAWTEXT_END_TAG_NAME",e[e.SCRIPT_DATA_LESS_THAN_SIGN=14]="SCRIPT_DATA_LESS_THAN_SIGN",e[e.SCRIPT_DATA_END_TAG_OPEN=15]="SCRIPT_DATA_END_TAG_OPEN",e[e.SCRIPT_DATA_END_TAG_NAME=16]="SCRIPT_DATA_END_TAG_NAME",e[e.SCRIPT_DATA_ESCAPE_START=17]="SCRIPT_DATA_ESCAPE_START",e[e.SCRIPT_DATA_ESCAPE_START_DASH=18]="SCRIPT_DATA_ESCAPE_START_DASH",e[e.SCRIPT_DATA_ESCAPED=19]="SCRIPT_DATA_ESCAPED",e[e.SCRIPT_DATA_ESCAPED_DASH=20]="SCRIPT_DATA_ESCAPED_DASH",e[e.SCRIPT_DATA_ESCAPED_DASH_DASH=21]="SCRIPT_DATA_ESCAPED_DASH_DASH",e[e.SCRIPT_DATA_ESCAPED_LESS_THAN_SIGN=22]="SCRIPT_DATA_ESCAPED_LESS_THAN_SIGN",e[e.SCRIPT_DATA_ESCAPED_END_TAG_OPEN=23]="SCRIPT_DATA_ESCAPED_END_TAG_OPEN",e[e.SCRIPT_DATA_ESCAPED_END_TAG_NAME=24]="SCRIPT_DATA_ESCAPED_END_TAG_NAME",e[e.SCRIPT_DATA_DOUBLE_ESCAPE_START=25]="SCRIPT_DATA_DOUBLE_ESCAPE_START",e[e.SCRIPT_DATA_DOUBLE_ESCAPED=26]="SCRIPT_DATA_DOUBLE_ESCAPED",e[e.SCRIPT_DATA_DOUBLE_ESCAPED_DASH=27]="SCRIPT_DATA_DOUBLE_ESCAPED_DASH",e[e.SCRIPT_DATA_DOUBLE_ESCAPED_DASH_DASH=28]="SCRIPT_DATA_DOUBLE_ESCAPED_DASH_DASH",e[e.SCRIPT_DATA_DOUBLE_ESCAPED_LESS_THAN_SIGN=29]="SCRIPT_DATA_DOUBLE_ESCAPED_LESS_THAN_SIGN",e[e.SCRIPT_DATA_DOUBLE_ESCAPE_END=30]="SCRIPT_DATA_DOUBLE_ESCAPE_END",e[e.BEFORE_ATTRIBUTE_NAME=31]="BEFORE_ATTRIBUTE_NAME",e[e.ATTRIBUTE_NAME=32]="ATTRIBUTE_NAME",e[e.AFTER_ATTRIBUTE_NAME=33]="AFTER_ATTRIBUTE_NAME",e[e.BEFORE_ATTRIBUTE_VALUE=34]="BEFORE_ATTRIBUTE_VALUE",e[e.ATTRIBUTE_VALUE_DOUBLE_QUOTED=35]="ATTRIBUTE_VALUE_DOUBLE_QUOTED",e[e.ATTRIBUTE_VALUE_SINGLE_QUOTED=36]="ATTRIBUTE_VALUE_SINGLE_QUOTED",e[e.ATTRIBUTE_VALUE_UNQUOTED=37]="ATTRIBUTE_VALUE_UNQUOTED",e[e.AFTER_ATTRIBUTE_VALUE_QUOTED=38]="AFTER_ATTRIBUTE_VALUE_QUOTED",e[e.SELF_CLOSING_START_TAG=39]="SELF_CLOSING_START_TAG",e[e.BOGUS_COMMENT=40]="BOGUS_COMMENT",e[e.MARKUP_DECLARATION_OPEN=41]="MARKUP_DECLARATION_OPEN",e[e.COMMENT_START=42]="COMMENT_START",e[e.COMMENT_START_DASH=43]="COMMENT_START_DASH",e[e.COMMENT=44]="COMMENT",e[e.COMMENT_LESS_THAN_SIGN=45]="COMMENT_LESS_THAN_SIGN",e[e.COMMENT_LESS_THAN_SIGN_BANG=46]="COMMENT_LESS_THAN_SIGN_BANG",e[e.COMMENT_LESS_THAN_SIGN_BANG_DASH=47]="COMMENT_LESS_THAN_SIGN_BANG_DASH",e[e.COMMENT_LESS_THAN_SIGN_BANG_DASH_DASH=48]="COMMENT_LESS_THAN_SIGN_BANG_DASH_DASH",e[e.COMMENT_END_DASH=49]="COMMENT_END_DASH",e[e.COMMENT_END=50]="COMMENT_END",e[e.COMMENT_END_BANG=51]="COMMENT_END_BANG",e[e.DOCTYPE=52]="DOCTYPE",e[e.BEFORE_DOCTYPE_NAME=53]="BEFORE_DOCTYPE_NAME",e[e.DOCTYPE_NAME=54]="DOCTYPE_NAME",e[e.AFTER_DOCTYPE_NAME=55]="AFTER_DOCTYPE_NAME",e[e.AFTER_DOCTYPE_PUBLIC_KEYWORD=56]="AFTER_DOCTYPE_PUBLIC_KEYWORD",e[e.BEFORE_DOCTYPE_PUBLIC_IDENTIFIER=57]="BEFORE_DOCTYPE_PUBLIC_IDENTIFIER",e[e.DOCTYPE_PUBLIC_IDENTIFIER_DOUBLE_QUOTED=58]="DOCTYPE_PUBLIC_IDENTIFIER_DOUBLE_QUOTED",e[e.DOCTYPE_PUBLIC_IDENTIFIER_SINGLE_QUOTED=59]="DOCTYPE_PUBLIC_IDENTIFIER_SINGLE_QUOTED",e[e.AFTER_DOCTYPE_PUBLIC_IDENTIFIER=60]="AFTER_DOCTYPE_PUBLIC_IDENTIFIER",e[e.BETWEEN_DOCTYPE_PUBLIC_AND_SYSTEM_IDENTIFIERS=61]="BETWEEN_DOCTYPE_PUBLIC_AND_SYSTEM_IDENTIFIERS",e[e.AFTER_DOCTYPE_SYSTEM_KEYWORD=62]="AFTER_DOCTYPE_SYSTEM_KEYWORD",e[e.BEFORE_DOCTYPE_SYSTEM_IDENTIFIER=63]="BEFORE_DOCTYPE_SYSTEM_IDENTIFIER",e[e.DOCTYPE_SYSTEM_IDENTIFIER_DOUBLE_QUOTED=64]="DOCTYPE_SYSTEM_IDENTIFIER_DOUBLE_QUOTED",e[e.DOCTYPE_SYSTEM_IDENTIFIER_SINGLE_QUOTED=65]="DOCTYPE_SYSTEM_IDENTIFIER_SINGLE_QUOTED",e[e.AFTER_DOCTYPE_SYSTEM_IDENTIFIER=66]="AFTER_DOCTYPE_SYSTEM_IDENTIFIER",e[e.BOGUS_DOCTYPE=67]="BOGUS_DOCTYPE",e[e.CDATA_SECTION=68]="CDATA_SECTION",e[e.CDATA_SECTION_BRACKET=69]="CDATA_SECTION_BRACKET",e[e.CDATA_SECTION_END=70]="CDATA_SECTION_END",e[e.CHARACTER_REFERENCE=71]="CHARACTER_REFERENCE",e[e.AMBIGUOUS_AMPERSAND=72]="AMBIGUOUS_AMPERSAND"})(U||(U={}));const zn={DATA:U.DATA,RCDATA:U.RCDATA,RAWTEXT:U.RAWTEXT,SCRIPT_DATA:U.SCRIPT_DATA,PLAINTEXT:U.PLAINTEXT,CDATA_SECTION:U.CDATA_SECTION};function oU(e){return e>=P.DIGIT_0&&e<=P.DIGIT_9}function Hc(e){return e>=P.LATIN_CAPITAL_A&&e<=P.LATIN_CAPITAL_Z}function uU(e){return e>=P.LATIN_SMALL_A&&e<=P.LATIN_SMALL_Z}function Vs(e){return uU(e)||Hc(e)}function kE(e){return Vs(e)||oU(e)}function dh(e){return e+32}function bT(e){return e===P.SPACE||e===P.LINE_FEED||e===P.TABULATION||e===P.FORM_FEED}function CE(e){return bT(e)||e===P.SOLIDUS||e===P.GREATER_THAN_SIGN}function cU(e){return e===P.NULL?ve.nullCharacterReference:e>1114111?ve.characterReferenceOutsideUnicodeRange:fT(e)?ve.surrogateCharacterReference:pT(e)?ve.noncharacterCharacterReference:mT(e)||e===P.CARRIAGE_RETURN?ve.controlCharacterReference:null}class dU{constructor(t,n){this.options=t,this.handler=n,this.paused=!1,this.inLoop=!1,this.inForeignNode=!1,this.lastStartTagName="",this.active=!1,this.state=U.DATA,this.returnState=U.DATA,this.entityStartPos=0,this.consumedAfterSnapshot=-1,this.currentCharacterToken=null,this.currentToken=null,this.currentAttr={name:"",value:""},this.preprocessor=new XB(n),this.currentLocation=this.getCurrentLocation(-1),this.entityDecoder=new aU(QB,(r,a)=>{this.preprocessor.pos=this.entityStartPos+a-1,this._flushCodePointConsumedAsCharacterReference(r)},n.onParseError?{missingSemicolonAfterCharacterReference:()=>{this._err(ve.missingSemicolonAfterCharacterReference,1)},absenceOfDigitsInNumericCharacterReference:r=>{this._err(ve.absenceOfDigitsInNumericCharacterReference,this.entityStartPos-this.preprocessor.pos+r)},validateNumericCharacterReference:r=>{const a=cU(r);a&&this._err(a,1)}}:void 0)}_err(t,n=0){var r,a;(a=(r=this.handler).onParseError)===null||a===void 0||a.call(r,this.preprocessor.getError(t,n))}getCurrentLocation(t){return this.options.sourceCodeLocationInfo?{startLine:this.preprocessor.line,startCol:this.preprocessor.col-t,startOffset:this.preprocessor.offset-t,endLine:-1,endCol:-1,endOffset:-1}:null}_runParsingLoop(){if(!this.inLoop){for(this.inLoop=!0;this.active&&!this.paused;){this.consumedAfterSnapshot=0;const t=this._consume();this._ensureHibernation()||this._callState(t)}this.inLoop=!1}}pause(){this.paused=!0}resume(t){if(!this.paused)throw new Error("Parser was already resumed");this.paused=!1,!this.inLoop&&(this._runParsingLoop(),this.paused||t==null||t())}write(t,n,r){this.active=!0,this.preprocessor.write(t,n),this._runParsingLoop(),this.paused||r==null||r()}insertHtmlAtCurrentPos(t){this.active=!0,this.preprocessor.insertHtmlAtCurrentPos(t),this._runParsingLoop()}_ensureHibernation(){return this.preprocessor.endOfChunkHit?(this.preprocessor.retreat(this.consumedAfterSnapshot),this.consumedAfterSnapshot=0,this.active=!1,!0):!1}_consume(){return this.consumedAfterSnapshot++,this.preprocessor.advance()}_advanceBy(t){this.consumedAfterSnapshot+=t;for(let n=0;n<t;n++)this.preprocessor.advance()}_consumeSequenceIfMatch(t,n){return this.preprocessor.startsWith(t,n)?(this._advanceBy(t.length-1),!0):!1}_createStartTagToken(){this.currentToken={type:Ct.START_TAG,tagName:"",tagID:_.UNKNOWN,selfClosing:!1,ackSelfClosing:!1,attrs:[],location:this.getCurrentLocation(1)}}_createEndTagToken(){this.currentToken={type:Ct.END_TAG,tagName:"",tagID:_.UNKNOWN,selfClosing:!1,ackSelfClosing:!1,attrs:[],location:this.getCurrentLocation(2)}}_createCommentToken(t){this.currentToken={type:Ct.COMMENT,data:"",location:this.getCurrentLocation(t)}}_createDoctypeToken(t){this.currentToken={type:Ct.DOCTYPE,name:t,forceQuirks:!1,publicId:null,systemId:null,location:this.currentLocation}}_createCharacterToken(t,n){this.currentCharacterToken={type:t,chars:n,location:this.currentLocation}}_createAttr(t){this.currentAttr={name:t,value:""},this.currentLocation=this.getCurrentLocation(0)}_leaveAttrName(){var t,n;const r=this.currentToken;if(gT(r,this.currentAttr.name)===null){if(r.attrs.push(this.currentAttr),r.location&&this.currentLocation){const a=(t=(n=r.location).attrs)!==null&&t!==void 0?t:n.attrs=Object.create(null);a[this.currentAttr.name]=this.currentLocation,this._leaveAttrValue()}}else this._err(ve.duplicateAttribute)}_leaveAttrValue(){this.currentLocation&&(this.currentLocation.endLine=this.preprocessor.line,this.currentLocation.endCol=this.preprocessor.col,this.currentLocation.endOffset=this.preprocessor.offset)}prepareToken(t){this._emitCurrentCharacterToken(t.location),this.currentToken=null,t.location&&(t.location.endLine=this.preprocessor.line,t.location.endCol=this.preprocessor.col+1,t.location.endOffset=this.preprocessor.offset+1),this.currentLocation=this.getCurrentLocation(-1)}emitCurrentTagToken(){const t=this.currentToken;this.prepareToken(t),t.tagID=gu(t.tagName),t.type===Ct.START_TAG?(this.lastStartTagName=t.tagName,this.handler.onStartTag(t)):(t.attrs.length>0&&this._err(ve.endTagWithAttributes),t.selfClosing&&this._err(ve.endTagWithTrailingSolidus),this.handler.onEndTag(t)),this.preprocessor.dropParsedChunk()}emitCurrentComment(t){this.prepareToken(t),this.handler.onComment(t),this.preprocessor.dropParsedChunk()}emitCurrentDoctype(t){this.prepareToken(t),this.handler.onDoctype(t),this.preprocessor.dropParsedChunk()}_emitCurrentCharacterToken(t){if(this.currentCharacterToken){switch(t&&this.currentCharacterToken.location&&(this.currentCharacterToken.location.endLine=t.startLine,this.currentCharacterToken.location.endCol=t.startCol,this.currentCharacterToken.location.endOffset=t.startOffset),this.currentCharacterToken.type){case Ct.CHARACTER:{this.handler.onCharacter(this.currentCharacterToken);break}case Ct.NULL_CHARACTER:{this.handler.onNullCharacter(this.currentCharacterToken);break}case Ct.WHITESPACE_CHARACTER:{this.handler.onWhitespaceCharacter(this.currentCharacterToken);break}}this.currentCharacterToken=null}}_emitEOFToken(){const t=this.getCurrentLocation(0);t&&(t.endLine=t.startLine,t.endCol=t.startCol,t.endOffset=t.startOffset),this._emitCurrentCharacterToken(t),this.handler.onEof({type:Ct.EOF,location:t}),this.active=!1}_appendCharToCurrentCharacterToken(t,n){if(this.currentCharacterToken)if(this.currentCharacterToken.type===t){this.currentCharacterToken.chars+=n;return}else this.currentLocation=this.getCurrentLocation(0),this._emitCurrentCharacterToken(this.currentLocation),this.preprocessor.dropParsedChunk();this._createCharacterToken(t,n)}_emitCodePoint(t){const n=bT(t)?Ct.WHITESPACE_CHARACTER:t===P.NULL?Ct.NULL_CHARACTER:Ct.CHARACTER;this._appendCharToCurrentCharacterToken(n,String.fromCodePoint(t))}_emitChars(t){this._appendCharToCurrentCharacterToken(Ct.CHARACTER,t)}_startCharacterReference(){this.returnState=this.state,this.state=U.CHARACTER_REFERENCE,this.entityStartPos=this.preprocessor.pos,this.entityDecoder.startEntity(this._isCharacterReferenceInAttribute()?Xi.Attribute:Xi.Legacy)}_isCharacterReferenceInAttribute(){return this.returnState===U.ATTRIBUTE_VALUE_DOUBLE_QUOTED||this.returnState===U.ATTRIBUTE_VALUE_SINGLE_QUOTED||this.returnState===U.ATTRIBUTE_VALUE_UNQUOTED}_flushCodePointConsumedAsCharacterReference(t){this._isCharacterReferenceInAttribute()?this.currentAttr.value+=String.fromCodePoint(t):this._emitCodePoint(t)}_callState(t){switch(this.state){case U.DATA:{this._stateData(t);break}case U.RCDATA:{this._stateRcdata(t);break}case U.RAWTEXT:{this._stateRawtext(t);break}case U.SCRIPT_DATA:{this._stateScriptData(t);break}case U.PLAINTEXT:{this._statePlaintext(t);break}case U.TAG_OPEN:{this._stateTagOpen(t);break}case U.END_TAG_OPEN:{this._stateEndTagOpen(t);break}case U.TAG_NAME:{this._stateTagName(t);break}case U.RCDATA_LESS_THAN_SIGN:{this._stateRcdataLessThanSign(t);break}case U.RCDATA_END_TAG_OPEN:{this._stateRcdataEndTagOpen(t);break}case U.RCDATA_END_TAG_NAME:{this._stateRcdataEndTagName(t);break}case U.RAWTEXT_LESS_THAN_SIGN:{this._stateRawtextLessThanSign(t);break}case U.RAWTEXT_END_TAG_OPEN:{this._stateRawtextEndTagOpen(t);break}case U.RAWTEXT_END_TAG_NAME:{this._stateRawtextEndTagName(t);break}case U.SCRIPT_DATA_LESS_THAN_SIGN:{this._stateScriptDataLessThanSign(t);break}case U.SCRIPT_DATA_END_TAG_OPEN:{this._stateScriptDataEndTagOpen(t);break}case U.SCRIPT_DATA_END_TAG_NAME:{this._stateScriptDataEndTagName(t);break}case U.SCRIPT_DATA_ESCAPE_START:{this._stateScriptDataEscapeStart(t);break}case U.SCRIPT_DATA_ESCAPE_START_DASH:{this._stateScriptDataEscapeStartDash(t);break}case U.SCRIPT_DATA_ESCAPED:{this._stateScriptDataEscaped(t);break}case U.SCRIPT_DATA_ESCAPED_DASH:{this._stateScriptDataEscapedDash(t);break}case U.SCRIPT_DATA_ESCAPED_DASH_DASH:{this._stateScriptDataEscapedDashDash(t);break}case U.SCRIPT_DATA_ESCAPED_LESS_THAN_SIGN:{this._stateScriptDataEscapedLessThanSign(t);break}case U.SCRIPT_DATA_ESCAPED_END_TAG_OPEN:{this._stateScriptDataEscapedEndTagOpen(t);break}case U.SCRIPT_DATA_ESCAPED_END_TAG_NAME:{this._stateScriptDataEscapedEndTagName(t);break}case U.SCRIPT_DATA_DOUBLE_ESCAPE_START:{this._stateScriptDataDoubleEscapeStart(t);break}case U.SCRIPT_DATA_DOUBLE_ESCAPED:{this._stateScriptDataDoubleEscaped(t);break}case U.SCRIPT_DATA_DOUBLE_ESCAPED_DASH:{this._stateScriptDataDoubleEscapedDash(t);break}case U.SCRIPT_DATA_DOUBLE_ESCAPED_DASH_DASH:{this._stateScriptDataDoubleEscapedDashDash(t);break}case U.SCRIPT_DATA_DOUBLE_ESCAPED_LESS_THAN_SIGN:{this._stateScriptDataDoubleEscapedLessThanSign(t);break}case U.SCRIPT_DATA_DOUBLE_ESCAPE_END:{this._stateScriptDataDoubleEscapeEnd(t);break}case U.BEFORE_ATTRIBUTE_NAME:{this._stateBeforeAttributeName(t);break}case U.ATTRIBUTE_NAME:{this._stateAttributeName(t);break}case U.AFTER_ATTRIBUTE_NAME:{this._stateAfterAttributeName(t);break}case U.BEFORE_ATTRIBUTE_VALUE:{this._stateBeforeAttributeValue(t);break}case U.ATTRIBUTE_VALUE_DOUBLE_QUOTED:{this._stateAttributeValueDoubleQuoted(t);break}case U.ATTRIBUTE_VALUE_SINGLE_QUOTED:{this._stateAttributeValueSingleQuoted(t);break}case U.ATTRIBUTE_VALUE_UNQUOTED:{this._stateAttributeValueUnquoted(t);break}case U.AFTER_ATTRIBUTE_VALUE_QUOTED:{this._stateAfterAttributeValueQuoted(t);break}case U.SELF_CLOSING_START_TAG:{this._stateSelfClosingStartTag(t);break}case U.BOGUS_COMMENT:{this._stateBogusComment(t);break}case U.MARKUP_DECLARATION_OPEN:{this._stateMarkupDeclarationOpen(t);break}case U.COMMENT_START:{this._stateCommentStart(t);break}case U.COMMENT_START_DASH:{this._stateCommentStartDash(t);break}case U.COMMENT:{this._stateComment(t);break}case U.COMMENT_LESS_THAN_SIGN:{this._stateCommentLessThanSign(t);break}case U.COMMENT_LESS_THAN_SIGN_BANG:{this._stateCommentLessThanSignBang(t);break}case U.COMMENT_LESS_THAN_SIGN_BANG_DASH:{this._stateCommentLessThanSignBangDash(t);break}case U.COMMENT_LESS_THAN_SIGN_BANG_DASH_DASH:{this._stateCommentLessThanSignBangDashDash(t);break}case U.COMMENT_END_DASH:{this._stateCommentEndDash(t);break}case U.COMMENT_END:{this._stateCommentEnd(t);break}case U.COMMENT_END_BANG:{this._stateCommentEndBang(t);break}case U.DOCTYPE:{this._stateDoctype(t);break}case U.BEFORE_DOCTYPE_NAME:{this._stateBeforeDoctypeName(t);break}case U.DOCTYPE_NAME:{this._stateDoctypeName(t);break}case U.AFTER_DOCTYPE_NAME:{this._stateAfterDoctypeName(t);break}case U.AFTER_DOCTYPE_PUBLIC_KEYWORD:{this._stateAfterDoctypePublicKeyword(t);break}case U.BEFORE_DOCTYPE_PUBLIC_IDENTIFIER:{this._stateBeforeDoctypePublicIdentifier(t);break}case U.DOCTYPE_PUBLIC_IDENTIFIER_DOUBLE_QUOTED:{this._stateDoctypePublicIdentifierDoubleQuoted(t);break}case U.DOCTYPE_PUBLIC_IDENTIFIER_SINGLE_QUOTED:{this._stateDoctypePublicIdentifierSingleQuoted(t);break}case U.AFTER_DOCTYPE_PUBLIC_IDENTIFIER:{this._stateAfterDoctypePublicIdentifier(t);break}case U.BETWEEN_DOCTYPE_PUBLIC_AND_SYSTEM_IDENTIFIERS:{this._stateBetweenDoctypePublicAndSystemIdentifiers(t);break}case U.AFTER_DOCTYPE_SYSTEM_KEYWORD:{this._stateAfterDoctypeSystemKeyword(t);break}case U.BEFORE_DOCTYPE_SYSTEM_IDENTIFIER:{this._stateBeforeDoctypeSystemIdentifier(t);break}case U.DOCTYPE_SYSTEM_IDENTIFIER_DOUBLE_QUOTED:{this._stateDoctypeSystemIdentifierDoubleQuoted(t);break}case U.DOCTYPE_SYSTEM_IDENTIFIER_SINGLE_QUOTED:{this._stateDoctypeSystemIdentifierSingleQuoted(t);break}case U.AFTER_DOCTYPE_SYSTEM_IDENTIFIER:{this._stateAfterDoctypeSystemIdentifier(t);break}case U.BOGUS_DOCTYPE:{this._stateBogusDoctype(t);break}case U.CDATA_SECTION:{this._stateCdataSection(t);break}case U.CDATA_SECTION_BRACKET:{this._stateCdataSectionBracket(t);break}case U.CDATA_SECTION_END:{this._stateCdataSectionEnd(t);break}case U.CHARACTER_REFERENCE:{this._stateCharacterReference();break}case U.AMBIGUOUS_AMPERSAND:{this._stateAmbiguousAmpersand(t);break}default:throw new Error("Unknown state")}}_stateData(t){switch(t){case P.LESS_THAN_SIGN:{this.state=U.TAG_OPEN;break}case P.AMPERSAND:{this._startCharacterReference();break}case P.NULL:{this._err(ve.unexpectedNullCharacter),this._emitCodePoint(t);break}case P.EOF:{this._emitEOFToken();break}default:this._emitCodePoint(t)}}_stateRcdata(t){switch(t){case P.AMPERSAND:{this._startCharacterReference();break}case P.LESS_THAN_SIGN:{this.state=U.RCDATA_LESS_THAN_SIGN;break}case P.NULL:{this._err(ve.unexpectedNullCharacter),this._emitChars(Tn);break}case P.EOF:{this._emitEOFToken();break}default:this._emitCodePoint(t)}}_stateRawtext(t){switch(t){case P.LESS_THAN_SIGN:{this.state=U.RAWTEXT_LESS_THAN_SIGN;break}case P.NULL:{this._err(ve.unexpectedNullCharacter),this._emitChars(Tn);break}case P.EOF:{this._emitEOFToken();break}default:this._emitCodePoint(t)}}_stateScriptData(t){switch(t){case P.LESS_THAN_SIGN:{this.state=U.SCRIPT_DATA_LESS_THAN_SIGN;break}case P.NULL:{this._err(ve.unexpectedNullCharacter),this._emitChars(Tn);break}case P.EOF:{this._emitEOFToken();break}default:this._emitCodePoint(t)}}_statePlaintext(t){switch(t){case P.NULL:{this._err(ve.unexpectedNullCharacter),this._emitChars(Tn);break}case P.EOF:{this._emitEOFToken();break}default:this._emitCodePoint(t)}}_stateTagOpen(t){if(Vs(t))this._createStartTagToken(),this.state=U.TAG_NAME,this._stateTagName(t);else switch(t){case P.EXCLAMATION_MARK:{this.state=U.MARKUP_DECLARATION_OPEN;break}case P.SOLIDUS:{this.state=U.END_TAG_OPEN;break}case P.QUESTION_MARK:{this._err(ve.unexpectedQuestionMarkInsteadOfTagName),this._createCommentToken(1),this.state=U.BOGUS_COMMENT,this._stateBogusComment(t);break}case P.EOF:{this._err(ve.eofBeforeTagName),this._emitChars("<"),this._emitEOFToken();break}default:this._err(ve.invalidFirstCharacterOfTagName),this._emitChars("<"),this.state=U.DATA,this._stateData(t)}}_stateEndTagOpen(t){if(Vs(t))this._createEndTagToken(),this.state=U.TAG_NAME,this._stateTagName(t);else switch(t){case P.GREATER_THAN_SIGN:{this._err(ve.missingEndTagName),this.state=U.DATA;break}case P.EOF:{this._err(ve.eofBeforeTagName),this._emitChars("</"),this._emitEOFToken();break}default:this._err(ve.invalidFirstCharacterOfTagName),this._createCommentToken(2),this.state=U.BOGUS_COMMENT,this._stateBogusComment(t)}}_stateTagName(t){const n=this.currentToken;switch(t){case P.SPACE:case P.LINE_FEED:case P.TABULATION:case P.FORM_FEED:{this.state=U.BEFORE_ATTRIBUTE_NAME;break}case P.SOLIDUS:{this.state=U.SELF_CLOSING_START_TAG;break}case P.GREATER_THAN_SIGN:{this.state=U.DATA,this.emitCurrentTagToken();break}case P.NULL:{this._err(ve.unexpectedNullCharacter),n.tagName+=Tn;break}case P.EOF:{this._err(ve.eofInTag),this._emitEOFToken();break}default:n.tagName+=String.fromCodePoint(Hc(t)?dh(t):t)}}_stateRcdataLessThanSign(t){t===P.SOLIDUS?this.state=U.RCDATA_END_TAG_OPEN:(this._emitChars("<"),this.state=U.RCDATA,this._stateRcdata(t))}_stateRcdataEndTagOpen(t){Vs(t)?(this.state=U.RCDATA_END_TAG_NAME,this._stateRcdataEndTagName(t)):(this._emitChars("</"),this.state=U.RCDATA,this._stateRcdata(t))}handleSpecialEndTag(t){if(!this.preprocessor.startsWith(this.lastStartTagName,!1))return!this._ensureHibernation();this._createEndTagToken();const n=this.currentToken;switch(n.tagName=this.lastStartTagName,this.preprocessor.peek(this.lastStartTagName.length)){case P.SPACE:case P.LINE_FEED:case P.TABULATION:case P.FORM_FEED:return this._advanceBy(this.lastStartTagName.length),this.state=U.BEFORE_ATTRIBUTE_NAME,!1;case P.SOLIDUS:return this._advanceBy(this.lastStartTagName.length),this.state=U.SELF_CLOSING_START_TAG,!1;case P.GREATER_THAN_SIGN:return this._advanceBy(this.lastStartTagName.length),this.emitCurrentTagToken(),this.state=U.DATA,!1;default:return!this._ensureHibernation()}}_stateRcdataEndTagName(t){this.handleSpecialEndTag(t)&&(this._emitChars("</"),this.state=U.RCDATA,this._stateRcdata(t))}_stateRawtextLessThanSign(t){t===P.SOLIDUS?this.state=U.RAWTEXT_END_TAG_OPEN:(this._emitChars("<"),this.state=U.RAWTEXT,this._stateRawtext(t))}_stateRawtextEndTagOpen(t){Vs(t)?(this.state=U.RAWTEXT_END_TAG_NAME,this._stateRawtextEndTagName(t)):(this._emitChars("</"),this.state=U.RAWTEXT,this._stateRawtext(t))}_stateRawtextEndTagName(t){this.handleSpecialEndTag(t)&&(this._emitChars("</"),this.state=U.RAWTEXT,this._stateRawtext(t))}_stateScriptDataLessThanSign(t){switch(t){case P.SOLIDUS:{this.state=U.SCRIPT_DATA_END_TAG_OPEN;break}case P.EXCLAMATION_MARK:{this.state=U.SCRIPT_DATA_ESCAPE_START,this._emitChars("<!");break}default:this._emitChars("<"),this.state=U.SCRIPT_DATA,this._stateScriptData(t)}}_stateScriptDataEndTagOpen(t){Vs(t)?(this.state=U.SCRIPT_DATA_END_TAG_NAME,this._stateScriptDataEndTagName(t)):(this._emitChars("</"),this.state=U.SCRIPT_DATA,this._stateScriptData(t))}_stateScriptDataEndTagName(t){this.handleSpecialEndTag(t)&&(this._emitChars("</"),this.state=U.SCRIPT_DATA,this._stateScriptData(t))}_stateScriptDataEscapeStart(t){t===P.HYPHEN_MINUS?(this.state=U.SCRIPT_DATA_ESCAPE_START_DASH,this._emitChars("-")):(this.state=U.SCRIPT_DATA,this._stateScriptData(t))}_stateScriptDataEscapeStartDash(t){t===P.HYPHEN_MINUS?(this.state=U.SCRIPT_DATA_ESCAPED_DASH_DASH,this._emitChars("-")):(this.state=U.SCRIPT_DATA,this._stateScriptData(t))}_stateScriptDataEscaped(t){switch(t){case P.HYPHEN_MINUS:{this.state=U.SCRIPT_DATA_ESCAPED_DASH,this._emitChars("-");break}case P.LESS_THAN_SIGN:{this.state=U.SCRIPT_DATA_ESCAPED_LESS_THAN_SIGN;break}case P.NULL:{this._err(ve.unexpectedNullCharacter),this._emitChars(Tn);break}case P.EOF:{this._err(ve.eofInScriptHtmlCommentLikeText),this._emitEOFToken();break}default:this._emitCodePoint(t)}}_stateScriptDataEscapedDash(t){switch(t){case P.HYPHEN_MINUS:{this.state=U.SCRIPT_DATA_ESCAPED_DASH_DASH,this._emitChars("-");break}case P.LESS_THAN_SIGN:{this.state=U.SCRIPT_DATA_ESCAPED_LESS_THAN_SIGN;break}case P.NULL:{this._err(ve.unexpectedNullCharacter),this.state=U.SCRIPT_DATA_ESCAPED,this._emitChars(Tn);break}case P.EOF:{this._err(ve.eofInScriptHtmlCommentLikeText),this._emitEOFToken();break}default:this.state=U.SCRIPT_DATA_ESCAPED,this._emitCodePoint(t)}}_stateScriptDataEscapedDashDash(t){switch(t){case P.HYPHEN_MINUS:{this._emitChars("-");break}case P.LESS_THAN_SIGN:{this.state=U.SCRIPT_DATA_ESCAPED_LESS_THAN_SIGN;break}case P.GREATER_THAN_SIGN:{this.state=U.SCRIPT_DATA,this._emitChars(">");break}case P.NULL:{this._err(ve.unexpectedNullCharacter),this.state=U.SCRIPT_DATA_ESCAPED,this._emitChars(Tn);break}case P.EOF:{this._err(ve.eofInScriptHtmlCommentLikeText),this._emitEOFToken();break}default:this.state=U.SCRIPT_DATA_ESCAPED,this._emitCodePoint(t)}}_stateScriptDataEscapedLessThanSign(t){t===P.SOLIDUS?this.state=U.SCRIPT_DATA_ESCAPED_END_TAG_OPEN:Vs(t)?(this._emitChars("<"),this.state=U.SCRIPT_DATA_DOUBLE_ESCAPE_START,this._stateScriptDataDoubleEscapeStart(t)):(this._emitChars("<"),this.state=U.SCRIPT_DATA_ESCAPED,this._stateScriptDataEscaped(t))}_stateScriptDataEscapedEndTagOpen(t){Vs(t)?(this.state=U.SCRIPT_DATA_ESCAPED_END_TAG_NAME,this._stateScriptDataEscapedEndTagName(t)):(this._emitChars("</"),this.state=U.SCRIPT_DATA_ESCAPED,this._stateScriptDataEscaped(t))}_stateScriptDataEscapedEndTagName(t){this.handleSpecialEndTag(t)&&(this._emitChars("</"),this.state=U.SCRIPT_DATA_ESCAPED,this._stateScriptDataEscaped(t))}_stateScriptDataDoubleEscapeStart(t){if(this.preprocessor.startsWith(Qr.SCRIPT,!1)&&CE(this.preprocessor.peek(Qr.SCRIPT.length))){this._emitCodePoint(t);for(let n=0;n<Qr.SCRIPT.length;n++)this._emitCodePoint(this._consume());this.state=U.SCRIPT_DATA_DOUBLE_ESCAPED}else this._ensureHibernation()||(this.state=U.SCRIPT_DATA_ESCAPED,this._stateScriptDataEscaped(t))}_stateScriptDataDoubleEscaped(t){switch(t){case P.HYPHEN_MINUS:{this.state=U.SCRIPT_DATA_DOUBLE_ESCAPED_DASH,this._emitChars("-");break}case P.LESS_THAN_SIGN:{this.state=U.SCRIPT_DATA_DOUBLE_ESCAPED_LESS_THAN_SIGN,this._emitChars("<");break}case P.NULL:{this._err(ve.unexpectedNullCharacter),this._emitChars(Tn);break}case P.EOF:{this._err(ve.eofInScriptHtmlCommentLikeText),this._emitEOFToken();break}default:this._emitCodePoint(t)}}_stateScriptDataDoubleEscapedDash(t){switch(t){case P.HYPHEN_MINUS:{this.state=U.SCRIPT_DATA_DOUBLE_ESCAPED_DASH_DASH,this._emitChars("-");break}case P.LESS_THAN_SIGN:{this.state=U.SCRIPT_DATA_DOUBLE_ESCAPED_LESS_THAN_SIGN,this._emitChars("<");break}case P.NULL:{this._err(ve.unexpectedNullCharacter),this.state=U.SCRIPT_DATA_DOUBLE_ESCAPED,this._emitChars(Tn);break}case P.EOF:{this._err(ve.eofInScriptHtmlCommentLikeText),this._emitEOFToken();break}default:this.state=U.SCRIPT_DATA_DOUBLE_ESCAPED,this._emitCodePoint(t)}}_stateScriptDataDoubleEscapedDashDash(t){switch(t){case P.HYPHEN_MINUS:{this._emitChars("-");break}case P.LESS_THAN_SIGN:{this.state=U.SCRIPT_DATA_DOUBLE_ESCAPED_LESS_THAN_SIGN,this._emitChars("<");break}case P.GREATER_THAN_SIGN:{this.state=U.SCRIPT_DATA,this._emitChars(">");break}case P.NULL:{this._err(ve.unexpectedNullCharacter),this.state=U.SCRIPT_DATA_DOUBLE_ESCAPED,this._emitChars(Tn);break}case P.EOF:{this._err(ve.eofInScriptHtmlCommentLikeText),this._emitEOFToken();break}default:this.state=U.SCRIPT_DATA_DOUBLE_ESCAPED,this._emitCodePoint(t)}}_stateScriptDataDoubleEscapedLessThanSign(t){t===P.SOLIDUS?(this.state=U.SCRIPT_DATA_DOUBLE_ESCAPE_END,this._emitChars("/")):(this.state=U.SCRIPT_DATA_DOUBLE_ESCAPED,this._stateScriptDataDoubleEscaped(t))}_stateScriptDataDoubleEscapeEnd(t){if(this.preprocessor.startsWith(Qr.SCRIPT,!1)&&CE(this.preprocessor.peek(Qr.SCRIPT.length))){this._emitCodePoint(t);for(let n=0;n<Qr.SCRIPT.length;n++)this._emitCodePoint(this._consume());this.state=U.SCRIPT_DATA_ESCAPED}else this._ensureHibernation()||(this.state=U.SCRIPT_DATA_DOUBLE_ESCAPED,this._stateScriptDataDoubleEscaped(t))}_stateBeforeAttributeName(t){switch(t){case P.SPACE:case P.LINE_FEED:case P.TABULATION:case P.FORM_FEED:break;case P.SOLIDUS:case P.GREATER_THAN_SIGN:case P.EOF:{this.state=U.AFTER_ATTRIBUTE_NAME,this._stateAfterAttributeName(t);break}case P.EQUALS_SIGN:{this._err(ve.unexpectedEqualsSignBeforeAttributeName),this._createAttr("="),this.state=U.ATTRIBUTE_NAME;break}default:this._createAttr(""),this.state=U.ATTRIBUTE_NAME,this._stateAttributeName(t)}}_stateAttributeName(t){switch(t){case P.SPACE:case P.LINE_FEED:case P.TABULATION:case P.FORM_FEED:case P.SOLIDUS:case P.GREATER_THAN_SIGN:case P.EOF:{this._leaveAttrName(),this.state=U.AFTER_ATTRIBUTE_NAME,this._stateAfterAttributeName(t);break}case P.EQUALS_SIGN:{this._leaveAttrName(),this.state=U.BEFORE_ATTRIBUTE_VALUE;break}case P.QUOTATION_MARK:case P.APOSTROPHE:case P.LESS_THAN_SIGN:{this._err(ve.unexpectedCharacterInAttributeName),this.currentAttr.name+=String.fromCodePoint(t);break}case P.NULL:{this._err(ve.unexpectedNullCharacter),this.currentAttr.name+=Tn;break}default:this.currentAttr.name+=String.fromCodePoint(Hc(t)?dh(t):t)}}_stateAfterAttributeName(t){switch(t){case P.SPACE:case P.LINE_FEED:case P.TABULATION:case P.FORM_FEED:break;case P.SOLIDUS:{this.state=U.SELF_CLOSING_START_TAG;break}case P.EQUALS_SIGN:{this.state=U.BEFORE_ATTRIBUTE_VALUE;break}case P.GREATER_THAN_SIGN:{this.state=U.DATA,this.emitCurrentTagToken();break}case P.EOF:{this._err(ve.eofInTag),this._emitEOFToken();break}default:this._createAttr(""),this.state=U.ATTRIBUTE_NAME,this._stateAttributeName(t)}}_stateBeforeAttributeValue(t){switch(t){case P.SPACE:case P.LINE_FEED:case P.TABULATION:case P.FORM_FEED:break;case P.QUOTATION_MARK:{this.state=U.ATTRIBUTE_VALUE_DOUBLE_QUOTED;break}case P.APOSTROPHE:{this.state=U.ATTRIBUTE_VALUE_SINGLE_QUOTED;break}case P.GREATER_THAN_SIGN:{this._err(ve.missingAttributeValue),this.state=U.DATA,this.emitCurrentTagToken();break}default:this.state=U.ATTRIBUTE_VALUE_UNQUOTED,this._stateAttributeValueUnquoted(t)}}_stateAttributeValueDoubleQuoted(t){switch(t){case P.QUOTATION_MARK:{this.state=U.AFTER_ATTRIBUTE_VALUE_QUOTED;break}case P.AMPERSAND:{this._startCharacterReference();break}case P.NULL:{this._err(ve.unexpectedNullCharacter),this.currentAttr.value+=Tn;break}case P.EOF:{this._err(ve.eofInTag),this._emitEOFToken();break}default:this.currentAttr.value+=String.fromCodePoint(t)}}_stateAttributeValueSingleQuoted(t){switch(t){case P.APOSTROPHE:{this.state=U.AFTER_ATTRIBUTE_VALUE_QUOTED;break}case P.AMPERSAND:{this._startCharacterReference();break}case P.NULL:{this._err(ve.unexpectedNullCharacter),this.currentAttr.value+=Tn;break}case P.EOF:{this._err(ve.eofInTag),this._emitEOFToken();break}default:this.currentAttr.value+=String.fromCodePoint(t)}}_stateAttributeValueUnquoted(t){switch(t){case P.SPACE:case P.LINE_FEED:case P.TABULATION:case P.FORM_FEED:{this._leaveAttrValue(),this.state=U.BEFORE_ATTRIBUTE_NAME;break}case P.AMPERSAND:{this._startCharacterReference();break}case P.GREATER_THAN_SIGN:{this._leaveAttrValue(),this.state=U.DATA,this.emitCurrentTagToken();break}case P.NULL:{this._err(ve.unexpectedNullCharacter),this.currentAttr.value+=Tn;break}case P.QUOTATION_MARK:case P.APOSTROPHE:case P.LESS_THAN_SIGN:case P.EQUALS_SIGN:case P.GRAVE_ACCENT:{this._err(ve.unexpectedCharacterInUnquotedAttributeValue),this.currentAttr.value+=String.fromCodePoint(t);break}case P.EOF:{this._err(ve.eofInTag),this._emitEOFToken();break}default:this.currentAttr.value+=String.fromCodePoint(t)}}_stateAfterAttributeValueQuoted(t){switch(t){case P.SPACE:case P.LINE_FEED:case P.TABULATION:case P.FORM_FEED:{this._leaveAttrValue(),this.state=U.BEFORE_ATTRIBUTE_NAME;break}case P.SOLIDUS:{this._leaveAttrValue(),this.state=U.SELF_CLOSING_START_TAG;break}case P.GREATER_THAN_SIGN:{this._leaveAttrValue(),this.state=U.DATA,this.emitCurrentTagToken();break}case P.EOF:{this._err(ve.eofInTag),this._emitEOFToken();break}default:this._err(ve.missingWhitespaceBetweenAttributes),this.state=U.BEFORE_ATTRIBUTE_NAME,this._stateBeforeAttributeName(t)}}_stateSelfClosingStartTag(t){switch(t){case P.GREATER_THAN_SIGN:{const n=this.currentToken;n.selfClosing=!0,this.state=U.DATA,this.emitCurrentTagToken();break}case P.EOF:{this._err(ve.eofInTag),this._emitEOFToken();break}default:this._err(ve.unexpectedSolidusInTag),this.state=U.BEFORE_ATTRIBUTE_NAME,this._stateBeforeAttributeName(t)}}_stateBogusComment(t){const n=this.currentToken;switch(t){case P.GREATER_THAN_SIGN:{this.state=U.DATA,this.emitCurrentComment(n);break}case P.EOF:{this.emitCurrentComment(n),this._emitEOFToken();break}case P.NULL:{this._err(ve.unexpectedNullCharacter),n.data+=Tn;break}default:n.data+=String.fromCodePoint(t)}}_stateMarkupDeclarationOpen(t){this._consumeSequenceIfMatch(Qr.DASH_DASH,!0)?(this._createCommentToken(Qr.DASH_DASH.length+1),this.state=U.COMMENT_START):this._consumeSequenceIfMatch(Qr.DOCTYPE,!1)?(this.currentLocation=this.getCurrentLocation(Qr.DOCTYPE.length+1),this.state=U.DOCTYPE):this._consumeSequenceIfMatch(Qr.CDATA_START,!0)?this.inForeignNode?this.state=U.CDATA_SECTION:(this._err(ve.cdataInHtmlContent),this._createCommentToken(Qr.CDATA_START.length+1),this.currentToken.data="[CDATA[",this.state=U.BOGUS_COMMENT):this._ensureHibernation()||(this._err(ve.incorrectlyOpenedComment),this._createCommentToken(2),this.state=U.BOGUS_COMMENT,this._stateBogusComment(t))}_stateCommentStart(t){switch(t){case P.HYPHEN_MINUS:{this.state=U.COMMENT_START_DASH;break}case P.GREATER_THAN_SIGN:{this._err(ve.abruptClosingOfEmptyComment),this.state=U.DATA;const n=this.currentToken;this.emitCurrentComment(n);break}default:this.state=U.COMMENT,this._stateComment(t)}}_stateCommentStartDash(t){const n=this.currentToken;switch(t){case P.HYPHEN_MINUS:{this.state=U.COMMENT_END;break}case P.GREATER_THAN_SIGN:{this._err(ve.abruptClosingOfEmptyComment),this.state=U.DATA,this.emitCurrentComment(n);break}case P.EOF:{this._err(ve.eofInComment),this.emitCurrentComment(n),this._emitEOFToken();break}default:n.data+="-",this.state=U.COMMENT,this._stateComment(t)}}_stateComment(t){const n=this.currentToken;switch(t){case P.HYPHEN_MINUS:{this.state=U.COMMENT_END_DASH;break}case P.LESS_THAN_SIGN:{n.data+="<",this.state=U.COMMENT_LESS_THAN_SIGN;break}case P.NULL:{this._err(ve.unexpectedNullCharacter),n.data+=Tn;break}case P.EOF:{this._err(ve.eofInComment),this.emitCurrentComment(n),this._emitEOFToken();break}default:n.data+=String.fromCodePoint(t)}}_stateCommentLessThanSign(t){const n=this.currentToken;switch(t){case P.EXCLAMATION_MARK:{n.data+="!",this.state=U.COMMENT_LESS_THAN_SIGN_BANG;break}case P.LESS_THAN_SIGN:{n.data+="<";break}default:this.state=U.COMMENT,this._stateComment(t)}}_stateCommentLessThanSignBang(t){t===P.HYPHEN_MINUS?this.state=U.COMMENT_LESS_THAN_SIGN_BANG_DASH:(this.state=U.COMMENT,this._stateComment(t))}_stateCommentLessThanSignBangDash(t){t===P.HYPHEN_MINUS?this.state=U.COMMENT_LESS_THAN_SIGN_BANG_DASH_DASH:(this.state=U.COMMENT_END_DASH,this._stateCommentEndDash(t))}_stateCommentLessThanSignBangDashDash(t){t!==P.GREATER_THAN_SIGN&&t!==P.EOF&&this._err(ve.nestedComment),this.state=U.COMMENT_END,this._stateCommentEnd(t)}_stateCommentEndDash(t){const n=this.currentToken;switch(t){case P.HYPHEN_MINUS:{this.state=U.COMMENT_END;break}case P.EOF:{this._err(ve.eofInComment),this.emitCurrentComment(n),this._emitEOFToken();break}default:n.data+="-",this.state=U.COMMENT,this._stateComment(t)}}_stateCommentEnd(t){const n=this.currentToken;switch(t){case P.GREATER_THAN_SIGN:{this.state=U.DATA,this.emitCurrentComment(n);break}case P.EXCLAMATION_MARK:{this.state=U.COMMENT_END_BANG;break}case P.HYPHEN_MINUS:{n.data+="-";break}case P.EOF:{this._err(ve.eofInComment),this.emitCurrentComment(n),this._emitEOFToken();break}default:n.data+="--",this.state=U.COMMENT,this._stateComment(t)}}_stateCommentEndBang(t){const n=this.currentToken;switch(t){case P.HYPHEN_MINUS:{n.data+="--!",this.state=U.COMMENT_END_DASH;break}case P.GREATER_THAN_SIGN:{this._err(ve.incorrectlyClosedComment),this.state=U.DATA,this.emitCurrentComment(n);break}case P.EOF:{this._err(ve.eofInComment),this.emitCurrentComment(n),this._emitEOFToken();break}default:n.data+="--!",this.state=U.COMMENT,this._stateComment(t)}}_stateDoctype(t){switch(t){case P.SPACE:case P.LINE_FEED:case P.TABULATION:case P.FORM_FEED:{this.state=U.BEFORE_DOCTYPE_NAME;break}case P.GREATER_THAN_SIGN:{this.state=U.BEFORE_DOCTYPE_NAME,this._stateBeforeDoctypeName(t);break}case P.EOF:{this._err(ve.eofInDoctype),this._createDoctypeToken(null);const n=this.currentToken;n.forceQuirks=!0,this.emitCurrentDoctype(n),this._emitEOFToken();break}default:this._err(ve.missingWhitespaceBeforeDoctypeName),this.state=U.BEFORE_DOCTYPE_NAME,this._stateBeforeDoctypeName(t)}}_stateBeforeDoctypeName(t){if(Hc(t))this._createDoctypeToken(String.fromCharCode(dh(t))),this.state=U.DOCTYPE_NAME;else switch(t){case P.SPACE:case P.LINE_FEED:case P.TABULATION:case P.FORM_FEED:break;case P.NULL:{this._err(ve.unexpectedNullCharacter),this._createDoctypeToken(Tn),this.state=U.DOCTYPE_NAME;break}case P.GREATER_THAN_SIGN:{this._err(ve.missingDoctypeName),this._createDoctypeToken(null);const n=this.currentToken;n.forceQuirks=!0,this.emitCurrentDoctype(n),this.state=U.DATA;break}case P.EOF:{this._err(ve.eofInDoctype),this._createDoctypeToken(null);const n=this.currentToken;n.forceQuirks=!0,this.emitCurrentDoctype(n),this._emitEOFToken();break}default:this._createDoctypeToken(String.fromCodePoint(t)),this.state=U.DOCTYPE_NAME}}_stateDoctypeName(t){const n=this.currentToken;switch(t){case P.SPACE:case P.LINE_FEED:case P.TABULATION:case P.FORM_FEED:{this.state=U.AFTER_DOCTYPE_NAME;break}case P.GREATER_THAN_SIGN:{this.state=U.DATA,this.emitCurrentDoctype(n);break}case P.NULL:{this._err(ve.unexpectedNullCharacter),n.name+=Tn;break}case P.EOF:{this._err(ve.eofInDoctype),n.forceQuirks=!0,this.emitCurrentDoctype(n),this._emitEOFToken();break}default:n.name+=String.fromCodePoint(Hc(t)?dh(t):t)}}_stateAfterDoctypeName(t){const n=this.currentToken;switch(t){case P.SPACE:case P.LINE_FEED:case P.TABULATION:case P.FORM_FEED:break;case P.GREATER_THAN_SIGN:{this.state=U.DATA,this.emitCurrentDoctype(n);break}case P.EOF:{this._err(ve.eofInDoctype),n.forceQuirks=!0,this.emitCurrentDoctype(n),this._emitEOFToken();break}default:this._consumeSequenceIfMatch(Qr.PUBLIC,!1)?this.state=U.AFTER_DOCTYPE_PUBLIC_KEYWORD:this._consumeSequenceIfMatch(Qr.SYSTEM,!1)?this.state=U.AFTER_DOCTYPE_SYSTEM_KEYWORD:this._ensureHibernation()||(this._err(ve.invalidCharacterSequenceAfterDoctypeName),n.forceQuirks=!0,this.state=U.BOGUS_DOCTYPE,this._stateBogusDoctype(t))}}_stateAfterDoctypePublicKeyword(t){const n=this.currentToken;switch(t){case P.SPACE:case P.LINE_FEED:case P.TABULATION:case P.FORM_FEED:{this.state=U.BEFORE_DOCTYPE_PUBLIC_IDENTIFIER;break}case P.QUOTATION_MARK:{this._err(ve.missingWhitespaceAfterDoctypePublicKeyword),n.publicId="",this.state=U.DOCTYPE_PUBLIC_IDENTIFIER_DOUBLE_QUOTED;break}case P.APOSTROPHE:{this._err(ve.missingWhitespaceAfterDoctypePublicKeyword),n.publicId="",this.state=U.DOCTYPE_PUBLIC_IDENTIFIER_SINGLE_QUOTED;break}case P.GREATER_THAN_SIGN:{this._err(ve.missingDoctypePublicIdentifier),n.forceQuirks=!0,this.state=U.DATA,this.emitCurrentDoctype(n);break}case P.EOF:{this._err(ve.eofInDoctype),n.forceQuirks=!0,this.emitCurrentDoctype(n),this._emitEOFToken();break}default:this._err(ve.missingQuoteBeforeDoctypePublicIdentifier),n.forceQuirks=!0,this.state=U.BOGUS_DOCTYPE,this._stateBogusDoctype(t)}}_stateBeforeDoctypePublicIdentifier(t){const n=this.currentToken;switch(t){case P.SPACE:case P.LINE_FEED:case P.TABULATION:case P.FORM_FEED:break;case P.QUOTATION_MARK:{n.publicId="",this.state=U.DOCTYPE_PUBLIC_IDENTIFIER_DOUBLE_QUOTED;break}case P.APOSTROPHE:{n.publicId="",this.state=U.DOCTYPE_PUBLIC_IDENTIFIER_SINGLE_QUOTED;break}case P.GREATER_THAN_SIGN:{this._err(ve.missingDoctypePublicIdentifier),n.forceQuirks=!0,this.state=U.DATA,this.emitCurrentDoctype(n);break}case P.EOF:{this._err(ve.eofInDoctype),n.forceQuirks=!0,this.emitCurrentDoctype(n),this._emitEOFToken();break}default:this._err(ve.missingQuoteBeforeDoctypePublicIdentifier),n.forceQuirks=!0,this.state=U.BOGUS_DOCTYPE,this._stateBogusDoctype(t)}}_stateDoctypePublicIdentifierDoubleQuoted(t){const n=this.currentToken;switch(t){case P.QUOTATION_MARK:{this.state=U.AFTER_DOCTYPE_PUBLIC_IDENTIFIER;break}case P.NULL:{this._err(ve.unexpectedNullCharacter),n.publicId+=Tn;break}case P.GREATER_THAN_SIGN:{this._err(ve.abruptDoctypePublicIdentifier),n.forceQuirks=!0,this.emitCurrentDoctype(n),this.state=U.DATA;break}case P.EOF:{this._err(ve.eofInDoctype),n.forceQuirks=!0,this.emitCurrentDoctype(n),this._emitEOFToken();break}default:n.publicId+=String.fromCodePoint(t)}}_stateDoctypePublicIdentifierSingleQuoted(t){const n=this.currentToken;switch(t){case P.APOSTROPHE:{this.state=U.AFTER_DOCTYPE_PUBLIC_IDENTIFIER;break}case P.NULL:{this._err(ve.unexpectedNullCharacter),n.publicId+=Tn;break}case P.GREATER_THAN_SIGN:{this._err(ve.abruptDoctypePublicIdentifier),n.forceQuirks=!0,this.emitCurrentDoctype(n),this.state=U.DATA;break}case P.EOF:{this._err(ve.eofInDoctype),n.forceQuirks=!0,this.emitCurrentDoctype(n),this._emitEOFToken();break}default:n.publicId+=String.fromCodePoint(t)}}_stateAfterDoctypePublicIdentifier(t){const n=this.currentToken;switch(t){case P.SPACE:case P.LINE_FEED:case P.TABULATION:case P.FORM_FEED:{this.state=U.BETWEEN_DOCTYPE_PUBLIC_AND_SYSTEM_IDENTIFIERS;break}case P.GREATER_THAN_SIGN:{this.state=U.DATA,this.emitCurrentDoctype(n);break}case P.QUOTATION_MARK:{this._err(ve.missingWhitespaceBetweenDoctypePublicAndSystemIdentifiers),n.systemId="",this.state=U.DOCTYPE_SYSTEM_IDENTIFIER_DOUBLE_QUOTED;break}case P.APOSTROPHE:{this._err(ve.missingWhitespaceBetweenDoctypePublicAndSystemIdentifiers),n.systemId="",this.state=U.DOCTYPE_SYSTEM_IDENTIFIER_SINGLE_QUOTED;break}case P.EOF:{this._err(ve.eofInDoctype),n.forceQuirks=!0,this.emitCurrentDoctype(n),this._emitEOFToken();break}default:this._err(ve.missingQuoteBeforeDoctypeSystemIdentifier),n.forceQuirks=!0,this.state=U.BOGUS_DOCTYPE,this._stateBogusDoctype(t)}}_stateBetweenDoctypePublicAndSystemIdentifiers(t){const n=this.currentToken;switch(t){case P.SPACE:case P.LINE_FEED:case P.TABULATION:case P.FORM_FEED:break;case P.GREATER_THAN_SIGN:{this.emitCurrentDoctype(n),this.state=U.DATA;break}case P.QUOTATION_MARK:{n.systemId="",this.state=U.DOCTYPE_SYSTEM_IDENTIFIER_DOUBLE_QUOTED;break}case P.APOSTROPHE:{n.systemId="",this.state=U.DOCTYPE_SYSTEM_IDENTIFIER_SINGLE_QUOTED;break}case P.EOF:{this._err(ve.eofInDoctype),n.forceQuirks=!0,this.emitCurrentDoctype(n),this._emitEOFToken();break}default:this._err(ve.missingQuoteBeforeDoctypeSystemIdentifier),n.forceQuirks=!0,this.state=U.BOGUS_DOCTYPE,this._stateBogusDoctype(t)}}_stateAfterDoctypeSystemKeyword(t){const n=this.currentToken;switch(t){case P.SPACE:case P.LINE_FEED:case P.TABULATION:case P.FORM_FEED:{this.state=U.BEFORE_DOCTYPE_SYSTEM_IDENTIFIER;break}case P.QUOTATION_MARK:{this._err(ve.missingWhitespaceAfterDoctypeSystemKeyword),n.systemId="",this.state=U.DOCTYPE_SYSTEM_IDENTIFIER_DOUBLE_QUOTED;break}case P.APOSTROPHE:{this._err(ve.missingWhitespaceAfterDoctypeSystemKeyword),n.systemId="",this.state=U.DOCTYPE_SYSTEM_IDENTIFIER_SINGLE_QUOTED;break}case P.GREATER_THAN_SIGN:{this._err(ve.missingDoctypeSystemIdentifier),n.forceQuirks=!0,this.state=U.DATA,this.emitCurrentDoctype(n);break}case P.EOF:{this._err(ve.eofInDoctype),n.forceQuirks=!0,this.emitCurrentDoctype(n),this._emitEOFToken();break}default:this._err(ve.missingQuoteBeforeDoctypeSystemIdentifier),n.forceQuirks=!0,this.state=U.BOGUS_DOCTYPE,this._stateBogusDoctype(t)}}_stateBeforeDoctypeSystemIdentifier(t){const n=this.currentToken;switch(t){case P.SPACE:case P.LINE_FEED:case P.TABULATION:case P.FORM_FEED:break;case P.QUOTATION_MARK:{n.systemId="",this.state=U.DOCTYPE_SYSTEM_IDENTIFIER_DOUBLE_QUOTED;break}case P.APOSTROPHE:{n.systemId="",this.state=U.DOCTYPE_SYSTEM_IDENTIFIER_SINGLE_QUOTED;break}case P.GREATER_THAN_SIGN:{this._err(ve.missingDoctypeSystemIdentifier),n.forceQuirks=!0,this.state=U.DATA,this.emitCurrentDoctype(n);break}case P.EOF:{this._err(ve.eofInDoctype),n.forceQuirks=!0,this.emitCurrentDoctype(n),this._emitEOFToken();break}default:this._err(ve.missingQuoteBeforeDoctypeSystemIdentifier),n.forceQuirks=!0,this.state=U.BOGUS_DOCTYPE,this._stateBogusDoctype(t)}}_stateDoctypeSystemIdentifierDoubleQuoted(t){const n=this.currentToken;switch(t){case P.QUOTATION_MARK:{this.state=U.AFTER_DOCTYPE_SYSTEM_IDENTIFIER;break}case P.NULL:{this._err(ve.unexpectedNullCharacter),n.systemId+=Tn;break}case P.GREATER_THAN_SIGN:{this._err(ve.abruptDoctypeSystemIdentifier),n.forceQuirks=!0,this.emitCurrentDoctype(n),this.state=U.DATA;break}case P.EOF:{this._err(ve.eofInDoctype),n.forceQuirks=!0,this.emitCurrentDoctype(n),this._emitEOFToken();break}default:n.systemId+=String.fromCodePoint(t)}}_stateDoctypeSystemIdentifierSingleQuoted(t){const n=this.currentToken;switch(t){case P.APOSTROPHE:{this.state=U.AFTER_DOCTYPE_SYSTEM_IDENTIFIER;break}case P.NULL:{this._err(ve.unexpectedNullCharacter),n.systemId+=Tn;break}case P.GREATER_THAN_SIGN:{this._err(ve.abruptDoctypeSystemIdentifier),n.forceQuirks=!0,this.emitCurrentDoctype(n),this.state=U.DATA;break}case P.EOF:{this._err(ve.eofInDoctype),n.forceQuirks=!0,this.emitCurrentDoctype(n),this._emitEOFToken();break}default:n.systemId+=String.fromCodePoint(t)}}_stateAfterDoctypeSystemIdentifier(t){const n=this.currentToken;switch(t){case P.SPACE:case P.LINE_FEED:case P.TABULATION:case P.FORM_FEED:break;case P.GREATER_THAN_SIGN:{this.emitCurrentDoctype(n),this.state=U.DATA;break}case P.EOF:{this._err(ve.eofInDoctype),n.forceQuirks=!0,this.emitCurrentDoctype(n),this._emitEOFToken();break}default:this._err(ve.unexpectedCharacterAfterDoctypeSystemIdentifier),this.state=U.BOGUS_DOCTYPE,this._stateBogusDoctype(t)}}_stateBogusDoctype(t){const n=this.currentToken;switch(t){case P.GREATER_THAN_SIGN:{this.emitCurrentDoctype(n),this.state=U.DATA;break}case P.NULL:{this._err(ve.unexpectedNullCharacter);break}case P.EOF:{this.emitCurrentDoctype(n),this._emitEOFToken();break}}}_stateCdataSection(t){switch(t){case P.RIGHT_SQUARE_BRACKET:{this.state=U.CDATA_SECTION_BRACKET;break}case P.EOF:{this._err(ve.eofInCdata),this._emitEOFToken();break}default:this._emitCodePoint(t)}}_stateCdataSectionBracket(t){t===P.RIGHT_SQUARE_BRACKET?this.state=U.CDATA_SECTION_END:(this._emitChars("]"),this.state=U.CDATA_SECTION,this._stateCdataSection(t))}_stateCdataSectionEnd(t){switch(t){case P.GREATER_THAN_SIGN:{this.state=U.DATA;break}case P.RIGHT_SQUARE_BRACKET:{this._emitChars("]");break}default:this._emitChars("]]"),this.state=U.CDATA_SECTION,this._stateCdataSection(t)}}_stateCharacterReference(){let t=this.entityDecoder.write(this.preprocessor.html,this.preprocessor.pos);if(t<0)if(this.preprocessor.lastChunkWritten)t=this.entityDecoder.end();else{this.active=!1,this.preprocessor.pos=this.preprocessor.html.length-1,this.consumedAfterSnapshot=0,this.preprocessor.endOfChunkHit=!0;return}t===0?(this.preprocessor.pos=this.entityStartPos,this._flushCodePointConsumedAsCharacterReference(P.AMPERSAND),this.state=!this._isCharacterReferenceInAttribute()&&kE(this.preprocessor.peek(1))?U.AMBIGUOUS_AMPERSAND:this.returnState):this.state=this.returnState}_stateAmbiguousAmpersand(t){kE(t)?this._flushCodePointConsumedAsCharacterReference(t):(t===P.SEMICOLON&&this._err(ve.unknownNamedCharacterReference),this.state=this.returnState,this._callState(t))}}const vT=new Set([_.DD,_.DT,_.LI,_.OPTGROUP,_.OPTION,_.P,_.RB,_.RP,_.RT,_.RTC]),NE=new Set([...vT,_.CAPTION,_.COLGROUP,_.TBODY,_.TD,_.TFOOT,_.TH,_.THEAD,_.TR]),rf=new Set([_.APPLET,_.CAPTION,_.HTML,_.MARQUEE,_.OBJECT,_.TABLE,_.TD,_.TEMPLATE,_.TH]),hU=new Set([...rf,_.OL,_.UL]),fU=new Set([...rf,_.BUTTON]),OE=new Set([_.ANNOTATION_XML,_.MI,_.MN,_.MO,_.MS,_.MTEXT]),RE=new Set([_.DESC,_.FOREIGN_OBJECT,_.TITLE]),mU=new Set([_.TR,_.TEMPLATE,_.HTML]),pU=new Set([_.TBODY,_.TFOOT,_.THEAD,_.TEMPLATE,_.HTML]),gU=new Set([_.TABLE,_.TEMPLATE,_.HTML]),bU=new Set([_.TD,_.TH]);class vU{get currentTmplContentOrNode(){return this._isInTemplate()?this.treeAdapter.getTemplateContent(this.current):this.current}constructor(t,n,r){this.treeAdapter=n,this.handler=r,this.items=[],this.tagIDs=[],this.stackTop=-1,this.tmplCount=0,this.currentTagId=_.UNKNOWN,this.current=t}_indexOf(t){return this.items.lastIndexOf(t,this.stackTop)}_isInTemplate(){return this.currentTagId===_.TEMPLATE&&this.treeAdapter.getNamespaceURI(this.current)===Ae.HTML}_updateCurrentElement(){this.current=this.items[this.stackTop],this.currentTagId=this.tagIDs[this.stackTop]}push(t,n){this.stackTop++,this.items[this.stackTop]=t,this.current=t,this.tagIDs[this.stackTop]=n,this.currentTagId=n,this._isInTemplate()&&this.tmplCount++,this.handler.onItemPush(t,n,!0)}pop(){const t=this.current;this.tmplCount>0&&this._isInTemplate()&&this.tmplCount--,this.stackTop--,this._updateCurrentElement(),this.handler.onItemPop(t,!0)}replace(t,n){const r=this._indexOf(t);this.items[r]=n,r===this.stackTop&&(this.current=n)}insertAfter(t,n,r){const a=this._indexOf(t)+1;this.items.splice(a,0,n),this.tagIDs.splice(a,0,r),this.stackTop++,a===this.stackTop&&this._updateCurrentElement(),this.current&&this.currentTagId!==void 0&&this.handler.onItemPush(this.current,this.currentTagId,a===this.stackTop)}popUntilTagNamePopped(t){let n=this.stackTop+1;do n=this.tagIDs.lastIndexOf(t,n-1);while(n>0&&this.treeAdapter.getNamespaceURI(this.items[n])!==Ae.HTML);this.shortenToLength(Math.max(n,0))}shortenToLength(t){for(;this.stackTop>=t;){const n=this.current;this.tmplCount>0&&this._isInTemplate()&&(this.tmplCount-=1),this.stackTop--,this._updateCurrentElement(),this.handler.onItemPop(n,this.stackTop<t)}}popUntilElementPopped(t){const n=this._indexOf(t);this.shortenToLength(Math.max(n,0))}popUntilPopped(t,n){const r=this._indexOfTagNames(t,n);this.shortenToLength(Math.max(r,0))}popUntilNumberedHeaderPopped(){this.popUntilPopped(hb,Ae.HTML)}popUntilTableCellPopped(){this.popUntilPopped(bU,Ae.HTML)}popAllUpToHtmlElement(){this.tmplCount=0,this.shortenToLength(1)}_indexOfTagNames(t,n){for(let r=this.stackTop;r>=0;r--)if(t.has(this.tagIDs[r])&&this.treeAdapter.getNamespaceURI(this.items[r])===n)return r;return-1}clearBackTo(t,n){const r=this._indexOfTagNames(t,n);this.shortenToLength(r+1)}clearBackToTableContext(){this.clearBackTo(gU,Ae.HTML)}clearBackToTableBodyContext(){this.clearBackTo(pU,Ae.HTML)}clearBackToTableRowContext(){this.clearBackTo(mU,Ae.HTML)}remove(t){const n=this._indexOf(t);n>=0&&(n===this.stackTop?this.pop():(this.items.splice(n,1),this.tagIDs.splice(n,1),this.stackTop--,this._updateCurrentElement(),this.handler.onItemPop(t,!1)))}tryPeekProperlyNestedBodyElement(){return this.stackTop>=1&&this.tagIDs[1]===_.BODY?this.items[1]:null}contains(t){return this._indexOf(t)>-1}getCommonAncestor(t){const n=this._indexOf(t)-1;return n>=0?this.items[n]:null}isRootHtmlElementCurrent(){return this.stackTop===0&&this.tagIDs[0]===_.HTML}hasInDynamicScope(t,n){for(let r=this.stackTop;r>=0;r--){const a=this.tagIDs[r];switch(this.treeAdapter.getNamespaceURI(this.items[r])){case Ae.HTML:{if(a===t)return!0;if(n.has(a))return!1;break}case Ae.SVG:{if(RE.has(a))return!1;break}case Ae.MATHML:{if(OE.has(a))return!1;break}}}return!0}hasInScope(t){return this.hasInDynamicScope(t,rf)}hasInListItemScope(t){return this.hasInDynamicScope(t,hU)}hasInButtonScope(t){return this.hasInDynamicScope(t,fU)}hasNumberedHeaderInScope(){for(let t=this.stackTop;t>=0;t--){const n=this.tagIDs[t];switch(this.treeAdapter.getNamespaceURI(this.items[t])){case Ae.HTML:{if(hb.has(n))return!0;if(rf.has(n))return!1;break}case Ae.SVG:{if(RE.has(n))return!1;break}case Ae.MATHML:{if(OE.has(n))return!1;break}}}return!0}hasInTableScope(t){for(let n=this.stackTop;n>=0;n--)if(this.treeAdapter.getNamespaceURI(this.items[n])===Ae.HTML)switch(this.tagIDs[n]){case t:return!0;case _.TABLE:case _.HTML:return!1}return!0}hasTableBodyContextInTableScope(){for(let t=this.stackTop;t>=0;t--)if(this.treeAdapter.getNamespaceURI(this.items[t])===Ae.HTML)switch(this.tagIDs[t]){case _.TBODY:case _.THEAD:case _.TFOOT:return!0;case _.TABLE:case _.HTML:return!1}return!0}hasInSelectScope(t){for(let n=this.stackTop;n>=0;n--)if(this.treeAdapter.getNamespaceURI(this.items[n])===Ae.HTML)switch(this.tagIDs[n]){case t:return!0;case _.OPTION:case _.OPTGROUP:break;default:return!1}return!0}generateImpliedEndTags(){for(;this.currentTagId!==void 0&&vT.has(this.currentTagId);)this.pop()}generateImpliedEndTagsThoroughly(){for(;this.currentTagId!==void 0&&NE.has(this.currentTagId);)this.pop()}generateImpliedEndTagsWithExclusion(t){for(;this.currentTagId!==void 0&&this.currentTagId!==t&&NE.has(this.currentTagId);)this.pop()}}const Q1=3;var ui;(function(e){e[e.Marker=0]="Marker",e[e.Element=1]="Element"})(ui||(ui={}));const ME={type:ui.Marker};class yU{constructor(t){this.treeAdapter=t,this.entries=[],this.bookmark=null}_getNoahArkConditionCandidates(t,n){const r=[],a=n.length,s=this.treeAdapter.getTagName(t),o=this.treeAdapter.getNamespaceURI(t);for(let c=0;c<this.entries.length;c++){const d=this.entries[c];if(d.type===ui.Marker)break;const{element:f}=d;if(this.treeAdapter.getTagName(f)===s&&this.treeAdapter.getNamespaceURI(f)===o){const m=this.treeAdapter.getAttrList(f);m.length===a&&r.push({idx:c,attrs:m})}}return r}_ensureNoahArkCondition(t){if(this.entries.length<Q1)return;const n=this.treeAdapter.getAttrList(t),r=this._getNoahArkConditionCandidates(t,n);if(r.length<Q1)return;const a=new Map(n.map(o=>[o.name,o.value]));let s=0;for(let o=0;o<r.length;o++){const c=r[o];c.attrs.every(d=>a.get(d.name)===d.value)&&(s+=1,s>=Q1&&this.entries.splice(c.idx,1))}}insertMarker(){this.entries.unshift(ME)}pushElement(t,n){this._ensureNoahArkCondition(t),this.entries.unshift({type:ui.Element,element:t,token:n})}insertElementAfterBookmark(t,n){const r=this.entries.indexOf(this.bookmark);this.entries.splice(r,0,{type:ui.Element,element:t,token:n})}removeEntry(t){const n=this.entries.indexOf(t);n!==-1&&this.entries.splice(n,1)}clearToLastMarker(){const t=this.entries.indexOf(ME);t===-1?this.entries.length=0:this.entries.splice(0,t+1)}getElementEntryInScopeWithTagName(t){const n=this.entries.find(r=>r.type===ui.Marker||this.treeAdapter.getTagName(r.element)===t);return n&&n.type===ui.Element?n:null}getElementEntry(t){return this.entries.find(n=>n.type===ui.Element&&n.element===t)}}const Ks={createDocument(){return{nodeName:"#document",mode:Ma.NO_QUIRKS,childNodes:[]}},createDocumentFragment(){return{nodeName:"#document-fragment",childNodes:[]}},createElement(e,t,n){return{nodeName:e,tagName:e,attrs:n,namespaceURI:t,childNodes:[],parentNode:null}},createCommentNode(e){return{nodeName:"#comment",data:e,parentNode:null}},createTextNode(e){return{nodeName:"#text",value:e,parentNode:null}},appendChild(e,t){e.childNodes.push(t),t.parentNode=e},insertBefore(e,t,n){const r=e.childNodes.indexOf(n);e.childNodes.splice(r,0,t),t.parentNode=e},setTemplateContent(e,t){e.content=t},getTemplateContent(e){return e.content},setDocumentType(e,t,n,r){const a=e.childNodes.find(s=>s.nodeName==="#documentType");if(a)a.name=t,a.publicId=n,a.systemId=r;else{const s={nodeName:"#documentType",name:t,publicId:n,systemId:r,parentNode:null};Ks.appendChild(e,s)}},setDocumentMode(e,t){e.mode=t},getDocumentMode(e){return e.mode},detachNode(e){if(e.parentNode){const t=e.parentNode.childNodes.indexOf(e);e.parentNode.childNodes.splice(t,1),e.parentNode=null}},insertText(e,t){if(e.childNodes.length>0){const n=e.childNodes[e.childNodes.length-1];if(Ks.isTextNode(n)){n.value+=t;return}}Ks.appendChild(e,Ks.createTextNode(t))},insertTextBefore(e,t,n){const r=e.childNodes[e.childNodes.indexOf(n)-1];r&&Ks.isTextNode(r)?r.value+=t:Ks.insertBefore(e,Ks.createTextNode(t),n)},adoptAttributes(e,t){const n=new Set(e.attrs.map(r=>r.name));for(let r=0;r<t.length;r++)n.has(t[r].name)||e.attrs.push(t[r])},getFirstChild(e){return e.childNodes[0]},getChildNodes(e){return e.childNodes},getParentNode(e){return e.parentNode},getAttrList(e){return e.attrs},getTagName(e){return e.tagName},getNamespaceURI(e){return e.namespaceURI},getTextNodeContent(e){return e.value},getCommentNodeContent(e){return e.data},getDocumentTypeNodeName(e){return e.name},getDocumentTypeNodePublicId(e){return e.publicId},getDocumentTypeNodeSystemId(e){return e.systemId},isTextNode(e){return e.nodeName==="#text"},isCommentNode(e){return e.nodeName==="#comment"},isDocumentTypeNode(e){return e.nodeName==="#documentType"},isElementNode(e){return Object.prototype.hasOwnProperty.call(e,"tagName")},setNodeSourceCodeLocation(e,t){e.sourceCodeLocation=t},getNodeSourceCodeLocation(e){return e.sourceCodeLocation},updateNodeSourceCodeLocation(e,t){e.sourceCodeLocation={...e.sourceCodeLocation,...t}}},yT="html",_U="about:legacy-compat",EU="http://www.ibm.com/data/dtd/v11/ibmxhtml1-transitional.dtd",_T=["+//silmaril//dtd html pro v0r11 19970101//","-//as//dtd html 3.0 aswedit + extensions//","-//advasoft ltd//dtd html 3.0 aswedit + extensions//","-//ietf//dtd html 2.0 level 1//","-//ietf//dtd html 2.0 level 2//","-//ietf//dtd html 2.0 strict level 1//","-//ietf//dtd html 2.0 strict level 2//","-//ietf//dtd html 2.0 strict//","-//ietf//dtd html 2.0//","-//ietf//dtd html 2.1e//","-//ietf//dtd html 3.0//","-//ietf//dtd html 3.2 final//","-//ietf//dtd html 3.2//","-//ietf//dtd html 3//","-//ietf//dtd html level 0//","-//ietf//dtd html level 1//","-//ietf//dtd html level 2//","-//ietf//dtd html level 3//","-//ietf//dtd html strict level 0//","-//ietf//dtd html strict level 1//","-//ietf//dtd html strict level 2//","-//ietf//dtd html strict level 3//","-//ietf//dtd html strict//","-//ietf//dtd html//","-//metrius//dtd metrius presentational//","-//microsoft//dtd internet explorer 2.0 html strict//","-//microsoft//dtd internet explorer 2.0 html//","-//microsoft//dtd internet explorer 2.0 tables//","-//microsoft//dtd internet explorer 3.0 html strict//","-//microsoft//dtd internet explorer 3.0 html//","-//microsoft//dtd internet explorer 3.0 tables//","-//netscape comm. corp.//dtd html//","-//netscape comm. corp.//dtd strict html//","-//o'reilly and associates//dtd html 2.0//","-//o'reilly and associates//dtd html extended 1.0//","-//o'reilly and associates//dtd html extended relaxed 1.0//","-//sq//dtd html 2.0 hotmetal + extensions//","-//softquad software//dtd hotmetal pro 6.0::19990601::extensions to html 4.0//","-//softquad//dtd hotmetal pro 4.0::19971010::extensions to html 4.0//","-//spyglass//dtd html 2.0 extended//","-//sun microsystems corp.//dtd hotjava html//","-//sun microsystems corp.//dtd hotjava strict html//","-//w3c//dtd html 3 1995-03-24//","-//w3c//dtd html 3.2 draft//","-//w3c//dtd html 3.2 final//","-//w3c//dtd html 3.2//","-//w3c//dtd html 3.2s draft//","-//w3c//dtd html 4.0 frameset//","-//w3c//dtd html 4.0 transitional//","-//w3c//dtd html experimental 19960712//","-//w3c//dtd html experimental 970421//","-//w3c//dtd w3 html//","-//w3o//dtd w3 html 3.0//","-//webtechs//dtd mozilla html 2.0//","-//webtechs//dtd mozilla html//"],TU=[..._T,"-//w3c//dtd html 4.01 frameset//","-//w3c//dtd html 4.01 transitional//"],xU=new Set(["-//w3o//dtd w3 html strict 3.0//en//","-/w3c/dtd html 4.0 transitional/en","html"]),ET=["-//w3c//dtd xhtml 1.0 frameset//","-//w3c//dtd xhtml 1.0 transitional//"],SU=[...ET,"-//w3c//dtd html 4.01 frameset//","-//w3c//dtd html 4.01 transitional//"];function IE(e,t){return t.some(n=>e.startsWith(n))}function wU(e){return e.name===yT&&e.publicId===null&&(e.systemId===null||e.systemId===_U)}function AU(e){if(e.name!==yT)return Ma.QUIRKS;const{systemId:t}=e;if(t&&t.toLowerCase()===EU)return Ma.QUIRKS;let{publicId:n}=e;if(n!==null){if(n=n.toLowerCase(),xU.has(n))return Ma.QUIRKS;let r=t===null?TU:_T;if(IE(n,r))return Ma.QUIRKS;if(r=t===null?ET:SU,IE(n,r))return Ma.LIMITED_QUIRKS}return Ma.NO_QUIRKS}const DE={TEXT_HTML:"text/html",APPLICATION_XML:"application/xhtml+xml"},kU="definitionurl",CU="definitionURL",NU=new Map(["attributeName","attributeType","baseFrequency","baseProfile","calcMode","clipPathUnits","diffuseConstant","edgeMode","filterUnits","glyphRef","gradientTransform","gradientUnits","kernelMatrix","kernelUnitLength","keyPoints","keySplines","keyTimes","lengthAdjust","limitingConeAngle","markerHeight","markerUnits","markerWidth","maskContentUnits","maskUnits","numOctaves","pathLength","patternContentUnits","patternTransform","patternUnits","pointsAtX","pointsAtY","pointsAtZ","preserveAlpha","preserveAspectRatio","primitiveUnits","refX","refY","repeatCount","repeatDur","requiredExtensions","requiredFeatures","specularConstant","specularExponent","spreadMethod","startOffset","stdDeviation","stitchTiles","surfaceScale","systemLanguage","tableValues","targetX","targetY","textLength","viewBox","viewTarget","xChannelSelector","yChannelSelector","zoomAndPan"].map(e=>[e.toLowerCase(),e])),OU=new Map([["xlink:actuate",{prefix:"xlink",name:"actuate",namespace:Ae.XLINK}],["xlink:arcrole",{prefix:"xlink",name:"arcrole",namespace:Ae.XLINK}],["xlink:href",{prefix:"xlink",name:"href",namespace:Ae.XLINK}],["xlink:role",{prefix:"xlink",name:"role",namespace:Ae.XLINK}],["xlink:show",{prefix:"xlink",name:"show",namespace:Ae.XLINK}],["xlink:title",{prefix:"xlink",name:"title",namespace:Ae.XLINK}],["xlink:type",{prefix:"xlink",name:"type",namespace:Ae.XLINK}],["xml:lang",{prefix:"xml",name:"lang",namespace:Ae.XML}],["xml:space",{prefix:"xml",name:"space",namespace:Ae.XML}],["xmlns",{prefix:"",name:"xmlns",namespace:Ae.XMLNS}],["xmlns:xlink",{prefix:"xmlns",name:"xlink",namespace:Ae.XMLNS}]]),RU=new Map(["altGlyph","altGlyphDef","altGlyphItem","animateColor","animateMotion","animateTransform","clipPath","feBlend","feColorMatrix","feComponentTransfer","feComposite","feConvolveMatrix","feDiffuseLighting","feDisplacementMap","feDistantLight","feFlood","feFuncA","feFuncB","feFuncG","feFuncR","feGaussianBlur","feImage","feMerge","feMergeNode","feMorphology","feOffset","fePointLight","feSpecularLighting","feSpotLight","feTile","feTurbulence","foreignObject","glyphRef","linearGradient","radialGradient","textPath"].map(e=>[e.toLowerCase(),e])),MU=new Set([_.B,_.BIG,_.BLOCKQUOTE,_.BODY,_.BR,_.CENTER,_.CODE,_.DD,_.DIV,_.DL,_.DT,_.EM,_.EMBED,_.H1,_.H2,_.H3,_.H4,_.H5,_.H6,_.HEAD,_.HR,_.I,_.IMG,_.LI,_.LISTING,_.MENU,_.META,_.NOBR,_.OL,_.P,_.PRE,_.RUBY,_.S,_.SMALL,_.SPAN,_.STRONG,_.STRIKE,_.SUB,_.SUP,_.TABLE,_.TT,_.U,_.UL,_.VAR]);function IU(e){const t=e.tagID;return t===_.FONT&&e.attrs.some(({name:r})=>r===$l.COLOR||r===$l.SIZE||r===$l.FACE)||MU.has(t)}function TT(e){for(let t=0;t<e.attrs.length;t++)if(e.attrs[t].name===kU){e.attrs[t].name=CU;break}}function xT(e){for(let t=0;t<e.attrs.length;t++){const n=NU.get(e.attrs[t].name);n!=null&&(e.attrs[t].name=n)}}function R2(e){for(let t=0;t<e.attrs.length;t++){const n=OU.get(e.attrs[t].name);n&&(e.attrs[t].prefix=n.prefix,e.attrs[t].name=n.name,e.attrs[t].namespace=n.namespace)}}function DU(e){const t=RU.get(e.tagName);t!=null&&(e.tagName=t,e.tagID=gu(e.tagName))}function LU(e,t){return t===Ae.MATHML&&(e===_.MI||e===_.MO||e===_.MN||e===_.MS||e===_.MTEXT)}function PU(e,t,n){if(t===Ae.MATHML&&e===_.ANNOTATION_XML){for(let r=0;r<n.length;r++)if(n[r].name===$l.ENCODING){const a=n[r].value.toLowerCase();return a===DE.TEXT_HTML||a===DE.APPLICATION_XML}}return t===Ae.SVG&&(e===_.FOREIGN_OBJECT||e===_.DESC||e===_.TITLE)}function BU(e,t,n,r){return(!r||r===Ae.HTML)&&PU(e,t,n)||(!r||r===Ae.MATHML)&&LU(e,t)}const UU="hidden",FU=8,zU=3;var j;(function(e){e[e.INITIAL=0]="INITIAL",e[e.BEFORE_HTML=1]="BEFORE_HTML",e[e.BEFORE_HEAD=2]="BEFORE_HEAD",e[e.IN_HEAD=3]="IN_HEAD",e[e.IN_HEAD_NO_SCRIPT=4]="IN_HEAD_NO_SCRIPT",e[e.AFTER_HEAD=5]="AFTER_HEAD",e[e.IN_BODY=6]="IN_BODY",e[e.TEXT=7]="TEXT",e[e.IN_TABLE=8]="IN_TABLE",e[e.IN_TABLE_TEXT=9]="IN_TABLE_TEXT",e[e.IN_CAPTION=10]="IN_CAPTION",e[e.IN_COLUMN_GROUP=11]="IN_COLUMN_GROUP",e[e.IN_TABLE_BODY=12]="IN_TABLE_BODY",e[e.IN_ROW=13]="IN_ROW",e[e.IN_CELL=14]="IN_CELL",e[e.IN_SELECT=15]="IN_SELECT",e[e.IN_SELECT_IN_TABLE=16]="IN_SELECT_IN_TABLE",e[e.IN_TEMPLATE=17]="IN_TEMPLATE",e[e.AFTER_BODY=18]="AFTER_BODY",e[e.IN_FRAMESET=19]="IN_FRAMESET",e[e.AFTER_FRAMESET=20]="AFTER_FRAMESET",e[e.AFTER_AFTER_BODY=21]="AFTER_AFTER_BODY",e[e.AFTER_AFTER_FRAMESET=22]="AFTER_AFTER_FRAMESET"})(j||(j={}));const HU={startLine:-1,startCol:-1,startOffset:-1,endLine:-1,endCol:-1,endOffset:-1},ST=new Set([_.TABLE,_.TBODY,_.TFOOT,_.THEAD,_.TR]),LE={scriptingEnabled:!0,sourceCodeLocationInfo:!1,treeAdapter:Ks,onParseError:null};let PE=class{constructor(t,n,r=null,a=null){this.fragmentContext=r,this.scriptHandler=a,this.currentToken=null,this.stopped=!1,this.insertionMode=j.INITIAL,this.originalInsertionMode=j.INITIAL,this.headElement=null,this.formElement=null,this.currentNotInHTML=!1,this.tmplInsertionModeStack=[],this.pendingCharacterTokens=[],this.hasNonWhitespacePendingCharacterToken=!1,this.framesetOk=!0,this.skipNextNewLine=!1,this.fosterParentingEnabled=!1,this.options={...LE,...t},this.treeAdapter=this.options.treeAdapter,this.onParseError=this.options.onParseError,this.onParseError&&(this.options.sourceCodeLocationInfo=!0),this.document=n??this.treeAdapter.createDocument(),this.tokenizer=new dU(this.options,this),this.activeFormattingElements=new yU(this.treeAdapter),this.fragmentContextID=r?gu(this.treeAdapter.getTagName(r)):_.UNKNOWN,this._setContextModes(r??this.document,this.fragmentContextID),this.openElements=new vU(this.document,this.treeAdapter,this)}static parse(t,n){const r=new this(n);return r.tokenizer.write(t,!0),r.document}static getFragmentParser(t,n){const r={...LE,...n};t??(t=r.treeAdapter.createElement(ne.TEMPLATE,Ae.HTML,[]));const a=r.treeAdapter.createElement("documentmock",Ae.HTML,[]),s=new this(r,a,t);return s.fragmentContextID===_.TEMPLATE&&s.tmplInsertionModeStack.unshift(j.IN_TEMPLATE),s._initTokenizerForFragmentParsing(),s._insertFakeRootElement(),s._resetInsertionMode(),s._findFormInFragmentContext(),s}getFragment(){const t=this.treeAdapter.getFirstChild(this.document),n=this.treeAdapter.createDocumentFragment();return this._adoptNodes(t,n),n}_err(t,n,r){var a;if(!this.onParseError)return;const s=(a=t.location)!==null&&a!==void 0?a:HU,o={code:n,startLine:s.startLine,startCol:s.startCol,startOffset:s.startOffset,endLine:r?s.startLine:s.endLine,endCol:r?s.startCol:s.endCol,endOffset:r?s.startOffset:s.endOffset};this.onParseError(o)}onItemPush(t,n,r){var a,s;(s=(a=this.treeAdapter).onItemPush)===null||s===void 0||s.call(a,t),r&&this.openElements.stackTop>0&&this._setContextModes(t,n)}onItemPop(t,n){var r,a;if(this.options.sourceCodeLocationInfo&&this._setEndLocation(t,this.currentToken),(a=(r=this.treeAdapter).onItemPop)===null||a===void 0||a.call(r,t,this.openElements.current),n){let s,o;this.openElements.stackTop===0&&this.fragmentContext?(s=this.fragmentContext,o=this.fragmentContextID):{current:s,currentTagId:o}=this.openElements,this._setContextModes(s,o)}}_setContextModes(t,n){const r=t===this.document||t&&this.treeAdapter.getNamespaceURI(t)===Ae.HTML;this.currentNotInHTML=!r,this.tokenizer.inForeignNode=!r&&t!==void 0&&n!==void 0&&!this._isIntegrationPoint(n,t)}_switchToTextParsing(t,n){this._insertElement(t,Ae.HTML),this.tokenizer.state=n,this.originalInsertionMode=this.insertionMode,this.insertionMode=j.TEXT}switchToPlaintextParsing(){this.insertionMode=j.TEXT,this.originalInsertionMode=j.IN_BODY,this.tokenizer.state=zn.PLAINTEXT}_getAdjustedCurrentElement(){return this.openElements.stackTop===0&&this.fragmentContext?this.fragmentContext:this.openElements.current}_findFormInFragmentContext(){let t=this.fragmentContext;for(;t;){if(this.treeAdapter.getTagName(t)===ne.FORM){this.formElement=t;break}t=this.treeAdapter.getParentNode(t)}}_initTokenizerForFragmentParsing(){if(!(!this.fragmentContext||this.treeAdapter.getNamespaceURI(this.fragmentContext)!==Ae.HTML))switch(this.fragmentContextID){case _.TITLE:case _.TEXTAREA:{this.tokenizer.state=zn.RCDATA;break}case _.STYLE:case _.XMP:case _.IFRAME:case _.NOEMBED:case _.NOFRAMES:case _.NOSCRIPT:{this.tokenizer.state=zn.RAWTEXT;break}case _.SCRIPT:{this.tokenizer.state=zn.SCRIPT_DATA;break}case _.PLAINTEXT:{this.tokenizer.state=zn.PLAINTEXT;break}}}_setDocumentType(t){const n=t.name||"",r=t.publicId||"",a=t.systemId||"";if(this.treeAdapter.setDocumentType(this.document,n,r,a),t.location){const o=this.treeAdapter.getChildNodes(this.document).find(c=>this.treeAdapter.isDocumentTypeNode(c));o&&this.treeAdapter.setNodeSourceCodeLocation(o,t.location)}}_attachElementToTree(t,n){if(this.options.sourceCodeLocationInfo){const r=n&&{...n,startTag:n};this.treeAdapter.setNodeSourceCodeLocation(t,r)}if(this._shouldFosterParentOnInsertion())this._fosterParentElement(t);else{const r=this.openElements.currentTmplContentOrNode;this.treeAdapter.appendChild(r??this.document,t)}}_appendElement(t,n){const r=this.treeAdapter.createElement(t.tagName,n,t.attrs);this._attachElementToTree(r,t.location)}_insertElement(t,n){const r=this.treeAdapter.createElement(t.tagName,n,t.attrs);this._attachElementToTree(r,t.location),this.openElements.push(r,t.tagID)}_insertFakeElement(t,n){const r=this.treeAdapter.createElement(t,Ae.HTML,[]);this._attachElementToTree(r,null),this.openElements.push(r,n)}_insertTemplate(t){const n=this.treeAdapter.createElement(t.tagName,Ae.HTML,t.attrs),r=this.treeAdapter.createDocumentFragment();this.treeAdapter.setTemplateContent(n,r),this._attachElementToTree(n,t.location),this.openElements.push(n,t.tagID),this.options.sourceCodeLocationInfo&&this.treeAdapter.setNodeSourceCodeLocation(r,null)}_insertFakeRootElement(){const t=this.treeAdapter.createElement(ne.HTML,Ae.HTML,[]);this.options.sourceCodeLocationInfo&&this.treeAdapter.setNodeSourceCodeLocation(t,null),this.treeAdapter.appendChild(this.openElements.current,t),this.openElements.push(t,_.HTML)}_appendCommentNode(t,n){const r=this.treeAdapter.createCommentNode(t.data);this.treeAdapter.appendChild(n,r),this.options.sourceCodeLocationInfo&&this.treeAdapter.setNodeSourceCodeLocation(r,t.location)}_insertCharacters(t){let n,r;if(this._shouldFosterParentOnInsertion()?({parent:n,beforeElement:r}=this._findFosterParentingLocation(),r?this.treeAdapter.insertTextBefore(n,t.chars,r):this.treeAdapter.insertText(n,t.chars)):(n=this.openElements.currentTmplContentOrNode,this.treeAdapter.insertText(n,t.chars)),!t.location)return;const a=this.treeAdapter.getChildNodes(n),s=r?a.lastIndexOf(r):a.length,o=a[s-1];if(this.treeAdapter.getNodeSourceCodeLocation(o)){const{endLine:d,endCol:f,endOffset:m}=t.location;this.treeAdapter.updateNodeSourceCodeLocation(o,{endLine:d,endCol:f,endOffset:m})}else this.options.sourceCodeLocationInfo&&this.treeAdapter.setNodeSourceCodeLocation(o,t.location)}_adoptNodes(t,n){for(let r=this.treeAdapter.getFirstChild(t);r;r=this.treeAdapter.getFirstChild(t))this.treeAdapter.detachNode(r),this.treeAdapter.appendChild(n,r)}_setEndLocation(t,n){if(this.treeAdapter.getNodeSourceCodeLocation(t)&&n.location){const r=n.location,a=this.treeAdapter.getTagName(t),s=n.type===Ct.END_TAG&&a===n.tagName?{endTag:{...r},endLine:r.endLine,endCol:r.endCol,endOffset:r.endOffset}:{endLine:r.startLine,endCol:r.startCol,endOffset:r.startOffset};this.treeAdapter.updateNodeSourceCodeLocation(t,s)}}shouldProcessStartTagTokenInForeignContent(t){if(!this.currentNotInHTML)return!1;let n,r;return this.openElements.stackTop===0&&this.fragmentContext?(n=this.fragmentContext,r=this.fragmentContextID):{current:n,currentTagId:r}=this.openElements,t.tagID===_.SVG&&this.treeAdapter.getTagName(n)===ne.ANNOTATION_XML&&this.treeAdapter.getNamespaceURI(n)===Ae.MATHML?!1:this.tokenizer.inForeignNode||(t.tagID===_.MGLYPH||t.tagID===_.MALIGNMARK)&&r!==void 0&&!this._isIntegrationPoint(r,n,Ae.HTML)}_processToken(t){switch(t.type){case Ct.CHARACTER:{this.onCharacter(t);break}case Ct.NULL_CHARACTER:{this.onNullCharacter(t);break}case Ct.COMMENT:{this.onComment(t);break}case Ct.DOCTYPE:{this.onDoctype(t);break}case Ct.START_TAG:{this._processStartTag(t);break}case Ct.END_TAG:{this.onEndTag(t);break}case Ct.EOF:{this.onEof(t);break}case Ct.WHITESPACE_CHARACTER:{this.onWhitespaceCharacter(t);break}}}_isIntegrationPoint(t,n,r){const a=this.treeAdapter.getNamespaceURI(n),s=this.treeAdapter.getAttrList(n);return BU(t,a,s,r)}_reconstructActiveFormattingElements(){const t=this.activeFormattingElements.entries.length;if(t){const n=this.activeFormattingElements.entries.findIndex(a=>a.type===ui.Marker||this.openElements.contains(a.element)),r=n===-1?t-1:n-1;for(let a=r;a>=0;a--){const s=this.activeFormattingElements.entries[a];this._insertElement(s.token,this.treeAdapter.getNamespaceURI(s.element)),s.element=this.openElements.current}}}_closeTableCell(){this.openElements.generateImpliedEndTags(),this.openElements.popUntilTableCellPopped(),this.activeFormattingElements.clearToLastMarker(),this.insertionMode=j.IN_ROW}_closePElement(){this.openElements.generateImpliedEndTagsWithExclusion(_.P),this.openElements.popUntilTagNamePopped(_.P)}_resetInsertionMode(){for(let t=this.openElements.stackTop;t>=0;t--)switch(t===0&&this.fragmentContext?this.fragmentContextID:this.openElements.tagIDs[t]){case _.TR:{this.insertionMode=j.IN_ROW;return}case _.TBODY:case _.THEAD:case _.TFOOT:{this.insertionMode=j.IN_TABLE_BODY;return}case _.CAPTION:{this.insertionMode=j.IN_CAPTION;return}case _.COLGROUP:{this.insertionMode=j.IN_COLUMN_GROUP;return}case _.TABLE:{this.insertionMode=j.IN_TABLE;return}case _.BODY:{this.insertionMode=j.IN_BODY;return}case _.FRAMESET:{this.insertionMode=j.IN_FRAMESET;return}case _.SELECT:{this._resetInsertionModeForSelect(t);return}case _.TEMPLATE:{this.insertionMode=this.tmplInsertionModeStack[0];return}case _.HTML:{this.insertionMode=this.headElement?j.AFTER_HEAD:j.BEFORE_HEAD;return}case _.TD:case _.TH:{if(t>0){this.insertionMode=j.IN_CELL;return}break}case _.HEAD:{if(t>0){this.insertionMode=j.IN_HEAD;return}break}}this.insertionMode=j.IN_BODY}_resetInsertionModeForSelect(t){if(t>0)for(let n=t-1;n>0;n--){const r=this.openElements.tagIDs[n];if(r===_.TEMPLATE)break;if(r===_.TABLE){this.insertionMode=j.IN_SELECT_IN_TABLE;return}}this.insertionMode=j.IN_SELECT}_isElementCausesFosterParenting(t){return ST.has(t)}_shouldFosterParentOnInsertion(){return this.fosterParentingEnabled&&this.openElements.currentTagId!==void 0&&this._isElementCausesFosterParenting(this.openElements.currentTagId)}_findFosterParentingLocation(){for(let t=this.openElements.stackTop;t>=0;t--){const n=this.openElements.items[t];switch(this.openElements.tagIDs[t]){case _.TEMPLATE:{if(this.treeAdapter.getNamespaceURI(n)===Ae.HTML)return{parent:this.treeAdapter.getTemplateContent(n),beforeElement:null};break}case _.TABLE:{const r=this.treeAdapter.getParentNode(n);return r?{parent:r,beforeElement:n}:{parent:this.openElements.items[t-1],beforeElement:null}}}}return{parent:this.openElements.items[0],beforeElement:null}}_fosterParentElement(t){const n=this._findFosterParentingLocation();n.beforeElement?this.treeAdapter.insertBefore(n.parent,t,n.beforeElement):this.treeAdapter.appendChild(n.parent,t)}_isSpecialElement(t,n){const r=this.treeAdapter.getNamespaceURI(t);return lU[r].has(n)}onCharacter(t){if(this.skipNextNewLine=!1,this.tokenizer.inForeignNode){bz(this,t);return}switch(this.insertionMode){case j.INITIAL:{Cc(this,t);break}case j.BEFORE_HTML:{t0(this,t);break}case j.BEFORE_HEAD:{n0(this,t);break}case j.IN_HEAD:{r0(this,t);break}case j.IN_HEAD_NO_SCRIPT:{a0(this,t);break}case j.AFTER_HEAD:{i0(this,t);break}case j.IN_BODY:case j.IN_CAPTION:case j.IN_CELL:case j.IN_TEMPLATE:{AT(this,t);break}case j.TEXT:case j.IN_SELECT:case j.IN_SELECT_IN_TABLE:{this._insertCharacters(t);break}case j.IN_TABLE:case j.IN_TABLE_BODY:case j.IN_ROW:{Z1(this,t);break}case j.IN_TABLE_TEXT:{MT(this,t);break}case j.IN_COLUMN_GROUP:{af(this,t);break}case j.AFTER_BODY:{sf(this,t);break}case j.AFTER_AFTER_BODY:{jh(this,t);break}}}onNullCharacter(t){if(this.skipNextNewLine=!1,this.tokenizer.inForeignNode){gz(this,t);return}switch(this.insertionMode){case j.INITIAL:{Cc(this,t);break}case j.BEFORE_HTML:{t0(this,t);break}case j.BEFORE_HEAD:{n0(this,t);break}case j.IN_HEAD:{r0(this,t);break}case j.IN_HEAD_NO_SCRIPT:{a0(this,t);break}case j.AFTER_HEAD:{i0(this,t);break}case j.TEXT:{this._insertCharacters(t);break}case j.IN_TABLE:case j.IN_TABLE_BODY:case j.IN_ROW:{Z1(this,t);break}case j.IN_COLUMN_GROUP:{af(this,t);break}case j.AFTER_BODY:{sf(this,t);break}case j.AFTER_AFTER_BODY:{jh(this,t);break}}}onComment(t){if(this.skipNextNewLine=!1,this.currentNotInHTML){fb(this,t);return}switch(this.insertionMode){case j.INITIAL:case j.BEFORE_HTML:case j.BEFORE_HEAD:case j.IN_HEAD:case j.IN_HEAD_NO_SCRIPT:case j.AFTER_HEAD:case j.IN_BODY:case j.IN_TABLE:case j.IN_CAPTION:case j.IN_COLUMN_GROUP:case j.IN_TABLE_BODY:case j.IN_ROW:case j.IN_CELL:case j.IN_SELECT:case j.IN_SELECT_IN_TABLE:case j.IN_TEMPLATE:case j.IN_FRAMESET:case j.AFTER_FRAMESET:{fb(this,t);break}case j.IN_TABLE_TEXT:{Nc(this,t);break}case j.AFTER_BODY:{KU(this,t);break}case j.AFTER_AFTER_BODY:case j.AFTER_AFTER_FRAMESET:{WU(this,t);break}}}onDoctype(t){switch(this.skipNextNewLine=!1,this.insertionMode){case j.INITIAL:{XU(this,t);break}case j.BEFORE_HEAD:case j.IN_HEAD:case j.IN_HEAD_NO_SCRIPT:case j.AFTER_HEAD:{this._err(t,ve.misplacedDoctype);break}case j.IN_TABLE_TEXT:{Nc(this,t);break}}}onStartTag(t){this.skipNextNewLine=!1,this.currentToken=t,this._processStartTag(t),t.selfClosing&&!t.ackSelfClosing&&this._err(t,ve.nonVoidHtmlElementStartTagWithTrailingSolidus)}_processStartTag(t){this.shouldProcessStartTagTokenInForeignContent(t)?vz(this,t):this._startTagOutsideForeignContent(t)}_startTagOutsideForeignContent(t){switch(this.insertionMode){case j.INITIAL:{Cc(this,t);break}case j.BEFORE_HTML:{QU(this,t);break}case j.BEFORE_HEAD:{JU(this,t);break}case j.IN_HEAD:{Ja(this,t);break}case j.IN_HEAD_NO_SCRIPT:{nF(this,t);break}case j.AFTER_HEAD:{aF(this,t);break}case j.IN_BODY:{Nr(this,t);break}case j.IN_TABLE:{nu(this,t);break}case j.IN_TABLE_TEXT:{Nc(this,t);break}case j.IN_CAPTION:{ez(this,t);break}case j.IN_COLUMN_GROUP:{D2(this,t);break}case j.IN_TABLE_BODY:{Qf(this,t);break}case j.IN_ROW:{Zf(this,t);break}case j.IN_CELL:{rz(this,t);break}case j.IN_SELECT:{LT(this,t);break}case j.IN_SELECT_IN_TABLE:{iz(this,t);break}case j.IN_TEMPLATE:{lz(this,t);break}case j.AFTER_BODY:{uz(this,t);break}case j.IN_FRAMESET:{cz(this,t);break}case j.AFTER_FRAMESET:{hz(this,t);break}case j.AFTER_AFTER_BODY:{mz(this,t);break}case j.AFTER_AFTER_FRAMESET:{pz(this,t);break}}}onEndTag(t){this.skipNextNewLine=!1,this.currentToken=t,this.currentNotInHTML?yz(this,t):this._endTagOutsideForeignContent(t)}_endTagOutsideForeignContent(t){switch(this.insertionMode){case j.INITIAL:{Cc(this,t);break}case j.BEFORE_HTML:{ZU(this,t);break}case j.BEFORE_HEAD:{eF(this,t);break}case j.IN_HEAD:{tF(this,t);break}case j.IN_HEAD_NO_SCRIPT:{rF(this,t);break}case j.AFTER_HEAD:{iF(this,t);break}case j.IN_BODY:{Xf(this,t);break}case j.TEXT:{$F(this,t);break}case j.IN_TABLE:{m0(this,t);break}case j.IN_TABLE_TEXT:{Nc(this,t);break}case j.IN_CAPTION:{tz(this,t);break}case j.IN_COLUMN_GROUP:{nz(this,t);break}case j.IN_TABLE_BODY:{mb(this,t);break}case j.IN_ROW:{DT(this,t);break}case j.IN_CELL:{az(this,t);break}case j.IN_SELECT:{PT(this,t);break}case j.IN_SELECT_IN_TABLE:{sz(this,t);break}case j.IN_TEMPLATE:{oz(this,t);break}case j.AFTER_BODY:{UT(this,t);break}case j.IN_FRAMESET:{dz(this,t);break}case j.AFTER_FRAMESET:{fz(this,t);break}case j.AFTER_AFTER_BODY:{jh(this,t);break}}}onEof(t){switch(this.insertionMode){case j.INITIAL:{Cc(this,t);break}case j.BEFORE_HTML:{t0(this,t);break}case j.BEFORE_HEAD:{n0(this,t);break}case j.IN_HEAD:{r0(this,t);break}case j.IN_HEAD_NO_SCRIPT:{a0(this,t);break}case j.AFTER_HEAD:{i0(this,t);break}case j.IN_BODY:case j.IN_TABLE:case j.IN_CAPTION:case j.IN_COLUMN_GROUP:case j.IN_TABLE_BODY:case j.IN_ROW:case j.IN_CELL:case j.IN_SELECT:case j.IN_SELECT_IN_TABLE:{OT(this,t);break}case j.TEXT:{YF(this,t);break}case j.IN_TABLE_TEXT:{Nc(this,t);break}case j.IN_TEMPLATE:{BT(this,t);break}case j.AFTER_BODY:case j.IN_FRAMESET:case j.AFTER_FRAMESET:case j.AFTER_AFTER_BODY:case j.AFTER_AFTER_FRAMESET:{I2(this,t);break}}}onWhitespaceCharacter(t){if(this.skipNextNewLine&&(this.skipNextNewLine=!1,t.chars.charCodeAt(0)===P.LINE_FEED)){if(t.chars.length===1)return;t.chars=t.chars.substr(1)}if(this.tokenizer.inForeignNode){this._insertCharacters(t);return}switch(this.insertionMode){case j.IN_HEAD:case j.IN_HEAD_NO_SCRIPT:case j.AFTER_HEAD:case j.TEXT:case j.IN_COLUMN_GROUP:case j.IN_SELECT:case j.IN_SELECT_IN_TABLE:case j.IN_FRAMESET:case j.AFTER_FRAMESET:{this._insertCharacters(t);break}case j.IN_BODY:case j.IN_CAPTION:case j.IN_CELL:case j.IN_TEMPLATE:case j.AFTER_BODY:case j.AFTER_AFTER_BODY:case j.AFTER_AFTER_FRAMESET:{wT(this,t);break}case j.IN_TABLE:case j.IN_TABLE_BODY:case j.IN_ROW:{Z1(this,t);break}case j.IN_TABLE_TEXT:{RT(this,t);break}}}};function qU(e,t){let n=e.activeFormattingElements.getElementEntryInScopeWithTagName(t.tagName);return n?e.openElements.contains(n.element)?e.openElements.hasInScope(t.tagID)||(n=null):(e.activeFormattingElements.removeEntry(n),n=null):NT(e,t),n}function jU(e,t){let n=null,r=e.openElements.stackTop;for(;r>=0;r--){const a=e.openElements.items[r];if(a===t.element)break;e._isSpecialElement(a,e.openElements.tagIDs[r])&&(n=a)}return n||(e.openElements.shortenToLength(Math.max(r,0)),e.activeFormattingElements.removeEntry(t)),n}function $U(e,t,n){let r=t,a=e.openElements.getCommonAncestor(t);for(let s=0,o=a;o!==n;s++,o=a){a=e.openElements.getCommonAncestor(o);const c=e.activeFormattingElements.getElementEntry(o),d=c&&s>=zU;!c||d?(d&&e.activeFormattingElements.removeEntry(c),e.openElements.remove(o)):(o=YU(e,c),r===t&&(e.activeFormattingElements.bookmark=c),e.treeAdapter.detachNode(r),e.treeAdapter.appendChild(o,r),r=o)}return r}function YU(e,t){const n=e.treeAdapter.getNamespaceURI(t.element),r=e.treeAdapter.createElement(t.token.tagName,n,t.token.attrs);return e.openElements.replace(t.element,r),t.element=r,r}function GU(e,t,n){const r=e.treeAdapter.getTagName(t),a=gu(r);if(e._isElementCausesFosterParenting(a))e._fosterParentElement(n);else{const s=e.treeAdapter.getNamespaceURI(t);a===_.TEMPLATE&&s===Ae.HTML&&(t=e.treeAdapter.getTemplateContent(t)),e.treeAdapter.appendChild(t,n)}}function VU(e,t,n){const r=e.treeAdapter.getNamespaceURI(n.element),{token:a}=n,s=e.treeAdapter.createElement(a.tagName,r,a.attrs);e._adoptNodes(t,s),e.treeAdapter.appendChild(t,s),e.activeFormattingElements.insertElementAfterBookmark(s,a),e.activeFormattingElements.removeEntry(n),e.openElements.remove(n.element),e.openElements.insertAfter(t,s,a.tagID)}function M2(e,t){for(let n=0;n<FU;n++){const r=qU(e,t);if(!r)break;const a=jU(e,r);if(!a)break;e.activeFormattingElements.bookmark=r;const s=$U(e,a,r.element),o=e.openElements.getCommonAncestor(r.element);e.treeAdapter.detachNode(s),o&&GU(e,o,s),VU(e,a,r)}}function fb(e,t){e._appendCommentNode(t,e.openElements.currentTmplContentOrNode)}function KU(e,t){e._appendCommentNode(t,e.openElements.items[0])}function WU(e,t){e._appendCommentNode(t,e.document)}function I2(e,t){if(e.stopped=!0,t.location){const n=e.fragmentContext?0:2;for(let r=e.openElements.stackTop;r>=n;r--)e._setEndLocation(e.openElements.items[r],t);if(!e.fragmentContext&&e.openElements.stackTop>=0){const r=e.openElements.items[0],a=e.treeAdapter.getNodeSourceCodeLocation(r);if(a&&!a.endTag&&(e._setEndLocation(r,t),e.openElements.stackTop>=1)){const s=e.openElements.items[1],o=e.treeAdapter.getNodeSourceCodeLocation(s);o&&!o.endTag&&e._setEndLocation(s,t)}}}}function XU(e,t){e._setDocumentType(t);const n=t.forceQuirks?Ma.QUIRKS:AU(t);wU(t)||e._err(t,ve.nonConformingDoctype),e.treeAdapter.setDocumentMode(e.document,n),e.insertionMode=j.BEFORE_HTML}function Cc(e,t){e._err(t,ve.missingDoctype,!0),e.treeAdapter.setDocumentMode(e.document,Ma.QUIRKS),e.insertionMode=j.BEFORE_HTML,e._processToken(t)}function QU(e,t){t.tagID===_.HTML?(e._insertElement(t,Ae.HTML),e.insertionMode=j.BEFORE_HEAD):t0(e,t)}function ZU(e,t){const n=t.tagID;(n===_.HTML||n===_.HEAD||n===_.BODY||n===_.BR)&&t0(e,t)}function t0(e,t){e._insertFakeRootElement(),e.insertionMode=j.BEFORE_HEAD,e._processToken(t)}function JU(e,t){switch(t.tagID){case _.HTML:{Nr(e,t);break}case _.HEAD:{e._insertElement(t,Ae.HTML),e.headElement=e.openElements.current,e.insertionMode=j.IN_HEAD;break}default:n0(e,t)}}function eF(e,t){const n=t.tagID;n===_.HEAD||n===_.BODY||n===_.HTML||n===_.BR?n0(e,t):e._err(t,ve.endTagWithoutMatchingOpenElement)}function n0(e,t){e._insertFakeElement(ne.HEAD,_.HEAD),e.headElement=e.openElements.current,e.insertionMode=j.IN_HEAD,e._processToken(t)}function Ja(e,t){switch(t.tagID){case _.HTML:{Nr(e,t);break}case _.BASE:case _.BASEFONT:case _.BGSOUND:case _.LINK:case _.META:{e._appendElement(t,Ae.HTML),t.ackSelfClosing=!0;break}case _.TITLE:{e._switchToTextParsing(t,zn.RCDATA);break}case _.NOSCRIPT:{e.options.scriptingEnabled?e._switchToTextParsing(t,zn.RAWTEXT):(e._insertElement(t,Ae.HTML),e.insertionMode=j.IN_HEAD_NO_SCRIPT);break}case _.NOFRAMES:case _.STYLE:{e._switchToTextParsing(t,zn.RAWTEXT);break}case _.SCRIPT:{e._switchToTextParsing(t,zn.SCRIPT_DATA);break}case _.TEMPLATE:{e._insertTemplate(t),e.activeFormattingElements.insertMarker(),e.framesetOk=!1,e.insertionMode=j.IN_TEMPLATE,e.tmplInsertionModeStack.unshift(j.IN_TEMPLATE);break}case _.HEAD:{e._err(t,ve.misplacedStartTagForHeadElement);break}default:r0(e,t)}}function tF(e,t){switch(t.tagID){case _.HEAD:{e.openElements.pop(),e.insertionMode=j.AFTER_HEAD;break}case _.BODY:case _.BR:case _.HTML:{r0(e,t);break}case _.TEMPLATE:{Jl(e,t);break}default:e._err(t,ve.endTagWithoutMatchingOpenElement)}}function Jl(e,t){e.openElements.tmplCount>0?(e.openElements.generateImpliedEndTagsThoroughly(),e.openElements.currentTagId!==_.TEMPLATE&&e._err(t,ve.closingOfElementWithOpenChildElements),e.openElements.popUntilTagNamePopped(_.TEMPLATE),e.activeFormattingElements.clearToLastMarker(),e.tmplInsertionModeStack.shift(),e._resetInsertionMode()):e._err(t,ve.endTagWithoutMatchingOpenElement)}function r0(e,t){e.openElements.pop(),e.insertionMode=j.AFTER_HEAD,e._processToken(t)}function nF(e,t){switch(t.tagID){case _.HTML:{Nr(e,t);break}case _.BASEFONT:case _.BGSOUND:case _.HEAD:case _.LINK:case _.META:case _.NOFRAMES:case _.STYLE:{Ja(e,t);break}case _.NOSCRIPT:{e._err(t,ve.nestedNoscriptInHead);break}default:a0(e,t)}}function rF(e,t){switch(t.tagID){case _.NOSCRIPT:{e.openElements.pop(),e.insertionMode=j.IN_HEAD;break}case _.BR:{a0(e,t);break}default:e._err(t,ve.endTagWithoutMatchingOpenElement)}}function a0(e,t){const n=t.type===Ct.EOF?ve.openElementsLeftAfterEof:ve.disallowedContentInNoscriptInHead;e._err(t,n),e.openElements.pop(),e.insertionMode=j.IN_HEAD,e._processToken(t)}function aF(e,t){switch(t.tagID){case _.HTML:{Nr(e,t);break}case _.BODY:{e._insertElement(t,Ae.HTML),e.framesetOk=!1,e.insertionMode=j.IN_BODY;break}case _.FRAMESET:{e._insertElement(t,Ae.HTML),e.insertionMode=j.IN_FRAMESET;break}case _.BASE:case _.BASEFONT:case _.BGSOUND:case _.LINK:case _.META:case _.NOFRAMES:case _.SCRIPT:case _.STYLE:case _.TEMPLATE:case _.TITLE:{e._err(t,ve.abandonedHeadElementChild),e.openElements.push(e.headElement,_.HEAD),Ja(e,t),e.openElements.remove(e.headElement);break}case _.HEAD:{e._err(t,ve.misplacedStartTagForHeadElement);break}default:i0(e,t)}}function iF(e,t){switch(t.tagID){case _.BODY:case _.HTML:case _.BR:{i0(e,t);break}case _.TEMPLATE:{Jl(e,t);break}default:e._err(t,ve.endTagWithoutMatchingOpenElement)}}function i0(e,t){e._insertFakeElement(ne.BODY,_.BODY),e.insertionMode=j.IN_BODY,Wf(e,t)}function Wf(e,t){switch(t.type){case Ct.CHARACTER:{AT(e,t);break}case Ct.WHITESPACE_CHARACTER:{wT(e,t);break}case Ct.COMMENT:{fb(e,t);break}case Ct.START_TAG:{Nr(e,t);break}case Ct.END_TAG:{Xf(e,t);break}case Ct.EOF:{OT(e,t);break}}}function wT(e,t){e._reconstructActiveFormattingElements(),e._insertCharacters(t)}function AT(e,t){e._reconstructActiveFormattingElements(),e._insertCharacters(t),e.framesetOk=!1}function sF(e,t){e.openElements.tmplCount===0&&e.treeAdapter.adoptAttributes(e.openElements.items[0],t.attrs)}function lF(e,t){const n=e.openElements.tryPeekProperlyNestedBodyElement();n&&e.openElements.tmplCount===0&&(e.framesetOk=!1,e.treeAdapter.adoptAttributes(n,t.attrs))}function oF(e,t){const n=e.openElements.tryPeekProperlyNestedBodyElement();e.framesetOk&&n&&(e.treeAdapter.detachNode(n),e.openElements.popAllUpToHtmlElement(),e._insertElement(t,Ae.HTML),e.insertionMode=j.IN_FRAMESET)}function uF(e,t){e.openElements.hasInButtonScope(_.P)&&e._closePElement(),e._insertElement(t,Ae.HTML)}function cF(e,t){e.openElements.hasInButtonScope(_.P)&&e._closePElement(),e.openElements.currentTagId!==void 0&&hb.has(e.openElements.currentTagId)&&e.openElements.pop(),e._insertElement(t,Ae.HTML)}function dF(e,t){e.openElements.hasInButtonScope(_.P)&&e._closePElement(),e._insertElement(t,Ae.HTML),e.skipNextNewLine=!0,e.framesetOk=!1}function hF(e,t){const n=e.openElements.tmplCount>0;(!e.formElement||n)&&(e.openElements.hasInButtonScope(_.P)&&e._closePElement(),e._insertElement(t,Ae.HTML),n||(e.formElement=e.openElements.current))}function fF(e,t){e.framesetOk=!1;const n=t.tagID;for(let r=e.openElements.stackTop;r>=0;r--){const a=e.openElements.tagIDs[r];if(n===_.LI&&a===_.LI||(n===_.DD||n===_.DT)&&(a===_.DD||a===_.DT)){e.openElements.generateImpliedEndTagsWithExclusion(a),e.openElements.popUntilTagNamePopped(a);break}if(a!==_.ADDRESS&&a!==_.DIV&&a!==_.P&&e._isSpecialElement(e.openElements.items[r],a))break}e.openElements.hasInButtonScope(_.P)&&e._closePElement(),e._insertElement(t,Ae.HTML)}function mF(e,t){e.openElements.hasInButtonScope(_.P)&&e._closePElement(),e._insertElement(t,Ae.HTML),e.tokenizer.state=zn.PLAINTEXT}function pF(e,t){e.openElements.hasInScope(_.BUTTON)&&(e.openElements.generateImpliedEndTags(),e.openElements.popUntilTagNamePopped(_.BUTTON)),e._reconstructActiveFormattingElements(),e._insertElement(t,Ae.HTML),e.framesetOk=!1}function gF(e,t){const n=e.activeFormattingElements.getElementEntryInScopeWithTagName(ne.A);n&&(M2(e,t),e.openElements.remove(n.element),e.activeFormattingElements.removeEntry(n)),e._reconstructActiveFormattingElements(),e._insertElement(t,Ae.HTML),e.activeFormattingElements.pushElement(e.openElements.current,t)}function bF(e,t){e._reconstructActiveFormattingElements(),e._insertElement(t,Ae.HTML),e.activeFormattingElements.pushElement(e.openElements.current,t)}function vF(e,t){e._reconstructActiveFormattingElements(),e.openElements.hasInScope(_.NOBR)&&(M2(e,t),e._reconstructActiveFormattingElements()),e._insertElement(t,Ae.HTML),e.activeFormattingElements.pushElement(e.openElements.current,t)}function yF(e,t){e._reconstructActiveFormattingElements(),e._insertElement(t,Ae.HTML),e.activeFormattingElements.insertMarker(),e.framesetOk=!1}function _F(e,t){e.treeAdapter.getDocumentMode(e.document)!==Ma.QUIRKS&&e.openElements.hasInButtonScope(_.P)&&e._closePElement(),e._insertElement(t,Ae.HTML),e.framesetOk=!1,e.insertionMode=j.IN_TABLE}function kT(e,t){e._reconstructActiveFormattingElements(),e._appendElement(t,Ae.HTML),e.framesetOk=!1,t.ackSelfClosing=!0}function CT(e){const t=gT(e,$l.TYPE);return t!=null&&t.toLowerCase()===UU}function EF(e,t){e._reconstructActiveFormattingElements(),e._appendElement(t,Ae.HTML),CT(t)||(e.framesetOk=!1),t.ackSelfClosing=!0}function TF(e,t){e._appendElement(t,Ae.HTML),t.ackSelfClosing=!0}function xF(e,t){e.openElements.hasInButtonScope(_.P)&&e._closePElement(),e._appendElement(t,Ae.HTML),e.framesetOk=!1,t.ackSelfClosing=!0}function SF(e,t){t.tagName=ne.IMG,t.tagID=_.IMG,kT(e,t)}function wF(e,t){e._insertElement(t,Ae.HTML),e.skipNextNewLine=!0,e.tokenizer.state=zn.RCDATA,e.originalInsertionMode=e.insertionMode,e.framesetOk=!1,e.insertionMode=j.TEXT}function AF(e,t){e.openElements.hasInButtonScope(_.P)&&e._closePElement(),e._reconstructActiveFormattingElements(),e.framesetOk=!1,e._switchToTextParsing(t,zn.RAWTEXT)}function kF(e,t){e.framesetOk=!1,e._switchToTextParsing(t,zn.RAWTEXT)}function BE(e,t){e._switchToTextParsing(t,zn.RAWTEXT)}function CF(e,t){e._reconstructActiveFormattingElements(),e._insertElement(t,Ae.HTML),e.framesetOk=!1,e.insertionMode=e.insertionMode===j.IN_TABLE||e.insertionMode===j.IN_CAPTION||e.insertionMode===j.IN_TABLE_BODY||e.insertionMode===j.IN_ROW||e.insertionMode===j.IN_CELL?j.IN_SELECT_IN_TABLE:j.IN_SELECT}function NF(e,t){e.openElements.currentTagId===_.OPTION&&e.openElements.pop(),e._reconstructActiveFormattingElements(),e._insertElement(t,Ae.HTML)}function OF(e,t){e.openElements.hasInScope(_.RUBY)&&e.openElements.generateImpliedEndTags(),e._insertElement(t,Ae.HTML)}function RF(e,t){e.openElements.hasInScope(_.RUBY)&&e.openElements.generateImpliedEndTagsWithExclusion(_.RTC),e._insertElement(t,Ae.HTML)}function MF(e,t){e._reconstructActiveFormattingElements(),TT(t),R2(t),t.selfClosing?e._appendElement(t,Ae.MATHML):e._insertElement(t,Ae.MATHML),t.ackSelfClosing=!0}function IF(e,t){e._reconstructActiveFormattingElements(),xT(t),R2(t),t.selfClosing?e._appendElement(t,Ae.SVG):e._insertElement(t,Ae.SVG),t.ackSelfClosing=!0}function UE(e,t){e._reconstructActiveFormattingElements(),e._insertElement(t,Ae.HTML)}function Nr(e,t){switch(t.tagID){case _.I:case _.S:case _.B:case _.U:case _.EM:case _.TT:case _.BIG:case _.CODE:case _.FONT:case _.SMALL:case _.STRIKE:case _.STRONG:{bF(e,t);break}case _.A:{gF(e,t);break}case _.H1:case _.H2:case _.H3:case _.H4:case _.H5:case _.H6:{cF(e,t);break}case _.P:case _.DL:case _.OL:case _.UL:case _.DIV:case _.DIR:case _.NAV:case _.MAIN:case _.MENU:case _.ASIDE:case _.CENTER:case _.FIGURE:case _.FOOTER:case _.HEADER:case _.HGROUP:case _.DIALOG:case _.DETAILS:case _.ADDRESS:case _.ARTICLE:case _.SEARCH:case _.SECTION:case _.SUMMARY:case _.FIELDSET:case _.BLOCKQUOTE:case _.FIGCAPTION:{uF(e,t);break}case _.LI:case _.DD:case _.DT:{fF(e,t);break}case _.BR:case _.IMG:case _.WBR:case _.AREA:case _.EMBED:case _.KEYGEN:{kT(e,t);break}case _.HR:{xF(e,t);break}case _.RB:case _.RTC:{OF(e,t);break}case _.RT:case _.RP:{RF(e,t);break}case _.PRE:case _.LISTING:{dF(e,t);break}case _.XMP:{AF(e,t);break}case _.SVG:{IF(e,t);break}case _.HTML:{sF(e,t);break}case _.BASE:case _.LINK:case _.META:case _.STYLE:case _.TITLE:case _.SCRIPT:case _.BGSOUND:case _.BASEFONT:case _.TEMPLATE:{Ja(e,t);break}case _.BODY:{lF(e,t);break}case _.FORM:{hF(e,t);break}case _.NOBR:{vF(e,t);break}case _.MATH:{MF(e,t);break}case _.TABLE:{_F(e,t);break}case _.INPUT:{EF(e,t);break}case _.PARAM:case _.TRACK:case _.SOURCE:{TF(e,t);break}case _.IMAGE:{SF(e,t);break}case _.BUTTON:{pF(e,t);break}case _.APPLET:case _.OBJECT:case _.MARQUEE:{yF(e,t);break}case _.IFRAME:{kF(e,t);break}case _.SELECT:{CF(e,t);break}case _.OPTION:case _.OPTGROUP:{NF(e,t);break}case _.NOEMBED:case _.NOFRAMES:{BE(e,t);break}case _.FRAMESET:{oF(e,t);break}case _.TEXTAREA:{wF(e,t);break}case _.NOSCRIPT:{e.options.scriptingEnabled?BE(e,t):UE(e,t);break}case _.PLAINTEXT:{mF(e,t);break}case _.COL:case _.TH:case _.TD:case _.TR:case _.HEAD:case _.FRAME:case _.TBODY:case _.TFOOT:case _.THEAD:case _.CAPTION:case _.COLGROUP:break;default:UE(e,t)}}function DF(e,t){if(e.openElements.hasInScope(_.BODY)&&(e.insertionMode=j.AFTER_BODY,e.options.sourceCodeLocationInfo)){const n=e.openElements.tryPeekProperlyNestedBodyElement();n&&e._setEndLocation(n,t)}}function LF(e,t){e.openElements.hasInScope(_.BODY)&&(e.insertionMode=j.AFTER_BODY,UT(e,t))}function PF(e,t){const n=t.tagID;e.openElements.hasInScope(n)&&(e.openElements.generateImpliedEndTags(),e.openElements.popUntilTagNamePopped(n))}function BF(e){const t=e.openElements.tmplCount>0,{formElement:n}=e;t||(e.formElement=null),(n||t)&&e.openElements.hasInScope(_.FORM)&&(e.openElements.generateImpliedEndTags(),t?e.openElements.popUntilTagNamePopped(_.FORM):n&&e.openElements.remove(n))}function UF(e){e.openElements.hasInButtonScope(_.P)||e._insertFakeElement(ne.P,_.P),e._closePElement()}function FF(e){e.openElements.hasInListItemScope(_.LI)&&(e.openElements.generateImpliedEndTagsWithExclusion(_.LI),e.openElements.popUntilTagNamePopped(_.LI))}function zF(e,t){const n=t.tagID;e.openElements.hasInScope(n)&&(e.openElements.generateImpliedEndTagsWithExclusion(n),e.openElements.popUntilTagNamePopped(n))}function HF(e){e.openElements.hasNumberedHeaderInScope()&&(e.openElements.generateImpliedEndTags(),e.openElements.popUntilNumberedHeaderPopped())}function qF(e,t){const n=t.tagID;e.openElements.hasInScope(n)&&(e.openElements.generateImpliedEndTags(),e.openElements.popUntilTagNamePopped(n),e.activeFormattingElements.clearToLastMarker())}function jF(e){e._reconstructActiveFormattingElements(),e._insertFakeElement(ne.BR,_.BR),e.openElements.pop(),e.framesetOk=!1}function NT(e,t){const n=t.tagName,r=t.tagID;for(let a=e.openElements.stackTop;a>0;a--){const s=e.openElements.items[a],o=e.openElements.tagIDs[a];if(r===o&&(r!==_.UNKNOWN||e.treeAdapter.getTagName(s)===n)){e.openElements.generateImpliedEndTagsWithExclusion(r),e.openElements.stackTop>=a&&e.openElements.shortenToLength(a);break}if(e._isSpecialElement(s,o))break}}function Xf(e,t){switch(t.tagID){case _.A:case _.B:case _.I:case _.S:case _.U:case _.EM:case _.TT:case _.BIG:case _.CODE:case _.FONT:case _.NOBR:case _.SMALL:case _.STRIKE:case _.STRONG:{M2(e,t);break}case _.P:{UF(e);break}case _.DL:case _.UL:case _.OL:case _.DIR:case _.DIV:case _.NAV:case _.PRE:case _.MAIN:case _.MENU:case _.ASIDE:case _.BUTTON:case _.CENTER:case _.FIGURE:case _.FOOTER:case _.HEADER:case _.HGROUP:case _.DIALOG:case _.ADDRESS:case _.ARTICLE:case _.DETAILS:case _.SEARCH:case _.SECTION:case _.SUMMARY:case _.LISTING:case _.FIELDSET:case _.BLOCKQUOTE:case _.FIGCAPTION:{PF(e,t);break}case _.LI:{FF(e);break}case _.DD:case _.DT:{zF(e,t);break}case _.H1:case _.H2:case _.H3:case _.H4:case _.H5:case _.H6:{HF(e);break}case _.BR:{jF(e);break}case _.BODY:{DF(e,t);break}case _.HTML:{LF(e,t);break}case _.FORM:{BF(e);break}case _.APPLET:case _.OBJECT:case _.MARQUEE:{qF(e,t);break}case _.TEMPLATE:{Jl(e,t);break}default:NT(e,t)}}function OT(e,t){e.tmplInsertionModeStack.length>0?BT(e,t):I2(e,t)}function $F(e,t){var n;t.tagID===_.SCRIPT&&((n=e.scriptHandler)===null||n===void 0||n.call(e,e.openElements.current)),e.openElements.pop(),e.insertionMode=e.originalInsertionMode}function YF(e,t){e._err(t,ve.eofInElementThatCanContainOnlyText),e.openElements.pop(),e.insertionMode=e.originalInsertionMode,e.onEof(t)}function Z1(e,t){if(e.openElements.currentTagId!==void 0&&ST.has(e.openElements.currentTagId))switch(e.pendingCharacterTokens.length=0,e.hasNonWhitespacePendingCharacterToken=!1,e.originalInsertionMode=e.insertionMode,e.insertionMode=j.IN_TABLE_TEXT,t.type){case Ct.CHARACTER:{MT(e,t);break}case Ct.WHITESPACE_CHARACTER:{RT(e,t);break}}else D0(e,t)}function GF(e,t){e.openElements.clearBackToTableContext(),e.activeFormattingElements.insertMarker(),e._insertElement(t,Ae.HTML),e.insertionMode=j.IN_CAPTION}function VF(e,t){e.openElements.clearBackToTableContext(),e._insertElement(t,Ae.HTML),e.insertionMode=j.IN_COLUMN_GROUP}function KF(e,t){e.openElements.clearBackToTableContext(),e._insertFakeElement(ne.COLGROUP,_.COLGROUP),e.insertionMode=j.IN_COLUMN_GROUP,D2(e,t)}function WF(e,t){e.openElements.clearBackToTableContext(),e._insertElement(t,Ae.HTML),e.insertionMode=j.IN_TABLE_BODY}function XF(e,t){e.openElements.clearBackToTableContext(),e._insertFakeElement(ne.TBODY,_.TBODY),e.insertionMode=j.IN_TABLE_BODY,Qf(e,t)}function QF(e,t){e.openElements.hasInTableScope(_.TABLE)&&(e.openElements.popUntilTagNamePopped(_.TABLE),e._resetInsertionMode(),e._processStartTag(t))}function ZF(e,t){CT(t)?e._appendElement(t,Ae.HTML):D0(e,t),t.ackSelfClosing=!0}function JF(e,t){!e.formElement&&e.openElements.tmplCount===0&&(e._insertElement(t,Ae.HTML),e.formElement=e.openElements.current,e.openElements.pop())}function nu(e,t){switch(t.tagID){case _.TD:case _.TH:case _.TR:{XF(e,t);break}case _.STYLE:case _.SCRIPT:case _.TEMPLATE:{Ja(e,t);break}case _.COL:{KF(e,t);break}case _.FORM:{JF(e,t);break}case _.TABLE:{QF(e,t);break}case _.TBODY:case _.TFOOT:case _.THEAD:{WF(e,t);break}case _.INPUT:{ZF(e,t);break}case _.CAPTION:{GF(e,t);break}case _.COLGROUP:{VF(e,t);break}default:D0(e,t)}}function m0(e,t){switch(t.tagID){case _.TABLE:{e.openElements.hasInTableScope(_.TABLE)&&(e.openElements.popUntilTagNamePopped(_.TABLE),e._resetInsertionMode());break}case _.TEMPLATE:{Jl(e,t);break}case _.BODY:case _.CAPTION:case _.COL:case _.COLGROUP:case _.HTML:case _.TBODY:case _.TD:case _.TFOOT:case _.TH:case _.THEAD:case _.TR:break;default:D0(e,t)}}function D0(e,t){const n=e.fosterParentingEnabled;e.fosterParentingEnabled=!0,Wf(e,t),e.fosterParentingEnabled=n}function RT(e,t){e.pendingCharacterTokens.push(t)}function MT(e,t){e.pendingCharacterTokens.push(t),e.hasNonWhitespacePendingCharacterToken=!0}function Nc(e,t){let n=0;if(e.hasNonWhitespacePendingCharacterToken)for(;n<e.pendingCharacterTokens.length;n++)D0(e,e.pendingCharacterTokens[n]);else for(;n<e.pendingCharacterTokens.length;n++)e._insertCharacters(e.pendingCharacterTokens[n]);e.insertionMode=e.originalInsertionMode,e._processToken(t)}const IT=new Set([_.CAPTION,_.COL,_.COLGROUP,_.TBODY,_.TD,_.TFOOT,_.TH,_.THEAD,_.TR]);function ez(e,t){const n=t.tagID;IT.has(n)?e.openElements.hasInTableScope(_.CAPTION)&&(e.openElements.generateImpliedEndTags(),e.openElements.popUntilTagNamePopped(_.CAPTION),e.activeFormattingElements.clearToLastMarker(),e.insertionMode=j.IN_TABLE,nu(e,t)):Nr(e,t)}function tz(e,t){const n=t.tagID;switch(n){case _.CAPTION:case _.TABLE:{e.openElements.hasInTableScope(_.CAPTION)&&(e.openElements.generateImpliedEndTags(),e.openElements.popUntilTagNamePopped(_.CAPTION),e.activeFormattingElements.clearToLastMarker(),e.insertionMode=j.IN_TABLE,n===_.TABLE&&m0(e,t));break}case _.BODY:case _.COL:case _.COLGROUP:case _.HTML:case _.TBODY:case _.TD:case _.TFOOT:case _.TH:case _.THEAD:case _.TR:break;default:Xf(e,t)}}function D2(e,t){switch(t.tagID){case _.HTML:{Nr(e,t);break}case _.COL:{e._appendElement(t,Ae.HTML),t.ackSelfClosing=!0;break}case _.TEMPLATE:{Ja(e,t);break}default:af(e,t)}}function nz(e,t){switch(t.tagID){case _.COLGROUP:{e.openElements.currentTagId===_.COLGROUP&&(e.openElements.pop(),e.insertionMode=j.IN_TABLE);break}case _.TEMPLATE:{Jl(e,t);break}case _.COL:break;default:af(e,t)}}function af(e,t){e.openElements.currentTagId===_.COLGROUP&&(e.openElements.pop(),e.insertionMode=j.IN_TABLE,e._processToken(t))}function Qf(e,t){switch(t.tagID){case _.TR:{e.openElements.clearBackToTableBodyContext(),e._insertElement(t,Ae.HTML),e.insertionMode=j.IN_ROW;break}case _.TH:case _.TD:{e.openElements.clearBackToTableBodyContext(),e._insertFakeElement(ne.TR,_.TR),e.insertionMode=j.IN_ROW,Zf(e,t);break}case _.CAPTION:case _.COL:case _.COLGROUP:case _.TBODY:case _.TFOOT:case _.THEAD:{e.openElements.hasTableBodyContextInTableScope()&&(e.openElements.clearBackToTableBodyContext(),e.openElements.pop(),e.insertionMode=j.IN_TABLE,nu(e,t));break}default:nu(e,t)}}function mb(e,t){const n=t.tagID;switch(t.tagID){case _.TBODY:case _.TFOOT:case _.THEAD:{e.openElements.hasInTableScope(n)&&(e.openElements.clearBackToTableBodyContext(),e.openElements.pop(),e.insertionMode=j.IN_TABLE);break}case _.TABLE:{e.openElements.hasTableBodyContextInTableScope()&&(e.openElements.clearBackToTableBodyContext(),e.openElements.pop(),e.insertionMode=j.IN_TABLE,m0(e,t));break}case _.BODY:case _.CAPTION:case _.COL:case _.COLGROUP:case _.HTML:case _.TD:case _.TH:case _.TR:break;default:m0(e,t)}}function Zf(e,t){switch(t.tagID){case _.TH:case _.TD:{e.openElements.clearBackToTableRowContext(),e._insertElement(t,Ae.HTML),e.insertionMode=j.IN_CELL,e.activeFormattingElements.insertMarker();break}case _.CAPTION:case _.COL:case _.COLGROUP:case _.TBODY:case _.TFOOT:case _.THEAD:case _.TR:{e.openElements.hasInTableScope(_.TR)&&(e.openElements.clearBackToTableRowContext(),e.openElements.pop(),e.insertionMode=j.IN_TABLE_BODY,Qf(e,t));break}default:nu(e,t)}}function DT(e,t){switch(t.tagID){case _.TR:{e.openElements.hasInTableScope(_.TR)&&(e.openElements.clearBackToTableRowContext(),e.openElements.pop(),e.insertionMode=j.IN_TABLE_BODY);break}case _.TABLE:{e.openElements.hasInTableScope(_.TR)&&(e.openElements.clearBackToTableRowContext(),e.openElements.pop(),e.insertionMode=j.IN_TABLE_BODY,mb(e,t));break}case _.TBODY:case _.TFOOT:case _.THEAD:{(e.openElements.hasInTableScope(t.tagID)||e.openElements.hasInTableScope(_.TR))&&(e.openElements.clearBackToTableRowContext(),e.openElements.pop(),e.insertionMode=j.IN_TABLE_BODY,mb(e,t));break}case _.BODY:case _.CAPTION:case _.COL:case _.COLGROUP:case _.HTML:case _.TD:case _.TH:break;default:m0(e,t)}}function rz(e,t){const n=t.tagID;IT.has(n)?(e.openElements.hasInTableScope(_.TD)||e.openElements.hasInTableScope(_.TH))&&(e._closeTableCell(),Zf(e,t)):Nr(e,t)}function az(e,t){const n=t.tagID;switch(n){case _.TD:case _.TH:{e.openElements.hasInTableScope(n)&&(e.openElements.generateImpliedEndTags(),e.openElements.popUntilTagNamePopped(n),e.activeFormattingElements.clearToLastMarker(),e.insertionMode=j.IN_ROW);break}case _.TABLE:case _.TBODY:case _.TFOOT:case _.THEAD:case _.TR:{e.openElements.hasInTableScope(n)&&(e._closeTableCell(),DT(e,t));break}case _.BODY:case _.CAPTION:case _.COL:case _.COLGROUP:case _.HTML:break;default:Xf(e,t)}}function LT(e,t){switch(t.tagID){case _.HTML:{Nr(e,t);break}case _.OPTION:{e.openElements.currentTagId===_.OPTION&&e.openElements.pop(),e._insertElement(t,Ae.HTML);break}case _.OPTGROUP:{e.openElements.currentTagId===_.OPTION&&e.openElements.pop(),e.openElements.currentTagId===_.OPTGROUP&&e.openElements.pop(),e._insertElement(t,Ae.HTML);break}case _.HR:{e.openElements.currentTagId===_.OPTION&&e.openElements.pop(),e.openElements.currentTagId===_.OPTGROUP&&e.openElements.pop(),e._appendElement(t,Ae.HTML),t.ackSelfClosing=!0;break}case _.INPUT:case _.KEYGEN:case _.TEXTAREA:case _.SELECT:{e.openElements.hasInSelectScope(_.SELECT)&&(e.openElements.popUntilTagNamePopped(_.SELECT),e._resetInsertionMode(),t.tagID!==_.SELECT&&e._processStartTag(t));break}case _.SCRIPT:case _.TEMPLATE:{Ja(e,t);break}}}function PT(e,t){switch(t.tagID){case _.OPTGROUP:{e.openElements.stackTop>0&&e.openElements.currentTagId===_.OPTION&&e.openElements.tagIDs[e.openElements.stackTop-1]===_.OPTGROUP&&e.openElements.pop(),e.openElements.currentTagId===_.OPTGROUP&&e.openElements.pop();break}case _.OPTION:{e.openElements.currentTagId===_.OPTION&&e.openElements.pop();break}case _.SELECT:{e.openElements.hasInSelectScope(_.SELECT)&&(e.openElements.popUntilTagNamePopped(_.SELECT),e._resetInsertionMode());break}case _.TEMPLATE:{Jl(e,t);break}}}function iz(e,t){const n=t.tagID;n===_.CAPTION||n===_.TABLE||n===_.TBODY||n===_.TFOOT||n===_.THEAD||n===_.TR||n===_.TD||n===_.TH?(e.openElements.popUntilTagNamePopped(_.SELECT),e._resetInsertionMode(),e._processStartTag(t)):LT(e,t)}function sz(e,t){const n=t.tagID;n===_.CAPTION||n===_.TABLE||n===_.TBODY||n===_.TFOOT||n===_.THEAD||n===_.TR||n===_.TD||n===_.TH?e.openElements.hasInTableScope(n)&&(e.openElements.popUntilTagNamePopped(_.SELECT),e._resetInsertionMode(),e.onEndTag(t)):PT(e,t)}function lz(e,t){switch(t.tagID){case _.BASE:case _.BASEFONT:case _.BGSOUND:case _.LINK:case _.META:case _.NOFRAMES:case _.SCRIPT:case _.STYLE:case _.TEMPLATE:case _.TITLE:{Ja(e,t);break}case _.CAPTION:case _.COLGROUP:case _.TBODY:case _.TFOOT:case _.THEAD:{e.tmplInsertionModeStack[0]=j.IN_TABLE,e.insertionMode=j.IN_TABLE,nu(e,t);break}case _.COL:{e.tmplInsertionModeStack[0]=j.IN_COLUMN_GROUP,e.insertionMode=j.IN_COLUMN_GROUP,D2(e,t);break}case _.TR:{e.tmplInsertionModeStack[0]=j.IN_TABLE_BODY,e.insertionMode=j.IN_TABLE_BODY,Qf(e,t);break}case _.TD:case _.TH:{e.tmplInsertionModeStack[0]=j.IN_ROW,e.insertionMode=j.IN_ROW,Zf(e,t);break}default:e.tmplInsertionModeStack[0]=j.IN_BODY,e.insertionMode=j.IN_BODY,Nr(e,t)}}function oz(e,t){t.tagID===_.TEMPLATE&&Jl(e,t)}function BT(e,t){e.openElements.tmplCount>0?(e.openElements.popUntilTagNamePopped(_.TEMPLATE),e.activeFormattingElements.clearToLastMarker(),e.tmplInsertionModeStack.shift(),e._resetInsertionMode(),e.onEof(t)):I2(e,t)}function uz(e,t){t.tagID===_.HTML?Nr(e,t):sf(e,t)}function UT(e,t){var n;if(t.tagID===_.HTML){if(e.fragmentContext||(e.insertionMode=j.AFTER_AFTER_BODY),e.options.sourceCodeLocationInfo&&e.openElements.tagIDs[0]===_.HTML){e._setEndLocation(e.openElements.items[0],t);const r=e.openElements.items[1];r&&!(!((n=e.treeAdapter.getNodeSourceCodeLocation(r))===null||n===void 0)&&n.endTag)&&e._setEndLocation(r,t)}}else sf(e,t)}function sf(e,t){e.insertionMode=j.IN_BODY,Wf(e,t)}function cz(e,t){switch(t.tagID){case _.HTML:{Nr(e,t);break}case _.FRAMESET:{e._insertElement(t,Ae.HTML);break}case _.FRAME:{e._appendElement(t,Ae.HTML),t.ackSelfClosing=!0;break}case _.NOFRAMES:{Ja(e,t);break}}}function dz(e,t){t.tagID===_.FRAMESET&&!e.openElements.isRootHtmlElementCurrent()&&(e.openElements.pop(),!e.fragmentContext&&e.openElements.currentTagId!==_.FRAMESET&&(e.insertionMode=j.AFTER_FRAMESET))}function hz(e,t){switch(t.tagID){case _.HTML:{Nr(e,t);break}case _.NOFRAMES:{Ja(e,t);break}}}function fz(e,t){t.tagID===_.HTML&&(e.insertionMode=j.AFTER_AFTER_FRAMESET)}function mz(e,t){t.tagID===_.HTML?Nr(e,t):jh(e,t)}function jh(e,t){e.insertionMode=j.IN_BODY,Wf(e,t)}function pz(e,t){switch(t.tagID){case _.HTML:{Nr(e,t);break}case _.NOFRAMES:{Ja(e,t);break}}}function gz(e,t){t.chars=Tn,e._insertCharacters(t)}function bz(e,t){e._insertCharacters(t),e.framesetOk=!1}function FT(e){for(;e.treeAdapter.getNamespaceURI(e.openElements.current)!==Ae.HTML&&e.openElements.currentTagId!==void 0&&!e._isIntegrationPoint(e.openElements.currentTagId,e.openElements.current);)e.openElements.pop()}function vz(e,t){if(IU(t))FT(e),e._startTagOutsideForeignContent(t);else{const n=e._getAdjustedCurrentElement(),r=e.treeAdapter.getNamespaceURI(n);r===Ae.MATHML?TT(t):r===Ae.SVG&&(DU(t),xT(t)),R2(t),t.selfClosing?e._appendElement(t,r):e._insertElement(t,r),t.ackSelfClosing=!0}}function yz(e,t){if(t.tagID===_.P||t.tagID===_.BR){FT(e),e._endTagOutsideForeignContent(t);return}for(let n=e.openElements.stackTop;n>0;n--){const r=e.openElements.items[n];if(e.treeAdapter.getNamespaceURI(r)===Ae.HTML){e._endTagOutsideForeignContent(t);break}const a=e.treeAdapter.getTagName(r);if(a.toLowerCase()===t.tagName){t.tagName=a,e.openElements.shortenToLength(n);break}}}ne.AREA,ne.BASE,ne.BASEFONT,ne.BGSOUND,ne.BR,ne.COL,ne.EMBED,ne.FRAME,ne.HR,ne.IMG,ne.INPUT,ne.KEYGEN,ne.LINK,ne.META,ne.PARAM,ne.SOURCE,ne.TRACK,ne.WBR;const _z=/<(\/?)(iframe|noembed|noframes|plaintext|script|style|textarea|title|xmp)(?=[\t\n\f\r />])/gi,Ez=new Set(["mdxFlowExpression","mdxJsxFlowElement","mdxJsxTextElement","mdxTextExpression","mdxjsEsm"]),FE={sourceCodeLocationInfo:!0,scriptingEnabled:!1};function zT(e,t){const n=Rz(e),r=R7("type",{handlers:{root:Tz,element:xz,text:Sz,comment:qT,doctype:wz,raw:kz},unknown:Cz}),a={parser:n?new PE(FE):PE.getFragmentParser(void 0,FE),handle(c){r(c,a)},stitches:!1,options:t||{}};r(e,a),bu(a,yi());const s=n?a.parser.document:a.parser.getFragment(),o=TB(s,{file:a.options.file});return a.stitches&&R0(o,"comment",function(c,d,f){const m=c;if(m.value.stitch&&f&&d!==void 0){const p=f.children;return p[d]=m.value.stitch,d}}),o.type==="root"&&o.children.length===1&&o.children[0].type===e.type?o.children[0]:o}function HT(e,t){let n=-1;if(e)for(;++n<e.length;)t.handle(e[n])}function Tz(e,t){HT(e.children,t)}function xz(e,t){Nz(e,t),HT(e.children,t),Oz(e,t)}function Sz(e,t){t.parser.tokenizer.state>4&&(t.parser.tokenizer.state=0);const n={type:Ct.CHARACTER,chars:e.value,location:L0(e)};bu(t,yi(e)),t.parser.currentToken=n,t.parser._processToken(t.parser.currentToken)}function wz(e,t){const n={type:Ct.DOCTYPE,name:"html",forceQuirks:!1,publicId:"",systemId:"",location:L0(e)};bu(t,yi(e)),t.parser.currentToken=n,t.parser._processToken(t.parser.currentToken)}function Az(e,t){t.stitches=!0;const n=Mz(e);if("children"in e&&"children"in n){const r=zT({type:"root",children:e.children},t.options);n.children=r.children}qT({type:"comment",value:{stitch:n}},t)}function qT(e,t){const n=e.value,r={type:Ct.COMMENT,data:n,location:L0(e)};bu(t,yi(e)),t.parser.currentToken=r,t.parser._processToken(t.parser.currentToken)}function kz(e,t){if(t.parser.tokenizer.preprocessor.html="",t.parser.tokenizer.preprocessor.pos=-1,t.parser.tokenizer.preprocessor.lastGapPos=-2,t.parser.tokenizer.preprocessor.gapStack=[],t.parser.tokenizer.preprocessor.skipNextNewLine=!1,t.parser.tokenizer.preprocessor.lastChunkWritten=!1,t.parser.tokenizer.preprocessor.endOfChunkHit=!1,t.parser.tokenizer.preprocessor.isEol=!1,jT(t,yi(e)),t.parser.tokenizer.write(t.options.tagfilter?e.value.replace(_z,"&lt;$1$2"):e.value,!1),t.parser.tokenizer._runParsingLoop(),t.parser.tokenizer.state===72||t.parser.tokenizer.state===78){t.parser.tokenizer.preprocessor.lastChunkWritten=!0;const n=t.parser.tokenizer._consume();t.parser.tokenizer._callState(n)}}function Cz(e,t){const n=e;if(t.options.passThrough&&t.options.passThrough.includes(n.type))Az(n,t);else{let r="";throw Ez.has(n.type)&&(r=". It looks like you are using MDX nodes with `hast-util-raw` (or `rehype-raw`). If you use this because you are using remark or rehype plugins that inject `'html'` nodes, then please raise an issue with that plugin, as its a bad and slow idea. If you use this because you are using markdown syntax, then you have to configure this utility (or plugin) to pass through these nodes (see `passThrough` in docs), but you can also migrate to use the MDX syntax"),new Error("Cannot compile `"+n.type+"` node"+r)}}function bu(e,t){jT(e,t);const n=e.parser.tokenizer.currentCharacterToken;n&&n.location&&(n.location.endLine=e.parser.tokenizer.preprocessor.line,n.location.endCol=e.parser.tokenizer.preprocessor.col+1,n.location.endOffset=e.parser.tokenizer.preprocessor.offset+1,e.parser.currentToken=n,e.parser._processToken(e.parser.currentToken)),e.parser.tokenizer.paused=!1,e.parser.tokenizer.inLoop=!1,e.parser.tokenizer.active=!1,e.parser.tokenizer.returnState=zn.DATA,e.parser.tokenizer.charRefCode=-1,e.parser.tokenizer.consumedAfterSnapshot=-1,e.parser.tokenizer.currentLocation=null,e.parser.tokenizer.currentCharacterToken=null,e.parser.tokenizer.currentToken=null,e.parser.tokenizer.currentAttr={name:"",value:""}}function jT(e,t){if(t&&t.offset!==void 0){const n={startLine:t.line,startCol:t.column,startOffset:t.offset,endLine:-1,endCol:-1,endOffset:-1};e.parser.tokenizer.preprocessor.lineStartPos=-t.column+1,e.parser.tokenizer.preprocessor.droppedBufferSize=t.offset,e.parser.tokenizer.preprocessor.line=t.line,e.parser.tokenizer.currentLocation=n}}function Nz(e,t){const n=e.tagName.toLowerCase();if(t.parser.tokenizer.state===zn.PLAINTEXT)return;bu(t,yi(e));const r=t.parser.openElements.current;let a="namespaceURI"in r?r.namespaceURI:di.html;a===di.html&&n==="svg"&&(a=di.svg);const s=BB({...e,children:[]},{space:a===di.svg?"svg":"html"}),o={type:Ct.START_TAG,tagName:n,tagID:gu(n),selfClosing:!1,ackSelfClosing:!1,attrs:"attrs"in s?s.attrs:[],location:L0(e)};t.parser.currentToken=o,t.parser._processToken(t.parser.currentToken),t.parser.tokenizer.lastStartTagName=n}function Oz(e,t){const n=e.tagName.toLowerCase();if(!t.parser.tokenizer.inForeignNode&&YB.includes(n)||t.parser.tokenizer.state===zn.PLAINTEXT)return;bu(t,$f(e));const r={type:Ct.END_TAG,tagName:n,tagID:gu(n),selfClosing:!1,ackSelfClosing:!1,attrs:[],location:L0(e)};t.parser.currentToken=r,t.parser._processToken(t.parser.currentToken),n===t.parser.tokenizer.lastStartTagName&&(t.parser.tokenizer.state===zn.RCDATA||t.parser.tokenizer.state===zn.RAWTEXT||t.parser.tokenizer.state===zn.SCRIPT_DATA)&&(t.parser.tokenizer.state=zn.DATA)}function Rz(e){const t=e.type==="root"?e.children[0]:e;return!!(t&&(t.type==="doctype"||t.type==="element"&&t.tagName.toLowerCase()==="html"))}function L0(e){const t=yi(e)||{line:void 0,column:void 0,offset:void 0},n=$f(e)||{line:void 0,column:void 0,offset:void 0};return{startLine:t.line,startCol:t.column,startOffset:t.offset,endLine:n.line,endCol:n.column,endOffset:n.offset}}function Mz(e){return"children"in e?tu({...e,children:[]}):tu(e)}function Iz(e){return function(t,n){return zT(t,{...e,file:n})}}const zE=function(e,t,n){const r=O0(n);if(!e||!e.type||!e.children)throw new Error("Expected parent node");if(typeof t=="number"){if(t<0||t===Number.POSITIVE_INFINITY)throw new Error("Expected positive finite number as index")}else if(t=e.children.indexOf(t),t<0)throw new Error("Expected child node or index");for(;++t<e.children.length;)if(r(e.children[t],t,e))return e.children[t]},eo=function(e){if(e==null)return Pz;if(typeof e=="string")return Lz(e);if(typeof e=="object")return Dz(e);if(typeof e=="function")return L2(e);throw new Error("Expected function, string, or array as `test`")};function Dz(e){const t=[];let n=-1;for(;++n<e.length;)t[n]=eo(e[n]);return L2(r);function r(...a){let s=-1;for(;++s<t.length;)if(t[s].apply(this,a))return!0;return!1}}function Lz(e){return L2(t);function t(n){return n.tagName===e}}function L2(e){return t;function t(n,r,a){return!!(Bz(n)&&e.call(this,n,typeof r=="number"?r:void 0,a||void 0))}}function Pz(e){return!!(e&&typeof e=="object"&&"type"in e&&e.type==="element"&&"tagName"in e&&typeof e.tagName=="string")}function Bz(e){return e!==null&&typeof e=="object"&&"type"in e&&"tagName"in e}const HE=/\n/g,qE=/[\t ]+/g,pb=eo("br"),jE=eo($z),Uz=eo("p"),$E=eo("tr"),Fz=eo(["datalist","head","noembed","noframes","noscript","rp","script","style","template","title",jz,Yz]),$T=eo(["address","article","aside","blockquote","body","caption","center","dd","dialog","dir","dl","dt","div","figure","figcaption","footer","form,","h1","h2","h3","h4","h5","h6","header","hgroup","hr","html","legend","li","listing","main","menu","nav","ol","p","plaintext","pre","section","ul","xmp"]);function YT(e,t){const n=t||{},r="children"in e?e.children:[],a=$T(e),s=KT(e,{whitespace:n.whitespace||"normal"}),o=[];(e.type==="text"||e.type==="comment")&&o.push(...VT(e,{breakBefore:!0,breakAfter:!0}));let c=-1;for(;++c<r.length;)o.push(...GT(r[c],e,{whitespace:s,breakBefore:c?void 0:a,breakAfter:c<r.length-1?pb(r[c+1]):a}));const d=[];let f;for(c=-1;++c<o.length;){const m=o[c];typeof m=="number"?f!==void 0&&m>f&&(f=m):m&&(f!==void 0&&f>-1&&d.push(`
`.repeat(f)||" "),f=-1,d.push(m))}return d.join("")}function GT(e,t,n){return e.type==="element"?zz(e,t,n):e.type==="text"?n.whitespace==="normal"?VT(e,n):Hz(e):[]}function zz(e,t,n){const r=KT(e,n),a=e.children||[];let s=-1,o=[];if(Fz(e))return o;let c,d;for(pb(e)||$E(e)&&zE(t,e,$E)?d=`
`:Uz(e)?(c=2,d=2):$T(e)&&(c=1,d=1);++s<a.length;)o=o.concat(GT(a[s],e,{whitespace:r,breakBefore:s?void 0:c,breakAfter:s<a.length-1?pb(a[s+1]):d}));return jE(e)&&zE(t,e,jE)&&o.push("	"),c&&o.unshift(c),d&&o.push(d),o}function VT(e,t){const n=String(e.value),r=[],a=[];let s=0;for(;s<=n.length;){HE.lastIndex=s;const d=HE.exec(n),f=d&&"index"in d?d.index:n.length;r.push(qz(n.slice(s,f).replace(/[\u061C\u200E\u200F\u202A-\u202E\u2066-\u2069]/g,""),s===0?t.breakBefore:!0,f===n.length?t.breakAfter:!0)),s=f+1}let o=-1,c;for(;++o<r.length;)r[o].charCodeAt(r[o].length-1)===8203||o<r.length-1&&r[o+1].charCodeAt(0)===8203?(a.push(r[o]),c=void 0):r[o]?(typeof c=="number"&&a.push(c),a.push(r[o]),c=0):(o===0||o===r.length-1)&&a.push(0);return a}function Hz(e){return[String(e.value)]}function qz(e,t,n){const r=[];let a=0,s;for(;a<e.length;){qE.lastIndex=a;const o=qE.exec(e);s=o?o.index:e.length,!a&&!s&&o&&!t&&r.push(""),a!==s&&r.push(e.slice(a,s)),a=o?s+o[0].length:s}return a!==s&&!n&&r.push(""),r.join(" ")}function KT(e,t){if(e.type==="element"){const n=e.properties||{};switch(e.tagName){case"listing":case"plaintext":case"xmp":return"pre";case"nobr":return"nowrap";case"pre":return n.wrap?"pre-wrap":"pre";case"td":case"th":return n.noWrap?"nowrap":t.whitespace;case"textarea":return"pre-wrap"}}return t.whitespace}function jz(e){return!!(e.properties||{}).hidden}function $z(e){return e.tagName==="td"||e.tagName==="th"}function Yz(e){return e.tagName==="dialog"&&!(e.properties||{}).open}function Gz(e){const t=e.regex,n=e.COMMENT("//","$",{contains:[{begin:/\\\n/}]}),r="decltype\\(auto\\)",a="[a-zA-Z_]\\w*::",o="(?!struct)("+r+"|"+t.optional(a)+"[a-zA-Z_]\\w*"+t.optional("<[^<>]+>")+")",c={className:"type",begin:"\\b[a-z\\d_]*_t\\b"},f={className:"string",variants:[{begin:'(u8?|U|L)?"',end:'"',illegal:"\\n",contains:[e.BACKSLASH_ESCAPE]},{begin:"(u8?|U|L)?'("+"\\\\(x[0-9A-Fa-f]{2}|u[0-9A-Fa-f]{4,8}|[0-7]{3}|\\S)"+"|.)",end:"'",illegal:"."},e.END_SAME_AS_BEGIN({begin:/(?:u8?|U|L)?R"([^()\\ ]{0,16})\(/,end:/\)([^()\\ ]{0,16})"/})]},m={className:"number",variants:[{begin:"[+-]?(?:(?:[0-9](?:'?[0-9])*\\.(?:[0-9](?:'?[0-9])*)?|\\.[0-9](?:'?[0-9])*)(?:[Ee][+-]?[0-9](?:'?[0-9])*)?|[0-9](?:'?[0-9])*[Ee][+-]?[0-9](?:'?[0-9])*|0[Xx](?:[0-9A-Fa-f](?:'?[0-9A-Fa-f])*(?:\\.(?:[0-9A-Fa-f](?:'?[0-9A-Fa-f])*)?)?|\\.[0-9A-Fa-f](?:'?[0-9A-Fa-f])*)[Pp][+-]?[0-9](?:'?[0-9])*)(?:[Ff](?:16|32|64|128)?|(BF|bf)16|[Ll]|)"},{begin:"[+-]?\\b(?:0[Bb][01](?:'?[01])*|0[Xx][0-9A-Fa-f](?:'?[0-9A-Fa-f])*|0(?:'?[0-7])*|[1-9](?:'?[0-9])*)(?:[Uu](?:LL?|ll?)|[Uu][Zz]?|(?:LL?|ll?)[Uu]?|[Zz][Uu]|)"}],relevance:0},p={className:"meta",begin:/#\s*[a-z]+\b/,end:/$/,keywords:{keyword:"if else elif endif define undef warning error line pragma _Pragma ifdef ifndef include"},contains:[{begin:/\\\n/,relevance:0},e.inherit(f,{className:"string"}),{className:"string",begin:/<.*?>/},n,e.C_BLOCK_COMMENT_MODE]},b={className:"title",begin:t.optional(a)+e.IDENT_RE,relevance:0},y=t.optional(a)+e.IDENT_RE+"\\s*\\(",T=["alignas","alignof","and","and_eq","asm","atomic_cancel","atomic_commit","atomic_noexcept","auto","bitand","bitor","break","case","catch","class","co_await","co_return","co_yield","compl","concept","const_cast|10","consteval","constexpr","constinit","continue","decltype","default","delete","do","dynamic_cast|10","else","enum","explicit","export","extern","false","final","for","friend","goto","if","import","inline","module","mutable","namespace","new","noexcept","not","not_eq","nullptr","operator","or","or_eq","override","private","protected","public","reflexpr","register","reinterpret_cast|10","requires","return","sizeof","static_assert","static_cast|10","struct","switch","synchronized","template","this","thread_local","throw","transaction_safe","transaction_safe_dynamic","true","try","typedef","typeid","typename","union","using","virtual","volatile","while","xor","xor_eq"],x=["bool","char","char16_t","char32_t","char8_t","double","float","int","long","short","void","wchar_t","unsigned","signed","const","static"],C=["any","auto_ptr","barrier","binary_semaphore","bitset","complex","condition_variable","condition_variable_any","counting_semaphore","deque","false_type","flat_map","flat_set","future","imaginary","initializer_list","istringstream","jthread","latch","lock_guard","multimap","multiset","mutex","optional","ostringstream","packaged_task","pair","promise","priority_queue","queue","recursive_mutex","recursive_timed_mutex","scoped_lock","set","shared_future","shared_lock","shared_mutex","shared_timed_mutex","shared_ptr","stack","string_view","stringstream","timed_mutex","thread","true_type","tuple","unique_lock","unique_ptr","unordered_map","unordered_multimap","unordered_multiset","unordered_set","variant","vector","weak_ptr","wstring","wstring_view"],w=["abort","abs","acos","apply","as_const","asin","atan","atan2","calloc","ceil","cerr","cin","clog","cos","cosh","cout","declval","endl","exchange","exit","exp","fabs","floor","fmod","forward","fprintf","fputs","free","frexp","fscanf","future","invoke","isalnum","isalpha","iscntrl","isdigit","isgraph","islower","isprint","ispunct","isspace","isupper","isxdigit","labs","launder","ldexp","log","log10","make_pair","make_shared","make_shared_for_overwrite","make_tuple","make_unique","malloc","memchr","memcmp","memcpy","memset","modf","move","pow","printf","putchar","puts","realloc","scanf","sin","sinh","snprintf","sprintf","sqrt","sscanf","std","stderr","stdin","stdout","strcat","strchr","strcmp","strcpy","strcspn","strlen","strncat","strncmp","strncpy","strpbrk","strrchr","strspn","strstr","swap","tan","tanh","terminate","to_underlying","tolower","toupper","vfprintf","visit","vprintf","vsprintf"],Y={type:x,keyword:T,literal:["NULL","false","nullopt","nullptr","true"],built_in:["_Pragma"],_type_hints:C},H={className:"function.dispatch",relevance:0,keywords:{_hint:w},begin:t.concat(/\b/,/(?!decltype)/,/(?!if)/,/(?!for)/,/(?!switch)/,/(?!while)/,e.IDENT_RE,t.lookahead(/(<[^<>]+>|)\s*\(/))},B=[H,p,c,n,e.C_BLOCK_COMMENT_MODE,m,f],ee={variants:[{begin:/=/,end:/;/},{begin:/\(/,end:/\)/},{beginKeywords:"new throw return else",end:/;/}],keywords:Y,contains:B.concat([{begin:/\(/,end:/\)/,keywords:Y,contains:B.concat(["self"]),relevance:0}]),relevance:0},Z={className:"function",begin:"("+o+"[\\*&\\s]+)+"+y,returnBegin:!0,end:/[{;=]/,excludeEnd:!0,keywords:Y,illegal:/[^\w\s\*&:<>.]/,contains:[{begin:r,keywords:Y,relevance:0},{begin:y,returnBegin:!0,contains:[b],relevance:0},{begin:/::/,relevance:0},{begin:/:/,endsWithParent:!0,contains:[f,m]},{relevance:0,match:/,/},{className:"params",begin:/\(/,end:/\)/,keywords:Y,relevance:0,contains:[n,e.C_BLOCK_COMMENT_MODE,f,m,c,{begin:/\(/,end:/\)/,keywords:Y,relevance:0,contains:["self",n,e.C_BLOCK_COMMENT_MODE,f,m,c]}]},c,n,e.C_BLOCK_COMMENT_MODE,p]};return{name:"C++",aliases:["cc","c++","h++","hpp","hh","hxx","cxx"],keywords:Y,illegal:"</",classNameAliases:{"function.dispatch":"built_in"},contains:[].concat(ee,Z,H,B,[p,{begin:"\\b(deque|list|queue|priority_queue|pair|stack|vector|map|set|bitset|multiset|multimap|unordered_map|unordered_set|unordered_multiset|unordered_multimap|array|tuple|optional|variant|function|flat_map|flat_set)\\s*<(?!<)",end:">",keywords:Y,contains:["self",c]},{begin:e.IDENT_RE+"::",keywords:Y},{match:[/\b(?:enum(?:\s+(?:class|struct))?|class|struct|union)/,/\s+/,/\w+/],className:{1:"keyword",3:"title.class"}}])}}function Vz(e){const t={type:["boolean","byte","word","String"],built_in:["KeyboardController","MouseController","SoftwareSerial","EthernetServer","EthernetClient","LiquidCrystal","RobotControl","GSMVoiceCall","EthernetUDP","EsploraTFT","HttpClient","RobotMotor","WiFiClient","GSMScanner","FileSystem","Scheduler","GSMServer","YunClient","YunServer","IPAddress","GSMClient","GSMModem","Keyboard","Ethernet","Console","GSMBand","Esplora","Stepper","Process","WiFiUDP","GSM_SMS","Mailbox","USBHost","Firmata","PImage","Client","Server","GSMPIN","FileIO","Bridge","Serial","EEPROM","Stream","Mouse","Audio","Servo","File","Task","GPRS","WiFi","Wire","TFT","GSM","SPI","SD"],_hints:["setup","loop","runShellCommandAsynchronously","analogWriteResolution","retrieveCallingNumber","printFirmwareVersion","analogReadResolution","sendDigitalPortPair","noListenOnLocalhost","readJoystickButton","setFirmwareVersion","readJoystickSwitch","scrollDisplayRight","getVoiceCallStatus","scrollDisplayLeft","writeMicroseconds","delayMicroseconds","beginTransmission","getSignalStrength","runAsynchronously","getAsynchronously","listenOnLocalhost","getCurrentCarrier","readAccelerometer","messageAvailable","sendDigitalPorts","lineFollowConfig","countryNameWrite","runShellCommand","readStringUntil","rewindDirectory","readTemperature","setClockDivider","readLightSensor","endTransmission","analogReference","detachInterrupt","countryNameRead","attachInterrupt","encryptionType","readBytesUntil","robotNameWrite","readMicrophone","robotNameRead","cityNameWrite","userNameWrite","readJoystickY","readJoystickX","mouseReleased","openNextFile","scanNetworks","noInterrupts","digitalWrite","beginSpeaker","mousePressed","isActionDone","mouseDragged","displayLogos","noAutoscroll","addParameter","remoteNumber","getModifiers","keyboardRead","userNameRead","waitContinue","processInput","parseCommand","printVersion","readNetworks","writeMessage","blinkVersion","cityNameRead","readMessage","setDataMode","parsePacket","isListening","setBitOrder","beginPacket","isDirectory","motorsWrite","drawCompass","digitalRead","clearScreen","serialEvent","rightToLeft","setTextSize","leftToRight","requestFrom","keyReleased","compassRead","analogWrite","interrupts","WiFiServer","disconnect","playMelody","parseFloat","autoscroll","getPINUsed","setPINUsed","setTimeout","sendAnalog","readSlider","analogRead","beginWrite","createChar","motorsStop","keyPressed","tempoWrite","readButton","subnetMask","debugPrint","macAddress","writeGreen","randomSeed","attachGPRS","readString","sendString","remotePort","releaseAll","mouseMoved","background","getXChange","getYChange","answerCall","getResult","voiceCall","endPacket","constrain","getSocket","writeJSON","getButton","available","connected","findUntil","readBytes","exitValue","readGreen","writeBlue","startLoop","IPAddress","isPressed","sendSysex","pauseMode","gatewayIP","setCursor","getOemKey","tuneWrite","noDisplay","loadImage","switchPIN","onRequest","onReceive","changePIN","playFile","noBuffer","parseInt","overflow","checkPIN","knobRead","beginTFT","bitClear","updateIR","bitWrite","position","writeRGB","highByte","writeRed","setSpeed","readBlue","noStroke","remoteIP","transfer","shutdown","hangCall","beginSMS","endWrite","attached","maintain","noCursor","checkReg","checkPUK","shiftOut","isValid","shiftIn","pulseIn","connect","println","localIP","pinMode","getIMEI","display","noBlink","process","getBand","running","beginSD","drawBMP","lowByte","setBand","release","bitRead","prepare","pointTo","readRed","setMode","noFill","remove","listen","stroke","detach","attach","noTone","exists","buffer","height","bitSet","circle","config","cursor","random","IRread","setDNS","endSMS","getKey","micros","millis","begin","print","write","ready","flush","width","isPIN","blink","clear","press","mkdir","rmdir","close","point","yield","image","BSSID","click","delay","read","text","move","peek","beep","rect","line","open","seek","fill","size","turn","stop","home","find","step","tone","sqrt","RSSI","SSID","end","bit","tan","cos","sin","pow","map","abs","max","min","get","run","put"],literal:["DIGITAL_MESSAGE","FIRMATA_STRING","ANALOG_MESSAGE","REPORT_DIGITAL","REPORT_ANALOG","INPUT_PULLUP","SET_PIN_MODE","INTERNAL2V56","SYSTEM_RESET","LED_BUILTIN","INTERNAL1V1","SYSEX_START","INTERNAL","EXTERNAL","DEFAULT","OUTPUT","INPUT","HIGH","LOW"]},n=Gz(e),r=n.keywords;return r.type=[...r.type,...t.type],r.literal=[...r.literal,...t.literal],r.built_in=[...r.built_in,...t.built_in],r._hints=t._hints,n.name="Arduino",n.aliases=["ino"],n.supersetOf="cpp",n}function Kz(e){const t=e.regex,n={},r={begin:/\$\{/,end:/\}/,contains:["self",{begin:/:-/,contains:[n]}]};Object.assign(n,{className:"variable",variants:[{begin:t.concat(/\$[\w\d#@][\w\d_]*/,"(?![\\w\\d])(?![$])")},r]});const a={className:"subst",begin:/\$\(/,end:/\)/,contains:[e.BACKSLASH_ESCAPE]},s=e.inherit(e.COMMENT(),{match:[/(^|\s)/,/#.*$/],scope:{2:"comment"}}),o={begin:/<<-?\s*(?=\w+)/,starts:{contains:[e.END_SAME_AS_BEGIN({begin:/(\w+)/,end:/(\w+)/,className:"string"})]}},c={className:"string",begin:/"/,end:/"/,contains:[e.BACKSLASH_ESCAPE,n,a]};a.contains.push(c);const d={match:/\\"/},f={className:"string",begin:/'/,end:/'/},m={match:/\\'/},p={begin:/\$?\(\(/,end:/\)\)/,contains:[{begin:/\d+#[0-9a-f]+/,className:"number"},e.NUMBER_MODE,n]},b=["fish","bash","zsh","sh","csh","ksh","tcsh","dash","scsh"],y=e.SHEBANG({binary:`(${b.join("|")})`,relevance:10}),T={className:"function",begin:/\w[\w\d_]*\s*\(\s*\)\s*\{/,returnBegin:!0,contains:[e.inherit(e.TITLE_MODE,{begin:/\w[\w\d_]*/})],relevance:0},x=["if","then","else","elif","fi","time","for","while","until","in","do","done","case","esac","coproc","function","select"],C=["true","false"],w={match:/(\/[a-z._-]+)+/},O=["break","cd","continue","eval","exec","exit","export","getopts","hash","pwd","readonly","return","shift","test","times","trap","umask","unset"],D=["alias","bind","builtin","caller","command","declare","echo","enable","help","let","local","logout","mapfile","printf","read","readarray","source","sudo","type","typeset","ulimit","unalias"],Y=["autoload","bg","bindkey","bye","cap","chdir","clone","comparguments","compcall","compctl","compdescribe","compfiles","compgroups","compquote","comptags","comptry","compvalues","dirs","disable","disown","echotc","echoti","emulate","fc","fg","float","functions","getcap","getln","history","integer","jobs","kill","limit","log","noglob","popd","print","pushd","pushln","rehash","sched","setcap","setopt","stat","suspend","ttyctl","unfunction","unhash","unlimit","unsetopt","vared","wait","whence","where","which","zcompile","zformat","zftp","zle","zmodload","zparseopts","zprof","zpty","zregexparse","zsocket","zstyle","ztcp"],H=["chcon","chgrp","chown","chmod","cp","dd","df","dir","dircolors","ln","ls","mkdir","mkfifo","mknod","mktemp","mv","realpath","rm","rmdir","shred","sync","touch","truncate","vdir","b2sum","base32","base64","cat","cksum","comm","csplit","cut","expand","fmt","fold","head","join","md5sum","nl","numfmt","od","paste","ptx","pr","sha1sum","sha224sum","sha256sum","sha384sum","sha512sum","shuf","sort","split","sum","tac","tail","tr","tsort","unexpand","uniq","wc","arch","basename","chroot","date","dirname","du","echo","env","expr","factor","groups","hostid","id","link","logname","nice","nohup","nproc","pathchk","pinky","printenv","printf","pwd","readlink","runcon","seq","sleep","stat","stdbuf","stty","tee","test","timeout","tty","uname","unlink","uptime","users","who","whoami","yes"];return{name:"Bash",aliases:["sh","zsh"],keywords:{$pattern:/\b[a-z][a-z0-9._-]+\b/,keyword:x,literal:C,built_in:[...O,...D,"set","shopt",...Y,...H]},contains:[y,e.SHEBANG(),T,p,s,o,w,c,d,f,m,n]}}function Wz(e){const t=e.regex,n=e.COMMENT("//","$",{contains:[{begin:/\\\n/}]}),r="decltype\\(auto\\)",a="[a-zA-Z_]\\w*::",o="("+r+"|"+t.optional(a)+"[a-zA-Z_]\\w*"+t.optional("<[^<>]+>")+")",c={className:"type",variants:[{begin:"\\b[a-z\\d_]*_t\\b"},{match:/\batomic_[a-z]{3,6}\b/}]},f={className:"string",variants:[{begin:'(u8?|U|L)?"',end:'"',illegal:"\\n",contains:[e.BACKSLASH_ESCAPE]},{begin:"(u8?|U|L)?'("+"\\\\(x[0-9A-Fa-f]{2}|u[0-9A-Fa-f]{4,8}|[0-7]{3}|\\S)"+"|.)",end:"'",illegal:"."},e.END_SAME_AS_BEGIN({begin:/(?:u8?|U|L)?R"([^()\\ ]{0,16})\(/,end:/\)([^()\\ ]{0,16})"/})]},m={className:"number",variants:[{match:/\b(0b[01']+)/},{match:/(-?)\b([\d']+(\.[\d']*)?|\.[\d']+)((ll|LL|l|L)(u|U)?|(u|U)(ll|LL|l|L)?|f|F|b|B)/},{match:/(-?)\b(0[xX][a-fA-F0-9]+(?:'[a-fA-F0-9]+)*(?:\.[a-fA-F0-9]*(?:'[a-fA-F0-9]*)*)?(?:[pP][-+]?[0-9]+)?(l|L)?(u|U)?)/},{match:/(-?)\b\d+(?:'\d+)*(?:\.\d*(?:'\d*)*)?(?:[eE][-+]?\d+)?/}],relevance:0},p={className:"meta",begin:/#\s*[a-z]+\b/,end:/$/,keywords:{keyword:"if else elif endif define undef warning error line pragma _Pragma ifdef ifndef elifdef elifndef include"},contains:[{begin:/\\\n/,relevance:0},e.inherit(f,{className:"string"}),{className:"string",begin:/<.*?>/},n,e.C_BLOCK_COMMENT_MODE]},b={className:"title",begin:t.optional(a)+e.IDENT_RE,relevance:0},y=t.optional(a)+e.IDENT_RE+"\\s*\\(",C={keyword:["asm","auto","break","case","continue","default","do","else","enum","extern","for","fortran","goto","if","inline","register","restrict","return","sizeof","typeof","typeof_unqual","struct","switch","typedef","union","volatile","while","_Alignas","_Alignof","_Atomic","_Generic","_Noreturn","_Static_assert","_Thread_local","alignas","alignof","noreturn","static_assert","thread_local","_Pragma"],type:["float","double","signed","unsigned","int","short","long","char","void","_Bool","_BitInt","_Complex","_Imaginary","_Decimal32","_Decimal64","_Decimal96","_Decimal128","_Decimal64x","_Decimal128x","_Float16","_Float32","_Float64","_Float128","_Float32x","_Float64x","_Float128x","const","static","constexpr","complex","bool","imaginary"],literal:"true false NULL",built_in:"std string wstring cin cout cerr clog stdin stdout stderr stringstream istringstream ostringstream auto_ptr deque list queue stack vector map set pair bitset multiset multimap unordered_set unordered_map unordered_multiset unordered_multimap priority_queue make_pair array shared_ptr abort terminate abs acos asin atan2 atan calloc ceil cosh cos exit exp fabs floor fmod fprintf fputs free frexp fscanf future isalnum isalpha iscntrl isdigit isgraph islower isprint ispunct isspace isupper isxdigit tolower toupper labs ldexp log10 log malloc realloc memchr memcmp memcpy memset modf pow printf putchar puts scanf sinh sin snprintf sprintf sqrt sscanf strcat strchr strcmp strcpy strcspn strlen strncat strncmp strncpy strpbrk strrchr strspn strstr tanh tan vfprintf vprintf vsprintf endl initializer_list unique_ptr"},w=[p,c,n,e.C_BLOCK_COMMENT_MODE,m,f],O={variants:[{begin:/=/,end:/;/},{begin:/\(/,end:/\)/},{beginKeywords:"new throw return else",end:/;/}],keywords:C,contains:w.concat([{begin:/\(/,end:/\)/,keywords:C,contains:w.concat(["self"]),relevance:0}]),relevance:0},D={begin:"("+o+"[\\*&\\s]+)+"+y,returnBegin:!0,end:/[{;=]/,excludeEnd:!0,keywords:C,illegal:/[^\w\s\*&:<>.]/,contains:[{begin:r,keywords:C,relevance:0},{begin:y,returnBegin:!0,contains:[e.inherit(b,{className:"title.function"})],relevance:0},{relevance:0,match:/,/},{className:"params",begin:/\(/,end:/\)/,keywords:C,relevance:0,contains:[n,e.C_BLOCK_COMMENT_MODE,f,m,c,{begin:/\(/,end:/\)/,keywords:C,relevance:0,contains:["self",n,e.C_BLOCK_COMMENT_MODE,f,m,c]}]},c,n,e.C_BLOCK_COMMENT_MODE,p]};return{name:"C",aliases:["h"],keywords:C,disableAutodetect:!0,illegal:"</",contains:[].concat(O,D,w,[p,{begin:e.IDENT_RE+"::",keywords:C},{className:"class",beginKeywords:"enum class struct union",end:/[{;:<>=]/,contains:[{beginKeywords:"final class struct"},e.TITLE_MODE]}]),exports:{preprocessor:p,strings:f,keywords:C}}}function Xz(e){const t=e.regex,n=e.COMMENT("//","$",{contains:[{begin:/\\\n/}]}),r="decltype\\(auto\\)",a="[a-zA-Z_]\\w*::",o="(?!struct)("+r+"|"+t.optional(a)+"[a-zA-Z_]\\w*"+t.optional("<[^<>]+>")+")",c={className:"type",begin:"\\b[a-z\\d_]*_t\\b"},f={className:"string",variants:[{begin:'(u8?|U|L)?"',end:'"',illegal:"\\n",contains:[e.BACKSLASH_ESCAPE]},{begin:"(u8?|U|L)?'("+"\\\\(x[0-9A-Fa-f]{2}|u[0-9A-Fa-f]{4,8}|[0-7]{3}|\\S)"+"|.)",end:"'",illegal:"."},e.END_SAME_AS_BEGIN({begin:/(?:u8?|U|L)?R"([^()\\ ]{0,16})\(/,end:/\)([^()\\ ]{0,16})"/})]},m={className:"number",variants:[{begin:"[+-]?(?:(?:[0-9](?:'?[0-9])*\\.(?:[0-9](?:'?[0-9])*)?|\\.[0-9](?:'?[0-9])*)(?:[Ee][+-]?[0-9](?:'?[0-9])*)?|[0-9](?:'?[0-9])*[Ee][+-]?[0-9](?:'?[0-9])*|0[Xx](?:[0-9A-Fa-f](?:'?[0-9A-Fa-f])*(?:\\.(?:[0-9A-Fa-f](?:'?[0-9A-Fa-f])*)?)?|\\.[0-9A-Fa-f](?:'?[0-9A-Fa-f])*)[Pp][+-]?[0-9](?:'?[0-9])*)(?:[Ff](?:16|32|64|128)?|(BF|bf)16|[Ll]|)"},{begin:"[+-]?\\b(?:0[Bb][01](?:'?[01])*|0[Xx][0-9A-Fa-f](?:'?[0-9A-Fa-f])*|0(?:'?[0-7])*|[1-9](?:'?[0-9])*)(?:[Uu](?:LL?|ll?)|[Uu][Zz]?|(?:LL?|ll?)[Uu]?|[Zz][Uu]|)"}],relevance:0},p={className:"meta",begin:/#\s*[a-z]+\b/,end:/$/,keywords:{keyword:"if else elif endif define undef warning error line pragma _Pragma ifdef ifndef include"},contains:[{begin:/\\\n/,relevance:0},e.inherit(f,{className:"string"}),{className:"string",begin:/<.*?>/},n,e.C_BLOCK_COMMENT_MODE]},b={className:"title",begin:t.optional(a)+e.IDENT_RE,relevance:0},y=t.optional(a)+e.IDENT_RE+"\\s*\\(",T=["alignas","alignof","and","and_eq","asm","atomic_cancel","atomic_commit","atomic_noexcept","auto","bitand","bitor","break","case","catch","class","co_await","co_return","co_yield","compl","concept","const_cast|10","consteval","constexpr","constinit","continue","decltype","default","delete","do","dynamic_cast|10","else","enum","explicit","export","extern","false","final","for","friend","goto","if","import","inline","module","mutable","namespace","new","noexcept","not","not_eq","nullptr","operator","or","or_eq","override","private","protected","public","reflexpr","register","reinterpret_cast|10","requires","return","sizeof","static_assert","static_cast|10","struct","switch","synchronized","template","this","thread_local","throw","transaction_safe","transaction_safe_dynamic","true","try","typedef","typeid","typename","union","using","virtual","volatile","while","xor","xor_eq"],x=["bool","char","char16_t","char32_t","char8_t","double","float","int","long","short","void","wchar_t","unsigned","signed","const","static"],C=["any","auto_ptr","barrier","binary_semaphore","bitset","complex","condition_variable","condition_variable_any","counting_semaphore","deque","false_type","flat_map","flat_set","future","imaginary","initializer_list","istringstream","jthread","latch","lock_guard","multimap","multiset","mutex","optional","ostringstream","packaged_task","pair","promise","priority_queue","queue","recursive_mutex","recursive_timed_mutex","scoped_lock","set","shared_future","shared_lock","shared_mutex","shared_timed_mutex","shared_ptr","stack","string_view","stringstream","timed_mutex","thread","true_type","tuple","unique_lock","unique_ptr","unordered_map","unordered_multimap","unordered_multiset","unordered_set","variant","vector","weak_ptr","wstring","wstring_view"],w=["abort","abs","acos","apply","as_const","asin","atan","atan2","calloc","ceil","cerr","cin","clog","cos","cosh","cout","declval","endl","exchange","exit","exp","fabs","floor","fmod","forward","fprintf","fputs","free","frexp","fscanf","future","invoke","isalnum","isalpha","iscntrl","isdigit","isgraph","islower","isprint","ispunct","isspace","isupper","isxdigit","labs","launder","ldexp","log","log10","make_pair","make_shared","make_shared_for_overwrite","make_tuple","make_unique","malloc","memchr","memcmp","memcpy","memset","modf","move","pow","printf","putchar","puts","realloc","scanf","sin","sinh","snprintf","sprintf","sqrt","sscanf","std","stderr","stdin","stdout","strcat","strchr","strcmp","strcpy","strcspn","strlen","strncat","strncmp","strncpy","strpbrk","strrchr","strspn","strstr","swap","tan","tanh","terminate","to_underlying","tolower","toupper","vfprintf","visit","vprintf","vsprintf"],Y={type:x,keyword:T,literal:["NULL","false","nullopt","nullptr","true"],built_in:["_Pragma"],_type_hints:C},H={className:"function.dispatch",relevance:0,keywords:{_hint:w},begin:t.concat(/\b/,/(?!decltype)/,/(?!if)/,/(?!for)/,/(?!switch)/,/(?!while)/,e.IDENT_RE,t.lookahead(/(<[^<>]+>|)\s*\(/))},B=[H,p,c,n,e.C_BLOCK_COMMENT_MODE,m,f],ee={variants:[{begin:/=/,end:/;/},{begin:/\(/,end:/\)/},{beginKeywords:"new throw return else",end:/;/}],keywords:Y,contains:B.concat([{begin:/\(/,end:/\)/,keywords:Y,contains:B.concat(["self"]),relevance:0}]),relevance:0},Z={className:"function",begin:"("+o+"[\\*&\\s]+)+"+y,returnBegin:!0,end:/[{;=]/,excludeEnd:!0,keywords:Y,illegal:/[^\w\s\*&:<>.]/,contains:[{begin:r,keywords:Y,relevance:0},{begin:y,returnBegin:!0,contains:[b],relevance:0},{begin:/::/,relevance:0},{begin:/:/,endsWithParent:!0,contains:[f,m]},{relevance:0,match:/,/},{className:"params",begin:/\(/,end:/\)/,keywords:Y,relevance:0,contains:[n,e.C_BLOCK_COMMENT_MODE,f,m,c,{begin:/\(/,end:/\)/,keywords:Y,relevance:0,contains:["self",n,e.C_BLOCK_COMMENT_MODE,f,m,c]}]},c,n,e.C_BLOCK_COMMENT_MODE,p]};return{name:"C++",aliases:["cc","c++","h++","hpp","hh","hxx","cxx"],keywords:Y,illegal:"</",classNameAliases:{"function.dispatch":"built_in"},contains:[].concat(ee,Z,H,B,[p,{begin:"\\b(deque|list|queue|priority_queue|pair|stack|vector|map|set|bitset|multiset|multimap|unordered_map|unordered_set|unordered_multiset|unordered_multimap|array|tuple|optional|variant|function|flat_map|flat_set)\\s*<(?!<)",end:">",keywords:Y,contains:["self",c]},{begin:e.IDENT_RE+"::",keywords:Y},{match:[/\b(?:enum(?:\s+(?:class|struct))?|class|struct|union)/,/\s+/,/\w+/],className:{1:"keyword",3:"title.class"}}])}}function Qz(e){const t=["bool","byte","char","decimal","delegate","double","dynamic","enum","float","int","long","nint","nuint","object","sbyte","short","string","ulong","uint","ushort"],n=["public","private","protected","static","internal","protected","abstract","async","extern","override","unsafe","virtual","new","sealed","partial"],r=["default","false","null","true"],a=["abstract","as","base","break","case","catch","class","const","continue","do","else","event","explicit","extern","finally","fixed","for","foreach","goto","if","implicit","in","interface","internal","is","lock","namespace","new","operator","out","override","params","private","protected","public","readonly","record","ref","return","scoped","sealed","sizeof","stackalloc","static","struct","switch","this","throw","try","typeof","unchecked","unsafe","using","virtual","void","volatile","while"],s=["add","alias","and","ascending","args","async","await","by","descending","dynamic","equals","file","from","get","global","group","init","into","join","let","nameof","not","notnull","on","or","orderby","partial","record","remove","required","scoped","select","set","unmanaged","value|0","var","when","where","with","yield"],o={keyword:a.concat(s),built_in:t,literal:r},c=e.inherit(e.TITLE_MODE,{begin:"[a-zA-Z](\\.?\\w)*"}),d={className:"number",variants:[{begin:"\\b(0b[01']+)"},{begin:"(-?)\\b([\\d']+(\\.[\\d']*)?|\\.[\\d']+)(u|U|l|L|ul|UL|f|F|b|B)"},{begin:"(-?)(\\b0[xX][a-fA-F0-9']+|(\\b[\\d']+(\\.[\\d']*)?|\\.[\\d']+)([eE][-+]?[\\d']+)?)"}],relevance:0},f={className:"string",begin:/"""("*)(?!")(.|\n)*?"""\1/,relevance:1},m={className:"string",begin:'@"',end:'"',contains:[{begin:'""'}]},p=e.inherit(m,{illegal:/\n/}),b={className:"subst",begin:/\{/,end:/\}/,keywords:o},y=e.inherit(b,{illegal:/\n/}),T={className:"string",begin:/\$"/,end:'"',illegal:/\n/,contains:[{begin:/\{\{/},{begin:/\}\}/},e.BACKSLASH_ESCAPE,y]},x={className:"string",begin:/\$@"/,end:'"',contains:[{begin:/\{\{/},{begin:/\}\}/},{begin:'""'},b]},C=e.inherit(x,{illegal:/\n/,contains:[{begin:/\{\{/},{begin:/\}\}/},{begin:'""'},y]});b.contains=[x,T,m,e.APOS_STRING_MODE,e.QUOTE_STRING_MODE,d,e.C_BLOCK_COMMENT_MODE],y.contains=[C,T,p,e.APOS_STRING_MODE,e.QUOTE_STRING_MODE,d,e.inherit(e.C_BLOCK_COMMENT_MODE,{illegal:/\n/})];const w={variants:[f,x,T,m,e.APOS_STRING_MODE,e.QUOTE_STRING_MODE]},O={begin:"<",end:">",contains:[{beginKeywords:"in out"},c]},D=e.IDENT_RE+"(<"+e.IDENT_RE+"(\\s*,\\s*"+e.IDENT_RE+")*>)?(\\[\\])?",Y={begin:"@"+e.IDENT_RE,relevance:0};return{name:"C#",aliases:["cs","c#"],keywords:o,illegal:/::/,contains:[e.COMMENT("///","$",{returnBegin:!0,contains:[{className:"doctag",variants:[{begin:"///",relevance:0},{begin:"<!--|-->"},{begin:"</?",end:">"}]}]}),e.C_LINE_COMMENT_MODE,e.C_BLOCK_COMMENT_MODE,{className:"meta",begin:"#",end:"$",keywords:{keyword:"if else elif endif define undef warning error line region endregion pragma checksum"}},w,d,{beginKeywords:"class interface",relevance:0,end:/[{;=]/,illegal:/[^\s:,]/,contains:[{beginKeywords:"where class"},c,O,e.C_LINE_COMMENT_MODE,e.C_BLOCK_COMMENT_MODE]},{beginKeywords:"namespace",relevance:0,end:/[{;=]/,illegal:/[^\s:]/,contains:[c,e.C_LINE_COMMENT_MODE,e.C_BLOCK_COMMENT_MODE]},{beginKeywords:"record",relevance:0,end:/[{;=]/,illegal:/[^\s:]/,contains:[c,O,e.C_LINE_COMMENT_MODE,e.C_BLOCK_COMMENT_MODE]},{className:"meta",begin:"^\\s*\\[(?=[\\w])",excludeBegin:!0,end:"\\]",excludeEnd:!0,contains:[{className:"string",begin:/"/,end:/"/}]},{beginKeywords:"new return throw await else",relevance:0},{className:"function",begin:"("+D+"\\s+)+"+e.IDENT_RE+"\\s*(<[^=]+>\\s*)?\\(",returnBegin:!0,end:/\s*[{;=]/,excludeEnd:!0,keywords:o,contains:[{beginKeywords:n.join(" "),relevance:0},{begin:e.IDENT_RE+"\\s*(<[^=]+>\\s*)?\\(",returnBegin:!0,contains:[e.TITLE_MODE,O],relevance:0},{match:/\(\)/},{className:"params",begin:/\(/,end:/\)/,excludeBegin:!0,excludeEnd:!0,keywords:o,relevance:0,contains:[w,d,e.C_BLOCK_COMMENT_MODE]},e.C_LINE_COMMENT_MODE,e.C_BLOCK_COMMENT_MODE]},Y]}}const Zz=e=>({IMPORTANT:{scope:"meta",begin:"!important"},BLOCK_COMMENT:e.C_BLOCK_COMMENT_MODE,HEXCOLOR:{scope:"number",begin:/#(([0-9a-fA-F]{3,4})|(([0-9a-fA-F]{2}){3,4}))\b/},FUNCTION_DISPATCH:{className:"built_in",begin:/[\w-]+(?=\()/},ATTRIBUTE_SELECTOR_MODE:{scope:"selector-attr",begin:/\[/,end:/\]/,illegal:"$",contains:[e.APOS_STRING_MODE,e.QUOTE_STRING_MODE]},CSS_NUMBER_MODE:{scope:"number",begin:e.NUMBER_RE+"(%|em|ex|ch|rem|vw|vh|vmin|vmax|cm|mm|in|pt|pc|px|deg|grad|rad|turn|s|ms|Hz|kHz|dpi|dpcm|dppx)?",relevance:0},CSS_VARIABLE:{className:"attr",begin:/--[A-Za-z_][A-Za-z0-9_-]*/}}),Jz=["a","abbr","address","article","aside","audio","b","blockquote","body","button","canvas","caption","cite","code","dd","del","details","dfn","div","dl","dt","em","fieldset","figcaption","figure","footer","form","h1","h2","h3","h4","h5","h6","header","hgroup","html","i","iframe","img","input","ins","kbd","label","legend","li","main","mark","menu","nav","object","ol","optgroup","option","p","picture","q","quote","samp","section","select","source","span","strong","summary","sup","table","tbody","td","textarea","tfoot","th","thead","time","tr","ul","var","video"],eH=["defs","g","marker","mask","pattern","svg","switch","symbol","feBlend","feColorMatrix","feComponentTransfer","feComposite","feConvolveMatrix","feDiffuseLighting","feDisplacementMap","feFlood","feGaussianBlur","feImage","feMerge","feMorphology","feOffset","feSpecularLighting","feTile","feTurbulence","linearGradient","radialGradient","stop","circle","ellipse","image","line","path","polygon","polyline","rect","text","use","textPath","tspan","foreignObject","clipPath"],tH=[...Jz,...eH],nH=["any-hover","any-pointer","aspect-ratio","color","color-gamut","color-index","device-aspect-ratio","device-height","device-width","display-mode","forced-colors","grid","height","hover","inverted-colors","monochrome","orientation","overflow-block","overflow-inline","pointer","prefers-color-scheme","prefers-contrast","prefers-reduced-motion","prefers-reduced-transparency","resolution","scan","scripting","update","width","min-width","max-width","min-height","max-height"].sort().reverse(),rH=["active","any-link","blank","checked","current","default","defined","dir","disabled","drop","empty","enabled","first","first-child","first-of-type","fullscreen","future","focus","focus-visible","focus-within","has","host","host-context","hover","indeterminate","in-range","invalid","is","lang","last-child","last-of-type","left","link","local-link","not","nth-child","nth-col","nth-last-child","nth-last-col","nth-last-of-type","nth-of-type","only-child","only-of-type","optional","out-of-range","past","placeholder-shown","read-only","read-write","required","right","root","scope","target","target-within","user-invalid","valid","visited","where"].sort().reverse(),aH=["after","backdrop","before","cue","cue-region","first-letter","first-line","grammar-error","marker","part","placeholder","selection","slotted","spelling-error"].sort().reverse(),iH=["accent-color","align-content","align-items","align-self","alignment-baseline","all","anchor-name","animation","animation-composition","animation-delay","animation-direction","animation-duration","animation-fill-mode","animation-iteration-count","animation-name","animation-play-state","animation-range","animation-range-end","animation-range-start","animation-timeline","animation-timing-function","appearance","aspect-ratio","backdrop-filter","backface-visibility","background","background-attachment","background-blend-mode","background-clip","background-color","background-image","background-origin","background-position","background-position-x","background-position-y","background-repeat","background-size","baseline-shift","block-size","border","border-block","border-block-color","border-block-end","border-block-end-color","border-block-end-style","border-block-end-width","border-block-start","border-block-start-color","border-block-start-style","border-block-start-width","border-block-style","border-block-width","border-bottom","border-bottom-color","border-bottom-left-radius","border-bottom-right-radius","border-bottom-style","border-bottom-width","border-collapse","border-color","border-end-end-radius","border-end-start-radius","border-image","border-image-outset","border-image-repeat","border-image-slice","border-image-source","border-image-width","border-inline","border-inline-color","border-inline-end","border-inline-end-color","border-inline-end-style","border-inline-end-width","border-inline-start","border-inline-start-color","border-inline-start-style","border-inline-start-width","border-inline-style","border-inline-width","border-left","border-left-color","border-left-style","border-left-width","border-radius","border-right","border-right-color","border-right-style","border-right-width","border-spacing","border-start-end-radius","border-start-start-radius","border-style","border-top","border-top-color","border-top-left-radius","border-top-right-radius","border-top-style","border-top-width","border-width","bottom","box-align","box-decoration-break","box-direction","box-flex","box-flex-group","box-lines","box-ordinal-group","box-orient","box-pack","box-shadow","box-sizing","break-after","break-before","break-inside","caption-side","caret-color","clear","clip","clip-path","clip-rule","color","color-interpolation","color-interpolation-filters","color-profile","color-rendering","color-scheme","column-count","column-fill","column-gap","column-rule","column-rule-color","column-rule-style","column-rule-width","column-span","column-width","columns","contain","contain-intrinsic-block-size","contain-intrinsic-height","contain-intrinsic-inline-size","contain-intrinsic-size","contain-intrinsic-width","container","container-name","container-type","content","content-visibility","counter-increment","counter-reset","counter-set","cue","cue-after","cue-before","cursor","cx","cy","direction","display","dominant-baseline","empty-cells","enable-background","field-sizing","fill","fill-opacity","fill-rule","filter","flex","flex-basis","flex-direction","flex-flow","flex-grow","flex-shrink","flex-wrap","float","flood-color","flood-opacity","flow","font","font-display","font-family","font-feature-settings","font-kerning","font-language-override","font-optical-sizing","font-palette","font-size","font-size-adjust","font-smooth","font-smoothing","font-stretch","font-style","font-synthesis","font-synthesis-position","font-synthesis-small-caps","font-synthesis-style","font-synthesis-weight","font-variant","font-variant-alternates","font-variant-caps","font-variant-east-asian","font-variant-emoji","font-variant-ligatures","font-variant-numeric","font-variant-position","font-variation-settings","font-weight","forced-color-adjust","gap","glyph-orientation-horizontal","glyph-orientation-vertical","grid","grid-area","grid-auto-columns","grid-auto-flow","grid-auto-rows","grid-column","grid-column-end","grid-column-start","grid-gap","grid-row","grid-row-end","grid-row-start","grid-template","grid-template-areas","grid-template-columns","grid-template-rows","hanging-punctuation","height","hyphenate-character","hyphenate-limit-chars","hyphens","icon","image-orientation","image-rendering","image-resolution","ime-mode","initial-letter","initial-letter-align","inline-size","inset","inset-area","inset-block","inset-block-end","inset-block-start","inset-inline","inset-inline-end","inset-inline-start","isolation","justify-content","justify-items","justify-self","kerning","left","letter-spacing","lighting-color","line-break","line-height","line-height-step","list-style","list-style-image","list-style-position","list-style-type","margin","margin-block","margin-block-end","margin-block-start","margin-bottom","margin-inline","margin-inline-end","margin-inline-start","margin-left","margin-right","margin-top","margin-trim","marker","marker-end","marker-mid","marker-start","marks","mask","mask-border","mask-border-mode","mask-border-outset","mask-border-repeat","mask-border-slice","mask-border-source","mask-border-width","mask-clip","mask-composite","mask-image","mask-mode","mask-origin","mask-position","mask-repeat","mask-size","mask-type","masonry-auto-flow","math-depth","math-shift","math-style","max-block-size","max-height","max-inline-size","max-width","min-block-size","min-height","min-inline-size","min-width","mix-blend-mode","nav-down","nav-index","nav-left","nav-right","nav-up","none","normal","object-fit","object-position","offset","offset-anchor","offset-distance","offset-path","offset-position","offset-rotate","opacity","order","orphans","outline","outline-color","outline-offset","outline-style","outline-width","overflow","overflow-anchor","overflow-block","overflow-clip-margin","overflow-inline","overflow-wrap","overflow-x","overflow-y","overlay","overscroll-behavior","overscroll-behavior-block","overscroll-behavior-inline","overscroll-behavior-x","overscroll-behavior-y","padding","padding-block","padding-block-end","padding-block-start","padding-bottom","padding-inline","padding-inline-end","padding-inline-start","padding-left","padding-right","padding-top","page","page-break-after","page-break-before","page-break-inside","paint-order","pause","pause-after","pause-before","perspective","perspective-origin","place-content","place-items","place-self","pointer-events","position","position-anchor","position-visibility","print-color-adjust","quotes","r","resize","rest","rest-after","rest-before","right","rotate","row-gap","ruby-align","ruby-position","scale","scroll-behavior","scroll-margin","scroll-margin-block","scroll-margin-block-end","scroll-margin-block-start","scroll-margin-bottom","scroll-margin-inline","scroll-margin-inline-end","scroll-margin-inline-start","scroll-margin-left","scroll-margin-right","scroll-margin-top","scroll-padding","scroll-padding-block","scroll-padding-block-end","scroll-padding-block-start","scroll-padding-bottom","scroll-padding-inline","scroll-padding-inline-end","scroll-padding-inline-start","scroll-padding-left","scroll-padding-right","scroll-padding-top","scroll-snap-align","scroll-snap-stop","scroll-snap-type","scroll-timeline","scroll-timeline-axis","scroll-timeline-name","scrollbar-color","scrollbar-gutter","scrollbar-width","shape-image-threshold","shape-margin","shape-outside","shape-rendering","speak","speak-as","src","stop-color","stop-opacity","stroke","stroke-dasharray","stroke-dashoffset","stroke-linecap","stroke-linejoin","stroke-miterlimit","stroke-opacity","stroke-width","tab-size","table-layout","text-align","text-align-all","text-align-last","text-anchor","text-combine-upright","text-decoration","text-decoration-color","text-decoration-line","text-decoration-skip","text-decoration-skip-ink","text-decoration-style","text-decoration-thickness","text-emphasis","text-emphasis-color","text-emphasis-position","text-emphasis-style","text-indent","text-justify","text-orientation","text-overflow","text-rendering","text-shadow","text-size-adjust","text-transform","text-underline-offset","text-underline-position","text-wrap","text-wrap-mode","text-wrap-style","timeline-scope","top","touch-action","transform","transform-box","transform-origin","transform-style","transition","transition-behavior","transition-delay","transition-duration","transition-property","transition-timing-function","translate","unicode-bidi","user-modify","user-select","vector-effect","vertical-align","view-timeline","view-timeline-axis","view-timeline-inset","view-timeline-name","view-transition-name","visibility","voice-balance","voice-duration","voice-family","voice-pitch","voice-range","voice-rate","voice-stress","voice-volume","white-space","white-space-collapse","widows","width","will-change","word-break","word-spacing","word-wrap","writing-mode","x","y","z-index","zoom"].sort().reverse();function sH(e){const t=e.regex,n=Zz(e),r={begin:/-(webkit|moz|ms|o)-(?=[a-z])/},a="and or not only",s=/@-?\w[\w]*(-\w+)*/,o="[a-zA-Z-][a-zA-Z0-9_-]*",c=[e.APOS_STRING_MODE,e.QUOTE_STRING_MODE];return{name:"CSS",case_insensitive:!0,illegal:/[=|'\$]/,keywords:{keyframePosition:"from to"},classNameAliases:{keyframePosition:"selector-tag"},contains:[n.BLOCK_COMMENT,r,n.CSS_NUMBER_MODE,{className:"selector-id",begin:/#[A-Za-z0-9_-]+/,relevance:0},{className:"selector-class",begin:"\\."+o,relevance:0},n.ATTRIBUTE_SELECTOR_MODE,{className:"selector-pseudo",variants:[{begin:":("+rH.join("|")+")"},{begin:":(:)?("+aH.join("|")+")"}]},n.CSS_VARIABLE,{className:"attribute",begin:"\\b("+iH.join("|")+")\\b"},{begin:/:/,end:/[;}{]/,contains:[n.BLOCK_COMMENT,n.HEXCOLOR,n.IMPORTANT,n.CSS_NUMBER_MODE,...c,{begin:/(url|data-uri)\(/,end:/\)/,relevance:0,keywords:{built_in:"url data-uri"},contains:[...c,{className:"string",begin:/[^)]/,endsWithParent:!0,excludeEnd:!0}]},n.FUNCTION_DISPATCH]},{begin:t.lookahead(/@/),end:"[{;]",relevance:0,illegal:/:/,contains:[{className:"keyword",begin:s},{begin:/\s/,endsWithParent:!0,excludeEnd:!0,relevance:0,keywords:{$pattern:/[a-z-]+/,keyword:a,attribute:nH.join(" ")},contains:[{begin:/[a-z-]+(?=:)/,className:"attribute"},...c,n.CSS_NUMBER_MODE]}]},{className:"selector-tag",begin:"\\b("+tH.join("|")+")\\b"}]}}function lH(e){const t=e.regex;return{name:"Diff",aliases:["patch"],contains:[{className:"meta",relevance:10,match:t.either(/^@@ +-\d+,\d+ +\+\d+,\d+ +@@/,/^\*\*\* +\d+,\d+ +\*\*\*\*$/,/^--- +\d+,\d+ +----$/)},{className:"comment",variants:[{begin:t.either(/Index: /,/^index/,/={3,}/,/^-{3}/,/^\*{3} /,/^\+{3}/,/^diff --git/),end:/$/},{match:/^\*{15}$/}]},{className:"addition",begin:/^\+/,end:/$/},{className:"deletion",begin:/^-/,end:/$/},{className:"addition",begin:/^!/,end:/$/}]}}function oH(e){const s={keyword:["break","case","chan","const","continue","default","defer","else","fallthrough","for","func","go","goto","if","import","interface","map","package","range","return","select","struct","switch","type","var"],type:["bool","byte","complex64","complex128","error","float32","float64","int8","int16","int32","int64","string","uint8","uint16","uint32","uint64","int","uint","uintptr","rune"],literal:["true","false","iota","nil"],built_in:["append","cap","close","complex","copy","imag","len","make","new","panic","print","println","real","recover","delete"]};return{name:"Go",aliases:["golang"],keywords:s,illegal:"</",contains:[e.C_LINE_COMMENT_MODE,e.C_BLOCK_COMMENT_MODE,{className:"string",variants:[e.QUOTE_STRING_MODE,e.APOS_STRING_MODE,{begin:"`",end:"`"}]},{className:"number",variants:[{match:/-?\b0[xX]\.[a-fA-F0-9](_?[a-fA-F0-9])*[pP][+-]?\d(_?\d)*i?/,relevance:0},{match:/-?\b0[xX](_?[a-fA-F0-9])+((\.([a-fA-F0-9](_?[a-fA-F0-9])*)?)?[pP][+-]?\d(_?\d)*)?i?/,relevance:0},{match:/-?\b0[oO](_?[0-7])*i?/,relevance:0},{match:/-?\.\d(_?\d)*([eE][+-]?\d(_?\d)*)?i?/,relevance:0},{match:/-?\b\d(_?\d)*(\.(\d(_?\d)*)?)?([eE][+-]?\d(_?\d)*)?i?/,relevance:0}]},{begin:/:=/},{className:"function",beginKeywords:"func",end:"\\s*(\\{|$)",excludeEnd:!0,contains:[e.TITLE_MODE,{className:"params",begin:/\(/,end:/\)/,endsParent:!0,keywords:s,illegal:/["']/}]}]}}function uH(e){const t=e.regex,n=/[_A-Za-z][_0-9A-Za-z]*/;return{name:"GraphQL",aliases:["gql"],case_insensitive:!0,disableAutodetect:!1,keywords:{keyword:["query","mutation","subscription","type","input","schema","directive","interface","union","scalar","fragment","enum","on"],literal:["true","false","null"]},contains:[e.HASH_COMMENT_MODE,e.QUOTE_STRING_MODE,e.NUMBER_MODE,{scope:"punctuation",match:/[.]{3}/,relevance:0},{scope:"punctuation",begin:/[\!\(\)\:\=\[\]\{\|\}]{1}/,relevance:0},{scope:"variable",begin:/\$/,end:/\W/,excludeEnd:!0,relevance:0},{scope:"meta",match:/@\w+/,excludeEnd:!0},{scope:"symbol",begin:t.concat(n,t.lookahead(/\s*:/)),relevance:0}],illegal:[/[;<']/,/BEGIN/]}}function cH(e){const t=e.regex,n={className:"number",relevance:0,variants:[{begin:/([+-]+)?[\d]+_[\d_]+/},{begin:e.NUMBER_RE}]},r=e.COMMENT();r.variants=[{begin:/;/,end:/$/},{begin:/#/,end:/$/}];const a={className:"variable",variants:[{begin:/\$[\w\d"][\w\d_]*/},{begin:/\$\{(.*?)\}/}]},s={className:"literal",begin:/\bon|off|true|false|yes|no\b/},o={className:"string",contains:[e.BACKSLASH_ESCAPE],variants:[{begin:"'''",end:"'''",relevance:10},{begin:'"""',end:'"""',relevance:10},{begin:'"',end:'"'},{begin:"'",end:"'"}]},c={begin:/\[/,end:/\]/,contains:[r,s,a,o,n,"self"],relevance:0},d=/[A-Za-z0-9_-]+/,f=/"(\\"|[^"])*"/,m=/'[^']*'/,p=t.either(d,f,m),b=t.concat(p,"(\\s*\\.\\s*",p,")*",t.lookahead(/\s*=\s*[^#\s]/));return{name:"TOML, also INI",aliases:["toml"],case_insensitive:!0,illegal:/\S/,contains:[r,{className:"section",begin:/\[+/,end:/\]+/},{begin:b,className:"attr",starts:{end:/$/,contains:[r,c,s,a,o,n]}}]}}var jo="[0-9](_*[0-9])*",hh=`\\.(${jo})`,fh="[0-9a-fA-F](_*[0-9a-fA-F])*",YE={className:"number",variants:[{begin:`(\\b(${jo})((${hh})|\\.)?|(${hh}))[eE][+-]?(${jo})[fFdD]?\\b`},{begin:`\\b(${jo})((${hh})[fFdD]?\\b|\\.([fFdD]\\b)?)`},{begin:`(${hh})[fFdD]?\\b`},{begin:`\\b(${jo})[fFdD]\\b`},{begin:`\\b0[xX]((${fh})\\.?|(${fh})?\\.(${fh}))[pP][+-]?(${jo})[fFdD]?\\b`},{begin:"\\b(0|[1-9](_*[0-9])*)[lL]?\\b"},{begin:`\\b0[xX](${fh})[lL]?\\b`},{begin:"\\b0(_*[0-7])*[lL]?\\b"},{begin:"\\b0[bB][01](_*[01])*[lL]?\\b"}],relevance:0};function WT(e,t,n){return n===-1?"":e.replace(t,r=>WT(e,t,n-1))}function dH(e){const t=e.regex,n="[À-ʸa-zA-Z_$][À-ʸa-zA-Z_$0-9]*",r=n+WT("(?:<"+n+"~~~(?:\\s*,\\s*"+n+"~~~)*>)?",/~~~/g,2),d={keyword:["synchronized","abstract","private","var","static","if","const ","for","while","strictfp","finally","protected","import","native","final","void","enum","else","break","transient","catch","instanceof","volatile","case","assert","package","default","public","try","switch","continue","throws","protected","public","private","module","requires","exports","do","sealed","yield","permits","goto","when"],literal:["false","true","null"],type:["char","boolean","long","float","int","byte","short","double"],built_in:["super","this"]},f={className:"meta",begin:"@"+n,contains:[{begin:/\(/,end:/\)/,contains:["self"]}]},m={className:"params",begin:/\(/,end:/\)/,keywords:d,relevance:0,contains:[e.C_BLOCK_COMMENT_MODE],endsParent:!0};return{name:"Java",aliases:["jsp"],keywords:d,illegal:/<\/|#/,contains:[e.COMMENT("/\\*\\*","\\*/",{relevance:0,contains:[{begin:/\w+@/,relevance:0},{className:"doctag",begin:"@[A-Za-z]+"}]}),{begin:/import java\.[a-z]+\./,keywords:"import",relevance:2},e.C_LINE_COMMENT_MODE,e.C_BLOCK_COMMENT_MODE,{begin:/"""/,end:/"""/,className:"string",contains:[e.BACKSLASH_ESCAPE]},e.APOS_STRING_MODE,e.QUOTE_STRING_MODE,{match:[/\b(?:class|interface|enum|extends|implements|new)/,/\s+/,n],className:{1:"keyword",3:"title.class"}},{match:/non-sealed/,scope:"keyword"},{begin:[t.concat(/(?!else)/,n),/\s+/,n,/\s+/,/=(?!=)/],className:{1:"type",3:"variable",5:"operator"}},{begin:[/record/,/\s+/,n],className:{1:"keyword",3:"title.class"},contains:[m,e.C_LINE_COMMENT_MODE,e.C_BLOCK_COMMENT_MODE]},{beginKeywords:"new throw return else",relevance:0},{begin:["(?:"+r+"\\s+)",e.UNDERSCORE_IDENT_RE,/\s*(?=\()/],className:{2:"title.function"},keywords:d,contains:[{className:"params",begin:/\(/,end:/\)/,keywords:d,relevance:0,contains:[f,e.APOS_STRING_MODE,e.QUOTE_STRING_MODE,YE,e.C_BLOCK_COMMENT_MODE]},e.C_LINE_COMMENT_MODE,e.C_BLOCK_COMMENT_MODE]},YE,f]}}const GE="[A-Za-z$_][0-9A-Za-z$_]*",hH=["as","in","of","if","for","while","finally","var","new","function","do","return","void","else","break","catch","instanceof","with","throw","case","default","try","switch","continue","typeof","delete","let","yield","const","class","debugger","async","await","static","import","from","export","extends","using"],fH=["true","false","null","undefined","NaN","Infinity"],XT=["Object","Function","Boolean","Symbol","Math","Date","Number","BigInt","String","RegExp","Array","Float32Array","Float64Array","Int8Array","Uint8Array","Uint8ClampedArray","Int16Array","Int32Array","Uint16Array","Uint32Array","BigInt64Array","BigUint64Array","Set","Map","WeakSet","WeakMap","ArrayBuffer","SharedArrayBuffer","Atomics","DataView","JSON","Promise","Generator","GeneratorFunction","AsyncFunction","Reflect","Proxy","Intl","WebAssembly"],QT=["Error","EvalError","InternalError","RangeError","ReferenceError","SyntaxError","TypeError","URIError"],ZT=["setInterval","setTimeout","clearInterval","clearTimeout","require","exports","eval","isFinite","isNaN","parseFloat","parseInt","decodeURI","decodeURIComponent","encodeURI","encodeURIComponent","escape","unescape"],mH=["arguments","this","super","console","window","document","localStorage","sessionStorage","module","global"],pH=[].concat(ZT,XT,QT);function gH(e){const t=e.regex,n=(pe,{after:Be})=>{const k="</"+pe[0].slice(1);return pe.input.indexOf(k,Be)!==-1},r=GE,a={begin:"<>",end:"</>"},s=/<[A-Za-z0-9\\._:-]+\s*\/>/,o={begin:/<[A-Za-z0-9\\._:-]+/,end:/\/[A-Za-z0-9\\._:-]+>|\/>/,isTrulyOpeningTag:(pe,Be)=>{const k=pe[0].length+pe.index,ie=pe.input[k];if(ie==="<"||ie===","){Be.ignoreMatch();return}ie===">"&&(n(pe,{after:k})||Be.ignoreMatch());let ge;const L=pe.input.substring(k);if(ge=L.match(/^\s*=/)){Be.ignoreMatch();return}if((ge=L.match(/^\s+extends\s+/))&&ge.index===0){Be.ignoreMatch();return}}},c={$pattern:GE,keyword:hH,literal:fH,built_in:pH,"variable.language":mH},d="[0-9](_?[0-9])*",f=`\\.(${d})`,m="0|[1-9](_?[0-9])*|0[0-7]*[89][0-9]*",p={className:"number",variants:[{begin:`(\\b(${m})((${f})|\\.)?|(${f}))[eE][+-]?(${d})\\b`},{begin:`\\b(${m})\\b((${f})\\b|\\.)?|(${f})\\b`},{begin:"\\b(0|[1-9](_?[0-9])*)n\\b"},{begin:"\\b0[xX][0-9a-fA-F](_?[0-9a-fA-F])*n?\\b"},{begin:"\\b0[bB][0-1](_?[0-1])*n?\\b"},{begin:"\\b0[oO][0-7](_?[0-7])*n?\\b"},{begin:"\\b0[0-7]+n?\\b"}],relevance:0},b={className:"subst",begin:"\\$\\{",end:"\\}",keywords:c,contains:[]},y={begin:".?html`",end:"",starts:{end:"`",returnEnd:!1,contains:[e.BACKSLASH_ESCAPE,b],subLanguage:"xml"}},T={begin:".?css`",end:"",starts:{end:"`",returnEnd:!1,contains:[e.BACKSLASH_ESCAPE,b],subLanguage:"css"}},x={begin:".?gql`",end:"",starts:{end:"`",returnEnd:!1,contains:[e.BACKSLASH_ESCAPE,b],subLanguage:"graphql"}},C={className:"string",begin:"`",end:"`",contains:[e.BACKSLASH_ESCAPE,b]},O={className:"comment",variants:[e.COMMENT(/\/\*\*(?!\/)/,"\\*/",{relevance:0,contains:[{begin:"(?=@[A-Za-z]+)",relevance:0,contains:[{className:"doctag",begin:"@[A-Za-z]+"},{className:"type",begin:"\\{",end:"\\}",excludeEnd:!0,excludeBegin:!0,relevance:0},{className:"variable",begin:r+"(?=\\s*(-)|$)",endsParent:!0,relevance:0},{begin:/(?=[^\n])\s/,relevance:0}]}]}),e.C_BLOCK_COMMENT_MODE,e.C_LINE_COMMENT_MODE]},D=[e.APOS_STRING_MODE,e.QUOTE_STRING_MODE,y,T,x,C,{match:/\$\d+/},p];b.contains=D.concat({begin:/\{/,end:/\}/,keywords:c,contains:["self"].concat(D)});const Y=[].concat(O,b.contains),H=Y.concat([{begin:/(\s*)\(/,end:/\)/,keywords:c,contains:["self"].concat(Y)}]),B={className:"params",begin:/(\s*)\(/,end:/\)/,excludeBegin:!0,excludeEnd:!0,keywords:c,contains:H},ee={variants:[{match:[/class/,/\s+/,r,/\s+/,/extends/,/\s+/,t.concat(r,"(",t.concat(/\./,r),")*")],scope:{1:"keyword",3:"title.class",5:"keyword",7:"title.class.inherited"}},{match:[/class/,/\s+/,r],scope:{1:"keyword",3:"title.class"}}]},Z={relevance:0,match:t.either(/\bJSON/,/\b[A-Z][a-z]+([A-Z][a-z]*|\d)*/,/\b[A-Z]{2,}([A-Z][a-z]+|\d)+([A-Z][a-z]*)*/,/\b[A-Z]{2,}[a-z]+([A-Z][a-z]+|\d)*([A-Z][a-z]*)*/),className:"title.class",keywords:{_:[...XT,...QT]}},Q={label:"use_strict",className:"meta",relevance:10,begin:/^\s*['"]use (strict|asm)['"]/},G={variants:[{match:[/function/,/\s+/,r,/(?=\s*\()/]},{match:[/function/,/\s*(?=\()/]}],className:{1:"keyword",3:"title.function"},label:"func.def",contains:[B],illegal:/%/},se={relevance:0,match:/\b[A-Z][A-Z_0-9]+\b/,className:"variable.constant"};function oe(pe){return t.concat("(?!",pe.join("|"),")")}const ye={match:t.concat(/\b/,oe([...ZT,"super","import"].map(pe=>`${pe}\\s*\\(`)),r,t.lookahead(/\s*\(/)),className:"title.function",relevance:0},fe={begin:t.concat(/\./,t.lookahead(t.concat(r,/(?![0-9A-Za-z$_(])/))),end:r,excludeBegin:!0,keywords:"prototype",className:"property",relevance:0},de={match:[/get|set/,/\s+/,r,/(?=\()/],className:{1:"keyword",3:"title.function"},contains:[{begin:/\(\)/},B]},V="(\\([^()]*(\\([^()]*(\\([^()]*\\)[^()]*)*\\)[^()]*)*\\)|"+e.UNDERSCORE_IDENT_RE+")\\s*=>",me={match:[/const|var|let/,/\s+/,r,/\s*/,/=\s*/,/(async\s*)?/,t.lookahead(V)],keywords:"async",className:{1:"keyword",3:"title.function"},contains:[B]};return{name:"JavaScript",aliases:["js","jsx","mjs","cjs"],keywords:c,exports:{PARAMS_CONTAINS:H,CLASS_REFERENCE:Z},illegal:/#(?![$_A-z])/,contains:[e.SHEBANG({label:"shebang",binary:"node",relevance:5}),Q,e.APOS_STRING_MODE,e.QUOTE_STRING_MODE,y,T,x,C,O,{match:/\$\d+/},p,Z,{scope:"attr",match:r+t.lookahead(":"),relevance:0},me,{begin:"("+e.RE_STARTERS_RE+"|\\b(case|return|throw)\\b)\\s*",keywords:"return throw case",relevance:0,contains:[O,e.REGEXP_MODE,{className:"function",begin:V,returnBegin:!0,end:"\\s*=>",contains:[{className:"params",variants:[{begin:e.UNDERSCORE_IDENT_RE,relevance:0},{className:null,begin:/\(\s*\)/,skip:!0},{begin:/(\s*)\(/,end:/\)/,excludeBegin:!0,excludeEnd:!0,keywords:c,contains:H}]}]},{begin:/,/,relevance:0},{match:/\s+/,relevance:0},{variants:[{begin:a.begin,end:a.end},{match:s},{begin:o.begin,"on:begin":o.isTrulyOpeningTag,end:o.end}],subLanguage:"xml",contains:[{begin:o.begin,end:o.end,skip:!0,contains:["self"]}]}]},G,{beginKeywords:"while if switch catch for"},{begin:"\\b(?!function)"+e.UNDERSCORE_IDENT_RE+"\\([^()]*(\\([^()]*(\\([^()]*\\)[^()]*)*\\)[^()]*)*\\)\\s*\\{",returnBegin:!0,label:"func.def",contains:[B,e.inherit(e.TITLE_MODE,{begin:r,className:"title.function"})]},{match:/\.\.\./,relevance:0},fe,{match:"\\$"+r,relevance:0},{match:[/\bconstructor(?=\s*\()/],className:{1:"title.function"},contains:[B]},ye,se,ee,de,{match:/\$[(.]/}]}}function bH(e){const t={className:"attr",begin:/"(\\.|[^\\"\r\n])*"(?=\s*:)/,relevance:1.01},n={match:/[{}[\],:]/,className:"punctuation",relevance:0},r=["true","false","null"],a={scope:"literal",beginKeywords:r.join(" ")};return{name:"JSON",aliases:["jsonc"],keywords:{literal:r},contains:[t,n,e.QUOTE_STRING_MODE,a,e.C_NUMBER_MODE,e.C_LINE_COMMENT_MODE,e.C_BLOCK_COMMENT_MODE],illegal:"\\S"}}var $o="[0-9](_*[0-9])*",mh=`\\.(${$o})`,ph="[0-9a-fA-F](_*[0-9a-fA-F])*",vH={className:"number",variants:[{begin:`(\\b(${$o})((${mh})|\\.)?|(${mh}))[eE][+-]?(${$o})[fFdD]?\\b`},{begin:`\\b(${$o})((${mh})[fFdD]?\\b|\\.([fFdD]\\b)?)`},{begin:`(${mh})[fFdD]?\\b`},{begin:`\\b(${$o})[fFdD]\\b`},{begin:`\\b0[xX]((${ph})\\.?|(${ph})?\\.(${ph}))[pP][+-]?(${$o})[fFdD]?\\b`},{begin:"\\b(0|[1-9](_*[0-9])*)[lL]?\\b"},{begin:`\\b0[xX](${ph})[lL]?\\b`},{begin:"\\b0(_*[0-7])*[lL]?\\b"},{begin:"\\b0[bB][01](_*[01])*[lL]?\\b"}],relevance:0};function yH(e){const t={keyword:"abstract as val var vararg get set class object open private protected public noinline crossinline dynamic final enum if else do while for when throw try catch finally import package is in fun override companion reified inline lateinit init interface annotation data sealed internal infix operator out by constructor super tailrec where const inner suspend typealias external expect actual",built_in:"Byte Short Char Int Long Boolean Float Double Void Unit Nothing",literal:"true false null"},n={className:"keyword",begin:/\b(break|continue|return|this)\b/,starts:{contains:[{className:"symbol",begin:/@\w+/}]}},r={className:"symbol",begin:e.UNDERSCORE_IDENT_RE+"@"},a={className:"subst",begin:/\$\{/,end:/\}/,contains:[e.C_NUMBER_MODE]},s={className:"variable",begin:"\\$"+e.UNDERSCORE_IDENT_RE},o={className:"string",variants:[{begin:'"""',end:'"""(?=[^"])',contains:[s,a]},{begin:"'",end:"'",illegal:/\n/,contains:[e.BACKSLASH_ESCAPE]},{begin:'"',end:'"',illegal:/\n/,contains:[e.BACKSLASH_ESCAPE,s,a]}]};a.contains.push(o);const c={className:"meta",begin:"@(?:file|property|field|get|set|receiver|param|setparam|delegate)\\s*:(?:\\s*"+e.UNDERSCORE_IDENT_RE+")?"},d={className:"meta",begin:"@"+e.UNDERSCORE_IDENT_RE,contains:[{begin:/\(/,end:/\)/,contains:[e.inherit(o,{className:"string"}),"self"]}]},f=vH,m=e.COMMENT("/\\*","\\*/",{contains:[e.C_BLOCK_COMMENT_MODE]}),p={variants:[{className:"type",begin:e.UNDERSCORE_IDENT_RE},{begin:/\(/,end:/\)/,contains:[]}]},b=p;return b.variants[1].contains=[p],p.variants[1].contains=[b],{name:"Kotlin",aliases:["kt","kts"],keywords:t,contains:[e.COMMENT("/\\*\\*","\\*/",{relevance:0,contains:[{className:"doctag",begin:"@[A-Za-z]+"}]}),e.C_LINE_COMMENT_MODE,m,n,r,c,d,{className:"function",beginKeywords:"fun",end:"[(]|$",returnBegin:!0,excludeEnd:!0,keywords:t,relevance:5,contains:[{begin:e.UNDERSCORE_IDENT_RE+"\\s*\\(",returnBegin:!0,relevance:0,contains:[e.UNDERSCORE_TITLE_MODE]},{className:"type",begin:/</,end:/>/,keywords:"reified",relevance:0},{className:"params",begin:/\(/,end:/\)/,endsParent:!0,keywords:t,relevance:0,contains:[{begin:/:/,end:/[=,\/]/,endsWithParent:!0,contains:[p,e.C_LINE_COMMENT_MODE,m],relevance:0},e.C_LINE_COMMENT_MODE,m,c,d,o,e.C_NUMBER_MODE]},m]},{begin:[/class|interface|trait/,/\s+/,e.UNDERSCORE_IDENT_RE],beginScope:{3:"title.class"},keywords:"class interface trait",end:/[:\{(]|$/,excludeEnd:!0,illegal:"extends implements",contains:[{beginKeywords:"public protected internal private constructor"},e.UNDERSCORE_TITLE_MODE,{className:"type",begin:/</,end:/>/,excludeBegin:!0,excludeEnd:!0,relevance:0},{className:"type",begin:/[,:]\s*/,end:/[<\(,){\s]|$/,excludeBegin:!0,returnEnd:!0},c,d]},o,{className:"meta",begin:"^#!/usr/bin/env",end:"$",illegal:`
`},f]}}const _H=e=>({IMPORTANT:{scope:"meta",begin:"!important"},BLOCK_COMMENT:e.C_BLOCK_COMMENT_MODE,HEXCOLOR:{scope:"number",begin:/#(([0-9a-fA-F]{3,4})|(([0-9a-fA-F]{2}){3,4}))\b/},FUNCTION_DISPATCH:{className:"built_in",begin:/[\w-]+(?=\()/},ATTRIBUTE_SELECTOR_MODE:{scope:"selector-attr",begin:/\[/,end:/\]/,illegal:"$",contains:[e.APOS_STRING_MODE,e.QUOTE_STRING_MODE]},CSS_NUMBER_MODE:{scope:"number",begin:e.NUMBER_RE+"(%|em|ex|ch|rem|vw|vh|vmin|vmax|cm|mm|in|pt|pc|px|deg|grad|rad|turn|s|ms|Hz|kHz|dpi|dpcm|dppx)?",relevance:0},CSS_VARIABLE:{className:"attr",begin:/--[A-Za-z_][A-Za-z0-9_-]*/}}),EH=["a","abbr","address","article","aside","audio","b","blockquote","body","button","canvas","caption","cite","code","dd","del","details","dfn","div","dl","dt","em","fieldset","figcaption","figure","footer","form","h1","h2","h3","h4","h5","h6","header","hgroup","html","i","iframe","img","input","ins","kbd","label","legend","li","main","mark","menu","nav","object","ol","optgroup","option","p","picture","q","quote","samp","section","select","source","span","strong","summary","sup","table","tbody","td","textarea","tfoot","th","thead","time","tr","ul","var","video"],TH=["defs","g","marker","mask","pattern","svg","switch","symbol","feBlend","feColorMatrix","feComponentTransfer","feComposite","feConvolveMatrix","feDiffuseLighting","feDisplacementMap","feFlood","feGaussianBlur","feImage","feMerge","feMorphology","feOffset","feSpecularLighting","feTile","feTurbulence","linearGradient","radialGradient","stop","circle","ellipse","image","line","path","polygon","polyline","rect","text","use","textPath","tspan","foreignObject","clipPath"],xH=[...EH,...TH],SH=["any-hover","any-pointer","aspect-ratio","color","color-gamut","color-index","device-aspect-ratio","device-height","device-width","display-mode","forced-colors","grid","height","hover","inverted-colors","monochrome","orientation","overflow-block","overflow-inline","pointer","prefers-color-scheme","prefers-contrast","prefers-reduced-motion","prefers-reduced-transparency","resolution","scan","scripting","update","width","min-width","max-width","min-height","max-height"].sort().reverse(),JT=["active","any-link","blank","checked","current","default","defined","dir","disabled","drop","empty","enabled","first","first-child","first-of-type","fullscreen","future","focus","focus-visible","focus-within","has","host","host-context","hover","indeterminate","in-range","invalid","is","lang","last-child","last-of-type","left","link","local-link","not","nth-child","nth-col","nth-last-child","nth-last-col","nth-last-of-type","nth-of-type","only-child","only-of-type","optional","out-of-range","past","placeholder-shown","read-only","read-write","required","right","root","scope","target","target-within","user-invalid","valid","visited","where"].sort().reverse(),ex=["after","backdrop","before","cue","cue-region","first-letter","first-line","grammar-error","marker","part","placeholder","selection","slotted","spelling-error"].sort().reverse(),wH=["accent-color","align-content","align-items","align-self","alignment-baseline","all","anchor-name","animation","animation-composition","animation-delay","animation-direction","animation-duration","animation-fill-mode","animation-iteration-count","animation-name","animation-play-state","animation-range","animation-range-end","animation-range-start","animation-timeline","animation-timing-function","appearance","aspect-ratio","backdrop-filter","backface-visibility","background","background-attachment","background-blend-mode","background-clip","background-color","background-image","background-origin","background-position","background-position-x","background-position-y","background-repeat","background-size","baseline-shift","block-size","border","border-block","border-block-color","border-block-end","border-block-end-color","border-block-end-style","border-block-end-width","border-block-start","border-block-start-color","border-block-start-style","border-block-start-width","border-block-style","border-block-width","border-bottom","border-bottom-color","border-bottom-left-radius","border-bottom-right-radius","border-bottom-style","border-bottom-width","border-collapse","border-color","border-end-end-radius","border-end-start-radius","border-image","border-image-outset","border-image-repeat","border-image-slice","border-image-source","border-image-width","border-inline","border-inline-color","border-inline-end","border-inline-end-color","border-inline-end-style","border-inline-end-width","border-inline-start","border-inline-start-color","border-inline-start-style","border-inline-start-width","border-inline-style","border-inline-width","border-left","border-left-color","border-left-style","border-left-width","border-radius","border-right","border-right-color","border-right-style","border-right-width","border-spacing","border-start-end-radius","border-start-start-radius","border-style","border-top","border-top-color","border-top-left-radius","border-top-right-radius","border-top-style","border-top-width","border-width","bottom","box-align","box-decoration-break","box-direction","box-flex","box-flex-group","box-lines","box-ordinal-group","box-orient","box-pack","box-shadow","box-sizing","break-after","break-before","break-inside","caption-side","caret-color","clear","clip","clip-path","clip-rule","color","color-interpolation","color-interpolation-filters","color-profile","color-rendering","color-scheme","column-count","column-fill","column-gap","column-rule","column-rule-color","column-rule-style","column-rule-width","column-span","column-width","columns","contain","contain-intrinsic-block-size","contain-intrinsic-height","contain-intrinsic-inline-size","contain-intrinsic-size","contain-intrinsic-width","container","container-name","container-type","content","content-visibility","counter-increment","counter-reset","counter-set","cue","cue-after","cue-before","cursor","cx","cy","direction","display","dominant-baseline","empty-cells","enable-background","field-sizing","fill","fill-opacity","fill-rule","filter","flex","flex-basis","flex-direction","flex-flow","flex-grow","flex-shrink","flex-wrap","float","flood-color","flood-opacity","flow","font","font-display","font-family","font-feature-settings","font-kerning","font-language-override","font-optical-sizing","font-palette","font-size","font-size-adjust","font-smooth","font-smoothing","font-stretch","font-style","font-synthesis","font-synthesis-position","font-synthesis-small-caps","font-synthesis-style","font-synthesis-weight","font-variant","font-variant-alternates","font-variant-caps","font-variant-east-asian","font-variant-emoji","font-variant-ligatures","font-variant-numeric","font-variant-position","font-variation-settings","font-weight","forced-color-adjust","gap","glyph-orientation-horizontal","glyph-orientation-vertical","grid","grid-area","grid-auto-columns","grid-auto-flow","grid-auto-rows","grid-column","grid-column-end","grid-column-start","grid-gap","grid-row","grid-row-end","grid-row-start","grid-template","grid-template-areas","grid-template-columns","grid-template-rows","hanging-punctuation","height","hyphenate-character","hyphenate-limit-chars","hyphens","icon","image-orientation","image-rendering","image-resolution","ime-mode","initial-letter","initial-letter-align","inline-size","inset","inset-area","inset-block","inset-block-end","inset-block-start","inset-inline","inset-inline-end","inset-inline-start","isolation","justify-content","justify-items","justify-self","kerning","left","letter-spacing","lighting-color","line-break","line-height","line-height-step","list-style","list-style-image","list-style-position","list-style-type","margin","margin-block","margin-block-end","margin-block-start","margin-bottom","margin-inline","margin-inline-end","margin-inline-start","margin-left","margin-right","margin-top","margin-trim","marker","marker-end","marker-mid","marker-start","marks","mask","mask-border","mask-border-mode","mask-border-outset","mask-border-repeat","mask-border-slice","mask-border-source","mask-border-width","mask-clip","mask-composite","mask-image","mask-mode","mask-origin","mask-position","mask-repeat","mask-size","mask-type","masonry-auto-flow","math-depth","math-shift","math-style","max-block-size","max-height","max-inline-size","max-width","min-block-size","min-height","min-inline-size","min-width","mix-blend-mode","nav-down","nav-index","nav-left","nav-right","nav-up","none","normal","object-fit","object-position","offset","offset-anchor","offset-distance","offset-path","offset-position","offset-rotate","opacity","order","orphans","outline","outline-color","outline-offset","outline-style","outline-width","overflow","overflow-anchor","overflow-block","overflow-clip-margin","overflow-inline","overflow-wrap","overflow-x","overflow-y","overlay","overscroll-behavior","overscroll-behavior-block","overscroll-behavior-inline","overscroll-behavior-x","overscroll-behavior-y","padding","padding-block","padding-block-end","padding-block-start","padding-bottom","padding-inline","padding-inline-end","padding-inline-start","padding-left","padding-right","padding-top","page","page-break-after","page-break-before","page-break-inside","paint-order","pause","pause-after","pause-before","perspective","perspective-origin","place-content","place-items","place-self","pointer-events","position","position-anchor","position-visibility","print-color-adjust","quotes","r","resize","rest","rest-after","rest-before","right","rotate","row-gap","ruby-align","ruby-position","scale","scroll-behavior","scroll-margin","scroll-margin-block","scroll-margin-block-end","scroll-margin-block-start","scroll-margin-bottom","scroll-margin-inline","scroll-margin-inline-end","scroll-margin-inline-start","scroll-margin-left","scroll-margin-right","scroll-margin-top","scroll-padding","scroll-padding-block","scroll-padding-block-end","scroll-padding-block-start","scroll-padding-bottom","scroll-padding-inline","scroll-padding-inline-end","scroll-padding-inline-start","scroll-padding-left","scroll-padding-right","scroll-padding-top","scroll-snap-align","scroll-snap-stop","scroll-snap-type","scroll-timeline","scroll-timeline-axis","scroll-timeline-name","scrollbar-color","scrollbar-gutter","scrollbar-width","shape-image-threshold","shape-margin","shape-outside","shape-rendering","speak","speak-as","src","stop-color","stop-opacity","stroke","stroke-dasharray","stroke-dashoffset","stroke-linecap","stroke-linejoin","stroke-miterlimit","stroke-opacity","stroke-width","tab-size","table-layout","text-align","text-align-all","text-align-last","text-anchor","text-combine-upright","text-decoration","text-decoration-color","text-decoration-line","text-decoration-skip","text-decoration-skip-ink","text-decoration-style","text-decoration-thickness","text-emphasis","text-emphasis-color","text-emphasis-position","text-emphasis-style","text-indent","text-justify","text-orientation","text-overflow","text-rendering","text-shadow","text-size-adjust","text-transform","text-underline-offset","text-underline-position","text-wrap","text-wrap-mode","text-wrap-style","timeline-scope","top","touch-action","transform","transform-box","transform-origin","transform-style","transition","transition-behavior","transition-delay","transition-duration","transition-property","transition-timing-function","translate","unicode-bidi","user-modify","user-select","vector-effect","vertical-align","view-timeline","view-timeline-axis","view-timeline-inset","view-timeline-name","view-transition-name","visibility","voice-balance","voice-duration","voice-family","voice-pitch","voice-range","voice-rate","voice-stress","voice-volume","white-space","white-space-collapse","widows","width","will-change","word-break","word-spacing","word-wrap","writing-mode","x","y","z-index","zoom"].sort().reverse(),AH=JT.concat(ex).sort().reverse();function kH(e){const t=_H(e),n=AH,r="and or not only",a="[\\w-]+",s="("+a+"|@\\{"+a+"\\})",o=[],c=[],d=function(D){return{className:"string",begin:"~?"+D+".*?"+D}},f=function(D,Y,H){return{className:D,begin:Y,relevance:H}},m={$pattern:/[a-z-]+/,keyword:r,attribute:SH.join(" ")},p={begin:"\\(",end:"\\)",contains:c,keywords:m,relevance:0};c.push(e.C_LINE_COMMENT_MODE,e.C_BLOCK_COMMENT_MODE,d("'"),d('"'),t.CSS_NUMBER_MODE,{begin:"(url|data-uri)\\(",starts:{className:"string",end:"[\\)\\n]",excludeEnd:!0}},t.HEXCOLOR,p,f("variable","@@?"+a,10),f("variable","@\\{"+a+"\\}"),f("built_in","~?`[^`]*?`"),{className:"attribute",begin:a+"\\s*:",end:":",returnBegin:!0,excludeEnd:!0},t.IMPORTANT,{beginKeywords:"and not"},t.FUNCTION_DISPATCH);const b=c.concat({begin:/\{/,end:/\}/,contains:o}),y={beginKeywords:"when",endsWithParent:!0,contains:[{beginKeywords:"and not"}].concat(c)},T={begin:s+"\\s*:",returnBegin:!0,end:/[;}]/,relevance:0,contains:[{begin:/-(webkit|moz|ms|o)-/},t.CSS_VARIABLE,{className:"attribute",begin:"\\b("+wH.join("|")+")\\b",end:/(?=:)/,starts:{endsWithParent:!0,illegal:"[<=$]",relevance:0,contains:c}}]},x={className:"keyword",begin:"@(import|media|charset|font-face|(-[a-z]+-)?keyframes|supports|document|namespace|page|viewport|host)\\b",starts:{end:"[;{}]",keywords:m,returnEnd:!0,contains:c,relevance:0}},C={className:"variable",variants:[{begin:"@"+a+"\\s*:",relevance:15},{begin:"@"+a}],starts:{end:"[;}]",returnEnd:!0,contains:b}},w={variants:[{begin:"[\\.#:&\\[>]",end:"[;{}]"},{begin:s,end:/\{/}],returnBegin:!0,returnEnd:!0,illegal:`[<='$"]`,relevance:0,contains:[e.C_LINE_COMMENT_MODE,e.C_BLOCK_COMMENT_MODE,y,f("keyword","all\\b"),f("variable","@\\{"+a+"\\}"),{begin:"\\b("+xH.join("|")+")\\b",className:"selector-tag"},t.CSS_NUMBER_MODE,f("selector-tag",s,0),f("selector-id","#"+s),f("selector-class","\\."+s,0),f("selector-tag","&",0),t.ATTRIBUTE_SELECTOR_MODE,{className:"selector-pseudo",begin:":("+JT.join("|")+")"},{className:"selector-pseudo",begin:":(:)?("+ex.join("|")+")"},{begin:/\(/,end:/\)/,relevance:0,contains:b},{begin:"!important"},t.FUNCTION_DISPATCH]},O={begin:a+`:(:)?(${n.join("|")})`,returnBegin:!0,contains:[w]};return o.push(e.C_LINE_COMMENT_MODE,e.C_BLOCK_COMMENT_MODE,x,C,O,T,w,y,t.FUNCTION_DISPATCH),{name:"Less",case_insensitive:!0,illegal:`[=>'/<($"]`,contains:o}}function CH(e){const t="\\[=*\\[",n="\\]=*\\]",r={begin:t,end:n,contains:["self"]},a=[e.COMMENT("--(?!"+t+")","$"),e.COMMENT("--"+t,n,{contains:[r],relevance:10})];return{name:"Lua",aliases:["pluto"],keywords:{$pattern:e.UNDERSCORE_IDENT_RE,literal:"true false nil",keyword:"and break do else elseif end for goto if in local not or repeat return then until while",built_in:"_G _ENV _VERSION __index __newindex __mode __call __metatable __tostring __len __gc __add __sub __mul __div __mod __pow __concat __unm __eq __lt __le assert collectgarbage dofile error getfenv getmetatable ipairs load loadfile loadstring module next pairs pcall print rawequal rawget rawset require select setfenv setmetatable tonumber tostring type unpack xpcall arg self coroutine resume yield status wrap create running debug getupvalue debug sethook getmetatable gethook setmetatable setlocal traceback setfenv getinfo setupvalue getlocal getregistry getfenv io lines write close flush open output type read stderr stdin input stdout popen tmpfile math log max acos huge ldexp pi cos tanh pow deg tan cosh sinh random randomseed frexp ceil floor rad abs sqrt modf asin min mod fmod log10 atan2 exp sin atan os exit setlocale date getenv difftime remove time clock tmpname rename execute package preload loadlib loaded loaders cpath config path seeall string sub upper len gfind rep find match char dump gmatch reverse byte format gsub lower table setn insert getn foreachi maxn foreach concat sort remove"},contains:a.concat([{className:"function",beginKeywords:"function",end:"\\)",contains:[e.inherit(e.TITLE_MODE,{begin:"([_a-zA-Z]\\w*\\.)*([_a-zA-Z]\\w*:)?[_a-zA-Z]\\w*"}),{className:"params",begin:"\\(",endsWithParent:!0,contains:a}].concat(a)},e.C_NUMBER_MODE,e.APOS_STRING_MODE,e.QUOTE_STRING_MODE,{className:"string",begin:t,end:n,contains:[r],relevance:5}])}}function NH(e){const t={className:"variable",variants:[{begin:"\\$\\("+e.UNDERSCORE_IDENT_RE+"\\)",contains:[e.BACKSLASH_ESCAPE]},{begin:/\$[@%<?\^\+\*]/}]},n={className:"string",begin:/"/,end:/"/,contains:[e.BACKSLASH_ESCAPE,t]},r={className:"variable",begin:/\$\([\w-]+\s/,end:/\)/,keywords:{built_in:"subst patsubst strip findstring filter filter-out sort word wordlist firstword lastword dir notdir suffix basename addsuffix addprefix join wildcard realpath abspath error warning shell origin flavor foreach if or and call eval file value"},contains:[t,n]},a={begin:"^"+e.UNDERSCORE_IDENT_RE+"\\s*(?=[:+?]?=)"},s={className:"meta",begin:/^\.PHONY:/,end:/$/,keywords:{$pattern:/[\.\w]+/,keyword:".PHONY"}},o={className:"section",begin:/^[^\s]+:/,end:/$/,contains:[t]};return{name:"Makefile",aliases:["mk","mak","make"],keywords:{$pattern:/[\w-]+/,keyword:"define endef undefine ifdef ifndef ifeq ifneq else endif include -include sinclude override export unexport private vpath"},contains:[e.HASH_COMMENT_MODE,t,n,r,a,s,o]}}function OH(e){const t=e.regex,n={begin:/<\/?[A-Za-z_]/,end:">",subLanguage:"xml",relevance:0},r={begin:"^[-\\*]{3,}",end:"$"},a={className:"code",variants:[{begin:"(`{3,})[^`](.|\\n)*?\\1`*[ ]*"},{begin:"(~{3,})[^~](.|\\n)*?\\1~*[ ]*"},{begin:"```",end:"```+[ ]*$"},{begin:"~~~",end:"~~~+[ ]*$"},{begin:"`.+?`"},{begin:"(?=^( {4}|\\t))",contains:[{begin:"^( {4}|\\t)",end:"(\\n)$"}],relevance:0}]},s={className:"bullet",begin:"^[ 	]*([*+-]|(\\d+\\.))(?=\\s+)",end:"\\s+",excludeEnd:!0},o={begin:/^\[[^\n]+\]:/,returnBegin:!0,contains:[{className:"symbol",begin:/\[/,end:/\]/,excludeBegin:!0,excludeEnd:!0},{className:"link",begin:/:\s*/,end:/$/,excludeBegin:!0}]},c=/[A-Za-z][A-Za-z0-9+.-]*/,d={variants:[{begin:/\[.+?\]\[.*?\]/,relevance:0},{begin:/\[.+?\]\(((data|javascript|mailto):|(?:http|ftp)s?:\/\/).*?\)/,relevance:2},{begin:t.concat(/\[.+?\]\(/,c,/:\/\/.*?\)/),relevance:2},{begin:/\[.+?\]\([./?&#].*?\)/,relevance:1},{begin:/\[.*?\]\(.*?\)/,relevance:0}],returnBegin:!0,contains:[{match:/\[(?=\])/},{className:"string",relevance:0,begin:"\\[",end:"\\]",excludeBegin:!0,returnEnd:!0},{className:"link",relevance:0,begin:"\\]\\(",end:"\\)",excludeBegin:!0,excludeEnd:!0},{className:"symbol",relevance:0,begin:"\\]\\[",end:"\\]",excludeBegin:!0,excludeEnd:!0}]},f={className:"strong",contains:[],variants:[{begin:/_{2}(?!\s)/,end:/_{2}/},{begin:/\*{2}(?!\s)/,end:/\*{2}/}]},m={className:"emphasis",contains:[],variants:[{begin:/\*(?![*\s])/,end:/\*/},{begin:/_(?![_\s])/,end:/_/,relevance:0}]},p=e.inherit(f,{contains:[]}),b=e.inherit(m,{contains:[]});f.contains.push(b),m.contains.push(p);let y=[n,d];return[f,m,p,b].forEach(w=>{w.contains=w.contains.concat(y)}),y=y.concat(f,m),{name:"Markdown",aliases:["md","mkdown","mkd"],contains:[{className:"section",variants:[{begin:"^#{1,6}",end:"$",contains:y},{begin:"(?=^.+?\\n[=-]{2,}$)",contains:[{begin:"^[=-]*$"},{begin:"^",end:"\\n",contains:y}]}]},n,s,f,m,{className:"quote",begin:"^>\\s+",contains:y,end:"$"},a,r,d,o,{scope:"literal",match:/&([a-zA-Z0-9]+|#[0-9]{1,7}|#[Xx][0-9a-fA-F]{1,6});/}]}}function RH(e){const t={className:"built_in",begin:"\\b(AV|CA|CF|CG|CI|CL|CM|CN|CT|MK|MP|MTK|MTL|NS|SCN|SK|UI|WK|XC)\\w+"},n=/[a-zA-Z@][a-zA-Z0-9_]*/,c={"variable.language":["this","super"],$pattern:n,keyword:["while","export","sizeof","typedef","const","struct","for","union","volatile","static","mutable","if","do","return","goto","enum","else","break","extern","asm","case","default","register","explicit","typename","switch","continue","inline","readonly","assign","readwrite","self","@synchronized","id","typeof","nonatomic","IBOutlet","IBAction","strong","weak","copy","in","out","inout","bycopy","byref","oneway","__strong","__weak","__block","__autoreleasing","@private","@protected","@public","@try","@property","@end","@throw","@catch","@finally","@autoreleasepool","@synthesize","@dynamic","@selector","@optional","@required","@encode","@package","@import","@defs","@compatibility_alias","__bridge","__bridge_transfer","__bridge_retained","__bridge_retain","__covariant","__contravariant","__kindof","_Nonnull","_Nullable","_Null_unspecified","__FUNCTION__","__PRETTY_FUNCTION__","__attribute__","getter","setter","retain","unsafe_unretained","nonnull","nullable","null_unspecified","null_resettable","class","instancetype","NS_DESIGNATED_INITIALIZER","NS_UNAVAILABLE","NS_REQUIRES_SUPER","NS_RETURNS_INNER_POINTER","NS_INLINE","NS_AVAILABLE","NS_DEPRECATED","NS_ENUM","NS_OPTIONS","NS_SWIFT_UNAVAILABLE","NS_ASSUME_NONNULL_BEGIN","NS_ASSUME_NONNULL_END","NS_REFINED_FOR_SWIFT","NS_SWIFT_NAME","NS_SWIFT_NOTHROW","NS_DURING","NS_HANDLER","NS_ENDHANDLER","NS_VALUERETURN","NS_VOIDRETURN"],literal:["false","true","FALSE","TRUE","nil","YES","NO","NULL"],built_in:["dispatch_once_t","dispatch_queue_t","dispatch_sync","dispatch_async","dispatch_once"],type:["int","float","char","unsigned","signed","short","long","double","wchar_t","unichar","void","bool","BOOL","id|0","_Bool"]},d={$pattern:n,keyword:["@interface","@class","@protocol","@implementation"]};return{name:"Objective-C",aliases:["mm","objc","obj-c","obj-c++","objective-c++"],keywords:c,illegal:"</",contains:[t,e.C_LINE_COMMENT_MODE,e.C_BLOCK_COMMENT_MODE,e.C_NUMBER_MODE,e.QUOTE_STRING_MODE,e.APOS_STRING_MODE,{className:"string",variants:[{begin:'@"',end:'"',illegal:"\\n",contains:[e.BACKSLASH_ESCAPE]}]},{className:"meta",begin:/#\s*[a-z]+\b/,end:/$/,keywords:{keyword:"if else elif endif define undef warning error line pragma ifdef ifndef include"},contains:[{begin:/\\\n/,relevance:0},e.inherit(e.QUOTE_STRING_MODE,{className:"string"}),{className:"string",begin:/<.*?>/,end:/$/,illegal:"\\n"},e.C_LINE_COMMENT_MODE,e.C_BLOCK_COMMENT_MODE]},{className:"class",begin:"("+d.keyword.join("|")+")\\b",end:/(\{|$)/,excludeEnd:!0,keywords:d,contains:[e.UNDERSCORE_TITLE_MODE]},{begin:"\\."+e.UNDERSCORE_IDENT_RE,relevance:0}]}}function MH(e){const t=e.regex,n=["abs","accept","alarm","and","atan2","bind","binmode","bless","break","caller","chdir","chmod","chomp","chop","chown","chr","chroot","class","close","closedir","connect","continue","cos","crypt","dbmclose","dbmopen","defined","delete","die","do","dump","each","else","elsif","endgrent","endhostent","endnetent","endprotoent","endpwent","endservent","eof","eval","exec","exists","exit","exp","fcntl","field","fileno","flock","for","foreach","fork","format","formline","getc","getgrent","getgrgid","getgrnam","gethostbyaddr","gethostbyname","gethostent","getlogin","getnetbyaddr","getnetbyname","getnetent","getpeername","getpgrp","getpriority","getprotobyname","getprotobynumber","getprotoent","getpwent","getpwnam","getpwuid","getservbyname","getservbyport","getservent","getsockname","getsockopt","given","glob","gmtime","goto","grep","gt","hex","if","index","int","ioctl","join","keys","kill","last","lc","lcfirst","length","link","listen","local","localtime","log","lstat","lt","ma","map","method","mkdir","msgctl","msgget","msgrcv","msgsnd","my","ne","next","no","not","oct","open","opendir","or","ord","our","pack","package","pipe","pop","pos","print","printf","prototype","push","q|0","qq","quotemeta","qw","qx","rand","read","readdir","readline","readlink","readpipe","recv","redo","ref","rename","require","reset","return","reverse","rewinddir","rindex","rmdir","say","scalar","seek","seekdir","select","semctl","semget","semop","send","setgrent","sethostent","setnetent","setpgrp","setpriority","setprotoent","setpwent","setservent","setsockopt","shift","shmctl","shmget","shmread","shmwrite","shutdown","sin","sleep","socket","socketpair","sort","splice","split","sprintf","sqrt","srand","stat","state","study","sub","substr","symlink","syscall","sysopen","sysread","sysseek","system","syswrite","tell","telldir","tie","tied","time","times","tr","truncate","uc","ucfirst","umask","undef","unless","unlink","unpack","unshift","untie","until","use","utime","values","vec","wait","waitpid","wantarray","warn","when","while","write","x|0","xor","y|0"],r=/[dualxmsipngr]{0,12}/,a={$pattern:/[\w.]+/,keyword:n.join(" ")},s={className:"subst",begin:"[$@]\\{",end:"\\}",keywords:a},o={begin:/->\{/,end:/\}/},c={scope:"attr",match:/\s+:\s*\w+(\s*\(.*?\))?/},d={scope:"variable",variants:[{begin:/\$\d/},{begin:t.concat(/[$%@](?!")(\^\w\b|#\w+(::\w+)*|\{\w+\}|\w+(::\w*)*)/,"(?![A-Za-z])(?![@$%])")},{begin:/[$%@](?!")[^\s\w{=]|\$=/,relevance:0}],contains:[c]},f={className:"number",variants:[{match:/0?\.[0-9][0-9_]+\b/},{match:/\bv?(0|[1-9][0-9_]*(\.[0-9_]+)?|[1-9][0-9_]*)\b/},{match:/\b0[0-7][0-7_]*\b/},{match:/\b0x[0-9a-fA-F][0-9a-fA-F_]*\b/},{match:/\b0b[0-1][0-1_]*\b/}],relevance:0},m=[e.BACKSLASH_ESCAPE,s,d],p=[/!/,/\//,/\|/,/\?/,/'/,/"/,/#/],b=(x,C,w="\\1")=>{const O=w==="\\1"?w:t.concat(w,C);return t.concat(t.concat("(?:",x,")"),C,/(?:\\.|[^\\\/])*?/,O,/(?:\\.|[^\\\/])*?/,w,r)},y=(x,C,w)=>t.concat(t.concat("(?:",x,")"),C,/(?:\\.|[^\\\/])*?/,w,r),T=[d,e.HASH_COMMENT_MODE,e.COMMENT(/^=\w/,/=cut/,{endsWithParent:!0}),o,{className:"string",contains:m,variants:[{begin:"q[qwxr]?\\s*\\(",end:"\\)",relevance:5},{begin:"q[qwxr]?\\s*\\[",end:"\\]",relevance:5},{begin:"q[qwxr]?\\s*\\{",end:"\\}",relevance:5},{begin:"q[qwxr]?\\s*\\|",end:"\\|",relevance:5},{begin:"q[qwxr]?\\s*<",end:">",relevance:5},{begin:"qw\\s+q",end:"q",relevance:5},{begin:"'",end:"'",contains:[e.BACKSLASH_ESCAPE]},{begin:'"',end:'"'},{begin:"`",end:"`",contains:[e.BACKSLASH_ESCAPE]},{begin:/\{\w+\}/,relevance:0},{begin:"-?\\w+\\s*=>",relevance:0}]},f,{begin:"(\\/\\/|"+e.RE_STARTERS_RE+"|\\b(split|return|print|reverse|grep)\\b)\\s*",keywords:"split return print reverse grep",relevance:0,contains:[e.HASH_COMMENT_MODE,{className:"regexp",variants:[{begin:b("s|tr|y",t.either(...p,{capture:!0}))},{begin:b("s|tr|y","\\(","\\)")},{begin:b("s|tr|y","\\[","\\]")},{begin:b("s|tr|y","\\{","\\}")}],relevance:2},{className:"regexp",variants:[{begin:/(m|qr)\/\//,relevance:0},{begin:y("(?:m|qr)?",/\//,/\//)},{begin:y("m|qr",t.either(...p,{capture:!0}),/\1/)},{begin:y("m|qr",/\(/,/\)/)},{begin:y("m|qr",/\[/,/\]/)},{begin:y("m|qr",/\{/,/\}/)}]}]},{className:"function",beginKeywords:"sub method",end:"(\\s*\\(.*?\\))?[;{]",excludeEnd:!0,relevance:5,contains:[e.TITLE_MODE,c]},{className:"class",beginKeywords:"class",end:"[;{]",excludeEnd:!0,relevance:5,contains:[e.TITLE_MODE,c,f]},{begin:"-\\w\\b",relevance:0},{begin:"^__DATA__$",end:"^__END__$",subLanguage:"mojolicious",contains:[{begin:"^@@.*",end:"$",className:"comment"}]}];return s.contains=T,o.contains=T,{name:"Perl",aliases:["pl","pm"],keywords:a,contains:T}}function IH(e){const t=e.regex,n=/(?![A-Za-z0-9])(?![$])/,r=t.concat(/[a-zA-Z_\x7f-\xff][a-zA-Z0-9_\x7f-\xff]*/,n),a=t.concat(/(\\?[A-Z][a-z0-9_\x7f-\xff]+|\\?[A-Z]+(?=[A-Z][a-z0-9_\x7f-\xff])){1,}/,n),s=t.concat(/[A-Z]+/,n),o={scope:"variable",match:"\\$+"+r},c={scope:"meta",variants:[{begin:/<\?php/,relevance:10},{begin:/<\?=/},{begin:/<\?/,relevance:.1},{begin:/\?>/}]},d={scope:"subst",variants:[{begin:/\$\w+/},{begin:/\{\$/,end:/\}/}]},f=e.inherit(e.APOS_STRING_MODE,{illegal:null}),m=e.inherit(e.QUOTE_STRING_MODE,{illegal:null,contains:e.QUOTE_STRING_MODE.contains.concat(d)}),p={begin:/<<<[ \t]*(?:(\w+)|"(\w+)")\n/,end:/[ \t]*(\w+)\b/,contains:e.QUOTE_STRING_MODE.contains.concat(d),"on:begin":(fe,de)=>{de.data._beginMatch=fe[1]||fe[2]},"on:end":(fe,de)=>{de.data._beginMatch!==fe[1]&&de.ignoreMatch()}},b=e.END_SAME_AS_BEGIN({begin:/<<<[ \t]*'(\w+)'\n/,end:/[ \t]*(\w+)\b/}),y=`[ 	
]`,T={scope:"string",variants:[m,f,p,b]},x={scope:"number",variants:[{begin:"\\b0[bB][01]+(?:_[01]+)*\\b"},{begin:"\\b0[oO][0-7]+(?:_[0-7]+)*\\b"},{begin:"\\b0[xX][\\da-fA-F]+(?:_[\\da-fA-F]+)*\\b"},{begin:"(?:\\b\\d+(?:_\\d+)*(\\.(?:\\d+(?:_\\d+)*))?|\\B\\.\\d+)(?:[eE][+-]?\\d+)?"}],relevance:0},C=["false","null","true"],w=["__CLASS__","__DIR__","__FILE__","__FUNCTION__","__COMPILER_HALT_OFFSET__","__LINE__","__METHOD__","__NAMESPACE__","__TRAIT__","die","echo","exit","include","include_once","print","require","require_once","array","abstract","and","as","binary","bool","boolean","break","callable","case","catch","class","clone","const","continue","declare","default","do","double","else","elseif","empty","enddeclare","endfor","endforeach","endif","endswitch","endwhile","enum","eval","extends","final","finally","float","for","foreach","from","global","goto","if","implements","instanceof","insteadof","int","integer","interface","isset","iterable","list","match|0","mixed","new","never","object","or","private","protected","public","readonly","real","return","string","switch","throw","trait","try","unset","use","var","void","while","xor","yield"],O=["Error|0","AppendIterator","ArgumentCountError","ArithmeticError","ArrayIterator","ArrayObject","AssertionError","BadFunctionCallException","BadMethodCallException","CachingIterator","CallbackFilterIterator","CompileError","Countable","DirectoryIterator","DivisionByZeroError","DomainException","EmptyIterator","ErrorException","Exception","FilesystemIterator","FilterIterator","GlobIterator","InfiniteIterator","InvalidArgumentException","IteratorIterator","LengthException","LimitIterator","LogicException","MultipleIterator","NoRewindIterator","OutOfBoundsException","OutOfRangeException","OuterIterator","OverflowException","ParentIterator","ParseError","RangeException","RecursiveArrayIterator","RecursiveCachingIterator","RecursiveCallbackFilterIterator","RecursiveDirectoryIterator","RecursiveFilterIterator","RecursiveIterator","RecursiveIteratorIterator","RecursiveRegexIterator","RecursiveTreeIterator","RegexIterator","RuntimeException","SeekableIterator","SplDoublyLinkedList","SplFileInfo","SplFileObject","SplFixedArray","SplHeap","SplMaxHeap","SplMinHeap","SplObjectStorage","SplObserver","SplPriorityQueue","SplQueue","SplStack","SplSubject","SplTempFileObject","TypeError","UnderflowException","UnexpectedValueException","UnhandledMatchError","ArrayAccess","BackedEnum","Closure","Fiber","Generator","Iterator","IteratorAggregate","Serializable","Stringable","Throwable","Traversable","UnitEnum","WeakReference","WeakMap","Directory","__PHP_Incomplete_Class","parent","php_user_filter","self","static","stdClass"],Y={keyword:w,literal:(fe=>{const de=[];return fe.forEach(V=>{de.push(V),V.toLowerCase()===V?de.push(V.toUpperCase()):de.push(V.toLowerCase())}),de})(C),built_in:O},H=fe=>fe.map(de=>de.replace(/\|\d+$/,"")),B={variants:[{match:[/new/,t.concat(y,"+"),t.concat("(?!",H(O).join("\\b|"),"\\b)"),a],scope:{1:"keyword",4:"title.class"}}]},ee=t.concat(r,"\\b(?!\\()"),Z={variants:[{match:[t.concat(/::/,t.lookahead(/(?!class\b)/)),ee],scope:{2:"variable.constant"}},{match:[/::/,/class/],scope:{2:"variable.language"}},{match:[a,t.concat(/::/,t.lookahead(/(?!class\b)/)),ee],scope:{1:"title.class",3:"variable.constant"}},{match:[a,t.concat("::",t.lookahead(/(?!class\b)/))],scope:{1:"title.class"}},{match:[a,/::/,/class/],scope:{1:"title.class",3:"variable.language"}}]},Q={scope:"attr",match:t.concat(r,t.lookahead(":"),t.lookahead(/(?!::)/))},G={relevance:0,begin:/\(/,end:/\)/,keywords:Y,contains:[Q,o,Z,e.C_BLOCK_COMMENT_MODE,T,x,B]},se={relevance:0,match:[/\b/,t.concat("(?!fn\\b|function\\b|",H(w).join("\\b|"),"|",H(O).join("\\b|"),"\\b)"),r,t.concat(y,"*"),t.lookahead(/(?=\()/)],scope:{3:"title.function.invoke"},contains:[G]};G.contains.push(se);const oe=[Q,Z,e.C_BLOCK_COMMENT_MODE,T,x,B],ye={begin:t.concat(/#\[\s*\\?/,t.either(a,s)),beginScope:"meta",end:/]/,endScope:"meta",keywords:{literal:C,keyword:["new","array"]},contains:[{begin:/\[/,end:/]/,keywords:{literal:C,keyword:["new","array"]},contains:["self",...oe]},...oe,{scope:"meta",variants:[{match:a},{match:s}]}]};return{case_insensitive:!1,keywords:Y,contains:[ye,e.HASH_COMMENT_MODE,e.COMMENT("//","$"),e.COMMENT("/\\*","\\*/",{contains:[{scope:"doctag",match:"@[A-Za-z]+"}]}),{match:/__halt_compiler\(\);/,keywords:"__halt_compiler",starts:{scope:"comment",end:e.MATCH_NOTHING_RE,contains:[{match:/\?>/,scope:"meta",endsParent:!0}]}},c,{scope:"variable.language",match:/\$this\b/},o,se,Z,{match:[/const/,/\s/,r],scope:{1:"keyword",3:"variable.constant"}},B,{scope:"function",relevance:0,beginKeywords:"fn function",end:/[;{]/,excludeEnd:!0,illegal:"[$%\\[]",contains:[{beginKeywords:"use"},e.UNDERSCORE_TITLE_MODE,{begin:"=>",endsParent:!0},{scope:"params",begin:"\\(",end:"\\)",excludeBegin:!0,excludeEnd:!0,keywords:Y,contains:["self",ye,o,Z,e.C_BLOCK_COMMENT_MODE,T,x]}]},{scope:"class",variants:[{beginKeywords:"enum",illegal:/[($"]/},{beginKeywords:"class interface trait",illegal:/[:($"]/}],relevance:0,end:/\{/,excludeEnd:!0,contains:[{beginKeywords:"extends implements"},e.UNDERSCORE_TITLE_MODE]},{beginKeywords:"namespace",relevance:0,end:";",illegal:/[.']/,contains:[e.inherit(e.UNDERSCORE_TITLE_MODE,{scope:"title.class"})]},{beginKeywords:"use",relevance:0,end:";",contains:[{match:/\b(as|const|function)\b/,scope:"keyword"},e.UNDERSCORE_TITLE_MODE]},T,x]}}function DH(e){return{name:"PHP template",subLanguage:"xml",contains:[{begin:/<\?(php|=)?/,end:/\?>/,subLanguage:"php",contains:[{begin:"/\\*",end:"\\*/",skip:!0},{begin:'b"',end:'"',skip:!0},{begin:"b'",end:"'",skip:!0},e.inherit(e.APOS_STRING_MODE,{illegal:null,className:null,contains:null,skip:!0}),e.inherit(e.QUOTE_STRING_MODE,{illegal:null,className:null,contains:null,skip:!0})]}]}}function LH(e){return{name:"Plain text",aliases:["text","txt"],disableAutodetect:!0}}function PH(e){const t=e.regex,n=new RegExp("[\\p{XID_Start}_]\\p{XID_Continue}*","u"),r=["and","as","assert","async","await","break","case","class","continue","def","del","elif","else","except","finally","for","from","global","if","import","in","is","lambda","match","nonlocal|10","not","or","pass","raise","return","try","while","with","yield"],c={$pattern:/[A-Za-z]\w+|__\w+__/,keyword:r,built_in:["__import__","abs","all","any","ascii","bin","bool","breakpoint","bytearray","bytes","callable","chr","classmethod","compile","complex","delattr","dict","dir","divmod","enumerate","eval","exec","filter","float","format","frozenset","getattr","globals","hasattr","hash","help","hex","id","input","int","isinstance","issubclass","iter","len","list","locals","map","max","memoryview","min","next","object","oct","open","ord","pow","print","property","range","repr","reversed","round","set","setattr","slice","sorted","staticmethod","str","sum","super","tuple","type","vars","zip"],literal:["__debug__","Ellipsis","False","None","NotImplemented","True"],type:["Any","Callable","Coroutine","Dict","List","Literal","Generic","Optional","Sequence","Set","Tuple","Type","Union"]},d={className:"meta",begin:/^(>>>|\.\.\.) /},f={className:"subst",begin:/\{/,end:/\}/,keywords:c,illegal:/#/},m={begin:/\{\{/,relevance:0},p={className:"string",contains:[e.BACKSLASH_ESCAPE],variants:[{begin:/([uU]|[bB]|[rR]|[bB][rR]|[rR][bB])?'''/,end:/'''/,contains:[e.BACKSLASH_ESCAPE,d],relevance:10},{begin:/([uU]|[bB]|[rR]|[bB][rR]|[rR][bB])?"""/,end:/"""/,contains:[e.BACKSLASH_ESCAPE,d],relevance:10},{begin:/([fF][rR]|[rR][fF]|[fF])'''/,end:/'''/,contains:[e.BACKSLASH_ESCAPE,d,m,f]},{begin:/([fF][rR]|[rR][fF]|[fF])"""/,end:/"""/,contains:[e.BACKSLASH_ESCAPE,d,m,f]},{begin:/([uU]|[rR])'/,end:/'/,relevance:10},{begin:/([uU]|[rR])"/,end:/"/,relevance:10},{begin:/([bB]|[bB][rR]|[rR][bB])'/,end:/'/},{begin:/([bB]|[bB][rR]|[rR][bB])"/,end:/"/},{begin:/([fF][rR]|[rR][fF]|[fF])'/,end:/'/,contains:[e.BACKSLASH_ESCAPE,m,f]},{begin:/([fF][rR]|[rR][fF]|[fF])"/,end:/"/,contains:[e.BACKSLASH_ESCAPE,m,f]},e.APOS_STRING_MODE,e.QUOTE_STRING_MODE]},b="[0-9](_?[0-9])*",y=`(\\b(${b}))?\\.(${b})|\\b(${b})\\.`,T=`\\b|${r.join("|")}`,x={className:"number",relevance:0,variants:[{begin:`(\\b(${b})|(${y}))[eE][+-]?(${b})[jJ]?(?=${T})`},{begin:`(${y})[jJ]?`},{begin:`\\b([1-9](_?[0-9])*|0+(_?0)*)[lLjJ]?(?=${T})`},{begin:`\\b0[bB](_?[01])+[lL]?(?=${T})`},{begin:`\\b0[oO](_?[0-7])+[lL]?(?=${T})`},{begin:`\\b0[xX](_?[0-9a-fA-F])+[lL]?(?=${T})`},{begin:`\\b(${b})[jJ](?=${T})`}]},C={className:"comment",begin:t.lookahead(/# type:/),end:/$/,keywords:c,contains:[{begin:/# type:/},{begin:/#/,end:/\b\B/,endsWithParent:!0}]},w={className:"params",variants:[{className:"",begin:/\(\s*\)/,skip:!0},{begin:/\(/,end:/\)/,excludeBegin:!0,excludeEnd:!0,keywords:c,contains:["self",d,x,p,e.HASH_COMMENT_MODE]}]};return f.contains=[p,x,d],{name:"Python",aliases:["py","gyp","ipython"],unicodeRegex:!0,keywords:c,illegal:/(<\/|\?)|=>/,contains:[d,x,{scope:"variable.language",match:/\bself\b/},{beginKeywords:"if",relevance:0},{match:/\bor\b/,scope:"keyword"},p,C,e.HASH_COMMENT_MODE,{match:[/\bdef/,/\s+/,n],scope:{1:"keyword",3:"title.function"},contains:[w]},{variants:[{match:[/\bclass/,/\s+/,n,/\s*/,/\(\s*/,n,/\s*\)/]},{match:[/\bclass/,/\s+/,n]}],scope:{1:"keyword",3:"title.class",6:"title.class.inherited"}},{className:"meta",begin:/^[\t ]*@/,end:/(?=#)|$/,contains:[x,w,p]}]}}function BH(e){return{aliases:["pycon"],contains:[{className:"meta.prompt",starts:{end:/ |$/,starts:{end:"$",subLanguage:"python"}},variants:[{begin:/^>>>(?=[ ]|$)/},{begin:/^\.\.\.(?=[ ]|$)/}]}]}}function UH(e){const t=e.regex,n=/(?:(?:[a-zA-Z]|\.[._a-zA-Z])[._a-zA-Z0-9]*)|\.(?!\d)/,r=t.either(/0[xX][0-9a-fA-F]+\.[0-9a-fA-F]*[pP][+-]?\d+i?/,/0[xX][0-9a-fA-F]+(?:[pP][+-]?\d+)?[Li]?/,/(?:\d+(?:\.\d*)?|\.\d+)(?:[eE][+-]?\d+)?[Li]?/),a=/[=!<>:]=|\|\||&&|:::?|<-|<<-|->>|->|\|>|[-+*\/?!$&|:<=>@^~]|\*\*/,s=t.either(/[()]/,/[{}]/,/\[\[/,/[[\]]/,/\\/,/,/);return{name:"R",keywords:{$pattern:n,keyword:"function if in break next repeat else for while",literal:"NULL NA TRUE FALSE Inf NaN NA_integer_|10 NA_real_|10 NA_character_|10 NA_complex_|10",built_in:"LETTERS letters month.abb month.name pi T F abs acos acosh all any anyNA Arg as.call as.character as.complex as.double as.environment as.integer as.logical as.null.default as.numeric as.raw asin asinh atan atanh attr attributes baseenv browser c call ceiling class Conj cos cosh cospi cummax cummin cumprod cumsum digamma dim dimnames emptyenv exp expression floor forceAndCall gamma gc.time globalenv Im interactive invisible is.array is.atomic is.call is.character is.complex is.double is.environment is.expression is.finite is.function is.infinite is.integer is.language is.list is.logical is.matrix is.na is.name is.nan is.null is.numeric is.object is.pairlist is.raw is.recursive is.single is.symbol lazyLoadDBfetch length lgamma list log max min missing Mod names nargs nzchar oldClass on.exit pos.to.env proc.time prod quote range Re rep retracemem return round seq_along seq_len seq.int sign signif sin sinh sinpi sqrt standardGeneric substitute sum switch tan tanh tanpi tracemem trigamma trunc unclass untracemem UseMethod xtfrm"},contains:[e.COMMENT(/#'/,/$/,{contains:[{scope:"doctag",match:/@examples/,starts:{end:t.lookahead(t.either(/\n^#'\s*(?=@[a-zA-Z]+)/,/\n^(?!#')/)),endsParent:!0}},{scope:"doctag",begin:"@param",end:/$/,contains:[{scope:"variable",variants:[{match:n},{match:/`(?:\\.|[^`\\])+`/}],endsParent:!0}]},{scope:"doctag",match:/@[a-zA-Z]+/},{scope:"keyword",match:/\\[a-zA-Z]+/}]}),e.HASH_COMMENT_MODE,{scope:"string",contains:[e.BACKSLASH_ESCAPE],variants:[e.END_SAME_AS_BEGIN({begin:/[rR]"(-*)\(/,end:/\)(-*)"/}),e.END_SAME_AS_BEGIN({begin:/[rR]"(-*)\{/,end:/\}(-*)"/}),e.END_SAME_AS_BEGIN({begin:/[rR]"(-*)\[/,end:/\](-*)"/}),e.END_SAME_AS_BEGIN({begin:/[rR]'(-*)\(/,end:/\)(-*)'/}),e.END_SAME_AS_BEGIN({begin:/[rR]'(-*)\{/,end:/\}(-*)'/}),e.END_SAME_AS_BEGIN({begin:/[rR]'(-*)\[/,end:/\](-*)'/}),{begin:'"',end:'"',relevance:0},{begin:"'",end:"'",relevance:0}]},{relevance:0,variants:[{scope:{1:"operator",2:"number"},match:[a,r]},{scope:{1:"operator",2:"number"},match:[/%[^%]*%/,r]},{scope:{1:"punctuation",2:"number"},match:[s,r]},{scope:{2:"number"},match:[/[^a-zA-Z0-9._]|^/,r]}]},{scope:{3:"operator"},match:[n,/\s+/,/<-/,/\s+/]},{scope:"operator",relevance:0,variants:[{match:a},{match:/%[^%]*%/}]},{scope:"punctuation",relevance:0,match:s},{begin:"`",end:"`",contains:[{begin:/\\./}]}]}}function FH(e){const t=e.regex,n="([a-zA-Z_]\\w*[!?=]?|[-+~]@|<<|>>|=~|===?|<=>|[<>]=?|\\*\\*|[-/+%^&*~`|]|\\[\\]=?)",r=t.either(/\b([A-Z]+[a-z0-9]+)+/,/\b([A-Z]+[a-z0-9]+)+[A-Z]+/),a=t.concat(r,/(::\w+)*/),o={"variable.constant":["__FILE__","__LINE__","__ENCODING__"],"variable.language":["self","super"],keyword:["alias","and","begin","BEGIN","break","case","class","defined","do","else","elsif","end","END","ensure","for","if","in","module","next","not","or","redo","require","rescue","retry","return","then","undef","unless","until","when","while","yield",...["include","extend","prepend","public","private","protected","raise","throw"]],built_in:["proc","lambda","attr_accessor","attr_reader","attr_writer","define_method","private_constant","module_function"],literal:["true","false","nil"]},c={className:"doctag",begin:"@[A-Za-z]+"},d={begin:"#<",end:">"},f=[e.COMMENT("#","$",{contains:[c]}),e.COMMENT("^=begin","^=end",{contains:[c],relevance:10}),e.COMMENT("^__END__",e.MATCH_NOTHING_RE)],m={className:"subst",begin:/#\{/,end:/\}/,keywords:o},p={className:"string",contains:[e.BACKSLASH_ESCAPE,m],variants:[{begin:/'/,end:/'/},{begin:/"/,end:/"/},{begin:/`/,end:/`/},{begin:/%[qQwWx]?\(/,end:/\)/},{begin:/%[qQwWx]?\[/,end:/\]/},{begin:/%[qQwWx]?\{/,end:/\}/},{begin:/%[qQwWx]?</,end:/>/},{begin:/%[qQwWx]?\//,end:/\//},{begin:/%[qQwWx]?%/,end:/%/},{begin:/%[qQwWx]?-/,end:/-/},{begin:/%[qQwWx]?\|/,end:/\|/},{begin:/\B\?(\\\d{1,3})/},{begin:/\B\?(\\x[A-Fa-f0-9]{1,2})/},{begin:/\B\?(\\u\{?[A-Fa-f0-9]{1,6}\}?)/},{begin:/\B\?(\\M-\\C-|\\M-\\c|\\c\\M-|\\M-|\\C-\\M-)[\x20-\x7e]/},{begin:/\B\?\\(c|C-)[\x20-\x7e]/},{begin:/\B\?\\?\S/},{begin:t.concat(/<<[-~]?'?/,t.lookahead(/(\w+)(?=\W)[^\n]*\n(?:[^\n]*\n)*?\s*\1\b/)),contains:[e.END_SAME_AS_BEGIN({begin:/(\w+)/,end:/(\w+)/,contains:[e.BACKSLASH_ESCAPE,m]})]}]},b="[1-9](_?[0-9])*|0",y="[0-9](_?[0-9])*",T={className:"number",relevance:0,variants:[{begin:`\\b(${b})(\\.(${y}))?([eE][+-]?(${y})|r)?i?\\b`},{begin:"\\b0[dD][0-9](_?[0-9])*r?i?\\b"},{begin:"\\b0[bB][0-1](_?[0-1])*r?i?\\b"},{begin:"\\b0[oO][0-7](_?[0-7])*r?i?\\b"},{begin:"\\b0[xX][0-9a-fA-F](_?[0-9a-fA-F])*r?i?\\b"},{begin:"\\b0(_?[0-7])+r?i?\\b"}]},x={variants:[{match:/\(\)/},{className:"params",begin:/\(/,end:/(?=\))/,excludeBegin:!0,endsParent:!0,keywords:o}]},B=[p,{variants:[{match:[/class\s+/,a,/\s+<\s+/,a]},{match:[/\b(class|module)\s+/,a]}],scope:{2:"title.class",4:"title.class.inherited"},keywords:o},{match:[/(include|extend)\s+/,a],scope:{2:"title.class"},keywords:o},{relevance:0,match:[a,/\.new[. (]/],scope:{1:"title.class"}},{relevance:0,match:/\b[A-Z][A-Z_0-9]+\b/,className:"variable.constant"},{relevance:0,match:r,scope:"title.class"},{match:[/def/,/\s+/,n],scope:{1:"keyword",3:"title.function"},contains:[x]},{begin:e.IDENT_RE+"::"},{className:"symbol",begin:e.UNDERSCORE_IDENT_RE+"(!|\\?)?:",relevance:0},{className:"symbol",begin:":(?!\\s)",contains:[p,{begin:n}],relevance:0},T,{className:"variable",begin:"(\\$\\W)|((\\$|@@?)(\\w+))(?=[^@$?])(?![A-Za-z])(?![@$?'])"},{className:"params",begin:/\|(?!=)/,end:/\|/,excludeBegin:!0,excludeEnd:!0,relevance:0,keywords:o},{begin:"("+e.RE_STARTERS_RE+"|unless)\\s*",keywords:"unless",contains:[{className:"regexp",contains:[e.BACKSLASH_ESCAPE,m],illegal:/\n/,variants:[{begin:"/",end:"/[a-z]*"},{begin:/%r\{/,end:/\}[a-z]*/},{begin:"%r\\(",end:"\\)[a-z]*"},{begin:"%r!",end:"![a-z]*"},{begin:"%r\\[",end:"\\][a-z]*"}]}].concat(d,f),relevance:0}].concat(d,f);m.contains=B,x.contains=B;const G=[{begin:/^\s*=>/,starts:{end:"$",contains:B}},{className:"meta.prompt",begin:"^("+"[>?]>"+"|"+"[\\w#]+\\(\\w+\\):\\d+:\\d+[>*]"+"|"+"(\\w+-)?\\d+\\.\\d+\\.\\d+(p\\d+)?[^\\d][^>]+>"+")(?=[ ])",starts:{end:"$",keywords:o,contains:B}}];return f.unshift(d),{name:"Ruby",aliases:["rb","gemspec","podspec","thor","irb"],keywords:o,illegal:/\/\*/,contains:[e.SHEBANG({binary:"ruby"})].concat(G).concat(f).concat(B)}}function zH(e){const t=e.regex,n=/(r#)?/,r=t.concat(n,e.UNDERSCORE_IDENT_RE),a=t.concat(n,e.IDENT_RE),s={className:"title.function.invoke",relevance:0,begin:t.concat(/\b/,/(?!let|for|while|if|else|match\b)/,a,t.lookahead(/\s*\(/))},o="([ui](8|16|32|64|128|size)|f(32|64))?",c=["abstract","as","async","await","become","box","break","const","continue","crate","do","dyn","else","enum","extern","false","final","fn","for","if","impl","in","let","loop","macro","match","mod","move","mut","override","priv","pub","ref","return","self","Self","static","struct","super","trait","true","try","type","typeof","union","unsafe","unsized","use","virtual","where","while","yield"],d=["true","false","Some","None","Ok","Err"],f=["drop ","Copy","Send","Sized","Sync","Drop","Fn","FnMut","FnOnce","ToOwned","Clone","Debug","PartialEq","PartialOrd","Eq","Ord","AsRef","AsMut","Into","From","Default","Iterator","Extend","IntoIterator","DoubleEndedIterator","ExactSizeIterator","SliceConcatExt","ToString","assert!","assert_eq!","bitflags!","bytes!","cfg!","col!","concat!","concat_idents!","debug_assert!","debug_assert_eq!","env!","eprintln!","panic!","file!","format!","format_args!","include_bytes!","include_str!","line!","local_data_key!","module_path!","option_env!","print!","println!","select!","stringify!","try!","unimplemented!","unreachable!","vec!","write!","writeln!","macro_rules!","assert_ne!","debug_assert_ne!"],m=["i8","i16","i32","i64","i128","isize","u8","u16","u32","u64","u128","usize","f32","f64","str","char","bool","Box","Option","Result","String","Vec"];return{name:"Rust",aliases:["rs"],keywords:{$pattern:e.IDENT_RE+"!?",type:m,keyword:c,literal:d,built_in:f},illegal:"</",contains:[e.C_LINE_COMMENT_MODE,e.COMMENT("/\\*","\\*/",{contains:["self"]}),e.inherit(e.QUOTE_STRING_MODE,{begin:/b?"/,illegal:null}),{className:"symbol",begin:/'[a-zA-Z_][a-zA-Z0-9_]*(?!')/},{scope:"string",variants:[{begin:/b?r(#*)"(.|\n)*?"\1(?!#)/},{begin:/b?'/,end:/'/,contains:[{scope:"char.escape",match:/\\('|\w|x\w{2}|u\w{4}|U\w{8})/}]}]},{className:"number",variants:[{begin:"\\b0b([01_]+)"+o},{begin:"\\b0o([0-7_]+)"+o},{begin:"\\b0x([A-Fa-f0-9_]+)"+o},{begin:"\\b(\\d[\\d_]*(\\.[0-9_]+)?([eE][+-]?[0-9_]+)?)"+o}],relevance:0},{begin:[/fn/,/\s+/,r],className:{1:"keyword",3:"title.function"}},{className:"meta",begin:"#!?\\[",end:"\\]",contains:[{className:"string",begin:/"/,end:/"/,contains:[e.BACKSLASH_ESCAPE]}]},{begin:[/let/,/\s+/,/(?:mut\s+)?/,r],className:{1:"keyword",3:"keyword",4:"variable"}},{begin:[/for/,/\s+/,r,/\s+/,/in/],className:{1:"keyword",3:"variable",5:"keyword"}},{begin:[/type/,/\s+/,r],className:{1:"keyword",3:"title.class"}},{begin:[/(?:trait|enum|struct|union|impl|for)/,/\s+/,r],className:{1:"keyword",3:"title.class"}},{begin:e.IDENT_RE+"::",keywords:{keyword:"Self",built_in:f,type:m}},{className:"punctuation",begin:"->"},s]}}const HH=e=>({IMPORTANT:{scope:"meta",begin:"!important"},BLOCK_COMMENT:e.C_BLOCK_COMMENT_MODE,HEXCOLOR:{scope:"number",begin:/#(([0-9a-fA-F]{3,4})|(([0-9a-fA-F]{2}){3,4}))\b/},FUNCTION_DISPATCH:{className:"built_in",begin:/[\w-]+(?=\()/},ATTRIBUTE_SELECTOR_MODE:{scope:"selector-attr",begin:/\[/,end:/\]/,illegal:"$",contains:[e.APOS_STRING_MODE,e.QUOTE_STRING_MODE]},CSS_NUMBER_MODE:{scope:"number",begin:e.NUMBER_RE+"(%|em|ex|ch|rem|vw|vh|vmin|vmax|cm|mm|in|pt|pc|px|deg|grad|rad|turn|s|ms|Hz|kHz|dpi|dpcm|dppx)?",relevance:0},CSS_VARIABLE:{className:"attr",begin:/--[A-Za-z_][A-Za-z0-9_-]*/}}),qH=["a","abbr","address","article","aside","audio","b","blockquote","body","button","canvas","caption","cite","code","dd","del","details","dfn","div","dl","dt","em","fieldset","figcaption","figure","footer","form","h1","h2","h3","h4","h5","h6","header","hgroup","html","i","iframe","img","input","ins","kbd","label","legend","li","main","mark","menu","nav","object","ol","optgroup","option","p","picture","q","quote","samp","section","select","source","span","strong","summary","sup","table","tbody","td","textarea","tfoot","th","thead","time","tr","ul","var","video"],jH=["defs","g","marker","mask","pattern","svg","switch","symbol","feBlend","feColorMatrix","feComponentTransfer","feComposite","feConvolveMatrix","feDiffuseLighting","feDisplacementMap","feFlood","feGaussianBlur","feImage","feMerge","feMorphology","feOffset","feSpecularLighting","feTile","feTurbulence","linearGradient","radialGradient","stop","circle","ellipse","image","line","path","polygon","polyline","rect","text","use","textPath","tspan","foreignObject","clipPath"],$H=[...qH,...jH],YH=["any-hover","any-pointer","aspect-ratio","color","color-gamut","color-index","device-aspect-ratio","device-height","device-width","display-mode","forced-colors","grid","height","hover","inverted-colors","monochrome","orientation","overflow-block","overflow-inline","pointer","prefers-color-scheme","prefers-contrast","prefers-reduced-motion","prefers-reduced-transparency","resolution","scan","scripting","update","width","min-width","max-width","min-height","max-height"].sort().reverse(),GH=["active","any-link","blank","checked","current","default","defined","dir","disabled","drop","empty","enabled","first","first-child","first-of-type","fullscreen","future","focus","focus-visible","focus-within","has","host","host-context","hover","indeterminate","in-range","invalid","is","lang","last-child","last-of-type","left","link","local-link","not","nth-child","nth-col","nth-last-child","nth-last-col","nth-last-of-type","nth-of-type","only-child","only-of-type","optional","out-of-range","past","placeholder-shown","read-only","read-write","required","right","root","scope","target","target-within","user-invalid","valid","visited","where"].sort().reverse(),VH=["after","backdrop","before","cue","cue-region","first-letter","first-line","grammar-error","marker","part","placeholder","selection","slotted","spelling-error"].sort().reverse(),KH=["accent-color","align-content","align-items","align-self","alignment-baseline","all","anchor-name","animation","animation-composition","animation-delay","animation-direction","animation-duration","animation-fill-mode","animation-iteration-count","animation-name","animation-play-state","animation-range","animation-range-end","animation-range-start","animation-timeline","animation-timing-function","appearance","aspect-ratio","backdrop-filter","backface-visibility","background","background-attachment","background-blend-mode","background-clip","background-color","background-image","background-origin","background-position","background-position-x","background-position-y","background-repeat","background-size","baseline-shift","block-size","border","border-block","border-block-color","border-block-end","border-block-end-color","border-block-end-style","border-block-end-width","border-block-start","border-block-start-color","border-block-start-style","border-block-start-width","border-block-style","border-block-width","border-bottom","border-bottom-color","border-bottom-left-radius","border-bottom-right-radius","border-bottom-style","border-bottom-width","border-collapse","border-color","border-end-end-radius","border-end-start-radius","border-image","border-image-outset","border-image-repeat","border-image-slice","border-image-source","border-image-width","border-inline","border-inline-color","border-inline-end","border-inline-end-color","border-inline-end-style","border-inline-end-width","border-inline-start","border-inline-start-color","border-inline-start-style","border-inline-start-width","border-inline-style","border-inline-width","border-left","border-left-color","border-left-style","border-left-width","border-radius","border-right","border-right-color","border-right-style","border-right-width","border-spacing","border-start-end-radius","border-start-start-radius","border-style","border-top","border-top-color","border-top-left-radius","border-top-right-radius","border-top-style","border-top-width","border-width","bottom","box-align","box-decoration-break","box-direction","box-flex","box-flex-group","box-lines","box-ordinal-group","box-orient","box-pack","box-shadow","box-sizing","break-after","break-before","break-inside","caption-side","caret-color","clear","clip","clip-path","clip-rule","color","color-interpolation","color-interpolation-filters","color-profile","color-rendering","color-scheme","column-count","column-fill","column-gap","column-rule","column-rule-color","column-rule-style","column-rule-width","column-span","column-width","columns","contain","contain-intrinsic-block-size","contain-intrinsic-height","contain-intrinsic-inline-size","contain-intrinsic-size","contain-intrinsic-width","container","container-name","container-type","content","content-visibility","counter-increment","counter-reset","counter-set","cue","cue-after","cue-before","cursor","cx","cy","direction","display","dominant-baseline","empty-cells","enable-background","field-sizing","fill","fill-opacity","fill-rule","filter","flex","flex-basis","flex-direction","flex-flow","flex-grow","flex-shrink","flex-wrap","float","flood-color","flood-opacity","flow","font","font-display","font-family","font-feature-settings","font-kerning","font-language-override","font-optical-sizing","font-palette","font-size","font-size-adjust","font-smooth","font-smoothing","font-stretch","font-style","font-synthesis","font-synthesis-position","font-synthesis-small-caps","font-synthesis-style","font-synthesis-weight","font-variant","font-variant-alternates","font-variant-caps","font-variant-east-asian","font-variant-emoji","font-variant-ligatures","font-variant-numeric","font-variant-position","font-variation-settings","font-weight","forced-color-adjust","gap","glyph-orientation-horizontal","glyph-orientation-vertical","grid","grid-area","grid-auto-columns","grid-auto-flow","grid-auto-rows","grid-column","grid-column-end","grid-column-start","grid-gap","grid-row","grid-row-end","grid-row-start","grid-template","grid-template-areas","grid-template-columns","grid-template-rows","hanging-punctuation","height","hyphenate-character","hyphenate-limit-chars","hyphens","icon","image-orientation","image-rendering","image-resolution","ime-mode","initial-letter","initial-letter-align","inline-size","inset","inset-area","inset-block","inset-block-end","inset-block-start","inset-inline","inset-inline-end","inset-inline-start","isolation","justify-content","justify-items","justify-self","kerning","left","letter-spacing","lighting-color","line-break","line-height","line-height-step","list-style","list-style-image","list-style-position","list-style-type","margin","margin-block","margin-block-end","margin-block-start","margin-bottom","margin-inline","margin-inline-end","margin-inline-start","margin-left","margin-right","margin-top","margin-trim","marker","marker-end","marker-mid","marker-start","marks","mask","mask-border","mask-border-mode","mask-border-outset","mask-border-repeat","mask-border-slice","mask-border-source","mask-border-width","mask-clip","mask-composite","mask-image","mask-mode","mask-origin","mask-position","mask-repeat","mask-size","mask-type","masonry-auto-flow","math-depth","math-shift","math-style","max-block-size","max-height","max-inline-size","max-width","min-block-size","min-height","min-inline-size","min-width","mix-blend-mode","nav-down","nav-index","nav-left","nav-right","nav-up","none","normal","object-fit","object-position","offset","offset-anchor","offset-distance","offset-path","offset-position","offset-rotate","opacity","order","orphans","outline","outline-color","outline-offset","outline-style","outline-width","overflow","overflow-anchor","overflow-block","overflow-clip-margin","overflow-inline","overflow-wrap","overflow-x","overflow-y","overlay","overscroll-behavior","overscroll-behavior-block","overscroll-behavior-inline","overscroll-behavior-x","overscroll-behavior-y","padding","padding-block","padding-block-end","padding-block-start","padding-bottom","padding-inline","padding-inline-end","padding-inline-start","padding-left","padding-right","padding-top","page","page-break-after","page-break-before","page-break-inside","paint-order","pause","pause-after","pause-before","perspective","perspective-origin","place-content","place-items","place-self","pointer-events","position","position-anchor","position-visibility","print-color-adjust","quotes","r","resize","rest","rest-after","rest-before","right","rotate","row-gap","ruby-align","ruby-position","scale","scroll-behavior","scroll-margin","scroll-margin-block","scroll-margin-block-end","scroll-margin-block-start","scroll-margin-bottom","scroll-margin-inline","scroll-margin-inline-end","scroll-margin-inline-start","scroll-margin-left","scroll-margin-right","scroll-margin-top","scroll-padding","scroll-padding-block","scroll-padding-block-end","scroll-padding-block-start","scroll-padding-bottom","scroll-padding-inline","scroll-padding-inline-end","scroll-padding-inline-start","scroll-padding-left","scroll-padding-right","scroll-padding-top","scroll-snap-align","scroll-snap-stop","scroll-snap-type","scroll-timeline","scroll-timeline-axis","scroll-timeline-name","scrollbar-color","scrollbar-gutter","scrollbar-width","shape-image-threshold","shape-margin","shape-outside","shape-rendering","speak","speak-as","src","stop-color","stop-opacity","stroke","stroke-dasharray","stroke-dashoffset","stroke-linecap","stroke-linejoin","stroke-miterlimit","stroke-opacity","stroke-width","tab-size","table-layout","text-align","text-align-all","text-align-last","text-anchor","text-combine-upright","text-decoration","text-decoration-color","text-decoration-line","text-decoration-skip","text-decoration-skip-ink","text-decoration-style","text-decoration-thickness","text-emphasis","text-emphasis-color","text-emphasis-position","text-emphasis-style","text-indent","text-justify","text-orientation","text-overflow","text-rendering","text-shadow","text-size-adjust","text-transform","text-underline-offset","text-underline-position","text-wrap","text-wrap-mode","text-wrap-style","timeline-scope","top","touch-action","transform","transform-box","transform-origin","transform-style","transition","transition-behavior","transition-delay","transition-duration","transition-property","transition-timing-function","translate","unicode-bidi","user-modify","user-select","vector-effect","vertical-align","view-timeline","view-timeline-axis","view-timeline-inset","view-timeline-name","view-transition-name","visibility","voice-balance","voice-duration","voice-family","voice-pitch","voice-range","voice-rate","voice-stress","voice-volume","white-space","white-space-collapse","widows","width","will-change","word-break","word-spacing","word-wrap","writing-mode","x","y","z-index","zoom"].sort().reverse();function WH(e){const t=HH(e),n=VH,r=GH,a="@[a-z-]+",s="and or not only",c={className:"variable",begin:"(\\$"+"[a-zA-Z-][a-zA-Z0-9_-]*"+")\\b",relevance:0};return{name:"SCSS",case_insensitive:!0,illegal:"[=/|']",contains:[e.C_LINE_COMMENT_MODE,e.C_BLOCK_COMMENT_MODE,t.CSS_NUMBER_MODE,{className:"selector-id",begin:"#[A-Za-z0-9_-]+",relevance:0},{className:"selector-class",begin:"\\.[A-Za-z0-9_-]+",relevance:0},t.ATTRIBUTE_SELECTOR_MODE,{className:"selector-tag",begin:"\\b("+$H.join("|")+")\\b",relevance:0},{className:"selector-pseudo",begin:":("+r.join("|")+")"},{className:"selector-pseudo",begin:":(:)?("+n.join("|")+")"},c,{begin:/\(/,end:/\)/,contains:[t.CSS_NUMBER_MODE]},t.CSS_VARIABLE,{className:"attribute",begin:"\\b("+KH.join("|")+")\\b"},{begin:"\\b(whitespace|wait|w-resize|visible|vertical-text|vertical-ideographic|uppercase|upper-roman|upper-alpha|underline|transparent|top|thin|thick|text|text-top|text-bottom|tb-rl|table-header-group|table-footer-group|sw-resize|super|strict|static|square|solid|small-caps|separate|se-resize|scroll|s-resize|rtl|row-resize|ridge|right|repeat|repeat-y|repeat-x|relative|progress|pointer|overline|outside|outset|oblique|nowrap|not-allowed|normal|none|nw-resize|no-repeat|no-drop|newspaper|ne-resize|n-resize|move|middle|medium|ltr|lr-tb|lowercase|lower-roman|lower-alpha|loose|list-item|line|line-through|line-edge|lighter|left|keep-all|justify|italic|inter-word|inter-ideograph|inside|inset|inline|inline-block|inherit|inactive|ideograph-space|ideograph-parenthesis|ideograph-numeric|ideograph-alpha|horizontal|hidden|help|hand|groove|fixed|ellipsis|e-resize|double|dotted|distribute|distribute-space|distribute-letter|distribute-all-lines|disc|disabled|default|decimal|dashed|crosshair|collapse|col-resize|circle|char|center|capitalize|break-word|break-all|bottom|both|bolder|bold|block|bidi-override|below|baseline|auto|always|all-scroll|absolute|table|table-cell)\\b"},{begin:/:/,end:/[;}{]/,relevance:0,contains:[t.BLOCK_COMMENT,c,t.HEXCOLOR,t.CSS_NUMBER_MODE,e.QUOTE_STRING_MODE,e.APOS_STRING_MODE,t.IMPORTANT,t.FUNCTION_DISPATCH]},{begin:"@(page|font-face)",keywords:{$pattern:a,keyword:"@page @font-face"}},{begin:"@",end:"[{;]",returnBegin:!0,keywords:{$pattern:/[a-z-]+/,keyword:s,attribute:YH.join(" ")},contains:[{begin:a,className:"keyword"},{begin:/[a-z-]+(?=:)/,className:"attribute"},c,e.QUOTE_STRING_MODE,e.APOS_STRING_MODE,t.HEXCOLOR,t.CSS_NUMBER_MODE]},t.FUNCTION_DISPATCH]}}function XH(e){return{name:"Shell Session",aliases:["console","shellsession"],contains:[{className:"meta.prompt",begin:/^\s{0,3}[/~\w\d[\]()@-]*[>%$#][ ]?/,starts:{end:/[^\\](?=\s*$)/,subLanguage:"bash"}}]}}function QH(e){const t=e.regex,n=e.COMMENT("--","$"),r={scope:"string",variants:[{begin:/'/,end:/'/,contains:[{match:/''/}]}]},a={begin:/"/,end:/"/,contains:[{match:/""/}]},s=["true","false","unknown"],o=["double precision","large object","with timezone","without timezone"],c=["bigint","binary","blob","boolean","char","character","clob","date","dec","decfloat","decimal","float","int","integer","interval","nchar","nclob","national","numeric","real","row","smallint","time","timestamp","varchar","varying","varbinary"],d=["add","asc","collation","desc","final","first","last","view"],f=["abs","acos","all","allocate","alter","and","any","are","array","array_agg","array_max_cardinality","as","asensitive","asin","asymmetric","at","atan","atomic","authorization","avg","begin","begin_frame","begin_partition","between","bigint","binary","blob","boolean","both","by","call","called","cardinality","cascaded","case","cast","ceil","ceiling","char","char_length","character","character_length","check","classifier","clob","close","coalesce","collate","collect","column","commit","condition","connect","constraint","contains","convert","copy","corr","corresponding","cos","cosh","count","covar_pop","covar_samp","create","cross","cube","cume_dist","current","current_catalog","current_date","current_default_transform_group","current_path","current_role","current_row","current_schema","current_time","current_timestamp","current_path","current_role","current_transform_group_for_type","current_user","cursor","cycle","date","day","deallocate","dec","decimal","decfloat","declare","default","define","delete","dense_rank","deref","describe","deterministic","disconnect","distinct","double","drop","dynamic","each","element","else","empty","end","end_frame","end_partition","end-exec","equals","escape","every","except","exec","execute","exists","exp","external","extract","false","fetch","filter","first_value","float","floor","for","foreign","frame_row","free","from","full","function","fusion","get","global","grant","group","grouping","groups","having","hold","hour","identity","in","indicator","initial","inner","inout","insensitive","insert","int","integer","intersect","intersection","interval","into","is","join","json_array","json_arrayagg","json_exists","json_object","json_objectagg","json_query","json_table","json_table_primitive","json_value","lag","language","large","last_value","lateral","lead","leading","left","like","like_regex","listagg","ln","local","localtime","localtimestamp","log","log10","lower","match","match_number","match_recognize","matches","max","member","merge","method","min","minute","mod","modifies","module","month","multiset","national","natural","nchar","nclob","new","no","none","normalize","not","nth_value","ntile","null","nullif","numeric","octet_length","occurrences_regex","of","offset","old","omit","on","one","only","open","or","order","out","outer","over","overlaps","overlay","parameter","partition","pattern","per","percent","percent_rank","percentile_cont","percentile_disc","period","portion","position","position_regex","power","precedes","precision","prepare","primary","procedure","ptf","range","rank","reads","real","recursive","ref","references","referencing","regr_avgx","regr_avgy","regr_count","regr_intercept","regr_r2","regr_slope","regr_sxx","regr_sxy","regr_syy","release","result","return","returns","revoke","right","rollback","rollup","row","row_number","rows","running","savepoint","scope","scroll","search","second","seek","select","sensitive","session_user","set","show","similar","sin","sinh","skip","smallint","some","specific","specifictype","sql","sqlexception","sqlstate","sqlwarning","sqrt","start","static","stddev_pop","stddev_samp","submultiset","subset","substring","substring_regex","succeeds","sum","symmetric","system","system_time","system_user","table","tablesample","tan","tanh","then","time","timestamp","timezone_hour","timezone_minute","to","trailing","translate","translate_regex","translation","treat","trigger","trim","trim_array","true","truncate","uescape","union","unique","unknown","unnest","update","upper","user","using","value","values","value_of","var_pop","var_samp","varbinary","varchar","varying","versioning","when","whenever","where","width_bucket","window","with","within","without","year"],m=["abs","acos","array_agg","asin","atan","avg","cast","ceil","ceiling","coalesce","corr","cos","cosh","count","covar_pop","covar_samp","cume_dist","dense_rank","deref","element","exp","extract","first_value","floor","json_array","json_arrayagg","json_exists","json_object","json_objectagg","json_query","json_table","json_table_primitive","json_value","lag","last_value","lead","listagg","ln","log","log10","lower","max","min","mod","nth_value","ntile","nullif","percent_rank","percentile_cont","percentile_disc","position","position_regex","power","rank","regr_avgx","regr_avgy","regr_count","regr_intercept","regr_r2","regr_slope","regr_sxx","regr_sxy","regr_syy","row_number","sin","sinh","sqrt","stddev_pop","stddev_samp","substring","substring_regex","sum","tan","tanh","translate","translate_regex","treat","trim","trim_array","unnest","upper","value_of","var_pop","var_samp","width_bucket"],p=["current_catalog","current_date","current_default_transform_group","current_path","current_role","current_schema","current_transform_group_for_type","current_user","session_user","system_time","system_user","current_time","localtime","current_timestamp","localtimestamp"],b=["create table","insert into","primary key","foreign key","not null","alter table","add constraint","grouping sets","on overflow","character set","respect nulls","ignore nulls","nulls first","nulls last","depth first","breadth first"],y=m,T=[...f,...d].filter(H=>!m.includes(H)),x={scope:"variable",match:/@[a-z0-9][a-z0-9_]*/},C={scope:"operator",match:/[-+*/=%^~]|&&?|\|\|?|!=?|<(?:=>?|<|>)?|>[>=]?/,relevance:0},w={match:t.concat(/\b/,t.either(...y),/\s*\(/),relevance:0,keywords:{built_in:y}};function O(H){return t.concat(/\b/,t.either(...H.map(B=>B.replace(/\s+/,"\\s+"))),/\b/)}const D={scope:"keyword",match:O(b),relevance:0};function Y(H,{exceptions:B,when:ee}={}){const Z=ee;return B=B||[],H.map(Q=>Q.match(/\|\d+$/)||B.includes(Q)?Q:Z(Q)?`${Q}|0`:Q)}return{name:"SQL",case_insensitive:!0,illegal:/[{}]|<\//,keywords:{$pattern:/\b[\w\.]+/,keyword:Y(T,{when:H=>H.length<3}),literal:s,type:c,built_in:p},contains:[{scope:"type",match:O(o)},D,w,x,r,a,e.C_NUMBER_MODE,e.C_BLOCK_COMMENT_MODE,n,C]}}function tx(e){return e?typeof e=="string"?e:e.source:null}function Oc(e){return tn("(?=",e,")")}function tn(...e){return e.map(n=>tx(n)).join("")}function ZH(e){const t=e[e.length-1];return typeof t=="object"&&t.constructor===Object?(e.splice(e.length-1,1),t):{}}function Br(...e){return"("+(ZH(e).capture?"":"?:")+e.map(r=>tx(r)).join("|")+")"}const P2=e=>tn(/\b/,e,/\w$/.test(e)?/\b/:/\B/),JH=["Protocol","Type"].map(P2),VE=["init","self"].map(P2),eq=["Any","Self"],J1=["actor","any","associatedtype","async","await",/as\?/,/as!/,"as","borrowing","break","case","catch","class","consume","consuming","continue","convenience","copy","default","defer","deinit","didSet","distributed","do","dynamic","each","else","enum","extension","fallthrough",/fileprivate\(set\)/,"fileprivate","final","for","func","get","guard","if","import","indirect","infix",/init\?/,/init!/,"inout",/internal\(set\)/,"internal","in","is","isolated","nonisolated","lazy","let","macro","mutating","nonmutating",/open\(set\)/,"open","operator","optional","override","package","postfix","precedencegroup","prefix",/private\(set\)/,"private","protocol",/public\(set\)/,"public","repeat","required","rethrows","return","set","some","static","struct","subscript","super","switch","throws","throw",/try\?/,/try!/,"try","typealias",/unowned\(safe\)/,/unowned\(unsafe\)/,"unowned","var","weak","where","while","willSet"],KE=["false","nil","true"],tq=["assignment","associativity","higherThan","left","lowerThan","none","right"],nq=["#colorLiteral","#column","#dsohandle","#else","#elseif","#endif","#error","#file","#fileID","#fileLiteral","#filePath","#function","#if","#imageLiteral","#keyPath","#line","#selector","#sourceLocation","#warning"],WE=["abs","all","any","assert","assertionFailure","debugPrint","dump","fatalError","getVaList","isKnownUniquelyReferenced","max","min","numericCast","pointwiseMax","pointwiseMin","precondition","preconditionFailure","print","readLine","repeatElement","sequence","stride","swap","swift_unboxFromSwiftValueWithType","transcode","type","unsafeBitCast","unsafeDowncast","withExtendedLifetime","withUnsafeMutablePointer","withUnsafePointer","withVaList","withoutActuallyEscaping","zip"],nx=Br(/[/=\-+!*%<>&|^~?]/,/[\u00A1-\u00A7]/,/[\u00A9\u00AB]/,/[\u00AC\u00AE]/,/[\u00B0\u00B1]/,/[\u00B6\u00BB\u00BF\u00D7\u00F7]/,/[\u2016-\u2017]/,/[\u2020-\u2027]/,/[\u2030-\u203E]/,/[\u2041-\u2053]/,/[\u2055-\u205E]/,/[\u2190-\u23FF]/,/[\u2500-\u2775]/,/[\u2794-\u2BFF]/,/[\u2E00-\u2E7F]/,/[\u3001-\u3003]/,/[\u3008-\u3020]/,/[\u3030]/),rx=Br(nx,/[\u0300-\u036F]/,/[\u1DC0-\u1DFF]/,/[\u20D0-\u20FF]/,/[\uFE00-\uFE0F]/,/[\uFE20-\uFE2F]/),eg=tn(nx,rx,"*"),ax=Br(/[a-zA-Z_]/,/[\u00A8\u00AA\u00AD\u00AF\u00B2-\u00B5\u00B7-\u00BA]/,/[\u00BC-\u00BE\u00C0-\u00D6\u00D8-\u00F6\u00F8-\u00FF]/,/[\u0100-\u02FF\u0370-\u167F\u1681-\u180D\u180F-\u1DBF]/,/[\u1E00-\u1FFF]/,/[\u200B-\u200D\u202A-\u202E\u203F-\u2040\u2054\u2060-\u206F]/,/[\u2070-\u20CF\u2100-\u218F\u2460-\u24FF\u2776-\u2793]/,/[\u2C00-\u2DFF\u2E80-\u2FFF]/,/[\u3004-\u3007\u3021-\u302F\u3031-\u303F\u3040-\uD7FF]/,/[\uF900-\uFD3D\uFD40-\uFDCF\uFDF0-\uFE1F\uFE30-\uFE44]/,/[\uFE47-\uFEFE\uFF00-\uFFFD]/),lf=Br(ax,/\d/,/[\u0300-\u036F\u1DC0-\u1DFF\u20D0-\u20FF\uFE20-\uFE2F]/),li=tn(ax,lf,"*"),gh=tn(/[A-Z]/,lf,"*"),rq=["attached","autoclosure",tn(/convention\(/,Br("swift","block","c"),/\)/),"discardableResult","dynamicCallable","dynamicMemberLookup","escaping","freestanding","frozen","GKInspectable","IBAction","IBDesignable","IBInspectable","IBOutlet","IBSegueAction","inlinable","main","nonobjc","NSApplicationMain","NSCopying","NSManaged",tn(/objc\(/,li,/\)/),"objc","objcMembers","propertyWrapper","requires_stored_property_inits","resultBuilder","Sendable","testable","UIApplicationMain","unchecked","unknown","usableFromInline","warn_unqualified_access"],aq=["iOS","iOSApplicationExtension","macOS","macOSApplicationExtension","macCatalyst","macCatalystApplicationExtension","watchOS","watchOSApplicationExtension","tvOS","tvOSApplicationExtension","swift"];function iq(e){const t={match:/\s+/,relevance:0},n=e.COMMENT("/\\*","\\*/",{contains:["self"]}),r=[e.C_LINE_COMMENT_MODE,n],a={match:[/\./,Br(...JH,...VE)],className:{2:"keyword"}},s={match:tn(/\./,Br(...J1)),relevance:0},o=J1.filter(Je=>typeof Je=="string").concat(["_|0"]),c=J1.filter(Je=>typeof Je!="string").concat(eq).map(P2),d={variants:[{className:"keyword",match:Br(...c,...VE)}]},f={$pattern:Br(/\b\w+/,/#\w+/),keyword:o.concat(nq),literal:KE},m=[a,s,d],p={match:tn(/\./,Br(...WE)),relevance:0},b={className:"built_in",match:tn(/\b/,Br(...WE),/(?=\()/)},y=[p,b],T={match:/->/,relevance:0},x={className:"operator",relevance:0,variants:[{match:eg},{match:`\\.(\\.|${rx})+`}]},C=[T,x],w="([0-9]_*)+",O="([0-9a-fA-F]_*)+",D={className:"number",relevance:0,variants:[{match:`\\b(${w})(\\.(${w}))?([eE][+-]?(${w}))?\\b`},{match:`\\b0x(${O})(\\.(${O}))?([pP][+-]?(${w}))?\\b`},{match:/\b0o([0-7]_*)+\b/},{match:/\b0b([01]_*)+\b/}]},Y=(Je="")=>({className:"subst",variants:[{match:tn(/\\/,Je,/[0\\tnr"']/)},{match:tn(/\\/,Je,/u\{[0-9a-fA-F]{1,8}\}/)}]}),H=(Je="")=>({className:"subst",match:tn(/\\/,Je,/[\t ]*(?:[\r\n]|\r\n)/)}),B=(Je="")=>({className:"subst",label:"interpol",begin:tn(/\\/,Je,/\(/),end:/\)/}),ee=(Je="")=>({begin:tn(Je,/"""/),end:tn(/"""/,Je),contains:[Y(Je),H(Je),B(Je)]}),Z=(Je="")=>({begin:tn(Je,/"/),end:tn(/"/,Je),contains:[Y(Je),B(Je)]}),Q={className:"string",variants:[ee(),ee("#"),ee("##"),ee("###"),Z(),Z("#"),Z("##"),Z("###")]},G=[e.BACKSLASH_ESCAPE,{begin:/\[/,end:/\]/,relevance:0,contains:[e.BACKSLASH_ESCAPE]}],se={begin:/\/[^\s](?=[^/\n]*\/)/,end:/\//,contains:G},oe=Je=>{const yt=tn(Je,/\//),mn=tn(/\//,Je);return{begin:yt,end:mn,contains:[...G,{scope:"comment",begin:`#(?!.*${mn})`,end:/$/}]}},ye={scope:"regexp",variants:[oe("###"),oe("##"),oe("#"),se]},fe={match:tn(/`/,li,/`/)},de={className:"variable",match:/\$\d+/},V={className:"variable",match:`\\$${lf}+`},me=[fe,de,V],pe={match:/(@|#(un)?)available/,scope:"keyword",starts:{contains:[{begin:/\(/,end:/\)/,keywords:aq,contains:[...C,D,Q]}]}},Be={scope:"keyword",match:tn(/@/,Br(...rq),Oc(Br(/\(/,/\s+/)))},k={scope:"meta",match:tn(/@/,li)},ie=[pe,Be,k],ge={match:Oc(/\b[A-Z]/),relevance:0,contains:[{className:"type",match:tn(/(AV|CA|CF|CG|CI|CL|CM|CN|CT|MK|MP|MTK|MTL|NS|SCN|SK|UI|WK|XC)/,lf,"+")},{className:"type",match:gh,relevance:0},{match:/[?!]+/,relevance:0},{match:/\.\.\./,relevance:0},{match:tn(/\s+&\s+/,Oc(gh)),relevance:0}]},L={begin:/</,end:/>/,keywords:f,contains:[...r,...m,...ie,T,ge]};ge.contains.push(L);const Oe={match:tn(li,/\s*:/),keywords:"_|0",relevance:0},je={begin:/\(/,end:/\)/,relevance:0,keywords:f,contains:["self",Oe,...r,ye,...m,...y,...C,D,Q,...me,...ie,ge]},De={begin:/</,end:/>/,keywords:"repeat each",contains:[...r,ge]},Ot={begin:Br(Oc(tn(li,/\s*:/)),Oc(tn(li,/\s+/,li,/\s*:/))),end:/:/,relevance:0,contains:[{className:"keyword",match:/\b_\b/},{className:"params",match:li}]},ot={begin:/\(/,end:/\)/,keywords:f,contains:[Ot,...r,...m,...C,D,Q,...ie,ge,je],endsParent:!0,illegal:/["']/},zt={match:[/(func|macro)/,/\s+/,Br(fe.match,li,eg)],className:{1:"keyword",3:"title.function"},contains:[De,ot,t],illegal:[/\[/,/%/]},wt={match:[/\b(?:subscript|init[?!]?)/,/\s*(?=[<(])/],className:{1:"keyword"},contains:[De,ot,t],illegal:/\[|%/},vn={match:[/operator/,/\s+/,eg],className:{1:"keyword",3:"title"}},mr={begin:[/precedencegroup/,/\s+/,gh],className:{1:"keyword",3:"title"},contains:[ge],keywords:[...tq,...KE],end:/}/},vt={match:[/class\b/,/\s+/,/func\b/,/\s+/,/\b[A-Za-z_][A-Za-z0-9_]*\b/],scope:{1:"keyword",3:"keyword",5:"title.function"}},gt={match:[/class\b/,/\s+/,/var\b/],scope:{1:"keyword",3:"keyword"}},Bt={begin:[/(struct|protocol|class|extension|enum|actor)/,/\s+/,li,/\s*/],beginScope:{1:"keyword",3:"title.class"},keywords:f,contains:[De,...m,{begin:/:/,end:/\{/,keywords:f,contains:[{scope:"title.class.inherited",match:gh},...m],relevance:0}]};for(const Je of Q.variants){const yt=Je.contains.find(Lt=>Lt.label==="interpol");yt.keywords=f;const mn=[...m,...y,...C,D,Q,...me];yt.contains=[...mn,{begin:/\(/,end:/\)/,contains:["self",...mn]}]}return{name:"Swift",keywords:f,contains:[...r,zt,wt,vt,gt,Bt,vn,mr,{beginKeywords:"import",end:/$/,contains:[...r],relevance:0},ye,...m,...y,...C,D,Q,...me,...ie,ge,je]}}const of="[A-Za-z$_][0-9A-Za-z$_]*",ix=["as","in","of","if","for","while","finally","var","new","function","do","return","void","else","break","catch","instanceof","with","throw","case","default","try","switch","continue","typeof","delete","let","yield","const","class","debugger","async","await","static","import","from","export","extends","using"],sx=["true","false","null","undefined","NaN","Infinity"],lx=["Object","Function","Boolean","Symbol","Math","Date","Number","BigInt","String","RegExp","Array","Float32Array","Float64Array","Int8Array","Uint8Array","Uint8ClampedArray","Int16Array","Int32Array","Uint16Array","Uint32Array","BigInt64Array","BigUint64Array","Set","Map","WeakSet","WeakMap","ArrayBuffer","SharedArrayBuffer","Atomics","DataView","JSON","Promise","Generator","GeneratorFunction","AsyncFunction","Reflect","Proxy","Intl","WebAssembly"],ox=["Error","EvalError","InternalError","RangeError","ReferenceError","SyntaxError","TypeError","URIError"],ux=["setInterval","setTimeout","clearInterval","clearTimeout","require","exports","eval","isFinite","isNaN","parseFloat","parseInt","decodeURI","decodeURIComponent","encodeURI","encodeURIComponent","escape","unescape"],cx=["arguments","this","super","console","window","document","localStorage","sessionStorage","module","global"],dx=[].concat(ux,lx,ox);function sq(e){const t=e.regex,n=(pe,{after:Be})=>{const k="</"+pe[0].slice(1);return pe.input.indexOf(k,Be)!==-1},r=of,a={begin:"<>",end:"</>"},s=/<[A-Za-z0-9\\._:-]+\s*\/>/,o={begin:/<[A-Za-z0-9\\._:-]+/,end:/\/[A-Za-z0-9\\._:-]+>|\/>/,isTrulyOpeningTag:(pe,Be)=>{const k=pe[0].length+pe.index,ie=pe.input[k];if(ie==="<"||ie===","){Be.ignoreMatch();return}ie===">"&&(n(pe,{after:k})||Be.ignoreMatch());let ge;const L=pe.input.substring(k);if(ge=L.match(/^\s*=/)){Be.ignoreMatch();return}if((ge=L.match(/^\s+extends\s+/))&&ge.index===0){Be.ignoreMatch();return}}},c={$pattern:of,keyword:ix,literal:sx,built_in:dx,"variable.language":cx},d="[0-9](_?[0-9])*",f=`\\.(${d})`,m="0|[1-9](_?[0-9])*|0[0-7]*[89][0-9]*",p={className:"number",variants:[{begin:`(\\b(${m})((${f})|\\.)?|(${f}))[eE][+-]?(${d})\\b`},{begin:`\\b(${m})\\b((${f})\\b|\\.)?|(${f})\\b`},{begin:"\\b(0|[1-9](_?[0-9])*)n\\b"},{begin:"\\b0[xX][0-9a-fA-F](_?[0-9a-fA-F])*n?\\b"},{begin:"\\b0[bB][0-1](_?[0-1])*n?\\b"},{begin:"\\b0[oO][0-7](_?[0-7])*n?\\b"},{begin:"\\b0[0-7]+n?\\b"}],relevance:0},b={className:"subst",begin:"\\$\\{",end:"\\}",keywords:c,contains:[]},y={begin:".?html`",end:"",starts:{end:"`",returnEnd:!1,contains:[e.BACKSLASH_ESCAPE,b],subLanguage:"xml"}},T={begin:".?css`",end:"",starts:{end:"`",returnEnd:!1,contains:[e.BACKSLASH_ESCAPE,b],subLanguage:"css"}},x={begin:".?gql`",end:"",starts:{end:"`",returnEnd:!1,contains:[e.BACKSLASH_ESCAPE,b],subLanguage:"graphql"}},C={className:"string",begin:"`",end:"`",contains:[e.BACKSLASH_ESCAPE,b]},O={className:"comment",variants:[e.COMMENT(/\/\*\*(?!\/)/,"\\*/",{relevance:0,contains:[{begin:"(?=@[A-Za-z]+)",relevance:0,contains:[{className:"doctag",begin:"@[A-Za-z]+"},{className:"type",begin:"\\{",end:"\\}",excludeEnd:!0,excludeBegin:!0,relevance:0},{className:"variable",begin:r+"(?=\\s*(-)|$)",endsParent:!0,relevance:0},{begin:/(?=[^\n])\s/,relevance:0}]}]}),e.C_BLOCK_COMMENT_MODE,e.C_LINE_COMMENT_MODE]},D=[e.APOS_STRING_MODE,e.QUOTE_STRING_MODE,y,T,x,C,{match:/\$\d+/},p];b.contains=D.concat({begin:/\{/,end:/\}/,keywords:c,contains:["self"].concat(D)});const Y=[].concat(O,b.contains),H=Y.concat([{begin:/(\s*)\(/,end:/\)/,keywords:c,contains:["self"].concat(Y)}]),B={className:"params",begin:/(\s*)\(/,end:/\)/,excludeBegin:!0,excludeEnd:!0,keywords:c,contains:H},ee={variants:[{match:[/class/,/\s+/,r,/\s+/,/extends/,/\s+/,t.concat(r,"(",t.concat(/\./,r),")*")],scope:{1:"keyword",3:"title.class",5:"keyword",7:"title.class.inherited"}},{match:[/class/,/\s+/,r],scope:{1:"keyword",3:"title.class"}}]},Z={relevance:0,match:t.either(/\bJSON/,/\b[A-Z][a-z]+([A-Z][a-z]*|\d)*/,/\b[A-Z]{2,}([A-Z][a-z]+|\d)+([A-Z][a-z]*)*/,/\b[A-Z]{2,}[a-z]+([A-Z][a-z]+|\d)*([A-Z][a-z]*)*/),className:"title.class",keywords:{_:[...lx,...ox]}},Q={label:"use_strict",className:"meta",relevance:10,begin:/^\s*['"]use (strict|asm)['"]/},G={variants:[{match:[/function/,/\s+/,r,/(?=\s*\()/]},{match:[/function/,/\s*(?=\()/]}],className:{1:"keyword",3:"title.function"},label:"func.def",contains:[B],illegal:/%/},se={relevance:0,match:/\b[A-Z][A-Z_0-9]+\b/,className:"variable.constant"};function oe(pe){return t.concat("(?!",pe.join("|"),")")}const ye={match:t.concat(/\b/,oe([...ux,"super","import"].map(pe=>`${pe}\\s*\\(`)),r,t.lookahead(/\s*\(/)),className:"title.function",relevance:0},fe={begin:t.concat(/\./,t.lookahead(t.concat(r,/(?![0-9A-Za-z$_(])/))),end:r,excludeBegin:!0,keywords:"prototype",className:"property",relevance:0},de={match:[/get|set/,/\s+/,r,/(?=\()/],className:{1:"keyword",3:"title.function"},contains:[{begin:/\(\)/},B]},V="(\\([^()]*(\\([^()]*(\\([^()]*\\)[^()]*)*\\)[^()]*)*\\)|"+e.UNDERSCORE_IDENT_RE+")\\s*=>",me={match:[/const|var|let/,/\s+/,r,/\s*/,/=\s*/,/(async\s*)?/,t.lookahead(V)],keywords:"async",className:{1:"keyword",3:"title.function"},contains:[B]};return{name:"JavaScript",aliases:["js","jsx","mjs","cjs"],keywords:c,exports:{PARAMS_CONTAINS:H,CLASS_REFERENCE:Z},illegal:/#(?![$_A-z])/,contains:[e.SHEBANG({label:"shebang",binary:"node",relevance:5}),Q,e.APOS_STRING_MODE,e.QUOTE_STRING_MODE,y,T,x,C,O,{match:/\$\d+/},p,Z,{scope:"attr",match:r+t.lookahead(":"),relevance:0},me,{begin:"("+e.RE_STARTERS_RE+"|\\b(case|return|throw)\\b)\\s*",keywords:"return throw case",relevance:0,contains:[O,e.REGEXP_MODE,{className:"function",begin:V,returnBegin:!0,end:"\\s*=>",contains:[{className:"params",variants:[{begin:e.UNDERSCORE_IDENT_RE,relevance:0},{className:null,begin:/\(\s*\)/,skip:!0},{begin:/(\s*)\(/,end:/\)/,excludeBegin:!0,excludeEnd:!0,keywords:c,contains:H}]}]},{begin:/,/,relevance:0},{match:/\s+/,relevance:0},{variants:[{begin:a.begin,end:a.end},{match:s},{begin:o.begin,"on:begin":o.isTrulyOpeningTag,end:o.end}],subLanguage:"xml",contains:[{begin:o.begin,end:o.end,skip:!0,contains:["self"]}]}]},G,{beginKeywords:"while if switch catch for"},{begin:"\\b(?!function)"+e.UNDERSCORE_IDENT_RE+"\\([^()]*(\\([^()]*(\\([^()]*\\)[^()]*)*\\)[^()]*)*\\)\\s*\\{",returnBegin:!0,label:"func.def",contains:[B,e.inherit(e.TITLE_MODE,{begin:r,className:"title.function"})]},{match:/\.\.\./,relevance:0},fe,{match:"\\$"+r,relevance:0},{match:[/\bconstructor(?=\s*\()/],className:{1:"title.function"},contains:[B]},ye,se,ee,de,{match:/\$[(.]/}]}}function lq(e){const t=e.regex,n=sq(e),r=of,a=["any","void","number","boolean","string","object","never","symbol","bigint","unknown"],s={begin:[/namespace/,/\s+/,e.IDENT_RE],beginScope:{1:"keyword",3:"title.class"}},o={beginKeywords:"interface",end:/\{/,excludeEnd:!0,keywords:{keyword:"interface extends",built_in:a},contains:[n.exports.CLASS_REFERENCE]},c={className:"meta",relevance:10,begin:/^\s*['"]use strict['"]/},d=["type","interface","public","private","protected","implements","declare","abstract","readonly","enum","override","satisfies"],f={$pattern:of,keyword:ix.concat(d),literal:sx,built_in:dx.concat(a),"variable.language":cx},m={className:"meta",begin:"@"+r},p=(x,C,w)=>{const O=x.contains.findIndex(D=>D.label===C);if(O===-1)throw new Error("can not find mode to replace");x.contains.splice(O,1,w)};Object.assign(n.keywords,f),n.exports.PARAMS_CONTAINS.push(m);const b=n.contains.find(x=>x.scope==="attr"),y=Object.assign({},b,{match:t.concat(r,t.lookahead(/\s*\?:/))});n.exports.PARAMS_CONTAINS.push([n.exports.CLASS_REFERENCE,b,y]),n.contains=n.contains.concat([m,s,o,y]),p(n,"shebang",e.SHEBANG()),p(n,"use_strict",c);const T=n.contains.find(x=>x.label==="func.def");return T.relevance=0,Object.assign(n,{name:"TypeScript",aliases:["ts","tsx","mts","cts"]}),n}function oq(e){const t=e.regex,n={className:"string",begin:/"(""|[^/n])"C\b/},r={className:"string",begin:/"/,end:/"/,illegal:/\n/,contains:[{begin:/""/}]},a=/\d{1,2}\/\d{1,2}\/\d{4}/,s=/\d{4}-\d{1,2}-\d{1,2}/,o=/(\d|1[012])(:\d+){0,2} *(AM|PM)/,c=/\d{1,2}(:\d{1,2}){1,2}/,d={className:"literal",variants:[{begin:t.concat(/# */,t.either(s,a),/ *#/)},{begin:t.concat(/# */,c,/ *#/)},{begin:t.concat(/# */,o,/ *#/)},{begin:t.concat(/# */,t.either(s,a),/ +/,t.either(o,c),/ *#/)}]},f={className:"number",relevance:0,variants:[{begin:/\b\d[\d_]*((\.[\d_]+(E[+-]?[\d_]+)?)|(E[+-]?[\d_]+))[RFD@!#]?/},{begin:/\b\d[\d_]*((U?[SIL])|[%&])?/},{begin:/&H[\dA-F_]+((U?[SIL])|[%&])?/},{begin:/&O[0-7_]+((U?[SIL])|[%&])?/},{begin:/&B[01_]+((U?[SIL])|[%&])?/}]},m={className:"label",begin:/^\w+:/},p=e.COMMENT(/'''/,/$/,{contains:[{className:"doctag",begin:/<\/?/,end:/>/}]}),b=e.COMMENT(null,/$/,{variants:[{begin:/'/},{begin:/([\t ]|^)REM(?=\s)/}]});return{name:"Visual Basic .NET",aliases:["vb"],case_insensitive:!0,classNameAliases:{label:"symbol"},keywords:{keyword:"addhandler alias aggregate ansi as async assembly auto binary by byref byval call case catch class compare const continue custom declare default delegate dim distinct do each equals else elseif end enum erase error event exit explicit finally for friend from function get global goto group handles if implements imports in inherits interface into iterator join key let lib loop me mid module mustinherit mustoverride mybase myclass namespace narrowing new next notinheritable notoverridable of off on operator option optional order overloads overridable overrides paramarray partial preserve private property protected public raiseevent readonly redim removehandler resume return select set shadows shared skip static step stop structure strict sub synclock take text then throw to try unicode until using when where while widening with withevents writeonly yield",built_in:"addressof and andalso await directcast gettype getxmlnamespace is isfalse isnot istrue like mod nameof new not or orelse trycast typeof xor cbool cbyte cchar cdate cdbl cdec cint clng cobj csbyte cshort csng cstr cuint culng cushort",type:"boolean byte char date decimal double integer long object sbyte short single string uinteger ulong ushort",literal:"true false nothing"},illegal:"//|\\{|\\}|endif|gosub|variant|wend|^\\$ ",contains:[n,r,d,f,m,p,b,{className:"meta",begin:/[\t ]*#(const|disable|else|elseif|enable|end|externalsource|if|region)\b/,end:/$/,keywords:{keyword:"const disable else elseif enable end externalsource if region then"},contains:[b]}]}}function uq(e){e.regex;const t=e.COMMENT(/\(;/,/;\)/);t.contains.push("self");const n=e.COMMENT(/;;/,/$/),r=["anyfunc","block","br","br_if","br_table","call","call_indirect","data","drop","elem","else","end","export","func","global.get","global.set","local.get","local.set","local.tee","get_global","get_local","global","if","import","local","loop","memory","memory.grow","memory.size","module","mut","nop","offset","param","result","return","select","set_global","set_local","start","table","tee_local","then","type","unreachable"],a={begin:[/(?:func|call|call_indirect)/,/\s+/,/\$[^\s)]+/],className:{1:"keyword",3:"title.function"}},s={className:"variable",begin:/\$[\w_]+/},o={match:/(\((?!;)|\))+/,className:"punctuation",relevance:0},c={className:"number",relevance:0,match:/[+-]?\b(?:\d(?:_?\d)*(?:\.\d(?:_?\d)*)?(?:[eE][+-]?\d(?:_?\d)*)?|0x[\da-fA-F](?:_?[\da-fA-F])*(?:\.[\da-fA-F](?:_?[\da-fA-D])*)?(?:[pP][+-]?\d(?:_?\d)*)?)\b|\binf\b|\bnan(?::0x[\da-fA-F](?:_?[\da-fA-D])*)?\b/},d={match:/(i32|i64|f32|f64)(?!\.)/,className:"type"},f={className:"keyword",match:/\b(f32|f64|i32|i64)(?:\.(?:abs|add|and|ceil|clz|const|convert_[su]\/i(?:32|64)|copysign|ctz|demote\/f64|div(?:_[su])?|eqz?|extend_[su]\/i32|floor|ge(?:_[su])?|gt(?:_[su])?|le(?:_[su])?|load(?:(?:8|16|32)_[su])?|lt(?:_[su])?|max|min|mul|nearest|neg?|or|popcnt|promote\/f32|reinterpret\/[fi](?:32|64)|rem_[su]|rot[lr]|shl|shr_[su]|store(?:8|16|32)?|sqrt|sub|trunc(?:_[su]\/f(?:32|64))?|wrap\/i64|xor))\b/};return{name:"WebAssembly",keywords:{$pattern:/[\w.]+/,keyword:r},contains:[n,t,{match:[/(?:offset|align)/,/\s*/,/=/],className:{1:"keyword",3:"operator"}},s,o,a,e.QUOTE_STRING_MODE,d,f,c]}}function cq(e){const t=e.regex,n=t.concat(/[\p{L}_]/u,t.optional(/[\p{L}0-9_.-]*:/u),/[\p{L}0-9_.-]*/u),r=/[\p{L}0-9._:-]+/u,a={className:"symbol",begin:/&[a-z]+;|&#[0-9]+;|&#x[a-f0-9]+;/},s={begin:/\s/,contains:[{className:"keyword",begin:/#?[a-z_][a-z1-9_-]+/,illegal:/\n/}]},o=e.inherit(s,{begin:/\(/,end:/\)/}),c=e.inherit(e.APOS_STRING_MODE,{className:"string"}),d=e.inherit(e.QUOTE_STRING_MODE,{className:"string"}),f={endsWithParent:!0,illegal:/</,relevance:0,contains:[{className:"attr",begin:r,relevance:0},{begin:/=\s*/,relevance:0,contains:[{className:"string",endsParent:!0,variants:[{begin:/"/,end:/"/,contains:[a]},{begin:/'/,end:/'/,contains:[a]},{begin:/[^\s"'=<>`]+/}]}]}]};return{name:"HTML, XML",aliases:["html","xhtml","rss","atom","xjb","xsd","xsl","plist","wsf","svg"],case_insensitive:!0,unicodeRegex:!0,contains:[{className:"meta",begin:/<![a-z]/,end:/>/,relevance:10,contains:[s,d,c,o,{begin:/\[/,end:/\]/,contains:[{className:"meta",begin:/<![a-z]/,end:/>/,contains:[s,o,d,c]}]}]},e.COMMENT(/<!--/,/-->/,{relevance:10}),{begin:/<!\[CDATA\[/,end:/\]\]>/,relevance:10},a,{className:"meta",end:/\?>/,variants:[{begin:/<\?xml/,relevance:10,contains:[d]},{begin:/<\?[a-z][a-z0-9]+/}]},{className:"tag",begin:/<style(?=\s|>)/,end:/>/,keywords:{name:"style"},contains:[f],starts:{end:/<\/style>/,returnEnd:!0,subLanguage:["css","xml"]}},{className:"tag",begin:/<script(?=\s|>)/,end:/>/,keywords:{name:"script"},contains:[f],starts:{end:/<\/script>/,returnEnd:!0,subLanguage:["javascript","handlebars","xml"]}},{className:"tag",begin:/<>|<\/>/},{className:"tag",begin:t.concat(/</,t.lookahead(t.concat(n,t.either(/\/>/,/>/,/\s/)))),end:/\/?>/,contains:[{className:"name",begin:n,relevance:0,starts:f}]},{className:"tag",begin:t.concat(/<\//,t.lookahead(t.concat(n,/>/))),contains:[{className:"name",begin:n,relevance:0},{begin:/>/,relevance:0,endsParent:!0}]}]}}function dq(e){const t="true false yes no null",n="[\\w#;/?:@&=+$,.~*'()[\\]]+",r={className:"attr",variants:[{begin:/[\w*@][\w*@ :()\./-]*:(?=[ \t]|$)/},{begin:/"[\w*@][\w*@ :()\./-]*":(?=[ \t]|$)/},{begin:/'[\w*@][\w*@ :()\./-]*':(?=[ \t]|$)/}]},a={className:"template-variable",variants:[{begin:/\{\{/,end:/\}\}/},{begin:/%\{/,end:/\}/}]},s={className:"string",relevance:0,begin:/'/,end:/'/,contains:[{match:/''/,scope:"char.escape",relevance:0}]},o={className:"string",relevance:0,variants:[{begin:/"/,end:/"/},{begin:/\S+/}],contains:[e.BACKSLASH_ESCAPE,a]},c=e.inherit(o,{variants:[{begin:/'/,end:/'/,contains:[{begin:/''/,relevance:0}]},{begin:/"/,end:/"/},{begin:/[^\s,{}[\]]+/}]}),b={className:"number",begin:"\\b"+"[0-9]{4}(-[0-9][0-9]){0,2}"+"([Tt \\t][0-9][0-9]?(:[0-9][0-9]){2})?"+"(\\.[0-9]*)?"+"([ \\t])*(Z|[-+][0-9][0-9]?(:[0-9][0-9])?)?"+"\\b"},y={end:",",endsWithParent:!0,excludeEnd:!0,keywords:t,relevance:0},T={begin:/\{/,end:/\}/,contains:[y],illegal:"\\n",relevance:0},x={begin:"\\[",end:"\\]",contains:[y],illegal:"\\n",relevance:0},C=[r,{className:"meta",begin:"^---\\s*$",relevance:10},{className:"string",begin:"[\\|>]([1-9]?[+-])?[ ]*\\n( +)[^ ][^\\n]*\\n(\\2[^\\n]+\\n?)*"},{begin:"<%[%=-]?",end:"[%-]?%>",subLanguage:"ruby",excludeBegin:!0,excludeEnd:!0,relevance:0},{className:"type",begin:"!\\w+!"+n},{className:"type",begin:"!<"+n+">"},{className:"type",begin:"!"+n},{className:"type",begin:"!!"+n},{className:"meta",begin:"&"+e.UNDERSCORE_IDENT_RE+"$"},{className:"meta",begin:"\\*"+e.UNDERSCORE_IDENT_RE+"$"},{className:"bullet",begin:"-(?=[ ]|$)",relevance:0},e.HASH_COMMENT_MODE,{beginKeywords:t,keywords:{literal:t}},b,{className:"number",begin:e.C_NUMBER_RE+"\\b",relevance:0},T,x,s,o],w=[...C];return w.pop(),w.push(c),y.contains=w,{name:"YAML",case_insensitive:!0,aliases:["yml"],contains:C}}const hq={arduino:Vz,bash:Kz,c:Wz,cpp:Xz,csharp:Qz,css:sH,diff:lH,go:oH,graphql:uH,ini:cH,java:dH,javascript:gH,json:bH,kotlin:yH,less:kH,lua:CH,makefile:NH,markdown:OH,objectivec:RH,perl:MH,php:IH,"php-template":DH,plaintext:LH,python:PH,"python-repl":BH,r:UH,ruby:FH,rust:zH,scss:WH,shell:XH,sql:QH,swift:iq,typescript:lq,vbnet:oq,wasm:uq,xml:cq,yaml:dq};var tg,XE;function fq(){if(XE)return tg;XE=1;function e(K){return K instanceof Map?K.clear=K.delete=K.set=function(){throw new Error("map is read-only")}:K instanceof Set&&(K.add=K.clear=K.delete=function(){throw new Error("set is read-only")}),Object.freeze(K),Object.getOwnPropertyNames(K).forEach(Ee=>{const Fe=K[Ee],ht=typeof Fe;(ht==="object"||ht==="function")&&!Object.isFrozen(Fe)&&e(Fe)}),K}class t{constructor(Ee){Ee.data===void 0&&(Ee.data={}),this.data=Ee.data,this.isMatchIgnored=!1}ignoreMatch(){this.isMatchIgnored=!0}}function n(K){return K.replace(/&/g,"&amp;").replace(/</g,"&lt;").replace(/>/g,"&gt;").replace(/"/g,"&quot;").replace(/'/g,"&#x27;")}function r(K,...Ee){const Fe=Object.create(null);for(const ht in K)Fe[ht]=K[ht];return Ee.forEach(function(ht){for(const yn in ht)Fe[yn]=ht[yn]}),Fe}const a="</span>",s=K=>!!K.scope,o=(K,{prefix:Ee})=>{if(K.startsWith("language:"))return K.replace("language:","language-");if(K.includes(".")){const Fe=K.split(".");return[`${Ee}${Fe.shift()}`,...Fe.map((ht,yn)=>`${ht}${"_".repeat(yn+1)}`)].join(" ")}return`${Ee}${K}`};class c{constructor(Ee,Fe){this.buffer="",this.classPrefix=Fe.classPrefix,Ee.walk(this)}addText(Ee){this.buffer+=n(Ee)}openNode(Ee){if(!s(Ee))return;const Fe=o(Ee.scope,{prefix:this.classPrefix});this.span(Fe)}closeNode(Ee){s(Ee)&&(this.buffer+=a)}value(){return this.buffer}span(Ee){this.buffer+=`<span class="${Ee}">`}}const d=(K={})=>{const Ee={children:[]};return Object.assign(Ee,K),Ee};class f{constructor(){this.rootNode=d(),this.stack=[this.rootNode]}get top(){return this.stack[this.stack.length-1]}get root(){return this.rootNode}add(Ee){this.top.children.push(Ee)}openNode(Ee){const Fe=d({scope:Ee});this.add(Fe),this.stack.push(Fe)}closeNode(){if(this.stack.length>1)return this.stack.pop()}closeAllNodes(){for(;this.closeNode(););}toJSON(){return JSON.stringify(this.rootNode,null,4)}walk(Ee){return this.constructor._walk(Ee,this.rootNode)}static _walk(Ee,Fe){return typeof Fe=="string"?Ee.addText(Fe):Fe.children&&(Ee.openNode(Fe),Fe.children.forEach(ht=>this._walk(Ee,ht)),Ee.closeNode(Fe)),Ee}static _collapse(Ee){typeof Ee!="string"&&Ee.children&&(Ee.children.every(Fe=>typeof Fe=="string")?Ee.children=[Ee.children.join("")]:Ee.children.forEach(Fe=>{f._collapse(Fe)}))}}class m extends f{constructor(Ee){super(),this.options=Ee}addText(Ee){Ee!==""&&this.add(Ee)}startScope(Ee){this.openNode(Ee)}endScope(){this.closeNode()}__addSublanguage(Ee,Fe){const ht=Ee.root;Fe&&(ht.scope=`language:${Fe}`),this.add(ht)}toHTML(){return new c(this,this.options).value()}finalize(){return this.closeAllNodes(),!0}}function p(K){return K?typeof K=="string"?K:K.source:null}function b(K){return x("(?=",K,")")}function y(K){return x("(?:",K,")*")}function T(K){return x("(?:",K,")?")}function x(...K){return K.map(Fe=>p(Fe)).join("")}function C(K){const Ee=K[K.length-1];return typeof Ee=="object"&&Ee.constructor===Object?(K.splice(K.length-1,1),Ee):{}}function w(...K){return"("+(C(K).capture?"":"?:")+K.map(ht=>p(ht)).join("|")+")"}function O(K){return new RegExp(K.toString()+"|").exec("").length-1}function D(K,Ee){const Fe=K&&K.exec(Ee);return Fe&&Fe.index===0}const Y=/\[(?:[^\\\]]|\\.)*\]|\(\??|\\([1-9][0-9]*)|\\./;function H(K,{joinWith:Ee}){let Fe=0;return K.map(ht=>{Fe+=1;const yn=Fe;let wn=p(ht),Ve="";for(;wn.length>0;){const Pe=Y.exec(wn);if(!Pe){Ve+=wn;break}Ve+=wn.substring(0,Pe.index),wn=wn.substring(Pe.index+Pe[0].length),Pe[0][0]==="\\"&&Pe[1]?Ve+="\\"+String(Number(Pe[1])+yn):(Ve+=Pe[0],Pe[0]==="("&&Fe++)}return Ve}).map(ht=>`(${ht})`).join(Ee)}const B=/\b\B/,ee="[a-zA-Z]\\w*",Z="[a-zA-Z_]\\w*",Q="\\b\\d+(\\.\\d+)?",G="(-?)(\\b0[xX][a-fA-F0-9]+|(\\b\\d+(\\.\\d*)?|\\.\\d+)([eE][-+]?\\d+)?)",se="\\b(0b[01]+)",oe="!|!=|!==|%|%=|&|&&|&=|\\*|\\*=|\\+|\\+=|,|-|-=|/=|/|:|;|<<|<<=|<=|<|===|==|=|>>>=|>>=|>=|>>>|>>|>|\\?|\\[|\\{|\\(|\\^|\\^=|\\||\\|=|\\|\\||~",ye=(K={})=>{const Ee=/^#![ ]*\//;return K.binary&&(K.begin=x(Ee,/.*\b/,K.binary,/\b.*/)),r({scope:"meta",begin:Ee,end:/$/,relevance:0,"on:begin":(Fe,ht)=>{Fe.index!==0&&ht.ignoreMatch()}},K)},fe={begin:"\\\\[\\s\\S]",relevance:0},de={scope:"string",begin:"'",end:"'",illegal:"\\n",contains:[fe]},V={scope:"string",begin:'"',end:'"',illegal:"\\n",contains:[fe]},me={begin:/\b(a|an|the|are|I'm|isn't|don't|doesn't|won't|but|just|should|pretty|simply|enough|gonna|going|wtf|so|such|will|you|your|they|like|more)\b/},pe=function(K,Ee,Fe={}){const ht=r({scope:"comment",begin:K,end:Ee,contains:[]},Fe);ht.contains.push({scope:"doctag",begin:"[ ]*(?=(TODO|FIXME|NOTE|BUG|OPTIMIZE|HACK|XXX):)",end:/(TODO|FIXME|NOTE|BUG|OPTIMIZE|HACK|XXX):/,excludeBegin:!0,relevance:0});const yn=w("I","a","is","so","us","to","at","if","in","it","on",/[A-Za-z]+['](d|ve|re|ll|t|s|n)/,/[A-Za-z]+[-][a-z]+/,/[A-Za-z][a-z]{2,}/);return ht.contains.push({begin:x(/[ ]+/,"(",yn,/[.]?[:]?([.][ ]|[ ])/,"){3}")}),ht},Be=pe("//","$"),k=pe("/\\*","\\*/"),ie=pe("#","$"),ge={scope:"number",begin:Q,relevance:0},L={scope:"number",begin:G,relevance:0},Oe={scope:"number",begin:se,relevance:0},je={scope:"regexp",begin:/\/(?=[^/\n]*\/)/,end:/\/[gimuy]*/,contains:[fe,{begin:/\[/,end:/\]/,relevance:0,contains:[fe]}]},De={scope:"title",begin:ee,relevance:0},Ot={scope:"title",begin:Z,relevance:0},ot={begin:"\\.\\s*"+Z,relevance:0};var wt=Object.freeze({__proto__:null,APOS_STRING_MODE:de,BACKSLASH_ESCAPE:fe,BINARY_NUMBER_MODE:Oe,BINARY_NUMBER_RE:se,COMMENT:pe,C_BLOCK_COMMENT_MODE:k,C_LINE_COMMENT_MODE:Be,C_NUMBER_MODE:L,C_NUMBER_RE:G,END_SAME_AS_BEGIN:function(K){return Object.assign(K,{"on:begin":(Ee,Fe)=>{Fe.data._beginMatch=Ee[1]},"on:end":(Ee,Fe)=>{Fe.data._beginMatch!==Ee[1]&&Fe.ignoreMatch()}})},HASH_COMMENT_MODE:ie,IDENT_RE:ee,MATCH_NOTHING_RE:B,METHOD_GUARD:ot,NUMBER_MODE:ge,NUMBER_RE:Q,PHRASAL_WORDS_MODE:me,QUOTE_STRING_MODE:V,REGEXP_MODE:je,RE_STARTERS_RE:oe,SHEBANG:ye,TITLE_MODE:De,UNDERSCORE_IDENT_RE:Z,UNDERSCORE_TITLE_MODE:Ot});function vn(K,Ee){K.input[K.index-1]==="."&&Ee.ignoreMatch()}function mr(K,Ee){K.className!==void 0&&(K.scope=K.className,delete K.className)}function vt(K,Ee){Ee&&K.beginKeywords&&(K.begin="\\b("+K.beginKeywords.split(" ").join("|")+")(?!\\.)(?=\\b|\\s)",K.__beforeBegin=vn,K.keywords=K.keywords||K.beginKeywords,delete K.beginKeywords,K.relevance===void 0&&(K.relevance=0))}function gt(K,Ee){Array.isArray(K.illegal)&&(K.illegal=w(...K.illegal))}function Bt(K,Ee){if(K.match){if(K.begin||K.end)throw new Error("begin & end are not supported with match");K.begin=K.match,delete K.match}}function Je(K,Ee){K.relevance===void 0&&(K.relevance=1)}const yt=(K,Ee)=>{if(!K.beforeMatch)return;if(K.starts)throw new Error("beforeMatch cannot be used with starts");const Fe=Object.assign({},K);Object.keys(K).forEach(ht=>{delete K[ht]}),K.keywords=Fe.keywords,K.begin=x(Fe.beforeMatch,b(Fe.begin)),K.starts={relevance:0,contains:[Object.assign(Fe,{endsParent:!0})]},K.relevance=0,delete Fe.beforeMatch},mn=["of","and","for","in","not","or","if","then","parent","list","value"],Lt="keyword";function ln(K,Ee,Fe=Lt){const ht=Object.create(null);return typeof K=="string"?yn(Fe,K.split(" ")):Array.isArray(K)?yn(Fe,K):Object.keys(K).forEach(function(wn){Object.assign(ht,ln(K[wn],Ee,wn))}),ht;function yn(wn,Ve){Ee&&(Ve=Ve.map(Pe=>Pe.toLowerCase())),Ve.forEach(function(Pe){const et=Pe.split("|");ht[et[0]]=[wn,We(et[0],et[1])]})}}function We(K,Ee){return Ee?Number(Ee):on(K)?0:1}function on(K){return mn.includes(K.toLowerCase())}const qn={},Ln=K=>{console.error(K)},nn=(K,...Ee)=>{console.log(`WARN: ${K}`,...Ee)},he=(K,Ee)=>{qn[`${K}/${Ee}`]||(console.log(`Deprecated as of ${K}. ${Ee}`),qn[`${K}/${Ee}`]=!0)},Se=new Error;function Ue(K,Ee,{key:Fe}){let ht=0;const yn=K[Fe],wn={},Ve={};for(let Pe=1;Pe<=Ee.length;Pe++)Ve[Pe+ht]=yn[Pe],wn[Pe+ht]=!0,ht+=O(Ee[Pe-1]);K[Fe]=Ve,K[Fe]._emit=wn,K[Fe]._multi=!0}function ze(K){if(Array.isArray(K.begin)){if(K.skip||K.excludeBegin||K.returnBegin)throw Ln("skip, excludeBegin, returnBegin not compatible with beginScope: {}"),Se;if(typeof K.beginScope!="object"||K.beginScope===null)throw Ln("beginScope must be object"),Se;Ue(K,K.begin,{key:"beginScope"}),K.begin=H(K.begin,{joinWith:""})}}function tt(K){if(Array.isArray(K.end)){if(K.skip||K.excludeEnd||K.returnEnd)throw Ln("skip, excludeEnd, returnEnd not compatible with endScope: {}"),Se;if(typeof K.endScope!="object"||K.endScope===null)throw Ln("endScope must be object"),Se;Ue(K,K.end,{key:"endScope"}),K.end=H(K.end,{joinWith:""})}}function er(K){K.scope&&typeof K.scope=="object"&&K.scope!==null&&(K.beginScope=K.scope,delete K.scope)}function $r(K){er(K),typeof K.beginScope=="string"&&(K.beginScope={_wrap:K.beginScope}),typeof K.endScope=="string"&&(K.endScope={_wrap:K.endScope}),ze(K),tt(K)}function Pn(K){function Ee(Ve,Pe){return new RegExp(p(Ve),"m"+(K.case_insensitive?"i":"")+(K.unicodeRegex?"u":"")+(Pe?"g":""))}class Fe{constructor(){this.matchIndexes={},this.regexes=[],this.matchAt=1,this.position=0}addRule(Pe,et){et.position=this.position++,this.matchIndexes[this.matchAt]=et,this.regexes.push([et,Pe]),this.matchAt+=O(Pe)+1}compile(){this.regexes.length===0&&(this.exec=()=>null);const Pe=this.regexes.map(et=>et[1]);this.matcherRe=Ee(H(Pe,{joinWith:"|"}),!0),this.lastIndex=0}exec(Pe){this.matcherRe.lastIndex=this.lastIndex;const et=this.matcherRe.exec(Pe);if(!et)return null;const pn=et.findIndex((cl,wu)=>wu>0&&cl!==void 0),_n=this.matchIndexes[pn];return et.splice(0,pn),Object.assign(et,_n)}}class ht{constructor(){this.rules=[],this.multiRegexes=[],this.count=0,this.lastIndex=0,this.regexIndex=0}getMatcher(Pe){if(this.multiRegexes[Pe])return this.multiRegexes[Pe];const et=new Fe;return this.rules.slice(Pe).forEach(([pn,_n])=>et.addRule(pn,_n)),et.compile(),this.multiRegexes[Pe]=et,et}resumingScanAtSamePosition(){return this.regexIndex!==0}considerAll(){this.regexIndex=0}addRule(Pe,et){this.rules.push([Pe,et]),et.type==="begin"&&this.count++}exec(Pe){const et=this.getMatcher(this.regexIndex);et.lastIndex=this.lastIndex;let pn=et.exec(Pe);if(this.resumingScanAtSamePosition()&&!(pn&&pn.index===this.lastIndex)){const _n=this.getMatcher(0);_n.lastIndex=this.lastIndex+1,pn=_n.exec(Pe)}return pn&&(this.regexIndex+=pn.position+1,this.regexIndex===this.count&&this.considerAll()),pn}}function yn(Ve){const Pe=new ht;return Ve.contains.forEach(et=>Pe.addRule(et.begin,{rule:et,type:"begin"})),Ve.terminatorEnd&&Pe.addRule(Ve.terminatorEnd,{type:"end"}),Ve.illegal&&Pe.addRule(Ve.illegal,{type:"illegal"}),Pe}function wn(Ve,Pe){const et=Ve;if(Ve.isCompiled)return et;[mr,Bt,$r,yt].forEach(_n=>_n(Ve,Pe)),K.compilerExtensions.forEach(_n=>_n(Ve,Pe)),Ve.__beforeBegin=null,[vt,gt,Je].forEach(_n=>_n(Ve,Pe)),Ve.isCompiled=!0;let pn=null;return typeof Ve.keywords=="object"&&Ve.keywords.$pattern&&(Ve.keywords=Object.assign({},Ve.keywords),pn=Ve.keywords.$pattern,delete Ve.keywords.$pattern),pn=pn||/\w+/,Ve.keywords&&(Ve.keywords=ln(Ve.keywords,K.case_insensitive)),et.keywordPatternRe=Ee(pn,!0),Pe&&(Ve.begin||(Ve.begin=/\B|\b/),et.beginRe=Ee(et.begin),!Ve.end&&!Ve.endsWithParent&&(Ve.end=/\B|\b/),Ve.end&&(et.endRe=Ee(et.end)),et.terminatorEnd=p(et.end)||"",Ve.endsWithParent&&Pe.terminatorEnd&&(et.terminatorEnd+=(Ve.end?"|":"")+Pe.terminatorEnd)),Ve.illegal&&(et.illegalRe=Ee(Ve.illegal)),Ve.contains||(Ve.contains=[]),Ve.contains=[].concat(...Ve.contains.map(function(_n){return Or(_n==="self"?Ve:_n)})),Ve.contains.forEach(function(_n){wn(_n,et)}),Ve.starts&&wn(Ve.starts,Pe),et.matcher=yn(et),et}if(K.compilerExtensions||(K.compilerExtensions=[]),K.contains&&K.contains.includes("self"))throw new Error("ERR: contains `self` is not supported at the top-level of a language.  See documentation.");return K.classNameAliases=r(K.classNameAliases||{}),wn(K)}function tr(K){return K?K.endsWithParent||tr(K.starts):!1}function Or(K){return K.variants&&!K.cachedVariants&&(K.cachedVariants=K.variants.map(function(Ee){return r(K,{variants:null},Ee)})),K.cachedVariants?K.cachedVariants:tr(K)?r(K,{starts:K.starts?r(K.starts):null}):Object.isFrozen(K)?r(K):K}var jn="11.11.1";class Yr extends Error{constructor(Ee,Fe){super(Ee),this.name="HTMLInjectionError",this.html=Fe}}const Rr=n,Tu=r,xu=Symbol("nomatch"),Su=7,hs=function(K){const Ee=Object.create(null),Fe=Object.create(null),ht=[];let yn=!0;const wn="Could not find the language '{}', did you forget to load/include a language module?",Ve={disableAutodetect:!0,name:"Plain text",contains:[]};let Pe={ignoreUnescapedHTML:!1,throwUnescapedHTML:!1,noHighlightRe:/^(no-?highlight)$/i,languageDetectRe:/\blang(?:uage)?-([\w-]+)\b/i,classPrefix:"hljs-",cssSelector:"pre code",languages:null,__emitter:m};function et(Re){return Pe.noHighlightRe.test(Re)}function pn(Re){let rt=Re.className+" ";rt+=Re.parentNode?Re.parentNode.className:"";const _t=Pe.languageDetectRe.exec(rt);if(_t){const $t=Mr(_t[1]);return $t||(nn(wn.replace("{}",_t[1])),nn("Falling back to no-highlight mode for this block.",Re)),$t?_t[1]:"no-highlight"}return rt.split(/\s+/).find($t=>et($t)||Mr($t))}function _n(Re,rt,_t){let $t="",kn="";typeof rt=="object"?($t=Re,_t=rt.ignoreIllegals,kn=rt.language):(he("10.7.0","highlight(lang, code, ...args) has been deprecated."),he("10.7.0",`Please use highlight(code, options) instead.
https://github.com/highlightjs/highlight.js/issues/2277`),kn=Re,$t=rt),_t===void 0&&(_t=!0);const ia={code:$t,language:kn};Ai("before:highlight",ia);const qa=ia.result?ia.result:cl(ia.language,ia.code,_t);return qa.code=ia.code,Ai("after:highlight",qa),qa}function cl(Re,rt,_t,$t){const kn=Object.create(null);function ia(He,Qe){return He.keywords[Qe]}function qa(){if(!bt.keywords){$n.addText(Yt);return}let He=0;bt.keywordPatternRe.lastIndex=0;let Qe=bt.keywordPatternRe.exec(Yt),pt="";for(;Qe;){pt+=Yt.substring(He,Qe.index);const Ht=sa.case_insensitive?Qe[0].toLowerCase():Qe[0],Yn=ia(bt,Ht);if(Yn){const[xa,bl]=Yn;if($n.addText(pt),pt="",kn[Ht]=(kn[Ht]||0)+1,kn[Ht]<=Su&&(ao+=bl),xa.startsWith("_"))pt+=Qe[0];else{const ja=sa.classNameAliases[xa]||xa;Ta(Qe[0],ja)}}else pt+=Qe[0];He=bt.keywordPatternRe.lastIndex,Qe=bt.keywordPatternRe.exec(Yt)}pt+=Yt.substring(He),$n.addText(pt)}function pr(){if(Yt==="")return;let He=null;if(typeof bt.subLanguage=="string"){if(!Ee[bt.subLanguage]){$n.addText(Yt);return}He=cl(bt.subLanguage,Yt,!0,Nu[bt.subLanguage]),Nu[bt.subLanguage]=He._top}else He=no(Yt,bt.subLanguage.length?bt.subLanguage:null);bt.relevance>0&&(ao+=He.relevance),$n.__addSublanguage(He._emitter,He.language)}function Tr(){bt.subLanguage!=null?pr():qa(),Yt=""}function Ta(He,Qe){He!==""&&($n.startScope(Qe),$n.addText(He),$n.endScope())}function ml(He,Qe){let pt=1;const Ht=Qe.length-1;for(;pt<=Ht;){if(!He._emit[pt]){pt++;continue}const Yn=sa.classNameAliases[He[pt]]||He[pt],xa=Qe[pt];Yn?Ta(xa,Yn):(Yt=xa,qa(),Yt=""),pt++}}function ku(He,Qe){return He.scope&&typeof He.scope=="string"&&$n.openNode(sa.classNameAliases[He.scope]||He.scope),He.beginScope&&(He.beginScope._wrap?(Ta(Yt,sa.classNameAliases[He.beginScope._wrap]||He.beginScope._wrap),Yt=""):He.beginScope._multi&&(ml(He.beginScope,Qe),Yt="")),bt=Object.create(He,{parent:{value:bt}}),bt}function pl(He,Qe,pt){let Ht=D(He.endRe,pt);if(Ht){if(He["on:end"]){const Yn=new t(He);He["on:end"](Qe,Yn),Yn.isMatchIgnored&&(Ht=!1)}if(Ht){for(;He.endsParent&&He.parent;)He=He.parent;return He}}if(He.endsWithParent)return pl(He.parent,Qe,pt)}function dm(He){return bt.matcher.regexIndex===0?(Yt+=He[0],1):(gl=!0,0)}function Gr(He){const Qe=He[0],pt=He.rule,Ht=new t(pt),Yn=[pt.__beforeBegin,pt["on:begin"]];for(const xa of Yn)if(xa&&(xa(He,Ht),Ht.isMatchIgnored))return dm(Qe);return pt.skip?Yt+=Qe:(pt.excludeBegin&&(Yt+=Qe),Tr(),!pt.returnBegin&&!pt.excludeBegin&&(Yt=Qe)),ku(pt,He),pt.returnBegin?0:Qe.length}function Cu(He){const Qe=He[0],pt=rt.substring(He.index),Ht=pl(bt,He,pt);if(!Ht)return xu;const Yn=bt;bt.endScope&&bt.endScope._wrap?(Tr(),Ta(Qe,bt.endScope._wrap)):bt.endScope&&bt.endScope._multi?(Tr(),ml(bt.endScope,He)):Yn.skip?Yt+=Qe:(Yn.returnEnd||Yn.excludeEnd||(Yt+=Qe),Tr(),Yn.excludeEnd&&(Yt=Qe));do bt.scope&&$n.closeNode(),!bt.skip&&!bt.subLanguage&&(ao+=bt.relevance),bt=bt.parent;while(bt!==Ht.parent);return Ht.starts&&ku(Ht.starts,He),Yn.returnEnd?0:Qe.length}function K0(){const He=[];for(let Qe=bt;Qe!==sa;Qe=Qe.parent)Qe.scope&&He.unshift(Qe.scope);He.forEach(Qe=>$n.openNode(Qe))}let gs={};function ki(He,Qe){const pt=Qe&&Qe[0];if(Yt+=He,pt==null)return Tr(),0;if(gs.type==="begin"&&Qe.type==="end"&&gs.index===Qe.index&&pt===""){if(Yt+=rt.slice(Qe.index,Qe.index+1),!yn){const Ht=new Error(`0 width match regex (${Re})`);throw Ht.languageName=Re,Ht.badRule=gs.rule,Ht}return 1}if(gs=Qe,Qe.type==="begin")return Gr(Qe);if(Qe.type==="illegal"&&!_t){const Ht=new Error('Illegal lexeme "'+pt+'" for mode "'+(bt.scope||"<unnamed>")+'"');throw Ht.mode=bt,Ht}else if(Qe.type==="end"){const Ht=Cu(Qe);if(Ht!==xu)return Ht}if(Qe.type==="illegal"&&pt==="")return Yt+=`
`,1;if(bs>1e5&&bs>Qe.index*3)throw new Error("potential infinite loop, way more iterations than matches");return Yt+=pt,pt.length}const sa=Mr(Re);if(!sa)throw Ln(wn.replace("{}",Re)),new Error('Unknown language: "'+Re+'"');const W0=Pn(sa);let ei="",bt=$t||W0;const Nu={},$n=new Pe.__emitter(Pe);K0();let Yt="",ao=0,Ci=0,bs=0,gl=!1;try{if(sa.__emitTokens)sa.__emitTokens(rt,$n);else{for(bt.matcher.considerAll();;){bs++,gl?gl=!1:bt.matcher.considerAll(),bt.matcher.lastIndex=Ci;const He=bt.matcher.exec(rt);if(!He)break;const Qe=rt.substring(Ci,He.index),pt=ki(Qe,He);Ci=He.index+pt}ki(rt.substring(Ci))}return $n.finalize(),ei=$n.toHTML(),{language:Re,value:ei,relevance:ao,illegal:!1,_emitter:$n,_top:bt}}catch(He){if(He.message&&He.message.includes("Illegal"))return{language:Re,value:Rr(rt),illegal:!0,relevance:0,_illegalBy:{message:He.message,index:Ci,context:rt.slice(Ci-100,Ci+100),mode:He.mode,resultSoFar:ei},_emitter:$n};if(yn)return{language:Re,value:Rr(rt),illegal:!1,relevance:0,errorRaised:He,_emitter:$n,_top:bt};throw He}}function wu(Re){const rt={value:Rr(Re),illegal:!1,relevance:0,_top:Ve,_emitter:new Pe.__emitter(Pe)};return rt._emitter.addText(Re),rt}function no(Re,rt){rt=rt||Pe.languages||Object.keys(Ee);const _t=wu(Re),$t=rt.filter(Mr).filter(Au).map(Tr=>cl(Tr,Re,!1));$t.unshift(_t);const kn=$t.sort((Tr,Ta)=>{if(Tr.relevance!==Ta.relevance)return Ta.relevance-Tr.relevance;if(Tr.language&&Ta.language){if(Mr(Tr.language).supersetOf===Ta.language)return 1;if(Mr(Ta.language).supersetOf===Tr.language)return-1}return 0}),[ia,qa]=kn,pr=ia;return pr.secondBest=qa,pr}function dl(Re,rt,_t){const $t=rt&&Fe[rt]||_t;Re.classList.add("hljs"),Re.classList.add(`language-${$t}`)}function hl(Re){let rt=null;const _t=pn(Re);if(et(_t))return;if(Ai("before:highlightElement",{el:Re,language:_t}),Re.dataset.highlighted){console.log("Element previously highlighted. To highlight again, first unset `dataset.highlighted`.",Re);return}if(Re.children.length>0&&(Pe.ignoreUnescapedHTML||(console.warn("One of your code blocks includes unescaped HTML. This is a potentially serious security risk."),console.warn("https://github.com/highlightjs/highlight.js/wiki/security"),console.warn("The element with unescaped HTML:"),console.warn(Re)),Pe.throwUnescapedHTML))throw new Yr("One of your code blocks includes unescaped HTML.",Re.innerHTML);rt=Re;const $t=rt.textContent,kn=_t?_n($t,{language:_t,ignoreIllegals:!0}):no($t);Re.innerHTML=kn.value,Re.dataset.highlighted="yes",dl(Re,_t,kn.language),Re.result={language:kn.language,re:kn.relevance,relevance:kn.relevance},kn.secondBest&&(Re.secondBest={language:kn.secondBest.language,relevance:kn.secondBest.relevance}),Ai("after:highlightElement",{el:Re,result:kn,text:$t})}function ms(Re){Pe=Tu(Pe,Re)}const ps=()=>{An(),he("10.6.0","initHighlighting() deprecated.  Use highlightAll() now.")};function fl(){An(),he("10.6.0","initHighlightingOnLoad() deprecated.  Use highlightAll() now.")}let Si=!1;function An(){function Re(){An()}if(document.readyState==="loading"){Si||window.addEventListener("DOMContentLoaded",Re,!1),Si=!0;return}document.querySelectorAll(Pe.cssSelector).forEach(hl)}function $0(Re,rt){let _t=null;try{_t=rt(K)}catch($t){if(Ln("Language definition for '{}' could not be registered.".replace("{}",Re)),yn)Ln($t);else throw $t;_t=Ve}_t.name||(_t.name=Re),Ee[Re]=_t,_t.rawDefinition=rt.bind(null,K),_t.aliases&&G0(_t.aliases,{languageName:Re})}function Y0(Re){delete Ee[Re];for(const rt of Object.keys(Fe))Fe[rt]===Re&&delete Fe[rt]}function wi(){return Object.keys(Ee)}function Mr(Re){return Re=(Re||"").toLowerCase(),Ee[Re]||Ee[Fe[Re]]}function G0(Re,{languageName:rt}){typeof Re=="string"&&(Re=[Re]),Re.forEach(_t=>{Fe[_t.toLowerCase()]=rt})}function Au(Re){const rt=Mr(Re);return rt&&!rt.disableAutodetect}function V0(Re){Re["before:highlightBlock"]&&!Re["before:highlightElement"]&&(Re["before:highlightElement"]=rt=>{Re["before:highlightBlock"](Object.assign({block:rt.el},rt))}),Re["after:highlightBlock"]&&!Re["after:highlightElement"]&&(Re["after:highlightElement"]=rt=>{Re["after:highlightBlock"](Object.assign({block:rt.el},rt))})}function cm(Re){V0(Re),ht.push(Re)}function ro(Re){const rt=ht.indexOf(Re);rt!==-1&&ht.splice(rt,1)}function Ai(Re,rt){const _t=Re;ht.forEach(function($t){$t[_t]&&$t[_t](rt)})}function Ha(Re){return he("10.7.0","highlightBlock will be removed entirely in v12.0"),he("10.7.0","Please use highlightElement now."),hl(Re)}Object.assign(K,{highlight:_n,highlightAuto:no,highlightAll:An,highlightElement:hl,highlightBlock:Ha,configure:ms,initHighlighting:ps,initHighlightingOnLoad:fl,registerLanguage:$0,unregisterLanguage:Y0,listLanguages:wi,getLanguage:Mr,registerAliases:G0,autoDetection:Au,inherit:Tu,addPlugin:cm,removePlugin:ro}),K.debugMode=function(){yn=!1},K.safeMode=function(){yn=!0},K.versionString=jn,K.regex={concat:x,lookahead:b,either:w,optional:T,anyNumberOfTimes:y};for(const Re in wt)typeof wt[Re]=="object"&&e(wt[Re]);return Object.assign(K,wt),K},fs=hs({});return fs.newInstance=()=>hs({}),tg=fs,fs.HighlightJS=fs,fs.default=fs,tg}var mq=fq();const pq=Mf(mq),QE={},gq="hljs-";function bq(e){const t=pq.newInstance();return e&&s(e),{highlight:n,highlightAuto:r,listLanguages:a,register:s,registerAlias:o,registered:c};function n(d,f,m){const p=m||QE,b=typeof p.prefix=="string"?p.prefix:gq;if(!t.getLanguage(d))throw new Error("Unknown language: `"+d+"` is not registered");t.configure({__emitter:vq,classPrefix:b});const y=t.highlight(f,{ignoreIllegals:!0,language:d});if(y.errorRaised)throw new Error("Could not highlight with `Highlight.js`",{cause:y.errorRaised});const T=y._emitter.root,x=T.data;return x.language=y.language,x.relevance=y.relevance,T}function r(d,f){const p=(f||QE).subset||a();let b=-1,y=0,T;for(;++b<p.length;){const x=p[b];if(!t.getLanguage(x))continue;const C=n(x,d,f);C.data&&C.data.relevance!==void 0&&C.data.relevance>y&&(y=C.data.relevance,T=C)}return T||{type:"root",children:[],data:{language:void 0,relevance:y}}}function a(){return t.listLanguages()}function s(d,f){if(typeof d=="string")t.registerLanguage(d,f);else{let m;for(m in d)Object.hasOwn(d,m)&&t.registerLanguage(m,d[m])}}function o(d,f){if(typeof d=="string")t.registerAliases(typeof f=="string"?f:[...f],{languageName:d});else{let m;for(m in d)if(Object.hasOwn(d,m)){const p=d[m];t.registerAliases(typeof p=="string"?p:[...p],{languageName:m})}}}function c(d){return!!t.getLanguage(d)}}class vq{constructor(t){this.options=t,this.root={type:"root",children:[],data:{language:void 0,relevance:0}},this.stack=[this.root]}addText(t){if(t==="")return;const n=this.stack[this.stack.length-1],r=n.children[n.children.length-1];r&&r.type==="text"?r.value+=t:n.children.push({type:"text",value:t})}startScope(t){this.openNode(String(t))}endScope(){this.closeNode()}__addSublanguage(t,n){const r=this.stack[this.stack.length-1],a=t.root.children;n?r.children.push({type:"element",tagName:"span",properties:{className:[n]},children:a}):r.children.push(...a)}openNode(t){const n=this,r=t.split(".").map(function(o,c){return c?o+"_".repeat(c):n.options.classPrefix+o}),a=this.stack[this.stack.length-1],s={type:"element",tagName:"span",properties:{className:r},children:[]};a.children.push(s),this.stack.push(s)}closeNode(){this.stack.pop()}finalize(){}toHTML(){return""}}const yq={};function _q(e){const t=e||yq,n=t.aliases,r=t.detect||!1,a=t.languages||hq,s=t.plainText,o=t.prefix,c=t.subset;let d="hljs";const f=bq(a);if(n&&f.registerAlias(n),o){const m=o.indexOf("-");d=m===-1?o:o.slice(0,m)}return function(m,p){R0(m,"element",function(b,y,T){if(b.tagName!=="code"||!T||T.type!=="element"||T.tagName!=="pre")return;const x=Eq(b);if(x===!1||!x&&!r||x&&s&&s.includes(x))return;Array.isArray(b.properties.className)||(b.properties.className=[]),b.properties.className.includes(d)||b.properties.className.unshift(d);const C=YT(b,{whitespace:"pre"});let w;try{w=x?f.highlight(x,C,{prefix:o}):f.highlightAuto(C,{prefix:o,subset:c})}catch(O){const D=O;if(x&&/Unknown language/.test(D.message)){p.message("Cannot highlight as `"+x+"`, it’s not registered",{ancestors:[T,b],cause:D,place:b.position,ruleId:"missing-language",source:"rehype-highlight"});return}throw D}!x&&w.data&&w.data.language&&b.properties.className.push("language-"+w.data.language),w.children.length>0&&(b.children=w.children)})}}function Eq(e){const t=e.properties.className;let n=-1;if(!Array.isArray(t))return;let r;for(;++n<t.length;){const a=String(t[n]);if(a==="no-highlight"||a==="nohighlight")return!1;!r&&a.slice(0,5)==="lang-"&&(r=a.slice(5)),!r&&a.slice(0,9)==="language-"&&(r=a.slice(9))}return r}function Tq(e,t){return hx(e,{})||{type:"root",children:[]}}function hx(e,t){const n=xq(e,t);return n&&t.afterTransform&&t.afterTransform(e,n),n}function xq(e,t){switch(e.nodeType){case 1:return kq(e,t);case 3:return wq(e);case 8:return Aq(e);case 9:return ZE(e,t);case 10:return Sq();case 11:return ZE(e,t);default:return}}function ZE(e,t){return{type:"root",children:fx(e,t)}}function Sq(){return{type:"doctype"}}function wq(e){return{type:"text",value:e.nodeValue||""}}function Aq(e){return{type:"comment",value:e.nodeValue||""}}function kq(e,t){const n=e.namespaceURI,r=n===di.svg?eT:J7,a=n===di.html?e.tagName.toLowerCase():e.tagName,s=n===di.html&&a==="template"?e.content:e,o=e.getAttributeNames(),c={};let d=-1;for(;++d<o.length;)c[o[d]]=e.getAttribute(o[d])||"";return r(a,c,fx(s,t))}function fx(e,t){const n=e.childNodes,r=[];let a=-1;for(;++a<n.length;){const s=hx(n[a],t);s!==void 0&&r.push(s)}return r}new DOMParser;function Cq(e,t){const n=Nq(e);return Tq(n)}function Nq(e){const t=document.createElement("template");return t.innerHTML=e,t.content}class ba{constructor(t,n,r){this.lexer=void 0,this.start=void 0,this.end=void 0,this.lexer=t,this.start=n,this.end=r}static range(t,n){return n?!t||!t.loc||!n.loc||t.loc.lexer!==n.loc.lexer?null:new ba(t.loc.lexer,t.loc.start,n.loc.end):t&&t.loc}}class Da{constructor(t,n){this.text=void 0,this.loc=void 0,this.noexpand=void 0,this.treatAsRelax=void 0,this.text=t,this.loc=n}range(t,n){return new Da(n,ba.range(this,t))}}class Ne{constructor(t,n){this.name=void 0,this.position=void 0,this.length=void 0,this.rawMessage=void 0;var r="KaTeX parse error: "+t,a,s,o=n&&n.loc;if(o&&o.start<=o.end){var c=o.lexer.input;a=o.start,s=o.end,a===c.length?r+=" at end of input: ":r+=" at position "+(a+1)+": ";var d=c.slice(a,s).replace(/[^]/g,"$&̲"),f;a>15?f="…"+c.slice(a-15,a):f=c.slice(0,a);var m;s+15<c.length?m=c.slice(s,s+15)+"…":m=c.slice(s),r+=f+d+m}var p=new Error(r);return p.name="ParseError",p.__proto__=Ne.prototype,p.position=a,a!=null&&s!=null&&(p.length=s-a),p.rawMessage=t,p}}Ne.prototype.__proto__=Error.prototype;var Oq=function(t,n){return t.indexOf(n)!==-1},Rq=function(t,n){return t===void 0?n:t},Mq=/([A-Z])/g,Iq=function(t){return t.replace(Mq,"-$1").toLowerCase()},Dq={"&":"&amp;",">":"&gt;","<":"&lt;",'"':"&quot;","'":"&#x27;"},Lq=/[&><"']/g;function Pq(e){return String(e).replace(Lq,t=>Dq[t])}var mx=function e(t){return t.type==="ordgroup"||t.type==="color"?t.body.length===1?e(t.body[0]):t:t.type==="font"?e(t.body):t},Bq=function(t){var n=mx(t);return n.type==="mathord"||n.type==="textord"||n.type==="atom"},Uq=function(t){if(!t)throw new Error("Expected non-null, but got "+String(t));return t},Fq=function(t){var n=/^[\x00-\x20]*([^\\/#?]*?)(:|&#0*58|&#x0*3a|&colon)/i.exec(t);return n?n[2]!==":"||!/^[a-zA-Z][a-zA-Z0-9+\-.]*$/.test(n[1])?null:n[1].toLowerCase():"_relative"},it={contains:Oq,deflt:Rq,escape:Pq,hyphenate:Iq,getBaseElem:mx,isCharacterBox:Bq,protocolFromUrl:Fq},$h={displayMode:{type:"boolean",description:"Render math in display mode, which puts the math in display style (so \\int and \\sum are large, for example), and centers the math on the page on its own line.",cli:"-d, --display-mode"},output:{type:{enum:["htmlAndMathml","html","mathml"]},description:"Determines the markup language of the output.",cli:"-F, --format <type>"},leqno:{type:"boolean",description:"Render display math in leqno style (left-justified tags)."},fleqn:{type:"boolean",description:"Render display math flush left."},throwOnError:{type:"boolean",default:!0,cli:"-t, --no-throw-on-error",cliDescription:"Render errors (in the color given by --error-color) instead of throwing a ParseError exception when encountering an error."},errorColor:{type:"string",default:"#cc0000",cli:"-c, --error-color <color>",cliDescription:"A color string given in the format 'rgb' or 'rrggbb' (no #). This option determines the color of errors rendered by the -t option.",cliProcessor:e=>"#"+e},macros:{type:"object",cli:"-m, --macro <def>",cliDescription:"Define custom macro of the form '\\foo:expansion' (use multiple -m arguments for multiple macros).",cliDefault:[],cliProcessor:(e,t)=>(t.push(e),t)},minRuleThickness:{type:"number",description:"Specifies a minimum thickness, in ems, for fraction lines, `\\sqrt` top lines, `{array}` vertical lines, `\\hline`, `\\hdashline`, `\\underline`, `\\overline`, and the borders of `\\fbox`, `\\boxed`, and `\\fcolorbox`.",processor:e=>Math.max(0,e),cli:"--min-rule-thickness <size>",cliProcessor:parseFloat},colorIsTextColor:{type:"boolean",description:"Makes \\color behave like LaTeX's 2-argument \\textcolor, instead of LaTeX's one-argument \\color mode change.",cli:"-b, --color-is-text-color"},strict:{type:[{enum:["warn","ignore","error"]},"boolean","function"],description:"Turn on strict / LaTeX faithfulness mode, which throws an error if the input uses features that are not supported by LaTeX.",cli:"-S, --strict",cliDefault:!1},trust:{type:["boolean","function"],description:"Trust the input, enabling all HTML features such as \\url.",cli:"-T, --trust"},maxSize:{type:"number",default:1/0,description:"If non-zero, all user-specified sizes, e.g. in \\rule{500em}{500em}, will be capped to maxSize ems. Otherwise, elements and spaces can be arbitrarily large",processor:e=>Math.max(0,e),cli:"-s, --max-size <n>",cliProcessor:parseInt},maxExpand:{type:"number",default:1e3,description:"Limit the number of macro expansions to the specified number, to prevent e.g. infinite macro loops. If set to Infinity, the macro expander will try to fully expand as in LaTeX.",processor:e=>Math.max(0,e),cli:"-e, --max-expand <n>",cliProcessor:e=>e==="Infinity"?1/0:parseInt(e)},globalGroup:{type:"boolean",cli:!1}};function zq(e){if(e.default)return e.default;var t=e.type,n=Array.isArray(t)?t[0]:t;if(typeof n!="string")return n.enum[0];switch(n){case"boolean":return!1;case"string":return"";case"number":return 0;case"object":return{}}}class B2{constructor(t){this.displayMode=void 0,this.output=void 0,this.leqno=void 0,this.fleqn=void 0,this.throwOnError=void 0,this.errorColor=void 0,this.macros=void 0,this.minRuleThickness=void 0,this.colorIsTextColor=void 0,this.strict=void 0,this.trust=void 0,this.maxSize=void 0,this.maxExpand=void 0,this.globalGroup=void 0,t=t||{};for(var n in $h)if($h.hasOwnProperty(n)){var r=$h[n];this[n]=t[n]!==void 0?r.processor?r.processor(t[n]):t[n]:zq(r)}}reportNonstrict(t,n,r){var a=this.strict;if(typeof a=="function"&&(a=a(t,n,r)),!(!a||a==="ignore")){if(a===!0||a==="error")throw new Ne("LaTeX-incompatible input and strict mode is set to 'error': "+(n+" ["+t+"]"),r);a==="warn"?typeof console<"u"&&console.warn("LaTeX-incompatible input and strict mode is set to 'warn': "+(n+" ["+t+"]")):typeof console<"u"&&console.warn("LaTeX-incompatible input and strict mode is set to "+("unrecognized '"+a+"': "+n+" ["+t+"]"))}}useStrictBehavior(t,n,r){var a=this.strict;if(typeof a=="function")try{a=a(t,n,r)}catch{a="error"}return!a||a==="ignore"?!1:a===!0||a==="error"?!0:a==="warn"?(typeof console<"u"&&console.warn("LaTeX-incompatible input and strict mode is set to 'warn': "+(n+" ["+t+"]")),!1):(typeof console<"u"&&console.warn("LaTeX-incompatible input and strict mode is set to "+("unrecognized '"+a+"': "+n+" ["+t+"]")),!1)}isTrusted(t){if(t.url&&!t.protocol){var n=it.protocolFromUrl(t.url);if(n==null)return!1;t.protocol=n}var r=typeof this.trust=="function"?this.trust(t):this.trust;return!!r}}class Hs{constructor(t,n,r){this.id=void 0,this.size=void 0,this.cramped=void 0,this.id=t,this.size=n,this.cramped=r}sup(){return ci[Hq[this.id]]}sub(){return ci[qq[this.id]]}fracNum(){return ci[jq[this.id]]}fracDen(){return ci[$q[this.id]]}cramp(){return ci[Yq[this.id]]}text(){return ci[Gq[this.id]]}isTight(){return this.size>=2}}var U2=0,uf=1,Qo=2,es=3,p0=4,Ia=5,ru=6,Hr=7,ci=[new Hs(U2,0,!1),new Hs(uf,0,!0),new Hs(Qo,1,!1),new Hs(es,1,!0),new Hs(p0,2,!1),new Hs(Ia,2,!0),new Hs(ru,3,!1),new Hs(Hr,3,!0)],Hq=[p0,Ia,p0,Ia,ru,Hr,ru,Hr],qq=[Ia,Ia,Ia,Ia,Hr,Hr,Hr,Hr],jq=[Qo,es,p0,Ia,ru,Hr,ru,Hr],$q=[es,es,Ia,Ia,Hr,Hr,Hr,Hr],Yq=[uf,uf,es,es,Ia,Ia,Hr,Hr],Gq=[U2,uf,Qo,es,Qo,es,Qo,es],lt={DISPLAY:ci[U2],TEXT:ci[Qo],SCRIPT:ci[p0],SCRIPTSCRIPT:ci[ru]},gb=[{name:"latin",blocks:[[256,591],[768,879]]},{name:"cyrillic",blocks:[[1024,1279]]},{name:"armenian",blocks:[[1328,1423]]},{name:"brahmic",blocks:[[2304,4255]]},{name:"georgian",blocks:[[4256,4351]]},{name:"cjk",blocks:[[12288,12543],[19968,40879],[65280,65376]]},{name:"hangul",blocks:[[44032,55215]]}];function Vq(e){for(var t=0;t<gb.length;t++)for(var n=gb[t],r=0;r<n.blocks.length;r++){var a=n.blocks[r];if(e>=a[0]&&e<=a[1])return n.name}return null}var Yh=[];gb.forEach(e=>e.blocks.forEach(t=>Yh.push(...t)));function px(e){for(var t=0;t<Yh.length;t+=2)if(e>=Yh[t]&&e<=Yh[t+1])return!0;return!1}var Bo=80,Kq=function(t,n){return"M95,"+(622+t+n)+`
c-2.7,0,-7.17,-2.7,-13.5,-8c-5.8,-5.3,-9.5,-10,-9.5,-14
c0,-2,0.3,-3.3,1,-4c1.3,-2.7,23.83,-20.7,67.5,-54
c44.2,-33.3,65.8,-50.3,66.5,-51c1.3,-1.3,3,-2,5,-2c4.7,0,8.7,3.3,12,10
s173,378,173,378c0.7,0,35.3,-71,104,-213c68.7,-142,137.5,-285,206.5,-429
c69,-144,104.5,-217.7,106.5,-221
l`+t/2.075+" -"+t+`
c5.3,-9.3,12,-14,20,-14
H400000v`+(40+t)+`H845.2724
s-225.272,467,-225.272,467s-235,486,-235,486c-2.7,4.7,-9,7,-19,7
c-6,0,-10,-1,-12,-3s-194,-422,-194,-422s-65,47,-65,47z
M`+(834+t)+" "+n+"h400000v"+(40+t)+"h-400000z"},Wq=function(t,n){return"M263,"+(601+t+n)+`c0.7,0,18,39.7,52,119
c34,79.3,68.167,158.7,102.5,238c34.3,79.3,51.8,119.3,52.5,120
c340,-704.7,510.7,-1060.3,512,-1067
l`+t/2.084+" -"+t+`
c4.7,-7.3,11,-11,19,-11
H40000v`+(40+t)+`H1012.3
s-271.3,567,-271.3,567c-38.7,80.7,-84,175,-136,283c-52,108,-89.167,185.3,-111.5,232
c-22.3,46.7,-33.8,70.3,-34.5,71c-4.7,4.7,-12.3,7,-23,7s-12,-1,-12,-1
s-109,-253,-109,-253c-72.7,-168,-109.3,-252,-110,-252c-10.7,8,-22,16.7,-34,26
c-22,17.3,-33.3,26,-34,26s-26,-26,-26,-26s76,-59,76,-59s76,-60,76,-60z
M`+(1001+t)+" "+n+"h400000v"+(40+t)+"h-400000z"},Xq=function(t,n){return"M983 "+(10+t+n)+`
l`+t/3.13+" -"+t+`
c4,-6.7,10,-10,18,-10 H400000v`+(40+t)+`
H1013.1s-83.4,268,-264.1,840c-180.7,572,-277,876.3,-289,913c-4.7,4.7,-12.7,7,-24,7
s-12,0,-12,0c-1.3,-3.3,-3.7,-11.7,-7,-25c-35.3,-125.3,-106.7,-373.3,-214,-744
c-10,12,-21,25,-33,39s-32,39,-32,39c-6,-5.3,-15,-14,-27,-26s25,-30,25,-30
c26.7,-32.7,52,-63,76,-91s52,-60,52,-60s208,722,208,722
c56,-175.3,126.3,-397.3,211,-666c84.7,-268.7,153.8,-488.2,207.5,-658.5
c53.7,-170.3,84.5,-266.8,92.5,-289.5z
M`+(1001+t)+" "+n+"h400000v"+(40+t)+"h-400000z"},Qq=function(t,n){return"M424,"+(2398+t+n)+`
c-1.3,-0.7,-38.5,-172,-111.5,-514c-73,-342,-109.8,-513.3,-110.5,-514
c0,-2,-10.7,14.3,-32,49c-4.7,7.3,-9.8,15.7,-15.5,25c-5.7,9.3,-9.8,16,-12.5,20
s-5,7,-5,7c-4,-3.3,-8.3,-7.7,-13,-13s-13,-13,-13,-13s76,-122,76,-122s77,-121,77,-121
s209,968,209,968c0,-2,84.7,-361.7,254,-1079c169.3,-717.3,254.7,-1077.7,256,-1081
l`+t/4.223+" -"+t+`c4,-6.7,10,-10,18,-10 H400000
v`+(40+t)+`H1014.6
s-87.3,378.7,-272.6,1166c-185.3,787.3,-279.3,1182.3,-282,1185
c-2,6,-10,9,-24,9
c-8,0,-12,-0.7,-12,-2z M`+(1001+t)+" "+n+`
h400000v`+(40+t)+"h-400000z"},Zq=function(t,n){return"M473,"+(2713+t+n)+`
c339.3,-1799.3,509.3,-2700,510,-2702 l`+t/5.298+" -"+t+`
c3.3,-7.3,9.3,-11,18,-11 H400000v`+(40+t)+`H1017.7
s-90.5,478,-276.2,1466c-185.7,988,-279.5,1483,-281.5,1485c-2,6,-10,9,-24,9
c-8,0,-12,-0.7,-12,-2c0,-1.3,-5.3,-32,-16,-92c-50.7,-293.3,-119.7,-693.3,-207,-1200
c0,-1.3,-5.3,8.7,-16,30c-10.7,21.3,-21.3,42.7,-32,64s-16,33,-16,33s-26,-26,-26,-26
s76,-153,76,-153s77,-151,77,-151c0.7,0.7,35.7,202,105,604c67.3,400.7,102,602.7,104,
606zM`+(1001+t)+" "+n+"h400000v"+(40+t)+"H1017.7z"},Jq=function(t){var n=t/2;return"M400000 "+t+" H0 L"+n+" 0 l65 45 L145 "+(t-80)+" H400000z"},ej=function(t,n,r){var a=r-54-n-t;return"M702 "+(t+n)+"H400000"+(40+t)+`
H742v`+a+`l-4 4-4 4c-.667.7 -2 1.5-4 2.5s-4.167 1.833-6.5 2.5-5.5 1-9.5 1
h-12l-28-84c-16.667-52-96.667 -294.333-240-727l-212 -643 -85 170
c-4-3.333-8.333-7.667-13 -13l-13-13l77-155 77-156c66 199.333 139 419.667
219 661 l218 661zM702 `+n+"H400000v"+(40+t)+"H742z"},tj=function(t,n,r){n=1e3*n;var a="";switch(t){case"sqrtMain":a=Kq(n,Bo);break;case"sqrtSize1":a=Wq(n,Bo);break;case"sqrtSize2":a=Xq(n,Bo);break;case"sqrtSize3":a=Qq(n,Bo);break;case"sqrtSize4":a=Zq(n,Bo);break;case"sqrtTall":a=ej(n,Bo,r)}return a},nj=function(t,n){switch(t){case"⎜":return"M291 0 H417 V"+n+" H291z M291 0 H417 V"+n+" H291z";case"∣":return"M145 0 H188 V"+n+" H145z M145 0 H188 V"+n+" H145z";case"∥":return"M145 0 H188 V"+n+" H145z M145 0 H188 V"+n+" H145z"+("M367 0 H410 V"+n+" H367z M367 0 H410 V"+n+" H367z");case"⎟":return"M457 0 H583 V"+n+" H457z M457 0 H583 V"+n+" H457z";case"⎢":return"M319 0 H403 V"+n+" H319z M319 0 H403 V"+n+" H319z";case"⎥":return"M263 0 H347 V"+n+" H263z M263 0 H347 V"+n+" H263z";case"⎪":return"M384 0 H504 V"+n+" H384z M384 0 H504 V"+n+" H384z";case"⏐":return"M312 0 H355 V"+n+" H312z M312 0 H355 V"+n+" H312z";case"‖":return"M257 0 H300 V"+n+" H257z M257 0 H300 V"+n+" H257z"+("M478 0 H521 V"+n+" H478z M478 0 H521 V"+n+" H478z");default:return""}},JE={doubleleftarrow:`M262 157
l10-10c34-36 62.7-77 86-123 3.3-8 5-13.3 5-16 0-5.3-6.7-8-20-8-7.3
 0-12.2.5-14.5 1.5-2.3 1-4.8 4.5-7.5 10.5-49.3 97.3-121.7 169.3-217 216-28
 14-57.3 25-88 33-6.7 2-11 3.8-13 5.5-2 1.7-3 4.2-3 7.5s1 5.8 3 7.5
c2 1.7 6.3 3.5 13 5.5 68 17.3 128.2 47.8 180.5 91.5 52.3 43.7 93.8 96.2 124.5
 157.5 9.3 8 15.3 12.3 18 13h6c12-.7 18-4 18-10 0-2-1.7-7-5-15-23.3-46-52-87
-86-123l-10-10h399738v-40H218c328 0 0 0 0 0l-10-8c-26.7-20-65.7-43-117-69 2.7
-2 6-3.7 10-5 36.7-16 72.3-37.3 107-64l10-8h399782v-40z
m8 0v40h399730v-40zm0 194v40h399730v-40z`,doublerightarrow:`M399738 392l
-10 10c-34 36-62.7 77-86 123-3.3 8-5 13.3-5 16 0 5.3 6.7 8 20 8 7.3 0 12.2-.5
 14.5-1.5 2.3-1 4.8-4.5 7.5-10.5 49.3-97.3 121.7-169.3 217-216 28-14 57.3-25 88
-33 6.7-2 11-3.8 13-5.5 2-1.7 3-4.2 3-7.5s-1-5.8-3-7.5c-2-1.7-6.3-3.5-13-5.5-68
-17.3-128.2-47.8-180.5-91.5-52.3-43.7-93.8-96.2-124.5-157.5-9.3-8-15.3-12.3-18
-13h-6c-12 .7-18 4-18 10 0 2 1.7 7 5 15 23.3 46 52 87 86 123l10 10H0v40h399782
c-328 0 0 0 0 0l10 8c26.7 20 65.7 43 117 69-2.7 2-6 3.7-10 5-36.7 16-72.3 37.3
-107 64l-10 8H0v40zM0 157v40h399730v-40zm0 194v40h399730v-40z`,leftarrow:`M400000 241H110l3-3c68.7-52.7 113.7-120
 135-202 4-14.7 6-23 6-25 0-7.3-7-11-21-11-8 0-13.2.8-15.5 2.5-2.3 1.7-4.2 5.8
-5.5 12.5-1.3 4.7-2.7 10.3-4 17-12 48.7-34.8 92-68.5 130S65.3 228.3 18 247
c-10 4-16 7.7-18 11 0 8.7 6 14.3 18 17 47.3 18.7 87.8 47 121.5 85S196 441.3 208
 490c.7 2 1.3 5 2 9s1.2 6.7 1.5 8c.3 1.3 1 3.3 2 6s2.2 4.5 3.5 5.5c1.3 1 3.3
 1.8 6 2.5s6 1 10 1c14 0 21-3.7 21-11 0-2-2-10.3-6-25-20-79.3-65-146.7-135-202
 l-3-3h399890zM100 241v40h399900v-40z`,leftbrace:`M6 548l-6-6v-35l6-11c56-104 135.3-181.3 238-232 57.3-28.7 117
-45 179-50h399577v120H403c-43.3 7-81 15-113 26-100.7 33-179.7 91-237 174-2.7
 5-6 9-10 13-.7 1-7.3 1-20 1H6z`,leftbraceunder:`M0 6l6-6h17c12.688 0 19.313.3 20 1 4 4 7.313 8.3 10 13
 35.313 51.3 80.813 93.8 136.5 127.5 55.688 33.7 117.188 55.8 184.5 66.5.688
 0 2 .3 4 1 18.688 2.7 76 4.3 172 5h399450v120H429l-6-1c-124.688-8-235-61.7
-331-161C60.687 138.7 32.312 99.3 7 54L0 41V6z`,leftgroup:`M400000 80
H435C64 80 168.3 229.4 21 260c-5.9 1.2-18 0-18 0-2 0-3-1-3-3v-38C76 61 257 0
 435 0h399565z`,leftgroupunder:`M400000 262
H435C64 262 168.3 112.6 21 82c-5.9-1.2-18 0-18 0-2 0-3 1-3 3v38c76 158 257 219
 435 219h399565z`,leftharpoon:`M0 267c.7 5.3 3 10 7 14h399993v-40H93c3.3
-3.3 10.2-9.5 20.5-18.5s17.8-15.8 22.5-20.5c50.7-52 88-110.3 112-175 4-11.3 5
-18.3 3-21-1.3-4-7.3-6-18-6-8 0-13 .7-15 2s-4.7 6.7-8 16c-42 98.7-107.3 174.7
-196 228-6.7 4.7-10.7 8-12 10-1.3 2-2 5.7-2 11zm100-26v40h399900v-40z`,leftharpoonplus:`M0 267c.7 5.3 3 10 7 14h399993v-40H93c3.3-3.3 10.2-9.5
 20.5-18.5s17.8-15.8 22.5-20.5c50.7-52 88-110.3 112-175 4-11.3 5-18.3 3-21-1.3
-4-7.3-6-18-6-8 0-13 .7-15 2s-4.7 6.7-8 16c-42 98.7-107.3 174.7-196 228-6.7 4.7
-10.7 8-12 10-1.3 2-2 5.7-2 11zm100-26v40h399900v-40zM0 435v40h400000v-40z
m0 0v40h400000v-40z`,leftharpoondown:`M7 241c-4 4-6.333 8.667-7 14 0 5.333.667 9 2 11s5.333
 5.333 12 10c90.667 54 156 130 196 228 3.333 10.667 6.333 16.333 9 17 2 .667 5
 1 9 1h5c10.667 0 16.667-2 18-6 2-2.667 1-9.667-3-21-32-87.333-82.667-157.667
-152-211l-3-3h399907v-40zM93 281 H400000 v-40L7 241z`,leftharpoondownplus:`M7 435c-4 4-6.3 8.7-7 14 0 5.3.7 9 2 11s5.3 5.3 12
 10c90.7 54 156 130 196 228 3.3 10.7 6.3 16.3 9 17 2 .7 5 1 9 1h5c10.7 0 16.7
-2 18-6 2-2.7 1-9.7-3-21-32-87.3-82.7-157.7-152-211l-3-3h399907v-40H7zm93 0
v40h399900v-40zM0 241v40h399900v-40zm0 0v40h399900v-40z`,lefthook:`M400000 281 H103s-33-11.2-61-33.5S0 197.3 0 164s14.2-61.2 42.5
-83.5C70.8 58.2 104 47 142 47 c16.7 0 25 6.7 25 20 0 12-8.7 18.7-26 20-40 3.3
-68.7 15.7-86 37-10 12-15 25.3-15 40 0 22.7 9.8 40.7 29.5 54 19.7 13.3 43.5 21
 71.5 23h399859zM103 281v-40h399897v40z`,leftlinesegment:`M40 281 V428 H0 V94 H40 V241 H400000 v40z
M40 281 V428 H0 V94 H40 V241 H400000 v40z`,leftmapsto:`M40 281 V448H0V74H40V241H400000v40z
M40 281 V448H0V74H40V241H400000v40z`,leftToFrom:`M0 147h400000v40H0zm0 214c68 40 115.7 95.7 143 167h22c15.3 0 23
-.3 23-1 0-1.3-5.3-13.7-16-37-18-35.3-41.3-69-70-101l-7-8h399905v-40H95l7-8
c28.7-32 52-65.7 70-101 10.7-23.3 16-35.7 16-37 0-.7-7.7-1-23-1h-22C115.7 265.3
 68 321 0 361zm0-174v-40h399900v40zm100 154v40h399900v-40z`,longequal:`M0 50 h400000 v40H0z m0 194h40000v40H0z
M0 50 h400000 v40H0z m0 194h40000v40H0z`,midbrace:`M200428 334
c-100.7-8.3-195.3-44-280-108-55.3-42-101.7-93-139-153l-9-14c-2.7 4-5.7 8.7-9 14
-53.3 86.7-123.7 153-211 199-66.7 36-137.3 56.3-212 62H0V214h199568c178.3-11.7
 311.7-78.3 403-201 6-8 9.7-12 11-12 .7-.7 6.7-1 18-1s17.3.3 18 1c1.3 0 5 4 11
 12 44.7 59.3 101.3 106.3 170 141s145.3 54.3 229 60h199572v120z`,midbraceunder:`M199572 214
c100.7 8.3 195.3 44 280 108 55.3 42 101.7 93 139 153l9 14c2.7-4 5.7-8.7 9-14
 53.3-86.7 123.7-153 211-199 66.7-36 137.3-56.3 212-62h199568v120H200432c-178.3
 11.7-311.7 78.3-403 201-6 8-9.7 12-11 12-.7.7-6.7 1-18 1s-17.3-.3-18-1c-1.3 0
-5-4-11-12-44.7-59.3-101.3-106.3-170-141s-145.3-54.3-229-60H0V214z`,oiintSize1:`M512.6 71.6c272.6 0 320.3 106.8 320.3 178.2 0 70.8-47.7 177.6
-320.3 177.6S193.1 320.6 193.1 249.8c0-71.4 46.9-178.2 319.5-178.2z
m368.1 178.2c0-86.4-60.9-215.4-368.1-215.4-306.4 0-367.3 129-367.3 215.4 0 85.8
60.9 214.8 367.3 214.8 307.2 0 368.1-129 368.1-214.8z`,oiintSize2:`M757.8 100.1c384.7 0 451.1 137.6 451.1 230 0 91.3-66.4 228.8
-451.1 228.8-386.3 0-452.7-137.5-452.7-228.8 0-92.4 66.4-230 452.7-230z
m502.4 230c0-111.2-82.4-277.2-502.4-277.2s-504 166-504 277.2
c0 110 84 276 504 276s502.4-166 502.4-276z`,oiiintSize1:`M681.4 71.6c408.9 0 480.5 106.8 480.5 178.2 0 70.8-71.6 177.6
-480.5 177.6S202.1 320.6 202.1 249.8c0-71.4 70.5-178.2 479.3-178.2z
m525.8 178.2c0-86.4-86.8-215.4-525.7-215.4-437.9 0-524.7 129-524.7 215.4 0
85.8 86.8 214.8 524.7 214.8 438.9 0 525.7-129 525.7-214.8z`,oiiintSize2:`M1021.2 53c603.6 0 707.8 165.8 707.8 277.2 0 110-104.2 275.8
-707.8 275.8-606 0-710.2-165.8-710.2-275.8C311 218.8 415.2 53 1021.2 53z
m770.4 277.1c0-131.2-126.4-327.6-770.5-327.6S248.4 198.9 248.4 330.1
c0 130 128.8 326.4 772.7 326.4s770.5-196.4 770.5-326.4z`,rightarrow:`M0 241v40h399891c-47.3 35.3-84 78-110 128
-16.7 32-27.7 63.7-33 95 0 1.3-.2 2.7-.5 4-.3 1.3-.5 2.3-.5 3 0 7.3 6.7 11 20
 11 8 0 13.2-.8 15.5-2.5 2.3-1.7 4.2-5.5 5.5-11.5 2-13.3 5.7-27 11-41 14.7-44.7
 39-84.5 73-119.5s73.7-60.2 119-75.5c6-2 9-5.7 9-11s-3-9-9-11c-45.3-15.3-85
-40.5-119-75.5s-58.3-74.8-73-119.5c-4.7-14-8.3-27.3-11-40-1.3-6.7-3.2-10.8-5.5
-12.5-2.3-1.7-7.5-2.5-15.5-2.5-14 0-21 3.7-21 11 0 2 2 10.3 6 25 20.7 83.3 67
 151.7 139 205zm0 0v40h399900v-40z`,rightbrace:`M400000 542l
-6 6h-17c-12.7 0-19.3-.3-20-1-4-4-7.3-8.3-10-13-35.3-51.3-80.8-93.8-136.5-127.5
s-117.2-55.8-184.5-66.5c-.7 0-2-.3-4-1-18.7-2.7-76-4.3-172-5H0V214h399571l6 1
c124.7 8 235 61.7 331 161 31.3 33.3 59.7 72.7 85 118l7 13v35z`,rightbraceunder:`M399994 0l6 6v35l-6 11c-56 104-135.3 181.3-238 232-57.3
 28.7-117 45-179 50H-300V214h399897c43.3-7 81-15 113-26 100.7-33 179.7-91 237
-174 2.7-5 6-9 10-13 .7-1 7.3-1 20-1h17z`,rightgroup:`M0 80h399565c371 0 266.7 149.4 414 180 5.9 1.2 18 0 18 0 2 0
 3-1 3-3v-38c-76-158-257-219-435-219H0z`,rightgroupunder:`M0 262h399565c371 0 266.7-149.4 414-180 5.9-1.2 18 0 18
 0 2 0 3 1 3 3v38c-76 158-257 219-435 219H0z`,rightharpoon:`M0 241v40h399993c4.7-4.7 7-9.3 7-14 0-9.3
-3.7-15.3-11-18-92.7-56.7-159-133.7-199-231-3.3-9.3-6-14.7-8-16-2-1.3-7-2-15-2
-10.7 0-16.7 2-18 6-2 2.7-1 9.7 3 21 15.3 42 36.7 81.8 64 119.5 27.3 37.7 58
 69.2 92 94.5zm0 0v40h399900v-40z`,rightharpoonplus:`M0 241v40h399993c4.7-4.7 7-9.3 7-14 0-9.3-3.7-15.3-11
-18-92.7-56.7-159-133.7-199-231-3.3-9.3-6-14.7-8-16-2-1.3-7-2-15-2-10.7 0-16.7
 2-18 6-2 2.7-1 9.7 3 21 15.3 42 36.7 81.8 64 119.5 27.3 37.7 58 69.2 92 94.5z
m0 0v40h399900v-40z m100 194v40h399900v-40zm0 0v40h399900v-40z`,rightharpoondown:`M399747 511c0 7.3 6.7 11 20 11 8 0 13-.8 15-2.5s4.7-6.8
 8-15.5c40-94 99.3-166.3 178-217 13.3-8 20.3-12.3 21-13 5.3-3.3 8.5-5.8 9.5
-7.5 1-1.7 1.5-5.2 1.5-10.5s-2.3-10.3-7-15H0v40h399908c-34 25.3-64.7 57-92 95
-27.3 38-48.7 77.7-64 119-3.3 8.7-5 14-5 16zM0 241v40h399900v-40z`,rightharpoondownplus:`M399747 705c0 7.3 6.7 11 20 11 8 0 13-.8
 15-2.5s4.7-6.8 8-15.5c40-94 99.3-166.3 178-217 13.3-8 20.3-12.3 21-13 5.3-3.3
 8.5-5.8 9.5-7.5 1-1.7 1.5-5.2 1.5-10.5s-2.3-10.3-7-15H0v40h399908c-34 25.3
-64.7 57-92 95-27.3 38-48.7 77.7-64 119-3.3 8.7-5 14-5 16zM0 435v40h399900v-40z
m0-194v40h400000v-40zm0 0v40h400000v-40z`,righthook:`M399859 241c-764 0 0 0 0 0 40-3.3 68.7-15.7 86-37 10-12 15-25.3
 15-40 0-22.7-9.8-40.7-29.5-54-19.7-13.3-43.5-21-71.5-23-17.3-1.3-26-8-26-20 0
-13.3 8.7-20 26-20 38 0 71 11.2 99 33.5 0 0 7 5.6 21 16.7 14 11.2 21 33.5 21
 66.8s-14 61.2-42 83.5c-28 22.3-61 33.5-99 33.5L0 241z M0 281v-40h399859v40z`,rightlinesegment:`M399960 241 V94 h40 V428 h-40 V281 H0 v-40z
M399960 241 V94 h40 V428 h-40 V281 H0 v-40z`,rightToFrom:`M400000 167c-70.7-42-118-97.7-142-167h-23c-15.3 0-23 .3-23
 1 0 1.3 5.3 13.7 16 37 18 35.3 41.3 69 70 101l7 8H0v40h399905l-7 8c-28.7 32
-52 65.7-70 101-10.7 23.3-16 35.7-16 37 0 .7 7.7 1 23 1h23c24-69.3 71.3-125 142
-167z M100 147v40h399900v-40zM0 341v40h399900v-40z`,twoheadleftarrow:`M0 167c68 40
 115.7 95.7 143 167h22c15.3 0 23-.3 23-1 0-1.3-5.3-13.7-16-37-18-35.3-41.3-69
-70-101l-7-8h125l9 7c50.7 39.3 85 86 103 140h46c0-4.7-6.3-18.7-19-42-18-35.3
-40-67.3-66-96l-9-9h399716v-40H284l9-9c26-28.7 48-60.7 66-96 12.7-23.333 19
-37.333 19-42h-46c-18 54-52.3 100.7-103 140l-9 7H95l7-8c28.7-32 52-65.7 70-101
 10.7-23.333 16-35.7 16-37 0-.7-7.7-1-23-1h-22C115.7 71.3 68 127 0 167z`,twoheadrightarrow:`M400000 167
c-68-40-115.7-95.7-143-167h-22c-15.3 0-23 .3-23 1 0 1.3 5.3 13.7 16 37 18 35.3
 41.3 69 70 101l7 8h-125l-9-7c-50.7-39.3-85-86-103-140h-46c0 4.7 6.3 18.7 19 42
 18 35.3 40 67.3 66 96l9 9H0v40h399716l-9 9c-26 28.7-48 60.7-66 96-12.7 23.333
-19 37.333-19 42h46c18-54 52.3-100.7 103-140l9-7h125l-7 8c-28.7 32-52 65.7-70
 101-10.7 23.333-16 35.7-16 37 0 .7 7.7 1 23 1h22c27.3-71.3 75-127 143-167z`,tilde1:`M200 55.538c-77 0-168 73.953-177 73.953-3 0-7
-2.175-9-5.437L2 97c-1-2-2-4-2-6 0-4 2-7 5-9l20-12C116 12 171 0 207 0c86 0
 114 68 191 68 78 0 168-68 177-68 4 0 7 2 9 5l12 19c1 2.175 2 4.35 2 6.525 0
 4.35-2 7.613-5 9.788l-19 13.05c-92 63.077-116.937 75.308-183 76.128
-68.267.847-113-73.952-191-73.952z`,tilde2:`M344 55.266c-142 0-300.638 81.316-311.5 86.418
-8.01 3.762-22.5 10.91-23.5 5.562L1 120c-1-2-1-3-1-4 0-5 3-9 8-10l18.4-9C160.9
 31.9 283 0 358 0c148 0 188 122 331 122s314-97 326-97c4 0 8 2 10 7l7 21.114
c1 2.14 1 3.21 1 4.28 0 5.347-3 9.626-7 10.696l-22.3 12.622C852.6 158.372 751
 181.476 676 181.476c-149 0-189-126.21-332-126.21z`,tilde3:`M786 59C457 59 32 175.242 13 175.242c-6 0-10-3.457
-11-10.37L.15 138c-1-7 3-12 10-13l19.2-6.4C378.4 40.7 634.3 0 804.3 0c337 0
 411.8 157 746.8 157 328 0 754-112 773-112 5 0 10 3 11 9l1 14.075c1 8.066-.697
 16.595-6.697 17.492l-21.052 7.31c-367.9 98.146-609.15 122.696-778.15 122.696
 -338 0-409-156.573-744-156.573z`,tilde4:`M786 58C457 58 32 177.487 13 177.487c-6 0-10-3.345
-11-10.035L.15 143c-1-7 3-12 10-13l22-6.7C381.2 35 637.15 0 807.15 0c337 0 409
 177 744 177 328 0 754-127 773-127 5 0 10 3 11 9l1 14.794c1 7.805-3 13.38-9
 14.495l-20.7 5.574c-366.85 99.79-607.3 139.372-776.3 139.372-338 0-409
 -175.236-744-175.236z`,vec:`M377 20c0-5.333 1.833-10 5.5-14S391 0 397 0c4.667 0 8.667 1.667 12 5
3.333 2.667 6.667 9 10 19 6.667 24.667 20.333 43.667 41 57 7.333 4.667 11
10.667 11 18 0 6-1 10-3 12s-6.667 5-14 9c-28.667 14.667-53.667 35.667-75 63
-1.333 1.333-3.167 3.5-5.5 6.5s-4 4.833-5 5.5c-1 .667-2.5 1.333-4.5 2s-4.333 1
-7 1c-4.667 0-9.167-1.833-13.5-5.5S337 184 337 178c0-12.667 15.667-32.333 47-59
H213l-171-1c-8.667-6-13-12.333-13-19 0-4.667 4.333-11.333 13-20h359
c-16-25.333-24-45-24-59z`,widehat1:`M529 0h5l519 115c5 1 9 5 9 10 0 1-1 2-1 3l-4 22
c-1 5-5 9-11 9h-2L532 67 19 159h-2c-5 0-9-4-11-9l-5-22c-1-6 2-12 8-13z`,widehat2:`M1181 0h2l1171 176c6 0 10 5 10 11l-2 23c-1 6-5 10
-11 10h-1L1182 67 15 220h-1c-6 0-10-4-11-10l-2-23c-1-6 4-11 10-11z`,widehat3:`M1181 0h2l1171 236c6 0 10 5 10 11l-2 23c-1 6-5 10
-11 10h-1L1182 67 15 280h-1c-6 0-10-4-11-10l-2-23c-1-6 4-11 10-11z`,widehat4:`M1181 0h2l1171 296c6 0 10 5 10 11l-2 23c-1 6-5 10
-11 10h-1L1182 67 15 340h-1c-6 0-10-4-11-10l-2-23c-1-6 4-11 10-11z`,widecheck1:`M529,159h5l519,-115c5,-1,9,-5,9,-10c0,-1,-1,-2,-1,-3l-4,-22c-1,
-5,-5,-9,-11,-9h-2l-512,92l-513,-92h-2c-5,0,-9,4,-11,9l-5,22c-1,6,2,12,8,13z`,widecheck2:`M1181,220h2l1171,-176c6,0,10,-5,10,-11l-2,-23c-1,-6,-5,-10,
-11,-10h-1l-1168,153l-1167,-153h-1c-6,0,-10,4,-11,10l-2,23c-1,6,4,11,10,11z`,widecheck3:`M1181,280h2l1171,-236c6,0,10,-5,10,-11l-2,-23c-1,-6,-5,-10,
-11,-10h-1l-1168,213l-1167,-213h-1c-6,0,-10,4,-11,10l-2,23c-1,6,4,11,10,11z`,widecheck4:`M1181,340h2l1171,-296c6,0,10,-5,10,-11l-2,-23c-1,-6,-5,-10,
-11,-10h-1l-1168,273l-1167,-273h-1c-6,0,-10,4,-11,10l-2,23c-1,6,4,11,10,11z`,baraboveleftarrow:`M400000 620h-399890l3 -3c68.7 -52.7 113.7 -120 135 -202
c4 -14.7 6 -23 6 -25c0 -7.3 -7 -11 -21 -11c-8 0 -13.2 0.8 -15.5 2.5
c-2.3 1.7 -4.2 5.8 -5.5 12.5c-1.3 4.7 -2.7 10.3 -4 17c-12 48.7 -34.8 92 -68.5 130
s-74.2 66.3 -121.5 85c-10 4 -16 7.7 -18 11c0 8.7 6 14.3 18 17c47.3 18.7 87.8 47
121.5 85s56.5 81.3 68.5 130c0.7 2 1.3 5 2 9s1.2 6.7 1.5 8c0.3 1.3 1 3.3 2 6
s2.2 4.5 3.5 5.5c1.3 1 3.3 1.8 6 2.5s6 1 10 1c14 0 21 -3.7 21 -11
c0 -2 -2 -10.3 -6 -25c-20 -79.3 -65 -146.7 -135 -202l-3 -3h399890z
M100 620v40h399900v-40z M0 241v40h399900v-40zM0 241v40h399900v-40z`,rightarrowabovebar:`M0 241v40h399891c-47.3 35.3-84 78-110 128-16.7 32
-27.7 63.7-33 95 0 1.3-.2 2.7-.5 4-.3 1.3-.5 2.3-.5 3 0 7.3 6.7 11 20 11 8 0
13.2-.8 15.5-2.5 2.3-1.7 4.2-5.5 5.5-11.5 2-13.3 5.7-27 11-41 14.7-44.7 39
-84.5 73-119.5s73.7-60.2 119-75.5c6-2 9-5.7 9-11s-3-9-9-11c-45.3-15.3-85-40.5
-119-75.5s-58.3-74.8-73-119.5c-4.7-14-8.3-27.3-11-40-1.3-6.7-3.2-10.8-5.5
-12.5-2.3-1.7-7.5-2.5-15.5-2.5-14 0-21 3.7-21 11 0 2 2 10.3 6 25 20.7 83.3 67
151.7 139 205zm96 379h399894v40H0zm0 0h399904v40H0z`,baraboveshortleftharpoon:`M507,435c-4,4,-6.3,8.7,-7,14c0,5.3,0.7,9,2,11
c1.3,2,5.3,5.3,12,10c90.7,54,156,130,196,228c3.3,10.7,6.3,16.3,9,17
c2,0.7,5,1,9,1c0,0,5,0,5,0c10.7,0,16.7,-2,18,-6c2,-2.7,1,-9.7,-3,-21
c-32,-87.3,-82.7,-157.7,-152,-211c0,0,-3,-3,-3,-3l399351,0l0,-40
c-398570,0,-399437,0,-399437,0z M593 435 v40 H399500 v-40z
M0 281 v-40 H399908 v40z M0 281 v-40 H399908 v40z`,rightharpoonaboveshortbar:`M0,241 l0,40c399126,0,399993,0,399993,0
c4.7,-4.7,7,-9.3,7,-14c0,-9.3,-3.7,-15.3,-11,-18c-92.7,-56.7,-159,-133.7,-199,
-231c-3.3,-9.3,-6,-14.7,-8,-16c-2,-1.3,-7,-2,-15,-2c-10.7,0,-16.7,2,-18,6
c-2,2.7,-1,9.7,3,21c15.3,42,36.7,81.8,64,119.5c27.3,37.7,58,69.2,92,94.5z
M0 241 v40 H399908 v-40z M0 475 v-40 H399500 v40z M0 475 v-40 H399500 v40z`,shortbaraboveleftharpoon:`M7,435c-4,4,-6.3,8.7,-7,14c0,5.3,0.7,9,2,11
c1.3,2,5.3,5.3,12,10c90.7,54,156,130,196,228c3.3,10.7,6.3,16.3,9,17c2,0.7,5,1,9,
1c0,0,5,0,5,0c10.7,0,16.7,-2,18,-6c2,-2.7,1,-9.7,-3,-21c-32,-87.3,-82.7,-157.7,
-152,-211c0,0,-3,-3,-3,-3l399907,0l0,-40c-399126,0,-399993,0,-399993,0z
M93 435 v40 H400000 v-40z M500 241 v40 H400000 v-40z M500 241 v40 H400000 v-40z`,shortrightharpoonabovebar:`M53,241l0,40c398570,0,399437,0,399437,0
c4.7,-4.7,7,-9.3,7,-14c0,-9.3,-3.7,-15.3,-11,-18c-92.7,-56.7,-159,-133.7,-199,
-231c-3.3,-9.3,-6,-14.7,-8,-16c-2,-1.3,-7,-2,-15,-2c-10.7,0,-16.7,2,-18,6
c-2,2.7,-1,9.7,3,21c15.3,42,36.7,81.8,64,119.5c27.3,37.7,58,69.2,92,94.5z
M500 241 v40 H399408 v-40z M500 435 v40 H400000 v-40z`},rj=function(t,n){switch(t){case"lbrack":return"M403 1759 V84 H666 V0 H319 V1759 v"+n+` v1759 h347 v-84
H403z M403 1759 V0 H319 V1759 v`+n+" v1759 h84z";case"rbrack":return"M347 1759 V0 H0 V84 H263 V1759 v"+n+` v1759 H0 v84 H347z
M347 1759 V0 H263 V1759 v`+n+" v1759 h84z";case"vert":return"M145 15 v585 v"+n+` v585 c2.667,10,9.667,15,21,15
c10,0,16.667,-5,20,-15 v-585 v`+-n+` v-585 c-2.667,-10,-9.667,-15,-21,-15
c-10,0,-16.667,5,-20,15z M188 15 H145 v585 v`+n+" v585 h43z";case"doublevert":return"M145 15 v585 v"+n+` v585 c2.667,10,9.667,15,21,15
c10,0,16.667,-5,20,-15 v-585 v`+-n+` v-585 c-2.667,-10,-9.667,-15,-21,-15
c-10,0,-16.667,5,-20,15z M188 15 H145 v585 v`+n+` v585 h43z
M367 15 v585 v`+n+` v585 c2.667,10,9.667,15,21,15
c10,0,16.667,-5,20,-15 v-585 v`+-n+` v-585 c-2.667,-10,-9.667,-15,-21,-15
c-10,0,-16.667,5,-20,15z M410 15 H367 v585 v`+n+" v585 h43z";case"lfloor":return"M319 602 V0 H403 V602 v"+n+` v1715 h263 v84 H319z
MM319 602 V0 H403 V602 v`+n+" v1715 H319z";case"rfloor":return"M319 602 V0 H403 V602 v"+n+` v1799 H0 v-84 H319z
MM319 602 V0 H403 V602 v`+n+" v1715 H319z";case"lceil":return"M403 1759 V84 H666 V0 H319 V1759 v"+n+` v602 h84z
M403 1759 V0 H319 V1759 v`+n+" v602 h84z";case"rceil":return"M347 1759 V0 H0 V84 H263 V1759 v"+n+` v602 h84z
M347 1759 V0 h-84 V1759 v`+n+" v602 h84z";case"lparen":return`M863,9c0,-2,-2,-5,-6,-9c0,0,-17,0,-17,0c-12.7,0,-19.3,0.3,-20,1
c-5.3,5.3,-10.3,11,-15,17c-242.7,294.7,-395.3,682,-458,1162c-21.3,163.3,-33.3,349,
-36,557 l0,`+(n+84)+`c0.2,6,0,26,0,60c2,159.3,10,310.7,24,454c53.3,528,210,
949.7,470,1265c4.7,6,9.7,11.7,15,17c0.7,0.7,7,1,19,1c0,0,18,0,18,0c4,-4,6,-7,6,-9
c0,-2.7,-3.3,-8.7,-10,-18c-135.3,-192.7,-235.5,-414.3,-300.5,-665c-65,-250.7,-102.5,
-544.7,-112.5,-882c-2,-104,-3,-167,-3,-189
l0,-`+(n+92)+`c0,-162.7,5.7,-314,17,-454c20.7,-272,63.7,-513,129,-723c65.3,
-210,155.3,-396.3,270,-559c6.7,-9.3,10,-15.3,10,-18z`;case"rparen":return`M76,0c-16.7,0,-25,3,-25,9c0,2,2,6.3,6,13c21.3,28.7,42.3,60.3,
63,95c96.7,156.7,172.8,332.5,228.5,527.5c55.7,195,92.8,416.5,111.5,664.5
c11.3,139.3,17,290.7,17,454c0,28,1.7,43,3.3,45l0,`+(n+9)+`
c-3,4,-3.3,16.7,-3.3,38c0,162,-5.7,313.7,-17,455c-18.7,248,-55.8,469.3,-111.5,664
c-55.7,194.7,-131.8,370.3,-228.5,527c-20.7,34.7,-41.7,66.3,-63,95c-2,3.3,-4,7,-6,11
c0,7.3,5.7,11,17,11c0,0,11,0,11,0c9.3,0,14.3,-0.3,15,-1c5.3,-5.3,10.3,-11,15,-17
c242.7,-294.7,395.3,-681.7,458,-1161c21.3,-164.7,33.3,-350.7,36,-558
l0,-`+(n+144)+`c-2,-159.3,-10,-310.7,-24,-454c-53.3,-528,-210,-949.7,
-470,-1265c-4.7,-6,-9.7,-11.7,-15,-17c-0.7,-0.7,-6.7,-1,-18,-1z`;default:throw new Error("Unknown stretchy delimiter.")}};class P0{constructor(t){this.children=void 0,this.classes=void 0,this.height=void 0,this.depth=void 0,this.maxFontSize=void 0,this.style=void 0,this.children=t,this.classes=[],this.height=0,this.depth=0,this.maxFontSize=0,this.style={}}hasClass(t){return it.contains(this.classes,t)}toNode(){for(var t=document.createDocumentFragment(),n=0;n<this.children.length;n++)t.appendChild(this.children[n].toNode());return t}toMarkup(){for(var t="",n=0;n<this.children.length;n++)t+=this.children[n].toMarkup();return t}toText(){var t=n=>n.toText();return this.children.map(t).join("")}}var hi={"AMS-Regular":{32:[0,0,0,0,.25],65:[0,.68889,0,0,.72222],66:[0,.68889,0,0,.66667],67:[0,.68889,0,0,.72222],68:[0,.68889,0,0,.72222],69:[0,.68889,0,0,.66667],70:[0,.68889,0,0,.61111],71:[0,.68889,0,0,.77778],72:[0,.68889,0,0,.77778],73:[0,.68889,0,0,.38889],74:[.16667,.68889,0,0,.5],75:[0,.68889,0,0,.77778],76:[0,.68889,0,0,.66667],77:[0,.68889,0,0,.94445],78:[0,.68889,0,0,.72222],79:[.16667,.68889,0,0,.77778],80:[0,.68889,0,0,.61111],81:[.16667,.68889,0,0,.77778],82:[0,.68889,0,0,.72222],83:[0,.68889,0,0,.55556],84:[0,.68889,0,0,.66667],85:[0,.68889,0,0,.72222],86:[0,.68889,0,0,.72222],87:[0,.68889,0,0,1],88:[0,.68889,0,0,.72222],89:[0,.68889,0,0,.72222],90:[0,.68889,0,0,.66667],107:[0,.68889,0,0,.55556],160:[0,0,0,0,.25],165:[0,.675,.025,0,.75],174:[.15559,.69224,0,0,.94666],240:[0,.68889,0,0,.55556],295:[0,.68889,0,0,.54028],710:[0,.825,0,0,2.33334],732:[0,.9,0,0,2.33334],770:[0,.825,0,0,2.33334],771:[0,.9,0,0,2.33334],989:[.08167,.58167,0,0,.77778],1008:[0,.43056,.04028,0,.66667],8245:[0,.54986,0,0,.275],8463:[0,.68889,0,0,.54028],8487:[0,.68889,0,0,.72222],8498:[0,.68889,0,0,.55556],8502:[0,.68889,0,0,.66667],8503:[0,.68889,0,0,.44445],8504:[0,.68889,0,0,.66667],8513:[0,.68889,0,0,.63889],8592:[-.03598,.46402,0,0,.5],8594:[-.03598,.46402,0,0,.5],8602:[-.13313,.36687,0,0,1],8603:[-.13313,.36687,0,0,1],8606:[.01354,.52239,0,0,1],8608:[.01354,.52239,0,0,1],8610:[.01354,.52239,0,0,1.11111],8611:[.01354,.52239,0,0,1.11111],8619:[0,.54986,0,0,1],8620:[0,.54986,0,0,1],8621:[-.13313,.37788,0,0,1.38889],8622:[-.13313,.36687,0,0,1],8624:[0,.69224,0,0,.5],8625:[0,.69224,0,0,.5],8630:[0,.43056,0,0,1],8631:[0,.43056,0,0,1],8634:[.08198,.58198,0,0,.77778],8635:[.08198,.58198,0,0,.77778],8638:[.19444,.69224,0,0,.41667],8639:[.19444,.69224,0,0,.41667],8642:[.19444,.69224,0,0,.41667],8643:[.19444,.69224,0,0,.41667],8644:[.1808,.675,0,0,1],8646:[.1808,.675,0,0,1],8647:[.1808,.675,0,0,1],8648:[.19444,.69224,0,0,.83334],8649:[.1808,.675,0,0,1],8650:[.19444,.69224,0,0,.83334],8651:[.01354,.52239,0,0,1],8652:[.01354,.52239,0,0,1],8653:[-.13313,.36687,0,0,1],8654:[-.13313,.36687,0,0,1],8655:[-.13313,.36687,0,0,1],8666:[.13667,.63667,0,0,1],8667:[.13667,.63667,0,0,1],8669:[-.13313,.37788,0,0,1],8672:[-.064,.437,0,0,1.334],8674:[-.064,.437,0,0,1.334],8705:[0,.825,0,0,.5],8708:[0,.68889,0,0,.55556],8709:[.08167,.58167,0,0,.77778],8717:[0,.43056,0,0,.42917],8722:[-.03598,.46402,0,0,.5],8724:[.08198,.69224,0,0,.77778],8726:[.08167,.58167,0,0,.77778],8733:[0,.69224,0,0,.77778],8736:[0,.69224,0,0,.72222],8737:[0,.69224,0,0,.72222],8738:[.03517,.52239,0,0,.72222],8739:[.08167,.58167,0,0,.22222],8740:[.25142,.74111,0,0,.27778],8741:[.08167,.58167,0,0,.38889],8742:[.25142,.74111,0,0,.5],8756:[0,.69224,0,0,.66667],8757:[0,.69224,0,0,.66667],8764:[-.13313,.36687,0,0,.77778],8765:[-.13313,.37788,0,0,.77778],8769:[-.13313,.36687,0,0,.77778],8770:[-.03625,.46375,0,0,.77778],8774:[.30274,.79383,0,0,.77778],8776:[-.01688,.48312,0,0,.77778],8778:[.08167,.58167,0,0,.77778],8782:[.06062,.54986,0,0,.77778],8783:[.06062,.54986,0,0,.77778],8785:[.08198,.58198,0,0,.77778],8786:[.08198,.58198,0,0,.77778],8787:[.08198,.58198,0,0,.77778],8790:[0,.69224,0,0,.77778],8791:[.22958,.72958,0,0,.77778],8796:[.08198,.91667,0,0,.77778],8806:[.25583,.75583,0,0,.77778],8807:[.25583,.75583,0,0,.77778],8808:[.25142,.75726,0,0,.77778],8809:[.25142,.75726,0,0,.77778],8812:[.25583,.75583,0,0,.5],8814:[.20576,.70576,0,0,.77778],8815:[.20576,.70576,0,0,.77778],8816:[.30274,.79383,0,0,.77778],8817:[.30274,.79383,0,0,.77778],8818:[.22958,.72958,0,0,.77778],8819:[.22958,.72958,0,0,.77778],8822:[.1808,.675,0,0,.77778],8823:[.1808,.675,0,0,.77778],8828:[.13667,.63667,0,0,.77778],8829:[.13667,.63667,0,0,.77778],8830:[.22958,.72958,0,0,.77778],8831:[.22958,.72958,0,0,.77778],8832:[.20576,.70576,0,0,.77778],8833:[.20576,.70576,0,0,.77778],8840:[.30274,.79383,0,0,.77778],8841:[.30274,.79383,0,0,.77778],8842:[.13597,.63597,0,0,.77778],8843:[.13597,.63597,0,0,.77778],8847:[.03517,.54986,0,0,.77778],8848:[.03517,.54986,0,0,.77778],8858:[.08198,.58198,0,0,.77778],8859:[.08198,.58198,0,0,.77778],8861:[.08198,.58198,0,0,.77778],8862:[0,.675,0,0,.77778],8863:[0,.675,0,0,.77778],8864:[0,.675,0,0,.77778],8865:[0,.675,0,0,.77778],8872:[0,.69224,0,0,.61111],8873:[0,.69224,0,0,.72222],8874:[0,.69224,0,0,.88889],8876:[0,.68889,0,0,.61111],8877:[0,.68889,0,0,.61111],8878:[0,.68889,0,0,.72222],8879:[0,.68889,0,0,.72222],8882:[.03517,.54986,0,0,.77778],8883:[.03517,.54986,0,0,.77778],8884:[.13667,.63667,0,0,.77778],8885:[.13667,.63667,0,0,.77778],8888:[0,.54986,0,0,1.11111],8890:[.19444,.43056,0,0,.55556],8891:[.19444,.69224,0,0,.61111],8892:[.19444,.69224,0,0,.61111],8901:[0,.54986,0,0,.27778],8903:[.08167,.58167,0,0,.77778],8905:[.08167,.58167,0,0,.77778],8906:[.08167,.58167,0,0,.77778],8907:[0,.69224,0,0,.77778],8908:[0,.69224,0,0,.77778],8909:[-.03598,.46402,0,0,.77778],8910:[0,.54986,0,0,.76042],8911:[0,.54986,0,0,.76042],8912:[.03517,.54986,0,0,.77778],8913:[.03517,.54986,0,0,.77778],8914:[0,.54986,0,0,.66667],8915:[0,.54986,0,0,.66667],8916:[0,.69224,0,0,.66667],8918:[.0391,.5391,0,0,.77778],8919:[.0391,.5391,0,0,.77778],8920:[.03517,.54986,0,0,1.33334],8921:[.03517,.54986,0,0,1.33334],8922:[.38569,.88569,0,0,.77778],8923:[.38569,.88569,0,0,.77778],8926:[.13667,.63667,0,0,.77778],8927:[.13667,.63667,0,0,.77778],8928:[.30274,.79383,0,0,.77778],8929:[.30274,.79383,0,0,.77778],8934:[.23222,.74111,0,0,.77778],8935:[.23222,.74111,0,0,.77778],8936:[.23222,.74111,0,0,.77778],8937:[.23222,.74111,0,0,.77778],8938:[.20576,.70576,0,0,.77778],8939:[.20576,.70576,0,0,.77778],8940:[.30274,.79383,0,0,.77778],8941:[.30274,.79383,0,0,.77778],8994:[.19444,.69224,0,0,.77778],8995:[.19444,.69224,0,0,.77778],9416:[.15559,.69224,0,0,.90222],9484:[0,.69224,0,0,.5],9488:[0,.69224,0,0,.5],9492:[0,.37788,0,0,.5],9496:[0,.37788,0,0,.5],9585:[.19444,.68889,0,0,.88889],9586:[.19444,.74111,0,0,.88889],9632:[0,.675,0,0,.77778],9633:[0,.675,0,0,.77778],9650:[0,.54986,0,0,.72222],9651:[0,.54986,0,0,.72222],9654:[.03517,.54986,0,0,.77778],9660:[0,.54986,0,0,.72222],9661:[0,.54986,0,0,.72222],9664:[.03517,.54986,0,0,.77778],9674:[.11111,.69224,0,0,.66667],9733:[.19444,.69224,0,0,.94445],10003:[0,.69224,0,0,.83334],10016:[0,.69224,0,0,.83334],10731:[.11111,.69224,0,0,.66667],10846:[.19444,.75583,0,0,.61111],10877:[.13667,.63667,0,0,.77778],10878:[.13667,.63667,0,0,.77778],10885:[.25583,.75583,0,0,.77778],10886:[.25583,.75583,0,0,.77778],10887:[.13597,.63597,0,0,.77778],10888:[.13597,.63597,0,0,.77778],10889:[.26167,.75726,0,0,.77778],10890:[.26167,.75726,0,0,.77778],10891:[.48256,.98256,0,0,.77778],10892:[.48256,.98256,0,0,.77778],10901:[.13667,.63667,0,0,.77778],10902:[.13667,.63667,0,0,.77778],10933:[.25142,.75726,0,0,.77778],10934:[.25142,.75726,0,0,.77778],10935:[.26167,.75726,0,0,.77778],10936:[.26167,.75726,0,0,.77778],10937:[.26167,.75726,0,0,.77778],10938:[.26167,.75726,0,0,.77778],10949:[.25583,.75583,0,0,.77778],10950:[.25583,.75583,0,0,.77778],10955:[.28481,.79383,0,0,.77778],10956:[.28481,.79383,0,0,.77778],57350:[.08167,.58167,0,0,.22222],57351:[.08167,.58167,0,0,.38889],57352:[.08167,.58167,0,0,.77778],57353:[0,.43056,.04028,0,.66667],57356:[.25142,.75726,0,0,.77778],57357:[.25142,.75726,0,0,.77778],57358:[.41951,.91951,0,0,.77778],57359:[.30274,.79383,0,0,.77778],57360:[.30274,.79383,0,0,.77778],57361:[.41951,.91951,0,0,.77778],57366:[.25142,.75726,0,0,.77778],57367:[.25142,.75726,0,0,.77778],57368:[.25142,.75726,0,0,.77778],57369:[.25142,.75726,0,0,.77778],57370:[.13597,.63597,0,0,.77778],57371:[.13597,.63597,0,0,.77778]},"Caligraphic-Regular":{32:[0,0,0,0,.25],65:[0,.68333,0,.19445,.79847],66:[0,.68333,.03041,.13889,.65681],67:[0,.68333,.05834,.13889,.52653],68:[0,.68333,.02778,.08334,.77139],69:[0,.68333,.08944,.11111,.52778],70:[0,.68333,.09931,.11111,.71875],71:[.09722,.68333,.0593,.11111,.59487],72:[0,.68333,.00965,.11111,.84452],73:[0,.68333,.07382,0,.54452],74:[.09722,.68333,.18472,.16667,.67778],75:[0,.68333,.01445,.05556,.76195],76:[0,.68333,0,.13889,.68972],77:[0,.68333,0,.13889,1.2009],78:[0,.68333,.14736,.08334,.82049],79:[0,.68333,.02778,.11111,.79611],80:[0,.68333,.08222,.08334,.69556],81:[.09722,.68333,0,.11111,.81667],82:[0,.68333,0,.08334,.8475],83:[0,.68333,.075,.13889,.60556],84:[0,.68333,.25417,0,.54464],85:[0,.68333,.09931,.08334,.62583],86:[0,.68333,.08222,0,.61278],87:[0,.68333,.08222,.08334,.98778],88:[0,.68333,.14643,.13889,.7133],89:[.09722,.68333,.08222,.08334,.66834],90:[0,.68333,.07944,.13889,.72473],160:[0,0,0,0,.25]},"Fraktur-Regular":{32:[0,0,0,0,.25],33:[0,.69141,0,0,.29574],34:[0,.69141,0,0,.21471],38:[0,.69141,0,0,.73786],39:[0,.69141,0,0,.21201],40:[.24982,.74947,0,0,.38865],41:[.24982,.74947,0,0,.38865],42:[0,.62119,0,0,.27764],43:[.08319,.58283,0,0,.75623],44:[0,.10803,0,0,.27764],45:[.08319,.58283,0,0,.75623],46:[0,.10803,0,0,.27764],47:[.24982,.74947,0,0,.50181],48:[0,.47534,0,0,.50181],49:[0,.47534,0,0,.50181],50:[0,.47534,0,0,.50181],51:[.18906,.47534,0,0,.50181],52:[.18906,.47534,0,0,.50181],53:[.18906,.47534,0,0,.50181],54:[0,.69141,0,0,.50181],55:[.18906,.47534,0,0,.50181],56:[0,.69141,0,0,.50181],57:[.18906,.47534,0,0,.50181],58:[0,.47534,0,0,.21606],59:[.12604,.47534,0,0,.21606],61:[-.13099,.36866,0,0,.75623],63:[0,.69141,0,0,.36245],65:[0,.69141,0,0,.7176],66:[0,.69141,0,0,.88397],67:[0,.69141,0,0,.61254],68:[0,.69141,0,0,.83158],69:[0,.69141,0,0,.66278],70:[.12604,.69141,0,0,.61119],71:[0,.69141,0,0,.78539],72:[.06302,.69141,0,0,.7203],73:[0,.69141,0,0,.55448],74:[.12604,.69141,0,0,.55231],75:[0,.69141,0,0,.66845],76:[0,.69141,0,0,.66602],77:[0,.69141,0,0,1.04953],78:[0,.69141,0,0,.83212],79:[0,.69141,0,0,.82699],80:[.18906,.69141,0,0,.82753],81:[.03781,.69141,0,0,.82699],82:[0,.69141,0,0,.82807],83:[0,.69141,0,0,.82861],84:[0,.69141,0,0,.66899],85:[0,.69141,0,0,.64576],86:[0,.69141,0,0,.83131],87:[0,.69141,0,0,1.04602],88:[0,.69141,0,0,.71922],89:[.18906,.69141,0,0,.83293],90:[.12604,.69141,0,0,.60201],91:[.24982,.74947,0,0,.27764],93:[.24982,.74947,0,0,.27764],94:[0,.69141,0,0,.49965],97:[0,.47534,0,0,.50046],98:[0,.69141,0,0,.51315],99:[0,.47534,0,0,.38946],100:[0,.62119,0,0,.49857],101:[0,.47534,0,0,.40053],102:[.18906,.69141,0,0,.32626],103:[.18906,.47534,0,0,.5037],104:[.18906,.69141,0,0,.52126],105:[0,.69141,0,0,.27899],106:[0,.69141,0,0,.28088],107:[0,.69141,0,0,.38946],108:[0,.69141,0,0,.27953],109:[0,.47534,0,0,.76676],110:[0,.47534,0,0,.52666],111:[0,.47534,0,0,.48885],112:[.18906,.52396,0,0,.50046],113:[.18906,.47534,0,0,.48912],114:[0,.47534,0,0,.38919],115:[0,.47534,0,0,.44266],116:[0,.62119,0,0,.33301],117:[0,.47534,0,0,.5172],118:[0,.52396,0,0,.5118],119:[0,.52396,0,0,.77351],120:[.18906,.47534,0,0,.38865],121:[.18906,.47534,0,0,.49884],122:[.18906,.47534,0,0,.39054],160:[0,0,0,0,.25],8216:[0,.69141,0,0,.21471],8217:[0,.69141,0,0,.21471],58112:[0,.62119,0,0,.49749],58113:[0,.62119,0,0,.4983],58114:[.18906,.69141,0,0,.33328],58115:[.18906,.69141,0,0,.32923],58116:[.18906,.47534,0,0,.50343],58117:[0,.69141,0,0,.33301],58118:[0,.62119,0,0,.33409],58119:[0,.47534,0,0,.50073]},"Main-Bold":{32:[0,0,0,0,.25],33:[0,.69444,0,0,.35],34:[0,.69444,0,0,.60278],35:[.19444,.69444,0,0,.95833],36:[.05556,.75,0,0,.575],37:[.05556,.75,0,0,.95833],38:[0,.69444,0,0,.89444],39:[0,.69444,0,0,.31944],40:[.25,.75,0,0,.44722],41:[.25,.75,0,0,.44722],42:[0,.75,0,0,.575],43:[.13333,.63333,0,0,.89444],44:[.19444,.15556,0,0,.31944],45:[0,.44444,0,0,.38333],46:[0,.15556,0,0,.31944],47:[.25,.75,0,0,.575],48:[0,.64444,0,0,.575],49:[0,.64444,0,0,.575],50:[0,.64444,0,0,.575],51:[0,.64444,0,0,.575],52:[0,.64444,0,0,.575],53:[0,.64444,0,0,.575],54:[0,.64444,0,0,.575],55:[0,.64444,0,0,.575],56:[0,.64444,0,0,.575],57:[0,.64444,0,0,.575],58:[0,.44444,0,0,.31944],59:[.19444,.44444,0,0,.31944],60:[.08556,.58556,0,0,.89444],61:[-.10889,.39111,0,0,.89444],62:[.08556,.58556,0,0,.89444],63:[0,.69444,0,0,.54305],64:[0,.69444,0,0,.89444],65:[0,.68611,0,0,.86944],66:[0,.68611,0,0,.81805],67:[0,.68611,0,0,.83055],68:[0,.68611,0,0,.88194],69:[0,.68611,0,0,.75555],70:[0,.68611,0,0,.72361],71:[0,.68611,0,0,.90416],72:[0,.68611,0,0,.9],73:[0,.68611,0,0,.43611],74:[0,.68611,0,0,.59444],75:[0,.68611,0,0,.90138],76:[0,.68611,0,0,.69166],77:[0,.68611,0,0,1.09166],78:[0,.68611,0,0,.9],79:[0,.68611,0,0,.86388],80:[0,.68611,0,0,.78611],81:[.19444,.68611,0,0,.86388],82:[0,.68611,0,0,.8625],83:[0,.68611,0,0,.63889],84:[0,.68611,0,0,.8],85:[0,.68611,0,0,.88472],86:[0,.68611,.01597,0,.86944],87:[0,.68611,.01597,0,1.18888],88:[0,.68611,0,0,.86944],89:[0,.68611,.02875,0,.86944],90:[0,.68611,0,0,.70277],91:[.25,.75,0,0,.31944],92:[.25,.75,0,0,.575],93:[.25,.75,0,0,.31944],94:[0,.69444,0,0,.575],95:[.31,.13444,.03194,0,.575],97:[0,.44444,0,0,.55902],98:[0,.69444,0,0,.63889],99:[0,.44444,0,0,.51111],100:[0,.69444,0,0,.63889],101:[0,.44444,0,0,.52708],102:[0,.69444,.10903,0,.35139],103:[.19444,.44444,.01597,0,.575],104:[0,.69444,0,0,.63889],105:[0,.69444,0,0,.31944],106:[.19444,.69444,0,0,.35139],107:[0,.69444,0,0,.60694],108:[0,.69444,0,0,.31944],109:[0,.44444,0,0,.95833],110:[0,.44444,0,0,.63889],111:[0,.44444,0,0,.575],112:[.19444,.44444,0,0,.63889],113:[.19444,.44444,0,0,.60694],114:[0,.44444,0,0,.47361],115:[0,.44444,0,0,.45361],116:[0,.63492,0,0,.44722],117:[0,.44444,0,0,.63889],118:[0,.44444,.01597,0,.60694],119:[0,.44444,.01597,0,.83055],120:[0,.44444,0,0,.60694],121:[.19444,.44444,.01597,0,.60694],122:[0,.44444,0,0,.51111],123:[.25,.75,0,0,.575],124:[.25,.75,0,0,.31944],125:[.25,.75,0,0,.575],126:[.35,.34444,0,0,.575],160:[0,0,0,0,.25],163:[0,.69444,0,0,.86853],168:[0,.69444,0,0,.575],172:[0,.44444,0,0,.76666],176:[0,.69444,0,0,.86944],177:[.13333,.63333,0,0,.89444],184:[.17014,0,0,0,.51111],198:[0,.68611,0,0,1.04166],215:[.13333,.63333,0,0,.89444],216:[.04861,.73472,0,0,.89444],223:[0,.69444,0,0,.59722],230:[0,.44444,0,0,.83055],247:[.13333,.63333,0,0,.89444],248:[.09722,.54167,0,0,.575],305:[0,.44444,0,0,.31944],338:[0,.68611,0,0,1.16944],339:[0,.44444,0,0,.89444],567:[.19444,.44444,0,0,.35139],710:[0,.69444,0,0,.575],711:[0,.63194,0,0,.575],713:[0,.59611,0,0,.575],714:[0,.69444,0,0,.575],715:[0,.69444,0,0,.575],728:[0,.69444,0,0,.575],729:[0,.69444,0,0,.31944],730:[0,.69444,0,0,.86944],732:[0,.69444,0,0,.575],733:[0,.69444,0,0,.575],915:[0,.68611,0,0,.69166],916:[0,.68611,0,0,.95833],920:[0,.68611,0,0,.89444],923:[0,.68611,0,0,.80555],926:[0,.68611,0,0,.76666],928:[0,.68611,0,0,.9],931:[0,.68611,0,0,.83055],933:[0,.68611,0,0,.89444],934:[0,.68611,0,0,.83055],936:[0,.68611,0,0,.89444],937:[0,.68611,0,0,.83055],8211:[0,.44444,.03194,0,.575],8212:[0,.44444,.03194,0,1.14999],8216:[0,.69444,0,0,.31944],8217:[0,.69444,0,0,.31944],8220:[0,.69444,0,0,.60278],8221:[0,.69444,0,0,.60278],8224:[.19444,.69444,0,0,.51111],8225:[.19444,.69444,0,0,.51111],8242:[0,.55556,0,0,.34444],8407:[0,.72444,.15486,0,.575],8463:[0,.69444,0,0,.66759],8465:[0,.69444,0,0,.83055],8467:[0,.69444,0,0,.47361],8472:[.19444,.44444,0,0,.74027],8476:[0,.69444,0,0,.83055],8501:[0,.69444,0,0,.70277],8592:[-.10889,.39111,0,0,1.14999],8593:[.19444,.69444,0,0,.575],8594:[-.10889,.39111,0,0,1.14999],8595:[.19444,.69444,0,0,.575],8596:[-.10889,.39111,0,0,1.14999],8597:[.25,.75,0,0,.575],8598:[.19444,.69444,0,0,1.14999],8599:[.19444,.69444,0,0,1.14999],8600:[.19444,.69444,0,0,1.14999],8601:[.19444,.69444,0,0,1.14999],8636:[-.10889,.39111,0,0,1.14999],8637:[-.10889,.39111,0,0,1.14999],8640:[-.10889,.39111,0,0,1.14999],8641:[-.10889,.39111,0,0,1.14999],8656:[-.10889,.39111,0,0,1.14999],8657:[.19444,.69444,0,0,.70277],8658:[-.10889,.39111,0,0,1.14999],8659:[.19444,.69444,0,0,.70277],8660:[-.10889,.39111,0,0,1.14999],8661:[.25,.75,0,0,.70277],8704:[0,.69444,0,0,.63889],8706:[0,.69444,.06389,0,.62847],8707:[0,.69444,0,0,.63889],8709:[.05556,.75,0,0,.575],8711:[0,.68611,0,0,.95833],8712:[.08556,.58556,0,0,.76666],8715:[.08556,.58556,0,0,.76666],8722:[.13333,.63333,0,0,.89444],8723:[.13333,.63333,0,0,.89444],8725:[.25,.75,0,0,.575],8726:[.25,.75,0,0,.575],8727:[-.02778,.47222,0,0,.575],8728:[-.02639,.47361,0,0,.575],8729:[-.02639,.47361,0,0,.575],8730:[.18,.82,0,0,.95833],8733:[0,.44444,0,0,.89444],8734:[0,.44444,0,0,1.14999],8736:[0,.69224,0,0,.72222],8739:[.25,.75,0,0,.31944],8741:[.25,.75,0,0,.575],8743:[0,.55556,0,0,.76666],8744:[0,.55556,0,0,.76666],8745:[0,.55556,0,0,.76666],8746:[0,.55556,0,0,.76666],8747:[.19444,.69444,.12778,0,.56875],8764:[-.10889,.39111,0,0,.89444],8768:[.19444,.69444,0,0,.31944],8771:[.00222,.50222,0,0,.89444],8773:[.027,.638,0,0,.894],8776:[.02444,.52444,0,0,.89444],8781:[.00222,.50222,0,0,.89444],8801:[.00222,.50222,0,0,.89444],8804:[.19667,.69667,0,0,.89444],8805:[.19667,.69667,0,0,.89444],8810:[.08556,.58556,0,0,1.14999],8811:[.08556,.58556,0,0,1.14999],8826:[.08556,.58556,0,0,.89444],8827:[.08556,.58556,0,0,.89444],8834:[.08556,.58556,0,0,.89444],8835:[.08556,.58556,0,0,.89444],8838:[.19667,.69667,0,0,.89444],8839:[.19667,.69667,0,0,.89444],8846:[0,.55556,0,0,.76666],8849:[.19667,.69667,0,0,.89444],8850:[.19667,.69667,0,0,.89444],8851:[0,.55556,0,0,.76666],8852:[0,.55556,0,0,.76666],8853:[.13333,.63333,0,0,.89444],8854:[.13333,.63333,0,0,.89444],8855:[.13333,.63333,0,0,.89444],8856:[.13333,.63333,0,0,.89444],8857:[.13333,.63333,0,0,.89444],8866:[0,.69444,0,0,.70277],8867:[0,.69444,0,0,.70277],8868:[0,.69444,0,0,.89444],8869:[0,.69444,0,0,.89444],8900:[-.02639,.47361,0,0,.575],8901:[-.02639,.47361,0,0,.31944],8902:[-.02778,.47222,0,0,.575],8968:[.25,.75,0,0,.51111],8969:[.25,.75,0,0,.51111],8970:[.25,.75,0,0,.51111],8971:[.25,.75,0,0,.51111],8994:[-.13889,.36111,0,0,1.14999],8995:[-.13889,.36111,0,0,1.14999],9651:[.19444,.69444,0,0,1.02222],9657:[-.02778,.47222,0,0,.575],9661:[.19444,.69444,0,0,1.02222],9667:[-.02778,.47222,0,0,.575],9711:[.19444,.69444,0,0,1.14999],9824:[.12963,.69444,0,0,.89444],9825:[.12963,.69444,0,0,.89444],9826:[.12963,.69444,0,0,.89444],9827:[.12963,.69444,0,0,.89444],9837:[0,.75,0,0,.44722],9838:[.19444,.69444,0,0,.44722],9839:[.19444,.69444,0,0,.44722],10216:[.25,.75,0,0,.44722],10217:[.25,.75,0,0,.44722],10815:[0,.68611,0,0,.9],10927:[.19667,.69667,0,0,.89444],10928:[.19667,.69667,0,0,.89444],57376:[.19444,.69444,0,0,0]},"Main-BoldItalic":{32:[0,0,0,0,.25],33:[0,.69444,.11417,0,.38611],34:[0,.69444,.07939,0,.62055],35:[.19444,.69444,.06833,0,.94444],37:[.05556,.75,.12861,0,.94444],38:[0,.69444,.08528,0,.88555],39:[0,.69444,.12945,0,.35555],40:[.25,.75,.15806,0,.47333],41:[.25,.75,.03306,0,.47333],42:[0,.75,.14333,0,.59111],43:[.10333,.60333,.03306,0,.88555],44:[.19444,.14722,0,0,.35555],45:[0,.44444,.02611,0,.41444],46:[0,.14722,0,0,.35555],47:[.25,.75,.15806,0,.59111],48:[0,.64444,.13167,0,.59111],49:[0,.64444,.13167,0,.59111],50:[0,.64444,.13167,0,.59111],51:[0,.64444,.13167,0,.59111],52:[.19444,.64444,.13167,0,.59111],53:[0,.64444,.13167,0,.59111],54:[0,.64444,.13167,0,.59111],55:[.19444,.64444,.13167,0,.59111],56:[0,.64444,.13167,0,.59111],57:[0,.64444,.13167,0,.59111],58:[0,.44444,.06695,0,.35555],59:[.19444,.44444,.06695,0,.35555],61:[-.10889,.39111,.06833,0,.88555],63:[0,.69444,.11472,0,.59111],64:[0,.69444,.09208,0,.88555],65:[0,.68611,0,0,.86555],66:[0,.68611,.0992,0,.81666],67:[0,.68611,.14208,0,.82666],68:[0,.68611,.09062,0,.87555],69:[0,.68611,.11431,0,.75666],70:[0,.68611,.12903,0,.72722],71:[0,.68611,.07347,0,.89527],72:[0,.68611,.17208,0,.8961],73:[0,.68611,.15681,0,.47166],74:[0,.68611,.145,0,.61055],75:[0,.68611,.14208,0,.89499],76:[0,.68611,0,0,.69777],77:[0,.68611,.17208,0,1.07277],78:[0,.68611,.17208,0,.8961],79:[0,.68611,.09062,0,.85499],80:[0,.68611,.0992,0,.78721],81:[.19444,.68611,.09062,0,.85499],82:[0,.68611,.02559,0,.85944],83:[0,.68611,.11264,0,.64999],84:[0,.68611,.12903,0,.7961],85:[0,.68611,.17208,0,.88083],86:[0,.68611,.18625,0,.86555],87:[0,.68611,.18625,0,1.15999],88:[0,.68611,.15681,0,.86555],89:[0,.68611,.19803,0,.86555],90:[0,.68611,.14208,0,.70888],91:[.25,.75,.1875,0,.35611],93:[.25,.75,.09972,0,.35611],94:[0,.69444,.06709,0,.59111],95:[.31,.13444,.09811,0,.59111],97:[0,.44444,.09426,0,.59111],98:[0,.69444,.07861,0,.53222],99:[0,.44444,.05222,0,.53222],100:[0,.69444,.10861,0,.59111],101:[0,.44444,.085,0,.53222],102:[.19444,.69444,.21778,0,.4],103:[.19444,.44444,.105,0,.53222],104:[0,.69444,.09426,0,.59111],105:[0,.69326,.11387,0,.35555],106:[.19444,.69326,.1672,0,.35555],107:[0,.69444,.11111,0,.53222],108:[0,.69444,.10861,0,.29666],109:[0,.44444,.09426,0,.94444],110:[0,.44444,.09426,0,.64999],111:[0,.44444,.07861,0,.59111],112:[.19444,.44444,.07861,0,.59111],113:[.19444,.44444,.105,0,.53222],114:[0,.44444,.11111,0,.50167],115:[0,.44444,.08167,0,.48694],116:[0,.63492,.09639,0,.385],117:[0,.44444,.09426,0,.62055],118:[0,.44444,.11111,0,.53222],119:[0,.44444,.11111,0,.76777],120:[0,.44444,.12583,0,.56055],121:[.19444,.44444,.105,0,.56166],122:[0,.44444,.13889,0,.49055],126:[.35,.34444,.11472,0,.59111],160:[0,0,0,0,.25],168:[0,.69444,.11473,0,.59111],176:[0,.69444,0,0,.94888],184:[.17014,0,0,0,.53222],198:[0,.68611,.11431,0,1.02277],216:[.04861,.73472,.09062,0,.88555],223:[.19444,.69444,.09736,0,.665],230:[0,.44444,.085,0,.82666],248:[.09722,.54167,.09458,0,.59111],305:[0,.44444,.09426,0,.35555],338:[0,.68611,.11431,0,1.14054],339:[0,.44444,.085,0,.82666],567:[.19444,.44444,.04611,0,.385],710:[0,.69444,.06709,0,.59111],711:[0,.63194,.08271,0,.59111],713:[0,.59444,.10444,0,.59111],714:[0,.69444,.08528,0,.59111],715:[0,.69444,0,0,.59111],728:[0,.69444,.10333,0,.59111],729:[0,.69444,.12945,0,.35555],730:[0,.69444,0,0,.94888],732:[0,.69444,.11472,0,.59111],733:[0,.69444,.11472,0,.59111],915:[0,.68611,.12903,0,.69777],916:[0,.68611,0,0,.94444],920:[0,.68611,.09062,0,.88555],923:[0,.68611,0,0,.80666],926:[0,.68611,.15092,0,.76777],928:[0,.68611,.17208,0,.8961],931:[0,.68611,.11431,0,.82666],933:[0,.68611,.10778,0,.88555],934:[0,.68611,.05632,0,.82666],936:[0,.68611,.10778,0,.88555],937:[0,.68611,.0992,0,.82666],8211:[0,.44444,.09811,0,.59111],8212:[0,.44444,.09811,0,1.18221],8216:[0,.69444,.12945,0,.35555],8217:[0,.69444,.12945,0,.35555],8220:[0,.69444,.16772,0,.62055],8221:[0,.69444,.07939,0,.62055]},"Main-Italic":{32:[0,0,0,0,.25],33:[0,.69444,.12417,0,.30667],34:[0,.69444,.06961,0,.51444],35:[.19444,.69444,.06616,0,.81777],37:[.05556,.75,.13639,0,.81777],38:[0,.69444,.09694,0,.76666],39:[0,.69444,.12417,0,.30667],40:[.25,.75,.16194,0,.40889],41:[.25,.75,.03694,0,.40889],42:[0,.75,.14917,0,.51111],43:[.05667,.56167,.03694,0,.76666],44:[.19444,.10556,0,0,.30667],45:[0,.43056,.02826,0,.35778],46:[0,.10556,0,0,.30667],47:[.25,.75,.16194,0,.51111],48:[0,.64444,.13556,0,.51111],49:[0,.64444,.13556,0,.51111],50:[0,.64444,.13556,0,.51111],51:[0,.64444,.13556,0,.51111],52:[.19444,.64444,.13556,0,.51111],53:[0,.64444,.13556,0,.51111],54:[0,.64444,.13556,0,.51111],55:[.19444,.64444,.13556,0,.51111],56:[0,.64444,.13556,0,.51111],57:[0,.64444,.13556,0,.51111],58:[0,.43056,.0582,0,.30667],59:[.19444,.43056,.0582,0,.30667],61:[-.13313,.36687,.06616,0,.76666],63:[0,.69444,.1225,0,.51111],64:[0,.69444,.09597,0,.76666],65:[0,.68333,0,0,.74333],66:[0,.68333,.10257,0,.70389],67:[0,.68333,.14528,0,.71555],68:[0,.68333,.09403,0,.755],69:[0,.68333,.12028,0,.67833],70:[0,.68333,.13305,0,.65277],71:[0,.68333,.08722,0,.77361],72:[0,.68333,.16389,0,.74333],73:[0,.68333,.15806,0,.38555],74:[0,.68333,.14028,0,.525],75:[0,.68333,.14528,0,.76888],76:[0,.68333,0,0,.62722],77:[0,.68333,.16389,0,.89666],78:[0,.68333,.16389,0,.74333],79:[0,.68333,.09403,0,.76666],80:[0,.68333,.10257,0,.67833],81:[.19444,.68333,.09403,0,.76666],82:[0,.68333,.03868,0,.72944],83:[0,.68333,.11972,0,.56222],84:[0,.68333,.13305,0,.71555],85:[0,.68333,.16389,0,.74333],86:[0,.68333,.18361,0,.74333],87:[0,.68333,.18361,0,.99888],88:[0,.68333,.15806,0,.74333],89:[0,.68333,.19383,0,.74333],90:[0,.68333,.14528,0,.61333],91:[.25,.75,.1875,0,.30667],93:[.25,.75,.10528,0,.30667],94:[0,.69444,.06646,0,.51111],95:[.31,.12056,.09208,0,.51111],97:[0,.43056,.07671,0,.51111],98:[0,.69444,.06312,0,.46],99:[0,.43056,.05653,0,.46],100:[0,.69444,.10333,0,.51111],101:[0,.43056,.07514,0,.46],102:[.19444,.69444,.21194,0,.30667],103:[.19444,.43056,.08847,0,.46],104:[0,.69444,.07671,0,.51111],105:[0,.65536,.1019,0,.30667],106:[.19444,.65536,.14467,0,.30667],107:[0,.69444,.10764,0,.46],108:[0,.69444,.10333,0,.25555],109:[0,.43056,.07671,0,.81777],110:[0,.43056,.07671,0,.56222],111:[0,.43056,.06312,0,.51111],112:[.19444,.43056,.06312,0,.51111],113:[.19444,.43056,.08847,0,.46],114:[0,.43056,.10764,0,.42166],115:[0,.43056,.08208,0,.40889],116:[0,.61508,.09486,0,.33222],117:[0,.43056,.07671,0,.53666],118:[0,.43056,.10764,0,.46],119:[0,.43056,.10764,0,.66444],120:[0,.43056,.12042,0,.46389],121:[.19444,.43056,.08847,0,.48555],122:[0,.43056,.12292,0,.40889],126:[.35,.31786,.11585,0,.51111],160:[0,0,0,0,.25],168:[0,.66786,.10474,0,.51111],176:[0,.69444,0,0,.83129],184:[.17014,0,0,0,.46],198:[0,.68333,.12028,0,.88277],216:[.04861,.73194,.09403,0,.76666],223:[.19444,.69444,.10514,0,.53666],230:[0,.43056,.07514,0,.71555],248:[.09722,.52778,.09194,0,.51111],338:[0,.68333,.12028,0,.98499],339:[0,.43056,.07514,0,.71555],710:[0,.69444,.06646,0,.51111],711:[0,.62847,.08295,0,.51111],713:[0,.56167,.10333,0,.51111],714:[0,.69444,.09694,0,.51111],715:[0,.69444,0,0,.51111],728:[0,.69444,.10806,0,.51111],729:[0,.66786,.11752,0,.30667],730:[0,.69444,0,0,.83129],732:[0,.66786,.11585,0,.51111],733:[0,.69444,.1225,0,.51111],915:[0,.68333,.13305,0,.62722],916:[0,.68333,0,0,.81777],920:[0,.68333,.09403,0,.76666],923:[0,.68333,0,0,.69222],926:[0,.68333,.15294,0,.66444],928:[0,.68333,.16389,0,.74333],931:[0,.68333,.12028,0,.71555],933:[0,.68333,.11111,0,.76666],934:[0,.68333,.05986,0,.71555],936:[0,.68333,.11111,0,.76666],937:[0,.68333,.10257,0,.71555],8211:[0,.43056,.09208,0,.51111],8212:[0,.43056,.09208,0,1.02222],8216:[0,.69444,.12417,0,.30667],8217:[0,.69444,.12417,0,.30667],8220:[0,.69444,.1685,0,.51444],8221:[0,.69444,.06961,0,.51444],8463:[0,.68889,0,0,.54028]},"Main-Regular":{32:[0,0,0,0,.25],33:[0,.69444,0,0,.27778],34:[0,.69444,0,0,.5],35:[.19444,.69444,0,0,.83334],36:[.05556,.75,0,0,.5],37:[.05556,.75,0,0,.83334],38:[0,.69444,0,0,.77778],39:[0,.69444,0,0,.27778],40:[.25,.75,0,0,.38889],41:[.25,.75,0,0,.38889],42:[0,.75,0,0,.5],43:[.08333,.58333,0,0,.77778],44:[.19444,.10556,0,0,.27778],45:[0,.43056,0,0,.33333],46:[0,.10556,0,0,.27778],47:[.25,.75,0,0,.5],48:[0,.64444,0,0,.5],49:[0,.64444,0,0,.5],50:[0,.64444,0,0,.5],51:[0,.64444,0,0,.5],52:[0,.64444,0,0,.5],53:[0,.64444,0,0,.5],54:[0,.64444,0,0,.5],55:[0,.64444,0,0,.5],56:[0,.64444,0,0,.5],57:[0,.64444,0,0,.5],58:[0,.43056,0,0,.27778],59:[.19444,.43056,0,0,.27778],60:[.0391,.5391,0,0,.77778],61:[-.13313,.36687,0,0,.77778],62:[.0391,.5391,0,0,.77778],63:[0,.69444,0,0,.47222],64:[0,.69444,0,0,.77778],65:[0,.68333,0,0,.75],66:[0,.68333,0,0,.70834],67:[0,.68333,0,0,.72222],68:[0,.68333,0,0,.76389],69:[0,.68333,0,0,.68056],70:[0,.68333,0,0,.65278],71:[0,.68333,0,0,.78472],72:[0,.68333,0,0,.75],73:[0,.68333,0,0,.36111],74:[0,.68333,0,0,.51389],75:[0,.68333,0,0,.77778],76:[0,.68333,0,0,.625],77:[0,.68333,0,0,.91667],78:[0,.68333,0,0,.75],79:[0,.68333,0,0,.77778],80:[0,.68333,0,0,.68056],81:[.19444,.68333,0,0,.77778],82:[0,.68333,0,0,.73611],83:[0,.68333,0,0,.55556],84:[0,.68333,0,0,.72222],85:[0,.68333,0,0,.75],86:[0,.68333,.01389,0,.75],87:[0,.68333,.01389,0,1.02778],88:[0,.68333,0,0,.75],89:[0,.68333,.025,0,.75],90:[0,.68333,0,0,.61111],91:[.25,.75,0,0,.27778],92:[.25,.75,0,0,.5],93:[.25,.75,0,0,.27778],94:[0,.69444,0,0,.5],95:[.31,.12056,.02778,0,.5],97:[0,.43056,0,0,.5],98:[0,.69444,0,0,.55556],99:[0,.43056,0,0,.44445],100:[0,.69444,0,0,.55556],101:[0,.43056,0,0,.44445],102:[0,.69444,.07778,0,.30556],103:[.19444,.43056,.01389,0,.5],104:[0,.69444,0,0,.55556],105:[0,.66786,0,0,.27778],106:[.19444,.66786,0,0,.30556],107:[0,.69444,0,0,.52778],108:[0,.69444,0,0,.27778],109:[0,.43056,0,0,.83334],110:[0,.43056,0,0,.55556],111:[0,.43056,0,0,.5],112:[.19444,.43056,0,0,.55556],113:[.19444,.43056,0,0,.52778],114:[0,.43056,0,0,.39167],115:[0,.43056,0,0,.39445],116:[0,.61508,0,0,.38889],117:[0,.43056,0,0,.55556],118:[0,.43056,.01389,0,.52778],119:[0,.43056,.01389,0,.72222],120:[0,.43056,0,0,.52778],121:[.19444,.43056,.01389,0,.52778],122:[0,.43056,0,0,.44445],123:[.25,.75,0,0,.5],124:[.25,.75,0,0,.27778],125:[.25,.75,0,0,.5],126:[.35,.31786,0,0,.5],160:[0,0,0,0,.25],163:[0,.69444,0,0,.76909],167:[.19444,.69444,0,0,.44445],168:[0,.66786,0,0,.5],172:[0,.43056,0,0,.66667],176:[0,.69444,0,0,.75],177:[.08333,.58333,0,0,.77778],182:[.19444,.69444,0,0,.61111],184:[.17014,0,0,0,.44445],198:[0,.68333,0,0,.90278],215:[.08333,.58333,0,0,.77778],216:[.04861,.73194,0,0,.77778],223:[0,.69444,0,0,.5],230:[0,.43056,0,0,.72222],247:[.08333,.58333,0,0,.77778],248:[.09722,.52778,0,0,.5],305:[0,.43056,0,0,.27778],338:[0,.68333,0,0,1.01389],339:[0,.43056,0,0,.77778],567:[.19444,.43056,0,0,.30556],710:[0,.69444,0,0,.5],711:[0,.62847,0,0,.5],713:[0,.56778,0,0,.5],714:[0,.69444,0,0,.5],715:[0,.69444,0,0,.5],728:[0,.69444,0,0,.5],729:[0,.66786,0,0,.27778],730:[0,.69444,0,0,.75],732:[0,.66786,0,0,.5],733:[0,.69444,0,0,.5],915:[0,.68333,0,0,.625],916:[0,.68333,0,0,.83334],920:[0,.68333,0,0,.77778],923:[0,.68333,0,0,.69445],926:[0,.68333,0,0,.66667],928:[0,.68333,0,0,.75],931:[0,.68333,0,0,.72222],933:[0,.68333,0,0,.77778],934:[0,.68333,0,0,.72222],936:[0,.68333,0,0,.77778],937:[0,.68333,0,0,.72222],8211:[0,.43056,.02778,0,.5],8212:[0,.43056,.02778,0,1],8216:[0,.69444,0,0,.27778],8217:[0,.69444,0,0,.27778],8220:[0,.69444,0,0,.5],8221:[0,.69444,0,0,.5],8224:[.19444,.69444,0,0,.44445],8225:[.19444,.69444,0,0,.44445],8230:[0,.123,0,0,1.172],8242:[0,.55556,0,0,.275],8407:[0,.71444,.15382,0,.5],8463:[0,.68889,0,0,.54028],8465:[0,.69444,0,0,.72222],8467:[0,.69444,0,.11111,.41667],8472:[.19444,.43056,0,.11111,.63646],8476:[0,.69444,0,0,.72222],8501:[0,.69444,0,0,.61111],8592:[-.13313,.36687,0,0,1],8593:[.19444,.69444,0,0,.5],8594:[-.13313,.36687,0,0,1],8595:[.19444,.69444,0,0,.5],8596:[-.13313,.36687,0,0,1],8597:[.25,.75,0,0,.5],8598:[.19444,.69444,0,0,1],8599:[.19444,.69444,0,0,1],8600:[.19444,.69444,0,0,1],8601:[.19444,.69444,0,0,1],8614:[.011,.511,0,0,1],8617:[.011,.511,0,0,1.126],8618:[.011,.511,0,0,1.126],8636:[-.13313,.36687,0,0,1],8637:[-.13313,.36687,0,0,1],8640:[-.13313,.36687,0,0,1],8641:[-.13313,.36687,0,0,1],8652:[.011,.671,0,0,1],8656:[-.13313,.36687,0,0,1],8657:[.19444,.69444,0,0,.61111],8658:[-.13313,.36687,0,0,1],8659:[.19444,.69444,0,0,.61111],8660:[-.13313,.36687,0,0,1],8661:[.25,.75,0,0,.61111],8704:[0,.69444,0,0,.55556],8706:[0,.69444,.05556,.08334,.5309],8707:[0,.69444,0,0,.55556],8709:[.05556,.75,0,0,.5],8711:[0,.68333,0,0,.83334],8712:[.0391,.5391,0,0,.66667],8715:[.0391,.5391,0,0,.66667],8722:[.08333,.58333,0,0,.77778],8723:[.08333,.58333,0,0,.77778],8725:[.25,.75,0,0,.5],8726:[.25,.75,0,0,.5],8727:[-.03472,.46528,0,0,.5],8728:[-.05555,.44445,0,0,.5],8729:[-.05555,.44445,0,0,.5],8730:[.2,.8,0,0,.83334],8733:[0,.43056,0,0,.77778],8734:[0,.43056,0,0,1],8736:[0,.69224,0,0,.72222],8739:[.25,.75,0,0,.27778],8741:[.25,.75,0,0,.5],8743:[0,.55556,0,0,.66667],8744:[0,.55556,0,0,.66667],8745:[0,.55556,0,0,.66667],8746:[0,.55556,0,0,.66667],8747:[.19444,.69444,.11111,0,.41667],8764:[-.13313,.36687,0,0,.77778],8768:[.19444,.69444,0,0,.27778],8771:[-.03625,.46375,0,0,.77778],8773:[-.022,.589,0,0,.778],8776:[-.01688,.48312,0,0,.77778],8781:[-.03625,.46375,0,0,.77778],8784:[-.133,.673,0,0,.778],8801:[-.03625,.46375,0,0,.77778],8804:[.13597,.63597,0,0,.77778],8805:[.13597,.63597,0,0,.77778],8810:[.0391,.5391,0,0,1],8811:[.0391,.5391,0,0,1],8826:[.0391,.5391,0,0,.77778],8827:[.0391,.5391,0,0,.77778],8834:[.0391,.5391,0,0,.77778],8835:[.0391,.5391,0,0,.77778],8838:[.13597,.63597,0,0,.77778],8839:[.13597,.63597,0,0,.77778],8846:[0,.55556,0,0,.66667],8849:[.13597,.63597,0,0,.77778],8850:[.13597,.63597,0,0,.77778],8851:[0,.55556,0,0,.66667],8852:[0,.55556,0,0,.66667],8853:[.08333,.58333,0,0,.77778],8854:[.08333,.58333,0,0,.77778],8855:[.08333,.58333,0,0,.77778],8856:[.08333,.58333,0,0,.77778],8857:[.08333,.58333,0,0,.77778],8866:[0,.69444,0,0,.61111],8867:[0,.69444,0,0,.61111],8868:[0,.69444,0,0,.77778],8869:[0,.69444,0,0,.77778],8872:[.249,.75,0,0,.867],8900:[-.05555,.44445,0,0,.5],8901:[-.05555,.44445,0,0,.27778],8902:[-.03472,.46528,0,0,.5],8904:[.005,.505,0,0,.9],8942:[.03,.903,0,0,.278],8943:[-.19,.313,0,0,1.172],8945:[-.1,.823,0,0,1.282],8968:[.25,.75,0,0,.44445],8969:[.25,.75,0,0,.44445],8970:[.25,.75,0,0,.44445],8971:[.25,.75,0,0,.44445],8994:[-.14236,.35764,0,0,1],8995:[-.14236,.35764,0,0,1],9136:[.244,.744,0,0,.412],9137:[.244,.745,0,0,.412],9651:[.19444,.69444,0,0,.88889],9657:[-.03472,.46528,0,0,.5],9661:[.19444,.69444,0,0,.88889],9667:[-.03472,.46528,0,0,.5],9711:[.19444,.69444,0,0,1],9824:[.12963,.69444,0,0,.77778],9825:[.12963,.69444,0,0,.77778],9826:[.12963,.69444,0,0,.77778],9827:[.12963,.69444,0,0,.77778],9837:[0,.75,0,0,.38889],9838:[.19444,.69444,0,0,.38889],9839:[.19444,.69444,0,0,.38889],10216:[.25,.75,0,0,.38889],10217:[.25,.75,0,0,.38889],10222:[.244,.744,0,0,.412],10223:[.244,.745,0,0,.412],10229:[.011,.511,0,0,1.609],10230:[.011,.511,0,0,1.638],10231:[.011,.511,0,0,1.859],10232:[.024,.525,0,0,1.609],10233:[.024,.525,0,0,1.638],10234:[.024,.525,0,0,1.858],10236:[.011,.511,0,0,1.638],10815:[0,.68333,0,0,.75],10927:[.13597,.63597,0,0,.77778],10928:[.13597,.63597,0,0,.77778],57376:[.19444,.69444,0,0,0]},"Math-BoldItalic":{32:[0,0,0,0,.25],48:[0,.44444,0,0,.575],49:[0,.44444,0,0,.575],50:[0,.44444,0,0,.575],51:[.19444,.44444,0,0,.575],52:[.19444,.44444,0,0,.575],53:[.19444,.44444,0,0,.575],54:[0,.64444,0,0,.575],55:[.19444,.44444,0,0,.575],56:[0,.64444,0,0,.575],57:[.19444,.44444,0,0,.575],65:[0,.68611,0,0,.86944],66:[0,.68611,.04835,0,.8664],67:[0,.68611,.06979,0,.81694],68:[0,.68611,.03194,0,.93812],69:[0,.68611,.05451,0,.81007],70:[0,.68611,.15972,0,.68889],71:[0,.68611,0,0,.88673],72:[0,.68611,.08229,0,.98229],73:[0,.68611,.07778,0,.51111],74:[0,.68611,.10069,0,.63125],75:[0,.68611,.06979,0,.97118],76:[0,.68611,0,0,.75555],77:[0,.68611,.11424,0,1.14201],78:[0,.68611,.11424,0,.95034],79:[0,.68611,.03194,0,.83666],80:[0,.68611,.15972,0,.72309],81:[.19444,.68611,0,0,.86861],82:[0,.68611,.00421,0,.87235],83:[0,.68611,.05382,0,.69271],84:[0,.68611,.15972,0,.63663],85:[0,.68611,.11424,0,.80027],86:[0,.68611,.25555,0,.67778],87:[0,.68611,.15972,0,1.09305],88:[0,.68611,.07778,0,.94722],89:[0,.68611,.25555,0,.67458],90:[0,.68611,.06979,0,.77257],97:[0,.44444,0,0,.63287],98:[0,.69444,0,0,.52083],99:[0,.44444,0,0,.51342],100:[0,.69444,0,0,.60972],101:[0,.44444,0,0,.55361],102:[.19444,.69444,.11042,0,.56806],103:[.19444,.44444,.03704,0,.5449],104:[0,.69444,0,0,.66759],105:[0,.69326,0,0,.4048],106:[.19444,.69326,.0622,0,.47083],107:[0,.69444,.01852,0,.6037],108:[0,.69444,.0088,0,.34815],109:[0,.44444,0,0,1.0324],110:[0,.44444,0,0,.71296],111:[0,.44444,0,0,.58472],112:[.19444,.44444,0,0,.60092],113:[.19444,.44444,.03704,0,.54213],114:[0,.44444,.03194,0,.5287],115:[0,.44444,0,0,.53125],116:[0,.63492,0,0,.41528],117:[0,.44444,0,0,.68102],118:[0,.44444,.03704,0,.56666],119:[0,.44444,.02778,0,.83148],120:[0,.44444,0,0,.65903],121:[.19444,.44444,.03704,0,.59028],122:[0,.44444,.04213,0,.55509],160:[0,0,0,0,.25],915:[0,.68611,.15972,0,.65694],916:[0,.68611,0,0,.95833],920:[0,.68611,.03194,0,.86722],923:[0,.68611,0,0,.80555],926:[0,.68611,.07458,0,.84125],928:[0,.68611,.08229,0,.98229],931:[0,.68611,.05451,0,.88507],933:[0,.68611,.15972,0,.67083],934:[0,.68611,0,0,.76666],936:[0,.68611,.11653,0,.71402],937:[0,.68611,.04835,0,.8789],945:[0,.44444,0,0,.76064],946:[.19444,.69444,.03403,0,.65972],947:[.19444,.44444,.06389,0,.59003],948:[0,.69444,.03819,0,.52222],949:[0,.44444,0,0,.52882],950:[.19444,.69444,.06215,0,.50833],951:[.19444,.44444,.03704,0,.6],952:[0,.69444,.03194,0,.5618],953:[0,.44444,0,0,.41204],954:[0,.44444,0,0,.66759],955:[0,.69444,0,0,.67083],956:[.19444,.44444,0,0,.70787],957:[0,.44444,.06898,0,.57685],958:[.19444,.69444,.03021,0,.50833],959:[0,.44444,0,0,.58472],960:[0,.44444,.03704,0,.68241],961:[.19444,.44444,0,0,.6118],962:[.09722,.44444,.07917,0,.42361],963:[0,.44444,.03704,0,.68588],964:[0,.44444,.13472,0,.52083],965:[0,.44444,.03704,0,.63055],966:[.19444,.44444,0,0,.74722],967:[.19444,.44444,0,0,.71805],968:[.19444,.69444,.03704,0,.75833],969:[0,.44444,.03704,0,.71782],977:[0,.69444,0,0,.69155],981:[.19444,.69444,0,0,.7125],982:[0,.44444,.03194,0,.975],1009:[.19444,.44444,0,0,.6118],1013:[0,.44444,0,0,.48333],57649:[0,.44444,0,0,.39352],57911:[.19444,.44444,0,0,.43889]},"Math-Italic":{32:[0,0,0,0,.25],48:[0,.43056,0,0,.5],49:[0,.43056,0,0,.5],50:[0,.43056,0,0,.5],51:[.19444,.43056,0,0,.5],52:[.19444,.43056,0,0,.5],53:[.19444,.43056,0,0,.5],54:[0,.64444,0,0,.5],55:[.19444,.43056,0,0,.5],56:[0,.64444,0,0,.5],57:[.19444,.43056,0,0,.5],65:[0,.68333,0,.13889,.75],66:[0,.68333,.05017,.08334,.75851],67:[0,.68333,.07153,.08334,.71472],68:[0,.68333,.02778,.05556,.82792],69:[0,.68333,.05764,.08334,.7382],70:[0,.68333,.13889,.08334,.64306],71:[0,.68333,0,.08334,.78625],72:[0,.68333,.08125,.05556,.83125],73:[0,.68333,.07847,.11111,.43958],74:[0,.68333,.09618,.16667,.55451],75:[0,.68333,.07153,.05556,.84931],76:[0,.68333,0,.02778,.68056],77:[0,.68333,.10903,.08334,.97014],78:[0,.68333,.10903,.08334,.80347],79:[0,.68333,.02778,.08334,.76278],80:[0,.68333,.13889,.08334,.64201],81:[.19444,.68333,0,.08334,.79056],82:[0,.68333,.00773,.08334,.75929],83:[0,.68333,.05764,.08334,.6132],84:[0,.68333,.13889,.08334,.58438],85:[0,.68333,.10903,.02778,.68278],86:[0,.68333,.22222,0,.58333],87:[0,.68333,.13889,0,.94445],88:[0,.68333,.07847,.08334,.82847],89:[0,.68333,.22222,0,.58056],90:[0,.68333,.07153,.08334,.68264],97:[0,.43056,0,0,.52859],98:[0,.69444,0,0,.42917],99:[0,.43056,0,.05556,.43276],100:[0,.69444,0,.16667,.52049],101:[0,.43056,0,.05556,.46563],102:[.19444,.69444,.10764,.16667,.48959],103:[.19444,.43056,.03588,.02778,.47697],104:[0,.69444,0,0,.57616],105:[0,.65952,0,0,.34451],106:[.19444,.65952,.05724,0,.41181],107:[0,.69444,.03148,0,.5206],108:[0,.69444,.01968,.08334,.29838],109:[0,.43056,0,0,.87801],110:[0,.43056,0,0,.60023],111:[0,.43056,0,.05556,.48472],112:[.19444,.43056,0,.08334,.50313],113:[.19444,.43056,.03588,.08334,.44641],114:[0,.43056,.02778,.05556,.45116],115:[0,.43056,0,.05556,.46875],116:[0,.61508,0,.08334,.36111],117:[0,.43056,0,.02778,.57246],118:[0,.43056,.03588,.02778,.48472],119:[0,.43056,.02691,.08334,.71592],120:[0,.43056,0,.02778,.57153],121:[.19444,.43056,.03588,.05556,.49028],122:[0,.43056,.04398,.05556,.46505],160:[0,0,0,0,.25],915:[0,.68333,.13889,.08334,.61528],916:[0,.68333,0,.16667,.83334],920:[0,.68333,.02778,.08334,.76278],923:[0,.68333,0,.16667,.69445],926:[0,.68333,.07569,.08334,.74236],928:[0,.68333,.08125,.05556,.83125],931:[0,.68333,.05764,.08334,.77986],933:[0,.68333,.13889,.05556,.58333],934:[0,.68333,0,.08334,.66667],936:[0,.68333,.11,.05556,.61222],937:[0,.68333,.05017,.08334,.7724],945:[0,.43056,.0037,.02778,.6397],946:[.19444,.69444,.05278,.08334,.56563],947:[.19444,.43056,.05556,0,.51773],948:[0,.69444,.03785,.05556,.44444],949:[0,.43056,0,.08334,.46632],950:[.19444,.69444,.07378,.08334,.4375],951:[.19444,.43056,.03588,.05556,.49653],952:[0,.69444,.02778,.08334,.46944],953:[0,.43056,0,.05556,.35394],954:[0,.43056,0,0,.57616],955:[0,.69444,0,0,.58334],956:[.19444,.43056,0,.02778,.60255],957:[0,.43056,.06366,.02778,.49398],958:[.19444,.69444,.04601,.11111,.4375],959:[0,.43056,0,.05556,.48472],960:[0,.43056,.03588,0,.57003],961:[.19444,.43056,0,.08334,.51702],962:[.09722,.43056,.07986,.08334,.36285],963:[0,.43056,.03588,0,.57141],964:[0,.43056,.1132,.02778,.43715],965:[0,.43056,.03588,.02778,.54028],966:[.19444,.43056,0,.08334,.65417],967:[.19444,.43056,0,.05556,.62569],968:[.19444,.69444,.03588,.11111,.65139],969:[0,.43056,.03588,0,.62245],977:[0,.69444,0,.08334,.59144],981:[.19444,.69444,0,.08334,.59583],982:[0,.43056,.02778,0,.82813],1009:[.19444,.43056,0,.08334,.51702],1013:[0,.43056,0,.05556,.4059],57649:[0,.43056,0,.02778,.32246],57911:[.19444,.43056,0,.08334,.38403]},"SansSerif-Bold":{32:[0,0,0,0,.25],33:[0,.69444,0,0,.36667],34:[0,.69444,0,0,.55834],35:[.19444,.69444,0,0,.91667],36:[.05556,.75,0,0,.55],37:[.05556,.75,0,0,1.02912],38:[0,.69444,0,0,.83056],39:[0,.69444,0,0,.30556],40:[.25,.75,0,0,.42778],41:[.25,.75,0,0,.42778],42:[0,.75,0,0,.55],43:[.11667,.61667,0,0,.85556],44:[.10556,.13056,0,0,.30556],45:[0,.45833,0,0,.36667],46:[0,.13056,0,0,.30556],47:[.25,.75,0,0,.55],48:[0,.69444,0,0,.55],49:[0,.69444,0,0,.55],50:[0,.69444,0,0,.55],51:[0,.69444,0,0,.55],52:[0,.69444,0,0,.55],53:[0,.69444,0,0,.55],54:[0,.69444,0,0,.55],55:[0,.69444,0,0,.55],56:[0,.69444,0,0,.55],57:[0,.69444,0,0,.55],58:[0,.45833,0,0,.30556],59:[.10556,.45833,0,0,.30556],61:[-.09375,.40625,0,0,.85556],63:[0,.69444,0,0,.51945],64:[0,.69444,0,0,.73334],65:[0,.69444,0,0,.73334],66:[0,.69444,0,0,.73334],67:[0,.69444,0,0,.70278],68:[0,.69444,0,0,.79445],69:[0,.69444,0,0,.64167],70:[0,.69444,0,0,.61111],71:[0,.69444,0,0,.73334],72:[0,.69444,0,0,.79445],73:[0,.69444,0,0,.33056],74:[0,.69444,0,0,.51945],75:[0,.69444,0,0,.76389],76:[0,.69444,0,0,.58056],77:[0,.69444,0,0,.97778],78:[0,.69444,0,0,.79445],79:[0,.69444,0,0,.79445],80:[0,.69444,0,0,.70278],81:[.10556,.69444,0,0,.79445],82:[0,.69444,0,0,.70278],83:[0,.69444,0,0,.61111],84:[0,.69444,0,0,.73334],85:[0,.69444,0,0,.76389],86:[0,.69444,.01528,0,.73334],87:[0,.69444,.01528,0,1.03889],88:[0,.69444,0,0,.73334],89:[0,.69444,.0275,0,.73334],90:[0,.69444,0,0,.67223],91:[.25,.75,0,0,.34306],93:[.25,.75,0,0,.34306],94:[0,.69444,0,0,.55],95:[.35,.10833,.03056,0,.55],97:[0,.45833,0,0,.525],98:[0,.69444,0,0,.56111],99:[0,.45833,0,0,.48889],100:[0,.69444,0,0,.56111],101:[0,.45833,0,0,.51111],102:[0,.69444,.07639,0,.33611],103:[.19444,.45833,.01528,0,.55],104:[0,.69444,0,0,.56111],105:[0,.69444,0,0,.25556],106:[.19444,.69444,0,0,.28611],107:[0,.69444,0,0,.53056],108:[0,.69444,0,0,.25556],109:[0,.45833,0,0,.86667],110:[0,.45833,0,0,.56111],111:[0,.45833,0,0,.55],112:[.19444,.45833,0,0,.56111],113:[.19444,.45833,0,0,.56111],114:[0,.45833,.01528,0,.37222],115:[0,.45833,0,0,.42167],116:[0,.58929,0,0,.40417],117:[0,.45833,0,0,.56111],118:[0,.45833,.01528,0,.5],119:[0,.45833,.01528,0,.74445],120:[0,.45833,0,0,.5],121:[.19444,.45833,.01528,0,.5],122:[0,.45833,0,0,.47639],126:[.35,.34444,0,0,.55],160:[0,0,0,0,.25],168:[0,.69444,0,0,.55],176:[0,.69444,0,0,.73334],180:[0,.69444,0,0,.55],184:[.17014,0,0,0,.48889],305:[0,.45833,0,0,.25556],567:[.19444,.45833,0,0,.28611],710:[0,.69444,0,0,.55],711:[0,.63542,0,0,.55],713:[0,.63778,0,0,.55],728:[0,.69444,0,0,.55],729:[0,.69444,0,0,.30556],730:[0,.69444,0,0,.73334],732:[0,.69444,0,0,.55],733:[0,.69444,0,0,.55],915:[0,.69444,0,0,.58056],916:[0,.69444,0,0,.91667],920:[0,.69444,0,0,.85556],923:[0,.69444,0,0,.67223],926:[0,.69444,0,0,.73334],928:[0,.69444,0,0,.79445],931:[0,.69444,0,0,.79445],933:[0,.69444,0,0,.85556],934:[0,.69444,0,0,.79445],936:[0,.69444,0,0,.85556],937:[0,.69444,0,0,.79445],8211:[0,.45833,.03056,0,.55],8212:[0,.45833,.03056,0,1.10001],8216:[0,.69444,0,0,.30556],8217:[0,.69444,0,0,.30556],8220:[0,.69444,0,0,.55834],8221:[0,.69444,0,0,.55834]},"SansSerif-Italic":{32:[0,0,0,0,.25],33:[0,.69444,.05733,0,.31945],34:[0,.69444,.00316,0,.5],35:[.19444,.69444,.05087,0,.83334],36:[.05556,.75,.11156,0,.5],37:[.05556,.75,.03126,0,.83334],38:[0,.69444,.03058,0,.75834],39:[0,.69444,.07816,0,.27778],40:[.25,.75,.13164,0,.38889],41:[.25,.75,.02536,0,.38889],42:[0,.75,.11775,0,.5],43:[.08333,.58333,.02536,0,.77778],44:[.125,.08333,0,0,.27778],45:[0,.44444,.01946,0,.33333],46:[0,.08333,0,0,.27778],47:[.25,.75,.13164,0,.5],48:[0,.65556,.11156,0,.5],49:[0,.65556,.11156,0,.5],50:[0,.65556,.11156,0,.5],51:[0,.65556,.11156,0,.5],52:[0,.65556,.11156,0,.5],53:[0,.65556,.11156,0,.5],54:[0,.65556,.11156,0,.5],55:[0,.65556,.11156,0,.5],56:[0,.65556,.11156,0,.5],57:[0,.65556,.11156,0,.5],58:[0,.44444,.02502,0,.27778],59:[.125,.44444,.02502,0,.27778],61:[-.13,.37,.05087,0,.77778],63:[0,.69444,.11809,0,.47222],64:[0,.69444,.07555,0,.66667],65:[0,.69444,0,0,.66667],66:[0,.69444,.08293,0,.66667],67:[0,.69444,.11983,0,.63889],68:[0,.69444,.07555,0,.72223],69:[0,.69444,.11983,0,.59722],70:[0,.69444,.13372,0,.56945],71:[0,.69444,.11983,0,.66667],72:[0,.69444,.08094,0,.70834],73:[0,.69444,.13372,0,.27778],74:[0,.69444,.08094,0,.47222],75:[0,.69444,.11983,0,.69445],76:[0,.69444,0,0,.54167],77:[0,.69444,.08094,0,.875],78:[0,.69444,.08094,0,.70834],79:[0,.69444,.07555,0,.73611],80:[0,.69444,.08293,0,.63889],81:[.125,.69444,.07555,0,.73611],82:[0,.69444,.08293,0,.64584],83:[0,.69444,.09205,0,.55556],84:[0,.69444,.13372,0,.68056],85:[0,.69444,.08094,0,.6875],86:[0,.69444,.1615,0,.66667],87:[0,.69444,.1615,0,.94445],88:[0,.69444,.13372,0,.66667],89:[0,.69444,.17261,0,.66667],90:[0,.69444,.11983,0,.61111],91:[.25,.75,.15942,0,.28889],93:[.25,.75,.08719,0,.28889],94:[0,.69444,.0799,0,.5],95:[.35,.09444,.08616,0,.5],97:[0,.44444,.00981,0,.48056],98:[0,.69444,.03057,0,.51667],99:[0,.44444,.08336,0,.44445],100:[0,.69444,.09483,0,.51667],101:[0,.44444,.06778,0,.44445],102:[0,.69444,.21705,0,.30556],103:[.19444,.44444,.10836,0,.5],104:[0,.69444,.01778,0,.51667],105:[0,.67937,.09718,0,.23889],106:[.19444,.67937,.09162,0,.26667],107:[0,.69444,.08336,0,.48889],108:[0,.69444,.09483,0,.23889],109:[0,.44444,.01778,0,.79445],110:[0,.44444,.01778,0,.51667],111:[0,.44444,.06613,0,.5],112:[.19444,.44444,.0389,0,.51667],113:[.19444,.44444,.04169,0,.51667],114:[0,.44444,.10836,0,.34167],115:[0,.44444,.0778,0,.38333],116:[0,.57143,.07225,0,.36111],117:[0,.44444,.04169,0,.51667],118:[0,.44444,.10836,0,.46111],119:[0,.44444,.10836,0,.68334],120:[0,.44444,.09169,0,.46111],121:[.19444,.44444,.10836,0,.46111],122:[0,.44444,.08752,0,.43472],126:[.35,.32659,.08826,0,.5],160:[0,0,0,0,.25],168:[0,.67937,.06385,0,.5],176:[0,.69444,0,0,.73752],184:[.17014,0,0,0,.44445],305:[0,.44444,.04169,0,.23889],567:[.19444,.44444,.04169,0,.26667],710:[0,.69444,.0799,0,.5],711:[0,.63194,.08432,0,.5],713:[0,.60889,.08776,0,.5],714:[0,.69444,.09205,0,.5],715:[0,.69444,0,0,.5],728:[0,.69444,.09483,0,.5],729:[0,.67937,.07774,0,.27778],730:[0,.69444,0,0,.73752],732:[0,.67659,.08826,0,.5],733:[0,.69444,.09205,0,.5],915:[0,.69444,.13372,0,.54167],916:[0,.69444,0,0,.83334],920:[0,.69444,.07555,0,.77778],923:[0,.69444,0,0,.61111],926:[0,.69444,.12816,0,.66667],928:[0,.69444,.08094,0,.70834],931:[0,.69444,.11983,0,.72222],933:[0,.69444,.09031,0,.77778],934:[0,.69444,.04603,0,.72222],936:[0,.69444,.09031,0,.77778],937:[0,.69444,.08293,0,.72222],8211:[0,.44444,.08616,0,.5],8212:[0,.44444,.08616,0,1],8216:[0,.69444,.07816,0,.27778],8217:[0,.69444,.07816,0,.27778],8220:[0,.69444,.14205,0,.5],8221:[0,.69444,.00316,0,.5]},"SansSerif-Regular":{32:[0,0,0,0,.25],33:[0,.69444,0,0,.31945],34:[0,.69444,0,0,.5],35:[.19444,.69444,0,0,.83334],36:[.05556,.75,0,0,.5],37:[.05556,.75,0,0,.83334],38:[0,.69444,0,0,.75834],39:[0,.69444,0,0,.27778],40:[.25,.75,0,0,.38889],41:[.25,.75,0,0,.38889],42:[0,.75,0,0,.5],43:[.08333,.58333,0,0,.77778],44:[.125,.08333,0,0,.27778],45:[0,.44444,0,0,.33333],46:[0,.08333,0,0,.27778],47:[.25,.75,0,0,.5],48:[0,.65556,0,0,.5],49:[0,.65556,0,0,.5],50:[0,.65556,0,0,.5],51:[0,.65556,0,0,.5],52:[0,.65556,0,0,.5],53:[0,.65556,0,0,.5],54:[0,.65556,0,0,.5],55:[0,.65556,0,0,.5],56:[0,.65556,0,0,.5],57:[0,.65556,0,0,.5],58:[0,.44444,0,0,.27778],59:[.125,.44444,0,0,.27778],61:[-.13,.37,0,0,.77778],63:[0,.69444,0,0,.47222],64:[0,.69444,0,0,.66667],65:[0,.69444,0,0,.66667],66:[0,.69444,0,0,.66667],67:[0,.69444,0,0,.63889],68:[0,.69444,0,0,.72223],69:[0,.69444,0,0,.59722],70:[0,.69444,0,0,.56945],71:[0,.69444,0,0,.66667],72:[0,.69444,0,0,.70834],73:[0,.69444,0,0,.27778],74:[0,.69444,0,0,.47222],75:[0,.69444,0,0,.69445],76:[0,.69444,0,0,.54167],77:[0,.69444,0,0,.875],78:[0,.69444,0,0,.70834],79:[0,.69444,0,0,.73611],80:[0,.69444,0,0,.63889],81:[.125,.69444,0,0,.73611],82:[0,.69444,0,0,.64584],83:[0,.69444,0,0,.55556],84:[0,.69444,0,0,.68056],85:[0,.69444,0,0,.6875],86:[0,.69444,.01389,0,.66667],87:[0,.69444,.01389,0,.94445],88:[0,.69444,0,0,.66667],89:[0,.69444,.025,0,.66667],90:[0,.69444,0,0,.61111],91:[.25,.75,0,0,.28889],93:[.25,.75,0,0,.28889],94:[0,.69444,0,0,.5],95:[.35,.09444,.02778,0,.5],97:[0,.44444,0,0,.48056],98:[0,.69444,0,0,.51667],99:[0,.44444,0,0,.44445],100:[0,.69444,0,0,.51667],101:[0,.44444,0,0,.44445],102:[0,.69444,.06944,0,.30556],103:[.19444,.44444,.01389,0,.5],104:[0,.69444,0,0,.51667],105:[0,.67937,0,0,.23889],106:[.19444,.67937,0,0,.26667],107:[0,.69444,0,0,.48889],108:[0,.69444,0,0,.23889],109:[0,.44444,0,0,.79445],110:[0,.44444,0,0,.51667],111:[0,.44444,0,0,.5],112:[.19444,.44444,0,0,.51667],113:[.19444,.44444,0,0,.51667],114:[0,.44444,.01389,0,.34167],115:[0,.44444,0,0,.38333],116:[0,.57143,0,0,.36111],117:[0,.44444,0,0,.51667],118:[0,.44444,.01389,0,.46111],119:[0,.44444,.01389,0,.68334],120:[0,.44444,0,0,.46111],121:[.19444,.44444,.01389,0,.46111],122:[0,.44444,0,0,.43472],126:[.35,.32659,0,0,.5],160:[0,0,0,0,.25],168:[0,.67937,0,0,.5],176:[0,.69444,0,0,.66667],184:[.17014,0,0,0,.44445],305:[0,.44444,0,0,.23889],567:[.19444,.44444,0,0,.26667],710:[0,.69444,0,0,.5],711:[0,.63194,0,0,.5],713:[0,.60889,0,0,.5],714:[0,.69444,0,0,.5],715:[0,.69444,0,0,.5],728:[0,.69444,0,0,.5],729:[0,.67937,0,0,.27778],730:[0,.69444,0,0,.66667],732:[0,.67659,0,0,.5],733:[0,.69444,0,0,.5],915:[0,.69444,0,0,.54167],916:[0,.69444,0,0,.83334],920:[0,.69444,0,0,.77778],923:[0,.69444,0,0,.61111],926:[0,.69444,0,0,.66667],928:[0,.69444,0,0,.70834],931:[0,.69444,0,0,.72222],933:[0,.69444,0,0,.77778],934:[0,.69444,0,0,.72222],936:[0,.69444,0,0,.77778],937:[0,.69444,0,0,.72222],8211:[0,.44444,.02778,0,.5],8212:[0,.44444,.02778,0,1],8216:[0,.69444,0,0,.27778],8217:[0,.69444,0,0,.27778],8220:[0,.69444,0,0,.5],8221:[0,.69444,0,0,.5]},"Script-Regular":{32:[0,0,0,0,.25],65:[0,.7,.22925,0,.80253],66:[0,.7,.04087,0,.90757],67:[0,.7,.1689,0,.66619],68:[0,.7,.09371,0,.77443],69:[0,.7,.18583,0,.56162],70:[0,.7,.13634,0,.89544],71:[0,.7,.17322,0,.60961],72:[0,.7,.29694,0,.96919],73:[0,.7,.19189,0,.80907],74:[.27778,.7,.19189,0,1.05159],75:[0,.7,.31259,0,.91364],76:[0,.7,.19189,0,.87373],77:[0,.7,.15981,0,1.08031],78:[0,.7,.3525,0,.9015],79:[0,.7,.08078,0,.73787],80:[0,.7,.08078,0,1.01262],81:[0,.7,.03305,0,.88282],82:[0,.7,.06259,0,.85],83:[0,.7,.19189,0,.86767],84:[0,.7,.29087,0,.74697],85:[0,.7,.25815,0,.79996],86:[0,.7,.27523,0,.62204],87:[0,.7,.27523,0,.80532],88:[0,.7,.26006,0,.94445],89:[0,.7,.2939,0,.70961],90:[0,.7,.24037,0,.8212],160:[0,0,0,0,.25]},"Size1-Regular":{32:[0,0,0,0,.25],40:[.35001,.85,0,0,.45834],41:[.35001,.85,0,0,.45834],47:[.35001,.85,0,0,.57778],91:[.35001,.85,0,0,.41667],92:[.35001,.85,0,0,.57778],93:[.35001,.85,0,0,.41667],123:[.35001,.85,0,0,.58334],125:[.35001,.85,0,0,.58334],160:[0,0,0,0,.25],710:[0,.72222,0,0,.55556],732:[0,.72222,0,0,.55556],770:[0,.72222,0,0,.55556],771:[0,.72222,0,0,.55556],8214:[-99e-5,.601,0,0,.77778],8593:[1e-5,.6,0,0,.66667],8595:[1e-5,.6,0,0,.66667],8657:[1e-5,.6,0,0,.77778],8659:[1e-5,.6,0,0,.77778],8719:[.25001,.75,0,0,.94445],8720:[.25001,.75,0,0,.94445],8721:[.25001,.75,0,0,1.05556],8730:[.35001,.85,0,0,1],8739:[-.00599,.606,0,0,.33333],8741:[-.00599,.606,0,0,.55556],8747:[.30612,.805,.19445,0,.47222],8748:[.306,.805,.19445,0,.47222],8749:[.306,.805,.19445,0,.47222],8750:[.30612,.805,.19445,0,.47222],8896:[.25001,.75,0,0,.83334],8897:[.25001,.75,0,0,.83334],8898:[.25001,.75,0,0,.83334],8899:[.25001,.75,0,0,.83334],8968:[.35001,.85,0,0,.47222],8969:[.35001,.85,0,0,.47222],8970:[.35001,.85,0,0,.47222],8971:[.35001,.85,0,0,.47222],9168:[-99e-5,.601,0,0,.66667],10216:[.35001,.85,0,0,.47222],10217:[.35001,.85,0,0,.47222],10752:[.25001,.75,0,0,1.11111],10753:[.25001,.75,0,0,1.11111],10754:[.25001,.75,0,0,1.11111],10756:[.25001,.75,0,0,.83334],10758:[.25001,.75,0,0,.83334]},"Size2-Regular":{32:[0,0,0,0,.25],40:[.65002,1.15,0,0,.59722],41:[.65002,1.15,0,0,.59722],47:[.65002,1.15,0,0,.81111],91:[.65002,1.15,0,0,.47222],92:[.65002,1.15,0,0,.81111],93:[.65002,1.15,0,0,.47222],123:[.65002,1.15,0,0,.66667],125:[.65002,1.15,0,0,.66667],160:[0,0,0,0,.25],710:[0,.75,0,0,1],732:[0,.75,0,0,1],770:[0,.75,0,0,1],771:[0,.75,0,0,1],8719:[.55001,1.05,0,0,1.27778],8720:[.55001,1.05,0,0,1.27778],8721:[.55001,1.05,0,0,1.44445],8730:[.65002,1.15,0,0,1],8747:[.86225,1.36,.44445,0,.55556],8748:[.862,1.36,.44445,0,.55556],8749:[.862,1.36,.44445,0,.55556],8750:[.86225,1.36,.44445,0,.55556],8896:[.55001,1.05,0,0,1.11111],8897:[.55001,1.05,0,0,1.11111],8898:[.55001,1.05,0,0,1.11111],8899:[.55001,1.05,0,0,1.11111],8968:[.65002,1.15,0,0,.52778],8969:[.65002,1.15,0,0,.52778],8970:[.65002,1.15,0,0,.52778],8971:[.65002,1.15,0,0,.52778],10216:[.65002,1.15,0,0,.61111],10217:[.65002,1.15,0,0,.61111],10752:[.55001,1.05,0,0,1.51112],10753:[.55001,1.05,0,0,1.51112],10754:[.55001,1.05,0,0,1.51112],10756:[.55001,1.05,0,0,1.11111],10758:[.55001,1.05,0,0,1.11111]},"Size3-Regular":{32:[0,0,0,0,.25],40:[.95003,1.45,0,0,.73611],41:[.95003,1.45,0,0,.73611],47:[.95003,1.45,0,0,1.04445],91:[.95003,1.45,0,0,.52778],92:[.95003,1.45,0,0,1.04445],93:[.95003,1.45,0,0,.52778],123:[.95003,1.45,0,0,.75],125:[.95003,1.45,0,0,.75],160:[0,0,0,0,.25],710:[0,.75,0,0,1.44445],732:[0,.75,0,0,1.44445],770:[0,.75,0,0,1.44445],771:[0,.75,0,0,1.44445],8730:[.95003,1.45,0,0,1],8968:[.95003,1.45,0,0,.58334],8969:[.95003,1.45,0,0,.58334],8970:[.95003,1.45,0,0,.58334],8971:[.95003,1.45,0,0,.58334],10216:[.95003,1.45,0,0,.75],10217:[.95003,1.45,0,0,.75]},"Size4-Regular":{32:[0,0,0,0,.25],40:[1.25003,1.75,0,0,.79167],41:[1.25003,1.75,0,0,.79167],47:[1.25003,1.75,0,0,1.27778],91:[1.25003,1.75,0,0,.58334],92:[1.25003,1.75,0,0,1.27778],93:[1.25003,1.75,0,0,.58334],123:[1.25003,1.75,0,0,.80556],125:[1.25003,1.75,0,0,.80556],160:[0,0,0,0,.25],710:[0,.825,0,0,1.8889],732:[0,.825,0,0,1.8889],770:[0,.825,0,0,1.8889],771:[0,.825,0,0,1.8889],8730:[1.25003,1.75,0,0,1],8968:[1.25003,1.75,0,0,.63889],8969:[1.25003,1.75,0,0,.63889],8970:[1.25003,1.75,0,0,.63889],8971:[1.25003,1.75,0,0,.63889],9115:[.64502,1.155,0,0,.875],9116:[1e-5,.6,0,0,.875],9117:[.64502,1.155,0,0,.875],9118:[.64502,1.155,0,0,.875],9119:[1e-5,.6,0,0,.875],9120:[.64502,1.155,0,0,.875],9121:[.64502,1.155,0,0,.66667],9122:[-99e-5,.601,0,0,.66667],9123:[.64502,1.155,0,0,.66667],9124:[.64502,1.155,0,0,.66667],9125:[-99e-5,.601,0,0,.66667],9126:[.64502,1.155,0,0,.66667],9127:[1e-5,.9,0,0,.88889],9128:[.65002,1.15,0,0,.88889],9129:[.90001,0,0,0,.88889],9130:[0,.3,0,0,.88889],9131:[1e-5,.9,0,0,.88889],9132:[.65002,1.15,0,0,.88889],9133:[.90001,0,0,0,.88889],9143:[.88502,.915,0,0,1.05556],10216:[1.25003,1.75,0,0,.80556],10217:[1.25003,1.75,0,0,.80556],57344:[-.00499,.605,0,0,1.05556],57345:[-.00499,.605,0,0,1.05556],57680:[0,.12,0,0,.45],57681:[0,.12,0,0,.45],57682:[0,.12,0,0,.45],57683:[0,.12,0,0,.45]},"Typewriter-Regular":{32:[0,0,0,0,.525],33:[0,.61111,0,0,.525],34:[0,.61111,0,0,.525],35:[0,.61111,0,0,.525],36:[.08333,.69444,0,0,.525],37:[.08333,.69444,0,0,.525],38:[0,.61111,0,0,.525],39:[0,.61111,0,0,.525],40:[.08333,.69444,0,0,.525],41:[.08333,.69444,0,0,.525],42:[0,.52083,0,0,.525],43:[-.08056,.53055,0,0,.525],44:[.13889,.125,0,0,.525],45:[-.08056,.53055,0,0,.525],46:[0,.125,0,0,.525],47:[.08333,.69444,0,0,.525],48:[0,.61111,0,0,.525],49:[0,.61111,0,0,.525],50:[0,.61111,0,0,.525],51:[0,.61111,0,0,.525],52:[0,.61111,0,0,.525],53:[0,.61111,0,0,.525],54:[0,.61111,0,0,.525],55:[0,.61111,0,0,.525],56:[0,.61111,0,0,.525],57:[0,.61111,0,0,.525],58:[0,.43056,0,0,.525],59:[.13889,.43056,0,0,.525],60:[-.05556,.55556,0,0,.525],61:[-.19549,.41562,0,0,.525],62:[-.05556,.55556,0,0,.525],63:[0,.61111,0,0,.525],64:[0,.61111,0,0,.525],65:[0,.61111,0,0,.525],66:[0,.61111,0,0,.525],67:[0,.61111,0,0,.525],68:[0,.61111,0,0,.525],69:[0,.61111,0,0,.525],70:[0,.61111,0,0,.525],71:[0,.61111,0,0,.525],72:[0,.61111,0,0,.525],73:[0,.61111,0,0,.525],74:[0,.61111,0,0,.525],75:[0,.61111,0,0,.525],76:[0,.61111,0,0,.525],77:[0,.61111,0,0,.525],78:[0,.61111,0,0,.525],79:[0,.61111,0,0,.525],80:[0,.61111,0,0,.525],81:[.13889,.61111,0,0,.525],82:[0,.61111,0,0,.525],83:[0,.61111,0,0,.525],84:[0,.61111,0,0,.525],85:[0,.61111,0,0,.525],86:[0,.61111,0,0,.525],87:[0,.61111,0,0,.525],88:[0,.61111,0,0,.525],89:[0,.61111,0,0,.525],90:[0,.61111,0,0,.525],91:[.08333,.69444,0,0,.525],92:[.08333,.69444,0,0,.525],93:[.08333,.69444,0,0,.525],94:[0,.61111,0,0,.525],95:[.09514,0,0,0,.525],96:[0,.61111,0,0,.525],97:[0,.43056,0,0,.525],98:[0,.61111,0,0,.525],99:[0,.43056,0,0,.525],100:[0,.61111,0,0,.525],101:[0,.43056,0,0,.525],102:[0,.61111,0,0,.525],103:[.22222,.43056,0,0,.525],104:[0,.61111,0,0,.525],105:[0,.61111,0,0,.525],106:[.22222,.61111,0,0,.525],107:[0,.61111,0,0,.525],108:[0,.61111,0,0,.525],109:[0,.43056,0,0,.525],110:[0,.43056,0,0,.525],111:[0,.43056,0,0,.525],112:[.22222,.43056,0,0,.525],113:[.22222,.43056,0,0,.525],114:[0,.43056,0,0,.525],115:[0,.43056,0,0,.525],116:[0,.55358,0,0,.525],117:[0,.43056,0,0,.525],118:[0,.43056,0,0,.525],119:[0,.43056,0,0,.525],120:[0,.43056,0,0,.525],121:[.22222,.43056,0,0,.525],122:[0,.43056,0,0,.525],123:[.08333,.69444,0,0,.525],124:[.08333,.69444,0,0,.525],125:[.08333,.69444,0,0,.525],126:[0,.61111,0,0,.525],127:[0,.61111,0,0,.525],160:[0,0,0,0,.525],176:[0,.61111,0,0,.525],184:[.19445,0,0,0,.525],305:[0,.43056,0,0,.525],567:[.22222,.43056,0,0,.525],711:[0,.56597,0,0,.525],713:[0,.56555,0,0,.525],714:[0,.61111,0,0,.525],715:[0,.61111,0,0,.525],728:[0,.61111,0,0,.525],730:[0,.61111,0,0,.525],770:[0,.61111,0,0,.525],771:[0,.61111,0,0,.525],776:[0,.61111,0,0,.525],915:[0,.61111,0,0,.525],916:[0,.61111,0,0,.525],920:[0,.61111,0,0,.525],923:[0,.61111,0,0,.525],926:[0,.61111,0,0,.525],928:[0,.61111,0,0,.525],931:[0,.61111,0,0,.525],933:[0,.61111,0,0,.525],934:[0,.61111,0,0,.525],936:[0,.61111,0,0,.525],937:[0,.61111,0,0,.525],8216:[0,.61111,0,0,.525],8217:[0,.61111,0,0,.525],8242:[0,.61111,0,0,.525],9251:[.11111,.21944,0,0,.525]}},bh={slant:[.25,.25,.25],space:[0,0,0],stretch:[0,0,0],shrink:[0,0,0],xHeight:[.431,.431,.431],quad:[1,1.171,1.472],extraSpace:[0,0,0],num1:[.677,.732,.925],num2:[.394,.384,.387],num3:[.444,.471,.504],denom1:[.686,.752,1.025],denom2:[.345,.344,.532],sup1:[.413,.503,.504],sup2:[.363,.431,.404],sup3:[.289,.286,.294],sub1:[.15,.143,.2],sub2:[.247,.286,.4],supDrop:[.386,.353,.494],subDrop:[.05,.071,.1],delim1:[2.39,1.7,1.98],delim2:[1.01,1.157,1.42],axisHeight:[.25,.25,.25],defaultRuleThickness:[.04,.049,.049],bigOpSpacing1:[.111,.111,.111],bigOpSpacing2:[.166,.166,.166],bigOpSpacing3:[.2,.2,.2],bigOpSpacing4:[.6,.611,.611],bigOpSpacing5:[.1,.143,.143],sqrtRuleThickness:[.04,.04,.04],ptPerEm:[10,10,10],doubleRuleSep:[.2,.2,.2],arrayRuleWidth:[.04,.04,.04],fboxsep:[.3,.3,.3],fboxrule:[.04,.04,.04]},e6={Å:"A",Ð:"D",Þ:"o",å:"a",ð:"d",þ:"o",А:"A",Б:"B",В:"B",Г:"F",Д:"A",Е:"E",Ж:"K",З:"3",И:"N",Й:"N",К:"K",Л:"N",М:"M",Н:"H",О:"O",П:"N",Р:"P",С:"C",Т:"T",У:"y",Ф:"O",Х:"X",Ц:"U",Ч:"h",Ш:"W",Щ:"W",Ъ:"B",Ы:"X",Ь:"B",Э:"3",Ю:"X",Я:"R",а:"a",б:"b",в:"a",г:"r",д:"y",е:"e",ж:"m",з:"e",и:"n",й:"n",к:"n",л:"n",м:"m",н:"n",о:"o",п:"n",р:"p",с:"c",т:"o",у:"y",ф:"b",х:"x",ц:"n",ч:"n",ш:"w",щ:"w",ъ:"a",ы:"m",ь:"a",э:"e",ю:"m",я:"r"};function aj(e,t){hi[e]=t}function F2(e,t,n){if(!hi[t])throw new Error("Font metrics not found for font: "+t+".");var r=e.charCodeAt(0),a=hi[t][r];if(!a&&e[0]in e6&&(r=e6[e[0]].charCodeAt(0),a=hi[t][r]),!a&&n==="text"&&px(r)&&(a=hi[t][77]),a)return{depth:a[0],height:a[1],italic:a[2],skew:a[3],width:a[4]}}var ng={};function ij(e){var t;if(e>=5?t=0:e>=3?t=1:t=2,!ng[t]){var n=ng[t]={cssEmPerMu:bh.quad[t]/18};for(var r in bh)bh.hasOwnProperty(r)&&(n[r]=bh[r][t])}return ng[t]}var sj=[[1,1,1],[2,1,1],[3,1,1],[4,2,1],[5,2,1],[6,3,1],[7,4,2],[8,6,3],[9,7,6],[10,8,7],[11,10,9]],t6=[.5,.6,.7,.8,.9,1,1.2,1.44,1.728,2.074,2.488],n6=function(t,n){return n.size<2?t:sj[t-1][n.size-1]};class Ki{constructor(t){this.style=void 0,this.color=void 0,this.size=void 0,this.textSize=void 0,this.phantom=void 0,this.font=void 0,this.fontFamily=void 0,this.fontWeight=void 0,this.fontShape=void 0,this.sizeMultiplier=void 0,this.maxSize=void 0,this.minRuleThickness=void 0,this._fontMetrics=void 0,this.style=t.style,this.color=t.color,this.size=t.size||Ki.BASESIZE,this.textSize=t.textSize||this.size,this.phantom=!!t.phantom,this.font=t.font||"",this.fontFamily=t.fontFamily||"",this.fontWeight=t.fontWeight||"",this.fontShape=t.fontShape||"",this.sizeMultiplier=t6[this.size-1],this.maxSize=t.maxSize,this.minRuleThickness=t.minRuleThickness,this._fontMetrics=void 0}extend(t){var n={style:this.style,size:this.size,textSize:this.textSize,color:this.color,phantom:this.phantom,font:this.font,fontFamily:this.fontFamily,fontWeight:this.fontWeight,fontShape:this.fontShape,maxSize:this.maxSize,minRuleThickness:this.minRuleThickness};for(var r in t)t.hasOwnProperty(r)&&(n[r]=t[r]);return new Ki(n)}havingStyle(t){return this.style===t?this:this.extend({style:t,size:n6(this.textSize,t)})}havingCrampedStyle(){return this.havingStyle(this.style.cramp())}havingSize(t){return this.size===t&&this.textSize===t?this:this.extend({style:this.style.text(),size:t,textSize:t,sizeMultiplier:t6[t-1]})}havingBaseStyle(t){t=t||this.style.text();var n=n6(Ki.BASESIZE,t);return this.size===n&&this.textSize===Ki.BASESIZE&&this.style===t?this:this.extend({style:t,size:n})}havingBaseSizing(){var t;switch(this.style.id){case 4:case 5:t=3;break;case 6:case 7:t=1;break;default:t=6}return this.extend({style:this.style.text(),size:t})}withColor(t){return this.extend({color:t})}withPhantom(){return this.extend({phantom:!0})}withFont(t){return this.extend({font:t})}withTextFontFamily(t){return this.extend({fontFamily:t,font:""})}withTextFontWeight(t){return this.extend({fontWeight:t,font:""})}withTextFontShape(t){return this.extend({fontShape:t,font:""})}sizingClasses(t){return t.size!==this.size?["sizing","reset-size"+t.size,"size"+this.size]:[]}baseSizingClasses(){return this.size!==Ki.BASESIZE?["sizing","reset-size"+this.size,"size"+Ki.BASESIZE]:[]}fontMetrics(){return this._fontMetrics||(this._fontMetrics=ij(this.size)),this._fontMetrics}getColor(){return this.phantom?"transparent":this.color}}Ki.BASESIZE=6;var bb={pt:1,mm:7227/2540,cm:7227/254,in:72.27,bp:803/800,pc:12,dd:1238/1157,cc:14856/1157,nd:685/642,nc:1370/107,sp:1/65536,px:803/800},lj={ex:!0,em:!0,mu:!0},gx=function(t){return typeof t!="string"&&(t=t.unit),t in bb||t in lj||t==="ex"},Rn=function(t,n){var r;if(t.unit in bb)r=bb[t.unit]/n.fontMetrics().ptPerEm/n.sizeMultiplier;else if(t.unit==="mu")r=n.fontMetrics().cssEmPerMu;else{var a;if(n.style.isTight()?a=n.havingStyle(n.style.text()):a=n,t.unit==="ex")r=a.fontMetrics().xHeight;else if(t.unit==="em")r=a.fontMetrics().quad;else throw new Ne("Invalid unit: '"+t.unit+"'");a!==n&&(r*=a.sizeMultiplier/n.sizeMultiplier)}return Math.min(t.number*r,n.maxSize)},Le=function(t){return+t.toFixed(4)+"em"},il=function(t){return t.filter(n=>n).join(" ")},bx=function(t,n,r){if(this.classes=t||[],this.attributes={},this.height=0,this.depth=0,this.maxFontSize=0,this.style=r||{},n){n.style.isTight()&&this.classes.push("mtight");var a=n.getColor();a&&(this.style.color=a)}},vx=function(t){var n=document.createElement(t);n.className=il(this.classes);for(var r in this.style)this.style.hasOwnProperty(r)&&(n.style[r]=this.style[r]);for(var a in this.attributes)this.attributes.hasOwnProperty(a)&&n.setAttribute(a,this.attributes[a]);for(var s=0;s<this.children.length;s++)n.appendChild(this.children[s].toNode());return n},oj=/[\s"'>/=\x00-\x1f]/,yx=function(t){var n="<"+t;this.classes.length&&(n+=' class="'+it.escape(il(this.classes))+'"');var r="";for(var a in this.style)this.style.hasOwnProperty(a)&&(r+=it.hyphenate(a)+":"+this.style[a]+";");r&&(n+=' style="'+it.escape(r)+'"');for(var s in this.attributes)if(this.attributes.hasOwnProperty(s)){if(oj.test(s))throw new Ne("Invalid attribute name '"+s+"'");n+=" "+s+'="'+it.escape(this.attributes[s])+'"'}n+=">";for(var o=0;o<this.children.length;o++)n+=this.children[o].toMarkup();return n+="</"+t+">",n};class B0{constructor(t,n,r,a){this.children=void 0,this.attributes=void 0,this.classes=void 0,this.height=void 0,this.depth=void 0,this.width=void 0,this.maxFontSize=void 0,this.style=void 0,bx.call(this,t,r,a),this.children=n||[]}setAttribute(t,n){this.attributes[t]=n}hasClass(t){return it.contains(this.classes,t)}toNode(){return vx.call(this,"span")}toMarkup(){return yx.call(this,"span")}}class z2{constructor(t,n,r,a){this.children=void 0,this.attributes=void 0,this.classes=void 0,this.height=void 0,this.depth=void 0,this.maxFontSize=void 0,this.style=void 0,bx.call(this,n,a),this.children=r||[],this.setAttribute("href",t)}setAttribute(t,n){this.attributes[t]=n}hasClass(t){return it.contains(this.classes,t)}toNode(){return vx.call(this,"a")}toMarkup(){return yx.call(this,"a")}}class uj{constructor(t,n,r){this.src=void 0,this.alt=void 0,this.classes=void 0,this.height=void 0,this.depth=void 0,this.maxFontSize=void 0,this.style=void 0,this.alt=n,this.src=t,this.classes=["mord"],this.style=r}hasClass(t){return it.contains(this.classes,t)}toNode(){var t=document.createElement("img");t.src=this.src,t.alt=this.alt,t.className="mord";for(var n in this.style)this.style.hasOwnProperty(n)&&(t.style[n]=this.style[n]);return t}toMarkup(){var t='<img src="'+it.escape(this.src)+'"'+(' alt="'+it.escape(this.alt)+'"'),n="";for(var r in this.style)this.style.hasOwnProperty(r)&&(n+=it.hyphenate(r)+":"+this.style[r]+";");return n&&(t+=' style="'+it.escape(n)+'"'),t+="'/>",t}}var cj={î:"ı̂",ï:"ı̈",í:"ı́",ì:"ı̀"};class Ua{constructor(t,n,r,a,s,o,c,d){this.text=void 0,this.height=void 0,this.depth=void 0,this.italic=void 0,this.skew=void 0,this.width=void 0,this.maxFontSize=void 0,this.classes=void 0,this.style=void 0,this.text=t,this.height=n||0,this.depth=r||0,this.italic=a||0,this.skew=s||0,this.width=o||0,this.classes=c||[],this.style=d||{},this.maxFontSize=0;var f=Vq(this.text.charCodeAt(0));f&&this.classes.push(f+"_fallback"),/[îïíì]/.test(this.text)&&(this.text=cj[this.text])}hasClass(t){return it.contains(this.classes,t)}toNode(){var t=document.createTextNode(this.text),n=null;this.italic>0&&(n=document.createElement("span"),n.style.marginRight=Le(this.italic)),this.classes.length>0&&(n=n||document.createElement("span"),n.className=il(this.classes));for(var r in this.style)this.style.hasOwnProperty(r)&&(n=n||document.createElement("span"),n.style[r]=this.style[r]);return n?(n.appendChild(t),n):t}toMarkup(){var t=!1,n="<span";this.classes.length&&(t=!0,n+=' class="',n+=it.escape(il(this.classes)),n+='"');var r="";this.italic>0&&(r+="margin-right:"+this.italic+"em;");for(var a in this.style)this.style.hasOwnProperty(a)&&(r+=it.hyphenate(a)+":"+this.style[a]+";");r&&(t=!0,n+=' style="'+it.escape(r)+'"');var s=it.escape(this.text);return t?(n+=">",n+=s,n+="</span>",n):s}}class as{constructor(t,n){this.children=void 0,this.attributes=void 0,this.children=t||[],this.attributes=n||{}}toNode(){var t="http://www.w3.org/2000/svg",n=document.createElementNS(t,"svg");for(var r in this.attributes)Object.prototype.hasOwnProperty.call(this.attributes,r)&&n.setAttribute(r,this.attributes[r]);for(var a=0;a<this.children.length;a++)n.appendChild(this.children[a].toNode());return n}toMarkup(){var t='<svg xmlns="http://www.w3.org/2000/svg"';for(var n in this.attributes)Object.prototype.hasOwnProperty.call(this.attributes,n)&&(t+=" "+n+'="'+it.escape(this.attributes[n])+'"');t+=">";for(var r=0;r<this.children.length;r++)t+=this.children[r].toMarkup();return t+="</svg>",t}}class sl{constructor(t,n){this.pathName=void 0,this.alternate=void 0,this.pathName=t,this.alternate=n}toNode(){var t="http://www.w3.org/2000/svg",n=document.createElementNS(t,"path");return this.alternate?n.setAttribute("d",this.alternate):n.setAttribute("d",JE[this.pathName]),n}toMarkup(){return this.alternate?'<path d="'+it.escape(this.alternate)+'"/>':'<path d="'+it.escape(JE[this.pathName])+'"/>'}}class vb{constructor(t){this.attributes=void 0,this.attributes=t||{}}toNode(){var t="http://www.w3.org/2000/svg",n=document.createElementNS(t,"line");for(var r in this.attributes)Object.prototype.hasOwnProperty.call(this.attributes,r)&&n.setAttribute(r,this.attributes[r]);return n}toMarkup(){var t="<line";for(var n in this.attributes)Object.prototype.hasOwnProperty.call(this.attributes,n)&&(t+=" "+n+'="'+it.escape(this.attributes[n])+'"');return t+="/>",t}}function r6(e){if(e instanceof Ua)return e;throw new Error("Expected symbolNode but got "+String(e)+".")}function dj(e){if(e instanceof B0)return e;throw new Error("Expected span<HtmlDomNode> but got "+String(e)+".")}var hj={bin:1,close:1,inner:1,open:1,punct:1,rel:1},fj={"accent-token":1,mathord:1,"op-token":1,spacing:1,textord:1},fn={math:{},text:{}};function E(e,t,n,r,a,s){fn[e][a]={font:t,group:n,replace:r},s&&r&&(fn[e][r]=fn[e][a])}var S="math",we="text",N="main",F="ams",Sn="accent-token",Ke="bin",jr="close",vu="inner",st="mathord",Qn="op-token",Ea="open",Jf="punct",q="rel",cs="spacing",X="textord";E(S,N,q,"≡","\\equiv",!0);E(S,N,q,"≺","\\prec",!0);E(S,N,q,"≻","\\succ",!0);E(S,N,q,"∼","\\sim",!0);E(S,N,q,"⊥","\\perp");E(S,N,q,"⪯","\\preceq",!0);E(S,N,q,"⪰","\\succeq",!0);E(S,N,q,"≃","\\simeq",!0);E(S,N,q,"∣","\\mid",!0);E(S,N,q,"≪","\\ll",!0);E(S,N,q,"≫","\\gg",!0);E(S,N,q,"≍","\\asymp",!0);E(S,N,q,"∥","\\parallel");E(S,N,q,"⋈","\\bowtie",!0);E(S,N,q,"⌣","\\smile",!0);E(S,N,q,"⊑","\\sqsubseteq",!0);E(S,N,q,"⊒","\\sqsupseteq",!0);E(S,N,q,"≐","\\doteq",!0);E(S,N,q,"⌢","\\frown",!0);E(S,N,q,"∋","\\ni",!0);E(S,N,q,"∝","\\propto",!0);E(S,N,q,"⊢","\\vdash",!0);E(S,N,q,"⊣","\\dashv",!0);E(S,N,q,"∋","\\owns");E(S,N,Jf,".","\\ldotp");E(S,N,Jf,"⋅","\\cdotp");E(S,N,X,"#","\\#");E(we,N,X,"#","\\#");E(S,N,X,"&","\\&");E(we,N,X,"&","\\&");E(S,N,X,"ℵ","\\aleph",!0);E(S,N,X,"∀","\\forall",!0);E(S,N,X,"ℏ","\\hbar",!0);E(S,N,X,"∃","\\exists",!0);E(S,N,X,"∇","\\nabla",!0);E(S,N,X,"♭","\\flat",!0);E(S,N,X,"ℓ","\\ell",!0);E(S,N,X,"♮","\\natural",!0);E(S,N,X,"♣","\\clubsuit",!0);E(S,N,X,"℘","\\wp",!0);E(S,N,X,"♯","\\sharp",!0);E(S,N,X,"♢","\\diamondsuit",!0);E(S,N,X,"ℜ","\\Re",!0);E(S,N,X,"♡","\\heartsuit",!0);E(S,N,X,"ℑ","\\Im",!0);E(S,N,X,"♠","\\spadesuit",!0);E(S,N,X,"§","\\S",!0);E(we,N,X,"§","\\S");E(S,N,X,"¶","\\P",!0);E(we,N,X,"¶","\\P");E(S,N,X,"†","\\dag");E(we,N,X,"†","\\dag");E(we,N,X,"†","\\textdagger");E(S,N,X,"‡","\\ddag");E(we,N,X,"‡","\\ddag");E(we,N,X,"‡","\\textdaggerdbl");E(S,N,jr,"⎱","\\rmoustache",!0);E(S,N,Ea,"⎰","\\lmoustache",!0);E(S,N,jr,"⟯","\\rgroup",!0);E(S,N,Ea,"⟮","\\lgroup",!0);E(S,N,Ke,"∓","\\mp",!0);E(S,N,Ke,"⊖","\\ominus",!0);E(S,N,Ke,"⊎","\\uplus",!0);E(S,N,Ke,"⊓","\\sqcap",!0);E(S,N,Ke,"∗","\\ast");E(S,N,Ke,"⊔","\\sqcup",!0);E(S,N,Ke,"◯","\\bigcirc",!0);E(S,N,Ke,"∙","\\bullet",!0);E(S,N,Ke,"‡","\\ddagger");E(S,N,Ke,"≀","\\wr",!0);E(S,N,Ke,"⨿","\\amalg");E(S,N,Ke,"&","\\And");E(S,N,q,"⟵","\\longleftarrow",!0);E(S,N,q,"⇐","\\Leftarrow",!0);E(S,N,q,"⟸","\\Longleftarrow",!0);E(S,N,q,"⟶","\\longrightarrow",!0);E(S,N,q,"⇒","\\Rightarrow",!0);E(S,N,q,"⟹","\\Longrightarrow",!0);E(S,N,q,"↔","\\leftrightarrow",!0);E(S,N,q,"⟷","\\longleftrightarrow",!0);E(S,N,q,"⇔","\\Leftrightarrow",!0);E(S,N,q,"⟺","\\Longleftrightarrow",!0);E(S,N,q,"↦","\\mapsto",!0);E(S,N,q,"⟼","\\longmapsto",!0);E(S,N,q,"↗","\\nearrow",!0);E(S,N,q,"↩","\\hookleftarrow",!0);E(S,N,q,"↪","\\hookrightarrow",!0);E(S,N,q,"↘","\\searrow",!0);E(S,N,q,"↼","\\leftharpoonup",!0);E(S,N,q,"⇀","\\rightharpoonup",!0);E(S,N,q,"↙","\\swarrow",!0);E(S,N,q,"↽","\\leftharpoondown",!0);E(S,N,q,"⇁","\\rightharpoondown",!0);E(S,N,q,"↖","\\nwarrow",!0);E(S,N,q,"⇌","\\rightleftharpoons",!0);E(S,F,q,"≮","\\nless",!0);E(S,F,q,"","\\@nleqslant");E(S,F,q,"","\\@nleqq");E(S,F,q,"⪇","\\lneq",!0);E(S,F,q,"≨","\\lneqq",!0);E(S,F,q,"","\\@lvertneqq");E(S,F,q,"⋦","\\lnsim",!0);E(S,F,q,"⪉","\\lnapprox",!0);E(S,F,q,"⊀","\\nprec",!0);E(S,F,q,"⋠","\\npreceq",!0);E(S,F,q,"⋨","\\precnsim",!0);E(S,F,q,"⪹","\\precnapprox",!0);E(S,F,q,"≁","\\nsim",!0);E(S,F,q,"","\\@nshortmid");E(S,F,q,"∤","\\nmid",!0);E(S,F,q,"⊬","\\nvdash",!0);E(S,F,q,"⊭","\\nvDash",!0);E(S,F,q,"⋪","\\ntriangleleft");E(S,F,q,"⋬","\\ntrianglelefteq",!0);E(S,F,q,"⊊","\\subsetneq",!0);E(S,F,q,"","\\@varsubsetneq");E(S,F,q,"⫋","\\subsetneqq",!0);E(S,F,q,"","\\@varsubsetneqq");E(S,F,q,"≯","\\ngtr",!0);E(S,F,q,"","\\@ngeqslant");E(S,F,q,"","\\@ngeqq");E(S,F,q,"⪈","\\gneq",!0);E(S,F,q,"≩","\\gneqq",!0);E(S,F,q,"","\\@gvertneqq");E(S,F,q,"⋧","\\gnsim",!0);E(S,F,q,"⪊","\\gnapprox",!0);E(S,F,q,"⊁","\\nsucc",!0);E(S,F,q,"⋡","\\nsucceq",!0);E(S,F,q,"⋩","\\succnsim",!0);E(S,F,q,"⪺","\\succnapprox",!0);E(S,F,q,"≆","\\ncong",!0);E(S,F,q,"","\\@nshortparallel");E(S,F,q,"∦","\\nparallel",!0);E(S,F,q,"⊯","\\nVDash",!0);E(S,F,q,"⋫","\\ntriangleright");E(S,F,q,"⋭","\\ntrianglerighteq",!0);E(S,F,q,"","\\@nsupseteqq");E(S,F,q,"⊋","\\supsetneq",!0);E(S,F,q,"","\\@varsupsetneq");E(S,F,q,"⫌","\\supsetneqq",!0);E(S,F,q,"","\\@varsupsetneqq");E(S,F,q,"⊮","\\nVdash",!0);E(S,F,q,"⪵","\\precneqq",!0);E(S,F,q,"⪶","\\succneqq",!0);E(S,F,q,"","\\@nsubseteqq");E(S,F,Ke,"⊴","\\unlhd");E(S,F,Ke,"⊵","\\unrhd");E(S,F,q,"↚","\\nleftarrow",!0);E(S,F,q,"↛","\\nrightarrow",!0);E(S,F,q,"⇍","\\nLeftarrow",!0);E(S,F,q,"⇏","\\nRightarrow",!0);E(S,F,q,"↮","\\nleftrightarrow",!0);E(S,F,q,"⇎","\\nLeftrightarrow",!0);E(S,F,q,"△","\\vartriangle");E(S,F,X,"ℏ","\\hslash");E(S,F,X,"▽","\\triangledown");E(S,F,X,"◊","\\lozenge");E(S,F,X,"Ⓢ","\\circledS");E(S,F,X,"®","\\circledR");E(we,F,X,"®","\\circledR");E(S,F,X,"∡","\\measuredangle",!0);E(S,F,X,"∄","\\nexists");E(S,F,X,"℧","\\mho");E(S,F,X,"Ⅎ","\\Finv",!0);E(S,F,X,"⅁","\\Game",!0);E(S,F,X,"‵","\\backprime");E(S,F,X,"▲","\\blacktriangle");E(S,F,X,"▼","\\blacktriangledown");E(S,F,X,"■","\\blacksquare");E(S,F,X,"⧫","\\blacklozenge");E(S,F,X,"★","\\bigstar");E(S,F,X,"∢","\\sphericalangle",!0);E(S,F,X,"∁","\\complement",!0);E(S,F,X,"ð","\\eth",!0);E(we,N,X,"ð","ð");E(S,F,X,"╱","\\diagup");E(S,F,X,"╲","\\diagdown");E(S,F,X,"□","\\square");E(S,F,X,"□","\\Box");E(S,F,X,"◊","\\Diamond");E(S,F,X,"¥","\\yen",!0);E(we,F,X,"¥","\\yen",!0);E(S,F,X,"✓","\\checkmark",!0);E(we,F,X,"✓","\\checkmark");E(S,F,X,"ℶ","\\beth",!0);E(S,F,X,"ℸ","\\daleth",!0);E(S,F,X,"ℷ","\\gimel",!0);E(S,F,X,"ϝ","\\digamma",!0);E(S,F,X,"ϰ","\\varkappa");E(S,F,Ea,"┌","\\@ulcorner",!0);E(S,F,jr,"┐","\\@urcorner",!0);E(S,F,Ea,"└","\\@llcorner",!0);E(S,F,jr,"┘","\\@lrcorner",!0);E(S,F,q,"≦","\\leqq",!0);E(S,F,q,"⩽","\\leqslant",!0);E(S,F,q,"⪕","\\eqslantless",!0);E(S,F,q,"≲","\\lesssim",!0);E(S,F,q,"⪅","\\lessapprox",!0);E(S,F,q,"≊","\\approxeq",!0);E(S,F,Ke,"⋖","\\lessdot");E(S,F,q,"⋘","\\lll",!0);E(S,F,q,"≶","\\lessgtr",!0);E(S,F,q,"⋚","\\lesseqgtr",!0);E(S,F,q,"⪋","\\lesseqqgtr",!0);E(S,F,q,"≑","\\doteqdot");E(S,F,q,"≓","\\risingdotseq",!0);E(S,F,q,"≒","\\fallingdotseq",!0);E(S,F,q,"∽","\\backsim",!0);E(S,F,q,"⋍","\\backsimeq",!0);E(S,F,q,"⫅","\\subseteqq",!0);E(S,F,q,"⋐","\\Subset",!0);E(S,F,q,"⊏","\\sqsubset",!0);E(S,F,q,"≼","\\preccurlyeq",!0);E(S,F,q,"⋞","\\curlyeqprec",!0);E(S,F,q,"≾","\\precsim",!0);E(S,F,q,"⪷","\\precapprox",!0);E(S,F,q,"⊲","\\vartriangleleft");E(S,F,q,"⊴","\\trianglelefteq");E(S,F,q,"⊨","\\vDash",!0);E(S,F,q,"⊪","\\Vvdash",!0);E(S,F,q,"⌣","\\smallsmile");E(S,F,q,"⌢","\\smallfrown");E(S,F,q,"≏","\\bumpeq",!0);E(S,F,q,"≎","\\Bumpeq",!0);E(S,F,q,"≧","\\geqq",!0);E(S,F,q,"⩾","\\geqslant",!0);E(S,F,q,"⪖","\\eqslantgtr",!0);E(S,F,q,"≳","\\gtrsim",!0);E(S,F,q,"⪆","\\gtrapprox",!0);E(S,F,Ke,"⋗","\\gtrdot");E(S,F,q,"⋙","\\ggg",!0);E(S,F,q,"≷","\\gtrless",!0);E(S,F,q,"⋛","\\gtreqless",!0);E(S,F,q,"⪌","\\gtreqqless",!0);E(S,F,q,"≖","\\eqcirc",!0);E(S,F,q,"≗","\\circeq",!0);E(S,F,q,"≜","\\triangleq",!0);E(S,F,q,"∼","\\thicksim");E(S,F,q,"≈","\\thickapprox");E(S,F,q,"⫆","\\supseteqq",!0);E(S,F,q,"⋑","\\Supset",!0);E(S,F,q,"⊐","\\sqsupset",!0);E(S,F,q,"≽","\\succcurlyeq",!0);E(S,F,q,"⋟","\\curlyeqsucc",!0);E(S,F,q,"≿","\\succsim",!0);E(S,F,q,"⪸","\\succapprox",!0);E(S,F,q,"⊳","\\vartriangleright");E(S,F,q,"⊵","\\trianglerighteq");E(S,F,q,"⊩","\\Vdash",!0);E(S,F,q,"∣","\\shortmid");E(S,F,q,"∥","\\shortparallel");E(S,F,q,"≬","\\between",!0);E(S,F,q,"⋔","\\pitchfork",!0);E(S,F,q,"∝","\\varpropto");E(S,F,q,"◀","\\blacktriangleleft");E(S,F,q,"∴","\\therefore",!0);E(S,F,q,"∍","\\backepsilon");E(S,F,q,"▶","\\blacktriangleright");E(S,F,q,"∵","\\because",!0);E(S,F,q,"⋘","\\llless");E(S,F,q,"⋙","\\gggtr");E(S,F,Ke,"⊲","\\lhd");E(S,F,Ke,"⊳","\\rhd");E(S,F,q,"≂","\\eqsim",!0);E(S,N,q,"⋈","\\Join");E(S,F,q,"≑","\\Doteq",!0);E(S,F,Ke,"∔","\\dotplus",!0);E(S,F,Ke,"∖","\\smallsetminus");E(S,F,Ke,"⋒","\\Cap",!0);E(S,F,Ke,"⋓","\\Cup",!0);E(S,F,Ke,"⩞","\\doublebarwedge",!0);E(S,F,Ke,"⊟","\\boxminus",!0);E(S,F,Ke,"⊞","\\boxplus",!0);E(S,F,Ke,"⋇","\\divideontimes",!0);E(S,F,Ke,"⋉","\\ltimes",!0);E(S,F,Ke,"⋊","\\rtimes",!0);E(S,F,Ke,"⋋","\\leftthreetimes",!0);E(S,F,Ke,"⋌","\\rightthreetimes",!0);E(S,F,Ke,"⋏","\\curlywedge",!0);E(S,F,Ke,"⋎","\\curlyvee",!0);E(S,F,Ke,"⊝","\\circleddash",!0);E(S,F,Ke,"⊛","\\circledast",!0);E(S,F,Ke,"⋅","\\centerdot");E(S,F,Ke,"⊺","\\intercal",!0);E(S,F,Ke,"⋒","\\doublecap");E(S,F,Ke,"⋓","\\doublecup");E(S,F,Ke,"⊠","\\boxtimes",!0);E(S,F,q,"⇢","\\dashrightarrow",!0);E(S,F,q,"⇠","\\dashleftarrow",!0);E(S,F,q,"⇇","\\leftleftarrows",!0);E(S,F,q,"⇆","\\leftrightarrows",!0);E(S,F,q,"⇚","\\Lleftarrow",!0);E(S,F,q,"↞","\\twoheadleftarrow",!0);E(S,F,q,"↢","\\leftarrowtail",!0);E(S,F,q,"↫","\\looparrowleft",!0);E(S,F,q,"⇋","\\leftrightharpoons",!0);E(S,F,q,"↶","\\curvearrowleft",!0);E(S,F,q,"↺","\\circlearrowleft",!0);E(S,F,q,"↰","\\Lsh",!0);E(S,F,q,"⇈","\\upuparrows",!0);E(S,F,q,"↿","\\upharpoonleft",!0);E(S,F,q,"⇃","\\downharpoonleft",!0);E(S,N,q,"⊶","\\origof",!0);E(S,N,q,"⊷","\\imageof",!0);E(S,F,q,"⊸","\\multimap",!0);E(S,F,q,"↭","\\leftrightsquigarrow",!0);E(S,F,q,"⇉","\\rightrightarrows",!0);E(S,F,q,"⇄","\\rightleftarrows",!0);E(S,F,q,"↠","\\twoheadrightarrow",!0);E(S,F,q,"↣","\\rightarrowtail",!0);E(S,F,q,"↬","\\looparrowright",!0);E(S,F,q,"↷","\\curvearrowright",!0);E(S,F,q,"↻","\\circlearrowright",!0);E(S,F,q,"↱","\\Rsh",!0);E(S,F,q,"⇊","\\downdownarrows",!0);E(S,F,q,"↾","\\upharpoonright",!0);E(S,F,q,"⇂","\\downharpoonright",!0);E(S,F,q,"⇝","\\rightsquigarrow",!0);E(S,F,q,"⇝","\\leadsto");E(S,F,q,"⇛","\\Rrightarrow",!0);E(S,F,q,"↾","\\restriction");E(S,N,X,"‘","`");E(S,N,X,"$","\\$");E(we,N,X,"$","\\$");E(we,N,X,"$","\\textdollar");E(S,N,X,"%","\\%");E(we,N,X,"%","\\%");E(S,N,X,"_","\\_");E(we,N,X,"_","\\_");E(we,N,X,"_","\\textunderscore");E(S,N,X,"∠","\\angle",!0);E(S,N,X,"∞","\\infty",!0);E(S,N,X,"′","\\prime");E(S,N,X,"△","\\triangle");E(S,N,X,"Γ","\\Gamma",!0);E(S,N,X,"Δ","\\Delta",!0);E(S,N,X,"Θ","\\Theta",!0);E(S,N,X,"Λ","\\Lambda",!0);E(S,N,X,"Ξ","\\Xi",!0);E(S,N,X,"Π","\\Pi",!0);E(S,N,X,"Σ","\\Sigma",!0);E(S,N,X,"Υ","\\Upsilon",!0);E(S,N,X,"Φ","\\Phi",!0);E(S,N,X,"Ψ","\\Psi",!0);E(S,N,X,"Ω","\\Omega",!0);E(S,N,X,"A","Α");E(S,N,X,"B","Β");E(S,N,X,"E","Ε");E(S,N,X,"Z","Ζ");E(S,N,X,"H","Η");E(S,N,X,"I","Ι");E(S,N,X,"K","Κ");E(S,N,X,"M","Μ");E(S,N,X,"N","Ν");E(S,N,X,"O","Ο");E(S,N,X,"P","Ρ");E(S,N,X,"T","Τ");E(S,N,X,"X","Χ");E(S,N,X,"¬","\\neg",!0);E(S,N,X,"¬","\\lnot");E(S,N,X,"⊤","\\top");E(S,N,X,"⊥","\\bot");E(S,N,X,"∅","\\emptyset");E(S,F,X,"∅","\\varnothing");E(S,N,st,"α","\\alpha",!0);E(S,N,st,"β","\\beta",!0);E(S,N,st,"γ","\\gamma",!0);E(S,N,st,"δ","\\delta",!0);E(S,N,st,"ϵ","\\epsilon",!0);E(S,N,st,"ζ","\\zeta",!0);E(S,N,st,"η","\\eta",!0);E(S,N,st,"θ","\\theta",!0);E(S,N,st,"ι","\\iota",!0);E(S,N,st,"κ","\\kappa",!0);E(S,N,st,"λ","\\lambda",!0);E(S,N,st,"μ","\\mu",!0);E(S,N,st,"ν","\\nu",!0);E(S,N,st,"ξ","\\xi",!0);E(S,N,st,"ο","\\omicron",!0);E(S,N,st,"π","\\pi",!0);E(S,N,st,"ρ","\\rho",!0);E(S,N,st,"σ","\\sigma",!0);E(S,N,st,"τ","\\tau",!0);E(S,N,st,"υ","\\upsilon",!0);E(S,N,st,"ϕ","\\phi",!0);E(S,N,st,"χ","\\chi",!0);E(S,N,st,"ψ","\\psi",!0);E(S,N,st,"ω","\\omega",!0);E(S,N,st,"ε","\\varepsilon",!0);E(S,N,st,"ϑ","\\vartheta",!0);E(S,N,st,"ϖ","\\varpi",!0);E(S,N,st,"ϱ","\\varrho",!0);E(S,N,st,"ς","\\varsigma",!0);E(S,N,st,"φ","\\varphi",!0);E(S,N,Ke,"∗","*",!0);E(S,N,Ke,"+","+");E(S,N,Ke,"−","-",!0);E(S,N,Ke,"⋅","\\cdot",!0);E(S,N,Ke,"∘","\\circ",!0);E(S,N,Ke,"÷","\\div",!0);E(S,N,Ke,"±","\\pm",!0);E(S,N,Ke,"×","\\times",!0);E(S,N,Ke,"∩","\\cap",!0);E(S,N,Ke,"∪","\\cup",!0);E(S,N,Ke,"∖","\\setminus",!0);E(S,N,Ke,"∧","\\land");E(S,N,Ke,"∨","\\lor");E(S,N,Ke,"∧","\\wedge",!0);E(S,N,Ke,"∨","\\vee",!0);E(S,N,X,"√","\\surd");E(S,N,Ea,"⟨","\\langle",!0);E(S,N,Ea,"∣","\\lvert");E(S,N,Ea,"∥","\\lVert");E(S,N,jr,"?","?");E(S,N,jr,"!","!");E(S,N,jr,"⟩","\\rangle",!0);E(S,N,jr,"∣","\\rvert");E(S,N,jr,"∥","\\rVert");E(S,N,q,"=","=");E(S,N,q,":",":");E(S,N,q,"≈","\\approx",!0);E(S,N,q,"≅","\\cong",!0);E(S,N,q,"≥","\\ge");E(S,N,q,"≥","\\geq",!0);E(S,N,q,"←","\\gets");E(S,N,q,">","\\gt",!0);E(S,N,q,"∈","\\in",!0);E(S,N,q,"","\\@not");E(S,N,q,"⊂","\\subset",!0);E(S,N,q,"⊃","\\supset",!0);E(S,N,q,"⊆","\\subseteq",!0);E(S,N,q,"⊇","\\supseteq",!0);E(S,F,q,"⊈","\\nsubseteq",!0);E(S,F,q,"⊉","\\nsupseteq",!0);E(S,N,q,"⊨","\\models");E(S,N,q,"←","\\leftarrow",!0);E(S,N,q,"≤","\\le");E(S,N,q,"≤","\\leq",!0);E(S,N,q,"<","\\lt",!0);E(S,N,q,"→","\\rightarrow",!0);E(S,N,q,"→","\\to");E(S,F,q,"≱","\\ngeq",!0);E(S,F,q,"≰","\\nleq",!0);E(S,N,cs," ","\\ ");E(S,N,cs," ","\\space");E(S,N,cs," ","\\nobreakspace");E(we,N,cs," ","\\ ");E(we,N,cs," "," ");E(we,N,cs," ","\\space");E(we,N,cs," ","\\nobreakspace");E(S,N,cs,null,"\\nobreak");E(S,N,cs,null,"\\allowbreak");E(S,N,Jf,",",",");E(S,N,Jf,";",";");E(S,F,Ke,"⊼","\\barwedge",!0);E(S,F,Ke,"⊻","\\veebar",!0);E(S,N,Ke,"⊙","\\odot",!0);E(S,N,Ke,"⊕","\\oplus",!0);E(S,N,Ke,"⊗","\\otimes",!0);E(S,N,X,"∂","\\partial",!0);E(S,N,Ke,"⊘","\\oslash",!0);E(S,F,Ke,"⊚","\\circledcirc",!0);E(S,F,Ke,"⊡","\\boxdot",!0);E(S,N,Ke,"△","\\bigtriangleup");E(S,N,Ke,"▽","\\bigtriangledown");E(S,N,Ke,"†","\\dagger");E(S,N,Ke,"⋄","\\diamond");E(S,N,Ke,"⋆","\\star");E(S,N,Ke,"◃","\\triangleleft");E(S,N,Ke,"▹","\\triangleright");E(S,N,Ea,"{","\\{");E(we,N,X,"{","\\{");E(we,N,X,"{","\\textbraceleft");E(S,N,jr,"}","\\}");E(we,N,X,"}","\\}");E(we,N,X,"}","\\textbraceright");E(S,N,Ea,"{","\\lbrace");E(S,N,jr,"}","\\rbrace");E(S,N,Ea,"[","\\lbrack",!0);E(we,N,X,"[","\\lbrack",!0);E(S,N,jr,"]","\\rbrack",!0);E(we,N,X,"]","\\rbrack",!0);E(S,N,Ea,"(","\\lparen",!0);E(S,N,jr,")","\\rparen",!0);E(we,N,X,"<","\\textless",!0);E(we,N,X,">","\\textgreater",!0);E(S,N,Ea,"⌊","\\lfloor",!0);E(S,N,jr,"⌋","\\rfloor",!0);E(S,N,Ea,"⌈","\\lceil",!0);E(S,N,jr,"⌉","\\rceil",!0);E(S,N,X,"\\","\\backslash");E(S,N,X,"∣","|");E(S,N,X,"∣","\\vert");E(we,N,X,"|","\\textbar",!0);E(S,N,X,"∥","\\|");E(S,N,X,"∥","\\Vert");E(we,N,X,"∥","\\textbardbl");E(we,N,X,"~","\\textasciitilde");E(we,N,X,"\\","\\textbackslash");E(we,N,X,"^","\\textasciicircum");E(S,N,q,"↑","\\uparrow",!0);E(S,N,q,"⇑","\\Uparrow",!0);E(S,N,q,"↓","\\downarrow",!0);E(S,N,q,"⇓","\\Downarrow",!0);E(S,N,q,"↕","\\updownarrow",!0);E(S,N,q,"⇕","\\Updownarrow",!0);E(S,N,Qn,"∐","\\coprod");E(S,N,Qn,"⋁","\\bigvee");E(S,N,Qn,"⋀","\\bigwedge");E(S,N,Qn,"⨄","\\biguplus");E(S,N,Qn,"⋂","\\bigcap");E(S,N,Qn,"⋃","\\bigcup");E(S,N,Qn,"∫","\\int");E(S,N,Qn,"∫","\\intop");E(S,N,Qn,"∬","\\iint");E(S,N,Qn,"∭","\\iiint");E(S,N,Qn,"∏","\\prod");E(S,N,Qn,"∑","\\sum");E(S,N,Qn,"⨂","\\bigotimes");E(S,N,Qn,"⨁","\\bigoplus");E(S,N,Qn,"⨀","\\bigodot");E(S,N,Qn,"∮","\\oint");E(S,N,Qn,"∯","\\oiint");E(S,N,Qn,"∰","\\oiiint");E(S,N,Qn,"⨆","\\bigsqcup");E(S,N,Qn,"∫","\\smallint");E(we,N,vu,"…","\\textellipsis");E(S,N,vu,"…","\\mathellipsis");E(we,N,vu,"…","\\ldots",!0);E(S,N,vu,"…","\\ldots",!0);E(S,N,vu,"⋯","\\@cdots",!0);E(S,N,vu,"⋱","\\ddots",!0);E(S,N,X,"⋮","\\varvdots");E(we,N,X,"⋮","\\varvdots");E(S,N,Sn,"ˊ","\\acute");E(S,N,Sn,"ˋ","\\grave");E(S,N,Sn,"¨","\\ddot");E(S,N,Sn,"~","\\tilde");E(S,N,Sn,"ˉ","\\bar");E(S,N,Sn,"˘","\\breve");E(S,N,Sn,"ˇ","\\check");E(S,N,Sn,"^","\\hat");E(S,N,Sn,"⃗","\\vec");E(S,N,Sn,"˙","\\dot");E(S,N,Sn,"˚","\\mathring");E(S,N,st,"","\\@imath");E(S,N,st,"","\\@jmath");E(S,N,X,"ı","ı");E(S,N,X,"ȷ","ȷ");E(we,N,X,"ı","\\i",!0);E(we,N,X,"ȷ","\\j",!0);E(we,N,X,"ß","\\ss",!0);E(we,N,X,"æ","\\ae",!0);E(we,N,X,"œ","\\oe",!0);E(we,N,X,"ø","\\o",!0);E(we,N,X,"Æ","\\AE",!0);E(we,N,X,"Œ","\\OE",!0);E(we,N,X,"Ø","\\O",!0);E(we,N,Sn,"ˊ","\\'");E(we,N,Sn,"ˋ","\\`");E(we,N,Sn,"ˆ","\\^");E(we,N,Sn,"˜","\\~");E(we,N,Sn,"ˉ","\\=");E(we,N,Sn,"˘","\\u");E(we,N,Sn,"˙","\\.");E(we,N,Sn,"¸","\\c");E(we,N,Sn,"˚","\\r");E(we,N,Sn,"ˇ","\\v");E(we,N,Sn,"¨",'\\"');E(we,N,Sn,"˝","\\H");E(we,N,Sn,"◯","\\textcircled");var _x={"--":!0,"---":!0,"``":!0,"''":!0};E(we,N,X,"–","--",!0);E(we,N,X,"–","\\textendash");E(we,N,X,"—","---",!0);E(we,N,X,"—","\\textemdash");E(we,N,X,"‘","`",!0);E(we,N,X,"‘","\\textquoteleft");E(we,N,X,"’","'",!0);E(we,N,X,"’","\\textquoteright");E(we,N,X,"“","``",!0);E(we,N,X,"“","\\textquotedblleft");E(we,N,X,"”","''",!0);E(we,N,X,"”","\\textquotedblright");E(S,N,X,"°","\\degree",!0);E(we,N,X,"°","\\degree");E(we,N,X,"°","\\textdegree",!0);E(S,N,X,"£","\\pounds");E(S,N,X,"£","\\mathsterling",!0);E(we,N,X,"£","\\pounds");E(we,N,X,"£","\\textsterling",!0);E(S,F,X,"✠","\\maltese");E(we,F,X,"✠","\\maltese");var a6='0123456789/@."';for(var rg=0;rg<a6.length;rg++){var i6=a6.charAt(rg);E(S,N,X,i6,i6)}var s6='0123456789!@*()-=+";:?/.,';for(var ag=0;ag<s6.length;ag++){var l6=s6.charAt(ag);E(we,N,X,l6,l6)}var cf="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";for(var ig=0;ig<cf.length;ig++){var vh=cf.charAt(ig);E(S,N,st,vh,vh),E(we,N,X,vh,vh)}E(S,F,X,"C","ℂ");E(we,F,X,"C","ℂ");E(S,F,X,"H","ℍ");E(we,F,X,"H","ℍ");E(S,F,X,"N","ℕ");E(we,F,X,"N","ℕ");E(S,F,X,"P","ℙ");E(we,F,X,"P","ℙ");E(S,F,X,"Q","ℚ");E(we,F,X,"Q","ℚ");E(S,F,X,"R","ℝ");E(we,F,X,"R","ℝ");E(S,F,X,"Z","ℤ");E(we,F,X,"Z","ℤ");E(S,N,st,"h","ℎ");E(we,N,st,"h","ℎ");var mt="";for(var Dr=0;Dr<cf.length;Dr++){var Dn=cf.charAt(Dr);mt=String.fromCharCode(55349,56320+Dr),E(S,N,st,Dn,mt),E(we,N,X,Dn,mt),mt=String.fromCharCode(55349,56372+Dr),E(S,N,st,Dn,mt),E(we,N,X,Dn,mt),mt=String.fromCharCode(55349,56424+Dr),E(S,N,st,Dn,mt),E(we,N,X,Dn,mt),mt=String.fromCharCode(55349,56580+Dr),E(S,N,st,Dn,mt),E(we,N,X,Dn,mt),mt=String.fromCharCode(55349,56684+Dr),E(S,N,st,Dn,mt),E(we,N,X,Dn,mt),mt=String.fromCharCode(55349,56736+Dr),E(S,N,st,Dn,mt),E(we,N,X,Dn,mt),mt=String.fromCharCode(55349,56788+Dr),E(S,N,st,Dn,mt),E(we,N,X,Dn,mt),mt=String.fromCharCode(55349,56840+Dr),E(S,N,st,Dn,mt),E(we,N,X,Dn,mt),mt=String.fromCharCode(55349,56944+Dr),E(S,N,st,Dn,mt),E(we,N,X,Dn,mt),Dr<26&&(mt=String.fromCharCode(55349,56632+Dr),E(S,N,st,Dn,mt),E(we,N,X,Dn,mt),mt=String.fromCharCode(55349,56476+Dr),E(S,N,st,Dn,mt),E(we,N,X,Dn,mt))}mt="𝕜";E(S,N,st,"k",mt);E(we,N,X,"k",mt);for(var Pl=0;Pl<10;Pl++){var qs=Pl.toString();mt=String.fromCharCode(55349,57294+Pl),E(S,N,st,qs,mt),E(we,N,X,qs,mt),mt=String.fromCharCode(55349,57314+Pl),E(S,N,st,qs,mt),E(we,N,X,qs,mt),mt=String.fromCharCode(55349,57324+Pl),E(S,N,st,qs,mt),E(we,N,X,qs,mt),mt=String.fromCharCode(55349,57334+Pl),E(S,N,st,qs,mt),E(we,N,X,qs,mt)}var yb="ÐÞþ";for(var sg=0;sg<yb.length;sg++){var yh=yb.charAt(sg);E(S,N,st,yh,yh),E(we,N,X,yh,yh)}var _h=[["mathbf","textbf","Main-Bold"],["mathbf","textbf","Main-Bold"],["mathnormal","textit","Math-Italic"],["mathnormal","textit","Math-Italic"],["boldsymbol","boldsymbol","Main-BoldItalic"],["boldsymbol","boldsymbol","Main-BoldItalic"],["mathscr","textscr","Script-Regular"],["","",""],["","",""],["","",""],["mathfrak","textfrak","Fraktur-Regular"],["mathfrak","textfrak","Fraktur-Regular"],["mathbb","textbb","AMS-Regular"],["mathbb","textbb","AMS-Regular"],["mathboldfrak","textboldfrak","Fraktur-Regular"],["mathboldfrak","textboldfrak","Fraktur-Regular"],["mathsf","textsf","SansSerif-Regular"],["mathsf","textsf","SansSerif-Regular"],["mathboldsf","textboldsf","SansSerif-Bold"],["mathboldsf","textboldsf","SansSerif-Bold"],["mathitsf","textitsf","SansSerif-Italic"],["mathitsf","textitsf","SansSerif-Italic"],["","",""],["","",""],["mathtt","texttt","Typewriter-Regular"],["mathtt","texttt","Typewriter-Regular"]],o6=[["mathbf","textbf","Main-Bold"],["","",""],["mathsf","textsf","SansSerif-Regular"],["mathboldsf","textboldsf","SansSerif-Bold"],["mathtt","texttt","Typewriter-Regular"]],mj=function(t,n){var r=t.charCodeAt(0),a=t.charCodeAt(1),s=(r-55296)*1024+(a-56320)+65536,o=n==="math"?0:1;if(119808<=s&&s<120484){var c=Math.floor((s-119808)/26);return[_h[c][2],_h[c][o]]}else if(120782<=s&&s<=120831){var d=Math.floor((s-120782)/10);return[o6[d][2],o6[d][o]]}else{if(s===120485||s===120486)return[_h[0][2],_h[0][o]];if(120486<s&&s<120782)return["",""];throw new Ne("Unsupported character: "+t)}},em=function(t,n,r){return fn[r][t]&&fn[r][t].replace&&(t=fn[r][t].replace),{value:t,metrics:F2(t,n,r)}},Ka=function(t,n,r,a,s){var o=em(t,n,r),c=o.metrics;t=o.value;var d;if(c){var f=c.italic;(r==="text"||a&&a.font==="mathit")&&(f=0),d=new Ua(t,c.height,c.depth,f,c.skew,c.width,s)}else typeof console<"u"&&console.warn("No character metrics "+("for '"+t+"' in style '"+n+"' and mode '"+r+"'")),d=new Ua(t,0,0,0,0,0,s);if(a){d.maxFontSize=a.sizeMultiplier,a.style.isTight()&&d.classes.push("mtight");var m=a.getColor();m&&(d.style.color=m)}return d},pj=function(t,n,r,a){return a===void 0&&(a=[]),r.font==="boldsymbol"&&em(t,"Main-Bold",n).metrics?Ka(t,"Main-Bold",n,r,a.concat(["mathbf"])):t==="\\"||fn[n][t].font==="main"?Ka(t,"Main-Regular",n,r,a):Ka(t,"AMS-Regular",n,r,a.concat(["amsrm"]))},gj=function(t,n,r,a,s){return s!=="textord"&&em(t,"Math-BoldItalic",n).metrics?{fontName:"Math-BoldItalic",fontClass:"boldsymbol"}:{fontName:"Main-Bold",fontClass:"mathbf"}},bj=function(t,n,r){var a=t.mode,s=t.text,o=["mord"],c=a==="math"||a==="text"&&n.font,d=c?n.font:n.fontFamily,f="",m="";if(s.charCodeAt(0)===55349&&([f,m]=mj(s,a)),f.length>0)return Ka(s,f,a,n,o.concat(m));if(d){var p,b;if(d==="boldsymbol"){var y=gj(s,a,n,o,r);p=y.fontName,b=[y.fontClass]}else c?(p=xx[d].fontName,b=[d]):(p=Eh(d,n.fontWeight,n.fontShape),b=[d,n.fontWeight,n.fontShape]);if(em(s,p,a).metrics)return Ka(s,p,a,n,o.concat(b));if(_x.hasOwnProperty(s)&&p.slice(0,10)==="Typewriter"){for(var T=[],x=0;x<s.length;x++)T.push(Ka(s[x],p,a,n,o.concat(b)));return Tx(T)}}if(r==="mathord")return Ka(s,"Math-Italic",a,n,o.concat(["mathnormal"]));if(r==="textord"){var C=fn[a][s]&&fn[a][s].font;if(C==="ams"){var w=Eh("amsrm",n.fontWeight,n.fontShape);return Ka(s,w,a,n,o.concat("amsrm",n.fontWeight,n.fontShape))}else if(C==="main"||!C){var O=Eh("textrm",n.fontWeight,n.fontShape);return Ka(s,O,a,n,o.concat(n.fontWeight,n.fontShape))}else{var D=Eh(C,n.fontWeight,n.fontShape);return Ka(s,D,a,n,o.concat(D,n.fontWeight,n.fontShape))}}else throw new Error("unexpected type: "+r+" in makeOrd")},vj=(e,t)=>{if(il(e.classes)!==il(t.classes)||e.skew!==t.skew||e.maxFontSize!==t.maxFontSize)return!1;if(e.classes.length===1){var n=e.classes[0];if(n==="mbin"||n==="mord")return!1}for(var r in e.style)if(e.style.hasOwnProperty(r)&&e.style[r]!==t.style[r])return!1;for(var a in t.style)if(t.style.hasOwnProperty(a)&&e.style[a]!==t.style[a])return!1;return!0},yj=e=>{for(var t=0;t<e.length-1;t++){var n=e[t],r=e[t+1];n instanceof Ua&&r instanceof Ua&&vj(n,r)&&(n.text+=r.text,n.height=Math.max(n.height,r.height),n.depth=Math.max(n.depth,r.depth),n.italic=r.italic,e.splice(t+1,1),t--)}return e},H2=function(t){for(var n=0,r=0,a=0,s=0;s<t.children.length;s++){var o=t.children[s];o.height>n&&(n=o.height),o.depth>r&&(r=o.depth),o.maxFontSize>a&&(a=o.maxFontSize)}t.height=n,t.depth=r,t.maxFontSize=a},Jr=function(t,n,r,a){var s=new B0(t,n,r,a);return H2(s),s},Ex=(e,t,n,r)=>new B0(e,t,n,r),_j=function(t,n,r){var a=Jr([t],[],n);return a.height=Math.max(r||n.fontMetrics().defaultRuleThickness,n.minRuleThickness),a.style.borderBottomWidth=Le(a.height),a.maxFontSize=1,a},Ej=function(t,n,r,a){var s=new z2(t,n,r,a);return H2(s),s},Tx=function(t){var n=new P0(t);return H2(n),n},Tj=function(t,n){return t instanceof P0?Jr([],[t],n):t},xj=function(t){if(t.positionType==="individualShift"){for(var n=t.children,r=[n[0]],a=-n[0].shift-n[0].elem.depth,s=a,o=1;o<n.length;o++){var c=-n[o].shift-s-n[o].elem.depth,d=c-(n[o-1].elem.height+n[o-1].elem.depth);s=s+c,r.push({type:"kern",size:d}),r.push(n[o])}return{children:r,depth:a}}var f;if(t.positionType==="top"){for(var m=t.positionData,p=0;p<t.children.length;p++){var b=t.children[p];m-=b.type==="kern"?b.size:b.elem.height+b.elem.depth}f=m}else if(t.positionType==="bottom")f=-t.positionData;else{var y=t.children[0];if(y.type!=="elem")throw new Error('First child must have type "elem".');if(t.positionType==="shift")f=-y.elem.depth-t.positionData;else if(t.positionType==="firstBaseline")f=-y.elem.depth;else throw new Error("Invalid positionType "+t.positionType+".")}return{children:t.children,depth:f}},Sj=function(t,n){for(var{children:r,depth:a}=xj(t),s=0,o=0;o<r.length;o++){var c=r[o];if(c.type==="elem"){var d=c.elem;s=Math.max(s,d.maxFontSize,d.height)}}s+=2;var f=Jr(["pstrut"],[]);f.style.height=Le(s);for(var m=[],p=a,b=a,y=a,T=0;T<r.length;T++){var x=r[T];if(x.type==="kern")y+=x.size;else{var C=x.elem,w=x.wrapperClasses||[],O=x.wrapperStyle||{},D=Jr(w,[f,C],void 0,O);D.style.top=Le(-s-y-C.depth),x.marginLeft&&(D.style.marginLeft=x.marginLeft),x.marginRight&&(D.style.marginRight=x.marginRight),m.push(D),y+=C.height+C.depth}p=Math.min(p,y),b=Math.max(b,y)}var Y=Jr(["vlist"],m);Y.style.height=Le(b);var H;if(p<0){var B=Jr([],[]),ee=Jr(["vlist"],[B]);ee.style.height=Le(-p);var Z=Jr(["vlist-s"],[new Ua("​")]);H=[Jr(["vlist-r"],[Y,Z]),Jr(["vlist-r"],[ee])]}else H=[Jr(["vlist-r"],[Y])];var Q=Jr(["vlist-t"],H);return H.length===2&&Q.classes.push("vlist-t2"),Q.height=b,Q.depth=-p,Q},wj=(e,t)=>{var n=Jr(["mspace"],[],t),r=Rn(e,t);return n.style.marginRight=Le(r),n},Eh=function(t,n,r){var a="";switch(t){case"amsrm":a="AMS";break;case"textrm":a="Main";break;case"textsf":a="SansSerif";break;case"texttt":a="Typewriter";break;default:a=t}var s;return n==="textbf"&&r==="textit"?s="BoldItalic":n==="textbf"?s="Bold":n==="textit"?s="Italic":s="Regular",a+"-"+s},xx={mathbf:{variant:"bold",fontName:"Main-Bold"},mathrm:{variant:"normal",fontName:"Main-Regular"},textit:{variant:"italic",fontName:"Main-Italic"},mathit:{variant:"italic",fontName:"Main-Italic"},mathnormal:{variant:"italic",fontName:"Math-Italic"},mathsfit:{variant:"sans-serif-italic",fontName:"SansSerif-Italic"},mathbb:{variant:"double-struck",fontName:"AMS-Regular"},mathcal:{variant:"script",fontName:"Caligraphic-Regular"},mathfrak:{variant:"fraktur",fontName:"Fraktur-Regular"},mathscr:{variant:"script",fontName:"Script-Regular"},mathsf:{variant:"sans-serif",fontName:"SansSerif-Regular"},mathtt:{variant:"monospace",fontName:"Typewriter-Regular"}},Sx={vec:["vec",.471,.714],oiintSize1:["oiintSize1",.957,.499],oiintSize2:["oiintSize2",1.472,.659],oiiintSize1:["oiiintSize1",1.304,.499],oiiintSize2:["oiiintSize2",1.98,.659]},Aj=function(t,n){var[r,a,s]=Sx[t],o=new sl(r),c=new as([o],{width:Le(a),height:Le(s),style:"width:"+Le(a),viewBox:"0 0 "+1e3*a+" "+1e3*s,preserveAspectRatio:"xMinYMin"}),d=Ex(["overlay"],[c],n);return d.height=s,d.style.height=Le(s),d.style.width=Le(a),d},ae={fontMap:xx,makeSymbol:Ka,mathsym:pj,makeSpan:Jr,makeSvgSpan:Ex,makeLineSpan:_j,makeAnchor:Ej,makeFragment:Tx,wrapFragment:Tj,makeVList:Sj,makeOrd:bj,makeGlue:wj,staticSvg:Aj,svgData:Sx,tryCombineChars:yj},Nn={number:3,unit:"mu"},Bl={number:4,unit:"mu"},ji={number:5,unit:"mu"},kj={mord:{mop:Nn,mbin:Bl,mrel:ji,minner:Nn},mop:{mord:Nn,mop:Nn,mrel:ji,minner:Nn},mbin:{mord:Bl,mop:Bl,mopen:Bl,minner:Bl},mrel:{mord:ji,mop:ji,mopen:ji,minner:ji},mopen:{},mclose:{mop:Nn,mbin:Bl,mrel:ji,minner:Nn},mpunct:{mord:Nn,mop:Nn,mrel:ji,mopen:Nn,mclose:Nn,mpunct:Nn,minner:Nn},minner:{mord:Nn,mop:Nn,mbin:Bl,mrel:ji,mopen:Nn,mpunct:Nn,minner:Nn}},Cj={mord:{mop:Nn},mop:{mord:Nn,mop:Nn},mbin:{},mrel:{},mopen:{},mclose:{mop:Nn},mpunct:{},minner:{mop:Nn}},wx={},df={},hf={};function Ye(e){for(var{type:t,names:n,props:r,handler:a,htmlBuilder:s,mathmlBuilder:o}=e,c={type:t,numArgs:r.numArgs,argTypes:r.argTypes,allowedInArgument:!!r.allowedInArgument,allowedInText:!!r.allowedInText,allowedInMath:r.allowedInMath===void 0?!0:r.allowedInMath,numOptionalArgs:r.numOptionalArgs||0,infix:!!r.infix,primitive:!!r.primitive,handler:a},d=0;d<n.length;++d)wx[n[d]]=c;t&&(s&&(df[t]=s),o&&(hf[t]=o))}function to(e){var{type:t,htmlBuilder:n,mathmlBuilder:r}=e;Ye({type:t,names:[],props:{numArgs:0},handler(){throw new Error("Should never be called.")},htmlBuilder:n,mathmlBuilder:r})}var ff=function(t){return t.type==="ordgroup"&&t.body.length===1?t.body[0]:t},Hn=function(t){return t.type==="ordgroup"?t.body:[t]},is=ae.makeSpan,Nj=["leftmost","mbin","mopen","mrel","mop","mpunct"],Oj=["rightmost","mrel","mclose","mpunct"],Rj={display:lt.DISPLAY,text:lt.TEXT,script:lt.SCRIPT,scriptscript:lt.SCRIPTSCRIPT},Mj={mord:"mord",mop:"mop",mbin:"mbin",mrel:"mrel",mopen:"mopen",mclose:"mclose",mpunct:"mpunct",minner:"minner"},sr=function(t,n,r,a){a===void 0&&(a=[null,null]);for(var s=[],o=0;o<t.length;o++){var c=Ft(t[o],n);if(c instanceof P0){var d=c.children;s.push(...d)}else s.push(c)}if(ae.tryCombineChars(s),!r)return s;var f=n;if(t.length===1){var m=t[0];m.type==="sizing"?f=n.havingSize(m.size):m.type==="styling"&&(f=n.havingStyle(Rj[m.style]))}var p=is([a[0]||"leftmost"],[],n),b=is([a[1]||"rightmost"],[],n),y=r==="root";return u6(s,(T,x)=>{var C=x.classes[0],w=T.classes[0];C==="mbin"&&it.contains(Oj,w)?x.classes[0]="mord":w==="mbin"&&it.contains(Nj,C)&&(T.classes[0]="mord")},{node:p},b,y),u6(s,(T,x)=>{var C=_b(x),w=_b(T),O=C&&w?T.hasClass("mtight")?Cj[C][w]:kj[C][w]:null;if(O)return ae.makeGlue(O,f)},{node:p},b,y),s},u6=function e(t,n,r,a,s){a&&t.push(a);for(var o=0;o<t.length;o++){var c=t[o],d=Ax(c);if(d){e(d.children,n,r,null,s);continue}var f=!c.hasClass("mspace");if(f){var m=n(c,r.node);m&&(r.insertAfter?r.insertAfter(m):(t.unshift(m),o++))}f?r.node=c:s&&c.hasClass("newline")&&(r.node=is(["leftmost"])),r.insertAfter=(p=>b=>{t.splice(p+1,0,b),o++})(o)}a&&t.pop()},Ax=function(t){return t instanceof P0||t instanceof z2||t instanceof B0&&t.hasClass("enclosing")?t:null},Ij=function e(t,n){var r=Ax(t);if(r){var a=r.children;if(a.length){if(n==="right")return e(a[a.length-1],"right");if(n==="left")return e(a[0],"left")}}return t},_b=function(t,n){return t?(n&&(t=Ij(t,n)),Mj[t.classes[0]]||null):null},g0=function(t,n){var r=["nulldelimiter"].concat(t.baseSizingClasses());return is(n.concat(r))},Ft=function(t,n,r){if(!t)return is();if(df[t.type]){var a=df[t.type](t,n);if(r&&n.size!==r.size){a=is(n.sizingClasses(r),[a],n);var s=n.sizeMultiplier/r.sizeMultiplier;a.height*=s,a.depth*=s}return a}else throw new Ne("Got group of unknown type: '"+t.type+"'")};function Th(e,t){var n=is(["base"],e,t),r=is(["strut"]);return r.style.height=Le(n.height+n.depth),n.depth&&(r.style.verticalAlign=Le(-n.depth)),n.children.unshift(r),n}function Eb(e,t){var n=null;e.length===1&&e[0].type==="tag"&&(n=e[0].tag,e=e[0].body);var r=sr(e,t,"root"),a;r.length===2&&r[1].hasClass("tag")&&(a=r.pop());for(var s=[],o=[],c=0;c<r.length;c++)if(o.push(r[c]),r[c].hasClass("mbin")||r[c].hasClass("mrel")||r[c].hasClass("allowbreak")){for(var d=!1;c<r.length-1&&r[c+1].hasClass("mspace")&&!r[c+1].hasClass("newline");)c++,o.push(r[c]),r[c].hasClass("nobreak")&&(d=!0);d||(s.push(Th(o,t)),o=[])}else r[c].hasClass("newline")&&(o.pop(),o.length>0&&(s.push(Th(o,t)),o=[]),s.push(r[c]));o.length>0&&s.push(Th(o,t));var f;n?(f=Th(sr(n,t,!0)),f.classes=["tag"],s.push(f)):a&&s.push(a);var m=is(["katex-html"],s);if(m.setAttribute("aria-hidden","true"),f){var p=f.children[0];p.style.height=Le(m.height+m.depth),m.depth&&(p.style.verticalAlign=Le(-m.depth))}return m}function kx(e){return new P0(e)}class va{constructor(t,n,r){this.type=void 0,this.attributes=void 0,this.children=void 0,this.classes=void 0,this.type=t,this.attributes={},this.children=n||[],this.classes=r||[]}setAttribute(t,n){this.attributes[t]=n}getAttribute(t){return this.attributes[t]}toNode(){var t=document.createElementNS("http://www.w3.org/1998/Math/MathML",this.type);for(var n in this.attributes)Object.prototype.hasOwnProperty.call(this.attributes,n)&&t.setAttribute(n,this.attributes[n]);this.classes.length>0&&(t.className=il(this.classes));for(var r=0;r<this.children.length;r++)if(this.children[r]instanceof fi&&this.children[r+1]instanceof fi){for(var a=this.children[r].toText()+this.children[++r].toText();this.children[r+1]instanceof fi;)a+=this.children[++r].toText();t.appendChild(new fi(a).toNode())}else t.appendChild(this.children[r].toNode());return t}toMarkup(){var t="<"+this.type;for(var n in this.attributes)Object.prototype.hasOwnProperty.call(this.attributes,n)&&(t+=" "+n+'="',t+=it.escape(this.attributes[n]),t+='"');this.classes.length>0&&(t+=' class ="'+it.escape(il(this.classes))+'"'),t+=">";for(var r=0;r<this.children.length;r++)t+=this.children[r].toMarkup();return t+="</"+this.type+">",t}toText(){return this.children.map(t=>t.toText()).join("")}}class fi{constructor(t){this.text=void 0,this.text=t}toNode(){return document.createTextNode(this.text)}toMarkup(){return it.escape(this.toText())}toText(){return this.text}}class Dj{constructor(t){this.width=void 0,this.character=void 0,this.width=t,t>=.05555&&t<=.05556?this.character=" ":t>=.1666&&t<=.1667?this.character=" ":t>=.2222&&t<=.2223?this.character=" ":t>=.2777&&t<=.2778?this.character="  ":t>=-.05556&&t<=-.05555?this.character=" ⁣":t>=-.1667&&t<=-.1666?this.character=" ⁣":t>=-.2223&&t<=-.2222?this.character=" ⁣":t>=-.2778&&t<=-.2777?this.character=" ⁣":this.character=null}toNode(){if(this.character)return document.createTextNode(this.character);var t=document.createElementNS("http://www.w3.org/1998/Math/MathML","mspace");return t.setAttribute("width",Le(this.width)),t}toMarkup(){return this.character?"<mtext>"+this.character+"</mtext>":'<mspace width="'+Le(this.width)+'"/>'}toText(){return this.character?this.character:" "}}var Ce={MathNode:va,TextNode:fi,SpaceNode:Dj,newDocumentFragment:kx},Fa=function(t,n,r){return fn[n][t]&&fn[n][t].replace&&t.charCodeAt(0)!==55349&&!(_x.hasOwnProperty(t)&&r&&(r.fontFamily&&r.fontFamily.slice(4,6)==="tt"||r.font&&r.font.slice(4,6)==="tt"))&&(t=fn[n][t].replace),new Ce.TextNode(t)},q2=function(t){return t.length===1?t[0]:new Ce.MathNode("mrow",t)},j2=function(t,n){if(n.fontFamily==="texttt")return"monospace";if(n.fontFamily==="textsf")return n.fontShape==="textit"&&n.fontWeight==="textbf"?"sans-serif-bold-italic":n.fontShape==="textit"?"sans-serif-italic":n.fontWeight==="textbf"?"bold-sans-serif":"sans-serif";if(n.fontShape==="textit"&&n.fontWeight==="textbf")return"bold-italic";if(n.fontShape==="textit")return"italic";if(n.fontWeight==="textbf")return"bold";var r=n.font;if(!r||r==="mathnormal")return null;var a=t.mode;if(r==="mathit")return"italic";if(r==="boldsymbol")return t.type==="textord"?"bold":"bold-italic";if(r==="mathbf")return"bold";if(r==="mathbb")return"double-struck";if(r==="mathsfit")return"sans-serif-italic";if(r==="mathfrak")return"fraktur";if(r==="mathscr"||r==="mathcal")return"script";if(r==="mathsf")return"sans-serif";if(r==="mathtt")return"monospace";var s=t.text;if(it.contains(["\\imath","\\jmath"],s))return null;fn[a][s]&&fn[a][s].replace&&(s=fn[a][s].replace);var o=ae.fontMap[r].fontName;return F2(s,o,a)?ae.fontMap[r].variant:null};function lg(e){if(!e)return!1;if(e.type==="mi"&&e.children.length===1){var t=e.children[0];return t instanceof fi&&t.text==="."}else if(e.type==="mo"&&e.children.length===1&&e.getAttribute("separator")==="true"&&e.getAttribute("lspace")==="0em"&&e.getAttribute("rspace")==="0em"){var n=e.children[0];return n instanceof fi&&n.text===","}else return!1}var aa=function(t,n,r){if(t.length===1){var a=sn(t[0],n);return r&&a instanceof va&&a.type==="mo"&&(a.setAttribute("lspace","0em"),a.setAttribute("rspace","0em")),[a]}for(var s=[],o,c=0;c<t.length;c++){var d=sn(t[c],n);if(d instanceof va&&o instanceof va){if(d.type==="mtext"&&o.type==="mtext"&&d.getAttribute("mathvariant")===o.getAttribute("mathvariant")){o.children.push(...d.children);continue}else if(d.type==="mn"&&o.type==="mn"){o.children.push(...d.children);continue}else if(lg(d)&&o.type==="mn"){o.children.push(...d.children);continue}else if(d.type==="mn"&&lg(o))d.children=[...o.children,...d.children],s.pop();else if((d.type==="msup"||d.type==="msub")&&d.children.length>=1&&(o.type==="mn"||lg(o))){var f=d.children[0];f instanceof va&&f.type==="mn"&&(f.children=[...o.children,...f.children],s.pop())}else if(o.type==="mi"&&o.children.length===1){var m=o.children[0];if(m instanceof fi&&m.text==="̸"&&(d.type==="mo"||d.type==="mi"||d.type==="mn")){var p=d.children[0];p instanceof fi&&p.text.length>0&&(p.text=p.text.slice(0,1)+"̸"+p.text.slice(1),s.pop())}}}s.push(d),o=d}return s},ll=function(t,n,r){return q2(aa(t,n,r))},sn=function(t,n){if(!t)return new Ce.MathNode("mrow");if(hf[t.type]){var r=hf[t.type](t,n);return r}else throw new Ne("Got group of unknown type: '"+t.type+"'")};function c6(e,t,n,r,a){var s=aa(e,n),o;s.length===1&&s[0]instanceof va&&it.contains(["mrow","mtable"],s[0].type)?o=s[0]:o=new Ce.MathNode("mrow",s);var c=new Ce.MathNode("annotation",[new Ce.TextNode(t)]);c.setAttribute("encoding","application/x-tex");var d=new Ce.MathNode("semantics",[o,c]),f=new Ce.MathNode("math",[d]);f.setAttribute("xmlns","http://www.w3.org/1998/Math/MathML"),r&&f.setAttribute("display","block");var m=a?"katex":"katex-mathml";return ae.makeSpan([m],[f])}var Cx=function(t){return new Ki({style:t.displayMode?lt.DISPLAY:lt.TEXT,maxSize:t.maxSize,minRuleThickness:t.minRuleThickness})},Nx=function(t,n){if(n.displayMode){var r=["katex-display"];n.leqno&&r.push("leqno"),n.fleqn&&r.push("fleqn"),t=ae.makeSpan(r,[t])}return t},Lj=function(t,n,r){var a=Cx(r),s;if(r.output==="mathml")return c6(t,n,a,r.displayMode,!0);if(r.output==="html"){var o=Eb(t,a);s=ae.makeSpan(["katex"],[o])}else{var c=c6(t,n,a,r.displayMode,!1),d=Eb(t,a);s=ae.makeSpan(["katex"],[c,d])}return Nx(s,r)},Pj=function(t,n,r){var a=Cx(r),s=Eb(t,a),o=ae.makeSpan(["katex"],[s]);return Nx(o,r)},Bj={widehat:"^",widecheck:"ˇ",widetilde:"~",utilde:"~",overleftarrow:"←",underleftarrow:"←",xleftarrow:"←",overrightarrow:"→",underrightarrow:"→",xrightarrow:"→",underbrace:"⏟",overbrace:"⏞",overgroup:"⏠",undergroup:"⏡",overleftrightarrow:"↔",underleftrightarrow:"↔",xleftrightarrow:"↔",Overrightarrow:"⇒",xRightarrow:"⇒",overleftharpoon:"↼",xleftharpoonup:"↼",overrightharpoon:"⇀",xrightharpoonup:"⇀",xLeftarrow:"⇐",xLeftrightarrow:"⇔",xhookleftarrow:"↩",xhookrightarrow:"↪",xmapsto:"↦",xrightharpoondown:"⇁",xleftharpoondown:"↽",xrightleftharpoons:"⇌",xleftrightharpoons:"⇋",xtwoheadleftarrow:"↞",xtwoheadrightarrow:"↠",xlongequal:"=",xtofrom:"⇄",xrightleftarrows:"⇄",xrightequilibrium:"⇌",xleftequilibrium:"⇋","\\cdrightarrow":"→","\\cdleftarrow":"←","\\cdlongequal":"="},Uj=function(t){var n=new Ce.MathNode("mo",[new Ce.TextNode(Bj[t.replace(/^\\/,"")])]);return n.setAttribute("stretchy","true"),n},Fj={overrightarrow:[["rightarrow"],.888,522,"xMaxYMin"],overleftarrow:[["leftarrow"],.888,522,"xMinYMin"],underrightarrow:[["rightarrow"],.888,522,"xMaxYMin"],underleftarrow:[["leftarrow"],.888,522,"xMinYMin"],xrightarrow:[["rightarrow"],1.469,522,"xMaxYMin"],"\\cdrightarrow":[["rightarrow"],3,522,"xMaxYMin"],xleftarrow:[["leftarrow"],1.469,522,"xMinYMin"],"\\cdleftarrow":[["leftarrow"],3,522,"xMinYMin"],Overrightarrow:[["doublerightarrow"],.888,560,"xMaxYMin"],xRightarrow:[["doublerightarrow"],1.526,560,"xMaxYMin"],xLeftarrow:[["doubleleftarrow"],1.526,560,"xMinYMin"],overleftharpoon:[["leftharpoon"],.888,522,"xMinYMin"],xleftharpoonup:[["leftharpoon"],.888,522,"xMinYMin"],xleftharpoondown:[["leftharpoondown"],.888,522,"xMinYMin"],overrightharpoon:[["rightharpoon"],.888,522,"xMaxYMin"],xrightharpoonup:[["rightharpoon"],.888,522,"xMaxYMin"],xrightharpoondown:[["rightharpoondown"],.888,522,"xMaxYMin"],xlongequal:[["longequal"],.888,334,"xMinYMin"],"\\cdlongequal":[["longequal"],3,334,"xMinYMin"],xtwoheadleftarrow:[["twoheadleftarrow"],.888,334,"xMinYMin"],xtwoheadrightarrow:[["twoheadrightarrow"],.888,334,"xMaxYMin"],overleftrightarrow:[["leftarrow","rightarrow"],.888,522],overbrace:[["leftbrace","midbrace","rightbrace"],1.6,548],underbrace:[["leftbraceunder","midbraceunder","rightbraceunder"],1.6,548],underleftrightarrow:[["leftarrow","rightarrow"],.888,522],xleftrightarrow:[["leftarrow","rightarrow"],1.75,522],xLeftrightarrow:[["doubleleftarrow","doublerightarrow"],1.75,560],xrightleftharpoons:[["leftharpoondownplus","rightharpoonplus"],1.75,716],xleftrightharpoons:[["leftharpoonplus","rightharpoondownplus"],1.75,716],xhookleftarrow:[["leftarrow","righthook"],1.08,522],xhookrightarrow:[["lefthook","rightarrow"],1.08,522],overlinesegment:[["leftlinesegment","rightlinesegment"],.888,522],underlinesegment:[["leftlinesegment","rightlinesegment"],.888,522],overgroup:[["leftgroup","rightgroup"],.888,342],undergroup:[["leftgroupunder","rightgroupunder"],.888,342],xmapsto:[["leftmapsto","rightarrow"],1.5,522],xtofrom:[["leftToFrom","rightToFrom"],1.75,528],xrightleftarrows:[["baraboveleftarrow","rightarrowabovebar"],1.75,901],xrightequilibrium:[["baraboveshortleftharpoon","rightharpoonaboveshortbar"],1.75,716],xleftequilibrium:[["shortbaraboveleftharpoon","shortrightharpoonabovebar"],1.75,716]},zj=function(t){return t.type==="ordgroup"?t.body.length:1},Hj=function(t,n){function r(){var c=4e5,d=t.label.slice(1);if(it.contains(["widehat","widecheck","widetilde","utilde"],d)){var f=t,m=zj(f.base),p,b,y;if(m>5)d==="widehat"||d==="widecheck"?(p=420,c=2364,y=.42,b=d+"4"):(p=312,c=2340,y=.34,b="tilde4");else{var T=[1,1,2,2,3,3][m];d==="widehat"||d==="widecheck"?(c=[0,1062,2364,2364,2364][T],p=[0,239,300,360,420][T],y=[0,.24,.3,.3,.36,.42][T],b=d+T):(c=[0,600,1033,2339,2340][T],p=[0,260,286,306,312][T],y=[0,.26,.286,.3,.306,.34][T],b="tilde"+T)}var x=new sl(b),C=new as([x],{width:"100%",height:Le(y),viewBox:"0 0 "+c+" "+p,preserveAspectRatio:"none"});return{span:ae.makeSvgSpan([],[C],n),minWidth:0,height:y}}else{var w=[],O=Fj[d],[D,Y,H]=O,B=H/1e3,ee=D.length,Z,Q;if(ee===1){var G=O[3];Z=["hide-tail"],Q=[G]}else if(ee===2)Z=["halfarrow-left","halfarrow-right"],Q=["xMinYMin","xMaxYMin"];else if(ee===3)Z=["brace-left","brace-center","brace-right"],Q=["xMinYMin","xMidYMin","xMaxYMin"];else throw new Error(`Correct katexImagesData or update code here to support
                    `+ee+" children.");for(var se=0;se<ee;se++){var oe=new sl(D[se]),ye=new as([oe],{width:"400em",height:Le(B),viewBox:"0 0 "+c+" "+H,preserveAspectRatio:Q[se]+" slice"}),fe=ae.makeSvgSpan([Z[se]],[ye],n);if(ee===1)return{span:fe,minWidth:Y,height:B};fe.style.height=Le(B),w.push(fe)}return{span:ae.makeSpan(["stretchy"],w,n),minWidth:Y,height:B}}}var{span:a,minWidth:s,height:o}=r();return a.height=o,a.style.height=Le(o),s>0&&(a.style.minWidth=Le(s)),a},qj=function(t,n,r,a,s){var o,c=t.height+t.depth+r+a;if(/fbox|color|angl/.test(n)){if(o=ae.makeSpan(["stretchy",n],[],s),n==="fbox"){var d=s.color&&s.getColor();d&&(o.style.borderColor=d)}}else{var f=[];/^[bx]cancel$/.test(n)&&f.push(new vb({x1:"0",y1:"0",x2:"100%",y2:"100%","stroke-width":"0.046em"})),/^x?cancel$/.test(n)&&f.push(new vb({x1:"0",y1:"100%",x2:"100%",y2:"0","stroke-width":"0.046em"}));var m=new as(f,{width:"100%",height:Le(c)});o=ae.makeSvgSpan([],[m],s)}return o.height=c,o.style.height=Le(c),o},ss={encloseSpan:qj,mathMLnode:Uj,svgSpan:Hj};function St(e,t){if(!e||e.type!==t)throw new Error("Expected node of type "+t+", but got "+(e?"node of type "+e.type:String(e)));return e}function $2(e){var t=tm(e);if(!t)throw new Error("Expected node of symbol group type, but got "+(e?"node of type "+e.type:String(e)));return t}function tm(e){return e&&(e.type==="atom"||fj.hasOwnProperty(e.type))?e:null}var Y2=(e,t)=>{var n,r,a;e&&e.type==="supsub"?(r=St(e.base,"accent"),n=r.base,e.base=n,a=dj(Ft(e,t)),e.base=r):(r=St(e,"accent"),n=r.base);var s=Ft(n,t.havingCrampedStyle()),o=r.isShifty&&it.isCharacterBox(n),c=0;if(o){var d=it.getBaseElem(n),f=Ft(d,t.havingCrampedStyle());c=r6(f).skew}var m=r.label==="\\c",p=m?s.height+s.depth:Math.min(s.height,t.fontMetrics().xHeight),b;if(r.isStretchy)b=ss.svgSpan(r,t),b=ae.makeVList({positionType:"firstBaseline",children:[{type:"elem",elem:s},{type:"elem",elem:b,wrapperClasses:["svg-align"],wrapperStyle:c>0?{width:"calc(100% - "+Le(2*c)+")",marginLeft:Le(2*c)}:void 0}]},t);else{var y,T;r.label==="\\vec"?(y=ae.staticSvg("vec",t),T=ae.svgData.vec[1]):(y=ae.makeOrd({mode:r.mode,text:r.label},t,"textord"),y=r6(y),y.italic=0,T=y.width,m&&(p+=y.depth)),b=ae.makeSpan(["accent-body"],[y]);var x=r.label==="\\textcircled";x&&(b.classes.push("accent-full"),p=s.height);var C=c;x||(C-=T/2),b.style.left=Le(C),r.label==="\\textcircled"&&(b.style.top=".2em"),b=ae.makeVList({positionType:"firstBaseline",children:[{type:"elem",elem:s},{type:"kern",size:-p},{type:"elem",elem:b}]},t)}var w=ae.makeSpan(["mord","accent"],[b],t);return a?(a.children[0]=w,a.height=Math.max(w.height,a.height),a.classes[0]="mord",a):w},Ox=(e,t)=>{var n=e.isStretchy?ss.mathMLnode(e.label):new Ce.MathNode("mo",[Fa(e.label,e.mode)]),r=new Ce.MathNode("mover",[sn(e.base,t),n]);return r.setAttribute("accent","true"),r},jj=new RegExp(["\\acute","\\grave","\\ddot","\\tilde","\\bar","\\breve","\\check","\\hat","\\vec","\\dot","\\mathring"].map(e=>"\\"+e).join("|"));Ye({type:"accent",names:["\\acute","\\grave","\\ddot","\\tilde","\\bar","\\breve","\\check","\\hat","\\vec","\\dot","\\mathring","\\widecheck","\\widehat","\\widetilde","\\overrightarrow","\\overleftarrow","\\Overrightarrow","\\overleftrightarrow","\\overgroup","\\overlinesegment","\\overleftharpoon","\\overrightharpoon"],props:{numArgs:1},handler:(e,t)=>{var n=ff(t[0]),r=!jj.test(e.funcName),a=!r||e.funcName==="\\widehat"||e.funcName==="\\widetilde"||e.funcName==="\\widecheck";return{type:"accent",mode:e.parser.mode,label:e.funcName,isStretchy:r,isShifty:a,base:n}},htmlBuilder:Y2,mathmlBuilder:Ox});Ye({type:"accent",names:["\\'","\\`","\\^","\\~","\\=","\\u","\\.",'\\"',"\\c","\\r","\\H","\\v","\\textcircled"],props:{numArgs:1,allowedInText:!0,allowedInMath:!0,argTypes:["primitive"]},handler:(e,t)=>{var n=t[0],r=e.parser.mode;return r==="math"&&(e.parser.settings.reportNonstrict("mathVsTextAccents","LaTeX's accent "+e.funcName+" works only in text mode"),r="text"),{type:"accent",mode:r,label:e.funcName,isStretchy:!1,isShifty:!0,base:n}},htmlBuilder:Y2,mathmlBuilder:Ox});Ye({type:"accentUnder",names:["\\underleftarrow","\\underrightarrow","\\underleftrightarrow","\\undergroup","\\underlinesegment","\\utilde"],props:{numArgs:1},handler:(e,t)=>{var{parser:n,funcName:r}=e,a=t[0];return{type:"accentUnder",mode:n.mode,label:r,base:a}},htmlBuilder:(e,t)=>{var n=Ft(e.base,t),r=ss.svgSpan(e,t),a=e.label==="\\utilde"?.12:0,s=ae.makeVList({positionType:"top",positionData:n.height,children:[{type:"elem",elem:r,wrapperClasses:["svg-align"]},{type:"kern",size:a},{type:"elem",elem:n}]},t);return ae.makeSpan(["mord","accentunder"],[s],t)},mathmlBuilder:(e,t)=>{var n=ss.mathMLnode(e.label),r=new Ce.MathNode("munder",[sn(e.base,t),n]);return r.setAttribute("accentunder","true"),r}});var xh=e=>{var t=new Ce.MathNode("mpadded",e?[e]:[]);return t.setAttribute("width","+0.6em"),t.setAttribute("lspace","0.3em"),t};Ye({type:"xArrow",names:["\\xleftarrow","\\xrightarrow","\\xLeftarrow","\\xRightarrow","\\xleftrightarrow","\\xLeftrightarrow","\\xhookleftarrow","\\xhookrightarrow","\\xmapsto","\\xrightharpoondown","\\xrightharpoonup","\\xleftharpoondown","\\xleftharpoonup","\\xrightleftharpoons","\\xleftrightharpoons","\\xlongequal","\\xtwoheadrightarrow","\\xtwoheadleftarrow","\\xtofrom","\\xrightleftarrows","\\xrightequilibrium","\\xleftequilibrium","\\\\cdrightarrow","\\\\cdleftarrow","\\\\cdlongequal"],props:{numArgs:1,numOptionalArgs:1},handler(e,t,n){var{parser:r,funcName:a}=e;return{type:"xArrow",mode:r.mode,label:a,body:t[0],below:n[0]}},htmlBuilder(e,t){var n=t.style,r=t.havingStyle(n.sup()),a=ae.wrapFragment(Ft(e.body,r,t),t),s=e.label.slice(0,2)==="\\x"?"x":"cd";a.classes.push(s+"-arrow-pad");var o;e.below&&(r=t.havingStyle(n.sub()),o=ae.wrapFragment(Ft(e.below,r,t),t),o.classes.push(s+"-arrow-pad"));var c=ss.svgSpan(e,t),d=-t.fontMetrics().axisHeight+.5*c.height,f=-t.fontMetrics().axisHeight-.5*c.height-.111;(a.depth>.25||e.label==="\\xleftequilibrium")&&(f-=a.depth);var m;if(o){var p=-t.fontMetrics().axisHeight+o.height+.5*c.height+.111;m=ae.makeVList({positionType:"individualShift",children:[{type:"elem",elem:a,shift:f},{type:"elem",elem:c,shift:d},{type:"elem",elem:o,shift:p}]},t)}else m=ae.makeVList({positionType:"individualShift",children:[{type:"elem",elem:a,shift:f},{type:"elem",elem:c,shift:d}]},t);return m.children[0].children[0].children[1].classes.push("svg-align"),ae.makeSpan(["mrel","x-arrow"],[m],t)},mathmlBuilder(e,t){var n=ss.mathMLnode(e.label);n.setAttribute("minsize",e.label.charAt(0)==="x"?"1.75em":"3.0em");var r;if(e.body){var a=xh(sn(e.body,t));if(e.below){var s=xh(sn(e.below,t));r=new Ce.MathNode("munderover",[n,s,a])}else r=new Ce.MathNode("mover",[n,a])}else if(e.below){var o=xh(sn(e.below,t));r=new Ce.MathNode("munder",[n,o])}else r=xh(),r=new Ce.MathNode("mover",[n,r]);return r}});var $j=ae.makeSpan;function Rx(e,t){var n=sr(e.body,t,!0);return $j([e.mclass],n,t)}function Mx(e,t){var n,r=aa(e.body,t);return e.mclass==="minner"?n=new Ce.MathNode("mpadded",r):e.mclass==="mord"?e.isCharacterBox?(n=r[0],n.type="mi"):n=new Ce.MathNode("mi",r):(e.isCharacterBox?(n=r[0],n.type="mo"):n=new Ce.MathNode("mo",r),e.mclass==="mbin"?(n.attributes.lspace="0.22em",n.attributes.rspace="0.22em"):e.mclass==="mpunct"?(n.attributes.lspace="0em",n.attributes.rspace="0.17em"):e.mclass==="mopen"||e.mclass==="mclose"?(n.attributes.lspace="0em",n.attributes.rspace="0em"):e.mclass==="minner"&&(n.attributes.lspace="0.0556em",n.attributes.width="+0.1111em")),n}Ye({type:"mclass",names:["\\mathord","\\mathbin","\\mathrel","\\mathopen","\\mathclose","\\mathpunct","\\mathinner"],props:{numArgs:1,primitive:!0},handler(e,t){var{parser:n,funcName:r}=e,a=t[0];return{type:"mclass",mode:n.mode,mclass:"m"+r.slice(5),body:Hn(a),isCharacterBox:it.isCharacterBox(a)}},htmlBuilder:Rx,mathmlBuilder:Mx});var nm=e=>{var t=e.type==="ordgroup"&&e.body.length?e.body[0]:e;return t.type==="atom"&&(t.family==="bin"||t.family==="rel")?"m"+t.family:"mord"};Ye({type:"mclass",names:["\\@binrel"],props:{numArgs:2},handler(e,t){var{parser:n}=e;return{type:"mclass",mode:n.mode,mclass:nm(t[0]),body:Hn(t[1]),isCharacterBox:it.isCharacterBox(t[1])}}});Ye({type:"mclass",names:["\\stackrel","\\overset","\\underset"],props:{numArgs:2},handler(e,t){var{parser:n,funcName:r}=e,a=t[1],s=t[0],o;r!=="\\stackrel"?o=nm(a):o="mrel";var c={type:"op",mode:a.mode,limits:!0,alwaysHandleSupSub:!0,parentIsSupSub:!1,symbol:!1,suppressBaseShift:r!=="\\stackrel",body:Hn(a)},d={type:"supsub",mode:s.mode,base:c,sup:r==="\\underset"?null:s,sub:r==="\\underset"?s:null};return{type:"mclass",mode:n.mode,mclass:o,body:[d],isCharacterBox:it.isCharacterBox(d)}},htmlBuilder:Rx,mathmlBuilder:Mx});Ye({type:"pmb",names:["\\pmb"],props:{numArgs:1,allowedInText:!0},handler(e,t){var{parser:n}=e;return{type:"pmb",mode:n.mode,mclass:nm(t[0]),body:Hn(t[0])}},htmlBuilder(e,t){var n=sr(e.body,t,!0),r=ae.makeSpan([e.mclass],n,t);return r.style.textShadow="0.02em 0.01em 0.04px",r},mathmlBuilder(e,t){var n=aa(e.body,t),r=new Ce.MathNode("mstyle",n);return r.setAttribute("style","text-shadow: 0.02em 0.01em 0.04px"),r}});var Yj={">":"\\\\cdrightarrow","<":"\\\\cdleftarrow","=":"\\\\cdlongequal",A:"\\uparrow",V:"\\downarrow","|":"\\Vert",".":"no arrow"},d6=()=>({type:"styling",body:[],mode:"math",style:"display"}),h6=e=>e.type==="textord"&&e.text==="@",Gj=(e,t)=>(e.type==="mathord"||e.type==="atom")&&e.text===t;function Vj(e,t,n){var r=Yj[e];switch(r){case"\\\\cdrightarrow":case"\\\\cdleftarrow":return n.callFunction(r,[t[0]],[t[1]]);case"\\uparrow":case"\\downarrow":{var a=n.callFunction("\\\\cdleft",[t[0]],[]),s={type:"atom",text:r,mode:"math",family:"rel"},o=n.callFunction("\\Big",[s],[]),c=n.callFunction("\\\\cdright",[t[1]],[]),d={type:"ordgroup",mode:"math",body:[a,o,c]};return n.callFunction("\\\\cdparent",[d],[])}case"\\\\cdlongequal":return n.callFunction("\\\\cdlongequal",[],[]);case"\\Vert":{var f={type:"textord",text:"\\Vert",mode:"math"};return n.callFunction("\\Big",[f],[])}default:return{type:"textord",text:" ",mode:"math"}}}function Kj(e){var t=[];for(e.gullet.beginGroup(),e.gullet.macros.set("\\cr","\\\\\\relax"),e.gullet.beginGroup();;){t.push(e.parseExpression(!1,"\\\\")),e.gullet.endGroup(),e.gullet.beginGroup();var n=e.fetch().text;if(n==="&"||n==="\\\\")e.consume();else if(n==="\\end"){t[t.length-1].length===0&&t.pop();break}else throw new Ne("Expected \\\\ or \\cr or \\end",e.nextToken)}for(var r=[],a=[r],s=0;s<t.length;s++){for(var o=t[s],c=d6(),d=0;d<o.length;d++)if(!h6(o[d]))c.body.push(o[d]);else{r.push(c),d+=1;var f=$2(o[d]).text,m=new Array(2);if(m[0]={type:"ordgroup",mode:"math",body:[]},m[1]={type:"ordgroup",mode:"math",body:[]},!("=|.".indexOf(f)>-1))if("<>AV".indexOf(f)>-1)for(var p=0;p<2;p++){for(var b=!0,y=d+1;y<o.length;y++){if(Gj(o[y],f)){b=!1,d=y;break}if(h6(o[y]))throw new Ne("Missing a "+f+" character to complete a CD arrow.",o[y]);m[p].body.push(o[y])}if(b)throw new Ne("Missing a "+f+" character to complete a CD arrow.",o[d])}else throw new Ne('Expected one of "<>AV=|." after @',o[d]);var T=Vj(f,m,e),x={type:"styling",body:[T],mode:"math",style:"display"};r.push(x),c=d6()}s%2===0?r.push(c):r.shift(),r=[],a.push(r)}e.gullet.endGroup(),e.gullet.endGroup();var C=new Array(a[0].length).fill({type:"align",align:"c",pregap:.25,postgap:.25});return{type:"array",mode:"math",body:a,arraystretch:1,addJot:!0,rowGaps:[null],cols:C,colSeparationType:"CD",hLinesBeforeRow:new Array(a.length+1).fill([])}}Ye({type:"cdlabel",names:["\\\\cdleft","\\\\cdright"],props:{numArgs:1},handler(e,t){var{parser:n,funcName:r}=e;return{type:"cdlabel",mode:n.mode,side:r.slice(4),label:t[0]}},htmlBuilder(e,t){var n=t.havingStyle(t.style.sup()),r=ae.wrapFragment(Ft(e.label,n,t),t);return r.classes.push("cd-label-"+e.side),r.style.bottom=Le(.8-r.depth),r.height=0,r.depth=0,r},mathmlBuilder(e,t){var n=new Ce.MathNode("mrow",[sn(e.label,t)]);return n=new Ce.MathNode("mpadded",[n]),n.setAttribute("width","0"),e.side==="left"&&n.setAttribute("lspace","-1width"),n.setAttribute("voffset","0.7em"),n=new Ce.MathNode("mstyle",[n]),n.setAttribute("displaystyle","false"),n.setAttribute("scriptlevel","1"),n}});Ye({type:"cdlabelparent",names:["\\\\cdparent"],props:{numArgs:1},handler(e,t){var{parser:n}=e;return{type:"cdlabelparent",mode:n.mode,fragment:t[0]}},htmlBuilder(e,t){var n=ae.wrapFragment(Ft(e.fragment,t),t);return n.classes.push("cd-vert-arrow"),n},mathmlBuilder(e,t){return new Ce.MathNode("mrow",[sn(e.fragment,t)])}});Ye({type:"textord",names:["\\@char"],props:{numArgs:1,allowedInText:!0},handler(e,t){for(var{parser:n}=e,r=St(t[0],"ordgroup"),a=r.body,s="",o=0;o<a.length;o++){var c=St(a[o],"textord");s+=c.text}var d=parseInt(s),f;if(isNaN(d))throw new Ne("\\@char has non-numeric argument "+s);if(d<0||d>=1114111)throw new Ne("\\@char with invalid code point "+s);return d<=65535?f=String.fromCharCode(d):(d-=65536,f=String.fromCharCode((d>>10)+55296,(d&1023)+56320)),{type:"textord",mode:n.mode,text:f}}});var Ix=(e,t)=>{var n=sr(e.body,t.withColor(e.color),!1);return ae.makeFragment(n)},Dx=(e,t)=>{var n=aa(e.body,t.withColor(e.color)),r=new Ce.MathNode("mstyle",n);return r.setAttribute("mathcolor",e.color),r};Ye({type:"color",names:["\\textcolor"],props:{numArgs:2,allowedInText:!0,argTypes:["color","original"]},handler(e,t){var{parser:n}=e,r=St(t[0],"color-token").color,a=t[1];return{type:"color",mode:n.mode,color:r,body:Hn(a)}},htmlBuilder:Ix,mathmlBuilder:Dx});Ye({type:"color",names:["\\color"],props:{numArgs:1,allowedInText:!0,argTypes:["color"]},handler(e,t){var{parser:n,breakOnTokenText:r}=e,a=St(t[0],"color-token").color;n.gullet.macros.set("\\current@color",a);var s=n.parseExpression(!0,r);return{type:"color",mode:n.mode,color:a,body:s}},htmlBuilder:Ix,mathmlBuilder:Dx});Ye({type:"cr",names:["\\\\"],props:{numArgs:0,numOptionalArgs:0,allowedInText:!0},handler(e,t,n){var{parser:r}=e,a=r.gullet.future().text==="["?r.parseSizeGroup(!0):null,s=!r.settings.displayMode||!r.settings.useStrictBehavior("newLineInDisplayMode","In LaTeX, \\\\ or \\newline does nothing in display mode");return{type:"cr",mode:r.mode,newLine:s,size:a&&St(a,"size").value}},htmlBuilder(e,t){var n=ae.makeSpan(["mspace"],[],t);return e.newLine&&(n.classes.push("newline"),e.size&&(n.style.marginTop=Le(Rn(e.size,t)))),n},mathmlBuilder(e,t){var n=new Ce.MathNode("mspace");return e.newLine&&(n.setAttribute("linebreak","newline"),e.size&&n.setAttribute("height",Le(Rn(e.size,t)))),n}});var Tb={"\\global":"\\global","\\long":"\\\\globallong","\\\\globallong":"\\\\globallong","\\def":"\\gdef","\\gdef":"\\gdef","\\edef":"\\xdef","\\xdef":"\\xdef","\\let":"\\\\globallet","\\futurelet":"\\\\globalfuture"},Lx=e=>{var t=e.text;if(/^(?:[\\{}$&#^_]|EOF)$/.test(t))throw new Ne("Expected a control sequence",e);return t},Wj=e=>{var t=e.gullet.popToken();return t.text==="="&&(t=e.gullet.popToken(),t.text===" "&&(t=e.gullet.popToken())),t},Px=(e,t,n,r)=>{var a=e.gullet.macros.get(n.text);a==null&&(n.noexpand=!0,a={tokens:[n],numArgs:0,unexpandable:!e.gullet.isExpandable(n.text)}),e.gullet.macros.set(t,a,r)};Ye({type:"internal",names:["\\global","\\long","\\\\globallong"],props:{numArgs:0,allowedInText:!0},handler(e){var{parser:t,funcName:n}=e;t.consumeSpaces();var r=t.fetch();if(Tb[r.text])return(n==="\\global"||n==="\\\\globallong")&&(r.text=Tb[r.text]),St(t.parseFunction(),"internal");throw new Ne("Invalid token after macro prefix",r)}});Ye({type:"internal",names:["\\def","\\gdef","\\edef","\\xdef"],props:{numArgs:0,allowedInText:!0,primitive:!0},handler(e){var{parser:t,funcName:n}=e,r=t.gullet.popToken(),a=r.text;if(/^(?:[\\{}$&#^_]|EOF)$/.test(a))throw new Ne("Expected a control sequence",r);for(var s=0,o,c=[[]];t.gullet.future().text!=="{";)if(r=t.gullet.popToken(),r.text==="#"){if(t.gullet.future().text==="{"){o=t.gullet.future(),c[s].push("{");break}if(r=t.gullet.popToken(),!/^[1-9]$/.test(r.text))throw new Ne('Invalid argument number "'+r.text+'"');if(parseInt(r.text)!==s+1)throw new Ne('Argument number "'+r.text+'" out of order');s++,c.push([])}else{if(r.text==="EOF")throw new Ne("Expected a macro definition");c[s].push(r.text)}var{tokens:d}=t.gullet.consumeArg();return o&&d.unshift(o),(n==="\\edef"||n==="\\xdef")&&(d=t.gullet.expandTokens(d),d.reverse()),t.gullet.macros.set(a,{tokens:d,numArgs:s,delimiters:c},n===Tb[n]),{type:"internal",mode:t.mode}}});Ye({type:"internal",names:["\\let","\\\\globallet"],props:{numArgs:0,allowedInText:!0,primitive:!0},handler(e){var{parser:t,funcName:n}=e,r=Lx(t.gullet.popToken());t.gullet.consumeSpaces();var a=Wj(t);return Px(t,r,a,n==="\\\\globallet"),{type:"internal",mode:t.mode}}});Ye({type:"internal",names:["\\futurelet","\\\\globalfuture"],props:{numArgs:0,allowedInText:!0,primitive:!0},handler(e){var{parser:t,funcName:n}=e,r=Lx(t.gullet.popToken()),a=t.gullet.popToken(),s=t.gullet.popToken();return Px(t,r,s,n==="\\\\globalfuture"),t.gullet.pushToken(s),t.gullet.pushToken(a),{type:"internal",mode:t.mode}}});var qc=function(t,n,r){var a=fn.math[t]&&fn.math[t].replace,s=F2(a||t,n,r);if(!s)throw new Error("Unsupported symbol "+t+" and font size "+n+".");return s},G2=function(t,n,r,a){var s=r.havingBaseStyle(n),o=ae.makeSpan(a.concat(s.sizingClasses(r)),[t],r),c=s.sizeMultiplier/r.sizeMultiplier;return o.height*=c,o.depth*=c,o.maxFontSize=s.sizeMultiplier,o},Bx=function(t,n,r){var a=n.havingBaseStyle(r),s=(1-n.sizeMultiplier/a.sizeMultiplier)*n.fontMetrics().axisHeight;t.classes.push("delimcenter"),t.style.top=Le(s),t.height-=s,t.depth+=s},Xj=function(t,n,r,a,s,o){var c=ae.makeSymbol(t,"Main-Regular",s,a),d=G2(c,n,a,o);return r&&Bx(d,a,n),d},Qj=function(t,n,r,a){return ae.makeSymbol(t,"Size"+n+"-Regular",r,a)},Ux=function(t,n,r,a,s,o){var c=Qj(t,n,s,a),d=G2(ae.makeSpan(["delimsizing","size"+n],[c],a),lt.TEXT,a,o);return r&&Bx(d,a,lt.TEXT),d},og=function(t,n,r){var a;n==="Size1-Regular"?a="delim-size1":a="delim-size4";var s=ae.makeSpan(["delimsizinginner",a],[ae.makeSpan([],[ae.makeSymbol(t,n,r)])]);return{type:"elem",elem:s}},ug=function(t,n,r){var a=hi["Size4-Regular"][t.charCodeAt(0)]?hi["Size4-Regular"][t.charCodeAt(0)][4]:hi["Size1-Regular"][t.charCodeAt(0)][4],s=new sl("inner",nj(t,Math.round(1e3*n))),o=new as([s],{width:Le(a),height:Le(n),style:"width:"+Le(a),viewBox:"0 0 "+1e3*a+" "+Math.round(1e3*n),preserveAspectRatio:"xMinYMin"}),c=ae.makeSvgSpan([],[o],r);return c.height=n,c.style.height=Le(n),c.style.width=Le(a),{type:"elem",elem:c}},xb=.008,Sh={type:"kern",size:-1*xb},Zj=["|","\\lvert","\\rvert","\\vert"],Jj=["\\|","\\lVert","\\rVert","\\Vert"],Fx=function(t,n,r,a,s,o){var c,d,f,m,p="",b=0;c=f=m=t,d=null;var y="Size1-Regular";t==="\\uparrow"?f=m="⏐":t==="\\Uparrow"?f=m="‖":t==="\\downarrow"?c=f="⏐":t==="\\Downarrow"?c=f="‖":t==="\\updownarrow"?(c="\\uparrow",f="⏐",m="\\downarrow"):t==="\\Updownarrow"?(c="\\Uparrow",f="‖",m="\\Downarrow"):it.contains(Zj,t)?(f="∣",p="vert",b=333):it.contains(Jj,t)?(f="∥",p="doublevert",b=556):t==="["||t==="\\lbrack"?(c="⎡",f="⎢",m="⎣",y="Size4-Regular",p="lbrack",b=667):t==="]"||t==="\\rbrack"?(c="⎤",f="⎥",m="⎦",y="Size4-Regular",p="rbrack",b=667):t==="\\lfloor"||t==="⌊"?(f=c="⎢",m="⎣",y="Size4-Regular",p="lfloor",b=667):t==="\\lceil"||t==="⌈"?(c="⎡",f=m="⎢",y="Size4-Regular",p="lceil",b=667):t==="\\rfloor"||t==="⌋"?(f=c="⎥",m="⎦",y="Size4-Regular",p="rfloor",b=667):t==="\\rceil"||t==="⌉"?(c="⎤",f=m="⎥",y="Size4-Regular",p="rceil",b=667):t==="("||t==="\\lparen"?(c="⎛",f="⎜",m="⎝",y="Size4-Regular",p="lparen",b=875):t===")"||t==="\\rparen"?(c="⎞",f="⎟",m="⎠",y="Size4-Regular",p="rparen",b=875):t==="\\{"||t==="\\lbrace"?(c="⎧",d="⎨",m="⎩",f="⎪",y="Size4-Regular"):t==="\\}"||t==="\\rbrace"?(c="⎫",d="⎬",m="⎭",f="⎪",y="Size4-Regular"):t==="\\lgroup"||t==="⟮"?(c="⎧",m="⎩",f="⎪",y="Size4-Regular"):t==="\\rgroup"||t==="⟯"?(c="⎫",m="⎭",f="⎪",y="Size4-Regular"):t==="\\lmoustache"||t==="⎰"?(c="⎧",m="⎭",f="⎪",y="Size4-Regular"):(t==="\\rmoustache"||t==="⎱")&&(c="⎫",m="⎩",f="⎪",y="Size4-Regular");var T=qc(c,y,s),x=T.height+T.depth,C=qc(f,y,s),w=C.height+C.depth,O=qc(m,y,s),D=O.height+O.depth,Y=0,H=1;if(d!==null){var B=qc(d,y,s);Y=B.height+B.depth,H=2}var ee=x+D+Y,Z=Math.max(0,Math.ceil((n-ee)/(H*w))),Q=ee+Z*H*w,G=a.fontMetrics().axisHeight;r&&(G*=a.sizeMultiplier);var se=Q/2-G,oe=[];if(p.length>0){var ye=Q-x-D,fe=Math.round(Q*1e3),de=rj(p,Math.round(ye*1e3)),V=new sl(p,de),me=(b/1e3).toFixed(3)+"em",pe=(fe/1e3).toFixed(3)+"em",Be=new as([V],{width:me,height:pe,viewBox:"0 0 "+b+" "+fe}),k=ae.makeSvgSpan([],[Be],a);k.height=fe/1e3,k.style.width=me,k.style.height=pe,oe.push({type:"elem",elem:k})}else{if(oe.push(og(m,y,s)),oe.push(Sh),d===null){var ie=Q-x-D+2*xb;oe.push(ug(f,ie,a))}else{var ge=(Q-x-D-Y)/2+2*xb;oe.push(ug(f,ge,a)),oe.push(Sh),oe.push(og(d,y,s)),oe.push(Sh),oe.push(ug(f,ge,a))}oe.push(Sh),oe.push(og(c,y,s))}var L=a.havingBaseStyle(lt.TEXT),Oe=ae.makeVList({positionType:"bottom",positionData:se,children:oe},L);return G2(ae.makeSpan(["delimsizing","mult"],[Oe],L),lt.TEXT,a,o)},cg=80,dg=.08,hg=function(t,n,r,a,s){var o=tj(t,a,r),c=new sl(t,o),d=new as([c],{width:"400em",height:Le(n),viewBox:"0 0 400000 "+r,preserveAspectRatio:"xMinYMin slice"});return ae.makeSvgSpan(["hide-tail"],[d],s)},e$=function(t,n){var r=n.havingBaseSizing(),a=jx("\\surd",t*r.sizeMultiplier,qx,r),s=r.sizeMultiplier,o=Math.max(0,n.minRuleThickness-n.fontMetrics().sqrtRuleThickness),c,d=0,f=0,m=0,p;return a.type==="small"?(m=1e3+1e3*o+cg,t<1?s=1:t<1.4&&(s=.7),d=(1+o+dg)/s,f=(1+o)/s,c=hg("sqrtMain",d,m,o,n),c.style.minWidth="0.853em",p=.833/s):a.type==="large"?(m=(1e3+cg)*s0[a.size],f=(s0[a.size]+o)/s,d=(s0[a.size]+o+dg)/s,c=hg("sqrtSize"+a.size,d,m,o,n),c.style.minWidth="1.02em",p=1/s):(d=t+o+dg,f=t+o,m=Math.floor(1e3*t+o)+cg,c=hg("sqrtTall",d,m,o,n),c.style.minWidth="0.742em",p=1.056),c.height=f,c.style.height=Le(d),{span:c,advanceWidth:p,ruleWidth:(n.fontMetrics().sqrtRuleThickness+o)*s}},zx=["(","\\lparen",")","\\rparen","[","\\lbrack","]","\\rbrack","\\{","\\lbrace","\\}","\\rbrace","\\lfloor","\\rfloor","⌊","⌋","\\lceil","\\rceil","⌈","⌉","\\surd"],t$=["\\uparrow","\\downarrow","\\updownarrow","\\Uparrow","\\Downarrow","\\Updownarrow","|","\\|","\\vert","\\Vert","\\lvert","\\rvert","\\lVert","\\rVert","\\lgroup","\\rgroup","⟮","⟯","\\lmoustache","\\rmoustache","⎰","⎱"],Hx=["<",">","\\langle","\\rangle","/","\\backslash","\\lt","\\gt"],s0=[0,1.2,1.8,2.4,3],n$=function(t,n,r,a,s){if(t==="<"||t==="\\lt"||t==="⟨"?t="\\langle":(t===">"||t==="\\gt"||t==="⟩")&&(t="\\rangle"),it.contains(zx,t)||it.contains(Hx,t))return Ux(t,n,!1,r,a,s);if(it.contains(t$,t))return Fx(t,s0[n],!1,r,a,s);throw new Ne("Illegal delimiter: '"+t+"'")},r$=[{type:"small",style:lt.SCRIPTSCRIPT},{type:"small",style:lt.SCRIPT},{type:"small",style:lt.TEXT},{type:"large",size:1},{type:"large",size:2},{type:"large",size:3},{type:"large",size:4}],a$=[{type:"small",style:lt.SCRIPTSCRIPT},{type:"small",style:lt.SCRIPT},{type:"small",style:lt.TEXT},{type:"stack"}],qx=[{type:"small",style:lt.SCRIPTSCRIPT},{type:"small",style:lt.SCRIPT},{type:"small",style:lt.TEXT},{type:"large",size:1},{type:"large",size:2},{type:"large",size:3},{type:"large",size:4},{type:"stack"}],i$=function(t){if(t.type==="small")return"Main-Regular";if(t.type==="large")return"Size"+t.size+"-Regular";if(t.type==="stack")return"Size4-Regular";throw new Error("Add support for delim type '"+t.type+"' here.")},jx=function(t,n,r,a){for(var s=Math.min(2,3-a.style.size),o=s;o<r.length&&r[o].type!=="stack";o++){var c=qc(t,i$(r[o]),"math"),d=c.height+c.depth;if(r[o].type==="small"){var f=a.havingBaseStyle(r[o].style);d*=f.sizeMultiplier}if(d>n)return r[o]}return r[r.length-1]},$x=function(t,n,r,a,s,o){t==="<"||t==="\\lt"||t==="⟨"?t="\\langle":(t===">"||t==="\\gt"||t==="⟩")&&(t="\\rangle");var c;it.contains(Hx,t)?c=r$:it.contains(zx,t)?c=qx:c=a$;var d=jx(t,n,c,a);return d.type==="small"?Xj(t,d.style,r,a,s,o):d.type==="large"?Ux(t,d.size,r,a,s,o):Fx(t,n,r,a,s,o)},s$=function(t,n,r,a,s,o){var c=a.fontMetrics().axisHeight*a.sizeMultiplier,d=901,f=5/a.fontMetrics().ptPerEm,m=Math.max(n-c,r+c),p=Math.max(m/500*d,2*m-f);return $x(t,p,!0,a,s,o)},ts={sqrtImage:e$,sizedDelim:n$,sizeToMaxHeight:s0,customSizedDelim:$x,leftRightDelim:s$},f6={"\\bigl":{mclass:"mopen",size:1},"\\Bigl":{mclass:"mopen",size:2},"\\biggl":{mclass:"mopen",size:3},"\\Biggl":{mclass:"mopen",size:4},"\\bigr":{mclass:"mclose",size:1},"\\Bigr":{mclass:"mclose",size:2},"\\biggr":{mclass:"mclose",size:3},"\\Biggr":{mclass:"mclose",size:4},"\\bigm":{mclass:"mrel",size:1},"\\Bigm":{mclass:"mrel",size:2},"\\biggm":{mclass:"mrel",size:3},"\\Biggm":{mclass:"mrel",size:4},"\\big":{mclass:"mord",size:1},"\\Big":{mclass:"mord",size:2},"\\bigg":{mclass:"mord",size:3},"\\Bigg":{mclass:"mord",size:4}},l$=["(","\\lparen",")","\\rparen","[","\\lbrack","]","\\rbrack","\\{","\\lbrace","\\}","\\rbrace","\\lfloor","\\rfloor","⌊","⌋","\\lceil","\\rceil","⌈","⌉","<",">","\\langle","⟨","\\rangle","⟩","\\lt","\\gt","\\lvert","\\rvert","\\lVert","\\rVert","\\lgroup","\\rgroup","⟮","⟯","\\lmoustache","\\rmoustache","⎰","⎱","/","\\backslash","|","\\vert","\\|","\\Vert","\\uparrow","\\Uparrow","\\downarrow","\\Downarrow","\\updownarrow","\\Updownarrow","."];function rm(e,t){var n=tm(e);if(n&&it.contains(l$,n.text))return n;throw n?new Ne("Invalid delimiter '"+n.text+"' after '"+t.funcName+"'",e):new Ne("Invalid delimiter type '"+e.type+"'",e)}Ye({type:"delimsizing",names:["\\bigl","\\Bigl","\\biggl","\\Biggl","\\bigr","\\Bigr","\\biggr","\\Biggr","\\bigm","\\Bigm","\\biggm","\\Biggm","\\big","\\Big","\\bigg","\\Bigg"],props:{numArgs:1,argTypes:["primitive"]},handler:(e,t)=>{var n=rm(t[0],e);return{type:"delimsizing",mode:e.parser.mode,size:f6[e.funcName].size,mclass:f6[e.funcName].mclass,delim:n.text}},htmlBuilder:(e,t)=>e.delim==="."?ae.makeSpan([e.mclass]):ts.sizedDelim(e.delim,e.size,t,e.mode,[e.mclass]),mathmlBuilder:e=>{var t=[];e.delim!=="."&&t.push(Fa(e.delim,e.mode));var n=new Ce.MathNode("mo",t);e.mclass==="mopen"||e.mclass==="mclose"?n.setAttribute("fence","true"):n.setAttribute("fence","false"),n.setAttribute("stretchy","true");var r=Le(ts.sizeToMaxHeight[e.size]);return n.setAttribute("minsize",r),n.setAttribute("maxsize",r),n}});function m6(e){if(!e.body)throw new Error("Bug: The leftright ParseNode wasn't fully parsed.")}Ye({type:"leftright-right",names:["\\right"],props:{numArgs:1,primitive:!0},handler:(e,t)=>{var n=e.parser.gullet.macros.get("\\current@color");if(n&&typeof n!="string")throw new Ne("\\current@color set to non-string in \\right");return{type:"leftright-right",mode:e.parser.mode,delim:rm(t[0],e).text,color:n}}});Ye({type:"leftright",names:["\\left"],props:{numArgs:1,primitive:!0},handler:(e,t)=>{var n=rm(t[0],e),r=e.parser;++r.leftrightDepth;var a=r.parseExpression(!1);--r.leftrightDepth,r.expect("\\right",!1);var s=St(r.parseFunction(),"leftright-right");return{type:"leftright",mode:r.mode,body:a,left:n.text,right:s.delim,rightColor:s.color}},htmlBuilder:(e,t)=>{m6(e);for(var n=sr(e.body,t,!0,["mopen","mclose"]),r=0,a=0,s=!1,o=0;o<n.length;o++)n[o].isMiddle?s=!0:(r=Math.max(n[o].height,r),a=Math.max(n[o].depth,a));r*=t.sizeMultiplier,a*=t.sizeMultiplier;var c;if(e.left==="."?c=g0(t,["mopen"]):c=ts.leftRightDelim(e.left,r,a,t,e.mode,["mopen"]),n.unshift(c),s)for(var d=1;d<n.length;d++){var f=n[d],m=f.isMiddle;m&&(n[d]=ts.leftRightDelim(m.delim,r,a,m.options,e.mode,[]))}var p;if(e.right===".")p=g0(t,["mclose"]);else{var b=e.rightColor?t.withColor(e.rightColor):t;p=ts.leftRightDelim(e.right,r,a,b,e.mode,["mclose"])}return n.push(p),ae.makeSpan(["minner"],n,t)},mathmlBuilder:(e,t)=>{m6(e);var n=aa(e.body,t);if(e.left!=="."){var r=new Ce.MathNode("mo",[Fa(e.left,e.mode)]);r.setAttribute("fence","true"),n.unshift(r)}if(e.right!=="."){var a=new Ce.MathNode("mo",[Fa(e.right,e.mode)]);a.setAttribute("fence","true"),e.rightColor&&a.setAttribute("mathcolor",e.rightColor),n.push(a)}return q2(n)}});Ye({type:"middle",names:["\\middle"],props:{numArgs:1,primitive:!0},handler:(e,t)=>{var n=rm(t[0],e);if(!e.parser.leftrightDepth)throw new Ne("\\middle without preceding \\left",n);return{type:"middle",mode:e.parser.mode,delim:n.text}},htmlBuilder:(e,t)=>{var n;if(e.delim===".")n=g0(t,[]);else{n=ts.sizedDelim(e.delim,1,t,e.mode,[]);var r={delim:e.delim,options:t};n.isMiddle=r}return n},mathmlBuilder:(e,t)=>{var n=e.delim==="\\vert"||e.delim==="|"?Fa("|","text"):Fa(e.delim,e.mode),r=new Ce.MathNode("mo",[n]);return r.setAttribute("fence","true"),r.setAttribute("lspace","0.05em"),r.setAttribute("rspace","0.05em"),r}});var V2=(e,t)=>{var n=ae.wrapFragment(Ft(e.body,t),t),r=e.label.slice(1),a=t.sizeMultiplier,s,o=0,c=it.isCharacterBox(e.body);if(r==="sout")s=ae.makeSpan(["stretchy","sout"]),s.height=t.fontMetrics().defaultRuleThickness/a,o=-.5*t.fontMetrics().xHeight;else if(r==="phase"){var d=Rn({number:.6,unit:"pt"},t),f=Rn({number:.35,unit:"ex"},t),m=t.havingBaseSizing();a=a/m.sizeMultiplier;var p=n.height+n.depth+d+f;n.style.paddingLeft=Le(p/2+d);var b=Math.floor(1e3*p*a),y=Jq(b),T=new as([new sl("phase",y)],{width:"400em",height:Le(b/1e3),viewBox:"0 0 400000 "+b,preserveAspectRatio:"xMinYMin slice"});s=ae.makeSvgSpan(["hide-tail"],[T],t),s.style.height=Le(p),o=n.depth+d+f}else{/cancel/.test(r)?c||n.classes.push("cancel-pad"):r==="angl"?n.classes.push("anglpad"):n.classes.push("boxpad");var x=0,C=0,w=0;/box/.test(r)?(w=Math.max(t.fontMetrics().fboxrule,t.minRuleThickness),x=t.fontMetrics().fboxsep+(r==="colorbox"?0:w),C=x):r==="angl"?(w=Math.max(t.fontMetrics().defaultRuleThickness,t.minRuleThickness),x=4*w,C=Math.max(0,.25-n.depth)):(x=c?.2:0,C=x),s=ss.encloseSpan(n,r,x,C,t),/fbox|boxed|fcolorbox/.test(r)?(s.style.borderStyle="solid",s.style.borderWidth=Le(w)):r==="angl"&&w!==.049&&(s.style.borderTopWidth=Le(w),s.style.borderRightWidth=Le(w)),o=n.depth+C,e.backgroundColor&&(s.style.backgroundColor=e.backgroundColor,e.borderColor&&(s.style.borderColor=e.borderColor))}var O;if(e.backgroundColor)O=ae.makeVList({positionType:"individualShift",children:[{type:"elem",elem:s,shift:o},{type:"elem",elem:n,shift:0}]},t);else{var D=/cancel|phase/.test(r)?["svg-align"]:[];O=ae.makeVList({positionType:"individualShift",children:[{type:"elem",elem:n,shift:0},{type:"elem",elem:s,shift:o,wrapperClasses:D}]},t)}return/cancel/.test(r)&&(O.height=n.height,O.depth=n.depth),/cancel/.test(r)&&!c?ae.makeSpan(["mord","cancel-lap"],[O],t):ae.makeSpan(["mord"],[O],t)},K2=(e,t)=>{var n=0,r=new Ce.MathNode(e.label.indexOf("colorbox")>-1?"mpadded":"menclose",[sn(e.body,t)]);switch(e.label){case"\\cancel":r.setAttribute("notation","updiagonalstrike");break;case"\\bcancel":r.setAttribute("notation","downdiagonalstrike");break;case"\\phase":r.setAttribute("notation","phasorangle");break;case"\\sout":r.setAttribute("notation","horizontalstrike");break;case"\\fbox":r.setAttribute("notation","box");break;case"\\angl":r.setAttribute("notation","actuarial");break;case"\\fcolorbox":case"\\colorbox":if(n=t.fontMetrics().fboxsep*t.fontMetrics().ptPerEm,r.setAttribute("width","+"+2*n+"pt"),r.setAttribute("height","+"+2*n+"pt"),r.setAttribute("lspace",n+"pt"),r.setAttribute("voffset",n+"pt"),e.label==="\\fcolorbox"){var a=Math.max(t.fontMetrics().fboxrule,t.minRuleThickness);r.setAttribute("style","border: "+a+"em solid "+String(e.borderColor))}break;case"\\xcancel":r.setAttribute("notation","updiagonalstrike downdiagonalstrike");break}return e.backgroundColor&&r.setAttribute("mathbackground",e.backgroundColor),r};Ye({type:"enclose",names:["\\colorbox"],props:{numArgs:2,allowedInText:!0,argTypes:["color","text"]},handler(e,t,n){var{parser:r,funcName:a}=e,s=St(t[0],"color-token").color,o=t[1];return{type:"enclose",mode:r.mode,label:a,backgroundColor:s,body:o}},htmlBuilder:V2,mathmlBuilder:K2});Ye({type:"enclose",names:["\\fcolorbox"],props:{numArgs:3,allowedInText:!0,argTypes:["color","color","text"]},handler(e,t,n){var{parser:r,funcName:a}=e,s=St(t[0],"color-token").color,o=St(t[1],"color-token").color,c=t[2];return{type:"enclose",mode:r.mode,label:a,backgroundColor:o,borderColor:s,body:c}},htmlBuilder:V2,mathmlBuilder:K2});Ye({type:"enclose",names:["\\fbox"],props:{numArgs:1,argTypes:["hbox"],allowedInText:!0},handler(e,t){var{parser:n}=e;return{type:"enclose",mode:n.mode,label:"\\fbox",body:t[0]}}});Ye({type:"enclose",names:["\\cancel","\\bcancel","\\xcancel","\\sout","\\phase"],props:{numArgs:1},handler(e,t){var{parser:n,funcName:r}=e,a=t[0];return{type:"enclose",mode:n.mode,label:r,body:a}},htmlBuilder:V2,mathmlBuilder:K2});Ye({type:"enclose",names:["\\angl"],props:{numArgs:1,argTypes:["hbox"],allowedInText:!1},handler(e,t){var{parser:n}=e;return{type:"enclose",mode:n.mode,label:"\\angl",body:t[0]}}});var Yx={};function Ei(e){for(var{type:t,names:n,props:r,handler:a,htmlBuilder:s,mathmlBuilder:o}=e,c={type:t,numArgs:r.numArgs||0,allowedInText:!1,numOptionalArgs:0,handler:a},d=0;d<n.length;++d)Yx[n[d]]=c;s&&(df[t]=s),o&&(hf[t]=o)}var Gx={};function I(e,t){Gx[e]=t}function p6(e){var t=[];e.consumeSpaces();var n=e.fetch().text;for(n==="\\relax"&&(e.consume(),e.consumeSpaces(),n=e.fetch().text);n==="\\hline"||n==="\\hdashline";)e.consume(),t.push(n==="\\hdashline"),e.consumeSpaces(),n=e.fetch().text;return t}var am=e=>{var t=e.parser.settings;if(!t.displayMode)throw new Ne("{"+e.envName+"} can be used only in display mode.")};function W2(e){if(e.indexOf("ed")===-1)return e.indexOf("*")===-1}function ul(e,t,n){var{hskipBeforeAndAfter:r,addJot:a,cols:s,arraystretch:o,colSeparationType:c,autoTag:d,singleRow:f,emptySingleRow:m,maxNumCols:p,leqno:b}=t;if(e.gullet.beginGroup(),f||e.gullet.macros.set("\\cr","\\\\\\relax"),!o){var y=e.gullet.expandMacroAsText("\\arraystretch");if(y==null)o=1;else if(o=parseFloat(y),!o||o<0)throw new Ne("Invalid \\arraystretch: "+y)}e.gullet.beginGroup();var T=[],x=[T],C=[],w=[],O=d!=null?[]:void 0;function D(){d&&e.gullet.macros.set("\\@eqnsw","1",!0)}function Y(){O&&(e.gullet.macros.get("\\df@tag")?(O.push(e.subparse([new Da("\\df@tag")])),e.gullet.macros.set("\\df@tag",void 0,!0)):O.push(!!d&&e.gullet.macros.get("\\@eqnsw")==="1"))}for(D(),w.push(p6(e));;){var H=e.parseExpression(!1,f?"\\end":"\\\\");e.gullet.endGroup(),e.gullet.beginGroup(),H={type:"ordgroup",mode:e.mode,body:H},n&&(H={type:"styling",mode:e.mode,style:n,body:[H]}),T.push(H);var B=e.fetch().text;if(B==="&"){if(p&&T.length===p){if(f||c)throw new Ne("Too many tab characters: &",e.nextToken);e.settings.reportNonstrict("textEnv","Too few columns specified in the {array} column argument.")}e.consume()}else if(B==="\\end"){Y(),T.length===1&&H.type==="styling"&&H.body[0].body.length===0&&(x.length>1||!m)&&x.pop(),w.length<x.length+1&&w.push([]);break}else if(B==="\\\\"){e.consume();var ee=void 0;e.gullet.future().text!==" "&&(ee=e.parseSizeGroup(!0)),C.push(ee?ee.value:null),Y(),w.push(p6(e)),T=[],x.push(T),D()}else throw new Ne("Expected & or \\\\ or \\cr or \\end",e.nextToken)}return e.gullet.endGroup(),e.gullet.endGroup(),{type:"array",mode:e.mode,addJot:a,arraystretch:o,body:x,cols:s,rowGaps:C,hskipBeforeAndAfter:r,hLinesBeforeRow:w,colSeparationType:c,tags:O,leqno:b}}function X2(e){return e.slice(0,1)==="d"?"display":"text"}var Ti=function(t,n){var r,a,s=t.body.length,o=t.hLinesBeforeRow,c=0,d=new Array(s),f=[],m=Math.max(n.fontMetrics().arrayRuleWidth,n.minRuleThickness),p=1/n.fontMetrics().ptPerEm,b=5*p;if(t.colSeparationType&&t.colSeparationType==="small"){var y=n.havingStyle(lt.SCRIPT).sizeMultiplier;b=.2778*(y/n.sizeMultiplier)}var T=t.colSeparationType==="CD"?Rn({number:3,unit:"ex"},n):12*p,x=3*p,C=t.arraystretch*T,w=.7*C,O=.3*C,D=0;function Y(mn){for(var Lt=0;Lt<mn.length;++Lt)Lt>0&&(D+=.25),f.push({pos:D,isDashed:mn[Lt]})}for(Y(o[0]),r=0;r<t.body.length;++r){var H=t.body[r],B=w,ee=O;c<H.length&&(c=H.length);var Z=new Array(H.length);for(a=0;a<H.length;++a){var Q=Ft(H[a],n);ee<Q.depth&&(ee=Q.depth),B<Q.height&&(B=Q.height),Z[a]=Q}var G=t.rowGaps[r],se=0;G&&(se=Rn(G,n),se>0&&(se+=O,ee<se&&(ee=se),se=0)),t.addJot&&(ee+=x),Z.height=B,Z.depth=ee,D+=B,Z.pos=D,D+=ee+se,d[r]=Z,Y(o[r+1])}var oe=D/2+n.fontMetrics().axisHeight,ye=t.cols||[],fe=[],de,V,me=[];if(t.tags&&t.tags.some(mn=>mn))for(r=0;r<s;++r){var pe=d[r],Be=pe.pos-oe,k=t.tags[r],ie=void 0;k===!0?ie=ae.makeSpan(["eqn-num"],[],n):k===!1?ie=ae.makeSpan([],[],n):ie=ae.makeSpan([],sr(k,n,!0),n),ie.depth=pe.depth,ie.height=pe.height,me.push({type:"elem",elem:ie,shift:Be})}for(a=0,V=0;a<c||V<ye.length;++a,++V){for(var ge=ye[V]||{},L=!0;ge.type==="separator";){if(L||(de=ae.makeSpan(["arraycolsep"],[]),de.style.width=Le(n.fontMetrics().doubleRuleSep),fe.push(de)),ge.separator==="|"||ge.separator===":"){var Oe=ge.separator==="|"?"solid":"dashed",je=ae.makeSpan(["vertical-separator"],[],n);je.style.height=Le(D),je.style.borderRightWidth=Le(m),je.style.borderRightStyle=Oe,je.style.margin="0 "+Le(-m/2);var De=D-oe;De&&(je.style.verticalAlign=Le(-De)),fe.push(je)}else throw new Ne("Invalid separator type: "+ge.separator);V++,ge=ye[V]||{},L=!1}if(!(a>=c)){var Ot=void 0;(a>0||t.hskipBeforeAndAfter)&&(Ot=it.deflt(ge.pregap,b),Ot!==0&&(de=ae.makeSpan(["arraycolsep"],[]),de.style.width=Le(Ot),fe.push(de)));var ot=[];for(r=0;r<s;++r){var zt=d[r],wt=zt[a];if(wt){var vn=zt.pos-oe;wt.depth=zt.depth,wt.height=zt.height,ot.push({type:"elem",elem:wt,shift:vn})}}ot=ae.makeVList({positionType:"individualShift",children:ot},n),ot=ae.makeSpan(["col-align-"+(ge.align||"c")],[ot]),fe.push(ot),(a<c-1||t.hskipBeforeAndAfter)&&(Ot=it.deflt(ge.postgap,b),Ot!==0&&(de=ae.makeSpan(["arraycolsep"],[]),de.style.width=Le(Ot),fe.push(de)))}}if(d=ae.makeSpan(["mtable"],fe),f.length>0){for(var mr=ae.makeLineSpan("hline",n,m),vt=ae.makeLineSpan("hdashline",n,m),gt=[{type:"elem",elem:d,shift:0}];f.length>0;){var Bt=f.pop(),Je=Bt.pos-oe;Bt.isDashed?gt.push({type:"elem",elem:vt,shift:Je}):gt.push({type:"elem",elem:mr,shift:Je})}d=ae.makeVList({positionType:"individualShift",children:gt},n)}if(me.length===0)return ae.makeSpan(["mord"],[d],n);var yt=ae.makeVList({positionType:"individualShift",children:me},n);return yt=ae.makeSpan(["tag"],[yt],n),ae.makeFragment([d,yt])},o$={c:"center ",l:"left ",r:"right "},xi=function(t,n){for(var r=[],a=new Ce.MathNode("mtd",[],["mtr-glue"]),s=new Ce.MathNode("mtd",[],["mml-eqn-num"]),o=0;o<t.body.length;o++){for(var c=t.body[o],d=[],f=0;f<c.length;f++)d.push(new Ce.MathNode("mtd",[sn(c[f],n)]));t.tags&&t.tags[o]&&(d.unshift(a),d.push(a),t.leqno?d.unshift(s):d.push(s)),r.push(new Ce.MathNode("mtr",d))}var m=new Ce.MathNode("mtable",r),p=t.arraystretch===.5?.1:.16+t.arraystretch-1+(t.addJot?.09:0);m.setAttribute("rowspacing",Le(p));var b="",y="";if(t.cols&&t.cols.length>0){var T=t.cols,x="",C=!1,w=0,O=T.length;T[0].type==="separator"&&(b+="top ",w=1),T[T.length-1].type==="separator"&&(b+="bottom ",O-=1);for(var D=w;D<O;D++)T[D].type==="align"?(y+=o$[T[D].align],C&&(x+="none "),C=!0):T[D].type==="separator"&&C&&(x+=T[D].separator==="|"?"solid ":"dashed ",C=!1);m.setAttribute("columnalign",y.trim()),/[sd]/.test(x)&&m.setAttribute("columnlines",x.trim())}if(t.colSeparationType==="align"){for(var Y=t.cols||[],H="",B=1;B<Y.length;B++)H+=B%2?"0em ":"1em ";m.setAttribute("columnspacing",H.trim())}else t.colSeparationType==="alignat"||t.colSeparationType==="gather"?m.setAttribute("columnspacing","0em"):t.colSeparationType==="small"?m.setAttribute("columnspacing","0.2778em"):t.colSeparationType==="CD"?m.setAttribute("columnspacing","0.5em"):m.setAttribute("columnspacing","1em");var ee="",Z=t.hLinesBeforeRow;b+=Z[0].length>0?"left ":"",b+=Z[Z.length-1].length>0?"right ":"";for(var Q=1;Q<Z.length-1;Q++)ee+=Z[Q].length===0?"none ":Z[Q][0]?"dashed ":"solid ";return/[sd]/.test(ee)&&m.setAttribute("rowlines",ee.trim()),b!==""&&(m=new Ce.MathNode("menclose",[m]),m.setAttribute("notation",b.trim())),t.arraystretch&&t.arraystretch<1&&(m=new Ce.MathNode("mstyle",[m]),m.setAttribute("scriptlevel","1")),m},Vx=function(t,n){t.envName.indexOf("ed")===-1&&am(t);var r=[],a=t.envName.indexOf("at")>-1?"alignat":"align",s=t.envName==="split",o=ul(t.parser,{cols:r,addJot:!0,autoTag:s?void 0:W2(t.envName),emptySingleRow:!0,colSeparationType:a,maxNumCols:s?2:void 0,leqno:t.parser.settings.leqno},"display"),c,d=0,f={type:"ordgroup",mode:t.mode,body:[]};if(n[0]&&n[0].type==="ordgroup"){for(var m="",p=0;p<n[0].body.length;p++){var b=St(n[0].body[p],"textord");m+=b.text}c=Number(m),d=c*2}var y=!d;o.body.forEach(function(w){for(var O=1;O<w.length;O+=2){var D=St(w[O],"styling"),Y=St(D.body[0],"ordgroup");Y.body.unshift(f)}if(y)d<w.length&&(d=w.length);else{var H=w.length/2;if(c<H)throw new Ne("Too many math in a row: "+("expected "+c+", but got "+H),w[0])}});for(var T=0;T<d;++T){var x="r",C=0;T%2===1?x="l":T>0&&y&&(C=1),r[T]={type:"align",align:x,pregap:C,postgap:0}}return o.colSeparationType=y?"align":"alignat",o};Ei({type:"array",names:["array","darray"],props:{numArgs:1},handler(e,t){var n=tm(t[0]),r=n?[t[0]]:St(t[0],"ordgroup").body,a=r.map(function(o){var c=$2(o),d=c.text;if("lcr".indexOf(d)!==-1)return{type:"align",align:d};if(d==="|")return{type:"separator",separator:"|"};if(d===":")return{type:"separator",separator:":"};throw new Ne("Unknown column alignment: "+d,o)}),s={cols:a,hskipBeforeAndAfter:!0,maxNumCols:a.length};return ul(e.parser,s,X2(e.envName))},htmlBuilder:Ti,mathmlBuilder:xi});Ei({type:"array",names:["matrix","pmatrix","bmatrix","Bmatrix","vmatrix","Vmatrix","matrix*","pmatrix*","bmatrix*","Bmatrix*","vmatrix*","Vmatrix*"],props:{numArgs:0},handler(e){var t={matrix:null,pmatrix:["(",")"],bmatrix:["[","]"],Bmatrix:["\\{","\\}"],vmatrix:["|","|"],Vmatrix:["\\Vert","\\Vert"]}[e.envName.replace("*","")],n="c",r={hskipBeforeAndAfter:!1,cols:[{type:"align",align:n}]};if(e.envName.charAt(e.envName.length-1)==="*"){var a=e.parser;if(a.consumeSpaces(),a.fetch().text==="["){if(a.consume(),a.consumeSpaces(),n=a.fetch().text,"lcr".indexOf(n)===-1)throw new Ne("Expected l or c or r",a.nextToken);a.consume(),a.consumeSpaces(),a.expect("]"),a.consume(),r.cols=[{type:"align",align:n}]}}var s=ul(e.parser,r,X2(e.envName)),o=Math.max(0,...s.body.map(c=>c.length));return s.cols=new Array(o).fill({type:"align",align:n}),t?{type:"leftright",mode:e.mode,body:[s],left:t[0],right:t[1],rightColor:void 0}:s},htmlBuilder:Ti,mathmlBuilder:xi});Ei({type:"array",names:["smallmatrix"],props:{numArgs:0},handler(e){var t={arraystretch:.5},n=ul(e.parser,t,"script");return n.colSeparationType="small",n},htmlBuilder:Ti,mathmlBuilder:xi});Ei({type:"array",names:["subarray"],props:{numArgs:1},handler(e,t){var n=tm(t[0]),r=n?[t[0]]:St(t[0],"ordgroup").body,a=r.map(function(o){var c=$2(o),d=c.text;if("lc".indexOf(d)!==-1)return{type:"align",align:d};throw new Ne("Unknown column alignment: "+d,o)});if(a.length>1)throw new Ne("{subarray} can contain only one column");var s={cols:a,hskipBeforeAndAfter:!1,arraystretch:.5};if(s=ul(e.parser,s,"script"),s.body.length>0&&s.body[0].length>1)throw new Ne("{subarray} can contain only one column");return s},htmlBuilder:Ti,mathmlBuilder:xi});Ei({type:"array",names:["cases","dcases","rcases","drcases"],props:{numArgs:0},handler(e){var t={arraystretch:1.2,cols:[{type:"align",align:"l",pregap:0,postgap:1},{type:"align",align:"l",pregap:0,postgap:0}]},n=ul(e.parser,t,X2(e.envName));return{type:"leftright",mode:e.mode,body:[n],left:e.envName.indexOf("r")>-1?".":"\\{",right:e.envName.indexOf("r")>-1?"\\}":".",rightColor:void 0}},htmlBuilder:Ti,mathmlBuilder:xi});Ei({type:"array",names:["align","align*","aligned","split"],props:{numArgs:0},handler:Vx,htmlBuilder:Ti,mathmlBuilder:xi});Ei({type:"array",names:["gathered","gather","gather*"],props:{numArgs:0},handler(e){it.contains(["gather","gather*"],e.envName)&&am(e);var t={cols:[{type:"align",align:"c"}],addJot:!0,colSeparationType:"gather",autoTag:W2(e.envName),emptySingleRow:!0,leqno:e.parser.settings.leqno};return ul(e.parser,t,"display")},htmlBuilder:Ti,mathmlBuilder:xi});Ei({type:"array",names:["alignat","alignat*","alignedat"],props:{numArgs:1},handler:Vx,htmlBuilder:Ti,mathmlBuilder:xi});Ei({type:"array",names:["equation","equation*"],props:{numArgs:0},handler(e){am(e);var t={autoTag:W2(e.envName),emptySingleRow:!0,singleRow:!0,maxNumCols:1,leqno:e.parser.settings.leqno};return ul(e.parser,t,"display")},htmlBuilder:Ti,mathmlBuilder:xi});Ei({type:"array",names:["CD"],props:{numArgs:0},handler(e){return am(e),Kj(e.parser)},htmlBuilder:Ti,mathmlBuilder:xi});I("\\nonumber","\\gdef\\@eqnsw{0}");I("\\notag","\\nonumber");Ye({type:"text",names:["\\hline","\\hdashline"],props:{numArgs:0,allowedInText:!0,allowedInMath:!0},handler(e,t){throw new Ne(e.funcName+" valid only within array environment")}});var g6=Yx;Ye({type:"environment",names:["\\begin","\\end"],props:{numArgs:1,argTypes:["text"]},handler(e,t){var{parser:n,funcName:r}=e,a=t[0];if(a.type!=="ordgroup")throw new Ne("Invalid environment name",a);for(var s="",o=0;o<a.body.length;++o)s+=St(a.body[o],"textord").text;if(r==="\\begin"){if(!g6.hasOwnProperty(s))throw new Ne("No such environment: "+s,a);var c=g6[s],{args:d,optArgs:f}=n.parseArguments("\\begin{"+s+"}",c),m={mode:n.mode,envName:s,parser:n},p=c.handler(m,d,f);n.expect("\\end",!1);var b=n.nextToken,y=St(n.parseFunction(),"environment");if(y.name!==s)throw new Ne("Mismatch: \\begin{"+s+"} matched by \\end{"+y.name+"}",b);return p}return{type:"environment",mode:n.mode,name:s,nameGroup:a}}});var Kx=(e,t)=>{var n=e.font,r=t.withFont(n);return Ft(e.body,r)},Wx=(e,t)=>{var n=e.font,r=t.withFont(n);return sn(e.body,r)},b6={"\\Bbb":"\\mathbb","\\bold":"\\mathbf","\\frak":"\\mathfrak","\\bm":"\\boldsymbol"};Ye({type:"font",names:["\\mathrm","\\mathit","\\mathbf","\\mathnormal","\\mathsfit","\\mathbb","\\mathcal","\\mathfrak","\\mathscr","\\mathsf","\\mathtt","\\Bbb","\\bold","\\frak"],props:{numArgs:1,allowedInArgument:!0},handler:(e,t)=>{var{parser:n,funcName:r}=e,a=ff(t[0]),s=r;return s in b6&&(s=b6[s]),{type:"font",mode:n.mode,font:s.slice(1),body:a}},htmlBuilder:Kx,mathmlBuilder:Wx});Ye({type:"mclass",names:["\\boldsymbol","\\bm"],props:{numArgs:1},handler:(e,t)=>{var{parser:n}=e,r=t[0],a=it.isCharacterBox(r);return{type:"mclass",mode:n.mode,mclass:nm(r),body:[{type:"font",mode:n.mode,font:"boldsymbol",body:r}],isCharacterBox:a}}});Ye({type:"font",names:["\\rm","\\sf","\\tt","\\bf","\\it","\\cal"],props:{numArgs:0,allowedInText:!0},handler:(e,t)=>{var{parser:n,funcName:r,breakOnTokenText:a}=e,{mode:s}=n,o=n.parseExpression(!0,a),c="math"+r.slice(1);return{type:"font",mode:s,font:c,body:{type:"ordgroup",mode:n.mode,body:o}}},htmlBuilder:Kx,mathmlBuilder:Wx});var Xx=(e,t)=>{var n=t;return e==="display"?n=n.id>=lt.SCRIPT.id?n.text():lt.DISPLAY:e==="text"&&n.size===lt.DISPLAY.size?n=lt.TEXT:e==="script"?n=lt.SCRIPT:e==="scriptscript"&&(n=lt.SCRIPTSCRIPT),n},Q2=(e,t)=>{var n=Xx(e.size,t.style),r=n.fracNum(),a=n.fracDen(),s;s=t.havingStyle(r);var o=Ft(e.numer,s,t);if(e.continued){var c=8.5/t.fontMetrics().ptPerEm,d=3.5/t.fontMetrics().ptPerEm;o.height=o.height<c?c:o.height,o.depth=o.depth<d?d:o.depth}s=t.havingStyle(a);var f=Ft(e.denom,s,t),m,p,b;e.hasBarLine?(e.barSize?(p=Rn(e.barSize,t),m=ae.makeLineSpan("frac-line",t,p)):m=ae.makeLineSpan("frac-line",t),p=m.height,b=m.height):(m=null,p=0,b=t.fontMetrics().defaultRuleThickness);var y,T,x;n.size===lt.DISPLAY.size||e.size==="display"?(y=t.fontMetrics().num1,p>0?T=3*b:T=7*b,x=t.fontMetrics().denom1):(p>0?(y=t.fontMetrics().num2,T=b):(y=t.fontMetrics().num3,T=3*b),x=t.fontMetrics().denom2);var C;if(m){var O=t.fontMetrics().axisHeight;y-o.depth-(O+.5*p)<T&&(y+=T-(y-o.depth-(O+.5*p))),O-.5*p-(f.height-x)<T&&(x+=T-(O-.5*p-(f.height-x)));var D=-(O-.5*p);C=ae.makeVList({positionType:"individualShift",children:[{type:"elem",elem:f,shift:x},{type:"elem",elem:m,shift:D},{type:"elem",elem:o,shift:-y}]},t)}else{var w=y-o.depth-(f.height-x);w<T&&(y+=.5*(T-w),x+=.5*(T-w)),C=ae.makeVList({positionType:"individualShift",children:[{type:"elem",elem:f,shift:x},{type:"elem",elem:o,shift:-y}]},t)}s=t.havingStyle(n),C.height*=s.sizeMultiplier/t.sizeMultiplier,C.depth*=s.sizeMultiplier/t.sizeMultiplier;var Y;n.size===lt.DISPLAY.size?Y=t.fontMetrics().delim1:n.size===lt.SCRIPTSCRIPT.size?Y=t.havingStyle(lt.SCRIPT).fontMetrics().delim2:Y=t.fontMetrics().delim2;var H,B;return e.leftDelim==null?H=g0(t,["mopen"]):H=ts.customSizedDelim(e.leftDelim,Y,!0,t.havingStyle(n),e.mode,["mopen"]),e.continued?B=ae.makeSpan([]):e.rightDelim==null?B=g0(t,["mclose"]):B=ts.customSizedDelim(e.rightDelim,Y,!0,t.havingStyle(n),e.mode,["mclose"]),ae.makeSpan(["mord"].concat(s.sizingClasses(t)),[H,ae.makeSpan(["mfrac"],[C]),B],t)},Z2=(e,t)=>{var n=new Ce.MathNode("mfrac",[sn(e.numer,t),sn(e.denom,t)]);if(!e.hasBarLine)n.setAttribute("linethickness","0px");else if(e.barSize){var r=Rn(e.barSize,t);n.setAttribute("linethickness",Le(r))}var a=Xx(e.size,t.style);if(a.size!==t.style.size){n=new Ce.MathNode("mstyle",[n]);var s=a.size===lt.DISPLAY.size?"true":"false";n.setAttribute("displaystyle",s),n.setAttribute("scriptlevel","0")}if(e.leftDelim!=null||e.rightDelim!=null){var o=[];if(e.leftDelim!=null){var c=new Ce.MathNode("mo",[new Ce.TextNode(e.leftDelim.replace("\\",""))]);c.setAttribute("fence","true"),o.push(c)}if(o.push(n),e.rightDelim!=null){var d=new Ce.MathNode("mo",[new Ce.TextNode(e.rightDelim.replace("\\",""))]);d.setAttribute("fence","true"),o.push(d)}return q2(o)}return n};Ye({type:"genfrac",names:["\\dfrac","\\frac","\\tfrac","\\dbinom","\\binom","\\tbinom","\\\\atopfrac","\\\\bracefrac","\\\\brackfrac"],props:{numArgs:2,allowedInArgument:!0},handler:(e,t)=>{var{parser:n,funcName:r}=e,a=t[0],s=t[1],o,c=null,d=null,f="auto";switch(r){case"\\dfrac":case"\\frac":case"\\tfrac":o=!0;break;case"\\\\atopfrac":o=!1;break;case"\\dbinom":case"\\binom":case"\\tbinom":o=!1,c="(",d=")";break;case"\\\\bracefrac":o=!1,c="\\{",d="\\}";break;case"\\\\brackfrac":o=!1,c="[",d="]";break;default:throw new Error("Unrecognized genfrac command")}switch(r){case"\\dfrac":case"\\dbinom":f="display";break;case"\\tfrac":case"\\tbinom":f="text";break}return{type:"genfrac",mode:n.mode,continued:!1,numer:a,denom:s,hasBarLine:o,leftDelim:c,rightDelim:d,size:f,barSize:null}},htmlBuilder:Q2,mathmlBuilder:Z2});Ye({type:"genfrac",names:["\\cfrac"],props:{numArgs:2},handler:(e,t)=>{var{parser:n,funcName:r}=e,a=t[0],s=t[1];return{type:"genfrac",mode:n.mode,continued:!0,numer:a,denom:s,hasBarLine:!0,leftDelim:null,rightDelim:null,size:"display",barSize:null}}});Ye({type:"infix",names:["\\over","\\choose","\\atop","\\brace","\\brack"],props:{numArgs:0,infix:!0},handler(e){var{parser:t,funcName:n,token:r}=e,a;switch(n){case"\\over":a="\\frac";break;case"\\choose":a="\\binom";break;case"\\atop":a="\\\\atopfrac";break;case"\\brace":a="\\\\bracefrac";break;case"\\brack":a="\\\\brackfrac";break;default:throw new Error("Unrecognized infix genfrac command")}return{type:"infix",mode:t.mode,replaceWith:a,token:r}}});var v6=["display","text","script","scriptscript"],y6=function(t){var n=null;return t.length>0&&(n=t,n=n==="."?null:n),n};Ye({type:"genfrac",names:["\\genfrac"],props:{numArgs:6,allowedInArgument:!0,argTypes:["math","math","size","text","math","math"]},handler(e,t){var{parser:n}=e,r=t[4],a=t[5],s=ff(t[0]),o=s.type==="atom"&&s.family==="open"?y6(s.text):null,c=ff(t[1]),d=c.type==="atom"&&c.family==="close"?y6(c.text):null,f=St(t[2],"size"),m,p=null;f.isBlank?m=!0:(p=f.value,m=p.number>0);var b="auto",y=t[3];if(y.type==="ordgroup"){if(y.body.length>0){var T=St(y.body[0],"textord");b=v6[Number(T.text)]}}else y=St(y,"textord"),b=v6[Number(y.text)];return{type:"genfrac",mode:n.mode,numer:r,denom:a,continued:!1,hasBarLine:m,barSize:p,leftDelim:o,rightDelim:d,size:b}},htmlBuilder:Q2,mathmlBuilder:Z2});Ye({type:"infix",names:["\\above"],props:{numArgs:1,argTypes:["size"],infix:!0},handler(e,t){var{parser:n,funcName:r,token:a}=e;return{type:"infix",mode:n.mode,replaceWith:"\\\\abovefrac",size:St(t[0],"size").value,token:a}}});Ye({type:"genfrac",names:["\\\\abovefrac"],props:{numArgs:3,argTypes:["math","size","math"]},handler:(e,t)=>{var{parser:n,funcName:r}=e,a=t[0],s=Uq(St(t[1],"infix").size),o=t[2],c=s.number>0;return{type:"genfrac",mode:n.mode,numer:a,denom:o,continued:!1,hasBarLine:c,barSize:s,leftDelim:null,rightDelim:null,size:"auto"}},htmlBuilder:Q2,mathmlBuilder:Z2});var Qx=(e,t)=>{var n=t.style,r,a;e.type==="supsub"?(r=e.sup?Ft(e.sup,t.havingStyle(n.sup()),t):Ft(e.sub,t.havingStyle(n.sub()),t),a=St(e.base,"horizBrace")):a=St(e,"horizBrace");var s=Ft(a.base,t.havingBaseStyle(lt.DISPLAY)),o=ss.svgSpan(a,t),c;if(a.isOver?(c=ae.makeVList({positionType:"firstBaseline",children:[{type:"elem",elem:s},{type:"kern",size:.1},{type:"elem",elem:o}]},t),c.children[0].children[0].children[1].classes.push("svg-align")):(c=ae.makeVList({positionType:"bottom",positionData:s.depth+.1+o.height,children:[{type:"elem",elem:o},{type:"kern",size:.1},{type:"elem",elem:s}]},t),c.children[0].children[0].children[0].classes.push("svg-align")),r){var d=ae.makeSpan(["mord",a.isOver?"mover":"munder"],[c],t);a.isOver?c=ae.makeVList({positionType:"firstBaseline",children:[{type:"elem",elem:d},{type:"kern",size:.2},{type:"elem",elem:r}]},t):c=ae.makeVList({positionType:"bottom",positionData:d.depth+.2+r.height+r.depth,children:[{type:"elem",elem:r},{type:"kern",size:.2},{type:"elem",elem:d}]},t)}return ae.makeSpan(["mord",a.isOver?"mover":"munder"],[c],t)},u$=(e,t)=>{var n=ss.mathMLnode(e.label);return new Ce.MathNode(e.isOver?"mover":"munder",[sn(e.base,t),n])};Ye({type:"horizBrace",names:["\\overbrace","\\underbrace"],props:{numArgs:1},handler(e,t){var{parser:n,funcName:r}=e;return{type:"horizBrace",mode:n.mode,label:r,isOver:/^\\over/.test(r),base:t[0]}},htmlBuilder:Qx,mathmlBuilder:u$});Ye({type:"href",names:["\\href"],props:{numArgs:2,argTypes:["url","original"],allowedInText:!0},handler:(e,t)=>{var{parser:n}=e,r=t[1],a=St(t[0],"url").url;return n.settings.isTrusted({command:"\\href",url:a})?{type:"href",mode:n.mode,href:a,body:Hn(r)}:n.formatUnsupportedCmd("\\href")},htmlBuilder:(e,t)=>{var n=sr(e.body,t,!1);return ae.makeAnchor(e.href,[],n,t)},mathmlBuilder:(e,t)=>{var n=ll(e.body,t);return n instanceof va||(n=new va("mrow",[n])),n.setAttribute("href",e.href),n}});Ye({type:"href",names:["\\url"],props:{numArgs:1,argTypes:["url"],allowedInText:!0},handler:(e,t)=>{var{parser:n}=e,r=St(t[0],"url").url;if(!n.settings.isTrusted({command:"\\url",url:r}))return n.formatUnsupportedCmd("\\url");for(var a=[],s=0;s<r.length;s++){var o=r[s];o==="~"&&(o="\\textasciitilde"),a.push({type:"textord",mode:"text",text:o})}var c={type:"text",mode:n.mode,font:"\\texttt",body:a};return{type:"href",mode:n.mode,href:r,body:Hn(c)}}});Ye({type:"hbox",names:["\\hbox"],props:{numArgs:1,argTypes:["text"],allowedInText:!0,primitive:!0},handler(e,t){var{parser:n}=e;return{type:"hbox",mode:n.mode,body:Hn(t[0])}},htmlBuilder(e,t){var n=sr(e.body,t,!1);return ae.makeFragment(n)},mathmlBuilder(e,t){return new Ce.MathNode("mrow",aa(e.body,t))}});Ye({type:"html",names:["\\htmlClass","\\htmlId","\\htmlStyle","\\htmlData"],props:{numArgs:2,argTypes:["raw","original"],allowedInText:!0},handler:(e,t)=>{var{parser:n,funcName:r,token:a}=e,s=St(t[0],"raw").string,o=t[1];n.settings.strict&&n.settings.reportNonstrict("htmlExtension","HTML extension is disabled on strict mode");var c,d={};switch(r){case"\\htmlClass":d.class=s,c={command:"\\htmlClass",class:s};break;case"\\htmlId":d.id=s,c={command:"\\htmlId",id:s};break;case"\\htmlStyle":d.style=s,c={command:"\\htmlStyle",style:s};break;case"\\htmlData":{for(var f=s.split(","),m=0;m<f.length;m++){var p=f[m].split("=");if(p.length!==2)throw new Ne("Error parsing key-value for \\htmlData");d["data-"+p[0].trim()]=p[1].trim()}c={command:"\\htmlData",attributes:d};break}default:throw new Error("Unrecognized html command")}return n.settings.isTrusted(c)?{type:"html",mode:n.mode,attributes:d,body:Hn(o)}:n.formatUnsupportedCmd(r)},htmlBuilder:(e,t)=>{var n=sr(e.body,t,!1),r=["enclosing"];e.attributes.class&&r.push(...e.attributes.class.trim().split(/\s+/));var a=ae.makeSpan(r,n,t);for(var s in e.attributes)s!=="class"&&e.attributes.hasOwnProperty(s)&&a.setAttribute(s,e.attributes[s]);return a},mathmlBuilder:(e,t)=>ll(e.body,t)});Ye({type:"htmlmathml",names:["\\html@mathml"],props:{numArgs:2,allowedInText:!0},handler:(e,t)=>{var{parser:n}=e;return{type:"htmlmathml",mode:n.mode,html:Hn(t[0]),mathml:Hn(t[1])}},htmlBuilder:(e,t)=>{var n=sr(e.html,t,!1);return ae.makeFragment(n)},mathmlBuilder:(e,t)=>ll(e.mathml,t)});var fg=function(t){if(/^[-+]? *(\d+(\.\d*)?|\.\d+)$/.test(t))return{number:+t,unit:"bp"};var n=/([-+]?) *(\d+(?:\.\d*)?|\.\d+) *([a-z]{2})/.exec(t);if(!n)throw new Ne("Invalid size: '"+t+"' in \\includegraphics");var r={number:+(n[1]+n[2]),unit:n[3]};if(!gx(r))throw new Ne("Invalid unit: '"+r.unit+"' in \\includegraphics.");return r};Ye({type:"includegraphics",names:["\\includegraphics"],props:{numArgs:1,numOptionalArgs:1,argTypes:["raw","url"],allowedInText:!1},handler:(e,t,n)=>{var{parser:r}=e,a={number:0,unit:"em"},s={number:.9,unit:"em"},o={number:0,unit:"em"},c="";if(n[0])for(var d=St(n[0],"raw").string,f=d.split(","),m=0;m<f.length;m++){var p=f[m].split("=");if(p.length===2){var b=p[1].trim();switch(p[0].trim()){case"alt":c=b;break;case"width":a=fg(b);break;case"height":s=fg(b);break;case"totalheight":o=fg(b);break;default:throw new Ne("Invalid key: '"+p[0]+"' in \\includegraphics.")}}}var y=St(t[0],"url").url;return c===""&&(c=y,c=c.replace(/^.*[\\/]/,""),c=c.substring(0,c.lastIndexOf("."))),r.settings.isTrusted({command:"\\includegraphics",url:y})?{type:"includegraphics",mode:r.mode,alt:c,width:a,height:s,totalheight:o,src:y}:r.formatUnsupportedCmd("\\includegraphics")},htmlBuilder:(e,t)=>{var n=Rn(e.height,t),r=0;e.totalheight.number>0&&(r=Rn(e.totalheight,t)-n);var a=0;e.width.number>0&&(a=Rn(e.width,t));var s={height:Le(n+r)};a>0&&(s.width=Le(a)),r>0&&(s.verticalAlign=Le(-r));var o=new uj(e.src,e.alt,s);return o.height=n,o.depth=r,o},mathmlBuilder:(e,t)=>{var n=new Ce.MathNode("mglyph",[]);n.setAttribute("alt",e.alt);var r=Rn(e.height,t),a=0;if(e.totalheight.number>0&&(a=Rn(e.totalheight,t)-r,n.setAttribute("valign",Le(-a))),n.setAttribute("height",Le(r+a)),e.width.number>0){var s=Rn(e.width,t);n.setAttribute("width",Le(s))}return n.setAttribute("src",e.src),n}});Ye({type:"kern",names:["\\kern","\\mkern","\\hskip","\\mskip"],props:{numArgs:1,argTypes:["size"],primitive:!0,allowedInText:!0},handler(e,t){var{parser:n,funcName:r}=e,a=St(t[0],"size");if(n.settings.strict){var s=r[1]==="m",o=a.value.unit==="mu";s?(o||n.settings.reportNonstrict("mathVsTextUnits","LaTeX's "+r+" supports only mu units, "+("not "+a.value.unit+" units")),n.mode!=="math"&&n.settings.reportNonstrict("mathVsTextUnits","LaTeX's "+r+" works only in math mode")):o&&n.settings.reportNonstrict("mathVsTextUnits","LaTeX's "+r+" doesn't support mu units")}return{type:"kern",mode:n.mode,dimension:a.value}},htmlBuilder(e,t){return ae.makeGlue(e.dimension,t)},mathmlBuilder(e,t){var n=Rn(e.dimension,t);return new Ce.SpaceNode(n)}});Ye({type:"lap",names:["\\mathllap","\\mathrlap","\\mathclap"],props:{numArgs:1,allowedInText:!0},handler:(e,t)=>{var{parser:n,funcName:r}=e,a=t[0];return{type:"lap",mode:n.mode,alignment:r.slice(5),body:a}},htmlBuilder:(e,t)=>{var n;e.alignment==="clap"?(n=ae.makeSpan([],[Ft(e.body,t)]),n=ae.makeSpan(["inner"],[n],t)):n=ae.makeSpan(["inner"],[Ft(e.body,t)]);var r=ae.makeSpan(["fix"],[]),a=ae.makeSpan([e.alignment],[n,r],t),s=ae.makeSpan(["strut"]);return s.style.height=Le(a.height+a.depth),a.depth&&(s.style.verticalAlign=Le(-a.depth)),a.children.unshift(s),a=ae.makeSpan(["thinbox"],[a],t),ae.makeSpan(["mord","vbox"],[a],t)},mathmlBuilder:(e,t)=>{var n=new Ce.MathNode("mpadded",[sn(e.body,t)]);if(e.alignment!=="rlap"){var r=e.alignment==="llap"?"-1":"-0.5";n.setAttribute("lspace",r+"width")}return n.setAttribute("width","0px"),n}});Ye({type:"styling",names:["\\(","$"],props:{numArgs:0,allowedInText:!0,allowedInMath:!1},handler(e,t){var{funcName:n,parser:r}=e,a=r.mode;r.switchMode("math");var s=n==="\\("?"\\)":"$",o=r.parseExpression(!1,s);return r.expect(s),r.switchMode(a),{type:"styling",mode:r.mode,style:"text",body:o}}});Ye({type:"text",names:["\\)","\\]"],props:{numArgs:0,allowedInText:!0,allowedInMath:!1},handler(e,t){throw new Ne("Mismatched "+e.funcName)}});var _6=(e,t)=>{switch(t.style.size){case lt.DISPLAY.size:return e.display;case lt.TEXT.size:return e.text;case lt.SCRIPT.size:return e.script;case lt.SCRIPTSCRIPT.size:return e.scriptscript;default:return e.text}};Ye({type:"mathchoice",names:["\\mathchoice"],props:{numArgs:4,primitive:!0},handler:(e,t)=>{var{parser:n}=e;return{type:"mathchoice",mode:n.mode,display:Hn(t[0]),text:Hn(t[1]),script:Hn(t[2]),scriptscript:Hn(t[3])}},htmlBuilder:(e,t)=>{var n=_6(e,t),r=sr(n,t,!1);return ae.makeFragment(r)},mathmlBuilder:(e,t)=>{var n=_6(e,t);return ll(n,t)}});var Zx=(e,t,n,r,a,s,o)=>{e=ae.makeSpan([],[e]);var c=n&&it.isCharacterBox(n),d,f;if(t){var m=Ft(t,r.havingStyle(a.sup()),r);f={elem:m,kern:Math.max(r.fontMetrics().bigOpSpacing1,r.fontMetrics().bigOpSpacing3-m.depth)}}if(n){var p=Ft(n,r.havingStyle(a.sub()),r);d={elem:p,kern:Math.max(r.fontMetrics().bigOpSpacing2,r.fontMetrics().bigOpSpacing4-p.height)}}var b;if(f&&d){var y=r.fontMetrics().bigOpSpacing5+d.elem.height+d.elem.depth+d.kern+e.depth+o;b=ae.makeVList({positionType:"bottom",positionData:y,children:[{type:"kern",size:r.fontMetrics().bigOpSpacing5},{type:"elem",elem:d.elem,marginLeft:Le(-s)},{type:"kern",size:d.kern},{type:"elem",elem:e},{type:"kern",size:f.kern},{type:"elem",elem:f.elem,marginLeft:Le(s)},{type:"kern",size:r.fontMetrics().bigOpSpacing5}]},r)}else if(d){var T=e.height-o;b=ae.makeVList({positionType:"top",positionData:T,children:[{type:"kern",size:r.fontMetrics().bigOpSpacing5},{type:"elem",elem:d.elem,marginLeft:Le(-s)},{type:"kern",size:d.kern},{type:"elem",elem:e}]},r)}else if(f){var x=e.depth+o;b=ae.makeVList({positionType:"bottom",positionData:x,children:[{type:"elem",elem:e},{type:"kern",size:f.kern},{type:"elem",elem:f.elem,marginLeft:Le(s)},{type:"kern",size:r.fontMetrics().bigOpSpacing5}]},r)}else return e;var C=[b];if(d&&s!==0&&!c){var w=ae.makeSpan(["mspace"],[],r);w.style.marginRight=Le(s),C.unshift(w)}return ae.makeSpan(["mop","op-limits"],C,r)},Jx=["\\smallint"],yu=(e,t)=>{var n,r,a=!1,s;e.type==="supsub"?(n=e.sup,r=e.sub,s=St(e.base,"op"),a=!0):s=St(e,"op");var o=t.style,c=!1;o.size===lt.DISPLAY.size&&s.symbol&&!it.contains(Jx,s.name)&&(c=!0);var d;if(s.symbol){var f=c?"Size2-Regular":"Size1-Regular",m="";if((s.name==="\\oiint"||s.name==="\\oiiint")&&(m=s.name.slice(1),s.name=m==="oiint"?"\\iint":"\\iiint"),d=ae.makeSymbol(s.name,f,"math",t,["mop","op-symbol",c?"large-op":"small-op"]),m.length>0){var p=d.italic,b=ae.staticSvg(m+"Size"+(c?"2":"1"),t);d=ae.makeVList({positionType:"individualShift",children:[{type:"elem",elem:d,shift:0},{type:"elem",elem:b,shift:c?.08:0}]},t),s.name="\\"+m,d.classes.unshift("mop"),d.italic=p}}else if(s.body){var y=sr(s.body,t,!0);y.length===1&&y[0]instanceof Ua?(d=y[0],d.classes[0]="mop"):d=ae.makeSpan(["mop"],y,t)}else{for(var T=[],x=1;x<s.name.length;x++)T.push(ae.mathsym(s.name[x],s.mode,t));d=ae.makeSpan(["mop"],T,t)}var C=0,w=0;return(d instanceof Ua||s.name==="\\oiint"||s.name==="\\oiiint")&&!s.suppressBaseShift&&(C=(d.height-d.depth)/2-t.fontMetrics().axisHeight,w=d.italic),a?Zx(d,n,r,t,o,w,C):(C&&(d.style.position="relative",d.style.top=Le(C)),d)},U0=(e,t)=>{var n;if(e.symbol)n=new va("mo",[Fa(e.name,e.mode)]),it.contains(Jx,e.name)&&n.setAttribute("largeop","false");else if(e.body)n=new va("mo",aa(e.body,t));else{n=new va("mi",[new fi(e.name.slice(1))]);var r=new va("mo",[Fa("⁡","text")]);e.parentIsSupSub?n=new va("mrow",[n,r]):n=kx([n,r])}return n},c$={"∏":"\\prod","∐":"\\coprod","∑":"\\sum","⋀":"\\bigwedge","⋁":"\\bigvee","⋂":"\\bigcap","⋃":"\\bigcup","⨀":"\\bigodot","⨁":"\\bigoplus","⨂":"\\bigotimes","⨄":"\\biguplus","⨆":"\\bigsqcup"};Ye({type:"op",names:["\\coprod","\\bigvee","\\bigwedge","\\biguplus","\\bigcap","\\bigcup","\\intop","\\prod","\\sum","\\bigotimes","\\bigoplus","\\bigodot","\\bigsqcup","\\smallint","∏","∐","∑","⋀","⋁","⋂","⋃","⨀","⨁","⨂","⨄","⨆"],props:{numArgs:0},handler:(e,t)=>{var{parser:n,funcName:r}=e,a=r;return a.length===1&&(a=c$[a]),{type:"op",mode:n.mode,limits:!0,parentIsSupSub:!1,symbol:!0,name:a}},htmlBuilder:yu,mathmlBuilder:U0});Ye({type:"op",names:["\\mathop"],props:{numArgs:1,primitive:!0},handler:(e,t)=>{var{parser:n}=e,r=t[0];return{type:"op",mode:n.mode,limits:!1,parentIsSupSub:!1,symbol:!1,body:Hn(r)}},htmlBuilder:yu,mathmlBuilder:U0});var d$={"∫":"\\int","∬":"\\iint","∭":"\\iiint","∮":"\\oint","∯":"\\oiint","∰":"\\oiiint"};Ye({type:"op",names:["\\arcsin","\\arccos","\\arctan","\\arctg","\\arcctg","\\arg","\\ch","\\cos","\\cosec","\\cosh","\\cot","\\cotg","\\coth","\\csc","\\ctg","\\cth","\\deg","\\dim","\\exp","\\hom","\\ker","\\lg","\\ln","\\log","\\sec","\\sin","\\sinh","\\sh","\\tan","\\tanh","\\tg","\\th"],props:{numArgs:0},handler(e){var{parser:t,funcName:n}=e;return{type:"op",mode:t.mode,limits:!1,parentIsSupSub:!1,symbol:!1,name:n}},htmlBuilder:yu,mathmlBuilder:U0});Ye({type:"op",names:["\\det","\\gcd","\\inf","\\lim","\\max","\\min","\\Pr","\\sup"],props:{numArgs:0},handler(e){var{parser:t,funcName:n}=e;return{type:"op",mode:t.mode,limits:!0,parentIsSupSub:!1,symbol:!1,name:n}},htmlBuilder:yu,mathmlBuilder:U0});Ye({type:"op",names:["\\int","\\iint","\\iiint","\\oint","\\oiint","\\oiiint","∫","∬","∭","∮","∯","∰"],props:{numArgs:0},handler(e){var{parser:t,funcName:n}=e,r=n;return r.length===1&&(r=d$[r]),{type:"op",mode:t.mode,limits:!1,parentIsSupSub:!1,symbol:!0,name:r}},htmlBuilder:yu,mathmlBuilder:U0});var eS=(e,t)=>{var n,r,a=!1,s;e.type==="supsub"?(n=e.sup,r=e.sub,s=St(e.base,"operatorname"),a=!0):s=St(e,"operatorname");var o;if(s.body.length>0){for(var c=s.body.map(p=>{var b=p.text;return typeof b=="string"?{type:"textord",mode:p.mode,text:b}:p}),d=sr(c,t.withFont("mathrm"),!0),f=0;f<d.length;f++){var m=d[f];m instanceof Ua&&(m.text=m.text.replace(/\u2212/,"-").replace(/\u2217/,"*"))}o=ae.makeSpan(["mop"],d,t)}else o=ae.makeSpan(["mop"],[],t);return a?Zx(o,n,r,t,t.style,0,0):o},h$=(e,t)=>{for(var n=aa(e.body,t.withFont("mathrm")),r=!0,a=0;a<n.length;a++){var s=n[a];if(!(s instanceof Ce.SpaceNode))if(s instanceof Ce.MathNode)switch(s.type){case"mi":case"mn":case"ms":case"mspace":case"mtext":break;case"mo":{var o=s.children[0];s.children.length===1&&o instanceof Ce.TextNode?o.text=o.text.replace(/\u2212/,"-").replace(/\u2217/,"*"):r=!1;break}default:r=!1}else r=!1}if(r){var c=n.map(m=>m.toText()).join("");n=[new Ce.TextNode(c)]}var d=new Ce.MathNode("mi",n);d.setAttribute("mathvariant","normal");var f=new Ce.MathNode("mo",[Fa("⁡","text")]);return e.parentIsSupSub?new Ce.MathNode("mrow",[d,f]):Ce.newDocumentFragment([d,f])};Ye({type:"operatorname",names:["\\operatorname@","\\operatornamewithlimits"],props:{numArgs:1},handler:(e,t)=>{var{parser:n,funcName:r}=e,a=t[0];return{type:"operatorname",mode:n.mode,body:Hn(a),alwaysHandleSupSub:r==="\\operatornamewithlimits",limits:!1,parentIsSupSub:!1}},htmlBuilder:eS,mathmlBuilder:h$});I("\\operatorname","\\@ifstar\\operatornamewithlimits\\operatorname@");to({type:"ordgroup",htmlBuilder(e,t){return e.semisimple?ae.makeFragment(sr(e.body,t,!1)):ae.makeSpan(["mord"],sr(e.body,t,!0),t)},mathmlBuilder(e,t){return ll(e.body,t,!0)}});Ye({type:"overline",names:["\\overline"],props:{numArgs:1},handler(e,t){var{parser:n}=e,r=t[0];return{type:"overline",mode:n.mode,body:r}},htmlBuilder(e,t){var n=Ft(e.body,t.havingCrampedStyle()),r=ae.makeLineSpan("overline-line",t),a=t.fontMetrics().defaultRuleThickness,s=ae.makeVList({positionType:"firstBaseline",children:[{type:"elem",elem:n},{type:"kern",size:3*a},{type:"elem",elem:r},{type:"kern",size:a}]},t);return ae.makeSpan(["mord","overline"],[s],t)},mathmlBuilder(e,t){var n=new Ce.MathNode("mo",[new Ce.TextNode("‾")]);n.setAttribute("stretchy","true");var r=new Ce.MathNode("mover",[sn(e.body,t),n]);return r.setAttribute("accent","true"),r}});Ye({type:"phantom",names:["\\phantom"],props:{numArgs:1,allowedInText:!0},handler:(e,t)=>{var{parser:n}=e,r=t[0];return{type:"phantom",mode:n.mode,body:Hn(r)}},htmlBuilder:(e,t)=>{var n=sr(e.body,t.withPhantom(),!1);return ae.makeFragment(n)},mathmlBuilder:(e,t)=>{var n=aa(e.body,t);return new Ce.MathNode("mphantom",n)}});Ye({type:"hphantom",names:["\\hphantom"],props:{numArgs:1,allowedInText:!0},handler:(e,t)=>{var{parser:n}=e,r=t[0];return{type:"hphantom",mode:n.mode,body:r}},htmlBuilder:(e,t)=>{var n=ae.makeSpan([],[Ft(e.body,t.withPhantom())]);if(n.height=0,n.depth=0,n.children)for(var r=0;r<n.children.length;r++)n.children[r].height=0,n.children[r].depth=0;return n=ae.makeVList({positionType:"firstBaseline",children:[{type:"elem",elem:n}]},t),ae.makeSpan(["mord"],[n],t)},mathmlBuilder:(e,t)=>{var n=aa(Hn(e.body),t),r=new Ce.MathNode("mphantom",n),a=new Ce.MathNode("mpadded",[r]);return a.setAttribute("height","0px"),a.setAttribute("depth","0px"),a}});Ye({type:"vphantom",names:["\\vphantom"],props:{numArgs:1,allowedInText:!0},handler:(e,t)=>{var{parser:n}=e,r=t[0];return{type:"vphantom",mode:n.mode,body:r}},htmlBuilder:(e,t)=>{var n=ae.makeSpan(["inner"],[Ft(e.body,t.withPhantom())]),r=ae.makeSpan(["fix"],[]);return ae.makeSpan(["mord","rlap"],[n,r],t)},mathmlBuilder:(e,t)=>{var n=aa(Hn(e.body),t),r=new Ce.MathNode("mphantom",n),a=new Ce.MathNode("mpadded",[r]);return a.setAttribute("width","0px"),a}});Ye({type:"raisebox",names:["\\raisebox"],props:{numArgs:2,argTypes:["size","hbox"],allowedInText:!0},handler(e,t){var{parser:n}=e,r=St(t[0],"size").value,a=t[1];return{type:"raisebox",mode:n.mode,dy:r,body:a}},htmlBuilder(e,t){var n=Ft(e.body,t),r=Rn(e.dy,t);return ae.makeVList({positionType:"shift",positionData:-r,children:[{type:"elem",elem:n}]},t)},mathmlBuilder(e,t){var n=new Ce.MathNode("mpadded",[sn(e.body,t)]),r=e.dy.number+e.dy.unit;return n.setAttribute("voffset",r),n}});Ye({type:"internal",names:["\\relax"],props:{numArgs:0,allowedInText:!0,allowedInArgument:!0},handler(e){var{parser:t}=e;return{type:"internal",mode:t.mode}}});Ye({type:"rule",names:["\\rule"],props:{numArgs:2,numOptionalArgs:1,allowedInText:!0,allowedInMath:!0,argTypes:["size","size","size"]},handler(e,t,n){var{parser:r}=e,a=n[0],s=St(t[0],"size"),o=St(t[1],"size");return{type:"rule",mode:r.mode,shift:a&&St(a,"size").value,width:s.value,height:o.value}},htmlBuilder(e,t){var n=ae.makeSpan(["mord","rule"],[],t),r=Rn(e.width,t),a=Rn(e.height,t),s=e.shift?Rn(e.shift,t):0;return n.style.borderRightWidth=Le(r),n.style.borderTopWidth=Le(a),n.style.bottom=Le(s),n.width=r,n.height=a+s,n.depth=-s,n.maxFontSize=a*1.125*t.sizeMultiplier,n},mathmlBuilder(e,t){var n=Rn(e.width,t),r=Rn(e.height,t),a=e.shift?Rn(e.shift,t):0,s=t.color&&t.getColor()||"black",o=new Ce.MathNode("mspace");o.setAttribute("mathbackground",s),o.setAttribute("width",Le(n)),o.setAttribute("height",Le(r));var c=new Ce.MathNode("mpadded",[o]);return a>=0?c.setAttribute("height",Le(a)):(c.setAttribute("height",Le(a)),c.setAttribute("depth",Le(-a))),c.setAttribute("voffset",Le(a)),c}});function tS(e,t,n){for(var r=sr(e,t,!1),a=t.sizeMultiplier/n.sizeMultiplier,s=0;s<r.length;s++){var o=r[s].classes.indexOf("sizing");o<0?Array.prototype.push.apply(r[s].classes,t.sizingClasses(n)):r[s].classes[o+1]==="reset-size"+t.size&&(r[s].classes[o+1]="reset-size"+n.size),r[s].height*=a,r[s].depth*=a}return ae.makeFragment(r)}var E6=["\\tiny","\\sixptsize","\\scriptsize","\\footnotesize","\\small","\\normalsize","\\large","\\Large","\\LARGE","\\huge","\\Huge"],f$=(e,t)=>{var n=t.havingSize(e.size);return tS(e.body,n,t)};Ye({type:"sizing",names:E6,props:{numArgs:0,allowedInText:!0},handler:(e,t)=>{var{breakOnTokenText:n,funcName:r,parser:a}=e,s=a.parseExpression(!1,n);return{type:"sizing",mode:a.mode,size:E6.indexOf(r)+1,body:s}},htmlBuilder:f$,mathmlBuilder:(e,t)=>{var n=t.havingSize(e.size),r=aa(e.body,n),a=new Ce.MathNode("mstyle",r);return a.setAttribute("mathsize",Le(n.sizeMultiplier)),a}});Ye({type:"smash",names:["\\smash"],props:{numArgs:1,numOptionalArgs:1,allowedInText:!0},handler:(e,t,n)=>{var{parser:r}=e,a=!1,s=!1,o=n[0]&&St(n[0],"ordgroup");if(o)for(var c="",d=0;d<o.body.length;++d){var f=o.body[d];if(c=f.text,c==="t")a=!0;else if(c==="b")s=!0;else{a=!1,s=!1;break}}else a=!0,s=!0;var m=t[0];return{type:"smash",mode:r.mode,body:m,smashHeight:a,smashDepth:s}},htmlBuilder:(e,t)=>{var n=ae.makeSpan([],[Ft(e.body,t)]);if(!e.smashHeight&&!e.smashDepth)return n;if(e.smashHeight&&(n.height=0,n.children))for(var r=0;r<n.children.length;r++)n.children[r].height=0;if(e.smashDepth&&(n.depth=0,n.children))for(var a=0;a<n.children.length;a++)n.children[a].depth=0;var s=ae.makeVList({positionType:"firstBaseline",children:[{type:"elem",elem:n}]},t);return ae.makeSpan(["mord"],[s],t)},mathmlBuilder:(e,t)=>{var n=new Ce.MathNode("mpadded",[sn(e.body,t)]);return e.smashHeight&&n.setAttribute("height","0px"),e.smashDepth&&n.setAttribute("depth","0px"),n}});Ye({type:"sqrt",names:["\\sqrt"],props:{numArgs:1,numOptionalArgs:1},handler(e,t,n){var{parser:r}=e,a=n[0],s=t[0];return{type:"sqrt",mode:r.mode,body:s,index:a}},htmlBuilder(e,t){var n=Ft(e.body,t.havingCrampedStyle());n.height===0&&(n.height=t.fontMetrics().xHeight),n=ae.wrapFragment(n,t);var r=t.fontMetrics(),a=r.defaultRuleThickness,s=a;t.style.id<lt.TEXT.id&&(s=t.fontMetrics().xHeight);var o=a+s/4,c=n.height+n.depth+o+a,{span:d,ruleWidth:f,advanceWidth:m}=ts.sqrtImage(c,t),p=d.height-f;p>n.height+n.depth+o&&(o=(o+p-n.height-n.depth)/2);var b=d.height-n.height-o-f;n.style.paddingLeft=Le(m);var y=ae.makeVList({positionType:"firstBaseline",children:[{type:"elem",elem:n,wrapperClasses:["svg-align"]},{type:"kern",size:-(n.height+b)},{type:"elem",elem:d},{type:"kern",size:f}]},t);if(e.index){var T=t.havingStyle(lt.SCRIPTSCRIPT),x=Ft(e.index,T,t),C=.6*(y.height-y.depth),w=ae.makeVList({positionType:"shift",positionData:-C,children:[{type:"elem",elem:x}]},t),O=ae.makeSpan(["root"],[w]);return ae.makeSpan(["mord","sqrt"],[O,y],t)}else return ae.makeSpan(["mord","sqrt"],[y],t)},mathmlBuilder(e,t){var{body:n,index:r}=e;return r?new Ce.MathNode("mroot",[sn(n,t),sn(r,t)]):new Ce.MathNode("msqrt",[sn(n,t)])}});var T6={display:lt.DISPLAY,text:lt.TEXT,script:lt.SCRIPT,scriptscript:lt.SCRIPTSCRIPT};Ye({type:"styling",names:["\\displaystyle","\\textstyle","\\scriptstyle","\\scriptscriptstyle"],props:{numArgs:0,allowedInText:!0,primitive:!0},handler(e,t){var{breakOnTokenText:n,funcName:r,parser:a}=e,s=a.parseExpression(!0,n),o=r.slice(1,r.length-5);return{type:"styling",mode:a.mode,style:o,body:s}},htmlBuilder(e,t){var n=T6[e.style],r=t.havingStyle(n).withFont("");return tS(e.body,r,t)},mathmlBuilder(e,t){var n=T6[e.style],r=t.havingStyle(n),a=aa(e.body,r),s=new Ce.MathNode("mstyle",a),o={display:["0","true"],text:["0","false"],script:["1","false"],scriptscript:["2","false"]},c=o[e.style];return s.setAttribute("scriptlevel",c[0]),s.setAttribute("displaystyle",c[1]),s}});var m$=function(t,n){var r=t.base;if(r)if(r.type==="op"){var a=r.limits&&(n.style.size===lt.DISPLAY.size||r.alwaysHandleSupSub);return a?yu:null}else if(r.type==="operatorname"){var s=r.alwaysHandleSupSub&&(n.style.size===lt.DISPLAY.size||r.limits);return s?eS:null}else{if(r.type==="accent")return it.isCharacterBox(r.base)?Y2:null;if(r.type==="horizBrace"){var o=!t.sub;return o===r.isOver?Qx:null}else return null}else return null};to({type:"supsub",htmlBuilder(e,t){var n=m$(e,t);if(n)return n(e,t);var{base:r,sup:a,sub:s}=e,o=Ft(r,t),c,d,f=t.fontMetrics(),m=0,p=0,b=r&&it.isCharacterBox(r);if(a){var y=t.havingStyle(t.style.sup());c=Ft(a,y,t),b||(m=o.height-y.fontMetrics().supDrop*y.sizeMultiplier/t.sizeMultiplier)}if(s){var T=t.havingStyle(t.style.sub());d=Ft(s,T,t),b||(p=o.depth+T.fontMetrics().subDrop*T.sizeMultiplier/t.sizeMultiplier)}var x;t.style===lt.DISPLAY?x=f.sup1:t.style.cramped?x=f.sup3:x=f.sup2;var C=t.sizeMultiplier,w=Le(.5/f.ptPerEm/C),O=null;if(d){var D=e.base&&e.base.type==="op"&&e.base.name&&(e.base.name==="\\oiint"||e.base.name==="\\oiiint");(o instanceof Ua||D)&&(O=Le(-o.italic))}var Y;if(c&&d){m=Math.max(m,x,c.depth+.25*f.xHeight),p=Math.max(p,f.sub2);var H=f.defaultRuleThickness,B=4*H;if(m-c.depth-(d.height-p)<B){p=B-(m-c.depth)+d.height;var ee=.8*f.xHeight-(m-c.depth);ee>0&&(m+=ee,p-=ee)}var Z=[{type:"elem",elem:d,shift:p,marginRight:w,marginLeft:O},{type:"elem",elem:c,shift:-m,marginRight:w}];Y=ae.makeVList({positionType:"individualShift",children:Z},t)}else if(d){p=Math.max(p,f.sub1,d.height-.8*f.xHeight);var Q=[{type:"elem",elem:d,marginLeft:O,marginRight:w}];Y=ae.makeVList({positionType:"shift",positionData:p,children:Q},t)}else if(c)m=Math.max(m,x,c.depth+.25*f.xHeight),Y=ae.makeVList({positionType:"shift",positionData:-m,children:[{type:"elem",elem:c,marginRight:w}]},t);else throw new Error("supsub must have either sup or sub.");var G=_b(o,"right")||"mord";return ae.makeSpan([G],[o,ae.makeSpan(["msupsub"],[Y])],t)},mathmlBuilder(e,t){var n=!1,r,a;e.base&&e.base.type==="horizBrace"&&(a=!!e.sup,a===e.base.isOver&&(n=!0,r=e.base.isOver)),e.base&&(e.base.type==="op"||e.base.type==="operatorname")&&(e.base.parentIsSupSub=!0);var s=[sn(e.base,t)];e.sub&&s.push(sn(e.sub,t)),e.sup&&s.push(sn(e.sup,t));var o;if(n)o=r?"mover":"munder";else if(e.sub)if(e.sup){var f=e.base;f&&f.type==="op"&&f.limits&&t.style===lt.DISPLAY||f&&f.type==="operatorname"&&f.alwaysHandleSupSub&&(t.style===lt.DISPLAY||f.limits)?o="munderover":o="msubsup"}else{var d=e.base;d&&d.type==="op"&&d.limits&&(t.style===lt.DISPLAY||d.alwaysHandleSupSub)||d&&d.type==="operatorname"&&d.alwaysHandleSupSub&&(d.limits||t.style===lt.DISPLAY)?o="munder":o="msub"}else{var c=e.base;c&&c.type==="op"&&c.limits&&(t.style===lt.DISPLAY||c.alwaysHandleSupSub)||c&&c.type==="operatorname"&&c.alwaysHandleSupSub&&(c.limits||t.style===lt.DISPLAY)?o="mover":o="msup"}return new Ce.MathNode(o,s)}});to({type:"atom",htmlBuilder(e,t){return ae.mathsym(e.text,e.mode,t,["m"+e.family])},mathmlBuilder(e,t){var n=new Ce.MathNode("mo",[Fa(e.text,e.mode)]);if(e.family==="bin"){var r=j2(e,t);r==="bold-italic"&&n.setAttribute("mathvariant",r)}else e.family==="punct"?n.setAttribute("separator","true"):(e.family==="open"||e.family==="close")&&n.setAttribute("stretchy","false");return n}});var nS={mi:"italic",mn:"normal",mtext:"normal"};to({type:"mathord",htmlBuilder(e,t){return ae.makeOrd(e,t,"mathord")},mathmlBuilder(e,t){var n=new Ce.MathNode("mi",[Fa(e.text,e.mode,t)]),r=j2(e,t)||"italic";return r!==nS[n.type]&&n.setAttribute("mathvariant",r),n}});to({type:"textord",htmlBuilder(e,t){return ae.makeOrd(e,t,"textord")},mathmlBuilder(e,t){var n=Fa(e.text,e.mode,t),r=j2(e,t)||"normal",a;return e.mode==="text"?a=new Ce.MathNode("mtext",[n]):/[0-9]/.test(e.text)?a=new Ce.MathNode("mn",[n]):e.text==="\\prime"?a=new Ce.MathNode("mo",[n]):a=new Ce.MathNode("mi",[n]),r!==nS[a.type]&&a.setAttribute("mathvariant",r),a}});var mg={"\\nobreak":"nobreak","\\allowbreak":"allowbreak"},pg={" ":{},"\\ ":{},"~":{className:"nobreak"},"\\space":{},"\\nobreakspace":{className:"nobreak"}};to({type:"spacing",htmlBuilder(e,t){if(pg.hasOwnProperty(e.text)){var n=pg[e.text].className||"";if(e.mode==="text"){var r=ae.makeOrd(e,t,"textord");return r.classes.push(n),r}else return ae.makeSpan(["mspace",n],[ae.mathsym(e.text,e.mode,t)],t)}else{if(mg.hasOwnProperty(e.text))return ae.makeSpan(["mspace",mg[e.text]],[],t);throw new Ne('Unknown type of space "'+e.text+'"')}},mathmlBuilder(e,t){var n;if(pg.hasOwnProperty(e.text))n=new Ce.MathNode("mtext",[new Ce.TextNode(" ")]);else{if(mg.hasOwnProperty(e.text))return new Ce.MathNode("mspace");throw new Ne('Unknown type of space "'+e.text+'"')}return n}});var x6=()=>{var e=new Ce.MathNode("mtd",[]);return e.setAttribute("width","50%"),e};to({type:"tag",mathmlBuilder(e,t){var n=new Ce.MathNode("mtable",[new Ce.MathNode("mtr",[x6(),new Ce.MathNode("mtd",[ll(e.body,t)]),x6(),new Ce.MathNode("mtd",[ll(e.tag,t)])])]);return n.setAttribute("width","100%"),n}});var S6={"\\text":void 0,"\\textrm":"textrm","\\textsf":"textsf","\\texttt":"texttt","\\textnormal":"textrm"},w6={"\\textbf":"textbf","\\textmd":"textmd"},p$={"\\textit":"textit","\\textup":"textup"},A6=(e,t)=>{var n=e.font;if(n){if(S6[n])return t.withTextFontFamily(S6[n]);if(w6[n])return t.withTextFontWeight(w6[n]);if(n==="\\emph")return t.fontShape==="textit"?t.withTextFontShape("textup"):t.withTextFontShape("textit")}else return t;return t.withTextFontShape(p$[n])};Ye({type:"text",names:["\\text","\\textrm","\\textsf","\\texttt","\\textnormal","\\textbf","\\textmd","\\textit","\\textup","\\emph"],props:{numArgs:1,argTypes:["text"],allowedInArgument:!0,allowedInText:!0},handler(e,t){var{parser:n,funcName:r}=e,a=t[0];return{type:"text",mode:n.mode,body:Hn(a),font:r}},htmlBuilder(e,t){var n=A6(e,t),r=sr(e.body,n,!0);return ae.makeSpan(["mord","text"],r,n)},mathmlBuilder(e,t){var n=A6(e,t);return ll(e.body,n)}});Ye({type:"underline",names:["\\underline"],props:{numArgs:1,allowedInText:!0},handler(e,t){var{parser:n}=e;return{type:"underline",mode:n.mode,body:t[0]}},htmlBuilder(e,t){var n=Ft(e.body,t),r=ae.makeLineSpan("underline-line",t),a=t.fontMetrics().defaultRuleThickness,s=ae.makeVList({positionType:"top",positionData:n.height,children:[{type:"kern",size:a},{type:"elem",elem:r},{type:"kern",size:3*a},{type:"elem",elem:n}]},t);return ae.makeSpan(["mord","underline"],[s],t)},mathmlBuilder(e,t){var n=new Ce.MathNode("mo",[new Ce.TextNode("‾")]);n.setAttribute("stretchy","true");var r=new Ce.MathNode("munder",[sn(e.body,t),n]);return r.setAttribute("accentunder","true"),r}});Ye({type:"vcenter",names:["\\vcenter"],props:{numArgs:1,argTypes:["original"],allowedInText:!1},handler(e,t){var{parser:n}=e;return{type:"vcenter",mode:n.mode,body:t[0]}},htmlBuilder(e,t){var n=Ft(e.body,t),r=t.fontMetrics().axisHeight,a=.5*(n.height-r-(n.depth+r));return ae.makeVList({positionType:"shift",positionData:a,children:[{type:"elem",elem:n}]},t)},mathmlBuilder(e,t){return new Ce.MathNode("mpadded",[sn(e.body,t)],["vcenter"])}});Ye({type:"verb",names:["\\verb"],props:{numArgs:0,allowedInText:!0},handler(e,t,n){throw new Ne("\\verb ended by end of line instead of matching delimiter")},htmlBuilder(e,t){for(var n=k6(e),r=[],a=t.havingStyle(t.style.text()),s=0;s<n.length;s++){var o=n[s];o==="~"&&(o="\\textasciitilde"),r.push(ae.makeSymbol(o,"Typewriter-Regular",e.mode,a,["mord","texttt"]))}return ae.makeSpan(["mord","text"].concat(a.sizingClasses(t)),ae.tryCombineChars(r),a)},mathmlBuilder(e,t){var n=new Ce.TextNode(k6(e)),r=new Ce.MathNode("mtext",[n]);return r.setAttribute("mathvariant","monospace"),r}});var k6=e=>e.body.replace(/ /g,e.star?"␣":" "),Js=wx,rS=`[ \r
	]`,g$="\\\\[a-zA-Z@]+",b$="\\\\[^\uD800-\uDFFF]",v$="("+g$+")"+rS+"*",y$=`\\\\(
|[ \r	]+
?)[ \r	]*`,Sb="[̀-ͯ]",_$=new RegExp(Sb+"+$"),E$="("+rS+"+)|"+(y$+"|")+"([!-\\[\\]-‧‪-퟿豈-]"+(Sb+"*")+"|[\uD800-\uDBFF][\uDC00-\uDFFF]"+(Sb+"*")+"|\\\\verb\\*([^]).*?\\4|\\\\verb([^*a-zA-Z]).*?\\5"+("|"+v$)+("|"+b$+")");class C6{constructor(t,n){this.input=void 0,this.settings=void 0,this.tokenRegex=void 0,this.catcodes=void 0,this.input=t,this.settings=n,this.tokenRegex=new RegExp(E$,"g"),this.catcodes={"%":14,"~":13}}setCatcode(t,n){this.catcodes[t]=n}lex(){var t=this.input,n=this.tokenRegex.lastIndex;if(n===t.length)return new Da("EOF",new ba(this,n,n));var r=this.tokenRegex.exec(t);if(r===null||r.index!==n)throw new Ne("Unexpected character: '"+t[n]+"'",new Da(t[n],new ba(this,n,n+1)));var a=r[6]||r[3]||(r[2]?"\\ ":" ");if(this.catcodes[a]===14){var s=t.indexOf(`
`,this.tokenRegex.lastIndex);return s===-1?(this.tokenRegex.lastIndex=t.length,this.settings.reportNonstrict("commentAtEnd","% comment has no terminating newline; LaTeX would fail because of commenting the end of math mode (e.g. $)")):this.tokenRegex.lastIndex=s+1,this.lex()}return new Da(a,new ba(this,n,this.tokenRegex.lastIndex))}}class T${constructor(t,n){t===void 0&&(t={}),n===void 0&&(n={}),this.current=void 0,this.builtins=void 0,this.undefStack=void 0,this.current=n,this.builtins=t,this.undefStack=[]}beginGroup(){this.undefStack.push({})}endGroup(){if(this.undefStack.length===0)throw new Ne("Unbalanced namespace destruction: attempt to pop global namespace; please report this as a bug");var t=this.undefStack.pop();for(var n in t)t.hasOwnProperty(n)&&(t[n]==null?delete this.current[n]:this.current[n]=t[n])}endGroups(){for(;this.undefStack.length>0;)this.endGroup()}has(t){return this.current.hasOwnProperty(t)||this.builtins.hasOwnProperty(t)}get(t){return this.current.hasOwnProperty(t)?this.current[t]:this.builtins[t]}set(t,n,r){if(r===void 0&&(r=!1),r){for(var a=0;a<this.undefStack.length;a++)delete this.undefStack[a][t];this.undefStack.length>0&&(this.undefStack[this.undefStack.length-1][t]=n)}else{var s=this.undefStack[this.undefStack.length-1];s&&!s.hasOwnProperty(t)&&(s[t]=this.current[t])}n==null?delete this.current[t]:this.current[t]=n}}var x$=Gx;I("\\noexpand",function(e){var t=e.popToken();return e.isExpandable(t.text)&&(t.noexpand=!0,t.treatAsRelax=!0),{tokens:[t],numArgs:0}});I("\\expandafter",function(e){var t=e.popToken();return e.expandOnce(!0),{tokens:[t],numArgs:0}});I("\\@firstoftwo",function(e){var t=e.consumeArgs(2);return{tokens:t[0],numArgs:0}});I("\\@secondoftwo",function(e){var t=e.consumeArgs(2);return{tokens:t[1],numArgs:0}});I("\\@ifnextchar",function(e){var t=e.consumeArgs(3);e.consumeSpaces();var n=e.future();return t[0].length===1&&t[0][0].text===n.text?{tokens:t[1],numArgs:0}:{tokens:t[2],numArgs:0}});I("\\@ifstar","\\@ifnextchar *{\\@firstoftwo{#1}}");I("\\TextOrMath",function(e){var t=e.consumeArgs(2);return e.mode==="text"?{tokens:t[0],numArgs:0}:{tokens:t[1],numArgs:0}});var N6={0:0,1:1,2:2,3:3,4:4,5:5,6:6,7:7,8:8,9:9,a:10,A:10,b:11,B:11,c:12,C:12,d:13,D:13,e:14,E:14,f:15,F:15};I("\\char",function(e){var t=e.popToken(),n,r="";if(t.text==="'")n=8,t=e.popToken();else if(t.text==='"')n=16,t=e.popToken();else if(t.text==="`")if(t=e.popToken(),t.text[0]==="\\")r=t.text.charCodeAt(1);else{if(t.text==="EOF")throw new Ne("\\char` missing argument");r=t.text.charCodeAt(0)}else n=10;if(n){if(r=N6[t.text],r==null||r>=n)throw new Ne("Invalid base-"+n+" digit "+t.text);for(var a;(a=N6[e.future().text])!=null&&a<n;)r*=n,r+=a,e.popToken()}return"\\@char{"+r+"}"});var J2=(e,t,n,r)=>{var a=e.consumeArg().tokens;if(a.length!==1)throw new Ne("\\newcommand's first argument must be a macro name");var s=a[0].text,o=e.isDefined(s);if(o&&!t)throw new Ne("\\newcommand{"+s+"} attempting to redefine "+(s+"; use \\renewcommand"));if(!o&&!n)throw new Ne("\\renewcommand{"+s+"} when command "+s+" does not yet exist; use \\newcommand");var c=0;if(a=e.consumeArg().tokens,a.length===1&&a[0].text==="["){for(var d="",f=e.expandNextToken();f.text!=="]"&&f.text!=="EOF";)d+=f.text,f=e.expandNextToken();if(!d.match(/^\s*[0-9]+\s*$/))throw new Ne("Invalid number of arguments: "+d);c=parseInt(d),a=e.consumeArg().tokens}return o&&r||e.macros.set(s,{tokens:a,numArgs:c}),""};I("\\newcommand",e=>J2(e,!1,!0,!1));I("\\renewcommand",e=>J2(e,!0,!1,!1));I("\\providecommand",e=>J2(e,!0,!0,!0));I("\\message",e=>{var t=e.consumeArgs(1)[0];return console.log(t.reverse().map(n=>n.text).join("")),""});I("\\errmessage",e=>{var t=e.consumeArgs(1)[0];return console.error(t.reverse().map(n=>n.text).join("")),""});I("\\show",e=>{var t=e.popToken(),n=t.text;return console.log(t,e.macros.get(n),Js[n],fn.math[n],fn.text[n]),""});I("\\bgroup","{");I("\\egroup","}");I("~","\\nobreakspace");I("\\lq","`");I("\\rq","'");I("\\aa","\\r a");I("\\AA","\\r A");I("\\textcopyright","\\html@mathml{\\textcircled{c}}{\\char`©}");I("\\copyright","\\TextOrMath{\\textcopyright}{\\text{\\textcopyright}}");I("\\textregistered","\\html@mathml{\\textcircled{\\scriptsize R}}{\\char`®}");I("ℬ","\\mathscr{B}");I("ℰ","\\mathscr{E}");I("ℱ","\\mathscr{F}");I("ℋ","\\mathscr{H}");I("ℐ","\\mathscr{I}");I("ℒ","\\mathscr{L}");I("ℳ","\\mathscr{M}");I("ℛ","\\mathscr{R}");I("ℭ","\\mathfrak{C}");I("ℌ","\\mathfrak{H}");I("ℨ","\\mathfrak{Z}");I("\\Bbbk","\\Bbb{k}");I("·","\\cdotp");I("\\llap","\\mathllap{\\textrm{#1}}");I("\\rlap","\\mathrlap{\\textrm{#1}}");I("\\clap","\\mathclap{\\textrm{#1}}");I("\\mathstrut","\\vphantom{(}");I("\\underbar","\\underline{\\text{#1}}");I("\\not",'\\html@mathml{\\mathrel{\\mathrlap\\@not}}{\\char"338}');I("\\neq","\\html@mathml{\\mathrel{\\not=}}{\\mathrel{\\char`≠}}");I("\\ne","\\neq");I("≠","\\neq");I("\\notin","\\html@mathml{\\mathrel{{\\in}\\mathllap{/\\mskip1mu}}}{\\mathrel{\\char`∉}}");I("∉","\\notin");I("≘","\\html@mathml{\\mathrel{=\\kern{-1em}\\raisebox{0.4em}{$\\scriptsize\\frown$}}}{\\mathrel{\\char`≘}}");I("≙","\\html@mathml{\\stackrel{\\tiny\\wedge}{=}}{\\mathrel{\\char`≘}}");I("≚","\\html@mathml{\\stackrel{\\tiny\\vee}{=}}{\\mathrel{\\char`≚}}");I("≛","\\html@mathml{\\stackrel{\\scriptsize\\star}{=}}{\\mathrel{\\char`≛}}");I("≝","\\html@mathml{\\stackrel{\\tiny\\mathrm{def}}{=}}{\\mathrel{\\char`≝}}");I("≞","\\html@mathml{\\stackrel{\\tiny\\mathrm{m}}{=}}{\\mathrel{\\char`≞}}");I("≟","\\html@mathml{\\stackrel{\\tiny?}{=}}{\\mathrel{\\char`≟}}");I("⟂","\\perp");I("‼","\\mathclose{!\\mkern-0.8mu!}");I("∌","\\notni");I("⌜","\\ulcorner");I("⌝","\\urcorner");I("⌞","\\llcorner");I("⌟","\\lrcorner");I("©","\\copyright");I("®","\\textregistered");I("️","\\textregistered");I("\\ulcorner",'\\html@mathml{\\@ulcorner}{\\mathop{\\char"231c}}');I("\\urcorner",'\\html@mathml{\\@urcorner}{\\mathop{\\char"231d}}');I("\\llcorner",'\\html@mathml{\\@llcorner}{\\mathop{\\char"231e}}');I("\\lrcorner",'\\html@mathml{\\@lrcorner}{\\mathop{\\char"231f}}');I("\\vdots","{\\varvdots\\rule{0pt}{15pt}}");I("⋮","\\vdots");I("\\varGamma","\\mathit{\\Gamma}");I("\\varDelta","\\mathit{\\Delta}");I("\\varTheta","\\mathit{\\Theta}");I("\\varLambda","\\mathit{\\Lambda}");I("\\varXi","\\mathit{\\Xi}");I("\\varPi","\\mathit{\\Pi}");I("\\varSigma","\\mathit{\\Sigma}");I("\\varUpsilon","\\mathit{\\Upsilon}");I("\\varPhi","\\mathit{\\Phi}");I("\\varPsi","\\mathit{\\Psi}");I("\\varOmega","\\mathit{\\Omega}");I("\\substack","\\begin{subarray}{c}#1\\end{subarray}");I("\\colon","\\nobreak\\mskip2mu\\mathpunct{}\\mathchoice{\\mkern-3mu}{\\mkern-3mu}{}{}{:}\\mskip6mu\\relax");I("\\boxed","\\fbox{$\\displaystyle{#1}$}");I("\\iff","\\DOTSB\\;\\Longleftrightarrow\\;");I("\\implies","\\DOTSB\\;\\Longrightarrow\\;");I("\\impliedby","\\DOTSB\\;\\Longleftarrow\\;");I("\\dddot","{\\overset{\\raisebox{-0.1ex}{\\normalsize ...}}{#1}}");I("\\ddddot","{\\overset{\\raisebox{-0.1ex}{\\normalsize ....}}{#1}}");var O6={",":"\\dotsc","\\not":"\\dotsb","+":"\\dotsb","=":"\\dotsb","<":"\\dotsb",">":"\\dotsb","-":"\\dotsb","*":"\\dotsb",":":"\\dotsb","\\DOTSB":"\\dotsb","\\coprod":"\\dotsb","\\bigvee":"\\dotsb","\\bigwedge":"\\dotsb","\\biguplus":"\\dotsb","\\bigcap":"\\dotsb","\\bigcup":"\\dotsb","\\prod":"\\dotsb","\\sum":"\\dotsb","\\bigotimes":"\\dotsb","\\bigoplus":"\\dotsb","\\bigodot":"\\dotsb","\\bigsqcup":"\\dotsb","\\And":"\\dotsb","\\longrightarrow":"\\dotsb","\\Longrightarrow":"\\dotsb","\\longleftarrow":"\\dotsb","\\Longleftarrow":"\\dotsb","\\longleftrightarrow":"\\dotsb","\\Longleftrightarrow":"\\dotsb","\\mapsto":"\\dotsb","\\longmapsto":"\\dotsb","\\hookrightarrow":"\\dotsb","\\doteq":"\\dotsb","\\mathbin":"\\dotsb","\\mathrel":"\\dotsb","\\relbar":"\\dotsb","\\Relbar":"\\dotsb","\\xrightarrow":"\\dotsb","\\xleftarrow":"\\dotsb","\\DOTSI":"\\dotsi","\\int":"\\dotsi","\\oint":"\\dotsi","\\iint":"\\dotsi","\\iiint":"\\dotsi","\\iiiint":"\\dotsi","\\idotsint":"\\dotsi","\\DOTSX":"\\dotsx"};I("\\dots",function(e){var t="\\dotso",n=e.expandAfterFuture().text;return n in O6?t=O6[n]:(n.slice(0,4)==="\\not"||n in fn.math&&it.contains(["bin","rel"],fn.math[n].group))&&(t="\\dotsb"),t});var ev={")":!0,"]":!0,"\\rbrack":!0,"\\}":!0,"\\rbrace":!0,"\\rangle":!0,"\\rceil":!0,"\\rfloor":!0,"\\rgroup":!0,"\\rmoustache":!0,"\\right":!0,"\\bigr":!0,"\\biggr":!0,"\\Bigr":!0,"\\Biggr":!0,$:!0,";":!0,".":!0,",":!0};I("\\dotso",function(e){var t=e.future().text;return t in ev?"\\ldots\\,":"\\ldots"});I("\\dotsc",function(e){var t=e.future().text;return t in ev&&t!==","?"\\ldots\\,":"\\ldots"});I("\\cdots",function(e){var t=e.future().text;return t in ev?"\\@cdots\\,":"\\@cdots"});I("\\dotsb","\\cdots");I("\\dotsm","\\cdots");I("\\dotsi","\\!\\cdots");I("\\dotsx","\\ldots\\,");I("\\DOTSI","\\relax");I("\\DOTSB","\\relax");I("\\DOTSX","\\relax");I("\\tmspace","\\TextOrMath{\\kern#1#3}{\\mskip#1#2}\\relax");I("\\,","\\tmspace+{3mu}{.1667em}");I("\\thinspace","\\,");I("\\>","\\mskip{4mu}");I("\\:","\\tmspace+{4mu}{.2222em}");I("\\medspace","\\:");I("\\;","\\tmspace+{5mu}{.2777em}");I("\\thickspace","\\;");I("\\!","\\tmspace-{3mu}{.1667em}");I("\\negthinspace","\\!");I("\\negmedspace","\\tmspace-{4mu}{.2222em}");I("\\negthickspace","\\tmspace-{5mu}{.277em}");I("\\enspace","\\kern.5em ");I("\\enskip","\\hskip.5em\\relax");I("\\quad","\\hskip1em\\relax");I("\\qquad","\\hskip2em\\relax");I("\\tag","\\@ifstar\\tag@literal\\tag@paren");I("\\tag@paren","\\tag@literal{({#1})}");I("\\tag@literal",e=>{if(e.macros.get("\\df@tag"))throw new Ne("Multiple \\tag");return"\\gdef\\df@tag{\\text{#1}}"});I("\\bmod","\\mathchoice{\\mskip1mu}{\\mskip1mu}{\\mskip5mu}{\\mskip5mu}\\mathbin{\\rm mod}\\mathchoice{\\mskip1mu}{\\mskip1mu}{\\mskip5mu}{\\mskip5mu}");I("\\pod","\\allowbreak\\mathchoice{\\mkern18mu}{\\mkern8mu}{\\mkern8mu}{\\mkern8mu}(#1)");I("\\pmod","\\pod{{\\rm mod}\\mkern6mu#1}");I("\\mod","\\allowbreak\\mathchoice{\\mkern18mu}{\\mkern12mu}{\\mkern12mu}{\\mkern12mu}{\\rm mod}\\,\\,#1");I("\\newline","\\\\\\relax");I("\\TeX","\\textrm{\\html@mathml{T\\kern-.1667em\\raisebox{-.5ex}{E}\\kern-.125emX}{TeX}}");var aS=Le(hi["Main-Regular"][84][1]-.7*hi["Main-Regular"][65][1]);I("\\LaTeX","\\textrm{\\html@mathml{"+("L\\kern-.36em\\raisebox{"+aS+"}{\\scriptstyle A}")+"\\kern-.15em\\TeX}{LaTeX}}");I("\\KaTeX","\\textrm{\\html@mathml{"+("K\\kern-.17em\\raisebox{"+aS+"}{\\scriptstyle A}")+"\\kern-.15em\\TeX}{KaTeX}}");I("\\hspace","\\@ifstar\\@hspacer\\@hspace");I("\\@hspace","\\hskip #1\\relax");I("\\@hspacer","\\rule{0pt}{0pt}\\hskip #1\\relax");I("\\ordinarycolon",":");I("\\vcentcolon","\\mathrel{\\mathop\\ordinarycolon}");I("\\dblcolon",'\\html@mathml{\\mathrel{\\vcentcolon\\mathrel{\\mkern-.9mu}\\vcentcolon}}{\\mathop{\\char"2237}}');I("\\coloneqq",'\\html@mathml{\\mathrel{\\vcentcolon\\mathrel{\\mkern-1.2mu}=}}{\\mathop{\\char"2254}}');I("\\Coloneqq",'\\html@mathml{\\mathrel{\\dblcolon\\mathrel{\\mkern-1.2mu}=}}{\\mathop{\\char"2237\\char"3d}}');I("\\coloneq",'\\html@mathml{\\mathrel{\\vcentcolon\\mathrel{\\mkern-1.2mu}\\mathrel{-}}}{\\mathop{\\char"3a\\char"2212}}');I("\\Coloneq",'\\html@mathml{\\mathrel{\\dblcolon\\mathrel{\\mkern-1.2mu}\\mathrel{-}}}{\\mathop{\\char"2237\\char"2212}}');I("\\eqqcolon",'\\html@mathml{\\mathrel{=\\mathrel{\\mkern-1.2mu}\\vcentcolon}}{\\mathop{\\char"2255}}');I("\\Eqqcolon",'\\html@mathml{\\mathrel{=\\mathrel{\\mkern-1.2mu}\\dblcolon}}{\\mathop{\\char"3d\\char"2237}}');I("\\eqcolon",'\\html@mathml{\\mathrel{\\mathrel{-}\\mathrel{\\mkern-1.2mu}\\vcentcolon}}{\\mathop{\\char"2239}}');I("\\Eqcolon",'\\html@mathml{\\mathrel{\\mathrel{-}\\mathrel{\\mkern-1.2mu}\\dblcolon}}{\\mathop{\\char"2212\\char"2237}}');I("\\colonapprox",'\\html@mathml{\\mathrel{\\vcentcolon\\mathrel{\\mkern-1.2mu}\\approx}}{\\mathop{\\char"3a\\char"2248}}');I("\\Colonapprox",'\\html@mathml{\\mathrel{\\dblcolon\\mathrel{\\mkern-1.2mu}\\approx}}{\\mathop{\\char"2237\\char"2248}}');I("\\colonsim",'\\html@mathml{\\mathrel{\\vcentcolon\\mathrel{\\mkern-1.2mu}\\sim}}{\\mathop{\\char"3a\\char"223c}}');I("\\Colonsim",'\\html@mathml{\\mathrel{\\dblcolon\\mathrel{\\mkern-1.2mu}\\sim}}{\\mathop{\\char"2237\\char"223c}}');I("∷","\\dblcolon");I("∹","\\eqcolon");I("≔","\\coloneqq");I("≕","\\eqqcolon");I("⩴","\\Coloneqq");I("\\ratio","\\vcentcolon");I("\\coloncolon","\\dblcolon");I("\\colonequals","\\coloneqq");I("\\coloncolonequals","\\Coloneqq");I("\\equalscolon","\\eqqcolon");I("\\equalscoloncolon","\\Eqqcolon");I("\\colonminus","\\coloneq");I("\\coloncolonminus","\\Coloneq");I("\\minuscolon","\\eqcolon");I("\\minuscoloncolon","\\Eqcolon");I("\\coloncolonapprox","\\Colonapprox");I("\\coloncolonsim","\\Colonsim");I("\\simcolon","\\mathrel{\\sim\\mathrel{\\mkern-1.2mu}\\vcentcolon}");I("\\simcoloncolon","\\mathrel{\\sim\\mathrel{\\mkern-1.2mu}\\dblcolon}");I("\\approxcolon","\\mathrel{\\approx\\mathrel{\\mkern-1.2mu}\\vcentcolon}");I("\\approxcoloncolon","\\mathrel{\\approx\\mathrel{\\mkern-1.2mu}\\dblcolon}");I("\\notni","\\html@mathml{\\not\\ni}{\\mathrel{\\char`∌}}");I("\\limsup","\\DOTSB\\operatorname*{lim\\,sup}");I("\\liminf","\\DOTSB\\operatorname*{lim\\,inf}");I("\\injlim","\\DOTSB\\operatorname*{inj\\,lim}");I("\\projlim","\\DOTSB\\operatorname*{proj\\,lim}");I("\\varlimsup","\\DOTSB\\operatorname*{\\overline{lim}}");I("\\varliminf","\\DOTSB\\operatorname*{\\underline{lim}}");I("\\varinjlim","\\DOTSB\\operatorname*{\\underrightarrow{lim}}");I("\\varprojlim","\\DOTSB\\operatorname*{\\underleftarrow{lim}}");I("\\gvertneqq","\\html@mathml{\\@gvertneqq}{≩}");I("\\lvertneqq","\\html@mathml{\\@lvertneqq}{≨}");I("\\ngeqq","\\html@mathml{\\@ngeqq}{≱}");I("\\ngeqslant","\\html@mathml{\\@ngeqslant}{≱}");I("\\nleqq","\\html@mathml{\\@nleqq}{≰}");I("\\nleqslant","\\html@mathml{\\@nleqslant}{≰}");I("\\nshortmid","\\html@mathml{\\@nshortmid}{∤}");I("\\nshortparallel","\\html@mathml{\\@nshortparallel}{∦}");I("\\nsubseteqq","\\html@mathml{\\@nsubseteqq}{⊈}");I("\\nsupseteqq","\\html@mathml{\\@nsupseteqq}{⊉}");I("\\varsubsetneq","\\html@mathml{\\@varsubsetneq}{⊊}");I("\\varsubsetneqq","\\html@mathml{\\@varsubsetneqq}{⫋}");I("\\varsupsetneq","\\html@mathml{\\@varsupsetneq}{⊋}");I("\\varsupsetneqq","\\html@mathml{\\@varsupsetneqq}{⫌}");I("\\imath","\\html@mathml{\\@imath}{ı}");I("\\jmath","\\html@mathml{\\@jmath}{ȷ}");I("\\llbracket","\\html@mathml{\\mathopen{[\\mkern-3.2mu[}}{\\mathopen{\\char`⟦}}");I("\\rrbracket","\\html@mathml{\\mathclose{]\\mkern-3.2mu]}}{\\mathclose{\\char`⟧}}");I("⟦","\\llbracket");I("⟧","\\rrbracket");I("\\lBrace","\\html@mathml{\\mathopen{\\{\\mkern-3.2mu[}}{\\mathopen{\\char`⦃}}");I("\\rBrace","\\html@mathml{\\mathclose{]\\mkern-3.2mu\\}}}{\\mathclose{\\char`⦄}}");I("⦃","\\lBrace");I("⦄","\\rBrace");I("\\minuso","\\mathbin{\\html@mathml{{\\mathrlap{\\mathchoice{\\kern{0.145em}}{\\kern{0.145em}}{\\kern{0.1015em}}{\\kern{0.0725em}}\\circ}{-}}}{\\char`⦵}}");I("⦵","\\minuso");I("\\darr","\\downarrow");I("\\dArr","\\Downarrow");I("\\Darr","\\Downarrow");I("\\lang","\\langle");I("\\rang","\\rangle");I("\\uarr","\\uparrow");I("\\uArr","\\Uparrow");I("\\Uarr","\\Uparrow");I("\\N","\\mathbb{N}");I("\\R","\\mathbb{R}");I("\\Z","\\mathbb{Z}");I("\\alef","\\aleph");I("\\alefsym","\\aleph");I("\\Alpha","\\mathrm{A}");I("\\Beta","\\mathrm{B}");I("\\bull","\\bullet");I("\\Chi","\\mathrm{X}");I("\\clubs","\\clubsuit");I("\\cnums","\\mathbb{C}");I("\\Complex","\\mathbb{C}");I("\\Dagger","\\ddagger");I("\\diamonds","\\diamondsuit");I("\\empty","\\emptyset");I("\\Epsilon","\\mathrm{E}");I("\\Eta","\\mathrm{H}");I("\\exist","\\exists");I("\\harr","\\leftrightarrow");I("\\hArr","\\Leftrightarrow");I("\\Harr","\\Leftrightarrow");I("\\hearts","\\heartsuit");I("\\image","\\Im");I("\\infin","\\infty");I("\\Iota","\\mathrm{I}");I("\\isin","\\in");I("\\Kappa","\\mathrm{K}");I("\\larr","\\leftarrow");I("\\lArr","\\Leftarrow");I("\\Larr","\\Leftarrow");I("\\lrarr","\\leftrightarrow");I("\\lrArr","\\Leftrightarrow");I("\\Lrarr","\\Leftrightarrow");I("\\Mu","\\mathrm{M}");I("\\natnums","\\mathbb{N}");I("\\Nu","\\mathrm{N}");I("\\Omicron","\\mathrm{O}");I("\\plusmn","\\pm");I("\\rarr","\\rightarrow");I("\\rArr","\\Rightarrow");I("\\Rarr","\\Rightarrow");I("\\real","\\Re");I("\\reals","\\mathbb{R}");I("\\Reals","\\mathbb{R}");I("\\Rho","\\mathrm{P}");I("\\sdot","\\cdot");I("\\sect","\\S");I("\\spades","\\spadesuit");I("\\sub","\\subset");I("\\sube","\\subseteq");I("\\supe","\\supseteq");I("\\Tau","\\mathrm{T}");I("\\thetasym","\\vartheta");I("\\weierp","\\wp");I("\\Zeta","\\mathrm{Z}");I("\\argmin","\\DOTSB\\operatorname*{arg\\,min}");I("\\argmax","\\DOTSB\\operatorname*{arg\\,max}");I("\\plim","\\DOTSB\\mathop{\\operatorname{plim}}\\limits");I("\\bra","\\mathinner{\\langle{#1}|}");I("\\ket","\\mathinner{|{#1}\\rangle}");I("\\braket","\\mathinner{\\langle{#1}\\rangle}");I("\\Bra","\\left\\langle#1\\right|");I("\\Ket","\\left|#1\\right\\rangle");var iS=e=>t=>{var n=t.consumeArg().tokens,r=t.consumeArg().tokens,a=t.consumeArg().tokens,s=t.consumeArg().tokens,o=t.macros.get("|"),c=t.macros.get("\\|");t.macros.beginGroup();var d=p=>b=>{e&&(b.macros.set("|",o),a.length&&b.macros.set("\\|",c));var y=p;if(!p&&a.length){var T=b.future();T.text==="|"&&(b.popToken(),y=!0)}return{tokens:y?a:r,numArgs:0}};t.macros.set("|",d(!1)),a.length&&t.macros.set("\\|",d(!0));var f=t.consumeArg().tokens,m=t.expandTokens([...s,...f,...n]);return t.macros.endGroup(),{tokens:m.reverse(),numArgs:0}};I("\\bra@ket",iS(!1));I("\\bra@set",iS(!0));I("\\Braket","\\bra@ket{\\left\\langle}{\\,\\middle\\vert\\,}{\\,\\middle\\vert\\,}{\\right\\rangle}");I("\\Set","\\bra@set{\\left\\{\\:}{\\;\\middle\\vert\\;}{\\;\\middle\\Vert\\;}{\\:\\right\\}}");I("\\set","\\bra@set{\\{\\,}{\\mid}{}{\\,\\}}");I("\\angln","{\\angl n}");I("\\blue","\\textcolor{##6495ed}{#1}");I("\\orange","\\textcolor{##ffa500}{#1}");I("\\pink","\\textcolor{##ff00af}{#1}");I("\\red","\\textcolor{##df0030}{#1}");I("\\green","\\textcolor{##28ae7b}{#1}");I("\\gray","\\textcolor{gray}{#1}");I("\\purple","\\textcolor{##9d38bd}{#1}");I("\\blueA","\\textcolor{##ccfaff}{#1}");I("\\blueB","\\textcolor{##80f6ff}{#1}");I("\\blueC","\\textcolor{##63d9ea}{#1}");I("\\blueD","\\textcolor{##11accd}{#1}");I("\\blueE","\\textcolor{##0c7f99}{#1}");I("\\tealA","\\textcolor{##94fff5}{#1}");I("\\tealB","\\textcolor{##26edd5}{#1}");I("\\tealC","\\textcolor{##01d1c1}{#1}");I("\\tealD","\\textcolor{##01a995}{#1}");I("\\tealE","\\textcolor{##208170}{#1}");I("\\greenA","\\textcolor{##b6ffb0}{#1}");I("\\greenB","\\textcolor{##8af281}{#1}");I("\\greenC","\\textcolor{##74cf70}{#1}");I("\\greenD","\\textcolor{##1fab54}{#1}");I("\\greenE","\\textcolor{##0d923f}{#1}");I("\\goldA","\\textcolor{##ffd0a9}{#1}");I("\\goldB","\\textcolor{##ffbb71}{#1}");I("\\goldC","\\textcolor{##ff9c39}{#1}");I("\\goldD","\\textcolor{##e07d10}{#1}");I("\\goldE","\\textcolor{##a75a05}{#1}");I("\\redA","\\textcolor{##fca9a9}{#1}");I("\\redB","\\textcolor{##ff8482}{#1}");I("\\redC","\\textcolor{##f9685d}{#1}");I("\\redD","\\textcolor{##e84d39}{#1}");I("\\redE","\\textcolor{##bc2612}{#1}");I("\\maroonA","\\textcolor{##ffbde0}{#1}");I("\\maroonB","\\textcolor{##ff92c6}{#1}");I("\\maroonC","\\textcolor{##ed5fa6}{#1}");I("\\maroonD","\\textcolor{##ca337c}{#1}");I("\\maroonE","\\textcolor{##9e034e}{#1}");I("\\purpleA","\\textcolor{##ddd7ff}{#1}");I("\\purpleB","\\textcolor{##c6b9fc}{#1}");I("\\purpleC","\\textcolor{##aa87ff}{#1}");I("\\purpleD","\\textcolor{##7854ab}{#1}");I("\\purpleE","\\textcolor{##543b78}{#1}");I("\\mintA","\\textcolor{##f5f9e8}{#1}");I("\\mintB","\\textcolor{##edf2df}{#1}");I("\\mintC","\\textcolor{##e0e5cc}{#1}");I("\\grayA","\\textcolor{##f6f7f7}{#1}");I("\\grayB","\\textcolor{##f0f1f2}{#1}");I("\\grayC","\\textcolor{##e3e5e6}{#1}");I("\\grayD","\\textcolor{##d6d8da}{#1}");I("\\grayE","\\textcolor{##babec2}{#1}");I("\\grayF","\\textcolor{##888d93}{#1}");I("\\grayG","\\textcolor{##626569}{#1}");I("\\grayH","\\textcolor{##3b3e40}{#1}");I("\\grayI","\\textcolor{##21242c}{#1}");I("\\kaBlue","\\textcolor{##314453}{#1}");I("\\kaGreen","\\textcolor{##71B307}{#1}");var sS={"^":!0,_:!0,"\\limits":!0,"\\nolimits":!0};class S${constructor(t,n,r){this.settings=void 0,this.expansionCount=void 0,this.lexer=void 0,this.macros=void 0,this.stack=void 0,this.mode=void 0,this.settings=n,this.expansionCount=0,this.feed(t),this.macros=new T$(x$,n.macros),this.mode=r,this.stack=[]}feed(t){this.lexer=new C6(t,this.settings)}switchMode(t){this.mode=t}beginGroup(){this.macros.beginGroup()}endGroup(){this.macros.endGroup()}endGroups(){this.macros.endGroups()}future(){return this.stack.length===0&&this.pushToken(this.lexer.lex()),this.stack[this.stack.length-1]}popToken(){return this.future(),this.stack.pop()}pushToken(t){this.stack.push(t)}pushTokens(t){this.stack.push(...t)}scanArgument(t){var n,r,a;if(t){if(this.consumeSpaces(),this.future().text!=="[")return null;n=this.popToken(),{tokens:a,end:r}=this.consumeArg(["]"])}else({tokens:a,start:n,end:r}=this.consumeArg());return this.pushToken(new Da("EOF",r.loc)),this.pushTokens(a),n.range(r,"")}consumeSpaces(){for(;;){var t=this.future();if(t.text===" ")this.stack.pop();else break}}consumeArg(t){var n=[],r=t&&t.length>0;r||this.consumeSpaces();var a=this.future(),s,o=0,c=0;do{if(s=this.popToken(),n.push(s),s.text==="{")++o;else if(s.text==="}"){if(--o,o===-1)throw new Ne("Extra }",s)}else if(s.text==="EOF")throw new Ne("Unexpected end of input in a macro argument, expected '"+(t&&r?t[c]:"}")+"'",s);if(t&&r)if((o===0||o===1&&t[c]==="{")&&s.text===t[c]){if(++c,c===t.length){n.splice(-c,c);break}}else c=0}while(o!==0||r);return a.text==="{"&&n[n.length-1].text==="}"&&(n.pop(),n.shift()),n.reverse(),{tokens:n,start:a,end:s}}consumeArgs(t,n){if(n){if(n.length!==t+1)throw new Ne("The length of delimiters doesn't match the number of args!");for(var r=n[0],a=0;a<r.length;a++){var s=this.popToken();if(r[a]!==s.text)throw new Ne("Use of the macro doesn't match its definition",s)}}for(var o=[],c=0;c<t;c++)o.push(this.consumeArg(n&&n[c+1]).tokens);return o}countExpansion(t){if(this.expansionCount+=t,this.expansionCount>this.settings.maxExpand)throw new Ne("Too many expansions: infinite loop or need to increase maxExpand setting")}expandOnce(t){var n=this.popToken(),r=n.text,a=n.noexpand?null:this._getExpansion(r);if(a==null||t&&a.unexpandable){if(t&&a==null&&r[0]==="\\"&&!this.isDefined(r))throw new Ne("Undefined control sequence: "+r);return this.pushToken(n),!1}this.countExpansion(1);var s=a.tokens,o=this.consumeArgs(a.numArgs,a.delimiters);if(a.numArgs){s=s.slice();for(var c=s.length-1;c>=0;--c){var d=s[c];if(d.text==="#"){if(c===0)throw new Ne("Incomplete placeholder at end of macro body",d);if(d=s[--c],d.text==="#")s.splice(c+1,1);else if(/^[1-9]$/.test(d.text))s.splice(c,2,...o[+d.text-1]);else throw new Ne("Not a valid argument number",d)}}}return this.pushTokens(s),s.length}expandAfterFuture(){return this.expandOnce(),this.future()}expandNextToken(){for(;;)if(this.expandOnce()===!1){var t=this.stack.pop();return t.treatAsRelax&&(t.text="\\relax"),t}throw new Error}expandMacro(t){return this.macros.has(t)?this.expandTokens([new Da(t)]):void 0}expandTokens(t){var n=[],r=this.stack.length;for(this.pushTokens(t);this.stack.length>r;)if(this.expandOnce(!0)===!1){var a=this.stack.pop();a.treatAsRelax&&(a.noexpand=!1,a.treatAsRelax=!1),n.push(a)}return this.countExpansion(n.length),n}expandMacroAsText(t){var n=this.expandMacro(t);return n&&n.map(r=>r.text).join("")}_getExpansion(t){var n=this.macros.get(t);if(n==null)return n;if(t.length===1){var r=this.lexer.catcodes[t];if(r!=null&&r!==13)return}var a=typeof n=="function"?n(this):n;if(typeof a=="string"){var s=0;if(a.indexOf("#")!==-1)for(var o=a.replace(/##/g,"");o.indexOf("#"+(s+1))!==-1;)++s;for(var c=new C6(a,this.settings),d=[],f=c.lex();f.text!=="EOF";)d.push(f),f=c.lex();d.reverse();var m={tokens:d,numArgs:s};return m}return a}isDefined(t){return this.macros.has(t)||Js.hasOwnProperty(t)||fn.math.hasOwnProperty(t)||fn.text.hasOwnProperty(t)||sS.hasOwnProperty(t)}isExpandable(t){var n=this.macros.get(t);return n!=null?typeof n=="string"||typeof n=="function"||!n.unexpandable:Js.hasOwnProperty(t)&&!Js[t].primitive}}var R6=/^[₊₋₌₍₎₀₁₂₃₄₅₆₇₈₉ₐₑₕᵢⱼₖₗₘₙₒₚᵣₛₜᵤᵥₓᵦᵧᵨᵩᵪ]/,wh=Object.freeze({"₊":"+","₋":"-","₌":"=","₍":"(","₎":")","₀":"0","₁":"1","₂":"2","₃":"3","₄":"4","₅":"5","₆":"6","₇":"7","₈":"8","₉":"9","ₐ":"a","ₑ":"e","ₕ":"h","ᵢ":"i","ⱼ":"j","ₖ":"k","ₗ":"l","ₘ":"m","ₙ":"n","ₒ":"o","ₚ":"p","ᵣ":"r","ₛ":"s","ₜ":"t","ᵤ":"u","ᵥ":"v","ₓ":"x","ᵦ":"β","ᵧ":"γ","ᵨ":"ρ","ᵩ":"ϕ","ᵪ":"χ","⁺":"+","⁻":"-","⁼":"=","⁽":"(","⁾":")","⁰":"0","¹":"1","²":"2","³":"3","⁴":"4","⁵":"5","⁶":"6","⁷":"7","⁸":"8","⁹":"9","ᴬ":"A","ᴮ":"B","ᴰ":"D","ᴱ":"E","ᴳ":"G","ᴴ":"H","ᴵ":"I","ᴶ":"J","ᴷ":"K","ᴸ":"L","ᴹ":"M","ᴺ":"N","ᴼ":"O","ᴾ":"P","ᴿ":"R","ᵀ":"T","ᵁ":"U","ⱽ":"V","ᵂ":"W","ᵃ":"a","ᵇ":"b","ᶜ":"c","ᵈ":"d","ᵉ":"e","ᶠ":"f","ᵍ":"g",ʰ:"h","ⁱ":"i",ʲ:"j","ᵏ":"k",ˡ:"l","ᵐ":"m",ⁿ:"n","ᵒ":"o","ᵖ":"p",ʳ:"r",ˢ:"s","ᵗ":"t","ᵘ":"u","ᵛ":"v",ʷ:"w",ˣ:"x",ʸ:"y","ᶻ":"z","ᵝ":"β","ᵞ":"γ","ᵟ":"δ","ᵠ":"ϕ","ᵡ":"χ","ᶿ":"θ"}),gg={"́":{text:"\\'",math:"\\acute"},"̀":{text:"\\`",math:"\\grave"},"̈":{text:'\\"',math:"\\ddot"},"̃":{text:"\\~",math:"\\tilde"},"̄":{text:"\\=",math:"\\bar"},"̆":{text:"\\u",math:"\\breve"},"̌":{text:"\\v",math:"\\check"},"̂":{text:"\\^",math:"\\hat"},"̇":{text:"\\.",math:"\\dot"},"̊":{text:"\\r",math:"\\mathring"},"̋":{text:"\\H"},"̧":{text:"\\c"}},M6={á:"á",à:"à",ä:"ä",ǟ:"ǟ",ã:"ã",ā:"ā",ă:"ă",ắ:"ắ",ằ:"ằ",ẵ:"ẵ",ǎ:"ǎ",â:"â",ấ:"ấ",ầ:"ầ",ẫ:"ẫ",ȧ:"ȧ",ǡ:"ǡ",å:"å",ǻ:"ǻ",ḃ:"ḃ",ć:"ć",ḉ:"ḉ",č:"č",ĉ:"ĉ",ċ:"ċ",ç:"ç",ď:"ď",ḋ:"ḋ",ḑ:"ḑ",é:"é",è:"è",ë:"ë",ẽ:"ẽ",ē:"ē",ḗ:"ḗ",ḕ:"ḕ",ĕ:"ĕ",ḝ:"ḝ",ě:"ě",ê:"ê",ế:"ế",ề:"ề",ễ:"ễ",ė:"ė",ȩ:"ȩ",ḟ:"ḟ",ǵ:"ǵ",ḡ:"ḡ",ğ:"ğ",ǧ:"ǧ",ĝ:"ĝ",ġ:"ġ",ģ:"ģ",ḧ:"ḧ",ȟ:"ȟ",ĥ:"ĥ",ḣ:"ḣ",ḩ:"ḩ",í:"í",ì:"ì",ï:"ï",ḯ:"ḯ",ĩ:"ĩ",ī:"ī",ĭ:"ĭ",ǐ:"ǐ",î:"î",ǰ:"ǰ",ĵ:"ĵ",ḱ:"ḱ",ǩ:"ǩ",ķ:"ķ",ĺ:"ĺ",ľ:"ľ",ļ:"ļ",ḿ:"ḿ",ṁ:"ṁ",ń:"ń",ǹ:"ǹ",ñ:"ñ",ň:"ň",ṅ:"ṅ",ņ:"ņ",ó:"ó",ò:"ò",ö:"ö",ȫ:"ȫ",õ:"õ",ṍ:"ṍ",ṏ:"ṏ",ȭ:"ȭ",ō:"ō",ṓ:"ṓ",ṑ:"ṑ",ŏ:"ŏ",ǒ:"ǒ",ô:"ô",ố:"ố",ồ:"ồ",ỗ:"ỗ",ȯ:"ȯ",ȱ:"ȱ",ő:"ő",ṕ:"ṕ",ṗ:"ṗ",ŕ:"ŕ",ř:"ř",ṙ:"ṙ",ŗ:"ŗ",ś:"ś",ṥ:"ṥ",š:"š",ṧ:"ṧ",ŝ:"ŝ",ṡ:"ṡ",ş:"ş",ẗ:"ẗ",ť:"ť",ṫ:"ṫ",ţ:"ţ",ú:"ú",ù:"ù",ü:"ü",ǘ:"ǘ",ǜ:"ǜ",ǖ:"ǖ",ǚ:"ǚ",ũ:"ũ",ṹ:"ṹ",ū:"ū",ṻ:"ṻ",ŭ:"ŭ",ǔ:"ǔ",û:"û",ů:"ů",ű:"ű",ṽ:"ṽ",ẃ:"ẃ",ẁ:"ẁ",ẅ:"ẅ",ŵ:"ŵ",ẇ:"ẇ",ẘ:"ẘ",ẍ:"ẍ",ẋ:"ẋ",ý:"ý",ỳ:"ỳ",ÿ:"ÿ",ỹ:"ỹ",ȳ:"ȳ",ŷ:"ŷ",ẏ:"ẏ",ẙ:"ẙ",ź:"ź",ž:"ž",ẑ:"ẑ",ż:"ż",Á:"Á",À:"À",Ä:"Ä",Ǟ:"Ǟ",Ã:"Ã",Ā:"Ā",Ă:"Ă",Ắ:"Ắ",Ằ:"Ằ",Ẵ:"Ẵ",Ǎ:"Ǎ",Â:"Â",Ấ:"Ấ",Ầ:"Ầ",Ẫ:"Ẫ",Ȧ:"Ȧ",Ǡ:"Ǡ",Å:"Å",Ǻ:"Ǻ",Ḃ:"Ḃ",Ć:"Ć",Ḉ:"Ḉ",Č:"Č",Ĉ:"Ĉ",Ċ:"Ċ",Ç:"Ç",Ď:"Ď",Ḋ:"Ḋ",Ḑ:"Ḑ",É:"É",È:"È",Ë:"Ë",Ẽ:"Ẽ",Ē:"Ē",Ḗ:"Ḗ",Ḕ:"Ḕ",Ĕ:"Ĕ",Ḝ:"Ḝ",Ě:"Ě",Ê:"Ê",Ế:"Ế",Ề:"Ề",Ễ:"Ễ",Ė:"Ė",Ȩ:"Ȩ",Ḟ:"Ḟ",Ǵ:"Ǵ",Ḡ:"Ḡ",Ğ:"Ğ",Ǧ:"Ǧ",Ĝ:"Ĝ",Ġ:"Ġ",Ģ:"Ģ",Ḧ:"Ḧ",Ȟ:"Ȟ",Ĥ:"Ĥ",Ḣ:"Ḣ",Ḩ:"Ḩ",Í:"Í",Ì:"Ì",Ï:"Ï",Ḯ:"Ḯ",Ĩ:"Ĩ",Ī:"Ī",Ĭ:"Ĭ",Ǐ:"Ǐ",Î:"Î",İ:"İ",Ĵ:"Ĵ",Ḱ:"Ḱ",Ǩ:"Ǩ",Ķ:"Ķ",Ĺ:"Ĺ",Ľ:"Ľ",Ļ:"Ļ",Ḿ:"Ḿ",Ṁ:"Ṁ",Ń:"Ń",Ǹ:"Ǹ",Ñ:"Ñ",Ň:"Ň",Ṅ:"Ṅ",Ņ:"Ņ",Ó:"Ó",Ò:"Ò",Ö:"Ö",Ȫ:"Ȫ",Õ:"Õ",Ṍ:"Ṍ",Ṏ:"Ṏ",Ȭ:"Ȭ",Ō:"Ō",Ṓ:"Ṓ",Ṑ:"Ṑ",Ŏ:"Ŏ",Ǒ:"Ǒ",Ô:"Ô",Ố:"Ố",Ồ:"Ồ",Ỗ:"Ỗ",Ȯ:"Ȯ",Ȱ:"Ȱ",Ő:"Ő",Ṕ:"Ṕ",Ṗ:"Ṗ",Ŕ:"Ŕ",Ř:"Ř",Ṙ:"Ṙ",Ŗ:"Ŗ",Ś:"Ś",Ṥ:"Ṥ",Š:"Š",Ṧ:"Ṧ",Ŝ:"Ŝ",Ṡ:"Ṡ",Ş:"Ş",Ť:"Ť",Ṫ:"Ṫ",Ţ:"Ţ",Ú:"Ú",Ù:"Ù",Ü:"Ü",Ǘ:"Ǘ",Ǜ:"Ǜ",Ǖ:"Ǖ",Ǚ:"Ǚ",Ũ:"Ũ",Ṹ:"Ṹ",Ū:"Ū",Ṻ:"Ṻ",Ŭ:"Ŭ",Ǔ:"Ǔ",Û:"Û",Ů:"Ů",Ű:"Ű",Ṽ:"Ṽ",Ẃ:"Ẃ",Ẁ:"Ẁ",Ẅ:"Ẅ",Ŵ:"Ŵ",Ẇ:"Ẇ",Ẍ:"Ẍ",Ẋ:"Ẋ",Ý:"Ý",Ỳ:"Ỳ",Ÿ:"Ÿ",Ỹ:"Ỹ",Ȳ:"Ȳ",Ŷ:"Ŷ",Ẏ:"Ẏ",Ź:"Ź",Ž:"Ž",Ẑ:"Ẑ",Ż:"Ż",ά:"ά",ὰ:"ὰ",ᾱ:"ᾱ",ᾰ:"ᾰ",έ:"έ",ὲ:"ὲ",ή:"ή",ὴ:"ὴ",ί:"ί",ὶ:"ὶ",ϊ:"ϊ",ΐ:"ΐ",ῒ:"ῒ",ῑ:"ῑ",ῐ:"ῐ",ό:"ό",ὸ:"ὸ",ύ:"ύ",ὺ:"ὺ",ϋ:"ϋ",ΰ:"ΰ",ῢ:"ῢ",ῡ:"ῡ",ῠ:"ῠ",ώ:"ώ",ὼ:"ὼ",Ύ:"Ύ",Ὺ:"Ὺ",Ϋ:"Ϋ",Ῡ:"Ῡ",Ῠ:"Ῠ",Ώ:"Ώ",Ὼ:"Ὼ"};class im{constructor(t,n){this.mode=void 0,this.gullet=void 0,this.settings=void 0,this.leftrightDepth=void 0,this.nextToken=void 0,this.mode="math",this.gullet=new S$(t,n,this.mode),this.settings=n,this.leftrightDepth=0}expect(t,n){if(n===void 0&&(n=!0),this.fetch().text!==t)throw new Ne("Expected '"+t+"', got '"+this.fetch().text+"'",this.fetch());n&&this.consume()}consume(){this.nextToken=null}fetch(){return this.nextToken==null&&(this.nextToken=this.gullet.expandNextToken()),this.nextToken}switchMode(t){this.mode=t,this.gullet.switchMode(t)}parse(){this.settings.globalGroup||this.gullet.beginGroup(),this.settings.colorIsTextColor&&this.gullet.macros.set("\\color","\\textcolor");try{var t=this.parseExpression(!1);return this.expect("EOF"),this.settings.globalGroup||this.gullet.endGroup(),t}finally{this.gullet.endGroups()}}subparse(t){var n=this.nextToken;this.consume(),this.gullet.pushToken(new Da("}")),this.gullet.pushTokens(t);var r=this.parseExpression(!1);return this.expect("}"),this.nextToken=n,r}parseExpression(t,n){for(var r=[];;){this.mode==="math"&&this.consumeSpaces();var a=this.fetch();if(im.endOfExpression.indexOf(a.text)!==-1||n&&a.text===n||t&&Js[a.text]&&Js[a.text].infix)break;var s=this.parseAtom(n);if(s){if(s.type==="internal")continue}else break;r.push(s)}return this.mode==="text"&&this.formLigatures(r),this.handleInfixNodes(r)}handleInfixNodes(t){for(var n=-1,r,a=0;a<t.length;a++)if(t[a].type==="infix"){if(n!==-1)throw new Ne("only one infix operator per group",t[a].token);n=a,r=t[a].replaceWith}if(n!==-1&&r){var s,o,c=t.slice(0,n),d=t.slice(n+1);c.length===1&&c[0].type==="ordgroup"?s=c[0]:s={type:"ordgroup",mode:this.mode,body:c},d.length===1&&d[0].type==="ordgroup"?o=d[0]:o={type:"ordgroup",mode:this.mode,body:d};var f;return r==="\\\\abovefrac"?f=this.callFunction(r,[s,t[n],o],[]):f=this.callFunction(r,[s,o],[]),[f]}else return t}handleSupSubscript(t){var n=this.fetch(),r=n.text;this.consume(),this.consumeSpaces();var a;do{var s;a=this.parseGroup(t)}while(((s=a)==null?void 0:s.type)==="internal");if(!a)throw new Ne("Expected group after '"+r+"'",n);return a}formatUnsupportedCmd(t){for(var n=[],r=0;r<t.length;r++)n.push({type:"textord",mode:"text",text:t[r]});var a={type:"text",mode:this.mode,body:n},s={type:"color",mode:this.mode,color:this.settings.errorColor,body:[a]};return s}parseAtom(t){var n=this.parseGroup("atom",t);if((n==null?void 0:n.type)==="internal"||this.mode==="text")return n;for(var r,a;;){this.consumeSpaces();var s=this.fetch();if(s.text==="\\limits"||s.text==="\\nolimits"){if(n&&n.type==="op"){var o=s.text==="\\limits";n.limits=o,n.alwaysHandleSupSub=!0}else if(n&&n.type==="operatorname")n.alwaysHandleSupSub&&(n.limits=s.text==="\\limits");else throw new Ne("Limit controls must follow a math operator",s);this.consume()}else if(s.text==="^"){if(r)throw new Ne("Double superscript",s);r=this.handleSupSubscript("superscript")}else if(s.text==="_"){if(a)throw new Ne("Double subscript",s);a=this.handleSupSubscript("subscript")}else if(s.text==="'"){if(r)throw new Ne("Double superscript",s);var c={type:"textord",mode:this.mode,text:"\\prime"},d=[c];for(this.consume();this.fetch().text==="'";)d.push(c),this.consume();this.fetch().text==="^"&&d.push(this.handleSupSubscript("superscript")),r={type:"ordgroup",mode:this.mode,body:d}}else if(wh[s.text]){var f=R6.test(s.text),m=[];for(m.push(new Da(wh[s.text])),this.consume();;){var p=this.fetch().text;if(!wh[p]||R6.test(p)!==f)break;m.unshift(new Da(wh[p])),this.consume()}var b=this.subparse(m);f?a={type:"ordgroup",mode:"math",body:b}:r={type:"ordgroup",mode:"math",body:b}}else break}return r||a?{type:"supsub",mode:this.mode,base:n,sup:r,sub:a}:n}parseFunction(t,n){var r=this.fetch(),a=r.text,s=Js[a];if(!s)return null;if(this.consume(),n&&n!=="atom"&&!s.allowedInArgument)throw new Ne("Got function '"+a+"' with no arguments"+(n?" as "+n:""),r);if(this.mode==="text"&&!s.allowedInText)throw new Ne("Can't use function '"+a+"' in text mode",r);if(this.mode==="math"&&s.allowedInMath===!1)throw new Ne("Can't use function '"+a+"' in math mode",r);var{args:o,optArgs:c}=this.parseArguments(a,s);return this.callFunction(a,o,c,r,t)}callFunction(t,n,r,a,s){var o={funcName:t,parser:this,token:a,breakOnTokenText:s},c=Js[t];if(c&&c.handler)return c.handler(o,n,r);throw new Ne("No function handler for "+t)}parseArguments(t,n){var r=n.numArgs+n.numOptionalArgs;if(r===0)return{args:[],optArgs:[]};for(var a=[],s=[],o=0;o<r;o++){var c=n.argTypes&&n.argTypes[o],d=o<n.numOptionalArgs;(n.primitive&&c==null||n.type==="sqrt"&&o===1&&s[0]==null)&&(c="primitive");var f=this.parseGroupOfType("argument to '"+t+"'",c,d);if(d)s.push(f);else if(f!=null)a.push(f);else throw new Ne("Null argument, please report this as a bug")}return{args:a,optArgs:s}}parseGroupOfType(t,n,r){switch(n){case"color":return this.parseColorGroup(r);case"size":return this.parseSizeGroup(r);case"url":return this.parseUrlGroup(r);case"math":case"text":return this.parseArgumentGroup(r,n);case"hbox":{var a=this.parseArgumentGroup(r,"text");return a!=null?{type:"styling",mode:a.mode,body:[a],style:"text"}:null}case"raw":{var s=this.parseStringGroup("raw",r);return s!=null?{type:"raw",mode:"text",string:s.text}:null}case"primitive":{if(r)throw new Ne("A primitive argument cannot be optional");var o=this.parseGroup(t);if(o==null)throw new Ne("Expected group as "+t,this.fetch());return o}case"original":case null:case void 0:return this.parseArgumentGroup(r);default:throw new Ne("Unknown group type as "+t,this.fetch())}}consumeSpaces(){for(;this.fetch().text===" ";)this.consume()}parseStringGroup(t,n){var r=this.gullet.scanArgument(n);if(r==null)return null;for(var a="",s;(s=this.fetch()).text!=="EOF";)a+=s.text,this.consume();return this.consume(),r.text=a,r}parseRegexGroup(t,n){for(var r=this.fetch(),a=r,s="",o;(o=this.fetch()).text!=="EOF"&&t.test(s+o.text);)a=o,s+=a.text,this.consume();if(s==="")throw new Ne("Invalid "+n+": '"+r.text+"'",r);return r.range(a,s)}parseColorGroup(t){var n=this.parseStringGroup("color",t);if(n==null)return null;var r=/^(#[a-f0-9]{3}|#?[a-f0-9]{6}|[a-z]+)$/i.exec(n.text);if(!r)throw new Ne("Invalid color: '"+n.text+"'",n);var a=r[0];return/^[0-9a-f]{6}$/i.test(a)&&(a="#"+a),{type:"color-token",mode:this.mode,color:a}}parseSizeGroup(t){var n,r=!1;if(this.gullet.consumeSpaces(),!t&&this.gullet.future().text!=="{"?n=this.parseRegexGroup(/^[-+]? *(?:$|\d+|\d+\.\d*|\.\d*) *[a-z]{0,2} *$/,"size"):n=this.parseStringGroup("size",t),!n)return null;!t&&n.text.length===0&&(n.text="0pt",r=!0);var a=/([-+]?) *(\d+(?:\.\d*)?|\.\d+) *([a-z]{2})/.exec(n.text);if(!a)throw new Ne("Invalid size: '"+n.text+"'",n);var s={number:+(a[1]+a[2]),unit:a[3]};if(!gx(s))throw new Ne("Invalid unit: '"+s.unit+"'",n);return{type:"size",mode:this.mode,value:s,isBlank:r}}parseUrlGroup(t){this.gullet.lexer.setCatcode("%",13),this.gullet.lexer.setCatcode("~",12);var n=this.parseStringGroup("url",t);if(this.gullet.lexer.setCatcode("%",14),this.gullet.lexer.setCatcode("~",13),n==null)return null;var r=n.text.replace(/\\([#$%&~_^{}])/g,"$1");return{type:"url",mode:this.mode,url:r}}parseArgumentGroup(t,n){var r=this.gullet.scanArgument(t);if(r==null)return null;var a=this.mode;n&&this.switchMode(n),this.gullet.beginGroup();var s=this.parseExpression(!1,"EOF");this.expect("EOF"),this.gullet.endGroup();var o={type:"ordgroup",mode:this.mode,loc:r.loc,body:s};return n&&this.switchMode(a),o}parseGroup(t,n){var r=this.fetch(),a=r.text,s;if(a==="{"||a==="\\begingroup"){this.consume();var o=a==="{"?"}":"\\endgroup";this.gullet.beginGroup();var c=this.parseExpression(!1,o),d=this.fetch();this.expect(o),this.gullet.endGroup(),s={type:"ordgroup",mode:this.mode,loc:ba.range(r,d),body:c,semisimple:a==="\\begingroup"||void 0}}else if(s=this.parseFunction(n,t)||this.parseSymbol(),s==null&&a[0]==="\\"&&!sS.hasOwnProperty(a)){if(this.settings.throwOnError)throw new Ne("Undefined control sequence: "+a,r);s=this.formatUnsupportedCmd(a),this.consume()}return s}formLigatures(t){for(var n=t.length-1,r=0;r<n;++r){var a=t[r],s=a.text;s==="-"&&t[r+1].text==="-"&&(r+1<n&&t[r+2].text==="-"?(t.splice(r,3,{type:"textord",mode:"text",loc:ba.range(a,t[r+2]),text:"---"}),n-=2):(t.splice(r,2,{type:"textord",mode:"text",loc:ba.range(a,t[r+1]),text:"--"}),n-=1)),(s==="'"||s==="`")&&t[r+1].text===s&&(t.splice(r,2,{type:"textord",mode:"text",loc:ba.range(a,t[r+1]),text:s+s}),n-=1)}}parseSymbol(){var t=this.fetch(),n=t.text;if(/^\\verb[^a-zA-Z]/.test(n)){this.consume();var r=n.slice(5),a=r.charAt(0)==="*";if(a&&(r=r.slice(1)),r.length<2||r.charAt(0)!==r.slice(-1))throw new Ne(`\\verb assertion failed --
                    please report what input caused this bug`);return r=r.slice(1,-1),{type:"verb",mode:"text",body:r,star:a}}M6.hasOwnProperty(n[0])&&!fn[this.mode][n[0]]&&(this.settings.strict&&this.mode==="math"&&this.settings.reportNonstrict("unicodeTextInMathMode",'Accented Unicode text character "'+n[0]+'" used in math mode',t),n=M6[n[0]]+n.slice(1));var s=_$.exec(n);s&&(n=n.substring(0,s.index),n==="i"?n="ı":n==="j"&&(n="ȷ"));var o;if(fn[this.mode][n]){this.settings.strict&&this.mode==="math"&&yb.indexOf(n)>=0&&this.settings.reportNonstrict("unicodeTextInMathMode",'Latin-1/Unicode text character "'+n[0]+'" used in math mode',t);var c=fn[this.mode][n].group,d=ba.range(t),f;if(hj.hasOwnProperty(c)){var m=c;f={type:"atom",mode:this.mode,family:m,loc:d,text:n}}else f={type:c,mode:this.mode,loc:d,text:n};o=f}else if(n.charCodeAt(0)>=128)this.settings.strict&&(px(n.charCodeAt(0))?this.mode==="math"&&this.settings.reportNonstrict("unicodeTextInMathMode",'Unicode text character "'+n[0]+'" used in math mode',t):this.settings.reportNonstrict("unknownSymbol",'Unrecognized Unicode character "'+n[0]+'"'+(" ("+n.charCodeAt(0)+")"),t)),o={type:"textord",mode:"text",loc:ba.range(t),text:n};else return null;if(this.consume(),s)for(var p=0;p<s[0].length;p++){var b=s[0][p];if(!gg[b])throw new Ne("Unknown accent ' "+b+"'",t);var y=gg[b][this.mode]||gg[b].text;if(!y)throw new Ne("Accent "+b+" unsupported in "+this.mode+" mode",t);o={type:"accent",mode:this.mode,loc:ba.range(t),label:y,isStretchy:!1,isShifty:!0,base:o}}return o}}im.endOfExpression=["}","\\endgroup","\\end","\\right","&"];var tv=function(t,n){if(!(typeof t=="string"||t instanceof String))throw new TypeError("KaTeX can only parse string typed expression");var r=new im(t,n);delete r.gullet.macros.current["\\df@tag"];var a=r.parse();if(delete r.gullet.macros.current["\\current@color"],delete r.gullet.macros.current["\\color"],r.gullet.macros.get("\\df@tag")){if(!n.displayMode)throw new Ne("\\tag works only in display equations");a=[{type:"tag",mode:"text",body:a,tag:r.subparse([new Da("\\df@tag")])}]}return a},lS=function(t,n,r){n.textContent="";var a=nv(t,r).toNode();n.appendChild(a)};typeof document<"u"&&document.compatMode!=="CSS1Compat"&&(typeof console<"u"&&console.warn("Warning: KaTeX doesn't work in quirks mode. Make sure your website has a suitable doctype."),lS=function(){throw new Ne("KaTeX doesn't work in quirks mode.")});var w$=function(t,n){var r=nv(t,n).toMarkup();return r},A$=function(t,n){var r=new B2(n);return tv(t,r)},oS=function(t,n,r){if(r.throwOnError||!(t instanceof Ne))throw t;var a=ae.makeSpan(["katex-error"],[new Ua(n)]);return a.setAttribute("title",t.toString()),a.setAttribute("style","color:"+r.errorColor),a},nv=function(t,n){var r=new B2(n);try{var a=tv(t,r);return Lj(a,t,r)}catch(s){return oS(s,t,r)}},k$=function(t,n){var r=new B2(n);try{var a=tv(t,r);return Pj(a,t,r)}catch(s){return oS(s,t,r)}},C$="0.16.22",N$={Span:B0,Anchor:z2,SymbolNode:Ua,SvgNode:as,PathNode:sl,LineNode:vb},I6={version:C$,render:lS,renderToString:w$,ParseError:Ne,SETTINGS_SCHEMA:$h,__parse:A$,__renderToDomTree:nv,__renderToHTMLTree:k$,__setFontMetrics:aj,__defineSymbol:E,__defineFunction:Ye,__defineMacro:I,__domTree:N$};const O$={},R$=[];function M$(e){const t=e||O$;return function(n,r){x2(n,"element",function(a,s){const o=Array.isArray(a.properties.className)?a.properties.className:R$,c=o.includes("language-math"),d=o.includes("math-display"),f=o.includes("math-inline");let m=d;if(!c&&!d&&!f)return;let p=s[s.length-1],b=a;if(a.tagName==="code"&&c&&p&&p.type==="element"&&p.tagName==="pre"&&(b=p,p=s[s.length-2],m=!0),!p)return;const y=YT(b,{whitespace:"pre"});let T;try{T=I6.renderToString(y,{...t,displayMode:m,throwOnError:!0})}catch(C){const w=C,O=w.name.toLowerCase();r.message("Could not render math with KaTeX",{ancestors:[...s,a],cause:w,place:a.position,ruleId:O,source:"rehype-katex"});try{T=I6.renderToString(y,{...t,displayMode:m,strict:"ignore",throwOnError:!1})}catch{T=[{type:"element",tagName:"span",properties:{className:["katex-error"],style:"color:"+(t.errorColor||"#cc0000"),title:String(C)},children:[{type:"text",value:y}]}]}}typeof T=="string"&&(T=Cq(T).children);const x=p.children.indexOf(b);return p.children.splice(x,1,...T),w7})}}function I$(){return{enter:{mathFlow:e,mathFlowFenceMeta:t,mathText:s},exit:{mathFlow:a,mathFlowFence:r,mathFlowFenceMeta:n,mathFlowValue:c,mathText:o,mathTextData:c}};function e(d){const f={type:"element",tagName:"code",properties:{className:["language-math","math-display"]},children:[]};this.enter({type:"math",meta:null,value:"",data:{hName:"pre",hChildren:[f]}},d)}function t(){this.buffer()}function n(){const d=this.resume(),f=this.stack[this.stack.length-1];f.type,f.meta=d}function r(){this.data.mathFlowInside||(this.buffer(),this.data.mathFlowInside=!0)}function a(d){const f=this.resume().replace(/^(\r?\n|\r)|(\r?\n|\r)$/g,""),m=this.stack[this.stack.length-1];m.type,this.exit(d),m.value=f;const p=m.data.hChildren[0];p.type,p.tagName,p.children.push({type:"text",value:f}),this.data.mathFlowInside=void 0}function s(d){this.enter({type:"inlineMath",value:"",data:{hName:"code",hProperties:{className:["language-math","math-inline"]},hChildren:[]}},d),this.buffer()}function o(d){const f=this.resume(),m=this.stack[this.stack.length-1];m.type,this.exit(d),m.value=f,m.data.hChildren.push({type:"text",value:f})}function c(d){this.config.enter.data.call(this,d),this.config.exit.data.call(this,d)}}function D$(e){let t=(e||{}).singleDollarTextMath;return t==null&&(t=!0),r.peek=a,{unsafe:[{character:"\r",inConstruct:"mathFlowMeta"},{character:`
`,inConstruct:"mathFlowMeta"},{character:"$",after:t?void 0:"\\$",inConstruct:"phrasing"},{character:"$",inConstruct:"mathFlowMeta"},{atBreak:!0,character:"$",after:"\\$"}],handlers:{math:n,inlineMath:r}};function n(s,o,c,d){const f=s.value||"",m=c.createTracker(d),p="$".repeat(Math.max(M7(f,"$")+1,2)),b=c.enter("mathFlow");let y=m.move(p);if(s.meta){const T=c.enter("mathFlowMeta");y+=m.move(c.safe(s.meta,{after:`
`,before:y,encode:["$"],...m.current()})),T()}return y+=m.move(`
`),f&&(y+=m.move(f+`
`)),y+=m.move(p),b(),y}function r(s,o,c){let d=s.value||"",f=1;for(t||f++;new RegExp("(^|[^$])"+"\\$".repeat(f)+"([^$]|$)").test(d);)f++;const m="$".repeat(f);/[^ \r\n]/.test(d)&&(/^[ \r\n]/.test(d)&&/[ \r\n]$/.test(d)||/^\$|\$$/.test(d))&&(d=" "+d+" ");let p=-1;for(;++p<c.unsafe.length;){const b=c.unsafe[p];if(!b.atBreak)continue;const y=c.compilePattern(b);let T;for(;T=y.exec(d);){let x=T.index;d.codePointAt(x)===10&&d.codePointAt(x-1)===13&&x--,d=d.slice(0,x)+" "+d.slice(T.index+1)}}return m+d+m}function a(){return"$"}}const L$={tokenize:P$,concrete:!0,name:"mathFlow"},D6={tokenize:B$,partial:!0};function P$(e,t,n){const r=this,a=r.events[r.events.length-1],s=a&&a[1].type==="linePrefix"?a[2].sliceSerialize(a[1],!0).length:0;let o=0;return c;function c(O){return e.enter("mathFlow"),e.enter("mathFlowFence"),e.enter("mathFlowFenceSequence"),d(O)}function d(O){return O===36?(e.consume(O),o++,d):o<2?n(O):(e.exit("mathFlowFenceSequence"),Nt(e,f,"whitespace")(O))}function f(O){return O===null||nt(O)?p(O):(e.enter("mathFlowFenceMeta"),e.enter("chunkString",{contentType:"string"}),m(O))}function m(O){return O===null||nt(O)?(e.exit("chunkString"),e.exit("mathFlowFenceMeta"),p(O)):O===36?n(O):(e.consume(O),m)}function p(O){return e.exit("mathFlowFence"),r.interrupt?t(O):e.attempt(D6,b,C)(O)}function b(O){return e.attempt({tokenize:w,partial:!0},C,y)(O)}function y(O){return(s?Nt(e,T,"linePrefix",s+1):T)(O)}function T(O){return O===null?C(O):nt(O)?e.attempt(D6,b,C)(O):(e.enter("mathFlowValue"),x(O))}function x(O){return O===null||nt(O)?(e.exit("mathFlowValue"),T(O)):(e.consume(O),x)}function C(O){return e.exit("mathFlow"),t(O)}function w(O,D,Y){let H=0;return Nt(O,B,"linePrefix",r.parser.constructs.disable.null.includes("codeIndented")?void 0:4);function B(Q){return O.enter("mathFlowFence"),O.enter("mathFlowFenceSequence"),ee(Q)}function ee(Q){return Q===36?(H++,O.consume(Q),ee):H<o?Y(Q):(O.exit("mathFlowFenceSequence"),Nt(O,Z,"whitespace")(Q))}function Z(Q){return Q===null||nt(Q)?(O.exit("mathFlowFence"),D(Q)):Y(Q)}}}function B$(e,t,n){const r=this;return a;function a(o){return o===null?t(o):(e.enter("lineEnding"),e.consume(o),e.exit("lineEnding"),s)}function s(o){return r.parser.lazy[r.now().line]?n(o):t(o)}}function U$(e){let n=(e||{}).singleDollarTextMath;return n==null&&(n=!0),{tokenize:r,resolve:F$,previous:z$,name:"mathText"};function r(a,s,o){let c=0,d,f;return m;function m(x){return a.enter("mathText"),a.enter("mathTextSequence"),p(x)}function p(x){return x===36?(a.consume(x),c++,p):c<2&&!n?o(x):(a.exit("mathTextSequence"),b(x))}function b(x){return x===null?o(x):x===36?(f=a.enter("mathTextSequence"),d=0,T(x)):x===32?(a.enter("space"),a.consume(x),a.exit("space"),b):nt(x)?(a.enter("lineEnding"),a.consume(x),a.exit("lineEnding"),b):(a.enter("mathTextData"),y(x))}function y(x){return x===null||x===32||x===36||nt(x)?(a.exit("mathTextData"),b(x)):(a.consume(x),y)}function T(x){return x===36?(a.consume(x),d++,T):d===c?(a.exit("mathTextSequence"),a.exit("mathText"),s(x)):(f.type="mathTextData",y(x))}}}function F$(e){let t=e.length-4,n=3,r,a;if((e[n][1].type==="lineEnding"||e[n][1].type==="space")&&(e[t][1].type==="lineEnding"||e[t][1].type==="space")){for(r=n;++r<t;)if(e[r][1].type==="mathTextData"){e[t][1].type="mathTextPadding",e[n][1].type="mathTextPadding",n+=2,t-=2;break}}for(r=n-1,t++;++r<=t;)a===void 0?r!==t&&e[r][1].type!=="lineEnding"&&(a=r):(r===t||e[r][1].type==="lineEnding")&&(e[a][1].type="mathTextData",r!==a+2&&(e[a][1].end=e[r-1][1].end,e.splice(a+2,r-a-2),t-=r-a-2,r=a+2),a=void 0);return e}function z$(e){return e!==36||this.events[this.events.length-1][1].type==="characterEscape"}function H$(e){return{flow:{36:L$},text:{36:U$(e)}}}const q$={};function j$(e){const t=this,n=e||q$,r=t.data(),a=r.micromarkExtensions||(r.micromarkExtensions=[]),s=r.fromMarkdownExtensions||(r.fromMarkdownExtensions=[]),o=r.toMarkdownExtensions||(r.toMarkdownExtensions=[]);a.push(H$(n)),s.push(I$()),o.push(D$(n))}const $$=({children:e,className:t,node:n,inline:r,...a})=>{const[s,o]=z.useState(!1),c=z.useRef(null),d=async()=>{var b;try{const y=((b=c.current)==null?void 0:b.textContent)||"";await navigator.clipboard.writeText(y),o(!0),setTimeout(()=>o(!1),2e3)}catch(y){console.error("Failed to copy code:",y)}},f=b=>{var y;return typeof b=="string"?b:Array.isArray(b)?b.map(f).join(""):(y=b==null?void 0:b.props)!=null&&y.children?f(b.props.children):""},m=f(e);return r||!m.includes(`
`)?R.jsx("code",{className:t,...a,children:e}):R.jsxs("div",{className:"relative group",children:[R.jsx("button",{onClick:d,className:"absolute top-1 right-1 opacity-0 group-hover:opacity-100 transition-opacity inline-flex items-center gap-1.5 px-2.5 py-1.5 text-xs font-medium text-gray-600 hover:text-gray-900 bg-white hover:bg-gray-50 border border-gray-200 rounded-md shadow-sm z-10",title:"Copy code",children:s?R.jsxs(R.Fragment,{children:[R.jsx("svg",{width:"13",height:"13",viewBox:"0 0 24 24",fill:"none",stroke:"currentColor",strokeWidth:"2",strokeLinecap:"round",strokeLinejoin:"round",children:R.jsx("polyline",{points:"20 6 9 17 4 12"})}),R.jsx("span",{children:"Copied!"})]}):R.jsx(R.Fragment,{children:R.jsxs("svg",{width:"13",height:"13",viewBox:"0 0 24 24",fill:"none",stroke:"currentColor",strokeWidth:"2",strokeLinecap:"round",strokeLinejoin:"round",children:[R.jsx("rect",{x:"9",y:"9",width:"13",height:"13",rx:"2",ry:"2"}),R.jsx("path",{d:"M5 15H4a2 2 0 0 1-2-2V4a2 2 0 0 1 2-2h9a2 2 0 0 1 2 2v1"})]})})}),R.jsx("code",{ref:c,className:t,...a,children:e})]})},Y$=({children:e})=>R.jsx("div",{className:"not-prose",children:e}),G$=({src:e,alt:t,...n})=>{const[r,a]=z.useState(!1),s=z.useCallback(async()=>{try{const c=await(await fetch(e)).blob(),d=window.URL.createObjectURL(c),f=document.createElement("a");f.href=d,f.download=t||"image",document.body.appendChild(f),f.click(),document.body.removeChild(f),window.URL.revokeObjectURL(d)}catch(o){console.error("Failed to download image:",o)}},[e,t]);return R.jsxs("div",{className:"relative inline-block group",onMouseEnter:()=>a(!0),onMouseLeave:()=>a(!1),children:[R.jsx("img",{src:e,alt:t,...n}),r&&R.jsx("button",{onClick:s,className:"absolute top-8 right-2 p-2 rounded-lg transition-all duration-200",style:{backgroundColor:"var(--bg-primary)",border:"1px solid var(--border-primary)",color:"var(--text-primary)",boxShadow:"var(--shadow-md)"},title:"Download image","aria-label":"Download image",children:R.jsxs("svg",{xmlns:"http://www.w3.org/2000/svg",width:"18",height:"18",viewBox:"0 0 24 24",fill:"none",stroke:"currentColor",strokeWidth:"2",strokeLinecap:"round",strokeLinejoin:"round",children:[R.jsx("path",{d:"M21 15v4a2 2 0 0 1-2 2H5a2 2 0 0 1-2-2v-4"}),R.jsx("polyline",{points:"7 10 12 15 17 10"}),R.jsx("line",{x1:"12",y1:"15",x2:"12",y2:"3"})]})})]})},V$=({href:e,onSend:t,...n})=>{const r=z.useCallback(a=>{t(decodeURIComponent(e.slice(23))),a.preventDefault()},[e,t]);return e.startsWith("https://cycls.com/send/")?R.jsx("a",{...n,className:n.className||"underline decoration-[var(--text-tertiary)]/40 decoration-2 hover:decoration-[var(--text-tertiary)]/60",href:"#",onClick:r}):R.jsx("a",{...n,href:e,target:"_blank",rel:"noreferrer"})},K$=({children:e,...t})=>{const n=z.useRef(null),[r,a]=z.useState(!1),s=z.useCallback(()=>{try{if(!n.current)return;const c=[],d=n.current,f=Array.from(d.querySelectorAll("thead th")).map(x=>{var C;return((C=x.textContent)==null?void 0:C.trim())||""});f.length>0&&c.push(f),Array.from(d.querySelectorAll("tbody tr")).forEach(x=>{const C=Array.from(x.querySelectorAll("td")).map(w=>{var O;return((O=w.textContent)==null?void 0:O.trim())||""});C.length>0&&c.push(C)});const p=c.map(x=>x.map(C=>{const w=String(C);return w.includes(",")||w.includes('"')||w.includes(`
`)?`"${w.replace(/"/g,'""')}"`:w}).join(",")).join(`
`),b=new Blob([p],{type:"text/csv;charset=utf-8;"}),y=window.URL.createObjectURL(b),T=document.createElement("a");T.href=y,T.download=`table-export-${Date.now()}.csv`,document.body.appendChild(T),T.click(),document.body.removeChild(T),window.URL.revokeObjectURL(y)}catch(c){console.error("Failed to export table as CSV:",c)}},[]),o=z.useCallback(async()=>{try{if(!n.current)return;const c=[],d=n.current,f=Array.from(d.querySelectorAll("thead th")).map(p=>{var b;return((b=p.textContent)==null?void 0:b.trim())||""});f.length>0&&c.push(f.join("	")),Array.from(d.querySelectorAll("tbody tr")).forEach(p=>{const b=Array.from(p.querySelectorAll("td")).map(y=>{var T;return((T=y.textContent)==null?void 0:T.trim())||""});b.length>0&&c.push(b.join("	"))}),await navigator.clipboard.writeText(c.join(`
`)),a(!0),setTimeout(()=>a(!1),2e3)}catch(c){console.error("Failed to copy table:",c)}},[]);return R.jsxs("div",{className:"table-wrapper",children:[R.jsx("table",{ref:n,...t,children:e}),R.jsxs("div",{className:"!-mt-2",style:{display:"flex",alignItems:"center",justifyContent:"space-between",padding:"12px 16px",marginTop:"8px",borderRadius:"8px",backgroundColor:"var(--bg-secondary)",border:"1px solid var(--border-primary)"},children:[R.jsxs("button",{onClick:s,style:{display:"flex",alignItems:"center",gap:"8px",fontSize:"14px",color:"var(--text-secondary)",background:"none",border:"none",cursor:"pointer",padding:"0"},"aria-label":"Export table as CSV",children:[R.jsxs("svg",{xmlns:"http://www.w3.org/2000/svg",width:"16",height:"16",viewBox:"0 0 24 24",fill:"none",stroke:"currentColor",strokeWidth:"2",strokeLinecap:"round",strokeLinejoin:"round",children:[R.jsx("rect",{x:"3",y:"3",width:"18",height:"18",rx:"2",ry:"2"}),R.jsx("line",{x1:"3",y1:"9",x2:"21",y2:"9"}),R.jsx("line",{x1:"3",y1:"15",x2:"21",y2:"15"}),R.jsx("line",{x1:"9",y1:"3",x2:"9",y2:"21"}),R.jsx("line",{x1:"15",y1:"3",x2:"15",y2:"21"})]}),R.jsx("span",{children:"Export as CSV"})]}),R.jsx("button",{onClick:o,style:{display:"flex",alignItems:"center",gap:"6px",padding:"6px",borderRadius:"4px",background:"none",border:"none",cursor:"pointer",color:r?"var(--text-success, #22c55e)":"var(--text-tertiary)"},title:r?"Copied!":"Copy table","aria-label":"Copy table",children:r?R.jsx("svg",{xmlns:"http://www.w3.org/2000/svg",width:"16",height:"16",viewBox:"0 0 24 24",fill:"none",stroke:"currentColor",strokeWidth:"2",strokeLinecap:"round",strokeLinejoin:"round",children:R.jsx("polyline",{points:"20 6 9 17 4 12"})}):R.jsxs("svg",{xmlns:"http://www.w3.org/2000/svg",width:"16",height:"16",viewBox:"0 0 24 24",fill:"none",stroke:"currentColor",strokeWidth:"2",strokeLinecap:"round",strokeLinejoin:"round",children:[R.jsx("rect",{x:"9",y:"9",width:"13",height:"13",rx:"2",ry:"2"}),R.jsx("path",{d:"M5 15H4a2 2 0 0 1-2-2V4a2 2 0 0 1 2-2h9a2 2 0 0 1 2 2v1"})]})})]})]})},mf=ce.memo(({markdown:e,onSend:t})=>{const n=z.useMemo(()=>({raw:Y$,a:r=>R.jsx(V$,{...r,onSend:t}),code:$$,img:G$,table:K$}),[t]);return R.jsx("div",{dir:"auto",children:R.jsx(tL,{components:n,remarkPlugins:[fB,j$],rehypePlugins:[Iz,M$,[_q,{ignoreMissing:!0,detect:!0}]],children:e})})});mf.displayName="MarkdownRenderer";const Zi={MAX_FILE_BYTES:10*1024*1024,ALLOWED_MIME:new Set(["image/jpeg","image/png","image/webp","application/pdf","text/plain","text/markdown","text/csv","video/mp4","video/webm","video/ogg","video/quicktime"]),MAX_CHATS:15};function F0(e){const t=/<!--\s*cycls:attachments\s*([\s\S]*?)\s*cycls:attachments\s*-->/m,n=e.match(t);if(!n)return{clean:e,attachments:[]};let r=[];try{r=JSON.parse(n[1])}catch{r=[]}return{clean:e.replace(t,"").trimEnd(),attachments:r}}function W$(e){return e?e.startsWith("image/")?"image":e.startsWith("video/")?"video":e==="application/pdf"?"pdf":e==="text/csv"?"csv":e==="text/markdown"?"md":e==="text/plain"?"txt":"file":"file"}async function X$(e,t="anon",n="anon"){if(e.size>Zi.MAX_FILE_BYTES)throw new Error("File too large (max 10MB)");if(!Zi.ALLOWED_MIME.has(e.type))throw new Error(`Type not allowed: ${e.type}`);const r=new FormData;r.append("file",e),r.append("orgId",t),r.append("userId",n);const a=await fetch("https://cfywthfcvbajcekvwajz.supabase.co/functions/v1/upload",{method:"POST",body:r});if(!a.ok)throw new Error(`Upload failed (${a.status})`);return await a.json()}const pf=e=>e<1024?`${e}B`:e<1024*1024?`${(e/1024).toFixed(2)}kB`:`${(e/(1024*1024)).toFixed(2)}MB`,Q$=e=>{if(e.size>Zi.MAX_FILE_BYTES)return{valid:!1,error:`File "${e.name}" is too large (${pf(e.size)}). Maximum size is ${pf(Zi.MAX_FILE_BYTES)}.`};if(!Zi.ALLOWED_MIME.has(e.type)){const t=Array.from(Zi.ALLOWED_MIME).map(n=>n.split("/")[1]).join(", ");return{valid:!1,error:`File type "${e.type}" is not supported. Allowed types: ${t}`}}return{valid:!0}},Z$=({attachments:e=[]})=>e.length?R.jsx("div",{className:"mt-3 flex flex-wrap gap-2",children:e.map((t,n)=>{const r=W$(t.mime);if(r==="image")return R.jsx("a",{href:t.url,target:"_blank",rel:"noopener noreferrer",className:"group relative overflow-hidden rounded-lg shadow-sm transition-all duration-200 theme-transition",style:{border:"1px solid var(--border-primary)"},children:R.jsx("img",{src:t.url,alt:"",loading:"lazy",className:"mb-1 w-40 rounded-md"})},n);const a={pdf:"📄",csv:"📊",md:"📝",txt:"📝"}[r]||"📎";return R.jsxs("a",{href:t.url,target:"_blank",rel:"noopener noreferrer",className:"inline-flex items-center gap-2.5 rounded-xl backdrop-blur-sm px-4 py-2.5 text-sm transition-all shadow-sm hover:bg-[var(--bg-hover)] theme-transition",style:{border:"1px solid var(--border-primary)",backgroundColor:"var(--bg-primary)"},children:[R.jsx("span",{className:"text-xl",children:a}),R.jsx("span",{className:"text-xs uppercase font-semibold tracking-wide",style:{color:"var(--text-tertiary)"},children:r||"file"})]},n)})}):null,J$=({error:e,onRetry:t,isRetrying:n})=>{const r=a=>a?a.type==="network"?"Network error. Please check your connection and try again.":a.type==="timeout"?"Request timed out. The server took too long to respond.":a.type==="rate_limit"?a.message||"Too many requests. Please wait a moment and try again.":a.type==="server"?`Server error (${a.status||"unknown"}). Please try again later.`:a.type==="abort"?"Request was cancelled.":a.message||"An error occurred while processing your request.":"An unexpected error occurred";return R.jsx("div",{className:"rounded-lg p-4 my-2 theme-transition",style:{backgroundColor:"rgba(239, 68, 68, 0.1)",border:"1px solid rgba(239, 68, 68, 0.3)"},children:R.jsxs("div",{className:"flex items-start gap-3",children:[R.jsx("div",{className:"flex-shrink-0 mt-0.5",children:R.jsxs("svg",{className:"h-5 w-5 text-red-500",viewBox:"0 0 24 24",fill:"none",stroke:"currentColor",strokeWidth:"2",children:[R.jsx("circle",{cx:"12",cy:"12",r:"10"}),R.jsx("line",{x1:"12",y1:"8",x2:"12",y2:"12"}),R.jsx("line",{x1:"12",y1:"16",x2:"12.01",y2:"16"})]})}),R.jsxs("div",{className:"flex-1 min-w-0",children:[R.jsx("p",{className:"text-sm font-medium text-red-500",children:"Error"}),R.jsx("p",{className:"text-sm mt-1",style:{color:"var(--text-secondary)"},children:r(e)}),t&&e.type!=="abort"&&R.jsx("button",{onClick:t,disabled:n,className:"mt-3 inline-flex items-center gap-1.5 px-3 py-1.5 text-sm font-medium text-red-500 hover:text-red-400 border border-red-500/30 hover:border-red-500/50 rounded-lg transition-colors disabled:opacity-50 disabled:cursor-not-allowed theme-transition",style:{backgroundColor:"var(--bg-primary)"},children:n?R.jsxs(R.Fragment,{children:[R.jsxs("svg",{className:"animate-spin h-4 w-4",xmlns:"http://www.w3.org/2000/svg",fill:"none",viewBox:"0 0 24 24",children:[R.jsx("circle",{className:"opacity-25",cx:"12",cy:"12",r:"10",stroke:"currentColor",strokeWidth:"4"}),R.jsx("path",{className:"opacity-75",fill:"currentColor",d:"M4 12a8 8 0 018-8V0C5.373 0 0 5.373 0 12h4zm2 5.291A7.962 7.962 0 014 12H0c0 3.042 1.135 5.824 3 7.938l3-2.647z"})]}),R.jsx("span",{children:"Retrying..."})]}):R.jsxs(R.Fragment,{children:[R.jsxs("svg",{width:"14",height:"14",viewBox:"0 0 24 24",fill:"none",stroke:"currentColor",strokeWidth:"2",strokeLinecap:"round",strokeLinejoin:"round",children:[R.jsx("polyline",{points:"23 4 23 10 17 10"}),R.jsx("polyline",{points:"1 20 1 14 7 14"}),R.jsx("path",{d:"M3.51 9a9 9 0 0 1 14.85-3.36L23 10M1 14l4.64 4.36A9 9 0 0 0 20.49 15"})]}),R.jsx("span",{children:"Retry"})]})})]})]})})},eY=({message:e,onCopy:t,onRegenerate:n,isLastAssistant:r})=>{const[a,s]=z.useState(!1),o=async()=>{try{const{clean:c}=F0(e.content);await navigator.clipboard.writeText(c),s(!0),setTimeout(()=>s(!1),2e3)}catch(c){console.error("Failed to copy:",c)}};return R.jsxs("div",{className:"flex items-center gap-1 opacity-0 group-hover:opacity-100 transition-opacity duration-200",children:[R.jsx("button",{onClick:o,className:"inline-flex items-center gap-1.5 px-2.5 py-1.5 text-xs font-medium rounded-lg transition-colors hover:bg-[var(--bg-hover)] theme-transition",style:{color:"var(--text-tertiary)"},title:"Copy message",children:a?R.jsxs(R.Fragment,{children:[R.jsx("svg",{width:"14",height:"14",viewBox:"0 0 24 24",fill:"none",stroke:"currentColor",strokeWidth:"2",strokeLinecap:"round",strokeLinejoin:"round",children:R.jsx("polyline",{points:"20 6 9 17 4 12"})}),R.jsx("span",{children:"Copied!"})]}):R.jsxs(R.Fragment,{children:[R.jsxs("svg",{width:"14",height:"14",viewBox:"0 0 24 24",fill:"none",stroke:"currentColor",strokeWidth:"2",strokeLinecap:"round",strokeLinejoin:"round",children:[R.jsx("rect",{x:"9",y:"9",width:"13",height:"13",rx:"2",ry:"2"}),R.jsx("path",{d:"M5 15H4a2 2 0 0 1-2-2V4a2 2 0 0 1 2-2h9a2 2 0 0 1 2 2v1"})]}),R.jsx("span",{children:"Copy"})]})}),r&&R.jsxs("button",{onClick:n,className:"inline-flex items-center gap-1.5 px-2.5 py-1.5 text-xs font-medium rounded-lg transition-colors hover:bg-[var(--bg-hover)] theme-transition",style:{color:"var(--text-tertiary)"},title:"Regenerate response",children:[R.jsxs("svg",{width:"14",height:"14",viewBox:"0 0 24 24",fill:"none",stroke:"currentColor",strokeWidth:"2",strokeLinecap:"round",strokeLinejoin:"round",children:[R.jsx("polyline",{points:"23 4 23 10 17 10"}),R.jsx("polyline",{points:"1 20 1 14 7 14"}),R.jsx("path",{d:"M3.51 9a9 9 0 0 1 14.85-3.36L23 10M1 14l4.64 4.36A9 9 0 0 0 20.49 15"})]}),R.jsx("span",{children:"Regenerate"})]})]})},uS=ce.memo(({messages:e,onSend:t,onRegenerate:n,isGenerating:r,onRetry:a,retryingIndex:s})=>{const o=z.useMemo(()=>e.map((c,d)=>{if(c.type==="request"){const{clean:m,attachments:p}=F0(c.content),b=/[\u0591-\u07FF\uFB1D-\uFDFD\uFE70-\uFEFC]/.test(m);return R.jsx("div",{className:`flex mb-4 ${b?"justify-start":"justify-end"}`,children:R.jsxs("div",{className:"max-w-[85%] md:max-w-[75%]",children:[p.length>0&&R.jsx("div",{className:"mb-1",children:R.jsx(Z$,{attachments:p})}),m&&R.jsx("div",{className:"px-4 py-2 rounded-3xl shadow-sm theme-transition",style:{backgroundColor:"var(--msg-user-bg)",color:"var(--text-primary)"},children:R.jsx("div",{dir:"auto",className:"text-[15px] leading-relaxed whitespace-pre-wrap",children:m})})]})},d)}const f=c.type==="response"&&d===e.length-1&&!r;return R.jsx("div",{className:"flex mb-4 justify-start group",children:R.jsx("div",{className:"w-full",children:c.error?R.jsx(J$,{error:c.error,onRetry:()=>a(d),isRetrying:s===d}):R.jsxs(R.Fragment,{children:[R.jsxs("div",{className:"prose prose-sm max-w-none",children:[R.jsx(mf,{markdown:c.content,onSend:t}),c.content===""&&R.jsx("div",{className:"h-4 w-32 rounded animate-pulse",style:{backgroundColor:"var(--bg-tertiary)"}})]}),c.content&&c.content!==""&&R.jsx(eY,{message:c,onRegenerate:()=>n(d),isLastAssistant:f})]})})},d)}),[e,t,n,r,a,s]);return R.jsx("div",{className:"space-y-1",children:o})});uS.displayName="MessageList";const tY=({file:e,onRemove:t})=>{const[n,r]=z.useState(null),a=e.type.startsWith("image/");return z.useEffect(()=>{if(!a)return;const s=new FileReader;s.onloadend=()=>r(String(s.result)),s.readAsDataURL(e)},[e,a]),R.jsxs("div",{className:"group inline-flex items-start gap-2 rounded-xl shadow-sm px-2.5 py-1 theme-transition",style:{backgroundColor:"var(--bg-primary)",border:"1px solid var(--border-primary)"},title:e.name,children:[R.jsx("div",{className:"w-10 h-10 rounded-md overflow-hidden flex items-center justify-center",style:{backgroundColor:"var(--bg-tertiary)"},children:a&&n?R.jsx("img",{src:n,alt:e.name,className:"w-full h-full object-cover"}):R.jsx("span",{className:"text-xs",children:"📄"})}),R.jsxs("div",{className:"min-w-0 leading-tight",children:[R.jsx("div",{className:"text-xs font-medium truncate max-w-[100px]",style:{color:"var(--text-primary)"},children:e.name}),R.jsx("div",{className:"text-xs text-left",style:{color:"var(--text-tertiary)"},children:pf(e.size)})]}),R.jsx("button",{onClick:t,className:"ml-1 h-5 w-5 rounded-full grid place-items-center hover:bg-[var(--bg-hover)] theme-transition",style:{backgroundColor:"var(--bg-tertiary)",color:"var(--text-secondary)"},"aria-label":"Remove attachment",children:R.jsx("svg",{width:"12",height:"12",viewBox:"0 0 12 12",fill:"none",children:R.jsx("path",{d:"M9 3L3 9M3 3L9 9",stroke:"currentColor",strokeWidth:"1.5",strokeLinecap:"round"})})})]})},nY=({error:e,onDismiss:t})=>(z.useEffect(()=>{const n=setTimeout(t,5e3);return()=>clearTimeout(n)},[t]),R.jsx("div",{className:"fixed bottom-40 left-1/2 transform -translate-x-1/2 bg-red-50 border border-red-200 rounded-xl shadow-lg px-4 py-3 min-w-[320px] max-w-[90vw] z-50 animate-in slide-in-from-bottom-2 duration-300",children:R.jsxs("div",{className:"flex items-start gap-3",children:[R.jsx("div",{className:"flex-shrink-0 mt-0.5",children:R.jsxs("svg",{className:"h-5 w-5 text-red-600",viewBox:"0 0 24 24",fill:"none",stroke:"currentColor",strokeWidth:"2",children:[R.jsx("circle",{cx:"12",cy:"12",r:"10"}),R.jsx("line",{x1:"12",y1:"8",x2:"12",y2:"12"}),R.jsx("line",{x1:"12",y1:"16",x2:"12.01",y2:"16"})]})}),R.jsxs("div",{className:"flex-1 min-w-0",children:[R.jsx("p",{className:"text-sm font-medium text-red-900",children:"Upload Error"}),R.jsx("p",{className:"text-sm text-red-700 mt-1",children:e})]}),R.jsx("button",{onClick:t,className:"flex-shrink-0 text-red-400 hover:text-red-600 transition-colors","aria-label":"Dismiss",children:R.jsx("svg",{width:"20",height:"20",viewBox:"0 0 20 20",fill:"currentColor",children:R.jsx("path",{fillRule:"evenodd",d:"M4.293 4.293a1 1 0 011.414 0L10 8.586l4.293-4.293a1 1 0 111.414 1.414L11.414 10l4.293 4.293a1 1 0 01-1.414 1.414L10 11.414l-4.293 4.293a1 1 0 01-1.414-1.414L8.586 10 4.293 5.707a1 1 0 010-1.414z",clipRule:"evenodd"})})})]})})),rY=({onSend:e,isLoading:t=!1,onStop:n,disabled:r=!1,onUpgradeClick:a,sidebarWidth:s=0})=>{const[o,c]=z.useState(""),[d,f]=z.useState([]),[m,p]=z.useState(!1),[b,y]=z.useState(null),[T,x]=z.useState(!1),C=z.useRef(null),w=z.useRef(null),O=z.useRef(0);z.useEffect(()=>{const Q=()=>p(window.innerWidth<=768);return Q(),window.addEventListener("resize",Q),()=>window.removeEventListener("resize",Q)},[]);const D=z.useCallback(Q=>{if(r)return;const G=[];for(const se of Q){const oe=Q$(se);if(oe.valid)G.push(se);else{y(oe.error);break}}G.length>0&&f(se=>[...se,...G])},[r]);z.useEffect(()=>{const Q=ye=>{ye.preventDefault(),ye.stopPropagation(),!r&&(O.current++,ye.dataTransfer.types.includes("Files")&&x(!0))},G=ye=>{ye.preventDefault(),ye.stopPropagation(),O.current--,O.current===0&&x(!1)},se=ye=>{ye.preventDefault(),ye.stopPropagation()},oe=ye=>{if(ye.preventDefault(),ye.stopPropagation(),x(!1),O.current=0,r)return;const fe=Array.from(ye.dataTransfer.files||[]);fe.length>0&&D(fe)};return document.addEventListener("dragenter",Q),document.addEventListener("dragleave",G),document.addEventListener("dragover",se),document.addEventListener("drop",oe),()=>{document.removeEventListener("dragenter",Q),document.removeEventListener("dragleave",G),document.removeEventListener("dragover",se),document.removeEventListener("drop",oe)}},[D,r]),z.useEffect(()=>{const Q=G=>{var fe;if(r||!(document.activeElement===w.current))return;const ye=Array.from(((fe=G.clipboardData)==null?void 0:fe.items)||[]).filter(de=>de.type.startsWith("image/"));if(ye.length>0){G.preventDefault();const de=ye.map(V=>V.getAsFile()).filter(V=>V!==null);de.length>0&&D(de)}};return document.addEventListener("paste",Q),()=>document.removeEventListener("paste",Q)},[D,r]);const Y=()=>{var Q;r||(Q=C.current)==null||Q.click()},H=Q=>{if(r)return;const G=Array.from(Q.target.files||[]);D(G),Q.target.value=""},B=Q=>f(G=>G.filter((se,oe)=>oe!==Q)),ee=async()=>{t||r||(e({text:o,files:d}),c(""),f([]))},Z=Q=>{Q.key==="Enter"&&!Q.shiftKey&&!m&&!t&&!r&&(Q.preventDefault(),ee())};return R.jsxs(R.Fragment,{children:[b&&R.jsx(nY,{error:b,onDismiss:()=>y(null)}),T&&!r&&R.jsx("div",{className:"fixed inset-0 z-50 backdrop-blur-sm flex items-center justify-center pointer-events-none",style:{backgroundColor:"var(--bg-overlay)"},children:R.jsx("div",{className:"rounded-2xl shadow-2xl p-8 max-w-md mx-4 border-2 border-dashed theme-transition",style:{backgroundColor:"var(--bg-primary)",borderColor:"var(--text-primary)"},children:R.jsxs("div",{className:"text-center",children:[R.jsx("div",{className:"mx-auto w-16 h-16 rounded-full flex items-center justify-center mb-4",style:{backgroundColor:"var(--bg-tertiary)"},children:R.jsx("svg",{className:"w-8 h-8",style:{color:"var(--text-primary)"},fill:"none",viewBox:"0 0 24 24",stroke:"currentColor",children:R.jsx("path",{strokeLinecap:"round",strokeLinejoin:"round",strokeWidth:2,d:"M7 16a4 4 0 01-.88-7.903A5 5 0 1115.9 6L16 6a5 5 0 011 9.9M15 13l-3-3m0 0l-3 3m3-3v12"})})}),R.jsx("h3",{className:"text-lg font-semibold mb-2",style:{color:"var(--text-primary)"},children:"Drop files to upload"}),R.jsx("p",{className:"text-sm",style:{color:"var(--text-secondary)"},children:"Images, PDFs, and text files supported"}),R.jsxs("p",{className:"text-xs mt-2",style:{color:"var(--text-tertiary)"},children:["Max ",pf(Zi.MAX_FILE_BYTES)," per file"]})]})})}),R.jsx("div",{className:"fixed right-0 z-40 transition-all duration-300 ease-in-out bottom-[max(0.75rem,env(safe-area-inset-bottom))] md:bottom-[max(1rem,env(safe-area-inset-bottom))]",style:{left:m?"0px":`${s}px`},children:R.jsxs("div",{className:"mx-auto max-w-3xl px-3 md:px-4",children:[R.jsxs("div",{className:`cursor-text rounded-3xl relative p-0 pt-1 backdrop-blur-xl w-full theme-transition ${r?"opacity-75":""}`,style:{backgroundColor:"var(--input-bg)",border:"1px solid var(--border-primary)"},children:[d.length>0&&R.jsx("div",{className:"px-3 pt-3",children:R.jsx("div",{className:"flex flex-wrap gap-2",children:d.map((Q,G)=>R.jsx(tY,{file:Q,onRemove:()=>B(G)},G))})}),R.jsxs("div",{className:"px-3 pb-3 pt-2",children:[R.jsx("textarea",{ref:w,className:`w-full resize-none border-0 bg-transparent focus:ring-0 focus:outline-none text-base leading-6 max-h-40 mb-2 disabled:opacity-50 disabled:cursor-not-allowed ${r?"cursor-not-allowed":""}`,style:{color:"var(--text-primary)",caretColor:"var(--accent-primary)"},dir:"auto",autoFocus:!r,tabIndex:0,rows:Math.max(1,Math.min(6,o.split(`
`).length)),placeholder:r?"Upgrade to continue chatting...":t?"Waiting for response...":"What's on your mind?",onChange:Q=>c(Q.target.value),onKeyDown:Z,value:o,disabled:t||r}),R.jsxs("div",{className:"flex items-center gap-2",children:[R.jsx("button",{onClick:Y,disabled:r,className:`rounded-full w-8 h-8 inline-flex items-center justify-center p-1 theme-transition ${r?"opacity-50 cursor-not-allowed":"hover:bg-[var(--bg-hover)] active:scale-[0.98]"}`,style:{backgroundColor:"var(--bg-primary)",border:"1px solid var(--border-primary)",color:"var(--text-primary)"},"aria-label":"Attach file",title:r?"Upgrade to attach files":"Attach files",children:R.jsx("svg",{xmlns:"http://www.w3.org/2000/svg",width:"20",height:"20",viewBox:"0 0 24 24",fill:"none",stroke:"currentColor",strokeWidth:"1.5",strokeLinecap:"round",strokeLinejoin:"round",children:R.jsx("path",{d:"m16 6-8.414 8.586a2 2 0 0 0 2.829 2.829l8.414-8.586a4 4 0 1 0-5.657-5.657l-8.379 8.551a6 6 0 1 0 8.485 8.485l8.379-8.551"})})}),R.jsx("input",{ref:C,type:"file",multiple:!0,className:"hidden",onChange:H,accept:".jpg,.jpeg,.png,.webp,.pdf,.txt,.md,.csv",disabled:r}),R.jsx("div",{className:"flex-1"}),t&&!r&&R.jsx("button",{onClick:n,className:"rounded-full hover:bg-[var(--bg-hover)] active:scale-95 transition-all inline-flex items-center justify-center w-8 h-8 flex-shrink-0 theme-transition",style:{backgroundColor:"var(--bg-primary)",border:"1px solid var(--border-primary)",color:"var(--text-primary)",boxShadow:"var(--shadow-sm)"},"aria-label":"Stop generating",title:"Stop",children:R.jsx("svg",{width:"18",height:"18",viewBox:"0 0 24 24",fill:"currentColor",children:R.jsx("rect",{x:"6",y:"6",width:"12",height:"12",rx:"2"})})}),r&&R.jsxs("button",{onClick:a,className:"rounded-full bg-gradient-to-r from-blue-600 to-indigo-600 text-white hover:from-blue-700 hover:to-indigo-700 active:scale-[0.98] transition-all inline-flex items-center justify-center gap-1.5 px-4 h-9 flex-shrink-0 shadow-md font-medium text-sm","aria-label":"Upgrade to continue",title:"Upgrade",children:[R.jsx("svg",{className:"w-4 h-4",fill:"currentColor",viewBox:"0 0 20 20",children:R.jsx("path",{fillRule:"evenodd",d:"M11.3 1.046A1 1 0 0112 2v5h4a1 1 0 01.82 1.573l-7 10A1 1 0 018 18v-5H4a1 1 0 01-.82-1.573l7-10a1 1 0 011.12-.38z",clipRule:"evenodd"})}),"Upgrade"]}),!t&&!r&&R.jsx("button",{onClick:ee,disabled:t,className:"rounded-full hover:opacity-90 active:scale-[0.98] inline-flex items-center justify-center w-9 h-9 flex-shrink-0 disabled:opacity-50 disabled:cursor-not-allowed disabled:active:scale-100 theme-transition",style:{backgroundColor:"var(--btn-primary-bg)",color:"var(--btn-primary-text)"},"aria-label":"Send message",title:"Send",children:R.jsx("svg",{width:"20",height:"20",viewBox:"0 0 20 20",fill:"currentColor",xmlns:"http://www.w3.org/2000/svg",children:R.jsx("path",{d:"M8.99992 16V6.41407L5.70696 9.70704C5.31643 10.0976 4.68342 10.0976 4.29289 9.70704C3.90237 9.31652 3.90237 8.6835 4.29289 8.29298L9.29289 3.29298L9.36907 3.22462C9.76184 2.90427 10.3408 2.92686 10.707 3.29298L15.707 8.29298L15.7753 8.36915C16.0957 8.76192 16.0731 9.34092 15.707 9.70704C15.3408 10.0732 14.7618 10.0958 14.3691 9.7754L14.2929 9.70704L10.9999 6.41407V16C10.9999 16.5523 10.5522 17 9.99992 17C9.44764 17 8.99992 16.5523 8.99992 16Z"})})})]})]})]}),r&&R.jsx("p",{className:"text-xs text-center mt-2",style:{color:"var(--text-tertiary)"},children:"You've reached your free message limit"})]})}),R.jsx("style",{children:`
        textarea::placeholder {
          color: var(--text-muted);
        }
      `})]})},aY=({fileName:e,progress:t})=>R.jsx("div",{className:"fixed bottom-40 left-1/2 transform -translate-x-1/2 rounded-xl shadow-lg px-4 py-3 min-w-[280px] theme-transition",style:{backgroundColor:"var(--bg-primary)",border:"1px solid var(--border-primary)",boxShadow:"var(--shadow-lg)"},children:R.jsxs("div",{className:"flex items-center gap-3",children:[R.jsx("div",{className:"flex-shrink-0",children:R.jsxs("svg",{className:"animate-spin h-5 w-5",style:{color:"var(--text-secondary)"},xmlns:"http://www.w3.org/2000/svg",fill:"none",viewBox:"0 0 24 24",children:[R.jsx("circle",{className:"opacity-25",cx:"12",cy:"12",r:"10",stroke:"currentColor",strokeWidth:"4"}),R.jsx("path",{className:"opacity-75",fill:"currentColor",d:"M4 12a8 8 0 018-8V0C5.373 0 0 5.373 0 12h4zm2 5.291A7.962 7.962 0 014 12H0c0 3.042 1.135 5.824 3 7.938l3-2.647z"})]})}),R.jsxs("div",{className:"flex-1 min-w-0",children:[R.jsx("p",{className:"text-sm font-medium truncate",style:{color:"var(--text-primary)"},children:e}),R.jsx("div",{className:"mt-1 w-full rounded-full h-1.5",style:{backgroundColor:"var(--bg-tertiary)"},children:R.jsx("div",{className:"h-1.5 rounded-full transition-all duration-300",style:{width:`${t}%`,backgroundColor:"var(--text-primary)"}})})]})]})}),gf={collapsed:56,expanded:275},dr=(...e)=>e.filter(Boolean).join(" "),L6=({children:e})=>R.jsx("span",{className:"grid place-items-center w-5 h-5",children:e}),P6=e=>R.jsxs("svg",{"aria-hidden":"true",viewBox:"0 0 24 24",className:dr("w-5 h-5",e==null?void 0:e.className),fill:"none",stroke:"currentColor",strokeWidth:"1.7",strokeLinecap:"round",strokeLinejoin:"round",children:[R.jsx("path",{d:"M12 3a9 9 0 1 1-6.22 15.5"}),R.jsx("path",{d:"M12 8v8"}),R.jsx("path",{d:"M16 12H8"})]}),B6=e=>R.jsxs("svg",{"aria-hidden":"true",viewBox:"0 0 24 24",className:dr("w-[18px] h-[18px]",e==null?void 0:e.className),fill:"none",stroke:"currentColor",strokeWidth:"1.7",strokeLinecap:"round",strokeLinejoin:"round",children:[R.jsx("rect",{x:"3",y:"3",width:"18",height:"18",rx:"2"}),R.jsx("path",{d:"M9 3v18"})]}),iY=e=>R.jsx("svg",{"aria-hidden":"true",viewBox:"0 0 24 24",className:dr("w-5 h-5",e==null?void 0:e.className),fill:"none",stroke:"currentColor",strokeWidth:"1.5",strokeLinecap:"round",strokeLinejoin:"round",children:R.jsx("path",{d:"M10 7l5 5-5 5"})}),sY=e=>R.jsxs("svg",{"aria-hidden":"true",viewBox:"0 0 24 24",className:dr("w-4 h-4",e==null?void 0:e.className),fill:"none",stroke:"currentColor",strokeWidth:"1.5",strokeLinecap:"round",strokeLinejoin:"round",children:[R.jsx("path",{d:"M3 6h18"}),R.jsx("path",{d:"M8 6V4a2 2 0 0 1 2-2h4a2 2 0 0 1 2 2v2"}),R.jsx("path",{d:"M19 6v14a2 2 0 0 1-2 2H7a2 2 0 0 1-2-2V6"}),R.jsx("path",{d:"M10 11v6"}),R.jsx("path",{d:"M14 11v6"})]}),lY=e=>R.jsxs("svg",{xmlns:"http://www.w3.org/2000/svg",width:"20",height:"20",viewBox:"0 0 24 24",fill:"none",stroke:"currentColor",strokeWidth:"1.5",strokeLinecap:"round",strokeLinejoin:"round",className:dr("lucide lucide-ellipsis-icon lucide-ellipsis",e==null?void 0:e.className),children:[R.jsx("circle",{cx:"12",cy:"12",r:"1"}),R.jsx("circle",{cx:"19",cy:"12",r:"1"}),R.jsx("circle",{cx:"5",cy:"12",r:"1"})]});function oY(e,t,n=!0){z.useEffect(()=>{if(!n)return;const r=a=>{(Array.isArray(e)?e:[e]).some(c=>(c==null?void 0:c.current)&&c.current.contains(a.target))||t(a)};return document.addEventListener("mousedown",r),document.addEventListener("touchstart",r,{passive:!0}),()=>{document.removeEventListener("mousedown",r),document.removeEventListener("touchstart",r)}},[e,t,n])}const uY=({icon:e,children:t,isCollapsed:n,...r})=>R.jsxs("button",{...r,className:dr("group w-full flex items-center gap-3 rounded-xl px-2.5 py-2 text-left","hover:bg-[var(--bg-hover)] active:bg-[var(--bg-active)]","focus:outline-none focus-visible:ring-2 focus-visible:ring-[var(--text-primary)]/20","theme-transition",n&&"justify-center px-2"),style:{color:"var(--text-primary)"},children:[R.jsx("span",{className:"shrink-0",style:{color:"var(--text-primary)"},children:e}),!n&&R.jsx("span",{className:"text-sm font-medium leading-5",children:t})]}),cY=({children:e})=>R.jsx("div",{className:"px-2.5 py-1.5 text-[11px] font-semibold tracking-wide uppercase",style:{color:"var(--text-tertiary)"},children:e}),dY=({chat:e,isActive:t,onSelect:n,onDelete:r})=>{const[a,s]=z.useState(!1),o=z.useRef(null),c=z.useRef(null),d=z.useId();return oY([o,c],()=>s(!1),a),R.jsxs("div",{className:"relative",children:[R.jsxs("button",{onClick:()=>n(e.id),className:dr("group w-full flex items-center gap-2 rounded-xl px-2.5 py-2 text-left","hover:bg-[var(--bg-hover)] active:bg-[var(--bg-active)]","focus:outline-none focus-visible:ring-2 focus-visible:ring-[var(--text-primary)]/20","theme-transition"),style:{backgroundColor:t?"var(--bg-active)":"transparent"},"aria-current":t?"page":void 0,children:[R.jsx("span",{className:"min-w-0 flex-1 truncate text-sm",style:{color:"var(--text-primary)"},children:e.title||"Untitled"}),R.jsx("button",{ref:c,type:"button",onClick:f=>{f.stopPropagation(),s(m=>!m)},className:dr("shrink-0 rounded-lg p-1.5","hover:bg-[var(--bg-hover)] active:bg-[var(--bg-active)]","opacity-0 group-hover:opacity-100 focus:opacity-100","focus:outline-none focus-visible:ring-2 focus-visible:ring-[var(--text-primary)]/20","theme-transition"),style:{color:"var(--text-tertiary)"},"aria-label":"Chat options","aria-haspopup":"menu","aria-expanded":a,"aria-controls":d,children:R.jsx(lY,{})})]}),a&&R.jsx("div",{id:d,ref:o,role:"menu",className:dr("absolute right-2 top-[calc(100%+6px)] z-50 min-w-[170px]","rounded-xl shadow-lg","overflow-hidden dropdown-menu"),style:{backgroundColor:"var(--bg-primary)",border:"1px solid var(--border-primary)"},children:R.jsxs("button",{role:"menuitem",onClick:f=>{f.stopPropagation(),r(e.id),s(!1)},className:dr("w-full flex items-center gap-2 px-3 py-2.5 text-sm","text-red-500 hover:bg-red-500/10 focus:bg-red-500/10","focus:outline-none theme-transition"),children:[R.jsx(sY,{}),"Delete"]})})]})},hY=({user:e,isOnPaidPlan:t,tierName:n,UserButtonComponent:r})=>{var o,c;const a=(e==null?void 0:e.fullName)||(e==null?void 0:e.firstName)||((c=(o=e==null?void 0:e.emailAddresses)==null?void 0:o[0])==null?void 0:c.emailAddress)||"User",s=t?n||"Pro":"Free plan";return R.jsxs("div",{className:"flex items-center gap-3 rounded-xl p-2 hover:bg-[var(--bg-hover)] transition-colors theme-transition",children:[R.jsx("div",{className:"shrink-0",children:r}),R.jsxs("div",{className:"min-w-0 flex-1",children:[R.jsx("div",{className:"truncate text-sm font-semibold",style:{color:"var(--text-primary)"},children:a}),R.jsx("div",{className:"truncate text-xs",style:{color:"var(--text-tertiary)"},children:s})]})]})},fY=({UserButtonComponent:e})=>R.jsx("div",{className:"grid place-items-center p-1",children:e}),mY=({isOpen:e,onClose:t,onToggle:n,chats:r,activeChat:a,onSelectChat:s,onNewChat:o,onDeleteChat:c,user:d,isOnPaidPlan:f,tierName:m,UserButtonComponent:p,isAuthenticated:b})=>{const y=z.useMemo(()=>[...r||[]].sort((C,w)=>new Date(w.updatedAt)-new Date(C.updatedAt)),[r]),T=z.useMemo(()=>{const C={today:[],yesterday:[],previous7Days:[],previous30Days:[],older:[]},w=new Date,O=new Date(w.getFullYear(),w.getMonth(),w.getDate()),D=new Date(O);D.setDate(D.getDate()-1);const Y=new Date(O);Y.setDate(Y.getDate()-7);const H=new Date(O);return H.setDate(H.getDate()-30),y.forEach(B=>{const ee=new Date(B.updatedAt);ee>=O?C.today.push(B):ee>=D?C.yesterday.push(B):ee>=Y?C.previous7Days.push(B):ee>=H?C.previous30Days.push(B):C.older.push(B)}),C},[y]),x=(C,w)=>w!=null&&w.length?R.jsxs("div",{className:"mb-4",children:[R.jsx(cY,{children:C}),R.jsx("div",{className:"space-y-1",children:w.map(O=>R.jsx(dY,{chat:O,isActive:a===O.id,onSelect:s,onDelete:c},O.id))})]}):null;return R.jsxs(R.Fragment,{children:[R.jsx("div",{className:dr("fixed inset-0 z-40 md:hidden transition-opacity duration-300 ease-in-out",e?"opacity-100":"opacity-0 pointer-events-none"),style:{backgroundColor:"var(--bg-overlay)"},onClick:t,"aria-hidden":"true"}),R.jsxs("aside",{className:dr("fixed left-0 top-0 z-50 h-full","backdrop-blur supports-[backdrop-filter]:backdrop-blur-xl","transition-all duration-300 ease-in-out","flex flex-col",e?"translate-x-0":"-translate-x-full md:translate-x-0"),style:{width:`${e?gf.expanded:gf.collapsed}px`,backgroundColor:"var(--bg-sidebar)",borderRight:"1px solid var(--border-primary)"},children:[R.jsxs("div",{className:dr("flex h-full w-full flex-col","transition-opacity duration-200",e?"opacity-100":"opacity-0 pointer-events-none absolute inset-0"),children:[R.jsxs("div",{className:"flex h-14 items-center justify-between px-2",children:[R.jsxs("button",{onClick:o,className:dr("inline-flex items-center gap-2 rounded-xl px-2.5 py-2","hover:bg-[var(--bg-hover)] active:bg-[var(--bg-active)]","focus:outline-none focus-visible:ring-2 focus-visible:ring-[var(--text-primary)]/20","theme-transition"),style:{color:"var(--text-primary)"},"aria-label":"New chat",title:"New chat",children:[R.jsx(L6,{children:R.jsx(P6,{})}),R.jsx("span",{className:"text-sm font-semibold",children:"New"})]}),R.jsx("button",{onClick:n,className:dr("rounded-xl p-2 hover:bg-[var(--bg-hover)] active:bg-[var(--bg-active)]","focus:outline-none focus-visible:ring-2 focus-visible:ring-[var(--text-primary)]/20","theme-transition"),style:{color:"var(--text-primary)"},"aria-label":"Collapse sidebar",title:"Collapse",children:R.jsx(B6,{})})]}),R.jsx("div",{className:"mx-3",style:{borderTop:"1px solid var(--border-primary)"}}),R.jsx("div",{className:"flex-1 overflow-y-auto px-2 py-3 scrollbar-thin",children:y.length===0?R.jsx("div",{className:"px-2 py-10 text-center text-sm",style:{color:"var(--text-tertiary)"},children:"No chats yet"}):R.jsxs(R.Fragment,{children:[x("Today",T.today),x("Yesterday",T.yesterday),x("Previous 7 days",T.previous7Days),x("Previous 30 days",T.previous30Days),x("Older",T.older)]})}),R.jsx("div",{className:"p-2",style:{borderTop:"1px solid var(--border-primary)"},children:b&&d?R.jsx(hY,{user:d,isOnPaidPlan:f,tierName:m,UserButtonComponent:p}):R.jsxs("div",{className:"px-2 py-2 text-center text-xs",style:{color:"var(--text-tertiary)"},children:[r.length," / ",Zi.MAX_CHATS," chats"]})})]}),R.jsxs("div",{className:dr("hidden md:flex h-full w-full flex-col","transition-opacity duration-200",e?"opacity-0 pointer-events-none absolute inset-0":"opacity-100"),children:[R.jsx("div",{className:"flex h-14 items-center justify-center",children:R.jsx("button",{onClick:n,className:dr("rounded-xl p-2 hover:bg-[var(--bg-hover)] active:bg-[var(--bg-active)]","focus:outline-none focus-visible:ring-2 focus-visible:ring-[var(--text-primary)]/20","theme-transition"),style:{color:"var(--text-primary)"},"aria-label":"Expand sidebar",title:"Expand",children:R.jsx(B6,{})})}),R.jsx("div",{className:"px-1 py-2",children:R.jsx(uY,{onClick:o,icon:R.jsx(L6,{children:R.jsx(P6,{})}),isCollapsed:!0,"aria-label":"New chat",title:"New chat"})}),R.jsx("div",{className:"flex-1"}),b&&p&&R.jsx("div",{className:"p-2",style:{borderTop:"1px solid var(--border-primary)"},children:R.jsx(fY,{UserButtonComponent:p})})]})]}),R.jsx("button",{onClick:n,className:dr("fixed left-3 top-3 z-30 md:hidden","rounded-xl backdrop-blur","p-2 hover:bg-[var(--bg-hover)] active:bg-[var(--bg-active)]","focus:outline-none focus-visible:ring-2 focus-visible:ring-[var(--text-primary)]/20","theme-transition transition-opacity duration-300 ease-in-out",e?"opacity-0 pointer-events-none":"opacity-100"),style:{backgroundColor:"var(--bg-sidebar)",border:"1px solid var(--border-primary)",color:"var(--text-primary)"},"aria-label":"Open sidebar",title:"Open",children:R.jsx(iY,{})})]})},pY=({open:e,onClose:t,tier:n})=>e?R.jsx("div",{className:"fixed inset-0 z-50 flex items-center justify-center backdrop-blur-md px-4",style:{backgroundColor:"var(--bg-overlay)"},children:R.jsxs("div",{className:"relative w-full max-w-xl rounded-3xl shadow-2xl overflow-hidden theme-transition",style:{background:"linear-gradient(to bottom, var(--bg-primary), var(--bg-secondary))",border:"1px solid var(--border-primary)"},children:[R.jsx("div",{className:"absolute top-0 left-0 right-0 h-32 pointer-events-none opacity-50",style:{background:"linear-gradient(to bottom right, rgba(59, 130, 246, 0.1), rgba(99, 102, 241, 0.05), transparent)"}}),R.jsx("button",{type:"button",onClick:t,className:"absolute right-4 top-4 z-10 inline-flex h-9 w-9 items-center justify-center rounded-full backdrop-blur-sm transition-all duration-200 shadow-sm theme-transition",style:{backgroundColor:"var(--bg-primary)",border:"1px solid var(--border-primary)",color:"var(--text-tertiary)"},"aria-label":"Close",children:R.jsx("span",{className:"text-xl leading-none",children:"×"})}),R.jsxs("div",{className:"relative p-8 md:p-10",children:[R.jsxs("div",{className:"space-y-3 pr-8 mb-6",children:[R.jsxs("div",{className:"inline-flex items-center gap-2 px-3 py-1 rounded-full bg-blue-500/10 text-blue-500 border border-blue-500/20",children:[R.jsx("svg",{className:"w-3.5 h-3.5",fill:"currentColor",viewBox:"0 0 20 20",children:R.jsx("path",{fillRule:"evenodd",d:"M11.3 1.046A1 1 0 0112 2v5h4a1 1 0 01.82 1.573l-7 10A1 1 0 018 18v-5H4a1 1 0 01-.82-1.573l7-10a1 1 0 011.12-.38z",clipRule:"evenodd"})}),R.jsx("span",{className:"text-xs font-semibold uppercase tracking-wide",children:"Upgrade Available"})]}),R.jsxs("h2",{className:"text-2xl md:text-3xl font-bold leading-tight",style:{color:"var(--text-primary)"},children:["Unlock"," ",R.jsx("span",{className:"bg-gradient-to-r from-blue-600 to-indigo-600 bg-clip-text text-transparent",children:"Cycls Pass"})," ","features"]}),R.jsxs("p",{className:"text-base leading-relaxed",style:{color:"var(--text-secondary)"},children:["You've reached your free message limit. Upgrade to access the full"," ",R.jsx("span",{className:"font-semibold",style:{color:"var(--text-primary)"},children:"Cycls Pass"})," ","experience, priority AI, and unlimited messages."]})]}),R.jsx("div",{className:"relative -mx-2 mb-6",children:R.jsx("div",{className:"max-h-[420px] overflow-y-auto rounded-2xl backdrop-blur-sm shadow-inner scrollbar-thin",style:{backgroundColor:"var(--bg-tertiary)",border:"1px solid var(--border-primary)"},children:R.jsx(pO,{})})}),R.jsx("div",{className:"flex justify-end",children:R.jsx("button",{type:"button",onClick:t,className:"inline-flex items-center gap-2 rounded-xl px-5 py-2.5 text-sm font-medium transition-all duration-200 hover:bg-[var(--bg-hover)] theme-transition",style:{color:"var(--text-secondary)"},children:"Maybe later"})})]})]})}):null,gY=({afterUrl:e})=>{const t=ls(),{resolvedTheme:n}=j5(),r=t.hash==="#sign-up/"||t.hash==="#sign-up"?"sign-up":"sign-in",a={elements:{card:"shadow-none border-none",footer:"hidden",formButtonPrimary:"py-2.5 rounded-lg transition-colors",formFieldInput:"border-gray-300 focus:border-blue-500 focus:ring-blue-500 rounded-lg",formFieldLabel:"text-gray-700 font-medium text-sm",dividerLine:"bg-gray-200",dividerText:"text-gray-500 text-sm"}};return R.jsxs("div",{className:"min-h-screen flex items-center justify-center px-4 theme-transition",style:{backgroundColor:"var(--bg-secondary)"},children:[R.jsx("div",{className:"fixed top-4 right-4",children:R.jsx($5,{})}),R.jsx("div",{className:"w-full max-w-md",children:r==="sign-in"?R.jsxs("div",{className:"space-y-4",children:[R.jsx(sO,{routing:"virtual",fallbackRedirectUrl:e,signUpUrl:"#sign-up",appearance:a}),R.jsx("div",{style:{marginLeft:"-30px"},className:"text-center",children:R.jsxs("p",{className:"text-sm",style:{color:"var(--text-secondary)"},children:["Don't have an account?"," ",R.jsx("a",{href:"#sign-up/",className:"font-medium hover:underline transition-colors",style:{color:"var(--accent-primary)"},children:"Sign up"})]})})]}):R.jsxs("div",{className:"space-y-4",children:[R.jsx(lO,{routing:"virtual",fallbackRedirectUrl:e,signInUrl:"#sign-in",appearance:a}),R.jsx("div",{style:{marginLeft:"-40px"},className:"text-center",children:R.jsxs("p",{className:"text-sm",style:{color:"var(--text-secondary)"},children:["Already have an account?"," ",R.jsx("a",{href:"#sign-in/",className:"font-medium hover:underline transition-colors",style:{color:"var(--accent-primary)"},children:"Sign in"})]})})]})})]})},bY=({children:e})=>R.jsxs(R.Fragment,{children:[R.jsx(XN,{children:e}),R.jsx(QN,{children:R.jsx(t8,{to:"/auth",replace:!0})})]});Qt(({clerk:e,children:t,...n})=>{const{planId:r,planPeriod:a,for:s,onSubscriptionComplete:o,newSubscriptionRedirectUrl:c,checkoutProps:d,...f}=n,{userId:m,orgId:p}=w0();if(m===null)throw new Error("Clerk: Ensure that `<CheckoutButton />` is rendered inside a `<SignedIn />` component.");if(p===null&&s==="organization")throw new Error('Clerk: Wrap `<CheckoutButton for="organization" />` with a check for an active organization. Retrieve `orgId` from `useAuth()` and confirm it is defined. For SSR, see: https://clerk.com/docs/reference/backend/types/auth-object#how-to-access-the-auth-object');t=Wl(t,"Checkout");const b=Kl(t)("CheckoutButton"),y=()=>{if(e)return e.__internal_openCheckout({planId:r,planPeriod:a,for:s,onSubscriptionComplete:o,newSubscriptionRedirectUrl:c,...d})},x={...f,onClick:async C=>(b&&typeof b=="object"&&"props"in b&&await Xl(b.props.onClick)(C),y())};return ce.cloneElement(b,x)},{component:"CheckoutButton",renderWhileLoading:!0});Qt(({clerk:e,children:t,...n})=>{const{plan:r,planId:a,initialPlanPeriod:s,planDetailsProps:o,...c}=n;t=Wl(t,"Plan details");const d=Kl(t)("PlanDetailsButton"),f=()=>{if(e)return e.__internal_openPlanDetails({plan:r,planId:a,initialPlanPeriod:s,...o})},p={...c,onClick:async b=>(d&&typeof d=="object"&&"props"in d&&await Xl(d.props.onClick)(b),f())};return ce.cloneElement(d,p)},{component:"PlanDetailsButton",renderWhileLoading:!0});Qt(({clerk:e,children:t,...n})=>{const{for:r,subscriptionDetailsProps:a,onSubscriptionCancel:s,...o}=n;t=Wl(t,"Subscription details");const c=Kl(t)("SubscriptionDetailsButton"),{userId:d,orgId:f}=w0();if(d===null)throw new Error("Clerk: Ensure that `<SubscriptionDetailsButton />` is rendered inside a `<SignedIn />` component.");if(f===null&&r==="organization")throw new Error('Clerk: Wrap `<SubscriptionDetailsButton for="organization" />` with a check for an active organization. Retrieve `orgId` from `useAuth()` and confirm it is defined. For SSR, see: https://clerk.com/docs/reference/backend/types/auth-object#how-to-access-the-auth-object');const m=()=>{if(e)return e.__internal_openSubscriptionDetails({for:r,onSubscriptionCancel:s,...a})},b={...o,onClick:async y=>(c&&typeof c=="object"&&"props"in c&&await Xl(c.props.onClick)(y),m())};return ce.cloneElement(c,b)},{component:"SubscriptionDetailsButton",renderWhileLoading:!0});const cS=z.createContext({subscription:null,isLoading:!1,error:null,revalidate:()=>{}}),vY=()=>z.useContext(cS),yY=({tier:e,children:t})=>{const{data:n,isLoading:r,error:a,revalidate:s}=bN({enabled:e==="cycls_pass"}),o=z.useMemo(()=>({subscription:n,isLoading:r,error:a,revalidate:s}),[n,r,a,s]);return R.jsx(cS.Provider,{value:o,children:t})};async function _Y(e,t,{signal:n}={}){var m,p,b,y,T,x;const r=e.body.getReader(),a=new TextDecoder,s=e.headers.get("content-type")||"",o=s.includes("text/event-stream")||s.includes("stream");let c="",d="",f=null;for(;;){if(n!=null&&n.aborted)throw new DOMException("Aborted","AbortError");const{value:C,done:w}=await r.read();if(w)break;if(C){const O=a.decode(C,{stream:!0});if(c+=O,f===null&&c.length>0&&(f=c.trimStart().startsWith("data: ")),o||f){let D;for(;(D=c.indexOf(`
`))!==-1;){const Y=c.slice(0,D).trim();if(c=c.slice(D+1),!!Y&&Y.startsWith("data: ")){const H=Y.slice(6);if(H==="[DONE]")return t(d),d;try{const B=JSON.parse(H),ee=((b=(p=(m=B==null?void 0:B.choices)==null?void 0:m[0])==null?void 0:p.delta)==null?void 0:b.content)||"";ee&&(d+=ee,t(d))}catch(B){console.error("Failed to parse SSE data:",B,H)}}}}else d+=O,t(d),c=""}}if((o||f)&&c.trim()){const C=c.trim();if(C.startsWith("data: ")){const w=C.slice(6);if(w!=="[DONE]")try{const O=JSON.parse(w),D=((x=(T=(y=O==null?void 0:O.choices)==null?void 0:y[0])==null?void 0:T.delta)==null?void 0:x.content)||"";D&&(d+=D)}catch(O){console.error("Failed to parse final SSE data:",O,w)}}}return t(d),d}function EY(e){return e.replace(/<think>[\s\S]*?<\/think>/g,"")}async function TY(e){const t=await fetch("/metadata",{method:"GET",credentials:"include",signal:e});if(!t.ok)throw new Error("Metadata request failed: "+t.status);return t.json()}async function wb({messages:e,auth:t,getToken:n,setActive:r,org:a,onDelta:s,signal:o}){const c=e.map(({type:p,content:b})=>{const y=p==="request"?"user":"assistant",{clean:T,attachments:x}=F0(b);if(x.length===0)return{role:y,content:T};const C=[];T.trim()&&C.push({type:"text",text:T});for(const w of x)w.mime&&w.mime.startsWith("image/")?C.push({type:"image_url",image_url:{url:w.url}}):C.push({type:"file",text:"[File attached: "+w.name+`]
File URL: `+w.url+`
File type: `+(w.mime||"unknown")});return{role:y,content:C}}),d={"Content-Type":"application/json"};if(t&&n)try{const p=await n({template:"template"});if(p&&(d.Authorization="Bearer "+p),r&&a)try{await r({organization:a})}catch{}}catch{}const f=await fetch("/chat/completions",{method:"POST",headers:d,body:JSON.stringify({messages:c,stream:!0}),signal:o});if(!f.ok){let p="server",b="Server returned "+f.status;if(f.status===429){p="rate_limit";const y=f.headers.get("Retry-After");b=y?"Too many requests. Please try again in "+y+" seconds.":"Too many requests. Please wait a moment and try again."}else f.status>=500?b="Server error. Please try again later.":(f.status===401||f.status===403)&&(b="Authentication error. Please refresh and try again.");throw{type:p,status:f.status,message:b}}await _Y(f,p=>{const b=EY(p);s(b)},{signal:o})}const bg="cycls_chat_history",Ah="cycls_active_chat",rv="cycls_message_count";function dS(){try{const e=localStorage.getItem(rv);return e?parseInt(e,10):0}catch{return 0}}function xY(){try{const e=dS();return localStorage.setItem(rv,String(e+1)),e+1}catch{return 0}}function SY(){try{localStorage.removeItem(rv)}catch{}}class ar{static getAllChats(){try{const t=localStorage.getItem(bg);return t?JSON.parse(t):[]}catch(t){return console.error("Error loading chat history:",t),[]}}static saveAllChats(t){try{localStorage.setItem(bg,JSON.stringify(t))}catch(n){console.error("Error saving chat history:",n)}}static getActiveChat(){try{return localStorage.getItem(Ah)}catch{return null}}static setActiveChat(t){try{localStorage.setItem(Ah,t)}catch(n){console.error("Error saving active chat:",n)}}static createChat(t=[]){const n=this.getAllChats(),r={id:Date.now().toString(),title:this.generateTitle(t),messages:t,createdAt:new Date().toISOString(),updatedAt:new Date().toISOString()};return n.length>=Zi.MAX_CHATS&&(n.sort((a,s)=>new Date(a.createdAt)-new Date(s.createdAt)),n.shift()),n.push(r),this.saveAllChats(n),this.setActiveChat(r.id),r}static updateChat(t,n){const r=this.getAllChats(),a=r.findIndex(s=>s.id===t);a!==-1&&(r[a].messages=n,r[a].updatedAt=new Date().toISOString(),r[a].title=this.generateTitle(n),this.saveAllChats(r))}static deleteChat(t){const r=this.getAllChats().filter(a=>a.id!==t);this.saveAllChats(r),this.getActiveChat()===t&&localStorage.removeItem(Ah)}static getChat(t){return this.getAllChats().find(r=>r.id===t)}static generateTitle(t){if(!t||t.length===0)return"New Chat";const n=t.find(s=>s.type==="request");if(!n)return"New Chat";const{clean:r}=F0(n.content);return r.trim().slice(0,50)||"New Chat"}static clearAll(){try{localStorage.removeItem(bg),localStorage.removeItem(Ah)}catch(t){console.error("Error clearing chat history:",t)}}}var be=typeof window<"u"?window:void 0,_r=typeof globalThis<"u"?globalThis:be;typeof self>"u"&&(_r.self=_r),typeof File>"u"&&(_r.File=function(){});var hS=Array.prototype,U6=hS.forEach,F6=hS.indexOf,ya=_r==null?void 0:_r.navigator,$e=_r==null?void 0:_r.document,ta=_r==null?void 0:_r.location,Ab=_r==null?void 0:_r.fetch,kb=_r!=null&&_r.XMLHttpRequest&&"withCredentials"in new _r.XMLHttpRequest?_r.XMLHttpRequest:void 0,z6=_r==null?void 0:_r.AbortController,ea=ya==null?void 0:ya.userAgent,dt=be??{},Yi={DEBUG:!1,LIB_VERSION:"1.306.2"};function H6(e,t,n,r,a,s,o){try{var c=e[s](o),d=c.value}catch(f){return void n(f)}c.done?t(d):Promise.resolve(d).then(r,a)}function q6(e){return function(){var t=this,n=arguments;return new Promise(function(r,a){var s=e.apply(t,n);function o(d){H6(s,r,a,o,c,"next",d)}function c(d){H6(s,r,a,o,c,"throw",d)}o(void 0)})}}function Xe(){return Xe=Object.assign?Object.assign.bind():function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var r in n)({}).hasOwnProperty.call(n,r)&&(e[r]=n[r])}return e},Xe.apply(null,arguments)}function fS(e,t){if(e==null)return{};var n={};for(var r in e)if({}.hasOwnProperty.call(e,r)){if(t.indexOf(r)!==-1)continue;n[r]=e[r]}return n}var wY=["amazonbot","amazonproductbot","app.hypefactors.com","applebot","archive.org_bot","awariobot","backlinksextendedbot","baiduspider","bingbot","bingpreview","chrome-lighthouse","dataforseobot","deepscan","duckduckbot","facebookexternal","facebookcatalog","http://yandex.com/bots","hubspot","ia_archiver","leikibot","linkedinbot","meta-externalagent","mj12bot","msnbot","nessus","petalbot","pinterest","prerender","rogerbot","screaming frog","sebot-wa","sitebulb","slackbot","slurp","trendictionbot","turnitin","twitterbot","vercel-screenshot","vercelbot","yahoo! slurp","yandexbot","zoombot","bot.htm","bot.php","(bot;","bot/","crawler","ahrefsbot","ahrefssiteaudit","semrushbot","siteauditbot","splitsignalbot","gptbot","oai-searchbot","chatgpt-user","perplexitybot","better uptime bot","sentryuptimebot","uptimerobot","headlesschrome","cypress","google-hoteladsverifier","adsbot-google","apis-google","duplexweb-google","feedfetcher-google","google favicon","google web preview","google-read-aloud","googlebot","googleother","google-cloudvertexbot","googleweblight","mediapartners-google","storebot-google","google-inspectiontool","bytespider"],j6=function(e,t){if(t===void 0&&(t=[]),!e)return!1;var n=e.toLowerCase();return wY.concat(t).some(r=>{var a=r.toLowerCase();return n.indexOf(a)!==-1})},AY=["$snapshot","$pageview","$pageleave","$set","survey dismissed","survey sent","survey shown","$identify","$groupidentify","$create_alias","$$client_ingestion_warning","$web_experiment_applied","$feature_enrollment_update","$feature_flag_called"];function It(e,t){return e.indexOf(t)!==-1}var sm=function(e){return e.trim()},Cb=function(e){return e.replace(/^\$/,"")},kY=Array.isArray,mS=Object.prototype,pS=mS.hasOwnProperty,lm=mS.toString,Vt=kY||function(e){return lm.call(e)==="[object Array]"},tl=e=>typeof e=="function",hr=e=>e===Object(e)&&!Vt(e),Vo=e=>{if(hr(e)){for(var t in e)if(pS.call(e,t))return!1;return!0}return!1},qe=e=>e===void 0,ir=e=>lm.call(e)=="[object String]",Nb=e=>ir(e)&&e.trim().length===0,ds=e=>e===null,Gt=e=>qe(e)||ds(e),bi=e=>lm.call(e)=="[object Number]",nl=e=>lm.call(e)==="[object Boolean]",CY=e=>e instanceof FormData,NY=e=>It(AY,e);function Ob(e){return e===null||typeof e!="object"}function bf(e,t){return Object.prototype.toString.call(e)==="[object "+t+"]"}function gS(e){return!qe(Event)&&function(t,n){try{return t instanceof n}catch{return!1}}(e,Event)}var OY=[!0,"true",1,"1","yes"],vg=e=>It(OY,e),RY=[!1,"false",0,"0","no"];function mi(e,t,n,r,a){return t>n&&(r.warn("min cannot be greater than max."),t=n),bi(e)?e>n?(r.warn(" cannot be  greater than max: "+n+". Using max value instead."),n):e<t?(r.warn(" cannot be less than min: "+t+". Using min value instead."),t):e:(r.warn(" must be a number. using max or fallback. max: "+n+", fallback: "+a),mi(a||n,t,n,r))}class MY{constructor(t){this.t={},this.i=t.i,this.o=mi(t.bucketSize,0,100,t.h),this.m=mi(t.refillRate,0,this.o,t.h),this.S=mi(t.refillInterval,0,864e5,t.h)}$(t,n){var r=n-t.lastAccess,a=Math.floor(r/this.S);if(a>0){var s=a*this.m;t.tokens=Math.min(t.tokens+s,this.o),t.lastAccess=t.lastAccess+a*this.S}}consumeRateLimit(t){var n,r=Date.now(),a=String(t),s=this.t[a];return s?this.$(s,r):(s={tokens:this.o,lastAccess:r},this.t[a]=s),s.tokens===0||(s.tokens--,s.tokens===0&&((n=this.i)==null||n.call(this,t)),s.tokens===0)}stop(){this.t={}}}var kh,$6,yg,IY=e=>e instanceof Error;function DY(e){var t=globalThis._posthogChunkIds;if(t){var n=Object.keys(t);return yg&&n.length===$6||($6=n.length,yg=n.reduce((r,a)=>{kh||(kh={});var s=kh[a];if(s)r[s[0]]=s[1];else for(var o=e(a),c=o.length-1;c>=0;c--){var d=o[c],f=d==null?void 0:d.filename,m=t[a];if(f&&m){r[f]=m,kh[a]=[f,m];break}}return r},{})),yg}}class LY{constructor(t,n,r){r===void 0&&(r=[]),this.coercers=t,this.stackParser=n,this.modifiers=r}buildFromUnknown(t,n){n===void 0&&(n={});var r=n&&n.mechanism||{handled:!0,type:"generic"},a=this.buildCoercingContext(r,n,0).apply(t),s=this.buildParsingContext(),o=this.parseStacktrace(a,s);return{$exception_list:this.convertToExceptionList(o,r),$exception_level:"error"}}modifyFrames(t){var n=this;return q6(function*(){for(var r of t)r.stacktrace&&r.stacktrace.frames&&Vt(r.stacktrace.frames)&&(r.stacktrace.frames=yield n.applyModifiers(r.stacktrace.frames));return t})()}coerceFallback(t){var n;return{type:"Error",value:"Unknown error",stack:(n=t.syntheticException)==null?void 0:n.stack,synthetic:!0}}parseStacktrace(t,n){var r,a;return t.cause!=null&&(r=this.parseStacktrace(t.cause,n)),t.stack!=""&&t.stack!=null&&(a=this.applyChunkIds(this.stackParser(t.stack,t.synthetic?1:0),n.chunkIdMap)),Xe({},t,{cause:r,stack:a})}applyChunkIds(t,n){return t.map(r=>(r.filename&&n&&(r.chunk_id=n[r.filename]),r))}applyCoercers(t,n){for(var r of this.coercers)if(r.match(t))return r.coerce(t,n);return this.coerceFallback(n)}applyModifiers(t){var n=this;return q6(function*(){var r=t;for(var a of n.modifiers)r=yield a(r);return r})()}convertToExceptionList(t,n){var r,a,s,o={type:t.type,value:t.value,mechanism:{type:(r=n.type)!==null&&r!==void 0?r:"generic",handled:(a=n.handled)===null||a===void 0||a,synthetic:(s=t.synthetic)!==null&&s!==void 0&&s}};t.stack&&(o.stacktrace={type:"raw",frames:t.stack});var c=[o];return t.cause!=null&&c.push(...this.convertToExceptionList(t.cause,Xe({},n,{handled:!0}))),c}buildParsingContext(){return{chunkIdMap:DY(this.stackParser)}}buildCoercingContext(t,n,r){r===void 0&&(r=0);var a=(s,o)=>{if(o<=4){var c=this.buildCoercingContext(t,n,o);return this.applyCoercers(s,c)}};return Xe({},n,{syntheticException:r==0?n.syntheticException:void 0,mechanism:t,apply:s=>a(s,r),next:s=>a(s,r+1)})}}var au="?";function Rb(e,t,n,r,a){var s={platform:e,filename:t,function:n==="<anonymous>"?au:n,in_app:!0};return qe(r)||(s.lineno=r),qe(a)||(s.colno=a),s}var bS=(e,t)=>{var n=e.indexOf("safari-extension")!==-1,r=e.indexOf("safari-web-extension")!==-1;return n||r?[e.indexOf("@")!==-1?e.split("@")[0]:au,n?"safari-extension:"+t:"safari-web-extension:"+t]:[e,t]},PY=/^\s*at (\S+?)(?::(\d+))(?::(\d+))\s*$/i,BY=/^\s*at (?:(.+?\)(?: \[.+\])?|.*?) ?\((?:address at )?)?(?:async )?((?:<anonymous>|[-a-z]+:|.*bundle|\/)?.*?)(?::(\d+))?(?::(\d+))?\)?\s*$/i,UY=/\((\S*)(?::(\d+))(?::(\d+))\)/,FY=(e,t)=>{var n=PY.exec(e);if(n){var[,r,a,s]=n;return Rb(t,r,au,+a,+s)}var o=BY.exec(e);if(o){if(o[2]&&o[2].indexOf("eval")===0){var c=UY.exec(o[2]);c&&(o[2]=c[1],o[3]=c[2],o[4]=c[3])}var[d,f]=bS(o[1]||au,o[2]);return Rb(t,f,d,o[3]?+o[3]:void 0,o[4]?+o[4]:void 0)}},zY=/^\s*(.*?)(?:\((.*?)\))?(?:^|@)?((?:[-a-z]+)?:\/.*?|\[native code\]|[^@]*(?:bundle|\d+\.js)|\/[\w\-. /=]+)(?::(\d+))?(?::(\d+))?\s*$/i,HY=/(\S+) line (\d+)(?: > eval line \d+)* > eval/i,qY=(e,t)=>{var n=zY.exec(e);if(n){if(n[3]&&n[3].indexOf(" > eval")>-1){var r=HY.exec(n[3]);r&&(n[1]=n[1]||"eval",n[3]=r[1],n[4]=r[2],n[5]="")}var a=n[3],s=n[1]||au;return[s,a]=bS(s,a),Rb(t,a,s,n[4]?+n[4]:void 0,n[5]?+n[5]:void 0)}},Y6=/\(error: (.*)\)/,G6=50;function jY(e){for(var t=arguments.length,n=new Array(t>1?t-1:0),r=1;r<t;r++)n[r-1]=arguments[r];return function(a,s){s===void 0&&(s=0);for(var o=[],c=a.split(`
`),d=s;d<c.length;d++){var f=c[d];if(!(f.length>1024)){var m=Y6.test(f)?f.replace(Y6,"$1"):f;if(!m.match(/\S*Error: /)){for(var p of n){var b=p(m,e);if(b){o.push(b);break}}if(o.length>=G6)break}}}return function(y){if(!y.length)return[];var T=Array.from(y);return T.reverse(),T.slice(0,G6).map(x=>{return Xe({},x,{filename:x.filename||(C=T,C[C.length-1]||{}).filename,function:x.function||au});var C})}(o)}}class $Y{match(t){return this.isDOMException(t)||this.isDOMError(t)}coerce(t,n){var r=ir(t.stack);return{type:this.getType(t),value:this.getValue(t),stack:r?t.stack:void 0,cause:t.cause?n.next(t.cause):void 0,synthetic:!1}}getType(t){return this.isDOMError(t)?"DOMError":"DOMException"}getValue(t){var n=t.name||(this.isDOMError(t)?"DOMError":"DOMException");return t.message?n+": "+t.message:n}isDOMException(t){return bf(t,"DOMException")}isDOMError(t){return bf(t,"DOMError")}}class YY{match(t){return(n=>n instanceof Error)(t)}coerce(t,n){return{type:this.getType(t),value:this.getMessage(t,n),stack:this.getStack(t),cause:t.cause?n.next(t.cause):void 0,synthetic:!1}}getType(t){return t.name||t.constructor.name}getMessage(t,n){var r=t.message;return r.error&&typeof r.error.message=="string"?String(r.error.message):String(r)}getStack(t){return t.stacktrace||t.stack||void 0}}class GY{constructor(){}match(t){return bf(t,"ErrorEvent")&&t.error!=null}coerce(t,n){var r,a=n.apply(t.error);return a||{type:"ErrorEvent",value:t.message,stack:(r=n.syntheticException)==null?void 0:r.stack,synthetic:!0}}}var VY=/^(?:[Uu]ncaught (?:exception: )?)?(?:((?:Eval|Internal|Range|Reference|Syntax|Type|URI|)Error): )?(.*)$/i;class KY{match(t){return typeof t=="string"}coerce(t,n){var r,[a,s]=this.getInfos(t);return{type:a??"Error",value:s??t,stack:(r=n.syntheticException)==null?void 0:r.stack,synthetic:!0}}getInfos(t){var n="Error",r=t,a=t.match(VY);return a&&(n=a[1],r=a[2]),[n,r]}}var WY=["fatal","error","warning","log","info","debug"];function vS(e,t){t===void 0&&(t=40);var n=Object.keys(e);if(n.sort(),!n.length)return"[object has no keys]";for(var r=n.length;r>0;r--){var a=n.slice(0,r).join(", ");if(!(a.length>t))return r===n.length||a.length<=t?a:a.slice(0,t)+"..."}return""}class XY{match(t){return typeof t=="object"&&t!==null}coerce(t,n){var r,a=this.getErrorPropertyFromObject(t);return a?n.apply(a):{type:this.getType(t),value:this.getValue(t),stack:(r=n.syntheticException)==null?void 0:r.stack,level:this.isSeverityLevel(t.level)?t.level:"error",synthetic:!0}}getType(t){return gS(t)?t.constructor.name:"Error"}getValue(t){if("name"in t&&typeof t.name=="string"){var n="'"+t.name+"' captured as exception";return"message"in t&&typeof t.message=="string"&&(n+=" with message: '"+t.message+"'"),n}if("message"in t&&typeof t.message=="string")return t.message;var r=this.getObjectClassName(t);return(r&&r!=="Object"?"'"+r+"'":"Object")+" captured as exception with keys: "+vS(t)}isSeverityLevel(t){return ir(t)&&!Nb(t)&&WY.indexOf(t)>=0}getErrorPropertyFromObject(t){for(var n in t)if(Object.prototype.hasOwnProperty.call(t,n)){var r=t[n];if(IY(r))return r}}getObjectClassName(t){try{var n=Object.getPrototypeOf(t);return n?n.constructor.name:void 0}catch{return}}}class QY{match(t){return gS(t)}coerce(t,n){var r,a=t.constructor.name;return{type:a,value:a+" captured as exception with keys: "+vS(t),stack:(r=n.syntheticException)==null?void 0:r.stack,synthetic:!0}}}class ZY{match(t){return Ob(t)}coerce(t,n){var r;return{type:"Error",value:"Primitive value captured as exception: "+String(t),stack:(r=n.syntheticException)==null?void 0:r.stack,synthetic:!0}}}class JY{match(t){return bf(t,"PromiseRejectionEvent")}coerce(t,n){var r,a=this.getUnhandledRejectionReason(t);return Ob(a)?{type:"UnhandledRejection",value:"Non-Error promise rejection captured with value: "+String(a),stack:(r=n.syntheticException)==null?void 0:r.stack,synthetic:!0}:n.apply(a)}getUnhandledRejectionReason(t){if(Ob(t))return t;try{if("reason"in t)return t.reason;if("detail"in t&&"reason"in t.detail)return t.detail.reason}catch{}return t}}var yS=function(e,t){var{debugEnabled:n}=t===void 0?{}:t,r={k:function(a){if(be&&(Yi.DEBUG||dt.POSTHOG_DEBUG||n)&&!qe(be.console)&&be.console){for(var s=("__rrweb_original__"in be.console[a])?be.console[a].__rrweb_original__:be.console[a],o=arguments.length,c=new Array(o>1?o-1:0),d=1;d<o;d++)c[d-1]=arguments[d];s(e,...c)}},info:function(){for(var a=arguments.length,s=new Array(a),o=0;o<a;o++)s[o]=arguments[o];r.k("log",...s)},warn:function(){for(var a=arguments.length,s=new Array(a),o=0;o<a;o++)s[o]=arguments[o];r.k("warn",...s)},error:function(){for(var a=arguments.length,s=new Array(a),o=0;o<a;o++)s[o]=arguments[o];r.k("error",...s)},critical:function(){for(var a=arguments.length,s=new Array(a),o=0;o<a;o++)s[o]=arguments[o];console.error(e,...s)},uninitializedWarning:a=>{r.error("You must initialize PostHog before calling "+a)},createLogger:(a,s)=>yS(e+" "+a,s)};return r},Ge=yS("[PostHog.js]"),Zn=Ge.createLogger,eG=Zn("[ExternalScriptsLoader]"),V6=(e,t,n)=>{if(e.config.disable_external_dependency_loading)return eG.warn(t+" was requested but loading of external scripts is disabled."),n("Loading of external scripts is disabled");var r=$e==null?void 0:$e.querySelectorAll("script");if(r){for(var a,s=function(){if(r[o].src===t){var d=r[o];return d.__posthog_loading_callback_fired?{v:n()}:(d.addEventListener("load",f=>{d.__posthog_loading_callback_fired=!0,n(void 0,f)}),d.onerror=f=>n(f),{v:void 0})}},o=0;o<r.length;o++)if(a=s())return a.v}var c=()=>{if(!$e)return n("document not found");var d=$e.createElement("script");if(d.type="text/javascript",d.crossOrigin="anonymous",d.src=t,d.onload=p=>{d.__posthog_loading_callback_fired=!0,n(void 0,p)},d.onerror=p=>n(p),e.config.prepare_external_dependency_script&&(d=e.config.prepare_external_dependency_script(d)),!d)return n("prepare_external_dependency_script returned null");var f,m=$e.querySelectorAll("body > script");m.length>0?(f=m[0].parentNode)==null||f.insertBefore(d,m[0]):$e.body.appendChild(d)};$e!=null&&$e.body?c():$e==null||$e.addEventListener("DOMContentLoaded",c)};dt.__PosthogExtensions__=dt.__PosthogExtensions__||{},dt.__PosthogExtensions__.loadExternalDependency=(e,t,n)=>{var r="/static/"+t+".js?v="+e.version;if(t==="remote-config"&&(r="/array/"+e.config.token+"/config.js"),t==="toolbar"){var a=3e5;r=r+"&t="+Math.floor(Date.now()/a)*a}var s=e.requestRouter.endpointFor("assets",r);V6(e,s,n)},dt.__PosthogExtensions__.loadSiteApp=(e,t,n)=>{var r=e.requestRouter.endpointFor("api",t);V6(e,r,n)};var vf={};function rl(e,t,n){if(Vt(e)){if(U6&&e.forEach===U6)e.forEach(t,n);else if("length"in e&&e.length===+e.length){for(var r=0,a=e.length;r<a;r++)if(r in e&&t.call(n,e[r],r)===vf)return}}}function hn(e,t,n){if(!Gt(e)){if(Vt(e))return rl(e,t,n);if(CY(e)){for(var r of e.entries())if(t.call(n,r[1],r[0])===vf)return}else for(var a in e)if(pS.call(e,a)&&t.call(n,e[a],a)===vf)return}}var Fn=function(e){for(var t=arguments.length,n=new Array(t>1?t-1:0),r=1;r<t;r++)n[r-1]=arguments[r];return rl(n,function(a){for(var s in a)a[s]!==void 0&&(e[s]=a[s])}),e},_u=function(e){for(var t=arguments.length,n=new Array(t>1?t-1:0),r=1;r<t;r++)n[r-1]=arguments[r];return rl(n,function(a){rl(a,function(s){e.push(s)})}),e};function Gh(e){for(var t=Object.keys(e),n=t.length,r=new Array(n);n--;)r[n]=[t[n],e[t[n]]];return r}var K6=function(e){try{return e()}catch{return}},tG=function(e){return function(){try{for(var t=arguments.length,n=new Array(t),r=0;r<t;r++)n[r]=arguments[r];return e.apply(this,n)}catch(a){Ge.critical("Implementation error. Please turn on debug mode and open a ticket on https://app.posthog.com/home#panel=support%3Asupport%3A."),Ge.critical(a)}}},av=function(e){var t={};return hn(e,function(n,r){(ir(n)&&n.length>0||bi(n))&&(t[r]=n)}),t};function nG(e,t){return n=e,r=s=>ir(s)&&!ds(t)?s.slice(0,t):s,a=new Set,function s(o,c){return o!==Object(o)?r?r(o,c):o:a.has(o)?void 0:(a.add(o),Vt(o)?(d=[],rl(o,f=>{d.push(s(f))})):(d={},hn(o,(f,m)=>{a.has(f)||(d[m]=s(f,m))})),d);var d}(n);var n,r,a}var rG=["herokuapp.com","vercel.app","netlify.app"];function aG(e){var t=e==null?void 0:e.hostname;if(!ir(t))return!1;var n=t.split(".").slice(-2).join(".");for(var r of rG)if(n===r)return!1;return!0}function _S(e,t){for(var n=0;n<e.length;n++)if(t(e[n]))return e[n]}function Xn(e,t,n,r){var{capture:a=!1,passive:s=!0}=r??{};e==null||e.addEventListener(t,n,{capture:a,passive:s})}var ES="$people_distinct_id",jc="__alias",$c="__timers",W6="$autocapture_disabled_server_side",Mb="$heatmaps_enabled_server_side",X6="$exception_capture_enabled_server_side",Ib="$error_tracking_suppression_rules",Q6="$error_tracking_capture_extension_exceptions",Z6="$web_vitals_enabled_server_side",TS="$dead_clicks_enabled_server_side",J6="$web_vitals_allowed_metrics",_g="$session_recording_remote_config",yf="$sesid",xS="$session_is_sampled",Ko="$enabled_feature_flags",Yc="$early_access_features",Db="$feature_flag_details",Gc="$stored_person_properties",Hl="$stored_group_properties",Lb="$surveys",Ch="$surveys_activated",_f="$flag_call_reported",Gi="$user_state",Pb="$client_session_props",Bb="$capture_rate_limit",Ub="$initial_campaign_params",Fb="$initial_referrer_info",Ef="$initial_person_info",Tf="$epp",SS="__POSTHOG_TOOLBAR__",Rc="$posthog_cookieless",iG=[ES,jc,"__cmpns",$c,"$session_recording_enabled_server_side",Mb,yf,Ko,Ib,Gi,Yc,Db,Hl,Gc,Lb,_f,Pb,Bb,Ub,Fb,Tf,Ef];function e5(e){return e instanceof Element&&(e.id===SS||!(e.closest==null||!e.closest(".toolbar-global-fade-container")))}function om(e){return!!e&&e.nodeType===1}function al(e,t){return!!e&&!!e.tagName&&e.tagName.toLowerCase()===t.toLowerCase()}function wS(e){return!!e&&e.nodeType===3}function AS(e){return!!e&&e.nodeType===11}function iv(e){return e?sm(e).split(/\s+/):[]}function t5(e){var t=be==null?void 0:be.location.href;return!!(t&&e&&e.some(n=>t.match(n)))}function xf(e){var t="";switch(typeof e.className){case"string":t=e.className;break;case"object":t=(e.className&&"baseVal"in e.className?e.className.baseVal:null)||e.getAttribute("class")||"";break;default:t=""}return iv(t)}function kS(e){return Gt(e)?null:sm(e).split(/(\s+)/).filter(t=>b0(t)).join("").replace(/[\r\n]/g," ").replace(/[ ]+/g," ").substring(0,255)}function z0(e){var t="";return zb(e)&&!MS(e)&&e.childNodes&&e.childNodes.length&&hn(e.childNodes,function(n){var r;wS(n)&&n.textContent&&(t+=(r=kS(n.textContent))!==null&&r!==void 0?r:"")}),sm(t)}function CS(e){return qe(e.target)?e.srcElement||null:(t=e.target)!=null&&t.shadowRoot?e.composedPath()[0]||null:e.target||null;var t}var sv=["a","button","form","input","select","textarea","label"];function NS(e,t){if(qe(t))return!0;var n,r=function(s){if(t.some(o=>s.matches(o)))return{v:!0}};for(var a of e)if(n=r(a))return n.v;return!1}function OS(e){var t=e.parentNode;return!(!t||!om(t))&&t}var sG=["next","previous","prev",">","<"],n5=10,r5=[".ph-no-rageclick",".ph-no-capture"];function lG(e,t){if(!be||lv(e))return!1;var n,r,a;if(nl(t)?(n=!!t&&r5,r=void 0):(n=(a=t==null?void 0:t.css_selector_ignorelist)!==null&&a!==void 0?a:r5,r=t==null?void 0:t.content_ignorelist),n===!1)return!1;var{targetElementList:s}=RS(e,!1);return!function(o,c){if(o===!1||qe(o))return!1;var d;if(o===!0)d=sG;else{if(!Vt(o))return!1;if(o.length>n5)return Ge.error("[PostHog] content_ignorelist array cannot exceed "+n5+" items. Use css_selector_ignorelist for more complex matching."),!1;d=o.map(f=>f.toLowerCase())}return c.some(f=>{var{safeText:m,ariaLabel:p}=f;return d.some(b=>m.includes(b)||p.includes(b))})}(r,s.map(o=>{var c;return{safeText:z0(o).toLowerCase(),ariaLabel:((c=o.getAttribute("aria-label"))==null?void 0:c.toLowerCase().trim())||""}}))&&!NS(s,n)}var lv=e=>!e||al(e,"html")||!om(e),RS=(e,t)=>{if(!be||lv(e))return{parentIsUsefulElement:!1,targetElementList:[]};for(var n=!1,r=[e],a=e;a.parentNode&&!al(a,"body");)if(AS(a.parentNode))r.push(a.parentNode.host),a=a.parentNode.host;else{var s=OS(a);if(!s)break;if(t||sv.indexOf(s.tagName.toLowerCase())>-1)n=!0;else{var o=be.getComputedStyle(s);o&&o.getPropertyValue("cursor")==="pointer"&&(n=!0)}r.push(s),a=s}return{parentIsUsefulElement:n,targetElementList:r}};function oG(e,t,n,r,a){var s,o,c,d;if(n===void 0&&(n=void 0),!be||lv(e)||(s=n)!=null&&s.url_allowlist&&!t5(n.url_allowlist)||(o=n)!=null&&o.url_ignorelist&&t5(n.url_ignorelist))return!1;if((c=n)!=null&&c.dom_event_allowlist){var f=n.dom_event_allowlist;if(f&&!f.some(T=>t.type===T))return!1}var{parentIsUsefulElement:m,targetElementList:p}=RS(e,r);if(!function(T,x){var C=x==null?void 0:x.element_allowlist;if(qe(C))return!0;var w,O=function(Y){if(C.some(H=>Y.tagName.toLowerCase()===H))return{v:!0}};for(var D of T)if(w=O(D))return w.v;return!1}(p,n)||!NS(p,(d=n)==null?void 0:d.css_selector_allowlist))return!1;var b=be.getComputedStyle(e);if(b&&b.getPropertyValue("cursor")==="pointer"&&t.type==="click")return!0;var y=e.tagName.toLowerCase();switch(y){case"html":return!1;case"form":return(a||["submit"]).indexOf(t.type)>=0;case"input":case"select":case"textarea":return(a||["change","click"]).indexOf(t.type)>=0;default:return m?(a||["click"]).indexOf(t.type)>=0:(a||["click"]).indexOf(t.type)>=0&&(sv.indexOf(y)>-1||e.getAttribute("contenteditable")==="true")}}function zb(e){for(var t=e;t.parentNode&&!al(t,"body");t=t.parentNode){var n=xf(t);if(It(n,"ph-sensitive")||It(n,"ph-no-capture"))return!1}if(It(xf(e),"ph-include"))return!0;var r=e.type||"";if(ir(r))switch(r.toLowerCase()){case"hidden":case"password":return!1}var a=e.name||e.id||"";return!(ir(a)&&/^cc|cardnum|ccnum|creditcard|csc|cvc|cvv|exp|pass|pwd|routing|seccode|securitycode|securitynum|socialsec|socsec|ssn/i.test(a.replace(/[^a-zA-Z0-9]/g,"")))}function MS(e){return!!(al(e,"input")&&!["button","checkbox","submit","reset"].includes(e.type)||al(e,"select")||al(e,"textarea")||e.getAttribute("contenteditable")==="true")}var IS="(4[0-9]{12}(?:[0-9]{3})?)|(5[1-5][0-9]{14})|(6(?:011|5[0-9]{2})[0-9]{12})|(3[47][0-9]{13})|(3(?:0[0-5]|[68][0-9])[0-9]{11})|((?:2131|1800|35[0-9]{3})[0-9]{11})",uG=new RegExp("^(?:"+IS+")$"),cG=new RegExp(IS),DS="\\d{3}-?\\d{2}-?\\d{4}",dG=new RegExp("^("+DS+")$"),hG=new RegExp("("+DS+")");function b0(e,t){return t===void 0&&(t=!0),!(Gt(e)||ir(e)&&(e=sm(e),(t?uG:cG).test((e||"").replace(/[- ]/g,""))||(t?dG:hG).test(e)))}function LS(e){var t=z0(e);return b0(t=(t+" "+PS(e)).trim())?t:""}function PS(e){var t="";return e&&e.childNodes&&e.childNodes.length&&hn(e.childNodes,function(n){var r;if(n&&((r=n.tagName)==null?void 0:r.toLowerCase())==="span")try{var a=z0(n);t=(t+" "+a).trim(),n.childNodes&&n.childNodes.length&&(t=(t+" "+PS(n)).trim())}catch(s){Ge.error("[AutoCapture]",s)}}),t}function fG(e){return function(t){var n=t.map(r=>{var a,s,o="";if(r.tag_name&&(o+=r.tag_name),r.attr_class)for(var c of(r.attr_class.sort(),r.attr_class))o+="."+c.replace(/"/g,"");var d=Xe({},r.text?{text:r.text}:{},{"nth-child":(a=r.nth_child)!==null&&a!==void 0?a:0,"nth-of-type":(s=r.nth_of_type)!==null&&s!==void 0?s:0},r.href?{href:r.href}:{},r.attr_id?{attr_id:r.attr_id}:{},r.attributes),f={};return Gh(d).sort((m,p)=>{var[b]=m,[y]=p;return b.localeCompare(y)}).forEach(m=>{var[p,b]=m;return f[a5(p.toString())]=a5(b.toString())}),o+=":",o+=Gh(f).map(m=>{var[p,b]=m;return p+'="'+b+'"'}).join("")});return n.join(";")}(function(t){return t.map(n=>{var r,a,s={text:(r=n.$el_text)==null?void 0:r.slice(0,400),tag_name:n.tag_name,href:(a=n.attr__href)==null?void 0:a.slice(0,2048),attr_class:mG(n),attr_id:n.attr__id,nth_child:n.nth_child,nth_of_type:n.nth_of_type,attributes:{}};return Gh(n).filter(o=>{var[c]=o;return c.indexOf("attr__")===0}).forEach(o=>{var[c,d]=o;return s.attributes[c]=d}),s})}(e))}function a5(e){return e.replace(/"|\\"/g,'\\"')}function mG(e){var t=e.attr__class;return t?Vt(t)?t:iv(t):void 0}class BS{constructor(t){this.disabled=t===!1;var n=hr(t)?t:{};this.thresholdPx=n.threshold_px||30,this.timeoutMs=n.timeout_ms||1e3,this.clickCount=n.click_count||3,this.clicks=[]}isRageClick(t,n,r){if(this.disabled)return!1;var a=this.clicks[this.clicks.length-1];if(a&&Math.abs(t-a.x)+Math.abs(n-a.y)<this.thresholdPx&&r-a.timestamp<this.timeoutMs){if(this.clicks.push({x:t,y:n,timestamp:r}),this.clicks.length===this.clickCount)return!0}else this.clicks=[{x:t,y:n,timestamp:r}];return!1}}var Eg="$copy_autocapture",Qi=function(e){return e.GZipJS="gzip-js",e.Base64="base64",e}({}),Sf=e=>{var t=$e==null?void 0:$e.createElement("a");return qe(t)?null:(t.href=e,t)},pG=function(e,t){var n,r;t===void 0&&(t="&");var a=[];return hn(e,function(s,o){qe(s)||qe(o)||o==="undefined"||(n=encodeURIComponent((c=>c instanceof File)(s)?s.name:s.toString()),r=encodeURIComponent(o),a[a.length]=r+"="+n)}),a.join(t)},wf=function(e,t){for(var n,r=((e.split("#")[0]||"").split(/\?(.*)/)[1]||"").replace(/^\?+/g,"").split("&"),a=0;a<r.length;a++){var s=r[a].split("=");if(s[0]===t){n=s;break}}if(!Vt(n)||n.length<2)return"";var o=n[1];try{o=decodeURIComponent(o)}catch{Ge.error("Skipping decoding for malformed query param: "+o)}return o.replace(/\+/g," ")},H0=function(e,t,n){if(!e||!t||!t.length)return e;for(var r=e.split("#"),a=r[0]||"",s=r[1],o=a.split("?"),c=o[1],d=o[0],f=(c||"").split("&"),m=[],p=0;p<f.length;p++){var b=f[p].split("=");Vt(b)&&(t.includes(b[0])?m.push(b[0]+"="+n):m.push(f[p]))}var y=d;return c!=null&&(y+="?"+m.join("&")),s!=null&&(y+="#"+s),y},Af=function(e,t){var n=e.match(new RegExp(t+"=([^&]*)"));return n?n[1]:null},i5=Zn("[AutoCapture]");function Tg(e,t){return t.length>e?t.slice(0,e)+"...":t}function gG(e){if(e.previousElementSibling)return e.previousElementSibling;var t=e;do t=t.previousSibling;while(t&&!om(t));return t}function bG(e,t,n,r){var a=e.tagName.toLowerCase(),s={tag_name:a};sv.indexOf(a)>-1&&!n&&(a.toLowerCase()==="a"||a.toLowerCase()==="button"?s.$el_text=Tg(1024,LS(e)):s.$el_text=Tg(1024,z0(e)));var o=xf(e);o.length>0&&(s.classes=o.filter(function(m){return m!==""})),hn(e.attributes,function(m){var p;if((!MS(e)||["name","id","class","aria-label"].indexOf(m.name)!==-1)&&(r==null||!r.includes(m.name))&&!t&&b0(m.value)&&(p=m.name,!ir(p)||p.substring(0,10)!=="_ngcontent"&&p.substring(0,7)!=="_nghost")){var b=m.value;m.name==="class"&&(b=iv(b).join(" ")),s["attr__"+m.name]=Tg(1024,b)}});for(var c=1,d=1,f=e;f=gG(f);)c++,f.tagName===e.tagName&&d++;return s.nth_child=c,s.nth_of_type=d,s}function vG(e,t){for(var n,r,{e:a,maskAllElementAttributes:s,maskAllText:o,elementAttributeIgnoreList:c,elementsChainAsString:d}=t,f=[e],m=e;m.parentNode&&!al(m,"body");)AS(m.parentNode)?(f.push(m.parentNode.host),m=m.parentNode.host):(f.push(m.parentNode),m=m.parentNode);var p,b=[],y={},T=!1,x=!1;if(hn(f,Y=>{var H=zb(Y);Y.tagName.toLowerCase()==="a"&&(T=Y.getAttribute("href"),T=H&&T&&b0(T)&&T),It(xf(Y),"ph-no-capture")&&(x=!0),b.push(bG(Y,s,o,c));var B=function(ee){if(!zb(ee))return{};var Z={};return hn(ee.attributes,function(Q){if(Q.name&&Q.name.indexOf("data-ph-capture-attribute")===0){var G=Q.name.replace("data-ph-capture-attribute-",""),se=Q.value;G&&se&&b0(se)&&(Z[G]=se)}}),Z}(Y);Fn(y,B)}),x)return{props:{},explicitNoCapture:x};if(o||(e.tagName.toLowerCase()==="a"||e.tagName.toLowerCase()==="button"?b[0].$el_text=LS(e):b[0].$el_text=z0(e)),T){var C,w;b[0].attr__href=T;var O=(C=Sf(T))==null?void 0:C.host,D=be==null||(w=be.location)==null?void 0:w.host;O&&D&&O!==D&&(p=T)}return{props:Fn({$event_type:a.type,$ce_version:1},d?{}:{$elements:b},{$elements_chain:fG(b)},(n=b[0])!=null&&n.$el_text?{$el_text:(r=b[0])==null?void 0:r.$el_text}:{},p&&a.type==="click"?{$external_click_url:p}:{},y)}}class yG{constructor(t){this.P=!1,this.T=null,this.R=!1,this.instance=t,this.rageclicks=new BS(t.config.rageclick),this.C=null}get I(){var t,n,r=hr(this.instance.config.autocapture)?this.instance.config.autocapture:{};return r.url_allowlist=(t=r.url_allowlist)==null?void 0:t.map(a=>new RegExp(a)),r.url_ignorelist=(n=r.url_ignorelist)==null?void 0:n.map(a=>new RegExp(a)),r}F(){if(this.isBrowserSupported()){if(be&&$e){var t=r=>{r=r||(be==null?void 0:be.event);try{this.M(r)}catch(a){i5.error("Failed to capture event",a)}};if(Xn($e,"submit",t,{capture:!0}),Xn($e,"change",t,{capture:!0}),Xn($e,"click",t,{capture:!0}),this.I.capture_copied_text){var n=r=>{r=r||(be==null?void 0:be.event),this.M(r,Eg)};Xn($e,"copy",n,{capture:!0}),Xn($e,"cut",n,{capture:!0})}}}else i5.info("Disabling Automatic Event Collection because this browser is not supported")}startIfEnabled(){this.isEnabled&&!this.P&&(this.F(),this.P=!0)}onRemoteConfig(t){t.elementsChainAsString&&(this.R=t.elementsChainAsString),this.instance.persistence&&this.instance.persistence.register({[W6]:!!t.autocapture_opt_out}),this.T=!!t.autocapture_opt_out,this.startIfEnabled()}setElementSelectors(t){this.C=t}getElementSelectors(t){var n,r=[];return(n=this.C)==null||n.forEach(a=>{var s=$e==null?void 0:$e.querySelectorAll(a);s==null||s.forEach(o=>{t===o&&r.push(a)})}),r}get isEnabled(){var t,n,r=(t=this.instance.persistence)==null?void 0:t.props[W6],a=this.T;if(ds(a)&&!nl(r)&&!this.instance.O())return!1;var s=(n=this.T)!==null&&n!==void 0?n:!!r;return!!this.instance.config.autocapture&&!s}M(t,n){if(n===void 0&&(n="$autocapture"),this.isEnabled){var r,a=CS(t);wS(a)&&(a=a.parentNode||null),n==="$autocapture"&&t.type==="click"&&t instanceof MouseEvent&&this.instance.config.rageclick&&(r=this.rageclicks)!=null&&r.isRageClick(t.clientX,t.clientY,t.timeStamp||new Date().getTime())&&lG(a,this.instance.config.rageclick)&&this.M(t,"$rageclick");var s=n===Eg;if(a&&oG(a,t,this.I,s,s?["copy","cut"]:void 0)){var{props:o,explicitNoCapture:c}=vG(a,{e:t,maskAllElementAttributes:this.instance.config.mask_all_element_attributes,maskAllText:this.instance.config.mask_all_text,elementAttributeIgnoreList:this.I.element_attribute_ignorelist,elementsChainAsString:this.R});if(c)return!1;var d=this.getElementSelectors(a);if(d&&d.length>0&&(o.$element_selectors=d),n===Eg){var f,m=kS(be==null||(f=be.getSelection())==null?void 0:f.toString()),p=t.type||"clipboard";if(!m)return!1;o.$selected_content=m,o.$copy_type=p}return this.instance.capture(n,o),!0}}}isBrowserSupported(){return tl($e==null?void 0:$e.querySelectorAll)}}Math.trunc||(Math.trunc=function(e){return e<0?Math.ceil(e):Math.floor(e)}),Number.isInteger||(Number.isInteger=function(e){return bi(e)&&isFinite(e)&&Math.floor(e)===e});var s5="0123456789abcdef";class kf{constructor(t){if(this.bytes=t,t.length!==16)throw new TypeError("not 128-bit length")}static fromFieldsV7(t,n,r,a){if(!Number.isInteger(t)||!Number.isInteger(n)||!Number.isInteger(r)||!Number.isInteger(a)||t<0||n<0||r<0||a<0||t>0xffffffffffff||n>4095||r>1073741823||a>4294967295)throw new RangeError("invalid field value");var s=new Uint8Array(16);return s[0]=t/Math.pow(2,40),s[1]=t/Math.pow(2,32),s[2]=t/Math.pow(2,24),s[3]=t/Math.pow(2,16),s[4]=t/Math.pow(2,8),s[5]=t,s[6]=112|n>>>8,s[7]=n,s[8]=128|r>>>24,s[9]=r>>>16,s[10]=r>>>8,s[11]=r,s[12]=a>>>24,s[13]=a>>>16,s[14]=a>>>8,s[15]=a,new kf(s)}toString(){for(var t="",n=0;n<this.bytes.length;n++)t=t+s5.charAt(this.bytes[n]>>>4)+s5.charAt(15&this.bytes[n]),n!==3&&n!==5&&n!==7&&n!==9||(t+="-");if(t.length!==36)throw new Error("Invalid UUIDv7 was generated");return t}clone(){return new kf(this.bytes.slice(0))}equals(t){return this.compareTo(t)===0}compareTo(t){for(var n=0;n<16;n++){var r=this.bytes[n]-t.bytes[n];if(r!==0)return Math.sign(r)}return 0}}class _G{constructor(){this.A=0,this.D=0,this.j=new EG}generate(){var t=this.generateOrAbort();if(qe(t)){this.A=0;var n=this.generateOrAbort();if(qe(n))throw new Error("Could not generate UUID after timestamp reset");return n}return t}generateOrAbort(){var t=Date.now();if(t>this.A)this.A=t,this.L();else{if(!(t+1e4>this.A))return;this.D++,this.D>4398046511103&&(this.A++,this.L())}return kf.fromFieldsV7(this.A,Math.trunc(this.D/Math.pow(2,30)),this.D&Math.pow(2,30)-1,this.j.nextUint32())}L(){this.D=1024*this.j.nextUint32()+(1023&this.j.nextUint32())}}var l5,US=e=>{if(typeof UUIDV7_DENY_WEAK_RNG<"u"&&UUIDV7_DENY_WEAK_RNG)throw new Error("no cryptographically strong RNG available");for(var t=0;t<e.length;t++)e[t]=65536*Math.trunc(65536*Math.random())+Math.trunc(65536*Math.random());return e};be&&!qe(be.crypto)&&crypto.getRandomValues&&(US=e=>crypto.getRandomValues(e));class EG{constructor(){this.N=new Uint32Array(8),this.U=1/0}nextUint32(){return this.U>=this.N.length&&(US(this.N),this.U=0),this.N[this.U++]}}var el=()=>TG().toString(),TG=()=>(l5||(l5=new _G)).generate(),Mc="",xG=/[a-z0-9][a-z0-9-]+\.[a-z]{2,}$/i;function SG(e,t){if(t){var n=function(a,s){if(s===void 0&&(s=$e),Mc)return Mc;if(!s||["localhost","127.0.0.1"].includes(a))return"";for(var o=a.split("."),c=Math.min(o.length,8),d="dmn_chk_"+el();!Mc&&c--;){var f=o.slice(c).join("."),m=d+"=1;domain=."+f+";path=/";s.cookie=m+";max-age=3",s.cookie.includes(d)&&(s.cookie=m+";max-age=0",Mc=f)}return Mc}(e);if(!n){var r=(a=>{var s=a.match(xG);return s?s[0]:""})(e);r!==n&&Ge.info("Warning: cookie subdomain discovery mismatch",r,n),n=r}return n?"; domain=."+n:""}return""}var gi={H:()=>!!$e,B:function(e){Ge.error("cookieStore error: "+e)},q:function(e){if($e){try{for(var t=e+"=",n=$e.cookie.split(";").filter(s=>s.length),r=0;r<n.length;r++){for(var a=n[r];a.charAt(0)==" ";)a=a.substring(1,a.length);if(a.indexOf(t)===0)return decodeURIComponent(a.substring(t.length,a.length))}}catch{}return null}},W:function(e){var t;try{t=JSON.parse(gi.q(e))||{}}catch{}return t},G:function(e,t,n,r,a){if($e)try{var s="",o="",c=SG($e.location.hostname,r);if(n){var d=new Date;d.setTime(d.getTime()+24*n*60*60*1e3),s="; expires="+d.toUTCString()}a&&(o="; secure");var f=e+"="+encodeURIComponent(JSON.stringify(t))+s+"; SameSite=Lax; path=/"+c+o;return f.length>3686.4&&Ge.warn("cookieStore warning: large cookie, len="+f.length),$e.cookie=f,f}catch{return}},V:function(e,t){if($e!=null&&$e.cookie)try{gi.G(e,"",-1,t)}catch{return}}},xg=null,On={H:function(){if(!ds(xg))return xg;var e=!0;if(qe(be))e=!1;else try{var t="__mplssupport__";On.G(t,"xyz"),On.q(t)!=='"xyz"'&&(e=!1),On.V(t)}catch{e=!1}return e||Ge.error("localStorage unsupported; falling back to cookie store"),xg=e,e},B:function(e){Ge.error("localStorage error: "+e)},q:function(e){try{return be==null?void 0:be.localStorage.getItem(e)}catch(t){On.B(t)}return null},W:function(e){try{return JSON.parse(On.q(e))||{}}catch{}return null},G:function(e,t){try{be==null||be.localStorage.setItem(e,JSON.stringify(t))}catch(n){On.B(n)}},V:function(e){try{be==null||be.localStorage.removeItem(e)}catch(t){On.B(t)}}},wG=["distinct_id",yf,xS,Tf,Ef],Nh=Xe({},On,{W:function(e){try{var t={};try{t=gi.W(e)||{}}catch{}var n=Fn(t,JSON.parse(On.q(e)||"{}"));return On.G(e,n),n}catch{}return null},G:function(e,t,n,r,a,s){try{On.G(e,t,void 0,void 0,s);var o={};wG.forEach(c=>{t[c]&&(o[c]=t[c])}),Object.keys(o).length&&gi.G(e,o,n,r,a,s)}catch(c){On.B(c)}},V:function(e,t){try{be==null||be.localStorage.removeItem(e),gi.V(e,t)}catch(n){On.B(n)}}}),Oh={},AG={H:function(){return!0},B:function(e){Ge.error("memoryStorage error: "+e)},q:function(e){return Oh[e]||null},W:function(e){return Oh[e]||null},G:function(e,t){Oh[e]=t},V:function(e){delete Oh[e]}},Ul=null,yr={H:function(){if(!ds(Ul))return Ul;if(Ul=!0,qe(be))Ul=!1;else try{var e="__support__";yr.G(e,"xyz"),yr.q(e)!=='"xyz"'&&(Ul=!1),yr.V(e)}catch{Ul=!1}return Ul},B:function(e){Ge.error("sessionStorage error: ",e)},q:function(e){try{return be==null?void 0:be.sessionStorage.getItem(e)}catch(t){yr.B(t)}return null},W:function(e){try{return JSON.parse(yr.q(e))||null}catch{}return null},G:function(e,t){try{be==null||be.sessionStorage.setItem(e,JSON.stringify(t))}catch(n){yr.B(n)}},V:function(e){try{be==null||be.sessionStorage.removeItem(e)}catch(t){yr.B(t)}}},Vi=function(e){return e[e.PENDING=-1]="PENDING",e[e.DENIED=0]="DENIED",e[e.GRANTED=1]="GRANTED",e}({});class kG{constructor(t){this._instance=t}get I(){return this._instance.config}get consent(){return this.J()?Vi.DENIED:this.K}isOptedOut(){return this.I.cookieless_mode==="always"||this.consent===Vi.DENIED||this.consent===Vi.PENDING&&(this.I.opt_out_capturing_by_default||this.I.cookieless_mode==="on_reject")}isOptedIn(){return!this.isOptedOut()}isExplicitlyOptedOut(){return this.consent===Vi.DENIED}optInOut(t){this.Y.G(this.X,t?1:0,this.I.cookie_expiration,this.I.cross_subdomain_cookie,this.I.secure_cookie)}reset(){this.Y.V(this.X,this.I.cross_subdomain_cookie)}get X(){var{token:t,opt_out_capturing_cookie_prefix:n,consent_persistence_name:r}=this._instance.config;return r||(n?n+t:"__ph_opt_in_out_"+t)}get K(){var t=this.Y.q(this.X);return vg(t)?Vi.GRANTED:It(RY,t)?Vi.DENIED:Vi.PENDING}get Y(){if(!this.Z){var t=this.I.opt_out_capturing_persistence_type;this.Z=t==="localStorage"?On:gi;var n=t==="localStorage"?gi:On;n.q(this.X)&&(this.Z.q(this.X)||this.optInOut(vg(n.q(this.X))),n.V(this.X,this.I.cross_subdomain_cookie))}return this.Z}J(){return!!this.I.respect_dnt&&!!_S([ya==null?void 0:ya.doNotTrack,ya==null?void 0:ya.msDoNotTrack,dt.doNotTrack],t=>vg(t))}}var Rh=Zn("[Dead Clicks]"),CG=()=>!0,NG=e=>{var t,n=!((t=e.instance.persistence)==null||!t.get_property(TS)),r=e.instance.config.capture_dead_clicks;return nl(r)?r:!!hr(r)||n};class FS{get lazyLoadedDeadClicksAutocapture(){return this.tt}constructor(t,n,r){this.instance=t,this.isEnabled=n,this.onCapture=r,this.startIfEnabled()}onRemoteConfig(t){this.instance.persistence&&this.instance.persistence.register({[TS]:t==null?void 0:t.captureDeadClicks}),this.startIfEnabled()}startIfEnabled(){this.isEnabled(this)&&this.it(()=>{this.et()})}it(t){var n,r;(n=dt.__PosthogExtensions__)!=null&&n.initDeadClicksAutocapture&&t(),(r=dt.__PosthogExtensions__)==null||r.loadExternalDependency==null||r.loadExternalDependency(this.instance,"dead-clicks-autocapture",a=>{a?Rh.error("failed to load script",a):t()})}et(){var t;if($e){if(!this.tt&&(t=dt.__PosthogExtensions__)!=null&&t.initDeadClicksAutocapture){var n=hr(this.instance.config.capture_dead_clicks)?this.instance.config.capture_dead_clicks:{};n.__onCapture=this.onCapture,this.tt=dt.__PosthogExtensions__.initDeadClicksAutocapture(this.instance,n),this.tt.start($e),Rh.info("starting...")}}else Rh.error("`document` not found. Cannot start.")}stop(){this.tt&&(this.tt.stop(),this.tt=void 0,Rh.info("stopping..."))}}var Ic=Zn("[ExceptionAutocapture]");class OG{constructor(t){var n,r,a;this.rt=()=>{var s;if(be&&this.isEnabled&&(s=dt.__PosthogExtensions__)!=null&&s.errorWrappingFunctions){var o=dt.__PosthogExtensions__.errorWrappingFunctions.wrapOnError,c=dt.__PosthogExtensions__.errorWrappingFunctions.wrapUnhandledRejection,d=dt.__PosthogExtensions__.errorWrappingFunctions.wrapConsoleError;try{!this.st&&this.I.capture_unhandled_errors&&(this.st=o(this.captureException.bind(this))),!this.nt&&this.I.capture_unhandled_rejections&&(this.nt=c(this.captureException.bind(this))),!this.ot&&this.I.capture_console_errors&&(this.ot=d(this.captureException.bind(this)))}catch(f){Ic.error("failed to start",f),this.lt()}}},this._instance=t,this.ut=!((n=this._instance.persistence)==null||!n.props[X6]),this.I=this.ht(),this.dt=new MY({refillRate:(r=this._instance.config.error_tracking.__exceptionRateLimiterRefillRate)!==null&&r!==void 0?r:1,bucketSize:(a=this._instance.config.error_tracking.__exceptionRateLimiterBucketSize)!==null&&a!==void 0?a:10,refillInterval:1e4,h:Ic}),this.startIfEnabled()}ht(){var t=this._instance.config.capture_exceptions,n={capture_unhandled_errors:!1,capture_unhandled_rejections:!1,capture_console_errors:!1};return hr(t)?n=Xe({},n,t):(qe(t)?this.ut:t)&&(n=Xe({},n,{capture_unhandled_errors:!0,capture_unhandled_rejections:!0})),n}get isEnabled(){return this.I.capture_console_errors||this.I.capture_unhandled_errors||this.I.capture_unhandled_rejections}startIfEnabled(){this.isEnabled&&(Ic.info("enabled"),this.it(this.rt))}it(t){var n,r;(n=dt.__PosthogExtensions__)!=null&&n.errorWrappingFunctions&&t(),(r=dt.__PosthogExtensions__)==null||r.loadExternalDependency==null||r.loadExternalDependency(this._instance,"exception-autocapture",a=>{if(a)return Ic.error("failed to load script",a);t()})}lt(){var t,n,r;(t=this.st)==null||t.call(this),this.st=void 0,(n=this.nt)==null||n.call(this),this.nt=void 0,(r=this.ot)==null||r.call(this),this.ot=void 0}onRemoteConfig(t){var n=t.autocaptureExceptions;this.ut=!!n||!1,this.I=this.ht(),this._instance.persistence&&this._instance.persistence.register({[X6]:this.ut}),this.startIfEnabled()}captureException(t){var n,r,a=(n=t==null||(r=t.$exception_list)==null||(r=r[0])==null?void 0:r.type)!==null&&n!==void 0?n:"Exception";this.dt.consumeRateLimit(a)?Ic.info("Skipping exception capture because of client rate limiting.",{exception:a}):this._instance.exceptions.sendExceptionEvent(t)}}function o5(e,t,n){try{if(!(t in e))return()=>{};var r=e[t],a=n(r);return tl(a)&&(a.prototype=a.prototype||{},Object.defineProperties(a,{__posthog_wrapped__:{enumerable:!1,value:!0}})),e[t]=a,()=>{e[t]=r}}catch{return()=>{}}}class RG{constructor(t){var n;this._instance=t,this.vt=(be==null||(n=be.location)==null?void 0:n.pathname)||""}get isEnabled(){return this._instance.config.capture_pageview==="history_change"}startIfEnabled(){this.isEnabled&&(Ge.info("History API monitoring enabled, starting..."),this.monitorHistoryChanges())}stop(){this.ct&&this.ct(),this.ct=void 0,Ge.info("History API monitoring stopped")}monitorHistoryChanges(){var t,n;if(be&&be.history){var r=this;(t=be.history.pushState)!=null&&t.__posthog_wrapped__||o5(be.history,"pushState",a=>function(s,o,c){a.call(this,s,o,c),r.ft("pushState")}),(n=be.history.replaceState)!=null&&n.__posthog_wrapped__||o5(be.history,"replaceState",a=>function(s,o,c){a.call(this,s,o,c),r.ft("replaceState")}),this.gt()}}ft(t){try{var n,r=be==null||(n=be.location)==null?void 0:n.pathname;if(!r)return;r!==this.vt&&this.isEnabled&&this._instance.capture("$pageview",{navigation_type:t}),this.vt=r}catch(a){Ge.error("Error capturing "+t+" pageview",a)}}gt(){if(!this.ct){var t=()=>{this.ft("popstate")};Xn(be,"popstate",t),this.ct=()=>{be&&be.removeEventListener("popstate",t)}}}}var Sg=Zn("[SegmentIntegration]");function MG(e,t){var n=e.config.segment;if(!n)return t();(function(r,a){var s=r.config.segment;if(!s)return a();var o=d=>{var f=()=>d.anonymousId()||el();r.config.get_device_id=f,d.id()&&(r.register({distinct_id:d.id(),$device_id:f()}),r.persistence.set_property(Gi,"identified")),a()},c=s.user();"then"in c&&tl(c.then)?c.then(d=>o(d)):o(c)})(e,()=>{n.register((r=>{Promise&&Promise.resolve||Sg.warn("This browser does not have Promise support, and can not use the segment integration");var a=(s,o)=>{if(!o)return s;s.event.userId||s.event.anonymousId===r.get_distinct_id()||(Sg.info("No userId set, resetting PostHog"),r.reset()),s.event.userId&&s.event.userId!==r.get_distinct_id()&&(Sg.info("UserId set, identifying with PostHog"),r.identify(s.event.userId));var c=r.calculateEventProperties(o,s.event.properties);return s.event.properties=Object.assign({},c,s.event.properties),s};return{name:"PostHog JS",type:"enrichment",version:"1.0.0",isLoaded:()=>!0,load:()=>Promise.resolve(),track:s=>a(s,s.event.event),page:s=>a(s,"$pageview"),identify:s=>a(s,"$identify"),screen:s=>a(s,"$screen")}})(e)).then(()=>{t()})})}var zS="posthog-js";function HS(e,t){var{organization:n,projectId:r,prefix:a,severityAllowList:s=["error"],sendExceptionsToPostHog:o=!0}=t===void 0?{}:t;return c=>{var d,f,m,p,b;if(!(s==="*"||s.includes(c.level))||!e.__loaded)return c;c.tags||(c.tags={});var y=e.requestRouter.endpointFor("ui","/project/"+e.config.token+"/person/"+e.get_distinct_id());c.tags["PostHog Person URL"]=y,e.sessionRecordingStarted()&&(c.tags["PostHog Recording URL"]=e.get_session_replay_url({withTimestamp:!0}));var T=((d=c.exception)==null?void 0:d.values)||[],x=T.map(w=>Xe({},w,{stacktrace:w.stacktrace?Xe({},w.stacktrace,{type:"raw",frames:(w.stacktrace.frames||[]).map(O=>Xe({},O,{platform:"web:javascript"}))}):void 0})),C={$exception_message:((f=T[0])==null?void 0:f.value)||c.message,$exception_type:(m=T[0])==null?void 0:m.type,$exception_level:c.level,$exception_list:x,$sentry_event_id:c.event_id,$sentry_exception:c.exception,$sentry_exception_message:((p=T[0])==null?void 0:p.value)||c.message,$sentry_exception_type:(b=T[0])==null?void 0:b.type,$sentry_tags:c.tags};return n&&r&&(C.$sentry_url=(a||"https://sentry.io/organizations/")+n+"/issues/?project="+r+"&query="+c.event_id),o&&e.exceptions.sendExceptionEvent(C),c}}class IG{constructor(t,n,r,a,s,o){this.name=zS,this.setupOnce=function(c){c(HS(t,{organization:n,projectId:r,prefix:a,severityAllowList:s,sendExceptionsToPostHog:o==null||o}))}}}var DG=be!=null&&be.location?Af(be.location.hash,"__posthog")||Af(location.hash,"state"):null,u5="_postHogToolbarParams",c5=Zn("[Toolbar]"),Ws=function(e){return e[e.UNINITIALIZED=0]="UNINITIALIZED",e[e.LOADING=1]="LOADING",e[e.LOADED=2]="LOADED",e}(Ws||{});class LG{constructor(t){this.instance=t}_t(t){dt.ph_toolbar_state=t}yt(){var t;return(t=dt.ph_toolbar_state)!==null&&t!==void 0?t:Ws.UNINITIALIZED}maybeLoadToolbar(t,n,r){if(t===void 0&&(t=void 0),n===void 0&&(n=void 0),r===void 0&&(r=void 0),!be||!$e)return!1;t=t??be.location,r=r??be.history;try{if(!n){try{be.localStorage.setItem("test","test"),be.localStorage.removeItem("test")}catch{return!1}n=be==null?void 0:be.localStorage}var a,s=DG||Af(t.hash,"__posthog")||Af(t.hash,"state"),o=s?K6(()=>JSON.parse(atob(decodeURIComponent(s))))||K6(()=>JSON.parse(decodeURIComponent(s))):null;return o&&o.action==="ph_authorize"?((a=o).source="url",a&&Object.keys(a).length>0&&(o.desiredHash?t.hash=o.desiredHash:r?r.replaceState(r.state,"",t.pathname+t.search):t.hash="")):((a=JSON.parse(n.getItem(u5)||"{}")).source="localstorage",delete a.userIntent),!(!a.token||this.instance.config.token!==a.token)&&(this.loadToolbar(a),!0)}catch{return!1}}bt(t){var n=dt.ph_load_toolbar||dt.ph_load_editor;!Gt(n)&&tl(n)?n(t,this.instance):c5.warn("No toolbar load function found")}loadToolbar(t){var n=!($e==null||!$e.getElementById(SS));if(!be||n)return!1;var r=this.instance.requestRouter.region==="custom"&&this.instance.config.advanced_disable_toolbar_metrics,a=Xe({token:this.instance.config.token},t,{apiURL:this.instance.requestRouter.endpointFor("ui")},r?{instrument:!1}:{});if(be.localStorage.setItem(u5,JSON.stringify(Xe({},a,{source:void 0}))),this.yt()===Ws.LOADED)this.bt(a);else if(this.yt()===Ws.UNINITIALIZED){var s;this._t(Ws.LOADING),(s=dt.__PosthogExtensions__)==null||s.loadExternalDependency==null||s.loadExternalDependency(this.instance,"toolbar",o=>{if(o)return c5.error("[Toolbar] Failed to load",o),void this._t(Ws.UNINITIALIZED);this._t(Ws.LOADED),this.bt(a)}),Xn(be,"turbolinks:load",()=>{this._t(Ws.UNINITIALIZED),this.loadToolbar(a)})}return!0}wt(t){return this.loadToolbar(t)}maybeLoadEditor(t,n,r){return t===void 0&&(t=void 0),n===void 0&&(n=void 0),r===void 0&&(r=void 0),this.maybeLoadToolbar(t,n,r)}}var PG=Zn("[TracingHeaders]");class BG{constructor(t){this.xt=void 0,this.Et=void 0,this.rt=()=>{var n,r;qe(this.xt)&&((n=dt.__PosthogExtensions__)==null||(n=n.tracingHeadersPatchFns)==null||n._patchXHR(this._instance.config.__add_tracing_headers||[],this._instance.get_distinct_id(),this._instance.sessionManager)),qe(this.Et)&&((r=dt.__PosthogExtensions__)==null||(r=r.tracingHeadersPatchFns)==null||r._patchFetch(this._instance.config.__add_tracing_headers||[],this._instance.get_distinct_id(),this._instance.sessionManager))},this._instance=t}it(t){var n,r;(n=dt.__PosthogExtensions__)!=null&&n.tracingHeadersPatchFns&&t(),(r=dt.__PosthogExtensions__)==null||r.loadExternalDependency==null||r.loadExternalDependency(this._instance,"tracing-headers",a=>{if(a)return PG.error("failed to load script",a);t()})}startIfEnabledOrStop(){var t,n;this._instance.config.__add_tracing_headers?this.it(this.rt):((t=this.xt)==null||t.call(this),(n=this.Et)==null||n.call(this),this.xt=void 0,this.Et=void 0)}}var La="Mobile",Cf="iOS",pi="Android",v0="Tablet",qS=pi+" "+v0,jS="iPad",$S="Apple",YS=$S+" Watch",y0="Safari",iu="BlackBerry",GS="Samsung",VS=GS+"Browser",KS=GS+" Internet",Gl="Chrome",UG=Gl+" OS",WS=Gl+" "+Cf,ov="Internet Explorer",XS=ov+" "+La,uv="Opera",FG=uv+" Mini",cv="Edge",QS="Microsoft "+cv,Zo="Firefox",ZS=Zo+" "+Cf,_0="Nintendo",E0="PlayStation",Jo="Xbox",JS=pi+" "+La,e9=La+" "+y0,Vc="Windows",Hb=Vc+" Phone",d5="Nokia",qb="Ouya",t9="Generic",zG=t9+" "+La.toLowerCase(),n9=t9+" "+v0.toLowerCase(),jb="Konqueror",Pr="(\\d+(\\.\\d+)?)",wg=new RegExp("Version/"+Pr),HG=new RegExp(Jo,"i"),qG=new RegExp(E0+" \\w+","i"),jG=new RegExp(_0+" \\w+","i"),dv=new RegExp(iu+"|PlayBook|BB10","i"),$G={"NT3.51":"NT 3.11","NT4.0":"NT 4.0","5.0":"2000",5.1:"XP",5.2:"XP","6.0":"Vista",6.1:"7",6.2:"8",6.3:"8.1",6.4:"10","10.0":"10"},YG=(e,t)=>t&&It(t,$S)||function(n){return It(n,y0)&&!It(n,Gl)&&!It(n,pi)}(e),r9=function(e,t){return t=t||"",It(e," OPR/")&&It(e,"Mini")?FG:It(e," OPR/")?uv:dv.test(e)?iu:It(e,"IE"+La)||It(e,"WPDesktop")?XS:It(e,VS)?KS:It(e,cv)||It(e,"Edg/")?QS:It(e,"FBIOS")?"Facebook "+La:It(e,"UCWEB")||It(e,"UCBrowser")?"UC Browser":It(e,"CriOS")?WS:It(e,"CrMo")||It(e,Gl)?Gl:It(e,pi)&&It(e,y0)?JS:It(e,"FxiOS")?ZS:It(e.toLowerCase(),jb.toLowerCase())?jb:YG(e,t)?It(e,La)?e9:y0:It(e,Zo)?Zo:It(e,"MSIE")||It(e,"Trident/")?ov:It(e,"Gecko")?Zo:""},GG={[XS]:[new RegExp("rv:"+Pr)],[QS]:[new RegExp(cv+"?\\/"+Pr)],[Gl]:[new RegExp("("+Gl+"|CrMo)\\/"+Pr)],[WS]:[new RegExp("CriOS\\/"+Pr)],"UC Browser":[new RegExp("(UCBrowser|UCWEB)\\/"+Pr)],[y0]:[wg],[e9]:[wg],[uv]:[new RegExp("(Opera|OPR)\\/"+Pr)],[Zo]:[new RegExp(Zo+"\\/"+Pr)],[ZS]:[new RegExp("FxiOS\\/"+Pr)],[jb]:[new RegExp("Konqueror[:/]?"+Pr,"i")],[iu]:[new RegExp(iu+" "+Pr),wg],[JS]:[new RegExp("android\\s"+Pr,"i")],[KS]:[new RegExp(VS+"\\/"+Pr)],[ov]:[new RegExp("(rv:|MSIE )"+Pr)],Mozilla:[new RegExp("rv:"+Pr)]},VG=function(e,t){var n=r9(e,t),r=GG[n];if(qe(r))return null;for(var a=0;a<r.length;a++){var s=r[a],o=e.match(s);if(o)return parseFloat(o[o.length-2])}return null},h5=[[new RegExp(Jo+"; "+Jo+" (.*?)[);]","i"),e=>[Jo,e&&e[1]||""]],[new RegExp(_0,"i"),[_0,""]],[new RegExp(E0,"i"),[E0,""]],[dv,[iu,""]],[new RegExp(Vc,"i"),(e,t)=>{if(/Phone/.test(t)||/WPDesktop/.test(t))return[Hb,""];if(new RegExp(La).test(t)&&!/IEMobile\b/.test(t))return[Vc+" "+La,""];var n=/Windows NT ([0-9.]+)/i.exec(t);if(n&&n[1]){var r=n[1],a=$G[r]||"";return/arm/i.test(t)&&(a="RT"),[Vc,a]}return[Vc,""]}],[/((iPhone|iPad|iPod).*?OS (\d+)_(\d+)_?(\d+)?|iPhone)/,e=>{if(e&&e[3]){var t=[e[3],e[4],e[5]||"0"];return[Cf,t.join(".")]}return[Cf,""]}],[/(watch.*\/(\d+\.\d+\.\d+)|watch os,(\d+\.\d+),)/i,e=>{var t="";return e&&e.length>=3&&(t=qe(e[2])?e[3]:e[2]),["watchOS",t]}],[new RegExp("("+pi+" (\\d+)\\.(\\d+)\\.?(\\d+)?|"+pi+")","i"),e=>{if(e&&e[2]){var t=[e[2],e[3],e[4]||"0"];return[pi,t.join(".")]}return[pi,""]}],[/Mac OS X (\d+)[_.](\d+)[_.]?(\d+)?/i,e=>{var t=["Mac OS X",""];if(e&&e[1]){var n=[e[1],e[2],e[3]||"0"];t[1]=n.join(".")}return t}],[/Mac/i,["Mac OS X",""]],[/CrOS/,[UG,""]],[/Linux|debian/i,["Linux",""]]],f5=function(e){return jG.test(e)?_0:qG.test(e)?E0:HG.test(e)?Jo:new RegExp(qb,"i").test(e)?qb:new RegExp("("+Hb+"|WPDesktop)","i").test(e)?Hb:/iPad/.test(e)?jS:/iPod/.test(e)?"iPod Touch":/iPhone/.test(e)?"iPhone":/(watch)(?: ?os[,/]|\d,\d\/)[\d.]+/i.test(e)?YS:dv.test(e)?iu:/(kobo)\s(ereader|touch)/i.test(e)?"Kobo":new RegExp(d5,"i").test(e)?d5:/(kf[a-z]{2}wi|aeo[c-r]{2})( bui|\))/i.test(e)||/(kf[a-z]+)( bui|\)).+silk\//i.test(e)?"Kindle Fire":/(Android|ZTE)/i.test(e)?!new RegExp(La).test(e)||/(9138B|TB782B|Nexus [97]|pixel c|HUAWEISHT|BTV|noble nook|smart ultra 6)/i.test(e)?/pixel[\daxl ]{1,6}/i.test(e)&&!/pixel c/i.test(e)||/(huaweimed-al00|tah-|APA|SM-G92|i980|zte|U304AA)/i.test(e)||/lmy47v/i.test(e)&&!/QTAQZ3/i.test(e)?pi:qS:pi:new RegExp("(pda|"+La+")","i").test(e)?zG:new RegExp(v0,"i").test(e)&&!new RegExp(v0+" pc","i").test(e)?n9:""},Mh="https?://(.*)",Eu=["gclid","gclsrc","dclid","gbraid","wbraid","fbclid","msclkid","twclid","li_fat_id","igshid","ttclid","rdt_cid","epik","qclid","sccid","irclid","_kx"],KG=_u(["utm_source","utm_medium","utm_campaign","utm_content","utm_term","gad_source","mc_cid"],Eu),q0="<masked>",WG=["li_fat_id"];function a9(e,t,n){if(!$e)return{};var r,a=t?_u([],Eu,n||[]):[],s=i9(H0($e.URL,a,q0),e),o=(r={},hn(WG,function(c){var d=gi.q(c);r[c]=d||null}),r);return Fn(o,s)}function i9(e,t){var n=KG.concat(t||[]),r={};return hn(n,function(a){var s=wf(e,a);r[a]=s||null}),r}function s9(e){var t=function(s){return s?s.search(Mh+"google.([^/?]*)")===0?"google":s.search(Mh+"bing.com")===0?"bing":s.search(Mh+"yahoo.com")===0?"yahoo":s.search(Mh+"duckduckgo.com")===0?"duckduckgo":null:null}(e),n=t!="yahoo"?"q":"p",r={};if(!ds(t)){r.$search_engine=t;var a=$e?wf($e.referrer,n):"";a.length&&(r.ph_keyword=a)}return r}function m5(){return navigator.language||navigator.userLanguage}function l9(){return($e==null?void 0:$e.referrer)||"$direct"}function o9(e,t){var n=e?_u([],Eu,t||[]):[],r=ta==null?void 0:ta.href.substring(0,1e3);return{r:l9().substring(0,1e3),u:r?H0(r,n,q0):void 0}}function u9(e){var t,{r:n,u:r}=e,a={$referrer:n,$referring_domain:n==null?void 0:n=="$direct"?"$direct":(t=Sf(n))==null?void 0:t.host};if(r){a.$current_url=r;var s=Sf(r);a.$host=s==null?void 0:s.host,a.$pathname=s==null?void 0:s.pathname;var o=i9(r);Fn(a,o)}if(n){var c=s9(n);Fn(a,c)}return a}function c9(){try{return Intl.DateTimeFormat().resolvedOptions().timeZone}catch{return}}function XG(){try{return new Date().getTimezoneOffset()}catch{return}}function QG(e,t){if(!ea)return{};var n,r,a,s=e?_u([],Eu,t||[]):[],[o,c]=function(d){for(var f=0;f<h5.length;f++){var[m,p]=h5[f],b=m.exec(d),y=b&&(tl(p)?p(b,d):p);if(y)return y}return["",""]}(ea);return Fn(av({$os:o,$os_version:c,$browser:r9(ea,navigator.vendor),$device:f5(ea),$device_type:(r=ea,a=f5(r),a===jS||a===qS||a==="Kobo"||a==="Kindle Fire"||a===n9?v0:a===_0||a===Jo||a===E0||a===qb?"Console":a===YS?"Wearable":a?La:"Desktop"),$timezone:c9(),$timezone_offset:XG()}),{$current_url:H0(ta==null?void 0:ta.href,s,q0),$host:ta==null?void 0:ta.host,$pathname:ta==null?void 0:ta.pathname,$raw_user_agent:ea.length>1e3?ea.substring(0,997)+"...":ea,$browser_version:VG(ea,navigator.vendor),$browser_language:m5(),$browser_language_prefix:(n=m5(),typeof n=="string"?n.split("-")[0]:void 0),$screen_height:be==null?void 0:be.screen.height,$screen_width:be==null?void 0:be.screen.width,$viewport_height:be==null?void 0:be.innerHeight,$viewport_width:be==null?void 0:be.innerWidth,$lib:"web",$lib_version:Yi.LIB_VERSION,$insert_id:Math.random().toString(36).substring(2,10)+Math.random().toString(36).substring(2,10),$time:Date.now()/1e3})}var js=Zn("[Web Vitals]"),p5=9e5;class ZG{constructor(t){var n;this.St=!1,this.P=!1,this.N={url:void 0,metrics:[],firstMetricTimestamp:void 0},this.$t=()=>{clearTimeout(this.kt),this.N.metrics.length!==0&&(this._instance.capture("$web_vitals",this.N.metrics.reduce((r,a)=>Xe({},r,{["$web_vitals_"+a.name+"_event"]:Xe({},a),["$web_vitals_"+a.name+"_value"]:a.value}),{})),this.N={url:void 0,metrics:[],firstMetricTimestamp:void 0})},this.Pt=r=>{var a,s=(a=this._instance.sessionManager)==null?void 0:a.checkAndGetSessionAndWindowId(!0);if(qe(s))js.error("Could not read session ID. Dropping metrics!");else{this.N=this.N||{url:void 0,metrics:[],firstMetricTimestamp:void 0};var o=this.Tt();qe(o)||(Gt(r==null?void 0:r.name)||Gt(r==null?void 0:r.value)?js.error("Invalid metric received",r):this.Rt&&r.value>=this.Rt?js.error("Ignoring metric with value >= "+this.Rt,r):(this.N.url!==o&&(this.$t(),this.kt=setTimeout(this.$t,this.flushToCaptureTimeoutMs)),qe(this.N.url)&&(this.N.url=o),this.N.firstMetricTimestamp=qe(this.N.firstMetricTimestamp)?Date.now():this.N.firstMetricTimestamp,r.attribution&&r.attribution.interactionTargetElement&&(r.attribution.interactionTargetElement=void 0),this.N.metrics.push(Xe({},r,{$current_url:o,$session_id:s.sessionId,$window_id:s.windowId,timestamp:Date.now()})),this.N.metrics.length===this.allowedMetrics.length&&this.$t()))}},this.rt=()=>{var r,a,s,o,c=dt.__PosthogExtensions__;qe(c)||qe(c.postHogWebVitalsCallbacks)||({onLCP:r,onCLS:a,onFCP:s,onINP:o}=c.postHogWebVitalsCallbacks),r&&a&&s&&o?(this.allowedMetrics.indexOf("LCP")>-1&&r(this.Pt.bind(this)),this.allowedMetrics.indexOf("CLS")>-1&&a(this.Pt.bind(this)),this.allowedMetrics.indexOf("FCP")>-1&&s(this.Pt.bind(this)),this.allowedMetrics.indexOf("INP")>-1&&o(this.Pt.bind(this)),this.P=!0):js.error("web vitals callbacks not loaded - not starting")},this._instance=t,this.St=!((n=this._instance.persistence)==null||!n.props[Z6]),this.startIfEnabled()}get allowedMetrics(){var t,n,r=hr(this._instance.config.capture_performance)?(t=this._instance.config.capture_performance)==null?void 0:t.web_vitals_allowed_metrics:void 0;return qe(r)?((n=this._instance.persistence)==null?void 0:n.props[J6])||["CLS","FCP","INP","LCP"]:r}get flushToCaptureTimeoutMs(){return(hr(this._instance.config.capture_performance)?this._instance.config.capture_performance.web_vitals_delayed_flush_ms:void 0)||5e3}get Rt(){var t=hr(this._instance.config.capture_performance)&&bi(this._instance.config.capture_performance.__web_vitals_max_value)?this._instance.config.capture_performance.__web_vitals_max_value:p5;return 0<t&&t<=6e4?p5:t}get isEnabled(){var t=ta==null?void 0:ta.protocol;if(t!=="http:"&&t!=="https:")return js.info("Web Vitals are disabled on non-http/https protocols"),!1;var n=hr(this._instance.config.capture_performance)?this._instance.config.capture_performance.web_vitals:nl(this._instance.config.capture_performance)?this._instance.config.capture_performance:void 0;return nl(n)?n:this.St}startIfEnabled(){this.isEnabled&&!this.P&&(js.info("enabled, starting..."),this.it(this.rt))}onRemoteConfig(t){var n=hr(t.capturePerformance)&&!!t.capturePerformance.web_vitals,r=hr(t.capturePerformance)?t.capturePerformance.web_vitals_allowed_metrics:void 0;this._instance.persistence&&(this._instance.persistence.register({[Z6]:n}),this._instance.persistence.register({[J6]:r})),this.St=n,this.startIfEnabled()}it(t){var n,r;(n=dt.__PosthogExtensions__)!=null&&n.postHogWebVitalsCallbacks&&t(),(r=dt.__PosthogExtensions__)==null||r.loadExternalDependency==null||r.loadExternalDependency(this._instance,"web-vitals",a=>{a?js.error("failed to load script",a):t()})}Tt(){var t=be?be.location.href:void 0;if(t){var n=this._instance.config.mask_personal_data_properties,r=this._instance.config.custom_personal_data_properties,a=n?_u([],Eu,r||[]):[];return H0(t,a,q0)}js.error("Could not determine current URL")}}var JG=Zn("[Heatmaps]");function g5(e){return hr(e)&&"clientX"in e&&"clientY"in e&&bi(e.clientX)&&bi(e.clientY)}class eV{constructor(t){var n;this.St=!1,this.P=!1,this.Ct=null,this.instance=t,this.St=!((n=this.instance.persistence)==null||!n.props[Mb]),this.rageclicks=new BS(t.config.rageclick)}get flushIntervalMilliseconds(){var t=5e3;return hr(this.instance.config.capture_heatmaps)&&this.instance.config.capture_heatmaps.flush_interval_milliseconds&&(t=this.instance.config.capture_heatmaps.flush_interval_milliseconds),t}get isEnabled(){return qe(this.instance.config.capture_heatmaps)?qe(this.instance.config.enable_heatmaps)?this.St:this.instance.config.enable_heatmaps:this.instance.config.capture_heatmaps!==!1}startIfEnabled(){if(this.isEnabled){if(this.P)return;JG.info("starting..."),this.It(),this.Ft()}else{var t;clearInterval((t=this.Ct)!==null&&t!==void 0?t:void 0),this.Mt(),this.getAndClearBuffer()}}onRemoteConfig(t){var n=!!t.heatmaps;this.instance.persistence&&this.instance.persistence.register({[Mb]:n}),this.St=n,this.startIfEnabled()}getAndClearBuffer(){var t=this.N;return this.N=void 0,t}Ot(t){this.At(t.originalEvent,"deadclick")}Ft(){this.Ct&&clearInterval(this.Ct),this.Ct=function(t){return(t==null?void 0:t.visibilityState)==="visible"}($e)?setInterval(this.Dt.bind(this),this.flushIntervalMilliseconds):null}It(){be&&$e&&(this.jt=this.Dt.bind(this),Xn(be,"beforeunload",this.jt),this.Lt=t=>this.At(t||(be==null?void 0:be.event)),Xn($e,"click",this.Lt,{capture:!0}),this.Nt=t=>this.Ut(t||(be==null?void 0:be.event)),Xn($e,"mousemove",this.Nt,{capture:!0}),this.zt=new FS(this.instance,CG,this.Ot.bind(this)),this.zt.startIfEnabled(),this.Ht=this.Ft.bind(this),Xn($e,"visibilitychange",this.Ht),this.P=!0)}Mt(){var t;be&&$e&&(this.jt&&be.removeEventListener("beforeunload",this.jt),this.Lt&&$e.removeEventListener("click",this.Lt,{capture:!0}),this.Nt&&$e.removeEventListener("mousemove",this.Nt,{capture:!0}),this.Ht&&$e.removeEventListener("visibilitychange",this.Ht),clearTimeout(this.Bt),(t=this.zt)==null||t.stop(),this.P=!1)}qt(t,n){var r=this.instance.scrollManager.scrollY(),a=this.instance.scrollManager.scrollX(),s=this.instance.scrollManager.scrollElement(),o=function(c,d,f){for(var m=c;m&&om(m)&&!al(m,"body");){if(m===f)return!1;if(It(d,be==null?void 0:be.getComputedStyle(m).position))return!0;m=OS(m)}return!1}(CS(t),["fixed","sticky"],s);return{x:t.clientX+(o?0:a),y:t.clientY+(o?0:r),target_fixed:o,type:n}}At(t,n){var r;if(n===void 0&&(n="click"),!e5(t.target)&&g5(t)){var a=this.qt(t,n);(r=this.rageclicks)!=null&&r.isRageClick(t.clientX,t.clientY,new Date().getTime())&&this.Wt(Xe({},a,{type:"rageclick"})),this.Wt(a)}}Ut(t){!e5(t.target)&&g5(t)&&(clearTimeout(this.Bt),this.Bt=setTimeout(()=>{this.Wt(this.qt(t,"mousemove"))},500))}Wt(t){if(be){var n=be.location.href,r=this.instance.config.mask_personal_data_properties,a=this.instance.config.custom_personal_data_properties,s=r?_u([],Eu,a||[]):[],o=H0(n,s,q0);this.N=this.N||{},this.N[o]||(this.N[o]=[]),this.N[o].push(t)}}Dt(){this.N&&!Vo(this.N)&&this.instance.capture("$$heatmap",{$heatmap_data:this.getAndClearBuffer()})}}class tV{constructor(t){this._instance=t}doPageView(t,n){var r,a=this.Gt(t,n);return this.Vt={pathname:(r=be==null?void 0:be.location.pathname)!==null&&r!==void 0?r:"",pageViewId:n,timestamp:t},this._instance.scrollManager.resetContext(),a}doPageLeave(t){var n;return this.Gt(t,(n=this.Vt)==null?void 0:n.pageViewId)}doEvent(){var t;return{$pageview_id:(t=this.Vt)==null?void 0:t.pageViewId}}Gt(t,n){var r=this.Vt;if(!r)return{$pageview_id:n};var a={$pageview_id:n,$prev_pageview_id:r.pageViewId},s=this._instance.scrollManager.getContext();if(s&&!this._instance.config.disable_scroll_properties){var{maxScrollHeight:o,lastScrollY:c,maxScrollY:d,maxContentHeight:f,lastContentY:m,maxContentY:p}=s;if(!(qe(o)||qe(c)||qe(d)||qe(f)||qe(m)||qe(p))){o=Math.ceil(o),c=Math.ceil(c),d=Math.ceil(d),f=Math.ceil(f),m=Math.ceil(m),p=Math.ceil(p);var b=o<=1?1:mi(c/o,0,1,Ge),y=o<=1?1:mi(d/o,0,1,Ge),T=f<=1?1:mi(m/f,0,1,Ge),x=f<=1?1:mi(p/f,0,1,Ge);a=Fn(a,{$prev_pageview_last_scroll:c,$prev_pageview_last_scroll_percentage:b,$prev_pageview_max_scroll:d,$prev_pageview_max_scroll_percentage:y,$prev_pageview_last_content:m,$prev_pageview_last_content_percentage:T,$prev_pageview_max_content:p,$prev_pageview_max_content_percentage:x})}}return r.pathname&&(a.$prev_pageview_pathname=r.pathname),r.timestamp&&(a.$prev_pageview_duration=(t.getTime()-r.timestamp.getTime())/1e3),a}}var Pa=Uint8Array,na=Uint16Array,su=Uint32Array,hv=new Pa([0,0,0,0,0,0,0,0,1,1,1,1,2,2,2,2,3,3,3,3,4,4,4,4,5,5,5,5,0,0,0,0]),fv=new Pa([0,0,0,0,1,1,2,2,3,3,4,4,5,5,6,6,7,7,8,8,9,9,10,10,11,11,12,12,13,13,0,0]),b5=new Pa([16,17,18,0,8,7,9,6,10,5,11,4,12,3,13,2,14,1,15]),d9=function(e,t){for(var n=new na(31),r=0;r<31;++r)n[r]=t+=1<<e[r-1];var a=new su(n[30]);for(r=1;r<30;++r)for(var s=n[r];s<n[r+1];++s)a[s]=s-n[r]<<5|r;return[n,a]},h9=d9(hv,2),nV=h9[0],$b=h9[1];nV[28]=258,$b[258]=28;for(var v5=d9(fv,0)[1],f9=new na(32768),bn=0;bn<32768;++bn){var Fl=(43690&bn)>>>1|(21845&bn)<<1;Fl=(61680&(Fl=(52428&Fl)>>>2|(13107&Fl)<<2))>>>4|(3855&Fl)<<4,f9[bn]=((65280&Fl)>>>8|(255&Fl)<<8)>>>1}var l0=function(e,t,n){for(var r=e.length,a=0,s=new na(t);a<r;++a)++s[e[a]-1];var o,c=new na(t);for(a=0;a<t;++a)c[a]=c[a-1]+s[a-1]<<1;for(o=new na(r),a=0;a<r;++a)o[a]=f9[c[e[a]-1]++]>>>15-e[a];return o},Vl=new Pa(288);for(bn=0;bn<144;++bn)Vl[bn]=8;for(bn=144;bn<256;++bn)Vl[bn]=9;for(bn=256;bn<280;++bn)Vl[bn]=7;for(bn=280;bn<288;++bn)Vl[bn]=8;var Nf=new Pa(32);for(bn=0;bn<32;++bn)Nf[bn]=5;var rV=l0(Vl,9),aV=l0(Nf,5),m9=function(e){return(e/8>>0)+(7&e&&1)},p9=function(e,t,n){(n==null||n>e.length)&&(n=e.length);var r=new(e instanceof na?na:e instanceof su?su:Pa)(n-t);return r.set(e.subarray(t,n)),r},$i=function(e,t,n){n<<=7&t;var r=t/8>>0;e[r]|=n,e[r+1]|=n>>>8},Dc=function(e,t,n){n<<=7&t;var r=t/8>>0;e[r]|=n,e[r+1]|=n>>>8,e[r+2]|=n>>>16},Ag=function(e,t){for(var n=[],r=0;r<e.length;++r)e[r]&&n.push({s:r,f:e[r]});var a=n.length,s=n.slice();if(!a)return[new Pa(0),0];if(a==1){var o=new Pa(n[0].s+1);return o[n[0].s]=1,[o,1]}n.sort(function(H,B){return H.f-B.f}),n.push({s:-1,f:25001});var c=n[0],d=n[1],f=0,m=1,p=2;for(n[0]={s:-1,f:c.f+d.f,l:c,r:d};m!=a-1;)c=n[n[f].f<n[p].f?f++:p++],d=n[f!=m&&n[f].f<n[p].f?f++:p++],n[m++]={s:-1,f:c.f+d.f,l:c,r:d};var b=s[0].s;for(r=1;r<a;++r)s[r].s>b&&(b=s[r].s);var y=new na(b+1),T=Yb(n[m-1],y,0);if(T>t){r=0;var x=0,C=T-t,w=1<<C;for(s.sort(function(H,B){return y[B.s]-y[H.s]||H.f-B.f});r<a;++r){var O=s[r].s;if(!(y[O]>t))break;x+=w-(1<<T-y[O]),y[O]=t}for(x>>>=C;x>0;){var D=s[r].s;y[D]<t?x-=1<<t-y[D]++-1:++r}for(;r>=0&&x;--r){var Y=s[r].s;y[Y]==t&&(--y[Y],++x)}T=t}return[new Pa(y),T]},Yb=function(e,t,n){return e.s==-1?Math.max(Yb(e.l,t,n+1),Yb(e.r,t,n+1)):t[e.s]=n},y5=function(e){for(var t=e.length;t&&!e[--t];);for(var n=new na(++t),r=0,a=e[0],s=1,o=function(d){n[r++]=d},c=1;c<=t;++c)if(e[c]==a&&c!=t)++s;else{if(!a&&s>2){for(;s>138;s-=138)o(32754);s>2&&(o(s>10?s-11<<5|28690:s-3<<5|12305),s=0)}else if(s>3){for(o(a),--s;s>6;s-=6)o(8304);s>2&&(o(s-3<<5|8208),s=0)}for(;s--;)o(a);s=1,a=e[c]}return[n.subarray(0,r),t]},Lc=function(e,t){for(var n=0,r=0;r<t.length;++r)n+=e[r]*t[r];return n},Gb=function(e,t,n){var r=n.length,a=m9(t+2);e[a]=255&r,e[a+1]=r>>>8,e[a+2]=255^e[a],e[a+3]=255^e[a+1];for(var s=0;s<r;++s)e[a+s+4]=n[s];return 8*(a+4+r)},_5=function(e,t,n,r,a,s,o,c,d,f,m){$i(t,m++,n),++a[256];for(var p=Ag(a,15),b=p[0],y=p[1],T=Ag(s,15),x=T[0],C=T[1],w=y5(b),O=w[0],D=w[1],Y=y5(x),H=Y[0],B=Y[1],ee=new na(19),Z=0;Z<O.length;++Z)ee[31&O[Z]]++;for(Z=0;Z<H.length;++Z)ee[31&H[Z]]++;for(var Q=Ag(ee,7),G=Q[0],se=Q[1],oe=19;oe>4&&!G[b5[oe-1]];--oe);var ye,fe,de,V,me=f+5<<3,pe=Lc(a,Vl)+Lc(s,Nf)+o,Be=Lc(a,b)+Lc(s,x)+o+14+3*oe+Lc(ee,G)+(2*ee[16]+3*ee[17]+7*ee[18]);if(me<=pe&&me<=Be)return Gb(t,m,e.subarray(d,d+f));if($i(t,m,1+(Be<pe)),m+=2,Be<pe){ye=l0(b,y),fe=b,de=l0(x,C),V=x;var k=l0(G,se);for($i(t,m,D-257),$i(t,m+5,B-1),$i(t,m+10,oe-4),m+=14,Z=0;Z<oe;++Z)$i(t,m+3*Z,G[b5[Z]]);m+=3*oe;for(var ie=[O,H],ge=0;ge<2;++ge){var L=ie[ge];for(Z=0;Z<L.length;++Z){var Oe=31&L[Z];$i(t,m,k[Oe]),m+=G[Oe],Oe>15&&($i(t,m,L[Z]>>>5&127),m+=L[Z]>>>12)}}}else ye=rV,fe=Vl,de=aV,V=Nf;for(Z=0;Z<c;++Z)if(r[Z]>255){Oe=r[Z]>>>18&31,Dc(t,m,ye[Oe+257]),m+=fe[Oe+257],Oe>7&&($i(t,m,r[Z]>>>23&31),m+=hv[Oe]);var je=31&r[Z];Dc(t,m,de[je]),m+=V[je],je>3&&(Dc(t,m,r[Z]>>>5&8191),m+=fv[je])}else Dc(t,m,ye[r[Z]]),m+=fe[r[Z]];return Dc(t,m,ye[256]),m+fe[256]},iV=new su([65540,131080,131088,131104,262176,1048704,1048832,2114560,2117632]),sV=function(){for(var e=new su(256),t=0;t<256;++t){for(var n=t,r=9;--r;)n=(1&n&&3988292384)^n>>>1;e[t]=n}return e}(),lV=function(e,t,n,r,a){return function(s,o,c,d,f,m){var p=s.length,b=new Pa(d+p+5*(1+Math.floor(p/7e3))+f),y=b.subarray(d,b.length-f),T=0;if(!o||p<8)for(var x=0;x<=p;x+=65535){var C=x+65535;C<p?T=Gb(y,T,s.subarray(x,C)):(y[x]=m,T=Gb(y,T,s.subarray(x,p)))}else{for(var w=iV[o-1],O=w>>>13,D=8191&w,Y=(1<<c)-1,H=new na(32768),B=new na(Y+1),ee=Math.ceil(c/3),Z=2*ee,Q=function(yt){return(s[yt]^s[yt+1]<<ee^s[yt+2]<<Z)&Y},G=new su(25e3),se=new na(288),oe=new na(32),ye=0,fe=0,de=(x=0,0),V=0,me=0;x<p;++x){var pe=Q(x),Be=32767&x,k=B[pe];if(H[Be]=k,B[pe]=Be,V<=x){var ie=p-x;if((ye>7e3||de>24576)&&ie>423){T=_5(s,y,0,G,se,oe,fe,de,me,x-me,T),de=ye=fe=0,me=x;for(var ge=0;ge<286;++ge)se[ge]=0;for(ge=0;ge<30;++ge)oe[ge]=0}var L=2,Oe=0,je=D,De=Be-k&32767;if(ie>2&&pe==Q(x-De))for(var Ot=Math.min(O,ie)-1,ot=Math.min(32767,x),zt=Math.min(258,ie);De<=ot&&--je&&Be!=k;){if(s[x+L]==s[x+L-De]){for(var wt=0;wt<zt&&s[x+wt]==s[x+wt-De];++wt);if(wt>L){if(L=wt,Oe=De,wt>Ot)break;var vn=Math.min(De,wt-2),mr=0;for(ge=0;ge<vn;++ge){var vt=x-De+ge+32768&32767,gt=vt-H[vt]+32768&32767;gt>mr&&(mr=gt,k=vt)}}}De+=(Be=k)-(k=H[Be])+32768&32767}if(Oe){G[de++]=268435456|$b[L]<<18|v5[Oe];var Bt=31&$b[L],Je=31&v5[Oe];fe+=hv[Bt]+fv[Je],++se[257+Bt],++oe[Je],V=x+L,++ye}else G[de++]=s[x],++se[s[x]]}}T=_5(s,y,m,G,se,oe,fe,de,me,x-me,T)}return p9(b,0,d+m9(T)+f)}(e,t.level==null?6:t.level,t.mem==null?Math.ceil(1.5*Math.max(8,Math.min(13,Math.log(e.length)))):12+t.mem,n,r,!0)},kg=function(e,t,n){for(;n;++t)e[t]=n,n>>>=8};function oV(e,t){t===void 0&&(t={});var n=function(){var c=4294967295;return{p:function(d){for(var f=c,m=0;m<d.length;++m)f=sV[255&f^d[m]]^f>>>8;c=f},d:function(){return 4294967295^c}}}(),r=e.length;n.p(e);var a,s=lV(e,t,10+((a=t).filename&&a.filename.length+1||0),8),o=s.length;return function(c,d){var f=d.filename;if(c[0]=31,c[1]=139,c[2]=8,c[8]=d.level<2?4:d.level==9?2:0,c[9]=3,d.mtime!=0&&kg(c,4,Math.floor(new Date(d.mtime||Date.now())/1e3)),f){c[3]=8;for(var m=0;m<=f.length;++m)c[m+10]=f.charCodeAt(m)}}(s,t),kg(s,o-8,n.d()),kg(s,o-4,r),s}var uV=function(e){var t,n,r,a,s="";for(t=n=0,r=(e=(e+"").replace(/\r\n/g,`
`).replace(/\r/g,`
`)).length,a=0;a<r;a++){var o=e.charCodeAt(a),c=null;o<128?n++:c=o>127&&o<2048?String.fromCharCode(o>>6|192,63&o|128):String.fromCharCode(o>>12|224,o>>6&63|128,63&o|128),ds(c)||(n>t&&(s+=e.substring(t,n)),s+=c,t=n=a+1)}return n>t&&(s+=e.substring(t,e.length)),s},cV=!!kb||!!Ab,E5="text/plain",Of=function(e,t,n){var r;n===void 0&&(n=!0);var[a,s]=e.split("?"),o=Xe({},t),c=(r=s==null?void 0:s.split("&").map(f=>{var m,[p,b]=f.split("="),y=n&&(m=o[p])!==null&&m!==void 0?m:b;return delete o[p],p+"="+y}))!==null&&r!==void 0?r:[],d=pG(o);return d&&c.push(d),a+"?"+c.join("&")},Kc=(e,t)=>JSON.stringify(e,(n,r)=>typeof r=="bigint"?r.toString():r,t),Cg=e=>{var{data:t,compression:n}=e;if(t){if(n===Qi.GZipJS){var r=oV(function(d,f){var m=d.length;if(typeof TextEncoder<"u")return new TextEncoder().encode(d);for(var p=new Pa(d.length+(d.length>>>1)),b=0,y=function(w){p[b++]=w},T=0;T<m;++T){if(b+5>p.length){var x=new Pa(b+8+(m-T<<1));x.set(p),p=x}var C=d.charCodeAt(T);C<128||f?y(C):C<2048?(y(192|C>>>6),y(128|63&C)):C>55295&&C<57344?(y(240|(C=65536+(1047552&C)|1023&d.charCodeAt(++T))>>>18),y(128|C>>>12&63),y(128|C>>>6&63),y(128|63&C)):(y(224|C>>>12),y(128|C>>>6&63),y(128|63&C))}return p9(p,0,b)}(Kc(t)),{mtime:0}),a=new Blob([r],{type:E5});return{contentType:E5,body:a,estimatedSize:a.size}}if(n===Qi.Base64){var s=function(d){var f,m,p,b,y,T="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=",x=0,C=0,w="",O=[];if(!d)return d;d=uV(d);do f=(y=d.charCodeAt(x++)<<16|d.charCodeAt(x++)<<8|d.charCodeAt(x++))>>18&63,m=y>>12&63,p=y>>6&63,b=63&y,O[C++]=T.charAt(f)+T.charAt(m)+T.charAt(p)+T.charAt(b);while(x<d.length);switch(w=O.join(""),d.length%3){case 1:w=w.slice(0,-2)+"==";break;case 2:w=w.slice(0,-1)+"="}return w}(Kc(t)),o=(d=>"data="+encodeURIComponent(typeof d=="string"?d:Kc(d)))(s);return{contentType:"application/x-www-form-urlencoded",body:o,estimatedSize:new Blob([o]).size}}var c=Kc(t);return{contentType:"application/json",body:c,estimatedSize:new Blob([c]).size}}},Vh=[];Ab&&Vh.push({transport:"fetch",method:e=>{var t,n,{contentType:r,body:a,estimatedSize:s}=(t=Cg(e))!==null&&t!==void 0?t:{},o=new Headers;hn(e.headers,function(m,p){o.append(p,m)}),r&&o.append("Content-Type",r);var c=e.url,d=null;if(z6){var f=new z6;d={signal:f.signal,timeout:setTimeout(()=>f.abort(),e.timeout)}}Ab(c,Xe({method:(e==null?void 0:e.method)||"GET",headers:o,keepalive:e.method==="POST"&&(s||0)<52428.8,body:a,signal:(n=d)==null?void 0:n.signal},e.fetchOptions)).then(m=>m.text().then(p=>{var b={statusCode:m.status,text:p};if(m.status===200)try{b.json=JSON.parse(p)}catch(y){Ge.error(y)}e.callback==null||e.callback(b)})).catch(m=>{Ge.error(m),e.callback==null||e.callback({statusCode:0,text:m})}).finally(()=>d?clearTimeout(d.timeout):null)}}),kb&&Vh.push({transport:"XHR",method:e=>{var t,n=new kb;n.open(e.method||"GET",e.url,!0);var{contentType:r,body:a}=(t=Cg(e))!==null&&t!==void 0?t:{};hn(e.headers,function(s,o){n.setRequestHeader(o,s)}),r&&n.setRequestHeader("Content-Type",r),e.timeout&&(n.timeout=e.timeout),e.disableXHRCredentials||(n.withCredentials=!0),n.onreadystatechange=()=>{if(n.readyState===4){var s={statusCode:n.status,text:n.responseText};if(n.status===200)try{s.json=JSON.parse(n.responseText)}catch{}e.callback==null||e.callback(s)}},n.send(a)}}),ya!=null&&ya.sendBeacon&&Vh.push({transport:"sendBeacon",method:e=>{var t=Of(e.url,{beacon:"1"});try{var n,{contentType:r,body:a}=(n=Cg(e))!==null&&n!==void 0?n:{},s=typeof a=="string"?new Blob([a],{type:r}):a;ya.sendBeacon(t,s)}catch{}}});var Rf=function(e,t){if(!function(n){try{new RegExp(n)}catch{return!1}return!0}(t))return!1;try{return new RegExp(t).test(e)}catch{return!1}};function T5(e,t,n){return Kc({distinct_id:e,userPropertiesToSet:t,userPropertiesToSetOnce:n})}var g9={exact:(e,t)=>t.some(n=>e.some(r=>n===r)),is_not:(e,t)=>t.every(n=>e.every(r=>n!==r)),regex:(e,t)=>t.some(n=>e.some(r=>Rf(n,r))),not_regex:(e,t)=>t.every(n=>e.every(r=>!Rf(n,r))),icontains:(e,t)=>t.map(Ih).some(n=>e.map(Ih).some(r=>n.includes(r))),not_icontains:(e,t)=>t.map(Ih).every(n=>e.map(Ih).every(r=>!n.includes(r)))},Ih=e=>e.toLowerCase();function b9(e,t){return!e||Object.entries(e).every(n=>{var[r,a]=n,s=t==null?void 0:t[r];if(qe(s)||ds(s))return!1;var o=[String(s)],c=g9[a.operator];return!!c&&c(a.values,o)})}var Ng=Zn("[Error tracking]");class dV{constructor(t){var n,r;this.Jt=[],this.Kt=new LY([new $Y,new JY,new GY,new YY,new QY,new XY,new KY,new ZY],jY("web:javascript",FY,qY)),this._instance=t,this.Jt=(n=(r=this._instance.persistence)==null?void 0:r.get_property(Ib))!==null&&n!==void 0?n:[]}onRemoteConfig(t){var n,r,a,s=(n=(r=t.errorTracking)==null?void 0:r.suppressionRules)!==null&&n!==void 0?n:[],o=(a=t.errorTracking)==null?void 0:a.captureExtensionExceptions;this.Jt=s,this._instance.persistence&&this._instance.persistence.register({[Ib]:this.Jt,[Q6]:o})}get Yt(){var t,n=!!this._instance.get_property(Q6),r=this._instance.config.error_tracking.captureExtensionExceptions;return(t=r??n)!==null&&t!==void 0&&t}buildProperties(t,n){return this.Kt.buildFromUnknown(t,{syntheticException:n==null?void 0:n.syntheticException,mechanism:{handled:n==null?void 0:n.handled}})}sendExceptionEvent(t){var n=t.$exception_list;if(this.Xt(n)){if(this.Qt(n))return void Ng.info("Skipping exception capture because a suppression rule matched");if(!this.Yt&&this.Zt(n))return void Ng.info("Skipping exception capture because it was thrown by an extension");if(!this._instance.config.error_tracking.__capturePostHogExceptions&&this.ti(n))return void Ng.info("Skipping exception capture because it was thrown by the PostHog SDK")}return this._instance.capture("$exception",t,{_noTruncate:!0,_batchKey:"exceptionEvent"})}Qt(t){if(t.length===0)return!1;var n=t.reduce((r,a)=>{var{type:s,value:o}=a;return ir(s)&&s.length>0&&r.$exception_types.push(s),ir(o)&&o.length>0&&r.$exception_values.push(o),r},{$exception_types:[],$exception_values:[]});return this.Jt.some(r=>{var a=r.values.map(s=>{var o,c=g9[s.operator],d=Vt(s.value)?s.value:[s.value],f=(o=n[s.key])!==null&&o!==void 0?o:[];return d.length>0&&c(d,f)});return r.type==="OR"?a.some(Boolean):a.every(Boolean)})}Zt(t){return t.flatMap(n=>{var r,a;return(r=(a=n.stacktrace)==null?void 0:a.frames)!==null&&r!==void 0?r:[]}).some(n=>n.filename&&n.filename.startsWith("chrome-extension://"))}ti(t){if(t.length>0){var n,r,a,s,o=(n=(r=t[0].stacktrace)==null?void 0:r.frames)!==null&&n!==void 0?n:[],c=o[o.length-1];return(a=c==null||(s=c.filename)==null?void 0:s.includes("posthog.com/static"))!==null&&a!==void 0&&a}return!1}Xt(t){return!Gt(t)&&Vt(t)}}var Va=Zn("[FeatureFlags]"),Pc=Zn("[FeatureFlags]",{debugEnabled:!0}),Og="$active_feature_flags",Uo="$override_feature_flags",x5="$feature_flag_payloads",Bc="$override_feature_flag_payloads",S5="$feature_flag_request_id",w5="$feature_flag_evaluated_at",A5=e=>{var t={};for(var[n,r]of Gh(e||{}))r&&(t[n]=r);return t},hV=e=>{var t=e.flags;return t?(e.featureFlags=Object.fromEntries(Object.keys(t).map(n=>{var r;return[n,(r=t[n].variant)!==null&&r!==void 0?r:t[n].enabled]})),e.featureFlagPayloads=Object.fromEntries(Object.keys(t).filter(n=>t[n].enabled).filter(n=>{var r;return(r=t[n].metadata)==null?void 0:r.payload}).map(n=>{var r;return[n,(r=t[n].metadata)==null?void 0:r.payload]}))):Va.warn("Using an older version of the feature flags endpoint. Please upgrade your PostHog server to the latest version"),e},fV=function(e){return e.FeatureFlags="feature_flags",e.Recordings="recordings",e}({});class mV{constructor(t){this.ii=!1,this.ei=!1,this.ri=!1,this.si=!1,this.ni=!1,this.oi=!1,this.ai=!1,this._instance=t,this.featureFlagEventHandlers=[]}li(){var t=this._instance.config.evaluation_environments;return t!=null&&t.length?t.filter(n=>{var r=n&&typeof n=="string"&&n.trim().length>0;return r||Va.error("Invalid evaluation environment found:",n,"Expected non-empty string"),r}):[]}ui(){return this.li().length>0}flags(){if(this._instance.config.__preview_remote_config)this.oi=!0;else{var t=!this.hi&&(this._instance.config.advanced_disable_feature_flags||this._instance.config.advanced_disable_feature_flags_on_first_load);this.di({disableFlags:t})}}get hasLoadedFlags(){return this.ei}getFlags(){return Object.keys(this.getFlagVariants())}getFlagsWithDetails(){var t=this._instance.get_property(Db),n=this._instance.get_property(Uo),r=this._instance.get_property(Bc);if(!r&&!n)return t||{};var a=Fn({},t||{}),s=[...new Set([...Object.keys(r||{}),...Object.keys(n||{})])];for(var o of s){var c,d,f=a[o],m=n==null?void 0:n[o],p=qe(m)?(c=f==null?void 0:f.enabled)!==null&&c!==void 0&&c:!!m,b=qe(m)?f.variant:typeof m=="string"?m:void 0,y=r==null?void 0:r[o],T=Xe({},f,{enabled:p,variant:p?b??(f==null?void 0:f.variant):void 0});p!==(f==null?void 0:f.enabled)&&(T.original_enabled=f==null?void 0:f.enabled),b!==(f==null?void 0:f.variant)&&(T.original_variant=f==null?void 0:f.variant),y&&(T.metadata=Xe({},f==null?void 0:f.metadata,{payload:y,original_payload:f==null||(d=f.metadata)==null?void 0:d.payload})),a[o]=T}return this.ii||(Va.warn(" Overriding feature flag details!",{flagDetails:t,overriddenPayloads:r,finalDetails:a}),this.ii=!0),a}getFlagVariants(){var t=this._instance.get_property(Ko),n=this._instance.get_property(Uo);if(!n)return t||{};for(var r=Fn({},t),a=Object.keys(n),s=0;s<a.length;s++)r[a[s]]=n[a[s]];return this.ii||(Va.warn(" Overriding feature flags!",{enabledFlags:t,overriddenFlags:n,finalFlags:r}),this.ii=!0),r}getFlagPayloads(){var t=this._instance.get_property(x5),n=this._instance.get_property(Bc);if(!n)return t||{};for(var r=Fn({},t||{}),a=Object.keys(n),s=0;s<a.length;s++)r[a[s]]=n[a[s]];return this.ii||(Va.warn(" Overriding feature flag payloads!",{flagPayloads:t,overriddenPayloads:n,finalPayloads:r}),this.ii=!0),r}reloadFeatureFlags(){this.si||this._instance.config.advanced_disable_feature_flags||this.hi||(this.hi=setTimeout(()=>{this.di()},5))}vi(){clearTimeout(this.hi),this.hi=void 0}ensureFlagsLoaded(){this.ei||this.ri||this.hi||this.reloadFeatureFlags()}setAnonymousDistinctId(t){this.$anon_distinct_id=t}setReloadingPaused(t){this.si=t}di(t){var n;if(this.vi(),!this._instance.O())if(this.ri)this.ni=!0;else{var r={token:this._instance.config.token,distinct_id:this._instance.get_distinct_id(),groups:this._instance.getGroups(),$anon_distinct_id:this.$anon_distinct_id,person_properties:Xe({},((n=this._instance.persistence)==null?void 0:n.get_initial_props())||{},this._instance.get_property(Gc)||{}),group_properties:this._instance.get_property(Hl)};(t!=null&&t.disableFlags||this._instance.config.advanced_disable_feature_flags)&&(r.disable_flags=!0),this.ui()&&(r.evaluation_environments=this.li());var a=this._instance.config.__preview_remote_config,s=a?"/flags/?v=2":"/flags/?v=2&config=true",o=this._instance.config.advanced_only_evaluate_survey_feature_flags?"&only_evaluate_survey_feature_flags=true":"",c=this._instance.requestRouter.endpointFor("flags",s+o);a&&(r.timezone=c9()),this.ri=!0,this._instance.ci({method:"POST",url:c,data:r,compression:this._instance.config.disable_compression?void 0:Qi.Base64,timeout:this._instance.config.feature_flag_request_timeout_ms,callback:d=>{var f,m,p=!0;if(d.statusCode===200&&(this.ni||(this.$anon_distinct_id=void 0),p=!1),this.ri=!1,this.oi||(this.oi=!0,this._instance.fi((m=d.json)!==null&&m!==void 0?m:{})),!r.disable_flags||this.ni)if(this.ai=!p,d.json&&(f=d.json.quotaLimited)!=null&&f.includes(fV.FeatureFlags))Va.warn("You have hit your feature flags quota limit, and will not be able to load feature flags until the quota is reset.  Please visit https://posthog.com/docs/billing/limits-alerts to learn more.");else{var b;r.disable_flags||this.receivedFeatureFlags((b=d.json)!==null&&b!==void 0?b:{},p),this.ni&&(this.ni=!1,this.di())}}})}}getFeatureFlag(t,n){if(n===void 0&&(n={}),this.ei||this.getFlags()&&this.getFlags().length>0){var r=this.getFlagVariants()[t],a=""+r,s=this._instance.get_property(S5)||void 0,o=this._instance.get_property(w5)||void 0,c=this._instance.get_property(_f)||{};if((n.send_event||!("send_event"in n))&&(!(t in c)||!c[t].includes(a))){var d,f,m,p,b,y,T,x,C;Vt(c[t])?c[t].push(a):c[t]=[a],(d=this._instance.persistence)==null||d.register({[_f]:c});var w=this.getFeatureFlagDetails(t),O={$feature_flag:t,$feature_flag_response:r,$feature_flag_payload:this.getFeatureFlagPayload(t)||null,$feature_flag_request_id:s,$feature_flag_evaluated_at:o,$feature_flag_bootstrapped_response:((f=this._instance.config.bootstrap)==null||(f=f.featureFlags)==null?void 0:f[t])||null,$feature_flag_bootstrapped_payload:((m=this._instance.config.bootstrap)==null||(m=m.featureFlagPayloads)==null?void 0:m[t])||null,$used_bootstrap_value:!this.ai};qe(w==null||(p=w.metadata)==null?void 0:p.version)||(O.$feature_flag_version=w.metadata.version);var D,Y=(b=w==null||(y=w.reason)==null?void 0:y.description)!==null&&b!==void 0?b:w==null||(T=w.reason)==null?void 0:T.code;Y&&(O.$feature_flag_reason=Y),w!=null&&(x=w.metadata)!=null&&x.id&&(O.$feature_flag_id=w.metadata.id),qe(w==null?void 0:w.original_variant)&&qe(w==null?void 0:w.original_enabled)||(O.$feature_flag_original_response=qe(w.original_variant)?w.original_enabled:w.original_variant),w!=null&&(C=w.metadata)!=null&&C.original_payload&&(O.$feature_flag_original_payload=w==null||(D=w.metadata)==null?void 0:D.original_payload),this._instance.capture("$feature_flag_called",O)}return r}Va.warn('getFeatureFlag for key "'+t+`" failed. Feature flags didn't load in time.`)}getFeatureFlagDetails(t){return this.getFlagsWithDetails()[t]}getFeatureFlagPayload(t){return this.getFlagPayloads()[t]}getRemoteConfigPayload(t,n){var r=this._instance.config.token,a={distinct_id:this._instance.get_distinct_id(),token:r};this.ui()&&(a.evaluation_environments=this.li()),this._instance.ci({method:"POST",url:this._instance.requestRouter.endpointFor("flags","/flags/?v=2&config=true"),data:a,compression:this._instance.config.disable_compression?void 0:Qi.Base64,timeout:this._instance.config.feature_flag_request_timeout_ms,callback:s=>{var o,c=(o=s.json)==null?void 0:o.featureFlagPayloads;n((c==null?void 0:c[t])||void 0)}})}isFeatureEnabled(t,n){if(n===void 0&&(n={}),this.ei||this.getFlags()&&this.getFlags().length>0){var r=this.getFeatureFlag(t,n);return qe(r)?void 0:!!r}Va.warn('isFeatureEnabled for key "'+t+`" failed. Feature flags didn't load in time.`)}addFeatureFlagsHandler(t){this.featureFlagEventHandlers.push(t)}removeFeatureFlagsHandler(t){this.featureFlagEventHandlers=this.featureFlagEventHandlers.filter(n=>n!==t)}receivedFeatureFlags(t,n){if(this._instance.persistence){this.ei=!0;var r=this.getFlagVariants(),a=this.getFlagPayloads(),s=this.getFlagsWithDetails();(function(o,c,d,f,m){d===void 0&&(d={}),f===void 0&&(f={}),m===void 0&&(m={});var p=hV(o),b=p.flags,y=p.featureFlags,T=p.featureFlagPayloads;if(y){var x=o.requestId,C=o.evaluatedAt;if(Vt(y)){Va.warn("v1 of the feature flags endpoint is deprecated. Please use the latest version.");var w={};if(y)for(var O=0;O<y.length;O++)w[y[O]]=!0;c&&c.register({[Og]:y,[Ko]:w})}else{var D=y,Y=T,H=b;o.errorsWhileComputingFlags&&(D=Xe({},d,D),Y=Xe({},f,Y),H=Xe({},m,H)),c&&c.register(Xe({[Og]:Object.keys(A5(D)),[Ko]:D||{},[x5]:Y||{},[Db]:H||{}},x?{[S5]:x}:{},C?{[w5]:C}:{}))}}})(t,this._instance.persistence,r,a,s),this.pi(n)}}override(t,n){n===void 0&&(n=!1),Va.warn("override is deprecated. Please use overrideFeatureFlags instead."),this.overrideFeatureFlags({flags:t,suppressWarning:n})}overrideFeatureFlags(t){if(!this._instance.__loaded||!this._instance.persistence)return Va.uninitializedWarning("posthog.featureFlags.overrideFeatureFlags");if(t===!1)return this._instance.persistence.unregister(Uo),this._instance.persistence.unregister(Bc),this.pi(),Pc.info("All overrides cleared");if(t&&typeof t=="object"&&("flags"in t||"payloads"in t)){var n,r=t;if(this.ii=!!((n=r.suppressWarning)!==null&&n!==void 0&&n),"flags"in r){if(r.flags===!1)this._instance.persistence.unregister(Uo),Pc.info("Flag overrides cleared");else if(r.flags){if(Vt(r.flags)){for(var a={},s=0;s<r.flags.length;s++)a[r.flags[s]]=!0;this._instance.persistence.register({[Uo]:a})}else this._instance.persistence.register({[Uo]:r.flags});Pc.info("Flag overrides set",{flags:r.flags})}}return"payloads"in r&&(r.payloads===!1?(this._instance.persistence.unregister(Bc),Pc.info("Payload overrides cleared")):r.payloads&&(this._instance.persistence.register({[Bc]:r.payloads}),Pc.info("Payload overrides set",{payloads:r.payloads}))),void this.pi()}this.pi()}onFeatureFlags(t){if(this.addFeatureFlagsHandler(t),this.ei){var{flags:n,flagVariants:r}=this.gi();t(n,r)}return()=>this.removeFeatureFlagsHandler(t)}updateEarlyAccessFeatureEnrollment(t,n,r){var a,s=(this._instance.get_property(Yc)||[]).find(f=>f.flagKey===t),o={["$feature_enrollment/"+t]:n},c={$feature_flag:t,$feature_enrollment:n,$set:o};s&&(c.$early_access_feature_name=s.name),r&&(c.$feature_enrollment_stage=r),this._instance.capture("$feature_enrollment_update",c),this.setPersonPropertiesForFlags(o,!1);var d=Xe({},this.getFlagVariants(),{[t]:n});(a=this._instance.persistence)==null||a.register({[Og]:Object.keys(A5(d)),[Ko]:d}),this.pi()}getEarlyAccessFeatures(t,n,r){n===void 0&&(n=!1);var a=this._instance.get_property(Yc),s=r?"&"+r.map(o=>"stage="+o).join("&"):"";if(a&&!n)return t(a);this._instance.ci({url:this._instance.requestRouter.endpointFor("api","/api/early_access_features/?token="+this._instance.config.token+s),method:"GET",callback:o=>{var c,d;if(o.json){var f=o.json.earlyAccessFeatures;return(c=this._instance.persistence)==null||c.unregister(Yc),(d=this._instance.persistence)==null||d.register({[Yc]:f}),t(f)}}})}gi(){var t=this.getFlags(),n=this.getFlagVariants();return{flags:t.filter(r=>n[r]),flagVariants:Object.keys(n).filter(r=>n[r]).reduce((r,a)=>(r[a]=n[a],r),{})}}pi(t){var{flags:n,flagVariants:r}=this.gi();this.featureFlagEventHandlers.forEach(a=>a(n,r,{errorsLoading:t}))}setPersonPropertiesForFlags(t,n){n===void 0&&(n=!0);var r=this._instance.get_property(Gc)||{};this._instance.register({[Gc]:Xe({},r,t)}),n&&this._instance.reloadFeatureFlags()}resetPersonPropertiesForFlags(){this._instance.unregister(Gc)}setGroupPropertiesForFlags(t,n){n===void 0&&(n=!0);var r=this._instance.get_property(Hl)||{};Object.keys(r).length!==0&&Object.keys(r).forEach(a=>{r[a]=Xe({},r[a],t[a]),delete t[a]}),this._instance.register({[Hl]:Xe({},r,t)}),n&&this._instance.reloadFeatureFlags()}resetGroupPropertiesForFlags(t){if(t){var n=this._instance.get_property(Hl)||{};this._instance.register({[Hl]:Xe({},n,{[t]:{}})})}else this._instance.unregister(Hl)}reset(){this.ei=!1,this.ri=!1,this.si=!1,this.ni=!1,this.oi=!1,this.ai=!1,this.$anon_distinct_id=void 0,this.vi(),this.ii=!1}}var pV=["cookie","localstorage","localstorage+cookie","sessionstorage","memory"];class Rg{constructor(t,n){this.I=t,this.props={},this.mi=!1,this.yi=(r=>{var a="";return r.token&&(a=r.token.replace(/\+/g,"PL").replace(/\//g,"SL").replace(/=/g,"EQ")),r.persistence_name?"ph_"+r.persistence_name:"ph_"+a+"_posthog"})(t),this.Y=this.bi(t),this.load(),t.debug&&Ge.info("Persistence loaded",t.persistence,Xe({},this.props)),this.update_config(t,t,n),this.save()}isDisabled(){return!!this.wi}bi(t){pV.indexOf(t.persistence.toLowerCase())===-1&&(Ge.critical("Unknown persistence type "+t.persistence+"; falling back to localStorage+cookie"),t.persistence="localStorage+cookie");var n=t.persistence.toLowerCase();return n==="localstorage"&&On.H()?On:n==="localstorage+cookie"&&Nh.H()?Nh:n==="sessionstorage"&&yr.H()?yr:n==="memory"?AG:n==="cookie"?gi:Nh.H()?Nh:gi}properties(){var t={};return hn(this.props,function(n,r){if(r===Ko&&hr(n))for(var a=Object.keys(n),s=0;s<a.length;s++)t["$feature/"+a[s]]=n[a[s]];else c=r,d=!1,(ds(o=iG)?d:F6&&o.indexOf===F6?o.indexOf(c)!=-1:(hn(o,function(f){if(d||(d=f===c))return vf}),d))||(t[r]=n);var o,c,d}),t}load(){if(!this.wi){var t=this.Y.W(this.yi);t&&(this.props=Fn({},t))}}save(){this.wi||this.Y.G(this.yi,this.props,this.xi,this.Ei,this.Si,this.I.debug)}remove(){this.Y.V(this.yi,!1),this.Y.V(this.yi,!0)}clear(){this.remove(),this.props={}}register_once(t,n,r){if(hr(t)){qe(n)&&(n="None"),this.xi=qe(r)?this.$i:r;var a=!1;if(hn(t,(s,o)=>{this.props.hasOwnProperty(o)&&this.props[o]!==n||(this.props[o]=s,a=!0)}),a)return this.save(),!0}return!1}register(t,n){if(hr(t)){this.xi=qe(n)?this.$i:n;var r=!1;if(hn(t,(a,s)=>{t.hasOwnProperty(s)&&this.props[s]!==a&&(this.props[s]=a,r=!0)}),r)return this.save(),!0}return!1}unregister(t){t in this.props&&(delete this.props[t],this.save())}update_campaign_params(){if(!this.mi){var t=a9(this.I.custom_campaign_params,this.I.mask_personal_data_properties,this.I.custom_personal_data_properties);Vo(av(t))||this.register(t),this.mi=!0}}update_search_keyword(){var t;this.register((t=$e==null?void 0:$e.referrer)?s9(t):{})}update_referrer_info(){var t;this.register_once({$referrer:l9(),$referring_domain:$e!=null&&$e.referrer&&((t=Sf($e.referrer))==null?void 0:t.host)||"$direct"},void 0)}set_initial_person_info(){this.props[Ub]||this.props[Fb]||this.register_once({[Ef]:o9(this.I.mask_personal_data_properties,this.I.custom_personal_data_properties)},void 0)}get_initial_props(){var t={};hn([Fb,Ub],o=>{var c=this.props[o];c&&hn(c,function(d,f){t["$initial_"+Cb(f)]=d})});var n,r,a=this.props[Ef];if(a){var s=(n=u9(a),r={},hn(n,function(o,c){r["$initial_"+Cb(c)]=o}),r);Fn(t,s)}return t}safe_merge(t){return hn(this.props,function(n,r){r in t||(t[r]=n)}),t}update_config(t,n,r){if(this.$i=this.xi=t.cookie_expiration,this.set_disabled(t.disable_persistence||!!r),this.set_cross_subdomain(t.cross_subdomain_cookie),this.set_secure(t.secure_cookie),t.persistence!==n.persistence){var a=this.bi(t),s=this.props;this.clear(),this.Y=a,this.props=s,this.save()}}set_disabled(t){this.wi=t,this.wi?this.remove():this.save()}set_cross_subdomain(t){t!==this.Ei&&(this.Ei=t,this.remove(),this.save())}set_secure(t){t!==this.Si&&(this.Si=t,this.remove(),this.save())}set_event_timer(t,n){var r=this.props[$c]||{};r[t]=n,this.props[$c]=r,this.save()}remove_event_timer(t){var n=(this.props[$c]||{})[t];return qe(n)||(delete this.props[$c][t],this.save()),n}get_property(t){return this.props[t]}set_property(t,n){this.props[t]=n,this.save()}}var Lr=Zn("[Product Tours]"),Mg="ph_product_tours";class gV{constructor(t){this.ki=null,this.Pi=null,this.Ti=void 0,this.Ri=!1,this._instance=t}onRemoteConfig(t){if(this._instance.config.disable_product_tours)Lr.info("Product tours disabled via config");else{var n=t.productTours;Gt(n)?Lr.info("Product tours not enabled in remote config"):(this.Ti=n,Lr.info("Remote config received, isProductToursEnabled: "+this.Ti),this.loadIfEnabled())}}loadIfEnabled(){var t;if(!this.Pi)if(this.Ri)Lr.info("Already initializing product tours, skipping...");else if(this._instance.config.disable_product_tours)Lr.info("Product tours disabled via config");else{var n=dt==null?void 0:dt.__PosthogExtensions__;if(n){var r=(t=this._instance.featureFlags)==null?void 0:t.isFeatureEnabled("product-tours-2025");if(!qe(this.Ti)||r){var a=this.Ti||r;if(a){this.Ri=!0;try{var s=n.generateProductTours;if(s)return void this.Ci(s,a);var o=n.loadExternalDependency;if(!o)return Lr.error("PostHog loadExternalDependency extension not found."),void(this.Ri=!1);o(this._instance,"product-tours",c=>{c||!n.generateProductTours?Lr.error("Could not load product tours script",c):this.Ci(n.generateProductTours,a),this.Ri=!1})}catch(c){Lr.error("Error initializing product tours",c),this.Ri=!1}}else Lr.info("Product tours not enabled")}else Lr.info("Waiting for remote config or feature flag to enable product tours")}else Lr.error("PostHog Extensions not found.")}}Ci(t,n){this.Pi=t(this._instance,n)||null,Lr.info("Product tours loaded successfully")}getProductTours(t,n){if(n===void 0&&(n=!1),!Vt(this.ki)||n){var r=this._instance.persistence;if(r){var a=r.props[Mg];if(Vt(a)&&!n)return this.ki=a,void t(a,{isLoaded:!0})}this._instance.ci({url:this._instance.requestRouter.endpointFor("api","/api/product_tours/?token="+this._instance.config.token),method:"GET",callback:s=>{var o=s.statusCode;if(o!==200||!s.json){var c="Product Tours API could not be loaded, status: "+o;return Lr.error(c),void t([],{isLoaded:!1,error:c})}var d=Vt(s.json.product_tours)?s.json.product_tours:[];this.ki=d,r&&r.register({[Mg]:d}),t(d,{isLoaded:!0})}})}else t(this.ki,{isLoaded:!0})}getActiveProductTours(t){if(Gt(this.Pi))return Lr.warn("Product tours not loaded yet"),void t([],{isLoaded:!1,error:"Product tours not loaded"});this.Pi.getActiveProductTours(t)}showProductTour(t){var n;Lr.info("showProductTour("+t+")"),(n=this.Pi)==null||n.showTourById(t)}dismissProductTour(){var t;(t=this.Pi)==null||t.dismissTour("user_clicked_skip")}nextStep(){var t;(t=this.Pi)==null||t.nextStep()}previousStep(){var t;(t=this.Pi)==null||t.previousStep()}clearCache(){this.ki=null;var t=this._instance.persistence;t&&t.unregister(Mg)}resetTour(t){var n;(n=this.Pi)==null||n.resetTour(t)}resetAllTours(){var t;(t=this.Pi)==null||t.resetAllTours()}}var Uc=function(e){return e.Activation="events",e.Cancellation="cancelEvents",e}({});(function(e){return e.Button="button",e.Tab="tab",e.Selector="selector",e})({});(function(e){return e.TopLeft="top_left",e.TopRight="top_right",e.TopCenter="top_center",e.MiddleLeft="middle_left",e.MiddleRight="middle_right",e.MiddleCenter="middle_center",e.Left="left",e.Center="center",e.Right="right",e.NextToTrigger="next_to_trigger",e})({});(function(e){return e.Top="top",e.Left="left",e.Right="right",e.Bottom="bottom",e})({});var Ig=function(e){return e.Popover="popover",e.API="api",e.Widget="widget",e.ExternalSurvey="external_survey",e}({});(function(e){return e.Open="open",e.MultipleChoice="multiple_choice",e.SingleChoice="single_choice",e.Rating="rating",e.Link="link",e})({});(function(e){return e.NextQuestion="next_question",e.End="end",e.ResponseBased="response_based",e.SpecificQuestion="specific_question",e})({});(function(e){return e.Once="once",e.Recurring="recurring",e.Always="always",e})({});var Kh=function(e){return e.SHOWN="survey shown",e.DISMISSED="survey dismissed",e.SENT="survey sent",e}({}),k5=function(e){return e.SURVEY_ID="$survey_id",e.SURVEY_NAME="$survey_name",e.SURVEY_RESPONSE="$survey_response",e.SURVEY_ITERATION="$survey_iteration",e.SURVEY_ITERATION_START_DATE="$survey_iteration_start_date",e.SURVEY_PARTIALLY_COMPLETED="$survey_partially_completed",e.SURVEY_SUBMISSION_ID="$survey_submission_id",e.SURVEY_QUESTIONS="$survey_questions",e.SURVEY_COMPLETED="$survey_completed",e}({}),v9=function(e){return e.Popover="popover",e.Inline="inline",e}({});class mv{constructor(){this.Ii={},this.Ii={}}on(t,n){return this.Ii[t]||(this.Ii[t]=[]),this.Ii[t].push(n),()=>{this.Ii[t]=this.Ii[t].filter(r=>r!==n)}}emit(t,n){for(var r of this.Ii[t]||[])r(n);for(var a of this.Ii["*"]||[])a(t,n)}}function Fo(e,t,n){if(Gt(e))return!1;switch(n){case"exact":return e===t;case"contains":var r=t.replace(/[.*+?^${}()|[\]\\]/g,"\\$&").replace(/_/g,".").replace(/%/g,".*");return new RegExp(r,"i").test(e);case"regex":try{return new RegExp(t).test(e)}catch{return!1}default:return!1}}class bV{constructor(t){this.Fi=new mv,this.Mi=(n,r)=>this.Oi(n,r)&&this.Ai(n,r)&&this.Di(n,r)&&this.ji(n,r),this.Oi=(n,r)=>r==null||!r.event||(n==null?void 0:n.event)===(r==null?void 0:r.event),this._instance=t,this.Li=new Set,this.Ni=new Set}init(){var t;if(!qe((t=this._instance)==null?void 0:t.Ui)){var n;(n=this._instance)==null||n.Ui((r,a)=>{this.on(r,a)})}}register(t){var n,r;if(!qe((n=this._instance)==null?void 0:n.Ui)&&(t.forEach(o=>{var c,d;(c=this.Ni)==null||c.add(o),(d=o.steps)==null||d.forEach(f=>{var m;(m=this.Li)==null||m.add((f==null?void 0:f.event)||"")})}),(r=this._instance)!=null&&r.autocapture)){var a,s=new Set;t.forEach(o=>{var c;(c=o.steps)==null||c.forEach(d=>{d!=null&&d.selector&&s.add(d==null?void 0:d.selector)})}),(a=this._instance)==null||a.autocapture.setElementSelectors(s)}}on(t,n){var r;n!=null&&t.length!=0&&(this.Li.has(t)||this.Li.has(n==null?void 0:n.event))&&this.Ni&&((r=this.Ni)==null?void 0:r.size)>0&&this.Ni.forEach(a=>{this.zi(n,a)&&this.Fi.emit("actionCaptured",a.name)})}Hi(t){this.onAction("actionCaptured",n=>t(n))}zi(t,n){if((n==null?void 0:n.steps)==null)return!1;for(var r of n.steps)if(this.Mi(t,r))return!0;return!1}onAction(t,n){return this.Fi.on(t,n)}Ai(t,n){if(n!=null&&n.url){var r,a=t==null||(r=t.properties)==null?void 0:r.$current_url;if(!a||typeof a!="string"||!Fo(a,n.url,n.url_matching||"contains"))return!1}return!0}Di(t,n){return!!this.Bi(t,n)&&!!this.qi(t,n)&&!!this.Wi(t,n)}Bi(t,n){var r;if(n==null||!n.href)return!0;var a=this.Gi(t);if(a.length>0)return a.some(c=>Fo(c.href,n.href,n.href_matching||"exact"));var s,o=(t==null||(r=t.properties)==null?void 0:r.$elements_chain)||"";return!!o&&Fo((s=o.match(/(?::|")href="(.*?)"/))?s[1]:"",n.href,n.href_matching||"exact")}qi(t,n){var r;if(n==null||!n.text)return!0;var a=this.Gi(t);if(a.length>0)return a.some(f=>Fo(f.text,n.text,n.text_matching||"exact")||Fo(f.$el_text,n.text,n.text_matching||"exact"));var s,o,c,d=(t==null||(r=t.properties)==null?void 0:r.$elements_chain)||"";return!!d&&(s=function(f){for(var m,p=[],b=/(?::|")text="(.*?)"/g;!Gt(m=b.exec(f));)p.includes(m[1])||p.push(m[1]);return p}(d),o=n.text,c=n.text_matching||"exact",s.some(f=>Fo(f,o,c)))}Wi(t,n){var r,a;if(n==null||!n.selector)return!0;var s=t==null||(r=t.properties)==null?void 0:r.$element_selectors;if(s!=null&&s.includes(n.selector))return!0;var o=(t==null||(a=t.properties)==null?void 0:a.$elements_chain)||"";if(n.selector_regex&&o)try{return new RegExp(n.selector_regex).test(o)}catch{return!1}return!1}Gi(t){var n;return(t==null||(n=t.properties)==null?void 0:n.$elements)==null?[]:t==null?void 0:t.properties.$elements}ji(t,n){return n==null||!n.properties||n.properties.length===0||b9(n.properties.reduce((r,a)=>{var s=Vt(a.value)?a.value.map(String):a.value!=null?[String(a.value)]:[];return r[a.key]={values:s,operator:a.operator||"exact"},r},{}),t==null?void 0:t.properties)}}var Xt=Zn("[Surveys]"),Vb="seenSurvey_",vV=(e,t)=>{var n="$survey_"+t+"/"+e.id;return e.current_iteration&&e.current_iteration>0&&(n="$survey_"+t+"/"+e.id+"/"+e.current_iteration),n},C5=e=>{var t=""+Vb+e.id;return e.current_iteration&&e.current_iteration>0&&(t=""+Vb+e.id+"_"+e.current_iteration),t},yV=[Ig.Popover,Ig.Widget,Ig.API],_V={ignoreConditions:!1,ignoreDelay:!1,displayType:v9.Popover};class EV{constructor(t){this._instance=t,this.Vi=new Map,this.Ji=new Map,this.Ki=new Map}Yi(t,n){return!!t&&b9(t.propertyFilters,n==null?void 0:n.properties)}Xi(t,n){var r=new Map;return t.forEach(a=>{var s;(s=a.conditions)==null||(s=s[n])==null||(s=s.values)==null||s.forEach(o=>{if(o!=null&&o.name){var c=r.get(o.name)||[];c.push(a.id),r.set(o.name,c)}})}),r}Qi(t,n,r){var a,s=(r===Uc.Activation?this.Vi:this.Ji).get(t),o=[];return(a=this._instance)==null||a.getSurveys(c=>{o=c.filter(d=>s==null?void 0:s.includes(d.id))}),o.filter(c=>{var d,f=(d=c.conditions)==null||(d=d[r])==null||(d=d.values)==null?void 0:d.find(m=>m.name===t);return this.Yi(f,n)})}register(t){var n;qe((n=this._instance)==null?void 0:n.Ui)||(this.Zi(t),this.te(t))}te(t){var n=t.filter(r=>{var a,s;return((a=r.conditions)==null?void 0:a.actions)&&((s=r.conditions)==null||(s=s.actions)==null||(s=s.values)==null?void 0:s.length)>0});n.length!==0&&(this.ie==null&&(this.ie=new bV(this._instance),this.ie.init(),this.ie.Hi(r=>{this.onAction(r)})),n.forEach(r=>{var a,s,o,c,d;r.conditions&&(a=r.conditions)!=null&&a.actions&&(s=r.conditions)!=null&&(s=s.actions)!=null&&s.values&&((o=r.conditions)==null||(o=o.actions)==null||(o=o.values)==null?void 0:o.length)>0&&((c=this.ie)==null||c.register(r.conditions.actions.values),(d=r.conditions)==null||(d=d.actions)==null||(d=d.values)==null||d.forEach(f=>{if(f&&f.name){var m=this.Ki.get(f.name);m&&m.push(r.id),this.Ki.set(f.name,m||[r.id])}}))}))}Zi(t){var n,r=t.filter(s=>{var o,c;return((o=s.conditions)==null?void 0:o.events)&&((c=s.conditions)==null||(c=c.events)==null||(c=c.values)==null?void 0:c.length)>0}),a=t.filter(s=>{var o,c;return((o=s.conditions)==null?void 0:o.cancelEvents)&&((c=s.conditions)==null||(c=c.cancelEvents)==null||(c=c.values)==null?void 0:c.length)>0});(r.length!==0||a.length!==0)&&((n=this._instance)==null||n.Ui((s,o)=>{this.onEvent(s,o)}),this.Vi=this.Xi(t,Uc.Activation),this.Ji=this.Xi(t,Uc.Cancellation))}onEvent(t,n){var r,a=((r=this._instance)==null||(r=r.persistence)==null?void 0:r.props[Ch])||[];if(Kh.SHOWN===t&&n&&a.length>0){var s;Xt.info("survey event matched, removing survey from activated surveys",{event:t,eventPayload:n,existingActivatedSurveys:a});var o=n==null||(s=n.properties)==null?void 0:s.$survey_id;if(o){var c=a.indexOf(o);c>=0&&(a.splice(c,1),this.ee(a))}}else{if(this.Ji.has(t)){var d=this.Qi(t,n,Uc.Cancellation);d.length>0&&(Xt.info("cancel event matched, cancelling surveys",{event:t,surveysToCancel:d.map(m=>m.id)}),d.forEach(m=>{var p,b=a.indexOf(m.id);b>=0&&a.splice(b,1),(p=this._instance)==null||p.cancelPendingSurvey(m.id)}),this.ee(a))}if(this.Vi.has(t)){Xt.info("survey event name matched",{event:t,eventPayload:n,surveys:this.Vi.get(t)});var f=this.Qi(t,n,Uc.Activation);this.ee(a.concat(f.map(m=>m.id)||[]))}}}onAction(t){var n,r=((n=this._instance)==null||(n=n.persistence)==null?void 0:n.props[Ch])||[];this.Ki.has(t)&&this.ee(r.concat(this.Ki.get(t)||[]))}ee(t){var n;Xt.info("updating activated surveys",{activatedSurveys:t}),(n=this._instance)==null||(n=n.persistence)==null||n.register({[Ch]:[...new Set(t)]})}getSurveys(){var t,n=(t=this._instance)==null||(t=t.persistence)==null?void 0:t.props[Ch];return n||[]}getEventToSurveys(){return this.Vi}re(){return this.ie}}class TV{constructor(t){this.se=void 0,this._surveyManager=null,this.ne=!1,this.oe=!1,this.ae=[],this._instance=t,this._surveyEventReceiver=null}onRemoteConfig(t){if(!this._instance.config.disable_surveys){var n=t.surveys;if(Gt(n))return Xt.warn("Flags not loaded yet. Not loading surveys.");var r=Vt(n);this.se=r?n.length>0:n,Xt.info("flags response received, isSurveysEnabled: "+this.se),this.loadIfEnabled()}}reset(){localStorage.removeItem("lastSeenSurveyDate");for(var t=[],n=0;n<localStorage.length;n++){var r=localStorage.key(n);(r!=null&&r.startsWith(Vb)||r!=null&&r.startsWith("inProgressSurvey_"))&&t.push(r)}t.forEach(a=>localStorage.removeItem(a))}loadIfEnabled(){if(!this._surveyManager)if(this.oe)Xt.info("Already initializing surveys, skipping...");else if(this._instance.config.disable_surveys)Xt.info("Disabled. Not loading surveys.");else if(this._instance.config.cookieless_mode&&this._instance.consent.isOptedOut())Xt.info("Not loading surveys in cookieless mode without consent.");else{var t=dt==null?void 0:dt.__PosthogExtensions__;if(t){if(!qe(this.se)||this._instance.config.advanced_enable_surveys){var n=this.se||this._instance.config.advanced_enable_surveys;this.oe=!0;try{var r=t.generateSurveys;if(r)return void this.le(r,n);var a=t.loadExternalDependency;if(!a)return void this.ue("PostHog loadExternalDependency extension not found.");a(this._instance,"surveys",s=>{s||!t.generateSurveys?this.ue("Could not load surveys script",s):this.le(t.generateSurveys,n)})}catch(s){throw this.ue("Error initializing surveys",s),s}finally{this.oe=!1}}}else Xt.error("PostHog Extensions not found.")}}le(t,n){this._surveyManager=t(this._instance,n),this._surveyEventReceiver=new EV(this._instance),Xt.info("Surveys loaded successfully"),this.he({isLoaded:!0})}ue(t,n){Xt.error(t,n),this.he({isLoaded:!1,error:t})}onSurveysLoaded(t){return this.ae.push(t),this._surveyManager&&this.he({isLoaded:!0}),()=>{this.ae=this.ae.filter(n=>n!==t)}}getSurveys(t,n){if(n===void 0&&(n=!1),this._instance.config.disable_surveys)return Xt.info("Disabled. Not loading surveys."),t([]);var r=this._instance.get_property(Lb);if(r&&!n)return t(r,{isLoaded:!0});if(this.ne)return t([],{isLoaded:!1,error:"Surveys are already being loaded"});try{this.ne=!0,this._instance.ci({url:this._instance.requestRouter.endpointFor("api","/api/surveys/?token="+this._instance.config.token),method:"GET",timeout:this._instance.config.surveys_request_timeout_ms,callback:a=>{var s;this.ne=!1;var o=a.statusCode;if(o!==200||!a.json){var c="Surveys API could not be loaded, status: "+o;return Xt.error(c),t([],{isLoaded:!1,error:c})}var d,f=a.json.surveys||[],m=f.filter(p=>function(b){return!(!b.start_date||b.end_date)}(p)&&(function(b){var y;return!((y=b.conditions)==null||(y=y.events)==null||(y=y.values)==null||!y.length)}(p)||function(b){var y;return!((y=b.conditions)==null||(y=y.actions)==null||(y=y.values)==null||!y.length)}(p)));return m.length>0&&((d=this._surveyEventReceiver)==null||d.register(m)),(s=this._instance.persistence)==null||s.register({[Lb]:f}),t(f,{isLoaded:!0})}})}catch(a){throw this.ne=!1,a}}he(t){for(var n of this.ae)try{if(!t.isLoaded)return n([],t);this.getSurveys(n)}catch(r){Xt.error("Error in survey callback",r)}}getActiveMatchingSurveys(t,n){if(n===void 0&&(n=!1),!Gt(this._surveyManager))return this._surveyManager.getActiveMatchingSurveys(t,n);Xt.warn("init was not called")}de(t){var n=null;return this.getSurveys(r=>{var a;n=(a=r.find(s=>s.id===t))!==null&&a!==void 0?a:null}),n}ve(t){if(Gt(this._surveyManager))return{eligible:!1,reason:"SDK is not enabled or survey functionality is not yet loaded"};var n=typeof t=="string"?this.de(t):t;return n?this._surveyManager.checkSurveyEligibility(n):{eligible:!1,reason:"Survey not found"}}canRenderSurvey(t){if(Gt(this._surveyManager))return Xt.warn("init was not called"),{visible:!1,disabledReason:"SDK is not enabled or survey functionality is not yet loaded"};var n=this.ve(t);return{visible:n.eligible,disabledReason:n.reason}}canRenderSurveyAsync(t,n){return Gt(this._surveyManager)?(Xt.warn("init was not called"),Promise.resolve({visible:!1,disabledReason:"SDK is not enabled or survey functionality is not yet loaded"})):new Promise(r=>{this.getSurveys(a=>{var s,o=(s=a.find(d=>d.id===t))!==null&&s!==void 0?s:null;if(o){var c=this.ve(o);r({visible:c.eligible,disabledReason:c.reason})}else r({visible:!1,disabledReason:"Survey not found"})},n)})}renderSurvey(t,n){var r;if(Gt(this._surveyManager))Xt.warn("init was not called");else{var a=typeof t=="string"?this.de(t):t;if(a!=null&&a.id)if(yV.includes(a.type)){var s=$e==null?void 0:$e.querySelector(n);if(s)return(r=a.appearance)!=null&&r.surveyPopupDelaySeconds?(Xt.info("Rendering survey "+a.id+" with delay of "+a.appearance.surveyPopupDelaySeconds+" seconds"),void setTimeout(()=>{var o,c;Xt.info("Rendering survey "+a.id+" with delay of "+((o=a.appearance)==null?void 0:o.surveyPopupDelaySeconds)+" seconds"),(c=this._surveyManager)==null||c.renderSurvey(a,s),Xt.info("Survey "+a.id+" rendered")},1e3*a.appearance.surveyPopupDelaySeconds)):void this._surveyManager.renderSurvey(a,s);Xt.warn("Survey element not found")}else Xt.warn("Surveys of type "+a.type+" cannot be rendered in the app");else Xt.warn("Survey not found")}}displaySurvey(t,n){var r;if(Gt(this._surveyManager))Xt.warn("init was not called");else{var a=this.de(t);if(a){var s=a;if((r=a.appearance)!=null&&r.surveyPopupDelaySeconds&&n.ignoreDelay&&(s=Xe({},a,{appearance:Xe({},a.appearance,{surveyPopupDelaySeconds:0})})),n.ignoreConditions===!1){var o=this.canRenderSurvey(a);if(!o.visible)return void Xt.warn("Survey is not eligible to be displayed: ",o.disabledReason)}n.displayType!==v9.Inline?this._surveyManager.handlePopoverSurvey(s):this.renderSurvey(s,n.selector)}else Xt.warn("Survey not found")}}cancelPendingSurvey(t){Gt(this._surveyManager)?Xt.warn("init was not called"):this._surveyManager.cancelSurvey(t)}}var N5=Zn("[RateLimiter]");class xV{constructor(t){var n,r;this.serverLimits={},this.lastEventRateLimited=!1,this.checkForLimiting=a=>{var s=a.text;if(s&&s.length)try{(JSON.parse(s).quota_limited||[]).forEach(o=>{N5.info((o||"events")+" is quota limited."),this.serverLimits[o]=new Date().getTime()+6e4})}catch(o){return void N5.warn('could not rate limit - continuing. Error: "'+(o==null?void 0:o.message)+'"',{text:s})}},this.instance=t,this.captureEventsPerSecond=((n=t.config.rate_limiting)==null?void 0:n.events_per_second)||10,this.captureEventsBurstLimit=Math.max(((r=t.config.rate_limiting)==null?void 0:r.events_burst_limit)||10*this.captureEventsPerSecond,this.captureEventsPerSecond),this.lastEventRateLimited=this.clientRateLimitContext(!0).isRateLimited}clientRateLimitContext(t){var n,r,a;t===void 0&&(t=!1);var s=new Date().getTime(),o=(n=(r=this.instance.persistence)==null?void 0:r.get_property(Bb))!==null&&n!==void 0?n:{tokens:this.captureEventsBurstLimit,last:s};o.tokens+=(s-o.last)/1e3*this.captureEventsPerSecond,o.last=s,o.tokens>this.captureEventsBurstLimit&&(o.tokens=this.captureEventsBurstLimit);var c=o.tokens<1;return c||t||(o.tokens=Math.max(0,o.tokens-1)),!c||this.lastEventRateLimited||t||this.instance.capture("$$client_ingestion_warning",{$$client_ingestion_warning_message:"posthog-js client rate limited. Config is set to "+this.captureEventsPerSecond+" events per second and "+this.captureEventsBurstLimit+" events burst limit."},{skip_client_rate_limiting:!0}),this.lastEventRateLimited=c,(a=this.instance.persistence)==null||a.set_property(Bb,o),{isRateLimited:c,remainingTokens:o.tokens}}isServerRateLimited(t){var n=this.serverLimits[t||"events"]||!1;return n!==!1&&new Date().getTime()<n}}var zl=Zn("[RemoteConfig]");class SV{constructor(t){this._instance=t}get remoteConfig(){var t;return(t=dt._POSTHOG_REMOTE_CONFIG)==null||(t=t[this._instance.config.token])==null?void 0:t.config}ce(t){var n,r;(n=dt.__PosthogExtensions__)!=null&&n.loadExternalDependency?(r=dt.__PosthogExtensions__)==null||r.loadExternalDependency==null||r.loadExternalDependency(this._instance,"remote-config",()=>t(this.remoteConfig)):(zl.error("PostHog Extensions not found. Cannot load remote config."),t())}fe(t){this._instance.ci({method:"GET",url:this._instance.requestRouter.endpointFor("assets","/array/"+this._instance.config.token+"/config"),callback:n=>{t(n.json)}})}load(){try{if(this.remoteConfig)return zl.info("Using preloaded remote config",this.remoteConfig),void this.fi(this.remoteConfig);if(this._instance.O())return void zl.warn("Remote config is disabled. Falling back to local config.");this.ce(t=>{if(!t)return zl.info("No config found after loading remote JS config. Falling back to JSON."),void this.fe(n=>{this.fi(n)});this.fi(t)})}catch(t){zl.error("Error loading remote config",t)}}fi(t){t?this._instance.config.__preview_remote_config?(this._instance.fi(t),t.hasFeatureFlags!==!1&&this._instance.featureFlags.ensureFlagsLoaded()):zl.info("__preview_remote_config is disabled. Logging config instead",t):zl.error("Failed to fetch remote config from PostHog.")}}var Kb=3e3;class wV{constructor(t,n){this.pe=!0,this.ge=[],this._e=mi((n==null?void 0:n.flush_interval_ms)||Kb,250,5e3,Ge.createLogger("flush interval"),Kb),this.me=t}enqueue(t){this.ge.push(t),this.ye||this.be()}unload(){this.we();var t=this.ge.length>0?this.xe():{},n=Object.values(t);[...n.filter(r=>r.url.indexOf("/e")===0),...n.filter(r=>r.url.indexOf("/e")!==0)].map(r=>{this.me(Xe({},r,{transport:"sendBeacon"}))})}enable(){this.pe=!1,this.be()}be(){var t=this;this.pe||(this.ye=setTimeout(()=>{if(this.we(),this.ge.length>0){var n=this.xe(),r=function(){var s=n[a],o=new Date().getTime();s.data&&Vt(s.data)&&hn(s.data,c=>{c.offset=Math.abs(c.timestamp-o),delete c.timestamp}),t.me(s)};for(var a in n)r()}},this._e))}we(){clearTimeout(this.ye),this.ye=void 0}xe(){var t={};return hn(this.ge,n=>{var r,a=n,s=(a?a.batchKey:null)||a.url;qe(t[s])&&(t[s]=Xe({},a,{data:[]})),(r=t[s].data)==null||r.push(a.data)}),this.ge=[],t}}var AV=["retriesPerformedSoFar"];class kV{constructor(t){this.Ee=!1,this.Se=3e3,this.ge=[],this._instance=t,this.ge=[],this.$e=!0,!qe(be)&&"onLine"in be.navigator&&(this.$e=be.navigator.onLine,this.ke=()=>{this.$e=!0,this.Dt()},this.Pe=()=>{this.$e=!1},Xn(be,"online",this.ke),Xn(be,"offline",this.Pe))}get length(){return this.ge.length}retriableRequest(t){var{retriesPerformedSoFar:n}=t,r=fS(t,AV);bi(n)&&n>0&&(r.url=Of(r.url,{retry_count:n})),this._instance.ci(Xe({},r,{callback:a=>{a.statusCode!==200&&(a.statusCode<400||a.statusCode>=500)&&(n??0)<10?this.Te(Xe({retriesPerformedSoFar:n},r)):r.callback==null||r.callback(a)}}))}Te(t){var n=t.retriesPerformedSoFar||0;t.retriesPerformedSoFar=n+1;var r=function(o){var c=3e3*Math.pow(2,o),d=c/2,f=Math.min(18e5,c),m=(Math.random()-.5)*(f-d);return Math.ceil(f+m)}(n),a=Date.now()+r;this.ge.push({retryAt:a,requestOptions:t});var s="Enqueued failed request for retry in "+r;navigator.onLine||(s+=" (Browser is offline)"),Ge.warn(s),this.Ee||(this.Ee=!0,this.Re())}Re(){if(this.Ce&&clearTimeout(this.Ce),this.ge.length===0)return this.Ee=!1,void(this.Ce=void 0);this.Ce=setTimeout(()=>{this.$e&&this.ge.length>0&&this.Dt(),this.Re()},this.Se)}Dt(){var t=Date.now(),n=[],r=this.ge.filter(s=>s.retryAt<t||(n.push(s),!1));if(this.ge=n,r.length>0)for(var{requestOptions:a}of r)this.retriableRequest(a)}unload(){for(var{requestOptions:t}of(this.Ce&&(clearTimeout(this.Ce),this.Ce=void 0),this.Ee=!1,qe(be)||(this.ke&&(be.removeEventListener("online",this.ke),this.ke=void 0),this.Pe&&(be.removeEventListener("offline",this.Pe),this.Pe=void 0)),this.ge))try{this._instance.ci(Xe({},t,{transport:"sendBeacon"}))}catch(n){Ge.error(n)}this.ge=[]}}class CV{constructor(t){this.Ie=()=>{var n,r,a,s;this.Fe||(this.Fe={});var o=this.scrollElement(),c=this.scrollY(),d=o?Math.max(0,o.scrollHeight-o.clientHeight):0,f=c+((o==null?void 0:o.clientHeight)||0),m=(o==null?void 0:o.scrollHeight)||0;this.Fe.lastScrollY=Math.ceil(c),this.Fe.maxScrollY=Math.max(c,(n=this.Fe.maxScrollY)!==null&&n!==void 0?n:0),this.Fe.maxScrollHeight=Math.max(d,(r=this.Fe.maxScrollHeight)!==null&&r!==void 0?r:0),this.Fe.lastContentY=f,this.Fe.maxContentY=Math.max(f,(a=this.Fe.maxContentY)!==null&&a!==void 0?a:0),this.Fe.maxContentHeight=Math.max(m,(s=this.Fe.maxContentHeight)!==null&&s!==void 0?s:0)},this._instance=t}getContext(){return this.Fe}resetContext(){var t=this.Fe;return setTimeout(this.Ie,0),t}startMeasuringScrollPosition(){Xn(be,"scroll",this.Ie,{capture:!0}),Xn(be,"scrollend",this.Ie,{capture:!0}),Xn(be,"resize",this.Ie)}scrollElement(){if(!this._instance.config.scroll_root_selector)return be==null?void 0:be.document.documentElement;var t=Vt(this._instance.config.scroll_root_selector)?this._instance.config.scroll_root_selector:[this._instance.config.scroll_root_selector];for(var n of t){var r=be==null?void 0:be.document.querySelector(n);if(r)return r}}scrollY(){if(this._instance.config.scroll_root_selector){var t=this.scrollElement();return t&&t.scrollTop||0}return be&&(be.scrollY||be.pageYOffset||be.document.documentElement.scrollTop)||0}scrollX(){if(this._instance.config.scroll_root_selector){var t=this.scrollElement();return t&&t.scrollLeft||0}return be&&(be.scrollX||be.pageXOffset||be.document.documentElement.scrollLeft)||0}}var NV=e=>o9(e==null?void 0:e.config.mask_personal_data_properties,e==null?void 0:e.config.custom_personal_data_properties);class O5{constructor(t,n,r,a){this.Me=s=>{var o=this.Oe();if(!o||o.sessionId!==s){var c={sessionId:s,props:this.Ae(this._instance)};this.De.register({[Pb]:c})}},this._instance=t,this.je=n,this.De=r,this.Ae=a||NV,this.je.onSessionId(this.Me)}Oe(){return this.De.props[Pb]}getSetOnceProps(){var t,n=(t=this.Oe())==null?void 0:t.props;return n?"r"in n?u9(n):{$referring_domain:n.referringDomain,$pathname:n.initialPathName,utm_source:n.utm_source,utm_campaign:n.utm_campaign,utm_medium:n.utm_medium,utm_content:n.utm_content,utm_term:n.utm_term}:{}}getSessionProps(){var t={};return hn(av(this.getSetOnceProps()),(n,r)=>{r==="$current_url"&&(r="url"),t["$session_entry_"+Cb(r)]=n}),t}}var Dg=Zn("[SessionId]");class R5{on(t,n){return this.Le.on(t,n)}constructor(t,n,r){var a;if(this.Ne=[],this.Ue=void 0,this.Le=new mv,this.ze=(m,p)=>Math.abs(m-p)>this.sessionTimeoutMs,!t.persistence)throw new Error("SessionIdManager requires a PostHogPersistence instance");if(t.config.cookieless_mode==="always")throw new Error('SessionIdManager cannot be used with cookieless_mode="always"');this.I=t.config,this.De=t.persistence,this.He=void 0,this.Be=void 0,this._sessionStartTimestamp=null,this._sessionActivityTimestamp=null,this.qe=n||el,this.We=r||el;var s=this.I.persistence_name||this.I.token,o=this.I.session_idle_timeout_seconds||1800;if(this._sessionTimeoutMs=1e3*mi(o,60,36e3,Dg.createLogger("session_idle_timeout_seconds"),1800),t.register({$configured_session_timeout_ms:this._sessionTimeoutMs}),this.Ge(),this.Ve="ph_"+s+"_window_id",this.Je="ph_"+s+"_primary_window_exists",this.Ke()){var c=yr.W(this.Ve),d=yr.W(this.Je);c&&!d?this.He=c:yr.V(this.Ve),yr.G(this.Je,!0)}if((a=this.I.bootstrap)!=null&&a.sessionID)try{var f=(m=>{var p=m.replace(/-/g,"");if(p.length!==32)throw new Error("Not a valid UUID");if(p[12]!=="7")throw new Error("Not a UUIDv7");return parseInt(p.substring(0,12),16)})(this.I.bootstrap.sessionID);this.Ye(this.I.bootstrap.sessionID,new Date().getTime(),f)}catch(m){Dg.error("Invalid sessionID in bootstrap",m)}this.Xe()}get sessionTimeoutMs(){return this._sessionTimeoutMs}onSessionId(t){return qe(this.Ne)&&(this.Ne=[]),this.Ne.push(t),this.Be&&t(this.Be,this.He),()=>{this.Ne=this.Ne.filter(n=>n!==t)}}Ke(){return this.I.persistence!=="memory"&&!this.De.wi&&yr.H()}Qe(t){t!==this.He&&(this.He=t,this.Ke()&&yr.G(this.Ve,t))}Ze(){return this.He?this.He:this.Ke()?yr.W(this.Ve):null}Ye(t,n,r){t===this.Be&&n===this._sessionActivityTimestamp&&r===this._sessionStartTimestamp||(this._sessionStartTimestamp=r,this._sessionActivityTimestamp=n,this.Be=t,this.De.register({[yf]:[n,t,r]}))}tr(){var t=this.De.props[yf];return Vt(t)&&t.length===2&&t.push(t[0]),t||[0,null,0]}resetSessionId(){this.Ye(null,null,null)}destroy(){clearTimeout(this.ir),this.ir=void 0,this.Ue&&be&&(be.removeEventListener("beforeunload",this.Ue,{capture:!1}),this.Ue=void 0),this.Ne=[]}Xe(){this.Ue=()=>{this.Ke()&&yr.V(this.Je)},Xn(be,"beforeunload",this.Ue,{capture:!1})}checkAndGetSessionAndWindowId(t,n){if(t===void 0&&(t=!1),n===void 0&&(n=null),this.I.cookieless_mode==="always")throw new Error('checkAndGetSessionAndWindowId should not be called with cookieless_mode="always"');var r=n||new Date().getTime(),[a,s,o]=this.tr(),c=this.Ze(),d=bi(o)&&o>0&&Math.abs(r-o)>864e5,f=!1,m=!s,p=!m&&!t&&this.ze(r,a);m||p||d?(s=this.qe(),c=this.We(),Dg.info("new session ID generated",{sessionId:s,windowId:c,changeReason:{noSessionId:m,activityTimeout:p,sessionPastMaximumLength:d}}),o=r,f=!0):c||(c=this.We(),f=!0);var b=a===0||!t||d?r:a,y=o===0?new Date().getTime():o;return this.Qe(c),this.Ye(s,b,y),t||this.Ge(),f&&this.Ne.forEach(T=>T(s,c,f?{noSessionId:m,activityTimeout:p,sessionPastMaximumLength:d}:void 0)),{sessionId:s,windowId:c,sessionStartTimestamp:y,changeReason:f?{noSessionId:m,activityTimeout:p,sessionPastMaximumLength:d}:void 0,lastActivityTimestamp:a}}Ge(){clearTimeout(this.ir),this.ir=setTimeout(()=>{var[t]=this.tr();if(this.ze(new Date().getTime(),t)){var n=this.Be;this.resetSessionId(),this.Le.emit("forcedIdleReset",{idleSessionId:n})}},1.1*this.sessionTimeoutMs)}}var OV=["$set_once","$set"],$s=Zn("[SiteApps]");class RV{constructor(t){this._instance=t,this.er=[],this.apps={}}get isEnabled(){return!!this._instance.config.opt_in_site_apps}rr(t,n){if(n){var r=this.globalsForEvent(n);this.er.push(r),this.er.length>1e3&&(this.er=this.er.slice(10))}}get siteAppLoaders(){var t;return(t=dt._POSTHOG_REMOTE_CONFIG)==null||(t=t[this._instance.config.token])==null?void 0:t.siteApps}init(){if(this.isEnabled){var t=this._instance.Ui(this.rr.bind(this));this.sr=()=>{t(),this.er=[],this.sr=void 0}}}globalsForEvent(t){var n,r,a,s,o,c,d;if(!t)throw new Error("Event payload is required");var f={},m=this._instance.get_property("$groups")||[],p=this._instance.get_property("$stored_group_properties")||{};for(var[b,y]of Object.entries(p))f[b]={id:m[b],type:b,properties:y};var{$set_once:T,$set:x}=t;return{event:Xe({},fS(t,OV),{properties:Xe({},t.properties,x?{$set:Xe({},(n=(r=t.properties)==null?void 0:r.$set)!==null&&n!==void 0?n:{},x)}:{},T?{$set_once:Xe({},(a=(s=t.properties)==null?void 0:s.$set_once)!==null&&a!==void 0?a:{},T)}:{}),elements_chain:(o=(c=t.properties)==null?void 0:c.$elements_chain)!==null&&o!==void 0?o:"",distinct_id:(d=t.properties)==null?void 0:d.distinct_id}),person:{properties:this._instance.get_property("$stored_person_properties")},groups:f}}setupSiteApp(t){var n=this.apps[t.id],r=()=>{var c;!n.errored&&this.er.length&&($s.info("Processing "+this.er.length+" events for site app with id "+t.id),this.er.forEach(d=>n.processEvent==null?void 0:n.processEvent(d)),n.processedBuffer=!0),Object.values(this.apps).every(d=>d.processedBuffer||d.errored)&&((c=this.sr)==null||c.call(this))},a=!1,s=c=>{n.errored=!c,n.loaded=!0,$s.info("Site app with id "+t.id+" "+(c?"loaded":"errored")),a&&r()};try{var{processEvent:o}=t.init({posthog:this._instance,callback:c=>{s(c)}});o&&(n.processEvent=o),a=!0}catch(c){$s.error("Error while initializing PostHog app with config id "+t.id,c),s(!1)}if(a&&n.loaded)try{r()}catch(c){$s.error("Error while processing buffered events PostHog app with config id "+t.id,c),n.errored=!0}}nr(){var t=this.siteAppLoaders||[];for(var n of t)this.apps[n.id]={id:n.id,loaded:!1,errored:!1,processedBuffer:!1};for(var r of t)this.setupSiteApp(r)}ar(t){if(Object.keys(this.apps).length!==0){var n=this.globalsForEvent(t);for(var r of Object.values(this.apps))try{r.processEvent==null||r.processEvent(n)}catch(a){$s.error("Error while processing event "+t.event+" for site app "+r.id,a)}}}onRemoteConfig(t){var n,r,a,s=this;if((n=this.siteAppLoaders)!=null&&n.length)return this.isEnabled?(this.nr(),void this._instance.on("eventCaptured",f=>this.ar(f))):void $s.error('PostHog site apps are disabled. Enable the "opt_in_site_apps" config to proceed.');if((r=this.sr)==null||r.call(this),(a=t.siteApps)!=null&&a.length)if(this.isEnabled){var o=function(f){var m;dt["__$$ph_site_app_"+f]=s._instance,(m=dt.__PosthogExtensions__)==null||m.loadSiteApp==null||m.loadSiteApp(s._instance,d,p=>{if(p)return $s.error("Error while initializing PostHog app with config id "+f,p)})};for(var{id:c,url:d}of t.siteApps)o(c)}else $s.error('PostHog site apps are disabled. Enable the "opt_in_site_apps" config to proceed.')}}var y9=function(e,t){if(!e)return!1;var n=e.userAgent;if(n&&j6(n,t))return!0;try{var r=e==null?void 0:e.userAgentData;if(r!=null&&r.brands&&r.brands.some(a=>j6(a==null?void 0:a.brand,t)))return!0}catch{}return!!e.webdriver},Wc=function(e){return e.US="us",e.EU="eu",e.CUSTOM="custom",e}({}),M5="i.posthog.com";class MV{constructor(t){this.lr={},this.instance=t}get apiHost(){var t=this.instance.config.api_host.trim().replace(/\/$/,"");return t==="https://app.posthog.com"?"https://us.i.posthog.com":t}get flagsApiHost(){var t=this.instance.config.flags_api_host;return t?t.trim().replace(/\/$/,""):this.apiHost}get uiHost(){var t,n=(t=this.instance.config.ui_host)==null?void 0:t.replace(/\/$/,"");return n||(n=this.apiHost.replace("."+M5,".posthog.com")),n==="https://app.posthog.com"?"https://us.posthog.com":n}get region(){return this.lr[this.apiHost]||(/https:\/\/(app|us|us-assets)(\.i)?\.posthog\.com/i.test(this.apiHost)?this.lr[this.apiHost]=Wc.US:/https:\/\/(eu|eu-assets)(\.i)?\.posthog\.com/i.test(this.apiHost)?this.lr[this.apiHost]=Wc.EU:this.lr[this.apiHost]=Wc.CUSTOM),this.lr[this.apiHost]}endpointFor(t,n){if(n===void 0&&(n=""),n&&(n=n[0]==="/"?n:"/"+n),t==="ui")return this.uiHost+n;if(t==="flags")return this.flagsApiHost+n;if(this.region===Wc.CUSTOM)return this.apiHost+n;var r=M5+n;switch(t){case"assets":return"https://"+this.region+"-assets."+r;case"api":return"https://"+this.region+"."+r}}}var IV={icontains:(e,t)=>!!be&&t.href.toLowerCase().indexOf(e.toLowerCase())>-1,not_icontains:(e,t)=>!!be&&t.href.toLowerCase().indexOf(e.toLowerCase())===-1,regex:(e,t)=>!!be&&Rf(t.href,e),not_regex:(e,t)=>!!be&&!Rf(t.href,e),exact:(e,t)=>t.href===e,is_not:(e,t)=>t.href!==e};class wr{constructor(t){var n=this;this.getWebExperimentsAndEvaluateDisplayLogic=function(r){r===void 0&&(r=!1),n.getWebExperiments(a=>{wr.ur("retrieved web experiments from the server"),n.hr=new Map,a.forEach(s=>{if(s.feature_flag_key){var o;n.hr&&(wr.ur("setting flag key ",s.feature_flag_key," to web experiment ",s),(o=n.hr)==null||o.set(s.feature_flag_key,s));var c=n._instance.getFeatureFlag(s.feature_flag_key);ir(c)&&s.variants[c]&&n.dr(s.name,c,s.variants[c].transforms)}else if(s.variants)for(var d in s.variants){var f=s.variants[d];wr.vr(f)&&n.dr(s.name,d,f.transforms)}})},r)},this._instance=t,this._instance.onFeatureFlags(r=>{this.onFeatureFlags(r)})}onFeatureFlags(t){if(this._is_bot())wr.ur("Refusing to render web experiment since the viewer is a likely bot");else if(!this._instance.config.disable_web_experiments){if(Gt(this.hr))return this.hr=new Map,this.loadIfEnabled(),void this.previewWebExperiment();wr.ur("applying feature flags",t),t.forEach(n=>{var r;if(this.hr&&(r=this.hr)!=null&&r.has(n)){var a,s=this._instance.getFeatureFlag(n),o=(a=this.hr)==null?void 0:a.get(n);s&&o!=null&&o.variants[s]&&this.dr(o.name,s,o.variants[s].transforms)}})}}previewWebExperiment(){var t=wr.getWindowLocation();if(t!=null&&t.search){var n=wf(t==null?void 0:t.search,"__experiment_id"),r=wf(t==null?void 0:t.search,"__experiment_variant");n&&r&&(wr.ur("previewing web experiments "+n+" && "+r),this.getWebExperiments(a=>{this.cr(parseInt(n),r,a)},!1,!0))}}loadIfEnabled(){this._instance.config.disable_web_experiments||this.getWebExperimentsAndEvaluateDisplayLogic()}getWebExperiments(t,n,r){if(this._instance.config.disable_web_experiments&&!r)return t([]);var a=this._instance.get_property("$web_experiments");if(a&&!n)return t(a);this._instance.ci({url:this._instance.requestRouter.endpointFor("api","/api/web_experiments/?token="+this._instance.config.token),method:"GET",callback:s=>{if(s.statusCode!==200||!s.json)return t([]);var o=s.json.experiments||[];return t(o)}})}cr(t,n,r){var a=r.filter(s=>s.id===t);a&&a.length>0&&(wr.ur("Previewing web experiment ["+a[0].name+"] with variant ["+n+"]"),this.dr(a[0].name,n,a[0].variants[n].transforms))}static vr(t){return!Gt(t.conditions)&&wr.pr(t)&&wr.gr(t)}static pr(t){var n;if(Gt(t.conditions)||Gt((n=t.conditions)==null?void 0:n.url))return!0;var r,a,s,o=wr.getWindowLocation();return!!o&&((r=t.conditions)==null||!r.url||IV[(a=(s=t.conditions)==null?void 0:s.urlMatchType)!==null&&a!==void 0?a:"icontains"](t.conditions.url,o))}static getWindowLocation(){return be==null?void 0:be.location}static gr(t){var n;if(Gt(t.conditions)||Gt((n=t.conditions)==null?void 0:n.utm))return!0;var r=a9();if(r.utm_source){var a,s,o,c,d,f,m,p,b=(a=t.conditions)==null||(a=a.utm)==null||!a.utm_campaign||((s=t.conditions)==null||(s=s.utm)==null?void 0:s.utm_campaign)==r.utm_campaign,y=(o=t.conditions)==null||(o=o.utm)==null||!o.utm_source||((c=t.conditions)==null||(c=c.utm)==null?void 0:c.utm_source)==r.utm_source,T=(d=t.conditions)==null||(d=d.utm)==null||!d.utm_medium||((f=t.conditions)==null||(f=f.utm)==null?void 0:f.utm_medium)==r.utm_medium,x=(m=t.conditions)==null||(m=m.utm)==null||!m.utm_term||((p=t.conditions)==null||(p=p.utm)==null?void 0:p.utm_term)==r.utm_term;return b&&T&&x&&y}return!1}static ur(t){for(var n=arguments.length,r=new Array(n>1?n-1:0),a=1;a<n;a++)r[a-1]=arguments[a];Ge.info("[WebExperiments] "+t,r)}dr(t,n,r){this._is_bot()?wr.ur("Refusing to render web experiment since the viewer is a likely bot"):n!=="control"?r.forEach(a=>{if(a.selector){var s;wr.ur("applying transform of variant "+n+" for experiment "+t+" ",a);var o=(s=document)==null?void 0:s.querySelectorAll(a.selector);o==null||o.forEach(c=>{var d=c;a.html&&(d.innerHTML=a.html),a.css&&d.setAttribute("style",a.css)})}}):wr.ur("Control variants leave the page unmodified.")}_is_bot(){return ya&&this._instance?y9(ya,this._instance.config.custom_blocked_useragents):void 0}}var DV=Zn("[PostHog ExternalIntegrations]"),LV={intercom:"intercom-integration",crispChat:"crisp-chat-integration"};class PV{constructor(t){this._instance=t}it(t,n){var r;(r=dt.__PosthogExtensions__)==null||r.loadExternalDependency==null||r.loadExternalDependency(this._instance,t,a=>{if(a)return DV.error("failed to load script",a);n()})}startIfEnabledOrStop(){var t=this,n=function(o){var c,d,f;!a||(c=dt.__PosthogExtensions__)!=null&&(c=c.integrations)!=null&&c[o]||t.it(LV[o],()=>{var m;(m=dt.__PosthogExtensions__)==null||(m=m.integrations)==null||(m=m[o])==null||m.start(t._instance)}),!a&&(d=dt.__PosthogExtensions__)!=null&&(d=d.integrations)!=null&&d[o]&&((f=dt.__PosthogExtensions__)==null||(f=f.integrations)==null||(f=f[o])==null||f.stop())};for(var[r,a]of Object.entries((s=this._instance.config.integrations)!==null&&s!==void 0?s:{})){var s;n(r)}}}var Wb="[SessionRecording]",Fc=Zn(Wb);class I5{get started(){var t;return!((t=this._r)==null||!t.isStarted)}get status(){return this._r?this._r.status:this.mr&&!this.yr?"disabled":"lazy_loading"}constructor(t){if(this._forceAllowLocalhostNetworkCapture=!1,this.mr=!1,this.br=void 0,this._instance=t,!this._instance.sessionManager)throw Fc.error("started without valid sessionManager"),new Error(Wb+" started without valid sessionManager. This is a bug.");if(this._instance.config.cookieless_mode==="always")throw new Error(Wb+' cannot be used with cookieless_mode="always"')}get yr(){var t,n=!((t=this._instance.get_property(_g))==null||!t.enabled),r=!this._instance.config.disable_session_recording,a=this._instance.config.disable_session_recording||this._instance.consent.isOptedOut();return be&&n&&r&&!a}startIfEnabledOrStop(t){var n;if(!this.yr||(n=this._r)==null||!n.isStarted){var r=!qe(Object.assign)&&!qe(Array.from);this.yr&&r?(this.wr(t),Fc.info("starting")):this.stopRecording()}}wr(t){var n,r,a;this.yr&&(dt!=null&&(n=dt.__PosthogExtensions__)!=null&&(n=n.rrweb)!=null&&n.record&&(r=dt.__PosthogExtensions__)!=null&&r.initSessionRecording?this.Er(t):(a=dt.__PosthogExtensions__)==null||a.loadExternalDependency==null||a.loadExternalDependency(this._instance,this.Sr,s=>{if(s)return Fc.error("could not load recorder",s);this.Er(t)}))}stopRecording(){var t,n;(t=this.br)==null||t.call(this),this.br=void 0,(n=this._r)==null||n.stop()}$r(){var t;(t=this._instance.persistence)==null||t.unregister(xS)}kr(t){if(this._instance.persistence){var n,r,a=this._instance.persistence,s=()=>{var o=t.sessionRecording===!1?void 0:t.sessionRecording,c=o==null?void 0:o.sampleRate,d=Gt(c)?null:parseFloat(c);Gt(d)&&this.$r();var f=o==null?void 0:o.minimumDurationMilliseconds;a.register({[_g]:Xe({enabled:!!o},o,{networkPayloadCapture:Xe({capturePerformance:t.capturePerformance},o==null?void 0:o.networkPayloadCapture),canvasRecording:{enabled:o==null?void 0:o.recordCanvas,fps:o==null?void 0:o.canvasFps,quality:o==null?void 0:o.canvasQuality},sampleRate:d,minimumDurationMilliseconds:qe(f)?null:f,endpoint:o==null?void 0:o.endpoint,triggerMatchType:o==null?void 0:o.triggerMatchType,masking:o==null?void 0:o.masking,urlTriggers:o==null?void 0:o.urlTriggers})})};s(),(n=this.br)==null||n.call(this),this.br=(r=this._instance.sessionManager)==null?void 0:r.onSessionId(s)}}onRemoteConfig(t){"sessionRecording"in t?t.sessionRecording!==!1?(this.kr(t),this.mr=!0,this.startIfEnabledOrStop()):this.mr=!0:Fc.info("skipping remote config with no sessionRecording",t)}log(t,n){var r;n===void 0&&(n="log"),(r=this._r)!=null&&r.log?this._r.log(t,n):Fc.warn("log called before recorder was ready")}get Sr(){var t,n,r=(t=this._instance)==null||(t=t.persistence)==null?void 0:t.get_property(_g);return(r==null||(n=r.scriptConfig)==null?void 0:n.script)||"lazy-recorder"}Er(t){var n,r;if((n=dt.__PosthogExtensions__)==null||!n.initSessionRecording)throw Error("Called on script loaded before session recording is available");this._r||(this._r=(r=dt.__PosthogExtensions__)==null?void 0:r.initSessionRecording(this._instance),this._r._forceAllowLocalhostNetworkCapture=this._forceAllowLocalhostNetworkCapture),this._r.start(t)}onRRwebEmit(t){var n;(n=this._r)==null||n.onRRwebEmit==null||n.onRRwebEmit(t)}overrideLinkedFlag(){var t,n;this._r||(n=this._instance.persistence)==null||n.register({$replay_override_linked_flag:!0}),(t=this._r)==null||t.overrideLinkedFlag()}overrideSampling(){var t,n;this._r||(n=this._instance.persistence)==null||n.register({$replay_override_sampling:!0}),(t=this._r)==null||t.overrideSampling()}overrideTrigger(t){var n,r;this._r||(r=this._instance.persistence)==null||r.register({[t==="url"?"$replay_override_url_trigger":"$replay_override_event_trigger"]:!0}),(n=this._r)==null||n.overrideTrigger(t)}get sdkDebugProperties(){var t;return((t=this._r)==null?void 0:t.sdkDebugProperties)||{$recording_status:this.status}}tryAddCustomEvent(t,n){var r;return!((r=this._r)==null||!r.tryAddCustomEvent(t,n))}}var o0={},Xb=()=>{},Yo="posthog",_9=!cV&&(ea==null?void 0:ea.indexOf("MSIE"))===-1&&(ea==null?void 0:ea.indexOf("Mozilla"))===-1,D5=e=>{var t;return Xe({api_host:"https://us.i.posthog.com",flags_api_host:null,ui_host:null,token:"",autocapture:!0,cross_subdomain_cookie:aG($e==null?void 0:$e.location),persistence:"localStorage+cookie",persistence_name:"",loaded:Xb,save_campaign_params:!0,custom_campaign_params:[],custom_blocked_useragents:[],save_referrer:!0,capture_pageleave:"if_capture_pageview",defaults:e??"unset",__preview_deferred_init_extensions:!1,debug:ta&&ir(ta==null?void 0:ta.search)&&ta.search.indexOf("__posthog_debug=true")!==-1||!1,cookie_expiration:365,upgrade:!1,disable_session_recording:!1,disable_persistence:!1,disable_web_experiments:!0,disable_surveys:!1,disable_surveys_automatic_display:!1,disable_product_tours:!0,disable_external_dependency_loading:!1,enable_recording_console_log:void 0,secure_cookie:(be==null||(t=be.location)==null?void 0:t.protocol)==="https:",ip:!1,opt_out_capturing_by_default:!1,opt_out_persistence_by_default:!1,opt_out_useragent_filter:!1,opt_out_capturing_persistence_type:"localStorage",consent_persistence_name:null,opt_out_capturing_cookie_prefix:null,opt_in_site_apps:!1,property_denylist:[],respect_dnt:!1,sanitize_properties:null,request_headers:{},request_batching:!0,properties_string_max_length:65535,mask_all_element_attributes:!1,mask_all_text:!1,mask_personal_data_properties:!1,custom_personal_data_properties:[],advanced_disable_flags:!1,advanced_disable_decide:!1,advanced_disable_feature_flags:!1,advanced_disable_feature_flags_on_first_load:!1,advanced_only_evaluate_survey_feature_flags:!1,advanced_enable_surveys:!1,advanced_disable_toolbar_metrics:!1,feature_flag_request_timeout_ms:3e3,surveys_request_timeout_ms:1e4,on_request_error:n=>{var r="Bad HTTP status: "+n.statusCode+" "+n.text;Ge.error(r)},get_device_id:n=>n,capture_performance:void 0,name:"posthog",bootstrap:{},disable_compression:!1,session_idle_timeout_seconds:1800,person_profiles:"identified_only",before_send:void 0,request_queue_config:{flush_interval_ms:Kb},error_tracking:{},_onCapture:Xb,__preview_eager_load_replay:!1},(n=>({rageclick:!(n&&n>="2025-11-30")||{content_ignorelist:!0},capture_pageview:!(n&&n>="2025-05-24")||"history_change",session_recording:n&&n>="2025-11-30"?{strictMinimumDuration:!0}:{}}))(e))},L5=e=>{var t={};qe(e.process_person)||(t.person_profiles=e.process_person),qe(e.xhr_headers)||(t.request_headers=e.xhr_headers),qe(e.cookie_name)||(t.persistence_name=e.cookie_name),qe(e.disable_cookie)||(t.disable_persistence=e.disable_cookie),qe(e.store_google)||(t.save_campaign_params=e.store_google),qe(e.verbose)||(t.debug=e.verbose);var n=Fn({},t,e);return Vt(e.property_blacklist)&&(qe(e.property_denylist)?n.property_denylist=e.property_blacklist:Vt(e.property_denylist)?n.property_denylist=[...e.property_blacklist,...e.property_denylist]:Ge.error("Invalid value for property_denylist config: "+e.property_denylist)),n};class BV{constructor(){this.__forceAllowLocalhost=!1}get Pr(){return this.__forceAllowLocalhost}set Pr(t){Ge.error("WebPerformanceObserver is deprecated and has no impact on network capture. Use `_forceAllowLocalhostNetworkCapture` on `posthog.sessionRecording`"),this.__forceAllowLocalhost=t}}class um{get decideEndpointWasHit(){var t,n;return(t=(n=this.featureFlags)==null?void 0:n.hasLoadedFlags)!==null&&t!==void 0&&t}get flagsEndpointWasHit(){var t,n;return(t=(n=this.featureFlags)==null?void 0:n.hasLoadedFlags)!==null&&t!==void 0&&t}constructor(){this.webPerformance=new BV,this.Tr=!1,this.version=Yi.LIB_VERSION,this.Rr=new mv,this._calculate_event_properties=this.calculateEventProperties.bind(this),this.config=D5(),this.SentryIntegration=IG,this.sentryIntegration=t=>function(n,r){var a=HS(n,r);return{name:zS,processEvent:s=>a(s)}}(this,t),this.__request_queue=[],this.__loaded=!1,this.analyticsDefaultEndpoint="/e/",this.Cr=!1,this.Ir=null,this.Fr=null,this.Mr=null,this.featureFlags=new mV(this),this.toolbar=new LG(this),this.scrollManager=new CV(this),this.pageViewManager=new tV(this),this.surveys=new TV(this),this.productTours=new gV(this),this.experiments=new wr(this),this.exceptions=new dV(this),this.rateLimiter=new xV(this),this.requestRouter=new MV(this),this.consent=new kG(this),this.externalIntegrations=new PV(this),this.people={set:(t,n,r)=>{var a=ir(t)?{[t]:n}:t;this.setPersonProperties(a),r==null||r({})},set_once:(t,n,r)=>{var a=ir(t)?{[t]:n}:t;this.setPersonProperties(void 0,a),r==null||r({})}},this.on("eventCaptured",t=>Ge.info('send "'+(t==null?void 0:t.event)+'"',t))}init(t,n,r){if(r&&r!==Yo){var a,s=(a=o0[r])!==null&&a!==void 0?a:new um;return s._init(t,n,r),o0[r]=s,o0[Yo][r]=s,s}return this._init(t,n,r)}_init(t,n,r){var a;if(n===void 0&&(n={}),qe(t)||Nb(t))return Ge.critical("PostHog was initialized without a token. This likely indicates a misconfiguration. Please check the first argument passed to posthog.init()"),this;if(this.__loaded)return console.warn("[PostHog.js]","You have already initialized PostHog! Re-initializing is a no-op"),this;this.__loaded=!0,this.config={},n.debug=this.Or(n.debug),this.Ar=n,this.Dr=[],n.person_profiles&&(this.Fr=n.person_profiles),this.set_config(Fn({},D5(n.defaults),L5(n),{name:r,token:t})),this.config.on_xhr_error&&Ge.error("on_xhr_error is deprecated. Use on_request_error instead"),this.compression=n.disable_compression?void 0:Qi.GZipJS;var s=this.jr();this.persistence=new Rg(this.config,s),this.sessionPersistence=this.config.persistence==="sessionStorage"||this.config.persistence==="memory"?this.persistence:new Rg(Xe({},this.config,{persistence:"sessionStorage"}),s);var o=Xe({},this.persistence.props),c=Xe({},this.sessionPersistence.props);this.register({$initialization_time:new Date().toISOString()}),this.Lr=new wV(O=>this.Nr(O),this.config.request_queue_config),this.Ur=new kV(this),this.__request_queue=[];var d=this.config.cookieless_mode==="always"||this.config.cookieless_mode==="on_reject"&&this.consent.isExplicitlyOptedOut();if(d||(this.sessionManager=new R5(this),this.sessionPropsManager=new O5(this,this.sessionManager,this.persistence)),this.config.__preview_deferred_init_extensions?(Ge.info("Deferring extension initialization to improve startup performance"),setTimeout(()=>{this.zr(d)},0)):(Ge.info("Initializing extensions synchronously"),this.zr(d)),Yi.DEBUG=Yi.DEBUG||this.config.debug,Yi.DEBUG&&Ge.info("Starting in debug mode",{this:this,config:n,thisC:Xe({},this.config),p:o,s:c}),((a=n.bootstrap)==null?void 0:a.distinctID)!==void 0){var f,m,p=this.config.get_device_id(el()),b=(f=n.bootstrap)!=null&&f.isIdentifiedID?p:n.bootstrap.distinctID;this.persistence.set_property(Gi,(m=n.bootstrap)!=null&&m.isIdentifiedID?"identified":"anonymous"),this.register({distinct_id:n.bootstrap.distinctID,$device_id:b})}if(this.Hr()){var y,T,x=Object.keys(((y=n.bootstrap)==null?void 0:y.featureFlags)||{}).filter(O=>{var D;return!((D=n.bootstrap)==null||(D=D.featureFlags)==null||!D[O])}).reduce((O,D)=>{var Y;return O[D]=((Y=n.bootstrap)==null||(Y=Y.featureFlags)==null?void 0:Y[D])||!1,O},{}),C=Object.keys(((T=n.bootstrap)==null?void 0:T.featureFlagPayloads)||{}).filter(O=>x[O]).reduce((O,D)=>{var Y,H;return(Y=n.bootstrap)!=null&&(Y=Y.featureFlagPayloads)!=null&&Y[D]&&(O[D]=(H=n.bootstrap)==null||(H=H.featureFlagPayloads)==null?void 0:H[D]),O},{});this.featureFlags.receivedFeatureFlags({featureFlags:x,featureFlagPayloads:C})}if(d)this.register_once({distinct_id:Rc,$device_id:null},"");else if(!this.get_distinct_id()){var w=this.config.get_device_id(el());this.register_once({distinct_id:w,$device_id:w},""),this.persistence.set_property(Gi,"anonymous")}return Xn(be,"onpagehide"in self?"pagehide":"unload",this._handle_unload.bind(this),{passive:!1}),this.toolbar.maybeLoadToolbar(),n.segment?MG(this,()=>this.Br()):this.Br(),tl(this.config._onCapture)&&this.config._onCapture!==Xb&&(Ge.warn("onCapture is deprecated. Please use `before_send` instead"),this.on("eventCaptured",O=>this.config._onCapture(O.event,O))),this.config.ip&&Ge.warn('The `ip` config option has NO EFFECT AT ALL and has been deprecated. Use a custom transformation or "Discard IP data" project setting instead. See https://posthog.com/tutorials/web-redact-properties#hiding-customer-ip-address for more information.'),this}zr(t){var n=performance.now();this.historyAutocapture=new RG(this),this.historyAutocapture.startIfEnabled();var r=[];r.push(()=>{new BG(this).startIfEnabledOrStop()}),r.push(()=>{var a;this.siteApps=new RV(this),(a=this.siteApps)==null||a.init()}),t||r.push(()=>{this.sessionRecording=new I5(this),this.sessionRecording.startIfEnabledOrStop()}),this.config.disable_scroll_properties||r.push(()=>{this.scrollManager.startMeasuringScrollPosition()}),r.push(()=>{this.autocapture=new yG(this),this.autocapture.startIfEnabled()}),r.push(()=>{this.surveys.loadIfEnabled()}),r.push(()=>{this.productTours.loadIfEnabled()}),r.push(()=>{this.heatmaps=new eV(this),this.heatmaps.startIfEnabled()}),r.push(()=>{this.webVitalsAutocapture=new ZG(this)}),r.push(()=>{this.exceptionObserver=new OG(this),this.exceptionObserver.startIfEnabled()}),r.push(()=>{this.deadClicksAutocapture=new FS(this,NG),this.deadClicksAutocapture.startIfEnabled()}),r.push(()=>{if(this.qr){var a=this.qr;this.qr=void 0,this.fi(a)}}),this.Wr(r,n)}Wr(t,n){for(;t.length>0;){if(this.config.__preview_deferred_init_extensions&&performance.now()-n>=30&&t.length>0)return void setTimeout(()=>{this.Wr(t,n)},0);var r=t.shift();if(r)try{r()}catch(s){Ge.error("Error initializing extension:",s)}}var a=Math.round(performance.now()-n);this.register_for_session({$sdk_debug_extensions_init_method:this.config.__preview_deferred_init_extensions?"deferred":"synchronous",$sdk_debug_extensions_init_time_ms:a}),this.config.__preview_deferred_init_extensions&&Ge.info("PostHog extensions initialized ("+a+"ms)")}fi(t){var n,r,a,s,o,c,d,f;if(!$e||!$e.body)return Ge.info("document not ready yet, trying again in 500 milliseconds..."),void setTimeout(()=>{this.fi(t)},500);this.config.__preview_deferred_init_extensions&&(this.qr=t),this.compression=void 0,t.supportedCompression&&!this.config.disable_compression&&(this.compression=It(t.supportedCompression,Qi.GZipJS)?Qi.GZipJS:It(t.supportedCompression,Qi.Base64)?Qi.Base64:void 0),(n=t.analytics)!=null&&n.endpoint&&(this.analyticsDefaultEndpoint=t.analytics.endpoint),this.set_config({person_profiles:this.Fr?this.Fr:"identified_only"}),(r=this.siteApps)==null||r.onRemoteConfig(t),(a=this.sessionRecording)==null||a.onRemoteConfig(t),(s=this.autocapture)==null||s.onRemoteConfig(t),(o=this.heatmaps)==null||o.onRemoteConfig(t),this.surveys.onRemoteConfig(t),this.productTours.onRemoteConfig(t),(c=this.webVitalsAutocapture)==null||c.onRemoteConfig(t),(d=this.exceptionObserver)==null||d.onRemoteConfig(t),this.exceptions.onRemoteConfig(t),(f=this.deadClicksAutocapture)==null||f.onRemoteConfig(t)}Br(){try{this.config.loaded(this)}catch(t){Ge.critical("`loaded` function failed",t)}this.Gr(),this.config.capture_pageview&&setTimeout(()=>{(this.consent.isOptedIn()||this.config.cookieless_mode==="always")&&this.Vr()},1),new SV(this).load(),this.featureFlags.flags()}Gr(){var t;this.is_capturing()&&this.config.request_batching&&((t=this.Lr)==null||t.enable())}_dom_loaded(){this.is_capturing()&&rl(this.__request_queue,t=>this.Nr(t)),this.__request_queue=[],this.Gr()}_handle_unload(){var t,n;this.config.request_batching?(this.Jr()&&this.capture("$pageleave"),(t=this.Lr)==null||t.unload(),(n=this.Ur)==null||n.unload()):this.Jr()&&this.capture("$pageleave",null,{transport:"sendBeacon"})}ci(t){this.__loaded&&(_9?this.__request_queue.push(t):this.rateLimiter.isServerRateLimited(t.batchKey)||(t.transport=t.transport||this.config.api_transport,t.url=Of(t.url,{ip:this.config.ip?1:0}),t.headers=Xe({},this.config.request_headers),t.compression=t.compression==="best-available"?this.compression:t.compression,t.disableXHRCredentials=this.config.__preview_disable_xhr_credentials,this.config.__preview_disable_beacon&&(t.disableTransport=["sendBeacon"]),t.fetchOptions=t.fetchOptions||this.config.fetch_options,(n=>{var r,a,s,o=Xe({},n);o.timeout=o.timeout||6e4,o.url=Of(o.url,{_:new Date().getTime().toString(),ver:Yi.LIB_VERSION,compression:o.compression});var c=(r=o.transport)!==null&&r!==void 0?r:"fetch",d=Vh.filter(m=>!o.disableTransport||!m.transport||!o.disableTransport.includes(m.transport)),f=(a=(s=_S(d,m=>m.transport===c))==null?void 0:s.method)!==null&&a!==void 0?a:d[0].method;if(!f)throw new Error("No available transport method");f(o)})(Xe({},t,{callback:n=>{var r,a;this.rateLimiter.checkForLimiting(n),n.statusCode>=400&&((r=(a=this.config).on_request_error)==null||r.call(a,n)),t.callback==null||t.callback(n)}}))))}Nr(t){this.Ur?this.Ur.retriableRequest(t):this.ci(t)}_execute_array(t){var n,r=[],a=[],s=[];rl(t,c=>{c&&(n=c[0],Vt(n)?s.push(c):tl(c)?c.call(this):Vt(c)&&n==="alias"?r.push(c):Vt(c)&&n.indexOf("capture")!==-1&&tl(this[n])?s.push(c):a.push(c))});var o=function(c,d){rl(c,function(f){if(Vt(f[0])){var m=d;hn(f,function(p){m=m[p[0]].apply(m,p.slice(1))})}else this[f[0]].apply(this,f.slice(1))},d)};o(r,this),o(a,this),o(s,this)}Hr(){var t,n;return((t=this.config.bootstrap)==null?void 0:t.featureFlags)&&Object.keys((n=this.config.bootstrap)==null?void 0:n.featureFlags).length>0||!1}push(t){this._execute_array([t])}capture(t,n,r){var a;if(this.__loaded&&this.persistence&&this.sessionPersistence&&this.Lr){if(this.is_capturing())if(!qe(t)&&ir(t)){var s=!this.config.opt_out_useragent_filter&&this._is_bot();if(!(s&&!this.config.__preview_capture_bot_pageviews)){var o=r!=null&&r.skip_client_rate_limiting?void 0:this.rateLimiter.clientRateLimitContext();if(o==null||!o.isRateLimited){n!=null&&n.$current_url&&!ir(n==null?void 0:n.$current_url)&&(Ge.error("Invalid `$current_url` property provided to `posthog.capture`. Input must be a string. Ignoring provided value."),n==null||delete n.$current_url),this.sessionPersistence.update_search_keyword(),this.config.save_campaign_params&&this.sessionPersistence.update_campaign_params(),this.config.save_referrer&&this.sessionPersistence.update_referrer_info(),(this.config.save_campaign_params||this.config.save_referrer)&&this.persistence.set_initial_person_info();var c=new Date,d=(r==null?void 0:r.timestamp)||c,f=el(),m={uuid:f,event:t,properties:this.calculateEventProperties(t,n||{},d,f)};t==="$pageview"&&this.config.__preview_capture_bot_pageviews&&s&&(m.event="$bot_pageview",m.properties.$browser_type="bot"),o&&(m.properties.$lib_rate_limit_remaining_tokens=o.remainingTokens),r!=null&&r.$set&&(m.$set=r==null?void 0:r.$set);var p,b=t!=="$groupidentify",y=this.Kr(r==null?void 0:r.$set_once,b);if(y&&(m.$set_once=y),(m=nG(m,r!=null&&r._noTruncate?null:this.config.properties_string_max_length)).timestamp=d,qe(r==null?void 0:r.timestamp)||(m.properties.$event_time_override_provided=!0,m.properties.$event_time_override_system_time=c),t===Kh.DISMISSED||t===Kh.SENT){var T=n==null?void 0:n[k5.SURVEY_ID],x=n==null?void 0:n[k5.SURVEY_ITERATION];p={id:T,current_iteration:x},localStorage.getItem(C5(p))||localStorage.setItem(C5(p),"true"),m.$set=Xe({},m.$set,{[vV({id:T,current_iteration:x},t===Kh.SENT?"responded":"dismissed")]:!0})}var C=Xe({},m.properties.$set,m.$set);if(Vo(C)||this.setPersonPropertiesForFlags(C),!Gt(this.config.before_send)){var w=this.Yr(m);if(!w)return;m=w}this.Rr.emit("eventCaptured",m);var O={method:"POST",url:(a=r==null?void 0:r._url)!==null&&a!==void 0?a:this.requestRouter.endpointFor("api",this.analyticsDefaultEndpoint),data:m,compression:"best-available",batchKey:r==null?void 0:r._batchKey};return!this.config.request_batching||r&&(r==null||!r._batchKey)||r!=null&&r.send_instantly?this.Nr(O):this.Lr.enqueue(O),m}Ge.critical("This capture call is ignored due to client rate limiting.")}}else Ge.error("No event name provided to posthog.capture")}else Ge.uninitializedWarning("posthog.capture")}Ui(t){return this.on("eventCaptured",n=>t(n.event,n))}calculateEventProperties(t,n,r,a,s){if(r=r||new Date,!this.persistence||!this.sessionPersistence)return n;var o=s?void 0:this.persistence.remove_event_timer(t),c=Xe({},n);if(c.token=this.config.token,c.$config_defaults=this.config.defaults,(this.config.cookieless_mode=="always"||this.config.cookieless_mode=="on_reject"&&this.consent.isExplicitlyOptedOut())&&(c.$cookieless_mode=!0),t==="$snapshot"){var d=Xe({},this.persistence.properties(),this.sessionPersistence.properties());return c.distinct_id=d.distinct_id,(!ir(c.distinct_id)&&!bi(c.distinct_id)||Nb(c.distinct_id))&&Ge.error("Invalid distinct_id for replay event. This indicates a bug in your implementation"),c}var f,m=QG(this.config.mask_personal_data_properties,this.config.custom_personal_data_properties);if(this.sessionManager){var{sessionId:p,windowId:b}=this.sessionManager.checkAndGetSessionAndWindowId(s,r.getTime());c.$session_id=p,c.$window_id=b}this.sessionPropsManager&&Fn(c,this.sessionPropsManager.getSessionProps());try{var y;this.sessionRecording&&Fn(c,this.sessionRecording.sdkDebugProperties),c.$sdk_debug_retry_queue_size=(y=this.Ur)==null?void 0:y.length}catch(w){c.$sdk_debug_error_capturing_properties=String(w)}if(this.requestRouter.region===Wc.CUSTOM&&(c.$lib_custom_api_host=this.config.api_host),f=t!=="$pageview"||s?t!=="$pageleave"||s?this.pageViewManager.doEvent():this.pageViewManager.doPageLeave(r):this.pageViewManager.doPageView(r,a),c=Fn(c,f),t==="$pageview"&&$e&&(c.title=$e.title),!qe(o)){var T=r.getTime()-o;c.$duration=parseFloat((T/1e3).toFixed(3))}ea&&this.config.opt_out_useragent_filter&&(c.$browser_type=this._is_bot()?"bot":"browser"),(c=Fn({},m,this.persistence.properties(),this.sessionPersistence.properties(),c)).$is_identified=this._isIdentified(),Vt(this.config.property_denylist)?hn(this.config.property_denylist,function(w){delete c[w]}):Ge.error("Invalid value for property_denylist config: "+this.config.property_denylist+" or property_blacklist config: "+this.config.property_blacklist);var x=this.config.sanitize_properties;x&&(Ge.error("sanitize_properties is deprecated. Use before_send instead"),c=x(c,t));var C=this.Xr();return c.$process_person_profile=C,C&&!s&&this.Qr("_calculate_event_properties"),c}Kr(t,n){var r;if(n===void 0&&(n=!0),!this.persistence||!this.Xr()||this.Tr)return t;var a=this.persistence.get_initial_props(),s=(r=this.sessionPropsManager)==null?void 0:r.getSetOnceProps(),o=Fn({},a,s||{},t||{}),c=this.config.sanitize_properties;return c&&(Ge.error("sanitize_properties is deprecated. Use before_send instead"),o=c(o,"$set_once")),n&&(this.Tr=!0),Vo(o)?void 0:o}register(t,n){var r;(r=this.persistence)==null||r.register(t,n)}register_once(t,n,r){var a;(a=this.persistence)==null||a.register_once(t,n,r)}register_for_session(t){var n;(n=this.sessionPersistence)==null||n.register(t)}unregister(t){var n;(n=this.persistence)==null||n.unregister(t)}unregister_for_session(t){var n;(n=this.sessionPersistence)==null||n.unregister(t)}Zr(t,n){this.register({[t]:n})}getFeatureFlag(t,n){return this.featureFlags.getFeatureFlag(t,n)}getFeatureFlagPayload(t){var n=this.featureFlags.getFeatureFlagPayload(t);try{return JSON.parse(n)}catch{return n}}isFeatureEnabled(t,n){return this.featureFlags.isFeatureEnabled(t,n)}reloadFeatureFlags(){this.featureFlags.reloadFeatureFlags()}updateEarlyAccessFeatureEnrollment(t,n,r){this.featureFlags.updateEarlyAccessFeatureEnrollment(t,n,r)}getEarlyAccessFeatures(t,n,r){return n===void 0&&(n=!1),this.featureFlags.getEarlyAccessFeatures(t,n,r)}on(t,n){return this.Rr.on(t,n)}onFeatureFlags(t){return this.featureFlags.onFeatureFlags(t)}onSurveysLoaded(t){return this.surveys.onSurveysLoaded(t)}onSessionId(t){var n,r;return(n=(r=this.sessionManager)==null?void 0:r.onSessionId(t))!==null&&n!==void 0?n:()=>{}}getSurveys(t,n){n===void 0&&(n=!1),this.surveys.getSurveys(t,n)}getActiveMatchingSurveys(t,n){n===void 0&&(n=!1),this.surveys.getActiveMatchingSurveys(t,n)}renderSurvey(t,n){this.surveys.renderSurvey(t,n)}displaySurvey(t,n){n===void 0&&(n=_V),this.surveys.displaySurvey(t,n)}cancelPendingSurvey(t){this.surveys.cancelPendingSurvey(t)}canRenderSurvey(t){return this.surveys.canRenderSurvey(t)}canRenderSurveyAsync(t,n){return n===void 0&&(n=!1),this.surveys.canRenderSurveyAsync(t,n)}identify(t,n,r){if(!this.__loaded||!this.persistence)return Ge.uninitializedWarning("posthog.identify");if(bi(t)&&(t=t.toString(),Ge.warn("The first argument to posthog.identify was a number, but it should be a string. It has been converted to a string.")),t)if(["distinct_id","distinctid"].includes(t.toLowerCase()))Ge.critical('The string "'+t+'" was set in posthog.identify which indicates an error. This ID should be unique to the user and not a hardcoded string.');else if(t!==Rc){if(this.Qr("posthog.identify")){var a=this.get_distinct_id();if(this.register({$user_id:t}),!this.get_property("$device_id")){var s=a;this.register_once({$had_persisted_distinct_id:!0,$device_id:s},"")}t!==a&&t!==this.get_property(jc)&&(this.unregister(jc),this.register({distinct_id:t}));var o=(this.persistence.get_property(Gi)||"anonymous")==="anonymous";t!==a&&o?(this.persistence.set_property(Gi,"identified"),this.setPersonPropertiesForFlags(Xe({},r||{},n||{}),!1),this.capture("$identify",{distinct_id:t,$anon_distinct_id:a},{$set:n||{},$set_once:r||{}}),this.Mr=T5(t,n,r),this.featureFlags.setAnonymousDistinctId(a)):(n||r)&&this.setPersonProperties(n,r),t!==a&&(this.reloadFeatureFlags(),this.unregister(_f))}}else Ge.critical('The string "'+Rc+'" was set in posthog.identify which indicates an error. This ID is only used as a sentinel value.');else Ge.error("Unique user id has not been set in posthog.identify")}setPersonProperties(t,n){if((t||n)&&this.Qr("posthog.setPersonProperties")){var r=T5(this.get_distinct_id(),t,n);this.Mr!==r?(this.setPersonPropertiesForFlags(Xe({},n||{},t||{})),this.capture("$set",{$set:t||{},$set_once:n||{}}),this.Mr=r):Ge.info("A duplicate setPersonProperties call was made with the same properties. It has been ignored.")}}group(t,n,r){if(t&&n){var a=this.getGroups();a[t]!==n&&this.resetGroupPropertiesForFlags(t),this.register({$groups:Xe({},a,{[t]:n})}),r&&(this.capture("$groupidentify",{$group_type:t,$group_key:n,$group_set:r}),this.setGroupPropertiesForFlags({[t]:r})),a[t]===n||r||this.reloadFeatureFlags()}else Ge.error("posthog.group requires a group type and group key")}resetGroups(){this.register({$groups:{}}),this.resetGroupPropertiesForFlags(),this.reloadFeatureFlags()}setPersonPropertiesForFlags(t,n){n===void 0&&(n=!0),this.featureFlags.setPersonPropertiesForFlags(t,n)}resetPersonPropertiesForFlags(){this.featureFlags.resetPersonPropertiesForFlags()}setGroupPropertiesForFlags(t,n){n===void 0&&(n=!0),this.Qr("posthog.setGroupPropertiesForFlags")&&this.featureFlags.setGroupPropertiesForFlags(t,n)}resetGroupPropertiesForFlags(t){this.featureFlags.resetGroupPropertiesForFlags(t)}reset(t){var n,r,a,s;if(Ge.info("reset"),!this.__loaded)return Ge.uninitializedWarning("posthog.reset");var o=this.get_property("$device_id");if(this.consent.reset(),(n=this.persistence)==null||n.clear(),(r=this.sessionPersistence)==null||r.clear(),this.surveys.reset(),this.featureFlags.reset(),(a=this.persistence)==null||a.set_property(Gi,"anonymous"),(s=this.sessionManager)==null||s.resetSessionId(),this.Mr=null,this.config.cookieless_mode==="always")this.register_once({distinct_id:Rc,$device_id:null},"");else{var c=this.config.get_device_id(el());this.register_once({distinct_id:c,$device_id:t?c:o},"")}this.register({$last_posthog_reset:new Date().toISOString()},1)}get_distinct_id(){return this.get_property("distinct_id")}getGroups(){return this.get_property("$groups")||{}}get_session_id(){var t,n;return(t=(n=this.sessionManager)==null?void 0:n.checkAndGetSessionAndWindowId(!0).sessionId)!==null&&t!==void 0?t:""}get_session_replay_url(t){if(!this.sessionManager)return"";var{sessionId:n,sessionStartTimestamp:r}=this.sessionManager.checkAndGetSessionAndWindowId(!0),a=this.requestRouter.endpointFor("ui","/project/"+this.config.token+"/replay/"+n);if(t!=null&&t.withTimestamp&&r){var s,o=(s=t.timestampLookBack)!==null&&s!==void 0?s:10;if(!r)return a;a+="?t="+Math.max(Math.floor((new Date().getTime()-r)/1e3)-o,0)}return a}alias(t,n){return t===this.get_property(ES)?(Ge.critical("Attempting to create alias for existing People user - aborting."),-2):this.Qr("posthog.alias")?(qe(n)&&(n=this.get_distinct_id()),t!==n?(this.Zr(jc,t),this.capture("$create_alias",{alias:t,distinct_id:n})):(Ge.warn("alias matches current distinct_id - skipping api call."),this.identify(t),-1)):void 0}set_config(t){var n=Xe({},this.config);if(hr(t)){var r,a,s,o,c;Fn(this.config,L5(t));var d=this.jr();(r=this.persistence)==null||r.update_config(this.config,n,d),this.sessionPersistence=this.config.persistence==="sessionStorage"||this.config.persistence==="memory"?this.persistence:new Rg(Xe({},this.config,{persistence:"sessionStorage"}),d);var f=this.Or(this.config.debug);nl(f)&&(this.config.debug=f),nl(this.config.debug)&&(this.config.debug?(Yi.DEBUG=!0,On.H()&&On.G("ph_debug","true"),Ge.info("set_config",{config:t,oldConfig:n,newConfig:Xe({},this.config)})):(Yi.DEBUG=!1,On.H()&&On.V("ph_debug"))),(a=this.sessionRecording)==null||a.startIfEnabledOrStop(),(s=this.autocapture)==null||s.startIfEnabled(),(o=this.heatmaps)==null||o.startIfEnabled(),this.surveys.loadIfEnabled(),this.ts(),(c=this.externalIntegrations)==null||c.startIfEnabledOrStop()}}startSessionRecording(t){var n=t===!0,r={sampling:n||!(t==null||!t.sampling),linked_flag:n||!(t==null||!t.linked_flag),url_trigger:n||!(t==null||!t.url_trigger),event_trigger:n||!(t==null||!t.event_trigger)};if(Object.values(r).some(Boolean)){var a,s,o,c,d;(a=this.sessionManager)==null||a.checkAndGetSessionAndWindowId(),r.sampling&&((s=this.sessionRecording)==null||s.overrideSampling()),r.linked_flag&&((o=this.sessionRecording)==null||o.overrideLinkedFlag()),r.url_trigger&&((c=this.sessionRecording)==null||c.overrideTrigger("url")),r.event_trigger&&((d=this.sessionRecording)==null||d.overrideTrigger("event"))}this.set_config({disable_session_recording:!1})}stopSessionRecording(){this.set_config({disable_session_recording:!0})}sessionRecordingStarted(){var t;return!((t=this.sessionRecording)==null||!t.started)}captureException(t,n){var r=new Error("PostHog syntheticException"),a=this.exceptions.buildProperties(t,{handled:!0,syntheticException:r});return this.exceptions.sendExceptionEvent(Xe({},a,n))}loadToolbar(t){return this.toolbar.loadToolbar(t)}get_property(t){var n;return(n=this.persistence)==null?void 0:n.props[t]}getSessionProperty(t){var n;return(n=this.sessionPersistence)==null?void 0:n.props[t]}toString(){var t,n=(t=this.config.name)!==null&&t!==void 0?t:Yo;return n!==Yo&&(n=Yo+"."+n),n}_isIdentified(){var t,n;return((t=this.persistence)==null?void 0:t.get_property(Gi))==="identified"||((n=this.sessionPersistence)==null?void 0:n.get_property(Gi))==="identified"}Xr(){var t,n;return!(this.config.person_profiles==="never"||this.config.person_profiles==="identified_only"&&!this._isIdentified()&&Vo(this.getGroups())&&((t=this.persistence)==null||(t=t.props)==null||!t[jc])&&((n=this.persistence)==null||(n=n.props)==null||!n[Tf]))}Jr(){return this.config.capture_pageleave===!0||this.config.capture_pageleave==="if_capture_pageview"&&(this.config.capture_pageview===!0||this.config.capture_pageview==="history_change")}createPersonProfile(){this.Xr()||this.Qr("posthog.createPersonProfile")&&this.setPersonProperties({},{})}Qr(t){return this.config.person_profiles==="never"?(Ge.error(t+' was called, but process_person is set to "never". This call will be ignored.'),!1):(this.Zr(Tf,!0),!0)}jr(){if(this.config.cookieless_mode==="always")return!0;var t=this.consent.isOptedOut(),n=this.config.opt_out_persistence_by_default||this.config.cookieless_mode==="on_reject";return this.config.disable_persistence||t&&!!n}ts(){var t,n,r,a,s=this.jr();return((t=this.persistence)==null?void 0:t.wi)!==s&&((r=this.persistence)==null||r.set_disabled(s)),((n=this.sessionPersistence)==null?void 0:n.wi)!==s&&((a=this.sessionPersistence)==null||a.set_disabled(s)),s}opt_in_capturing(t){var n;if(this.config.cookieless_mode!=="always"){var r,a;this.config.cookieless_mode==="on_reject"&&this.consent.isExplicitlyOptedOut()&&(this.reset(!0),(r=this.sessionManager)==null||r.destroy(),this.sessionManager=new R5(this),this.persistence&&(this.sessionPropsManager=new O5(this,this.sessionManager,this.persistence)),this.sessionRecording=new I5(this),this.sessionRecording.startIfEnabledOrStop()),this.consent.optInOut(!0),this.ts(),this.Gr(),(n=this.sessionRecording)==null||n.startIfEnabledOrStop(),this.config.cookieless_mode=="on_reject"&&this.surveys.loadIfEnabled(),(qe(t==null?void 0:t.captureEventName)||t!=null&&t.captureEventName)&&this.capture((a=t==null?void 0:t.captureEventName)!==null&&a!==void 0?a:"$opt_in",t==null?void 0:t.captureProperties,{send_instantly:!0}),this.config.capture_pageview&&this.Vr()}else Ge.warn('Consent opt in/out is not valid with cookieless_mode="always" and will be ignored')}opt_out_capturing(){var t,n;this.config.cookieless_mode!=="always"?(this.config.cookieless_mode==="on_reject"&&this.consent.isOptedIn()&&this.reset(!0),this.consent.optInOut(!1),this.ts(),this.config.cookieless_mode==="on_reject"&&(this.register({distinct_id:Rc,$device_id:null}),(t=this.sessionManager)==null||t.destroy(),this.sessionManager=void 0,this.sessionPropsManager=void 0,(n=this.sessionRecording)==null||n.stopRecording(),this.sessionRecording=void 0,this.Vr())):Ge.warn('Consent opt in/out is not valid with cookieless_mode="always" and will be ignored')}has_opted_in_capturing(){return this.consent.isOptedIn()}has_opted_out_capturing(){return this.consent.isOptedOut()}get_explicit_consent_status(){var t=this.consent.consent;return t===Vi.GRANTED?"granted":t===Vi.DENIED?"denied":"pending"}is_capturing(){return this.config.cookieless_mode==="always"||(this.config.cookieless_mode==="on_reject"?this.consent.isExplicitlyOptedOut()||this.consent.isOptedIn():!this.has_opted_out_capturing())}clear_opt_in_out_capturing(){this.consent.reset(),this.ts()}_is_bot(){return ya?y9(ya,this.config.custom_blocked_useragents):void 0}Vr(){$e&&($e.visibilityState==="visible"?this.Cr||(this.Cr=!0,this.capture("$pageview",{title:$e.title},{send_instantly:!0}),this.Ir&&($e.removeEventListener("visibilitychange",this.Ir),this.Ir=null)):this.Ir||(this.Ir=this.Vr.bind(this),Xn($e,"visibilitychange",this.Ir)))}debug(t){t===!1?(be==null||be.console.log("You've disabled debug mode."),this.set_config({debug:!1})):(be==null||be.console.log("You're now in debug mode. All calls to PostHog will be logged in your console.\nYou can disable this with `posthog.debug(false)`."),this.set_config({debug:!0}))}O(){var t,n,r,a,s,o,c,d=this.Ar||{};return"advanced_disable_flags"in d?!!d.advanced_disable_flags:this.config.advanced_disable_flags!==!1?!!this.config.advanced_disable_flags:this.config.advanced_disable_decide===!0?(Ge.warn("Config field 'advanced_disable_decide' is deprecated. Please use 'advanced_disable_flags' instead. The old field will be removed in a future major version."),!0):(r="advanced_disable_decide",a=!1,s=Ge,o=(n="advanced_disable_flags")in(t=d)&&!qe(t[n]),c=r in t&&!qe(t[r]),o?t[n]:c?(s&&s.warn("Config field '"+r+"' is deprecated. Please use '"+n+"' instead. The old field will be removed in a future major version."),t[r]):a)}Yr(t){if(Gt(this.config.before_send))return t;var n=Vt(this.config.before_send)?this.config.before_send:[this.config.before_send],r=t;for(var a of n){if(r=a(r),Gt(r)){var s="Event '"+t.event+"' was rejected in beforeSend function";return NY(t.event)?Ge.warn(s+". This can cause unexpected behavior."):Ge.info(s),null}r.properties&&!Vo(r.properties)||Ge.warn("Event '"+t.event+"' has no properties after beforeSend function, this is likely an error.")}return r}getPageViewId(){var t;return(t=this.pageViewManager.Vt)==null?void 0:t.pageViewId}captureTraceFeedback(t,n){this.capture("$ai_feedback",{$ai_trace_id:String(t),$ai_feedback_text:n})}captureTraceMetric(t,n,r){this.capture("$ai_metric",{$ai_trace_id:String(t),$ai_metric_name:n,$ai_metric_value:String(r)})}Or(t){var n=nl(t)&&!t,r=On.H()&&On.q("ph_debug")==="true";return!n&&(!!r||t)}}(function(e,t){for(var n=0;n<t.length;n++)e.prototype[t[n]]=tG(e.prototype[t[n]])})(um,["identify"]);var P5,j0=(P5=o0[Yo]=new um,function(){function e(){e.done||(e.done=!0,_9=!1,hn(o0,function(t){t._dom_loaded()}))}$e!=null&&$e.addEventListener?$e.readyState==="complete"?e():Xn($e,"DOMContentLoaded",e,{capture:!1}):be&&Ge.error("Browser doesn't support `document.addEventListener` so PostHog couldn't be initialized")}(),P5);let lu=!1;function UV(){lu||(j0.init("phc_2qafhOCTgCnygXsPEHOA0RBtJf5nvVsi7yIene4DWaF",{api_host:"https://us.i.posthog.com",person_profiles:"identified_only",capture_pageview:!0,capture_pageleave:!0,autocapture:!1,persistence:"localStorage",loaded:e=>{var t,n;(n=(t=import.meta)==null?void 0:t.env)!=null&&n.DEV&&console.log("[PostHog] Initialized in development mode")}}),lu=!0)}function FV(e,t={}){var a,s,o,c,d;if(!e||!lu)return;const n={email:((a=e.primaryEmailAddress)==null?void 0:a.emailAddress)||((o=(s=e.emailAddresses)==null?void 0:s[0])==null?void 0:o.emailAddress),name:e.fullName||`${e.firstName||""} ${e.lastName||""}`.trim(),first_name:e.firstName,last_name:e.lastName,avatar_url:e.imageUrl,created_at:e.createdAt,...t},r=Object.fromEntries(Object.entries(n).filter(([f,m])=>m!=null&&m!==""));j0.identify(e.id,r),(d=(c=import.meta)==null?void 0:c.env)!=null&&d.DEV&&console.log("[PostHog] User identified:",e.id,r)}function zV(){var e,t;lu&&(j0.reset(),(t=(e=import.meta)==null?void 0:e.env)!=null&&t.DEV&&console.log("[PostHog] User reset"))}function E9(){return typeof window>"u"?"unknown":window.location.hostname}function HV(){var t,n;if(!lu)return;const e=E9();j0.register({agent_domain:e,agent_subdomain:e.split(".")[0]}),(n=(t=import.meta)==null?void 0:t.env)!=null&&n.DEV&&console.log("[PostHog] Agent domain set:",e)}function Jn(e,t={}){var n,r;lu&&(j0.capture(e,{...t,agent_domain:E9()}),(r=(n=import.meta)==null?void 0:n.env)!=null&&r.DEV&&console.log("[PostHog] Event:",e,t))}function qV(){Jn("user_signed_in")}function jV(){Jn("user_signed_out")}function $V(e={}){Jn("message_sent",{has_attachments:e.hasAttachments||!1,attachment_count:e.attachmentCount||0,message_length:e.messageLength||0,chat_id:e.chatId,is_new_chat:e.isNewChat||!1})}function YV(e){Jn("chat_created",{chat_id:e})}function GV(e){Jn("chat_selected",{chat_id:e})}function VV(e){Jn("chat_deleted",{chat_id:e})}function KV(){Jn("new_chat_started")}function WV(e){Jn("message_regenerated",{message_index:e})}function XV(e){Jn("message_retried",{message_index:e})}function Qb(){Jn("generation_stopped")}function QV(e,t){Jn("file_upload_started",{file_name:e,file_type:t})}function ZV(e,t){Jn("file_upload_completed",{file_name:e,file_type:t})}function JV(e,t){Jn("file_upload_failed",{file_name:e,error_message:t})}function B5(e){Jn("tier_modal_shown",{tier:e})}function eK(e){Jn("tier_modal_dismissed",{tier:e})}function tK(e){Jn("upgrade_clicked",{tier:e})}function nK(e){Jn("free_limit_reached",{message_count:e})}function Lg(e,t,n={}){Jn("error_occurred",{error_type:e,error_message:t,...n})}function rK(){Jn("sidebar_opened")}function U5(){Jn("sidebar_closed")}const aK=({messages:e,setMessages:t,setShouldFocus:n,setIsLoading:r,setUploadProgress:a,setIsUserScrolled:s,onFirstSend:o,auth:c,org:d,getToken:f,setActive:m,activeChatId:p,setActiveChatId:b,onMessageSuccess:y,analyticsEnabled:T})=>{const x=z.useRef(null),C=z.useCallback(async O=>{const D=typeof O=="object"&&O!==null,Y=D?O.text||"":String(O||""),H=D?O.files||[]:[];if(!Y.trim()&&H.length===0){n(!0);return}if(Y===":clear"&&H.length===0){t([]),b(null),ar.setActiveChat(""),window.scrollTo(0,0);return}e.length===0&&o(),r(!0),s(!1);const B=[];for(let de=0;de<H.length;de++){const V=H[de];try{a({fileName:V.name,progress:0}),T&&QV(V.name,V.type);const me=await X$(V);a({fileName:V.name,progress:100}),T&&ZV(V.name,V.type),B.push(me),setTimeout(()=>a(null),500)}catch(me){console.error("Upload error:",me),T&&JV(V.name,me.message),a(null),r(!1),t(pe=>[...pe,{type:"request",content:Y},{type:"response",content:"",error:{type:"server",message:"Failed to upload "+V.name+": "+me.message,status:null}}]);return}}let ee=Y;if(B.length>0){const de=`<!--cycls:attachments
`+JSON.stringify(B)+`
cycls:attachments-->`;ee=Y+`

`+de}const Z={type:"request",content:ee},Q={type:"response",content:""};t(de=>[...de,Z,Q]);const G=[...e,Z];let se=p;const oe=!se;se?ar.updateChat(se,G):(se=ar.createChat(G).id,b(se),T&&YV(se)),T&&$V({hasAttachments:B.length>0,attachmentCount:B.length,messageLength:Y.length,chatId:se,isNewChat:oe}),x.current=new AbortController;let ye=!1;const fe=setTimeout(()=>{x.current&&(ye=!0,x.current.abort())},12e4);try{await wb({messages:G,auth:c,getToken:f,setActive:m,org:d,onDelta:V=>{t(me=>{const pe=[...me];return pe[pe.length-1].content=V,pe})},signal:x.current.signal}),clearTimeout(fe);const de=[...G,Q];ar.updateChat(se,de),y&&y(),r(!1),n(!0)}catch(de){clearTimeout(fe);let V={type:"server",message:"An unexpected error occurred",status:null};if(de.name==="AbortError")if(ye)V={type:"timeout",message:"Request timed out. The server took too long to respond."},T&&Lg("timeout",V.message);else{console.log("Generation stopped by user"),T&&Qb(),t(me=>{var Be;const pe=[...me];if(((Be=pe[pe.length-1])==null?void 0:Be.type)==="response"){pe[pe.length-1].content+=`

*[Generation stopped]*`;const k=e.concat([Z,pe[pe.length-1]]);se&&ar.updateChat(se,k)}return pe}),r(!1),n(!0);return}else de instanceof TypeError&&de.message.includes("fetch")?(V={type:"network",message:"Network error. Please check your connection and try again."},T&&Lg("network",V.message)):de.type&&(V=de,T&&Lg(de.type,de.message,{status:de.status}));console.error("Request error:",de),t(me=>{var Be;const pe=[...me];return((Be=pe[pe.length-1])==null?void 0:Be.type)==="response"&&(pe[pe.length-1].error=V),pe}),r(!1),n(!0)}},[e,c,d,f,m,p,b,y,T,t,n,r,a,s,o]),w=z.useCallback(()=>{x.current&&(x.current.abort(),x.current=null)},[]);return{send:C,handleStop:w}},iK=e=>{const[t,n]=z.useState(!1);return z.useEffect(()=>{const r=()=>n(window.innerWidth<768);return r(),window.addEventListener("resize",r),()=>window.removeEventListener("resize",r)},[]),t?0:e?gf.expanded:gf.collapsed},sK=(e,t,n=null,r=!0)=>{const a=z.useRef(null);z.useEffect(()=>{!t||!r||(n?a.current!==n.id&&(FV(n,{subscription_status:e?"paid":"free"}),a.current===null&&qV(),a.current=n.id):a.current!==null&&(jV(),zV(),a.current=null))},[n,r,e,t])},lK=({HEADER:e,INTRO:t,AUTH:n,ORG:r,TITLE:a,TIER:s})=>R.jsx(T9,{HEADER:e,INTRO:t,AUTH:n,ORG:r,TITLE:a,TIER:s,authApi:null,clerkApi:null,userApi:{user:null,isLoaded:!0},subscription:null,isSubscriptionLoading:!1,subscriptionError:null}),oK=({HEADER:e,INTRO:t,AUTH:n,ORG:r,TITLE:a,TIER:s})=>{const o=w0(),c=mN(),d=pN(),{subscription:f,isLoading:m,error:p}=vY();return R.jsx(T9,{HEADER:e,INTRO:t,AUTH:n,ORG:r,TITLE:a,TIER:s,authApi:o,clerkApi:c,userApi:d,subscription:f,isSubscriptionLoading:m,subscriptionError:p})},F5=e=>e.AUTH?R.jsx(oK,{...e}):R.jsx(lK,{...e}),T9=({HEADER:e,INTRO:t,AUTH:n,ORG:r,TITLE:a,TIER:s,authApi:o,clerkApi:c,userApi:d,subscription:f,isSubscriptionLoading:m,subscriptionError:p})=>{const b=DA(),[y,T]=z.useState([]),[x,C]=z.useState(!1),[w,O]=z.useState(!1),[D,Y]=z.useState(null),[H,B]=z.useState(!1),[ee,Z]=z.useState(!1),[Q,G]=z.useState(null),se=z.useRef(null),[oe,ye]=z.useState(!1),[fe,de]=z.useState([]),[V,me]=z.useState(null),pe=iK(oe),[Be,k]=z.useState(!1),[ie,ge]=z.useState(0),L=5,Oe=z.useCallback(()=>{k(!1),b&&eK(s)},[s,b]),je=z.useMemo(()=>{var We,on,qn;return f?((qn=(on=(We=f==null?void 0:f.subscriptionItems)==null?void 0:We[0])==null?void 0:on.plan)==null?void 0:qn.name)==="Cycls Pass":!1},[f]),De=z.useMemo(()=>{var We,on,qn;return f&&((qn=(on=(We=f==null?void 0:f.subscriptionItems)==null?void 0:We[0])==null?void 0:on.plan)==null?void 0:qn.name)||null},[f]);sK(n&&je,b&&n,(d==null?void 0:d.user)||null,(d==null?void 0:d.isLoaded)??!0);const Ot=z.useMemo(()=>!n||s!=="cycls_pass"||je?!1:ie>=L,[n,s,je,ie,L]);z.useEffect(()=>{je&&(SY(),ge(0),k(!1))},[je]),z.useEffect(()=>{const We=ar.getAllChats();de(We),ge(dS())},[]),z.useEffect(()=>{y.length>0&&V&&(ar.updateChat(V,y),de(ar.getAllChats()))},[y,V]),z.useEffect(()=>{document.title=a},[a]),z.useEffect(()=>{const We=()=>{const on=window.scrollY||document.documentElement.scrollTop,qn=document.documentElement.scrollHeight,Ln=window.innerHeight,nn=qn-on-Ln<100;Z(!nn)};return window.addEventListener("scroll",We),()=>window.removeEventListener("scroll",We)},[]),z.useEffect(()=>{!ee&&se.current&&se.current.scrollIntoView({behavior:"smooth"})},[y,ee]),z.useEffect(()=>{if(x){const We=document.querySelector("textarea");We==null||We.focus(),C(!1)}},[x]);const ot=z.useCallback(()=>{if(!n||s!=="cycls_pass"||je)return;const We=xY();ge(We),We>=L&&(k(!0),b&&nK(We),b&&B5(s))},[n,s,je,L,b]),{send:zt,handleStop:wt}=aK({messages:y,setMessages:T,setShouldFocus:C,setIsLoading:O,setUploadProgress:Y,setIsUserScrolled:Z,onFirstSend:()=>B(!0),auth:n,org:r,getToken:o==null?void 0:o.getToken,setActive:c==null?void 0:c.setActive,activeChatId:V,setActiveChatId:me,onMessageSuccess:ot,analyticsEnabled:b}),vn=z.useCallback(()=>{T([]),me(null),B(!1),ar.setActiveChat(""),window.scrollTo(0,0),b&&KV()},[b]),mr=z.useCallback(We=>{const on=ar.getChat(We);on&&(T(on.messages),me(We),B(on.messages.length>0),ar.setActiveChat(We),window.innerWidth<768&&ye(!1),window.scrollTo(0,0),b&&GV(We))},[b]),vt=z.useCallback(We=>{ar.deleteChat(We),de(ar.getAllChats()),b&&VV(We),We===V&&vn()},[V,vn,b]),gt=z.useCallback(()=>{const We=!oe;ye(We),b&&(We?rK():U5())},[oe,b]),Bt=z.useCallback(()=>{ye(!1),b&&U5()},[b]),Je=z.useCallback(async We=>{const on=We-1;if(on<0||(b&&WV(We),T(Ue=>{var tt;const ze=[...Ue];return((tt=ze[We])==null?void 0:tt.type)==="response"&&(ze[We]={type:"response",content:""}),ze}),y[on].type!=="request"))return;const Ln=y.slice(0,We);O(!0),Z(!1),Ln.map(({type:Ue,content:ze})=>{const tt=Ue==="request"?"user":"assistant",{clean:er,attachments:$r}=F0(ze);if($r.length===0)return{role:tt,content:er};const Pn=[];er.trim()&&Pn.push({type:"text",text:er});for(const tr of $r)tr.mime&&tr.mime.startsWith("image/")?Pn.push({type:"image_url",image_url:{url:tr.url}}):Pn.push({type:"file",text:"[File: "+tr.name+"] URL: "+tr.url});return{role:tt,content:Pn}});const nn=new AbortController;let he=!1;const Se=setTimeout(()=>{he=!0,nn.abort()},12e4);try{await wb({messages:Ln,auth:n,getToken:o==null?void 0:o.getToken,setActive:c==null?void 0:c.setActive,org:r,onDelta:Ue=>{T(ze=>{const tt=[...ze];return tt[We].content=Ue,tt})},signal:nn.signal}),clearTimeout(Se),V&&T(Ue=>(ar.updateChat(V,Ue),de(ar.getAllChats()),Ue)),O(!1),C(!0)}catch(Ue){if(clearTimeout(Se),Ue.name==="AbortError"&&!he){b&&Qb(),T(ze=>{const tt=[...ze];return tt[We].content+=`

*[Generation stopped]*`,V&&ar.updateChat(V,tt),tt}),O(!1),C(!0);return}console.error("Regeneration error:",Ue),T(ze=>{const tt=[...ze];return tt[We].error={type:"server",message:Ue.message||"Error"},tt}),O(!1)}},[y,n,r,o==null?void 0:o.getToken,c==null?void 0:c.setActive,V,b]),yt=z.useCallback(async We=>{const on=We-1;if(on<0)return;if(b&&XV(We),G(We),T(Ue=>{var tt;const ze=[...Ue];return((tt=ze[We])==null?void 0:tt.type)==="response"&&(ze[We]={type:"response",content:""}),ze}),y[on].type!=="request"){G(null);return}const Ln=y.slice(0,We);O(!0),Z(!1);const nn=new AbortController;let he=!1;const Se=setTimeout(()=>{he=!0,nn.abort()},12e4);try{await wb({messages:Ln,auth:n,getToken:o==null?void 0:o.getToken,setActive:c==null?void 0:c.setActive,org:r,onDelta:Ue=>{T(ze=>{const tt=[...ze];return tt[We].content=Ue,tt})},signal:nn.signal}),clearTimeout(Se),V&&T(Ue=>(ar.updateChat(V,Ue),de(ar.getAllChats()),Ue)),O(!1),G(null),C(!0)}catch(Ue){if(clearTimeout(Se),Ue.name==="AbortError"&&!he){b&&Qb(),T(ze=>{const tt=[...ze];return tt[We].content+=`

*[Generation stopped]*`,V&&ar.updateChat(V,tt),tt}),O(!1),G(null),C(!0);return}console.error("Retry error:",Ue),T(ze=>{const tt=[...ze];return tt[We].error={type:"server",message:Ue.message||"Error"},tt}),O(!1),G(null)}},[y,n,r,o==null?void 0:o.getToken,c==null?void 0:c.setActive,V,b]),mn=z.useCallback(()=>{k(!0),b&&(tK(s),B5(s))},[s,b]),Lt=y.length===0,ln=n?R.jsx(dO,{appearance:{elements:{avatarBox:"w-6 h-6",userButtonTrigger:"focus:shadow-none"}}}):null;return R.jsxs("div",{className:"min-h-screen theme-transition",style:{backgroundColor:"var(--bg-primary)"},children:[R.jsx(mY,{isOpen:oe,onClose:Bt,onToggle:gt,chats:fe,activeChat:V,onSelectChat:mr,onNewChat:vn,onDeleteChat:vt,user:d==null?void 0:d.user,isOnPaidPlan:je,tierName:De,UserButtonComponent:ln,isAuthenticated:n}),R.jsx(pY,{open:Be&&s==="cycls_pass",onClose:Oe,tier:s||"Pro"}),R.jsxs("main",{className:"min-h-screen transition-all duration-300 ease-in-out",style:{marginLeft:`${pe}px`},children:[R.jsx("div",{className:"p-10",children:R.jsxs("div",{className:"fixed max-sm:top-[2%] max-sm:right-1 top-1 right-4 flex items-center gap-1 z-20",children:[R.jsx($5,{}),n&&R.jsxs("a",{href:"https://cycls.ai/",type:"button",className:`
                inline-flex items-center gap-2
                rounded-xl px-3 py-2
                text-sm font-medium
                hover:bg-[var(--bg-hover)]
                active:bg-[var(--bg-active)]
                transition-colors
                focus:outline-none focus-visible:ring-2 focus-visible:ring-[var(--text-primary)]/20
              `,style:{color:"var(--text-primary)"},children:[R.jsxs("svg",{xmlns:"http://www.w3.org/2000/svg",width:"18",height:"18",viewBox:"0 0 24 24",fill:"none",stroke:"currentColor",strokeWidth:"1.7",strokeLinecap:"round",strokeLinejoin:"round",children:[R.jsx("path",{d:"M15 21v-8a1 1 0 0 0-1-1h-4a1 1 0 0 0-1 1v8"}),R.jsx("path",{d:"M3 10a2 2 0 0 1 .709-1.528l7-6a2 2 0 0 1 2.582 0l7 6A2 2 0 0 1 21 10v9a2 2 0 0 1-2 2H5a2 2 0 0 1-2-2z"})]}),R.jsx("span",{children:"Home"})]})]})}),R.jsxs("div",{className:"mx-auto max-w-3xl min-h-screen px-4 pb-[200px] md:pb-[180px]",children:[D&&R.jsx(aY,{fileName:D.fileName,progress:D.progress}),R.jsx("div",{className:"prose m-2 mx-auto max-w-3xl p-2 prose-pre:p-0 pt-14 md:pt-2",children:R.jsx(mf,{markdown:e,onSend:zt})}),!Lt&&R.jsxs("div",{className:`transition-opacity duration-500 ${H?"opacity-100":"opacity-0"}`,children:[R.jsx(uS,{messages:y,onSend:zt,onRegenerate:Je,onRetry:yt,retryingIndex:Q,isGenerating:w}),R.jsx("div",{ref:se})]}),Lt&&!H&&t&&t.trim()!==""&&R.jsx("div",{className:"mx-auto max-w-3xl prose prose-sm prose-pre:p-0",children:R.jsx(mf,{markdown:t,onSend:zt})})]})]}),R.jsx(rY,{onSend:zt,isLoading:w,onStop:wt,disabled:Ot,onUpgradeClick:mn,sidebarWidth:pe})]})},z5=({meta:e})=>{const t=!!e.prod,n=!!e.auth,r=String(e.header??""),a=String(e.intro??""),s=String(e.title??"AI Agent"),o=e.tier||null,c=t?e.pk_live:e.pk_test,d=e.org||void 0,f=typeof window<"u"?window.location.origin+"/":"/";if(!n)return R.jsx(F5,{HEADER:r,INTRO:a,AUTH:n,ORG:d,TITLE:s,TIER:o});const m=t?{isSatellite:!1,domain:"cycls.ai",fallbackRedirectUrl:f,forceRedirectUrl:f}:{fallbackRedirectUrl:f,forceRedirectUrl:f};return R.jsx($8,{publishableKey:c,...m,children:R.jsx(yY,{tier:o,children:R.jsx(Kk,{children:R.jsxs(wk,{children:[R.jsx(Dh,{path:"/auth",element:R.jsx(gY,{afterUrl:f})}),R.jsx(Dh,{path:"/",element:R.jsx(bY,{children:R.jsx(F5,{HEADER:r,INTRO:a,AUTH:n,ORG:d,TITLE:s,TIER:o})})}),R.jsx(Dh,{path:"*",element:R.jsx(t8,{to:"/",replace:!0})})]})})})})};function uK(){const[e,t]=ce.useState({status:"loading"});return ce.useEffect(()=>{const n=new AbortController;return(async()=>{try{const r=await TY(n.signal);t({status:"ready",data:r})}catch(r){t({status:"error",error:r})}})(),()=>n.abort()},[]),e}const cK=()=>{var n,r,a,s;const e=uK();if(z.useEffect(()=>{var o;e.status==="ready"&&((o=e.data)==null?void 0:o.analytics)===!0&&(UV(),HV())},[e.status,(n=e.data)==null?void 0:n.analytics]),e.status==="loading")return R.jsx(p1,{children:R.jsx("div",{className:"min-h-screen grid place-items-center text-sm",style:{backgroundColor:"var(--bg-primary)",color:"var(--text-tertiary)"},children:"Loading…"})});if(e.status==="error"){console.error("Failed to load /metadata:",e.error);const o={header:"",intro:"",prod:((a=(r=import.meta)==null?void 0:r.env)==null?void 0:a.PROD)||!1,auth:!1,title:"AI Agent",tier:null,analytics:!1};return R.jsx(p1,{children:R.jsx(Bg.Provider,{value:!1,children:R.jsx(z5,{meta:o})})})}const t=((s=e.data)==null?void 0:s.analytics)===!0;return R.jsx(p1,{children:R.jsx(Bg.Provider,{value:t,children:R.jsx(z5,{meta:e.data})})})},Wh=document.getElementById("root");Wh._reactRoot||(Wh._reactRoot=IA.createRoot(Wh));Wh._reactRoot.render(R.jsx(cK,{}));
